-- Information about category:
-- Only one drop will be given per category, except in the cases when category is -1.
-- Category -1 is used for SPOIL/SWEEP drops only!  Do NOT change this.
-- In general, category 0 is for adena and 1 seal stone color (each seal stone color should be in a different category).
-- In general, category 1 is for full drops and parts of equipable items.  However, you can change this.
-- In general, category 2 is for all other items.  However, you can change this.
-- In general, more categories are only used for RBs who have 1 item per category (i.e. do not really drop categorized),
-- with the sole exception of category 200 that is used for Life Stones. However, you can change this too.
-- You can create more categories as you see fit.  Just make sure the "category" number is non-negative!!
-- Also, it is NOT a problem if category numbers are skipped (so you can have -1, 1, 5, 10 as your categories).
--
-- If you wish to allow more than one item to be given from the same category, you can
-- split them up over several categories.
-- In addition, RBs and Grandbosses (mainly) may have the exact same item repeated in multiple categories.
-- This allows mobs to give 1 copy of the drop to each of several people (if they are lucky enough to get the drops).
-- Calculation for each drop, when in categories, is equivallent in chance as when outside of categories.
-- First, the sum of chances for each category is calculated as category chance.  If the category is selected
-- for drops (i.e. its chance is successful), then exactly 1 item from that category will be selected, with 
-- such a chance that the overall probability is maintained unchanged. 

DROP TABLE IF EXISTS `droplist`;
CREATE TABLE `droplist` (
  `mobId` INT NOT NULL DEFAULT '0',
  `itemId` INT NOT NULL DEFAULT '0',
  `min` INT NOT NULL DEFAULT '0',
  `max` INT NOT NULL DEFAULT '0',
  `category` INT NOT NULL DEFAULT '0',
  `chance` INT NOT NULL DEFAULT '0',
  PRIMARY KEY (`mobId`,`itemId`,`category`),
  KEY `key_mobId` (`mobId`)
);

-- Category Explanation (non-RaidBoss & non-GrandBoss)
--   -1 is sweep
--    0 is adena and one of the seal stones (your choice)
--    1 is any item that is in weapon.sql, any item in armor.sql, and some of the items from etcitem. Before dropping the column, I had those marked in etcitem.sql. However, you can recognize them easily. If they are parts unique for crafting equipable items, it's category 1.
--    2 is almost all of the other items except:
--    3 and 4 are used for the other two seal stones (1 category each).

INSERT INTO `droplist` VALUES 
(18001,1806,1,1,-1,10868),-- Recipe: Soulshot: B-Grade
(18001,57,765,1528,0,700000),-- Adena
(18001,4069,1,1,1,2102),-- Tunic of Zubei Fabric
(18001,4070,1,1,1,3192),-- Stockings of Zubei Fabric
(18001,4071,1,1,1,1615),-- Avadon Robe Fabric
(18001,1419,1,1,2,200000),-- Blood Mark
(18001,1864,1,1,2,166667),-- Stem
(18001,1866,1,1,2,62500),-- Suede
(18001,1878,1,1,2,37037),-- Braided Hemp
(18001,1885,1,1,2,7092),-- High Grade Suede
(18001,1889,1,1,2,5435),-- Compound Braid
(18001,4197,1,1,2,8),-- Recipe: Demon Dagger
(18003,1871,1,1,-1,9797),-- Charcoal
(18010,57,173,347,0,700000),-- Adena
(18012,57,148,293,0,700000),-- Adena
(18013,57,148,293,0,700000),-- Adena
(18014,57,228,455,0,700000),-- Adena
(18015,57,194,387,0,700000),-- Adena
(18017,57,194,387,0,700000),-- Adena
(18018,57,253,504,0,700000),-- Adena
(18020,57,305,611,0,700000),-- Adena
(18022,57,265,529,0,700000),-- Adena
(18023,57,265,529,0,700000),-- Adena
(18024,57,381,761,0,700000),-- Adena
(18025,57,335,669,0,700000),-- Adena
(18027,57,335,669,0,700000),-- Adena
(18028,57,414,829,0,700000),-- Adena
(18030,57,486,972,0,700000),-- Adena
(18032,57,431,863,0,700000),-- Adena
(18033,57,431,863,0,700000),-- Adena
(18034,57,585,1171,0,700000),-- Adena
(18035,57,525,1049,0,700000),-- Adena
(18037,57,525,1049,0,700000),-- Adena
(18038,57,629,1257,0,700000),-- Adena
(18040,57,721,1443,0,700000),-- Adena
(18042,57,651,1302,0,700000),-- Adena
(18043,57,651,1302,0,700000),-- Adena
(18044,57,848,1697,0,700000),-- Adena
(18045,57,770,1541,0,700000),-- Adena
(18047,57,770,1541,0,700000),-- Adena
(18048,57,902,1804,0,700000),-- Adena
(18050,57,902,1804,0,700000),-- Adena
(18052,57,902,1804,0,700000),-- Adena
(18053,57,902,1804,0,700000),-- Adena
(18054,57,902,1804,0,700000),-- Adena
(18055,57,902,1804,0,700000),-- Adena
(18057,57,902,1804,0,700000),-- Adena
(18058,57,902,1804,0,700000),-- Adena
(18060,57,173,347,0,700000),-- Adena
(18062,57,148,293,0,700000),-- Adena
(18063,57,148,293,0,700000),-- Adena
(18064,57,228,455,0,700000),-- Adena
(18065,57,194,387,0,700000),-- Adena
(18067,57,194,387,0,700000),-- Adena
(18068,57,253,504,0,700000),-- Adena
(18070,57,305,611,0,700000),-- Adena
(18072,57,265,529,0,700000),-- Adena
(18073,57,265,529,0,700000),-- Adena
(18074,57,381,761,0,700000),-- Adena
(18075,57,335,669,0,700000),-- Adena
(18077,57,335,669,0,700000),-- Adena
(18078,57,414,829,0,700000),-- Adena
(18080,57,486,972,0,700000),-- Adena
(18082,57,431,863,0,700000),-- Adena
(18083,57,431,863,0,700000),-- Adena
(18084,57,585,1171,0,700000),-- Adena
(18085,57,525,1049,0,700000),-- Adena
(18087,57,525,1049,0,700000),-- Adena
(18088,57,629,1257,0,700000),-- Adena
(18090,57,721,1443,0,700000),-- Adena
(18092,57,651,1302,0,700000),-- Adena
(18093,57,651,1302,0,700000),-- Adena
(18094,57,848,1697,0,700000),-- Adena
(18095,57,770,1541,0,700000),-- Adena
(18097,57,770,1541,0,700000),-- Adena
(18098,57,902,1804,0,700000),-- Adena
(18100,57,902,1804,0,700000),-- Adena
(18102,57,902,1804,0,700000),-- Adena
(18103,57,902,1804,0,700000),-- Adena
(18104,57,902,1804,0,700000),-- Adena
(18105,57,902,1804,0,700000),-- Adena
(18107,57,902,1804,0,700000),-- Adena
(18108,57,902,1804,0,700000),-- Adena
(18109,955,1,1,2,2000),-- Scroll: Enchant Weapon (Grade D)
(18109,956,1,1,2,15000),-- Scroll: Enchant Armor (Grade D)
(18109,1060,1,1,2,50000),-- Lesser Healing Potion
(18109,1061,1,1,2,50000),-- Healing Potion
(18110,951,1,1,2,1500),-- Scroll: Enchant Weapon (Grade C)
(18110,952,1,1,2,12000),-- Scroll: Enchant Armor (Grade C)
(18110,1061,1,1,2,50000),-- Healing Potion
(18110,1832,1,1,2,25000),-- Greater Antidote
(18110,1834,1,1,2,25000),-- Emergency Dressing
(18111,734,1,1,2,25000),-- Haste Potion
(18111,735,1,1,2,25000),-- Potion of Alacrity
(18111,951,1,1,2,2000),-- Scroll: Enchant Weapon (Grade C)
(18111,952,1,1,2,18000),-- Scroll: Enchant Armor (Grade C)
(18111,1539,1,1,2,50000),-- Greater Healing Potion
(18112,734,1,1,2,25000),-- Haste Potion
(18112,735,1,1,2,25000),-- Potion of Alacrity
(18112,947,1,1,2,500),-- Scroll: Enchant Weapon (Grade B)
(18112,948,1,1,2,7000),-- Scroll: Enchant Armor (Grade B)
(18112,1539,1,1,2,50000),-- Greater Healing Potion
(18113,729,1,1,2,300),-- Scroll: Enchant Weapon (Grade A)
(18113,730,1,1,2,2500),-- Scroll: Enchant Armor (Grade A)
(18113,734,1,1,2,25000),-- Haste Potion
(18113,735,1,1,2,25000),-- Potion of Alacrity
(18113,1539,1,1,2,50000),-- Greater Healing Potion
(18114,955,1,1,2,2000),-- Scroll: Enchant Weapon (Grade D)
(18114,956,1,1,2,15000),-- Scroll: Enchant Armor (Grade D)
(18114,1060,1,1,2,50000),-- Lesser Healing Potion
(18114,1061,1,1,2,50000),-- Healing Potion
(18115,951,1,1,2,1500),-- Scroll: Enchant Weapon (Grade C)
(18115,952,1,1,2,12000),-- Scroll: Enchant Armor (Grade C)
(18115,1061,1,1,2,50000),-- Healing Potion
(18115,1832,1,1,2,25000),-- Greater Antidote
(18115,1834,1,1,2,25000),-- Emergency Dressing
(18116,734,1,1,2,25000),-- Haste Potion
(18116,735,1,1,2,25000),-- Potion of Alacrity
(18116,951,1,1,2,2000),-- Scroll: Enchant Weapon (Grade C)
(18116,952,1,1,2,18000),-- Scroll: Enchant Armor (Grade C)
(18116,1539,1,1,2,50000),-- Greater Healing Potion
(18117,734,1,1,2,25000),-- Haste Potion
(18117,735,1,1,2,25000),-- Potion of Alacrity
(18117,947,1,1,2,500),-- Scroll: Enchant Weapon (Grade B)
(18117,948,1,1,2,7000),-- Scroll: Enchant Armor (Grade B)
(18117,1539,1,1,2,50000),-- Greater Healing Potion
(18118,729,1,1,2,300),-- Scroll: Enchant Weapon (Grade A)
(18118,730,1,1,2,2500),-- Scroll: Enchant Armor (Grade A)
(18118,734,1,1,2,25000),-- Haste Potion
(18118,735,1,1,2,25000),-- Potion of Alacrity
(18118,1539,1,1,2,50000),-- Greater Healing Potion
(18256,57,1370,2743,0,700000),-- Adena
(18256,81,1,1,1,2),-- Dragon Slayer
(18256,151,1,1,1,2),-- Sword of Miracles 
(18256,2500,1,1,1,2),-- Dark Legion\'s Edge
(18256,5529,1,1,1,440),-- Dragon Slayer Edge
(18256,5545,1,1,1,440),-- Dark Legion\'s Edge Blade
(18256,5546,1,1,1,483),-- Sword of Miracles Edge
(18256,959,1,1,2,8),-- Scroll: Enchant Weapon (Grade S)
(18256,1895,1,1,2,54858),-- Metallic Fiber
(18256,4040,1,1,2,3841),-- Mold Lubricant
(18256,4042,1,1,2,3201),-- Enria
(18256,4043,1,1,2,6401),-- Asofe
(18265,57,480,858,0,700000),-- Adena
(18265,955,1,1,2,3740),-- Scroll: Enchant Weapon (Grade D)
(18265,956,1,1,2,31162),-- Scroll: Enchant Armor (Grade D)
(18266,57,483,891,0,700000),-- Adena
(18266,955,1,1,2,3855),-- Scroll: Enchant Weapon (Grade D)
(18266,956,1,1,2,32122),-- Scroll: Enchant Armor (Grade D)
(18267,57,484,932,0,700000),-- Adena
(18267,955,1,1,2,3905),-- Scroll: Enchant Weapon (Grade D)
(18267,956,1,1,2,32541),-- Scroll: Enchant Armor (Grade D)
(18268,57,708,1417,0,700000),-- Adena
(18268,955,1,1,2,5804),-- Scroll: Enchant Weapon (Grade D)
(18268,956,1,1,2,48367),-- Scroll: Enchant Armor (Grade D)
(18269,57,732,1465,0,700000),-- Adena
(18269,955,1,1,2,5940),-- Scroll: Enchant Weapon (Grade D)
(18269,956,1,1,2,49492),-- Scroll: Enchant Armor (Grade D)
(18270,57,758,1520,0,700000),-- Adena
(18270,955,1,1,2,6163),-- Scroll: Enchant Weapon (Grade D)
(18270,956,1,1,2,51357),-- Scroll: Enchant Armor (Grade D)
(18271,57,789,1572,0,700000),-- Adena
(18271,955,1,1,2,6338),-- Scroll: Enchant Weapon (Grade D)
(18271,956,1,1,2,52816),-- Scroll: Enchant Armor (Grade D)
(18272,57,1257,2519,0,700000),-- Adena
(18272,951,1,1,2,4563),-- Scroll: Enchant Weapon (Grade C)
(18272,952,1,1,2,33462),-- Scroll: Enchant Armor (Grade C)
(18273,57,1293,2582,0,700000),-- Adena
(18273,951,1,1,2,4604),-- Scroll: Enchant Weapon (Grade C)
(18273,952,1,1,2,33756),-- Scroll: Enchant Armor (Grade C)
(18274,57,1330,2656,0,700000),-- Adena
(18274,951,1,1,2,4652),-- Scroll: Enchant Weapon (Grade C)
(18274,952,1,1,2,34111),-- Scroll: Enchant Armor (Grade C)
(18275,57,1952,3909,0,700000),-- Adena
(18275,951,1,1,2,6718),-- Scroll: Enchant Weapon (Grade C)
(18275,952,1,1,2,49264),-- Scroll: Enchant Armor (Grade C)
(18276,57,1997,3995,0,700000),-- Adena
(18276,947,1,1,2,1468),-- Scroll: Enchant Weapon (Grade B)
(18276,948,1,1,2,9176),-- Scroll: Enchant Armor (Grade B)
(18277,57,2049,4095,0,700000),-- Adena
(18277,947,1,1,2,1433),-- Scroll: Enchant Weapon (Grade B)
(18277,948,1,1,2,8957),-- Scroll: Enchant Armor (Grade B)
(18278,57,2683,5361,0,700000),-- Adena
(18278,947,1,1,2,1877),-- Scroll: Enchant Weapon (Grade B)
(18278,948,1,1,2,11732),-- Scroll: Enchant Armor (Grade B)
(18279,57,2736,5473,0,700000),-- Adena
(18279,947,1,1,2,1916),-- Scroll: Enchant Weapon (Grade B)
(18279,948,1,1,2,11969),-- Scroll: Enchant Armor (Grade B)
(18280,57,2790,5576,0,700000),-- Adena
(18280,729,1,1,2,543),-- Scroll: Enchant Weapon (Grade A)
(18280,730,1,1,2,4067),-- Scroll: Enchant Armor (Grade A)
(18281,57,2849,5701,0,700000),-- Adena
(18281,729,1,1,2,555),-- Scroll: Enchant Weapon (Grade A)
(18281,730,1,1,2,4157),-- Scroll: Enchant Armor (Grade A)
(18282,57,4009,8028,0,700000),-- Adena
(18282,729,1,1,2,781),-- Scroll: Enchant Weapon (Grade A)
(18282,730,1,1,2,5854),-- Scroll: Enchant Armor (Grade A)
(18283,57,4078,8168,0,700000),-- Adena
(18283,959,1,1,2,286),-- Scroll: Enchant Weapon (Grade S)
(18283,960,1,1,2,2860),-- Scroll: Enchant Armor (Grade S)
(18284,57,4146,8295,0,700000),-- Adena
(18284,959,1,1,2,291),-- Scroll: Enchant Weapon (Grade S)
(18284,960,1,1,2,2904),-- Scroll: Enchant Armor (Grade S)
(18285,57,5312,10638,0,700000),-- Adena
(18285,959,1,1,2,373),-- Scroll: Enchant Weapon (Grade S)
(18285,960,1,1,2,3724),-- Scroll: Enchant Armor (Grade S)
(18286,57,5393,10798,0,700000),-- Adena
(18286,959,1,1,2,378),-- Scroll: Enchant Weapon (Grade S)
(18286,960,1,1,2,3780),-- Scroll: Enchant Armor (Grade S)
(18287,57,186,372,0,700000),-- Adena
(18287,955,1,1,2,1508),-- Scroll: Enchant Weapon (Grade D)
(18287,956,1,1,2,12560),-- Scroll: Enchant Armor (Grade D)
(18288,57,207,413,0,700000),-- Adena
(18288,955,1,1,2,1680),-- Scroll: Enchant Weapon (Grade D)
(18288,956,1,1,2,13997),-- Scroll: Enchant Armor (Grade D)
(18289,57,308,617,0,700000),-- Adena
(18289,951,1,1,2,1119),-- Scroll: Enchant Weapon (Grade C)
(18289,952,1,1,2,8205),-- Scroll: Enchant Armor (Grade C)
(18290,57,338,676,0,700000),-- Adena
(18290,951,1,1,2,1205),-- Scroll: Enchant Weapon (Grade C)
(18290,952,1,1,2,8833),-- Scroll: Enchant Armor (Grade C)
(18291,57,472,944,0,700000),-- Adena
(18291,947,1,1,2,347),-- Scroll: Enchant Weapon (Grade B)
(18291,948,1,1,2,2168),-- Scroll: Enchant Armor (Grade B)
(18292,57,519,1010,0,700000),-- Adena
(18292,947,1,1,2,375),-- Scroll: Enchant Weapon (Grade B)
(18292,948,1,1,2,2342),-- Scroll: Enchant Armor (Grade B)
(18293,57,681,1361,0,700000),-- Adena
(18293,729,1,1,2,133),-- Scroll: Enchant Weapon (Grade A)
(18293,730,1,1,2,993),-- Scroll: Enchant Armor (Grade A)
(18294,57,728,1457,0,700000),-- Adena
(18294,729,1,1,2,142),-- Scroll: Enchant Weapon (Grade A)
(18294,730,1,1,2,1063),-- Scroll: Enchant Armor (Grade A)
(18295,57,938,1879,0,700000),-- Adena
(18295,729,1,1,2,183),-- Scroll: Enchant Weapon (Grade A)
(18295,730,1,1,2,1370),-- Scroll: Enchant Armor (Grade A)
(18296,57,995,1993,0,700000),-- Adena
(18296,959,1,1,2,70),-- Scroll: Enchant Weapon (Grade S)
(18296,960,1,1,2,698),-- Scroll: Enchant Armor (Grade S)
(18297,57,1239,2481,0,700000),-- Adena
(18297,959,1,1,2,87),-- Scroll: Enchant Weapon (Grade S)
(18297,960,1,1,2,869),-- Scroll: Enchant Armor (Grade S)
(18298,57,1301,2604,0,700000),-- Adena
(18298,959,1,1,2,92),-- Scroll: Enchant Weapon (Grade S)
(18298,960,1,1,2,912),-- Scroll: Enchant Armor (Grade S)
(18319,6529,1,1,1,1159),-- Baby Duck Rod
(18319,6519,1,1,2,86953),-- Green Colored Lure - Low Grade
(18319,6522,1,1,2,57969),-- Purple Colored Lure - Low Grade
(18319,6525,1,1,2,72461),-- Yellow Colored Lure - Low Grade
(18319,6535,12,36,2,120768),-- Fishing Shot: non-grade
(18320,6530,1,1,1,527),-- Albatross Rod
(18320,6519,1,1,2,131712),-- Green Colored Lure - Low Grade
(18320,6522,1,1,2,87808),-- Purple Colored Lure - Low Grade
(18320,6525,1,1,2,109760),-- Yellow Colored Lure - Low Grade
(18320,6536,12,36,2,109760),-- Fishing Shot: D-grade
(18321,6531,1,1,1,200),-- Pelican Rod
(18321,6520,2,6,2,33251),-- Green Colored Lure 
(18321,6523,1,1,2,110836),-- Purple Colored Lure
(18321,6526,1,1,2,133003),-- Yellow Colored Lure
(18321,6537,12,36,2,118753),-- Fishing Shot: C-grade
(18322,6532,1,1,1,108),-- KingFisher Rod
(18322,6520,2,6,2,67551),-- Green Colored Lure 
(18322,6523,1,1,2,180134),-- Purple Colored Lure
(18322,6526,3,9,2,36027),-- Yellow Colored Lure
(18322,6538,15,45,2,43232),-- Fishing Shot: B-grade
(18323,6533,1,1,1,41),-- Cygnus Pole
(18323,6521,2,6,2,67649),-- Green Colored Lure - High Grade
(18323,6524,1,1,2,202945),-- Purple Colored Lure - High Grade
(18323,6527,2,6,2,45099),-- Yellow Colored Lure - High Grade
(18323,6539,3,9,2,169121),-- Fishing Shot: A-grade
(18324,6533,1,1,1,61),-- Cygnus Pole
(18324,6521,2,6,2,81602),-- Green Colored Lure - High Grade
(18324,6524,1,1,2,244805),-- Purple Colored Lure - High Grade
(18324,6527,2,6,2,54401),-- Yellow Colored Lure - High Grade
(18324,6539,3,9,2,204004),-- Fishing Shot: A-grade
(18325,6534,1,1,1,17),-- Triton Pole
(18325,6521,2,6,2,70774),-- Green Colored Lure - High Grade
(18325,6524,1,1,2,212320),-- Purple Colored Lure - High Grade
(18325,6527,2,6,2,47183),-- Yellow Colored Lure - High Grade
(18325,6540,3,9,2,141547),-- Fishing Shot: S-grade
(18326,6534,1,1,1,30),-- Triton Pole
(18326,6521,2,6,2,85774),-- Green Colored Lure - High Grade
(18326,6524,1,1,2,257320),-- Purple Colored Lure - High Grade
(18326,6527,2,6,2,57183),-- Yellow Colored Lure - High Grade
(18326,6540,3,9,2,171547),-- Fishing Shot: S-grade
(18345,8787,1,1,2,250000),-- Sprigant\'s Fruit
(18346,8787,1,1,2,250000),-- Sprigant\'s Fruit
(18485,9691,1,1,2,1000000),-- Water Dragon Scale
(18485,9700,1,1,3,360000),-- Water Dragon Claw
(20003,116,1,1,-1,343131),-- Magic Ring
(20003,1786,1,1,-1,45293),-- Recipe: Broad Sword
(20003,1871,1,1,-1,56617),-- Charcoal
(20003,57,30,42,0,700000),-- Adena
(20003,112,1,1,1,79637),-- Apprentice\'s Earring
(20003,116,1,1,1,118249),-- Magic Ring
(20003,118,1,1,1,59125),-- Necklace of Magic
(20003,1788,1,1,2,7805),-- Recipe: Bow 
(20003,1864,1,1,2,19512),-- Stem
(20003,1865,1,1,2,9756),-- Varnish
(20003,1866,1,1,2,6504),-- Suede
(20003,1868,1,1,2,19512),-- Thread
(20003,1873,1,1,2,3903),-- Silver Nugget
(20004,1866,1,1,-1,51827),-- Suede
(20004,2005,1,1,-1,5553),-- Broadsword Blade
(20004,2006,1,1,-1,5553),-- Willow Staff Head
(20004,57,41,58,0,700000),-- Adena
(20004,1119,1,1,1,5045),-- Short Leather Gloves
(20004,1122,1,1,1,5044),-- Cotton Shoes
(20004,1129,1,1,1,5044),-- Crude Leather Shoes
(20004,1666,1,1,2,5104),-- Recipe: Wooden Arrow
(20004,1866,1,1,2,10206),-- Suede
(20004,1871,1,1,2,15309),-- Charcoal
(20005,1060,1,1,-1,216259),-- Lesser Healing Potion
(20005,1666,1,1,-1,32439),-- Recipe: Wooden Arrow
(20005,1866,1,1,-1,64878),-- Suede
(20005,57,51,72,0,700000),-- Adena
(20005,19,1,1,1,9012),-- Small Shield
(20005,42,1,1,1,6311),-- Leather Cap
(20005,17,10,30,2,71865),-- Wooden Arrow
(20005,1789,1,1,2,2657),-- Recipe: Cedar Staff
(20005,1865,1,1,2,14373),-- Varnish
(20005,1870,1,1,2,14373),-- Coal
(20006,113,1,1,-1,31655),-- Mystic\'s Earring
(20006,1060,1,1,-1,285245),-- Lesser Healing Potion
(20006,2006,1,1,-1,9169),-- Willow Staff Head
(20006,57,69,94,0,700000),-- Adena
(20006,14,1,1,1,607),-- Bow
(20006,2007,1,1,1,2709),-- Bow Shaft
(20006,1866,1,1,2,16852),-- Suede
(20006,1871,1,1,2,25278),-- Charcoal
(20006,2136,1,1,2,7435),-- Recipe: Cokes
(20007,1122,1,1,-1,49042),-- Cotton Shoes
(20007,1866,1,1,-1,99228),-- Suede
(20007,1875,1,1,-1,9923),-- Stone of Purity
(20007,57,80,108,0,700000),-- Adena
(20007,22,1,1,1,3619),-- Leather Shirt
(20007,29,1,1,1,5786),-- Leather Pants
(20007,1796,1,1,2,36869),-- Recipe: Leather Tunic
(20007,1864,1,1,2,58621),-- Stem
(20007,1869,1,1,2,29311),-- Iron Ore
(20008,1793,1,1,-1,16388),-- Recipe: Sword of Reflection
(20008,1870,1,1,-1,222882),-- Coal
(20008,1871,1,1,-1,222882),-- Charcoal
(20008,57,114,168,0,700000),-- Adena
(20008,255,1,1,1,244),-- Fox Claw Gloves
(20008,2371,1,1,1,517),-- Fist of Butcher
(20008,1792,1,1,2,3228),-- Recipe: Short Spear
(20008,1865,1,1,2,43891),-- Varnish
(20008,1870,1,1,2,43891),-- Coal
(20009,43,1,1,-1,11200),-- Wooden Helmet
(20009,1831,1,1,-1,594351),-- Antidote
(20009,1870,1,1,-1,222882),-- Coal
(20009,57,114,168,0,700000),-- Adena
(20009,255,1,1,1,244),-- Fox Claw Gloves
(20009,2371,1,1,1,517),-- Fist of Butcher
(20009,736,1,1,2,21946),-- Scroll of Escape
(20009,1866,1,1,2,29261),-- Suede
(20009,1871,1,1,2,43891),-- Charcoal
(20010,1792,1,1,-1,17030),-- Recipe: Short Spear
(20010,1833,1,1,-1,617635),-- Bandage
(20010,1896,1,1,-1,98141),-- Necklace of Anguish Chain
(20010,57,116,177,0,700000),-- Adena
(20010,271,1,1,1,506),-- Hunting Bow
(20010,1798,1,1,2,44716),-- Recipe: Leather Helmet
(20010,1866,1,1,2,30407),-- Suede
(20010,1871,1,1,2,45610),-- Charcoal
(20011,1792,1,1,-1,17541),-- Recipe: Short Spear
(20011,1867,1,1,-1,318082),-- Animal Skin
(20011,1921,1,1,-1,153911),-- Leather Shoes Texture
(20011,57,120,182,0,700000),-- Adena
(20011,271,1,1,1,521),-- Hunting Bow
(20011,1799,1,1,2,69598),-- Recipe: Leather Gloves
(20011,1867,1,1,2,62638),-- Animal Skin
(20011,1872,1,1,2,62638),-- Animal Bone
(20012,734,1,1,-1,40386),-- Haste Potion
(20012,1865,1,2,-1,242317),-- Varnish
(20012,1897,1,1,-1,57694),-- Necklace of Wisdom Chain
(20012,57,120,187,0,700000),-- Adena
(20012,1101,1,1,1,706),-- Tunic of Devotion
(20012,1104,1,1,1,1127),-- Stockings of Devotion
(20012,1800,1,1,2,11255),-- Recipe: Piece Bone Breastplate
(20012,1865,1,1,2,35789),-- Varnish
(20012,1870,1,1,2,35789),-- Coal
(20012,6035,1,1,2,2983),-- Magic Haste Potion
(20013,1866,1,2,-1,143423),-- Suede
(20013,1896,1,1,-1,91159),-- Necklace of Anguish Chain
(20013,1922,1,1,-1,54881),-- Leather Tunic Pattern
(20013,57,113,160,0,700000),-- Adena
(20013,9,1,1,1,157),-- Cedar Staff
(20013,154,1,1,1,157),-- Dwarven Mace
(20013,2008,1,1,1,1649),-- Cedar Staff Head
(20013,1792,1,1,2,3115),-- Recipe: Short Spear
(20013,1865,1,1,2,42366),-- Varnish
(20013,1870,1,1,2,42366),-- Coal
(20014,1867,1,2,-1,328104),-- Animal Skin
(20014,2254,1,1,-1,10085),-- Recipe: Composition Bow
(20014,57,120,193,0,700000),-- Adena
(20014,2,1,1,1,54),-- Long Sword
(20014,120,1,1,1,54),-- Sword of Reflection
(20014,121,1,1,1,54),-- Sword of Watershadow
(20014,2012,1,1,1,758),-- Sword of Reflection Blade
(20014,1800,1,1,2,15239),-- Recipe: Piece Bone Breastplate
(20014,1867,1,1,2,64612),-- Animal Skin
(20014,1872,1,1,2,64612),-- Animal Bone
(20015,1798,1,1,-1,237113),-- Recipe: Leather Helmet
(20015,1864,1,2,-1,483710),-- Stem
(20015,1871,1,1,-1,241855),-- Charcoal
(20015,57,127,180,0,700000),-- Adena
(20015,9,1,1,1,177),-- Cedar Staff
(20015,154,1,1,1,177),-- Dwarven Mace
(20015,2008,1,1,1,1853),-- Cedar Staff Head
(20015,1798,1,1,2,46693),-- Recipe: Leather Helmet
(20015,1864,1,1,2,95254),-- Stem
(20015,1869,1,1,2,47627),-- Iron Ore
(20016,37,1,1,-1,18253),-- Leather Shoes
(20016,1833,1,1,-1,644946),-- Bandage
(20016,1865,1,2,-1,241855),-- Varnish
(20016,57,127,180,0,700000),-- Adena
(20016,271,1,1,1,528),-- Hunting Bow
(20016,736,1,1,2,23814),-- Scroll of Escape
(20016,1866,1,1,2,31752),-- Suede
(20016,1871,1,1,2,47627),-- Charcoal
(20017,1870,1,2,-1,251000),-- Coal
(20017,1897,1,1,-1,59762),-- Necklace of Wisdom Chain
(20017,57,122,197,0,700000),-- Adena
(20017,2,1,1,1,55),-- Long Sword
(20017,120,1,1,1,55),-- Sword of Reflection
(20017,121,1,1,1,55),-- Sword of Watershadow
(20017,2012,1,1,1,773),-- Sword of Reflection Blade
(20017,1817,1,1,2,7415),-- Recipe: Cord
(20017,1865,1,1,2,37071),-- Varnish
(20017,1870,1,1,2,37071),-- Coal
(20017,5268,1,1,2,1483),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20019,1792,1,1,-1,16388),-- Recipe: Short Spear
(20019,1864,1,1,-1,445764),-- Stem
(20019,1923,1,1,-1,91345),-- Leather Stocking Pattern
(20019,57,114,168,0,700000),-- Adena
(20019,429,1,1,1,828),-- Leather Tunic
(20019,464,1,1,1,1325),-- Leather Stockings
(20019,1922,1,1,1,8398),-- Leather Tunic Pattern
(20019,1923,1,1,1,13491),-- Leather Stocking Pattern
(20019,1793,1,1,2,3228),-- Recipe: Sword of Reflection
(20019,1866,1,1,2,29261),-- Suede
(20019,1871,1,1,2,43891),-- Charcoal
(20020,1799,1,1,-1,371205),-- Recipe: Leather Gloves
(20020,1831,1,1,-1,668170),-- Antidote
(20020,1896,1,1,-1,106171),-- Necklace of Anguish Chain
(20020,57,128,189,0,700000),-- Adena
(20020,37,1,1,1,3724),-- Leather Shoes
(20020,49,1,1,1,3724),-- Gloves
(20020,1921,1,1,1,31834),-- Leather Shoes Texture
(20020,1794,1,1,2,3628),-- Recipe: Forest Bow
(20020,1865,1,1,2,49342),-- Varnish
(20020,1870,1,1,2,49342),-- Coal
(20021,1792,1,1,-1,16388),-- Recipe: Short Spear
(20021,1793,1,1,-1,16388),-- Recipe: Sword of Reflection
(20021,1921,1,1,-1,143795),-- Leather Shoes Texture
(20021,57,114,168,0,700000),-- Adena
(20021,9,1,1,1,163),-- Cedar Staff
(20021,154,1,1,1,163),-- Dwarven Mace
(20021,2008,1,1,1,1708),-- Cedar Staff Head
(20021,1060,1,1,2,73151),-- Lesser Healing Potion
(20021,1798,1,1,2,32273),-- Recipe: Leather Helmet
(20021,1865,1,1,2,32918),-- Varnish
(20021,1870,1,1,2,32918),-- Coal
(20022,1831,1,1,-1,505199),-- Antidote
(20022,1872,1,1,-1,252599),-- Animal Bone
(20022,1921,1,1,-1,122225),-- Leather Shoes Texture
(20022,57,97,143,0,700000),-- Adena
(20022,20,1,1,1,4026),-- Buckler
(20022,43,1,1,1,2813),-- Wooden Helmet
(20022,1799,1,1,2,55270),-- Recipe: Leather Gloves
(20022,1864,1,1,2,74615),-- Stem
(20022,1869,1,1,2,37307),-- Iron Ore
(20023,1831,1,1,-1,617635),-- Antidote
(20023,1868,1,1,-1,463226),-- Thread
(20023,1922,1,1,-1,59085),-- Leather Tunic Pattern
(20023,57,116,177,0,700000),-- Adena
(20023,429,1,1,1,860),-- Leather Tunic
(20023,464,1,1,1,1377),-- Leather Stockings
(20023,1922,1,1,1,8727),-- Leather Tunic Pattern
(20023,1923,1,1,1,14020),-- Leather Stocking Pattern
(20023,735,1,1,2,2851),-- Potion of Alacrity
(20023,1803,1,1,2,28746),-- Recipe: Necklace of Wisdom
(20023,1868,1,1,2,68415),-- Thread
(20023,1873,1,1,2,13683),-- Silver Nugget
(20024,734,1,1,-1,41833),-- Haste Potion
(20024,1864,1,1,-1,501999),-- Stem
(20024,1897,1,1,-1,59762),-- Necklace of Wisdom Chain
(20024,57,122,197,0,700000),-- Adena
(20024,2,1,1,1,55),-- Long Sword
(20024,120,1,1,1,55),-- Sword of Reflection
(20024,121,1,1,1,55),-- Sword of Watershadow
(20024,2012,1,1,1,773),-- Sword of Reflection Blade
(20024,1801,1,1,2,24901),-- Recipe: Piece Bone Gaiters
(20024,1864,1,1,2,98856),-- Stem
(20024,1869,1,1,2,49428),-- Iron Ore
(20025,1799,1,1,-1,349993),-- Recipe: Leather Gloves
(20025,1866,1,2,-1,157497),-- Suede
(20025,1869,1,2,-1,236245),-- Iron Ore
(20025,57,119,180,0,700000),-- Adena
(20025,429,1,1,1,877),-- Leather Tunic
(20025,464,1,1,1,1405),-- Leather Stockings
(20025,1922,1,1,1,8902),-- Leather Tunic Pattern
(20025,1923,1,1,1,14301),-- Leather Stocking Pattern
(20025,736,1,1,2,23262),-- Scroll of Escape
(20025,1866,1,1,2,31015),-- Suede
(20025,1871,1,1,2,46523),-- Charcoal
(20026,1865,1,1,-1,260380),-- Varnish
(20026,1866,1,1,-1,173586),-- Suede
(20026,1923,1,1,-1,106713),-- Leather Stocking Pattern
(20026,57,131,199,0,700000),-- Adena
(20026,37,1,1,1,3870),-- Leather Shoes
(20026,49,1,1,1,3870),-- Gloves
(20026,1921,1,1,1,33081),-- Leather Shoes Texture
(20026,1792,1,1,2,3771),-- Recipe: Short Spear
(20026,1865,1,1,2,51275),-- Varnish
(20026,1870,1,1,2,51275),-- Coal
(20027,1800,1,1,-1,74706),-- Recipe: Piece Bone Breastplate
(20027,1801,1,1,-1,119680),-- Recipe: Piece Bone Gaiters
(20027,1924,1,1,-1,76634),-- Leather Helmet Design
(20027,57,118,183,0,700000),-- Adena
(20027,176,1,1,1,103),-- Journeyman\'s Staff
(20027,2501,1,1,1,103),-- Bone Club
(20027,1800,1,1,2,11034),-- Recipe: Piece Bone Breastplate
(20027,1865,1,1,2,35087),-- Varnish
(20027,1870,1,1,2,35087),-- Coal
(20027,5158,1,1,2,1404),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20028,734,1,1,-1,39594),-- Haste Potion
(20028,1801,1,1,-1,119680),-- Recipe: Piece Bone Gaiters
(20028,1868,1,2,-1,475131),-- Thread
(20028,57,118,183,0,700000),-- Adena
(20028,845,1,1,1,790),-- Cat\'s Eye Earring
(20028,877,1,1,1,1186),-- Ring of Wisdom
(20028,908,1,1,1,590),-- Necklace of Wisdom
(20028,1897,1,1,1,8354),-- Necklace of Wisdom Chain
(20028,1817,1,1,2,9357),-- Recipe: Cord
(20028,1864,1,1,2,93565),-- Stem
(20028,1869,1,1,2,46783),-- Iron Ore
(20029,1864,1,2,-1,534142),-- Stem
(20029,1897,1,1,-1,63588),-- Necklace of Wisdom Chain
(20029,2013,1,1,-1,5764),-- Forest Bow Shaft
(20029,57,132,206,0,700000),-- Adena
(20029,15,1,1,1,117),-- Short Spear
(20029,2011,1,1,1,1800),-- Short Spear Edge
(20029,1865,1,1,2,39445),-- Varnish
(20029,1870,1,1,2,39445),-- Coal
(20029,2139,1,1,2,7889),-- Recipe: Steel Mold
(20029,5268,1,1,2,1578),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20030,1794,1,1,-1,17030),-- Recipe: Forest Bow
(20030,1798,1,1,-1,227072),-- Recipe: Leather Helmet
(20030,1831,1,1,-1,617635),-- Antidote
(20030,57,116,177,0,700000),-- Adena
(20030,216,1,1,1,253),-- Dirk
(20030,2009,1,1,1,2427),-- Dirk Blade
(20030,1793,1,1,2,3354),-- Recipe: Sword of Reflection
(20030,1865,1,1,2,45611),-- Varnish
(20030,1870,1,1,2,45611),-- Coal
(20031,24,1,1,-1,2020),-- Bone Breastplate
(20031,1865,1,1,-1,205065),-- Varnish
(20031,1924,1,1,-1,66150),-- Leather Helmet Design
(20031,57,100,161,0,700000),-- Adena
(20031,24,1,1,1,299),-- Bone Breastplate
(20031,31,1,1,1,478),-- Bone Gaiters
(20031,390,1,1,1,299),-- Cotton Shirt
(20031,412,1,1,1,478),-- Cotton Pants
(20031,1865,1,1,2,30287),-- Varnish
(20031,1870,1,1,2,30287),-- Coal
(20031,2140,1,1,2,6058),-- Recipe: High Grade Suede
(20031,5273,1,1,2,1212),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20032,734,1,1,-1,41013),-- Haste Potion
(20032,1800,1,1,-1,77383),-- Recipe: Piece Bone Breastplate
(20032,1868,1,2,-1,492156),-- Thread
(20032,57,120,193,0,700000),-- Adena
(20032,24,1,1,1,359),-- Bone Breastplate
(20032,31,1,1,1,573),-- Bone Gaiters
(20032,390,1,1,1,359),-- Cotton Shirt
(20032,412,1,1,1,573),-- Cotton Pants
(20032,1865,1,1,2,48459),-- Varnish
(20032,1870,1,1,2,48459),-- Coal
(20032,2141,1,1,2,9692),-- Recipe: Silver Mold
(20033,1924,1,1,-1,66150),-- Leather Helmet Design
(20033,1925,1,1,-1,134469),-- Leather Gloves Lining
(20033,2139,1,1,-1,41013),-- Recipe: Steel Mold
(20033,57,100,161,0,700000),-- Adena
(20033,845,1,1,1,682),-- Cat\'s Eye Earring
(20033,877,1,1,1,1024),-- Ring of Wisdom
(20033,908,1,1,1,510),-- Necklace of Wisdom
(20033,1897,1,1,1,7212),-- Necklace of Wisdom Chain
(20033,1865,1,1,2,30287),-- Varnish
(20033,1870,1,1,2,30287),-- Coal
(20033,2142,1,1,2,6058),-- Recipe: Varnish of Purity
(20033,5158,1,1,2,1212),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20034,1868,1,2,-1,501999),-- Thread
(20034,1873,1,1,-1,100400),-- Silver Nugget
(20034,2254,1,1,-1,10287),-- Recipe: Composition Bow
(20034,57,122,197,0,700000),-- Adena
(20034,256,1,1,1,218),-- Cestus
(20034,1868,1,1,2,74142),-- Thread
(20034,1873,1,1,2,14829),-- Silver Nugget
(20034,2143,1,1,2,7415),-- Recipe: Synthetic Cokes
(20034,6035,1,1,2,3090),-- Magic Haste Potion
(20035,1868,1,2,-1,492156),-- Thread
(20035,1925,1,1,-1,161363),-- Leather Gloves Lining
(20035,2140,1,1,-1,49216),-- Recipe: High Grade Suede
(20035,57,120,193,0,700000),-- Adena
(20035,38,1,1,1,1432),-- Low Boots
(20035,50,1,1,1,1432),-- Leather Gloves
(20035,1925,1,1,1,31776),-- Leather Gloves Lining
(20035,1817,1,1,2,7269),-- Recipe: Cord
(20035,1868,1,1,2,72688),-- Thread
(20035,1873,1,1,2,14538),-- Silver Nugget
(20035,5268,1,1,2,1454),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20036,735,1,1,-1,21527),-- Potion of Alacrity
(20036,1873,1,1,-1,103328),-- Silver Nugget
(20036,2175,1,1,-1,210017),-- Recipe: Boots
(20036,57,124,204,0,700000),-- Adena
(20036,845,1,1,1,859),-- Cat\'s Eye Earring
(20036,877,1,1,1,1289),-- Ring of Wisdom
(20036,908,1,1,1,642),-- Necklace of Wisdom
(20036,1897,1,1,1,9084),-- Necklace of Wisdom Chain
(20036,1864,1,1,2,76304),-- Stem
(20036,1869,1,1,2,38152),-- Iron Ore
(20036,1878,1,1,2,15261),-- Braided Hemp
(20036,2173,1,1,2,10340),-- Recipe: Hard Leather Shirt
(20038,1868,1,4,-1,557163),-- Thread
(20038,2011,1,1,-1,6355),-- Short Spear Edge
(20038,2174,1,1,-1,121122),-- Recipe: Hard Leather Gaiters
(20038,57,133,220,0,700000),-- Adena
(20038,217,1,1,1,121),-- Shining Knife
(20038,218,1,1,1,121),-- Throwing Knife
(20038,1831,1,1,2,54859),-- Antidote
(20038,1867,1,1,2,27430),-- Animal Skin
(20038,1869,1,1,2,20573),-- Iron Ore
(20038,1870,1,1,2,20573),-- Coal
(20038,1871,1,1,2,20573),-- Charcoal
(20038,1872,1,1,2,27430),-- Animal Bone
(20038,2174,1,1,2,8945),-- Recipe: Hard Leather Gaiters
(20038,5273,1,1,2,823),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20039,1801,1,1,-1,127585),-- Recipe: Piece Bone Gaiters
(20039,1872,1,2,-1,337674),-- Animal Bone
(20039,2011,1,1,-1,5777),-- Short Spear Edge
(20039,57,121,200,0,700000),-- Adena
(20039,44,1,1,1,734),-- Leather Helmet
(20039,102,1,1,1,1053),-- Round Shield
(20039,945,1,1,1,2175),-- Skeleton Buckler
(20039,1924,1,1,1,12066),-- Leather Helmet Design
(20039,735,1,1,2,1782),-- Potion of Alacrity
(20039,1864,1,1,2,42748),-- Stem
(20039,1865,1,1,2,21374),-- Varnish
(20039,1866,1,1,2,14250),-- Suede
(20039,1868,1,1,2,42748),-- Thread
(20039,1873,1,1,2,8550),-- Silver Nugget
(20039,2175,1,1,2,17377),-- Recipe: Boots
(20040,2012,1,1,-1,4397),-- Sword of Reflection Blade
(20040,2140,1,1,-1,42209),-- Recipe: High Grade Suede
(20040,2175,1,1,-1,171583),-- Recipe: Boots
(20040,57,101,167,0,700000),-- Adena
(20040,44,1,1,1,612),-- Leather Helmet
(20040,102,1,1,1,877),-- Round Shield
(20040,945,1,1,1,1813),-- Skeleton Buckler
(20040,1924,1,1,1,10055),-- Leather Helmet Design
(20040,1868,1,1,2,62340),-- Thread
(20040,1873,1,1,2,12468),-- Silver Nugget
(20040,2252,1,1,2,1278),-- Recipe: Iron Hammer
(20040,5158,1,1,2,1247),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20041,1817,1,1,-1,47452),-- Recipe: Cord
(20041,1880,1,1,-1,23726),-- Steel
(20041,2012,1,1,-1,4943),-- Sword of Reflection Blade
(20041,57,114,187,0,700000),-- Adena
(20041,217,1,1,1,103),-- Shining Knife
(20041,218,1,1,1,103),-- Throwing Knife
(20041,1866,1,1,2,31148),-- Suede
(20041,1871,1,1,2,46722),-- Charcoal
(20041,2253,1,1,2,1915),-- Recipe: Sword Breaker
(20042,734,1,1,-1,43053),-- Haste Potion
(20042,2013,1,1,-1,5575),-- Forest Bow Shaft
(20042,57,124,204,0,700000),-- Adena
(20042,309,1,1,1,112),-- Tears of Eva
(20042,310,1,1,1,112),-- Relic of The Saints
(20042,1867,1,1,2,40696),-- Animal Skin
(20042,1872,1,1,2,40696),-- Animal Bone
(20042,1882,1,1,2,6783),-- Leather
(20042,2254,1,1,2,1251),-- Recipe: Composition Bow
(20042,5268,1,1,2,1221),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20043,1817,1,1,-1,58608),-- Recipe: Cord
(20043,1864,1,2,-1,586083),-- Stem
(20043,1924,1,1,-1,94529),-- Leather Helmet Design
(20043,57,137,234,0,700000),-- Adena
(20043,2,1,1,1,64),-- Long Sword
(20043,120,1,1,1,64),-- Sword of Reflection
(20043,121,1,1,1,64),-- Sword of Watershadow
(20043,2012,1,1,1,902),-- Sword of Reflection Blade
(20043,1831,1,1,2,57707),-- Antidote
(20043,1864,1,1,2,43280),-- Stem
(20043,1865,1,1,2,21640),-- Varnish
(20043,1866,1,1,2,14427),-- Suede
(20043,1868,1,1,2,43280),-- Thread
(20043,1873,1,1,2,8657),-- Silver Nugget
(20043,2143,1,1,2,4328),-- Recipe: Synthetic Cokes
(20043,5158,1,1,2,866),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20044,1865,1,1,-1,266401),-- Varnish
(20044,1872,1,1,-1,355202),-- Animal Bone
(20044,2142,1,1,-1,53280),-- Recipe: Varnish of Purity
(20044,57,125,213,0,700000),-- Adena
(20044,309,1,1,1,116),-- Tears of Eva
(20044,310,1,1,1,116),-- Relic of The Saints
(20044,1867,1,1,2,52461),-- Animal Skin
(20044,1872,1,1,2,52461),-- Animal Bone
(20044,1881,1,1,2,5246),-- Coarse Bone Powder
(20044,2144,1,1,2,7870),-- Recipe: Compound Braid
(20045,734,1,1,-1,44835),-- Haste Potion
(20045,1925,1,1,-1,176402),-- Leather Gloves Lining
(20045,57,126,215,0,700000),-- Adena
(20045,217,1,1,1,117),-- Shining Knife
(20045,218,1,1,1,117),-- Throwing Knife
(20045,1866,1,1,2,26488),-- Suede
(20045,1871,1,1,2,39732),-- Charcoal
(20045,2150,1,1,2,18654),-- Recipe: Blue Diamond Necklace
(20045,5268,1,1,2,1590),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20046,735,1,1,-1,20390),-- Potion of Alacrity
(20046,1897,1,1,-1,58257),-- Necklace of Wisdom Chain
(20046,2139,1,1,-1,48936),-- Recipe: Steel Mold
(20046,57,115,195,0,700000),-- Adena
(20046,845,1,1,1,813),-- Cat\'s Eye Earring
(20046,877,1,1,1,1221),-- Ring of Wisdom
(20046,908,1,1,1,608),-- Necklace of Wisdom
(20046,1897,1,1,1,8604),-- Necklace of Wisdom Chain
(20046,1867,1,1,2,48184),-- Animal Skin
(20046,1872,1,1,2,48184),-- Animal Bone
(20046,1882,1,1,2,8031),-- Leather
(20046,2173,1,1,2,9794),-- Recipe: Hard Leather Shirt
(20047,38,1,1,-1,6558),-- Low Boots
(20047,1869,1,2,-1,222001),-- Iron Ore
(20047,2139,1,1,-1,44400),-- Recipe: Steel Mold
(20047,57,104,177,0,700000),-- Adena
(20047,24,1,1,1,324),-- Bone Breastplate
(20047,31,1,1,1,517),-- Bone Gaiters
(20047,390,1,1,1,324),-- Cotton Shirt
(20047,412,1,1,1,517),-- Cotton Pants
(20047,734,1,1,2,3644),-- Haste Potion
(20047,1864,1,1,2,43717),-- Stem
(20047,1869,1,1,2,21859),-- Iron Ore
(20047,1878,1,1,2,8744),-- Braided Hemp
(20047,2174,1,1,2,9504),-- Recipe: Hard Leather Gaiters
(20047,5273,1,1,2,875),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20048,50,1,1,-1,6785),-- Leather Gloves
(20048,1924,1,1,-1,74091),-- Leather Helmet Design
(20048,2144,1,1,-1,45937),-- Recipe: Compound Braid
(20048,57,106,184,0,700000),-- Adena
(20048,256,1,1,1,200),-- Cestus
(20048,1868,1,1,2,90461),-- Thread
(20048,1873,1,1,2,18092),-- Silver Nugget
(20048,2142,1,1,2,9046),-- Recipe: Varnish of Purity
(20049,1882,1,1,-1,61249),-- Leather
(20049,1924,1,1,-1,88910),-- Leather Helmet Design
(20049,2140,1,1,-1,55124),-- Recipe: High Grade Suede
(20049,57,127,221,0,700000),-- Adena
(20049,256,1,1,1,239),-- Cestus
(20049,1867,1,1,2,43421),-- Animal Skin
(20049,1872,1,1,2,43421),-- Animal Bone
(20049,1881,1,1,2,4343),-- Coarse Bone Powder
(20049,2143,1,1,2,6514),-- Recipe: Synthetic Cokes
(20049,5158,1,1,2,1303),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20050,735,1,1,-1,25265),-- Potion of Alacrity
(20050,1866,1,1,-1,202121),-- Suede
(20050,1925,1,1,-1,198808),-- Leather Gloves Lining
(20050,57,140,243,0,700000),-- Adena
(20050,15,1,1,1,132),-- Short Spear
(20050,2011,1,1,1,2044),-- Short Spear Edge
(20050,1831,1,1,2,119407),-- Antidote
(20050,1866,1,1,2,29852),-- Suede
(20050,1871,1,1,2,44778),-- Charcoal
(20050,2144,1,1,2,8956),-- Recipe: Compound Braid
(20051,1867,1,2,-1,371096),-- Animal Skin
(20051,1925,1,1,-1,182506),-- Leather Gloves Lining
(20051,2254,1,1,-1,11407),-- Recipe: Composition Bow
(20051,57,129,223,0,700000),-- Adena
(20051,272,1,1,1,121),-- Forest Bow
(20051,2013,1,1,1,1775),-- Forest Bow Shaft
(20051,1867,1,1,2,43847),-- Animal Skin
(20051,1872,1,1,2,43847),-- Animal Bone
(20051,1881,1,1,2,4385),-- Coarse Bone Powder
(20051,2150,1,1,2,15439),-- Recipe: Blue Diamond Necklace
(20051,5268,1,1,2,1316),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20052,1866,1,1,-1,161368),-- Suede
(20052,1868,1,1,-1,484105),-- Thread
(20052,1869,1,1,-1,242053),-- Iron Ore
(20052,57,110,197,0,700000),-- Adena
(20052,219,1,1,1,59),-- Sword Breaker
(20052,2015,1,1,1,873),-- Sword Breaker Edge
(20052,1804,1,1,2,1145),-- Recipe: Soulshot: D-Grade
(20052,1864,1,1,2,57199),-- Stem
(20052,1869,1,1,2,28600),-- Iron Ore
(20052,1880,1,1,2,2860),-- Steel
(20052,5158,1,1,2,1144),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20053,1898,1,1,-1,30130),-- Blue Diamond Necklace Gem
(20053,1926,1,1,-1,29627),-- Piece Bone Breastplate Fragment
(20053,2015,1,1,-1,3577),-- Sword Breaker Edge
(20053,57,134,238,0,700000),-- Adena
(20053,39,1,1,1,940),-- Boots
(20053,51,1,1,1,940),-- Bracer
(20053,1930,1,1,1,14811),-- Boot Lining
(20053,1864,1,1,2,69312),-- Stem
(20053,1869,1,1,2,34656),-- Iron Ore
(20053,1878,1,1,2,13863),-- Braided Hemp
(20053,2151,1,1,2,9654),-- Recipe: Necklace of Devotion
(20053,5437,1,1,2,848),-- Recipe: Heavy Sword(100%)
(20054,1804,1,1,-1,11391),-- Recipe: Soulshot: D-Grade
(20054,1880,1,1,-1,28477),-- Steel
(20054,3032,1,1,-1,11391),-- Recipe: Spiritshot D
(20054,57,130,231,0,700000),-- Adena
(20054,39,1,1,1,912),-- Boots
(20054,51,1,1,1,912),-- Bracer
(20054,1930,1,1,1,14379),-- Boot Lining
(20054,1867,1,1,2,44862),-- Animal Skin
(20054,1872,1,1,2,44862),-- Animal Bone
(20054,1882,1,1,2,7477),-- Leather
(20054,2176,1,1,2,16099),-- Recipe: Leather Boots
(20054,5268,1,1,2,1346),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20055,1926,1,1,-1,28764),-- Piece Bone Breastplate Fragment
(20055,1928,1,1,-1,41481),-- Piece Bone Gaiters Fragment
(20055,2176,1,1,-1,136253),-- Recipe: Leather Boots
(20055,57,130,231,0,700000),-- Adena
(20055,219,1,1,1,69),-- Sword Breaker
(20055,2015,1,1,1,1027),-- Sword Breaker Edge
(20055,735,1,1,2,2804),-- Potion of Alacrity
(20055,1864,1,1,2,67293),-- Stem
(20055,1869,1,1,2,33647),-- Iron Ore
(20055,1880,1,1,2,3365),-- Steel
(20055,2177,1,1,2,10750),-- Recipe: Bone Helmet
(20056,1926,1,1,-1,28501),-- Piece Bone Breastplate Fragment
(20056,2151,1,1,-1,78595),-- Recipe: Necklace of Devotion
(20056,5163,1,1,-1,11286),-- Recipe: Blessed Spiritshot (D) Compressed Package (100%)
(20056,57,128,230,0,700000),-- Adena
(20056,27,1,1,1,226),-- Hard Leather Shirt
(20056,33,1,1,1,363),-- Hard Leather Gaiters
(20056,1927,1,1,1,3325),-- Hard Leather Shirt Pattern
(20056,1929,1,1,1,5378),-- Hard Leather Gaiters Material
(20056,734,1,1,2,6946),-- Haste Potion
(20056,1868,1,1,2,83345),-- Thread
(20056,1873,1,1,2,16669),-- Silver Nugget
(20056,2256,1,1,2,1019),-- Recipe: Assassin Knife
(20057,1927,1,1,-1,20463),-- Hard Leather Shirt Pattern
(20057,2176,1,1,-1,122730),-- Recipe: Leather Boots
(20057,2258,1,1,-1,6272),-- Recipe: Temptation of Abyss
(20057,57,116,209,0,700000),-- Adena
(20057,1102,1,1,1,411),-- Tunic of Magic
(20057,1105,1,1,1,660),-- Stockings of Magic
(20057,1865,1,1,2,37884),-- Varnish
(20057,1870,1,1,2,37884),-- Coal
(20057,2257,1,1,2,927),-- Recipe: Trident
(20057,5273,1,1,2,1516),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20058,1865,1,2,-1,301771),-- Varnish
(20058,1873,1,1,-1,120708),-- Silver Nugget
(20058,1926,1,1,-1,30482),-- Piece Bone Breastplate Fragment
(20058,57,137,246,0,700000),-- Adena
(20058,16,1,1,1,146),-- Long Spear
(20058,1865,1,1,2,44570),-- Varnish
(20058,1870,1,1,2,44570),-- Coal
(20058,1879,1,1,2,7429),-- Cokes
(20058,2258,1,1,2,1090),-- Recipe: Temptation of Abyss
(20059,32,1,1,-1,3033),-- Piece Bone Gaiters
(20059,1879,1,1,-1,50295),-- Cokes
(20059,2177,1,1,-1,96413),-- Recipe: Bone Helmet
(20059,57,137,246,0,700000),-- Adena
(20059,625,1,1,1,1383),-- Bone Shield
(20059,1148,1,1,1,969),-- Hard Leather Helmet
(20059,1865,1,1,2,44570),-- Varnish
(20059,1870,1,1,2,44570),-- Coal
(20059,3032,1,1,2,1783),-- Recipe: Spiritshot D
(20059,5158,1,1,2,1783),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20060,1869,1,2,-1,307807),-- Iron Ore
(20060,1928,1,1,-1,44837),-- Piece Bone Gaiters Fragment
(20060,57,140,251,0,700000),-- Adena
(20060,87,1,1,1,38),-- Iron Hammer
(20060,155,1,1,1,38),-- Buzdygan
(20060,177,1,1,1,38),-- Mage Staff
(20060,2014,1,1,1,523),-- Iron Hammer Head
(20060,735,1,1,2,3031),-- Potion of Alacrity
(20060,1865,1,1,2,36369),-- Varnish
(20060,1870,1,1,2,36369),-- Coal
(20060,3953,1,1,2,1455),-- Recipe: Blessed Spiritshot D
(20060,6666,1,1,2,36369),-- Deluxe Chest Key - Grade 2
(20061,1880,1,1,-1,32142),-- Steel
(20061,1927,1,1,-1,25642),-- Hard Leather Shirt Pattern
(20061,2176,1,1,-1,153791),-- Recipe: Leather Boots
(20061,57,147,260,0,700000),-- Adena
(20061,273,1,1,1,78),-- Composition Bow
(20061,2016,1,1,1,1113),-- Composition Bow Shaft
(20061,1866,1,1,2,25319),-- Suede
(20061,1871,1,1,2,37978),-- Charcoal
(20061,2257,1,1,2,929),-- Recipe: Trident
(20061,5268,1,1,2,1520),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20061,6666,1,1,2,37978),-- Deluxe Chest Key - Grade 2
(20062,27,1,1,-1,1708),-- Hard Leather Shirt
(20062,1929,1,1,-1,40661),-- Hard Leather Gaiters Material
(20062,2257,1,1,-1,7705),-- Recipe: Trident
(20062,57,144,255,0,700000),-- Adena
(20062,219,1,1,1,77),-- Sword Breaker
(20062,2015,1,1,1,1135),-- Sword Breaker Edge
(20062,1867,1,1,2,49644),-- Animal Skin
(20062,1872,1,1,2,49644),-- Animal Bone
(20062,1881,1,1,2,4965),-- Coarse Bone Powder
(20062,2258,1,1,2,911),-- Recipe: Temptation of Abyss
(20062,6666,1,1,2,37233),-- Deluxe Chest Key - Grade 2
(20063,34,1,1,-1,2233),-- Bronze Gaiters
(20063,2176,1,1,-1,164023),-- Recipe: Leather Boots
(20063,2177,1,1,-1,109523),-- Recipe: Bone Helmet
(20063,57,152,281,0,700000),-- Adena
(20063,273,1,1,1,84),-- Composition Bow
(20063,2016,1,1,1,1187),-- Composition Bow Shaft
(20063,1804,1,1,2,1621),-- Recipe: Soulshot: D-Grade
(20063,1867,1,1,2,54006),-- Animal Skin
(20063,1872,1,1,2,54006),-- Animal Bone
(20063,1882,1,1,2,9001),-- Leather
(20063,6666,1,1,2,40504),-- Deluxe Chest Key - Grade 2
(20064,1873,1,1,-1,135792),-- Silver Nugget
(20064,1926,1,1,-1,34291),-- Piece Bone Breastplate Fragment
(20064,1930,1,1,-1,87046),-- Boot Lining
(20064,57,151,278,0,700000),-- Adena
(20064,27,1,1,1,272),-- Hard Leather Shirt
(20064,33,1,1,1,436),-- Hard Leather Gaiters
(20064,1927,1,1,1,4000),-- Hard Leather Shirt Pattern
(20064,1929,1,1,1,6470),-- Hard Leather Gaiters Material
(20064,734,1,1,2,6686),-- Haste Potion
(20064,1866,1,1,2,26741),-- Suede
(20064,1871,1,1,2,40111),-- Charcoal
(20064,2179,1,1,2,19192),-- Recipe: Crafted Leather Gloves
(20064,5273,1,1,2,1605),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20066,347,1,1,-1,889),-- Ring Mail Breastplate
(20066,1865,1,3,-1,371004),-- Varnish
(20066,1873,1,3,-1,148401),-- Silver Nugget
(20066,57,196,367,0,700000),-- Adena
(20066,347,1,1,1,250),-- Ring Mail Breastplate
(20066,376,1,1,1,399),-- Iron Plate Gaiters
(20066,1868,1,1,2,111111),-- Thread
(20066,1873,1,1,2,16129),-- Silver Nugget
(20066,2178,1,1,2,6410),-- Recipe: Dark Stockings
(20066,8294,1,1,2,6410),-- Recipe: Steel Sword (100%)
(20066,5549,1,1,2,4878),-- Metallic Thread
(20067,1869,1,2,-1,349739),-- Iron Ore
(20067,2020,1,1,-1,6122),-- Temptation of Abyss Piece
(20067,2180,1,1,-1,26296),-- Recipe: Scale Mail
(20067,57,186,343,0,700000),-- Adena
(20067,847,1,1,1,365),-- Red Crescent Earring
(20067,890,1,1,1,546),-- Ring of Devotion
(20067,910,1,1,1,274),-- Necklace of Devotion
(20067,1899,1,1,1,7668),-- Necklace of Devotion Chain
(20067,1864,1,1,2,125000),-- Stem
(20067,1869,1,1,2,55556),-- Iron Ore
(20067,1878,1,1,2,18182),-- Braided Hemp
(20067,2180,1,1,2,6757),-- Recipe: Scale Mail
(20068,391,1,1,-1,1201),-- Puma Skin Shirt
(20068,1899,1,1,-1,58713),-- Necklace of Devotion Chain
(20068,2261,1,1,-1,5835),-- Recipe: Conjuror\'s Staff
(20068,57,196,374,0,700000),-- Adena
(20068,101,1,1,1,21),-- Scroll of Wisdom
(20068,312,1,1,1,21),-- Branch of Life
(20068,313,1,1,1,21),-- Temptation of Abyss
(20068,314,1,1,1,21),-- Proof of Revenge
(20068,2020,1,1,1,739),-- Temptation of Abyss Piece
(20068,1864,1,1,2,111111),-- Stem
(20068,1869,1,1,2,71429),-- Iron Ore
(20068,1878,1,1,2,23256),-- Braided Hemp
(20068,2262,1,1,2,2703),-- Recipe: Elven Bow
(20070,1872,1,2,-1,517936),-- Animal Bone
(20070,2152,1,1,-1,68389),-- Recipe: Enchanted Necklace
(20070,2182,1,1,-1,46633),-- Recipe: Scale Gaiters
(20070,57,201,388,0,700000),-- Adena
(20070,40,1,1,1,522),-- Leather Boots
(20070,604,1,1,1,522),-- Crafted Leather Gloves
(20070,1931,1,1,1,7125),-- Leather Boot Lining
(20070,1934,1,1,1,7125),-- Crafted Leather Gloves Lining
(20070,1867,1,1,2,66667),-- Animal Skin
(20070,1872,1,1,2,58824),-- Animal Bone
(20070,1875,1,1,2,4329),-- Stone of Purity
(20070,1881,1,1,2,7246),-- Coarse Bone Powder
(20070,2263,1,1,2,1927),-- Recipe: Dwarven Trident
(20072,1931,1,1,-1,53375),-- Leather Boot Lining
(20072,2263,1,1,-1,6340),-- Recipe: Dwarven Trident
(20072,3953,1,1,-1,16333),-- Recipe: Blessed Spiritshot D
(20072,57,210,411,0,700000),-- Adena
(20072,45,1,1,1,489),-- Bone Helmet
(20072,626,1,1,1,698),-- Bronze Shield
(20072,1932,1,1,1,12126),-- Bone Helmet Design
(20072,1866,1,1,2,23810),-- Suede
(20072,1871,1,1,2,35714),-- Charcoal
(20072,1885,1,1,2,3690),-- High Grade Suede
(20072,2262,1,1,2,1155),-- Recipe: Elven Bow
(20072,5220,1,1,2,2538),-- Metal Hardener
(20072,6035,1,1,2,3922),-- Magic Haste Potion
(20072,6666,1,1,2,37037),-- Deluxe Chest Key - Grade 2
(20074,1061,1,1,-1,256459),-- Healing Potion
(20074,1869,1,3,-1,423158),-- Iron Ore
(20074,1900,1,1,-1,46297),-- Enchanted Necklace Chain
(20074,57,216,426,0,700000),-- Adena
(20074,605,1,1,1,695),-- Leather Gauntlets
(20074,1123,1,1,1,695),-- Blue Buckskin Boots
(20074,1864,1,1,2,62500),-- Stem
(20074,1869,1,1,2,24390),-- Iron Ore
(20074,1880,1,1,2,8197),-- Steel
(20074,5550,1,1,2,514),-- Durable Metal Plate
(20074,6387,1,1,2,4082),-- Blessed Scroll of Resurrection for Pets
(20075,348,1,1,-1,583),-- Scale Mail
(20075,1061,1,2,-1,235088),-- Healing Potion
(20075,2153,1,1,-1,60326),-- Recipe: Tiger\'s Eye Earring
(20075,57,197,391,0,700000),-- Adena
(20075,848,1,1,1,249),-- Enchanted Earring
(20075,879,1,1,1,374),-- Enchanted Ring
(20075,911,1,1,1,187),-- Enchanted Necklace
(20075,1900,1,1,1,5798),-- Enchanted Necklace Chain
(20075,955,1,1,2,142),-- Scroll: Enchant Weapon (Grade D)
(20075,1831,1,1,2,125000),-- Antidote
(20075,1866,1,1,2,23256),-- Suede
(20075,1871,1,1,2,43478),-- Charcoal
(20075,1877,1,1,2,895),-- Adamantite Nugget
(20075,2193,1,1,2,6250),-- Recipe: Boots of Power
(20076,1880,1,1,-1,44889),-- Steel
(20076,1900,1,1,-1,49112),-- Enchanted Necklace Chain
(20076,2250,1,1,-1,112222),-- Recipe: Bone Arrow
(20076,57,228,452,0,700000),-- Adena
(20076,605,1,1,1,737),-- Leather Gauntlets
(20076,1123,1,1,1,737),-- Blue Buckskin Boots
(20076,956,1,1,2,824),-- Scroll: Enchant Armor (Grade D)
(20076,1864,1,1,2,83333),-- Stem
(20076,1869,1,1,2,47619),-- Iron Ore
(20076,1880,1,1,2,4950),-- Steel
(20076,2274,1,1,2,292),-- Recipe: Skull Breaker
(20076,5550,1,1,2,546),-- Durable Metal Plate
(20077,1933,1,1,-1,52057),-- Dark Stocking Fabric
(20077,2153,1,1,-1,68815),-- Recipe: Tiger\'s Eye Earring
(20077,2183,1,1,-1,29111),-- Recipe: Mithril Banded Mail
(20077,57,225,448,0,700000),-- Adena
(20077,46,1,1,1,484),-- Bronze Helmet
(20077,627,1,1,1,691),-- Aspis
(20077,955,1,1,2,242),-- Scroll: Enchant Weapon (Grade D)
(20077,1865,1,1,2,31250),-- Varnish
(20077,1870,1,1,2,35714),-- Coal
(20077,1874,1,1,2,4525),-- Oriharukon Ore
(20078,1900,1,1,-1,48411),-- Enchanted Necklace Chain
(20078,2023,1,1,-1,5483),-- Conjuror\'s Staff Head
(20078,2024,1,1,-1,6303),-- Elven Bow Shaft
(20078,57,224,448,0,700000),-- Adena
(20078,433,1,1,1,61),-- Elven Tunic
(20078,434,1,1,1,61),-- White Tunic
(20078,435,1,1,1,61),-- Mystic\'s Tunic
(20078,466,1,1,1,97),-- Elven Stockings
(20078,467,1,1,1,97),-- Dark Stockings
(20078,468,1,1,1,97),-- Mystic\'s Stockings
(20078,1933,1,1,1,3556),-- Dark Stocking Fabric
(20078,1936,1,1,1,2474),-- White Tunic Pattern
(20078,736,1,1,2,33333),-- Scroll of Escape
(20078,1868,1,1,2,100000),-- Thread
(20078,1873,1,1,2,16129),-- Silver Nugget
(20078,1889,1,1,2,4000),-- Compound Braid
(20078,5549,1,1,2,9346),-- Metallic Thread
(20079,1866,1,2,-1,294987),-- Suede
(20079,1881,1,1,-1,58997),-- Coarse Bone Powder
(20079,2266,1,1,-1,4576),-- Recipe: Shilen Knife
(20079,57,224,448,0,700000),-- Adena
(20079,848,1,1,1,284),-- Enchanted Earring
(20079,879,1,1,1,426),-- Enchanted Ring
(20079,911,1,1,1,213),-- Enchanted Necklace
(20079,1900,1,1,1,6614),-- Enchanted Necklace Chain
(20079,1061,1,1,2,20833),-- Healing Potion
(20079,1867,1,1,2,52632),-- Animal Skin
(20079,1872,1,1,2,50000),-- Animal Bone
(20079,1875,1,1,2,4329),-- Stone of Purity
(20079,1881,1,1,2,5291),-- Coarse Bone Powder
(20079,2183,1,1,2,2404),-- Recipe: Mithril Banded Mail
(20079,2278,1,1,2,247),-- Recipe: Strengthened Long Bow
(20080,1878,1,2,-1,191782),-- Braided Hemp
(20080,2021,1,1,-1,6054),-- Spinebone Sword Blade
(20080,2268,1,1,-1,4958),-- Recipe: Tome of Blood
(20080,57,243,485,0,700000),-- Adena
(20080,259,1,1,1,81),-- Single-Edged Jamadhr
(20080,737,1,1,2,5348),-- Scroll of Resurrection
(20080,1864,1,1,2,111111),-- Stem
(20080,1869,1,1,2,47619),-- Iron Ore
(20080,1878,1,1,2,16129),-- Braided Hemp
(20080,1884,1,1,2,35714),-- Cord
(20080,6387,1,1,2,3115),-- Blessed Scroll of Resurrection for Pets
(20081,1061,1,2,-1,486166),-- Healing Potion
(20081,1933,1,1,-1,94373),-- Dark Stocking Fabric
(20081,57,406,814,0,700000),-- Adena
(20081,848,1,1,1,515),-- Enchanted Earring
(20081,879,1,1,1,772),-- Enchanted Ring
(20081,911,1,1,1,386),-- Enchanted Necklace
(20081,1900,1,1,1,11990),-- Enchanted Necklace Chain
(20081,1874,1,1,2,7042),-- Oriharukon Ore
(20081,1878,1,1,2,27778),-- Braided Hemp
(20081,1884,1,1,2,37037),-- Cord
(20081,1889,1,1,2,5025),-- Compound Braid
(20081,2267,1,1,2,648),-- Recipe: Gastraphetes
(20081,2274,1,1,2,1006),-- Recipe: Skull Breaker
(20081,5220,1,1,2,2012),-- Metal Hardener
(20082,1884,1,1,-1,297748),-- Cord
(20082,1901,1,1,-1,66829),-- Tiger\'s Eye Earring Stone
(20082,2275,1,1,-1,3456),-- Recipe: Heavy Bone Club
(20082,57,245,491,0,700000),-- Adena
(20082,63,1,1,1,145),-- Gauntlets
(20082,553,1,1,1,145),-- Iron Boots
(20082,1940,1,1,1,5485),-- Iron Boots Design
(20082,2422,1,1,1,145),-- Reinforced Leather Boots
(20082,2423,1,1,1,145),-- Boots of Knowledge
(20082,2446,1,1,1,145),-- Reinforced Leather Gloves
(20082,2447,1,1,1,145),-- Gloves of Knowledge
(20082,1832,1,1,2,41667),-- Greater Antidote
(20082,1875,1,1,2,2688),-- Stone of Purity
(20082,1879,1,1,2,7519),-- Cokes
(20082,1885,1,1,2,6452),-- High Grade Suede
(20082,2154,1,1,2,4310),-- Recipe: Elven Earring
(20082,2190,1,1,2,2688),-- Recipe: Mithril Stockings
(20082,5549,1,1,2,4831),-- Metallic Thread
(20082,5550,1,1,2,538),-- Durable Metal Plate
(20083,2154,1,1,-1,63531),-- Recipe: Elven Earring
(20083,2975,1,1,-1,52981),-- Recipe: Brigandine Helmet
(20083,2987,1,1,-1,57610),-- Recipe: Plate Shield
(20083,57,301,602,0,700000),-- Adena
(20083,393,1,1,1,70),-- Mithril Banded Mail
(20083,394,1,1,1,70),-- Reinforced Leather Shirt
(20083,415,1,1,1,111),-- Mithril Banded Gaiters
(20083,416,1,1,1,111),-- Reinforced Leather Gaiters
(20083,1938,1,1,1,3536),-- Mithril Banded Mail Material
(20083,1939,1,1,1,4619),-- Mithril Banded Gaiters Material
(20083,1874,1,1,2,2882),-- Oriharukon Ore
(20083,1878,1,1,2,18182),-- Braided Hemp
(20083,1884,1,1,2,27778),-- Cord
(20083,1889,1,1,2,2309),-- Compound Braid
(20083,2155,1,1,2,7519),-- Recipe: Elven Ring
(20083,2286,1,1,2,288),-- Recipe: Bonebreaker
(20083,2985,1,1,2,3460),-- Recipe: Plate Boots
(20083,5220,1,1,2,2165),-- Metal Hardener
(20084,1885,1,1,-1,46006),-- High Grade Suede
(20084,1938,1,1,-1,37114),-- Mithril Banded Mail Material
(20084,2978,1,1,-1,73905),-- Recipe: Elven Mithril Boots
(20084,57,280,558,0,700000),-- Adena
(20084,47,1,1,1,386),-- Helmet
(20084,628,1,1,1,551),-- Hoplon
(20084,1341,10,30,2,142857),-- Bone Arrow
(20084,1876,1,1,2,8850),-- Mithril Ore
(20084,1877,1,1,2,2208),-- Adamantite Nugget
(20084,1880,1,1,2,5376),-- Steel
(20084,1881,1,1,2,4098),-- Coarse Bone Powder
(20084,1882,1,1,2,9174),-- Leather
(20084,2269,1,1,2,316),-- Recipe: Morning Star
(20084,2971,1,1,2,947),-- Recipe: Mithril Scale Gaiters
(20085,2155,1,2,-1,102468),-- Recipe: Elven Ring
(20085,2278,1,1,-1,4560),-- Recipe: Strengthened Long Bow
(20085,2422,1,1,-1,2513),-- Reinforced Leather Boots
(20085,57,324,645,0,700000),-- Adena
(20085,63,1,1,1,191),-- Gauntlets
(20085,553,1,1,1,191),-- Iron Boots
(20085,1940,1,1,1,7237),-- Iron Boots Design
(20085,2422,1,1,1,191),-- Reinforced Leather Boots
(20085,2423,1,1,1,191),-- Boots of Knowledge
(20085,2446,1,1,1,191),-- Reinforced Leather Gloves
(20085,2447,1,1,1,191),-- Gloves of Knowledge
(20085,1865,1,1,2,66667),-- Varnish
(20085,1870,1,1,2,66667),-- Coal
(20085,1874,1,1,2,4016),-- Oriharukon Ore
(20085,2270,1,1,2,430),-- Recipe: Goat Head Staff
(20085,2293,1,1,2,287),-- Recipe: Glaive
(20085,2986,1,1,2,5155),-- Recipe: Sage\'s Worn Gloves
(20086,469,1,1,-1,1030),-- Stockings of Knowledge
(20086,1868,1,3,-1,490187),-- Thread
(20086,1939,1,1,-1,43055),-- Mithril Banded Gaiters Material
(20086,57,248,496,0,700000),-- Adena
(20086,436,1,1,1,172),-- Tunic of Knowledge
(20086,469,1,1,1,274),-- Stockings of Knowledge
(20086,1874,1,1,2,3356),-- Oriharukon Ore
(20086,1878,1,1,2,18868),-- Braided Hemp
(20086,1884,1,1,2,27027),-- Cord
(20086,1889,1,1,2,2793),-- Compound Braid
(20086,2972,1,1,2,3356),-- Recipe: Brigandine Gaiters
(20086,8307,1,1,2,186),-- Recipe: Priest Mace (100%)
(20086,5220,1,1,2,1119),-- Metal Hardener
(20087,1866,1,2,-1,783985),-- Suede
(20087,1940,1,2,-1,175519),-- Iron Boots Design
(20087,2191,1,1,-1,157427),-- Recipe: RIP Gauntlets
(20087,57,596,1189,0,700000),-- Adena
(20087,393,1,1,1,138),-- Mithril Banded Mail
(20087,394,1,1,1,138),-- Reinforced Leather Shirt
(20087,415,1,1,1,219),-- Mithril Banded Gaiters
(20087,416,1,1,1,219),-- Reinforced Leather Gaiters
(20087,1938,1,1,1,7007),-- Mithril Banded Mail Material
(20087,1939,1,1,1,9154),-- Mithril Banded Gaiters Material
(20087,1832,1,1,2,100000),-- Greater Antidote
(20087,1866,1,1,2,66667),-- Suede
(20087,1871,1,1,2,90909),-- Charcoal
(20087,1877,1,1,2,1757),-- Adamantite Nugget
(20087,2975,1,1,2,10204),-- Recipe: Brigandine Helmet
(20087,2980,1,1,2,9091),-- Recipe: Brigandine Gauntlets
(20087,2984,1,1,2,8772),-- Recipe: Salamander Skin Boots
(20087,3020,1,1,2,702),-- Recipe: Elven Long Sword
(20088,1875,1,1,-1,87191),-- Stone of Purity
(20088,1895,1,2,-1,373675),-- Metallic Fiber
(20088,2149,1,1,-1,93419),-- Recipe: Metallic Fiber
(20088,57,662,1326,0,700000),-- Adena
(20088,293,1,1,1,72),-- War Hammer
(20088,296,1,1,1,72),-- Dwarven Pike
(20088,1867,1,1,2,200000),-- Animal Skin
(20088,1872,1,1,2,166667),-- Animal Bone
(20088,1875,1,1,2,6536),-- Stone of Purity
(20088,1881,1,1,2,22222),-- Coarse Bone Powder
(20088,2270,1,1,2,957),-- Recipe: Goat Head Staff
(20088,2288,1,1,2,319),-- Recipe: Ghost Staff
(20089,1884,1,1,-1,335320),-- Cord
(20089,1902,1,1,-1,55207),-- Elven Earring Beads
(20089,2159,1,1,-1,33635),-- Recipe: Necklace of Darkness
(20089,57,276,551,0,700000),-- Adena
(20089,395,1,1,1,63),-- Manticore Skin Shirt
(20089,417,1,1,1,101),-- Manticore Skin Gaiters
(20089,1942,1,1,1,3592),-- Manticore Skin Shirt Texture
(20089,1943,1,1,1,5593),-- Manticore Skin Gaiters Pattern
(20089,1825,1,1,2,2959),-- Recipe: Oriharukon
(20089,1875,1,1,2,3802),-- Stone of Purity
(20089,1879,1,1,2,9091),-- Cokes
(20089,1885,1,1,2,3175),-- High Grade Suede
(20089,2205,1,1,2,2538),-- Recipe: Chain Gaiters
(20089,2989,1,1,2,1479),-- Recipe: Chain Hood
(20089,5549,1,1,2,4854),-- Metallic Thread
(20089,5550,1,1,2,470),-- Durable Metal Plate
(20090,1825,1,1,-1,85633),-- Recipe: Oriharukon
(20090,1868,1,7,-1,799241),-- Thread
(20090,2197,1,1,-1,25133),-- Recipe: Salamander Skin Mail
(20090,57,608,1213,0,700000),-- Adena
(20090,352,1,1,1,70),-- Brigandine Tunic
(20090,1941,1,1,1,4108),-- Brigandine Temper
(20090,2377,1,1,1,111),-- Mithril Scale Gaiters
(20090,2378,1,1,1,111),-- Brigandine Gaiters
(20090,2918,1,1,1,6128),-- Mithril Scale Gaiters Material
(20090,2919,1,1,1,6128),-- Brigandine Gaiters Material
(20090,1341,10,30,2,333333),-- Bone Arrow
(20090,1876,1,1,2,19608),-- Mithril Ore
(20090,1877,1,1,2,4785),-- Adamantite Nugget
(20090,1880,1,1,2,10000),-- Steel
(20090,1881,1,1,2,13889),-- Coarse Bone Powder
(20090,1882,1,1,2,14925),-- Leather
(20090,2158,1,1,2,12987),-- Recipe: Mithril Ring
(20090,5472,1,1,2,8928),-- Recipe: Metallic Thread(100%)
(20092,1666,1,1,-1,49062),-- Recipe: Wooden Arrow
(20092,1866,1,1,-1,98124),-- Suede
(20092,2005,1,1,-1,10513),-- Broadsword Blade
(20092,57,79,107,0,700000),-- Adena
(20092,113,1,1,1,7148),-- Mystic\'s Earring
(20092,875,1,1,1,10735),-- Ring of Knowledge
(20092,906,1,1,1,5368),-- Necklace of Knowledge
(20092,1867,1,1,2,19323),-- Animal Skin
(20092,1869,1,1,2,14493),-- Iron Ore
(20092,1870,1,1,2,14493),-- Coal
(20092,1871,1,1,2,14493),-- Charcoal
(20092,1872,1,1,2,19323),-- Animal Bone
(20092,2137,1,1,2,4263),-- Recipe: Steel
(20093,1666,1,1,-1,60652),-- Recipe: Wooden Arrow
(20093,1867,1,1,-1,242609),-- Animal Skin
(20093,2007,1,1,-1,12997),-- Bow Shaft
(20093,57,98,132,0,700000),-- Adena
(20093,3,1,1,1,860),-- Broadsword
(20093,2005,1,1,1,3840),-- Broadsword Blade
(20093,1060,1,1,2,59720),-- Lesser Healing Potion
(20093,1867,1,1,2,35832),-- Animal Skin
(20093,1872,1,1,2,35832),-- Animal Bone
(20093,2138,1,1,2,7905),-- Recipe: Coarse Bone Powder (100%)
(20095,1864,1,1,-1,297315),-- Stem
(20095,2006,1,1,-1,10618),-- Willow Staff Head
(20095,57,80,108,0,700000),-- Adena
(20095,428,1,1,1,1808),-- Feriotic Tunic
(20095,463,1,1,1,2889),-- Feriotic Stockings
(20095,1100,1,1,1,1808),-- Cotton Tunic
(20095,1103,1,1,1,2889),-- Cotton Stockings
(20095,1789,1,1,2,5412),-- Recipe: Cedar Staff
(20095,1864,1,1,2,58548),-- Stem
(20095,1869,1,1,2,29274),-- Iron Ore
(20096,1792,1,1,-1,14822),-- Recipe: Short Spear
(20096,1799,1,1,-1,298635),-- Recipe: Leather Gloves
(20096,1921,1,1,-1,130051),-- Leather Shoes Texture
(20096,57,109,147,0,700000),-- Adena
(20096,37,1,1,1,2996),-- Leather Shoes
(20096,49,1,1,1,2996),-- Gloves
(20096,1921,1,1,1,25611),-- Leather Shoes Texture
(20096,736,1,1,2,9924),-- Scroll of Escape
(20096,1867,1,1,2,26464),-- Animal Skin
(20096,1869,1,1,2,19848),-- Iron Ore
(20096,1870,1,1,2,19848),-- Coal
(20096,1871,1,1,2,19848),-- Charcoal
(20096,1872,1,1,2,26464),-- Animal Bone
(20097,1793,1,1,-1,12352),-- Recipe: Sword of Reflection
(20097,1864,1,1,-1,335964),-- Stem
(20097,1872,1,1,-1,223976),-- Animal Bone
(20097,57,91,122,0,700000),-- Adena
(20097,23,1,1,1,832),-- Wooden Breastplate
(20097,30,1,1,1,1331),-- Hard Leather Pants
(20097,2386,1,1,1,1331),-- Wooden Gaiters
(20097,1792,1,1,2,2433),-- Recipe: Short Spear
(20097,1867,1,1,2,44106),-- Animal Skin
(20097,1872,1,1,2,44106),-- Animal Bone
(20098,1867,1,1,-1,292048),-- Animal Skin
(20098,1870,1,1,-1,219036),-- Coal
(20098,1896,1,1,-1,92812),-- Necklace of Anguish Chain
(20098,57,117,161,0,700000),-- Adena
(20098,66,1,1,1,160),-- Gladius
(20098,1333,1,1,1,160),-- Brandish
(20098,2010,1,1,1,1679),-- Brandish Blade
(20098,1793,1,1,2,3172),-- Recipe: Sword of Reflection
(20098,1865,1,1,2,43134),-- Varnish
(20098,1870,1,1,2,43134),-- Coal
(20099,1831,1,1,-1,473949),-- Antidote
(20099,1896,1,1,-1,75310),-- Necklace of Anguish Chain
(20099,2009,1,1,-1,6303),-- Dirk Blade
(20099,57,95,131,0,700000),-- Adena
(20099,66,1,1,1,130),-- Gladius
(20099,1333,1,1,1,130),-- Brandish
(20099,2010,1,1,1,1363),-- Brandish Blade
(20099,1794,1,1,2,1287),-- Recipe: Forest Bow
(20099,1864,1,1,2,35000),-- Stem
(20099,1865,1,1,2,17500),-- Varnish
(20099,1866,1,1,2,11667),-- Suede
(20099,1868,1,1,2,35000),-- Thread
(20099,1873,1,1,2,7000),-- Silver Nugget
(20100,1792,1,1,-1,13849),-- Recipe: Short Spear
(20100,1833,1,1,-1,502269),-- Bandage
(20100,1866,1,1,-1,125567),-- Suede
(20100,57,99,140,0,700000),-- Adena
(20100,271,1,1,1,411),-- Hunting Bow
(20100,1799,1,1,2,54950),-- Recipe: Leather Gloves
(20100,1866,1,1,2,24728),-- Suede
(20100,1871,1,1,2,37091),-- Charcoal
(20101,20,1,1,-1,18026),-- Buckler
(20101,1793,1,1,-1,18424),-- Recipe: Sword of Reflection
(20101,1921,1,1,-1,161654),-- Leather Shoes Texture
(20101,57,128,189,0,700000),-- Adena
(20101,114,1,1,1,1687),-- Earring of Strength
(20101,115,1,1,1,1687),-- Earring of Wisdom
(20101,876,1,1,1,2531),-- Ring of Anguish
(20101,907,1,1,1,1266),-- Necklace of Anguish
(20101,1896,1,1,1,12545),-- Necklace of Anguish Chain
(20101,1803,1,1,2,20732),-- Recipe: Necklace of Wisdom
(20101,1864,1,1,2,49342),-- Stem
(20101,1865,1,1,2,24671),-- Varnish
(20101,1866,1,1,2,16448),-- Suede
(20101,1868,1,1,2,49342),-- Thread
(20101,1873,1,1,2,9869),-- Silver Nugget
(20102,1793,1,1,-1,14067),-- Recipe: Sword of Reflection
(20102,1866,1,1,-1,127538),-- Suede
(20102,1922,1,1,-1,48803),-- Leather Tunic Pattern
(20102,57,98,145,0,700000),-- Adena
(20102,271,1,1,1,418),-- Hunting Bow
(20102,736,1,1,2,18837),-- Scroll of Escape
(20102,1864,1,1,2,75346),-- Stem
(20102,1869,1,1,2,37673),-- Iron Ore
(20103,1803,1,1,-1,198526),-- Recipe: Necklace of Wisdom
(20103,1871,1,2,-1,236245),-- Charcoal
(20103,1921,1,1,-1,152416),-- Leather Shoes Texture
(20103,57,119,180,0,700000),-- Adena
(20103,37,1,1,1,3512),-- Leather Shoes
(20103,49,1,1,1,3512),-- Gloves
(20103,1921,1,1,1,30015),-- Leather Shoes Texture
(20103,17,10,30,2,174458),-- Wooden Arrow
(20103,1794,1,1,2,2566),-- Recipe: Forest Bow
(20103,1867,1,1,2,46523),-- Animal Skin
(20103,1872,1,1,2,46523),-- Animal Bone
(20104,1868,1,1,-1,386022),-- Thread
(20104,1869,1,1,-1,193011),-- Iron Ore
(20104,57,97,147,0,700000),-- Adena
(20104,271,1,1,1,422),-- Hunting Bow
(20104,17,10,30,2,142531),-- Wooden Arrow
(20104,736,1,1,2,14254),-- Scroll of Escape
(20104,1868,1,1,2,57013),-- Thread
(20104,1873,1,1,2,11403),-- Silver Nugget
(20105,734,1,1,-1,39986),-- Haste Potion
(20105,1801,1,1,-1,120865),-- Recipe: Piece Bone Gaiters
(20105,1872,1,1,-1,319890),-- Animal Bone
(20105,57,119,185,0,700000),-- Adena
(20105,15,1,1,1,105),-- Short Spear
(20105,2011,1,1,1,1617),-- Short Spear Edge
(20105,735,1,1,2,2953),-- Potion of Alacrity
(20105,1867,1,1,2,47246),-- Animal Skin
(20105,1872,1,1,2,47246),-- Animal Bone
(20105,2140,1,1,2,7087),-- Recipe: High Grade Suede
(20106,734,1,1,-1,39594),-- Haste Potion
(20106,1897,1,1,-1,56563),-- Necklace of Wisdom Chain
(20106,2150,1,1,-1,111533),-- Recipe: Blue Diamond Necklace
(20106,57,118,183,0,700000),-- Adena
(20106,38,1,1,1,1383),-- Low Boots
(20106,50,1,1,1,1383),-- Leather Gloves
(20106,1925,1,1,1,30677),-- Leather Gloves Lining
(20106,1867,1,1,2,46783),-- Animal Skin
(20106,1872,1,1,2,46783),-- Animal Bone
(20106,2141,1,1,2,7018),-- Recipe: Silver Mold
(20106,5273,1,1,2,1404),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20107,734,1,1,-1,32995),-- Haste Potion
(20107,1869,1,1,-1,197971),-- Iron Ore
(20107,1897,1,1,-1,47136),-- Necklace of Wisdom Chain
(20107,57,98,153,0,700000),-- Adena
(20107,217,1,1,1,86),-- Shining Knife
(20107,218,1,1,1,86),-- Throwing Knife
(20107,1865,1,1,2,29239),-- Varnish
(20107,1870,1,1,2,29239),-- Coal
(20107,2142,1,1,2,5848),-- Recipe: Varnish of Purity
(20107,6035,1,1,2,2437),-- Magic Haste Potion
(20108,734,1,1,-1,41833),-- Haste Potion
(20108,1800,1,1,-1,78931),-- Recipe: Piece Bone Breastplate
(20108,1866,1,2,-1,167333),-- Suede
(20108,57,122,197,0,700000),-- Adena
(20108,38,1,1,1,1461),-- Low Boots
(20108,50,1,1,1,1461),-- Leather Gloves
(20108,1925,1,1,1,32412),-- Leather Gloves Lining
(20108,735,1,1,2,3090),-- Potion of Alacrity
(20108,1867,1,1,2,49428),-- Animal Skin
(20108,1872,1,1,2,49428),-- Animal Bone
(20108,2144,1,1,2,7415),-- Recipe: Compound Braid
(20109,845,1,1,-1,9042),-- Cat\'s Eye Earring
(20109,1865,1,2,-1,401928),-- Varnish
(20109,1924,1,1,-1,129654),-- Leather Helmet Design
(20109,57,196,315,0,700000),-- Adena
(20109,24,1,1,1,585),-- Bone Breastplate
(20109,31,1,1,1,935),-- Bone Gaiters
(20109,390,1,1,1,585),-- Cotton Shirt
(20109,412,1,1,1,935),-- Cotton Pants
(20109,1864,1,1,2,67842),-- Stem
(20109,1865,1,1,2,33921),-- Varnish
(20109,1866,1,1,2,22614),-- Suede
(20109,1868,1,1,2,67842),-- Thread
(20109,1873,1,1,2,13569),-- Silver Nugget
(20109,2150,1,1,2,15926),-- Recipe: Blue Diamond Necklace
(20109,5273,1,1,2,1357),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20110,734,1,1,-1,64842),-- Haste Potion
(20110,1801,1,1,-1,195995),-- Recipe: Piece Bone Gaiters
(20110,1897,1,1,-1,92631),-- Necklace of Wisdom Chain
(20110,57,190,305,0,700000),-- Adena
(20110,24,1,1,1,567),-- Bone Breastplate
(20110,31,1,1,1,905),-- Bone Gaiters
(20110,390,1,1,1,567),-- Cotton Shirt
(20110,412,1,1,1,905),-- Cotton Pants
(20110,1866,1,1,2,51076),-- Suede
(20110,1871,1,1,2,76613),-- Charcoal
(20110,2173,1,1,2,20763),-- Recipe: Hard Leather Shirt
(20112,1800,1,1,-1,123442),-- Recipe: Piece Bone Breastplate
(20112,1801,1,1,-1,197756),-- Recipe: Piece Bone Gaiters
(20112,1924,1,1,-1,126628),-- Leather Helmet Design
(20112,57,188,310,0,700000),-- Adena
(20112,1101,1,1,1,1143),-- Tunic of Devotion
(20112,1104,1,1,1,1826),-- Stockings of Devotion
(20112,734,1,1,2,7730),-- Haste Potion
(20112,1800,1,1,2,14586),-- Recipe: Piece Bone Breastplate
(20112,1864,1,1,2,92762),-- Stem
(20112,1869,1,1,2,46381),-- Iron Ore
(20112,1878,1,1,2,18553),-- Braided Hemp
(20113,1870,1,1,-1,400397),-- Coal
(20113,1879,1,1,-1,66733),-- Cokes
(20113,1925,1,1,-1,262556),-- Leather Gloves Lining
(20113,57,192,316,0,700000),-- Adena
(20113,24,1,1,1,583),-- Bone Breastplate
(20113,31,1,1,1,932),-- Bone Gaiters
(20113,390,1,1,1,583),-- Cotton Shirt
(20113,412,1,1,1,932),-- Cotton Pants
(20113,1801,1,1,2,23834),-- Recipe: Piece Bone Gaiters
(20113,1865,1,1,2,47309),-- Varnish
(20113,1870,1,1,2,47309),-- Coal
(20113,1879,1,1,2,7885),-- Cokes
(20113,5273,1,1,2,1893),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20114,735,1,1,-1,33735),-- Potion of Alacrity
(20114,1817,1,1,-1,80965),-- Recipe: Cord
(20114,2011,1,1,-1,9235),-- Short Spear Edge
(20114,57,190,323,0,700000),-- Adena
(20114,845,1,1,1,1346),-- Cat\'s Eye Earring
(20114,877,1,1,1,2020),-- Ring of Wisdom
(20114,908,1,1,1,1005),-- Necklace of Wisdom
(20114,1897,1,1,1,14236),-- Necklace of Wisdom Chain
(20114,1817,1,1,2,11958),-- Recipe: Cord
(20114,1866,1,1,2,39860),-- Suede
(20114,1871,1,1,2,59790),-- Charcoal
(20114,6035,1,1,2,4983),-- Magic Haste Potion
(20115,2012,1,1,-1,8603),-- Sword of Reflection Blade
(20115,2013,1,1,-1,8912),-- Forest Bow Shaft
(20115,2139,1,1,-1,82584),-- Recipe: Steel Mold
(20115,57,194,330,0,700000),-- Adena
(20115,38,1,1,1,2403),-- Low Boots
(20115,50,1,1,1,2403),-- Leather Gloves
(20115,1925,1,1,1,53321),-- Leather Gloves Lining
(20115,1867,1,1,2,46466),-- Animal Skin
(20115,1869,1,1,2,34849),-- Iron Ore
(20115,1870,1,1,2,34849),-- Coal
(20115,1871,1,1,2,34849),-- Charcoal
(20115,1872,1,1,2,46466),-- Animal Bone
(20115,2139,1,1,2,6970),-- Recipe: Steel Mold
(20115,5158,1,1,2,1394),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20117,734,1,1,-1,42109),-- Haste Potion
(20117,1868,1,2,-1,505303),-- Thread
(20117,1925,1,1,-1,165673),-- Leather Gloves Lining
(20117,57,117,203,0,700000),-- Adena
(20117,176,1,1,1,110),-- Journeyman\'s Staff
(20117,2501,1,1,1,110),-- Bone Club
(20117,735,1,1,2,2488),-- Potion of Alacrity
(20117,1867,1,1,2,39803),-- Animal Skin
(20117,1872,1,1,2,39803),-- Animal Bone
(20117,1882,1,1,2,6634),-- Leather
(20117,2173,1,1,2,8090),-- Recipe: Hard Leather Shirt
(20118,846,1,1,-1,3026),-- Coral Earring
(20118,1866,1,1,-1,161368),-- Suede
(20118,5549,1,1,-1,24205),-- Metallic Thread
(20118,57,110,197,0,700000),-- Adena
(20118,87,1,1,1,30),-- Iron Hammer
(20118,155,1,1,1,30),-- Buzdygan
(20118,177,1,1,1,30),-- Mage Staff
(20118,2014,1,1,1,412),-- Iron Hammer Head
(20118,1868,1,1,2,71499),-- Thread
(20118,1873,1,1,2,14300),-- Silver Nugget
(20118,2179,1,1,2,17105),-- Recipe: Crafted Leather Gloves
(20118,5273,1,1,2,1430),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20120,1787,1,1,-1,30885),-- Recipe: Willow Staff
(20120,1864,1,1,-1,77212),-- Stem
(20120,1874,1,1,-1,2574),-- Oriharukon Ore
(20120,57,19,29,0,700000),-- Adena
(20120,13,1,1,1,10394),-- Short Bow
(20120,1060,1,1,2,22174),-- Lesser Healing Potion
(20120,1786,1,1,2,7983),-- Recipe: Broad Sword
(20120,1865,1,1,2,9979),-- Varnish
(20120,1870,1,1,2,9979),-- Coal
(20121,1786,1,1,-1,45293),-- Recipe: Broad Sword
(20121,1788,1,1,-1,45293),-- Recipe: Bow 
(20121,1864,1,1,-1,113233),-- Stem
(20121,57,30,42,0,700000),-- Adena
(20121,4,1,1,1,7622),-- Club
(20121,7,1,1,1,7622),-- Apprentice\'s Rod
(20121,17,10,30,2,73166),-- Wooden Arrow
(20121,1786,1,1,2,11707),-- Recipe: Broad Sword
(20121,1866,1,1,2,9756),-- Suede
(20121,1871,1,1,2,14634),-- Charcoal
(20130,1060,1,1,-1,172757),-- Lesser Healing Potion
(20130,1666,1,1,-1,25914),-- Recipe: Wooden Arrow
(20130,2005,1,1,-1,5553),-- Broadsword Blade
(20130,57,41,58,0,700000),-- Adena
(20130,1119,1,1,1,5045),-- Short Leather Gloves
(20130,1122,1,1,1,5044),-- Cotton Shoes
(20130,1129,1,1,1,5044),-- Crude Leather Shoes
(20130,1666,1,1,2,5103),-- Recipe: Wooden Arrow
(20130,1864,1,1,2,30618),-- Stem
(20130,1869,1,1,2,15309),-- Iron Ore
(20131,1666,1,1,-1,32439),-- Recipe: Wooden Arrow
(20131,1789,1,1,-1,17988),-- Recipe: Cedar Staff
(20131,2006,1,1,-1,6951),-- Willow Staff Head
(20131,57,51,72,0,700000),-- Adena
(20131,19,1,1,1,9012),-- Small Shield
(20131,42,1,1,1,6311),-- Leather Cap
(20131,1797,1,1,2,38715),-- Recipe: Leather Stockings
(20131,1865,1,1,2,19164),-- Varnish
(20131,1870,1,1,2,19164),-- Coal
(20132,1060,1,1,-1,330759),-- Lesser Healing Potion
(20132,2006,1,1,-1,10632),-- Willow Staff Head
(20132,57,80,108,0,700000),-- Adena
(20132,254,1,1,1,1407),-- Iron Gloves
(20132,734,1,1,2,3664),-- Haste Potion
(20132,1797,1,1,2,44410),-- Recipe: Leather Stockings
(20132,1867,1,1,2,29311),-- Animal Skin
(20132,1872,1,1,2,29311),-- Animal Bone
(20134,356,1,1,-1,163),-- Full Plate Armor
(20134,947,1,1,-1,679),-- Scroll: Enchant Weapon (Grade B)
(20134,1871,1,4,-1,849303),-- Charcoal
(20134,57,822,1672,0,700000),-- Adena
(20134,357,1,1,1,7),-- Zubei\'s Breastplate
(20134,383,1,1,1,10),-- Zubei\'s Gaiters
(20134,2376,1,1,1,7),-- Avadon Breastplate
(20134,2379,1,1,1,10),-- Avadon Gaiters
(20134,4056,1,1,1,1136),-- Zubei\'s Breastplate Part
(20134,4057,1,1,1,1755),-- Zubei\'s Gaiter Material
(20134,4064,1,1,1,1136),-- Avadon Breastplate Part
(20134,4065,1,1,1,1755),-- Avadon Gaiters Material
(20134,947,1,1,2,26),-- Scroll: Enchant Weapon (Grade B)
(20134,1866,1,1,2,33333),-- Suede
(20134,1871,1,1,2,50000),-- Charcoal
(20134,1885,1,1,2,4184),-- High Grade Suede
(20134,4041,1,1,2,930),-- Mold Hardener
(20134,4192,1,1,2,48),-- Recipe: Sword of Damascus
(20134,4993,1,1,2,698),-- Recipe: Doom Gloves (60%)
(20134,5220,1,1,2,3021),-- Metal Hardener
(20134,8728,1,1,200,698),-- Life Stone: level 61
(20135,1882,1,1,-1,172318),-- Leather
(20135,2038,1,1,-1,8193),-- Maingauche Edge
(20135,5231,1,1,-1,55388),-- Recipe: Metal Hardener (100%)
(20135,57,390,779,0,700000),-- Adena
(20135,261,1,1,1,29),-- Bich\'Hwa
(20135,2039,1,1,1,2029),-- Bich\'Hwa Edge
(20135,1867,1,1,2,83333),-- Animal Skin
(20135,1872,1,1,2,83333),-- Animal Bone
(20135,1882,1,1,2,15152),-- Leather
(20135,1894,1,1,2,1972),-- Crafted Leather
(20135,2286,1,1,2,272),-- Recipe: Bonebreaker
(20135,2296,1,1,2,272),-- Recipe: Atuba Mace
(20135,2985,1,1,2,6711),-- Recipe: Plate Boots
(20136,1539,1,3,-1,589837),-- Greater Healing Potion
(20136,1914,1,1,-1,149116),-- Earring of Binding Gemstone
(20136,4601,1,1,-1,5898),-- Greater Dye of Strength <Str+3 Con-3>
(20136,57,1834,3679,0,700000),-- Adena
(20136,75,1,1,1,5),-- Caliburs
(20136,76,1,1,1,5),-- Sword of Delusion
(20136,77,1,1,1,5),-- Tsurugi
(20136,84,1,1,1,5),-- Homunkulus\'s Sword
(20136,134,1,1,1,5),-- Sword of Nightmare
(20136,145,1,1,1,5),-- Sword of Whispering Death
(20136,2089,1,1,1,499),-- Caliburs Edge
(20136,2090,1,1,1,499),-- Delusional Blade
(20136,2091,1,1,1,499),-- Tsurugi Blade
(20136,2092,1,1,1,499),-- Homunkulus\'s Sword Blade
(20136,2095,1,1,1,499),-- Sword of Nightmare Blade
(20136,2096,1,1,1,499),-- Sword of Whispering Death Blade
(20136,1864,1,3,2,142857),-- Stem
(20136,1869,1,1,2,200000),-- Iron Ore
(20136,1880,1,1,2,15385),-- Steel
(20136,2169,1,1,2,2776),-- Recipe: Necklace of Binding
(20136,2232,1,1,2,902),-- Recipe: Drake Leather Armor
(20136,4043,1,1,2,5227),-- Asofe
(20136,5550,1,1,2,2091),-- Durable Metal Plate
(20136,8725,1,1,200,3058),-- Life Stone: level 52
(20137,1877,1,1,-1,63802),-- Adamantite Nugget
(20137,2961,1,1,-1,86853),-- Full Plate Boots Part
(20137,4049,1,1,-1,83664),-- Adamantite Earring Gemstone
(20137,57,771,1572,0,700000),-- Adena
(20137,11889,1,1,1,64),-- Common Item - Great Sword
(20137,11916,1,1,1,64),-- Common Item - Keshanberk
(20137,11900,1,1,1,64),-- Common Item - Sword of Valhalla
(20137,4104,1,1,1,450),-- Great Sword Blade
(20137,4107,1,1,1,443),-- Keshanberk Blade
(20137,4108,1,1,1,443),-- Sword of Valhalla Blade
(20137,1864,1,1,2,83333),-- Stem
(20137,1869,1,1,2,50000),-- Iron Ore
(20137,1880,1,1,2,4630),-- Steel
(20137,4043,1,1,2,1350),-- Asofe
(20137,4163,1,1,2,193),-- Recipe: Blue Wolf Stockings
(20137,4952,1,1,2,385),-- Recipe: Avadon Circlet (60%)
(20137,4999,1,1,2,1156),-- Recipe: Doom Boots (60%)
(20137,5550,1,1,2,1350),-- Durable Metal Plate
(20137,6669,1,1,2,10000),-- Deluxe Chest Key - Grade 5
(20137,8727,1,1,200,963),-- Life Stone: level 58
(20137,8737,1,1,200,193),-- Mid-Grade Life Stone: level 58
(20143,1792,1,1,-1,15682),-- Recipe: Short Spear
(20143,1867,1,3,-1,284370),-- Animal Skin
(20143,1872,1,2,-1,284370),-- Animal Bone
(20143,57,114,157,0,700000),-- Adena
(20143,20,1,1,1,4533),-- Buckler
(20143,43,1,1,1,3166),-- Wooden Helmet
(20143,17,10,30,2,157499),-- Wooden Arrow
(20143,1798,1,1,2,30882),-- Recipe: Leather Helmet
(20143,1866,1,1,2,21000),-- Suede
(20143,1871,1,1,2,31500),-- Charcoal
(20144,1880,1,1,-1,59999),-- Steel
(20144,1901,1,1,-1,82871),-- Tiger\'s Eye Earring Stone
(20144,2272,1,1,-1,4286),-- Recipe: Sword of Revolution
(20144,57,304,607,0,700000),-- Adena
(20144,393,1,1,1,70),-- Mithril Banded Mail
(20144,394,1,1,1,70),-- Reinforced Leather Shirt
(20144,415,1,1,1,112),-- Mithril Banded Gaiters
(20144,416,1,1,1,112),-- Reinforced Leather Gaiters
(20144,1938,1,1,1,3575),-- Mithril Banded Mail Material
(20144,1939,1,1,1,4671),-- Mithril Banded Gaiters Material
(20144,1341,10,30,2,125000),-- Bone Arrow
(20144,1875,1,1,2,3096),-- Stone of Purity
(20144,1879,1,1,2,6897),-- Cokes
(20144,1885,1,1,2,4202),-- High Grade Suede
(20144,2274,1,1,2,315),-- Recipe: Skull Breaker
(20144,2976,1,1,2,5348),-- Recipe: Manticore Skin Boots
(20144,2985,1,1,2,4310),-- Recipe: Plate Boots
(20144,5549,1,1,2,3460),-- Metallic Thread
(20144,5550,1,1,2,315),-- Durable Metal Plate
(20144,6667,1,1,2,23810),-- Deluxe Chest Key - Grade 3
(20145,2153,1,2,-1,78158),-- Recipe: Tiger\'s Eye Earring
(20145,2183,1,1,-1,33063),-- Recipe: Mithril Banded Mail
(20145,2268,1,1,-1,5197),-- Recipe: Tome of Blood
(20145,57,254,510,0,700000),-- Adena
(20145,848,1,1,1,323),-- Enchanted Earring
(20145,879,1,1,1,484),-- Enchanted Ring
(20145,911,1,1,1,242),-- Enchanted Necklace
(20145,1900,1,1,1,7512),-- Enchanted Necklace Chain
(20145,1868,1,1,2,111111),-- Thread
(20145,1873,1,1,2,18182),-- Silver Nugget
(20145,1889,1,1,2,4115),-- Compound Braid
(20145,2268,1,1,2,457),-- Recipe: Tome of Blood
(20145,2276,1,1,2,913),-- Recipe: Maingauche
(20145,5549,1,1,2,3650),-- Metallic Thread
(20146,1869,1,3,-1,606685),-- Iron Ore
(20146,1914,1,1,-1,68167),-- Earring of Binding Gemstone
(20146,1915,1,1,-1,106905),-- Ring of Ages Gemstone
(20146,57,587,1194,0,700000),-- Adena
(20146,103,1,1,1,49),-- Tower Shield
(20146,107,1,1,1,49),-- Composite Shield
(20146,517,1,1,1,35),-- Composite Helmet
(20146,1149,1,1,1,35),-- Shining Circlet
(20146,1978,1,1,1,4021),-- Tower Shield Fragment
(20146,1983,1,1,1,2751),-- Shining Circlet Pattern
(20146,2957,1,1,1,2751),-- Composite Helmet Design
(20146,4055,1,1,1,4021),-- Composite Shield Fragment
(20146,1866,1,1,2,29412),-- Suede
(20146,1871,1,1,2,45455),-- Charcoal
(20146,1877,1,1,2,1171),-- Adamantite Nugget
(20146,2170,1,1,2,732),-- Recipe: Nassen\'s Earring
(20146,3014,1,1,2,585),-- Recipe: Full Plate Boots
(20146,4040,1,1,2,878),-- Mold Lubricant
(20146,4041,1,1,2,485),-- Mold Hardener
(20146,4188,1,1,2,65),-- Recipe: Hell Knife
(20146,4609,1,1,2,124),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20146,8725,1,1,200,585),-- Life Stone: level 52
(20146,8735,1,1,200,585),-- Mid-Grade Life Stone: level 52
(20146,8745,1,1,200,439),-- High-Grade Life Stone: level 52
(20147,1870,1,2,-1,284768),-- Coal
(20147,1876,1,1,-1,56954),-- Mithril Ore
(20147,57,130,231,0,700000),-- Adena
(20147,27,1,1,1,228),-- Hard Leather Shirt
(20147,33,1,1,1,366),-- Hard Leather Gaiters
(20147,1927,1,1,1,3356),-- Hard Leather Shirt Pattern
(20147,1929,1,1,1,5427),-- Hard Leather Gaiters Material
(20147,1868,1,1,2,84117),-- Thread
(20147,1873,1,1,2,16824),-- Silver Nugget
(20147,2255,1,1,2,1029),-- Recipe: Saber
(20147,6666,1,1,2,42059),-- Deluxe Chest Key - Grade 2
(20153,30,1,1,-1,8112),-- Hard Leather Pants
(20153,1831,1,1,-1,537542),-- Antidote
(20153,1867,1,2,-1,268771),-- Animal Skin
(20153,57,109,147,0,700000),-- Adena
(20153,429,1,1,1,749),-- Leather Tunic
(20153,464,1,1,1,1199),-- Leather Stockings
(20153,1922,1,1,1,7596),-- Leather Tunic Pattern
(20153,1923,1,1,1,12202),-- Leather Stocking Pattern
(20153,1793,1,1,2,2919),-- Recipe: Sword of Reflection
(20153,1867,1,1,2,52928),-- Animal Skin
(20153,1872,1,1,2,52928),-- Animal Bone
(20154,1898,1,1,-1,29252),-- Blue Diamond Necklace Gem
(20154,1929,1,1,-1,36744),-- Hard Leather Gaiters Material
(20154,2178,1,1,-1,45563),-- Recipe: Dark Stockings
(20154,57,130,231,0,700000),-- Adena
(20154,846,1,1,1,526),-- Coral Earring
(20154,878,1,1,1,787),-- Blue Coral Ring
(20154,909,1,1,1,395),-- Blue Diamond Necklace
(20154,1898,1,1,1,4321),-- Blue Diamond Necklace Gem
(20154,1868,1,1,2,84117),-- Thread
(20154,1873,1,1,2,16824),-- Silver Nugget
(20154,2256,1,1,2,1029),-- Recipe: Assassin Knife
(20154,5158,1,1,2,1683),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20155,1804,1,1,-1,12857),-- Recipe: Soulshot: D-Grade
(20155,1865,1,1,-1,321422),-- Varnish
(20155,1898,1,1,-1,33017),-- Blue Diamond Necklace Gem
(20155,57,147,260,0,700000),-- Adena
(20155,846,1,1,1,594),-- Coral Earring
(20155,878,1,1,1,888),-- Blue Coral Ring
(20155,909,1,1,1,446),-- Blue Diamond Necklace
(20155,1898,1,1,1,4877),-- Blue Diamond Necklace Gem
(20155,735,1,1,2,2261),-- Potion of Alacrity
(20155,1864,1,1,2,54254),-- Stem
(20155,1869,1,1,2,27127),-- Iron Ore
(20155,1880,1,1,2,2713),-- Steel
(20155,3032,1,1,2,1086),-- Recipe: Spiritshot D
(20155,5273,1,1,2,1086),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20155,6666,1,1,2,27127),-- Deluxe Chest Key - Grade 2
(20156,1899,1,1,-1,55740),-- Necklace of Devotion Chain
(20156,2181,1,1,-1,35709),-- Recipe: White Tunic
(20156,57,189,352,0,700000),-- Adena
(20156,847,1,1,1,373),-- Red Crescent Earring
(20156,890,1,1,1,557),-- Ring of Devotion
(20156,910,1,1,1,279),-- Necklace of Devotion
(20156,1899,1,1,1,7821),-- Necklace of Devotion Chain
(20156,1061,1,1,2,20408),-- Healing Potion
(20156,1876,1,1,2,7143),-- Mithril Ore
(20156,1877,1,1,2,2387),-- Adamantite Nugget
(20156,1880,1,1,2,7143),-- Steel
(20156,1881,1,1,2,5952),-- Coarse Bone Powder
(20156,1882,1,1,2,3571),-- Leather
(20156,2181,1,1,2,4762),-- Recipe: White Tunic
(20157,1879,1,1,-1,85605),-- Cokes
(20157,2024,1,1,-1,7317),-- Elven Bow Shaft
(20157,2153,1,2,-1,79880),-- Recipe: Tiger\'s Eye Earring
(20157,57,218,431,0,700000),-- Adena
(20157,86,1,1,1,9),-- Tomahawk
(20157,179,1,1,1,9),-- Mace of Prayer
(20157,180,1,1,1,9),-- Mace of Judgment
(20157,181,1,1,1,9),-- Mace of Miracle
(20157,182,1,1,1,9),-- Doom Hammer
(20157,183,1,1,1,9),-- Mystic Staff
(20157,184,1,1,1,9),-- Conjuror\'s Staff
(20157,185,1,1,1,9),-- Staff of Mana
(20157,2022,1,1,1,438),-- Mace of Judgment Head
(20157,2023,1,1,1,348),-- Conjuror\'s Staff Head
(20157,1061,1,1,2,24390),-- Healing Potion
(20157,1829,1,1,2,14493),-- Scroll of Escape: Clan Hall
(20157,1864,1,1,2,90909),-- Stem
(20157,1869,1,1,2,37037),-- Iron Ore
(20157,1878,1,1,2,16129),-- Braided Hemp
(20157,1884,1,1,2,24390),-- Cord
(20157,2274,1,1,2,106),-- Recipe: Skull Breaker
(20157,2282,1,1,2,422),-- Recipe: Cursed Maingauche
(20158,1876,1,1,-1,113569),-- Mithril Ore
(20158,1901,1,1,-1,78432),-- Tiger\'s Eye Earring Stone
(20158,1939,1,1,-1,49877),-- Mithril Banded Gaiters Material
(20158,57,288,574,0,700000),-- Adena
(20158,436,1,1,1,199),-- Tunic of Knowledge
(20158,469,1,1,1,318),-- Stockings of Knowledge
(20158,1864,1,1,2,125000),-- Stem
(20158,1869,1,1,2,55556),-- Iron Ore
(20158,1880,1,1,2,5076),-- Steel
(20158,2271,1,1,2,282),-- Recipe: Winged Spear
(20158,2972,1,1,2,2252),-- Recipe: Brigandine Gaiters
(20158,5550,1,1,2,1126),-- Durable Metal Plate
(20160,1874,1,1,-1,27494),-- Oriharukon Ore
(20160,2152,1,1,-1,72606),-- Recipe: Enchanted Necklace
(20160,2261,1,1,-1,6404),-- Recipe: Conjuror\'s Staff
(20160,57,212,415,0,700000),-- Adena
(20160,258,1,1,1,113),-- Bagh-Nakh
(20160,1874,1,1,2,2994),-- Oriharukon Ore
(20160,1878,1,1,2,18519),-- Braided Hemp
(20160,1884,1,1,2,28571),-- Cord
(20160,1889,1,1,2,3425),-- Compound Braid
(20160,2152,1,1,2,8333),-- Recipe: Enchanted Necklace
(20160,5220,1,1,2,2053),-- Metal Hardener
(20161,1806,1,1,-1,2737),-- Recipe: Soulshot: B-Grade
(20161,1872,1,5,-1,912482),-- Animal Bone
(20161,1984,1,1,-1,29340),-- Theca Leather Armor Pattern
(20161,57,643,1305,0,700000),-- Adena
(20161,266,1,1,1,9),-- Great Pata
(20161,2120,1,1,1,1214),-- Great Pata Blade
(20161,1806,1,1,2,221),-- Recipe: Soulshot: B-Grade
(20161,1867,1,1,2,50000),-- Animal Skin
(20161,1872,1,1,2,50000),-- Animal Bone
(20161,1882,1,1,2,9524),-- Leather
(20161,1894,1,1,2,1202),-- Crafted Leather
(20161,3955,1,1,2,74),-- Recipe: Blessed Spiritshot B
(20161,4042,1,1,2,735),-- Enria
(20161,4180,1,1,2,4405),-- Recipe: Silver Arrow
(20161,5165,1,1,2,172),-- Recipe: Blessed Spiritshot (B) Compressed Package (100%)
(20161,5280,1,1,2,49),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(20161,6669,1,1,2,9174),-- Deluxe Chest Key - Grade 5
(20161,8726,1,1,200,1054),-- Life Stone: level 55
(20161,8736,1,1,200,343),-- Mid-Grade Life Stone: level 55
(20161,8746,1,1,200,25),-- High-Grade Life Stone: level 55
(20168,955,1,1,-1,1770),-- Scroll: Enchant Weapon (Grade D)
(20168,1061,1,2,-1,268170),-- Healing Potion
(20168,1877,1,1,-1,17699),-- Adamantite Nugget
(20168,57,224,448,0,700000),-- Adena
(20168,46,1,1,1,484),-- Bronze Helmet
(20168,627,1,1,1,691),-- Aspis
(20168,1865,1,1,2,47619),-- Varnish
(20168,1870,1,1,2,47619),-- Coal
(20168,1875,1,1,2,2353),-- Stone of Purity
(20168,1879,1,1,2,8772),-- Cokes
(20168,2184,1,1,2,5587),-- Recipe: Mithril Banded Gaiters
(20168,2274,1,1,2,314),-- Recipe: Skull Breaker
(20171,1880,1,2,-1,37576),-- Steel
(20171,1899,1,1,-1,58713),-- Necklace of Devotion Chain
(20171,2019,1,1,-1,8081),-- Trident Edge
(20171,57,196,374,0,700000),-- Adena
(20171,101,1,1,1,21),-- Scroll of Wisdom
(20171,312,1,1,1,21),-- Branch of Life
(20171,313,1,1,1,21),-- Temptation of Abyss
(20171,314,1,1,1,21),-- Proof of Revenge
(20171,2020,1,1,1,739),-- Temptation of Abyss Piece
(20171,1061,1,1,2,32258),-- Healing Potion
(20171,1875,1,1,2,4329),-- Stone of Purity
(20171,1879,1,1,2,7519),-- Cokes
(20171,2263,1,1,2,227),-- Recipe: Dwarven Trident
(20171,5549,1,1,2,3413),-- Metallic Thread
(20171,6666,1,1,2,41667),-- Deluxe Chest Key - Grade 2
(20176,1873,1,1,-1,239995),-- Silver Nugget
(20176,2154,1,1,-1,64239),-- Recipe: Elven Earring
(20176,2273,1,1,-1,4286),-- Recipe: Tarbar
(20176,57,304,607,0,700000),-- Adena
(20176,63,1,1,1,180),-- Gauntlets
(20176,553,1,1,1,180),-- Iron Boots
(20176,1940,1,1,1,6802),-- Iron Boots Design
(20176,2422,1,1,1,180),-- Reinforced Leather Boots
(20176,2423,1,1,1,180),-- Boots of Knowledge
(20176,2446,1,1,1,180),-- Reinforced Leather Gloves
(20176,2447,1,1,1,180),-- Gloves of Knowledge
(20176,1867,1,1,2,71429),-- Animal Skin
(20176,1872,1,1,2,71429),-- Animal Bone
(20176,1882,1,1,2,11236),-- Leather
(20176,2286,1,1,2,296),-- Recipe: Bonebreaker
(20176,2977,1,1,2,7519),-- Recipe: Brigandine Boots
(20176,2981,1,1,2,6369),-- Recipe: Elven Mithril Gloves
(20176,2985,1,1,2,6897),-- Recipe: Plate Boots
(20190,1864,1,3,-1,889140),-- Stem
(20190,1900,1,1,-1,48640),-- Enchanted Necklace Chain
(20190,2025,1,1,-1,6157),-- Dwarven Trident Edge
(20190,57,226,448,0,700000),-- Adena
(20190,275,1,1,1,26),-- Long Bow
(20190,276,1,1,1,26),-- Elven Bow
(20190,2024,1,1,1,1154),-- Elven Bow Shaft
(20190,1866,1,1,2,34483),-- Suede
(20190,1871,1,1,2,55556),-- Charcoal
(20190,1885,1,1,2,5556),-- High Grade Suede
(20190,2153,1,1,2,12346),-- Recipe: Tiger\'s Eye Earring
(20190,5220,1,1,2,1712),-- Metal Hardener
(20191,1900,1,1,-1,49380),-- Enchanted Necklace Chain
(20191,2153,1,1,-1,70191),-- Recipe: Tiger\'s Eye Earring
(20191,2264,1,1,-1,4667),-- Recipe: Two-handed Sword
(20191,57,228,457,0,700000),-- Adena
(20191,605,1,1,1,741),-- Leather Gauntlets
(20191,1123,1,1,1,741),-- Blue Buckskin Boots
(20191,1867,1,1,2,55556),-- Animal Skin
(20191,1872,1,1,2,62500),-- Animal Bone
(20191,1882,1,1,2,11236),-- Leather
(20191,2185,1,1,2,4926),-- Recipe: Iron Boots
(20191,2274,1,1,2,252),-- Recipe: Skull Breaker
(20191,2279,1,1,2,289),-- Recipe: War Pick
(20191,6667,1,1,2,25641),-- Deluxe Chest Key - Grade 3
(20192,1901,1,1,-1,81246),-- Tiger\'s Eye Earring Stone
(20192,2146,1,1,-1,42016),-- Recipe: Artisan\'s frame
(20192,2447,1,1,-1,2316),-- Gloves of Knowledge
(20192,57,248,496,0,700000),-- Adena
(20192,223,1,1,1,13),-- Kukuri
(20192,238,1,1,1,13),-- Dagger of Mana
(20192,239,1,1,1,13),-- Mystic Knife
(20192,241,1,1,1,13),-- Shilen Knife
(20192,2028,1,1,1,744),-- Shilen Knife Edge
(20192,1864,1,1,2,90909),-- Stem
(20192,1869,1,1,2,52632),-- Iron Ore
(20192,1878,1,1,2,20833),-- Braided Hemp
(20192,1884,1,1,2,29412),-- Cord
(20192,2978,1,1,2,3597),-- Recipe: Elven Mithril Boots
(20192,8307,1,1,2,342),-- Recipe: Priest Mace (100%)
(20192,6667,1,1,2,41667),-- Deluxe Chest Key - Grade 3
(20193,1874,1,1,-1,42231),-- Oriharukon Ore
(20193,1882,1,1,-1,140771),-- Leather
(20193,2027,1,1,-1,8299),-- Spiked Club Head
(20193,57,330,642,0,700000),-- Adena
(20193,47,1,1,1,443),-- Helmet
(20193,628,1,1,1,632),-- Hoplon
(20193,1341,10,30,2,200000),-- Bone Arrow
(20193,1867,1,1,2,76923),-- Animal Skin
(20193,1872,1,1,2,76923),-- Animal Bone
(20193,1882,1,1,2,14286),-- Leather
(20193,1894,1,1,2,1689),-- Crafted Leather
(20193,2271,1,1,2,344),-- Recipe: Winged Spear
(20193,2286,1,1,2,268),-- Recipe: Bonebreaker
(20197,1876,1,1,-1,76167),-- Mithril Ore
(20197,2020,1,1,-1,6667),-- Temptation of Abyss Piece
(20197,2259,1,1,-1,5914),-- Recipe: Spinebone Sword
(20197,57,197,380,0,700000),-- Adena
(20197,847,1,1,1,398),-- Red Crescent Earring
(20197,890,1,1,1,594),-- Ring of Devotion
(20197,910,1,1,1,298),-- Necklace of Devotion
(20197,1899,1,1,1,8350),-- Necklace of Devotion Chain
(20197,1341,10,30,2,100000),-- Bone Arrow
(20197,1876,1,1,2,6369),-- Mithril Ore
(20197,1877,1,1,2,637),-- Adamantite Nugget
(20197,1880,1,1,2,2545),-- Steel
(20197,1881,1,1,2,3817),-- Coarse Bone Powder
(20197,1882,1,1,2,6993),-- Leather
(20197,8294,1,1,2,1912),-- Recipe: Steel Sword (100%)
(20197,2178,1,1,2,5102),-- Recipe: Dark Stockings
(20198,1832,1,3,-1,479579),-- Greater Antidote
(20198,57,220,435,0,700000),-- Adena
(20198,259,1,1,1,73),-- Single-Edged Jamadhr
(20198,1874,1,1,2,2513),-- Oriharukon Ore
(20198,1878,1,1,2,14925),-- Braided Hemp
(20198,1884,1,1,2,23256),-- Cord
(20198,1889,1,1,2,2427),-- Compound Braid
(20198,2183,1,1,2,3289),-- Recipe: Mithril Banded Mail
(20198,2274,1,1,2,173),-- Recipe: Skull Breaker
(20198,2982,1,1,2,4587),-- Recipe: Brigandine Shield
(20198,5220,1,1,2,1473),-- Metal Hardener
(20199,1834,1,2,-1,585058),-- Emergency Dressing
(20199,1882,1,1,-1,117012),-- Leather
(20199,1900,1,2,-1,57610),-- Enchanted Necklace Chain
(20199,57,267,533,0,700000),-- Adena
(20199,605,1,1,1,864),-- Leather Gauntlets
(20199,1123,1,1,1,864),-- Blue Buckskin Boots
(20199,1867,1,1,2,83333),-- Animal Skin
(20199,1872,1,1,2,83333),-- Animal Bone
(20199,1882,1,1,2,15873),-- Leather
(20199,2250,1,1,2,16949),-- Recipe: Bone Arrow
(20199,6035,1,1,2,3650); -- Magic Haste Potion

INSERT INTO `droplist` VALUES 
(20200,1061,1,2,-1,282061),-- Healing Potion
(20200,1877,1,1,-1,18616),-- Adamantite Nugget
(20200,2183,1,1,-1,30618),-- Recipe: Mithril Banded Mail
(20200,57,236,471,0,700000),-- Adena
(20200,433,1,1,1,64),-- Elven Tunic
(20200,434,1,1,1,64),-- White Tunic
(20200,435,1,1,1,64),-- Mystic\'s Tunic
(20200,466,1,1,1,102),-- Elven Stockings
(20200,467,1,1,1,102),-- Dark Stockings
(20200,468,1,1,1,102),-- Mystic\'s Stockings
(20200,1933,1,1,1,3741),-- Dark Stocking Fabric
(20200,1936,1,1,1,2602),-- White Tunic Pattern
(20200,955,1,1,2,64),-- Scroll: Enchant Weapon (Grade D)
(20200,1834,1,1,2,34483),-- Emergency Dressing
(20200,1875,1,1,2,2114),-- Stone of Purity
(20200,1879,1,1,2,4673),-- Cokes
(20200,1885,1,1,2,2817),-- High Grade Suede
(20200,2188,1,1,2,2564),-- Recipe: Manticore Skin Gaiters
(20200,5549,1,1,2,3584),-- Metallic Thread
(20200,5550,1,1,2,705),-- Durable Metal Plate
(20200,6667,1,1,2,21277),-- Deluxe Chest Key - Grade 3
(20201,466,1,1,-1,1772),-- Elven Stockings
(20201,1870,1,2,-1,553893),-- Coal
(20201,1933,1,1,-1,65164),-- Dark Stocking Fabric
(20201,57,280,562,0,700000),-- Adena
(20201,392,1,1,1,303),-- Lion Skin Shirt
(20201,414,1,1,1,485),-- Lion Skin Gaiters
(20201,737,1,1,2,5682),-- Scroll of Resurrection
(20201,1876,1,1,2,10000),-- Mithril Ore
(20201,1877,1,1,2,2532),-- Adamantite Nugget
(20201,1880,1,1,2,4587),-- Steel
(20201,1881,1,1,2,6803),-- Coarse Bone Powder
(20201,1882,1,1,2,10753),-- Leather
(20201,6387,1,1,2,1689),-- Blessed Scroll of Resurrection for Pets
(20202,1872,1,3,-1,734933),-- Animal Bone
(20202,2154,1,1,-1,59015),-- Recipe: Elven Earring
(20202,2198,1,1,-1,11556),-- Recipe: Sage\'s Rag
(20202,57,279,557,0,700000),-- Adena
(20202,849,1,1,1,228),-- Tiger\'s Eye Earring
(20202,880,1,1,1,343),-- Black Pearl Ring
(20202,912,1,1,1,171),-- Near Forest Necklace
(20202,1901,1,1,1,10120),-- Tiger\'s Eye Earring Stone
(20202,1874,1,1,2,3205),-- Oriharukon Ore
(20202,1878,1,1,2,17241),-- Braided Hemp
(20202,1884,1,1,2,31250),-- Cord
(20202,1889,1,1,2,2924),-- Compound Braid
(20202,2272,1,1,2,458),-- Recipe: Sword of Revolution
(20202,2274,1,1,2,229),-- Recipe: Skull Breaker
(20202,5220,1,1,2,1946),-- Metal Hardener
(20203,1882,1,2,-1,59453),-- Leather
(20203,1897,1,1,-1,63700),-- Necklace of Wisdom Chain
(20203,57,124,215,0,700000),-- Adena
(20203,176,1,1,1,116),-- Journeyman\'s Staff
(20203,2501,1,1,1,116),-- Bone Club
(20203,1867,1,1,2,42148),-- Animal Skin
(20203,1872,1,1,2,42148),-- Animal Bone
(20203,1882,1,1,2,7025),-- Leather
(20203,2174,1,1,2,13744),-- Recipe: Hard Leather Gaiters
(20203,5273,1,1,2,1265),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20204,1867,1,2,-1,457600),-- Animal Skin
(20204,1868,1,3,-1,686400),-- Thread
(20204,2014,1,1,-1,3945),-- Iron Hammer Head
(20204,57,157,277,0,700000),-- Adena
(20204,846,1,1,1,634),-- Coral Earring
(20204,878,1,1,1,948),-- Blue Coral Ring
(20204,909,1,1,1,476),-- Blue Diamond Necklace
(20204,1898,1,1,1,5207),-- Blue Diamond Necklace Gem
(20204,1831,1,1,2,108135),-- Antidote
(20204,1867,1,1,2,54068),-- Animal Skin
(20204,1872,1,1,2,54068),-- Animal Bone
(20204,1882,1,1,2,9012),-- Leather
(20204,3953,1,1,2,1623),-- Recipe: Blessed Spiritshot D
(20205,1868,1,3,-1,678960),-- Thread
(20205,1871,1,2,-1,339480),-- Charcoal
(20205,1928,1,1,-1,49451),-- Piece Bone Gaiters Fragment
(20205,57,151,278,0,700000),-- Adena
(20205,257,1,1,1,164),-- Viper\'s Fang
(20205,1868,1,1,2,100278),-- Thread
(20205,1873,1,1,2,20056),-- Silver Nugget
(20205,2255,1,1,2,1226),-- Recipe: Saber
(20205,6037,1,1,2,100278),-- Waking Scroll
(20206,376,1,1,-1,1327),-- Iron Plate Gaiters
(20206,1899,1,1,-1,54106),-- Necklace of Devotion Chain
(20206,2182,1,1,-1,41570),-- Recipe: Scale Gaiters
(20206,57,184,342,0,700000),-- Adena
(20206,391,1,1,1,311),-- Puma Skin Shirt
(20206,413,1,1,1,497),-- Puma Skin Gaiters
(20206,1864,1,1,2,80000),-- Stem
(20206,1869,1,1,2,43478),-- Iron Ore
(20206,1880,1,1,2,4608),-- Steel
(20206,2182,1,1,2,1536),-- Recipe: Scale Gaiters
(20206,6666,1,1,2,50000),-- Deluxe Chest Key - Grade 2
(20207,2152,1,1,-1,120636),-- Recipe: Enchanted Necklace
(20207,2181,1,2,-1,68590),-- Recipe: White Tunic
(20207,2263,1,1,-1,10640),-- Recipe: Dwarven Trident
(20207,57,357,682,0,700000),-- Adena
(20207,220,1,1,1,188),-- Crafted Dagger
(20207,1865,1,1,2,90909),-- Varnish
(20207,1870,1,1,2,90909),-- Coal
(20207,1875,1,1,2,6579),-- Stone of Purity
(20207,1879,1,1,2,14085),-- Cokes
(20207,2152,1,1,2,11765),-- Recipe: Enchanted Necklace
(20208,1869,1,2,-1,715437),-- Iron Ore
(20208,1872,1,3,-1,476958),-- Animal Bone
(20208,1931,1,1,-1,93521),-- Leather Boot Lining
(20208,57,371,714,0,700000),-- Adena
(20208,45,1,1,1,856),-- Bone Helmet
(20208,626,1,1,1,1223),-- Bronze Shield
(20208,1932,1,1,1,21245),-- Bone Helmet Design
(20208,1866,1,1,2,43478),-- Suede
(20208,1871,1,1,2,66667),-- Charcoal
(20208,1885,1,1,2,6135),-- High Grade Suede
(20208,2180,1,1,2,4785),-- Recipe: Scale Mail
(20208,5220,1,1,2,2469),-- Metal Hardener
(20208,6666,1,1,2,66667),-- Deluxe Chest Key - Grade 2
(20209,45,1,1,-1,4949),-- Bone Helmet
(20209,1889,1,1,-1,51639),-- Compound Braid
(20209,2178,1,1,-1,123934),-- Recipe: Dark Stockings
(20209,57,398,779,0,700000),-- Adena
(20209,274,1,1,1,213),-- Strengthened Bow
(20209,1061,1,1,2,71429),-- Healing Potion
(20209,1867,1,1,2,142857),-- Animal Skin
(20209,1872,1,1,2,125000),-- Animal Bone
(20209,1882,1,1,2,21739),-- Leather
(20209,2178,1,1,2,16393),-- Recipe: Dark Stockings
(20210,377,1,1,-1,1890),-- Scale Gaiters
(20210,1900,1,2,-1,86113),-- Enchanted Necklace Chain
(20210,2183,1,1,-1,51781),-- Recipe: Mithril Banded Mail
(20210,57,400,793,0,700000),-- Adena
(20210,348,1,1,1,162),-- Scale Mail
(20210,377,1,1,1,259),-- Scale Gaiters
(20210,1935,1,1,1,6008),-- Scale Mail Fragment
(20210,1937,1,1,1,9238),-- Scale Gaiters Fragment
(20210,1868,1,1,2,166667),-- Thread
(20210,1873,1,1,2,32258),-- Silver Nugget
(20210,1889,1,1,2,6211),-- Compound Braid
(20210,2184,1,1,2,7692),-- Recipe: Mithril Banded Gaiters
(20210,2274,1,1,2,733),-- Recipe: Skull Breaker
(20210,5549,1,1,2,8065),-- Metallic Thread
(20211,1865,1,3,-1,839475),-- Varnish
(20211,1869,1,6,-1,839475),-- Iron Ore
(20211,57,426,850,0,700000),-- Adena
(20211,86,1,1,1,15),-- Tomahawk
(20211,179,1,1,1,15),-- Mace of Prayer
(20211,180,1,1,1,15),-- Mace of Judgment
(20211,181,1,1,1,15),-- Mace of Miracle
(20211,182,1,1,1,15),-- Doom Hammer
(20211,183,1,1,1,15),-- Mystic Staff
(20211,184,1,1,1,15),-- Conjuror\'s Staff
(20211,185,1,1,1,15),-- Staff of Mana
(20211,2022,1,1,1,715),-- Mace of Judgment Head
(20211,2023,1,1,1,569),-- Conjuror\'s Staff Head
(20211,1864,1,1,2,166667),-- Stem
(20211,1869,1,1,2,90909),-- Iron Ore
(20211,1880,1,1,2,9259),-- Steel
(20211,2264,1,1,2,543),-- Recipe: Two-handed Sword
(20211,2280,1,1,2,271),-- Recipe: Crucifix of Blood
(20213,1884,1,5,-1,878893),-- Cord
(20213,1894,1,2,-1,50112),-- Crafted Leather
(20213,2292,1,1,-1,7934),-- Recipe: Light Crossbow
(20213,57,1401,2795,0,700000),-- Adena
(20213,352,1,1,1,83),-- Brigandine Tunic
(20213,1941,1,1,1,4893),-- Brigandine Temper
(20213,2377,1,1,1,132),-- Mithril Scale Gaiters
(20213,2378,1,1,1,132),-- Brigandine Gaiters
(20213,2918,1,1,1,7300),-- Mithril Scale Gaiters Material
(20213,2919,1,1,1,7300),-- Brigandine Gaiters Material
(20213,1341,80,240,2,58824),-- Bone Arrow
(20213,1864,9,27,2,14085),-- Stem
(20213,1869,2,6,2,27027),-- Iron Ore
(20213,1878,1,1,2,52632),-- Braided Hemp
(20213,1884,1,1,2,83333),-- Cord
(20213,2286,1,1,2,588),-- Recipe: Bonebreaker
(20213,2290,1,1,2,588),-- Recipe: Mithril Dagger
(20213,2983,1,1,2,8696),-- Recipe: Plate Helmet
(20214,1877,1,1,-1,52984),-- Adamantite Nugget
(20214,1879,1,1,-1,220767),-- Cokes
(20214,2997,1,2,-1,105127),-- Recipe: Karmian Gloves
(20214,57,883,1766,0,700000),-- Adena
(20214,851,1,1,1,184),-- Omen Beast\'s Eye Earring
(20214,882,1,1,1,275),-- Mithril Ring
(20214,914,1,1,1,137),-- Necklace of Darkness
(20214,1905,1,1,1,10784),-- Omen Beast\'s Eye Earring Gemstone
(20214,1906,1,1,1,15238),-- Mithril Ring Wire
(20214,1907,1,1,1,7712),-- Necklace of Darkness Gem
(20214,1805,1,1,2,393),-- Recipe: Soulshot: C-Grade
(20214,1865,1,1,2,166667),-- Varnish
(20214,1870,1,1,2,142857),-- Coal
(20214,1875,1,1,2,10753),-- Stone of Purity
(20214,1879,1,1,2,26316),-- Cokes
(20214,5269,1,1,2,393),-- Recipe: Greater Soulshot (C) Compressed Package(100%)
(20215,1872,1,7,-1,711850),-- Animal Bone
(20215,2206,1,1,-1,42260),-- Recipe: Mithril Shirt
(20215,4469,1,1,-1,9152),-- Dye of Strength <Str+1 Con-1>
(20215,57,1014,2048,0,700000),-- Adena
(20215,297,1,1,1,45),-- Glaive
(20215,2055,1,1,1,3326),-- Glaive Edge
(20215,1866,1,1,2,90909),-- Suede
(20215,1871,1,1,2,142857),-- Charcoal
(20215,1885,1,1,2,10526),-- High Grade Suede
(20215,2205,1,1,2,5263),-- Recipe: Chain Gaiters
(20215,2212,1,1,2,4386),-- Recipe: Plated Leather Gaiters
(20215,2299,1,1,2,876),-- Recipe: Big Hammer
(20215,2989,1,1,2,6993),-- Recipe: Chain Hood
(20215,5220,1,1,2,3953),-- Metal Hardener
(20216,1905,1,1,-1,133555),-- Omen Beast\'s Eye Earring Gemstone
(20216,2297,1,1,-1,5995),-- Recipe: Flamberge
(20216,2933,1,1,-1,153230),-- Sage\'s Worn Gloves Lining
(20216,57,891,1806,0,700000),-- Adena
(20216,61,1,1,1,118),-- Mithril Gloves
(20216,607,1,1,1,118),-- Ogre Power Gauntlets
(20216,1125,1,1,1,118),-- Assault Boots
(20216,1949,1,1,1,6412),-- Mithril Gloves Design
(20216,1957,1,1,1,6412),-- Assault Boots Part
(20216,1963,1,1,1,6412),-- Ogre Power Gauntlets Part
(20216,2427,1,1,1,118),-- Salamander Skin Boots
(20216,2428,1,1,1,118),-- Plate Boots
(20216,2451,1,1,1,118),-- Sage\'s Worn Gloves
(20216,2931,1,1,1,6412),-- Salamander Skin Boot Lining
(20216,2932,1,1,1,6412),-- Plate Boots Design
(20216,2933,1,1,1,6412),-- Sage\'s Worn Gloves Lining
(20216,1867,1,3,2,125000),-- Animal Skin
(20216,1872,1,1,2,250000),-- Animal Bone
(20216,1882,1,1,2,35714),-- Leather
(20216,1894,1,1,2,3802),-- Crafted Leather
(20216,2314,1,1,2,950),-- Recipe: Body Slasher
(20216,5269,1,1,2,384),-- Recipe: Greater Soulshot (C) Compressed Package(100%)
(20217,1873,1,3,-1,643598),-- Silver Nugget
(20217,2075,1,1,-1,12620),-- Orcish Glaive Blade
(20217,2163,1,1,-1,82092),-- Recipe: Earring of Protection
(20217,57,646,1324,0,700000),-- Adena
(20217,62,1,1,1,68),-- Mithril Boots
(20217,1958,1,1,1,3798),-- Mithril Boots Design
(20217,2429,1,1,1,68),-- Chain Boots
(20217,2430,1,1,1,68),-- Karmian Boots
(20217,2431,1,1,1,68),-- Plated Leather Boots
(20217,2432,1,1,1,68),-- Dwarven Chain Boots
(20217,2433,1,1,1,68),-- Boots of Seal
(20217,2452,1,1,1,68),-- Reinforced Mithril Gloves
(20217,2453,1,1,1,68),-- Chain Gloves
(20217,2454,1,1,1,68),-- Karmian Gloves
(20217,2937,1,1,1,3798),-- Chain Boots Part
(20217,2938,1,1,1,3798),-- Karmian Boots Fabric
(20217,2939,1,1,1,3798),-- Plate Leather Boot Lining
(20217,2940,1,1,1,3798),-- Dwarven Chain Boots Part
(20217,2941,1,1,1,3798),-- Boots of Seal Pattern
(20217,2942,1,1,1,3798),-- Reinforced Mithril Gloves Design
(20217,2943,1,1,1,3798),-- Chain Gloves Design
(20217,2944,1,1,1,3798),-- Karmian Gloves Fabric
(20217,1868,1,1,2,166667),-- Thread
(20217,1873,1,1,2,29412),-- Silver Nugget
(20217,1889,1,1,2,3984),-- Compound Braid
(20217,2163,1,1,2,2985),-- Recipe: Earring of Protection
(20217,2321,1,1,2,994),-- Recipe: Stiletto
(20217,5279,1,1,2,994),-- Recipe: Greater Blessed Spiritshot (C) Compressed Package(100%)
(20217,5549,1,1,2,9901),-- Metallic Thread
(20217,8723,1,1,200,2985),-- Life Stone: level 46
(20217,8733,1,1,200,994),-- Mid-Grade Life Stone: level 46
(20218,1876,1,1,-1,349385),-- Mithril Ore
(20218,1908,1,1,-1,165429),-- Moonstone Earring Wire
(20218,2073,1,1,-1,13503),-- Chakram Edge
(20218,57,1094,2185,0,700000),-- Adena
(20218,631,1,1,1,207),-- Eldarake
(20218,1964,1,1,1,11120),-- Eldarake Temper
(20218,2413,1,1,1,146),-- Chain Hood
(20218,2495,1,1,1,207),-- Chain Shield
(20218,2936,1,1,1,10100),-- Chain Hood Pattern
(20218,2945,1,1,1,11120),-- Chain Shield Fragment
(20218,1864,1,1,2,250000),-- Stem
(20218,1869,1,1,2,83333),-- Iron Ore
(20218,1878,1,1,2,27027),-- Braided Hemp
(20218,1884,1,1,2,20408),-- Cord
(20218,2217,1,1,2,6107),-- Recipe: Knight\'s Shield
(20218,2319,1,1,2,360),-- Recipe: Raid Sword
(20218,2320,1,1,2,360),-- Recipe: Cursed Staff
(20218,4039,1,1,2,3441),-- Mold Glue
(20218,8723,1,1,200,13514),-- Life Stone: level 46
(20218,8733,1,1,200,6803),-- Mid-Grade Life Stone: level 46
(20219,1873,1,3,-1,628922),-- Silver Nugget
(20219,2060,1,1,-1,12485),-- Stormbringer Blade
(20219,2164,1,1,-1,120947),-- Recipe: Ring of Protection
(20219,57,1209,2473,0,700000),-- Adena
(20219,281,1,1,1,33),-- Crystallized Ice Bow
(20219,2074,1,1,1,2632),-- Crystallized Ice Bow Shaft
(20219,1865,1,1,2,125000),-- Varnish
(20219,1870,1,1,2,125000),-- Coal
(20219,1874,1,1,2,6452),-- Oriharukon Ore
(20219,2230,1,1,2,2387),-- Recipe: Theca Leather Gaiters
(20219,2326,1,1,2,477),-- Recipe: Horn of Glory
(20219,4039,1,1,2,2865),-- Mold Glue
(20219,4043,1,1,2,5235),-- Asofe
(20219,8723,1,1,200,6211),-- Life Stone: level 46
(20219,8733,1,1,200,1193),-- Mid-Grade Life Stone: level 46
(20220,1895,1,1,-1,494901),-- Metallic Fiber
(20220,1911,1,1,-1,128833),-- Earring of Protection Gemstone
(20220,2944,1,1,-1,155699),-- Karmian Gloves Fabric
(20220,57,1229,2489,0,700000),-- Adena
(20220,1126,1,1,1,80),-- Crimson Boots
(20220,1974,1,1,1,5379),-- Crimson Boot Fabric
(20220,2434,1,1,1,80),-- Rind Leather Boots
(20220,2455,1,1,1,86),-- Plated Leather Gloves
(20220,2456,1,1,1,80),-- Dwarven Chain Gloves
(20220,2457,1,1,1,80),-- Gloves of Seal
(20220,2458,1,1,1,80),-- Rind Leather Gloves
(20220,2946,1,1,1,3575),-- Plate Leather Gloves Design
(20220,2948,1,1,1,5379),-- Rind Leather Boot Lining
(20220,2949,1,1,1,5379),-- Dwarven Chain Gloves Design
(20220,2950,1,1,1,5379),-- Gloves of Seal Pattern
(20220,2951,1,1,1,5379),-- Rind Leather Gloves Design
(20220,951,1,1,2,175),-- Scroll: Enchant Weapon (Grade C)
(20220,1866,1,1,2,76923),-- Suede
(20220,1871,1,1,2,111111),-- Charcoal
(20220,1877,1,1,2,4405),-- Adamantite Nugget
(20220,2343,1,1,2,401),-- Recipe: Mace of the Underworld
(20220,2345,1,1,2,401),-- Recipe: Dark Screamer
(20220,4040,1,1,2,2410),-- Mold Lubricant
(20220,4041,1,1,2,1205),-- Mold Hardener
(20220,8724,1,1,200,4016),-- Life Stone: level 49
(20220,8734,1,1,200,183),-- Mid-Grade Life Stone: level 49
(20221,951,1,1,-1,3773),-- Scroll: Enchant Weapon (Grade C)
(20221,2073,1,1,-1,16041),-- Chakram Edge
(20221,2352,1,1,-1,4342),-- Recipe: Widow Maker
(20221,57,1763,3582,0,700000),-- Adena
(20221,1126,1,1,1,96),-- Crimson Boots
(20221,1974,1,1,1,6444),-- Crimson Boot Fabric
(20221,2434,1,1,1,96),-- Rind Leather Boots
(20221,2455,1,1,1,103),-- Plated Leather Gloves
(20221,2456,1,1,1,96),-- Dwarven Chain Gloves
(20221,2457,1,1,1,96),-- Gloves of Seal
(20221,2458,1,1,1,96),-- Rind Leather Gloves
(20221,2946,1,1,1,4283),-- Plate Leather Gloves Design
(20221,2948,1,1,1,6444),-- Rind Leather Boot Lining
(20221,2949,1,1,1,6444),-- Dwarven Chain Gloves Design
(20221,2950,1,1,1,6444),-- Gloves of Seal Pattern
(20221,2951,1,1,1,6444),-- Rind Leather Gloves Design
(20221,1539,1,1,2,20833),-- Greater Healing Potion
(20221,1867,1,1,2,200000),-- Animal Skin
(20221,1872,1,1,2,200000),-- Animal Bone
(20221,1875,1,1,2,9524),-- Stone of Purity
(20221,1881,1,1,2,16949),-- Coarse Bone Powder
(20221,2334,1,1,2,214),-- Recipe: Sword of Whispering Death
(20221,2351,1,1,2,193),-- Recipe: Scorpion
(20221,3005,1,1,2,6329),-- Recipe: Demon\'s Boots
(20221,4041,1,1,2,800),-- Mold Hardener
(20221,6668,1,1,2,47619),-- Deluxe Chest Key - Grade 4
(20221,8724,1,1,200,3165),-- Life Stone: level 49
(20221,8734,1,1,200,1264),-- Mid-Grade Life Stone: level 49
(20221,8744,1,1,200,1264),-- High-Grade Life Stone: level 49
(20222,1876,1,1,-1,366230),-- Mithril Ore
(20222,1895,1,3,-1,523186),-- Metallic Fiber
(20222,2168,1,1,-1,46594),-- Recipe: Necklace of Mermaid
(20222,57,1340,2733,0,700000),-- Adena
(20222,500,1,1,1,112),-- Great Helmet
(20222,632,1,1,1,161),-- Knight\'s Shield
(20222,1971,1,1,1,8279),-- Great Helmet Material
(20222,1972,1,1,1,10819),-- Knight\'s Shield Fragment
(20222,2496,1,1,1,174),-- Dwarven Chain Shield
(20222,2947,1,1,1,9660),-- Dwarven Chain Shield Fragment
(20222,1539,1,1,2,23256),-- Greater Healing Potion
(20222,1868,1,1,2,250000),-- Thread
(20222,1873,1,1,2,43478),-- Silver Nugget
(20222,1876,1,1,2,18868),-- Mithril Ore
(20222,1895,1,1,2,38462),-- Metallic Fiber
(20222,2222,1,1,2,432),-- Recipe: Composite Armor
(20222,3006,1,1,2,6494),-- Recipe: Demon\'s Gloves
(20222,4042,1,1,2,3021),-- Enria
(20222,8724,1,1,200,6897),-- Life Stone: level 49
(20222,8734,1,1,200,864),-- Mid-Grade Life Stone: level 49
(20223,734,1,1,-1,49539),-- Haste Potion
(20223,1864,1,1,-1,594474),-- Stem
(20223,2011,1,1,-1,6781),-- Short Spear Edge
(20223,57,137,239,0,700000),-- Adena
(20223,845,1,1,1,988),-- Cat\'s Eye Earring
(20223,877,1,1,1,1484),-- Ring of Wisdom
(20223,908,1,1,1,738),-- Necklace of Wisdom
(20223,1897,1,1,1,10453),-- Necklace of Wisdom Chain
(20223,1864,1,1,2,87799),-- Stem
(20223,1869,1,1,2,43900),-- Iron Ore
(20223,1878,1,1,2,17560),-- Braided Hemp
(20223,2175,1,1,2,35691),-- Recipe: Boots
(20224,1865,1,3,-1,389152),-- Varnish
(20224,1898,1,1,-1,39975),-- Blue Diamond Necklace Gem
(20224,2178,1,1,-1,62264),-- Recipe: Dark Stockings
(20224,57,177,317,0,700000),-- Adena
(20224,27,1,1,1,312),-- Hard Leather Shirt
(20224,33,1,1,1,500),-- Hard Leather Gaiters
(20224,1927,1,1,1,4586),-- Hard Leather Shirt Pattern
(20224,1929,1,1,1,7417),-- Hard Leather Gaiters Material
(20224,1867,1,1,2,61307),-- Animal Skin
(20224,1872,1,1,2,61307),-- Animal Bone
(20224,1881,1,1,2,6131),-- Coarse Bone Powder
(20224,5153,1,1,2,1840),-- Recipe: Soulshot (D) Compressed Package (100%)
(20224,5268,1,1,2,1840),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20225,1931,1,1,-1,45718),-- Leather Boot Lining
(20225,2152,1,1,-1,61574),-- Recipe: Enchanted Necklace
(20225,2259,1,1,-1,5431),-- Recipe: Spinebone Sword
(20225,57,185,346,0,700000),-- Adena
(20225,40,1,1,1,470),-- Leather Boots
(20225,604,1,1,1,470),-- Crafted Leather Gloves
(20225,1931,1,1,1,6415),-- Leather Boot Lining
(20225,1934,1,1,1,6415),-- Crafted Leather Gloves Lining
(20225,1865,1,1,2,38462),-- Varnish
(20225,1870,1,1,2,41667),-- Coal
(20225,1875,1,1,2,3205),-- Stone of Purity
(20225,1879,1,1,2,5988),-- Cokes
(20225,2259,1,1,2,855),-- Recipe: Spinebone Sword
(20225,5437,1,1,2,855),-- Recipe: Heavy Sword(100%)
(20226,413,1,1,-1,1884),-- Puma Skin Gaiters
(20226,1868,1,1,-1,736790),-- Thread
(20226,1899,1,1,-1,57562),-- Necklace of Devotion Chain
(20226,57,192,367,0,700000),-- Adena
(20226,123,1,1,1,26),-- Saber
(20226,2017,1,1,1,687),-- Saber Blade
(20226,5285,1,1,1,26),-- Heavy Sword
(20226,5531,1,1,1,868),-- Heavy Sword Edge
(20226,1866,1,1,2,41667),-- Suede
(20226,1871,1,1,2,62500),-- Charcoal
(20226,2178,1,1,2,12048),-- Recipe: Dark Stockings
(20226,5220,1,1,2,1845),-- Metal Hardener
(20227,1899,1,1,-1,60696),-- Necklace of Devotion Chain
(20227,2178,1,1,-1,62152),-- Recipe: Dark Stockings
(20227,2260,1,1,-1,6032),-- Recipe: Mace of Judgment
(20227,57,201,388,0,700000),-- Adena
(20227,391,1,1,1,349),-- Puma Skin Shirt
(20227,413,1,1,1,558),-- Puma Skin Gaiters
(20227,1867,1,1,2,66667),-- Animal Skin
(20227,1872,1,1,2,66667),-- Animal Bone
(20227,1882,1,1,2,13158),-- Leather
(20227,2181,1,1,2,5155),-- Recipe: White Tunic
(20227,6035,1,1,2,4425),-- Magic Haste Potion
(20228,1884,1,1,-1,273691),-- Cord
(20228,1931,1,1,-1,58137),-- Leather Boot Lining
(20228,2180,1,1,-1,33440),-- Recipe: Scale Mail
(20228,57,229,448,0,700000),-- Adena
(20228,847,1,1,1,464),-- Red Crescent Earring
(20228,890,1,1,1,694),-- Ring of Devotion
(20228,910,1,1,1,348),-- Necklace of Devotion
(20228,1899,1,1,1,9751),-- Necklace of Devotion Chain
(20228,1831,1,1,2,125000),-- Antidote
(20228,1868,1,1,2,100000),-- Thread
(20228,1873,1,1,2,20408),-- Silver Nugget
(20228,1889,1,1,2,3584),-- Compound Braid
(20228,2180,1,1,2,4878),-- Recipe: Scale Mail
(20228,5549,1,1,2,6369),-- Metallic Thread
(20229,414,1,1,-1,1573),-- Lion Skin Gaiters
(20229,1061,1,1,-1,297908),-- Healing Potion
(20229,1864,1,3,-1,491548),-- Stem
(20229,57,249,498,0,700000),-- Adena
(20229,848,1,1,1,316),-- Enchanted Earring
(20229,879,1,1,1,473),-- Enchanted Ring
(20229,911,1,1,1,237),-- Enchanted Necklace
(20229,1900,1,1,1,7348),-- Enchanted Necklace Chain
(20229,1832,1,1,2,50000),-- Greater Antidote
(20229,1864,1,1,2,90909),-- Stem
(20229,1869,1,1,2,43478),-- Iron Ore
(20229,1880,1,1,2,3497),-- Steel
(20229,2265,1,1,2,570),-- Recipe: Spiked Club
(20229,2274,1,1,2,326),-- Recipe: Skull Breaker
(20229,5550,1,1,2,570),-- Durable Metal Plate
(20230,434,1,1,-1,1120),-- White Tunic
(20230,1834,1,2,-1,621516),-- Emergency Dressing
(20230,1870,1,2,-1,559364),-- Coal
(20230,57,284,566,0,700000),-- Adena
(20230,292,1,1,1,95),-- Pike
(20230,956,1,1,2,3344),-- Scroll: Enchant Armor (Grade D)
(20230,1865,1,1,2,83333),-- Varnish
(20230,1870,1,1,2,90909),-- Coal
(20230,1874,1,1,2,5618),-- Oriharukon Ore
(20231,1832,1,2,-1,602189),-- Greater Antidote
(20231,1877,1,1,-1,21679),-- Adamantite Nugget
(20231,2282,1,1,-1,3871),-- Recipe: Cursed Maingauche
(20231,57,274,550,0,700000),-- Adena
(20231,605,1,1,1,890),-- Leather Gauntlets
(20231,1123,1,1,1,890),-- Blue Buckskin Boots
(20231,955,1,1,2,293),-- Scroll: Enchant Weapon (Grade D)
(20231,1832,1,1,2,66667),-- Greater Antidote
(20231,1866,1,1,2,35714),-- Suede
(20231,1871,1,1,2,58824),-- Charcoal
(20231,1877,1,1,2,2247),-- Adamantite Nugget
(20231,2277,1,1,2,293),-- Recipe: Bich\'hwa
(20232,1901,1,1,-1,72904),-- Tiger\'s Eye Earring Stone
(20232,2027,1,1,-1,6915),-- Spiked Club Head
(20232,2147,1,1,-1,37702),-- Recipe: Blacksmith\'s frame
(20232,57,268,535,0,700000),-- Adena
(20232,47,1,1,1,369),-- Helmet
(20232,628,1,1,1,527),-- Hoplon
(20232,1867,1,1,2,71429),-- Animal Skin
(20232,1872,1,1,2,76923),-- Animal Bone
(20232,1875,1,1,2,3390),-- Stone of Purity
(20232,1881,1,1,2,6536),-- Coarse Bone Powder
(20232,2156,1,1,2,4274),-- Recipe: Elven Necklace
(20232,2191,1,1,2,6410),-- Recipe: RIP Gauntlets
(20233,955,1,1,-1,2474),-- Scroll: Enchant Weapon (Grade D)
(20233,2154,1,1,-1,66215),-- Recipe: Elven Earring
(20233,2280,1,1,-1,4417),-- Recipe: Crucifix of Blood
(20233,57,313,625,0,700000),-- Adena
(20233,849,1,1,1,256),-- Tiger\'s Eye Earring
(20233,880,1,1,1,385),-- Black Pearl Ring
(20233,912,1,1,1,192),-- Near Forest Necklace
(20233,1901,1,1,1,11355),-- Tiger\'s Eye Earring Stone
(20233,1832,1,1,2,62500),-- Greater Antidote
(20233,1868,1,1,2,111111),-- Thread
(20233,1873,1,1,2,20408),-- Silver Nugget
(20233,1876,1,1,2,9804),-- Mithril Ore
(20233,2273,1,1,2,180),-- Recipe: Tarbar
(20233,2274,1,1,2,420),-- Recipe: Skull Breaker
(20233,2975,1,1,2,5405),-- Recipe: Brigandine Helmet
(20234,1879,1,1,-1,103998),-- Cokes
(20234,1938,1,1,-1,41949),-- Mithril Banded Mail Material
(20234,2971,1,1,-1,33368),-- Recipe: Mithril Scale Gaiters
(20234,57,316,631,0,700000),-- Adena
(20234,293,1,1,1,35),-- War Hammer
(20234,296,1,1,1,35),-- Dwarven Pike
(20234,1868,1,1,2,166667),-- Thread
(20234,1873,1,1,2,31250),-- Silver Nugget
(20234,1876,1,1,2,15625),-- Mithril Ore
(20234,2188,1,1,2,5682),-- Recipe: Manticore Skin Gaiters
(20234,2979,1,1,2,9174),-- Recipe: Manticore Skin Gloves
(20235,2074,1,1,-1,6875),-- Crystallized Ice Bow Shaft
(20235,2163,1,1,-1,49522),-- Recipe: Earring of Protection
(20235,2340,1,1,-1,2257),-- Recipe: Pa\'agrian Hammer
(20235,57,479,955,0,700000),-- Adena
(20235,96,1,1,1,7),-- Scythe
(20235,298,1,1,1,7),-- Orcish Glaive
(20235,302,1,1,1,7),-- Body Slasher
(20235,2062,1,1,1,600),-- Scythe Edge
(20235,2075,1,1,1,600),-- Orcish Glaive Blade
(20235,2076,1,1,1,600),-- Body Slasher Blade
(20235,1865,1,1,2,55556),-- Varnish
(20235,1870,1,1,2,55556),-- Coal
(20235,1874,1,1,2,4587),-- Oriharukon Ore
(20235,2216,1,1,2,3205),-- Recipe: Great Helmet
(20235,4039,1,1,2,917),-- Mold Glue
(20235,4043,1,1,2,1835),-- Asofe
(20235,8723,1,1,200,2062),-- Life Stone: level 46
(20235,8733,1,1,200,688),-- Mid-Grade Life Stone: level 46
(20236,1908,1,1,-1,95118),-- Moonstone Earring Wire
(20236,2219,1,1,-1,62388),-- Recipe: Crimson Boots
(20236,3021,1,1,-1,3500),-- Recipe: Dwarven War Hammer
(20236,57,484,989,0,700000),-- Adena
(20236,62,1,1,1,42),-- Mithril Boots
(20236,1958,1,1,1,2371),-- Mithril Boots Design
(20236,2429,1,1,1,42),-- Chain Boots
(20236,2430,1,1,1,42),-- Karmian Boots
(20236,2431,1,1,1,42),-- Plated Leather Boots
(20236,2432,1,1,1,42),-- Dwarven Chain Boots
(20236,2433,1,1,1,42),-- Boots of Seal
(20236,2452,1,1,1,42),-- Reinforced Mithril Gloves
(20236,2453,1,1,1,42),-- Chain Gloves
(20236,2454,1,1,1,42),-- Karmian Gloves
(20236,2937,1,1,1,2371),-- Chain Boots Part
(20236,2938,1,1,1,2371),-- Karmian Boots Fabric
(20236,2939,1,1,1,2371),-- Plate Leather Boot Lining
(20236,2940,1,1,1,2371),-- Dwarven Chain Boots Part
(20236,2941,1,1,1,2371),-- Boots of Seal Pattern
(20236,2942,1,1,1,2371),-- Reinforced Mithril Gloves Design
(20236,2943,1,1,1,2371),-- Chain Gloves Design
(20236,2944,1,1,1,2371),-- Karmian Gloves Fabric
(20236,1865,1,1,2,62500),-- Varnish
(20236,1870,1,1,2,52632),-- Coal
(20236,1874,1,1,2,3472),-- Oriharukon Ore
(20236,2216,1,1,2,2481),-- Recipe: Great Helmet
(20236,4039,1,1,2,496),-- Mold Glue
(20236,4043,1,1,2,1488),-- Asofe
(20236,8723,1,1,200,1488),-- Life Stone: level 46
(20236,8733,1,1,200,496),-- Mid-Grade Life Stone: level 46
(20236,8753,1,1,200,496),-- Top-Grade Life Stone: level 46
(20237,951,1,1,-1,2457),-- Scroll: Enchant Weapon (Grade C)
(20237,2329,1,1,-1,3142),-- Recipe: Tsurugi
(20237,3010,1,1,-1,36717),-- Recipe: Composite Helmet
(20237,57,656,1322,0,700000),-- Adena
(20237,282,1,1,1,11),-- Elemental Bow
(20237,285,1,1,1,11),-- Noble Elven Bow
(20237,2085,1,1,1,1036),-- Crossbow Shaft
(20237,2086,1,1,1,1036),-- Elven Bow of Nobility Shaft
(20237,952,1,1,2,998),-- Scroll: Enchant Armor (Grade C)
(20237,1342,10,30,2,111111),-- Fine Steel Arrow
(20237,1864,1,1,2,125000),-- Stem
(20237,1869,1,1,2,58824),-- Iron Ore
(20237,1880,1,1,2,5952),-- Steel
(20237,2230,1,1,2,1451),-- Recipe: Theca Leather Gaiters
(20237,4043,1,1,2,1490),-- Asofe
(20237,5550,1,1,2,2976),-- Durable Metal Plate
(20237,8724,1,1,200,5952),-- Life Stone: level 49
(20237,8734,1,1,200,271),-- Mid-Grade Life Stone: level 49
(20238,1877,1,1,-1,52062),-- Adamantite Nugget
(20238,2357,1,1,-1,2123),-- Recipe: Crystal Dagger
(20238,4041,1,1,-1,11318),-- Mold Hardener
(20238,57,630,1281,0,700000),-- Adena
(20238,355,1,1,1,45),-- Dwarven Chain Mail Shirt
(20238,382,1,1,1,72),-- Dwarven Chain Gaiters
(20238,1968,1,1,1,3474),-- Dwarven Chain Mail Shirt Material
(20238,1969,1,1,1,5510),-- Dwarven Chain Gaiters Material
(20238,1866,1,1,2,40000),-- Suede
(20238,1871,1,1,2,62500),-- Charcoal
(20238,1877,1,1,2,2907),-- Adamantite Nugget
(20238,2335,1,1,2,728),-- Recipe: War Axe
(20238,2348,1,1,2,364),-- Recipe: Heathen\'s Book
(20238,4040,1,1,2,1092),-- Mold Lubricant
(20238,4041,1,1,2,728),-- Mold Hardener
(20238,6668,1,1,2,20408),-- Deluxe Chest Key - Grade 4
(20238,8724,1,1,200,1818),-- Life Stone: level 49
(20238,8734,1,1,200,364),-- Mid-Grade Life Stone: level 49
(20239,1867,1,5,-1,923295),-- Animal Skin
(20239,2079,1,1,-1,9515),-- Bec De Corbin Blade
(20239,3007,1,1,-1,50362),-- Recipe: Theca Leather Boots
(20239,57,669,1365,0,700000),-- Adena
(20239,73,1,1,1,6),-- Shamshir
(20239,74,1,1,1,6),-- Katana
(20239,131,1,1,1,6),-- Spirit Sword
(20239,133,1,1,1,6),-- Raid Sword
(20239,2077,1,1,1,510),-- Shamshir Blade
(20239,2078,1,1,1,510),-- Katana Blade
(20239,2080,1,1,1,510),-- Spirit Sword Blade
(20239,2081,1,1,1,510),-- Raid Sword Blade
(20239,952,1,1,2,601),-- Scroll: Enchant Armor (Grade C)
(20239,1867,1,1,2,76923),-- Animal Skin
(20239,1872,1,1,2,83333),-- Animal Bone
(20239,1875,1,1,2,3003),-- Stone of Purity
(20239,1881,1,1,2,7194),-- Coarse Bone Powder
(20239,2351,1,1,2,300),-- Recipe: Scorpion
(20239,3007,1,1,2,2703),-- Recipe: Theca Leather Boots
(20239,4041,1,1,2,601),-- Mold Hardener
(20239,8724,1,1,200,901),-- Life Stone: level 49
(20239,8734,1,1,200,1502),-- Mid-Grade Life Stone: level 49
(20240,2092,1,1,-1,8118),-- Homunkulus\'s Sword Blade
(20240,2353,1,1,-1,2430),-- Recipe: Samurai Longsword
(20240,4601,1,1,-1,3310),-- Greater Dye of Strength <Str+3 Con-3>
(20240,57,722,1466,0,700000),-- Adena
(20240,265,1,1,1,15),-- Fist Blade
(20240,2108,1,1,1,1650),-- Fist Blade Piece
(20240,1864,1,1,2,142857),-- Stem
(20240,1869,1,1,2,66667),-- Iron Ore
(20240,1878,1,1,2,27778),-- Braided Hemp
(20240,1884,1,1,2,41667),-- Cord
(20240,3019,1,1,2,2183),-- Recipe: Full Plate Shield
(20240,3022,1,1,2,144),-- Recipe: Yaksa Mace
(20240,4039,1,1,2,2415),-- Mold Glue
(20240,8725,1,1,200,1724),-- Life Stone: level 52
(20240,8735,1,1,200,230),-- Mid-Grade Life Stone: level 52
(20241,1539,1,2,-1,337590),-- Greater Healing Potion
(20241,2100,1,1,-1,8563),-- Paradia Staff Head
(20241,3015,1,1,-1,42434),-- Recipe: Drake Leather Gloves
(20241,57,720,1455,0,700000),-- Adena
(20241,854,1,1,1,71),-- Earring of Binding
(20241,885,1,1,1,107),-- Ring of Ages
(20241,917,1,1,1,54),-- Necklace of Mermaid
(20241,1914,1,1,1,5883),-- Earring of Binding Gemstone
(20241,1915,1,1,1,9226),-- Ring of Ages Gemstone
(20241,1916,1,1,1,4456),-- Necklace of Mermaid Teardrop
(20241,1867,1,1,2,71429),-- Animal Skin
(20241,1872,1,1,2,76923),-- Animal Bone
(20241,1875,1,1,2,3731),-- Stone of Purity
(20241,1881,1,1,2,7299),-- Coarse Bone Powder
(20241,2359,1,1,2,112),-- Recipe: Eminence Bow
(20241,3014,1,1,2,1218),-- Recipe: Full Plate Boots
(20241,4041,1,1,2,431),-- Mold Hardener
(20241,4604,1,1,2,150),-- Greater Dye of Constitution <Con+3 Dex-3>
(20241,4609,1,1,2,75),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20241,8725,1,1,200,1899),-- Life Stone: level 52
(20241,8735,1,1,200,393),-- Mid-Grade Life Stone: level 52
(20241,8745,1,1,200,13),-- High-Grade Life Stone: level 52
(20241,8755,1,1,200,13),-- Top-Grade Life Stone: level 52
(20242,1868,1,9,-1,790517),-- Thread
(20242,1918,1,1,-1,68473),-- Nassen\'s Earring Gemstone
(20242,1985,1,1,-1,54518),-- Theca Leather Gaiters Pattern
(20242,57,766,1555,0,700000),-- Adena
(20242,400,1,1,1,24),-- Theca Leather Armor
(20242,401,1,1,1,13),-- Drake Leather Armor
(20242,420,1,1,1,38),-- Theca Leather Gaiters
(20242,1984,1,1,1,2086),-- Theca Leather Armor Pattern
(20242,1985,1,1,1,3356),-- Theca Leather Gaiters Pattern
(20242,1987,1,1,1,1485),-- Drake Leather Armor Texture
(20242,1867,1,1,2,45455),-- Animal Skin
(20242,1872,1,1,2,47619),-- Animal Bone
(20242,1875,1,1,2,1795),-- Stone of Purity
(20242,1881,1,1,2,8065),-- Coarse Bone Powder
(20242,3034,1,1,2,117),-- Recipe: Spiritshot B
(20242,4041,1,1,2,508),-- Mold Hardener
(20242,4941,1,1,2,450),-- Recipe: Zubei\'s Gaiters  (60%)
(20242,4959,1,1,2,1125),-- Recipe: Avadon Boots (60%)
(20242,5270,1,1,2,117),-- Recipe: Greater Soulshot (B) Compressed Package(100%)
(20242,6669,1,1,2,9901),-- Deluxe Chest Key - Grade 5
(20242,8726,1,1,200,1795),-- Life Stone: level 55
(20242,8736,1,1,200,99),-- Mid-Grade Life Stone: level 55
(20243,1806,1,1,-1,2819),-- Recipe: Soulshot: B-Grade
(20243,1873,1,2,-1,563815),-- Silver Nugget
(20243,4599,1,1,-1,4698),-- Greater Dye of Wit <Wit+2 Int-2>
(20243,57,682,1386,0,700000),-- Adena
(20243,203,1,1,1,3),-- Pa\'agrian Axe
(20243,204,1,1,1,2),-- Deadman\'s Staff
(20243,205,1,1,1,2),-- Ghoul\'s Staff
(20243,206,1,1,1,2),-- Demon\'s Staff
(20243,2112,1,1,1,253),-- Pa\'agrian Axe Blade
(20243,2116,1,1,1,245),-- Deadman\'s Staff Head
(20243,2117,1,1,1,245),-- Ghoul\'s Staff Head
(20243,2118,1,1,1,245),-- Demon\'s Staff Head
(20243,2503,1,1,1,2),-- Yaksa Mace
(20243,2969,1,1,1,251),-- Yaksa Mace Head
(20243,1864,1,1,2,111111),-- Stem
(20243,1869,1,1,2,45455),-- Iron Ore
(20243,1880,1,1,2,6506),-- Steel
(20243,4043,1,1,2,2169),-- Asofe
(20243,4144,1,1,2,669),-- Recipe: Zubei\'s Leather Gaiters
(20243,4184,1,1,2,76),-- Recipe: Sprite\'s Staff
(20243,5155,1,1,2,131),-- Recipe: Soulshot (B) Compressed Package (100%)
(20243,5550,1,1,2,868),-- Durable Metal Plate
(20243,8726,1,1,200,127),-- Life Stone: level 55
(20243,8736,1,1,200,253),-- Mid-Grade Life Stone: level 55
(20244,2117,1,1,-1,6820),-- Ghoul\'s Staff Head
(20244,3955,1,1,-1,2899),-- Recipe: Blessed Spiritshot B
(20244,4592,1,1,-1,4832),-- Greater Dye of Constitution <Con+2 Dex-2>
(20244,57,701,1428,0,700000),-- Adena
(20244,400,1,1,1,22),-- Theca Leather Armor
(20244,401,1,1,1,12),-- Drake Leather Armor
(20244,420,1,1,1,35),-- Theca Leather Gaiters
(20244,1984,1,1,1,1913),-- Theca Leather Armor Pattern
(20244,1985,1,1,1,3077),-- Theca Leather Gaiters Pattern
(20244,1987,1,1,1,1362),-- Drake Leather Armor Texture
(20244,1867,1,1,2,76923),-- Animal Skin
(20244,1872,1,1,2,66667),-- Animal Bone
(20244,1875,1,1,2,2959),-- Stone of Purity
(20244,1881,1,1,2,5076),-- Coarse Bone Powder
(20244,4041,1,1,2,665),-- Mold Hardener
(20244,4185,1,1,2,89),-- Recipe: Keshanberk
(20244,5436,1,1,2,125),-- Recipe: Berserker Blade(100%)
(20244,8322,1,1,2,114),-- Recipe: Ice Storm Hammer (60%)
(20244,8726,1,1,200,983),-- Life Stone: level 55
(20244,8736,1,1,200,328),-- Mid-Grade Life Stone: level 55
(20244,8746,1,1,200,328),-- High-Grade Life Stone: level 55
(20245,1895,1,2,-1,430289),-- Metallic Fiber
(20245,4071,1,1,-1,27278),-- Avadon Robe Fabric
(20245,4129,1,1,-1,16299),-- Recipe: Earrings of Black Ore
(20245,57,728,1485,0,700000),-- Adena
(20245,12160,1,1,1,761),-- Common Item - Zubei\'s Helmet
(20245,12159,1,1,1,380),-- Common Item - Zubei\'s Shield
(20245,12171,1,1,1,1142),-- Common Item - Avadon Shield
(20245,12170,1,1,1,380),-- Common Item - Avadon Circlet
(20245,4072,1,1,1,2498),-- Avadon Circlet Pattern
(20245,4101,1,1,1,3225),-- Zubei\'s Shield Fragment
(20245,4102,1,1,1,2498),-- Zubei\'s Helmet Design
(20245,4439,1,1,1,3836),-- Avadon Shield Fragment
(20245,1864,1,1,2,71429),-- Stem
(20245,1869,1,1,2,41667),-- Iron Ore
(20245,1878,1,1,2,13333),-- Braided Hemp
(20245,1884,1,1,2,22222),-- Cord
(20245,4039,1,1,2,1522),-- Mold Glue
(20245,4945,1,1,2,428),-- Recipe: Avadon Gaiters (60%)
(20245,4999,1,1,2,692),-- Recipe: Doom Boots (60%)
(20245,8322,1,1,2,54),-- Recipe: Ice Storm Hammer (60%)
(20245,6036,1,1,2,1142),-- Greater Magic Haste Potion
(20245,8726,1,1,200,380),-- Life Stone: level 55
(20246,947,1,1,-1,772),-- Scroll: Enchant Weapon (Grade B)
(20246,2964,1,1,-1,105072),-- Divine Gloves Pattern
(20246,4049,1,1,-1,101214),-- Adamantite Earring Gemstone
(20246,57,934,1899,0,700000),-- Adena
(20246,12338,1,1,1,808),-- Common Item - Adamantite Earring
(20246,12340,1,1,1,1414),-- Common Item - Adamantite Ring
(20246,12339,1,1,1,1414),-- Common Item - Adamantite Necklace
(20246,4049,1,1,1,4984),-- Adamantite Earring Gemstone
(20246,4050,1,1,1,5888),-- Adamantite Ring Wire
(20246,4051,1,1,1,3686),-- Adamantite Necklace Chain
(20246,1864,1,1,2,111111),-- Stem
(20246,1869,1,1,2,52632),-- Iron Ore
(20246,1880,1,1,2,6849),-- Steel
(20246,4043,1,1,2,2833),-- Asofe
(20246,4157,1,1,2,355),-- Recipe: Blue Wolf Gaiters
(20246,4176,1,1,2,404),-- Recipe: Doom Boots
(20246,4969,1,1,2,83),-- Recipe: Hell Knife (60%)
(20246,5550,1,1,2,1414),-- Durable Metal Plate
(20246,8727,1,1,200,1414),-- Life Stone: level 58
(20246,8737,1,1,200,202),-- Mid-Grade Life Stone: level 58
(20248,955,1,1,-1,1880),-- Scroll: Enchant Weapon (Grade D)
(20248,1865,1,2,-1,470054),-- Varnish
(20248,2250,1,2,-1,117514),-- Recipe: Bone Arrow
(20248,57,239,476,0,700000),-- Adena
(20248,86,1,1,1,8),-- Tomahawk
(20248,179,1,1,1,8),-- Mace of Prayer
(20248,180,1,1,1,8),-- Mace of Judgment
(20248,181,1,1,1,8),-- Mace of Miracle
(20248,182,1,1,1,8),-- Doom Hammer
(20248,183,1,1,1,8),-- Mystic Staff
(20248,184,1,1,1,8),-- Conjuror\'s Staff
(20248,185,1,1,1,8),-- Staff of Mana
(20248,2022,1,1,1,400),-- Mace of Judgment Head
(20248,2023,1,1,1,319),-- Conjuror\'s Staff Head
(20248,1829,1,1,2,18182),-- Scroll of Escape: Clan Hall
(20248,1865,1,1,2,47619),-- Varnish
(20248,1870,1,1,2,45455),-- Coal
(20248,1874,1,1,2,1464),-- Oriharukon Ore
(20248,2192,1,1,2,6211),-- Recipe: Kite Shield
(20248,2274,1,1,2,306),-- Recipe: Skull Breaker
(20249,1865,1,3,-1,500696),-- Varnish
(20249,1873,1,2,-1,400557),-- Silver Nugget
(20249,57,508,1015,0,700000),-- Adena
(20249,260,1,1,1,110),-- Triple-Edged Jamadhr
(20249,1341,20,60,2,142857),-- Bone Arrow
(20249,1868,1,1,2,166667),-- Thread
(20249,1873,1,1,2,38462),-- Silver Nugget
(20249,1889,1,1,2,4739),-- Compound Braid
(20249,2186,1,1,2,3861),-- Recipe: Brigandine Tunic
(20249,5549,1,1,2,11111),-- Metallic Thread
(20249,6667,1,1,2,52632),-- Deluxe Chest Key - Grade 3
(20250,1884,1,1,-1,234360),-- Cord
(20250,1899,1,1,-1,59505),-- Necklace of Devotion Chain
(20250,2259,1,1,-1,5914),-- Recipe: Spinebone Sword
(20250,57,197,380,0,700000),-- Adena
(20250,847,1,1,1,398),-- Red Crescent Earring
(20250,890,1,1,1,594),-- Ring of Devotion
(20250,910,1,1,1,298),-- Necklace of Devotion
(20250,1899,1,1,1,8350),-- Necklace of Devotion Chain
(20250,1865,1,1,2,37037),-- Varnish
(20250,1870,1,1,2,58824),-- Coal
(20250,1874,1,1,2,3717),-- Oriharukon Ore
(20250,2182,1,1,2,11111),-- Recipe: Scale Gaiters
(20250,5437,1,1,2,1046),-- Recipe: Heavy Sword(100%)
(20251,1880,1,1,-1,40031),-- Steel
(20251,2017,1,1,-1,5320),-- Saber Blade
(20251,2152,1,1,-1,70478),-- Recipe: Enchanted Necklace
(20251,57,206,403,0,700000),-- Adena
(20251,123,1,1,1,28),-- Saber
(20251,2017,1,1,1,747),-- Saber Blade
(20251,5285,1,1,1,28),-- Heavy Sword
(20251,5531,1,1,1,943),-- Heavy Sword Edge
(20251,1866,1,1,2,35714),-- Suede
(20251,1871,1,1,2,55556),-- Charcoal
(20251,1877,1,1,2,3521),-- Adamantite Nugget
(20251,2181,1,1,2,2347),-- Recipe: White Tunic
(20252,1061,1,1,-1,253920),-- Healing Potion
(20252,1865,1,2,-1,418968),-- Varnish
(20252,1900,1,1,-1,45839),-- Enchanted Necklace Chain
(20252,57,213,422,0,700000),-- Adena
(20252,605,1,1,1,688),-- Leather Gauntlets
(20252,1123,1,1,1,688),-- Blue Buckskin Boots
(20252,1867,1,1,2,43478),-- Animal Skin
(20252,1872,1,1,2,41667),-- Animal Bone
(20252,1875,1,1,2,782),-- Stone of Purity
(20252,1881,1,1,2,3906),-- Coarse Bone Powder
(20252,2185,1,1,2,4695),-- Recipe: Iron Boots
(20252,2187,1,1,2,1304),-- Recipe: Manticore Skin Shirt
(20252,2274,1,1,2,205),-- Recipe: Skull Breaker
(20252,6666,1,1,2,28571),-- Deluxe Chest Key - Grade 2
(20253,1900,1,1,-1,48891),-- Enchanted Necklace Chain
(20253,2153,1,1,-1,69496),-- Recipe: Tiger\'s Eye Earring
(20253,2265,1,1,-1,4621),-- Recipe: Spiked Club
(20253,57,227,452,0,700000),-- Adena
(20253,348,1,1,1,92),-- Scale Mail
(20253,377,1,1,1,147),-- Scale Gaiters
(20253,1935,1,1,1,3411),-- Scale Mail Fragment
(20253,1937,1,1,1,5245),-- Scale Gaiters Fragment
(20253,1868,1,1,2,125000),-- Thread
(20253,1873,1,1,2,25641),-- Silver Nugget
(20253,1876,1,1,2,18868),-- Mithril Ore
(20253,2266,1,1,2,506),-- Recipe: Shilen Knife
(20253,2971,1,1,2,2612),-- Recipe: Mithril Scale Gaiters
(20254,1899,1,1,-1,64438),-- Necklace of Devotion Chain
(20254,2181,1,1,-1,41282),-- Recipe: White Tunic
(20254,2262,1,1,-1,6404),-- Recipe: Elven Bow
(20254,57,212,415,0,700000),-- Adena
(20254,291,1,1,1,57),-- Trident
(20254,2019,1,1,1,2489),-- Trident Edge
(20254,1876,1,1,2,8333),-- Mithril Ore
(20254,1877,1,1,2,1567),-- Adamantite Nugget
(20254,1880,1,1,2,3135),-- Steel
(20254,1881,1,1,2,6803),-- Coarse Bone Powder
(20254,1882,1,1,2,6803),-- Leather
(20254,2182,1,1,2,4695),-- Recipe: Scale Gaiters
(20265,1866,1,3,-1,374196),-- Suede
(20265,1872,1,5,-1,748392),-- Animal Bone
(20265,2015,1,1,-1,6845),-- Sword Breaker Edge
(20265,57,255,458,0,700000),-- Adena
(20265,219,1,1,1,136),-- Sword Breaker
(20265,2015,1,1,1,2023),-- Sword Breaker Edge
(20265,1867,1,1,2,110532),-- Animal Skin
(20265,1872,1,1,2,110532),-- Animal Bone
(20265,1882,1,1,2,18423),-- Leather
(20265,5158,1,1,2,3316),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20266,1866,1,1,-1,233160),-- Suede
(20266,1878,1,1,-1,139896),-- Braided Hemp
(20266,2180,1,1,-1,26296),-- Recipe: Scale Mail
(20266,57,185,346,0,700000),-- Adena
(20266,432,1,1,1,314),-- Cursed Tunic
(20266,465,1,1,1,503),-- Cursed Stockings
(20266,1874,1,1,2,5556),-- Oriharukon Ore
(20266,1878,1,1,2,22222),-- Braided Hemp
(20266,2260,1,1,2,617),-- Recipe: Mace of Judgment
(20266,5220,1,1,2,3704),-- Metal Hardener
(20267,1061,1,1,-1,284881),-- Healing Potion
(20267,1882,1,1,-1,104456),-- Leather
(20267,2203,1,1,-1,102408),-- Recipe: Mithril Boots
(20267,57,239,476,0,700000),-- Adena
(20267,69,1,1,1,16),-- Bastard Sword
(20267,125,1,1,1,16),-- Spinebone Sword
(20267,126,1,1,1,16),-- Artisan\'s Sword
(20267,128,1,1,1,16),-- Knight\'s Sword
(20267,2021,1,1,1,649),-- Spinebone Sword Blade
(20267,1868,1,1,2,111111),-- Thread
(20267,1873,1,1,2,31250),-- Silver Nugget
(20267,1876,1,1,2,14925),-- Mithril Ore
(20267,2153,1,1,2,8772),-- Recipe: Tiger\'s Eye Earring
(20267,2274,1,1,2,875),-- Recipe: Skull Breaker
(20268,1878,1,2,-1,256688),-- Braided Hemp
(20268,1900,1,1,-1,70210),-- Enchanted Necklace Chain
(20268,57,324,651,0,700000),-- Adena
(20268,275,1,1,1,37),-- Long Bow
(20268,276,1,1,1,37),-- Elven Bow
(20268,2024,1,1,1,1666),-- Elven Bow Shaft
(20268,956,1,1,2,4149),-- Scroll: Enchant Armor (Grade D)
(20268,1867,1,1,2,111111),-- Animal Skin
(20268,1872,1,1,2,111111),-- Animal Bone
(20268,1875,1,1,2,3322),-- Stone of Purity
(20268,1881,1,1,2,13333),-- Coarse Bone Powder
(20269,1879,1,1,-1,92785),-- Cokes
(20269,2029,1,1,-1,6641),-- Gastraphetes Shaft
(20269,2154,1,1,-1,59605),-- Recipe: Elven Earring
(20269,57,282,563,0,700000),-- Adena
(20269,157,1,1,1,21),-- Spiked Club
(20269,186,1,1,1,21),-- Staff of Magic
(20269,2027,1,1,1,1293),-- Spiked Club Head
(20269,1875,1,1,2,3472),-- Stone of Purity
(20269,1879,1,1,2,8696),-- Cokes
(20269,1885,1,1,2,3906),-- High Grade Suede
(20269,2275,1,1,2,217),-- Recipe: Heavy Bone Club
(20269,2286,1,1,2,435),-- Recipe: Bonebreaker
(20269,5549,1,1,2,4000),-- Metallic Thread
(20269,6667,1,1,2,27027),-- Deluxe Chest Key - Grade 3
(20270,1901,1,1,-1,108561),-- Tiger\'s Eye Earring Stone
(20270,2972,1,1,-1,42031),-- Recipe: Brigandine Gaiters
(20270,2976,1,1,-1,105219),-- Recipe: Manticore Skin Boots
(20270,57,398,795,0,700000),-- Adena
(20270,393,1,1,1,92),-- Mithril Banded Mail
(20270,394,1,1,1,92),-- Reinforced Leather Shirt
(20270,415,1,1,1,147),-- Mithril Banded Gaiters
(20270,416,1,1,1,147),-- Reinforced Leather Gaiters
(20270,1938,1,1,1,4683),-- Mithril Banded Mail Material
(20270,1939,1,1,1,6119),-- Mithril Banded Gaiters Material
(20270,1341,20,60,2,166667),-- Bone Arrow
(20270,1865,1,1,2,100000),-- Varnish
(20270,1870,1,1,2,100000),-- Coal
(20270,1874,1,1,2,8197),-- Oriharukon Ore
(20270,2980,1,1,2,11905),-- Recipe: Brigandine Gauntlets
(20271,1882,1,2,-1,142372),-- Leather
(20271,2155,1,2,-1,102837),-- Recipe: Elven Ring
(20271,2271,1,1,-1,4576),-- Recipe: Winged Spear
(20271,57,325,650,0,700000),-- Adena
(20271,849,1,1,1,265),-- Tiger\'s Eye Earring
(20271,880,1,1,1,398),-- Black Pearl Ring
(20271,912,1,1,1,199),-- Near Forest Necklace
(20271,1901,1,1,1,11763),-- Tiger\'s Eye Earring Stone
(20271,1874,1,1,2,4673),-- Oriharukon Ore
(20271,1878,1,1,2,20408),-- Braided Hemp
(20271,1884,1,1,2,33333),-- Cord
(20271,1889,1,1,2,4505),-- Compound Braid
(20271,2187,1,1,2,1730),-- Recipe: Manticore Skin Shirt
(20271,2269,1,1,2,346),-- Recipe: Morning Star
(20271,2293,1,1,2,173),-- Recipe: Glaive
(20271,5220,1,1,2,692),-- Metal Hardener
(20291,1879,1,1,-1,57609),-- Cokes
(20291,1898,1,1,-1,35506),-- Blue Diamond Necklace Gem
(20291,2255,1,1,-1,8451),-- Recipe: Saber
(20291,57,157,281,0,700000),-- Adena
(20291,27,1,1,1,277),-- Hard Leather Shirt
(20291,33,1,1,1,444),-- Hard Leather Gaiters
(20291,1927,1,1,1,4073),-- Hard Leather Shirt Pattern
(20291,1929,1,1,1,6587),-- Hard Leather Gaiters Material
(20291,734,1,1,2,6807),-- Haste Potion
(20291,1864,1,1,2,81680),-- Stem
(20291,1869,1,1,2,40840),-- Iron Ore
(20291,1878,1,1,2,16336),-- Braided Hemp
(20291,2257,1,1,2,999),-- Recipe: Trident
(20292,2139,1,1,-1,61663),-- Recipe: Steel Mold
(20292,2150,1,1,-1,144749),-- Recipe: Blue Diamond Necklace
(20292,5220,1,1,-1,12333),-- Metal Hardener
(20292,57,142,248,0,700000),-- Adena
(20292,845,1,1,1,1025),-- Cat\'s Eye Earring
(20292,877,1,1,1,1539),-- Ring of Wisdom
(20292,908,1,1,1,766),-- Necklace of Wisdom
(20292,1897,1,1,1,10842),-- Necklace of Wisdom Chain
(20292,1866,1,1,2,30358),-- Suede
(20292,1871,1,1,2,45537),-- Charcoal
(20292,2252,1,1,2,1867),-- Recipe: Iron Hammer
(20292,5158,1,1,2,1822),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20307,1060,1,1,-1,330759),-- Lesser Healing Potion
(20307,1867,1,2,-1,198456),-- Animal Skin
(20307,2137,1,1,-1,43777),-- Recipe: Steel
(20307,57,80,108,0,700000),-- Adena
(20307,5,1,1,1,352),-- Mace
(20307,8,1,1,1,352),-- Willow Staff
(20307,152,1,1,1,545),-- Heavy Chisel
(20307,2006,1,1,1,1570),-- Willow Staff Head
(20307,1802,1,1,2,31517),-- Recipe: Necklace of Anguish
(20307,1864,1,1,2,29311),-- Stem
(20307,1865,1,1,2,14656),-- Varnish
(20307,1866,1,1,2,9771),-- Suede
(20307,1868,1,1,2,29311),-- Thread
(20307,1873,1,1,2,5863),-- Silver Nugget
(20308,1868,1,2,-1,475131),-- Thread
(20308,1897,1,1,-1,56563),-- Necklace of Wisdom Chain
(20308,1924,1,1,-1,76634),-- Leather Helmet Design
(20308,57,118,183,0,700000),-- Adena
(20308,38,1,1,1,1383),-- Low Boots
(20308,50,1,1,1,1383),-- Leather Gloves
(20308,1925,1,1,1,30677),-- Leather Gloves Lining
(20308,1868,1,1,2,70174),-- Thread
(20308,1873,1,1,2,14035),-- Silver Nugget
(20308,2143,1,1,2,7018),-- Recipe: Synthetic Cokes
(20308,5158,1,1,2,1404),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20310,735,1,1,-1,19993),-- Potion of Alacrity
(20310,1800,1,1,-1,75446),-- Recipe: Piece Bone Breastplate
(20310,1872,1,3,-1,319890),-- Animal Bone
(20310,57,119,185,0,700000),-- Adena
(20310,38,1,1,1,1396),-- Low Boots
(20310,50,1,1,1,1396),-- Leather Gloves
(20310,1925,1,1,1,30981),-- Leather Gloves Lining
(20310,1865,1,1,2,47246),-- Varnish
(20310,1870,1,1,2,47246),-- Coal
(20310,2144,1,1,2,9450),-- Recipe: Compound Braid
(20311,1060,1,1,-1,172757),-- Lesser Healing Potion
(20311,2005,1,1,-1,5553),-- Broadsword Blade
(20311,57,41,58,0,700000),-- Adena
(20311,1119,1,1,1,5045),-- Short Leather Gloves
(20311,1122,1,1,1,5044),-- Cotton Shoes
(20311,1129,1,1,1,5044),-- Crude Leather Shoes
(20311,1060,1,1,2,25515),-- Lesser Healing Potion
(20311,1789,1,1,2,2123),-- Recipe: Cedar Staff
(20311,1868,1,1,2,22964),-- Thread
(20311,1873,1,1,2,4593),-- Silver Nugget
(20312,1666,1,1,-1,32439),-- Recipe: Wooden Arrow
(20312,1866,1,1,-1,64878),-- Suede
(20312,2136,1,1,-1,28622),-- Recipe: Cokes
(20312,57,51,72,0,700000),-- Adena
(20312,19,1,1,1,9012),-- Small Shield
(20312,42,1,1,1,6311),-- Leather Cap
(20312,734,1,1,2,2396),-- Haste Potion
(20312,1802,1,1,2,30910),-- Recipe: Necklace of Anguish
(20312,1864,1,1,2,28746),-- Stem
(20312,1869,1,1,2,14373),-- Iron Ore
(20313,22,1,1,-1,14976),-- Leather Shirt
(20313,1814,1,1,-1,95767),-- Recipe: Leather
(20313,2006,1,1,-1,12997),-- Willow Staff Head
(20313,57,98,132,0,700000),-- Adena
(20313,428,1,1,1,2212),-- Feriotic Tunic
(20313,463,1,1,1,3537),-- Feriotic Stockings
(20313,1100,1,1,1,2212),-- Cotton Tunic
(20313,1103,1,1,1,3537),-- Cotton Stockings
(20313,1790,1,1,2,4968),-- Recipe: Dirk
(20313,1864,1,1,2,53748),-- Stem
(20313,1869,1,1,2,26874),-- Iron Ore
(20313,6037,1,1,2,53747),-- Waking Scroll
(20314,1869,1,1,-1,710924),-- Iron Ore
(20314,1874,1,1,-1,47395),-- Oriharukon Ore
(20314,4472,1,1,-1,4062),-- Dye of Constitution <Con+1 Dex-1>
(20314,57,377,679,0,700000),-- Adena
(20314,70,1,1,1,10),-- Claymore
(20314,2047,1,1,1,785),-- Claymore Blade
(20314,2499,1,1,1,10),-- Elven Long Sword
(20314,2967,1,1,1,774),-- Elven Long Sword Blade
(20314,1864,1,1,2,101999),-- Stem
(20314,1869,1,1,2,51000),-- Iron Ore
(20314,1878,1,1,2,20400),-- Braided Hemp
(20314,1884,1,1,2,31385),-- Cord
(20314,2205,1,1,2,1614),-- Recipe: Chain Gaiters
(20314,2209,1,1,2,3835),-- Recipe: Eldarake
(20314,2299,1,1,2,223),-- Recipe: Big Hammer
(20316,1792,1,1,-1,14822),-- Recipe: Short Spear
(20316,1803,1,1,-1,169394),-- Recipe: Necklace of Wisdom
(20316,1864,1,2,-1,403157),-- Stem
(20316,57,109,147,0,700000),-- Adena
(20316,23,1,1,1,998),-- Wooden Breastplate
(20316,30,1,1,1,1598),-- Hard Leather Pants
(20316,2386,1,1,1,1598),-- Wooden Gaiters
(20316,734,1,1,2,4962),-- Haste Potion
(20316,1794,1,1,2,2190),-- Recipe: Forest Bow
(20316,1865,1,1,2,29772),-- Varnish
(20316,1870,1,1,2,29772),-- Coal
(20317,1786,1,1,-1,30885),-- Recipe: Broad Sword
(20317,1867,1,1,-1,51475),-- Animal Skin
(20317,57,19,29,0,700000),-- Adena
(20317,1,1,1,1,10394),-- Short Sword
(20317,1787,1,1,2,5322),-- Recipe: Willow Staff
(20317,1867,1,1,2,8870),-- Animal Skin
(20317,1869,1,1,2,6653),-- Iron Ore
(20317,1870,1,1,2,6653),-- Coal
(20317,1871,1,1,2,6653),-- Charcoal
(20317,1872,1,1,2,8870),-- Animal Bone
(20318,1060,1,1,-1,171046),-- Lesser Healing Potion
(20318,1866,1,1,-1,51314),-- Suede
(20318,2005,1,1,-1,5498),-- Broadsword Blade
(20318,57,41,58,0,700000),-- Adena
(20318,3,1,1,1,364),-- Broadsword
(20318,2005,1,1,1,1625),-- Broadsword Blade
(20318,1866,1,1,2,15158),-- Suede
(20318,1871,1,1,2,22736),-- Charcoal
(20319,1787,1,1,-1,45293),-- Recipe: Willow Staff
(20319,1865,1,1,-1,56617),-- Varnish
(20319,57,30,42,0,700000),-- Adena
(20319,112,1,1,1,79637),-- Apprentice\'s Earring
(20319,116,1,1,1,118249),-- Magic Ring
(20319,118,1,1,1,59125),-- Necklace of Magic
(20319,1787,1,1,2,7805),-- Recipe: Willow Staff
(20319,1864,1,1,2,19512),-- Stem
(20319,1865,1,1,2,9756),-- Varnish
(20319,1866,1,1,2,6504),-- Suede
(20319,1868,1,1,2,19512),-- Thread
(20319,1873,1,1,2,3903),-- Silver Nugget
(20320,1831,1,1,-1,557587),-- Antidote
(20320,1867,1,2,-1,278794),-- Animal Skin
(20320,2010,1,1,-1,8136),-- Brandish Blade
(20320,57,112,154,0,700000),-- Adena
(20320,216,1,1,1,229),-- Dirk
(20320,2009,1,1,1,2191),-- Dirk Blade
(20320,1799,1,1,2,61001),-- Recipe: Leather Gloves
(20320,1867,1,1,2,54902),-- Animal Skin
(20320,1872,1,1,2,54902),-- Animal Bone
(20321,1060,1,1,-1,396420),-- Lesser Healing Potion
(20321,1870,1,1,-1,178389),-- Coal
(20321,2005,1,1,-1,12742),-- Broadsword Blade
(20321,57,96,130,0,700000),-- Adena
(20321,5,1,1,1,422),-- Mace
(20321,8,1,1,1,422),-- Willow Staff
(20321,152,1,1,1,653),-- Heavy Chisel
(20321,2006,1,1,1,1882),-- Willow Staff Head
(20321,1791,1,1,2,2783),-- Recipe: Brandish
(20321,1833,1,1,2,40148),-- Bandage
(20321,1864,1,1,2,30111),-- Stem
(20321,1865,1,1,2,15056),-- Varnish
(20321,1866,1,1,2,10037),-- Suede
(20321,1868,1,1,2,30111),-- Thread
(20321,1873,1,1,2,6023),-- Silver Nugget
(20322,1796,1,1,-1,156757),-- Recipe: Leather Tunic
(20322,1870,1,2,-1,124622),-- Coal
(20322,2005,1,1,-1,8902),-- Broadsword Blade
(20322,57,67,91,0,700000),-- Adena
(20322,12,1,1,1,589),-- Knife
(20322,215,1,1,1,589),-- Doom Dagger
(20322,17,10,30,2,92028),-- Wooden Arrow
(20322,1867,1,1,2,24541),-- Animal Skin
(20322,1872,1,1,2,24541),-- Animal Bone
(20322,2137,1,1,2,5414),-- Recipe: Steel
(20323,1792,1,1,-1,16716),-- Recipe: Short Spear
(20323,1865,1,2,-1,227339),-- Varnish
(20323,1869,1,3,-1,227339),-- Iron Ore
(20323,57,116,172,0,700000),-- Adena
(20323,216,1,1,1,249),-- Dirk
(20323,2009,1,1,1,2382),-- Dirk Blade
(20323,1792,1,1,2,2469),-- Recipe: Short Spear
(20323,1866,1,1,2,22385),-- Suede
(20323,1871,1,1,2,33577),-- Charcoal
(20323,6037,1,1,2,67153),-- Waking Scroll
(20324,1791,1,1,-1,33633),-- Recipe: Brandish
(20324,1871,1,2,-1,181957),-- Charcoal
(20324,2006,1,1,-1,12997),-- Willow Staff Head
(20324,57,98,132,0,700000),-- Adena
(20324,5,1,1,1,431),-- Mace
(20324,8,1,1,1,431),-- Willow Staff
(20324,152,1,1,1,667),-- Heavy Chisel
(20324,2006,1,1,1,1920),-- Willow Staff Head
(20324,1795,1,1,2,67608),-- Recipe:Leather Shoes
(20324,1867,1,1,2,23888),-- Animal Skin
(20324,1869,1,1,2,17916),-- Iron Ore
(20324,1870,1,1,2,17916),-- Coal
(20324,1871,1,1,2,17916),-- Charcoal
(20324,1872,1,1,2,23888),-- Animal Bone
(20325,118,1,1,-1,171565),-- Necklace of Magic
(20325,1786,1,1,-1,45293),-- Recipe: Broad Sword
(20325,1787,1,1,-1,45293),-- Recipe: Willow Staff
(20325,57,30,42,0,700000),-- Adena
(20325,112,1,1,1,79637),-- Apprentice\'s Earring
(20325,116,1,1,1,118249),-- Magic Ring
(20325,118,1,1,1,59125),-- Necklace of Magic
(20325,1060,1,1,2,26015),-- Lesser Healing Potion
(20325,1788,1,1,2,9366),-- Recipe: Bow 
(20325,1865,1,1,2,11707),-- Varnish
(20325,1870,1,1,2,11707),-- Coal
(20325,6037,1,1,2,23414),-- Waking Scroll
(20326,2006,1,1,-1,8902),-- Willow Staff Head
(20326,2007,1,1,-1,8902),-- Bow Shaft
(20326,2135,1,1,-1,36653),-- Recipe: Braided Hemp
(20326,57,67,91,0,700000),-- Adena
(20326,22,1,1,1,3030),-- Leather Shirt
(20326,29,1,1,1,4844),-- Leather Pants
(20326,1864,1,1,2,24542),-- Stem
(20326,1865,1,1,2,12271),-- Varnish
(20326,1866,1,1,2,8181),-- Suede
(20326,1868,1,1,2,24542),-- Thread
(20326,1873,1,1,2,4909),-- Silver Nugget
(20326,2138,1,1,2,3609),-- Recipe: Coarse Bone Powder (100%)
(20327,1865,1,1,-1,56617),-- Varnish
(20327,1870,1,1,-1,56617),-- Coal
(20327,1872,1,1,-1,75489),-- Animal Bone
(20327,57,30,42,0,700000),-- Adena
(20327,112,1,1,1,79637),-- Apprentice\'s Earring
(20327,116,1,1,1,118249),-- Magic Ring
(20327,118,1,1,1,59125),-- Necklace of Magic
(20327,1786,1,1,2,15609),-- Recipe: Broad Sword
(20327,1867,1,1,2,26015),-- Animal Skin
(20327,1872,1,1,2,26015),-- Animal Bone
(20328,1865,1,1,-1,124622),-- Varnish
(20328,1869,1,1,-1,124622),-- Iron Ore
(20328,2137,1,1,-1,36653),-- Recipe: Steel
(20328,57,67,91,0,700000),-- Adena
(20328,12,1,1,1,589),-- Knife
(20328,215,1,1,1,589),-- Doom Dagger
(20328,1060,1,1,2,23373),-- Lesser Healing Potion
(20328,1666,1,1,2,3506),-- Recipe: Wooden Arrow
(20328,1867,1,1,2,14024),-- Animal Skin
(20328,1869,1,1,2,10518),-- Iron Ore
(20328,1870,1,1,2,10518),-- Coal
(20328,1871,1,1,2,10518),-- Charcoal
(20328,1872,1,1,2,14024),-- Animal Bone
(20329,1869,1,1,-1,239656),-- Iron Ore
(20329,1880,1,1,-1,23966),-- Steel
(20329,1898,1,1,-1,24618),-- Blue Diamond Necklace Gem
(20329,57,109,195,0,700000),-- Adena
(20329,16,1,1,1,116),-- Long Spear
(20329,1867,1,1,2,26968),-- Animal Skin
(20329,1869,1,1,2,20226),-- Iron Ore
(20329,1870,1,1,2,20226),-- Coal
(20329,1871,1,1,2,20226),-- Charcoal
(20329,1872,1,1,2,26968),-- Animal Bone
(20329,2258,1,1,2,495),-- Recipe: Temptation of Abyss
(20329,5268,1,1,2,810),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20330,1792,1,1,-1,12352),-- Recipe: Short Spear
(20330,1866,1,1,-1,111988),-- Suede
(20330,1896,1,1,-1,71179),-- Necklace of Anguish Chain
(20330,57,91,122,0,700000),-- Adena
(20330,37,1,1,1,2497),-- Leather Shoes
(20330,49,1,1,1,2497),-- Gloves
(20330,1921,1,1,1,21342),-- Leather Shoes Texture
(20330,1798,1,1,2,32431),-- Recipe: Leather Helmet
(20330,1864,1,1,2,66159),-- Stem
(20330,1869,1,1,2,33080),-- Iron Ore
(20331,1871,1,1,-1,148657),-- Charcoal
(20331,2005,1,1,-1,10618),-- Broadsword Blade
(20331,2007,1,1,-1,10618),-- Bow Shaft
(20331,57,80,108,0,700000),-- Adena
(20331,113,1,1,1,7220),-- Mystic\'s Earring
(20331,875,1,1,1,10843),-- Ring of Knowledge
(20331,906,1,1,1,5422),-- Necklace of Knowledge
(20331,734,1,1,2,3660),-- Haste Potion
(20331,1796,1,1,2,27617),-- Recipe: Leather Tunic
(20331,1865,1,1,2,21956),-- Varnish
(20331,1870,1,1,2,21956),-- Coal
(20333,1792,1,1,-1,17783),-- Recipe: Short Spear
(20333,1798,1,1,-1,237113),-- Recipe: Leather Helmet
(20333,2008,1,1,-1,9411),-- Cedar Staff Head
(20333,57,127,180,0,700000),-- Adena
(20333,255,1,1,1,264),-- Fox Claw Gloves
(20333,2371,1,1,1,561),-- Fist of Butcher
(20333,734,1,1,2,5954),-- Haste Potion
(20333,1803,1,1,2,30018),-- Recipe: Necklace of Wisdom
(20333,1868,1,1,2,71441),-- Thread
(20333,1873,1,1,2,14289),-- Silver Nugget
(20335,734,1,1,-1,40607),-- Haste Potion
(20335,1867,1,1,-1,324856),-- Animal Skin
(20335,1897,1,1,-1,58010),-- Necklace of Wisdom Chain
(20335,57,119,191,0,700000),-- Adena
(20335,2,1,1,1,53),-- Long Sword
(20335,120,1,1,1,53),-- Sword of Reflection
(20335,121,1,1,1,53),-- Sword of Watershadow
(20335,2012,1,1,1,750),-- Sword of Reflection Blade
(20335,1867,1,1,2,47979),-- Animal Skin
(20335,1872,1,1,2,47979),-- Animal Bone
(20335,2174,1,1,2,15646),-- Recipe: Hard Leather Gaiters
(20335,5158,1,1,2,1440),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20336,29,1,1,-1,16398),-- Leather Pants
(20336,1060,1,1,-1,276937),-- Lesser Healing Potion
(20336,1802,1,1,-1,268004),-- Recipe: Necklace of Anguish
(20336,57,67,91,0,700000),-- Adena
(20336,428,1,1,1,1515),-- Feriotic Tunic
(20336,463,1,1,1,2422),-- Feriotic Stockings
(20336,1100,1,1,1,1515),-- Cotton Tunic
(20336,1103,1,1,1,2422),-- Cotton Stockings
(20336,1789,1,1,2,2269),-- Recipe: Cedar Staff
(20336,1864,1,1,2,24541),-- Stem
(20336,1865,1,1,2,12271),-- Varnish
(20336,1866,1,1,2,8181),-- Suede
(20336,1868,1,1,2,24541),-- Thread
(20336,1873,1,1,2,4909),-- Silver Nugget
(20338,1898,1,1,-1,27080),-- Blue Diamond Necklace Gem
(20338,1927,1,1,-1,21031),-- Hard Leather Shirt Pattern
(20338,3032,1,1,-1,10545),-- Recipe: Spiritshot D
(20338,57,120,214,0,700000),-- Adena
(20338,16,1,1,1,128),-- Long Spear
(20338,1831,1,1,2,103827),-- Antidote
(20338,1868,1,1,2,77870),-- Thread
(20338,1873,1,1,2,15574),-- Silver Nugget
(20338,3032,1,1,2,1558),-- Recipe: Spiritshot D
(20340,1792,1,1,-1,13068),-- Recipe: Short Spear
(20340,57,95,131,0,700000),-- Adena
(20340,255,1,1,1,195),-- Fox Claw Gloves
(20340,2371,1,1,1,412),-- Fist of Butcher
(20340,1803,1,1,2,29412),-- Recipe: Necklace of Wisdom
(20340,1865,1,1,2,35000),-- Varnish
(20340,1870,1,1,2,35000),-- Coal
(20341,1897,1,1,-1,58257),-- Necklace of Wisdom Chain
(20341,2141,1,1,-1,48936),-- Recipe: Silver Mold
(20341,2143,1,1,-1,48936),-- Recipe: Synthetic Cokes
(20341,57,115,195,0,700000),-- Adena
(20341,44,1,1,1,709),-- Leather Helmet
(20341,102,1,1,1,1017),-- Round Shield
(20341,945,1,1,1,2101),-- Skeleton Buckler
(20341,1924,1,1,1,11657),-- Leather Helmet Design
(20341,1866,1,1,2,32123),-- Suede
(20341,1871,1,1,2,48184),-- Charcoal
(20341,2175,1,1,2,39174),-- Recipe: Boots
(20342,429,1,1,-1,5359),-- Leather Tunic
(20342,1793,1,1,-1,15682),-- Recipe: Sword of Reflection
(20342,1870,1,1,-1,213277),-- Coal
(20342,57,114,157,0,700000),-- Adena
(20342,255,1,1,1,233),-- Fox Claw Gloves
(20342,2371,1,1,1,495),-- Fist of Butcher
(20342,736,1,1,2,21000),-- Scroll of Escape
(20342,1864,1,1,2,83999),-- Stem
(20342,1869,1,1,2,42000),-- Iron Ore
(20343,1060,1,1,-1,404348),-- Lesser Healing Potion
(20343,1795,1,1,-1,686629),-- Recipe:Leather Shoes
(20343,2005,1,1,-1,12997),-- Broadsword Blade
(20343,57,98,132,0,700000),-- Adena
(20343,113,1,1,1,8837),-- Mystic\'s Earring
(20343,875,1,1,1,13272),-- Ring of Knowledge
(20343,906,1,1,1,6636),-- Necklace of Knowledge
(20343,1797,1,1,2,72387),-- Recipe: Leather Stockings
(20343,1864,1,1,2,71663),-- Stem
(20343,1869,1,1,2,35832),-- Iron Ore
(20345,1925,1,1,-1,145655),-- Leather Gloves Lining
(20345,2174,1,1,-1,96576),-- Recipe: Hard Leather Gaiters
(20345,5220,1,1,-1,8885),-- Metal Hardener
(20345,57,110,171,0,700000),-- Adena
(20345,24,1,1,1,324),-- Bone Breastplate
(20345,31,1,1,1,517),-- Bone Gaiters
(20345,390,1,1,1,324),-- Cotton Shirt
(20345,412,1,1,1,517),-- Cotton Pants
(20345,1800,1,1,2,8254),-- Recipe: Piece Bone Breastplate
(20345,1831,1,1,2,69986),-- Antidote
(20345,1866,1,1,2,17497),-- Suede
(20345,1871,1,1,2,26245),-- Charcoal
(20345,5268,1,1,2,1050),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20346,1803,1,1,-1,169366),-- Recipe: Necklace of Wisdom
(20346,1831,1,1,-1,537455),-- Antidote
(20346,1869,1,1,-1,201546),-- Iron Ore
(20346,57,106,150,0,700000),-- Adena
(20346,20,1,1,1,4284),-- Buckler
(20346,43,1,1,1,2992),-- Wooden Helmet
(20346,736,1,1,2,19845),-- Scroll of Escape
(20346,1866,1,1,2,26460),-- Suede
(20346,1871,1,1,2,39690),-- Charcoal
(20348,1794,1,1,-1,14192),-- Recipe: Forest Bow
(20348,1831,1,1,-1,514696),-- Antidote
(20348,1896,1,1,-1,81784),-- Necklace of Anguish Chain
(20348,57,97,147,0,700000),-- Adena
(20348,66,1,1,1,141),-- Gladius
(20348,1333,1,1,1,141),-- Brandish
(20348,2010,1,1,1,1479),-- Brandish Blade
(20348,1799,1,1,2,56309),-- Recipe: Leather Gloves
(20348,1866,1,1,2,25339),-- Suede
(20348,1871,1,1,2,38009),-- Charcoal
(20349,1794,1,1,-1,13182),-- Recipe: Forest Bow
(20349,1921,1,1,-1,115664),-- Leather Shoes Texture
(20349,2009,1,1,-1,6357),-- Dirk Blade
(20349,57,94,133,0,700000),-- Adena
(20349,23,1,1,1,887),-- Wooden Breastplate
(20349,30,1,1,1,1421),-- Hard Leather Pants
(20349,2386,1,1,1,1421),-- Wooden Gaiters
(20349,1792,1,1,2,2596),-- Recipe: Short Spear
(20349,1867,1,1,2,47073),-- Animal Skin
(20349,1872,1,1,2,47073),-- Animal Bone
(20350,1800,1,1,-1,63500),-- Recipe: Piece Bone Breastplate
(20350,1866,1,1,-1,134620),-- Suede
(20350,2150,1,1,-1,94803),-- Recipe: Blue Diamond Necklace
(20350,57,100,156,0,700000),-- Adena
(20350,845,1,1,1,671),-- Cat\'s Eye Earring
(20350,877,1,1,1,1008),-- Ring of Wisdom
(20350,908,1,1,1,502),-- Necklace of Wisdom
(20350,1897,1,1,1,7102),-- Necklace of Wisdom Chain
(20350,734,1,1,2,4971),-- Haste Potion
(20350,1868,1,1,2,59648),-- Thread
(20350,1873,1,1,2,11930),-- Silver Nugget
(20350,2150,1,1,2,14002),-- Recipe: Blue Diamond Necklace
(20351,735,1,1,-1,19140),-- Potion of Alacrity
(20351,1897,1,1,-1,54686),-- Necklace of Wisdom Chain
(20351,2173,1,1,-1,62245),-- Recipe: Hard Leather Shirt
(20351,57,106,184,0,700000),-- Adena
(20351,272,1,1,1,100),-- Forest Bow
(20351,2013,1,1,1,1465),-- Forest Bow Shaft
(20351,1864,1,1,2,67846),-- Stem
(20351,1869,1,1,2,33923),-- Iron Ore
(20351,1878,1,1,2,13570),-- Braided Hemp
(20351,2253,1,1,2,1391),-- Recipe: Sword Breaker
(20352,26,1,1,-1,1147),-- Bronze Breastplate
(20352,1878,1,1,-1,112862),-- Braided Hemp
(20352,57,128,230,0,700000),-- Adena
(20352,68,1,1,1,69),-- Falchion
(20352,5284,1,1,1,69),-- Zweihander
(20352,1864,1,1,2,66676),-- Stem
(20352,1869,1,1,2,33338),-- Iron Ore
(20352,1878,1,1,2,13336),-- Braided Hemp
(20352,5163,1,1,2,1334),-- Recipe: Blessed Spiritshot (D) Compressed Package (100%)
(20352,5273,1,1,2,1334),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20353,1864,1,1,-1,484105),-- Stem
(20353,1878,1,1,-1,96821),-- Braided Hemp
(20353,2256,1,1,-1,5918),-- Recipe: Assassin Knife
(20353,57,110,197,0,700000),-- Adena
(20353,16,1,1,1,117),-- Long Spear
(20353,1864,1,1,2,57199),-- Stem
(20353,1869,1,1,2,28600),-- Iron Ore
(20353,1878,1,1,2,11440),-- Braided Hemp
(20353,3953,1,1,2,1144),-- Recipe: Blessed Spiritshot D
(20353,5273,1,1,2,1144),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20356,1868,1,2,-1,713468),-- Thread
(20356,1873,1,1,-1,142694),-- Silver Nugget
(20356,1899,1,1,-1,55740),-- Necklace of Devotion Chain
(20356,57,188,353,0,700000),-- Adena
(20356,220,1,1,1,98),-- Crafted Dagger
(20356,1868,1,1,2,142857),-- Thread
(20356,1873,1,1,2,26316),-- Silver Nugget
(20356,2178,1,1,2,10417),-- Recipe: Dark Stockings
(20356,5549,1,1,2,5208),-- Metallic Thread
(20357,1804,1,1,-1,12071),-- Recipe: Soulshot: D-Grade
(20357,1898,1,1,-1,30999),-- Blue Diamond Necklace Gem
(20357,1926,1,1,-1,30482),-- Piece Bone Breastplate Fragment
(20357,57,137,246,0,700000),-- Adena
(20357,68,1,1,1,73),-- Falchion
(20357,5284,1,1,1,73),-- Zweihander
(20357,1864,1,1,2,89139),-- Stem
(20357,1869,1,1,2,44570),-- Iron Ore
(20357,1880,1,1,2,4457),-- Steel
(20357,2176,1,1,2,21326),-- Recipe: Leather Boots
(20358,1924,1,1,-1,81695),-- Leather Helmet Design
(20358,2140,1,1,-1,50651),-- Recipe: High Grade Suede
(20358,2141,1,1,-1,50651),-- Recipe: Silver Mold
(20358,57,121,200,0,700000),-- Adena
(20358,217,1,1,1,110),-- Shining Knife
(20358,218,1,1,1,110),-- Throwing Knife
(20358,1800,1,1,2,15684),-- Recipe: Piece Bone Breastplate
(20358,1866,1,1,2,33248),-- Suede
(20358,1871,1,1,2,49872),-- Charcoal
(20359,44,1,1,-1,5121),-- Leather Helmet
(20359,1872,1,3,-1,348237),-- Animal Bone
(20359,1924,1,1,-1,84251),-- Leather Helmet Design
(20359,57,122,208,0,700000),-- Adena
(20359,176,1,1,1,114),-- Journeyman\'s Staff
(20359,2501,1,1,1,114),-- Bone Club
(20359,1865,1,1,2,38575),-- Varnish
(20359,1870,1,1,2,38575),-- Coal
(20359,2252,1,1,2,1581),-- Recipe: Iron Hammer
(20359,5268,1,1,2,1543),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20360,908,1,1,-1,3993),-- Necklace of Wisdom
(20360,1801,1,1,-1,119680),-- Recipe: Piece Bone Gaiters
(20360,2175,1,1,-1,193143),-- Recipe: Boots
(20360,57,118,183,0,700000),-- Adena
(20360,44,1,1,1,688),-- Leather Helmet
(20360,102,1,1,1,987),-- Round Shield
(20360,945,1,1,1,2040),-- Skeleton Buckler
(20360,1924,1,1,1,11319),-- Leather Helmet Design
(20360,1867,1,1,2,46782),-- Animal Skin
(20360,1872,1,1,2,46782),-- Animal Bone
(20360,2173,1,1,2,9509),-- Recipe: Hard Leather Shirt
(20360,5273,1,1,2,1404),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20361,42,1,1,-1,33615),-- Leather Cap
(20361,1790,1,1,-1,28303),-- Recipe: Dirk
(20361,2005,1,1,-1,10937),-- Broadsword Blade
(20361,57,82,111,0,700000),-- Adena
(20361,14,1,1,1,724),-- Bow
(20361,2007,1,1,1,3231),-- Bow Shaft
(20361,735,1,1,2,1885),-- Potion of Alacrity
(20361,1802,1,1,2,48633),-- Recipe: Necklace of Anguish
(20361,1864,1,1,2,45229),-- Stem
(20361,1869,1,1,2,22615),-- Iron Ore
(20362,1792,1,1,-1,13068),-- Recipe: Short Spear
(20362,1799,1,1,-1,263305),-- Recipe: Leather Gloves
(20362,1867,1,1,-1,236975),-- Animal Skin
(20362,57,95,131,0,700000),-- Adena
(20362,37,1,1,1,2642),-- Leather Shoes
(20362,49,1,1,1,2642),-- Gloves
(20362,1921,1,1,1,22581),-- Leather Shoes Texture
(20362,1060,1,1,2,58333),-- Lesser Healing Potion
(20362,1792,1,1,2,1931),-- Recipe: Short Spear
(20362,1864,1,1,2,52500),-- Stem
(20362,1869,1,1,2,26250),-- Iron Ore
(20363,1867,1,2,-1,198456),-- Animal Skin
(20363,2005,1,1,-1,10632),-- Broadsword Blade
(20363,2006,1,1,-1,10632),-- Willow Staff Head
(20363,57,80,108,0,700000),-- Adena
(20363,254,1,1,1,1407),-- Iron Gloves
(20363,735,1,1,2,1832),-- Potion of Alacrity
(20363,1814,1,1,2,11570),-- Recipe: Leather
(20363,1864,1,1,2,43966),-- Stem
(20363,1869,1,1,2,21983),-- Iron Ore
(20364,1867,1,1,-1,303119),-- Animal Skin
(20364,1896,1,1,-1,96330),-- Necklace of Anguish Chain
(20364,2008,1,1,-1,8846),-- Cedar Staff Head
(20364,57,116,172,0,700000),-- Adena
(20364,255,1,1,1,249),-- Fox Claw Gloves
(20364,2371,1,1,1,527),-- Fist of Butcher
(20364,1793,1,1,2,3292),-- Recipe: Sword of Reflection
(20364,1865,1,1,2,44769),-- Varnish
(20364,1870,1,1,2,44769),-- Coal
(20365,1864,1,1,-1,249244),-- Stem
(20365,1866,1,1,-1,83081),-- Suede
(20365,1871,1,1,-1,124622),-- Charcoal
(20365,57,67,91,0,700000),-- Adena
(20365,22,1,1,1,3030),-- Leather Shirt
(20365,29,1,1,1,4844),-- Leather Pants
(20365,1666,1,1,2,8181),-- Recipe: Wooden Arrow
(20365,1866,1,1,2,16361),-- Suede
(20365,1871,1,1,2,24541),-- Charcoal
(20367,1831,1,1,-1,612488),-- Antidote
(20367,1864,1,1,-1,459366),-- Stem
(20367,1896,1,1,-1,97323),-- Necklace of Anguish Chain
(20367,57,106,184,0,700000),-- Adena
(20367,44,1,1,1,666),-- Leather Helmet
(20367,102,1,1,1,955),-- Round Shield
(20367,945,1,1,1,1973),-- Skeleton Buckler
(20367,1924,1,1,1,10943),-- Leather Helmet Design
(20367,735,1,1,2,2262),-- Potion of Alacrity
(20367,1864,1,1,2,54276),-- Stem
(20367,1869,1,1,2,27138),-- Iron Ore
(20367,1878,1,1,2,10856),-- Braided Hemp
(20367,5153,1,1,2,1086),-- Recipe: Soulshot (D) Compressed Package (100%)
(20368,1793,1,1,-1,14820),-- Recipe: Sword of Reflection
(20368,1833,1,1,-1,537455),-- Bandage
(20368,1873,1,1,-1,80618),-- Silver Nugget
(20368,57,106,150,0,700000),-- Adena
(20368,271,1,1,1,440),-- Hunting Bow
(20368,1793,1,1,2,2919),-- Recipe: Sword of Reflection
(20368,1866,1,1,2,26460),-- Suede
(20368,1871,1,1,2,39690),-- Charcoal
(20369,390,1,1,-1,2400),-- Cotton Shirt
(20369,1897,1,1,-1,58010),-- Necklace of Wisdom Chain
(20369,2143,1,1,-1,48728),-- Recipe: Synthetic Cokes
(20369,57,119,191,0,700000),-- Adena
(20369,1101,1,1,1,709),-- Tunic of Devotion
(20369,1104,1,1,1,1134),-- Stockings of Devotion
(20369,1868,1,1,2,95958),-- Thread
(20369,1873,1,1,2,19192),-- Silver Nugget
(20369,2175,1,1,2,39008),-- Recipe: Boots
(20370,1794,1,1,-1,15375),-- Recipe: Forest Bow
(20370,1864,1,2,-1,418191),-- Stem
(20370,1922,1,1,-1,53341),-- Leather Tunic Pattern
(20370,57,112,154,0,700000),-- Adena
(20370,23,1,1,1,1035),-- Wooden Breastplate
(20370,30,1,1,1,1658),-- Hard Leather Pants
(20370,2386,1,1,1,1658),-- Wooden Gaiters
(20370,1793,1,1,2,3028),-- Recipe: Sword of Reflection
(20370,1864,1,1,2,82352),-- Stem
(20370,1869,1,1,2,41176),-- Iron Ore
(20371,114,1,1,-1,11218),-- Earring of Strength
(20371,1869,1,1,-1,196871),-- Iron Ore
(20371,1896,1,1,-1,83420),-- Necklace of Anguish Chain
(20371,57,99,150,0,700000),-- Adena
(20371,429,1,1,1,731),-- Leather Tunic
(20371,464,1,1,1,1171),-- Leather Stockings
(20371,1922,1,1,1,7418),-- Leather Tunic Pattern
(20371,1923,1,1,1,11917),-- Leather Stocking Pattern
(20371,1803,1,1,2,32579),-- Recipe: Necklace of Wisdom
(20371,1866,1,1,2,25846),-- Suede
(20371,1871,1,1,2,38769),-- Charcoal
(20372,1864,1,1,-1,303261),-- Stem
(20372,1870,1,2,-1,151631),-- Coal
(20372,57,81,110,0,700000),-- Adena
(20372,3,1,1,1,717),-- Broadsword
(20372,2005,1,1,1,3200),-- Broadsword Blade
(20372,1814,1,1,2,15716),-- Recipe: Leather
(20372,1864,1,1,2,59719),-- Stem
(20372,1869,1,1,2,29860),-- Iron Ore
(20373,1793,1,1,-1,12352),-- Recipe: Sword of Reflection
(20373,1869,1,1,-1,167982),-- Iron Ore
(20373,1871,1,1,-1,167982),-- Charcoal
(20373,57,91,122,0,700000),-- Adena
(20373,255,1,1,1,184),-- Fox Claw Gloves
(20373,2371,1,1,1,390),-- Fist of Butcher
(20373,1799,1,1,2,49008),-- Recipe: Leather Gloves
(20373,1864,1,1,2,66160),-- Stem
(20373,1869,1,1,2,33080),-- Iron Ore
(20374,1871,1,1,-1,334740),-- Charcoal
(20374,1898,1,1,-1,34385),-- Blue Diamond Necklace Gem
(20374,2255,1,1,-1,8184),-- Recipe: Saber
(20374,57,152,273,0,700000),-- Adena
(20374,257,1,1,1,162),-- Viper\'s Fang
(20374,1868,1,1,2,98878),-- Thread
(20374,1873,1,1,2,19776),-- Silver Nugget
(20374,5158,1,1,2,1978),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20374,5273,1,1,2,1978),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20375,1804,1,1,-1,9682),-- Recipe: Soulshot: D-Grade
(20375,2014,1,1,-1,2782),-- Iron Hammer Head
(20375,2257,1,1,-1,5918),-- Recipe: Trident
(20375,57,110,197,0,700000),-- Adena
(20375,16,1,1,1,117),-- Long Spear
(20375,1865,1,1,2,28600),-- Varnish
(20375,1870,1,1,2,28600),-- Coal
(20375,1879,1,1,2,4767),-- Cokes
(20375,5158,1,1,2,1144),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20378,49,1,1,-1,15211),-- Gloves
(20378,1831,1,1,-1,537455),-- Antidote
(20378,1896,1,1,-1,85401),-- Necklace of Anguish Chain
(20378,57,106,150,0,700000),-- Adena
(20378,114,1,1,1,1357),-- Earring of Strength
(20378,115,1,1,1,1357),-- Earring of Wisdom
(20378,876,1,1,1,2036),-- Ring of Anguish
(20378,907,1,1,1,1018),-- Necklace of Anguish
(20378,1896,1,1,1,10091),-- Necklace of Anguish Chain
(20378,736,1,1,2,19845),-- Scroll of Escape
(20378,1866,1,1,2,26460),-- Suede
(20378,1871,1,1,2,39690),-- Charcoal
(20379,1792,1,1,-1,18424),-- Recipe: Short Spear
(20379,1871,1,3,-1,250564),-- Charcoal
(20379,1872,1,2,-1,334085),-- Animal Bone
(20379,57,128,189,0,700000),-- Adena
(20379,114,1,1,1,1687),-- Earring of Strength
(20379,115,1,1,1,1687),-- Earring of Wisdom
(20379,876,1,1,1,2531),-- Ring of Anguish
(20379,907,1,1,1,1266),-- Necklace of Anguish
(20379,1896,1,1,1,12545),-- Necklace of Anguish Chain
(20379,1794,1,1,2,3629),-- Recipe: Forest Bow
(20379,1867,1,1,2,65789),-- Animal Skin
(20379,1872,1,1,2,65789),-- Animal Bone
(20380,1866,1,1,-1,178047),-- Suede
(20380,1869,1,1,-1,267071),-- Iron Ore
(20380,1873,1,1,-1,106828),-- Silver Nugget
(20380,57,132,206,0,700000),-- Adena
(20380,176,1,1,1,116),-- Journeyman\'s Staff
(20380,2501,1,1,1,116),-- Bone Club
(20380,1866,1,1,2,35062),-- Suede
(20380,1871,1,1,2,52593),-- Charcoal
(20380,2174,1,1,2,22867),-- Recipe: Hard Leather Gaiters
(20381,1867,1,1,-1,198210),-- Animal Skin
(20381,1875,1,1,-1,9910),-- Stone of Purity
(20381,2006,1,1,-1,10618),-- Willow Staff Head
(20381,57,80,108,0,700000),-- Adena
(20381,99,1,1,1,1405),-- Apprentice\'s Spellbook
(20381,1865,1,1,2,29274),-- Varnish
(20381,1870,1,1,2,29274),-- Coal
(20381,2136,1,1,2,8611),-- Recipe: Cokes
(20382,1831,1,1,-1,447952),-- Antidote
(20382,1870,1,1,-1,167982),-- Coal
(20382,2008,1,1,-1,6536),-- Cedar Staff Head
(20382,57,91,122,0,700000),-- Adena
(20382,20,1,1,1,3570),-- Buckler
(20382,43,1,1,1,2494),-- Wooden Helmet
(20382,1803,1,1,2,27798),-- Recipe: Necklace of Wisdom
(20382,1866,1,1,2,22053),-- Suede
(20382,1871,1,1,2,33080),-- Charcoal
(20385,1060,1,1,-1,285245),-- Lesser Healing Potion
(20385,2005,1,1,-1,9169),-- Broadsword Blade
(20385,57,69,94,0,700000),-- Adena
(20385,14,1,1,1,607),-- Bow
(20385,2007,1,1,1,2709),-- Bow Shaft
(20385,1790,1,1,2,4673),-- Recipe: Dirk
(20385,1868,1,1,2,50555),-- Thread
(20385,1873,1,1,2,10111),-- Silver Nugget
(20386,1060,1,1,-1,404348),-- Lesser Healing Potion
(20386,1666,1,1,-1,60652),-- Recipe: Wooden Arrow
(20386,1796,1,1,-1,228876),-- Recipe: Leather Tunic
(20386,57,98,132,0,700000),-- Adena
(20386,3,1,1,1,860),-- Broadsword
(20386,2005,1,1,1,3840),-- Broadsword Blade
(20386,17,10,30,2,134369),-- Wooden Arrow
(20386,1666,1,1,2,8958),-- Recipe: Wooden Arrow
(20386,1864,1,1,2,53748),-- Stem
(20386,1869,1,1,2,26874),-- Iron Ore
(20387,1794,1,1,-1,15682),-- Recipe: Forest Bow
(20387,1833,1,1,-1,568739),-- Bandage
(20387,1896,1,1,-1,90372),-- Necklace of Anguish Chain
(20387,57,114,157,0,700000),-- Adena
(20387,66,1,1,1,156),-- Gladius
(20387,1333,1,1,1,156),-- Brandish
(20387,2010,1,1,1,1635),-- Brandish Blade
(20387,1794,1,1,2,3089),-- Recipe: Forest Bow
(20387,1864,1,1,2,83999),-- Stem
(20387,1869,1,1,2,42000),-- Iron Ore
(20388,1833,1,1,-1,537542),-- Bandage
(20388,1880,1,1,-1,20158),-- Steel
(20388,57,109,147,0,700000),-- Adena
(20388,37,1,1,1,2996),-- Leather Shoes
(20388,49,1,1,1,2996),-- Gloves
(20388,1921,1,1,1,25611),-- Leather Shoes Texture
(20388,735,1,1,2,2481),-- Potion of Alacrity
(20388,736,1,1,2,14886),-- Scroll of Escape
(20388,1864,1,1,2,59543),-- Stem
(20388,1869,1,1,2,29772),-- Iron Ore
(20389,734,1,1,-1,32995),-- Haste Potion
(20389,1872,1,2,-1,263962),-- Animal Bone
(20389,2011,1,1,-1,4516),-- Short Spear Edge
(20389,57,98,153,0,700000),-- Adena
(20389,44,1,1,1,574),-- Leather Helmet
(20389,102,1,1,1,823),-- Round Shield
(20389,945,1,1,1,1700),-- Skeleton Buckler
(20389,1924,1,1,1,9433),-- Leather Helmet Design
(20389,1867,1,1,2,38986),-- Animal Skin
(20389,1872,1,1,2,38986),-- Animal Bone
(20389,2175,1,1,2,23772),-- Recipe: Boots
(20389,5158,1,1,2,1170),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20390,1873,1,1,-1,84419),-- Silver Nugget
(20390,1881,1,1,-1,28140),-- Coarse Bone Powder
(20390,1897,1,1,-1,50249),-- Necklace of Wisdom Chain
(20390,57,101,167,0,700000),-- Adena
(20390,2,1,1,1,46),-- Long Sword
(20390,120,1,1,1,46),-- Sword of Reflection
(20390,121,1,1,1,46),-- Sword of Watershadow
(20390,2012,1,1,1,650),-- Sword of Reflection Blade
(20390,734,1,1,2,4157),-- Haste Potion
(20390,1801,1,1,2,12563),-- Recipe: Piece Bone Gaiters
(20390,1868,1,1,2,49872),-- Thread
(20390,1873,1,1,2,9975),-- Silver Nugget
(20390,5158,1,1,2,998),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20392,1798,1,1,-1,227072),-- Recipe: Leather Helmet
(20392,1831,1,1,-1,617635),-- Antidote
(20392,1833,1,1,-1,617635),-- Bandage
(20392,57,116,177,0,700000),-- Adena
(20392,23,1,1,1,1147),-- Wooden Breastplate
(20392,30,1,1,1,1836),-- Hard Leather Pants
(20392,2386,1,1,1,1836),-- Wooden Gaiters
(20392,736,1,1,2,17104),-- Scroll of Escape
(20392,1060,1,1,2,76017),-- Lesser Healing Potion
(20392,1864,1,1,2,68416),-- Stem
(20392,1869,1,1,2,34208),-- Iron Ore
(20393,1897,1,1,-1,56563),-- Necklace of Wisdom Chain
(20393,2252,1,1,-1,9736),-- Recipe: Iron Hammer
(20393,57,118,183,0,700000),-- Adena
(20393,217,1,1,1,103),-- Shining Knife
(20393,218,1,1,1,103),-- Throwing Knife
(20393,1867,1,1,2,62377),-- Animal Skin
(20393,1872,1,1,2,62377),-- Animal Bone
(20393,2252,1,1,2,1918),-- Recipe: Iron Hammer
(20400,1803,1,1,-1,175710),-- Recipe: Necklace of Wisdom
(20400,1921,1,1,-1,134900),-- Leather Shoes Texture
(20400,1923,1,1,-1,85695),-- Leather Stocking Pattern
(20400,57,112,154,0,700000),-- Adena
(20400,114,1,1,1,1408),-- Earring of Strength
(20400,115,1,1,1,1408),-- Earring of Wisdom
(20400,876,1,1,1,2112),-- Ring of Anguish
(20400,907,1,1,1,1056),-- Necklace of Anguish
(20400,1896,1,1,1,10469),-- Necklace of Anguish Chain
(20400,1798,1,1,2,40369),-- Recipe: Leather Helmet
(20400,1864,1,1,2,82352),-- Stem
(20400,1869,1,1,2,41176),-- Iron Ore
(20401,1866,1,1,-1,210080),-- Suede
(20401,1873,1,1,-1,126048),-- Silver Nugget
(20401,2151,1,1,-1,87777),-- Recipe: Necklace of Devotion
(20401,57,144,255,0,700000),-- Adena
(20401,87,1,1,1,39),-- Iron Hammer
(20401,155,1,1,1,39),-- Buzdygan
(20401,177,1,1,1,39),-- Mage Staff
(20401,2014,1,1,1,535),-- Iron Hammer Head
(20401,1866,1,1,2,31028),-- Suede
(20401,1871,1,1,2,46541),-- Charcoal
(20401,5153,1,1,2,1862),-- Recipe: Soulshot (D) Compressed Package (100%)
(20401,5158,1,1,2,1862),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20402,1871,1,2,-1,339480),-- Charcoal
(20402,1880,1,1,-1,33948),-- Steel
(20402,2179,1,1,-1,162431),-- Recipe: Crafted Leather Gloves
(20402,57,151,278,0,700000),-- Adena
(20402,25,1,1,1,211),-- Piece Bone Breastplate
(20402,26,1,1,1,136),-- Bronze Breastplate
(20402,32,1,1,1,336),-- Piece Bone Gaiters
(20402,34,1,1,1,218),-- Bronze Gaiters
(20402,1926,1,1,1,3377),-- Piece Bone Breastplate Fragment
(20402,1928,1,1,1,4869),-- Piece Bone Gaiters Fragment
(20402,1866,1,1,2,33426),-- Suede
(20402,1871,1,1,2,50139),-- Charcoal
(20402,2256,1,1,2,1226),-- Recipe: Assassin Knife
(20402,5158,1,1,2,2006),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20403,734,1,1,-1,39594),-- Haste Potion
(20403,1868,1,3,-1,475131),-- Thread
(20403,2012,1,1,-1,4949),-- Sword of Reflection Blade
(20403,57,118,183,0,700000),-- Adena
(20403,38,1,1,1,1383),-- Low Boots
(20403,50,1,1,1,1383),-- Leather Gloves
(20403,1925,1,1,1,30677),-- Leather Gloves Lining
(20403,735,1,1,2,2340),-- Potion of Alacrity
(20403,1867,1,1,2,37426),-- Animal Skin
(20403,1872,1,1,2,37426),-- Animal Bone
(20403,2253,1,1,2,1151),-- Recipe: Sword Breaker
(20403,5268,1,1,2,1123),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20404,735,1,1,-1,16828),-- Potion of Alacrity
(20404,1897,1,1,-1,48079),-- Necklace of Wisdom Chain
(20404,2253,1,1,-1,8276),-- Recipe: Sword Breaker
(20404,57,100,156,0,700000),-- Adena
(20404,1101,1,1,1,588),-- Tunic of Devotion
(20404,1104,1,1,1,940),-- Stockings of Devotion
(20404,1866,1,1,2,19883),-- Suede
(20404,1871,1,1,2,29824),-- Charcoal
(20404,2254,1,1,2,1223),-- Recipe: Composition Bow
(20404,6035,1,1,2,2486),-- Magic Haste Potion
(20410,1792,1,1,-1,17371),-- Recipe: Short Spear
(20410,1868,1,1,-1,472491),-- Thread
(20410,2009,1,1,-1,8377),-- Dirk Blade
(20410,57,119,180,0,700000),-- Adena
(20410,9,1,1,1,172),-- Cedar Staff
(20410,154,1,1,1,172),-- Dwarven Mace
(20410,2008,1,1,1,1811),-- Cedar Staff Head
(20410,1793,1,1,2,3421),-- Recipe: Sword of Reflection
(20410,1867,1,1,2,62030),-- Animal Skin
(20410,1872,1,1,2,62030),-- Animal Bone
(20411,735,1,1,-1,21105),-- Potion of Alacrity
(20411,1897,1,1,-1,60299),-- Necklace of Wisdom Chain
(20411,2141,1,1,-1,50651),-- Recipe: Silver Mold
(20411,57,121,200,0,700000),-- Adena
(20411,217,1,1,1,110),-- Shining Knife
(20411,218,1,1,1,110),-- Throwing Knife
(20411,1817,1,1,2,5985),-- Recipe: Cord
(20411,1864,1,1,2,59847),-- Stem
(20411,1869,1,1,2,29924),-- Iron Ore
(20411,1878,1,1,2,11970),-- Braided Hemp
(20411,6037,1,1,2,59847),-- Waking Scroll
(20415,735,1,1,-1,18798),-- Potion of Alacrity
(20415,1865,1,1,-1,225572),-- Varnish
(20415,2144,1,1,-1,45114),-- Recipe: Compound Braid
(20415,57,110,177,0,700000),-- Adena
(20415,309,1,1,1,98),-- Tears of Eva
(20415,310,1,1,1,98),-- Relic of The Saints
(20415,1864,1,1,2,44421),-- Stem
(20415,1865,1,1,2,22211),-- Varnish
(20415,1866,1,1,2,14807),-- Suede
(20415,1868,1,1,2,44421),-- Thread
(20415,1873,1,1,2,8885),-- Silver Nugget
(20415,2253,1,1,2,911),-- Recipe: Sword Breaker
(20416,102,1,1,-1,6530),-- Round Shield
(20416,1924,1,1,-1,74887),-- Leather Helmet Design
(20416,2254,1,1,-1,9514),-- Recipe: Composition Bow
(20416,57,111,183,0,700000),-- Adena
(20416,1101,1,1,1,676),-- Tunic of Devotion
(20416,1104,1,1,1,1081),-- Stockings of Devotion
(20416,1866,1,1,2,22859),-- Suede
(20416,1871,1,1,2,34288),-- Charcoal
(20416,2139,1,1,2,6858),-- Recipe: Steel Mold
(20416,5268,1,1,2,1372),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20418,1800,1,1,-1,78852),-- Recipe: Piece Bone Breastplate
(20418,1872,1,2,-1,334331),-- Animal Bone
(20418,1925,1,1,-1,164425),-- Leather Gloves Lining
(20418,57,120,198,0,700000),-- Adena
(20418,845,1,1,1,834),-- Cat\'s Eye Earring
(20418,877,1,1,1,1252),-- Ring of Wisdom
(20418,908,1,1,1,623),-- Necklace of Wisdom
(20418,1897,1,1,1,8818),-- Necklace of Wisdom Chain
(20418,1867,1,1,2,49379),-- Animal Skin
(20418,1872,1,1,2,49379),-- Animal Bone
(20418,1882,1,1,2,8230),-- Leather
(20418,2140,1,1,2,7407),-- Recipe: High Grade Suede
(20420,1804,1,1,-1,9682),-- Recipe: Soulshot: D-Grade
(20420,1866,1,1,-1,161368),-- Suede
(20420,1868,1,1,-1,484105),-- Thread
(20420,57,110,197,0,700000),-- Adena
(20420,1102,1,1,1,388),-- Tunic of Magic
(20420,1105,1,1,1,622),-- Stockings of Magic
(20420,1864,1,1,2,47666),-- Stem
(20420,1865,1,1,2,23833),-- Varnish
(20420,1866,1,1,2,15889),-- Suede
(20420,1868,1,1,2,47666),-- Thread
(20420,1873,1,1,2,9534),-- Silver Nugget
(20420,5163,1,1,2,954),-- Recipe: Blessed Spiritshot (D) Compressed Package (100%)
(20421,1866,1,1,-1,171004),-- Suede
(20421,1898,1,1,-1,26349),-- Blue Diamond Necklace Gem
(20421,1929,1,1,-1,33097),-- Hard Leather Gaiters Material
(20421,57,116,209,0,700000),-- Adena
(20421,273,1,1,1,63),-- Composition Bow
(20421,2016,1,1,1,888),-- Composition Bow Shaft
(20421,1867,1,1,2,33675),-- Animal Skin
(20421,1869,1,1,2,25256),-- Iron Ore
(20421,1870,1,1,2,25256),-- Coal
(20421,1871,1,1,2,25256),-- Charcoal
(20421,1872,1,1,2,33675),-- Animal Bone
(20421,5278,1,1,2,1011),-- Recipe: Greater Blessed Spiritshot (D) Compressed Package(100%)
(20422,1864,1,2,-1,497816),-- Stem
(20422,1869,1,1,-1,248908),-- Iron Ore
(20422,2013,1,1,-1,5372),-- Forest Bow Shaft
(20422,57,121,195,0,700000),-- Adena
(20422,176,1,1,1,108),-- Journeyman\'s Staff
(20422,2501,1,1,1,108),-- Bone Club
(20422,1864,1,1,2,73524),-- Stem
(20422,1869,1,1,2,36762),-- Iron Ore
(20422,2254,1,1,2,1507),-- Recipe: Composition Bow
(20422,5273,1,1,2,1471),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20423,1792,1,1,-1,14067),-- Recipe: Short Spear
(20423,1793,1,1,-1,14067),-- Recipe: Sword of Reflection
(20423,1921,1,1,-1,123424),-- Leather Shoes Texture
(20423,57,98,145,0,700000),-- Adena
(20423,271,1,1,1,418),-- Hunting Bow
(20423,1867,1,1,2,75346),-- Animal Skin
(20423,1872,1,1,2,75346),-- Animal Bone
(20424,1870,1,1,-1,227995),-- Coal
(20424,1880,1,1,-1,22799),-- Steel
(20424,2142,1,1,-1,45599),-- Recipe: Varnish of Purity
(20424,57,107,182,0,700000),-- Adena
(20424,2,1,1,1,50),-- Long Sword
(20424,120,1,1,1,50),-- Sword of Reflection
(20424,121,1,1,1,50),-- Sword of Watershadow
(20424,2012,1,1,1,702),-- Sword of Reflection Blade
(20424,1865,1,1,2,44898),-- Varnish
(20424,1870,1,1,2,44898),-- Coal
(20424,2253,1,1,2,1841),-- Recipe: Sword Breaker
(20425,1831,1,1,-1,473949),-- Antidote
(20425,1869,1,1,-1,177731),-- Iron Ore
(20425,57,95,131,0,700000),-- Adena
(20425,66,1,1,1,130),-- Gladius
(20425,1333,1,1,1,130),-- Brandish
(20425,2010,1,1,1,1363),-- Brandish Blade
(20425,1799,1,1,2,51851),-- Recipe: Leather Gloves
(20425,1866,1,1,2,23333),-- Suede
(20425,1871,1,1,2,35000),-- Charcoal
(20426,1796,1,1,-1,224389),-- Recipe: Leather Tunic
(20426,2005,1,1,-1,12742),-- Broadsword Blade
(20426,57,96,130,0,700000),-- Adena
(20426,113,1,1,1,8664),-- Mystic\'s Earring
(20426,875,1,1,1,13011),-- Ring of Knowledge
(20426,906,1,1,1,6506),-- Necklace of Knowledge
(20426,1060,1,1,2,58549),-- Lesser Healing Potion
(20426,1868,1,1,2,52694),-- Thread
(20426,1873,1,1,2,10539),-- Silver Nugget
(20426,2137,1,1,2,7750),-- Recipe: Steel
(20427,1060,1,1,-1,330350),-- Lesser Healing Potion
(20427,1666,1,1,-1,49552),-- Recipe: Wooden Arrow
(20427,1866,1,1,-1,99105),-- Suede
(20427,57,80,108,0,700000),-- Adena
(20427,254,1,1,1,1405),-- Iron Gloves
(20427,1864,1,1,2,58549),-- Stem
(20427,1869,1,1,2,29274),-- Iron Ore
(20427,2138,1,1,2,8611),-- Recipe: Coarse Bone Powder (100%)
(20428,1831,1,1,-1,594351),-- Antidote
(20428,1864,1,1,-1,445764),-- Stem
(20428,1896,1,1,-1,94441),-- Necklace of Anguish Chain
(20428,57,114,168,0,700000),-- Adena
(20428,114,1,1,1,1501),-- Earring of Strength
(20428,115,1,1,1,1501),-- Earring of Wisdom
(20428,876,1,1,1,2251),-- Ring of Anguish
(20428,907,1,1,1,1126),-- Necklace of Anguish
(20428,1896,1,1,1,11159),-- Necklace of Anguish Chain
(20428,1799,1,1,2,27868),-- Recipe: Leather Gloves
(20428,1833,1,1,2,50161),-- Bandage
(20428,1864,1,1,2,37621),-- Stem
(20428,1865,1,1,2,18811),-- Varnish
(20428,1866,1,1,2,12541),-- Suede
(20428,1868,1,1,2,37621),-- Thread
(20428,1873,1,1,2,7525),-- Silver Nugget
(20429,1792,1,1,-1,13657),-- Recipe: Short Spear
(20429,1921,1,1,-1,119829),-- Leather Shoes Texture
(20429,2009,1,1,-1,6586),-- Dirk Blade
(20429,57,95,140,0,700000),-- Adena
(20429,429,1,1,1,690),-- Leather Tunic
(20429,464,1,1,1,1104),-- Leather Stockings
(20429,1922,1,1,1,6999),-- Leather Tunic Pattern
(20429,1923,1,1,1,11243),-- Leather Stocking Pattern
(20429,1803,1,1,2,30736),-- Recipe: Necklace of Wisdom
(20429,1868,1,1,2,73152),-- Thread
(20429,1873,1,1,2,14631),-- Silver Nugget
(20432,1871,1,1,-1,15920),-- Charcoal
(20432,1873,1,1,-1,6368),-- Silver Nugget
(20432,57,8,12,0,700000),-- Adena
(20432,11,1,1,1,4286),-- Bone Dagger
(20432,17,10,30,2,11757),-- Wooden Arrow
(20432,1787,1,1,2,1881),-- Recipe: Willow Staff
(20432,1864,1,1,2,4703),-- Stem
(20432,1865,1,1,2,2352),-- Varnish
(20432,1866,1,1,2,1568),-- Suede
(20432,1868,1,1,2,4703),-- Thread
(20432,1873,1,1,2,941),-- Silver Nugget
(20433,1864,1,2,-1,430270),-- Stem
(20433,1865,1,1,-1,215135),-- Varnish
(20433,1921,1,1,-1,138797),-- Leather Shoes Texture
(20433,57,113,160,0,700000),-- Adena
(20433,429,1,1,1,799),-- Leather Tunic
(20433,464,1,1,1,1279),-- Leather Stockings
(20433,1922,1,1,1,8106),-- Leather Tunic Pattern
(20433,1923,1,1,1,13023),-- Leather Stocking Pattern
(20433,735,1,1,2,2648),-- Potion of Alacrity
(20433,1794,1,1,2,2337),-- Recipe: Forest Bow
(20433,1865,1,1,2,31774),-- Varnish
(20433,1870,1,1,2,31774),-- Coal
(20435,1793,1,1,-1,14332),-- Recipe: Sword of Reflection
(20435,1833,1,1,-1,519792),-- Bandage
(20435,1871,1,1,-1,194922),-- Charcoal
(20435,57,98,149,0,700000),-- Adena
(20435,216,1,1,1,213),-- Dirk
(20435,2009,1,1,1,2043),-- Dirk Blade
(20435,1794,1,1,2,2823),-- Recipe: Forest Bow
(20435,1867,1,1,2,51180),-- Animal Skin
(20435,1872,1,1,2,51180),-- Animal Bone
(20436,1869,1,2,-1,315120),-- Iron Ore
(20436,1898,1,1,-1,32370),-- Blue Diamond Necklace Gem
(20436,2016,1,1,-1,3693),-- Composition Bow Shaft
(20436,57,144,255,0,700000),-- Adena
(20436,16,1,1,1,153),-- Long Spear
(20436,1865,1,1,2,62055),-- Varnish
(20436,1870,1,1,2,62055),-- Coal
(20436,5158,1,1,2,2483),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20437,1804,1,1,-1,11391),-- Recipe: Soulshot: D-Grade
(20437,2016,1,1,-1,3337),-- Composition Bow Shaft
(20437,2258,1,1,-1,6963),-- Recipe: Temptation of Abyss
(20437,57,130,231,0,700000),-- Adena
(20437,39,1,1,1,912),-- Boots
(20437,51,1,1,1,912),-- Bracer
(20437,1930,1,1,1,14379),-- Boot Lining
(20437,1866,1,1,2,28039),-- Suede
(20437,1871,1,1,2,42058),-- Charcoal
(20437,5268,1,1,2,1683),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20437,5437,1,1,2,1029),-- Recipe: Heavy Sword(100%)
(20438,1865,1,2,-1,375763),-- Varnish
(20438,1869,1,2,-1,375763),-- Iron Ore
(20438,1899,1,1,-1,58713),-- Necklace of Devotion Chain
(20438,57,196,374,0,700000),-- Adena
(20438,291,1,1,1,52),-- Trident
(20438,2019,1,1,1,2268),-- Trident Edge
(20438,1875,1,1,2,2849),-- Stone of Purity
(20438,1879,1,1,2,8130),-- Cokes
(20438,2180,1,1,2,4274),-- Recipe: Scale Mail
(20438,5549,1,1,2,5714),-- Metallic Thread
(20438,6666,1,1,2,50000),-- Deluxe Chest Key - Grade 2
(20439,2015,1,1,-1,4059),-- Sword Breaker Edge
(20439,2151,1,1,-1,92709),-- Recipe: Necklace of Devotion
(20439,2255,1,1,-1,8137),-- Recipe: Saber
(20439,57,148,273,0,700000),-- Adena
(20439,25,1,1,1,207),-- Piece Bone Breastplate
(20439,26,1,1,1,134),-- Bronze Breastplate
(20439,32,1,1,1,330),-- Piece Bone Gaiters
(20439,34,1,1,1,214),-- Bronze Gaiters
(20439,1926,1,1,1,3311),-- Piece Bone Breastplate Fragment
(20439,1928,1,1,1,4774),-- Piece Bone Gaiters Fragment
(20439,1868,1,1,2,131082),-- Thread
(20439,1873,1,1,2,26217),-- Silver Nugget
(20439,2257,1,1,2,1603),-- Recipe: Trident
(20442,1786,1,1,-1,44845),-- Recipe: Broad Sword
(20442,1864,1,1,-1,112112),-- Stem
(20442,5549,1,1,-1,5606),-- Metallic Thread
(20442,57,30,42,0,700000),-- Adena
(20442,4,1,1,1,7546),-- Club
(20442,7,1,1,1,7546),-- Apprentice\'s Rod
(20442,17,10,30,2,72442),-- Wooden Arrow
(20442,1787,1,1,2,11591),-- Recipe: Willow Staff
(20442,1868,1,1,2,28977),-- Thread
(20442,1873,1,1,2,5796),-- Silver Nugget
(20446,1869,1,1,-1,77741),-- Iron Ore
(20446,1870,1,1,-1,77741),-- Coal
(20446,2007,1,1,-1,5553),-- Bow Shaft
(20446,57,41,58,0,700000),-- Adena
(20446,22,1,1,1,1890),-- Leather Shirt
(20446,29,1,1,1,3022),-- Leather Pants
(20446,1791,1,1,2,2830),-- Recipe: Brandish
(20446,1864,1,1,2,30618),-- Stem
(20446,1869,1,1,2,15309); -- Iron Ore

INSERT INTO `droplist` VALUES 
(20447,1060,1,1,-1,285245),-- Lesser Healing Potion
(20447,1797,1,1,-1,259314),-- Recipe: Leather Stockings
(20447,2138,1,1,-1,37753),-- Recipe: Coarse Bone Powder (100%)
(20447,57,69,94,0,700000),-- Adena
(20447,14,1,1,1,607),-- Bow
(20447,2007,1,1,1,2709),-- Bow Shaft
(20447,1791,1,1,2,4673),-- Recipe: Brandish
(20447,1868,1,1,2,50555),-- Thread
(20447,1873,1,1,2,10111),-- Silver Nugget
(20448,1666,1,1,-1,32439),-- Recipe: Wooden Arrow
(20448,1789,1,1,-1,17988),-- Recipe: Cedar Staff
(20448,2006,1,1,-1,6951),-- Willow Staff Head
(20448,57,51,72,0,700000),-- Adena
(20448,3,1,1,1,460),-- Broadsword
(20448,2005,1,1,1,2054),-- Broadsword Blade
(20448,1866,1,1,2,19164),-- Suede
(20448,1871,1,1,2,28746),-- Charcoal
(20449,115,1,1,-1,13164),-- Earring of Wisdom
(20449,1873,1,1,-1,92414),-- Silver Nugget
(20449,1896,1,1,-1,97896),-- Necklace of Anguish Chain
(20449,57,116,177,0,700000),-- Adena
(20449,429,1,1,1,858),-- Leather Tunic
(20449,464,1,1,1,1374),-- Leather Stockings
(20449,1922,1,1,1,8705),-- Leather Tunic Pattern
(20449,1923,1,1,1,13985),-- Leather Stocking Pattern
(20449,1798,1,1,2,44605),-- Recipe: Leather Helmet
(20449,1868,1,1,2,90993),-- Thread
(20449,1873,1,1,2,18199),-- Silver Nugget
(20450,1666,1,1,-1,41345),-- Recipe: Wooden Arrow
(20450,1870,1,1,-1,124035),-- Coal
(20450,57,67,90,0,700000),-- Adena
(20450,5,1,1,1,294),-- Mace
(20450,8,1,1,1,294),-- Willow Staff
(20450,152,1,1,1,455),-- Heavy Chisel
(20450,2006,1,1,1,1309),-- Willow Staff Head
(20450,1864,1,1,2,48851),-- Stem
(20450,1869,1,1,2,24426),-- Iron Ore
(20450,2135,1,1,2,7184),-- Recipe: Braided Hemp
(20455,1794,1,1,-1,13930),-- Recipe: Forest Bow
(20455,1869,1,1,-1,189449),-- Iron Ore
(20455,1896,1,1,-1,80275),-- Necklace of Anguish Chain
(20455,57,97,143,0,700000),-- Adena
(20455,20,1,1,1,4026),-- Buckler
(20455,43,1,1,1,2813),-- Wooden Helmet
(20455,736,1,1,2,18654),-- Scroll of Escape
(20455,1868,1,1,2,74614),-- Thread
(20455,1873,1,1,2,14923),-- Silver Nugget
(20456,1788,1,1,-1,30885),-- Recipe: Bow 
(20456,1868,1,1,-1,77212),-- Thread
(20456,1872,1,1,-1,51475),-- Animal Bone
(20456,57,19,29,0,700000),-- Adena
(20456,308,1,1,1,10394),-- Buffalo\'s Horn
(20456,17,10,30,2,49891),-- Wooden Arrow
(20456,1788,1,1,2,7983),-- Recipe: Bow 
(20456,1868,1,1,2,19957),-- Thread
(20456,1873,1,1,2,3992),-- Silver Nugget
(20457,1864,1,3,-1,587232),-- Stem
(20457,1866,1,2,-1,195744),-- Suede
(20457,2013,1,1,-1,6337),-- Forest Bow Shaft
(20457,57,138,234,0,700000),-- Adena
(20457,15,1,1,1,128),-- Short Spear
(20457,2011,1,1,1,1979),-- Short Spear Edge
(20457,1864,1,1,2,69384),-- Stem
(20457,1869,1,1,2,34692),-- Iron Ore
(20457,1880,1,1,2,3470),-- Steel
(20457,2254,1,1,2,1422),-- Recipe: Composition Bow
(20457,5273,1,1,2,1388),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20458,1898,1,1,-1,35546),-- Blue Diamond Necklace Gem
(20458,2176,1,1,-1,165568),-- Recipe: Leather Boots
(20458,2256,1,1,-1,8461),-- Recipe: Assassin Knife
(20458,57,157,282,0,700000),-- Adena
(20458,846,1,1,1,639),-- Coral Earring
(20458,878,1,1,1,956),-- Blue Coral Ring
(20458,909,1,1,1,480),-- Blue Diamond Necklace
(20458,1898,1,1,1,5250),-- Blue Diamond Necklace Gem
(20458,1804,1,1,2,1169),-- Recipe: Soulshot: D-Grade
(20458,1864,1,1,2,58409),-- Stem
(20458,1869,1,1,2,29205),-- Iron Ore
(20458,1880,1,1,2,2921),-- Steel
(20458,5268,1,1,2,1169),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20458,5437,1,1,2,715),-- Recipe: Heavy Sword(100%)
(20458,6666,1,1,2,29205),-- Deluxe Chest Key - Grade 2
(20460,1792,1,1,-1,17371),-- Recipe: Short Spear
(20460,1833,1,1,-1,629988),-- Bandage
(20460,1866,1,1,-1,157497),-- Suede
(20460,57,119,180,0,700000),-- Adena
(20460,66,1,1,1,172),-- Gladius
(20460,1333,1,1,1,172),-- Brandish
(20460,2010,1,1,1,1811),-- Brandish Blade
(20460,1799,1,1,2,68922),-- Recipe: Leather Gloves
(20460,1867,1,1,2,62030),-- Animal Skin
(20460,1872,1,1,2,62030),-- Animal Bone
(20461,1803,1,1,-1,165438),-- Recipe: Necklace of Wisdom
(20461,1896,1,1,-1,83420),-- Necklace of Anguish Chain
(20461,2010,1,1,-1,7660),-- Brandish Blade
(20461,57,99,150,0,700000),-- Adena
(20461,66,1,1,1,144),-- Gladius
(20461,1333,1,1,1,144),-- Brandish
(20461,2010,1,1,1,1509),-- Brandish Blade
(20461,1803,1,1,2,32579),-- Recipe: Necklace of Wisdom
(20461,1867,1,1,2,51692),-- Animal Skin
(20461,1872,1,1,2,51692),-- Animal Bone
(20462,1800,1,1,-1,65776),-- Recipe: Piece Bone Breastplate
(20462,1865,1,1,-1,209166),-- Varnish
(20462,1872,1,2,-1,278888),-- Animal Bone
(20462,57,102,164,0,700000),-- Adena
(20462,256,1,1,1,182),-- Cestus
(20462,1800,1,1,2,12953),-- Recipe: Piece Bone Breastplate
(20462,1867,1,1,2,54920),-- Animal Skin
(20462,1872,1,1,2,54920),-- Animal Bone
(20463,1799,1,1,-1,273567),-- Recipe: Leather Gloves
(20463,1921,1,1,-1,119134),-- Leather Shoes Texture
(20463,2010,1,1,-1,7185),-- Brandish Blade
(20463,57,97,137,0,700000),-- Adena
(20463,271,1,1,1,403),-- Hunting Bow
(20463,1798,1,1,2,35651),-- Recipe: Leather Helmet
(20463,1864,1,1,2,72727),-- Stem
(20463,1869,1,1,2,36364),-- Iron Ore
(20464,464,1,1,-1,7152),-- Leather Stockings
(20464,1870,1,1,-1,177731),-- Coal
(20464,1874,1,1,-1,11849),-- Oriharukon Ore
(20464,57,95,131,0,700000),-- Adena
(20464,9,1,1,1,130),-- Cedar Staff
(20464,154,1,1,1,130),-- Dwarven Mace
(20464,2008,1,1,1,1363),-- Cedar Staff Head
(20464,1803,1,1,2,29412),-- Recipe: Necklace of Wisdom
(20464,1865,1,1,2,35000),-- Varnish
(20464,1870,1,1,2,35000),-- Coal
(20466,734,1,1,-1,41833),-- Haste Potion
(20466,1868,1,2,-1,501999),-- Thread
(20466,1897,1,1,-1,59762),-- Necklace of Wisdom Chain
(20466,57,122,197,0,700000),-- Adena
(20466,24,1,1,1,366),-- Bone Breastplate
(20466,31,1,1,1,584),-- Bone Gaiters
(20466,390,1,1,1,366),-- Cotton Shirt
(20466,412,1,1,1,584),-- Cotton Pants
(20466,1801,1,1,2,18676),-- Recipe: Piece Bone Gaiters
(20466,1867,1,1,2,49428),-- Animal Skin
(20466,1872,1,1,2,49428),-- Animal Bone
(20466,5158,1,1,2,1483),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20468,1790,1,1,-1,14370),-- Recipe: Dirk
(20468,1814,1,1,-1,40916),-- Recipe: Leather
(20468,2005,1,1,-1,5553),-- Broadsword Blade
(20468,57,41,58,0,700000),-- Adena
(20468,99,1,1,1,735),-- Apprentice\'s Spellbook
(20468,1795,1,1,2,57770),-- Recipe:Leather Shoes
(20468,1864,1,1,2,30618),-- Stem
(20468,1869,1,1,2,15309),-- Iron Ore
(20469,1666,1,1,-1,42787),-- Recipe: Wooden Arrow
(20469,1864,1,1,-1,256721),-- Stem
(20469,2007,1,1,-1,9169),-- Bow Shaft
(20469,57,69,94,0,700000),-- Adena
(20469,14,1,1,1,607),-- Bow
(20469,2007,1,1,1,2709),-- Bow Shaft
(20469,1795,1,1,2,71539),-- Recipe:Leather Shoes
(20469,1833,1,1,2,50554),-- Bandage
(20469,1868,1,1,2,37916),-- Thread
(20469,1873,1,1,2,7584),-- Silver Nugget
(20470,1060,1,1,-1,216259),-- Lesser Healing Potion
(20470,1791,1,1,-1,17988),-- Recipe: Brandish
(20470,2006,1,1,-1,6951),-- Willow Staff Head
(20470,57,51,72,0,700000),-- Adena
(20470,19,1,1,1,9012),-- Small Shield
(20470,42,1,1,1,6311),-- Leather Cap
(20470,735,1,1,2,1198),-- Potion of Alacrity
(20470,1864,1,1,2,28746),-- Stem
(20470,1869,1,1,2,14373),-- Iron Ore
(20470,2135,1,1,2,4228),-- Recipe: Braided Hemp
(20471,428,1,1,-1,14976),-- Feriotic Tunic
(20471,1797,1,1,-1,367589),-- Recipe: Leather Stockings
(20471,1869,1,1,-1,181957),-- Iron Ore
(20471,57,98,132,0,700000),-- Adena
(20471,3,1,1,1,860),-- Broadsword
(20471,2005,1,1,1,3840),-- Broadsword Blade
(20471,1666,1,1,2,11944),-- Recipe: Wooden Arrow
(20471,1864,1,1,2,71663),-- Stem
(20471,1869,1,1,2,35832),-- Iron Ore
(20472,1864,1,2,-1,426554),-- Stem
(20472,1867,1,2,-1,284370),-- Animal Skin
(20472,1922,1,1,-1,54407),-- Leather Tunic Pattern
(20472,57,114,157,0,700000),-- Adena
(20472,66,1,1,1,156),-- Gladius
(20472,1333,1,1,1,156),-- Brandish
(20472,2010,1,1,1,1635),-- Brandish Blade
(20472,736,1,1,2,21000),-- Scroll of Escape
(20472,1864,1,1,2,83999),-- Stem
(20472,1869,1,1,2,42000),-- Iron Ore
(20473,23,1,1,-1,5065),-- Wooden Breastplate
(20473,1831,1,1,-1,537542),-- Antidote
(20473,1896,1,1,-1,85415),-- Necklace of Anguish Chain
(20473,57,109,147,0,700000),-- Adena
(20473,9,1,1,1,147),-- Cedar Staff
(20473,154,1,1,1,147),-- Dwarven Mace
(20473,2008,1,1,1,1545),-- Cedar Staff Head
(20473,1792,1,1,2,2919),-- Recipe: Short Spear
(20473,1864,1,1,2,79391),-- Stem
(20473,1869,1,1,2,39696),-- Iron Ore
(20474,1792,1,1,-1,17030),-- Recipe: Short Spear
(20474,1799,1,1,-1,343131),-- Recipe: Leather Gloves
(20474,1866,1,1,-1,154409),-- Suede
(20474,57,116,177,0,700000),-- Adena
(20474,216,1,1,1,253),-- Dirk
(20474,2009,1,1,1,2427),-- Dirk Blade
(20474,736,1,1,2,11403),-- Scroll of Escape
(20474,1867,1,1,2,30407),-- Animal Skin
(20474,1869,1,1,2,22806),-- Iron Ore
(20474,1870,1,1,2,22806),-- Coal
(20474,1871,1,1,2,22806),-- Charcoal
(20474,1872,1,1,2,30407),-- Animal Bone
(20475,426,1,1,-1,52525),-- Tunic
(20475,1786,1,1,-1,30885),-- Recipe: Broad Sword
(20475,1873,1,1,-1,15442),-- Silver Nugget
(20475,57,19,29,0,700000),-- Adena
(20475,253,1,1,1,10394),-- Spiked Gloves
(20475,1786,1,1,2,10644),-- Recipe: Broad Sword
(20475,1868,1,1,2,26609),-- Thread
(20475,1873,1,1,2,5322),-- Silver Nugget
(20476,945,1,1,-1,13812),-- Skeleton Buckler
(20476,1866,1,1,-1,158377),-- Suede
(20476,5549,1,1,-1,23757),-- Metallic Thread
(20476,57,118,183,0,700000),-- Adena
(20476,2,1,1,1,52),-- Long Sword
(20476,120,1,1,1,52),-- Sword of Reflection
(20476,121,1,1,1,52),-- Sword of Watershadow
(20476,2012,1,1,1,732),-- Sword of Reflection Blade
(20476,1800,1,1,2,11034),-- Recipe: Piece Bone Breastplate
(20476,1868,1,1,2,70174),-- Thread
(20476,1873,1,1,2,14035),-- Silver Nugget
(20476,5273,1,1,2,1404),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20477,906,1,1,-1,14254),-- Necklace of Knowledge
(20477,1060,1,1,-1,171046),-- Lesser Healing Potion
(20477,1872,1,1,-1,102628),-- Animal Bone
(20477,57,41,58,0,700000),-- Adena
(20477,12,1,1,1,364),-- Knife
(20477,215,1,1,1,364),-- Doom Dagger
(20477,1796,1,1,2,14300),-- Recipe: Leather Tunic
(20477,1833,1,1,2,30315),-- Bandage
(20477,1866,1,1,2,7579),-- Suede
(20477,1871,1,1,2,11369),-- Charcoal
(20478,735,1,1,-1,22557),-- Potion of Alacrity
(20478,1801,1,1,-1,136366),-- Recipe: Piece Bone Gaiters
(20478,1897,1,1,-1,64449),-- Necklace of Wisdom Chain
(20478,57,132,212,0,700000),-- Adena
(20478,217,1,1,1,118),-- Shining Knife
(20478,218,1,1,1,118),-- Throwing Knife
(20478,735,1,1,2,2221),-- Potion of Alacrity
(20478,1817,1,1,2,5331),-- Recipe: Cord
(20478,1831,1,1,2,71073),-- Antidote
(20478,1864,1,1,2,53305),-- Stem
(20478,1869,1,1,2,26653),-- Iron Ore
(20478,6037,1,1,2,53305),-- Waking Scroll
(20479,1792,1,1,-1,17199),-- Recipe: Short Spear
(20479,1793,1,1,-1,17199),-- Recipe: Sword of Reflection
(20479,1922,1,1,-1,59670),-- Leather Tunic Pattern
(20479,57,118,179,0,700000),-- Adena
(20479,9,1,1,1,171),-- Cedar Staff
(20479,154,1,1,1,171),-- Dwarven Mace
(20479,2008,1,1,1,1793),-- Cedar Staff Head
(20479,1792,1,1,2,2541),-- Recipe: Short Spear
(20479,1833,1,1,2,92124),-- Bandage
(20479,1867,1,1,2,46062),-- Animal Skin
(20479,1872,1,1,2,46062),-- Animal Bone
(20480,1864,1,3,-1,356778),-- Stem
(20480,2005,1,1,-1,12742),-- Broadsword Blade
(20480,2007,1,1,-1,12742),-- Bow Shaft
(20480,57,96,130,0,700000),-- Adena
(20480,22,1,1,1,4337),-- Leather Shirt
(20480,29,1,1,1,6934),-- Leather Pants
(20480,1789,1,1,2,6494),-- Recipe: Cedar Staff
(20480,1864,1,1,2,70258),-- Stem
(20480,1869,1,1,2,35129),-- Iron Ore
(20481,1786,1,1,-1,12736),-- Recipe: Broad Sword
(20481,1788,1,1,-1,12736),-- Recipe: Bow 
(20481,1868,1,1,-1,31839),-- Thread
(20481,57,8,12,0,700000),-- Adena
(20481,426,1,1,1,11197),-- Tunic
(20481,462,1,1,1,17890),-- Stockings
(20481,17,10,30,2,20574),-- Wooden Arrow
(20481,1786,1,1,2,3292),-- Recipe: Broad Sword
(20481,1868,1,1,2,8230),-- Thread
(20481,1873,1,1,2,1646),-- Silver Nugget
(20483,1833,1,1,-1,654568),-- Bandage
(20483,1868,1,2,-1,490926),-- Thread
(20483,57,120,192,0,700000),-- Adena
(20483,845,1,1,1,816),-- Cat\'s Eye Earring
(20483,877,1,1,1,1225),-- Ring of Wisdom
(20483,908,1,1,1,610),-- Necklace of Wisdom
(20483,1897,1,1,1,8632),-- Necklace of Wisdom Chain
(20483,1864,1,1,2,41433),-- Stem
(20483,1865,1,1,2,20716),-- Varnish
(20483,1866,1,1,2,13811),-- Suede
(20483,1868,1,1,2,41433),-- Thread
(20483,1873,1,1,2,8287),-- Silver Nugget
(20483,2139,1,1,2,4144),-- Recipe: Steel Mold
(20483,5268,1,1,2,829),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20487,1060,1,1,-1,439936),-- Lesser Healing Potion
(20487,1897,1,1,-1,47136),-- Necklace of Wisdom Chain
(20487,2136,1,1,-1,58227),-- Recipe: Cokes
(20487,57,98,153,0,700000),-- Adena
(20487,217,1,1,1,86),-- Shining Knife
(20487,218,1,1,1,86),-- Throwing Knife
(20487,1800,1,1,2,12260),-- Recipe: Piece Bone Breastplate
(20487,1865,1,1,2,38986),-- Varnish
(20487,1870,1,1,2,38986),-- Coal
(20488,412,1,1,-1,3229),-- Cotton Pants
(20488,734,1,1,-1,34178),-- Haste Potion
(20488,1800,1,1,-1,64486),-- Recipe: Piece Bone Breastplate
(20488,57,100,161,0,700000),-- Adena
(20488,256,1,1,1,178),-- Cestus
(20488,1864,1,1,2,80765),-- Stem
(20488,1869,1,1,2,40383),-- Iron Ore
(20488,2140,1,1,2,8077),-- Recipe: High Grade Suede
(20489,1060,1,1,-1,433160),-- Lesser Healing Potion
(20489,1872,1,2,-1,259896),-- Animal Bone
(20489,1896,1,1,-1,82594),-- Necklace of Anguish Chain
(20489,57,98,149,0,700000),-- Adena
(20489,429,1,1,1,724),-- Leather Tunic
(20489,464,1,1,1,1159),-- Leather Stockings
(20489,1922,1,1,1,7345),-- Leather Tunic Pattern
(20489,1923,1,1,1,11799),-- Leather Stocking Pattern
(20489,1793,1,1,2,2823),-- Recipe: Sword of Reflection
(20489,1868,1,1,2,76770),-- Thread
(20489,1873,1,1,2,15354),-- Silver Nugget
(20491,876,1,1,-1,16827),-- Ring of Anguish
(20491,1794,1,1,-1,14476),-- Recipe: Forest Bow
(20491,1803,1,1,-1,165438),-- Recipe: Necklace of Wisdom
(20491,57,99,150,0,700000),-- Adena
(20491,37,1,1,1,2926),-- Leather Shoes
(20491,49,1,1,1,2926),-- Gloves
(20491,1921,1,1,1,25013),-- Leather Shoes Texture
(20491,1794,1,1,2,2851),-- Recipe: Forest Bow
(20491,1866,1,1,2,25846),-- Suede
(20491,1871,1,1,2,38769),-- Charcoal
(20492,25,1,1,-1,1492),-- Piece Bone Breastplate
(20492,1865,1,2,-1,237307),-- Varnish
(20492,1879,1,1,-1,39551),-- Cokes
(20492,57,108,193,0,700000),-- Adena
(20492,27,1,1,1,190),-- Hard Leather Shirt
(20492,33,1,1,1,305),-- Hard Leather Gaiters
(20492,1927,1,1,1,2796),-- Hard Leather Shirt Pattern
(20492,1929,1,1,1,4523),-- Hard Leather Gaiters Material
(20492,1865,1,1,2,28039),-- Varnish
(20492,1870,1,1,2,28039),-- Coal
(20492,1879,1,1,2,4674),-- Cokes
(20492,5278,1,1,2,1122),-- Recipe: Greater Blessed Spiritshot (D) Compressed Package(100%)
(20492,5437,1,1,2,686),-- Recipe: Heavy Sword(100%)
(20494,1804,1,1,-1,24762),-- Recipe: Soulshot: D-Grade
(20494,1881,1,1,-1,82540),-- Coarse Bone Powder
(20494,2179,1,1,-1,296197),-- Recipe: Crafted Leather Gloves
(20494,57,275,507,0,700000),-- Adena
(20494,257,1,1,1,300),-- Viper\'s Fang
(20494,735,1,1,2,6096),-- Potion of Alacrity
(20494,1866,1,1,2,48763),-- Suede
(20494,1871,1,1,2,73144),-- Charcoal
(20494,2258,1,1,2,1789),-- Recipe: Temptation of Abyss
(20494,5268,1,1,2,2926),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20495,1900,1,1,-1,102868),-- Enchanted Necklace Chain
(20495,1936,1,1,-1,76940),-- White Tunic Pattern
(20495,2267,1,1,-1,9723),-- Recipe: Gastraphetes
(20495,57,476,952,0,700000),-- Adena
(20495,69,1,1,1,32),-- Bastard Sword
(20495,125,1,1,1,32),-- Spinebone Sword
(20495,126,1,1,1,32),-- Artisan\'s Sword
(20495,128,1,1,1,32),-- Knight\'s Sword
(20495,2021,1,1,1,1298),-- Spinebone Sword Blade
(20495,955,1,1,2,1044),-- Scroll: Enchant Weapon (Grade D)
(20495,1875,1,1,2,5208),-- Stone of Purity
(20495,1879,1,1,2,11111),-- Cokes
(20495,1885,1,1,2,4525),-- High Grade Suede
(20495,2274,1,1,2,408),-- Recipe: Skull Breaker
(20495,2975,1,1,2,10101),-- Recipe: Brigandine Helmet
(20495,5549,1,1,2,6944),-- Metallic Thread
(20495,5550,1,1,2,1044),-- Durable Metal Plate
(20495,6667,1,1,2,43478),-- Deluxe Chest Key - Grade 3
(20496,465,1,1,-1,3807),-- Cursed Stockings
(20496,1885,1,1,-1,62016),-- High Grade Suede
(20496,2152,1,1,-1,131020),-- Recipe: Enchanted Necklace
(20496,57,389,743,0,700000),-- Adena
(20496,274,1,1,1,204),-- Strengthened Bow
(20496,1876,1,1,2,19231),-- Mithril Ore
(20496,1877,1,1,2,6410),-- Adamantite Nugget
(20496,1880,1,1,2,1282),-- Steel
(20496,1881,1,1,2,7692),-- Coarse Bone Powder
(20496,1882,1,1,2,11494),-- Leather
(20496,2260,1,1,2,3846),-- Recipe: Mace of Judgment
(20496,6666,1,1,2,52632),-- Deluxe Chest Key - Grade 2
(20497,1899,1,2,-1,119855),-- Necklace of Devotion Chain
(20497,2018,1,1,-1,10194),-- Assassin Knife Edge
(20497,2259,1,1,-1,11911),-- Recipe: Spinebone Sword
(20497,57,396,772,0,700000),-- Adena
(20497,391,1,1,1,688),-- Puma Skin Shirt
(20497,413,1,1,1,1102),-- Puma Skin Gaiters
(20497,1874,1,1,2,7299),-- Oriharukon Ore
(20497,1878,1,1,2,34483),-- Braided Hemp
(20497,1884,1,1,2,58824),-- Cord
(20497,1889,1,1,2,6098),-- Compound Braid
(20497,2261,1,1,2,1828),-- Recipe: Conjuror\'s Staff
(20497,5220,1,1,2,1220),-- Metal Hardener
(20498,1885,1,1,-1,59620),-- High Grade Suede
(20498,2262,1,1,-1,11109),-- Recipe: Elven Bow
(20498,3953,1,1,-1,28617),-- Recipe: Blessed Spiritshot D
(20498,57,371,713,0,700000),-- Adena
(20498,45,1,1,1,856),-- Bone Helmet
(20498,626,1,1,1,1223),-- Bronze Shield
(20498,1932,1,1,1,21245),-- Bone Helmet Design
(20498,1875,1,1,2,3937),-- Stone of Purity
(20498,1879,1,1,2,13333),-- Cokes
(20498,1885,1,1,2,4739),-- High Grade Suede
(20498,2261,1,1,2,1577),-- Recipe: Conjuror\'s Staff
(20498,5549,1,1,2,5525),-- Metallic Thread
(20498,5550,1,1,2,789),-- Durable Metal Plate
(20499,1870,1,2,-1,685215),-- Coal
(20499,1880,1,1,-1,68521),-- Steel
(20499,2017,1,1,-1,9106),-- Saber Blade
(20499,57,358,682,0,700000),-- Adena
(20499,40,1,1,1,921),-- Leather Boots
(20499,604,1,1,1,921),-- Crafted Leather Gloves
(20499,1931,1,1,1,12568),-- Leather Boot Lining
(20499,1934,1,1,1,12568),-- Crafted Leather Gloves Lining
(20499,1876,1,1,2,9174),-- Mithril Ore
(20499,1877,1,1,2,3279),-- Adamantite Nugget
(20499,1880,1,1,2,6536),-- Steel
(20499,1881,1,1,2,11765),-- Coarse Bone Powder
(20499,1882,1,1,2,16393),-- Leather
(20499,2182,1,1,2,5882),-- Recipe: Scale Gaiters
(20500,1867,1,3,-1,867354),-- Animal Skin
(20500,1931,1,1,-1,85035),-- Leather Boot Lining
(20500,1932,1,1,-1,103256),-- Bone Helmet Design
(20500,57,344,643,0,700000),-- Adena
(20500,847,1,1,1,679),-- Red Crescent Earring
(20500,890,1,1,1,1015),-- Ring of Devotion
(20500,910,1,1,1,509),-- Necklace of Devotion
(20500,1899,1,1,1,14262),-- Necklace of Devotion Chain
(20500,1874,1,1,2,5025),-- Oriharukon Ore
(20500,1878,1,1,2,41667),-- Braided Hemp
(20500,2262,1,1,2,4016),-- Recipe: Elven Bow
(20500,5220,1,1,2,5025),-- Metal Hardener
(20501,1870,1,2,-1,787074),-- Coal
(20501,1933,1,1,-1,92597),-- Dark Stocking Fabric
(20501,2153,1,1,-1,122407),-- Recipe: Tiger\'s Eye Earring
(20501,57,400,793,0,700000),-- Adena
(20501,433,1,1,1,108),-- Elven Tunic
(20501,434,1,1,1,108),-- White Tunic
(20501,435,1,1,1,108),-- Mystic\'s Tunic
(20501,466,1,1,1,173),-- Elven Stockings
(20501,467,1,1,1,173),-- Dark Stockings
(20501,468,1,1,1,173),-- Mystic\'s Stockings
(20501,1933,1,1,1,6326),-- Dark Stocking Fabric
(20501,1936,1,1,1,4400),-- White Tunic Pattern
(20501,1875,1,1,2,5917),-- Stone of Purity
(20501,1879,1,1,2,15385),-- Cokes
(20501,1885,1,1,2,5917),-- High Grade Suede
(20501,2264,1,1,2,538),-- Recipe: Two-handed Sword
(20501,2274,1,1,2,806),-- Recipe: Skull Breaker
(20501,5549,1,1,2,6452),-- Metallic Thread
(20501,5550,1,1,2,1075),-- Durable Metal Plate
(20502,1871,1,2,-1,233906),-- Charcoal
(20502,1896,1,1,-1,99113),-- Necklace of Anguish Chain
(20502,1923,1,1,-1,95863),-- Leather Stocking Pattern
(20502,57,118,179,0,700000),-- Adena
(20502,23,1,1,1,1158),-- Wooden Breastplate
(20502,30,1,1,1,1854),-- Hard Leather Pants
(20502,2386,1,1,1,1854),-- Wooden Gaiters
(20502,1798,1,1,2,45159),-- Recipe: Leather Helmet
(20502,1868,1,1,2,92123),-- Thread
(20502,1873,1,1,2,18425),-- Silver Nugget
(20504,1792,1,1,-1,13930),-- Recipe: Short Spear
(20504,1921,1,1,-1,122225),-- Leather Shoes Texture
(20504,2010,1,1,-1,7372),-- Brandish Blade
(20504,57,97,143,0,700000),-- Adena
(20504,20,1,1,1,4026),-- Buckler
(20504,43,1,1,1,2813),-- Wooden Helmet
(20504,1792,1,1,2,2744),-- Recipe: Short Spear
(20504,1865,1,1,2,37307),-- Varnish
(20504,1870,1,1,2,37307),-- Coal
(20505,734,1,1,-1,45036),-- Haste Potion
(20505,1800,1,1,-1,84973),-- Recipe: Piece Bone Breastplate
(20505,1869,1,1,-1,270215),-- Iron Ore
(20505,57,125,217,0,700000),-- Adena
(20505,217,1,1,1,118),-- Shining Knife
(20505,218,1,1,1,118),-- Throwing Knife
(20505,1817,1,1,2,6386),-- Recipe: Cord
(20505,1864,1,1,2,63854),-- Stem
(20505,1869,1,1,2,31927),-- Iron Ore
(20505,1878,1,1,2,12771),-- Braided Hemp
(20505,5273,1,1,2,1278),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20506,1864,1,2,-1,492156),-- Stem
(20506,1924,1,1,-1,79380),-- Leather Helmet Design
(20506,2150,1,1,-1,115530),-- Recipe: Blue Diamond Necklace
(20506,57,120,193,0,700000),-- Adena
(20506,24,1,1,1,359),-- Bone Breastplate
(20506,31,1,1,1,573),-- Bone Gaiters
(20506,390,1,1,1,359),-- Cotton Shirt
(20506,412,1,1,1,573),-- Cotton Pants
(20506,1864,1,1,2,72688),-- Stem
(20506,1869,1,1,2,36344),-- Iron Ore
(20506,2141,1,1,2,7269),-- Recipe: Silver Mold
(20506,5273,1,1,2,1454),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20507,1866,1,1,-1,193642),-- Suede
(20507,1898,1,1,-1,29837),-- Blue Diamond Necklace Gem
(20507,3032,1,1,-1,11619),-- Recipe: Spiritshot D
(20507,57,132,236,0,700000),-- Adena
(20507,273,1,1,1,71),-- Composition Bow
(20507,2016,1,1,1,1006),-- Composition Bow Shaft
(20507,1804,1,1,2,1373),-- Recipe: Soulshot: D-Grade
(20507,1864,1,1,2,68639),-- Stem
(20507,1869,1,1,2,34320),-- Iron Ore
(20507,1878,1,1,2,13728),-- Braided Hemp
(20507,5273,1,1,2,1373),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20508,1800,1,1,-1,78931),-- Recipe: Piece Bone Breastplate
(20508,1866,1,2,-1,167333),-- Suede
(20508,57,122,197,0,700000),-- Adena
(20508,1101,1,1,1,731),-- Tunic of Devotion
(20508,1104,1,1,1,1168),-- Stockings of Devotion
(20508,1867,1,1,2,65904),-- Animal Skin
(20508,1872,1,1,2,65904),-- Animal Bone
(20508,2142,1,1,2,9886),-- Recipe: Varnish of Purity
(20509,1060,1,1,-1,330759),-- Lesser Healing Potion
(20509,1129,1,1,-1,49042),-- Crude Leather Shoes
(20509,2005,1,1,-1,10632),-- Broadsword Blade
(20509,57,80,108,0,700000),-- Adena
(20509,428,1,1,1,1810),-- Feriotic Tunic
(20509,463,1,1,1,2893),-- Feriotic Stockings
(20509,1100,1,1,1,1810),-- Cotton Tunic
(20509,1103,1,1,1,2893),-- Cotton Stockings
(20509,17,10,30,2,109915),-- Wooden Arrow
(20509,1864,1,1,2,43966),-- Stem
(20509,1869,1,1,2,21983),-- Iron Ore
(20509,2136,1,1,2,6466),-- Recipe: Cokes
(20510,1792,1,1,-1,15222),-- Recipe: Short Spear
(20510,1798,1,1,-1,202966),-- Recipe: Leather Helmet
(20510,1923,1,1,-1,84846),-- Leather Stocking Pattern
(20510,57,110,152,0,700000),-- Adena
(20510,23,1,1,1,1025),-- Wooden Breastplate
(20510,30,1,1,1,1641),-- Hard Leather Pants
(20510,2386,1,1,1,1641),-- Wooden Gaiters
(20510,1792,1,1,2,2998),-- Recipe: Short Spear
(20510,1868,1,1,2,81537),-- Thread
(20510,1873,1,1,2,16308),-- Silver Nugget
(20511,734,1,1,-1,40780),-- Haste Potion
(20511,1897,1,1,-1,58257),-- Necklace of Wisdom Chain
(20511,2144,1,1,-1,48936),-- Recipe: Compound Braid
(20511,57,115,195,0,700000),-- Adena
(20511,176,1,1,1,107),-- Journeyman\'s Staff
(20511,2501,1,1,1,107),-- Bone Club
(20511,735,1,1,2,1721),-- Potion of Alacrity
(20511,1800,1,1,2,6494),-- Recipe: Piece Bone Breastplate
(20511,1867,1,1,2,27534),-- Animal Skin
(20511,1869,1,1,2,20650),-- Iron Ore
(20511,1870,1,1,2,20650),-- Coal
(20511,1871,1,1,2,20650),-- Charcoal
(20511,1872,1,1,2,27534),-- Animal Bone
(20514,1870,1,2,-1,231613),-- Coal
(20514,1872,1,3,-1,308818),-- Animal Bone
(20514,57,116,177,0,700000),-- Adena
(20514,20,1,1,1,4923),-- Buckler
(20514,43,1,1,1,3438),-- Wooden Helmet
(20514,734,1,1,2,5702),-- Haste Potion
(20514,1799,1,1,2,50678),-- Recipe: Leather Gloves
(20514,1865,1,1,2,34208),-- Varnish
(20514,1870,1,1,2,34208),-- Coal
(20515,735,1,1,-1,19797),-- Potion of Alacrity
(20515,1865,1,2,-1,237565),-- Varnish
(20515,2013,1,1,-1,5127),-- Forest Bow Shaft
(20515,57,118,183,0,700000),-- Adena
(20515,15,1,1,1,104),-- Short Spear
(20515,2011,1,1,1,1601),-- Short Spear Edge
(20515,1801,1,1,2,17676),-- Recipe: Piece Bone Gaiters
(20515,1865,1,1,2,35087),-- Varnish
(20515,1870,1,1,2,35087),-- Coal
(20515,5158,1,1,2,1404),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20517,1831,1,1,-1,473949),-- Antidote
(20517,1869,1,1,-1,177731),-- Iron Ore
(20517,1921,1,1,-1,114665),-- Leather Shoes Texture
(20517,57,95,131,0,700000),-- Adena
(20517,271,1,1,1,388),-- Hunting Bow
(20517,1793,1,1,2,1931),-- Recipe: Sword of Reflection
(20517,1833,1,1,2,69999),-- Bandage
(20517,1864,1,1,2,52499),-- Stem
(20517,1869,1,1,2,26250),-- Iron Ore
(20518,1831,1,1,-1,492420),-- Antidote
(20518,1864,1,1,-1,369315),-- Stem
(20518,1866,1,1,-1,123105),-- Suede
(20518,57,97,137,0,700000),-- Adena
(20518,271,1,1,1,403),-- Hunting Bow
(20518,1799,1,1,2,53872),-- Recipe: Leather Gloves
(20518,1865,1,1,2,36364),-- Varnish
(20518,1870,1,1,2,36364),-- Coal
(20521,1896,1,1,-1,106171),-- Necklace of Anguish Chain
(20521,1897,1,1,-1,59658),-- Necklace of Wisdom Chain
(20521,2143,1,1,-1,50113),-- Recipe: Synthetic Cokes
(20521,57,128,189,0,700000),-- Adena
(20521,9,1,1,1,183),-- Cedar Staff
(20521,154,1,1,1,183),-- Dwarven Mace
(20521,2008,1,1,1,1921),-- Cedar Staff Head
(20521,734,1,1,2,6168),-- Haste Potion
(20521,1793,1,1,2,2722),-- Recipe: Sword of Reflection
(20521,1866,1,1,2,24671),-- Suede
(20521,1871,1,1,2,37007),-- Charcoal
(20525,1867,1,1,-1,51475),-- Animal Skin
(20525,1872,1,1,-1,51475),-- Animal Bone
(20525,5549,1,1,-1,3861),-- Metallic Thread
(20525,57,19,29,0,700000),-- Adena
(20525,1,1,1,1,10394),-- Short Sword
(20525,1060,1,1,2,22174),-- Lesser Healing Potion
(20525,1787,1,1,2,7983),-- Recipe: Willow Staff
(20525,1868,1,1,2,19957),-- Thread
(20525,1873,1,1,2,3992),-- Silver Nugget
(20526,1833,1,1,-1,694345),-- Bandage
(20526,1921,1,1,-1,167987),-- Leather Shoes Texture
(20526,5220,1,1,-1,10415),-- Metal Hardener
(20526,57,131,199,0,700000),-- Adena
(20526,114,1,1,1,1753),-- Earring of Strength
(20526,115,1,1,1,1753),-- Earring of Wisdom
(20526,876,1,1,1,2630),-- Ring of Anguish
(20526,907,1,1,1,1315),-- Necklace of Anguish
(20526,1896,1,1,1,13036),-- Necklace of Anguish Chain
(20526,1803,1,1,2,43088),-- Recipe: Necklace of Wisdom
(20526,1866,1,1,2,34184),-- Suede
(20526,1871,1,1,2,51275),-- Charcoal
(20528,1833,1,1,-1,568013),-- Bandage
(20528,1922,1,1,-1,54338),-- Leather Tunic Pattern
(20528,57,112,158,0,700000),-- Adena
(20528,216,1,1,1,233),-- Dirk
(20528,2009,1,1,1,2232),-- Dirk Blade
(20528,1803,1,1,2,35249),-- Recipe: Necklace of Wisdom
(20528,1865,1,1,2,41946),-- Varnish
(20528,1870,1,1,2,41946),-- Coal
(20529,1121,1,1,-1,454845),-- Apprentice\'s Shoes
(20529,1786,1,1,-1,12736),-- Recipe: Broad Sword
(20529,1867,1,1,-1,21226),-- Animal Skin
(20529,57,8,12,0,700000),-- Adena
(20529,35,1,1,1,22242),-- Cloth Shoes
(20529,36,1,1,1,22242),-- Leather Sandals
(20529,48,1,1,1,22241),-- Short Gloves
(20529,1121,1,1,1,117560),-- Apprentice\'s Shoes
(20529,1788,1,1,2,4389),-- Recipe: Bow 
(20529,1868,1,1,2,10973),-- Thread
(20529,1873,1,1,2,2195),-- Silver Nugget
(20530,1787,1,1,-1,12736),-- Recipe: Willow Staff
(20530,1867,1,1,-1,21226),-- Animal Skin
(20530,1868,1,1,-1,31839),-- Thread
(20530,57,8,12,0,700000),-- Adena
(20530,426,1,1,1,11197),-- Tunic
(20530,462,1,1,1,17890),-- Stockings
(20530,1060,1,1,2,9144),-- Lesser Healing Potion
(20530,1786,1,1,2,3292),-- Recipe: Broad Sword
(20530,1868,1,1,2,8230),-- Thread
(20530,1873,1,1,2,1646),-- Silver Nugget
(20531,1788,1,1,-1,12736),-- Recipe: Bow 
(20531,1872,1,1,-1,21226),-- Animal Bone
(20531,1873,1,1,-1,6368),-- Silver Nugget
(20531,57,8,12,0,700000),-- Adena
(20531,426,1,1,1,11197),-- Tunic
(20531,462,1,1,1,17890),-- Stockings
(20531,1787,1,1,2,4389),-- Recipe: Willow Staff
(20531,1868,1,1,2,10973),-- Thread
(20531,1873,1,1,2,2195),-- Silver Nugget
(20532,35,1,1,-1,96841),-- Cloth Shoes
(20532,1786,1,1,-1,14332),-- Recipe: Broad Sword
(20532,1787,1,1,-1,14332),-- Recipe: Willow Staff
(20532,57,8,13,0,700000),-- Adena
(20532,18,1,1,1,47493),-- Leather Shield
(20532,41,1,1,1,33677),-- Cloth Cap
(20532,1786,1,1,2,4940),-- Recipe: Broad Sword
(20532,1868,1,1,2,12349),-- Thread
(20532,1873,1,1,2,2470),-- Silver Nugget
(20533,1872,1,1,-1,23887),-- Animal Bone
(20533,5549,1,1,-1,1792),-- Metallic Thread
(20533,57,8,13,0,700000),-- Adena
(20533,13,1,1,1,4823),-- Short Bow
(20533,1060,1,1,2,10291),-- Lesser Healing Potion
(20533,1787,1,1,2,3705),-- Recipe: Willow Staff
(20533,1868,1,1,2,9262),-- Thread
(20533,1873,1,1,2,1853),-- Silver Nugget
(20534,1786,1,1,-1,14332),-- Recipe: Broad Sword
(20534,1868,1,1,-1,35831),-- Thread
(20534,57,8,13,0,700000),-- Adena
(20534,21,1,1,1,12601),-- Shirt
(20534,28,1,1,1,20133),-- Pants
(20534,1788,1,1,2,4940),-- Recipe: Bow 
(20534,1868,1,1,2,12349),-- Thread
(20534,1873,1,1,2,2470),-- Silver Nugget
(20535,36,1,1,-1,96841),-- Leather Sandals
(20535,1787,1,1,-1,14332),-- Recipe: Willow Staff
(20535,1867,1,1,-1,23887),-- Animal Skin
(20535,57,8,13,0,700000),-- Adena
(20535,21,1,1,1,12601),-- Shirt
(20535,28,1,1,1,20133),-- Pants
(20535,17,10,30,2,23153),-- Wooden Arrow
(20535,1786,1,1,2,3705),-- Recipe: Broad Sword
(20535,1865,1,1,2,4631),-- Varnish
(20535,1870,1,1,2,4631),-- Coal
(20536,18,1,1,-1,139007),-- Leather Shield
(20536,1788,1,1,-1,21685),-- Recipe: Bow 
(20536,1871,1,1,-1,27106),-- Charcoal
(20536,57,13,20,0,700000),-- Adena
(20536,13,1,1,1,7298),-- Short Bow
(20536,1787,1,1,2,7474),-- Recipe: Willow Staff
(20536,1864,1,1,2,18683),-- Stem
(20536,1869,1,1,2,9342),-- Iron Ore
(20537,1788,1,1,-1,21685),-- Recipe: Bow 
(20537,1864,1,1,-1,54213),-- Stem
(20537,1867,1,1,-1,36142),-- Animal Skin
(20537,57,13,20,0,700000),-- Adena
(20537,308,1,1,1,7298),-- Buffalo\'s Horn
(20537,1060,1,1,2,15569),-- Lesser Healing Potion
(20537,1788,1,1,2,5605),-- Recipe: Bow 
(20537,1864,1,1,2,14012),-- Stem
(20537,1869,1,1,2,7006),-- Iron Ore
(20538,1786,1,1,-1,21685),-- Recipe: Broad Sword
(20538,1865,1,1,-1,27106),-- Varnish
(20538,1869,1,1,-1,27106),-- Iron Ore
(20538,57,13,20,0,700000),-- Adena
(20538,1,1,1,1,7298),-- Short Sword
(20538,1786,1,1,2,7474),-- Recipe: Broad Sword
(20538,1864,1,1,2,18683),-- Stem
(20538,1869,1,1,2,9342),-- Iron Ore
(20539,41,1,1,-1,98569),-- Cloth Cap
(20539,1786,1,1,-1,21685),-- Recipe: Broad Sword
(20539,1787,1,1,-1,21685),-- Recipe: Willow Staff
(20539,57,13,20,0,700000),-- Adena
(20539,253,1,1,1,7298),-- Spiked Gloves
(20539,17,10,30,2,35031),-- Wooden Arrow
(20539,1787,1,1,2,5605),-- Recipe: Willow Staff
(20539,1864,1,1,2,14013),-- Stem
(20539,1869,1,1,2,7007),-- Iron Ore
(20540,1831,1,1,-1,460056),-- Antidote
(20540,1873,1,1,-1,69008),-- Silver Nugget
(20540,1896,1,1,-1,73102),-- Necklace of Anguish Chain
(20540,57,92,127,0,700000),-- Adena
(20540,23,1,1,1,854),-- Wooden Breastplate
(20540,30,1,1,1,1368),-- Hard Leather Pants
(20540,2386,1,1,1,1368),-- Wooden Gaiters
(20540,736,1,1,2,16987),-- Scroll of Escape
(20540,1868,1,1,2,67947),-- Thread
(20540,1873,1,1,2,13590),-- Silver Nugget
(20541,1897,1,1,-1,48342),-- Necklace of Wisdom Chain
(20541,1925,1,1,-1,133138),-- Leather Gloves Lining
(20541,2173,1,1,-1,55023),-- Recipe: Hard Leather Shirt
(20541,57,99,159,0,700000),-- Adena
(20541,272,1,1,1,89),-- Forest Bow
(20541,2013,1,1,1,1295),-- Forest Bow Shaft
(20541,734,1,1,2,2499),-- Haste Potion
(20541,1864,1,1,2,29987),-- Stem
(20541,1865,1,1,2,14994),-- Varnish
(20541,1866,1,1,2,9996),-- Suede
(20541,1868,1,1,2,29987),-- Thread
(20541,1873,1,1,2,5998),-- Silver Nugget
(20541,2143,1,1,2,2999),-- Recipe: Synthetic Cokes
(20541,5158,1,1,2,600),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20542,734,1,1,-1,37733),-- Haste Potion
(20542,1869,1,1,-1,226397),-- Iron Ore
(20542,1878,1,1,-1,90559),-- Braided Hemp
(20542,57,106,181,0,700000),-- Adena
(20542,272,1,1,1,99),-- Forest Bow
(20542,2013,1,1,1,1444),-- Forest Bow Shaft
(20542,1801,1,1,2,13476),-- Recipe: Piece Bone Gaiters
(20542,1864,1,1,2,53500),-- Stem
(20542,1869,1,1,2,26750),-- Iron Ore
(20542,1878,1,1,2,10700),-- Braided Hemp
(20542,5158,1,1,2,1070),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20543,908,1,1,-1,3822),-- Necklace of Wisdom
(20543,1882,1,1,-1,50535),-- Leather
(20543,2252,1,1,-1,9320),-- Recipe: Iron Hammer
(20543,57,105,183,0,700000),-- Adena
(20543,309,1,1,1,99),-- Tears of Eva
(20543,310,1,1,1,99),-- Relic of The Saints
(20543,1867,1,1,2,35826),-- Animal Skin
(20543,1872,1,1,2,35826),-- Animal Bone
(20543,1882,1,1,2,5971),-- Leather
(20543,2139,1,1,2,5374),-- Recipe: Steel Mold
(20543,6037,1,1,2,53739),-- Waking Scroll
(20544,1788,1,1,-1,21685),-- Recipe: Bow 
(20544,1870,1,1,-1,27106),-- Coal
(20544,1872,1,1,-1,36142),-- Animal Bone
(20544,57,13,20,0,700000),-- Adena
(20544,21,1,1,1,19065),-- Shirt
(20544,28,1,1,1,30461),-- Pants
(20544,1788,1,1,2,7474),-- Recipe: Bow 
(20544,1864,1,1,2,18683),-- Stem
(20544,1869,1,1,2,9342),-- Iron Ore
(20545,21,1,1,-1,21659),-- Shirt
(20545,1786,1,1,-1,12736),-- Recipe: Broad Sword
(20545,1873,1,1,-1,6368),-- Silver Nugget
(20545,57,8,12,0,700000),-- Adena
(20545,18,1,1,1,42201),-- Leather Shield
(20545,41,1,1,1,29925),-- Cloth Cap
(20545,17,10,30,2,16459),-- Wooden Arrow
(20545,1788,1,1,2,2634),-- Recipe: Bow 
(20545,1868,1,1,2,6584),-- Thread
(20545,1873,1,1,2,1317),-- Silver Nugget
(20545,6037,1,1,2,6584),-- Waking Scroll
(20546,1877,1,1,-1,22714),-- Adamantite Nugget
(20546,2279,1,1,-1,4056),-- Recipe: War Pick
(20546,2979,1,1,-1,76017),-- Recipe: Manticore Skin Gloves
(20546,57,288,574,0,700000),-- Adena
(20546,849,1,1,1,235),-- Tiger\'s Eye Earring
(20546,880,1,1,1,353),-- Black Pearl Ring
(20546,912,1,1,1,177),-- Near Forest Necklace
(20546,1901,1,1,1,10426),-- Tiger\'s Eye Earring Stone
(20546,1876,1,1,2,10989),-- Mithril Ore
(20546,1877,1,1,2,500),-- Adamantite Nugget
(20546,1880,1,1,2,4000),-- Steel
(20546,1881,1,1,2,3003),-- Coarse Bone Powder
(20546,1882,1,1,2,10989),-- Leather
(20546,2274,1,1,2,270),-- Recipe: Skull Breaker
(20546,2276,1,1,2,270),-- Recipe: Maingauche
(20546,2976,1,1,2,6494),-- Recipe: Manticore Skin Boots
(20547,1804,1,1,-1,12605),-- Recipe: Soulshot: D-Grade
(20547,1898,1,1,-1,32370),-- Blue Diamond Necklace Gem
(20547,2151,1,1,-1,87777),-- Recipe: Necklace of Devotion
(20547,57,144,255,0,700000),-- Adena
(20547,39,1,1,1,1009),-- Boots
(20547,51,1,1,1,1009),-- Bracer
(20547,1930,1,1,1,15912),-- Boot Lining
(20547,734,1,1,2,6206),-- Haste Potion
(20547,1865,1,1,2,37233),-- Varnish
(20547,1870,1,1,2,37233),-- Coal
(20547,5163,1,1,2,1490),-- Recipe: Blessed Spiritshot (D) Compressed Package (100%)
(20547,5268,1,1,2,1490),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20548,1804,1,1,-1,12857),-- Recipe: Soulshot: D-Grade
(20548,1865,1,3,-1,321422),-- Varnish
(20548,1930,1,1,-1,82416),-- Boot Lining
(20548,57,147,260,0,700000),-- Adena
(20548,625,1,1,1,1472),-- Bone Shield
(20548,1148,1,1,1,1033),-- Hard Leather Helmet
(20548,1864,1,1,2,94944),-- Stem
(20548,1869,1,1,2,47472),-- Iron Ore
(20548,1880,1,1,2,4748),-- Steel
(20548,5268,1,1,2,1899),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20549,1880,1,1,-1,116806),-- Steel
(20549,1901,1,1,-1,161334),-- Tiger\'s Eye Earring Stone
(20549,1938,1,1,-1,78525),-- Mithril Banded Mail Material
(20549,57,596,1178,0,700000),-- Adena
(20549,83,1,1,1,29),-- Sword of Magic
(20549,124,1,1,1,22),-- Two-Handed Sword
(20549,127,1,1,1,22),-- Crimson Sword
(20549,130,1,1,1,22),-- Elven Sword
(20549,144,1,1,1,29),-- Sword of Occult
(20549,2026,1,1,1,1246),-- Two-Handed Sword Edge
(20549,1341,20,60,2,200000),-- Bone Arrow
(20549,1866,1,1,2,90909),-- Suede
(20549,1871,1,1,2,142857),-- Charcoal
(20549,1877,1,1,2,3623),-- Adamantite Nugget
(20549,2277,1,1,2,740),-- Recipe: Bich\'hwa
(20549,6667,1,1,2,66667),-- Deluxe Chest Key - Grade 3
(20550,394,1,1,-1,850),-- Reinforced Leather Shirt
(20550,2154,1,1,-1,69153),-- Recipe: Elven Earring
(20550,2276,1,1,-1,4614),-- Recipe: Maingauche
(20550,57,327,655,0,700000),-- Adena
(20550,393,1,1,1,76),-- Mithril Banded Mail
(20550,394,1,1,1,76),-- Reinforced Leather Shirt
(20550,415,1,1,1,121),-- Mithril Banded Gaiters
(20550,416,1,1,1,121),-- Reinforced Leather Gaiters
(20550,1938,1,1,1,3849),-- Mithril Banded Mail Material
(20550,1939,1,1,1,5028),-- Mithril Banded Gaiters Material
(20550,1868,1,1,2,111111),-- Thread
(20550,1873,1,1,2,22222),-- Silver Nugget
(20550,1889,1,1,2,4238),-- Compound Braid
(20550,2148,1,1,2,4132),-- Recipe: Crafted Leather
(20550,2272,1,1,2,106),-- Recipe: Sword of Revolution
(20550,2274,1,1,2,318),-- Recipe: Skull Breaker
(20550,5549,1,1,2,5525),-- Metallic Thread
(20551,1874,1,1,-1,43592),-- Oriharukon Ore
(20551,1902,1,1,-1,66249),-- Elven Earring Beads
(20551,2034,1,1,-1,6909),-- Sword of Revolution Blade
(20551,57,332,662,0,700000),-- Adena
(20551,850,1,1,1,121),-- Elven Earring
(20551,881,1,1,1,181),-- Elven Ring
(20551,913,1,1,1,91),-- Elven Necklace
(20551,1902,1,1,1,5708),-- Elven Earring Beads
(20551,1903,1,1,1,9398),-- Elven Ring Piece
(20551,1904,1,1,1,5258),-- Elven Necklace Beads
(20551,1867,1,1,2,71429),-- Animal Skin
(20551,1872,1,1,2,76923),-- Animal Bone
(20551,1875,1,1,2,3676),-- Stone of Purity
(20551,1881,1,1,2,6711),-- Coarse Bone Powder
(20551,2145,1,1,2,3236),-- Recipe: Mithril Alloy
(20551,2287,1,1,2,335),-- Recipe: Atuba Hammer
(20551,2288,1,1,2,223),-- Recipe: Ghost Staff
(20552,1825,1,1,-1,100458),-- Recipe: Oriharukon
(20552,1902,1,1,-1,142493),-- Elven Earring Beads
(20552,1903,1,2,-1,234596),-- Elven Ring Piece
(20552,57,714,1424,0,700000),-- Adena
(20552,129,1,1,1,52),-- Sword of Revolution
(20552,2034,1,1,1,3842),-- Sword of Revolution Blade
(20552,1864,1,3,2,142857),-- Stem
(20552,1869,1,1,2,142857),-- Iron Ore
(20552,1878,1,1,2,55556),-- Braided Hemp
(20552,1884,1,1,2,90909),-- Cord
(20552,2208,1,1,2,15152),-- Recipe: Ogre Power Gauntlets
(20552,2285,1,1,2,971),-- Recipe: Claymore
(20553,1865,1,2,-1,724776),-- Varnish
(20553,1904,1,1,-1,67641),-- Elven Necklace Beads
(20553,2145,1,1,-1,51770),-- Recipe: Mithril Alloy
(20553,57,368,733,0,700000),-- Adena
(20553,318,1,1,1,27),-- Crucifix of Blood
(20553,2042,1,1,1,1732),-- Crucifix of Blood Piece
(20553,1865,1,1,2,76923),-- Varnish
(20553,1870,1,1,2,71429),-- Coal
(20553,1875,1,1,2,5814),-- Stone of Purity
(20553,1879,1,1,2,13158),-- Cokes
(20553,2208,1,1,2,6849),-- Recipe: Ogre Power Gauntlets
(20553,2285,1,1,2,558),-- Recipe: Claymore
(20554,1902,1,1,-1,85565),-- Elven Earring Beads
(20554,1941,1,1,-1,33586),-- Brigandine Temper
(20554,2146,1,1,-1,60324),-- Recipe: Artisan\'s frame
(20554,57,425,848,0,700000),-- Adena
(20554,224,1,1,1,16),-- Maingauche
(20554,1660,1,1,1,16),-- Cursed Maingauche
(20554,2038,1,1,1,1154),-- Maingauche Edge
(20554,2044,1,1,1,1154),-- Cursed Maingauche Edge
(20554,1864,1,1,2,142857),-- Stem
(20554,1869,1,1,2,71429),-- Iron Ore
(20554,1880,1,1,2,6329),-- Steel
(20554,2208,1,1,2,7874),-- Recipe: Ogre Power Gauntlets
(20554,5473,1,1,2,4739),-- Recipe: Durable Metal Plate(100%)
(20554,5550,1,1,2,1520),-- Durable Metal Plate
(20554,6036,1,1,2,2283),-- Greater Magic Haste Potion
(20555,1878,1,2,-1,304091),-- Braided Hemp
(20555,1903,1,1,-1,126810),-- Elven Ring Piece
(20555,2296,1,1,-1,4223),-- Recipe: Atuba Mace
(20555,57,383,764,0,700000),-- Adena
(20555,395,1,1,1,88),-- Manticore Skin Shirt
(20555,417,1,1,1,141),-- Manticore Skin Gaiters
(20555,1942,1,1,1,5011),-- Manticore Skin Shirt Texture
(20555,1943,1,1,1,7803),-- Manticore Skin Gaiters Pattern
(20555,1864,1,1,2,166667),-- Stem
(20555,1869,1,1,2,76923),-- Iron Ore
(20555,1880,1,1,2,7407),-- Steel
(20555,2208,1,1,2,8197),-- Recipe: Ogre Power Gauntlets
(20555,5473,1,1,2,4098),-- Recipe: Durable Metal Plate(100%)
(20555,5550,1,1,2,1279),-- Durable Metal Plate
(20556,1874,1,1,-1,53220),-- Oriharukon Ore
(20556,1957,1,1,-1,89096),-- Assault Boots Part
(20556,2305,1,1,-1,3486),-- Recipe: Crystal Staff
(20556,57,394,778,0,700000),-- Adena
(20556,438,1,1,1,94),-- Sage\'s Rag
(20556,1953,1,1,1,6421),-- Sage\'s Rag Lining
(20556,1865,1,1,2,76923),-- Varnish
(20556,1870,1,1,2,76923),-- Coal
(20556,1874,1,1,2,6211),-- Oriharukon Ore
(20556,2160,1,1,2,4348),-- Recipe: Moonstone Earring
(20556,2205,1,1,2,2070),-- Recipe: Chain Gaiters
(20556,5154,1,1,2,207),-- Recipe: Soulshot (C) Compressed Package (100%)
(20557,2055,1,1,-1,6984),-- Glaive Edge
(20557,3954,1,1,-1,2816),-- Recipe: Blessed Spiritshot C
(20557,4469,1,1,-1,4827),-- Dye of Strength <Str+1 Con-1>
(20557,57,414,837,0,700000),-- Adena
(20557,297,1,1,1,24),-- Glaive
(20557,2055,1,1,1,1754),-- Glaive Edge
(20557,1866,1,1,2,40000),-- Suede
(20557,1871,1,1,2,71429),-- Charcoal
(20557,1877,1,1,2,2062),-- Adamantite Nugget
(20557,2297,1,1,2,206),-- Recipe: Flamberge
(20557,2994,1,1,2,5348),-- Recipe: Boots of Seal
(20557,5154,1,1,2,412),-- Recipe: Soulshot (C) Compressed Package (100%)
(20557,6668,1,1,2,26316),-- Deluxe Chest Key - Grade 4
(20558,1905,1,1,-1,84475),-- Omen Beast\'s Eye Earring Gemstone
(20558,2310,1,1,-1,3792),-- Recipe: Dark Elven Dagger
(20558,2932,1,1,-1,96920),-- Plate Boots Design
(20558,57,421,853,0,700000),-- Adena
(20558,851,1,1,1,121),-- Omen Beast\'s Eye Earring
(20558,882,1,1,1,180),-- Mithril Ring
(20558,914,1,1,1,90),-- Necklace of Darkness
(20558,1905,1,1,1,7070),-- Omen Beast\'s Eye Earring Gemstone
(20558,1906,1,1,1,9990),-- Mithril Ring Wire
(20558,1907,1,1,1,5056),-- Necklace of Darkness Gem
(20558,1342,10,30,2,142857),-- Fine Steel Arrow
(20558,1875,1,1,2,4808),-- Stone of Purity
(20558,1879,1,1,2,12048),-- Cokes
(20558,1885,1,1,2,3215),-- High Grade Suede
(20558,2303,1,1,2,803),-- Recipe: Skull Graver
(20558,2988,1,1,2,4016),-- Recipe: Tempered Mithril Gaiters
(20558,5549,1,1,2,6410),-- Metallic Thread
(20558,5550,1,1,2,803),-- Durable Metal Plate
(20559,1873,1,1,-1,408624),-- Silver Nugget
(20559,2163,1,1,-1,52120),-- Recipe: Earring of Protection
(20559,2164,1,1,-1,78582),-- Recipe: Ring of Protection
(20559,57,493,1007,0,700000),-- Adena
(20559,71,1,1,1,11),-- Flamberge
(20559,72,1,1,1,11),-- Stormbringer
(20559,2059,1,1,1,897),-- Flamberge Blade
(20559,2060,1,1,1,959),-- Stormbringer Blade
(20559,1895,1,1,2,34487),-- Metallic Fiber
(20559,2164,1,1,2,9285),-- Recipe: Ring of Protection
(20559,2219,1,1,2,7497),-- Recipe: Crimson Boots
(20559,5154,1,1,2,403),-- Recipe: Soulshot (C) Compressed Package (100%)
(20559,8723,1,1,200,902),-- Life Stone: level 46 
(20559,8733,1,1,200,451),-- Mid-Grade Life Stone: level 46
(20560,1908,1,1,-1,90114),-- Moonstone Earring Wire
(20560,2165,1,1,-1,36460),-- Recipe: Necklace of Protection
(20560,2320,1,1,-1,3316),-- Recipe: Cursed Staff
(20560,57,462,935,0,700000),-- Adena
(20560,852,1,1,1,97),-- Moonstone Earring
(20560,883,1,1,1,146),-- Aquastone Ring
(20560,915,1,1,1,73),-- Aquastone Necklace
(20560,1908,1,1,1,7098),-- Moonstone Earring Wire
(20560,1909,1,1,1,10122),-- Aquastone Ring Wire
(20560,1910,1,1,1,5644),-- Aquastone Necklace Chain
(20560,1864,1,1,2,83333),-- Stem
(20560,1865,1,1,2,43478),-- Varnish
(20560,1866,1,1,2,22222),-- Suede
(20560,1868,1,1,2,83333),-- Thread
(20560,1873,1,1,2,15625),-- Silver Nugget
(20560,2225,1,1,2,680),-- Recipe: Demon\'s Stockings
(20560,2321,1,1,2,175),-- Recipe: Stiletto
(20560,2323,1,1,2,175),-- Recipe: Elemental Bow
(20560,3003,1,1,2,2717),-- Recipe: Gloves of Seal
(20560,8723,1,1,200,2041),-- Life Stone: level 46 
(20560,8733,1,1,200,280),-- Mid-Grade Life Stone: level 46 
(20561,1880,1,1,-1,112699),-- Steel
(20561,2164,1,1,-1,86691),-- Recipe: Ring of Protection
(20561,2940,1,1,-1,101302),-- Dwarven Chain Boots Part
(20561,57,543,1111,0,700000),-- Adena
(20561,96,1,1,1,8),-- Scythe
(20561,298,1,1,1,8),-- Orcish Glaive
(20561,302,1,1,1,8),-- Body Slasher
(20561,2062,1,1,1,697),-- Scythe Edge
(20561,2075,1,1,1,697),-- Orcish Glaive Blade
(20561,2076,1,1,1,697),-- Body Slasher Blade
(20561,1832,1,1,2,58824),-- Greater Antidote
(20561,1866,1,1,2,34483),-- Suede
(20561,1871,1,1,2,50000),-- Charcoal
(20561,1885,1,1,2,5952),-- High Grade Suede
(20561,4041,1,1,2,1052),-- Mold Hardener
(20561,5220,1,1,2,1751),-- Metal Hardener
(20561,6037,1,1,2,111111),-- Waking Scroll
(20561,6668,1,1,2,13333),-- Deluxe Chest Key - Grade 4
(20561,8723,1,1,200,1403),-- Life Stone: level 46
(20561,8733,1,1,200,701),-- Mid-Grade Life Stone: level 46
(20562,440,1,1,-1,307),-- Robe of Seal
(20562,1867,1,3,-1,723243),-- Animal Skin
(20562,2346,1,1,-1,2523),-- Recipe: Fist Blade
(20562,57,525,1069,0,700000),-- Adena
(20562,853,1,1,1,82),-- Earring of Protection
(20562,884,1,1,1,124),-- Ring of Protection
(20562,916,1,1,1,62),-- Necklace of Protection
(20562,1911,1,1,1,5960),-- Earring of Protection Gemstone
(20562,1912,1,1,1,8635),-- Ring of Protection Gemstone
(20562,1913,1,1,1,4873),-- Necklace of Protection Gemstone
(20562,1342,10,30,2,83333),-- Fine Steel Arrow
(20562,1864,1,1,2,90909),-- Stem
(20562,1869,1,1,2,38462),-- Iron Ore
(20562,1880,1,1,2,2924),-- Steel
(20562,2166,1,1,2,1462),-- Recipe: Earrings of Binding (100%)
(20562,2226,1,1,2,2439),-- Recipe: Mithril Gauntlets
(20562,2230,1,1,2,975),-- Recipe: Theca Leather Gaiters
(20562,4043,1,1,2,1949),-- Asofe
(20562,5550,1,1,2,713),-- Durable Metal Plate
(20562,8724,1,1,200,3906),-- Life Stone: level 49
(20562,8734,1,1,200,186),-- Mid-Grade Life Stone: level 49
(20563,1825,1,1,-1,64619),-- Recipe: Oriharukon
(20563,1864,1,5,-1,904670),-- Stem
(20563,1941,1,1,-1,35978),-- Brigandine Temper
(20563,57,355,909,0,700000),-- Adena
(20563,606,1,1,1,79),-- Rip Gauntlets
(20563,1124,1,1,1,79),-- Boots of Power
(20563,1946,1,1,1,3951),-- Rip Gauntlets Pattern
(20563,1948,1,1,1,3951),-- Boots of Power Pattern
(20563,2424,1,1,1,79),-- Manticore Skin Boots
(20563,2425,1,1,1,79),-- Brigandine Boots
(20563,2426,1,1,1,79),-- Elven Mithril Boots
(20563,2448,1,1,1,79),-- Manticore Skin Gloves
(20563,2449,1,1,1,79),-- Brigandine Gauntlets
(20563,2450,1,1,1,79),-- Elven Mithril Gloves
(20563,2923,1,1,1,3951),-- Manticore Skin Boot Lining
(20563,2924,1,1,1,3951),-- Brigandine Boots Design
(20563,2925,1,1,1,3951),-- Elven Mithril Boots Design
(20563,2926,1,1,1,3951),-- Manticore Skin Gloves Lining
(20563,2927,1,1,1,3951),-- Brigandine Gauntlets Pattern
(20563,2928,1,1,1,3951),-- Elven Mithril Gloves Design
(20563,1865,1,1,2,90909),-- Varnish
(20563,1870,1,1,2,90909),-- Coal
(20563,1874,1,1,2,7634),-- Oriharukon Ore
(20563,2160,1,1,2,5714),-- Recipe: Moonstone Earring
(20563,2205,1,1,2,3077),-- Recipe: Chain Gaiters
(20563,5154,1,1,2,293),-- Recipe: Soulshot (C) Compressed Package (100%)
(20564,1864,1,5,-1,814270),-- Stem
(20564,1885,1,1,-1,67856),-- High Grade Suede
(20564,2306,1,1,-1,3556),-- Recipe: Stick of Faith
(20564,57,402,804,0,700000),-- Adena
(20564,630,1,1,1,133),-- Square Shield
(20564,1956,1,1,1,7736),-- Square Shield Fragment
(20564,2412,1,1,1,93),-- Plate Helmet
(20564,2494,1,1,1,133),-- Plate Shield
(20564,2930,1,1,1,5494),-- Plate Helmet Design
(20564,2934,1,1,1,7736),-- Plate Shield Fragment
(20564,1866,1,1,2,45455),-- Suede
(20564,1871,1,1,2,76923),-- Charcoal
(20564,1877,1,1,2,2688),-- Adamantite Nugget
(20564,2297,1,1,2,518),-- Recipe: Flamberge
(20564,2994,1,1,2,5291),-- Recipe: Boots of Seal
(20564,5154,1,1,2,94),-- Recipe: Soulshot (C) Compressed Package (100%)
(20564,6668,1,1,2,26316),-- Deluxe Chest Key - Grade 4
(20565,1865,1,3,-1,940260),-- Varnish
(20565,3954,1,1,-1,3134),-- Recipe: Blessed Spiritshot C
(20565,4470,1,1,-1,5373),-- Dye of Strength <Str+1 Dex-1>
(20565,57,461,932,0,700000),-- Adena
(20565,262,1,1,1,27),-- Scallop Jamadhr
(20565,2053,1,1,1,1952),-- Scallop Jamadhr Edge
(20565,1875,1,1,2,5025),-- Stone of Purity
(20565,1879,1,1,2,13158),-- Cokes
(20565,1885,1,1,2,6944),-- High Grade Suede
(20565,2303,1,1,2,2096),-- Recipe: Skull Graver
(20565,2988,1,1,2,2994),-- Recipe: Tempered Mithril Gaiters
(20565,5549,1,1,2,7752),-- Metallic Thread
(20565,5550,1,1,2,1247),-- Durable Metal Plate
(20566,1905,1,1,-1,95916),-- Omen Beast\'s Eye Earring Gemstone
(20566,2995,1,1,-1,78255),-- Recipe: Reinforced Mithril Gloves
(20566,3000,1,1,-1,63206),-- Recipe: Dwarven Chain Shield
(20566,57,477,970,0,700000),-- Adena
(20566,630,1,1,1,161),-- Square Shield
(20566,1956,1,1,1,9368),-- Square Shield Fragment
(20566,2412,1,1,1,113),-- Plate Helmet
(20566,2494,1,1,1,161),-- Plate Shield
(20566,2930,1,1,1,6653),-- Plate Helmet Design
(20566,2934,1,1,1,9368),-- Plate Shield Fragment
(20566,1895,1,1,2,22222),-- Metallic Fiber
(20566,2164,1,1,2,5208),-- Recipe: Ring of Protection
(20566,2219,1,1,2,6452),-- Recipe: Crimson Boots
(20566,5154,1,1,2,289),-- Recipe: Soulshot (C) Compressed Package (100%)
(20566,8723,1,1,200,4329),-- Life Stone: level 46 
(20566,8733,1,1,200,347),-- Mid-Grade Life Stone: level 46
(20567,2061,1,1,-1,8442),-- Big Hammer Head
(20567,2165,1,1,-1,41845),-- Recipe: Necklace of Protection
(20567,3004,1,1,-1,67836),-- Recipe: Rind Leather Gloves
(20567,57,526,1078,0,700000),-- Adena
(20567,263,1,1,1,23),-- Chakram
(20567,2073,1,1,1,1995),-- Chakram Edge
(20567,1864,1,1,2,83333),-- Stem
(20567,1865,1,1,2,47619),-- Varnish
(20567,1866,1,1,2,35714),-- Suede
(20567,1868,1,1,2,100000),-- Thread
(20567,1873,1,1,2,12821),-- Silver Nugget
(20567,2225,1,1,2,1247),-- Recipe: Demon\'s Stockings
(20567,2321,1,1,2,416),-- Recipe: Stiletto
(20567,3003,1,1,2,3745),-- Recipe: Gloves of Seal
(20567,8723,1,1,200,2494),-- Life Stone: level 46 
(20567,8733,1,1,200,1247),-- Mid-Grade Life Stone: level 46 
(20568,1884,1,2,-1,591456),-- Cord
(20568,1908,1,1,-1,91015),-- Moonstone Earring Wire
(20568,1961,1,1,-1,30756),-- Mithril Shirt Fabric
(20568,57,465,945,0,700000),-- Adena
(20568,89,1,1,1,3),-- Big Hammer
(20568,160,1,1,1,3),-- Battle Axe
(20568,161,1,1,1,3),-- Silver Axe
(20568,173,1,1,1,3),-- Skull Graver
(20568,191,1,1,1,3),-- Heavy Doom Hammer
(20568,192,1,1,1,3),-- Crystal Staff
(20568,193,1,1,1,3),-- Stick of Faith
(20568,194,1,1,1,3),-- Heavy Doom Axe
(20568,2061,1,1,1,220),-- Big Hammer Head
(20568,2063,1,1,1,220),-- Battle Axe Head
(20568,2064,1,1,1,220),-- Silver Axe Head
(20568,2065,1,1,1,220),-- Skull Graver Head
(20568,2066,1,1,1,223),-- Heavy Doom Hammer Head
(20568,2067,1,1,1,205),-- Crystal Staff Head
(20568,2068,1,1,1,205),-- Stick of Faith Shaft
(20568,2069,1,1,1,223),-- Heavy Doom Axe Blade
(20568,1864,1,1,2,83333),-- Stem
(20568,1865,1,1,2,416672),-- Varnish
(20568,1866,1,1,2,31250),-- Suede
(20568,1868,1,1,2,83333),-- Thread
(20568,1873,1,1,2,17241),-- Silver Nugget
(20568,2321,1,1,2,303),-- Recipe: Stiletto
(20568,2324,1,1,2,182),-- Recipe: Elven Bow of Nobility
(20568,8723,1,1,200,1149),-- Life Stone: level 46
(20568,8733,1,1,200,424),-- Mid-Grade Life Stone: level 46
(20568,8743,1,1,200,61),-- High-Grade Life Stone: level 46
(20569,1961,1,1,-1,33428),-- Mithril Shirt Fabric
(20569,2213,1,1,-1,16221),-- Recipe: Dwarven Chain Mail Shirt
(20569,3001,1,1,-1,64883),-- Recipe: Rind Leather Boots
(20569,57,504,1030,0,700000),-- Adena
(20569,631,1,1,1,124),-- Eldarake
(20569,1964,1,1,1,6650),-- Eldarake Temper
(20569,2413,1,1,1,88),-- Chain Hood
(20569,2495,1,1,1,124),-- Chain Shield
(20569,2936,1,1,1,6040),-- Chain Hood Pattern
(20569,2945,1,1,1,6650),-- Chain Shield Fragment
(20569,1865,1,1,2,45455),-- Varnish
(20569,1870,1,1,2,47619),-- Coal
(20569,1875,1,1,2,3106),-- Stone of Purity
(20569,1879,1,1,2,8065),-- Cokes
(20569,2220,1,1,2,969),-- Recipe: Rind Leather Armor
(20569,2230,1,1,2,1325),-- Recipe: Theca Leather Gaiters
(20569,2508,1,1,2,40000),-- Cursed Bone
(20569,4040,1,1,2,743),-- Mold Lubricant
(20569,8723,1,1,200,1938),-- Life Stone: level 46
(20569,8733,1,1,200,452),-- Mid-Grade Life Stone: level 46
(20569,8743,1,1,200,32),-- High-Grade Life Stone: 46
(20570,1911,1,1,-1,77930),-- Earring of Protection Gemstone
(20570,2216,1,1,-1,43296),-- Recipe: Great Helmet
(20570,3008,1,1,-1,38101),-- Recipe: Theca Leather Gloves
(20570,57,506,1033,0,700000),-- Adena
(20570,355,1,1,1,37),-- Dwarven Chain Mail Shirt
(20570,382,1,1,1,58),-- Dwarven Chain Gaiters
(20570,1968,1,1,1,2797),-- Dwarven Chain Mail Shirt Material
(20570,1969,1,1,1,4436),-- Dwarven Chain Gaiters Material
(20570,1866,1,1,2,30303),-- Suede
(20570,1871,1,1,2,43478),-- Charcoal
(20570,1885,1,1,2,3937),-- High Grade Suede
(20570,2167,1,1,2,2681),-- Recipe: Ring of Ages
(20570,2343,1,1,2,149),-- Recipe: Mace of the Underworld
(20570,4041,1,1,2,239),-- Mold Hardener
(20570,5220,1,1,2,1701),-- Metal Hardener
(20570,8724,1,1,200,1401),-- Life Stone: level 49
(20570,8734,1,1,200,358),-- Mid-Grade Life Stone: level 49
(20570,8744,1,1,200,23),-- High-Grade Life Stone: level 49
(20571,951,1,1,-1,2251),-- Scroll: Enchant Weapon (Grade C)
(20571,2335,1,1,-1,2880),-- Recipe: War Axe
(20571,2968,1,1,-1,8355),-- Dwarven Warhammer Head
(20571,57,599,1219,0,700000),-- Adena
(20571,73,1,1,1,5),-- Shamshir
(20571,74,1,1,1,5),-- Katana
(20571,131,1,1,1,5),-- Spirit Sword
(20571,133,1,1,1,5),-- Raid Sword
(20571,2077,1,1,1,456),-- Shamshir Blade
(20571,2078,1,1,1,456),-- Katana Blade
(20571,2080,1,1,1,456),-- Spirit Sword Blade
(20571,2081,1,1,1,456),-- Raid Sword Blade
(20571,1866,1,1,2,32258),-- Suede
(20571,1871,1,1,2,55556),-- Charcoal
(20571,1885,1,1,2,4202),-- High Grade Suede
(20571,2336,1,1,2,47),-- Recipe: Nirvana Axe
(20571,2351,1,1,2,94),-- Recipe: Scorpion
(20571,2508,1,1,2,43478),-- Cursed Bone
(20571,4041,1,1,2,283),-- Mold Hardener
(20571,5220,1,1,2,2358),-- Metal Hardener
(20571,6668,1,1,2,20408),-- Deluxe Chest Key - Grade 4
(20571,8724,1,1,200,2075),-- Life Stone: level 49
(20571,8734,1,1,200,519),-- Mid-Grade Life Stone: level 49
(20572,1894,1,1,-1,47642),-- Crafted Leather
(20572,2950,1,1,-1,114196),-- Gloves of Seal Pattern
(20572,4125,1,1,-1,60346),-- Recipe: Maestro Mold
(20572,57,656,1338,0,700000),-- Adena
(20572,94,1,1,1,21),-- Bec de Corbin
(20572,2079,1,1,1,2068),-- Bec De Corbin Blade
(20572,1342,10,30,2,111111),-- Fine Steel Arrow
(20572,1867,1,1,2,76923),-- Animal Skin
(20572,1872,1,1,2,76923),-- Animal Bone
(20572,1882,1,1,2,11628),-- Leather
(20572,1894,1,1,2,2160),-- Crafted Leather
(20572,2224,1,1,2,1080),-- Recipe: Demon\'s Tunic
(20572,2508,1,1,2,55556),-- Cursed Bone
(20572,3009,1,1,2,1698),-- Recipe: Composite Boots
(20572,4042,1,1,2,463),-- Enria
(20572,8724,1,1,200,2160),-- Life Stone: level 49
(20572,8734,1,1,200,309),-- Mid-Grade Life Stone: level 49
(20572,8744,1,1,200,154),-- High-Grade Life Stone: level 49
(20573,2119,1,1,-1,5367),-- Crystal Dagger Blade
(20573,4122,1,1,-1,52347),-- Recipe: Maestro Holder
(20573,5549,1,1,-1,117780),-- Metallic Thread
(20573,57,472,959,0,700000),-- Adena
(20573,64,1,1,1,100),-- Composite Boots
(20573,608,1,1,1,100),-- Mithril Gauntlets
(20573,1981,1,1,1,7163),-- Mithril Gauntlets Design
(20573,2956,1,1,1,7163),-- Composite Boots Part
(20573,1342,10,30,2,76923),-- Fine Steel Arrow
(20573,1868,1,1,2,76923),-- Thread
(20573,1873,1,1,2,13889),-- Silver Nugget
(20573,1889,1,1,2,2703),-- Compound Braid
(20573,2231,1,1,2,239),-- Recipe: Full Plate Armor
(20573,3014,1,1,2,995),-- Recipe: Full Plate Boots
(20573,3019,1,1,2,1074),-- Recipe: Full Plate Shield
(20573,4044,1,1,2,1393),-- Thons
(20573,5549,1,1,2,3984),-- Metallic Thread
(20573,8725,1,1,200,835),-- Life Stone: level 52
(20573,8735,1,1,200,438),-- Mid-Grade Life Stone: level 52
(20573,8745,1,1,200,40),-- High-Grade Life Stone: level 52
(20574,1539,1,2,-1,278165),-- Greater Healing Potion
(20574,1867,1,3,-1,834496),-- Animal Skin
(20574,4601,1,1,-1,2782),-- Greater Dye of Strength <Str+3 Con-3>
(20574,57,497,1009,0,700000),-- Adena
(20574,75,1,1,1,3),-- Caliburs
(20574,76,1,1,1,3),-- Sword of Delusion
(20574,77,1,1,1,3),-- Tsurugi
(20574,84,1,1,1,3),-- Homunkulus\'s Sword
(20574,134,1,1,1,3),-- Sword of Nightmare
(20574,145,1,1,1,3),-- Sword of Whispering Death
(20574,2089,1,1,1,236),-- Caliburs Edge
(20574,2090,1,1,1,236),-- Delusional Blade
(20574,2091,1,1,1,236),-- Tsurugi Blade
(20574,2092,1,1,1,236),-- Homunkulus\'s Sword Blade
(20574,2095,1,1,1,236),-- Sword of Nightmare Blade
(20574,2096,1,1,1,236),-- Sword of Whispering Death Blade
(20574,1539,1,1,2,8403),-- Greater Healing Potion
(20574,1864,1,1,2,76923),-- Stem
(20574,1869,1,1,2,41667),-- Iron Ore
(20574,1880,1,1,2,4831),-- Steel
(20574,4043,1,1,2,1637),-- Asofe
(20574,4122,1,1,2,1473),-- Recipe: Maestro Holder
(20574,4123,1,1,2,1603),-- Recipe: Maestro Anvil Lock
(20574,4609,1,1,2,33),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20574,5550,1,1,2,556),-- Durable Metal Plate
(20574,8725,1,1,200,1342),-- Life Stone: level 52
(20574,8735,1,1,200,294),-- Mid-Grade Life Stone: level 52
(20574,8745,1,1,200,33),-- High-Grade Life Stone: level 52
(20574,8755,1,1,200,33),-- Top-Grade Life Stone: level 52
(20575,1806,1,1,-1,2791),-- Recipe: Soulshot: B-Grade
(20575,1870,1,3,-1,697860),-- Coal
(20575,4598,1,1,-1,4652),-- Greater Dye of Mental Strength <Men+2 Wit-2>
(20575,57,558,1137,0,700000),-- Adena
(20575,135,1,1,1,5),-- Samurai Longsword
(20575,2115,1,1,1,591),-- Samurai Longsword Blade
(20575,5286,1,1,1,5),-- Berserker Blade
(20575,5530,1,1,1,592),-- Berserker Blade Edge
(20575,1865,1,1,2,40000),-- Varnish
(20575,1870,1,1,2,40000),-- Coal
(20575,1874,1,1,2,2747),-- Oriharukon Ore
(20575,4039,1,1,2,1325),-- Mold Glue
(20575,4043,1,1,2,1385),-- Asofe
(20575,4145,1,1,2,207),-- Recipe: Avadon Leather Armor
(20575,4180,1,1,2,4975),-- Recipe: Silver Arrow
(20575,4184,1,1,2,62),-- Recipe: Sprite\'s Staff
(20575,5165,1,1,2,62),-- Recipe: Blessed Spiritshot (B) Compressed Package (100%)
(20575,8726,1,1,200,703),-- Life Stone: level 55
(20575,8736,1,1,200,372),-- Mid-Grade Life Stone: level 55
(20575,8746,1,1,200,83),-- High-Grade Life Stone: level 55
(20576,2119,1,1,-1,13217),-- Crystal Dagger Blade
(20576,4145,1,1,-1,12722),-- Recipe: Avadon Leather Armor
(20576,4591,1,1,-1,9669),-- Greater Dye of Constitution <Con+2 Str-2>
(20576,57,1121,2284,0,700000),-- Adena
(20576,119,1,1,1,64),-- Necklace of Binding
(20576,855,1,1,1,85),-- Nassen\'s Earring
(20576,886,1,1,1,127),-- Ring of Binding
(20576,1918,1,1,1,7730),-- Nassen\'s Earring Gemstone
(20576,1920,1,1,1,6107),-- Necklace of Binding Chain
(20576,2917,1,1,1,10252),-- Ring of Binding Gemstone
(20576,1865,1,1,2,76923),-- Varnish
(20576,1870,1,1,2,83333),-- Coal
(20576,1875,1,1,2,5405),-- Stone of Purity
(20576,1879,1,1,2,13699),-- Cokes
(20576,4040,1,1,2,1567),-- Mold Lubricant
(20576,4178,1,1,2,1506),-- Recipe: Zubei\'s Shield (100%)
(20576,4187,1,1,2,20),-- Recipe: Kris
(20576,5155,1,1,2,159),-- Recipe: Soulshot (B) Compressed Package (100%)
(20576,5275,1,1,2,99),-- Recipe: Greater Spiritshot (B) Compressed Package(100%)
(20576,8726,1,1,200,1696),-- Life Stone: level 55
(20576,8736,1,1,200,377),-- Mid-Grade Life Stone: level 55
(20576,8746,1,1,200,159),-- High-Grade Life Stone: level 55
(20577,351,1,1,-1,580),-- Blast Plate
(20577,2030,1,1,-1,6880),-- Tome of Blood Page
(20577,2154,1,1,-1,62979),-- Recipe: Elven Earring
(20577,57,298,594,0,700000),-- Adena
(20577,849,1,1,1,244),-- Tiger\'s Eye Earring
(20577,880,1,1,1,366),-- Black Pearl Ring
(20577,912,1,1,1,183),-- Near Forest Necklace
(20577,1901,1,1,1,10800),-- Tiger\'s Eye Earring Stone
(20577,1867,1,1,2,66667),-- Animal Skin
(20577,1872,1,1,2,71428),-- Animal Bone
(20577,1875,1,1,2,1916),-- Stone of Purity
(20577,1881,1,1,2,7246),-- Coarse Bone Powder
(20577,2154,1,1,2,5952),-- Recipe: Elven Earring
(20577,2270,1,1,2,320),-- Recipe: Goat Head Staff
(20577,2274,1,1,2,320),-- Recipe: Skull Breaker
(20578,1882,1,1,-1,142151),-- Leather
(20578,1901,1,1,-1,88354),-- Tiger\'s Eye Earring Stone
(20578,1939,1,1,-1,56186),-- Mithril Banded Gaiters Material
(20578,57,324,649,0,700000),-- Adena
(20578,278,1,1,1,36),-- Gastraphetes
(20578,2029,1,1,1,2029),-- Gastraphetes Shaft
(20578,1868,1,1,2,125000),-- Thread
(20578,1873,1,1,2,31250),-- Silver Nugget
(20578,1889,1,1,2,2924),-- Compound Braid
(20578,2273,1,1,2,268),-- Recipe: Tarbar
(20578,5549,1,1,2,7463),-- Metallic Thread
(20578,6037,1,1,2,125000),-- Waking Scroll
(20579,1889,1,1,-1,44463),-- Compound Braid
(20579,1943,1,1,-1,52975),-- Manticore Skin Gaiters Pattern
(20579,1946,1,1,-1,90129),-- Rip Gauntlets Pattern
(20579,57,338,675,0,700000),-- Adena
(20579,606,1,1,1,58),-- Rip Gauntlets
(20579,1124,1,1,1,58),-- Boots of Power
(20579,1946,1,1,1,2913),-- Rip Gauntlets Pattern
(20579,1948,1,1,1,2913),-- Boots of Power Pattern
(20579,2424,1,1,1,58),-- Manticore Skin Boots
(20579,2425,1,1,1,58),-- Brigandine Boots
(20579,2426,1,1,1,58),-- Elven Mithril Boots
(20579,2448,1,1,1,58),-- Manticore Skin Gloves
(20579,2449,1,1,1,58),-- Brigandine Gauntlets
(20579,2450,1,1,1,58),-- Elven Mithril Gloves
(20579,2923,1,1,1,2913),-- Manticore Skin Boot Lining
(20579,2924,1,1,1,2913),-- Brigandine Boots Design
(20579,2925,1,1,1,2913),-- Elven Mithril Boots Design
(20579,2926,1,1,1,2913),-- Manticore Skin Gloves Lining
(20579,2927,1,1,1,2913),-- Brigandine Gauntlets Pattern
(20579,2928,1,1,1,2913),-- Elven Mithril Gloves Design
(20579,1868,1,1,2,125000),-- Thread
(20579,1873,1,1,2,26316),-- Silver Nugget
(20579,1889,1,1,2,1259),-- Compound Braid
(20579,2146,1,1,2,7194),-- Recipe: Artisan\'s frame
(20579,2286,1,1,2,180),-- Recipe: Bonebreaker
(20579,5549,1,1,2,4505),-- Metallic Thread
(20579,6668,1,1,2,18868),-- Deluxe Chest Key - Grade 4
(20580,1868,1,7,-1,853135),-- Thread
(20580,1881,1,1,-1,170627),-- Coarse Bone Powder
(20580,2031,1,1,-1,12954),-- Morning Star Head
(20580,57,649,1245,0,700000),-- Adena
(20580,395,1,1,1,148),-- Manticore Skin Shirt
(20580,417,1,1,1,237),-- Manticore Skin Gaiters
(20580,1942,1,1,1,8435),-- Manticore Skin Shirt Texture
(20580,1943,1,1,1,13135),-- Manticore Skin Gaiters Pattern
(20580,1868,1,1,2,250000),-- Thread
(20580,1873,1,1,2,50000),-- Silver Nugget
(20580,1889,1,1,2,9259),-- Compound Braid
(20580,2194,1,1,2,12987),-- Recipe: Mithril Gloves
(20580,2288,1,1,2,402),-- Recipe: Ghost Staff
(20580,5549,1,1,2,13333),-- Metallic Thread
(20581,1902,1,1,-1,73432),-- Elven Earring Beads
(20581,2036,1,1,-1,7337),-- Skull Breaker Head
(20581,2202,1,1,-1,74108),-- Recipe: Assault Boots
(20581,57,368,733,0,700000),-- Adena
(20581,279,1,1,1,27),-- Strengthened Long Bow
(20581,2040,1,1,1,2168),-- Strengthened Long Bow Shaft
(20581,1341,10,30,2,166667),-- Bone Arrow
(20581,1875,1,1,2,2890),-- Stone of Purity
(20581,1879,1,1,2,7092),-- Cokes
(20581,1885,1,1,2,3534),-- High Grade Suede
(20581,2198,1,1,2,1608),-- Recipe: Sage\'s Rag
(20581,2287,1,1,2,964),-- Recipe: Atuba Hammer
(20581,2293,1,1,2,232),-- Recipe: Glaive
(20581,5549,1,1,2,6098),-- Metallic Thread
(20581,5550,1,1,2,322),-- Durable Metal Plate
(20582,1825,1,1,-1,103022),-- Recipe: Oriharukon
(20582,1894,1,1,-1,50607),-- Crafted Leather
(20582,1942,1,1,-1,73568),-- Manticore Skin Shirt Texture
(20582,57,728,1449,0,700000),-- Adena
(20582,629,1,1,1,317),-- Kite Shield
(20582,1947,1,1,1,17893),-- Kite Shield Fragment
(20582,2411,1,1,1,222),-- Brigandine Helmet
(20582,2493,1,1,1,317),-- Brigandine Shield
(20582,2922,1,1,1,4786),-- Brigandine Helmet Design
(20582,2929,1,1,1,17893),-- Brigandine Shield Fragment
(20582,1341,20,60,2,200000),-- Bone Arrow
(20582,1864,1,3,2,83333),-- Stem
(20582,1869,1,1,2,125000),-- Iron Ore
(20582,1880,1,1,2,11364),-- Steel
(20582,2987,1,1,2,11364),-- Recipe: Plate Shield
(20582,3020,1,1,2,1383),-- Recipe: Elven Long Sword
(20582,5550,1,1,2,2075),-- Durable Metal Plate
(20583,1903,1,1,-1,126810),-- Elven Ring Piece
(20583,1944,1,1,-1,38777),-- Mithril Tunic Fabric
(20583,2157,1,1,-1,62829),-- Recipe: Omen Beast\'s Eye Earring
(20583,57,384,763,0,700000),-- Adena
(20583,261,1,1,1,29),-- Bich\'Hwa
(20583,2039,1,1,1,1989),-- Bich\'Hwa Edge
(20583,1865,1,1,2,52632),-- Varnish
(20583,1870,1,1,2,90909),-- Coal
(20583,1874,1,1,2,1946),-- Oriharukon Ore
(20583,2160,1,1,2,3891),-- Recipe: Moonstone Earring
(20583,5154,1,1,2,1946),-- Recipe: Soulshot (C) Compressed Package (100%)
(20584,1805,1,1,-1,3432),-- Recipe: Soulshot: C-Grade
(20584,2211,1,1,-1,23082),-- Recipe: Plated Leather Armor
(20584,2988,1,2,-1,43437),-- Recipe: Tempered Mithril Gaiters
(20584,57,508,1017,0,700000),-- Adena
(20584,280,1,1,1,29),-- Light Crossbow
(20584,2054,1,1,1,2201),-- Light Crossbow Shaft
(20584,1866,1,1,2,62500),-- Suede
(20584,1871,1,1,2,90909),-- Charcoal
(20584,1877,1,1,2,1443),-- Adamantite Nugget
(20584,2994,1,1,2,5780),-- Recipe: Boots of Seal
(20584,6668,1,1,2,29412),-- Deluxe Chest Key - Grade 4
(20585,1905,1,1,-1,98755),-- Omen Beast\'s Eye Earring Gemstone
(20585,2204,1,1,-1,20103),-- Recipe: Chain Mail Shirt
(20585,2308,1,1,-1,4433),-- Recipe: Cursed Dagger
(20585,57,498,1006,0,700000),-- Adena
(20585,159,1,1,1,6),-- Bonebreaker
(20585,187,1,1,1,6),-- Atuba Hammer
(20585,188,1,1,1,6),-- Ghost Staff
(20585,189,1,1,1,6),-- Staff of Life
(20585,190,1,1,1,6),-- Atuba Mace
(20585,2048,1,1,1,422),-- Bonebreaker Head
(20585,2049,1,1,1,422),-- Atuba Hammer Head
(20585,2050,1,1,1,447),-- Ghost Staff Head
(20585,2051,1,1,1,447),-- Staff of Life Shaft
(20585,2058,1,1,1,422),-- Atuba Mace Head
(20585,1875,1,1,2,5263),-- Stone of Purity
(20585,1879,1,1,2,14493),-- Cokes
(20585,1885,1,1,2,6410),-- High Grade Suede
(20585,2303,1,1,2,564),-- Recipe: Skull Graver
(20585,2988,1,1,2,3759),-- Recipe: Tempered Mithril Gaiters
(20585,5549,1,1,2,10752),-- Metallic Thread
(20585,5550,1,1,2,940),-- Durable Metal Plate
(20586,1884,1,3,-1,599116),-- Cord
(20586,2314,1,1,-1,4251),-- Recipe: Body Slasher
(20586,3954,1,1,-1,3245),-- Recipe: Blessed Spiritshot C
(20586,57,471,956,0,700000),-- Adena
(20586,70,1,1,1,14),-- Claymore
(20586,2047,1,1,1,1075),-- Claymore Blade
(20586,2499,1,1,1,14),-- Elven Long Sword
(20586,2967,1,1,1,1060),-- Elven Long Sword Blade
(20586,1895,1,1,2,20833),-- Metallic Fiber
(20586,2164,1,1,2,7692),-- Recipe: Ring of Protection
(20586,2219,1,1,2,5208),-- Recipe: Crimson Boots
(20586,5154,1,1,2,743),-- Recipe: Soulshot (C) Compressed Package (100%)
(20586,8723,1,1,200,2475),-- Life Stone: level 46 
(20586,8733,1,1,200,1238),-- Mid-Grade Life Stone: level 46
(20587,1875,1,1,-1,61186),-- Stone of Purity
(20587,1953,1,1,-1,29397),-- Sage\'s Rag Lining
(20587,2062,1,1,-1,7198),-- Scythe Edge
(20587,57,442,905,0,700000),-- Adena
(20587,89,1,1,1,3),-- Big Hammer
(20587,160,1,1,1,3),-- Battle Axe
(20587,161,1,1,1,3),-- Silver Axe
(20587,173,1,1,1,3),-- Skull Graver
(20587,191,1,1,1,3),-- Heavy Doom Hammer
(20587,192,1,1,1,3),-- Crystal Staff
(20587,193,1,1,1,3),-- Stick of Faith
(20587,194,1,1,1,3),-- Heavy Doom Axe
(20587,2061,1,1,1,210),-- Big Hammer Head
(20587,2063,1,1,1,210),-- Battle Axe Head
(20587,2064,1,1,1,210),-- Silver Axe Head
(20587,2065,1,1,1,210),-- Skull Graver Head
(20587,2066,1,1,1,213),-- Heavy Doom Hammer Head
(20587,2067,1,1,1,196),-- Crystal Staff Head
(20587,2068,1,1,1,196),-- Stick of Faith Shaft
(20587,2069,1,1,1,213),-- Heavy Doom Axe Blade
(20587,1864,1,1,2,76923),-- Stem
(20587,1865,1,1,2,37037),-- Varnish
(20587,1866,1,1,2,27027),-- Suede
(20587,1868,1,1,2,62500),-- Thread
(20587,1873,1,1,2,14286),-- Silver Nugget
(20587,2225,1,1,2,752),-- Recipe: Demon\'s Stockings
(20587,2321,1,1,2,752),-- Recipe: Stiletto
(20587,2323,1,1,2,752),-- Recipe: Elemental Bow
(20587,3003,1,1,2,3012),-- Recipe: Gloves of Seal
(20587,8723,1,1,200,2257),-- Life Stone: level 46 
(20587,8733,1,1,200,1506),-- Mid-Grade Life Stone: level 46 
(20588,1959,1,1,-1,48799),-- Chain Mail Shirt Material
(20588,2324,1,1,-1,6312),-- Recipe: Elven Bow of Nobility
(20588,57,587,1184,0,700000),-- Adena
(20588,71,1,1,1,13),-- Flamberge
(20588,72,1,1,1,13),-- Stormbringer
(20588,2059,1,1,1,1060),-- Flamberge Blade
(20588,2060,1,1,1,1134),-- Stormbringer Blade
(20588,1868,1,1,2,166667),-- Thread
(20588,1873,1,1,2,25000),-- Silver Nugget
(20588,3003,1,1,2,3135),-- Recipe: Gloves of Seal
(20588,4044,1,1,2,1570),-- Thons
(20588,5549,1,1,2,9434),-- Metallic Thread
(20588,8723,1,1,200,7874),-- Life Stone: level 46
(20588,8743,1,1,200,949),-- High-Grade Life Stone: level 46 
(20589,1908,1,1,-1,170964),-- Moonstone Earring Wire
(20589,1961,1,1,-1,57772),-- Mithril Shirt Fabric
(20589,1962,1,1,-1,57772),-- Karmian Tunic Pattern
(20589,57,875,1775,0,700000),-- Adena
(20589,439,1,1,1,113),-- Karmian Tunic
(20589,471,1,1,1,181),-- Karmian Stockings
(20589,1954,1,1,1,11341),-- Karmian Stocking Design
(20589,1962,1,1,1,6827),-- Karmian Tunic Pattern
(20589,1868,1,1,2,125000),-- Thread
(20589,1873,1,1,2,32258),-- Silver Nugget
(20589,1889,1,1,2,3745),-- Compound Braid
(20589,2214,1,1,2,2247),-- Recipe: Dwarven Chain Gaiters
(20589,3003,1,1,2,5988),-- Recipe: Gloves of Seal
(20589,4044,1,1,2,2994),-- Thons
(20589,5549,1,1,2,4505),-- Metallic Thread
(20589,8723,1,1,200,2247),-- Life Stone: level 46
(20589,8733,1,1,200,749),-- Mid-Grade Life Stone: level 46
(20590,2068,1,1,-1,13465),-- Stick of Faith Shaft
(20590,2163,1,1,-1,95320),-- Recipe: Earring of Protection
(20590,2935,1,1,-1,99323),-- Tempered Mithril Gaiters Fragment
(20590,57,901,1840,0,700000),-- Adena
(20590,354,1,1,1,88),-- Chain Mail Shirt
(20590,381,1,1,1,140),-- Chain Gaiters
(20590,1959,1,1,1,5947),-- Chain Mail Shirt Material
(20590,1960,1,1,1,12097),-- Chain Gaiters Part
(20590,1867,1,1,2,111111),-- Animal Skin
(20590,1872,1,1,2,100000),-- Animal Bone
(20590,1882,1,1,2,14085),-- Leather
(20590,1894,1,1,2,1751),-- Crafted Leather
(20590,2230,1,1,2,876),-- Recipe: Theca Leather Gaiters
(20590,8723,1,1,200,4386),-- Life Stone: level 46
(20590,8733,1,1,200,312),-- Mid-Grade Life Stone: level 46
(20590,8743,1,1,200,312),-- High-Grade Life Stone: level 46 
(20591,951,1,1,-1,3983),-- Scroll: Enchant Weapon (Grade C)
(20591,1911,1,1,-1,162953),-- Earring of Protection Gemstone
(20591,2947,1,1,-1,156493),-- Dwarven Chain Shield Fragment
(20591,57,1058,2157,0,700000),-- Adena
(20591,94,1,1,1,34),-- Bec de Corbin
(20591,2079,1,1,1,3336),-- Bec De Corbin Blade
(20591,1867,1,1,2,125000),-- Animal Skin
(20591,1872,1,1,2,111111),-- Animal Bone
(20591,1882,1,1,2,21277),-- Leather
(20591,1894,1,1,2,4464),-- Crafted Leather
(20591,4042,1,1,2,1678),-- Enria
(20591,2230,1,1,2,2237),-- Recipe: Theca Leather Gaiters
(20591,2320,1,1,2,280),-- Recipe: Cursed Staff
(20591,8723,1,1,200,3356),-- Life Stone: level 46
(20591,8743,1,1,200,280),-- High-Grade Life Stone: level 46
(20592,1880,1,2,-1,301213),-- Steel
(20592,2166,1,2,-1,102106),-- Recipe: Earrings of Binding (100%)
(20592,2951,1,1,-1,253333),-- Rind Leather Gloves Design
(20592,57,1458,2964,0,700000),-- Adena
(20592,195,1,1,1,24),-- Cursed Staff
(20592,2082,1,1,1,2352),-- Cursed Staff Head
(20592,2502,1,1,1,24),-- Dwarven War Hammer
(20592,2968,1,1,1,2252),-- Dwarven Warhammer Head
(20592,1866,1,1,2,71429),-- Suede
(20592,1871,1,1,2,111111),-- Charcoal
(20592,1877,1,1,2,2326),-- Adamantite Nugget
(20592,4040,1,1,2,2326),-- Mold Lubricant
(20592,4041,1,1,2,1451),-- Mold Hardener
(20592,2340,1,1,2,290),-- Recipe: Pa\'agrian Hammer
(20592,3006,1,1,2,4219),-- Recipe: Demon\'s Gloves
(20592,8724,1,1,200,5076),-- Life Stone: level 49
(20592,8734,1,1,200,436),-- Mid-Grade Life Stone: level 49
(20592,8744,1,1,200,145),-- High-Grade Life Stone: level 49
(20593,1912,1,2,-1,348570),-- Ring of Protection Gemstone
(20593,2083,1,1,-1,21493),-- Stiletto Edge
(20593,2341,1,1,-1,7523),-- Recipe: Sage\'s Staff
(20593,57,1563,3187,0,700000),-- Adena
(20593,398,1,1,1,81),-- Plated Leather
(20593,399,1,1,1,75),-- Rind Leather Armor
(20593,418,1,1,1,129),-- Plated Leather Gaiters
(20593,419,1,1,1,119),-- Rind Leather Gaiters
(20593,1966,1,1,1,6541),-- Plated Leather Fragment
(20593,1967,1,1,1,9770),-- Plated Leather Gaiters Material
(20593,1975,1,1,1,5884),-- Rind Leather Armor Design
(20593,1976,1,1,1,10893),-- Rind Leather Gaiters Material
(20593,1866,1,1,2,100000),-- Suede
(20593,1871,1,1,2,142857),-- Charcoal
(20593,1877,1,1,2,5319),-- Adamantite Nugget
(20593,4040,1,1,2,2532),-- Mold Lubricant
(20593,4041,1,1,2,760),-- Mold Hardener
(20593,2340,1,1,2,253),-- Recipe: Pa\'agrian Hammer
(20593,3006,1,1,2,6579),-- Recipe: Demon\'s Gloves
(20593,8724,1,1,200,3802),-- Life Stone: level 49
(20593,8734,1,1,200,1267),-- Mid-Grade Life Stone: level 49
(20593,8744,1,1,200,507),-- High-Grade Life Stone: level 49
(20594,1871,1,5,-1,737538),-- Charcoal
(20594,1895,1,2,-1,632176),-- Metallic Fiber
(20594,1978,1,1,-1,141834),-- Tower Shield Fragment
(20594,57,1073,2178,0,700000),-- Adena
(20594,231,1,1,1,11),-- Grace Dagger
(20594,233,1,1,1,11),-- Dark Screamer
(20594,2106,1,1,1,1247),-- Grace Dagger Edge
(20594,2107,1,1,1,1247),-- Dark Screamer Edge
(20594,6669,1,1,2,19231),-- Deluxe Chest Key - Grade 5
(20594,1867,1,1,2,125000),-- Animal Skin
(20594,1872,1,1,2,100000),-- Animal Bone
(20594,1882,1,1,2,22727),-- Leather
(20594,1894,1,1,2,1876),-- Crafted Leather
(20594,4042,1,1,2,1876),-- Enria
(20594,2234,1,1,2,313),-- Recipe: Divine Stockings
(20594,2356,1,1,2,313),-- Recipe: Demon\'s Staff
(20594,8725,1,1,200,2188),-- Life Stone: level 52
(20594,8735,1,1,200,626),-- Mid-Grade Life Stone: level 52
(20595,1894,1,1,-1,80892),-- Crafted Leather
(20595,2096,1,1,-1,12564),-- Sword of Whispering Death Blade
(20595,2354,1,1,-1,3761),-- Recipe: Deadman\'s Staff
(20595,57,1117,2270,0,700000),-- Adena
(20595,231,1,1,1,12),-- Grace Dagger
(20595,233,1,1,1,12),-- Dark Screamer
(20595,2106,1,1,1,1299),-- Grace Dagger Edge
(20595,2107,1,1,1,1299),-- Dark Screamer Edge
(20595,1865,1,1,2,111111),-- Varnish
(20595,1870,1,1,2,100000),-- Coal
(20595,1874,1,1,2,4762),-- Oriharukon Ore
(20595,3014,1,1,2,2519),-- Recipe: Full Plate Boots
(20595,4039,1,1,2,3356),-- Mold Glue
(20595,4043,1,1,2,4484),-- Asofe
(20595,4125,1,1,2,4484),-- Recipe: Maestro Mold
(20595,8725,1,1,200,1681),-- Life Stone: level 52
(20595,8735,1,1,200,1120),-- Mid-Grade Life Stone: level 52
(20595,8745,1,1,200,171),-- High-Grade Life Stone: level 52
(20596,1539,1,3,-1,543597),-- Greater Healing Potion
(20596,1875,1,2,-1,163079),-- Stone of Purity
(20596,4609,1,1,-1,5436),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20596,57,1189,2403,0,700000),-- Adena
(20596,162,1,1,1,3),-- War Axe
(20596,174,1,1,1,3),-- Nirvana Axe
(20596,196,1,1,1,3),-- Stick of Eternity
(20596,197,1,1,1,3),-- Paradia Staff
(20596,199,1,1,1,3),-- Pa\'agrian Hammer
(20596,200,1,1,1,3),-- Sage\'s Staff
(20596,201,1,1,1,3),-- Club of Nature
(20596,202,1,1,1,3),-- Mace of The Underworld
(20596,2097,1,1,1,339),-- War Axe Blade
(20596,2098,1,1,1,339),-- Nirvana Axe Blade
(20596,2099,1,1,1,357),-- Stick of Eternity Shaft
(20596,2100,1,1,1,357),-- Paradia Staff Head
(20596,2102,1,1,1,344),-- Pa\'agrian Hammer Head
(20596,2103,1,1,1,357),-- Sage\'s Staff Head
(20596,2104,1,1,1,357),-- Club of Nature Head
(20596,2105,1,1,1,339),-- Mace of The Underworld Head
(20596,1868,1,1,2,166667),-- Thread
(20596,1873,1,1,2,30303),-- Silver Nugget
(20596,1876,1,1,2,18182),-- Mithril Ore
(20596,1895,1,1,2,22222),-- Metallic Fiber
(20596,3017,1,1,2,4545),-- Recipe: Divine Gloves
(20596,4042,1,1,2,606),-- Enria
(20596,4608,1,1,2,303),-- Greater Dye of Intelligence <Int+3 Wit-3>
(20596,5155,1,1,2,76),-- Recipe: Soulshot (B) Compressed Package (100%)
(20596,6669,1,1,2,19608),-- Deluxe Chest Key - Grade 5
(20596,8725,1,1,200,2653),-- Life Stone: level 52
(20596,8735,1,1,200,531),-- Mid-Grade Life Stone: level 52
(20596,8745,1,1,200,76),-- High-Grade Life Stone: level 52
(20597,1918,1,1,-1,176400),-- Nassen\'s Earring Gemstone
(20597,2113,1,1,-1,19724),-- Scorpion Blade
(20597,3034,1,1,-1,8146),-- Recipe: Spiritshot B
(20597,57,1974,4008,0,700000),-- Adena
(20597,442,1,1,1,71),-- Divine Tunic
(20597,473,1,1,1,113),-- Divine Stockings
(20597,1988,1,1,1,7225),-- Divine Tunic Fabric
(20597,1989,1,1,1,11333),-- Divine Stocking Pattern
(20597,1865,1,1,2,142857),-- Varnish
(20597,1870,1,1,2,142857),-- Coal
(20597,1874,1,1,2,9259),-- Oriharukon Ore
(20597,4039,1,1,2,4878),-- Mold Glue
(20597,4043,1,1,2,4630),-- Asofe
(20597,4133,1,1,2,1681),-- Recipe: Zubei\'s Breastplate
(20597,4941,1,1,2,588),-- Recipe: Zubei\'s Gaiters  (60%)
(20597,4959,1,1,2,2439),-- Recipe: Avadon Boots (60%)
(20597,8726,1,1,200,3534),-- Life Stone: level 55
(20597,8736,1,1,200,588),-- Mid-Grade Life Stone: level 55
(20597,8746,1,1,200,84),-- High-Grade Life Stone: level 55
(20598,1806,1,1,-1,7537),-- Recipe: Soulshot: B-Grade
(20598,2120,1,1,-1,18102),-- Great Pata Blade
(20598,4180,1,2,-1,418716),-- Recipe: Silver Arrow
(20598,57,1822,3711,0,700000),-- Adena
(20598,442,1,1,1,66),-- Divine Tunic
(20598,473,1,1,1,104),-- Divine Stockings
(20598,1988,1,1,1,6685),-- Divine Tunic Fabric
(20598,1989,1,1,1,10486),-- Divine Stocking Pattern
(20598,1864,1,3,2,125000),-- Stem
(20598,1869,1,1,2,125000),-- Iron Ore
(20598,1878,1,1,2,58824),-- Braided Hemp
(20598,1884,1,1,2,83333),-- Cord
(20598,4039,1,1,2,5076),-- Mold Glue
(20598,4178,1,1,2,3115),-- Recipe: Zubei\'s Shield (100%)
(20598,5160,1,1,2,519),-- Recipe: Spiritshot (B) Compressed Package (100%)
(20598,8726,1,1,200,3322),-- Life Stone: level 55
(20598,8736,1,1,200,1142),-- Mid-Grade Life Stone: level 55
(20598,8746,1,1,200,104),-- High-Grade Life Stone: level 55
(20599,2120,1,1,-1,18800),-- Great Pata Blade
(20599,4146,1,1,-1,25088),-- Recipe: Tunic of Zubei
(20599,4590,1,1,-1,13046),-- Greater Dye of Strength <Str+2 Dex-2>
(20599,57,1893,3855,0,700000),-- Adena
(20599,2436,1,1,1,76),-- Theca Leather Boots
(20599,2437,1,1,1,58),-- Drake Leather Boots
(20599,2438,1,1,1,58),-- Full Plate Boots
(20599,2460,1,1,1,76),-- Theca Leather Gloves
(20599,2461,1,1,1,58),-- Drake Leather Gloves
(20599,2462,1,1,1,58),-- Full Plate Gauntlets
(20599,2463,1,1,1,58),-- Divine Gloves
(20599,2954,1,1,1,6534),-- Theca Leather Boots Texture
(20599,2955,1,1,1,6534),-- Theca Leather Gloves Texture
(20599,2960,1,1,1,5622),-- Drake Leather Boots Design
(20599,2961,1,1,1,5622),-- Full Plate Boots Part
(20599,2962,1,1,1,5622),-- Drake Leather Gloves Design
(20599,2963,1,1,1,5622),-- Full Plate Gauntlets Part
(20599,2964,1,1,1,5622),-- Divine Gloves Pattern
(20599,1864,1,3,2,142857),-- Stem
(20599,1869,1,1,2,142857),-- Iron Ore
(20599,1878,1,1,2,55556),-- Braided Hemp
(20599,1884,1,1,2,83333),-- Cord
(20599,4039,1,1,2,3731),-- Mold Glue
(20599,4178,1,1,2,2755),-- Recipe: Zubei\'s Shield (100%)
(20599,4184,1,1,2,98),-- Recipe: Sprite\'s Staff
(20599,5160,1,1,2,295),-- Recipe: Spiritshot (B) Compressed Package (100%)
(20599,8726,1,1,200,3534),-- Life Stone: level 55
(20599,8736,1,1,200,393),-- Mid-Grade Life Stone: level 55
(20599,8756,1,1,200,98),-- Top-Grade Life Stone: level 55
(20600,1825,1,1,-1,63980),-- Recipe: Oriharukon
(20600,1873,1,1,-1,358285),-- Silver Nugget
(20600,1945,1,1,-1,71145),-- Mithril Stocking Design
(20600,57,451,900,0,700000),-- Adena
(20600,629,1,1,1,197),-- Kite Shield
(20600,1947,1,1,1,11112),-- Kite Shield Fragment
(20600,2411,1,1,1,138),-- Brigandine Helmet
(20600,2493,1,1,1,197),-- Brigandine Shield
(20600,2922,1,1,1,4202),-- Brigandine Helmet Design
(20600,2929,1,1,1,11112),-- Brigandine Shield Fragment
(20600,1868,1,1,2,200000),-- Thread
(20600,1873,1,1,2,40000),-- Silver Nugget
(20600,1889,1,1,2,6250),-- Compound Braid
(20600,2286,1,1,2,293),-- Recipe: Bonebreaker
(20600,5231,1,1,2,6623),-- Recipe: Metal Hardener (100%)
(20600,5549,1,1,2,9434),-- Metallic Thread
(20601,1880,1,1,-1,79830),-- Steel
(20601,1905,1,2,-1,77656),-- Omen Beast\'s Eye Earring Gemstone
(20601,2304,1,1,-1,3486),-- Recipe: Heavy Doom Hammer
(20601,57,394,788,0,700000),-- Adena
(20601,353,1,1,1,52),-- Half Plate Armor
(20601,380,1,1,1,83),-- Plate Gaiters
(20601,1950,1,1,1,3253),-- Half Plate Temper
(20601,1951,1,1,1,5059),-- Plate Gaiters Part
(20601,1864,1,1,2,166667),-- Stem
(20601,1869,1,1,2,83333),-- Iron Ore
(20601,1880,1,1,2,8696),-- Steel
(20601,2199,1,1,2,2404),-- Recipe: Karmian Stockings
(20601,5269,1,1,2,218),-- Recipe: Greater Soulshot (C) Compressed Package(100%)
(20601,5550,1,1,2,327),-- Durable Metal Plate
(20602,1873,1,1,-1,431425),-- Silver Nugget
(20602,1889,1,1,-1,71904),-- Compound Braid
(20602,2999,1,1,-1,72387),-- Recipe: Plated Leather Gloves
(20602,57,529,1069,0,700000),-- Adena
(20602,280,1,1,1,31),-- Light Crossbow
(20602,2054,1,1,1,2305),-- Light Crossbow Shaft
(20602,1868,1,1,2,166667),-- Thread
(20602,1873,1,1,2,38462),-- Silver Nugget
(20602,1876,1,1,2,13514),-- Mithril Ore
(20602,1895,1,1,2,22222),-- Metallic Fiber
(20602,2205,1,1,2,1667),-- Recipe: Chain Gaiters
(20602,2300,1,1,2,208),-- Recipe: Scythe
(20602,2311,1,1,2,1042),-- Recipe: Chakram
(20602,5154,1,1,2,625),-- Recipe: Soulshot (C) Compressed Package (100%)
(20603,1872,1,3,-1,590512),-- Animal Bone
(20603,1952,1,1,-1,28372),-- Salamander Skin Mail Texture
(20603,2311,1,1,-1,3868),-- Recipe: Chakram
(20603,57,429,871,0,700000),-- Adena
(20603,61,1,1,1,76),-- Mithril Gloves
(20603,607,1,1,1,76),-- Ogre Power Gauntlets
(20603,1125,1,1,1,76),-- Assault Boots
(20603,1949,1,1,1,4137),-- Mithril Gloves Design
(20603,1957,1,1,1,4137),-- Assault Boots Part
(20603,1963,1,1,1,4137),-- Ogre Power Gauntlets Part
(20603,2427,1,1,1,76),-- Salamander Skin Boots
(20603,2428,1,1,1,76),-- Plate Boots
(20603,2451,1,1,1,76),-- Sage\'s Worn Gloves
(20603,2931,1,1,1,4137),-- Salamander Skin Boot Lining
(20603,2932,1,1,1,4137),-- Plate Boots Design
(20603,2933,1,1,1,4137),-- Sage\'s Worn Gloves Lining
(20603,1867,1,1,2,100000),-- Animal Skin
(20603,1872,1,1,2,100000),-- Animal Bone
(20603,1875,1,1,2,5000),-- Stone of Purity
(20603,1881,1,1,2,9346),-- Coarse Bone Powder
(20603,2991,1,1,2,5618),-- Recipe: Karmian Boots
(20603,5279,1,1,2,212),-- Recipe: Greater Blessed Spiritshot (C) Compressed Package(100%)
(20603,6668,1,1,2,28571),-- Deluxe Chest Key - Grade 4
(20604,2322,1,1,-1,3768),-- Recipe: Soulfire Dirk
(20604,2323,1,1,-1,3768),-- Recipe: Elemental Bow
(20604,4553,1,1,-1,4325),-- Greater Dye of Strength <Str+1 Con-1>
(20604,57,522,1068,0,700000),-- Adena
(20604,852,1,1,1,110),-- Moonstone Earring
(20604,883,1,1,1,166),-- Aquastone Ring
(20604,915,1,1,1,83),-- Aquastone Necklace
(20604,1908,1,1,1,8066),-- Moonstone Earring Wire
(20604,1909,1,1,1,11502),-- Aquastone Ring Wire
(20604,1910,1,1,1,6414),-- Aquastone Necklace Chain
(20604,1867,1,1,2,111111),-- Animal Skin
(20604,1872,1,1,2,111111),-- Animal Bone
(20604,1875,1,1,2,6579),-- Stone of Purity
(20604,1881,1,1,2,10753),-- Coarse Bone Powder
(20604,2991,1,1,2,7634),-- Recipe: Karmian Boots
(20604,5279,1,1,2,525),-- Recipe: Greater Blessed Spiritshot (C) Compressed Package(100%)
(20604,6668,1,1,2,28571),-- Deluxe Chest Key - Grade 4
(20605,1867,1,3,-1,653559),-- Animal Skin
(20605,1908,1,1,-1,92835),-- Moonstone Earring Wire
(20605,2059,1,1,-1,7282),-- Flamberge Blade
(20605,57,475,964,0,700000),-- Adena
(20605,439,1,1,1,62),-- Karmian Tunic
(20605,471,1,1,1,98),-- Karmian Stockings
(20605,1954,1,1,1,6159),-- Karmian Stocking Design
(20605,1962,1,1,1,3707),-- Karmian Tunic Pattern
(20605,1868,1,1,2,100000),-- Thread
(20605,1873,1,1,2,20408),-- Silver Nugget
(20605,1889,1,1,2,3401),-- Compound Braid
(20605,2214,1,1,2,200),-- Recipe: Dwarven Chain Gaiters
(20605,3003,1,1,2,4785),-- Recipe: Gloves of Seal
(20605,4044,1,1,2,2198),-- Thons
(20605,5549,1,1,2,8621),-- Metallic Thread
(20605,8723,1,1,200,2793),-- Life Stone: level 46
(20605,8733,1,1,200,599),-- Mid-Grade Life Stone: level 46
(20606,1932,1,1,-1,104301),-- Bone Helmet Design
(20606,2152,1,1,-1,115686),-- Recipe: Enchanted Necklace
(20606,2260,1,1,-1,10203),-- Recipe: Mace of Judgment
(20606,57,340,656,0,700000),-- Adena
(20606,220,1,1,1,180),-- Crafted Dagger
(20606,1864,1,1,2,200000),-- Stem
(20606,1869,1,1,2,90909),-- Iron Ore
(20606,1880,1,1,2,8000),-- Steel
(20606,2262,1,1,2,1146),-- Recipe: Elven Bow
(20606,5550,1,1,2,2004),-- Durable Metal Plate
(20607,626,1,1,-1,6308),-- Bronze Shield
(20607,1899,1,1,-1,107923),-- Necklace of Devotion Chain
(20607,2020,1,1,-1,12091),-- Temptation of Abyss Piece
(20607,57,356,696,0,700000),-- Adena
(20607,101,1,1,1,38),-- Scroll of Wisdom
(20607,312,1,1,1,38),-- Branch of Life
(20607,313,1,1,1,38),-- Temptation of Abyss
(20607,314,1,1,1,38),-- Proof of Revenge
(20607,2020,1,1,1,1358),-- Temptation of Abyss Piece
(20607,1865,1,1,2,125000),-- Varnish
(20607,1870,1,1,2,111111),-- Coal
(20607,1874,1,1,2,5882),-- Oriharukon Ore
(20607,2262,1,1,2,1505),-- Recipe: Elven Bow
(20608,1874,1,1,-1,46394),-- Oriharukon Ore
(20608,2022,1,1,-1,10840),-- Mace of Judgment Head
(20608,2153,1,1,-1,108228),-- Recipe: Tiger\'s Eye Earring
(20608,57,354,700,0,700000),-- Adena
(20608,315,1,1,1,118),-- Divine Tome
(20608,1867,1,1,2,76923),-- Animal Skin
(20608,1872,1,1,2,100000),-- Animal Bone
(20608,1875,1,1,2,4464),-- Stone of Purity
(20608,1881,1,1,2,11236),-- Coarse Bone Powder
(20608,2189,1,1,2,2426),-- Recipe: Mithril Tunic
(20608,2265,1,1,2,562),-- Recipe: Spiked Club
(20608,2274,1,1,2,389),-- Recipe: Skull Breaker
(20609,433,1,1,-1,1447),-- Elven Tunic
(20609,1900,1,1,-1,79072),-- Enchanted Necklace Chain
(20609,2265,1,1,-1,7474),-- Recipe: Spiked Club
(20609,57,368,731,0,700000),-- Adena
(20609,259,1,1,1,123),-- Single-Edged Jamadhr
(20609,956,1,1,2,2195),-- Scroll: Enchant Armor (Grade D)
(20609,1866,1,1,2,66667),-- Suede
(20609,1871,1,1,2,83333),-- Charcoal
(20609,1885,1,1,2,12658),-- High Grade Suede
(20609,5220,1,1,2,2538),-- Metal Hardener
(20609,6667,1,1,2,40000),-- Deluxe Chest Key - Grade 3
(20610,1864,1,3,-1,728584),-- Stem
(20610,2025,1,1,-1,10091),-- Dwarven Trident Edge
(20610,2267,1,1,-1,7534),-- Recipe: Gastraphetes
(20610,57,371,736,0,700000),-- Adena
(20610,292,1,1,1,124),-- Pike
(20610,1341,20,60,2,90909),-- Bone Arrow
(20610,1864,1,1,2,111111),-- Stem
(20610,1865,1,1,2,66667),-- Varnish
(20610,1866,1,1,2,38462),-- Suede
(20610,1868,1,1,2,111111),-- Thread
(20610,1873,1,1,2,24390),-- Silver Nugget
(20610,2183,1,1,2,3521),-- Recipe: Mithril Banded Mail
(20610,2270,1,1,2,356),-- Recipe: Goat Head Staff
(20611,467,1,1,-1,2468),-- Dark Stockings
(20611,1884,1,1,-1,474660),-- Cord
(20611,2153,1,1,-1,119957),-- Recipe: Tiger\'s Eye Earring
(20611,57,390,782,0,700000),-- Adena
(20611,86,1,1,1,14),-- Tomahawk
(20611,179,1,1,1,14),-- Mace of Prayer
(20611,180,1,1,1,14),-- Mace of Judgment
(20611,181,1,1,1,14),-- Mace of Miracle
(20611,182,1,1,1,14),-- Doom Hammer
(20611,183,1,1,1,14),-- Mystic Staff
(20611,184,1,1,1,14),-- Conjuror\'s Staff
(20611,185,1,1,1,14),-- Staff of Mana
(20611,2022,1,1,1,657),-- Mace of Judgment Head
(20611,2023,1,1,1,523),-- Conjuror\'s Staff Head
(20611,1829,1,1,2,38462),-- Scroll of Escape: Clan Hall
(20611,1834,1,1,2,100000),-- Emergency Dressing
(20611,1864,1,1,2,166667),-- Stem
(20611,1869,1,1,2,62500),-- Iron Ore
(20611,1878,1,1,2,27027),-- Braided Hemp
(20611,1884,1,1,2,38462),-- Cord
(20611,2193,1,1,2,3906),-- Recipe: Boots of Power
(20612,349,1,1,-1,961),-- Compound Scale Mail
(20612,1869,2,3,-1,879358),-- Iron Ore
(20612,1885,1,1,-1,73280),-- High Grade Suede
(20612,57,447,892,0,700000),-- Adena
(20612,436,1,1,1,308),-- Tunic of Knowledge
(20612,469,1,1,1,492),-- Stockings of Knowledge
(20612,1832,1,1,2,76923),-- Greater Antidote
(20612,1866,1,1,2,45455),-- Suede
(20612,1871,1,1,2,90909),-- Charcoal
(20612,1885,1,1,2,4219),-- High Grade Suede
(20612,2188,1,1,2,4219),-- Recipe: Manticore Skin Gaiters
(20612,2274,1,1,2,1408),-- Recipe: Skull Breaker
(20612,5220,1,1,2,1408),-- Metal Hardener
(20613,40,1,1,-1,6288),-- Leather Boots
(20613,1899,1,1,-1,102672),-- Necklace of Devotion Chain
(20613,1934,1,1,-1,85895),-- Crafted Leather Gloves Lining
(20613,57,241,656,0,700000),-- Adena
(20613,432,1,1,1,589),-- Cursed Tunic
(20613,465,1,1,1,944),-- Cursed Stockings
(20613,1831,1,3,2,58824),-- Antidote
(20613,1864,1,1,2,142857),-- Stem
(20613,1865,1,1,2,58824),-- Varnish
(20613,1866,1,1,2,47619),-- Suede
(20613,1868,1,1,2,100000),-- Thread
(20613,1873,1,1,2,37037),-- Silver Nugget
(20613,2263,1,1,2,2618),-- Recipe: Dwarven Trident
(20614,1869,1,1,-1,645491),-- Iron Ore
(20614,1899,1,1,-1,100858),-- Necklace of Devotion Chain
(20614,2152,1,1,-1,113643),-- Recipe: Enchanted Necklace
(20614,57,340,6492,0,700000),-- Adena
(20614,220,1,1,1,177),-- Crafted Dagger
(20614,1834,1,1,2,111111),-- Emergency Dressing
(20614,1865,1,1,2,100000),-- Varnish
(20614,1870,1,1,2,100000),-- Coal
(20614,1874,1,1,2,5155),-- Oriharukon Ore
(20614,2263,1,1,2,1126),-- Recipe: Dwarven Trident
(20615,392,1,1,-1,1491),-- Lion Skin Shirt
(20615,1900,1,2,-1,81472),-- Enchanted Necklace Chain
(20615,2185,1,1,-1,146585),-- Recipe: Iron Boots
(20615,57,379,750,0,700000),-- Adena
(20615,46,1,1,1,814),-- Bronze Helmet
(20615,627,1,1,1,1163),-- Aspis
(20615,1834,1,1,2,83333),-- Emergency Dressing
(20615,1875,1,1,2,4202),-- Stone of Purity
(20615,1879,1,1,2,7194),-- Cokes
(20615,1885,1,1,2,7813),-- High Grade Suede
(20615,2266,1,1,2,2398),-- Recipe: Shilen Knife
(20615,5549,1,1,2,4808),-- Metallic Thread
(20615,5550,1,1,2,1199),-- Durable Metal Plate
(20616,1874,1,1,-1,51402),-- Oriharukon Ore
(20616,1937,1,1,-1,66239),-- Scale Gaiters Fragment
(20616,2153,1,1,-1,119911),-- Recipe: Tiger\'s Eye Earring
(20616,57,390,781,0,700000),-- Adena
(20616,433,1,1,1,106),-- Elven Tunic
(20616,434,1,1,1,106),-- White Tunic
(20616,435,1,1,1,106),-- Mystic\'s Tunic
(20616,466,1,1,1,169),-- Elven Stockings
(20616,467,1,1,1,169),-- Dark Stockings
(20616,468,1,1,1,169),-- Mystic\'s Stockings
(20616,1933,1,1,1,6197),-- Dark Stocking Fabric
(20616,1936,1,1,1,4310),-- White Tunic Pattern
(20616,737,1,1,2,1590),-- Scroll of Resurrection
(20616,1867,1,1,2,83333),-- Animal Skin
(20616,1872,1,1,2,100000),-- Animal Bone
(20616,1875,1,1,2,1193),-- Stone of Purity
(20616,1881,1,1,2,21277),-- Coarse Bone Powder
(20616,2274,1,1,2,398),-- Recipe: Skull Breaker
(20616,2976,1,1,2,6757),-- Recipe: Manticore Skin Boots
(20616,6667,1,1,2,52632),-- Deluxe Chest Key - Grade 3
(20617,435,1,1,-1,1473),-- Mystic\'s Tunic
(20617,1875,1,1,-1,49053),-- Stone of Purity
(20617,1933,1,1,-1,86564),-- Dark Stocking Fabric
(20617,57,374,743,0,700000),-- Adena
(20617,392,1,1,1,403),-- Lion Skin Shirt
(20617,414,1,1,1,644),-- Lion Skin Gaiters
(20617,1867,1,1,2,142857),-- Animal Skin
(20617,1872,1,1,2,125000),-- Animal Bone
(20617,1875,1,1,2,3226),-- Stone of Purity
(20617,1881,1,1,2,6452),-- Coarse Bone Powder
(20617,2184,1,1,2,17857),-- Recipe: Mithril Banded Gaiters
(20618,955,1,1,-1,3251),-- Scroll: Enchant Weapon (Grade D)
(20618,1885,1,1,-1,67733),-- High Grade Suede
(20618,1935,1,1,-1,45407),-- Scale Mail Fragment
(20618,57,412,823,0,700000),-- Adena
(20618,315,1,1,1,138),-- Divine Tome
(20618,1875,1,1,2,7813),-- Stone of Purity
(20618,1879,1,1,2,19608),-- Cokes
(20618,1885,1,1,2,4902),-- High Grade Suede
(20618,2153,1,1,2,8850),-- Recipe: Tiger\'s Eye Earring
(20618,5549,1,1,2,11765),-- Metallic Thread
(20618,5550,1,1,2,9798),-- Durable Metal Plate
(20619,1867,1,3,-1,556329),-- Animal Skin
(20619,2270,1,1,-1,5961),-- Recipe: Goat Head Staff
(20619,2981,1,2,-1,111713),-- Recipe: Elven Mithril Gloves
(20619,57,324,841,0,700000),-- Adena
(20619,157,1,1,1,31),-- Spiked Club
(20619,186,1,1,1,31),-- Staff of Magic
(20619,2027,1,1,1,1938),-- Spiked Club Head
(20619,1867,1,1,2,125000),-- Animal Skin
(20619,1872,1,1,2,142857),-- Animal Bone
(20619,1875,1,1,2,4386),-- Stone of Purity
(20619,1881,1,1,2,10989),-- Coarse Bone Powder
(20619,2154,1,1,2,10989),-- Recipe: Elven Earring
(20619,2278,1,1,2,528),-- Recipe: Strengthened Long Bow
(20620,948,1,1,-1,10769),-- Scroll: Enchant Armor (Grade B)
(20620,4039,1,2,-1,143581),-- Mold Glue
(20620,4051,1,1,-1,167052),-- Adamantite Necklace Chain
(20620,57,2090,4237,0,700000),-- Adena
(20620,78,1,1,1,5),-- Great Sword
(20620,142,1,1,1,5),-- Keshanberk
(20620,148,1,1,1,5),-- Sword of Valhalla
(20620,4104,1,1,1,1214),-- Great Sword Blade
(20620,4107,1,1,1,1195),-- Keshanberk Blade
(20620,4108,1,1,1,1195),-- Sword of Valhalla Blade
(20620,1894,1,1,2,3003),-- Crafted Leather
(20620,4039,1,1,2,3175),-- Mold Glue
(20620,4041,1,1,2,1412),-- Mold Hardener
(20620,4044,1,1,2,5464),-- Thons
(20620,4402,1,1,2,364),-- Blue Dimensional Stone
(20620,4614,1,1,2,251),-- Greater Dye of Strength <Str+4 Dex-4>
(20620,4999,1,1,2,1590),-- Recipe: Doom Boots (60%)
(20620,8728,1,1,200,3003),-- Life Stone: level 61
(20620,8738,1,1,200,383),-- Mid-Grade Life Stone: level 61
(20620,8748,1,1,200,153),-- High-Grade Life Stone: level 61
(20621,4052,1,2,-1,164789),-- Earring of Black Ore Piece
(20621,4199,1,1,-1,3553),-- Recipe: Bow of Peril
(20621,5230,1,1,-1,6206),-- Recipe: Mithril Arrow (100%)
(20621,57,2260,4576,0,700000),-- Adena
(20621,2391,1,1,1,20),-- Blue Wolf Leather Armor
(20621,2392,1,1,1,20),-- Leather Armor of Doom
(20621,4082,1,1,1,3884),-- Blue Wolf Leather Armor Texture
(20621,4083,1,1,1,3884),-- Leather Armor of Doom Design
(20621,1868,1,1,2,250000),-- Thread
(20621,1873,1,1,2,52632),-- Silver Nugget
(20621,1876,1,1,2,270272),-- Mithril Ore
(20621,1895,1,1,2,34483),-- Metallic Fiber
(20621,4042,1,1,2,1996),-- Enria
(20621,4623,1,1,2,271),-- Greater Dye of Wit <Wit+4 Int-4>
(20621,5276,1,1,2,262),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20621,8729,1,1,200,1567),-- Life Stone: level 64
(20621,8739,1,1,200,1140),-- Mid-Grade Life Stone: level 64
(20621,8749,1,1,200,143),-- High-Grade Life Stone: level 64
(20622,4042,1,1,-1,83042),-- Enria
(20622,4044,1,2,-1,166084),-- Thons
(20622,4053,1,2,-1,213567),-- Ring of Black Ore Gemstone
(20622,57,2419,4898,0,700000),-- Adena
(20622,864,1,1,1,54),-- Earring of Black Ore
(20622,895,1,1,1,80),-- Ring of Black Ore
(20622,926,1,1,1,40),-- Necklace of Black Ore
(20622,4052,1,1,1,8684),-- Earring of Black Ore Piece
(20622,4053,1,1,1,10513),-- Ring of Black Ore Gemstone
(20622,4054,1,1,1,4831),-- Necklace of Black Ore Beads
(20622,1867,1,1,2,200000),-- Animal Skin
(20622,1872,1,1,2,200000),-- Animal Bone
(20622,1882,1,1,2,31250),-- Leather
(20622,1894,1,1,2,5376),-- Crafted Leather
(20622,4042,1,1,2,2227),-- Enria
(20622,4401,1,1,2,413),-- Green Dimensional Stone
(20622,4988,1,1,2,867),-- Recipe: Blue Wolf Stockings (60%)
(20622,6670,1,1,2,24390),-- Deluxe Chest Key - Grade 6
(20622,8729,1,1,200,2105),-- Life Stone: level 64
(20622,8739,1,1,200,495),-- Mid-Grade Life Stone: level 64
(20622,8749,1,1,200,83),-- High-Grade Life Stone: level 64
(20623,4039,1,1,-1,222007),-- Mold Glue
(20623,4040,1,1,-1,133204),-- Mold Lubricant
(20623,5230,1,1,-1,8880),-- Recipe: Mithril Arrow (100%)
(20623,57,3236,6544,0,700000),-- Adena
(20623,110,1,1,1,78),-- Doom Shield
(20623,2416,1,1,1,55),-- Blue Wolf Helmet
(20623,2417,1,1,1,55),-- Doom Helmet
(20623,4077,1,1,1,14749),-- Doom Shield Fragment
(20623,4088,1,1,1,12392),-- Blue Wolf Helmet Design
(20623,4089,1,1,1,12392),-- Doom Helmet Pattern
(20623,1868,1,3,2,250000),-- Thread
(20623,1873,1,1,2,83333),-- Silver Nugget
(20623,1889,1,1,2,16667),-- Compound Braid
(20623,4044,1,1,2,7194),-- Thons
(20623,4177,1,1,2,4098),-- Recipe: Zubei\'s Gauntlets
(20623,5332,1,1,2,409),-- Recipe: Sealed Dark Crystal Leather Armor(60%)
(20623,5549,1,1,2,22727),-- Metallic Thread
(20623,8729,1,1,200,2564),-- Life Stone: level 64
(20623,8739,1,1,200,341),-- Mid-Grade Life Stone: level 64
(20623,8749,1,1,200,85),-- High-Grade Life Stone: level 64
(20624,1877,1,1,-1,271744),-- Adamantite Nugget
(20624,4052,1,1,-1,240524),-- Earring of Black Ore Piece
(20624,4054,1,1,-1,133811),-- Necklace of Black Ore Beads
(20624,57,3295,6681,0,700000),-- Adena
(20624,79,1,1,1,16),-- Sword of Damascus
(20624,7889,1,1,1,16),-- Wizard\'s Tear
(20624,7883,1,1,1,16),-- Guardian Sword
(20624,4114,1,1,1,4392),-- Sword of Damascus Blade
(20624,8336,1,1,1,1470),-- Blade of the Wizard\'s Tear
(20624,8330,1,1,1,1430),-- Guardian Sword Blade
(20624,1868,1,3,2,200000),-- Thread
(20624,1873,1,1,2,76923),-- Silver Nugget
(20624,1876,1,1,2,40000),-- Mithril Ore
(20624,1895,1,1,2,52632),-- Metallic Fiber
(20624,4042,1,1,2,3311),-- Enria
(20624,4154,1,1,2,2183),-- Recipe: Doom Shield
(20624,5340,1,1,2,989),-- Recipe: Sealed Dark Crystalline Leggings(60%)
(20624,4403,1,1,2,273),-- Red Dimensional Stone
(20624,5281,1,1,2,273),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(20624,8729,1,1,200,2288),-- Life Stone: level 64
(20624,8739,1,1,200,375),-- Mid-Grade Life Stone: level 64
(20624,8749,1,1,200,34),-- High-Grade Life Stone: level 64
(20624,8759,1,1,200,34),-- Top-Grade Life Stone: level 64
(20625,4043,1,1,-1,226724),-- Asofe
(20625,4044,1,1,-1,226724),-- Thons
(20625,5230,1,1,-1,9069),-- Recipe: Mithril Arrow (100%)
(20625,57,3298,6689,0,700000),-- Adena
(20625,268,1,1,1,16),-- Bellion Cestus
(20625,4120,1,1,1,4390),-- Bellion Cestus Edge
(20625,8336,1,1,1,52),-- Blade of the Wizard\'s Tear
(20625,8330,1,1,1,104),-- Guardian Sword Blade
(20625,1894,1,1,2,8772),-- Crafted Leather
(20625,4039,1,1,2,9434),-- Mold Glue
(20625,4041,1,1,2,2070),-- Mold Hardener
(20625,4044,1,1,2,7299),-- Thons
(20625,4192,1,1,2,59),-- Recipe: Sword of Damascus
(20625,8729,1,1,200,2725),-- Life Stone: level 64
(20625,8739,1,1,200,296),-- Mid-Grade Life Stone: level 64
(20625,8749,1,1,200,118),-- High-Grade Life Stone: level 64
(20625,8759,1,1,200,59),-- Top-Grade Life Stone: level 64
(20625,5276,1,1,2,118); -- Recipe: Greater Spiritshot (A) Compressed Package(100%)

INSERT INTO `droplist` VALUES 
(20626,1875,1,1,-1,560452),-- Stone of Purity
(20626,4053,1,1,-1,360342),-- Ring of Black Ore Gemstone
(20626,5230,1,1,-1,11209),-- Recipe: Mithril Arrow (100%)
(20626,57,4056,8290,0,700000),-- Adena
(20626,358,1,1,1,26),-- Blue Wolf Breastplate
(20626,2380,1,1,1,42),-- Blue Wolf Gaiters
(20626,2381,1,1,1,18),-- Doom Plate Armor
(20626,4078,1,1,1,6046),-- Blue Wolf Breastplate Part
(20626,4080,1,1,1,8514),-- Blue Wolf Gaiters Material
(20626,4081,1,1,1,4080),-- Doom Plate Armor Temper
(20626,1867,1,3,2,111111),-- Animal Skin
(20626,1872,1,3,2,166667),-- Animal Bone
(20626,1875,1,1,2,16393),-- Stone of Purity
(20626,1881,1,1,2,32258),-- Coarse Bone Powder
(20626,4041,1,1,2,1862),-- Mold Hardener
(20626,5334,1,1,2,890),-- Recipe: Sealed Tallum Leather Armor(60%)
(20626,4402,1,1,2,324),-- Blue Dimensional Stone
(20626,4615,1,1,2,81),-- Greater Dye of Constitution <Con+4 Str-4>
(20626,8730,1,1,200,2994),-- Life Stone: level 67
(20626,8740,1,1,200,405),-- Mid-Grade Life Stone: level 67
(20626,8760,1,1,200,81),-- Top-Grade Life Stone: level 67
(20627,1877,1,1,-1,337512),-- Adamantite Nugget
(20627,5478,1,2,-1,70954),-- Sealed Dark Crystal Leather Armor Pattern
(20627,57,4085,8305,0,700000),-- Adena
(20627,5290,1,1,1,54),-- Sealed Dark Crystal Gloves
(20627,5291,1,1,1,54),-- Sealed Dark Crystal Boots
(20627,5295,1,1,1,54),-- Sealed Tallum Gloves
(20627,5296,1,1,1,54),-- Sealed Tallum Boots
(20627,5496,1,1,1,7305),-- Sealed Dark Crystal Boots Lining
(20627,5497,1,1,1,7305),-- Sealed Tallum Boots Lining
(20627,5508,1,1,1,7305),-- Sealed Dark Crystal Gloves Design
(20627,5509,1,1,1,7305),-- Sealed Tallum Gloves Design
(20627,1866,1,1,2,166667),-- Suede
(20627,1871,1,1,2,250000),-- Charcoal
(20627,1877,1,1,2,9709),-- Adamantite Nugget
(20627,3956,1,1,2,304),-- Recipe: Blessed Spiritshot A
(20627,4040,1,1,2,5076),-- Mold Lubricant
(20627,4041,1,1,2,1420),-- Mold Hardener
(20627,4973,1,1,2,2941),-- Recipe: Earring of Black Ore (70%)
(20627,6670,1,1,2,41667),-- Deluxe Chest Key - Grade 6
(20627,8730,1,1,200,3040),-- Life Stone: level 67
(20627,8740,1,1,200,609),-- Mid-Grade Life Stone: level 67
(20627,8750,1,1,200,203),-- High-Grade Life Stone: level 67
(20628,1807,1,1,-1,12152),-- Recipe: Soulshot: A-Grade
(20628,1895,1,5,-1,868028),-- Metallic Fiber
(20628,5479,1,1,-1,53931),-- Sealed Tallum Leather Armor Pattern
(20628,57,4416,8962,0,700000),-- Adena
(20628,5304,1,1,1,26),-- Sealed Tallum Tunic
(20628,5305,1,1,1,42),-- Sealed Tallum Stockings
(20628,5308,1,1,1,18),-- Sealed Dark Crystal Robe
(20628,5485,1,1,1,3773),-- Sealed Tallum Tunic Texture
(20628,5486,1,1,1,2655),-- Sealed Dark Crystal Robe Fabric
(20628,5489,1,1,1,5922),-- Sealed Tallum Stockings Fabric
(20628,1895,1,1,2,76923),-- Metallic Fiber
(20628,4040,1,1,2,7246),-- Mold Lubricant
(20628,4042,1,1,2,5076),-- Enria
(20628,4043,1,1,2,13333),-- Asofe
(20628,4401,1,1,2,898),-- Green Dimensional Stone
(20628,4988,1,1,2,2538),-- Recipe: Blue Wolf Stockings (60%)
(20628,8730,1,1,200,3125),-- Life Stone: level 67
(20628,8740,1,1,200,1172),-- Mid-Grade Life Stone: level 67
(20629,3035,1,1,-1,14274),-- Recipe: Spiritshot A
(20629,4077,1,1,-1,481585),-- Doom Shield Fragment
(20629,4078,1,1,-1,156378),-- Blue Wolf Breastplate Part
(20629,57,5183,10540,0,700000),-- Adena
(20629,5289,1,1,1,61),-- Sealed Dark Crystal Helmet
(20629,5292,1,1,1,87),-- Sealed Dark Crystal Shield
(20629,5294,1,1,1,61),-- Sealed Tallum Helmet
(20629,5494,1,1,1,11069),-- Sealed Dark Crystal Shield Fragment
(20629,5525,1,1,1,8147),-- Sealed Dark Crystal Helmet Design
(20629,5526,1,1,1,8147),-- Sealed Tallum Helm Design
(20629,1868,3,9,2,125000),-- Thread
(20629,1873,1,1,2,142857),-- Silver Nugget
(20629,1876,1,1,2,71429),-- Mithril Ore
(20629,1895,1,1,2,100000),-- Metallic Fiber
(20629,4042,1,1,2,4831),-- Enria
(20629,5006,1,1,2,1464),-- Recipe: Bellion Cestus (60%)
(20629,8730,1,1,200,5405),-- Life Stone: level 67
(20629,8740,1,1,200,1171),-- Mid-Grade Life Stone: level 67
(20629,8750,1,1,200,185),-- High-Grade Life Stone: level 67
(20629,8760,1,1,200,62),-- Top-Grade Life Stone: level 67
(20630,1902,1,1,-1,77024),-- Elven Earring Beads
(20630,1943,1,1,-1,60383),-- Manticore Skin Gaiters Pattern
(20630,2291,1,1,-1,4223),-- Recipe: Scallop Jamadhr
(20630,57,383,764,0,700000),-- Adena
(20630,629,1,1,1,168),-- Kite Shield
(20630,1947,1,1,1,9431),-- Kite Shield Fragment
(20630,2411,1,1,1,117),-- Brigandine Helmet
(20630,2493,1,1,1,168),-- Brigandine Shield
(20630,2922,1,1,1,2523),-- Brigandine Helmet Design
(20630,2929,1,1,1,9431),-- Brigandine Shield Fragment
(20630,1866,1,1,2,41667),-- Suede
(20630,1871,1,1,2,71429),-- Charcoal
(20630,1885,1,1,2,6757),-- High Grade Suede
(20630,2286,1,1,2,312),-- Recipe: Bonebreaker
(20630,2987,1,1,2,4219),-- Recipe: Plate Shield
(20630,5220,1,1,2,2959),-- Metal Hardener
(20630,5472,1,1,2,6757),-- Recipe: Metallic Thread(100%)
(20631,1873,1,1,-1,411784),-- Silver Nugget
(20631,2996,1,1,-1,81703),-- Recipe: Chain Gloves
(20631,2998,1,1,-1,77403),-- Recipe: Chain Shield
(20631,57,508,1017,0,700000),-- Adena
(20631,61,1,1,1,89),-- Mithril Gloves
(20631,607,1,1,1,89),-- Ogre Power Gauntlets
(20631,1125,1,1,1,89),-- Assault Boots
(20631,1949,1,1,1,4808),-- Mithril Gloves Design
(20631,1957,1,1,1,4808),-- Assault Boots Part
(20631,1963,1,1,1,4808),-- Ogre Power Gauntlets Part
(20631,2427,1,1,1,89),-- Salamander Skin Boots
(20631,2428,1,1,1,89),-- Plate Boots
(20631,2451,1,1,1,89),-- Sage\'s Worn Gloves
(20631,2931,1,1,1,4808),-- Salamander Skin Boot Lining
(20631,2932,1,1,1,4808),-- Plate Boots Design
(20631,2933,1,1,1,4808),-- Sage\'s Worn Gloves Lining
(20631,1342,10,30,2,142857),-- Fine Steel Arrow
(20631,1868,1,1,2,142857),-- Thread
(20631,1873,1,1,2,38462),-- Silver Nugget
(20631,1889,1,1,2,8065),-- Compound Braid
(20631,2203,1,1,2,18868),-- Recipe: Mithril Boots
(20631,5164,1,1,2,216),-- Recipe: Blessed Spiritshot (C) Compressed Package (100%)
(20631,5274,1,1,2,216),-- Recipe: Greater Spiritshot (C) Compressed Package(100%)
(20631,5549,1,1,2,10753),-- Metallic Thread
(20632,1873,1,2,-1,410100),-- Silver Nugget
(20632,1880,1,1,-1,102525),-- Steel
(20632,1951,1,1,-1,51754),-- Plate Gaiters Part
(20632,57,505,1015,0,700000),-- Adena
(20632,353,1,1,1,66),-- Half Plate Armor
(20632,380,1,1,1,106),-- Plate Gaiters
(20632,1950,1,1,1,4178),-- Half Plate Temper
(20632,1951,1,1,1,6498),-- Plate Gaiters Part
(20632,1864,1,1,2,166667),-- Stem
(20632,1869,1,1,2,71429),-- Iron Ore
(20632,1880,1,1,2,6329),-- Steel
(20632,2205,1,1,2,1815),-- Recipe: Chain Gaiters
(20632,2301,1,1,2,907),-- Recipe: Battle Axe
(20632,5274,1,1,2,907),-- Recipe: Greater Spiritshot (C) Compressed Package(100%)
(20632,5550,1,1,2,907),-- Durable Metal Plate
(20632,6668,1,1,2,31250),-- Deluxe Chest Key - Grade 4
(20633,2212,1,1,-1,35869),-- Recipe: Plated Leather Gaiters
(20633,2313,1,1,-1,4370),-- Recipe: Orcish Glaive
(20633,4469,1,1,-1,5719),-- Dye of Strength <Str+1 Con-1>
(20633,57,485,984,0,700000),-- Adena
(20633,851,1,1,1,139),-- Omen Beast\'s Eye Earring
(20633,882,1,1,1,208),-- Mithril Ring
(20633,914,1,1,1,104),-- Necklace of Darkness
(20633,1905,1,1,1,8148),-- Omen Beast\'s Eye Earring Gemstone
(20633,1906,1,1,1,11513),-- Mithril Ring Wire
(20633,1907,1,1,1,5827),-- Necklace of Darkness Gem
(20633,1868,1,1,2,166667),-- Thread
(20633,1873,1,1,2,33333),-- Silver Nugget
(20633,1876,1,1,2,10753),-- Mithril Ore
(20633,1895,1,1,2,19231),-- Metallic Fiber
(20633,2211,1,1,2,770),-- Recipe: Plated Leather Armor
(20633,2992,1,1,2,9259),-- Recipe: Plated Leather Boots
(20633,5159,1,1,2,240),-- Recipe: Spiritshot (C) Compressed Package (100%)
(20634,1880,1,1,-1,191909),-- Steel
(20634,1908,1,1,-1,181732),-- Moonstone Earring Wire
(20634,2213,1,1,-1,29800),-- Recipe: Dwarven Chain Mail Shirt
(20634,57,925,1892,0,700000),-- Adena
(20634,354,1,1,1,90),-- Chain Mail Shirt
(20634,381,1,1,1,144),-- Chain Gaiters
(20634,1959,1,1,1,6109),-- Chain Mail Shirt Material
(20634,1960,1,1,1,12426),-- Chain Gaiters Part
(20634,1867,1,1,2,100000),-- Animal Skin
(20634,1872,1,1,2,100000),-- Animal Bone
(20634,1875,1,1,2,7042),-- Stone of Purity
(20634,1881,1,1,2,6536),-- Coarse Bone Powder
(20634,2215,1,1,2,1086),-- Recipe: Robe of Seal
(20634,2225,1,1,2,3257),-- Recipe: Demon\'s Stockings
(20634,2251,1,1,2,10309),-- Recipe: Steel Arrow
(20634,2321,1,1,2,543),-- Recipe: Stiletto
(20634,8723,1,1,200,4348),-- Life Stone: level 46
(20634,8733,1,1,200,543),-- Mid-Grade Life Stone: level 46
(20634,8743,1,1,200,1086),-- High-Grade Life Stone: level 46
(20635,1959,1,1,-1,26145),-- Chain Mail Shirt Material
(20635,2163,1,1,-1,49522),-- Recipe: Earring of Protection
(20635,2215,1,1,-1,12365),-- Recipe: Robe of Seal
(20635,57,470,954,0,700000),-- Adena
(20635,263,1,1,1,21),-- Chakram
(20635,2073,1,1,1,1773),-- Chakram Edge
(20635,1866,1,1,2,27778),-- Suede
(20635,1871,1,1,2,43478),-- Charcoal
(20635,1885,1,1,2,1555),-- High Grade Suede
(20635,3002,1,1,2,2591),-- Recipe: Dwarven Chain Gloves
(20635,3004,1,1,2,2591),-- Recipe: Rind Leather Gloves
(20635,4041,1,1,2,570),-- Mold Hardener
(20635,5220,1,1,2,1555),-- Metal Hardener
(20635,8723,1,1,200,2591),-- Life Stone: level 46
(20635,8733,1,1,200,1037),-- Mid-Grade Life Stone: level 46
(20636,1909,1,1,-1,145932),-- Aquastone Ring Wire
(20636,2163,1,1,-1,55134),-- Recipe: Earring of Protection
(20636,2214,1,1,-1,26815),-- Recipe: Dwarven Chain Gaiters
(20636,57,521,1066,0,700000),-- Adena
(20636,89,1,1,1,3),-- Big Hammer
(20636,160,1,1,1,3),-- Battle Axe
(20636,161,1,1,1,3),-- Silver Axe
(20636,173,1,1,1,3),-- Skull Graver
(20636,191,1,1,1,3),-- Heavy Doom Hammer
(20636,192,1,1,1,3),-- Crystal Staff
(20636,193,1,1,1,3),-- Stick of Faith
(20636,194,1,1,1,3),-- Heavy Doom Axe
(20636,2061,1,1,1,247),-- Big Hammer Head
(20636,2063,1,1,1,247),-- Battle Axe Head
(20636,2064,1,1,1,247),-- Silver Axe Head
(20636,2065,1,1,1,247),-- Skull Graver Head
(20636,2066,1,1,1,251),-- Heavy Doom Hammer Head
(20636,2067,1,1,1,231),-- Crystal Staff Head
(20636,2068,1,1,1,231),-- Stick of Faith Shaft
(20636,2069,1,1,1,251),-- Heavy Doom Axe Blade
(20636,1867,1,1,2,76923),-- Animal Skin
(20636,1872,1,1,2,71429),-- Animal Bone
(20636,1882,1,1,2,12658),-- Leather
(20636,1894,1,1,2,2660),-- Crafted Leather
(20636,2230,1,1,2,1724),-- Recipe: Theca Leather Gaiters
(20636,2320,1,1,2,157),-- Recipe: Cursed Staff
(20636,4042,1,1,2,1096),-- Enria
(20636,8723,1,1,200,3135),-- Life Stone: level 46
(20636,8733,1,1,200,470),-- Mid-Grade Life Stone: level 46
(20636,8743,1,1,200,157),-- High-Grade Life Stone: level 46
(20637,440,1,1,-1,319),-- Robe of Seal
(20637,2337,1,1,-1,2622),-- Recipe: Stick of Eternity
(20637,2338,1,1,-1,2622),-- Recipe: Paradia Staff
(20637,57,546,1111,0,700000),-- Adena
(20637,1126,1,1,1,52),-- Crimson Boots
(20637,1974,1,1,1,3501),-- Crimson Boot Fabric
(20637,2434,1,1,1,52),-- Rind Leather Boots
(20637,2455,1,1,1,56),-- Plated Leather Gloves
(20637,2456,1,1,1,52),-- Dwarven Chain Gloves
(20637,2457,1,1,1,52),-- Gloves of Seal
(20637,2458,1,1,1,52),-- Rind Leather Gloves
(20637,2946,1,1,1,2327),-- Plate Leather Gloves Design
(20637,2948,1,1,1,3501),-- Rind Leather Boot Lining
(20637,2949,1,1,1,3501),-- Dwarven Chain Gloves Design
(20637,2950,1,1,1,3501),-- Gloves of Seal Pattern
(20637,2951,1,1,1,3501),-- Rind Leather Gloves Design
(20637,1539,1,1,2,8333),-- Greater Healing Potion
(20637,1864,1,1,2,100000),-- Stem
(20637,1869,1,1,2,50000),-- Iron Ore
(20637,1880,1,1,2,4950),-- Steel
(20637,2222,1,1,2,354),-- Recipe: Composite Armor
(20637,4043,1,1,2,1946),-- Asofe
(20637,5550,1,1,2,1238),-- Durable Metal Plate
(20637,6668,1,1,2,16667),-- Deluxe Chest Key - Grade 4
(20637,8724,1,1,200,2833),-- Life Stone: level 49
(20637,8734,1,1,200,884),-- Mid-Grade Life Stone: level 49
(20637,8744,1,1,200,177),-- High-Grade Life Stone: level 49
(20638,1873,1,2,-1,470674),-- Silver Nugget
(20638,1911,1,2,-1,87518),-- Earring of Protection Gemstone
(20638,2946,1,1,-1,65773),-- Plate Leather Gloves Design
(20638,57,570,1158,0,700000),-- Adena
(20638,282,1,1,1,10),-- Elemental Bow
(20638,285,1,1,1,10),-- Noble Elven Bow
(20638,2085,1,1,1,902),-- Crossbow Shaft
(20638,2086,1,1,1,902),-- Elven Bow of Nobility Shaft
(20638,952,1,1,2,1193),-- Scroll: Enchant Armor (Grade C)
(20638,1867,1,1,2,66667),-- Animal Skin
(20638,1872,1,1,2,66667),-- Animal Bone
(20638,1882,1,1,2,10753),-- Leather
(20638,1894,1,1,2,2833),-- Crafted Leather
(20638,2338,1,1,2,135),-- Recipe: Paradia Staff
(20638,2351,1,1,2,122),-- Recipe: Scorpion
(20638,3005,1,1,2,2681),-- Recipe: Demon\'s Boots
(20638,4042,1,1,2,894),-- Enria
(20638,8724,1,1,200,1789),-- Life Stone: level 49
(20638,8734,1,1,200,298),-- Mid-Grade Life Stone: level 49
(20638,8744,1,1,200,38),-- High-Grade Life Stone: level 49
(20639,951,1,1,-1,4502),-- Scroll: Enchant Weapon (Grade C)
(20639,2334,1,1,-1,5758),-- Recipe: Sword of Whispering Death
(20639,2342,1,1,-1,5758),-- Recipe: Club of Nature
(20639,57,1196,2439,0,700000),-- Adena
(20639,227,1,1,1,20),-- Stiletto
(20639,242,1,1,1,20),-- Soulfire Dirk
(20639,2083,1,1,1,1823),-- Stiletto Edge
(20639,2084,1,1,1,1823),-- Soulfire Dirk Blade
(20639,1868,1,1,2,200000),-- Thread
(20639,1873,1,1,2,33333),-- Silver Nugget
(20639,1889,1,1,2,8929),-- Compound Braid
(20639,2328,1,1,2,746),-- Recipe: Sword of Delusion
(20639,4044,1,1,2,4484),-- Thons
(20639,4132,1,1,2,3731),-- Recipe: Composite Shield
(20639,5549,1,1,2,9709),-- Metallic Thread
(20639,6668,1,1,2,43478),-- Deluxe Chest Key - Grade 4
(20639,8724,1,1,200,3731),-- Life Stone: level 49
(20639,8734,1,1,200,2237),-- Mid-Grade Life Stone: level 49
(20640,1539,1,1,-1,264351),-- Greater Healing Potion
(20640,1868,1,7,-1,793052),-- Thread
(20640,1870,1,3,-1,594789),-- Coal
(20640,57,576,1171,0,700000),-- Adena
(20640,283,1,1,1,12),-- Akat Long Bow
(20640,2109,1,1,1,1380),-- Akat Longbow Shaft
(20640,1865,1,1,2,45455),-- Varnish
(20640,1870,1,1,2,52632),-- Coal
(20640,1875,1,1,2,2114),-- Stone of Purity
(20640,1879,1,1,2,8000),-- Cokes
(20640,2233,1,1,2,235),-- Recipe: Divine Tunic
(20640,2970,1,1,2,1876),-- Recipe: Ring of Binding
(20640,3014,1,1,2,1642),-- Recipe: Full Plate Boots
(20640,4040,1,1,2,1408),-- Mold Lubricant
(20640,6669,1,1,2,10989),-- Deluxe Chest Key - Grade 5
(20640,8725,1,1,200,839),-- Life Stone: level 52
(20640,8735,1,1,200,59),-- Mid-Grade Life Stone: level 52
(20641,2097,1,1,-1,7900),-- War Axe Blade
(20641,2355,1,1,-1,2406),-- Recipe: Ghoul\'s Staff
(20641,4042,1,1,-1,24583),-- Enria
(20641,57,715,1452,0,700000),-- Adena
(20641,854,1,1,1,69),-- Earring of Binding
(20641,885,1,1,1,104),-- Ring of Ages
(20641,917,1,1,1,52),-- Necklace of Mermaid
(20641,1914,1,1,1,5712),-- Earring of Binding Gemstone
(20641,1915,1,1,1,8958),-- Ring of Ages Gemstone
(20641,1916,1,1,1,4327),-- Necklace of Mermaid Teardrop
(20641,1867,1,1,2,76923),-- Animal Skin
(20641,1872,1,1,2,83333),-- Animal Bone
(20641,1882,1,1,2,15625),-- Leather
(20641,1894,1,1,2,2079),-- Crafted Leather
(20641,2353,1,1,2,125),-- Recipe: Samurai Longsword
(20641,4042,1,1,2,231),-- Enria
(20641,4124,1,1,2,1616),-- Recipe: Craftsman Mold
(20641,4609,1,1,2,170),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20641,8725,1,1,200,924),-- Life Stone: level 52
(20641,8735,1,1,200,55),-- Mid-Grade Life Stone: level 52
(20642,2103,1,1,-1,7789),-- Sage\'s Staff Head
(20642,2358,1,1,-1,2254),-- Recipe: Great Pata
(20642,4602,1,1,-1,3070),-- Greater Dye of Strength <Str+3 Dex-3>
(20642,57,672,1356,0,700000),-- Adena
(20642,283,1,1,1,14),-- Akat Long Bow
(20642,2109,1,1,1,1602),-- Akat Longbow Shaft
(20642,1342,10,30,2,200000),-- Fine Steel Arrow
(20642,1868,1,1,2,100000),-- Thread
(20642,1873,1,1,2,16667),-- Silver Nugget
(20642,1876,1,1,2,8772),-- Mithril Ore
(20642,1895,1,1,2,10526),-- Metallic Fiber
(20642,3015,1,1,2,878),-- Recipe: Drake Leather Gloves
(20642,3017,1,1,2,585),-- Recipe: Divine Gloves
(20642,4042,1,1,2,1464),-- Enria
(20642,4606,1,1,2,293),-- Greater Dye of Dexterity <Dex+3 Con-3>
(20642,5155,1,1,2,104),-- Recipe: Soulshot (B) Compressed Package (100%)
(20642,8725,1,1,200,878),-- Life Stone: level 52
(20642,8735,1,1,200,585),-- Mid-Grade Life Stone: level 52
(20643,1918,1,1,-1,61625),-- Nassen\'s Earring Gemstone
(20643,2114,1,1,-1,6891),-- Widowmaker Head
(20643,3955,1,1,-1,2846),-- Recipe: Blessed Spiritshot B
(20643,57,689,1399,0,700000),-- Adena
(20643,2414,1,1,1,50),-- Full Plate Helmet
(20643,2497,1,1,1,71),-- Full Plate Shield
(20643,2959,1,1,1,4544),-- Full Plate Helmet Design
(20643,2966,1,1,1,7187),-- Full Plate Shield Fragment
(20643,1894,1,1,2,1418),-- Crafted Leather
(20643,4039,1,1,2,1984),-- Mold Glue
(20643,4041,1,1,2,851),-- Mold Hardener
(20643,4044,1,1,2,1418),-- Thons
(20643,4126,1,1,2,284),-- Recipe: Adamantite Earrings
(20643,5270,1,1,2,284),-- Recipe: Greater Soulshot (B) Compressed Package(100%)
(20643,5275,1,1,2,284),-- Recipe: Greater Spiritshot (B) Compressed Package(100%)
(20643,6669,1,1,2,9615),-- Deluxe Chest Key - Grade 5
(20643,8726,1,1,200,1701),-- Life Stone: level 55
(20643,8736,1,1,200,851),-- Mid-Grade Life Stone: level 55
(20644,1806,1,1,-1,2791),-- Recipe: Soulshot: B-Grade
(20644,1865,1,3,-1,697860),-- Varnish
(20644,1987,1,1,-1,21294),-- Drake Leather Armor Texture
(20644,57,675,1374,0,700000),-- Adena
(20644,442,1,1,1,25),-- Divine Tunic
(20644,473,1,1,1,39),-- Divine Stockings
(20644,1988,1,1,1,2476),-- Divine Tunic Fabric
(20644,1989,1,1,1,3884),-- Divine Stocking Pattern
(20644,1864,1,1,2,100000),-- Stem
(20644,1869,1,1,2,50000),-- Iron Ore
(20644,1878,1,1,2,16129),-- Braided Hemp
(20644,1884,1,1,2,26316),-- Cord
(20644,4039,1,1,2,2392),-- Mold Glue
(20644,4142,1,1,2,478),-- Recipe: Avadon Gaiters
(20644,4147,1,1,2,239),-- Recipe: Stockings of Zubei
(20644,4184,1,1,2,66),-- Recipe: Sprite\'s Staff
(20644,5160,1,1,2,115),-- Recipe: Spiritshot (B) Compressed Package (100%)
(20644,8726,1,1,200,1675),-- Life Stone: level 55
(20644,8736,1,1,200,239),-- Mid-Grade Life Stone: level 55
(20644,8746,1,1,200,52),-- High-Grade Life Stone: level 55
(20645,1885,1,1,-1,261843),-- High Grade Suede
(20645,1920,1,1,-1,107515),-- Necklace of Binding Chain
(20645,2121,1,1,-1,14450),-- Eminence Bow Shaft
(20645,57,1520,3095,0,700000),-- Adena
(20645,286,1,1,1,19),-- Eminence Bow
(20645,2121,1,1,1,2668),-- Eminence Bow Shaft
(20645,1865,1,1,2,111111),-- Varnish
(20645,1870,1,1,2,125000),-- Coal
(20645,1874,1,1,2,6135),-- Oriharukon Ore
(20645,4039,1,1,2,4425),-- Mold Glue
(20645,4043,1,1,2,5405),-- Asofe
(20645,4179,1,1,2,1473),-- Recipe: Zubei\'s Helmet
(20645,4189,1,1,2,168),-- Recipe: Arthro Nail
(20645,5275,1,1,2,291),-- Recipe: Greater Spiritshot (B) Compressed Package(100%)
(20645,8726,1,1,200,2457),-- Life Stone: level 55
(20645,8746,1,1,200,246),-- High-Grade Life Stone: level 55
(20646,2122,1,1,-1,10807),-- Orcish Poleaxe Blade
(20646,3034,1,1,-1,4735),-- Recipe: Spiritshot B
(20646,4044,1,1,-1,78920),-- Thons
(20646,57,1146,2333,0,700000),-- Adena
(20646,299,1,1,1,5),-- Orcish Poleaxe
(20646,301,1,1,1,7),-- Scorpion
(20646,303,1,1,1,7),-- Widow Maker
(20646,2113,1,1,1,706),-- Scorpion Blade
(20646,2114,1,1,1,706),-- Widowmaker Head
(20646,2122,1,1,1,665),-- Orcish Poleaxe Blade
(20646,1864,1,1,2,166667),-- Stem
(20646,1869,1,1,2,90909),-- Iron Ore
(20646,1878,1,1,2,37037),-- Braided Hemp
(20646,1884,1,1,2,55556),-- Cord
(20646,4039,1,1,2,4608),-- Mold Glue
(20646,4186,1,1,2,126),-- Recipe: Sword of Valhalla
(20646,4190,1,1,2,126),-- Recipe: Dark Elven Long Bow
(20646,5165,1,1,2,219),-- Recipe: Blessed Spiritshot (B) Compressed Package (100%)
(20646,8726,1,1,200,2762),-- Life Stone: level 55
(20647,947,1,1,-1,952),-- Scroll: Enchant Weapon (Grade B)
(20647,2960,1,1,-1,129649),-- Drake Leather Boots Design
(20647,4049,1,1,-1,124889),-- Adamantite Earring Gemstone
(20647,57,1151,2346,0,700000),-- Adena
(20647,554,1,1,1,34),-- Zubei\'s Boots
(20647,600,1,1,1,34),-- Avadon Boots
(20647,612,1,1,1,34),-- Zubei\'s Gauntlets
(20647,2464,1,1,1,34),-- Avadon Gloves
(20647,4073,1,1,1,4575),-- Avadon Gloves Part
(20647,4097,1,1,1,4575),-- Zubei\'s Boots Design
(20647,4098,1,1,1,4575),-- Avadon Boots Design
(20647,4100,1,1,1,4575),-- Zubei\'s Gauntlets Part
(20647,1865,1,1,2,83333),-- Varnish
(20647,1870,1,1,2,71429),-- Coal
(20647,1875,1,1,2,4464),-- Stone of Purity
(20647,1879,1,1,2,9709),-- Cokes
(20647,4040,1,1,2,372),-- Mold Lubricant
(20647,4176,1,1,2,1116),-- Recipe: Doom Boots
(20647,4946,1,1,2,1488),-- Recipe: Zubei\'s Leather Shirt (60%)
(20647,5280,1,1,2,176),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(20647,8727,1,1,200,1116),-- Life Stone: level 58
(20647,8737,1,1,200,372),-- Mid-Grade Life Stone: level 58
(20648,1868,1,13,-1,855624),-- Thread
(20648,2963,1,1,-1,139770),-- Full Plate Gauntlets Part
(20648,4191,1,1,-1,2957),-- Recipe: Great Axe
(20648,57,1241,2530,0,700000),-- Adena
(20648,554,1,1,1,37),-- Zubei\'s Boots
(20648,600,1,1,1,37),-- Avadon Boots
(20648,612,1,1,1,37),-- Zubei\'s Gauntlets
(20648,2464,1,1,1,37),-- Avadon Gloves
(20648,4073,1,1,1,4932),-- Avadon Gloves Part
(20648,4097,1,1,1,4932),-- Zubei\'s Boots Design
(20648,4098,1,1,1,4932),-- Avadon Boots Design
(20648,4100,1,1,1,4932),-- Zubei\'s Gauntlets Part
(20648,1868,1,1,2,166667),-- Thread
(20648,1873,1,1,2,31250),-- Silver Nugget
(20648,1876,1,1,2,18868),-- Mithril Ore
(20648,1895,1,1,2,18868),-- Metallic Fiber
(20648,4042,1,1,2,1946),-- Enria
(20648,4961,1,1,2,486),-- Recipe: Zubei\'s Shield (60%)
(20648,4999,1,1,2,1179),-- Recipe: Doom Boots (60%)
(20648,6669,1,1,2,20408),-- Deluxe Chest Key - Grade 5
(20648,8727,1,1,200,3401),-- Life Stone: level 58
(20648,8737,1,1,200,118),-- Mid-Grade Life Stone: level 58
(20649,2112,1,1,-1,13082),-- Pa\'agrian Axe Blade
(20649,4133,1,1,-1,12988),-- Recipe: Zubei\'s Breastplate
(20649,5549,1,2,-1,270151),-- Metallic Thread
(20649,57,1306,2660,0,700000),-- Adena
(20649,78,1,1,1,4),-- Great Sword
(20649,142,1,1,1,4),-- Keshanberk
(20649,148,1,1,1,4),-- Sword of Valhalla
(20649,4104,1,1,1,761),-- Great Sword Blade
(20649,4107,1,1,1,750),-- Keshanberk Blade
(20649,4108,1,1,1,750),-- Sword of Valhalla Blade
(20649,1832,1,1,2,83333),-- Greater Antidote
(20649,1868,1,1,2,142857),-- Thread
(20649,1873,1,1,2,28571),-- Silver Nugget
(20649,1889,1,1,2,6579),-- Compound Braid
(20649,4044,1,1,2,2488),-- Thons
(20649,4194,1,1,2,73),-- Recipe: Deadman\'s Glory
(20649,4970,1,1,2,73),-- Recipe: Arthro Nail (60%)
(20649,8311,1,1,2,14),-- Recipe: Kaim Vanul\'s Bones (100%)
(20649,5549,1,1,2,7194),-- Metallic Thread
(20649,8727,1,1,200,1391),-- Life Stone: level 58
(20649,8737,1,1,200,585),-- Mid-Grade Life Stone: level 58
(20650,4049,1,1,-1,153727),-- Adamantite Earring Gemstone
(20650,4195,1,1,-1,2237),-- Recipe: Art of Battle Axe
(20650,4613,1,1,-1,4042),-- Greater Dye of Strength <Str+4 Con-4>
(20650,57,1417,2887,0,700000),-- Adena
(20650,856,1,1,1,48),-- Adamantite Earring
(20650,887,1,1,1,72),-- Adamantite Ring
(20650,918,1,1,1,36),-- Adamantite Necklace
(20650,4049,1,1,1,7570),-- Adamantite Earring Gemstone
(20650,4050,1,1,1,8942),-- Adamantite Ring Wire
(20650,4051,1,1,1,5597),-- Adamantite Necklace Chain
(20650,948,1,1,2,492),-- Scroll: Enchant Armor (Grade B)
(20650,1867,1,1,2,125000),-- Animal Skin
(20650,1872,1,1,2,111111),-- Animal Bone
(20650,1882,1,1,2,18182),-- Leather
(20650,1894,1,1,2,3484),-- Crafted Leather
(20650,4042,1,1,2,1410),-- Enria
(20650,4167,1,1,2,1149),-- Recipe: Blue Wolf Boots
(20650,4168,1,1,2,1182),-- Recipe: Doom Gloves
(20650,8728,1,1,200,1511),-- Life Stone: level 61
(20650,8738,1,1,200,492),-- Mid-Grade Life Stone: level 61
(20651,4050,1,2,-1,168983),-- Adamantite Ring Wire
(20651,4196,1,1,-1,2082),-- Recipe: Staff of Evil Spirits
(20651,57,1320,2684,0,700000),-- Adena
(20651,78,1,1,1,4),-- Great Sword
(20651,142,1,1,1,4),-- Keshanberk
(20651,148,1,1,1,4),-- Sword of Valhalla
(20651,4104,1,1,1,769),-- Great Sword Blade
(20651,4107,1,1,1,757),-- Keshanberk Blade
(20651,948,1,1,2,141),-- Scroll: Enchant Armor (Grade B)
(20651,1867,1,1,2,111111),-- Animal Skin
(20651,1872,1,1,2,111111),-- Animal Bone
(20651,1882,1,1,2,19608),-- Leather
(20651,1894,1,1,2,2545),-- Crafted Leather
(20651,4042,1,1,2,2119),-- Enria
(20651,4167,1,1,2,424),-- Recipe: Blue Wolf Boots
(20651,4168,1,1,2,848),-- Recipe: Doom Gloves
(20651,8728,1,1,200,424),-- Life Stone: level 61
(20651,8738,1,1,200,565),-- Mid-Grade Life Stone: level 61
(20652,4042,1,1,-1,48043),-- Enria
(20652,4043,1,1,-1,96085),-- Asofe
(20652,4129,1,1,-1,31197),-- Recipe: Earrings of Black Ore
(20652,57,1398,2835,0,700000),-- Adena
(20652,2384,1,1,1,19),-- Zubei\'s Leather Shirt
(20652,2388,1,1,1,30),-- Zubei\'s Leather Gaiters
(20652,2390,1,1,1,13),-- Avadon Leather Armor
(20652,4066,1,1,1,3345),-- Zubei\'s Leather Shirt Fabric
(20652,4067,1,1,1,5079),-- Zubei\'s Leather Gaiter Texture
(20652,4068,1,1,1,2570),-- Avadon Leather Armor Lining
(20652,1864,1,1,2,166667),-- Stem
(20652,1869,1,1,2,83333),-- Iron Ore
(20652,1880,1,1,2,11111),-- Steel
(20652,4043,1,1,2,2392),-- Asofe
(20652,4173,1,1,2,797),-- Recipe: Blue Wolf Gloves
(20652,4615,1,1,2,147),-- Greater Dye of Constitution <Con+4 Str-4>
(20652,4998,1,1,2,2924),-- Recipe: Blue Wolf Gloves (60%)
(20652,5550,1,1,2,1328),-- Durable Metal Plate
(20652,8728,1,1,200,2660),-- Life Stone: level 61
(20652,8738,1,1,200,532),-- Mid-Grade Life Stone: level 61
(20654,4052,1,1,-1,190253),-- Earring of Black Ore Piece
(20654,4064,1,1,-1,97262),-- Avadon Breastplate Part
(20654,5230,1,1,-1,7165),-- Recipe: Mithril Arrow (100%)
(20654,57,2608,5283,0,700000),-- Adena
(20654,2398,1,1,1,17),-- Blue Wolf Tunic
(20654,2399,1,1,1,17),-- Tunic of Doom
(20654,2403,1,1,1,27),-- Blue Wolf Stockings
(20654,2404,1,1,1,27),-- Stockings of Doom
(20654,4084,1,1,1,3447),-- Blue Wolf Tunic Fabric
(20654,4085,1,1,1,3447),-- Tunic of Doom Pattern
(20654,4086,1,1,1,4654),-- Blue Wolf Stockings Pattern
(20654,4087,1,1,1,4654),-- Stockings of Doom Pattern
(20654,1894,1,1,2,7692),-- Crafted Leather
(20654,4039,1,1,2,4808),-- Mold Glue
(20654,4041,1,1,2,962),-- Mold Hardener
(20654,4044,1,1,2,4484),-- Thons
(20654,4624,1,1,2,962),-- Greater Dye of Wit <Wit+4 Men-4>
(20654,5276,1,1,2,642),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20654,6036,1,1,2,6098),-- Greater Magic Haste Potion
(20654,8729,1,1,200,2890),-- Life Stone: level 64
(20654,8739,1,1,200,321),-- Mid-Grade Life Stone: level 64
(20655,4053,1,1,-1,207665),-- Ring of Black Ore Gemstone
(20655,4067,1,1,-1,173432),-- Zubei\'s Leather Gaiter Texture
(20655,4160,1,1,-1,13725),-- Recipe: Leather Armor of Doom
(20655,57,2356,4762,0,700000),-- Adena
(20655,110,1,1,1,57),-- Doom Shield
(20655,2416,1,1,1,40),-- Blue Wolf Helmet
(20655,2417,1,1,1,40),-- Doom Helmet
(20655,4077,1,1,1,10729),-- Doom Shield Fragment
(20655,4088,1,1,1,9014),-- Blue Wolf Helmet Design
(20655,4089,1,1,1,9014),-- Doom Helmet Pattern
(20655,1343,20,60,2,100000),-- Silver Arrow
(20655,1865,1,1,2,142857),-- Varnish
(20655,1870,1,1,2,111111),-- Coal
(20655,1874,1,1,2,5000),-- Oriharukon Ore
(20655,4039,1,1,2,5495),-- Mold Glue
(20655,4043,1,1,2,3247),-- Asofe
(20655,4989,1,1,2,1053),-- Recipe: Stockings of Doom (60%)
(20655,5276,1,1,2,212),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20655,6670,1,1,2,22727),-- Deluxe Chest Key - Grade 6
(20655,8729,1,1,200,750),-- Life Stone: level 64
(20655,8739,1,1,200,159),-- Mid-Grade Life Stone: level 64
(20656,4040,1,2,-1,93328),-- Mold Lubricant
(20656,4069,1,1,-1,110018),-- Tunic of Zubei Fabric
(20656,57,2265,4585,0,700000),-- Adena
(20656,2398,1,1,1,15),-- Blue Wolf Tunic
(20656,2399,1,1,1,15),-- Tunic of Doom
(20656,2403,1,1,1,23),-- Blue Wolf Stockings
(20656,2404,1,1,1,23),-- Stockings of Doom
(20656,4084,1,1,1,2994),-- Blue Wolf Tunic Fabric
(20656,4085,1,1,1,2994),-- Tunic of Doom Pattern
(20656,4086,1,1,1,4042),-- Blue Wolf Stockings Pattern
(20656,4087,1,1,1,4042),-- Stockings of Doom Pattern
(20656,1895,1,1,2,45455),-- Metallic Fiber
(20656,4040,1,1,2,2915),-- Mold Lubricant
(20656,4042,1,1,2,2667),-- Enria
(20656,4043,1,1,2,5814),-- Asofe
(20656,4990,1,1,2,485),-- Recipe: Blue Wolf Helmet (60%)
(20656,6670,1,1,2,27778),-- Deluxe Chest Key - Grade 6
(20656,8729,1,1,200,2915),-- Life Stone: level 64
(20656,8739,1,1,200,243),-- Mid-Grade Life Stone: level 64
(20656,8749,1,1,200,243),-- High-Grade Life Stone: level 64
(20657,1877,1,1,-1,137950),-- Adamantite Nugget
(20657,1914,1,2,-1,193750),-- Earring of Binding Gemstone
(20657,1920,1,1,-1,118007),-- Necklace of Binding Chain
(20657,57,1677,3388,0,700000),-- Adena
(20657,2398,1,1,1,11),-- Blue Wolf Tunic
(20657,2399,1,1,1,11),-- Tunic of Doom
(20657,2403,1,1,1,17),-- Blue Wolf Stockings
(20657,2404,1,1,1,17),-- Stockings of Doom
(20657,4084,1,1,1,2213),-- Blue Wolf Tunic Fabric
(20657,4085,1,1,1,2213),-- Tunic of Doom Pattern
(20657,4086,1,1,1,2987),-- Blue Wolf Stockings Pattern
(20657,4087,1,1,1,2987),-- Stockings of Doom Pattern
(20657,1866,1,1,2,66667),-- Suede
(20657,1871,1,1,2,111111),-- Charcoal
(20657,1877,1,1,2,4926),-- Adamantite Nugget
(20657,4040,1,1,2,2347),-- Mold Lubricant
(20657,4041,1,1,2,469),-- Mold Hardener
(20657,4960,1,1,2,2114),-- Recipe: Zubei\'s Gauntlets (60%)
(20657,5276,1,1,2,1408),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20657,8729,1,1,200,1408),-- Life Stone: level 64
(20657,8739,1,1,200,235),-- Mid-Grade Life Stone: level 64
(20657,8759,1,1,200,154),-- Top-Grade Life Stone: level 64
(20658,1960,1,2,-1,182200),-- Chain Gaiters Part
(20658,2220,1,1,-1,68844),-- Recipe: Rind Leather Armor
(20658,4054,1,1,-1,65494),-- Necklace of Black Ore Beads
(20658,57,1613,3270,0,700000),-- Adena
(20658,864,1,1,1,36),-- Earring of Black Ore
(20658,895,1,1,1,54),-- Ring of Black Ore
(20658,926,1,1,1,27),-- Necklace of Black Ore
(20658,4052,1,1,1,5796),-- Earring of Black Ore Piece
(20658,4053,1,1,1,7016),-- Ring of Black Ore Gemstone
(20658,4054,1,1,1,3224),-- Necklace of Black Ore Beads
(20658,730,1,1,2,103),-- Scroll: Enchant Armor (Grade A)
(20658,1832,1,1,2,111111),-- Greater Antidote
(20658,1866,1,1,2,58824),-- Suede
(20658,1871,1,1,2,111111),-- Charcoal
(20658,1885,1,1,2,9091),-- High Grade Suede
(20658,4041,1,1,2,912),-- Mold Hardener
(20658,4155,1,1,2,228),-- Recipe: Blue Wolf Breastplate
(20658,5220,1,1,2,4098),-- Metal Hardener
(20658,8729,1,1,200,1368),-- Life Stone: level 64
(20658,8749,1,1,200,228),-- High-Grade Life Stone: level 64
(20659,1885,1,1,-1,89149),-- High Grade Suede
(20659,2062,1,1,-1,8390),-- Scythe Edge
(20659,2163,1,1,-1,54581),-- Recipe: Earring of Protection
(20659,57,518,1052,0,700000),-- Adena
(20659,631,1,1,1,127),-- Eldarake
(20659,1964,1,1,1,6810),-- Eldarake Temper
(20659,2413,1,1,1,90),-- Chain Hood
(20659,2495,1,1,1,127),-- Chain Shield
(20659,2936,1,1,1,6185),-- Chain Hood Pattern
(20659,2945,1,1,1,6810),-- Chain Shield Fragment
(20659,1868,1,1,2,90909),-- Thread
(20659,1873,1,1,2,21277),-- Silver Nugget
(20659,1889,1,1,2,4049),-- Compound Braid
(20659,2215,1,1,2,716),-- Recipe: Robe of Seal
(20659,2225,1,1,2,290),-- Recipe: Demon\'s Stockings
(20659,2321,1,1,2,196),-- Recipe: Stiletto
(20659,3021,1,1,2,196),-- Recipe: Dwarven War Hammer
(20659,4044,1,1,2,869),-- Thons
(20659,5549,1,1,2,7519),-- Metallic Thread
(20659,8723,1,1,200,2028),-- Life Stone: level 46
(20659,8733,1,1,200,1447),-- Mid-Grade Life Stone: level 46
(20660,1908,1,1,-1,125113),-- Moonstone Earring Wire
(20660,2084,1,1,-1,8779),-- Soulfire Dirk Blade
(20660,2330,1,1,-1,3073),-- Recipe: Homunkulus\'s Sword
(20660,57,637,1303,0,700000),-- Adena
(20660,439,1,1,1,83),-- Karmian Tunic
(20660,471,1,1,1,132),-- Karmian Stockings
(20660,1954,1,1,1,8300),-- Karmian Stocking Design
(20660,1962,1,1,1,4996),-- Karmian Tunic Pattern
(20660,1864,1,1,2,142857),-- Stem
(20660,1869,1,1,2,76923),-- Iron Ore
(20660,1880,1,1,2,7353),-- Steel
(20660,2163,1,1,2,4132),-- Recipe: Earring of Protection
(20660,4043,1,1,2,3676),-- Asofe
(20660,5550,1,1,2,920),-- Durable Metal Plate
(20660,8723,1,1,200,3448),-- Life Stone: level 46
(20660,8733,1,1,200,920),-- Mid-Grade Life Stone: level 46
(20661,2166,1,1,-1,42266),-- Recipe: Earrings of Binding (100%)
(20661,2223,1,1,-1,48328),-- Recipe: Tower Shield
(20661,2347,1,1,-1,2900),-- Recipe: Akat Long Bow
(20661,57,603,1229,0,700000),-- Adena
(20661,500,1,1,1,77),-- Great Helmet
(20661,632,1,1,1,109),-- Knight\'s Shield
(20661,1971,1,1,1,5637),-- Great Helmet Material
(20661,1972,1,1,1,7367),-- Knight\'s Shield Fragment
(20661,2496,1,1,1,119),-- Dwarven Chain Shield
(20661,2947,1,1,1,6577),-- Dwarven Chain Shield Fragment
(20661,1343,10,30,2,76923),-- Silver Arrow
(20661,1868,1,1,2,90909),-- Thread
(20661,1873,1,1,2,23256),-- Silver Nugget
(20661,1876,1,1,2,9524),-- Mithril Ore
(20661,1895,1,1,2,11111),-- Metallic Fiber
(20661,2223,1,1,2,2288),-- Recipe: Tower Shield
(20661,2230,1,1,2,762),-- Recipe: Theca Leather Gaiters
(20661,3005,1,1,2,1905),-- Recipe: Demon\'s Boots
(20661,4042,1,1,2,381),-- Enria
(20661,6668,1,1,2,15152),-- Deluxe Chest Key - Grade 4
(20661,8724,1,1,200,2667),-- Life Stone: level 49
(20661,8734,1,1,200,381),-- Mid-Grade Life Stone: level 49
(20662,951,1,1,-1,2366),-- Scroll: Enchant Weapon (Grade C)
(20662,1873,1,2,-1,520615),-- Silver Nugget
(20662,1911,1,1,-1,96805),-- Earring of Protection Gemstone
(20662,57,630,1281,0,700000),-- Adena
(20662,73,1,1,1,6),-- Shamshir
(20662,74,1,1,1,6),-- Katana
(20662,131,1,1,1,6),-- Spirit Sword
(20662,133,1,1,1,6),-- Raid Sword
(20662,2077,1,1,1,479),-- Shamshir Blade
(20662,2078,1,1,1,479),-- Katana Blade
(20662,2080,1,1,1,479),-- Spirit Sword Blade
(20662,2081,1,1,1,479),-- Raid Sword Blade
(20662,1343,10,30,2,83333),-- Silver Arrow
(20662,1866,1,1,2,35714),-- Suede
(20662,1871,1,1,2,55556),-- Charcoal
(20662,1877,1,1,2,2079),-- Adamantite Nugget
(20662,2340,1,1,2,68),-- Recipe: Pa\'agrian Hammer
(20662,3006,1,1,2,1942),-- Recipe: Demon\'s Gloves
(20662,4040,1,1,2,886),-- Mold Lubricant
(20662,4041,1,1,2,238),-- Mold Hardener
(20662,8724,1,1,200,2008),-- Life Stone: level 49
(20662,8734,1,1,200,409),-- Mid-Grade Life Stone: level 49
(20662,8754,1,1,200,31),-- Top-Grade Life Stone: level 49
(20663,951,1,1,-1,2075),-- Scroll: Enchant Weapon (Grade C)
(20663,1981,1,1,-1,67117),-- Mithril Gauntlets Design
(20663,4123,1,1,-1,50711),-- Recipe: Maestro Anvil Lock
(20663,57,551,1124,0,700000),-- Adena
(20663,853,1,1,1,86),-- Earring of Protection
(20663,884,1,1,1,130),-- Ring of Protection
(20663,916,1,1,1,65),-- Necklace of Protection
(20663,1911,1,1,1,6268),-- Earring of Protection Gemstone
(20663,1912,1,1,1,9081),-- Ring of Protection Gemstone
(20663,1913,1,1,1,5125),-- Necklace of Protection Gemstone
(20663,1866,1,1,2,35714),-- Suede
(20663,1871,1,1,2,52632),-- Charcoal
(20663,1877,1,1,2,1613),-- Adamantite Nugget
(20663,2340,1,1,2,225),-- Recipe: Pa\'agrian Hammer
(20663,3006,1,1,2,2625),-- Recipe: Demon\'s Gloves
(20663,4040,1,1,2,1163),-- Mold Lubricant
(20663,4041,1,1,2,411),-- Mold Hardener
(20663,8724,1,1,200,2288),-- Life Stone: level 49
(20663,8734,1,1,200,525),-- Mid-Grade Life Stone: level 49
(20663,8754,1,1,200,75),-- Top-Grade Life Stone: level 49
(20664,1865,1,3,-1,600678),-- Varnish
(20664,1867,1,5,-1,800904),-- Animal Skin
(20664,2098,1,1,-1,6435),-- Nirvana Axe Blade
(20664,57,582,1182,0,700000),-- Adena
(20664,64,1,1,1,102),-- Composite Boots
(20664,608,1,1,1,102),-- Mithril Gauntlets
(20664,1981,1,1,1,7306),-- Mithril Gauntlets Design
(20664,2956,1,1,1,7306),-- Composite Boots Part
(20664,1539,1,1,2,12500),-- Greater Healing Potion
(20664,1867,1,1,2,58824),-- Animal Skin
(20664,1872,1,1,2,55556),-- Animal Bone
(20664,1882,1,1,2,8547),-- Leather
(20664,1894,1,1,2,2551),-- Crafted Leather
(20664,2234,1,1,2,650),-- Recipe: Divine Stockings
(20664,2356,1,1,2,4),-- Recipe: Demon\'s Staff
(20664,4042,1,1,2,1200),-- Enria
(20664,4609,1,1,2,590),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20664,6669,1,1,2,9346),-- Deluxe Chest Key - Grade 5
(20664,8725,1,1,200,2283),-- Life Stone: level 52
(20664,8735,1,1,200,31),-- Mid-Grade Life Stone: level 52
(20664,8745,1,1,200,6),-- High-Grade Life Stone: level 52
(20665,2104,1,1,-1,8314),-- Club of Nature Head
(20665,2359,1,1,-1,2406),-- Recipe: Eminence Bow
(20665,4043,1,1,-1,49166),-- Asofe
(20665,57,715,1452,0,700000),-- Adena
(20665,162,1,1,1,2),-- War Axe
(20665,174,1,1,1,2),-- Nirvana Axe
(20665,196,1,1,1,2),-- Stick of Eternity
(20665,197,1,1,1,2),-- Paradia Staff
(20665,199,1,1,1,2),-- Pa\'agrian Hammer
(20665,200,1,1,1,2),-- Sage\'s Staff
(20665,201,1,1,1,2),-- Club of Nature
(20665,202,1,1,1,2),-- Mace of The Underworld
(20665,2097,1,1,1,205),-- War Axe Blade
(20665,2098,1,1,1,205),-- Nirvana Axe Blade
(20665,2099,1,1,1,215),-- Stick of Eternity Shaft
(20665,2100,1,1,1,215),-- Paradia Staff Head
(20665,2102,1,1,1,208),-- Pa\'agrian Hammer Head
(20665,2103,1,1,1,215),-- Sage\'s Staff Head
(20665,2104,1,1,1,215),-- Club of Nature Head
(20665,2105,1,1,1,205),-- Mace of The Underworld Head
(20665,1865,1,1,2,58824),-- Varnish
(20665,1870,1,1,2,62500),-- Coal
(20665,1874,1,1,2,5000),-- Oriharukon Ore
(20665,3014,1,1,2,288),-- Recipe: Full Plate Boots
(20665,4039,1,1,2,1026),-- Mold Glue
(20665,4043,1,1,2,1580),-- Asofe
(20665,4125,1,1,2,1669),-- Recipe: Maestro Mold
(20665,8725,1,1,200,2237),-- Life Stone: level 52
(20665,8735,1,1,200,13),-- Mid-Grade Life Stone: level 52
(20665,8745,1,1,200,5),-- High-Grade Life Stone: level 52
(20666,1884,1,3,-1,944213),-- Cord
(20666,1977,1,1,-1,22697),-- Composite Armor Temper
(20666,4606,1,1,-1,3410),-- Greater Dye of Dexterity <Dex+3 Con-3>
(20666,57,746,1502,0,700000),-- Adena
(20666,64,1,1,1,130),-- Composite Boots
(20666,608,1,1,1,130),-- Mithril Gauntlets
(20666,1981,1,1,1,9332),-- Mithril Gauntlets Design
(20666,2956,1,1,1,9332),-- Composite Boots Part
(20666,1865,1,1,2,58824),-- Varnish
(20666,1870,1,1,2,62500),-- Coal
(20666,1874,1,1,2,5495),-- Oriharukon Ore
(20666,3955,1,1,2,142),-- Recipe: Blessed Spiritshot B
(20666,4039,1,1,2,2398),-- Mold Glue
(20666,4043,1,1,2,1976),-- Asofe
(20666,4607,1,1,2,157),-- Greater Dye of Intelligence <Int+3 Men-3>
(20666,4609,1,1,2,282),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20666,5280,1,1,2,282),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(20666,8725,1,1,200,2114),-- Life Stone: level 52
(20666,8735,1,1,200,564),-- Mid-Grade Life Stone: level 52
(20666,8745,1,1,200,141),-- High-Grade Life Stone: level 52
(20667,1988,1,1,-1,30378),-- Divine Tunic Fabric
(20667,4141,1,1,-1,7601),-- Recipe: Avadon Breastplate
(20667,4604,1,1,-1,3513),-- Greater Dye of Constitution <Con+3 Dex-3>
(20667,57,766,1556,0,700000),-- Adena
(20667,299,1,1,1,4),-- Orcish Poleaxe
(20667,301,1,1,1,5),-- Scorpion
(20667,303,1,1,1,5),-- Widow Maker
(20667,2113,1,1,1,471),-- Scorpion Blade
(20667,2114,1,1,1,471),-- Widowmaker Head
(20667,2122,1,1,1,444),-- Orcish Poleaxe Blade
(20667,1875,1,1,2,4274),-- Stone of Purity
(20667,1879,1,1,2,7092),-- Cokes
(20667,1885,1,1,2,7092),-- High Grade Suede
(20667,4127,1,1,2,2024),-- Recipe: Adamantite Ring
(20667,4184,1,1,2,75),-- Recipe: Sprite\'s Staff
(20667,5155,1,1,2,2),-- Recipe: Soulshot (B) Compressed Package (100%)
(20667,5165,1,1,2,104),-- Recipe: Blessed Spiritshot (B) Compressed Package (100%)
(20667,5549,1,1,2,5495),-- Metallic Thread
(20667,5550,1,1,2,121),-- Durable Metal Plate
(20667,8726,1,1,200,848),-- Life Stone: level 55
(20667,8735,1,1,200,588),-- Mid-Grade Life Stone: level 52
(20667,8745,1,1,200,141),-- High-Grade Life Stone: level 52
(20668,1806,1,1,-1,2819),-- Recipe: Soulshot: B-Grade
(20668,2917,1,1,-1,80961),-- Ring of Binding Gemstone
(20668,4147,1,1,-1,14487),-- Recipe: Stockings of Zubei
(20668,57,682,1388,0,700000),-- Adena
(20668,203,1,1,1,3),-- Pa\'agrian Axe
(20668,204,1,1,1,2),-- Deadman\'s Staff
(20668,205,1,1,1,2),-- Ghoul\'s Staff
(20668,206,1,1,1,2),-- Demon\'s Staff
(20668,2112,1,1,1,253),-- Pa\'agrian Axe Blade
(20668,2116,1,1,1,245),-- Deadman\'s Staff Head
(20668,2117,1,1,1,245),-- Ghoul\'s Staff Head
(20668,2118,1,1,1,245),-- Demon\'s Staff Head
(20668,2503,1,1,1,2),-- Yaksa Mace
(20668,2969,1,1,1,251),-- Yaksa Mace Head
(20668,1866,1,1,2,37037),-- Suede
(20668,1871,1,1,2,47619),-- Charcoal
(20668,1885,1,1,2,5650),-- High Grade Suede
(20668,4041,1,1,2,1431),-- Mold Hardener
(20668,4148,1,1,2,65),-- Recipe: Avadon Robe
(20668,4936,1,1,2,650),-- Recipe: Avadon Shield (60%)
(20668,8309,1,1,2,35),-- Recipe: Spell Breaker (100%)
(20668,5220,1,1,2,650),-- Metal Hardener
(20668,8726,1,1,200,520),-- Life Stone: level 55
(20668,8736,1,1,200,59),-- Mid-Grade Life Stone: level 55
(20669,4057,1,2,-1,49216),-- Zubei\'s Gaiter Material
(20669,4198,1,1,-1,1343),-- Recipe: Bellion Cestus
(20669,4589,1,1,-1,5865),-- Greater Dye of Strength <Str+2 Con-2>
(20669,57,851,1734,0,700000),-- Adena
(20669,286,1,1,1,11),-- Eminence Bow
(20669,2121,1,1,1,1494),-- Eminence Bow Shaft
(20669,1866,1,1,2,35714),-- Suede
(20669,1871,1,1,2,50000),-- Charcoal
(20669,1877,1,1,2,4255),-- Adamantite Nugget
(20669,4040,1,1,2,540),-- Mold Lubricant
(20669,4041,1,1,2,472),-- Mold Hardener
(20669,4186,1,1,2,13),-- Recipe: Sword of Valhalla
(20669,4191,1,1,2,27),-- Recipe: Great Axe
(20669,5160,1,1,2,27),-- Recipe: Spiritshot (B) Compressed Package (100%)
(20669,5275,1,1,2,13),-- Recipe: Greater Spiritshot (B) Compressed Package(100%)
(20669,6669,1,1,2,12048),-- Deluxe Chest Key - Grade 5
(20669,8726,1,1,200,607),-- Life Stone: level 55
(20669,8736,1,1,200,81),-- Mid-Grade Life Stone: level 55
(20669,8746,1,1,200,972),-- High-Grade Life Stone: level 55
(20670,948,1,1,-1,4558),-- Scroll: Enchant Armor (Grade B)
(20670,4065,1,1,-1,51000),-- Avadon Gaiters Material
(20670,4129,1,1,-1,19732),-- Recipe: Earrings of Black Ore
(20670,57,884,1793,0,700000),-- Adena
(20670,12160,1,1,1,234),-- Common Item - Zubei\'s Helmet
(20670,12159,1,1,1,1536),-- Common Item - Zubei\'s Shield
(20670,12171,1,1,1,1335),-- Common Item - Avadon Shield
(20670,12170,1,1,1,90),-- Common Item - Avadon Circlet
(20670,4072,1,1,1,3025),-- Avadon Circlet Pattern
(20670,4101,1,1,1,3904),-- Zubei\'s Shield Fragment
(20670,4102,1,1,1,3025),-- Zubei\'s Helmet Design
(20670,4439,1,1,1,4644),-- Avadon Shield Fragment
(20670,1864,1,1,2,125000),-- Stem
(20670,1869,1,1,2,55556),-- Iron Ore
(20670,1880,1,1,2,3344),-- Steel
(20670,4043,1,1,2,4292),-- Asofe
(20670,4164,1,1,2,847),-- Recipe: Stockings of Doom
(20670,4168,1,1,2,220),-- Recipe: Doom Gloves
(20670,4616,1,1,2,441),-- Greater Dye of Constitution <Con+4 Dex-4>
(20670,5550,1,1,2,2012),-- Durable Metal Plate
(20670,8728,1,1,200,517),-- Life Stone: level 61
(20670,8738,1,1,200,7),-- Mid-Grade Life Stone: level 61
(20670,8748,1,1,200,28),-- High-Grade Life Stone: level 61
(20671,947,1,1,-1,763),-- Scroll: Enchant Weapon (Grade B)
(20671,2098,1,1,-1,10215),-- Nirvana Axe Blade
(20671,4052,1,1,-1,67520),-- Earring of Black Ore Piece
(20671,57,926,1874,0,700000),-- Adena
(20671,12204,1,1,1,9),-- Common Item - Blue Wolf Breastplate
(20671,12193,1,1,1,130),-- Common Item - Blue Wolf Gaiters
(20671,12184,1,1,1,43),-- Common Item - Doom Plate Armor
(20671,4078,1,1,1,1372),-- Blue Wolf Breastplate Part
(20671,4080,1,1,1,1932),-- Blue Wolf Gaiters Material
(20671,4081,1,1,1,926),-- Doom Plate Armor Temper
(20671,1864,1,1,2,111111),-- Stem
(20671,1869,1,1,2,55556),-- Iron Ore
(20671,1880,1,1,2,9434),-- Steel
(20671,4043,1,1,2,1658),-- Asofe
(20671,4168,1,1,2,1218),-- Recipe: Doom Gloves
(20671,5550,1,1,2,1597),-- Durable Metal Plate
(20671,8728,1,1,200,1253),-- Life Stone: level 61
(20671,8738,1,1,200,35),-- Mid-Grade Life Stone: level 61
(20672,4052,1,1,-1,68521),-- Earring of Black Ore Piece
(20672,4070,1,1,-1,69281),-- Stockings of Zubei Fabric
(20672,4154,1,1,-1,22904),-- Recipe: Doom Shield
(20672,57,940,1902,0,700000),-- Adena
(20672,12188,1,1,1,148),-- Common Item - Boots of Doom
(20672,12189,1,1,1,326),-- Common Item - Blue Wolf Boots
(20672,12187,1,1,1,148),-- Common Item - Doom Gloves
(20672,12190,1,1,1,326),-- Common Item - Blue Wolf Gloves
(20672,4090,1,1,1,3208),-- Blue Wolf Boots Design
(20672,4091,1,1,1,3208),-- Doom Gloves Part
(20672,4096,1,1,1,3208),-- Blue Wolf Gloves Fabric
(20672,4099,1,1,1,3208),-- Doom Boots Part
(20672,1868,1,1,2,142857),-- Thread
(20672,1873,1,1,2,27027),-- Silver Nugget
(20672,1889,1,1,2,5952),-- Compound Braid
(20672,4044,1,1,2,2457),-- Thons
(20672,4991,1,1,2,267),-- Recipe: Doom Helmet (60%)
(20672,5276,1,1,2,30),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20672,5549,1,1,2,6211),-- Metallic Thread
(20672,8729,1,1,200,237),-- Life Stone: level 64
(20672,8739,1,1,200,119),-- Mid-Grade Life Stone: level 64
(20672,8749,1,1,200,30),-- High-Grade Life Stone: level 64
(20673,4041,1,1,-1,17584),-- Mold Hardener
(20673,4052,1,2,-1,71593),-- Earring of Black Ore Piece
(20673,5230,1,1,-1,2696),-- Recipe: Mithril Arrow (100%)
(20673,57,983,1987,0,700000),-- Adena
(20673,11930,1,1,1,44),-- Common Item - Sword of Damascus
(20673,11943,1,1,1,44),-- Common Item - Wizard\'s Tear
(20673,11929,1,1,1,22),-- Common Item - Guardian Sword
(20673,4114,1,1,1,1308),-- Sword of Damascus Blade
(20673,8336,1,1,1,362),-- Blade of the Wizard\'s Tear
(20673,8330,1,1,1,458),-- Guardian Sword Blade
(20673,1867,1,1,2,100000),-- Animal Skin
(20673,1872,1,1,2,90909),-- Animal Bone
(20673,1875,1,1,2,8065),-- Stone of Purity
(20673,1881,1,1,2,6757),-- Coarse Bone Powder
(20673,4041,1,1,2,240),-- Mold Hardener
(20673,4150,1,1,2,720),-- Recipe: Avadon Gloves
(20673,6037,1,1,2,142857),-- Waking Scroll
(20673,8729,1,1,200,371),-- Life Stone: level 64
(20673,8739,1,1,200,109),-- Mid-Grade Life Stone: level 64
(20673,8749,1,1,200,44),-- High-Grade Life Stone: level 64
(20674,2089,1,1,-1,15893),-- Caliburs Edge
(20674,4053,1,1,-1,125008),-- Ring of Black Ore Gemstone
(20674,4124,1,2,-1,129619),-- Recipe: Craftsman Mold
(20674,57,1415,2868,0,700000),-- Adena
(20674,601,1,1,1,27),-- Boots of Doom
(20674,2439,1,1,1,27),-- Blue Wolf Boots
(20674,2475,1,1,1,27),-- Doom Gloves
(20674,2487,1,1,1,27),-- Blue Wolf Gloves
(20674,4090,1,1,1,4833),-- Blue Wolf Boots Design
(20674,4091,1,1,1,4833),-- Doom Gloves Part
(20674,4096,1,1,1,4833),-- Blue Wolf Gloves Fabric
(20674,4099,1,1,1,4833),-- Doom Boots Part
(20674,1867,1,1,2,111111),-- Animal Skin
(20674,1872,1,1,2,125000),-- Animal Bone
(20674,1875,1,1,2,5128),-- Stone of Purity
(20674,1881,1,1,2,11236),-- Coarse Bone Powder
(20674,4041,1,1,2,606),-- Mold Hardener
(20674,4157,1,1,2,314),-- Recipe: Blue Wolf Gaiters
(20674,8316,1,1,2,22),-- Recipe: Flaming Dragon Skull (60%)
(20674,5276,1,1,2,101),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20674,8729,1,1,200,684),-- Life Stone: level 64
(20674,8739,1,1,200,168),-- Mid-Grade Life Stone: level 64
(20674,8749,1,1,200,34),-- High-Grade Life Stone: level 64
(20675,1539,1,2,-1,277568),-- Greater Healing Potion
(20675,4042,1,1,-1,20818),-- Enria
(20675,4602,1,1,-1,2776),-- Greater Dye of Strength <Str+3 Dex-3>
(20675,57,605,1229,0,700000),-- Adena
(20675,60,1,1,1,36),-- Composite Armor
(20675,1977,1,1,1,3821),-- Composite Armor Temper
(20675,1867,1,1,2,71429),-- Animal Skin
(20675,1872,1,1,2,66667),-- Animal Bone
(20675,1875,1,1,2,4566),-- Stone of Purity
(20675,1881,1,1,2,5952),-- Coarse Bone Powder
(20675,2353,1,1,2,106),-- Recipe: Samurai Longsword
(20675,4041,1,1,2,562),-- Mold Hardener
(20675,4184,1,1,2,199),-- Recipe: Sprite\'s Staff
(20675,5155,1,1,2,199),-- Recipe: Soulshot (B) Compressed Package (100%)
(20675,8725,1,1,200,2381),-- Life Stone: level 52
(20675,8735,1,1,200,596),-- Mid-Grade Life Stone: level 52
(20676,1807,1,1,-1,2790),-- Recipe: Soulshot: A-Grade
(20676,5478,1,1,-1,17595),-- Sealed Dark Crystal Leather Armor Pattern
(20676,5479,1,1,-1,12381),-- Sealed Tallum Leather Armor Pattern
(20676,57,1015,2058,0,700000),-- Adena
(20676,12341,1,1,1,488),-- Common Item - Earring of Black Ore
(20676,12343,1,1,1,732),-- Common Item - Ring of Black Ore
(20676,12342,1,1,1,557),-- Common Item - Necklace of Black Ore
(20676,4052,1,1,1,3647),-- Earring of Black Ore Piece
(20676,4053,1,1,1,4415),-- Ring of Black Ore Gemstone
(20676,4054,1,1,1,2029),-- Necklace of Black Ore Beads
(20676,1867,1,1,2,100000),-- Animal Skin
(20676,1872,1,1,2,100000),-- Animal Bone
(20676,1882,1,1,2,16393),-- Leather
(20676,1894,1,1,2,2611),-- Crafted Leather
(20676,4042,1,1,2,1149),-- Enria
(20676,6036,1,1,2,3030),-- Greater Magic Haste Potion
(20676,8729,1,1,200,801),-- Life Stone: level 64
(20676,8739,1,1,200,383),-- Mid-Grade Life Stone: level 64
(20676,8759,1,1,200,70),-- Top-Grade Life Stone: level 64
(20677,3035,1,1,-1,3141),-- Recipe: Spiritshot A
(20677,5230,1,1,-1,3141),-- Recipe: Mithril Arrow (100%)
(20677,5482,1,1,-1,31092),-- Sealed Dark Crystalline Leggings Design
(20677,57,1142,2317,0,700000),-- Adena
(20677,11944,1,1,1,256),-- Common Item - Bow of Peril
(20677,4121,1,1,1,1516),-- Bow of Peril Shaft
(20677,1864,1,1,2,166667),-- Stem
(20677,1869,1,1,2,83333),-- Iron Ore
(20677,1880,1,1,2,8403),-- Steel
(20677,4043,1,1,2,2475),-- Asofe
(20677,5550,1,1,2,1196),-- Durable Metal Plate
(20677,8316,1,1,2,90),-- Recipe: Flaming Dragon Skull (60%)
(20677,8729,1,1,200,939),-- Life Stone: level 64
(20677,8739,1,1,200,342),-- Mid-Grade Life Stone: level 64
(20678,1918,1,1,-1,58115),-- Nassen\'s Earring Gemstone
(20678,1989,1,1,-1,40443),-- Divine Stocking Pattern
(20678,4142,1,1,-1,10322),-- Recipe: Avadon Gaiters
(20678,57,650,1321,0,700000),-- Adena
(20678,266,1,1,1,9),-- Great Pata
(20678,2120,1,1,1,1190),-- Great Pata Blade
(20678,1895,1,1,2,7874),-- Metallic Fiber
(20678,4040,1,1,2,853),-- Mold Lubricant
(20678,4042,1,1,2,1764),-- Enria
(20678,4043,1,1,2,853),-- Asofe
(20678,4128,1,1,2,739),-- Recipe: Adamantite Necklace
(20678,4134,1,1,2,341),-- Recipe: Zubei\'s Gaiters
(20678,5280,1,1,2,142),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(20678,8726,1,1,200,683),-- Life Stone: level 55
(20678,8736,1,1,200,39),-- Mid-Grade Life Stone: level 55
(20678,8746,1,1,200,39),-- High-Grade Life Stone: level 55
(20679,1806,1,1,-1,2764),-- Recipe: Soulshot: B-Grade
(20679,1920,1,1,-1,47285),-- Necklace of Binding Chain
(20679,4148,1,1,-1,6061),-- Recipe: Avadon Robe
(20679,57,668,1361,0,700000),-- Adena
(20679,203,1,1,1,3),-- Pa\'agrian Axe
(20679,204,1,1,1,2),-- Deadman\'s Staff
(20679,205,1,1,1,2),-- Ghoul\'s Staff
(20679,206,1,1,1,2),-- Demon\'s Staff
(20679,2112,1,1,1,248),-- Pa\'agrian Axe Blade
(20679,2116,1,1,1,241),-- Deadman\'s Staff Head
(20679,2117,1,1,1,241),-- Ghoul\'s Staff Head
(20679,2118,1,1,1,241),-- Demon\'s Staff Head
(20679,2503,1,1,1,2),-- Yaksa Mace
(20679,2969,1,1,1,246),-- Yaksa Mace Head
(20679,1867,1,1,2,58824),-- Animal Skin
(20679,1869,1,1,2,52632),-- Iron Ore
(20679,1870,1,1,2,45455),-- Coal
(20679,1871,1,1,2,45455),-- Charcoal
(20679,1872,1,1,2,66667),-- Animal Bone
(20679,4143,1,1,2,322),-- Recipe: Zubei\'s Leather Shirt
(20679,4149,1,1,2,563),-- Recipe: Avadon Circlet
(20679,4186,1,1,2,241),-- Recipe: Sword of Valhalla
(20679,5160,1,1,2,80),-- Recipe: Spiritshot (B) Compressed Package (100%)
(20679,8726,1,1,200,724),-- Life Stone: level 55
(20679,8736,1,1,200,402),-- Mid-Grade Life Stone: level 55
(20679,8746,1,1,200,10),-- High-Grade Life Stone: level 55
(20680,1873,1,2,-1,585562),-- Silver Nugget
(20680,1876,1,1,-1,292781),-- Mithril Ore
(20680,3034,1,1,-1,2928),-- Recipe: Spiritshot B
(20680,57,708,1442,0,700000),-- Adena
(20680,299,1,1,1,3),-- Orcish Poleaxe
(20680,301,1,1,1,4),-- Scorpion
(20680,303,1,1,1,4),-- Widow Maker
(20680,2113,1,1,1,437),-- Scorpion Blade
(20680,2114,1,1,1,437),-- Widowmaker Head
(20680,2122,1,1,1,412),-- Orcish Poleaxe Blade
(20680,1874,1,1,2,3448),-- Oriharukon Ore
(20680,1878,1,1,2,22727),-- Braided Hemp
(20680,1884,1,1,2,32258),-- Cord
(20680,1889,1,1,2,3205),-- Compound Braid
(20680,4441,1,1,2,966),-- Recipe: Avadon Shield
(20680,5220,1,1,2,1631),-- Metal Hardener
(20680,5280,1,1,2,121),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(20680,8311,1,1,2,60),-- Recipe: Kaim Vanul\'s Bones (100%)
(20680,8726,1,1,200,1511),-- Life Stone: level 55
(20680,8736,1,1,200,302),-- Mid-Grade Life Stone: level 55
(20680,8746,1,1,200,14),-- High-Grade Life Stone: level 55
(20681,1873,1,2,-1,403669),-- Silver Nugget
(20681,1908,1,1,-1,95566),-- Moonstone Earring Wire
(20681,2063,1,1,-1,7800),-- Battle Axe Head
(20681,57,489,993,0,700000),-- Adena
(20681,71,1,1,1,11),-- Flamberge
(20681,72,1,1,1,11),-- Stormbringer
(20681,2059,1,1,1,886),-- Flamberge Blade
(20681,2060,1,1,1,947),-- Stormbringer Blade
(20681,1866,1,1,2,31250),-- Suede
(20681,1871,1,1,2,52632),-- Charcoal
(20681,1885,1,1,2,3279),-- High Grade Suede
(20681,2225,1,1,2,1247),-- Recipe: Demon\'s Stockings
(20681,2321,1,1,2,208),-- Recipe: Stiletto
(20681,4041,1,1,2,519),-- Mold Hardener
(20681,4440,1,1,2,208),-- Recipe: Knuckle Duster
(20681,5220,1,1,2,1715),-- Metal Hardener
(20681,8723,1,1,200,1247),-- Life Stone: level 46
(20681,8733,1,1,200,779),-- Mid-Grade Life Stone: level 46
(20682,951,1,1,-1,1826),-- Scroll: Enchant Weapon (Grade C)
(20682,1876,1,2,-1,200889),-- Mithril Ore
(20682,4039,1,1,-1,33482),-- Mold Glue
(20682,57,484,990,0,700000),-- Adena
(20682,397,1,1,1,63),-- Mithril Shirt
(20682,1961,1,1,1,3798),-- Mithril Shirt Fabric
(20682,2387,1,1,1,101),-- Tempered Mithril Gaiters
(20682,2935,1,1,1,6310),-- Tempered Mithril Gaiters Fragment
(20682,1865,1,1,2,52632),-- Varnish
(20682,1870,1,1,2,58824),-- Coal
(20682,1874,1,1,2,3953),-- Oriharukon Ore
(20682,2323,1,1,2,228),-- Recipe: Elemental Bow
(20682,4039,1,1,2,1901),-- Mold Glue
(20682,4043,1,1,2,2128),-- Asofe
(20682,8723,1,1,200,2358),-- Life Stone: level 46
(20682,8733,1,1,200,608),-- Mid-Grade Life Stone: level 46
(20682,8743,1,1,200,152),-- High-Grade Life Stone: level 46
(20683,1889,1,1,-1,87064),-- Compound Braid
(20683,1974,1,2,-1,109837),-- Crimson Boot Fabric
(20683,2336,1,1,-1,3037),-- Recipe: Nirvana Axe
(20683,57,631,1286,0,700000),-- Adena
(20683,227,1,1,1,11),-- Stiletto
(20683,242,1,1,1,11),-- Soulfire Dirk
(20683,2083,1,1,1,962),-- Stiletto Edge
(20683,2084,1,1,1,962),-- Soulfire Dirk Blade
(20683,1868,1,1,2,111111),-- Thread
(20683,1873,1,1,2,22222),-- Silver Nugget
(20683,1889,1,1,2,3378),-- Compound Braid
(20683,2224,1,1,2,977),-- Recipe: Demon\'s Tunic
(20683,2230,1,1,2,888),-- Recipe: Theca Leather Gaiters
(20683,3005,1,1,2,2398),-- Recipe: Demon\'s Boots
(20683,4044,1,1,2,1066),-- Thons
(20683,5549,1,1,2,5405),-- Metallic Thread
(20683,8724,1,1,200,2309),-- Life Stone: level 49
(20683,8734,1,1,200,799),-- Mid-Grade Life Stone: level 49
(20683,8744,1,1,200,55),-- High-Grade Life Stone: level 49
(20684,1876,1,1,-1,273432),-- Mithril Ore
(20684,1966,1,1,-1,49915),-- Plated Leather Fragment
(20684,2078,1,1,-1,9084),-- Katana Blade
(20684,57,662,1345,0,700000),-- Adena
(20684,398,1,1,1,34),-- Plated Leather
(20684,399,1,1,1,32),-- Rind Leather Armor
(20684,418,1,1,1,55),-- Plated Leather Gaiters
(20684,419,1,1,1,51),-- Rind Leather Gaiters
(20684,1966,1,1,1,2765),-- Plated Leather Fragment
(20684,1967,1,1,1,4130),-- Plated Leather Gaiters Material
(20684,1975,1,1,1,2487),-- Rind Leather Armor Design
(20684,1976,1,1,1,4604),-- Rind Leather Gaiters Material
(20684,1539,1,1,2,11494),-- Greater Healing Potion
(20684,1866,1,1,2,34483),-- Suede
(20684,1871,1,1,2,66667),-- Charcoal
(20684,1877,1,1,2,2710),-- Adamantite Nugget
(20684,2341,1,1,2,177),-- Recipe: Sage\'s Staff
(20684,4040,1,1,2,834),-- Mold Lubricant
(20684,4041,1,1,2,209),-- Mold Hardener
(20684,6668,1,1,2,20000),-- Deluxe Chest Key - Grade 4
(20684,8724,1,1,200,1252),-- Life Stone: level 49
(20684,8734,1,1,200,209),-- Mid-Grade Life Stone: level 49
(20685,1970,1,1,-1,25480),-- Robe of Seal Fabric
(20685,2090,1,1,-1,6218),-- Delusional Blade
(20685,2166,1,1,-1,38678),-- Recipe: Earrings of Binding (100%)
(20685,57,551,1123,0,700000),-- Adena
(20685,195,1,1,1,9),-- Cursed Staff
(20685,2082,1,1,1,891),-- Cursed Staff Head
(20685,2502,1,1,1,9),-- Dwarven War Hammer
(20685,2968,1,1,1,853),-- Dwarven Warhammer Head
(20685,1867,1,1,2,76923),-- Animal Skin
(20685,1872,1,1,2,76923),-- Animal Bone
(20685,1875,1,1,2,4405),-- Stone of Purity
(20685,1881,1,1,2,7353),-- Coarse Bone Powder
(20685,2330,1,1,2,210),-- Recipe: Homunkulus\'s Sword
(20685,2351,1,1,2,210),-- Recipe: Scorpion
(20685,4041,1,1,2,210),-- Mold Hardener
(20685,8724,1,1,200,840),-- Life Stone: level 49
(20685,8734,1,1,200,420),-- Mid-Grade Life Stone: level 49
(20685,8744,1,1,200,368),-- High-Grade Life Stone: level 49
(20686,1880,1,1,-1,238035),-- Steel
(20686,2169,1,1,-1,42130),-- Recipe: Necklace of Binding
(20686,2231,1,1,-1,10260),-- Recipe: Full Plate Armor
(20686,57,1153,2343,0,700000),-- Adena
(20686,326,1,1,1,23),-- Heathen\'s Book
(20686,2110,1,1,1,2775),-- Heathen\'s Book Page
(20686,1866,1,1,2,62500),-- Suede
(20686,1871,1,1,2,76923),-- Charcoal
(20686,1877,1,1,2,3425),-- Adamantite Nugget
(20686,2354,1,1,2,201),-- Recipe: Deadman\'s Staff
(20686,2358,1,1,2,201),-- Recipe: Great Pata
(20686,4040,1,1,2,2513),-- Mold Lubricant
(20686,4041,1,1,2,457),-- Mold Hardener
(20686,4609,1,1,2,229),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20686,8725,1,1,200,2513),-- Life Stone: level 52
(20686,8735,1,1,200,914),-- Mid-Grade Life Stone: level 52
(20745,848,1,1,-1,4157),-- Enchanted Earring
(20745,1832,1,3,-1,491857),-- Greater Antidote
(20745,1901,1,1,-1,122285),-- Tiger\'s Eye Earring Stone
(20745,57,449,894,0,700000),-- Adena
(20745,393,1,1,1,104),-- Mithril Banded Mail
(20745,394,1,1,1,104),-- Reinforced Leather Shirt
(20745,415,1,1,1,165),-- Mithril Banded Gaiters
(20745,416,1,1,1,165),-- Reinforced Leather Gaiters
(20745,1938,1,1,1,5275),-- Mithril Banded Mail Material
(20745,1939,1,1,1,6892),-- Mithril Banded Gaiters Material
(20745,1867,1,1,2,166666),-- Animal Skin
(20745,1872,1,1,2,125000),-- Animal Bone
(20745,1882,1,1,2,31250),-- Leather
(20745,2156,1,1,2,7143),-- Recipe: Elven Necklace
(20745,2272,1,1,2,673),-- Recipe: Sword of Revolution
(20746,1061,1,2,-1,420371),-- Healing Potion
(20746,1900,1,1,-1,75887),-- Enchanted Necklace Chain
(20746,2153,1,1,-1,107871),-- Recipe: Tiger\'s Eye Earring
(20746,57,353,700,0,700000),-- Adena
(20746,259,1,1,1,118),-- Single-Edged Jamadhr
(20746,1865,1,1,2,66667),-- Varnish
(20746,1870,1,1,2,66667),-- Coal
(20746,1875,1,1,2,7407),-- Stone of Purity
(20746,1879,1,1,2,20833),-- Cokes
(20746,2184,1,1,2,7407),-- Recipe: Mithril Banded Gaiters
(20746,2272,1,1,2,1484),-- Recipe: Sword of Revolution
(20747,1900,1,1,-1,106736),-- Enchanted Necklace Chain
(20747,2024,1,1,-1,13897),-- Elven Bow Shaft
(20747,2203,1,1,-1,212542),-- Recipe: Mithril Boots
(20747,57,494,989,0,700000),-- Adena
(20747,46,1,1,1,1067),-- Bronze Helmet
(20747,627,1,1,1,1523),-- Aspis
(20747,1867,1,1,2,142857),-- Animal Skin
(20747,1872,1,1,2,142857),-- Animal Bone
(20747,1875,1,1,2,17241),-- Stone of Purity
(20747,1881,1,1,2,6849),-- Coarse Bone Powder
(20747,2183,1,1,2,3425),-- Recipe: Mithril Banded Mail
(20747,2274,1,1,2,544),-- Recipe: Skull Breaker
(20747,2282,1,1,2,544),-- Recipe: Cursed Maingauche
(20748,1061,1,2,-1,556782),-- Healing Potion
(20748,1866,1,1,-1,612460),-- Suede
(20748,1871,1,3,-1,918690),-- Charcoal
(20748,57,468,924,0,700000),-- Adena
(20748,275,1,1,1,52),-- Long Bow
(20748,276,1,1,1,52),-- Elven Bow
(20748,2024,1,1,1,2385),-- Elven Bow Shaft
(20748,956,1,1,2,2391),-- Scroll: Enchant Armor (Grade D)
(20748,1831,1,1,2,250000),-- Antidote
(20748,1866,1,1,2,62500),-- Suede
(20748,1871,1,1,2,111111),-- Charcoal
(20748,1885,1,1,2,14706),-- High Grade Suede
(20748,2190,1,1,2,7299),-- Recipe: Mithril Stockings
(20748,5220,1,1,2,2869),-- Metal Hardener
(20749,1871,1,3,-1,487784),-- Charcoal
(20749,2025,1,1,-1,13512),-- Dwarven Trident Edge
(20749,2153,1,1,-1,151721),-- Recipe: Tiger\'s Eye Earring
(20749,57,522,960,0,700000),-- Adena
(20749,46,1,1,1,1067),-- Bronze Helmet
(20749,627,1,1,1,1523),-- Aspis
(20749,1866,1,1,2,55556),-- Suede
(20749,1871,1,1,2,250000),-- Charcoal
(20749,1877,1,1,2,5332),-- Adamantite Nugget
(20749,2184,1,1,2,14000),-- Recipe: Mithril Banded Gaiters
(20750,1933,1,1,-1,111906),-- Dark Stocking Fabric
(20750,1935,1,1,-1,53140),-- Scale Mail Fragment
(20750,2183,1,1,-1,62579),-- Recipe: Mithril Banded Mail
(20750,57,501,945,0,700000),-- Adena
(20750,222,1,1,1,161),-- Poniard Dagger
(20750,1829,1,1,2,16667),-- Scroll of Escape: Clan Hall
(20750,1831,1,3,2,166666),-- Antidote
(20750,1867,1,3,2,111111),-- Animal Skin
(20750,1872,1,1,2,200000),-- Animal Bone
(20750,1882,1,1,2,33333),-- Leather
(20751,1894,1,1,-1,77496),-- Crafted Leather
(20751,1949,1,1,-1,246500),-- Mithril Gloves Design
(20751,2161,1,1,-1,214431),-- Recipe: Aquastone Ring
(20751,57,1720,3517,0,700000),-- Adena
(20751,62,1,1,1,93),-- Mithril Boots
(20751,1958,1,1,1,5213),-- Mithril Boots Design
(20751,2429,1,1,1,93),-- Chain Boots
(20751,2430,1,1,1,93),-- Karmian Boots
(20751,2431,1,1,1,93),-- Plated Leather Boots
(20751,2432,1,1,1,93),-- Dwarven Chain Boots
(20751,2433,1,1,1,93),-- Boots of Seal
(20751,2452,1,1,1,93),-- Reinforced Mithril Gloves
(20751,2453,1,1,1,93),-- Chain Gloves
(20751,2454,1,1,1,93),-- Karmian Gloves
(20751,2937,1,1,1,5213),-- Chain Boots Part
(20751,2938,1,1,1,5213),-- Karmian Boots Fabric
(20751,2939,1,1,1,5213),-- Plate Leather Boot Lining
(20751,2940,1,1,1,5213),-- Dwarven Chain Boots Part
(20751,2941,1,1,1,5213),-- Boots of Seal Pattern
(20751,2942,1,1,1,5213),-- Reinforced Mithril Gloves Design
(20751,2943,1,1,1,5213),-- Chain Gloves Design
(20751,2944,1,1,1,5213),-- Karmian Gloves Fabric
(20751,1894,1,1,2,5348),-- Crafted Leather
(20751,2216,1,1,2,10638),-- Recipe: Great Helmet
(20751,2225,1,1,2,5673),-- Recipe: Demon\'s Stockings
(20751,2321,1,1,2,910),-- Recipe: Stiletto
(20752,1805,1,1,-1,6668),-- Recipe: Soulshot: C-Grade
(20752,1905,1,1,-1,194586),-- Omen Beast\'s Eye Earring Gemstone
(20752,5549,1,1,-1,200035),-- Metallic Thread
(20752,57,933,1879,0,700000),-- Adena
(20752,353,1,1,1,129),-- Half Plate Armor
(20752,380,1,1,1,206),-- Plate Gaiters
(20752,1950,1,1,1,8151),-- Half Plate Temper
(20752,1951,1,1,1,12677),-- Plate Gaiters Part
(20752,1868,1,1,2,333333),-- Thread
(20752,1873,1,1,2,71429),-- Silver Nugget
(20752,1889,1,1,2,3300),-- Compound Braid
(20752,2160,1,1,2,6623),-- Recipe: Moonstone Earring
(20752,2312,1,1,2,628),-- Recipe: Crystallized Ice Bow
(20752,5159,1,1,2,3300),-- Recipe: Spiritshot (C) Compressed Package (100%)
(20752,5549,1,1,2,20000),-- Metallic Thread
(20753,1874,1,1,-1,186818),-- Oriharukon Ore
(20753,1875,1,1,-1,186818),-- Stone of Purity
(20753,4601,1,1,-1,6227),-- Greater Dye of Strength <Str+3 Con-3>
(20753,57,1500,3018,0,700000),-- Adena
(20753,75,1,1,1,5),-- Caliburs
(20753,76,1,1,1,5),-- Sword of Delusion
(20753,77,1,1,1,5),-- Tsurugi
(20753,84,1,1,1,5),-- Homunkulus\'s Sword
(20753,134,1,1,1,5),-- Sword of Nightmare
(20753,145,1,1,1,5),-- Sword of Whispering Death
(20753,2089,1,1,1,527),-- Caliburs Edge
(20753,2090,1,1,1,527),-- Delusional Blade
(20753,2091,1,1,1,527),-- Tsurugi Blade
(20753,2092,1,1,1,527),-- Homunkulus\'s Sword Blade
(20753,2095,1,1,1,527),-- Sword of Nightmare Blade
(20753,2096,1,1,1,527),-- Sword of Whispering Death Blade
(20753,1342,10,30,2,200000),-- Fine Steel Arrow
(20753,1868,1,3,2,125000),-- Thread
(20753,1873,1,1,2,35714),-- Silver Nugget
(20753,1889,1,1,2,5025),-- Compound Braid
(20753,2355,1,1,2,210),-- Recipe: Ghoul\'s Staff
(20753,3019,1,1,2,3434),-- Recipe: Full Plate Shield
(20753,4044,1,1,2,5025),-- Thons
(20753,4609,1,1,2,1259),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20753,5549,1,1,2,7576),-- Metallic Thread
(20753,8725,1,1,200,3774),-- Life Stone: level 52
(20753,8735,1,1,200,602),-- Mid-Grade Life Stone: level 52
(20754,1869,1,9,-1,772301),-- Iron Ore
(20754,1911,1,1,-1,229766),-- Earring of Protection Gemstone
(20754,1967,1,1,-1,168487),-- Plated Leather Gaiters Material
(20754,57,1636,3317,0,700000),-- Adena
(20754,94,1,1,1,48),-- Bec de Corbin
(20754,2079,1,1,1,4703),-- Bec De Corbin Blade
(20754,1864,1,3,2,166667),-- Stem
(20754,1869,1,1,2,166667),-- Iron Ore
(20754,1880,1,1,2,14085),-- Steel
(20754,2167,1,1,2,9174),-- Recipe: Ring of Ages
(20754,3009,1,1,2,4950),-- Recipe: Composite Boots
(20754,4043,1,1,2,7092),-- Asofe
(20754,5550,1,1,2,4237),-- Durable Metal Plate
(20754,8724,1,1,200,8475),-- Life Stone: level 49
(20754,8734,1,1,200,3534),-- Mid-Grade Life Stone: level 49
(20754,8744,1,1,200,707),-- High-Grade Life Stone: level 49
(20755,1061,1,1,-1,403626),-- Healing Potion
(20755,1900,1,1,-1,72865),-- Enchanted Necklace Chain
(20755,2184,1,1,-1,69956),-- Recipe: Mithril Banded Gaiters
(20755,57,337,674,0,700000),-- Adena
(20755,222,1,1,1,113),-- Poniard Dagger
(20755,1341,10,30,2,200000),-- Bone Arrow
(20755,1829,1,1,2,18868),-- Scroll of Escape: Clan Hall
(20755,1876,1,1,2,7407),-- Mithril Ore
(20755,1877,1,1,2,3175),-- Adamantite Nugget
(20755,1880,1,1,2,7407),-- Steel
(20755,1881,1,1,2,5291),-- Coarse Bone Powder
(20755,1882,1,1,2,13699),-- Leather
(20755,2274,1,1,2,1058),-- Recipe: Skull Breaker
(20756,1899,1,2,-1,101019),-- Necklace of Devotion Chain
(20756,1932,1,1,-1,102623),-- Bone Helmet Design
(20756,2262,1,1,-1,10039),-- Recipe: Elven Bow
(20756,57,333,651,0,700000),-- Adena
(20756,274,1,1,1,177),-- Strengthened Bow
(20756,1868,1,1,2,166666),-- Thread
(20756,1873,1,1,2,32258),-- Silver Nugget
(20756,1876,1,1,2,23810),-- Mithril Ore
(20756,2152,1,1,2,10753),-- Recipe: Enchanted Necklace
(20756,6035,1,1,2,3571),-- Magic Haste Potion
(20757,847,1,1,-1,3546),-- Red Crescent Earring
(20757,1868,1,3,-1,476958),-- Thread
(20757,2180,1,1,-1,35861),-- Recipe: Scale Mail
(20757,57,248,476,0,700000),-- Adena
(20757,291,1,1,1,66),-- Trident
(20757,2019,1,1,1,2879),-- Trident Edge
(20757,1875,1,1,2,2506),-- Stone of Purity
(20757,1879,1,1,2,7519),-- Cokes
(20757,1885,1,1,2,3759),-- High Grade Suede
(20757,2152,1,1,2,10000),-- Recipe: Enchanted Necklace
(20757,5549,1,1,2,2506),-- Metallic Thread
(20757,5550,1,1,2,2506),-- Durable Metal Plate
(20758,2167,1,1,-1,87619),-- Recipe: Ring of Ages
(20758,2946,1,1,-1,95994),-- Plate Leather Gloves Design
(20758,57,830,1692,0,700000),-- Adena
(20758,853,1,1,1,130),-- Earring of Protection
(20758,884,1,1,1,196),-- Ring of Protection
(20758,916,1,1,1,98),-- Necklace of Protection
(20758,1911,1,1,1,9434),-- Earring of Protection Gemstone
(20758,1912,1,1,1,13668),-- Ring of Protection Gemstone
(20758,1913,1,1,1,7713),-- Necklace of Protection Gemstone
(20758,952,1,1,2,1364),-- Scroll: Enchant Armor (Grade C)
(20758,1867,1,1,2,125000),-- Animal Skin
(20758,1872,1,1,2,125000),-- Animal Bone
(20758,1875,1,1,2,5814),-- Stone of Purity
(20758,1881,1,1,2,12658),-- Coarse Bone Powder
(20758,4041,1,1,2,1364),-- Mold Hardener
(20758,8724,1,1,200,1706),-- Life Stone: level 49
(20758,8734,1,1,200,682),-- Mid-Grade Life Stone: level 49
(20758,8744,1,1,200,341),-- High-Grade Life Stone: level 49
(20759,1912,1,1,-1,173912),-- Ring of Protection Gemstone
(20759,2166,1,1,-1,54709),-- Recipe: Earrings of Binding (100%)
(20759,2167,1,1,-1,82342),-- Recipe: Ring of Ages
(20759,57,781,1588,0,700000),-- Adena
(20759,398,1,1,1,41),-- Plated Leather
(20759,399,1,1,1,38),-- Rind Leather Armor
(20759,418,1,1,1,65),-- Plated Leather Gaiters
(20759,419,1,1,1,60),-- Rind Leather Gaiters
(20759,1966,1,1,1,3263),-- Plated Leather Fragment
(20759,1967,1,1,1,4875),-- Plated Leather Gaiters Material
(20759,1975,1,1,1,2936),-- Rind Leather Armor Design
(20759,1976,1,1,1,5435),-- Rind Leather Gaiters Material
(20759,952,1,1,2,364),-- Scroll: Enchant Armor (Grade C)
(20759,1865,1,1,2,83333),-- Varnish
(20759,1870,1,1,2,76923),-- Coal
(20759,1874,1,1,2,4000),-- Oriharukon Ore
(20759,2168,1,1,2,14580),-- Recipe: Necklace of Mermaid
(20759,4039,1,1,2,3636),-- Mold Glue
(20759,4043,1,1,2,3279),-- Asofe
(20759,8724,1,1,200,2915),-- Life Stone: level 49
(20759,8734,1,1,200,728),-- Mid-Grade Life Stone: level 49
(20760,951,1,1,-1,3046),-- Scroll: Enchant Weapon (Grade C)
(20760,1871,1,5,-1,837724),-- Charcoal
(20760,1911,1,1,-1,124615),-- Earring of Protection Gemstone
(20760,57,809,1650,0,700000),-- Adena
(20760,398,1,1,1,42),-- Plated Leather
(20760,399,1,1,1,39),-- Rind Leather Armor
(20760,418,1,1,1,67),-- Plated Leather Gaiters
(20760,419,1,1,1,62),-- Rind Leather Gaiters
(20760,1966,1,1,1,3388),-- Plated Leather Fragment
(20760,1967,1,1,1,5061),-- Plated Leather Gaiters Material
(20760,1975,1,1,1,3048),-- Rind Leather Armor Design
(20760,1976,1,1,1,5642),-- Rind Leather Gaiters Material
(20760,951,1,1,2,366),-- Scroll: Enchant Weapon (Grade C)
(20760,1866,1,1,2,52632),-- Suede
(20760,1871,1,1,2,76923),-- Charcoal
(20760,1877,1,1,2,4032),-- Adamantite Nugget
(20760,2327,1,1,2,1098),-- Recipe: Caliburs
(20760,2343,1,1,2,216),-- Recipe: Mace of the Underworld
(20760,4040,1,1,2,732),-- Mold Lubricant
(20760,4041,1,1,2,732),-- Mold Hardener
(20760,8724,1,1,200,1828),-- Life Stone: level 49
(20760,8734,1,1,200,1464),-- Mid-Grade Life Stone: level 49
(20760,8744,1,1,200,366),-- High-Grade Life Stone: level 49
(20761,1807,1,1,-1,9041),-- Recipe: Soulshot: A-Grade
(20761,1874,1,1,-1,452069),-- Oriharukon Ore
(20761,1875,1,1,-1,452069),-- Stone of Purity
(20761,57,3286,6671,0,700000),-- Adena
(20761,5287,1,1,1,15),-- Sealed Dark Crystal breastplate
(20761,5288,1,1,1,24),-- Sealed Dark Crystal Gaiters
(20761,5293,1,1,1,10),-- Sealed Tallum Plate Armor
(20761,5520,1,1,1,2153),-- Sealed Dark Crystal Breastplate Pattern
(20761,5521,1,1,1,1520),-- Sealed Tallum Plate Armor Pattern
(20761,5524,1,1,1,3291),-- Sealed Dark Crystal Gaiters Pattern
(20761,1868,1,3,2,200000),-- Thread
(20761,1873,1,1,2,83333),-- Silver Nugget
(20761,1889,1,1,2,13889),-- Compound Braid
(20761,4044,1,1,2,4808),-- Thons
(20761,4403,1,1,2,385),-- Red Dimensional Stone
(20761,4989,1,1,2,962),-- Recipe: Stockings of Doom (60%)
(20761,5338,1,1,2,192),-- Recipe: Sealed Majestic Leather Armor(60%)
(20761,5549,1,1,2,23256),-- Metallic Thread
(20761,8730,1,1,200,2309),-- Life Stone: level 67
(20761,8740,1,1,200,192),-- Mid-Grade Life Stone: level 67
(20762,1807,1,1,-1,6365),-- Recipe: Soulshot: A-Grade
(20762,5478,1,1,-1,40140),-- Sealed Dark Crystal Leather Armor Pattern
(20762,57,2311,4698,0,700000),-- Adena
(20762,5297,1,1,1,14),-- Sealed Dark Crystal Leather Armor
(20762,5298,1,1,1,22),-- Sealed Dark Crystalline Leggings
(20762,5301,1,1,1,10),-- Sealed Tallum Leather Armor
(20762,5478,1,1,1,1976),-- Sealed Dark Crystal Leather Armor Pattern
(20762,5479,1,1,1,1391),-- Sealed Tallum Leather Armor Pattern
(20762,5482,1,1,1,3102),-- Sealed Dark Crystalline Leggings Design
(20762,1807,1,1,2,314),-- Recipe: Soulshot: A-Grade
(20762,5336,1,1,2,192),-- Recipe: Sealed Leather Armor of Nightmare(60%)
(20762,1867,1,1,2,200000),-- Animal Skin
(20762,1872,1,1,2,200000),-- Animal Bone
(20762,1875,1,1,2,8621),-- Stone of Purity
(20762,1881,1,1,2,20000),-- Coarse Bone Powder
(20762,4041,1,1,2,1441),-- Mold Hardener
(20762,8730,1,1,200,1634),-- Life Stone: level 67
(20762,8740,1,1,200,577),-- Mid-Grade Life Stone: level 67
(20762,8750,1,1,200,96),-- High-Grade Life Stone: level 67
(20763,1873,1,1,-1,505674),-- Silver Nugget
(20763,1902,1,1,-1,128084),-- Elven Earring Beads
(20763,1941,1,1,-1,50276),-- Brigandine Temper
(20763,57,639,1270,0,700000),-- Adena
(20763,129,1,1,1,47),-- Sword of Revolution
(20763,2034,1,1,1,3453),-- Sword of Revolution Blade
(20763,1867,1,1,2,200000),-- Animal Skin
(20763,1872,1,1,2,166667),-- Animal Bone
(20763,1875,1,1,2,10638),-- Stone of Purity
(20763,1881,1,1,2,9709),-- Coarse Bone Powder
(20763,2146,1,1,2,5848),-- Recipe: Artisan\'s frame
(20763,2196,1,1,2,7752),-- Recipe: Plate Gaiters
(20764,1825,1,1,-1,80925),-- Recipe: Oriharukon
(20764,1902,1,1,-1,114787),-- Elven Earring Beads
(20764,1903,1,1,-1,188982),-- Elven Ring Piece
(20764,57,577,1143,0,700000),-- Adena
(20764,437,1,1,1,131),-- Mithril Tunic
(20764,470,1,1,1,210),-- Mithril Stockings
(20764,1944,1,1,1,7468),-- Mithril Tunic Fabric
(20764,1945,1,1,1,11629),-- Mithril Stocking Design
(20764,1867,1,1,2,142857),-- Animal Skin
(20764,1872,1,1,2,166667),-- Animal Bone
(20764,1875,1,1,2,6944),-- Stone of Purity
(20764,1881,1,1,2,14925),-- Coarse Bone Powder
(20764,2149,1,1,2,6944),-- Recipe: Metallic Fiber
(20764,5473,1,1,2,4950),-- Recipe: Durable Metal Plate(100%)
(20765,1873,1,1,-1,392112),-- Silver Nugget
(20765,1904,1,1,-1,91487),-- Elven Necklace Beads
(20765,2145,1,1,-1,70020),-- Recipe: Mithril Alloy
(20765,57,497,993,0,700000),-- Adena
(20765,850,1,1,1,181),-- Elven Earring
(20765,881,1,1,1,272),-- Elven Ring
(20765,913,1,1,1,136),-- Elven Necklace
(20765,1902,1,1,1,8558),-- Elven Earring Beads
(20765,1903,1,1,1,14089),-- Elven Ring Piece
(20765,1904,1,1,1,7883),-- Elven Necklace Beads
(20765,1868,1,1,2,200000),-- Thread
(20765,1873,1,1,2,41667),-- Silver Nugget
(20765,1876,1,1,2,27027),-- Mithril Ore
(20765,1895,1,1,2,35714),-- Metallic Fiber
(20765,2145,1,1,2,13514),-- Recipe: Mithril Alloy
(20765,2158,1,1,2,12500),-- Recipe: Mithril Ring
(20766,1869,1,3,-1,876362),-- Iron Ore
(20766,1901,1,1,-1,121045),-- Tiger\'s Eye Earring Stone
(20766,2154,1,1,-1,93829),-- Recipe: Elven Earring
(20766,57,445,889,0,700000),-- Adena
(20766,63,1,1,1,263),-- Gauntlets
(20766,553,1,1,1,263),-- Iron Boots
(20766,1940,1,1,1,9935),-- Iron Boots Design
(20766,2422,1,1,1,263),-- Reinforced Leather Boots
(20766,2423,1,1,1,263),-- Boots of Knowledge
(20766,2446,1,1,1,263),-- Reinforced Leather Gloves
(20766,2447,1,1,1,263),-- Gloves of Knowledge
(20766,1864,1,1,2,200000),-- Stem
(20766,1869,1,1,2,90909),-- Iron Ore
(20766,1878,1,1,2,37037),-- Braided Hemp
(20766,1884,1,1,2,40000),-- Cord
(20766,2155,1,1,2,11628),-- Recipe: Elven Ring
(20766,2195,1,1,2,1295),-- Recipe: Half Plate Armor
(20766,6035,1,1,2,5181),-- Magic Haste Potion
(20767,1879,1,1,-1,261151),-- Cokes
(20767,2076,1,1,-1,12289),-- Body Slasher Blade
(20767,2220,1,1,-1,32441),-- Recipe: Rind Leather Armor
(20767,57,760,1541,0,700000),-- Adena
(20767,281,1,1,1,33),-- Crystallized Ice Bow
(20767,2074,1,1,1,2623),-- Crystallized Ice Bow Shaft
(20767,1865,1,1,2,76923),-- Varnish
(20767,1870,1,1,2,71429),-- Coal
(20767,1875,1,1,2,3012),-- Stone of Purity
(20767,1879,1,1,2,15152),-- Cokes
(20767,2225,1,1,2,3012),-- Recipe: Demon\'s Stockings
(20767,2321,1,1,2,323),-- Recipe: Stiletto
(20767,2322,1,1,2,323),-- Recipe: Soulfire Dirk
(20767,4040,1,1,2,752),-- Mold Lubricant
(20767,8723,1,1,200,3759),-- Life Stone: level 46
(20768,1908,1,1,-1,143002),-- Moonstone Earring Wire
(20768,1909,1,1,-1,203930),-- Aquastone Ring Wire
(20768,2163,1,1,-1,77046),-- Recipe: Earring of Protection
(20768,57,728,1490,0,700000),-- Adena
(20768,631,1,1,1,179),-- Eldarake
(20768,1964,1,1,1,9612),-- Eldarake Temper
(20768,2413,1,1,1,126),-- Chain Hood
(20768,2495,1,1,1,179),-- Chain Shield
(20768,2936,1,1,1,8730),-- Chain Hood Pattern
(20768,2945,1,1,1,9612),-- Chain Shield Fragment
(20768,1832,1,1,2,83333),-- Greater Antidote
(20768,1865,1,1,2,71429),-- Varnish
(20768,1870,1,1,2,66667),-- Coal
(20768,1875,1,1,2,2439),-- Stone of Purity
(20768,1879,1,1,2,11364),-- Cokes
(20768,2163,1,1,2,3247),-- Recipe: Earring of Protection
(20768,2315,1,1,2,311),-- Recipe: Shamshir
(20768,2319,1,1,2,311),-- Recipe: Raid Sword
(20768,8723,1,1,200,1623),-- Life Stone: level 46
(20769,1885,1,1,-1,100134),-- High Grade Suede
(20769,4469,1,1,-1,6866),-- Dye of Strength <Str+1 Con-1>
(20769,5220,1,1,-1,48064),-- Metal Hardener
(20769,57,583,1182,0,700000),-- Adena
(20769,70,1,1,1,17),-- Claymore
(20769,2047,1,1,1,1327),-- Claymore Blade
(20769,2499,1,1,1,17),-- Elven Long Sword
(20769,2967,1,1,1,1309),-- Elven Long Sword Blade
(20769,1866,1,1,2,71429),-- Suede
(20769,1871,1,1,2,111111),-- Charcoal
(20769,1885,1,1,2,14706),-- High Grade Suede
(20769,2161,1,1,2,10638),-- Recipe: Aquastone Ring
(20769,2297,1,1,2,377),-- Recipe: Flamberge
(20769,5220,1,1,2,5747),-- Metal Hardener
(20769,5269,1,1,2,288),-- Recipe: Greater Soulshot (C) Compressed Package(100%)
(20770,1805,1,1,-1,4701),-- Recipe: Soulshot: C-Grade
(20770,1905,1,1,-1,137195),-- Omen Beast\'s Eye Earring Gemstone
(20770,1906,1,1,-1,193864),-- Mithril Ring Wire
(20770,57,696,1393,0,700000),-- Adena
(20770,159,1,1,1,8),-- Bonebreaker
(20770,187,1,1,1,8),-- Atuba Hammer
(20770,188,1,1,1,8),-- Ghost Staff
(20770,189,1,1,1,8),-- Staff of Life
(20770,190,1,1,1,8),-- Atuba Mace
(20770,2048,1,1,1,586),-- Bonebreaker Head
(20770,2049,1,1,1,586),-- Atuba Hammer Head
(20770,2050,1,1,1,621),-- Ghost Staff Head
(20770,2051,1,1,1,621),-- Staff of Life Shaft
(20770,2058,1,1,1,586),-- Atuba Mace Head
(20770,1805,1,1,2,338),-- Recipe: Soulshot: C-Grade
(20770,1867,1,1,2,142857),-- Animal Skin
(20770,1872,1,1,2,166667),-- Animal Bone
(20770,1882,1,1,2,25641),-- Leather
(20770,1894,1,1,2,2857),-- Crafted Leather
(20770,2298,1,1,2,442),-- Recipe: Stormbringer
(20770,5269,1,1,2,338),-- Recipe: Greater Soulshot (C) Compressed Package(100%)
(20771,948,1,1,-1,10950),-- Scroll: Enchant Armor (Grade B)
(20771,1875,1,1,-1,291994),-- Stone of Purity
(20771,4042,1,1,-1,72999),-- Enria
(20771,57,2129,4296,0,700000),-- Adena
(20771,171,1,1,1,4),-- Deadman\'s Glory
(20771,175,1,1,1,4),-- Art of Battle Axe
(20771,210,1,1,1,4),-- Staff of Evil Spirits
(20771,4116,1,1,1,943),-- Deadman\'s Glory Stone
(20771,4117,1,1,1,943),-- Art of Battle Axe Blade
(20771,4118,1,1,1,893),-- Evil Spirit Head
(20771,1895,1,1,2,50000),-- Metallic Fiber
(20771,4040,1,1,2,4313),-- Mold Lubricant
(20771,4042,1,1,2,3594),-- Enria
(20771,4043,1,1,2,9259),-- Asofe
(20771,4953,1,1,2,1852),-- Recipe: Avadon Gloves (60%)
(20771,5276,1,1,2,288),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20771,8729,1,1,200,1852),-- Life Stone: level 64
(20772,1875,1,1,-1,281689),-- Stone of Purity
(20772,1986,1,1,-1,44649),-- Full Plate Armor Temper
(20772,4613,1,1,-1,5828),-- Greater Dye of Strength <Str+4 Con-4>
(20772,57,2052,4155,0,700000),-- Adena
(20772,300,1,1,1,15),-- Great Axe
(20772,4113,1,1,1,3211),-- Great Axe Head
(20772,1867,1,1,2,166667),-- Animal Skin
(20772,1872,1,1,2,200000),-- Animal Bone
(20772,1875,1,1,2,7937),-- Stone of Purity
(20772,1881,1,1,2,24390),-- Coarse Bone Powder
(20772,4041,1,1,2,1980),-- Mold Hardener
(20772,4163,1,1,2,1033),-- Recipe: Blue Wolf Stockings
(20772,4618,1,1,2,216),-- Greater Dye of Dexterity <Dex+4 Con-4>
(20772,4999,1,1,2,1941),-- Recipe: Doom Boots (60%)
(20772,8728,1,1,200,661),-- Life Stone: level 61
(20772,8738,1,1,200,187),-- Mid-Grade Life Stone: level 61
(20773,1807,1,1,-1,4827),-- Recipe: Soulshot: A-Grade
(20773,5478,1,1,-1,30441),-- Sealed Dark Crystal Leather Armor Pattern
(20773,5479,1,1,-1,21420),-- Sealed Tallum Leather Armor Pattern
(20773,57,1748,3569,0,700000),-- Adena
(20773,6323,1,1,1,21),-- Sealed Phoenix Necklace
(20773,6324,1,1,1,28),-- Sealed Phoenix Earring
(20773,6325,1,1,1,41),-- Sealed Phoenix Ring
(20773,6341,1,1,1,4930),-- Sealed Phoenix Earring Gemstone
(20773,6343,1,1,1,3811),-- Sealed Phoenix Necklace Beads
(20773,6345,1,1,1,6818),-- Sealed Phoenix Ring Gemstone
(20773,730,1,1,2,65),-- Scroll: Enchant Armor (Grade A)
(20773,1868,1,1,2,250000),-- Thread
(20773,1873,1,1,2,76923),-- Silver Nugget
(20773,1889,1,1,2,16393),-- Compound Braid
(20773,4044,1,1,2,2315),-- Thons
(20773,4157,1,1,2,284),-- Recipe: Blue Wolf Gaiters
(20773,5549,1,1,2,15385),-- Metallic Thread
(20773,8730,1,1,200,851),-- Life Stone: level 67
(20773,8740,1,1,200,458),-- Mid-Grade Life Stone: level 67
(20773,8750,1,1,200,22),-- High-Grade Life Stone: level 67
(20774,1807,1,1,-1,5079),-- Recipe: Soulshot: A-Grade
(20774,1889,1,1,-1,253956),-- Compound Braid
(20774,5478,1,1,-1,32033),-- Sealed Dark Crystal Leather Armor Pattern
(20774,57,1844,3751,0,700000),-- Adena
(20774,5290,1,1,1,25),-- Sealed Dark Crystal Gloves
(20774,5291,1,1,1,25),-- Sealed Dark Crystal Boots
(20774,5295,1,1,1,25),-- Sealed Tallum Gloves
(20774,5296,1,1,1,25),-- Sealed Tallum Boots
(20774,5496,1,1,1,3298),-- Sealed Dark Crystal Boots Lining
(20774,5497,1,1,1,3298),-- Sealed Tallum Boots Lining
(20774,5508,1,1,1,3298),-- Sealed Dark Crystal Gloves Design
(20774,5509,1,1,1,3298),-- Sealed Tallum Gloves Design
(20774,1868,1,1,2,333333),-- Thread
(20774,1873,1,1,2,55556),-- Silver Nugget
(20774,1876,1,1,2,29412),-- Mithril Ore
(20774,1895,1,1,2,45455),-- Metallic Fiber
(20774,4042,1,1,2,3125),-- Enria
(20774,5271,1,1,2,99),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(20774,8730,1,1,200,1030),-- Life Stone: level 67
(20774,8740,1,1,200,163),-- Mid-Grade Life Stone: level 67
(20774,8750,1,1,200,36),-- High-Grade Life Stone: level 67
(20775,1800,1,1,-1,76617),-- Recipe: Piece Bone Breastplate
(20775,1897,1,1,-1,58010),-- Necklace of Wisdom Chain
(20775,2253,1,1,-1,9985),-- Recipe: Sword Breaker
(20775,57,119,191,0,700000),-- Adena
(20775,15,1,1,1,106),-- Short Spear
(20775,2011,1,1,1,1643),-- Short Spear Edge
(20775,1868,1,1,2,95958),-- Thread
(20775,1873,1,1,2,19192),-- Silver Nugget
(20775,2144,1,1,2,9596),-- Recipe: Compound Braid
(20776,1865,1,2,-1,297237),-- Varnish
(20776,1866,1,1,-1,198158),-- Suede
(20776,1897,1,1,-1,70771),-- Necklace of Wisdom Chain
(20776,57,137,239,0,700000),-- Adena
(20776,1101,1,1,1,866),-- Tunic of Devotion
(20776,1104,1,1,1,1383),-- Stockings of Devotion
(20776,735,1,1,2,1626),-- Potion of Alacrity
(20776,1831,1,1,2,52030),-- Antidote
(20776,1867,1,1,2,26015),-- Animal Skin
(20776,1869,1,1,2,19511),-- Iron Ore
(20776,1870,1,1,2,19511),-- Coal
(20776,1871,1,1,2,19511),-- Charcoal
(20776,1872,1,1,2,26015),-- Animal Bone
(20776,2140,1,1,2,3903),-- Recipe: High Grade Suede
(20776,5158,1,1,2,781),-- Recipe: Spiritshot (D) Compressed Package (100%)
(20777,1792,1,1,-1,16388),-- Recipe: Short Spear
(20777,1831,1,1,-1,594351),-- Antidote
(20777,1896,1,1,-1,94441),-- Necklace of Anguish Chain
(20777,57,114,168,0,700000),-- Adena
(20777,37,1,1,1,3313),-- Leather Shoes
(20777,49,1,1,1,3313),-- Gloves
(20777,1921,1,1,1,28317),-- Leather Shoes Texture
(20777,1794,1,1,2,3228),-- Recipe: Forest Bow
(20777,1868,1,1,2,87782),-- Thread
(20777,1873,1,1,2,17557),-- Silver Nugget
(20778,1902,1,1,-1,82244),-- Elven Earring Beads
(20778,1903,1,1,-1,135404),-- Elven Ring Piece
(20778,2290,1,1,-1,4510),-- Recipe: Mithril Dagger
(20778,57,343,761,0,700000),-- Adena
(20778,352,1,1,1,47),-- Brigandine Tunic
(20778,1941,1,1,1,2782),-- Brigandine Temper
(20778,2377,1,1,1,75),-- Mithril Scale Gaiters
(20778,2378,1,1,1,75),-- Brigandine Gaiters
(20778,2918,1,1,1,4149),-- Mithril Scale Gaiters Material
(20778,2919,1,1,1,4149),-- Brigandine Gaiters Material
(20778,1866,1,1,2,46624),-- Suede
(20778,1871,1,1,2,69936),-- Charcoal
(20778,1877,1,1,2,2798),-- Adamantite Nugget
(20778,2202,1,1,2,7151),-- Recipe: Assault Boots
(20778,2288,1,1,2,389),-- Recipe: Ghost Staff
(20778,2289,1,1,2,389),-- Recipe: Staff of Life
(20779,1866,1,1,-1,483184),-- Suede
(20779,1904,1,1,-1,67641),-- Elven Necklace Beads
(20779,2158,1,1,-1,89700),-- Recipe: Mithril Ring
(20779,57,307,611,0,700000),-- Adena
(20779,318,1,1,1,27),-- Crucifix of Blood
(20779,2042,1,1,1,1732),-- Crucifix of Blood Piece
(20779,1866,1,1,2,49954),-- Suede
(20779,1871,1,1,2,74931),-- Charcoal
(20779,1877,1,1,2,2998),-- Adamantite Nugget
(20779,2286,1,1,2,417),-- Recipe: Bonebreaker
(20779,2290,1,1,2,417),-- Recipe: Mithril Dagger
(20779,8723,1,1,200,875),-- Life Stone: level 46
(20780,1877,1,1,-1,23762),-- Adamantite Nugget
(20780,1901,1,1,-1,82051),-- Tiger\'s Eye Earring Stone
(20780,2980,1,1,-1,79524),-- Recipe: Brigandine Gauntlets
(20780,57,301,601,0,700000),-- Adena
(20780,157,1,1,1,22),-- Spiked Club
(20780,186,1,1,1,22),-- Staff of Magic
(20780,2027,1,1,1,1380),-- Spiked Club Head
(20780,1866,1,1,2,62500),-- Suede
(20780,1871,1,1,2,100000),-- Charcoal
(20780,1877,1,1,2,2375),-- Adamantite Nugget
(20780,2186,1,1,2,2849),-- Recipe: Brigandine Tunic
(20781,1901,1,1,-1,81246),-- Tiger\'s Eye Earring Stone
(20781,1938,1,1,-1,39544),-- Mithril Banded Mail Material
(20781,2192,1,1,-1,75029),-- Recipe: Kite Shield
(20781,57,299,595,0,700000),-- Adena
(20781,436,1,1,1,206),-- Tunic of Knowledge
(20781,469,1,1,1,329),-- Stockings of Knowledge
(20781,1875,1,1,2,1712),-- Stone of Purity
(20781,1879,1,1,2,6849),-- Cokes
(20781,1885,1,1,2,4115),-- High Grade Suede
(20781,2187,1,1,2,685),-- Recipe: Manticore Skin Shirt
(20781,2273,1,1,2,280),-- Recipe: Tarbar
(20781,5549,1,1,2,3425),-- Metallic Thread
(20781,5550,1,1,2,342),-- Durable Metal Plate
(20781,6667,1,1,2,25000),-- Deluxe Chest Key - Grade 3
(20782,1800,1,1,-1,78931),-- Recipe: Piece Bone Breastplate
(20782,1801,1,1,-1,126448),-- Recipe: Piece Bone Gaiters
(20782,1924,1,1,-1,80968),-- Leather Helmet Design
(20782,57,122,197,0,700000),-- Adena
(20782,256,1,1,1,218),-- Cestus
(20782,1867,1,1,2,49428),-- Animal Skin
(20782,1872,1,1,2,49428),-- Animal Bone
(20782,2150,1,1,2,17405),-- Recipe: Blue Diamond Necklace
(20782,5268,1,1,2,1483),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(20783,1900,1,1,-1,29540),-- Enchanted Necklace Chain
(20783,1935,1,1,-1,15084),-- Scale Mail Fragment
(20783,2203,1,1,-1,58822),-- Recipe: Mithril Boots
(20783,57,137,273,0,700000),-- Adena
(20783,316,1,1,1,10),-- Blood of Saints
(20783,317,1,1,1,10),-- Tome of Blood
(20783,2030,1,1,1,560),-- Tome of Blood Page
(20783,1874,1,1,2,2381),-- Oriharukon Ore
(20783,1878,1,1,2,8333),-- Braided Hemp
(20783,1884,1,1,2,18868),-- Cord
(20783,1889,1,1,2,1368),-- Compound Braid
(20783,2153,1,1,2,5952),-- Recipe: Tiger\'s Eye Earring
(20783,2977,1,1,2,2381),-- Recipe: Brigandine Boots
(20783,5220,1,1,2,821),-- Metal Hardener
(20784,2021,1,1,-1,3853),-- Spinebone Sword Blade
(20784,2153,1,1,-1,47454),-- Recipe: Tiger\'s Eye Earring
(20784,2184,1,1,-1,32051),-- Recipe: Mithril Banded Gaiters
(20784,57,154,309,0,700000),-- Adena
(20784,83,1,1,1,8),-- Sword of Magic
(20784,124,1,1,1,6),-- Two-Handed Sword
(20784,127,1,1,1,6),-- Crimson Sword
(20784,130,1,1,1,6),-- Elven Sword
(20784,144,1,1,1,8),-- Sword of Occult
(20784,2026,1,1,1,326),-- Two-Handed Sword Edge
(20784,1875,1,1,2,2747),-- Stone of Purity
(20784,1879,1,1,2,4950),-- Cokes
(20784,1885,1,1,2,1372),-- High Grade Suede
(20784,2250,1,1,2,10753),-- Recipe: Bone Arrow
(20784,5549,1,1,2,3846),-- Metallic Thread
(20784,5550,1,1,2,274),-- Durable Metal Plate
(20785,1875,1,1,-1,19616),-- Stone of Purity
(20785,1933,1,1,-1,34617),-- Dark Stocking Fabric
(20785,2022,1,1,-1,4583),-- Mace of Judgment Head
(20785,57,149,298,0,700000),-- Adena
(20785,606,1,1,1,26),-- Rip Gauntlets
(20785,1124,1,1,1,26),-- Boots of Power
(20785,1946,1,1,1,1285),-- Rip Gauntlets Pattern
(20785,1948,1,1,1,1285),-- Boots of Power Pattern
(20785,2424,1,1,1,26),-- Manticore Skin Boots
(20785,2425,1,1,1,26),-- Brigandine Boots
(20785,2426,1,1,1,26),-- Elven Mithril Boots
(20785,2448,1,1,1,26),-- Manticore Skin Gloves
(20785,2449,1,1,1,26),-- Brigandine Gauntlets
(20785,2450,1,1,1,26),-- Elven Mithril Gloves
(20785,2923,1,1,1,1285),-- Manticore Skin Boot Lining
(20785,2924,1,1,1,1285),-- Brigandine Boots Design
(20785,2925,1,1,1,1285),-- Elven Mithril Boots Design
(20785,2926,1,1,1,1285),-- Manticore Skin Gloves Lining
(20785,2927,1,1,1,1285),-- Brigandine Gauntlets Pattern
(20785,2928,1,1,1,1285),-- Elven Mithril Gloves Design
(20785,1341,10,30,2,66667),-- Bone Arrow
(20785,1875,1,1,2,1572),-- Stone of Purity
(20785,1879,1,1,2,2755),-- Cokes
(20785,1885,1,1,2,1179),-- High Grade Suede
(20785,2185,1,1,2,3937),-- Recipe: Iron Boots
(20785,2982,1,1,2,3937),-- Recipe: Brigandine Shield
(20785,5549,1,1,2,2358),-- Metallic Thread
(20785,5550,1,1,2,197),-- Durable Metal Plate
(20785,6667,1,1,2,13514),-- Deluxe Chest Key - Grade 3
(20786,2026,1,1,-1,3925),-- Two-Handed Sword Edge
(20786,2155,1,1,-1,52351),-- Recipe: Elven Ring
(20786,5550,1,1,-1,4349),-- Durable Metal Plate
(20786,57,166,330,0,700000),-- Adena
(20786,395,1,1,1,38),-- Manticore Skin Shirt
(20786,417,1,1,1,61),-- Manticore Skin Gaiters
(20786,1942,1,1,1,2150),-- Manticore Skin Shirt Texture
(20786,1943,1,1,1,3348),-- Manticore Skin Gaiters Pattern
(20786,1875,1,1,2,1770),-- Stone of Purity
(20786,1879,1,1,2,3745),-- Cokes
(20786,1885,1,1,2,3745),-- High Grade Suede
(20786,2278,1,1,2,197),-- Recipe: Strengthened Long Bow
(20786,2286,1,1,2,118),-- Recipe: Bonebreaker
(20786,2977,1,1,2,2950),-- Recipe: Brigandine Boots
(20786,5549,1,1,2,3145),-- Metallic Thread
(20786,5550,1,1,2,281),-- Durable Metal Plate
(20787,1870,1,1,-1,395470),-- Coal
(20787,2029,1,1,-1,4717),-- Gastraphetes Shaft
(20787,5549,1,1,-1,39547),-- Metallic Thread
(20787,57,199,397,0,700000),-- Adena
(20787,850,1,1,1,73),-- Elven Earring
(20787,881,1,1,1,110),-- Elven Ring
(20787,913,1,1,1,55),-- Elven Necklace
(20787,1902,1,1,1,3453),-- Elven Earring Beads
(20787,1903,1,1,1,5684),-- Elven Ring Piece
(20787,1904,1,1,1,3181),-- Elven Necklace Beads
(20787,1341,10,30,2,83333),-- Bone Arrow
(20787,1832,1,1,2,30303),-- Greater Antidote
(20787,1875,1,1,2,1190),-- Stone of Purity
(20787,1879,1,1,2,5882),-- Cokes
(20787,1885,1,1,2,3401),-- High Grade Suede
(20787,2188,1,1,2,2976),-- Recipe: Manticore Skin Gaiters
(20787,5473,1,1,2,1616),-- Recipe: Durable Metal Plate(100%)
(20787,5549,1,1,2,2551),-- Metallic Thread
(20787,5550,1,1,2,425),-- Durable Metal Plate
(20788,1901,1,2,-1,96683),-- Tiger\'s Eye Earring Stone
(20788,2186,1,1,-1,23411),-- Recipe: Brigandine Tunic
(20788,57,356,708,0,700000),-- Adena
(20788,260,1,1,1,77),-- Triple-Edged Jamadhr
(20788,1874,1,1,2,3135),-- Oriharukon Ore
(20788,1878,1,1,2,25000),-- Braided Hemp
(20788,1884,1,1,2,31250),-- Cord
(20788,1889,1,1,2,3135),-- Compound Braid
(20788,2189,1,1,2,1883),-- Recipe: Mithril Tunic
(20788,2275,1,1,2,628),-- Recipe: Heavy Bone Club
(20788,2293,1,1,2,259),-- Recipe: Glaive
(20788,5220,1,1,2,25131),-- Metal Hardener
(20789,2030,1,1,-1,7705),-- Tome of Blood Page
(20789,2154,1,1,-1,70536),-- Recipe: Elven Earring
(20789,2188,1,1,-1,47058),-- Recipe: Manticore Skin Gaiters
(20789,57,333,668,0,700000),-- Adena
(20789,393,1,1,1,77),-- Mithril Banded Mail
(20789,394,1,1,1,77),-- Reinforced Leather Shirt
(20789,415,1,1,1,123),-- Mithril Banded Gaiters
(20789,416,1,1,1,123),-- Reinforced Leather Gaiters
(20789,1938,1,1,1,3925),-- Mithril Banded Mail Material
(20789,1939,1,1,1,5129),-- Mithril Banded Gaiters Material
(20789,1341,10,30,2,166667),-- Bone Arrow
(20789,1661,1,1,2,14286),-- Thief Key
(20789,1874,1,1,2,5000),-- Oriharukon Ore
(20789,1878,1,1,2,27027),-- Braided Hemp
(20789,1884,1,1,2,28571),-- Cord
(20789,1889,1,1,2,2857),-- Compound Braid
(20789,2275,1,1,2,313),-- Recipe: Heavy Bone Club
(20789,5220,1,1,2,2857),-- Metal Hardener
(20790,1868,1,5,-1,843210),-- Thread
(20790,1902,1,1,-1,85432),-- Elven Earring Beads
(20790,2039,1,1,-1,8536),-- Bich\'Hwa Edge
(20790,57,428,852,0,700000),-- Adena
(20790,224,1,1,1,16),-- Maingauche
(20790,1660,1,1,1,16),-- Cursed Maingauche
(20790,2038,1,1,1,1152),-- Maingauche Edge
(20790,2044,1,1,1,1152),-- Cursed Maingauche Edge
(20790,1341,10,30,2,166667),-- Bone Arrow
(20790,1874,1,1,2,4367),-- Oriharukon Ore
(20790,1878,1,1,2,20408),-- Braided Hemp
(20790,1884,1,1,2,32258),-- Cord
(20790,1889,1,1,2,4695),-- Compound Braid
(20790,2147,1,1,2,4695),-- Recipe: Blacksmith\'s frame
(20790,2289,1,1,2,336),-- Recipe: Staff of Life
(20790,2984,1,1,2,7042),-- Recipe: Salamander Skin Boots
(20790,5220,1,1,2,1757),-- Metal Hardener
(20790,6667,1,1,2,34483),-- Deluxe Chest Key - Grade 3
(20791,1825,1,1,-1,103491),-- Recipe: Oriharukon
(20791,2041,1,1,-1,16162),-- War Pick Head
(20791,2196,1,1,-1,59380),-- Recipe: Plate Gaiters
(20791,57,736,1466,0,700000),-- Adena
(20791,352,1,1,1,84),-- Brigandine Tunic
(20791,1941,1,1,1,4964),-- Brigandine Temper
(20791,2377,1,1,1,134),-- Mithril Scale Gaiters
(20791,2378,1,1,1,134),-- Brigandine Gaiters
(20791,2918,1,1,1,7405),-- Mithril Scale Gaiters Material
(20791,2919,1,1,1,7405),-- Brigandine Gaiters Material
(20791,1876,1,1,2,29412),-- Mithril Ore
(20791,1877,1,1,2,4219),-- Adamantite Nugget
(20791,1880,1,1,2,9804),-- Steel
(20791,1881,1,1,2,16129),-- Coarse Bone Powder
(20791,1882,1,1,2,30303),-- Leather
(20791,2198,1,1,2,4219),-- Recipe: Sage\'s Rag
(20791,3020,1,1,2,595),-- Recipe: Elven Long Sword
(20792,1903,1,1,-1,224868),-- Elven Ring Piece
(20792,2035,1,1,-1,13647),-- Tarbar Head
(20792,2202,1,1,-1,137841),-- Recipe: Assault Boots
(20792,57,684,1363,0,700000),-- Adena
(20792,88,1,1,1,10),-- Morning Star
(20792,90,1,1,1,10),-- Goat Head Staff
(20792,158,1,1,1,10),-- Tarbar
(20792,169,1,1,1,10),-- Skull Breaker
(20792,172,1,1,1,10),-- Heavy Bone Club
(20792,2031,1,1,1,706),-- Morning Star Head
(20792,2032,1,1,1,644),-- Goat Head Staff Head
(20792,2035,1,1,1,706),-- Tarbar Head
(20792,2036,1,1,1,706),-- Skull Breaker Head
(20792,2037,1,1,1,706),-- Heavy Bone Club Head
(20792,1876,1,1,2,22727),-- Mithril Ore
(20792,1877,1,1,2,3215),-- Adamantite Nugget
(20792,1880,1,1,2,12195),-- Steel
(20792,1881,1,1,2,20000),-- Coarse Bone Powder
(20792,1882,1,1,2,25000),-- Leather
(20792,2198,1,1,2,2571),-- Recipe: Sage\'s Rag
(20792,3020,1,1,2,1285),-- Recipe: Elven Long Sword
(20793,1877,1,2,-1,58846),-- Adamantite Nugget
(20793,1881,1,2,-1,196153),-- Coarse Bone Powder
(20793,2037,1,1,-1,14892),-- Heavy Bone Club Head
(20793,57,740,1478,0,700000),-- Adena
(20793,93,1,1,1,28),-- Winged Spear
(20793,294,1,1,1,28),-- War Pick
(20793,2033,1,1,1,2121),-- Winged Spear Blade
(20793,2041,1,1,1,2121),-- War Pick Head
(20793,1341,10,30,2,333333),-- Bone Arrow
(20793,1876,1,1,2,21277),-- Mithril Ore
(20793,1877,1,1,2,4032),-- Adamantite Nugget
(20793,1880,1,1,2,9091),-- Steel
(20793,1881,1,1,2,14286),-- Coarse Bone Powder
(20793,1882,1,1,2,23810),-- Leather
(20793,2147,1,1,2,6024),-- Recipe: Blacksmith\'s frame
(20793,2205,1,1,2,3135),-- Recipe: Chain Gaiters
(20794,1866,1,3,-1,915655),-- Suede
(20794,1952,1,1,-1,43994),-- Salamander Skin Mail Texture
(20794,2991,1,1,-1,109007),-- Recipe: Karmian Boots
(20794,57,678,1353,0,700000),-- Adena
(20794,225,1,1,1,39),-- Mithril Dagger
(20794,2052,1,1,1,2991),-- Mithril Dagger Blade
(20794,1866,1,1,2,71429),-- Suede
(20794,1871,1,1,2,100000),-- Charcoal
(20794,1885,1,1,2,3521),-- High Grade Suede
(20794,2206,1,1,2,2600),-- Recipe: Mithril Shirt
(20794,4556,1,1,2,395),-- Greater Dye of Constitution <Con+1 Dex-1>
(20794,5220,1,1,2,7042),-- Metal Hardener
(20794,5274,1,1,2,329),-- Recipe: Greater Spiritshot (C) Compressed Package(100%)
(20795,2047,1,1,-1,11397),-- Claymore Blade
(20795,2161,1,1,-1,125863),-- Recipe: Aquastone Ring
(20795,4469,1,1,-1,7408),-- Dye of Strength <Str+1 Con-1>
(20795,57,654,1246,0,700000),-- Adena
(20795,851,1,1,1,180),-- Omen Beast\'s Eye Earring
(20795,882,1,1,1,269),-- Mithril Ring
(20795,914,1,1,1,134),-- Necklace of Darkness
(20795,1905,1,1,1,10554),-- Omen Beast\'s Eye Earring Gemstone
(20795,1906,1,1,1,14914),-- Mithril Ring Wire
(20795,1907,1,1,1,7548),-- Necklace of Darkness Gem
(20795,736,1,1,2,83333),-- Scroll of Escape
(20795,1867,1,1,2,166667),-- Animal Skin
(20795,1872,1,1,2,166667),-- Animal Bone
(20795,1882,1,1,2,13699),-- Leather
(20795,1894,1,1,2,13699),-- Crafted Leather
(20795,2303,1,1,2,406),-- Recipe: Skull Graver
(20795,2304,1,1,2,407),-- Recipe: Heavy Doom Hammer
(20796,1905,1,1,-1,134890),-- Omen Beast\'s Eye Earring Gemstone
(20796,2199,1,1,-1,58509),-- Recipe: Karmian Stockings
(20796,2930,1,1,-1,111783),-- Plate Helmet Design
(20796,57,671,1364,0,700000),-- Adena
(20796,396,1,1,1,163),-- Salamander Skin Mail
(20796,1952,1,1,1,11152),-- Salamander Skin Mail Texture
(20796,737,1,1,2,8621),-- Scroll of Resurrection
(20796,1868,1,1,2,166667),-- Thread
(20796,1873,1,1,2,37037),-- Silver Nugget
(20796,1889,1,1,2,4926),-- Compound Braid
(20796,2998,1,1,2,1235),-- Recipe: Chain Shield
(20796,3033,1,1,2,233),-- Recipe: Spiritshot C
(20796,3954,1,1,2,233),-- Recipe: Blessed Spiritshot C
(20796,4557,1,1,2,279),-- Greater Dye of Dexterity <Dex+1 Str-1>
(20796,5549,1,1,2,4926),-- Metallic Thread
(20796,6668,1,1,2,27027),-- Deluxe Chest Key - Grade 4
(20797,1908,1,1,-1,199462),-- Moonstone Earring Wire
(20797,2163,1,1,-1,107465),-- Recipe: Earring of Protection
(20797,4440,1,1,-1,7339),-- Recipe: Knuckle Duster
(20797,57,1025,2077,0,700000),-- Adena
(20797,397,1,1,1,132),-- Mithril Shirt
(20797,1961,1,1,1,7965),-- Mithril Shirt Fabric
(20797,2387,1,1,1,211),-- Tempered Mithril Gaiters
(20797,2935,1,1,1,13232),-- Tempered Mithril Gaiters Fragment
(20797,1864,1,1,2,200000),-- Stem
(20797,1869,1,1,2,125000),-- Iron Ore
(20797,1880,1,1,2,12195),-- Steel
(20797,2220,1,1,2,5208),-- Recipe: Rind Leather Armor
(20797,2225,1,1,2,1736),-- Recipe: Demon\'s Stockings
(20797,2321,1,1,2,497),-- Recipe: Stiletto
(20797,5550,1,1,2,6944),-- Durable Metal Plate
(20797,8733,1,1,200,1736),-- Mid-Grade Life Stone: level 46
(20797,8723,1,1,200,3472),-- Life Stone: level 46
(20798,1908,1,1,-1,161471),-- Moonstone Earring Wire
(20798,4042,1,1,-1,28419),-- Enria
(20798,4043,1,1,-1,56838),-- Asofe
(20798,57,826,1677,0,700000),-- Adena
(20798,71,1,1,1,18),-- Flamberge
(20798,72,1,1,1,18),-- Stormbringer
(20798,2059,1,1,1,1497),-- Flamberge Blade
(20798,2060,1,1,1,1600),-- Stormbringer Blade
(20798,1864,1,1,2,166667),-- Stem
(20798,1869,1,1,2,83333),-- Iron Ore
(20798,1880,1,1,2,8621),-- Steel
(20798,2220,1,1,2,1236),-- Recipe: Rind Leather Armor
(20798,2225,1,1,2,618),-- Recipe: Demon\'s Stockings
(20798,5550,1,1,2,3704),-- Durable Metal Plate
(20798,8733,1,1,200,1236),-- Mid-Grade Life Stone: level 46
(20798,8723,1,1,200,3096),-- Life Stone: level 46
(20799,2065,1,1,-1,13270),-- Skull Graver Head
(20799,2164,1,1,-1,132064),-- Recipe: Ring of Protection
(20799,2317,1,1,-1,5982),-- Recipe: Bec de Corbin
(20799,57,828,1693,0,700000),-- Adena
(20799,354,1,1,1,81),-- Chain Mail Shirt
(20799,381,1,1,1,129),-- Chain Gaiters
(20799,1959,1,1,1,5465),-- Chain Mail Shirt Material
(20799,1960,1,1,1,11116),-- Chain Gaiters Part
(20799,1866,1,1,2,71429),-- Suede
(20799,1871,1,1,2,90909),-- Charcoal
(20799,1877,1,1,2,6579),-- Adamantite Nugget
(20799,2230,1,1,2,2625),-- Recipe: Theca Leather Gaiters
(20799,2326,1,1,2,404),-- Recipe: Horn of Glory
(20799,4040,1,1,2,2625),-- Mold Lubricant
(20799,4041,1,1,2,657),-- Mold Hardener
(20799,8723,1,1,200,1969),-- Life Stone: level 46
(20799,8733,1,1,200,1314),-- Mid-Grade Life Stone: level 46
(20799,8743,1,1,200,612),-- High-Grade Life Stone: level 46 
(20800,1911,1,1,-1,119596),-- Earring of Protection Gemstone
(20800,1966,1,1,-1,58706),-- Plated Leather Fragment
(20800,2223,1,1,-1,62324),-- Recipe: Tower Shield
(20800,57,777,1584,0,700000),-- Adena
(20800,355,1,1,1,56),-- Dwarven Chain Mail Shirt
(20800,382,1,1,1,89),-- Dwarven Chain Gaiters
(20800,1968,1,1,1,4292),-- Dwarven Chain Mail Shirt Material
(20800,1969,1,1,1,6808),-- Dwarven Chain Gaiters Material
(20800,1867,1,1,2,100000),-- Animal Skin
(20800,1872,1,1,2,111111),-- Animal Bone
(20800,1875,1,1,2,4386),-- Stone of Purity
(20800,1881,1,1,2,8772),-- Coarse Bone Powder
(20800,2226,1,1,2,2193),-- Recipe: Mithril Gauntlets
(20800,2333,1,1,2,237),-- Recipe: Sword of Nightmare
(20800,4041,1,1,2,885),-- Mold Hardener
(20800,8724,1,1,200,730),-- Life Stone: level 49
(20800,8734,1,1,200,300),-- Mid-Grade Life Stone: level 49
(20800,8744,1,1,200,730),-- High-Grade Life Stone: level 49
(20801,951,1,1,-1,3528),-- Scroll: Enchant Weapon (Grade C)
(20801,2330,1,1,-1,4513),-- Recipe: Homunkulus\'s Sword
(20801,2946,1,1,-1,108463),-- Plate Leather Gloves Design
(20801,57,942,1907,0,700000),-- Adena
(20801,4233,1,1,1,30),-- Knuckle Duster
(20801,1539,1,1,2,11765),-- Greater Healing Potion
(20801,1868,1,1,2,142857),-- Thread
(20801,1873,1,1,2,28571),-- Silver Nugget
(20801,1876,1,1,2,19231),-- Mithril Ore
(20801,1895,1,1,2,7463),-- Metallic Fiber
(20801,2347,1,1,2,200),-- Recipe: Akat Long Bow
(20801,2351,1,1,2,180),-- Recipe: Scorpion
(20801,3010,1,1,2,2137),-- Recipe: Composite Helmet
(20801,4042,1,1,2,4274),-- Enria
(20801,4438,1,1,2,3067),-- Knuckle Duster Edge
(20801,6668,1,1,2,27778),-- Deluxe Chest Key - Grade 4
(20801,8724,1,1,200,2137),-- Life Stone: level 49
(20801,8734,1,1,200,251),-- Mid-Grade Life Stone: level 49
(20802,1975,1,1,-1,60146),-- Rind Leather Armor Design
(20802,2333,1,1,-1,4258),-- Recipe: Sword of Nightmare
(20802,4039,1,1,-1,61038),-- Mold Glue
(20802,57,886,1798,0,700000),-- Adena
(20802,325,1,1,1,29),-- Horn of Glory
(20802,2088,1,1,1,2859),-- Horn of Glory Fragment
(20802,737,1,1,2,11591),-- Scroll of Resurrection
(20802,1342,10,30,2,200000),-- Fine Steel Arrow
(20802,1894,1,1,2,4067),-- Crafted Leather
(20802,2334,1,1,2,270),-- Recipe: Sword of Whispering Death
(20802,4039,1,1,2,3864),-- Mold Glue
(20802,4041,1,1,2,1008),-- Mold Hardener
(20802,4044,1,1,2,3864),-- Thons
(20802,8724,1,1,200,1718),-- Life Stone: level 49
(20802,8734,1,1,200,764),-- Mid-Grade Life Stone: level 49
(20803,4042,1,1,-1,57307),-- Enria
(20803,4043,1,1,-1,114615),-- Asofe
(20803,4044,1,3,-1,114615),-- Thons
(20803,57,1666,3379,0,700000),-- Adena
(20803,231,1,1,1,17),-- Grace Dagger
(20803,233,1,1,1,17),-- Dark Screamer
(20803,2106,1,1,1,1938),-- Grace Dagger Edge
(20803,2107,1,1,1,1938),-- Dark Screamer Edge
(20803,1895,1,1,2,47619),-- Metallic Fiber
(20803,2356,1,1,2,332),-- Recipe: Demon\'s Staff
(20803,2360,1,1,2,577),-- Recipe: Orcish Poleaxe
(20803,3014,1,1,2,1730),-- Recipe: Full Plate Boots
(20803,4040,1,1,2,2309),-- Mold Lubricant
(20803,4042,1,1,2,2309),-- Enria
(20803,4043,1,1,2,4032),-- Asofe
(20803,8725,1,1,200,5780),-- Life Stone: level 52
(20803,8735,1,1,200,1730),-- Mid-Grade Life Stone: level 52
(20804,1953,1,1,-1,53848),-- Sage\'s Rag Lining
(20804,2160,1,1,-1,108461),-- Recipe: Moonstone Earring
(20804,4470,1,1,-1,9607),-- Dye of Strength <Str+1 Dex-1>
(20804,57,830,1660,0,700000),-- Adena
(20804,630,1,1,1,274),-- Square Shield
(20804,1956,1,1,1,15971),-- Square Shield Fragment
(20804,2412,1,1,1,192),-- Plate Helmet
(20804,2494,1,1,1,274),-- Plate Shield
(20804,2930,1,1,1,11343),-- Plate Helmet Design
(20804,2934,1,1,1,15971),-- Plate Shield Fragment
(20804,735,1,1,2,9901),-- Potion of Alacrity
(20804,1864,1,3,2,125000),-- Stem
(20804,1869,1,1,2,125000),-- Iron Ore
(20804,1878,1,1,2,50000),-- Braided Hemp
(20804,1884,1,1,2,71429),-- Cord
(20804,2207,1,1,2,3460),-- Recipe: Karmian Tunic
(20804,2298,1,1,2,566),-- Recipe: Stormbringer
(20804,5274,1,1,2,200),-- Recipe: Greater Spiritshot (C) Compressed Package(100%)
(20805,1905,1,1,-1,176987),-- Omen Beast\'s Eye Earring Gemstone
(20805,2053,1,1,-1,15044),-- Scallop Jamadhr Edge
(20805,2162,1,1,-1,87895),-- Recipe: Aquastone Necklace
(20805,57,892,1903,0,700000),-- Adena
(20805,159,1,1,1,11),-- Bonebreaker
(20805,187,1,1,1,11),-- Atuba Hammer
(20805,188,1,1,1,11),-- Ghost Staff
(20805,189,1,1,1,11),-- Staff of Life
(20805,190,1,1,1,11),-- Atuba Mace
(20805,2048,1,1,1,756),-- Bonebreaker Head
(20805,2049,1,1,1,756),-- Atuba Hammer Head
(20805,2050,1,1,1,801),-- Ghost Staff Head
(20805,2051,1,1,1,801),-- Staff of Life Shaft
(20805,2058,1,1,1,756),-- Atuba Mace Head
(20805,1895,1,1,2,62500),-- Metallic Fiber
(20805,2205,1,1,2,6667),-- Recipe: Chain Gaiters
(20805,2305,1,1,2,794),-- Recipe: Crystal Staff
(20805,5164,1,1,2,1006),-- Recipe: Blessed Spiritshot (C) Compressed Package (100%)
(20805,6668,1,1,2,76923),-- Deluxe Chest Key - Grade 4
(20806,1805,1,1,-1,5824),-- Recipe: Soulshot: C-Grade
(20806,2204,1,1,-1,34598),-- Recipe: Chain Mail Shirt
(20806,2934,1,2,-1,198321),-- Plate Shield Fragment
(20806,57,846,1718,0,700000),-- Adena
(20806,70,1,1,1,25),-- Claymore
(20806,2047,1,1,1,1929),-- Claymore Blade
(20806,2499,1,1,1,25),-- Elven Long Sword
(20806,2967,1,1,1,1903),-- Elven Long Sword Blade
(20806,1865,1,1,2,142857),-- Varnish
(20806,1870,1,1,2,142857),-- Coal
(20806,1875,1,1,2,9346),-- Stone of Purity
(20806,1879,1,1,2,22727),-- Cokes
(20806,2300,1,1,2,835),-- Recipe: Scythe
(20806,2999,1,1,2,11765),-- Recipe: Plated Leather Gloves
(20806,5159,1,1,2,152); -- Recipe: Spiritshot (C) Compressed Package (100%)

INSERT INTO `droplist` VALUES 
(20807,1908,1,2,-1,173910),-- Moonstone Earring Wire
(20807,1910,1,1,-1,138290),-- Aquastone Necklace Chain
(20807,2164,1,2,-1,141269),-- Recipe: Ring of Protection
(20807,57,887,1813,0,700000),-- Adena
(20807,439,1,1,1,115),-- Karmian Tunic
(20807,471,1,1,1,184),-- Karmian Stockings
(20807,1954,1,1,1,11537),-- Karmian Stocking Design
(20807,1962,1,1,1,6944),-- Karmian Tunic Pattern
(20807,737,1,1,2,10204),-- Scroll of Resurrection
(20807,1866,1,1,2,41667),-- Suede
(20807,1871,1,1,2,71429),-- Charcoal
(20807,1885,1,1,2,4878),-- High Grade Suede
(20807,2221,1,1,2,1328),-- Recipe: Rind Leather Gaiters
(20807,2225,1,1,2,885),-- Recipe: Demon\'s Stockings
(20807,2316,1,1,2,442),-- Recipe: Katana
(20807,2321,1,1,2,336),-- Recipe: Stiletto
(20807,5220,1,1,2,4878),-- Metal Hardener
(20807,8723,1,1,200,1770),-- Life Stone: level 46
(20807,8733,1,1,200,885),-- Mid-Grade Life Stone: level 46
(20807,8743,1,1,200,442),-- High-Grade Life Stone: level 46
(20808,1908,1,1,-1,215579),-- Moonstone Earring Wire
(20808,1954,1,1,-1,121027),-- Karmian Stocking Design
(20808,2163,1,1,-1,116149),-- Recipe: Earring of Protection
(20808,57,1103,2236,0,700000),-- Adena
(20808,439,1,1,1,142),-- Karmian Tunic
(20808,471,1,1,1,227),-- Karmian Stockings
(20808,1954,1,1,1,14301),-- Karmian Stocking Design
(20808,1962,1,1,1,8608),-- Karmian Tunic Pattern
(20808,1894,1,1,2,5181),-- Crafted Leather
(20808,2324,1,1,2,625),-- Recipe: Elven Bow of Nobility
(20808,4039,1,1,2,4032),-- Mold Glue
(20808,4041,1,1,2,1152),-- Mold Hardener
(20808,4044,1,1,2,3460),-- Thons
(20808,6668,1,1,2,55556),-- Deluxe Chest Key - Grade 4
(20808,8723,1,1,200,4608),-- Life Stone: level 46
(20808,8733,1,1,200,576),-- Mid-Grade Life Stone: level 46
(20809,4049,1,1,-1,211817),-- Adamantite Earring Gemstone
(20809,4439,1,1,-1,278503),-- Avadon Shield Fragment
(20809,4973,1,1,-1,43704),-- Recipe: Earring of Black Ore (70%)
(20809,57,1955,3973,0,700000),-- Adena
(20809,357,1,1,1,15),-- Zubei\'s Breastplate
(20809,383,1,1,1,23),-- Zubei\'s Gaiters
(20809,2376,1,1,1,15),-- Avadon Breastplate
(20809,2379,1,1,1,23),-- Avadon Gaiters
(20809,4056,1,1,1,2699),-- Zubei\'s Breastplate Part
(20809,4057,1,1,1,4171),-- Zubei\'s Gaiter Material
(20809,4064,1,1,1,2699),-- Avadon Breastplate Part
(20809,4065,1,1,1,4171),-- Avadon Gaiters Material
(20809,1868,1,1,2,250000),-- Thread
(20809,1873,1,1,2,41667),-- Silver Nugget
(20809,1889,1,1,2,7194),-- Compound Braid
(20809,4044,1,1,2,3731),-- Thons
(20809,4158,1,1,2,317),-- Recipe: Doom Plate Armor
(20809,4981,1,1,2,464),-- Recipe: Blue Wolf Breastplate (60%)
(20809,4999,1,1,2,1799),-- Recipe: Doom Boots (60%)
(20809,5549,1,1,2,11905),-- Metallic Thread
(20809,8728,1,1,200,3322),-- Life Stone: level 61
(20809,8738,1,1,200,692),-- Mid-Grade Life Stone: level 61
(20810,947,1,1,-1,1615),-- Scroll: Enchant Weapon (Grade B)
(20810,4109,1,1,-1,22751),-- Kris Edge
(20810,4974,1,1,-1,65557),-- Recipe: Ring of Black Ore (70%)
(20810,57,1955,3974,0,700000),-- Adena
(20810,91,1,1,1,7),-- Heavy War Axe
(20810,92,1,1,1,7),-- Sprite\'s Staff
(20810,4105,1,1,1,1707),-- Heavy War Axe Head
(20810,4106,1,1,1,1679),-- Sprite\'s Staff Head
(20810,1874,1,1,2,6250),-- Oriharukon Ore
(20810,1878,1,1,2,52632),-- Braided Hemp
(20810,1884,1,1,2,83333),-- Cord
(20810,1889,1,1,2,6536),-- Compound Braid
(20810,4159,1,1,2,906),-- Recipe: Blue Wolf Leather Armor
(20810,5220,1,1,2,4525),-- Metal Hardener
(20810,8728,1,1,200,1812),-- Life Stone: level 61
(20810,8738,1,1,200,201),-- Mid-Grade Life Stone: level 61
(20810,8748,1,1,200,403),-- High-Grade Life Stone: level 61
(20810,8758,1,1,200,101),-- Top-Grade Life Stone: level 61
(20811,947,1,1,-1,1909),-- Scroll: Enchant Weapon (Grade B)
(20811,1875,1,2,-1,318218),-- Stone of Purity
(20811,4040,1,1,-1,95465),-- Mold Lubricant
(20811,57,2315,4695,0,700000),-- Adena
(20811,78,1,1,1,6),-- Great Sword
(20811,142,1,1,1,6),-- Keshanberk
(20811,148,1,1,1,6),-- Sword of Valhalla
(20811,4104,1,1,1,1345),-- Great Sword Blade
(20811,4107,1,1,1,1324),-- Keshanberk Blade
(20811,4108,1,1,1,1324),-- Sword of Valhalla Blade
(20811,1865,1,1,2,125000),-- Varnish
(20811,1870,1,1,2,125000),-- Coal
(20811,1875,1,1,2,10000),-- Stone of Purity
(20811,1879,1,1,2,23256),-- Cokes
(20811,4040,1,1,2,2933),-- Mold Lubricant
(20811,4167,1,1,2,1200),-- Recipe: Blue Wolf Boots
(20811,4401,1,1,2,133),-- Green Dimensional Stone
(20811,4619,1,1,2,133),-- Greater Dye of Intelligence <Int+4 Men-4>
(20811,6670,1,1,2,23256),-- Deluxe Chest Key - Grade 6
(20811,8728,1,1,200,1733),-- Life Stone: level 61
(20811,8738,1,1,200,933),-- Mid-Grade Life Stone: level 61
(20811,8748,1,1,200,118),-- High-Grade Life Stone: level 61
(20812,1875,1,1,-1,314143),-- Stone of Purity
(20812,4613,1,1,-1,6500),-- Greater Dye of Strength <Str+4 Con-4>
(20812,4617,1,1,-1,6500),-- Greater Dye of Dexterity <Dex+4 Str-4>
(20812,57,2288,4634,0,700000),-- Adena
(20812,284,1,1,1,17),-- Dark Elven Long Bow
(20812,4112,1,1,1,3940),-- Dark Elven Longbow Shaft
(20812,1867,1,1,2,250000),-- Animal Skin
(20812,1872,1,1,2,250000),-- Animal Bone
(20812,1875,1,1,2,7692),-- Stone of Purity
(20812,1881,1,1,2,24390),-- Coarse Bone Powder
(20812,4041,1,1,2,2410),-- Mold Hardener
(20812,4620,1,1,2,965),-- Greater Dye of Intelligence <Int+4 Wit-4>
(20812,8728,1,1,200,3861),-- Life Stone: level 61
(20812,8738,1,1,200,176),-- Mid-Grade Life Stone: level 61
(20813,4052,1,1,-1,151920),-- Earring of Black Ore Piece
(20813,4982,1,1,-1,21348),-- Recipe: Blue Wolf Gaiters (60%)
(20813,5230,1,1,-1,5721),-- Recipe: Mithril Arrow (100%)
(20813,57,2083,4219,0,700000),-- Adena
(20813,2398,1,1,1,14),-- Blue Wolf Tunic
(20813,2399,1,1,1,14),-- Tunic of Doom
(20813,2403,1,1,1,21),-- Blue Wolf Stockings
(20813,2404,1,1,1,21),-- Stockings of Doom
(20813,4084,1,1,1,2753),-- Blue Wolf Tunic Fabric
(20813,4085,1,1,1,2753),-- Tunic of Doom Pattern
(20813,4086,1,1,1,3717),-- Blue Wolf Stockings Pattern
(20813,4087,1,1,1,3717),-- Stockings of Doom Pattern
(20813,1867,1,1,2,200000),-- Animal Skin
(20813,1872,1,1,2,200000),-- Animal Bone
(20813,1875,1,1,2,7519),-- Stone of Purity
(20813,1881,1,1,2,17241),-- Coarse Bone Powder
(20813,4041,1,1,2,469),-- Mold Hardener
(20813,4620,1,1,2,469),-- Greater Dye of Intelligence <Int+4 Wit-4>
(20813,8728,1,1,200,938),-- Life Stone: level 61
(20813,8738,1,1,200,938),-- Mid-Grade Life Stone: level 61
(20814,948,1,1,-1,12387),-- Scroll: Enchant Armor (Grade B)
(20814,4053,1,1,-1,212371),-- Ring of Black Ore Gemstone
(20814,5230,1,1,-1,6606),-- Recipe: Mithril Arrow (100%)
(20814,57,2405,4872,0,700000),-- Adena
(20814,601,1,1,1,46),-- Boots of Doom
(20814,2439,1,1,1,46),-- Blue Wolf Boots
(20814,2475,1,1,1,46),-- Doom Gloves
(20814,2487,1,1,1,46),-- Blue Wolf Gloves
(20814,4090,1,1,1,8211),-- Blue Wolf Boots Design
(20814,4091,1,1,1,8211),-- Doom Gloves Part
(20814,4096,1,1,1,8211),-- Blue Wolf Gloves Fabric
(20814,4099,1,1,1,8211),-- Doom Boots Part
(20814,1866,1,1,2,100000),-- Suede
(20814,1871,1,1,2,166667),-- Charcoal
(20814,1877,1,1,2,5780),-- Adamantite Nugget
(20814,4040,1,1,2,3937),-- Mold Lubricant
(20814,4041,1,1,2,1445),-- Mold Hardener
(20814,4975,1,1,2,1052),-- Recipe: Necklace of Black Ore (70%)
(20814,5276,1,1,2,131),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20814,8729,1,1,200,2232),-- Life Stone: level 64
(20814,8739,1,1,200,263),-- Mid-Grade Life Stone: level 64
(20814,8749,1,1,200,131),-- High-Grade Life Stone: level 64
(20815,4052,1,1,-1,217631),-- Earring of Black Ore Piece
(20815,4054,1,1,-1,121075),-- Necklace of Black Ore Beads
(20815,4984,1,1,-1,17414),-- Recipe: Blue Wolf Leather Armor (60%)
(20815,57,2984,6041,0,700000),-- Adena
(20815,2391,1,1,1,26),-- Blue Wolf Leather Armor
(20815,2392,1,1,1,26),-- Leather Armor of Doom
(20815,4082,1,1,1,5129),-- Blue Wolf Leather Armor Texture
(20815,4083,1,1,1,5129),-- Leather Armor of Doom Design
(20815,1865,1,1,2,200000),-- Varnish
(20815,1870,1,1,2,200000),-- Coal
(20815,1874,1,1,2,13889),-- Oriharukon Ore
(20815,4039,1,1,2,6410),-- Mold Glue
(20815,4043,1,1,2,5587),-- Asofe
(20815,5000,1,1,2,413),-- Recipe: Sword of Damascus (60%)
(20815,6670,1,1,2,37037),-- Deluxe Chest Key - Grade 6
(20815,8729,1,1,200,2475),-- Life Stone: level 64
(20815,8739,1,1,200,1238),-- Mid-Grade Life Stone: level 64
(20816,947,1,1,-1,2051),-- Scroll: Enchant Weapon (Grade B)
(20816,4039,1,1,-1,170952),-- Mold Glue
(20816,5230,1,1,-1,6838),-- Recipe: Mithril Arrow (100%)
(20816,57,2490,5040,0,700000),-- Adena
(20816,2391,1,1,1,22),-- Blue Wolf Leather Armor
(20816,2392,1,1,1,22),-- Leather Armor of Doom
(20816,4082,1,1,1,4279),-- Blue Wolf Leather Armor Texture
(20816,4083,1,1,1,4279),-- Leather Armor of Doom Design
(20816,1864,1,3,2,142857),-- Stem
(20816,1869,1,1,2,142857),-- Iron Ore
(20816,1878,1,1,2,55556),-- Braided Hemp
(20816,1884,1,1,2,100000),-- Cord
(20816,4039,1,1,2,4348),-- Mold Glue
(20816,4402,1,1,2,379),-- Blue Dimensional Stone
(20816,5001,1,1,2,189),-- Recipe: Lance (60%)
(20816,5276,1,1,2,189),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20816,8729,1,1,200,1321),-- Life Stone: level 64
(20816,8739,1,1,200,377),-- Mid-Grade Life Stone: level 64
(20817,4040,1,1,-1,99540),-- Mold Lubricant
(20817,4053,1,1,-1,213331),-- Ring of Black Ore Gemstone
(20817,5230,1,1,-1,6636),-- Recipe: Mithril Arrow (100%)
(20817,57,2419,4891,0,700000),-- Adena
(20817,268,1,1,1,12),-- Bellion Cestus
(20817,4120,1,1,1,3212),-- Bellion Cestus Edge
(20817,1865,1,1,2,166667),-- Varnish
(20817,1870,1,1,2,166667),-- Coal
(20817,1875,1,1,2,12500),-- Stone of Purity
(20817,1879,1,1,2,22727),-- Cokes
(20817,4040,1,1,2,3861),-- Mold Lubricant
(20817,4174,1,1,2,2890),-- Recipe: Zubei\'s Boots
(20817,4402,1,1,2,241),-- Blue Dimensional Stone
(20817,8729,1,1,200,2653),-- Life Stone: level 64
(20817,8739,1,1,200,303),-- Mid-Grade Life Stone: level 64
(20817,8759,1,1,200,212),-- Top-Grade Life Stone: level 64
(20818,4077,1,1,-1,223455),-- Doom Shield Fragment
(20818,4105,1,1,-1,28426),-- Heavy War Axe Head
(20818,4620,1,1,-1,6852),-- Greater Dye of Intelligence <Int+4 Wit-4>
(20818,57,2415,4882,0,700000),-- Adena
(20818,287,1,1,1,12),-- Bow of Peril
(20818,4121,1,1,1,3196),-- Bow of Peril Shaft
(20818,1866,1,1,2,83333),-- Suede
(20818,1871,1,1,2,142857),-- Charcoal
(20818,1885,1,1,2,11494),-- High Grade Suede
(20818,4041,1,1,2,1359),-- Mold Hardener
(20818,4401,1,1,2,227),-- Green Dimensional Stone
(20818,4958,1,1,2,6329),-- Recipe: Zubei\'s Boots (60%)
(20818,5416,1,1,2,210),-- Recipe: Sealed Dark Crystal Breastplate(60%)
(20818,5220,1,1,2,7246),-- Metal Hardener
(20818,5276,1,1,2,245),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20818,8729,1,1,200,680),-- Life Stone: level 64
(20818,8739,1,1,200,160),-- Mid-Grade Life Stone: level 64
(20819,4053,1,1,-1,212010),-- Ring of Black Ore Gemstone
(20819,4078,1,1,-1,72249),-- Blue Wolf Breastplate Part
(20819,5230,1,1,-1,6595),-- Recipe: Mithril Arrow (100%)
(20819,57,2405,4861,0,700000),-- Adena
(20819,171,1,1,1,4),-- Deadman\'s Glory
(20819,175,1,1,1,4),-- Art of Battle Axe
(20819,210,1,1,1,4),-- Staff of Evil Spirits
(20819,4116,1,1,1,1065),-- Deadman\'s Glory Stone
(20819,4117,1,1,1,1065),-- Art of Battle Axe Blade
(20819,4118,1,1,1,1009),-- Evil Spirit Head
(20819,730,1,1,2,152),-- Scroll: Enchant Armor (Grade A)
(20819,1874,1,1,2,11494),-- Oriharukon Ore
(20819,1878,1,1,2,45455),-- Braided Hemp
(20819,1884,1,1,2,90909),-- Cord
(20819,1889,1,1,2,97096),-- Compound Braid
(20819,4159,1,1,2,518),-- Recipe: Blue Wolf Leather Armor
(20819,4402,1,1,2,366),-- Blue Dimensional Stone
(20819,5220,1,1,2,6173),-- Metal Hardener
(20819,8729,1,1,200,2208),-- Life Stone: level 64
(20819,8749,1,1,200,441),-- High-Grade Life Stone: level 64
(20820,4054,1,1,-1,167723),-- Necklace of Black Ore Beads
(20820,4080,1,1,-1,175157),-- Blue Wolf Gaiters Material
(20820,5230,1,1,-1,11354),-- Recipe: Mithril Arrow (100%)
(20820,57,4130,8372,0,700000),-- Adena
(20820,864,1,1,1,91),-- Earring of Black Ore
(20820,895,1,1,1,137),-- Ring of Black Ore
(20820,926,1,1,1,69),-- Necklace of Black Ore
(20820,4052,1,1,1,14841),-- Earring of Black Ore Piece
(20820,4053,1,1,1,17967),-- Ring of Black Ore Gemstone
(20820,4054,1,1,1,8257),-- Necklace of Black Ore Beads
(20820,1867,1,3,2,166667),-- Animal Skin
(20820,1872,1,3,2,111111),-- Animal Bone
(20820,1882,1,1,2,55556),-- Leather
(20820,1894,1,1,2,10309),-- Crafted Leather
(20820,4042,1,1,2,5263),-- Enria
(20820,4160,1,1,2,1605),-- Recipe: Leather Armor of Doom
(20820,5424,1,1,2,688),-- Recipe: Sealed Dark Crystal Gaiters(60%)
(20820,5276,1,1,2,479),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20820,8729,1,1,200,2525),-- Life Stone: level 64
(20820,8739,1,1,200,1147),-- Mid-Grade Life Stone: level 64
(20821,1894,1,1,-1,234416),-- Crafted Leather
(20821,4080,1,1,-1,137424),-- Blue Wolf Gaiters Material
(20821,4084,1,1,-1,116108),-- Blue Wolf Tunic Fabric
(20821,57,3241,6568,0,700000),-- Adena
(20821,97,1,1,1,16),-- Lance
(20821,4115,1,1,1,4305),-- Lance Blade
(20821,1867,1,3,2,166667),-- Animal Skin
(20821,1872,1,3,2,100000),-- Animal Bone
(20821,1882,1,1,2,43478),-- Leather
(20821,1894,1,1,2,9174),-- Crafted Leather
(20821,4042,1,1,2,5917),-- Enria
(20821,4195,1,1,2,890),-- Recipe: Art of Battle Axe
(20821,5276,1,1,2,376),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20821,8729,1,1,200,2075),-- Life Stone: level 64
(20821,8739,1,1,200,297),-- Mid-Grade Life Stone: level 64
(20822,1889,1,1,-1,447541),-- Compound Braid
(20822,1894,1,1,-1,235548),-- Crafted Leather
(20822,4077,1,1,-1,301984),-- Doom Shield Fragment
(20822,57,3256,6601,0,700000),-- Adena
(20822,97,1,1,1,16),-- Lance
(20822,4115,1,1,1,4326),-- Lance Blade
(20822,1868,1,3,2,200000),-- Thread
(20822,1873,1,1,2,76923),-- Silver Nugget
(20822,1889,1,1,2,9434),-- Compound Braid
(20822,4044,1,1,2,6452),-- Thons
(20822,4196,1,1,2,692),-- Recipe: Staff of Evil Spirits
(20822,4402,1,1,2,1153),-- Blue Dimensional Stone
(20822,5549,1,1,2,20408),-- Metallic Thread
(20822,6670,1,1,2,33333),-- Deluxe Chest Key - Grade 6
(20822,8729,1,1,200,3226),-- Life Stone: level 64
(20822,8739,1,1,200,923),-- Mid-Grade Life Stone: level 64
(20823,4043,1,1,-1,247340),-- Asofe
(20823,4088,1,1,-1,280431),-- Blue Wolf Helmet Design
(20823,4089,1,1,-1,280431),-- Doom Helmet Pattern
(20823,5416,1,1,-1,30303),-- Recipe: Sealed Dark Crystal Breastplate(60%)
(20823,57,3581,7317,0,700000),-- Adena
(20823,2398,1,1,1,23),-- Blue Wolf Tunic
(20823,2399,1,1,1,23),-- Tunic of Doom
(20823,2403,1,1,1,37),-- Blue Wolf Stockings
(20823,2404,1,1,1,37),-- Stockings of Doom
(20823,4084,1,1,1,4760),-- Blue Wolf Tunic Fabric
(20823,4085,1,1,1,4760),-- Tunic of Doom Pattern
(20823,4086,1,1,1,6427),-- Blue Wolf Stockings Pattern
(20823,4087,1,1,1,6427),-- Stockings of Doom Pattern
(20823,1864,1,3,2,200000),-- Stem
(20823,1869,1,1,2,200000),-- Iron Ore
(20823,1880,1,1,2,21277),-- Steel
(20823,4043,1,1,2,9709),-- Asofe
(20823,4402,1,1,2,691),-- Blue Dimensional Stone
(20823,8316,1,1,2,135),-- Recipe: Flaming Dragon Skull (60%)
(20823,4616,1,1,2,345),-- Greater Dye of Constitution <Con+4 Dex-4>
(20823,5550,1,1,2,2762),-- Durable Metal Plate
(20823,8730,1,1,200,3106),-- Life Stone: level 67
(20823,8740,1,1,200,2070),-- Mid-Grade Life Stone: level 67
(20824,4078,1,1,-1,102642),-- Blue Wolf Breastplate Part
(20824,4081,1,1,-1,69258),-- Doom Plate Armor Temper
(20824,5424,1,1,-1,19448),-- Recipe: Sealed Dark Crystal Gaiters(60%)
(20824,57,3391,6929,0,700000),-- Adena
(20824,864,1,1,1,75),-- Earring of Black Ore
(20824,895,1,1,1,113),-- Ring of Black Ore
(20824,926,1,1,1,57),-- Necklace of Black Ore
(20824,4052,1,1,1,12247),-- Earring of Black Ore Piece
(20824,4053,1,1,1,14827),-- Ring of Black Ore Gemstone
(20824,4054,1,1,1,6814),-- Necklace of Black Ore Beads
(20824,1865,1,1,2,200000),-- Varnish
(20824,1870,1,1,2,200000),-- Coal
(20824,1874,1,1,2,14085),-- Oriharukon Ore
(20824,4039,1,1,2,7194),-- Mold Glue
(20824,4043,1,1,2,6024),-- Asofe
(20824,4617,1,1,2,61),-- Greater Dye of Dexterity <Dex+4 Str-4>
(20824,5276,1,1,2,365),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20824,6036,1,1,2,7634),-- Greater Magic Haste Potion
(20824,8730,1,1,200,3226),-- Life Stone: level 67
(20824,8740,1,1,200,426),-- Mid-Grade Life Stone: level 67
(20824,8750,1,1,200,61),-- High-Grade Life Stone: level 67
(20825,1807,1,1,-1,9816),-- Recipe: Soulshot: A-Grade
(20825,1876,1,3,-1,736209),-- Mithril Ore
(20825,57,3566,7246,0,700000),-- Adena
(20825,5289,1,1,1,42),-- Sealed Dark Crystal Helmet
(20825,5292,1,1,1,60),-- Sealed Dark Crystal Shield
(20825,5294,1,1,1,42),-- Sealed Tallum Helmet
(20825,5494,1,1,1,7612),-- Sealed Dark Crystal Shield Fragment
(20825,5525,1,1,1,5603),-- Sealed Dark Crystal Helmet Design
(20825,5526,1,1,1,5603),-- Sealed Tallum Helm Design
(20825,1866,1,1,2,166667),-- Suede
(20825,1871,1,1,2,250000),-- Charcoal
(20825,1877,1,1,2,10000),-- Adamantite Nugget
(20825,4040,1,1,2,3906),-- Mold Lubricant
(20825,4041,1,1,2,2646),-- Mold Hardener
(20825,4402,1,1,2,529),-- Blue Dimensional Stone
(20825,4975,1,1,2,1786),-- Recipe: Necklace of Black Ore (70%)
(20825,8730,1,1,200,2513),-- Life Stone: level 67
(20825,8740,1,1,200,727),-- Mid-Grade Life Stone: level 67
(20825,8750,1,1,200,132),-- High-Grade Life Stone: level 67
(20826,4041,1,1,-1,58279),-- Mold Hardener
(20826,5478,1,1,-1,56358),-- Sealed Dark Crystal Leather Armor Pattern
(20826,57,3245,6597,0,700000),-- Adena
(20826,5287,1,1,1,15),-- Sealed Dark Crystal breastplate
(20826,5288,1,1,1,23),-- Sealed Dark Crystal Gaiters
(20826,5293,1,1,1,10),-- Sealed Tallum Plate Armor
(20826,5520,1,1,1,2129),-- Sealed Dark Crystal Breastplate Pattern
(20826,5521,1,1,1,1502),-- Sealed Tallum Plate Armor Pattern
(20826,5524,1,1,1,3253),-- Sealed Dark Crystal Gaiters Pattern
(20826,1867,1,1,2,250000),-- Animal Skin
(20826,1872,1,3,2,125000),-- Animal Bone
(20826,1875,1,1,2,15873),-- Stone of Purity
(20826,1881,1,1,2,27027),-- Coarse Bone Powder
(20826,3956,1,1,2,347),-- Recipe: Blessed Spiritshot A
(20826,4041,1,1,2,1733),-- Mold Hardener
(20826,4980,1,1,2,3817),-- Recipe: Doom Shield (60%)
(20826,6670,1,1,2,38462),-- Deluxe Chest Key - Grade 6
(20826,8730,1,1,200,4854),-- Life Stone: level 67
(20826,8740,1,1,200,1733),-- Mid-Grade Life Stone: level 67
(20827,3035,1,1,-1,10888),-- Recipe: Spiritshot A
(20827,4117,1,1,-1,35684),-- Art of Battle Axe Blade
(20827,4118,1,1,-1,33808),-- Evil Spirit Head
(20827,57,3958,8033,0,700000),-- Adena
(20827,5297,1,1,1,24),-- Sealed Dark Crystal Leather Armor
(20827,5298,1,1,1,38),-- Sealed Dark Crystalline Leggings
(20827,5301,1,1,1,16),-- Sealed Tallum Leather Armor
(20827,5478,1,1,1,3381),-- Sealed Dark Crystal Leather Armor Pattern
(20827,5479,1,1,1,2379),-- Sealed Tallum Leather Armor Pattern
(20827,5482,1,1,1,5306),-- Sealed Dark Crystalline Leggings Design
(20827,1868,1,3,2,250000),-- Thread
(20827,1873,1,1,2,90909),-- Silver Nugget
(20827,1876,1,1,2,55556),-- Mithril Ore
(20827,1895,1,1,2,71429),-- Metallic Fiber
(20827,4042,1,1,2,11111),-- Enria
(20827,4402,1,1,2,775),-- Blue Dimensional Stone
(20827,4990,1,1,2,1550),-- Recipe: Blue Wolf Helmet (60%)
(20827,5420,1,1,2,509),-- Recipe: Sealed Armor of Nightmare(60%)
(20827,8730,1,1,200,2584),-- Life Stone: level 67
(20827,8740,1,1,200,1034),-- Mid-Grade Life Stone: level 67
(20828,4120,1,1,-1,40698),-- Bellion Cestus Edge
(20828,4121,1,1,-1,40562),-- Bow of Peril Shaft
(20828,5479,1,1,-1,55110),-- Sealed Tallum Leather Armor Pattern
(20828,57,4514,9172,0,700000),-- Adena
(20828,5304,1,1,1,27),-- Sealed Tallum Tunic
(20828,5305,1,1,1,43),-- Sealed Tallum Stockings
(20828,5308,1,1,1,19),-- Sealed Dark Crystal Robe
(20828,5485,1,1,1,3855),-- Sealed Tallum Tunic Texture
(20828,5486,1,1,1,2713),-- Sealed Dark Crystal Robe Fabric
(20828,5489,1,1,1,6051),-- Sealed Tallum Stockings Fabric
(20828,1864,3,9,2,111111),-- Stem
(20828,1869,2,6,2,83333),-- Iron Ore
(20828,1878,1,1,2,142857),-- Braided Hemp
(20828,1884,2,6,2,34483),-- Cord
(20828,4039,1,1,2,7937),-- Mold Glue
(20828,4177,1,1,2,3676),-- Recipe: Zubei\'s Gauntlets
(20828,8730,1,1,200,3058),-- Life Stone: level 67
(20828,8740,1,1,200,612),-- Mid-Grade Life Stone: level 67
(20829,1894,1,1,-1,438023),-- Crafted Leather
(20829,5479,1,1,-1,73868),-- Sealed Tallum Leather Armor Pattern
(20829,5550,1,1,-1,166449),-- Durable Metal Plate
(20829,57,6036,12290,0,700000),-- Adena
(20829,5287,1,1,1,27),-- Sealed Dark Crystal breastplate
(20829,5288,1,1,1,43),-- Sealed Dark Crystal Gaiters
(20829,5293,1,1,1,19),-- Sealed Tallum Plate Armor
(20829,5520,1,1,1,3964),-- Sealed Dark Crystal Breastplate Pattern
(20829,5521,1,1,1,2797),-- Sealed Tallum Plate Armor Pattern
(20829,5524,1,1,1,6059),-- Sealed Dark Crystal Gaiters Pattern
(20829,1866,1,3,2,83333),-- Suede
(20829,1871,1,3,2,200000),-- Charcoal
(20829,1877,1,1,2,18868),-- Adamantite Nugget
(20829,4040,1,1,2,9174),-- Mold Lubricant
(20829,4041,1,1,2,4831),-- Mold Hardener
(20829,4129,1,1,2,3861),-- Recipe: Earrings of Black Ore
(20829,6671,1,1,2,38462),-- Deluxe Chest Key - Grade 7
(20829,8730,1,1,200,3378),-- Life Stone: level 67
(20829,8740,1,1,200,966),-- Mid-Grade Life Stone: level 67
(20829,8750,1,1,200,483),-- High-Grade Life Stone: level 67
(20832,1825,1,1,-1,63369),-- Recipe: Oriharukon
(20832,1902,1,1,-1,89886),-- Elven Earring Beads
(20832,5231,1,1,-1,63369),-- Recipe: Metal Hardener (100%)
(20832,57,430,872,0,700000),-- Adena
(20832,61,1,1,1,76),-- Mithril Gloves
(20832,607,1,1,1,76),-- Ogre Power Gauntlets
(20832,1125,1,1,1,76),-- Assault Boots
(20832,1949,1,1,1,4144),-- Mithril Gloves Design
(20832,1957,1,1,1,4144),-- Assault Boots Part
(20832,1963,1,1,1,4144),-- Ogre Power Gauntlets Part
(20832,2427,1,1,1,76),-- Salamander Skin Boots
(20832,2428,1,1,1,76),-- Plate Boots
(20832,2451,1,1,1,76),-- Sage\'s Worn Gloves
(20832,2931,1,1,1,4144),-- Salamander Skin Boot Lining
(20832,2932,1,1,1,4144),-- Plate Boots Design
(20832,2933,1,1,1,4144),-- Sage\'s Worn Gloves Lining
(20832,1865,1,1,2,111111),-- Varnish
(20832,1870,1,1,2,52632),-- Coal
(20832,1875,1,1,2,11628),-- Stone of Purity
(20832,1879,1,1,2,23256),-- Cokes
(20832,2162,1,1,2,3075),-- Recipe: Aquastone Necklace
(20832,2312,1,1,2,278),-- Recipe: Crystallized Ice Bow
(20832,5159,1,1,2,213),-- Recipe: Spiritshot (C) Compressed Package (100%)
(20833,1874,1,1,-1,65752),-- Oriharukon Ore
(20833,1875,1,1,-1,65752),-- Stone of Purity
(20833,1905,1,1,-1,95942),-- Omen Beast\'s Eye Earring Gemstone
(20833,57,479,969,0,700000),-- Adena
(20833,281,1,1,1,21),-- Crystallized Ice Bow
(20833,2074,1,1,1,1651),-- Crystallized Ice Bow Shaft
(20833,1867,1,1,2,66667),-- Animal Skin
(20833,1872,1,1,2,90909),-- Animal Bone
(20833,1875,1,1,2,4440),-- Stone of Purity
(20833,1881,1,1,2,8879),-- Coarse Bone Powder
(20833,2205,1,1,2,2108),-- Recipe: Chain Gaiters
(20833,2209,1,1,2,5007),-- Recipe: Eldarake
(20833,2299,1,1,2,291),-- Recipe: Big Hammer
(20834,1805,1,1,-1,4893),-- Recipe: Soulshot: C-Grade
(20834,1878,1,1,-1,587109),-- Braided Hemp
(20834,1885,1,1,-1,122314),-- High Grade Suede
(20834,57,600,1212,0,700000),-- Adena
(20834,353,1,1,1,95),-- Half Plate Armor
(20834,380,1,1,1,152),-- Plate Gaiters
(20834,1950,1,1,1,5981),-- Half Plate Temper
(20834,1951,1,1,1,9302),-- Plate Gaiters Part
(20834,1864,1,1,2,184262),-- Stem
(20834,1869,1,1,2,92132),-- Iron Ore
(20834,1878,1,1,2,36853),-- Braided Hemp
(20834,1884,1,1,2,56696),-- Cord
(20834,2306,1,1,2,403),-- Recipe: Stick of Faith
(20834,2996,1,1,2,7312),-- Recipe: Chain Gloves
(20834,5274,1,1,2,307),-- Recipe: Greater Spiritshot (C) Compressed Package(100%)
(20834,6668,1,1,2,32905),-- Deluxe Chest Key - Grade 4
(20835,1871,1,3,-1,808860),-- Charcoal
(20835,1879,1,1,-1,134810),-- Cokes
(20835,1906,1,1,-1,111184),-- Mithril Ring Wire
(20835,57,394,793,0,700000),-- Adena
(20835,439,1,1,1,51),-- Karmian Tunic
(20835,471,1,1,1,81),-- Karmian Stockings
(20835,1954,1,1,1,5081),-- Karmian Stocking Design
(20835,1962,1,1,1,3059),-- Karmian Tunic Pattern
(20835,1865,1,1,2,38229),-- Varnish
(20835,1870,1,1,2,38229),-- Coal
(20835,1875,1,1,2,2549),-- Stone of Purity
(20835,1879,1,1,2,6372),-- Cokes
(20835,2303,1,1,2,167),-- Recipe: Skull Graver
(20835,2990,1,1,2,3034),-- Recipe: Chain Boots
(20835,3000,1,1,2,2451),-- Recipe: Dwarven Chain Shield
(20835,3033,1,1,2,128),-- Recipe: Spiritshot C
(20835,3954,1,1,2,128),-- Recipe: Blessed Spiritshot C
(20835,4040,1,1,2,765),-- Mold Lubricant
(20836,1874,1,1,-1,113820),-- Oriharukon Ore
(20836,1875,1,2,-1,113820),-- Stone of Purity
(20836,57,827,1684,0,700000),-- Adena
(20836,96,1,1,1,12),-- Scythe
(20836,298,1,1,1,12),-- Orcish Glaive
(20836,302,1,1,1,12),-- Body Slasher
(20836,2062,1,1,1,1055),-- Scythe Edge
(20836,2075,1,1,1,1055),-- Orcish Glaive Blade
(20836,2076,1,1,1,1055),-- Body Slasher Blade
(20836,1867,1,1,2,179311),-- Animal Skin
(20836,1872,1,1,2,179311),-- Animal Bone
(20836,1875,1,1,2,8966),-- Stone of Purity
(20836,1881,1,1,2,17932),-- Coarse Bone Powder
(20836,2251,1,1,2,22414),-- Recipe: Steel Arrow
(20836,2319,1,1,2,469),-- Recipe: Raid Sword
(20836,8723,1,1,200,5495),-- Life Stone: level 46
(20836,8733,1,1,200,1832),-- Mid-grade Life Stone: level 46
(20837,1895,1,1,-1,478498),-- Metallic Fiber
(20837,1960,1,1,-1,91767),-- Chain Gaiters Part
(20837,2214,1,1,-1,41557),-- Recipe: Dwarven Chain Gaiters
(20837,57,676,1373,0,700000),-- Adena
(20837,62,1,1,1,70),-- Mithril Boots
(20837,1958,1,1,1,3953),-- Mithril Boots Design
(20837,2429,1,1,1,70),-- Chain Boots
(20837,2430,1,1,1,70),-- Karmian Boots
(20837,2431,1,1,1,70),-- Plated Leather Boots
(20837,2432,1,1,1,70),-- Dwarven Chain Boots
(20837,2433,1,1,1,70),-- Boots of Seal
(20837,2452,1,1,1,70),-- Reinforced Mithril Gloves
(20837,2453,1,1,1,70),-- Chain Gloves
(20837,2454,1,1,1,70),-- Karmian Gloves
(20837,2937,1,1,1,3953),-- Chain Boots Part
(20837,2938,1,1,1,3953),-- Karmian Boots Fabric
(20837,2939,1,1,1,3953),-- Plate Leather Boot Lining
(20837,2940,1,1,1,3953),-- Dwarven Chain Boots Part
(20837,2941,1,1,1,3953),-- Boots of Seal Pattern
(20837,2942,1,1,1,3953),-- Reinforced Mithril Gloves Design
(20837,2943,1,1,1,3953),-- Chain Gloves Design
(20837,2944,1,1,1,3953),-- Karmian Gloves Fabric
(20837,1868,1,1,2,226147),-- Thread
(20837,1873,1,1,2,45230),-- Silver Nugget
(20837,1876,1,1,2,22615),-- Mithril Ore
(20837,1895,1,1,2,32307),-- Metallic Fiber
(20837,2225,1,1,2,2459),-- Recipe: Demon\'s Stockings
(20837,2321,1,1,2,395),-- Recipe: Stiletto
(20837,4042,1,1,2,1885),-- Enria
(20838,1866,1,3,-1,523274),-- Suede
(20838,1964,1,1,-1,126854),-- Eldarake Temper
(20838,2216,1,1,-1,64869),-- Recipe: Great Helmet
(20838,57,633,1287,0,700000),-- Adena
(20838,852,1,1,1,160),-- Moonstone Earring
(20838,883,1,1,1,241),-- Aquastone Ring
(20838,915,1,1,1,120),-- Aquastone Necklace
(20838,1908,1,1,1,11709),-- Moonstone Earring Wire
(20838,1909,1,1,1,16698),-- Aquastone Ring Wire
(20838,1910,1,1,1,9311),-- Aquastone Necklace Chain
(20838,1866,1,1,2,82436),-- Suede
(20838,1871,1,1,2,123654),-- Charcoal
(20838,1885,1,1,2,10305),-- High Grade Suede
(20838,3004,1,1,2,7681),-- Recipe: Rind Leather Gloves
(20838,4041,1,1,2,1076),-- Mold Hardener
(20838,5220,1,1,2,4947),-- Metal Hardener
(20839,1908,1,1,-1,88935),-- Moonstone Earring Wire
(20839,2066,1,1,-1,7366),-- Heavy Doom Hammer Head
(20839,2320,1,1,-1,3272),-- Recipe: Cursed Staff
(20839,57,454,925,0,700000),-- Adena
(20839,631,1,1,1,112),-- Eldarake
(20839,1964,1,1,1,5978),-- Eldarake Temper
(20839,2413,1,1,1,79),-- Chain Hood
(20839,2495,1,1,1,112),-- Chain Shield
(20839,2936,1,1,1,5430),-- Chain Hood Pattern
(20839,2945,1,1,1,5978),-- Chain Shield Fragment
(20839,1867,1,1,2,73977),-- Animal Skin
(20839,1872,1,1,2,73977),-- Animal Bone
(20839,1882,1,1,2,12330),-- Leather
(20839,1894,1,1,2,1947),-- Crafted Leather
(20839,2230,1,1,2,1076),-- Recipe: Theca Leather Gaiters
(20839,2326,1,1,2,194),-- Recipe: Horn of Glory
(20839,4042,1,1,2,925),-- Enria
(20839,6668,1,1,2,19816),-- Deluxe Chest Key - Grade 4
(20840,1967,1,1,-1,52860),-- Plated Leather Gaiters Material
(20840,2166,1,1,-1,32854),-- Recipe: Earrings of Binding (100%)
(20840,2328,1,1,-1,2254),-- Recipe: Sword of Delusion
(20840,57,469,927,0,700000),-- Adena
(20840,227,1,1,1,8),-- Stiletto
(20840,242,1,1,1,8),-- Soulfire Dirk
(20840,2083,1,1,1,714),-- Stiletto Edge
(20840,2084,1,1,1,714),-- Soulfire Dirk Blade
(20840,952,1,1,2,818),-- Scroll: Enchant Armor (Grade C)
(20840,1868,1,1,2,122694),-- Thread
(20840,1873,1,1,2,24539),-- Silver Nugget
(20840,1889,1,1,2,4090),-- Compound Braid
(20840,2229,1,1,2,744),-- Recipe: Theca Leather Armor
(20840,4044,1,1,2,2045),-- Thons
(20840,5549,1,1,2,6135),-- Metallic Thread
(20841,951,1,1,-1,1811),-- Scroll: Enchant Weapon (Grade C)
(20841,1911,1,2,-1,74098),-- Earring of Protection Gemstone
(20841,4438,1,1,-1,7105),-- Knuckle Duster Edge
(20841,57,488,971,0,700000),-- Adena
(20841,282,1,1,1,8),-- Elemental Bow
(20841,285,1,1,1,8),-- Noble Elven Bow
(20841,2085,1,1,1,764),-- Crossbow Shaft
(20841,2086,1,1,1,764),-- Elven Bow of Nobility Shaft
(20841,952,1,1,2,736),-- Scroll: Enchant Armor (Grade C)
(20841,1864,1,1,2,110354),-- Stem
(20841,1869,1,1,2,55177),-- Iron Ore
(20841,1880,1,1,2,5518),-- Steel
(20841,2348,1,1,2,129),-- Recipe: Heathen\'s Book
(20841,2351,1,1,2,116),-- Recipe: Scorpion
(20841,4043,1,1,2,1840),-- Asofe
(20841,5550,1,1,2,736),-- Durable Metal Plate
(20842,1976,1,1,-1,57819),-- Rind Leather Gaiters Material
(20842,2167,1,1,-1,48512),-- Recipe: Ring of Ages
(20842,2335,1,1,-1,2211),-- Recipe: War Axe
(20842,57,467,920,0,700000),-- Adena
(20842,853,1,1,1,72),-- Earring of Protection
(20842,884,1,1,1,109),-- Ring of Protection
(20842,916,1,1,1,54),-- Necklace of Protection
(20842,1911,1,1,1,5224),-- Earring of Protection Gemstone
(20842,1912,1,1,1,7568),-- Ring of Protection Gemstone
(20842,1913,1,1,1,4271),-- Necklace of Protection Gemstone
(20842,952,1,1,2,937),-- Scroll: Enchant Armor (Grade C)
(20842,1865,1,1,2,70215),-- Varnish
(20842,1870,1,1,2,70215),-- Coal
(20842,1874,1,1,2,4682),-- Oriharukon Ore
(20842,4039,1,1,2,2341),-- Mold Glue
(20842,4043,1,1,2,2341),-- Asofe
(20843,1539,1,1,-1,267391),-- Greater Healing Potion
(20843,1914,1,1,-1,67599),-- Earring of Binding Gemstone
(20843,4601,1,1,-1,2674),-- Greater Dye of Strength <Str+3 Con-3>
(20843,57,602,1169,0,700000),-- Adena
(20843,283,1,1,1,12),-- Akat Long Bow
(20843,2109,1,1,1,1396),-- Akat Longbow Shaft
(20843,1866,1,1,2,41467),-- Suede
(20843,1871,1,1,2,62200),-- Charcoal
(20843,1877,1,1,2,2488),-- Adamantite Nugget
(20843,2357,1,1,2,102),-- Recipe: Crystal Dagger
(20843,3014,1,1,2,1738),-- Recipe: Full Plate Boots
(20843,4040,1,1,2,1245),-- Mold Lubricant
(20843,4041,1,1,2,541),-- Mold Hardener
(20843,4188,1,1,2,72),-- Recipe: Hell Knife
(20844,1539,1,1,-1,238805),-- Greater Healing Potion
(20844,4041,1,1,-1,9345),-- Mold Hardener
(20844,4603,1,1,-1,2388),-- Greater Dye of Constitution <Con+3 Str-3>
(20844,57,540,1036,0,700000),-- Adena
(20844,854,1,1,1,51),-- Earring of Binding
(20844,885,1,1,1,76),-- Ring of Ages
(20844,917,1,1,1,38),-- Necklace of Mermaid
(20844,1914,1,1,1,4162),-- Earring of Binding Gemstone
(20844,1915,1,1,1,6527),-- Ring of Ages Gemstone
(20844,1916,1,1,1,3152),-- Necklace of Mermaid Teardrop
(20844,1867,1,1,2,74067),-- Animal Skin
(20844,1872,1,1,2,74067),-- Animal Bone
(20844,1875,1,1,2,3704),-- Stone of Purity
(20844,1881,1,1,2,7407),-- Coarse Bone Powder
(20844,2355,1,1,2,91),-- Recipe: Ghoul\'s Staff
(20844,4041,1,1,2,484),-- Mold Hardener
(20844,4601,1,1,2,124),-- Greater Dye of Strength <Str+3 Con-3>
(20844,4609,1,1,2,124),-- Greater Dye of Mental Strength <Men+3 Int-3>
(20845,2100,1,1,-1,8254),-- Paradia Staff Head
(20845,2354,1,1,-1,2389),-- Recipe: Deadman\'s Staff
(20845,4042,1,1,-1,24403),-- Enria
(20845,57,731,1419,0,700000),-- Adena
(20845,75,1,1,1,3),-- Caliburs
(20845,76,1,1,1,3),-- Sword of Delusion
(20845,77,1,1,1,3),-- Tsurugi
(20845,84,1,1,1,3),-- Homunkulus\'s Sword
(20845,134,1,1,1,3),-- Sword of Nightmare
(20845,145,1,1,1,3),-- Sword of Whispering Death
(20845,2089,1,1,1,275),-- Caliburs Edge
(20845,2090,1,1,1,275),-- Delusional Blade
(20845,2091,1,1,1,275),-- Tsurugi Blade
(20845,2092,1,1,1,275),-- Homunkulus\'s Sword Blade
(20845,2095,1,1,1,275),-- Sword of Nightmare Blade
(20845,2096,1,1,1,275),-- Sword of Whispering Death Blade
(20845,1868,1,1,2,121101),-- Thread
(20845,1873,1,1,2,24221),-- Silver Nugget
(20845,1876,1,1,2,12111),-- Mithril Ore
(20845,1895,1,1,2,17301),-- Metallic Fiber
(20845,3017,1,1,2,1692),-- Recipe: Divine Gloves
(20845,4042,1,1,2,1010),-- Enria
(20845,4608,1,1,2,135),-- Greater Dye of Intelligence <Int+3 Wit-3>
(20845,5155,1,1,2,122),-- Recipe: Soulshot (B) Compressed Package (100%)
(20845,6669,1,1,2,14082),-- Deluxe Chest Key - Grade 5
(20845,8735,1,1,200,5000),-- Mid-Grade Life Stone: level 52
(20846,1918,1,1,-1,55929),-- Nassen\'s Earring Gemstone
(20846,2955,1,1,-1,81734),-- Theca Leather Gloves Texture
(20846,4042,1,1,-1,21523),-- Enria
(20846,57,995,1654,0,700000),-- Adena
(20846,400,1,1,1,20),-- Theca Leather Armor
(20846,401,1,1,1,11),-- Drake Leather Armor
(20846,420,1,1,1,31),-- Theca Leather Gaiters
(20846,1984,1,1,1,1704),-- Theca Leather Armor Pattern
(20846,1985,1,1,1,2741),-- Theca Leather Gaiters Pattern
(20846,1987,1,1,1,1213),-- Drake Leather Armor Texture
(20846,1865,1,1,2,59603),-- Varnish
(20846,1870,1,1,2,59603),-- Coal
(20846,1874,1,1,2,3974),-- Oriharukon Ore
(20846,4039,1,1,2,1987),-- Mold Glue
(20846,4043,1,1,2,1987),-- Asofe
(20846,4133,1,1,2,287),-- Recipe: Zubei\'s Breastplate
(20846,4941,1,1,2,459),-- Recipe: Zubei\'s Gaiters  (60%)
(20846,4959,1,1,2,1149),-- Recipe: Avadon Boots (60%)
(20847,1806,1,1,-1,2469),-- Recipe: Soulshot: B-Grade
(20847,1985,1,1,-1,42562),-- Theca Leather Gaiters Pattern
(20847,4937,1,1,-1,20368),-- Recipe: Adamantite Earrings (70%)
(20847,57,497,1455,0,700000),-- Adena
(20847,2436,1,1,1,24),-- Theca Leather Boots
(20847,2437,1,1,1,19),-- Drake Leather Boots
(20847,2438,1,1,1,19),-- Full Plate Boots
(20847,2460,1,1,1,24),-- Theca Leather Gloves
(20847,2461,1,1,1,19),-- Drake Leather Gloves
(20847,2462,1,1,1,19),-- Full Plate Gauntlets
(20847,2463,1,1,1,19),-- Divine Gloves
(20847,2954,1,1,1,2061),-- Theca Leather Boots Texture
(20847,2955,1,1,1,2061),-- Theca Leather Gloves Texture
(20847,2960,1,1,1,1773),-- Drake Leather Boots Design
(20847,2961,1,1,1,1773),-- Full Plate Boots Part
(20847,2962,1,1,1,1773),-- Drake Leather Gloves Design
(20847,2963,1,1,1,1773),-- Full Plate Gauntlets Part
(20847,2964,1,1,1,1773),-- Divine Gloves Pattern
(20847,1864,1,1,2,113935),-- Stem
(20847,1869,1,1,2,56968),-- Iron Ore
(20847,1878,1,1,2,22787),-- Braided Hemp
(20847,1884,1,1,2,35057),-- Cord
(20847,4039,1,1,2,1900),-- Mold Glue
(20847,4178,1,1,2,1046),-- Recipe: Zubei\'s Shield (100%)
(20847,4184,1,1,2,66),-- Recipe: Sprite\'s Staff
(20847,5160,1,1,2,114),-- Recipe: Spiritshot (B) Compressed Package (100%)
(20847,8726,1,1,200,3135),-- Life Stone: level 55
(20848,1871,1,3,-1,794014),-- Charcoal
(20848,1895,1,1,-1,453722),-- Metallic Fiber
(20848,1908,1,1,-1,150381),-- Moonstone Earring Wire
(20848,57,638,1538,0,700000),-- Adena
(20848,96,1,1,1,11),-- Scythe
(20848,298,1,1,1,11),-- Orcish Glaive
(20848,302,1,1,1,11),-- Body Slasher
(20848,2062,1,1,1,982),-- Scythe Edge
(20848,2075,1,1,1,982),-- Orcish Glaive Blade
(20848,2076,1,1,1,982),-- Body Slasher Blade
(20848,1864,1,3,2,125088),-- Stem
(20848,1865,1,1,2,125088),-- Varnish
(20848,1866,1,1,2,83392),-- Suede
(20848,1868,1,3,2,83392),-- Thread
(20848,1873,1,1,2,50036),-- Silver Nugget
(20848,2164,1,1,2,9622),-- Recipe: Ring of Protection
(20849,1805,1,1,-1,4207),-- Recipe: Soulshot: C-Grade
(20849,1956,1,1,-1,143260),-- Square Shield Fragment
(20849,4469,1,1,-1,7212),-- Dye of Strength <Str+1 Con-1>
(20849,57,653,1222,0,700000),-- Adena
(20849,438,1,1,1,148),-- Sage\'s Rag
(20849,1953,1,1,1,10151),-- Sage\'s Rag Lining
(20849,1867,1,1,2,93894),-- Animal Skin
(20849,1869,1,1,2,70420),-- Iron Ore
(20849,1870,1,1,2,70420),-- Coal
(20849,1871,1,1,2,70420),-- Charcoal
(20849,1872,1,1,2,93894),-- Animal Bone
(20849,2205,1,1,2,2229),-- Recipe: Chain Gaiters
(20849,2211,1,1,2,1580),-- Recipe: Plated Leather Armor
(20849,5274,1,1,2,235),-- Recipe: Greater Spiritshot (C) Compressed Package(100%)
(20849,5279,1,1,2,236),-- Recipe: Greater Blessed Spiritshot (C) Compressed Package(100%)
(20859,3035,1,1,-1,22803),-- Recipe: Spiritshot A
(20859,5479,1,1,-1,101196),-- Sealed Tallum Leather Armor Pattern
(20859,57,8261,16860,0,700000),-- Adena
(20859,5287,1,1,1,37),-- Sealed Dark Crystal breastplate
(20859,5288,1,1,1,59),-- Sealed Dark Crystal Gaiters
(20859,5293,1,1,1,25),-- Sealed Tallum Plate Armor
(20859,5520,1,1,1,5431),-- Sealed Dark Crystal Breastplate Pattern
(20859,5521,1,1,1,3832),-- Sealed Tallum Plate Armor Pattern
(20859,5524,1,1,1,8300),-- Sealed Dark Crystal Gaiters Pattern
(20859,1867,7,21,2,58824),-- Animal Skin
(20859,1872,4,12,2,111111),-- Animal Bone
(20859,1875,1,1,2,41667),-- Stone of Purity
(20859,1881,1,1,2,76923),-- Coarse Bone Powder
(20859,3035,1,1,2,285),-- Recipe: Spiritshot A
(20859,4041,1,1,2,5263),-- Mold Hardener
(20859,8730,1,1,200,6135),-- Life Stone: level 67
(20859,8740,1,1,200,1284),-- Mid-Grade Life Stone: level 67
(20859,8750,1,1,200,143),-- High-Grade Life Stone: level 67
(20860,5156,1,1,-1,32044),-- Recipe: Soulshot (A) Compressed Package (100%)
(20860,5482,1,1,-1,317205),-- Sealed Dark Crystalline Leggings Design
(20860,5485,1,1,-1,202094),-- Sealed Tallum Tunic Texture
(20860,57,9685,19719,0,700000),-- Adena
(20860,5287,1,1,1,52),-- Sealed Dark Crystal breastplate
(20860,5288,1,1,1,82),-- Sealed Dark Crystal Gaiters
(20860,5293,1,1,1,35),-- Sealed Tallum Plate Armor
(20860,5520,1,1,1,7631),-- Sealed Dark Crystal Breastplate Pattern
(20860,5521,1,1,1,5385),-- Sealed Tallum Plate Armor Pattern
(20860,5524,1,1,1,11663),-- Sealed Dark Crystal Gaiters Pattern
(20860,1868,15,45,2,55556),-- Thread
(20860,1873,2,6,2,76923),-- Silver Nugget
(20860,1876,1,1,2,166667),-- Mithril Ore
(20860,1895,3,9,2,43478),-- Metallic Fiber
(20860,4042,1,1,2,13889),-- Enria
(20860,5271,1,1,2,1842),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(20860,8731,1,1,200,6579),-- Life Stone: level 70
(20860,8741,1,1,200,526),-- Mid-Grade Life Stone: level 70
(20860,8761,1,1,200,263),-- Top-Grade Life Stone: level 70
(20919,734,1,1,-1,20266),-- Haste Potion
(20919,1865,1,1,-1,121597),-- Varnish
(20919,5220,1,1,-1,4864),-- Metal Hardener
(20919,57,56,98,0,700000),-- Adena
(20919,24,1,1,1,178),-- Bone Breastplate
(20919,31,1,1,1,283),-- Bone Gaiters
(20919,390,1,1,1,178),-- Cotton Shirt
(20919,412,1,1,1,283),-- Cotton Pants
(20919,1867,1,1,2,23946),-- Animal Skin
(20919,1872,1,1,2,23946),-- Animal Bone
(20919,1881,1,1,2,2395),-- Coarse Bone Powder
(20919,2141,1,1,2,3592),-- Recipe: Silver Mold
(20920,1898,1,1,-1,22414),-- Blue Diamond Necklace Gem
(20920,2014,1,1,-1,2508),-- Iron Hammer Head
(20920,5163,1,1,-1,8728),-- Recipe: Blessed Spiritshot (D) Compressed Package (100%)
(20920,57,99,178,0,700000),-- Adena
(20920,273,1,1,1,53),-- Composition Bow
(20920,2016,1,1,1,756),-- Composition Bow Shaft
(20920,735,1,1,2,1791),-- Potion of Alacrity
(20920,1060,1,1,2,47743),-- Lesser Healing Potion
(20920,1865,1,1,2,21485),-- Varnish
(20920,1870,1,1,2,21485),-- Coal
(20920,1879,1,1,2,3581),-- Cokes
(20921,1804,1,1,-1,6111),-- Recipe: Soulshot: D-Grade
(20921,1864,1,2,-1,305532),-- Stem
(20921,1928,1,1,-1,22253),-- Piece Bone Gaiters Fragment
(20921,57,68,125,0,700000),-- Adena
(20921,39,1,1,1,490),-- Boots
(20921,51,1,1,1,490),-- Bracer
(20921,1930,1,1,1,7714),-- Boot Lining
(20921,1880,1,1,2,1806),-- Steel
(20921,1881,1,1,2,2407),-- Coarse Bone Powder
(20921,1882,1,1,2,4012),-- Leather
(20921,3032,1,1,2,723),-- Recipe: Spiritshot D
(20921,6666,1,1,2,18051),-- Deluxe Chest Key - Grade 2
(20922,1870,1,1,-1,165778),-- Coal
(20922,1899,1,1,-1,25903),-- Necklace of Devotion Chain
(20922,1932,1,1,-1,26314),-- Bone Helmet Design
(20922,57,86,165,0,700000),-- Adena
(20922,123,1,1,1,12),-- Saber
(20922,2017,1,1,1,309),-- Saber Blade
(20922,5285,1,1,1,12),-- Heavy Sword
(20922,5531,1,1,1,391),-- Heavy Sword Edge
(20922,1060,1,1,2,50000),-- Lesser Healing Potion
(20922,1874,1,1,2,2994),-- Oriharukon Ore
(20922,1878,1,1,2,8621),-- Braided Hemp
(20922,2259,1,1,2,289),-- Recipe: Spinebone Sword
(20922,5220,1,1,2,997),-- Metal Hardener
(20923,2019,1,1,-1,5139),-- Trident Edge
(20923,2152,1,1,-1,42069),-- Recipe: Enchanted Necklace
(20923,2263,1,1,-1,3710),-- Recipe: Dwarven Trident
(20923,57,123,241,0,700000),-- Adena
(20923,432,1,1,1,215),-- Cursed Tunic
(20923,465,1,1,1,343),-- Cursed Stockings
(20923,735,1,1,2,2410),-- Potion of Alacrity
(20923,1875,1,1,2,2146),-- Stone of Purity
(20923,1879,1,1,2,5348),-- Cokes
(20923,1885,1,1,2,1608),-- High Grade Suede
(20923,5549,1,1,2,2950),-- Metallic Thread
(20923,5550,1,1,2,268),-- Durable Metal Plate
(20924,1900,1,1,-1,31603),-- Enchanted Necklace Chain
(20924,1937,1,1,-1,24815),-- Scale Gaiters Fragment
(20924,2250,1,1,-1,72213),-- Recipe: Bone Arrow
(20924,57,146,292,0,700000),-- Adena
(20924,259,1,1,1,49),-- Single-Edged Jamadhr
(20924,1876,1,1,2,3546),-- Mithril Ore
(20924,1877,1,1,2,1110),-- Adamantite Nugget
(20924,1880,1,1,2,2217),-- Steel
(20924,1881,1,1,2,2217),-- Coarse Bone Powder
(20924,1882,1,1,2,4651),-- Leather
(20924,2183,1,1,2,1110),-- Recipe: Mithril Banded Mail
(20924,2274,1,1,2,141),-- Recipe: Skull Breaker
(20924,6667,1,1,2,12658),-- Deluxe Chest Key - Grade 3
(20925,1874,1,1,-1,10384),-- Oriharukon Ore
(20925,2016,1,1,-1,1825),-- Composition Bow Shaft
(20925,2258,1,1,-1,3808),-- Recipe: Temptation of Abyss
(20925,57,69,128,0,700000),-- Adena
(20925,846,1,1,1,288),-- Coral Earring
(20925,878,1,1,1,431),-- Blue Coral Ring
(20925,909,1,1,1,217),-- Blue Diamond Necklace
(20925,1898,1,1,1,2364),-- Blue Diamond Necklace Gem
(20925,1060,1,1,2,40898),-- Lesser Healing Potion
(20925,1867,1,1,2,24539),-- Animal Skin
(20925,1872,1,1,2,24539),-- Animal Bone
(20925,1881,1,1,2,2454),-- Coarse Bone Powder
(20925,5273,1,1,2,737),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(20926,1869,1,1,-1,178367),-- Iron Ore
(20926,1875,1,1,-1,11891),-- Stone of Purity
(20926,2018,1,1,-1,2370),-- Assassin Knife Edge
(20926,57,94,177,0,700000),-- Adena
(20926,156,1,1,1,17),-- Hand Axe
(20926,167,1,1,1,17),-- Scalpel
(20926,178,1,1,1,17),-- Bone Staff
(20926,1831,1,1,2,47619),-- Antidote
(20926,1834,1,1,2,17857),-- Emergency Dressing
(20926,1875,1,1,2,962),-- Stone of Purity
(20926,1879,1,1,2,3460),-- Cokes
(20926,2178,1,1,2,2500),-- Recipe: Dark Stockings
(20926,5437,1,1,2,350),-- Recipe: Heavy Sword(100%)
(20926,5549,1,1,2,1155),-- Metallic Thread
(20927,1866,1,1,-1,167397),-- Suede
(20927,1899,1,1,-1,39234),-- Necklace of Devotion Chain
(20927,1932,1,1,-1,39856),-- Bone Helmet Design
(20927,57,130,251,0,700000),-- Adena
(20927,220,1,1,1,69),-- Crafted Dagger
(20927,1876,1,1,2,5917),-- Mithril Ore
(20927,1877,1,1,2,527),-- Adamantite Nugget
(20927,1880,1,1,2,3559),-- Steel
(20927,1881,1,1,2,3165),-- Coarse Bone Powder
(20927,1882,1,1,2,5917),-- Leather
(20927,2178,1,1,2,5405),-- Recipe: Dark Stockings
(20928,2025,1,1,-1,3438),-- Dwarven Trident Edge
(20928,2153,1,1,-1,38605),-- Recipe: Tiger\'s Eye Earring
(20928,2183,1,1,-1,16331),-- Recipe: Mithril Banded Mail
(20928,57,126,251,0,700000),-- Adena
(20928,348,1,1,1,52),-- Scale Mail
(20928,377,1,1,1,82),-- Scale Gaiters
(20928,1935,1,1,1,1895),-- Scale Mail Fragment
(20928,1937,1,1,1,2914),-- Scale Gaiters Fragment
(20928,1832,1,1,2,25000),-- Greater Antidote
(20928,1867,1,1,2,22727),-- Animal Skin
(20928,1872,1,1,2,21739),-- Animal Bone
(20928,1875,1,1,2,1742),-- Stone of Purity
(20928,1881,1,1,2,2618),-- Coarse Bone Powder
(20928,2184,1,1,2,1742),-- Recipe: Mithril Banded Gaiters
(20928,2274,1,1,2,122),-- Recipe: Skull Breaker
(20928,2978,1,1,2,5236),-- Recipe: Elven Mithril Boots
(20929,956,1,1,-1,7763),-- Scroll: Enchant Armor (Grade D)
(20929,1871,1,1,-1,232889),-- Charcoal
(20929,1900,1,1,-1,25480),-- Enchanted Necklace Chain
(20929,57,118,235,0,700000),-- Adena
(20929,315,1,1,1,40),-- Divine Tome
(20929,1060,1,1,2,47619),-- Lesser Healing Potion
(20929,1865,1,1,2,22727),-- Varnish
(20929,1870,1,1,2,15385),-- Coal
(20929,1875,1,1,2,3846),-- Stone of Purity
(20929,1879,1,1,2,3031),-- Cokes
(20929,2250,1,1,2,7634),-- Recipe: Bone Arrow
(20929,6387,1,1,2,1912),-- Blessed Scroll of Resurrection for Pets
(20930,2028,1,1,-1,4318),-- Shilen Knife Edge
(20930,2154,1,1,-1,38910),-- Recipe: Elven Earring
(20930,2156,1,1,-1,29074),-- Recipe: Elven Necklace
(20930,57,184,367,0,700000),-- Adena
(20930,83,1,1,1,10),-- Sword of Magic
(20930,124,1,1,1,7),-- Two-Handed Sword
(20930,127,1,1,1,7),-- Crimson Sword
(20930,130,1,1,1,7),-- Elven Sword
(20930,144,1,1,1,10),-- Sword of Occult
(20930,2026,1,1,1,388),-- Two-Handed Sword Edge
(20930,735,1,1,2,1558),-- Potion of Alacrity
(20930,1864,1,1,2,55556),-- Stem
(20930,1869,1,1,2,29412),-- Iron Ore
(20930,1880,1,1,2,1558),-- Steel
(20930,2279,1,1,2,779),-- Recipe: War Pick
(20930,8317,1,1,2,779),-- Recipe: Titan Hammer (100%)
(20930,5550,1,1,2,430),-- Durable Metal Plate
(20931,1901,1,1,-1,55757),-- Tiger\'s Eye Earring Stone
(20931,1940,1,1,-1,60251),-- Iron Boots Design
(20931,2271,1,1,-1,2883),-- Recipe: Winged Spear
(20931,57,204,409,0,700000),-- Adena
(20931,293,1,1,1,22),-- War Hammer
(20931,296,1,1,1,22),-- Dwarven Pike
(20931,1865,1,1,2,30303),-- Varnish
(20931,1870,1,1,2,32258),-- Coal
(20931,1875,1,1,2,2188),-- Stone of Purity
(20931,1879,1,1,2,5263),-- Cokes
(20931,2197,1,1,2,438),-- Recipe: Salamander Skin Mail
(20931,2276,1,1,2,438),-- Recipe: Maingauche
(20931,2286,1,1,2,171),-- Recipe: Bonebreaker
(20932,1825,1,1,-1,31452),-- Recipe: Oriharukon
(20932,2032,1,1,-1,4069),-- Goat Head Staff Head
(20932,5220,1,1,-1,17613),-- Metal Hardener
(20932,57,223,446,0,700000),-- Adena
(20932,850,1,1,1,82),-- Elven Earring
(20932,881,1,1,1,122),-- Elven Ring
(20932,913,1,1,1,61),-- Elven Necklace
(20932,1902,1,1,1,3844),-- Elven Earring Beads
(20932,1903,1,1,1,6329),-- Elven Ring Piece
(20932,1904,1,1,1,3541),-- Elven Necklace Beads
(20932,1060,1,1,2,90909),-- Lesser Healing Potion
(20932,1866,1,1,2,25641),-- Suede
(20932,1871,1,1,2,38462),-- Charcoal
(20932,1885,1,1,2,4348),-- High Grade Suede
(20932,2159,1,1,2,2008),-- Recipe: Necklace of Darkness
(20932,2201,1,1,2,3484),-- Recipe: Square Shield
(20932,5220,1,1,2,2174),-- Metal Hardener
(20933,1879,1,1,-1,83876),-- Cokes
(20933,1880,1,1,-1,50326),-- Steel
(20933,1894,1,1,-1,17658),-- Crafted Leather
(20933,57,253,505,0,700000),-- Adena
(20933,261,1,1,1,19),-- Bich\'Hwa
(20933,2039,1,1,1,1317),-- Bich\'Hwa Edge
(20933,1865,1,1,2,55556),-- Varnish
(20933,1870,1,1,2,50000),-- Coal
(20933,1875,1,1,2,3906),-- Stone of Purity
(20933,1879,1,1,2,9091),-- Cokes
(20933,2159,1,1,2,3650),-- Recipe: Necklace of Darkness
(20933,2298,1,1,2,261),-- Recipe: Stormbringer
(20934,1868,1,1,-1,545126),-- Thread
(20934,1938,1,1,-1,18324),-- Mithril Banded Mail Material
(20934,2028,1,1,-1,3238),-- Shilen Knife Edge
(20934,57,139,276,0,700000),-- Adena
(20934,436,1,1,1,96),-- Tunic of Knowledge
(20934,469,1,1,1,153),-- Stockings of Knowledge
(20934,1867,1,1,2,40000),-- Animal Skin
(20934,1872,1,1,2,32258),-- Animal Bone
(20934,1882,1,1,2,5348),-- Leather
(20934,2190,1,1,2,5348),-- Recipe: Mithril Stockings
(20934,3020,1,1,2,162),-- Recipe: Elven Long Sword
(20935,1825,1,1,-1,21225),-- Recipe: Oriharukon
(20935,1872,1,1,-1,396209),-- Animal Bone
(20935,2040,1,1,-1,3438),-- Strengthened Long Bow Shaft
(20935,57,151,301,0,700000),-- Adena
(20935,629,1,1,1,66),-- Kite Shield
(20935,1947,1,1,1,3687),-- Kite Shield Fragment
(20935,2411,1,1,1,46),-- Brigandine Helmet
(20935,2493,1,1,1,66),-- Brigandine Shield
(20935,2922,1,1,1,987),-- Brigandine Helmet Design
(20935,2929,1,1,1,3687),-- Brigandine Shield Fragment
(20935,1868,1,1,2,47619),-- Thread
(20935,1873,1,1,2,9174),-- Silver Nugget
(20935,1889,1,1,2,1282),-- Compound Braid
(20935,2148,1,1,2,2994),-- Recipe: Crafted Leather
(20935,2288,1,1,2,122),-- Recipe: Ghost Staff
(20935,2984,1,1,2,2353),-- Recipe: Salamander Skin Boots
(20935,5549,1,1,2,1923),-- Metallic Thread
(20936,1874,1,1,-1,37494),-- Oriharukon Ore
(20936,1875,1,1,-1,37494),-- Stone of Purity
(20936,1902,1,1,-1,56982),-- Elven Earring Beads
(20936,57,283,656,0,700000),-- Adena
(20936,437,1,1,1,65),-- Mithril Tunic
(20936,470,1,1,1,104),-- Mithril Stockings
(20936,1944,1,1,1,3707),-- Mithril Tunic Fabric
(20936,1945,1,1,1,5773),-- Mithril Stocking Design
(20936,1865,1,1,2,62500),-- Varnish
(20936,1870,1,1,2,58824),-- Coal
(20936,1875,1,1,2,2457),-- Stone of Purity
(20936,1879,1,1,2,12346),-- Cokes
(20936,2159,1,1,2,6135),-- Recipe: Necklace of Darkness
(20936,2298,1,1,2,212),-- Recipe: Stormbringer
(20937,1805,1,1,-1,2444),-- Recipe: Soulshot: C-Grade
(20937,2054,1,1,-1,6241),-- Light Crossbow Shaft
(20937,4480,1,1,-1,4191),-- Dye of Wit <Wit+1 Men-1>
(20937,57,363,727,0,700000),-- Adena
(20937,61,1,1,1,63),-- Mithril Gloves
(20937,607,1,1,1,63),-- Ogre Power Gauntlets
(20937,1125,1,1,1,63),-- Assault Boots
(20937,1949,1,1,1,3425),-- Mithril Gloves Design
(20937,1957,1,1,1,3425),-- Assault Boots Part
(20937,1963,1,1,1,3425),-- Ogre Power Gauntlets Part
(20937,2427,1,1,1,63),-- Salamander Skin Boots
(20937,2428,1,1,1,63),-- Plate Boots
(20937,2451,1,1,1,63),-- Sage\'s Worn Gloves
(20937,2931,1,1,1,3425),-- Salamander Skin Boot Lining
(20937,2932,1,1,1,3425),-- Plate Boots Design
(20937,2933,1,1,1,3425),-- Sage\'s Worn Gloves Lining
(20937,1866,1,1,2,38462),-- Suede
(20937,1871,1,1,2,76923),-- Charcoal
(20937,1885,1,1,2,9091),-- High Grade Suede
(20937,2205,1,1,2,2268),-- Recipe: Chain Gaiters
(20937,2307,1,1,2,230),-- Recipe: Heavy Doom Axe
(20937,5164,1,1,2,176),-- Recipe: Blessed Spiritshot (C) Compressed Package (100%)
(20937,5220,1,1,2,3401),-- Metal Hardener
(20938,1908,1,1,-1,76204),-- Moonstone Earring Wire
(20938,2070,1,1,-1,6390),-- Cursed Dagger Blade
(20938,2323,1,1,-1,2804),-- Recipe: Elemental Bow
(20938,57,388,794,0,700000),-- Adena
(20938,281,1,1,1,17),-- Crystallized Ice Bow
(20938,2074,1,1,1,1347),-- Crystallized Ice Bow Shaft
(20938,1867,1,1,2,55556),-- Animal Skin
(20938,1872,1,1,2,52632),-- Animal Bone
(20938,1882,1,1,2,8264),-- Leather
(20938,1894,1,1,2,4115),-- Crafted Leather
(20938,2315,1,1,2,190),-- Recipe: Shamshir
(20938,2317,1,1,2,1031),-- Recipe: Bec de Corbin
(20938,2321,1,1,2,190),-- Recipe: Stiletto
(20938,8723,1,1,200,2062),-- Life Stone: level 46
(20938,8733,1,1,200,1031),-- Mid-Grade Life Stone: level 46
(20939,1868,1,3,-1,898311),-- Thread
(20939,1873,1,1,-1,359325),-- Silver Nugget
(20939,2067,1,1,-1,6474),-- Crystal Staff Head
(20939,57,436,886,0,700000),-- Adena
(20939,397,1,1,1,57),-- Mithril Shirt
(20939,1961,1,1,1,3397),-- Mithril Shirt Fabric
(20939,2387,1,1,1,90),-- Tempered Mithril Gaiters
(20939,2935,1,1,1,5643),-- Tempered Mithril Gaiters Fragment
(20939,1868,1,1,2,90909),-- Thread
(20939,1873,1,1,2,17857),-- Silver Nugget
(20939,1889,1,1,2,3135),-- Compound Braid
(20939,2225,1,1,2,2072),-- Recipe: Demon\'s Stockings
(20939,2343,1,1,2,142),-- Recipe: Mace of the Underworld
(20939,4044,1,1,2,1046),-- Thons
(20939,5549,1,1,2,8403),-- Metallic Thread
(20939,8723,1,1,200,1046),-- Life Stone: level 46
(20939,8733,1,1,200,2072),-- Mid-Grade Life Stone: level 46
(20940,1970,1,1,-1,30149),-- Robe of Seal Fabric
(20940,4589,1,1,-1,4500),-- Greater Dye of Strength <Str+2 Con-2>
(20940,5550,1,1,-1,18001),-- Durable Metal Plate
(20940,57,653,1328,0,700000),-- Adena
(20940,4233,1,1,1,21),-- Knuckle Duster
(20940,1864,1,1,2,125000),-- Stem
(20940,1869,1,1,2,58824),-- Iron Ore
(20940,1880,1,1,2,5000),-- Steel
(20940,2350,1,1,2,157),-- Recipe: Pa\'agrian Axe
(20940,3005,1,1,2,3322),-- Recipe: Demon\'s Boots
(20940,4043,1,1,2,2119),-- Asofe
(20940,4132,1,1,2,3175),-- Recipe: Composite Shield
(20940,4438,1,1,2,2134),-- Knuckle Duster Edge
(20940,5550,1,1,2,907),-- Durable Metal Plate
(20940,8724,1,1,200,2725),-- Life Stone: level 49
(20940,8734,1,1,200,302),-- Mid-Grade Life Stone: level 49
(20941,2109,1,1,-1,5965),-- Akat Longbow Shaft
(20941,2233,1,1,-1,9943),-- Recipe: Divine Tunic
(20941,4601,1,1,-1,2364),-- Greater Dye of Strength <Str+3 Con-3>
(20941,57,516,1046,0,700000),-- Adena
(20941,60,1,1,1,31),-- Composite Armor
(20941,1977,1,1,1,3255),-- Composite Armor Temper
(20941,1865,1,1,2,45455),-- Varnish
(20941,1870,1,1,2,45455),-- Coal
(20941,1874,1,1,2,4049),-- Oriharukon Ore
(20941,3013,1,1,2,450),-- Recipe: Drake Leather Boots
(20941,3014,1,1,2,2247),-- Recipe: Full Plate Boots
(20941,4039,1,1,2,450),-- Mold Glue
(20941,4043,1,1,2,3145),-- Asofe
(20941,8725,1,1,200,450),-- Life Stone: level 52
(20942,1905,1,1,-1,52079),-- Omen Beast\'s Eye Earring Gemstone
(20942,2301,1,1,-1,2338),-- Recipe: Battle Axe
(20942,2931,1,1,-1,59751),-- Salamander Skin Boot Lining
(20942,57,259,526,0,700000),-- Adena
(20942,225,1,1,1,15),-- Mithril Dagger
(20942,2052,1,1,1,1167),-- Mithril Dagger Blade
(20942,1866,1,1,2,25000),-- Suede
(20942,1871,1,1,2,38462),-- Charcoal
(20942,1877,1,1,2,1502),-- Adamantite Nugget
(20942,8723,1,1,200,1502),-- Life Stone: level 46
(20942,8733,1,1,200,375),-- Mid-Grade Life Stone: level 46
(20943,1874,1,2,-1,51365),-- Oriharukon Ore
(20943,1875,1,1,-1,51365),-- Stone of Purity
(20943,2163,1,1,-1,39310),-- Recipe: Earring of Protection
(20943,57,373,758,0,700000),-- Adena
(20943,263,1,1,1,16),-- Chakram
(20943,2073,1,1,1,1408),-- Chakram Edge
(20943,1867,1,1,2,55556),-- Animal Skin
(20943,1872,1,1,2,58824),-- Animal Bone
(20943,1875,1,1,2,2994),-- Stone of Purity
(20943,1881,1,1,2,5495),-- Coarse Bone Powder
(20943,2225,1,1,2,832),-- Recipe: Demon\'s Stockings
(20943,4041,1,1,2,528),-- Mold Hardener
(20943,8723,1,1,200,1996),-- Life Stone: level 46
(20943,8733,1,1,200,832),-- Mid-Grade Life Stone: level 46
(20944,1911,1,1,-1,48520),-- Earring of Protection Gemstone
(20944,2329,1,1,-1,1517),-- Recipe: Tsurugi
(20944,2947,1,1,-1,46596),-- Dwarven Chain Shield Fragment
(20944,57,315,643,0,700000),-- Adena
(20944,853,1,1,1,50),-- Earring of Protection
(20944,884,1,1,1,75),-- Ring of Protection
(20944,916,1,1,1,37),-- Necklace of Protection
(20944,1911,1,1,1,3584),-- Earring of Protection Gemstone
(20944,1912,1,1,1,5192),-- Ring of Protection Gemstone
(20944,1913,1,1,1,2930),-- Necklace of Protection Gemstone
(20944,1868,1,1,2,71429),-- Thread
(20944,1873,1,1,2,13333),-- Silver Nugget
(20944,1876,1,1,2,6329),-- Mithril Ore
(20944,1895,1,1,2,9524),-- Metallic Fiber
(20944,2328,1,1,2,112),-- Recipe: Sword of Delusion
(20944,4042,1,1,2,317),-- Enria
(20944,8724,1,1,200,1267),-- Life Stone: level 49
(20944,8734,1,1,200,1585),-- Mid-Grade Life Stone: level 49
(20945,1868,1,3,-1,733375),-- Thread
(20945,1876,1,1,-1,146675),-- Mithril Ore
(20945,4601,1,1,-1,1630),-- Greater Dye of Strength <Str+3 Con-3>
(20945,57,355,722,0,700000),-- Adena
(20945,64,1,1,1,62),-- Composite Boots
(20945,608,1,1,1,62),-- Mithril Gauntlets
(20945,1981,1,1,1,4460),-- Mithril Gauntlets Design
(20945,2956,1,1,1,4460),-- Composite Boots Part
(20945,1868,1,1,2,71429),-- Thread
(20945,1873,1,1,2,12048),-- Silver Nugget
(20945,1876,1,1,2,7092),-- Mithril Ore
(20945,1895,1,1,2,10101),-- Metallic Fiber
(20945,2170,1,1,2,917),-- Recipe: Nassen\'s Earring
(20945,2970,1,1,2,1376),-- Recipe: Ring of Binding
(20945,4042,1,1,2,1032),-- Enria
(20945,8725,1,1,200,1147),-- Life Stone: level 52
(20945,8735,1,1,200,344),-- Mid-Grade Life Stone: level 52
(20945,8745,1,1,200,72),-- High-Grade Life Stone: level 52
(20946,1539,1,1,-1,251226),-- Greater Healing Potion
(20946,1914,1,1,-1,63512),-- Earring of Binding Gemstone
(20946,2169,1,1,-1,20009),-- Recipe: Necklace of Binding
(20946,57,550,1111,0,700000),-- Adena
(20946,103,1,1,1,46),-- Tower Shield
(20946,107,1,1,1,46),-- Composite Shield
(20946,517,1,1,1,32),-- Composite Helmet
(20946,1149,1,1,1,32),-- Shining Circlet
(20946,1978,1,1,1,3746),-- Tower Shield Fragment
(20946,1983,1,1,1,2563),-- Shining Circlet Pattern
(20946,2957,1,1,1,2564),-- Composite Helmet Design
(20946,4055,1,1,1,3746),-- Composite Shield Fragment
(20946,1864,1,1,2,111111),-- Stem
(20946,1869,1,1,2,55556),-- Iron Ore
(20946,1878,1,1,2,21277),-- Braided Hemp
(20946,1884,1,1,2,31250),-- Cord
(20946,2233,1,1,2,225),-- Recipe: Divine Tunic
(20946,4039,1,1,2,1351),-- Mold Glue
(20946,4180,1,1,2,4505),-- Recipe: Silver Arrow
(20946,8725,1,1,200,1802),-- Life Stone: level 52
(20946,8735,1,1,200,451),-- Mid-Grade Life Stone: level 52
(20947,1806,1,1,-1,2448),-- Recipe: Soulshot: B-Grade
(20947,4938,1,1,-1,30291),-- Recipe: Adamantite Ring (70%)
(20947,4967,1,1,-1,1410),-- Recipe: Sword of Valhalla (60%)
(20947,57,592,1205,0,700000),-- Adena
(20947,356,1,1,1,22),-- Full Plate Armor
(20947,1986,1,1,1,2388),-- Full Plate Armor Temper
(20947,1864,1,1,2,100000),-- Stem
(20947,1869,1,1,2,47619),-- Iron Ore
(20947,1880,1,1,2,5747),-- Steel
(20947,4043,1,1,2,1328),-- Asofe
(20947,4144,1,1,2,752),-- Recipe: Zubei\'s Leather Gaiters
(20947,4179,1,1,2,376),-- Recipe: Zubei\'s Helmet
(20947,4184,1,1,2,66),-- Recipe: Sprite\'s Staff
(20947,5550,1,1,2,188),-- Durable Metal Plate
(20947,8726,1,1,200,752),-- Life Stone: level 55
(20947,8736,1,1,200,134),-- Mid-Grade Life Stone: level 55
(20948,4049,1,1,-1,63959),-- Adamantite Earring Gemstone
(20948,4050,1,1,-1,75550),-- Adamantite Ring Wire
(20948,4129,1,1,-1,13197),-- Recipe: Earrings of Black Ore
(20948,57,589,1201,0,700000),-- Adena
(20948,11889,1,1,1,54),-- Common Item - Great Sword
(20948,11916,1,1,1,54),-- Common Item - Keshanberk
(20948,11900,1,1,1,54),-- Common Item - Sword of Valhalla
(20948,4104,1,1,1,344),-- Great Sword Blade
(20948,4107,1,1,1,339),-- Keshanberk Blade
(20948,4108,1,1,1,339),-- Sword of Valhalla Blade
(20948,948,1,1,2,113),-- Scroll: Enchant Armor (Grade B)
(20948,1865,1,1,2,38462),-- Varnish
(20948,1870,1,1,2,32258),-- Coal
(20948,1875,1,1,2,3165),-- Stone of Purity
(20948,1879,1,1,2,10000),-- Cokes
(20948,4040,1,1,2,1054),-- Mold Lubricant
(20948,4158,1,1,2,96),-- Recipe: Doom Plate Armor
(20948,4176,1,1,2,560),-- Recipe: Doom Boots
(20948,8727,1,1,200,1054),-- Life Stone: level 58
(20948,8737,1,1,200,164),-- Mid-Grade Life Stone: level 58
(20949,947,1,1,-1,577),-- Scroll: Enchant Weapon (Grade B)
(20949,1866,1,1,-1,480591),-- Suede
(20949,4041,1,1,-1,12537),-- Mold Hardener
(20949,57,697,1420,0,700000),-- Adena
(20949,11918,1,1,1,40),-- Common Item - Heavy War Axe
(20949,11908,1,1,1,40),-- Common Item - Sprite\'s Staff
(20949,4105,1,1,1,610),-- Heavy War Axe Head
(20949,4106,1,1,1,600),-- Sprite\'s Staff Head
(20949,1866,1,1,2,29412),-- Suede
(20949,1871,1,1,2,41667),-- Charcoal
(20949,1885,1,1,2,3145),-- High Grade Suede
(20949,4041,1,1,2,262),-- Mold Hardener
(20949,4129,1,1,2,262),-- Recipe: Earrings of Black Ore
(20949,4165,1,1,2,787),-- Recipe: Blue Wolf Helmet
(20949,4993,1,1,2,1049),-- Recipe: Doom Gloves (60%)
(20949,5220,1,1,2,1312),-- Metal Hardener
(20949,8727,1,1,200,787),-- Life Stone: level 58
(20950,947,1,1,-1,623),-- Scroll: Enchant Weapon (Grade B)
(20950,4049,1,1,-1,81733),-- Adamantite Earring Gemstone
(20950,4056,1,1,-1,28204),-- Zubei\'s Breastplate Part
(20950,57,755,1533,0,700000),-- Adena
(20950,11890,1,1,1,382),-- Common Item - Great Axe
(20950,4113,1,1,1,1184),-- Great Axe Head
(20950,1864,1,1,2,100000),-- Stem
(20950,1869,1,1,2,50000),-- Iron Ore
(20950,1880,1,1,2,6711),-- Steel
(20950,4043,1,1,2,1550),-- Asofe
(20950,5550,1,1,2,517),-- Durable Metal Plate
(20950,4159,1,1,2,1033),-- Recipe: Blue Wolf Leather Armor
(20950,8728,1,1,200,517),-- Life Stone: level 61
(20950,8738,1,1,200,517),-- Mid-Grade Life Stone: level 61
(20951,1806,1,1,-1,1655),-- Recipe: Soulshot: B-Grade
(20951,1918,1,1,-1,35828),-- Nassen\'s Earring Gemstone
(20951,1920,1,1,-1,28307),-- Necklace of Binding Chain
(20951,57,401,814,0,700000),-- Adena
(20951,11839,1,1,1,119),-- Common Item - Great Pata
(20951,2120,1,1,1,735),-- Great Pata Blade
(20951,1806,1,1,2,77),-- Recipe: Soulshot: B-Grade
(20951,1867,1,1,2,41667),-- Animal Skin
(20951,1872,1,1,2,40000),-- Animal Bone
(20951,1882,1,1,2,9259),-- Leather
(20951,1894,1,1,2,880),-- Crafted Leather
(20951,4042,1,1,2,660),-- Enria
(20951,5270,1,1,2,220),-- Recipe: Greater Soulshot (B) Compressed Package(100%)
(20951,5436,1,1,2,220),-- Recipe: Berserker Blade(100%)
(20951,8726,1,1,200,1099),-- Life Stone: level 55
(20951,8736,1,1,200,119),-- Mid-Grade Life Stone: level 55
(20952,1984,1,1,-1,32140),-- Theca Leather Armor Pattern
(20952,3034,1,1,-1,2999),-- Recipe: Spiritshot B
(20952,4044,1,1,-1,49978),-- Thons
(20952,57,726,1477,0,700000),-- Adena
(20952,11857,1,1,1,415),-- Common Item - Eminence Bow
(20952,2121,1,1,1,1273),-- Eminence Bow Shaft
(20952,1868,1,1,2,111111),-- Thread
(20952,1873,1,1,2,19231),-- Silver Nugget
(20952,1889,1,1,2,5814),-- Compound Braid
(20952,4044,1,1,2,1244),-- Thons
(20952,4128,1,1,2,857),-- Recipe: Adamantite Necklace
(20952,4187,1,1,2,80),-- Recipe: Kris
(20952,5275,1,1,2,415),-- Recipe: Greater Spiritshot (B) Compressed Package(100%)
(20952,5549,1,1,2,13699),-- Metallic Thread
(20952,8726,1,1,200,2075),-- Life Stone: level 55
(20952,8736,1,1,200,218),-- Mid-Grade Life Stone: level 55
(20952,8746,1,1,200,75),-- High-Grade Life Stone: level 55
(20953,1864,3,5,-1,932365),-- Stem
(20953,2962,1,1,-1,76153),-- Drake Leather Gloves Design
(20953,4948,1,1,-1,6134),-- Recipe: Avadon Leather Armor (60%)
(20953,57,563,1149,0,700000),-- Adena
(20953,12338,1,1,1,1253),-- Common Item - Adamantite Earring
(20953,12340,1,1,1,836),-- Common Item - Adamantite Ring
(20953,12339,1,1,1,418),-- Common Item - Adamantite Necklace
(20953,4049,1,1,1,3612),-- Adamantite Earring Gemstone
(20953,4050,1,1,1,4267),-- Adamantite Ring Wire
(20953,4051,1,1,1,2671),-- Adamantite Necklace Chain
(20953,1864,1,1,2,83333),-- Stem
(20953,1869,1,1,2,32258),-- Iron Ore
(20953,1878,1,1,2,14706),-- Braided Hemp
(20953,1884,1,1,2,27778),-- Cord
(20953,4039,1,1,2,1253),-- Mold Glue
(20953,4962,1,1,2,1253),-- Recipe: Zubei\'s Helmet (60%)
(20953,4999,1,1,2,418),-- Recipe: Doom Boots (60%)
(20953,8311,1,1,2,49),-- Recipe: Kaim Vanul\'s Bones (100%)
(20953,8727,1,1,200,1253),-- Life Stone: level 58
(20954,1869,1,5,-1,830753),-- Iron Ore
(20954,4049,1,1,-1,87150),-- Adamantite Earring Gemstone
(20954,4613,1,1,-1,2292),-- Greater Dye of Strength <Str+4 Con-4>
(20954,57,806,1635,0,700000),-- Adena
(20954,11915,1,1,1,55),-- Common Item - Kris
(20954,11919,1,1,1,55),-- Common Item - Hell Knife
(20954,4109,1,1,1,692),-- Kris Edge
(20954,4110,1,1,1,692),-- Hell Knife Edge
(20954,1864,1,1,2,111111),-- Stem
(20954,1869,1,1,2,55556),-- Iron Ore
(20954,1880,1,1,2,6623),-- Steel
(20954,4043,1,1,2,2041),-- Asofe
(20954,4159,1,1,2,199),-- Recipe: Blue Wolf Leather Armor
(20954,4990,1,1,2,2041),-- Recipe: Blue Wolf Helmet (60%)
(20954,5550,1,1,2,1020),-- Durable Metal Plate
(20954,8728,1,1,200,1020),-- Life Stone: level 61
(20954,8738,1,1,200,510),-- Mid-Grade Life Stone: level 61
(20955,4052,1,1,-1,68637),-- Earring of Black Ore Piece
(20955,5230,1,1,-1,2585),-- Recipe: Mithril Arrow (100%)
(20955,57,941,1905,0,700000),-- Adena
(20955,12194,1,1,1,123),-- Common Item - Blue Wolf Leather Armor
(20955,12186,1,1,1,123),-- Common Item - Leather Armor of Doom
(20955,4082,1,1,1,1618),-- Blue Wolf Leather Armor Texture
(20955,4083,1,1,1,1618),-- Leather Armor of Doom Design
(20955,1830,1,1,2,28571),-- Scroll of Escape: Castle
(20955,1865,1,1,2,66667),-- Varnish
(20955,1870,1,1,2,58824),-- Coal
(20955,1874,1,1,2,1883),-- Oriharukon Ore
(20955,4039,1,1,2,943),-- Mold Glue
(20955,4043,1,1,2,4237),-- Asofe
(20955,5276,1,1,2,110),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20955,8729,1,1,200,943),-- Life Stone: level 64
(20955,8739,1,1,200,471),-- Mid-Grade Life Stone: level 64
(20956,1884,1,3,-1,599263),-- Cord
(20956,4106,1,1,-1,10962),-- Sprite\'s Staff Head
(20956,4987,1,1,-1,8081),-- Recipe: Tunic of Doom (60%)
(20956,57,954,1912,0,700000),-- Adena
(20956,11939,1,1,1,186),-- Common Item - Demon Dagger
(20956,4119,1,1,1,1259),-- Demon Dagger Edge
(20956,1874,1,1,2,2571),-- Oriharukon Ore
(20956,1878,1,1,2,17857),-- Braided Hemp
(20956,1884,1,1,2,41667),-- Cord
(20956,1889,1,1,2,4291),-- Compound Braid
(20956,2508,1,1,2,62500),-- Cursed Bone
(20956,3956,1,1,2,97),-- Recipe: Blessed Spiritshot A
(20956,4175,1,1,2,857),-- Recipe: Avadon Boots
(20956,5220,1,1,2,3425),-- Metal Hardener
(20956,8729,1,1,200,64),-- Life Stone: level 64
(20956,8739,1,1,200,64),-- Mid-Grade Life Stone: level 64
(20957,1877,1,1,-1,81685),-- Adamantite Nugget
(20957,4052,1,1,-1,72301),-- Earring of Black Ore Piece
(20957,4053,1,1,-1,87533),-- Ring of Black Ore Gemstone
(20957,57,990,2008,0,700000),-- Adena
(20957,12183,1,1,1,519),-- Common Item - Doom Shield
(20957,12202,1,1,1,58),-- Common Item - Blue Wolf Helmet
(20957,12185,1,1,1,1038),-- Common Item - Doom Helmet
(20957,4077,1,1,1,4523),-- Doom Shield Fragment
(20957,4088,1,1,1,3800),-- Blue Wolf Helmet Design
(20957,4089,1,1,1,3800),-- Doom Helmet Pattern
(20957,1344,10,30,2,71429),-- Mithril Arrow
(20957,1830,1,1,2,25000),-- Scroll of Escape: Castle
(20957,1866,1,1,2,37037),-- Suede
(20957,1871,1,1,2,71429),-- Charcoal
(20957,1877,1,1,2,1845),-- Adamantite Nugget
(20957,4040,1,1,2,1038),-- Mold Lubricant
(20957,4041,1,1,2,1153),-- Mold Hardener
(20957,5276,1,1,2,58),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20957,8729,1,1,200,1095),-- Life Stone: level 64
(20957,8739,1,1,200,115),-- Mid-Grade Life Stone: level 64
(20958,1807,1,1,-1,2044),-- Recipe: Soulshot: A-Grade
(20958,5478,1,1,-1,12889),-- Sealed Dark Crystal Leather Armor Pattern
(20958,57,742,1509,0,700000),-- Adena
(20958,12226,1,1,1,40),-- Common Item - Sealed Dark Crystal Helmet
(20958,12225,1,1,1,40),-- Common Item - Sealed Dark Crystal Shield
(20958,12234,1,1,1,40),-- Common Item - Sealed Tallum Helmet
(20958,5494,1,1,1,1585),-- Sealed Dark Crystal Shield Fragment
(20958,5525,1,1,1,1167),-- Sealed Dark Crystal Helmet Design
(20958,5526,1,1,1,1167),-- Sealed Tallum Helm Design
(20958,1867,1,1,2,76923),-- Animal Skin
(20958,1872,1,1,2,76923),-- Animal Bone
(20958,1875,1,1,2,3610),-- Stone of Purity
(20958,1881,1,1,2,9346),-- Coarse Bone Powder
(20958,4041,1,1,2,317),-- Mold Hardener
(20958,5156,1,1,2,238),-- Recipe: Soulshot (A) Compressed Package (100%)
(20958,8730,1,1,200,832),-- Life Stone: level 67
(20958,8740,1,1,200,52),-- Mid-Grade Life Stone: level 67
(20959,1807,1,1,-1,3291),-- Recipe: Soulshot: A-Grade
(20959,4054,1,1,-1,48618),-- Necklace of Black Ore Beads
(20959,5478,1,1,-1,20756),-- Sealed Dark Crystal Leather Armor Pattern
(20959,57,1195,2431,0,700000),-- Adena
(20959,12232,1,1,1,195),-- Common Item - Sealed Tallum Tunic
(20959,12235,1,1,1,977),-- Common Item - Sealed Tallum Stockings
(20959,12223,1,1,1,195),-- Common Item - Sealed Dark Crystal Robe
(20959,5485,1,1,1,1022),-- Sealed Tallum Tunic Texture
(20959,5486,1,1,1,719),-- Sealed Dark Crystal Robe Fabric
(20959,5489,1,1,1,1604),-- Sealed Tallum Stockings Fabric
(20959,1866,1,1,2,55556),-- Suede
(20959,1871,1,1,2,71429),-- Charcoal
(20959,1877,1,1,2,3125),-- Adamantite Nugget
(20959,4040,1,1,2,1626),-- Mold Lubricant
(20959,4041,1,1,2,846),-- Mold Hardener
(20959,4953,1,1,2,2933),-- Recipe: Avadon Gloves (60%)
(20959,5281,1,1,2,139),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(20959,8730,1,1,200,1042),-- Life Stone: level 67
(20960,4052,1,1,-1,47533),-- Earring of Black Ore Piece
(20960,4053,1,1,-1,57546),-- Ring of Black Ore Gemstone
(20960,5230,1,1,-1,1790),-- Recipe: Mithril Arrow (100%)
(20960,57,652,1319,0,700000),-- Adena
(20960,11932,1,1,1,700),-- Common Item - Lance
(20960,4115,1,1,1,865),-- Lance Blade
(20960,1343,10,30,2,66667),-- Silver Arrow
(20960,1868,1,1,2,76923),-- Thread
(20960,1873,1,1,2,14085),-- Silver Nugget
(20960,1876,1,1,2,9709),-- Mithril Ore
(20960,1895,1,1,2,10309),-- Metallic Fiber
(20960,4042,1,1,2,233),-- Enria
(20960,5230,1,1,2,67),-- Recipe: Mithril Arrow (100%)
(20960,5276,1,1,2,67),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20960,8729,1,1,200,117),-- Life Stone: level 64
(20961,1884,1,3,-1,607931),-- Cord
(20961,4053,1,1,-1,84688),-- Ring of Black Ore Gemstone
(20961,5230,1,1,-1,2634),-- Recipe: Mithril Arrow (100%)
(20961,57,958,1943,0,700000),-- Adena
(20961,12188,1,1,1,377),-- Common Item - Boots of Doom
(20961,12189,1,1,1,377),-- Common Item - Blue Wolf Boots
(20961,12187,1,1,1,377),-- Common Item - Doom Gloves
(20961,12190,1,1,1,377),-- Common Item - Blue Wolf Gloves
(20961,4090,1,1,1,3275),-- Blue Wolf Boots Design
(20961,4091,1,1,1,3275),-- Doom Gloves Part
(20961,4096,1,1,1,3275),-- Blue Wolf Gloves Fabric
(20961,4099,1,1,1,3275),-- Doom Boots Part
(20961,1864,1,1,2,142857),-- Stem
(20961,1869,1,1,2,45455),-- Iron Ore
(20961,1878,1,1,2,22727),-- Braided Hemp
(20961,1884,1,1,2,29412),-- Cord
(20961,3956,1,1,2,112),-- Recipe: Blessed Spiritshot A
(20961,4039,1,1,2,1131),-- Mold Glue
(20961,5230,1,1,2,1319),-- Recipe: Mithril Arrow (100%)
(20961,8729,1,1,200,188),-- Life Stone: level 64
(20962,4054,1,1,-1,37144),-- Necklace of Black Ore Beads
(20962,4077,1,1,-1,84831),-- Doom Shield Fragment
(20962,5230,1,1,-1,2514),-- Recipe: Mithril Arrow (100%)
(20962,57,915,1854,0,700000),-- Adena
(20962,12194,1,1,1,308),-- Common Item - Blue Wolf Leather Armor
(20962,12186,1,1,1,308),-- Common Item - Leather Armor of Doom
(20962,4082,1,1,1,1574),-- Blue Wolf Leather Armor Texture
(20962,4083,1,1,1,1574),-- Leather Armor of Doom Design
(20962,1865,1,1,2,50000),-- Varnish
(20962,1870,1,1,2,55556),-- Coal
(20962,1875,1,1,2,4926),-- Stone of Purity
(20962,1879,1,1,2,14925),-- Cokes
(20962,3956,1,1,2,154),-- Recipe: Blessed Spiritshot A
(20962,4040,1,1,2,771),-- Mold Lubricant
(20962,8729,1,1,200,1080),-- Life Stone: level 64
(20962,5230,1,1,2,107),-- Recipe: Mithril Arrow (100%)
(20963,4116,1,1,-1,7834),-- Deadman\'s Glory Stone
(20963,5478,1,1,-1,15075),-- Sealed Dark Crystal Leather Armor Pattern
(20963,5479,1,1,-1,10608),-- Sealed Tallum Leather Armor Pattern
(20963,57,869,1764,0,700000),-- Adena
(20963,12221,1,1,1,117),-- Common Item - Sealed Dark Crystal Gloves
(20963,12224,1,1,1,117),-- Common Item - Sealed Dark Crystal Boots
(20963,12229,1,1,1,117),-- Common Item - Sealed Tallum Gloves
(20963,12231,1,1,1,117),-- Common Item - Sealed Tallum Boots
(20963,5496,1,1,1,1552),-- Sealed Dark Crystal Boots Lining
(20963,5497,1,1,1,1552),-- Sealed Tallum Boots Lining
(20963,5508,1,1,1,1552),-- Sealed Dark Crystal Gloves Design
(20963,5509,1,1,1,1552),-- Sealed Tallum Gloves Design
(20963,1864,1,1,2,142857),-- Stem
(20963,1869,1,1,2,66667),-- Iron Ore
(20963,1880,1,1,2,8475),-- Steel
(20963,4043,1,1,2,1055),-- Asofe
(20963,4984,1,1,2,352),-- Recipe: Blue Wolf Leather Armor (60%)
(20963,5550,1,1,2,1177),-- Durable Metal Plate
(20963,8730,1,1,200,820),-- Life Stone: level 67
(20963,8740,1,1,200,703),-- Mid-Grade Life Stone: level 67
(20964,1874,1,1,-1,122460),-- Oriharukon Ore
(20964,1875,1,1,-1,122460),-- Stone of Purity
(20964,3035,1,1,-1,2449),-- Recipe: Spiritshot A
(20964,57,890,1808,0,700000),-- Adena
(20964,12226,1,1,1,283),-- Common Item - Sealed Dark Crystal Helmet
(20964,12225,1,1,1,283),-- Common Item - Sealed Dark Crystal Shield
(20964,12234,1,1,1,283),-- Common Item - Sealed Tallum Helmet
(20964,5494,1,1,1,1899),-- Sealed Dark Crystal Shield Fragment
(20964,5525,1,1,1,1398),-- Sealed Dark Crystal Helmet Design
(20964,5526,1,1,1,1398),-- Sealed Tallum Helm Design
(20964,1868,1,1,2,142857),-- Thread
(20964,1873,1,1,2,23256),-- Silver Nugget
(20964,1876,1,1,2,15625),-- Mithril Ore
(20964,1895,1,1,2,18868),-- Metallic Fiber
(20964,4042,1,1,2,1701),-- Enria
(20964,4174,1,1,2,283),-- Recipe: Zubei\'s Boots
(20964,8730,1,1,200,567),-- Life Stone: level 67
(20964,8740,1,1,200,283),-- Mid-Grade Life Stone: level 67
(20964,8750,1,1,200,283),-- High-Grade Life Stone: level 67
(20965,4043,1,1,-1,82345),-- Asofe
(20965,4089,1,1,-1,93362),-- Doom Helmet Pattern
(20965,5482,1,1,-1,32605),-- Sealed Dark Crystalline Leggings Design
(20965,57,1193,2433,0,700000),-- Adena
(20965,12344,1,1,1,214),-- Common Item - Sealed Phoenix Necklace
(20965,12345,1,1,1,858),-- Common Item - Sealed Phoenix Earring
(20965,12346,1,1,1,858),-- Common Item - Sealed Phoenix Ring
(20965,6341,1,1,1,3364),-- Sealed Phoenix Earring Gemstone
(20965,6343,1,1,1,2601),-- Sealed Phoenix Necklace Beads
(20965,6345,1,1,1,4653),-- Sealed Phoenix Ring Gemstone
(20965,1864,1,1,2,166667),-- Stem
(20965,1869,1,1,2,90909),-- Iron Ore
(20965,1880,1,1,2,9901),-- Steel
(20965,4043,1,1,2,2358),-- Asofe
(20965,4161,1,1,2,643),-- Recipe: Blue Wolf Tunic
(20965,5550,1,1,2,1499),-- Durable Metal Plate
(20965,8730,1,1,200,1071),-- Life Stone: level 67
(20965,8740,1,1,200,214),-- Mid-Grade Life Stone: level 67
(20966,1894,1,1,-1,93879),-- Crafted Leather
(20966,4044,1,1,-1,89185),-- Thons
(20966,4085,1,1,-1,46499),-- Tunic of Doom Pattern
(20966,57,1294,2634,0,700000),-- Adena
(20966,12222,1,1,1,156),-- Common Item - Sealed Dark Crystal Leather Armor
(20966,12228,1,1,1,406),-- Common Item - Sealed Dark Crystalline Leggings
(20966,12230,1,1,1,117),-- Common Item - Sealed Tallum Leather Armor
(20966,5478,1,1,1,1108),-- Sealed Dark Crystal Leather Armor Pattern
(20966,5479,1,1,1,780),-- Sealed Tallum Leather Armor Pattern
(20966,5482,1,1,1,1739),-- Sealed Dark Crystalline Leggings Design
(20966,1894,1,1,2,2841),-- Crafted Leather
(20966,4039,1,1,2,3650),-- Mold Glue
(20966,4041,1,1,2,812),-- Mold Hardener
(20966,4044,1,1,2,3247),-- Thons
(20966,4617,1,1,2,182),-- Greater Dye of Dexterity <Dex+4 Str-4>
(20966,6037,1,1,2,200000),-- Waking Scroll
(20966,8731,1,1,200,1218),-- Life Stone: level 70
(20966,8741,1,1,200,812),-- Mid-Grade Life Stone: level 70
(20966,8751,1,1,200,406),-- High-Grade Life Stone: level 70
(20967,4041,1,1,-1,24181),-- Mold Hardener
(20967,5156,1,1,-1,3708),-- Recipe: Soulshot (A) Compressed Package (100%)
(20967,5485,1,1,-1,23384),-- Sealed Tallum Tunic Texture
(20967,57,1116,2286,0,700000),-- Adena
(20967,12344,1,1,1,265),-- Common Item - Sealed Phoenix Necklace
(20967,12345,1,1,1,795),-- Common Item - Sealed Phoenix Earring
(20967,12346,1,1,1,795),-- Common Item - Sealed Phoenix Ring
(20967,6341,1,1,1,3787),-- Sealed Phoenix Earring Gemstone
(20967,6343,1,1,1,2928),-- Sealed Phoenix Necklace Beads
(20967,6345,1,1,1,5238),-- Sealed Phoenix Ring Gemstone
(20967,1867,1,1,2,125000),-- Animal Skin
(20967,1872,1,1,2,125000),-- Animal Bone
(20967,1882,1,1,2,20833),-- Leather
(20967,1894,1,1,2,3175),-- Crafted Leather
(20967,4042,1,1,2,1590),-- Enria
(20967,5001,1,1,2,105),-- Recipe: Lance (60%)
(20967,8732,1,1,200,1192),-- Life Stone: level 76
(20967,8742,1,1,200,132),-- Mid-Grade Life Stone: level 76
(20967,8752,1,1,200,78),-- High-Grade Life Stone: level 76
(20968,5480,1,1,-1,12197),-- Sealed Leather Armor of Nightmare Fabric
(20968,5481,1,1,-1,12197),-- Sealed Majestic Leather Armor Fabric
(20968,5486,1,1,-1,16084),-- Sealed Dark Crystal Robe Fabric
(20968,57,1094,2233,0,700000),-- Adena
(20968,12221,1,1,1,397),-- Common Item - Sealed Dark Crystal Gloves
(20968,12224,1,1,1,397),-- Common Item - Sealed Dark Crystal Boots
(20968,12229,1,1,1,662),-- Common Item - Sealed Tallum Gloves
(20968,12231,1,1,1,1059),-- Common Item - Sealed Tallum Boots
(20968,5496,1,1,1,2354),-- Sealed Dark Crystal Boots Lining
(20968,5497,1,1,1,2354),-- Sealed Tallum Boots Lining
(20968,5508,1,1,1,2354),-- Sealed Dark Crystal Gloves Design
(20968,5509,1,1,1,2354),-- Sealed Tallum Gloves Design
(20968,729,1,1,2,12),-- Scroll: Enchant Weapon (Grade A)
(20968,1868,1,1,2,142857),-- Thread
(20968,1873,1,1,2,32258),-- Silver Nugget
(20968,1889,1,1,2,4762),-- Compound Braid
(20968,4044,1,1,2,3436),-- Thons
(20968,5007,1,1,2,77),-- Recipe: Bow of Peril (60%)
(20968,5549,1,1,2,5155),-- Metallic Thread
(20968,6036,1,1,2,2646),-- Greater Magic Haste Potion
(20968,8732,1,1,200,529),-- Life Stone: level 76
(20968,8742,1,1,200,132),-- Mid-Grade Life Stone: level 76
(20969,729,1,1,-1,253),-- Scroll: Enchant Weapon (Grade A)
(20969,5161,1,1,-1,3042),-- Recipe: Spiritshot (A) Compressed Package (100%)
(20969,5487,1,1,-1,10272),-- Sealed Robe of Nightmare Fabric
(20969,57,918,1873,0,700000),-- Adena
(20969,12264,1,1,1,256),-- Common Item - Sealed Gloves of Nightmare
(20969,12267,1,1,1,256),-- Common Item - Sealed Boots of Nightmare
(20969,12257,1,1,1,256),-- Common Item - Sealed Majestic Gloves
(20969,12260,1,1,1,256),-- Common Item - Sealed Majestic Boots
(20969,5502,1,1,1,1423),-- Sealed Boots of Nightmare Lining
(20969,5503,1,1,1,1423),-- Sealed Majestic Boots Lining
(20969,5514,1,1,1,1423),-- Sealed Gloves of Nightmare Design
(20969,5515,1,1,1,1423),-- Sealed Majestic Gloves Design
(20969,1894,1,1,2,2558),-- Crafted Leather
(20969,4039,1,1,2,3831),-- Mold Glue
(20969,4041,1,1,2,511),-- Mold Hardener
(20969,4044,1,1,2,4348),-- Thons
(20969,4150,1,1,2,2045),-- Recipe: Avadon Gloves
(20969,8732,1,1,200,1277),-- Life Stone: level 76
(20969,8742,1,1,200,273),-- Mid-Grade Life Stone: level 76
(20970,1895,1,2,-1,537200),-- Metallic Fiber
(20970,4042,1,1,-1,31337),-- Enria
(20970,5489,1,1,-1,24816),-- Sealed Tallum Stockings Fabric
(20970,57,909,1851,0,700000),-- Adena
(20970,12221,1,1,1,89),-- Common Item - Sealed Dark Crystal Gloves
(20970,12224,1,1,1,445),-- Common Item - Sealed Dark Crystal Boots
(20970,12229,1,1,1,711),-- Common Item - Sealed Tallum Gloves
(20970,12231,1,1,1,533),-- Common Item - Sealed Tallum Boots
(20970,5496,1,1,1,1628),-- Sealed Dark Crystal Boots Lining
(20970,5497,1,1,1,1628),-- Sealed Tallum Boots Lining
(20970,5508,1,1,1,1628),-- Sealed Dark Crystal Gloves Design
(20970,5509,1,1,1,1628),-- Sealed Tallum Gloves Design
(20970,10609,1,1,3,4098),-- Forgotten Scroll - Chant of Critical Attack
(20970,10581,1,1,3,3690),-- Forgotten Scroll - Improve Condition
(20970,10584,1,1,3,2873),-- Forgotten Scroll - Improve Movement
(20970,14217,1,1,3,3937),-- Forgotten Scroll - Condition of Paagrio
(20971,1876,1,1,-1,540396),-- Mithril Ore
(20971,1877,1,1,-1,108079),-- Adamantite Nugget
(20971,1895,1,1,-1,771995),-- Metallic Fiber
(20971,57,1088,2219,0,700000),-- Adena
(20971,12226,1,1,1,526),-- Common Item - Sealed Dark Crystal Helmet
(20971,12225,1,1,1,702),-- Common Item - Sealed Dark Crystal Shield
(20971,12234,1,1,1,702),-- Common Item - Sealed Tallum Helmet
(20971,5494,1,1,1,2794),-- Sealed Dark Crystal Shield Fragment
(20971,5525,1,1,1,2057),-- Sealed Dark Crystal Helmet Design
(20971,5526,1,1,1,2057),-- Sealed Tallum Helm Design
(20971,12769,1,1,3,7352),-- Forgotten Scroll - Evasion Chance
(20971,12768,1,1,3,8196),-- Forgotten Scroll - Evasion Counter
(20972,4077,1,1,-1,85462),-- Doom Shield Fragment
(20972,4087,1,1,-1,44581),-- Stockings of Doom Pattern
(20972,5494,1,1,-1,39900),-- Sealed Dark Crystal Shield Fragment
(20972,57,918,1872,0,700000),-- Adena
(20972,12221,1,1,1,507),-- Common Item - Sealed Dark Crystal Gloves
(20972,12224,1,1,1,169),-- Common Item - Sealed Dark Crystal Boots
(20972,12229,1,1,1,169),-- Common Item - Sealed Tallum Gloves
(20972,12231,1,1,1,507),-- Common Item - Sealed Tallum Boots
(20972,5496,1,1,1,1645),-- Sealed Dark Crystal Boots Lining
(20972,5497,1,1,1,1645),-- Sealed Tallum Boots Lining
(20972,5508,1,1,1,1645),-- Sealed Dark Crystal Gloves Design
(20972,5509,1,1,1,1645),-- Sealed Tallum Gloves Design
(20972,12771,1,1,3,5952),-- Forgotten Scroll - Shadow Step
(20972,12770,1,1,3,5714),-- Forgotten Scroll - Evasion Haste
(20973,4078,1,1,-1,43112),-- Blue Wolf Breastplate Part
(20973,5166,1,1,-1,3935),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(20973,5496,1,1,-1,69201),-- Sealed Dark Crystal Boots Lining
(20973,57,1187,2425,0,700000),-- Adena
(20973,12344,1,1,1,169),-- Common Item - Sealed Phoenix Necklace
(20973,12345,1,1,1,169),-- Common Item - Sealed Phoenix Earring
(20973,12346,1,1,1,1353),-- Common Item - Sealed Phoenix Ring
(20973,6341,1,1,1,4019),-- Sealed Phoenix Earring Gemstone
(20973,6343,1,1,1,3107),-- Sealed Phoenix Necklace Beads
(20973,6345,1,1,1,5559),-- Sealed Phoenix Ring Gemstone
(20974,4052,1,1,-1,142857),-- Earring of Black Ore Piece
(20974,5230,1,1,-1,5291),-- Recipe: Mithril Arrow (100%)
(20974,57,1926,3902,0,700000),-- Adena
(20974,1871,1,1,2,142857),-- Charcoal
(20974,1866,1,1,2,83333),-- Suede
(20974,1877,1,1,2,4854),-- Adamantite Nugget
(20974,4120,1,1,1,2386),-- Bellion Cestus Edge
(20974,4041,1,1,2,3247),-- Mold Hardener
(20974,4040,1,1,2,1081),-- Mold Lubricant
(20974,4164,1,1,2,2703),-- Recipe: Stockings of Doom
(20974,5276,1,1,2,224),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20974,8729,1,1,200,4329),-- Life Stone: level 64
(20974,8739,1,1,200,541),-- Mid-Grade Life Stone: level 64
(20975,4053,1,1,-1,150882),-- Ring of Black Ore Gemstone
(20975,4078,1,1,-1,51418),-- Blue Wolf Breastplate Part
(20975,5230,1,1,-1,4693),-- Recipe: Mithril Arrow (100%)
(20975,57,1711,3459,0,700000),-- Adena
(20975,110,1,1,1,42),-- Doom Shield
(20975,2416,1,1,1,29),-- Blue Wolf Helmet
(20975,2417,1,1,1,29),-- Doom Helmet
(20975,4077,1,1,1,7795),-- Doom Shield Fragment
(20975,4088,1,1,1,6549),-- Blue Wolf Helmet Design
(20975,4089,1,1,1,6549),-- Doom Helmet Pattern
(20975,1830,1,1,2,45455),-- Scroll of Escape: Castle
(20975,1866,1,1,2,76923),-- Suede
(20975,1871,1,1,2,142857),-- Charcoal
(20975,1885,1,1,2,8547),-- High Grade Suede
(20975,4041,1,1,2,1292),-- Mold Hardener
(20975,5220,1,1,2,3135),-- Metal Hardener
(20975,5276,1,1,2,199),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20975,8729,1,1,200,1712),-- Life Stone: level 64
(20975,8739,1,1,200,285),-- Mid-Grade Life Stone: level 64
(20975,8749,1,1,200,856),-- High-Grade Life Stone: level 64
(20976,4054,1,2,-1,58705),-- Necklace of Black Ore Beads
(20976,4080,1,1,-1,61307),-- Blue Wolf Gaiters Material
(20976,5230,1,1,-1,3974),-- Recipe: Mithril Arrow (100%)
(20976,57,1447,2929,0,700000),-- Adena
(20976,358,1,1,1,10),-- Blue Wolf Breastplate
(20976,2380,1,1,1,15),-- Blue Wolf Gaiters
(20976,2381,1,1,1,7),-- Doom Plate Armor
(20976,4078,1,1,1,2144),-- Blue Wolf Breastplate Part
(20976,4080,1,1,1,3019),-- Blue Wolf Gaiters Material
(20976,4081,1,1,1,1447),-- Doom Plate Armor Temper
(20976,1343,10,30,2,142857),-- Silver Arrow
(20976,1867,1,1,2,125000),-- Animal Skin
(20976,1872,1,1,2,125000),-- Animal Bone
(20976,1882,1,1,2,19231),-- Leather
(20976,1894,1,1,2,2101),-- Crafted Leather
(20976,4042,1,1,2,841),-- Enria
(20976,5230,1,1,2,147),-- Recipe: Mithril Arrow (100%)
(20976,5276,1,1,2,147),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20976,8729,1,1,200,421),-- Life Stone: level 64
(20976,8739,1,1,200,86),-- Mid-Grade Life Stone: level 64
(20976,8759,1,1,200,162),-- Top-Grade Life Stone: level 64
(20977,947,1,1,-1,1877),-- Scroll: Enchant Weapon (Grade B)
(20977,1876,1,1,-1,938515),-- Mithril Ore
(20977,4052,1,1,-1,166138),-- Earring of Black Ore Piece
(20977,57,2281,4615,0,700000),-- Adena
(20977,864,1,1,1,51),-- Earring of Black Ore
(20977,895,1,1,1,76),-- Ring of Black Ore
(20977,926,1,1,1,38),-- Necklace of Black Ore
(20977,4052,1,1,1,8178),-- Earring of Black Ore Piece
(20977,4053,1,1,1,9901),-- Ring of Black Ore Gemstone
(20977,4054,1,1,1,4550),-- Necklace of Black Ore Beads
(20977,730,1,1,2,166),-- Scroll: Enchant Armor (Grade A)
(20977,1866,1,1,2,100000),-- Suede
(20977,1871,1,1,2,125000),-- Charcoal
(20977,1877,1,1,2,4464),-- Adamantite Nugget
(20977,4040,1,1,2,3961),-- Mold Lubricant
(20977,4041,1,1,2,1722),-- Mold Hardener
(20977,4164,1,1,2,1312),-- Recipe: Stockings of Doom
(20977,8729,1,1,200,6667),-- Life Stone: level 64
(20978,4081,1,1,-1,49498),-- Doom Plate Armor Temper
(20978,5230,1,1,-1,6696),-- Recipe: Mithril Arrow (100%)
(20978,57,2442,4931,0,700000),-- Adena
(20978,79,1,1,1,12),-- Sword of Damascus
(20978,7889,1,1,1,15),-- Wizard\'s Tear
(20978,4114,1,1,1,3247),-- Sword of Damascus Blade
(20978,8336,1,1,1,1515),-- Blade of the Wizard's Tear
(20978,1830,1,1,2,76923),-- Scroll of Escape: Castle
(20978,1868,1,1,2,333333),-- Thread
(20978,1873,1,1,2,40000),-- Silver Nugget
(20978,1889,1,1,2,7042),-- Compound Braid
(20978,3956,1,1,2,283),-- Recipe: Blessed Spiritshot A
(20978,4044,1,1,2,11765),-- Thons
(20978,5549,1,1,2,27778),-- Metallic Thread
(20978,8729,1,1,200,4673),-- Life Stone: level 64
(20978,8739,1,1,200,2342),-- Mid-Grade Life Stone: level 64
(20979,4082,1,1,-1,50238),-- Blue Wolf Leather Armor Texture
(20979,5550,1,1,-1,59281),-- Durable Metal Plate
(20979,57,2165,4366,0,700000),-- Adena
(20979,171,1,1,1,4),-- Deadman\'s Glory
(20979,175,1,1,1,4),-- Art of Battle Axe
(20979,210,1,1,1,4),-- Staff of Evil Spirits
(20979,4116,1,1,1,957),-- Deadman\'s Glory Stone
(20979,4117,1,1,1,957),-- Art of Battle Axe Blade
(20979,4118,1,1,1,907),-- Evil Spirit Head
(20979,1343,10,30,2,200000),-- Silver Arrow
(20979,1864,1,3,2,100000),-- Stem
(20979,1869,1,1,2,90909),-- Iron Ore
(20979,1880,1,1,2,16417),-- Steel
(20979,4043,1,1,2,4739),-- Asofe
(20979,5230,1,1,2,2375),-- Recipe: Mithril Arrow (100%)
(20979,5276,1,1,2,219),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20979,5550,1,1,2,4739),-- Durable Metal Plate
(20980,4039,1,1,-1,172365),-- Mold Glue
(20980,4041,1,1,-1,44965),-- Mold Hardener
(20980,4085,1,1,-1,89867),-- Tunic of Doom Pattern
(20980,57,2504,5090,0,700000),-- Adena
(20980,5287,1,1,1,11),-- Sealed Dark Crystal breastplate
(20980,5288,1,1,1,18),-- Sealed Dark Crystal Gaiters
(20980,5293,1,1,1,8),-- Sealed Tallum Plate Armor
(20980,5520,1,1,1,1642),-- Sealed Dark Crystal Breastplate Pattern
(20980,5521,1,1,1,1159),-- Sealed Tallum Plate Armor Pattern
(20980,5524,1,1,1,2510),-- Sealed Dark Crystal Gaiters Pattern
(20980,1867,1,3,2,66667),-- Animal Skin
(20980,1869,2,6,2,34483),-- Iron Ore
(20980,1870,2,6,2,35714),-- Coal
(20980,1871,1,1,2,166667),-- Charcoal
(20980,1872,1,3,2,111111),-- Animal Bone
(20980,4985,1,1,2,339),-- Recipe: Leather Armor of Doom (60%)
(20980,8730,1,1,200,702),-- Life Stone: level 67
(20980,8740,1,1,200,339),-- Mid-Grade Life Stone: level 67
(20981,4083,1,1,-1,55065),-- Leather Armor of Doom Design
(20981,4085,1,1,-1,84693),-- Tunic of Doom Pattern
(20981,5230,1,1,-1,6498),-- Recipe: Mithril Arrow (100%)
(20981,57,2354,4802,0,700000),-- Adena
(20981,358,1,1,1,15),-- Blue Wolf Breastplate
(20981,2380,1,1,1,24),-- Blue Wolf Gaiters
(20981,2381,1,1,1,11),-- Doom Plate Armor
(20981,4078,1,1,1,3505),-- Blue Wolf Breastplate Part
(20981,4080,1,1,1,4935),-- Blue Wolf Gaiters Material
(20981,4081,1,1,1,2365),-- Doom Plate Armor Temper
(20981,1830,1,1,2,66667),-- Scroll of Escape: Castle
(20981,1864,1,3,2,166667),-- Stem
(20981,1869,1,1,2,166667),-- Iron Ore
(20981,1880,1,1,2,11628),-- Steel
(20981,4043,1,1,2,6024),-- Asofe
(20981,5550,1,1,2,3891),-- Durable Metal Plate
(20981,10303,1,1,1,159),-- Transform Sealbook - Vale Master
(20981,8730,1,1,200,707),-- Life Stone: level 67
(20981,8740,1,1,200,1060),-- Mid-Grade Life Stone: level 67
(20982,4084,1,1,-1,78645),-- Blue Wolf Tunic Fabric
(20982,4085,1,1,-1,78645),-- Tunic of Doom Pattern
(20982,5230,1,1,-1,6034),-- Recipe: Mithril Arrow (100%)
(20982,57,2185,4461,0,700000),-- Adena
(20982,2398,1,1,1,14),-- Blue Wolf Tunic
(20982,2399,1,1,1,14),-- Tunic of Doom
(20982,2403,1,1,1,23),-- Blue Wolf Stockings
(20982,2404,1,1,1,23),-- Stockings of Doom
(20982,4084,1,1,1,2903),-- Blue Wolf Tunic Fabric
(20982,4085,1,1,1,2903),-- Tunic of Doom Pattern
(20982,4086,1,1,1,3920),-- Blue Wolf Stockings Pattern
(20982,4087,1,1,1,3920),-- Stockings of Doom Pattern
(20982,1344,30,90,2,55556),-- Mithril Arrow
(20982,1830,1,1,2,66667),-- Scroll of Escape: Castle
(20982,1864,1,3,2,125000),-- Stem
(20982,1869,1,1,2,125000),-- Iron Ore
(20982,1880,1,1,2,10638),-- Steel
(20982,4043,1,1,2,3774),-- Asofe
(20982,5550,1,1,2,2062),-- Durable Metal Plate
(20982,5276,1,1,2,223),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(20982,10303,1,1,1,313),-- Transform Sealbook - Vale Master
(20982,8730,1,1,200,2747),-- Life Stone: level 67
(20982,8740,1,1,200,341),-- Mid-Grade Life Stone: level 67
(20983,4080,1,1,-1,139529),-- Blue Wolf Gaiters Material
(20983,5478,1,1,-1,57040),-- Sealed Dark Crystal Leather Armor Pattern
(20983,5549,1,1,-1,678321),-- Metallic Thread
(20983,57,3280,6678,0,700000),-- Adena
(20983,5297,1,1,1,20),-- Sealed Dark Crystal Leather Armor
(20983,5298,1,1,1,31),-- Sealed Dark Crystalline Leggings
(20983,5301,1,1,1,14),-- Sealed Tallum Leather Armor
(20983,5478,1,1,1,2808),-- Sealed Dark Crystal Leather Armor Pattern
(20983,5479,1,1,1,1976),-- Sealed Tallum Leather Armor Pattern
(20983,5482,1,1,1,4408),-- Sealed Dark Crystalline Leggings Design
(20983,730,1,1,2,239),-- Scroll: Enchant Armor (Grade A)
(20983,1867,1,3,2,166667),-- Animal Skin
(20983,1872,1,3,2,90909),-- Animal Bone
(20983,1882,1,1,2,58824),-- Leather
(20983,1894,1,1,2,7246),-- Crafted Leather
(20983,4042,1,1,2,3215),-- Enria
(20983,4199,1,1,2,219),-- Recipe: Bow of Peril
(20983,8731,1,1,200,410),-- Life Stone: level 70
(20983,8741,1,1,200,410),-- Mid-Grade Life Stone: level 70
(20984,4041,1,1,-1,54145),-- Mold Hardener
(20984,5478,1,1,-1,52361),-- Sealed Dark Crystal Leather Armor Pattern
(20984,57,3007,6136,0,700000),-- Adena
(20984,5304,1,1,1,18),-- Sealed Tallum Tunic
(20984,5305,1,1,1,29),-- Sealed Tallum Stockings
(20984,5308,1,1,1,13),-- Sealed Dark Crystal Robe
(20984,5485,1,1,1,2578),-- Sealed Tallum Tunic Texture
(20984,5486,1,1,1,1814),-- Sealed Dark Crystal Robe Fabric
(20984,5489,1,1,1,4046),-- Sealed Tallum Stockings Fabric
(20984,729,1,1,2,30),-- Scroll: Enchant Weapon (Grade A)
(20984,1865,1,1,2,200000),-- Varnish
(20984,1870,1,1,2,166667),-- Coal
(20984,1874,1,1,2,9709),-- Oriharukon Ore
(20984,4039,1,1,2,9709),-- Mold Glue
(20984,4043,1,1,2,8929),-- Asofe
(20984,5156,1,1,2,351),-- Recipe: Soulshot (A) Compressed Package (100%)
(20984,5420,1,1,2,808),-- Recipe: Sealed Armor of Nightmare(60%)
(20984,8730,1,1,200,1616),-- Life Stone: level 67
(20984,8740,1,1,200,808),-- Mid-Grade Life Stone: level 67
(20985,1807,1,1,-1,8840),-- Recipe: Soulshot: A-Grade
(20985,3035,1,1,-1,8840),-- Recipe: Spiritshot A
(20985,5479,1,1,-1,39229),-- Sealed Tallum Leather Armor Pattern
(20985,57,3203,6534,0,700000),-- Adena
(20985,6323,1,1,1,38),-- Sealed Phoenix Necklace
(20985,6324,1,1,1,50),-- Sealed Phoenix Earring
(20985,6325,1,1,1,75),-- Sealed Phoenix Ring
(20985,6341,1,1,1,9028),-- Sealed Phoenix Earring Gemstone
(20985,6343,1,1,1,6979),-- Sealed Phoenix Necklace Beads
(20985,6345,1,1,1,12486),-- Sealed Phoenix Ring Gemstone
(20985,1344,20,60,2,142857),-- Mithril Arrow
(20985,1866,1,1,2,142857),-- Suede
(20985,1871,1,3,2,71429),-- Charcoal
(20985,1877,1,1,2,8772),-- Adamantite Nugget
(20985,4040,1,1,2,3968),-- Mold Lubricant
(20985,4041,1,1,2,2381),-- Mold Hardener
(20985,5161,1,1,2,374),-- Recipe: Spiritshot (A) Compressed Package (100%)
(20985,8730,1,1,200,1590),-- Life Stone: level 67
(20985,8740,1,1,200,1590),-- Mid-Grade Life Stone: level 67
(20986,1806,1,1,-1,7272),-- Recipe: Soulshot: B-Grade
(20986,1984,1,1,-1,77945),-- Theca Leather Armor Pattern
(20986,4043,1,1,-1,121204),-- Asofe
(20986,57,1937,3158,0,700000),-- Adena
(20986,2414,1,1,1,126),-- Full Plate Helmet
(20986,2497,1,1,1,180),-- Full Plate Shield
(20986,2959,1,1,1,11610),-- Full Plate Helmet Design
(20986,2966,1,1,1,18366),-- Full Plate Shield Fragment
(20986,1894,1,1,2,6730),-- Crafted Leather
(20986,4039,1,1,2,6394),-- Mold Glue
(20986,4041,1,1,2,1668),-- Mold Hardener
(20986,4044,1,1,2,6394),-- Thons
(20986,4180,1,1,2,21311),-- Recipe: Silver Arrow
(20986,4184,1,1,2,221),-- Recipe: Sprite\'s Staff
(20986,5436,1,1,2,313),-- Recipe: Berserker Blade(100%)
(20986,8726,1,1,200,797),-- Life Stone: level 55
(20987,1864,1,14,-1,943145),-- Stem
(20987,1918,1,1,-1,142963),-- Nassen\'s Earring Gemstone
(20987,4039,1,1,-1,110034),-- Mold Glue
(20987,57,1658,3230,0,700000),-- Adena
(20987,299,1,1,1,7),-- Orcish Poleaxe
(20987,301,1,1,1,9),-- Scorpion
(20987,303,1,1,1,9),-- Widow Maker
(20987,2113,1,1,1,984),-- Scorpion Blade
(20987,2114,1,1,1,984),-- Widowmaker Head
(20987,2122,1,1,1,928),-- Orcish Poleaxe Blade
(20987,1864,1,3,2,152355),-- Stem
(20987,1869,1,1,2,152355),-- Iron Ore
(20987,1878,1,1,2,60942),-- Braided Hemp
(20987,1884,1,1,2,93757),-- Cord
(20987,3034,1,1,2,305),-- Recipe: Spiritshot B
(20987,3955,1,1,2,305),-- Recipe: Blessed Spiritshot B
(20987,4039,1,1,2,5079),-- Mold Glue
(20987,5270,1,1,2,305),-- Recipe: Greater Soulshot (B) Compressed Package(100%)
(20987,8726,1,1,200,420),-- Life Stone: level 55
(20988,1806,1,1,-1,7262),-- Recipe: Soulshot: B-Grade
(20988,1920,1,1,-1,124241),-- Necklace of Binding Chain
(20988,1984,1,1,-1,77834),-- Theca Leather Armor Pattern
(20988,57,2106,3561,0,700000),-- Adena
(20988,400,1,1,1,55),-- Theca Leather Armor
(20988,401,1,1,1,29),-- Drake Leather Armor
(20988,420,1,1,1,87),-- Theca Leather Gaiters
(20988,1984,1,1,1,4791),-- Theca Leather Armor Pattern
(20988,1985,1,1,1,7706),-- Theca Leather Gaiters Pattern
(20988,1987,1,1,1,3410),-- Drake Leather Armor Texture
(20988,1865,1,1,2,167582),-- Varnish
(20988,1870,1,1,2,167582),-- Coal
(20988,1875,1,1,2,11173),-- Stone of Purity
(20988,1879,1,1,2,27931),-- Cokes
(20988,3955,1,1,2,336),-- Recipe: Blessed Spiritshot B
(20988,4040,1,1,2,3352),-- Mold Lubricant
(20988,4127,1,1,2,4148),-- Recipe: Adamantite Ring
(20988,5270,1,1,2,336),-- Recipe: Greater Soulshot (B) Compressed Package(100%)
(20989,1825,1,1,-1,64329),-- Recipe: Oriharukon
(20989,1871,1,3,-1,900608),-- Charcoal
(20989,1902,1,1,-1,91247),-- Elven Earring Beads
(20989,57,457,910,0,700000),-- Adena
(20989,352,1,1,1,52),-- Brigandine Tunic
(20989,1941,1,1,1,3086),-- Brigandine Temper
(20989,2377,1,1,1,83),-- Mithril Scale Gaiters
(20989,2378,1,1,1,83),-- Brigandine Gaiters
(20989,2918,1,1,1,4603),-- Mithril Scale Gaiters Material
(20989,2919,1,1,1,4603),-- Brigandine Gaiters Material
(20989,1866,1,1,2,62500),-- Suede
(20989,1871,1,1,2,83333),-- Charcoal
(20989,1885,1,1,2,6993),-- High Grade Suede
(20989,2146,1,1,2,4975),-- Recipe: Artisan\'s frame
(20989,5220,1,1,2,3104),-- Metal Hardener
(20989,5473,1,1,2,10000),-- Recipe: Durable Metal Plate(100%)
(20990,1882,1,1,-1,189994),-- Leather
(20990,1903,1,1,-1,142615),-- Elven Ring Piece
(20990,2145,1,1,-1,61070),-- Recipe: Mithril Alloy
(20990,57,434,864,0,700000),-- Adena
(20990,437,1,1,1,99),-- Mithril Tunic
(20990,470,1,1,1,158),-- Mithril Stockings
(20990,1944,1,1,1,5636),-- Mithril Tunic Fabric
(20990,1945,1,1,1,8776),-- Mithril Stocking Design
(20990,1341,10,30,2,250000),-- Bone Arrow
(20990,1867,1,1,2,100000),-- Animal Skin
(20990,1872,1,1,2,100000),-- Animal Bone
(20990,1882,1,1,2,16393),-- Leather
(20990,1894,1,1,2,1931),-- Crafted Leather
(20990,2147,1,1,2,6757),-- Recipe: Blacksmith\'s frame
(20990,2194,1,1,2,6757),-- Recipe: Mithril Gloves
(20991,1908,1,1,-1,317634),-- Moonstone Earring Wire
(20991,1909,1,1,-1,452966),-- Aquastone Ring Wire
(20991,2064,1,1,-1,25926),-- Silver Axe Head
(20991,57,1632,3299,0,700000),-- Adena
(20991,354,1,1,1,157),-- Chain Mail Shirt
(20991,381,1,1,1,251),-- Chain Gaiters
(20991,1959,1,1,1,10677),-- Chain Mail Shirt Material
(20991,1960,1,1,1,21718),-- Chain Gaiters Part
(20991,1875,1,1,2,7752),-- Stone of Purity
(20991,1879,1,1,2,23256),-- Cokes
(20991,1885,1,1,2,16949),-- High Grade Suede
(20991,3021,1,1,2,1548),-- Recipe: Dwarven War Hammer
(20991,4556,1,1,2,1548),-- Greater Dye of Constitution <Con+1 Dex-1>
(20991,5549,1,1,2,16949),-- Metallic Thread
(20991,5550,1,1,2,6211),-- Durable Metal Plate
(20991,8723,1,1,200,9259),-- Life Stone: level 46
(20991,8733,1,1,200,1548),-- Mid-Grade Life Stone: level 46
(20992,1908,1,2,-1,284012),-- Moonstone Earring Wire
(20992,1909,1,2,-1,405019),-- Aquastone Ring Wire
(20992,2163,1,1,-1,153019),-- Recipe: Earring of Protection
(20992,57,1446,2961,0,700000),-- Adena
(20992,852,1,1,1,305),-- Moonstone Earring
(20992,883,1,1,1,459),-- Aquastone Ring
(20992,915,1,1,1,229),-- Aquastone Necklace
(20992,1908,1,1,1,22370),-- Moonstone Earring Wire
(20992,1909,1,1,1,31900),-- Aquastone Ring Wire
(20992,1910,1,1,1,17788),-- Aquastone Necklace Chain
(20992,1868,1,1,2,333333),-- Thread
(20992,1873,1,1,2,55556),-- Silver Nugget
(20992,1889,1,1,2,13514),-- Compound Braid
(20992,2164,1,1,2,20000),-- Recipe: Ring of Protection
(20992,2319,1,1,2,840),-- Recipe: Raid Sword
(20992,5549,1,1,2,18519),-- Metallic Thread
(20992,8723,1,1,200,4202),-- Life Stone: level 46
(20992,8733,1,1,200,840),-- Mid-Grade Life Stone: level 46
(20993,1864,1,13,-1,909305),-- Stem
(20993,1869,1,7,-1,795642),-- Iron Ore
(20993,2163,1,1,-1,162376),-- Recipe: Earring of Protection
(20993,57,1538,3139,0,700000),-- Adena
(20993,62,1,1,1,133),-- Mithril Boots
(20993,1958,1,1,1,7512),-- Mithril Boots Design
(20993,2429,1,1,1,133),-- Chain Boots
(20993,2430,1,1,1,133),-- Karmian Boots
(20993,2431,1,1,1,133),-- Plated Leather Boots
(20993,2432,1,1,1,133),-- Dwarven Chain Boots
(20993,2433,1,1,1,133),-- Boots of Seal
(20993,2452,1,1,1,133),-- Reinforced Mithril Gloves
(20993,2453,1,1,1,133),-- Chain Gloves
(20993,2454,1,1,1,133),-- Karmian Gloves
(20993,2937,1,1,1,7512),-- Chain Boots Part
(20993,2938,1,1,1,7512),-- Karmian Boots Fabric
(20993,2939,1,1,1,7512),-- Plate Leather Boot Lining
(20993,2940,1,1,1,7512),-- Dwarven Chain Boots Part
(20993,2941,1,1,1,7512),-- Boots of Seal Pattern
(20993,2942,1,1,1,7512),-- Reinforced Mithril Gloves Design
(20993,2943,1,1,1,7512),-- Chain Gloves Design
(20993,2944,1,1,1,7512),-- Karmian Gloves Fabric
(20993,1864,1,3,2,200000),-- Stem
(20993,1869,1,1,2,166667),-- Iron Ore
(20993,1880,1,1,2,19608),-- Steel
(20993,2165,1,1,2,3922),-- Recipe: Necklace of Protection
(20993,2318,1,1,2,1120),-- Recipe: Spirit Sword
(20993,5550,1,1,2,1120),-- Durable Metal Plate
(20993,8723,1,1,200,7299),-- Life Stone: level 46
(20993,8733,1,1,200,2801),-- Mid-Grade Life Stone: level 46
(20993,8743,1,1,200,87),-- High-Grade Life Stone: level 46
(20994,2169,1,1,-1,64202),-- Recipe: Necklace of Binding
(20994,4043,1,1,-1,120913),-- Asofe
(20994,4044,1,1,-1,120913),-- Thons
(20994,57,1756,3567,0,700000),-- Adena
(20994,265,1,1,1,35),-- Fist Blade
(20994,2108,1,1,1,4018),-- Fist Blade Piece
(20994,1894,1,1,2,6536),-- Crafted Leather
(20994,3015,1,1,2,1629),-- Recipe: Drake Leather Gloves
(20994,3016,1,1,2,4878),-- Recipe: Full Plate Gauntlets
(20994,4039,1,1,2,7353),-- Mold Glue
(20994,4041,1,1,2,1864),-- Mold Hardener
(20994,4044,1,1,2,5714),-- Thons
(20994,4186,1,1,2,247),-- Recipe: Sword of Valhalla
(20994,8725,1,1,200,5714),-- Life Stone: level 52
(20994,8735,1,1,200,815),-- Mid-Grade Life Stone: level 52
(20995,1911,1,2,-1,205015),-- Earring of Protection Gemstone
(20995,1966,1,1,-1,100636),-- Plated Leather Fragment
(20995,2166,1,1,-1,93438),-- Recipe: Earrings of Binding (100%)
(20995,57,1334,2711,0,700000),-- Adena
(20995,500,1,1,1,169),-- Great Helmet
(20995,632,1,1,1,241),-- Knight\'s Shield
(20995,1971,1,1,1,12462),-- Great Helmet Material
(20995,1972,1,1,1,16285),-- Knight\'s Shield Fragment
(20995,2496,1,1,1,261),-- Dwarven Chain Shield
(20995,2947,1,1,1,14540),-- Dwarven Chain Shield Fragment
(20995,1539,1,1,2,28571),-- Greater Healing Potion
(20995,1865,1,1,2,125000),-- Varnish
(20995,1870,1,1,2,142857),-- Coal
(20995,1874,1,1,2,7634),-- Oriharukon Ore
(20995,2222,1,1,2,2278),-- Recipe: Composite Armor
(20995,4039,1,1,2,5181),-- Mold Glue
(20995,4043,1,1,2,4132),-- Asofe
(20995,8724,1,1,200,5181),-- Life Stone: level 49
(20995,8734,1,1,200,1656),-- Mid-Grade Life Stone: level 49
(20995,8744,1,1,200,207),-- High-Grade Life Stone: level 49
(20996,1864,1,7,-1,927568),-- Stem
(20996,1869,1,3,-1,695676),-- Iron Ore
(20996,1914,1,1,-1,78166),-- Earring of Binding Gemstone
(20996,57,674,1369,0,700000),-- Adena
(20996,265,1,1,1,14),-- Fist Blade
(20996,2108,1,1,1,1541),-- Fist Blade Piece
(20996,1864,1,1,2,125000),-- Stem
(20996,1869,1,1,2,62500),-- Iron Ore
(20996,1878,1,1,2,25641),-- Braided Hemp
(20996,1884,1,1,2,38462),-- Cord
(20996,2169,1,1,2,2105),-- Recipe: Necklace of Binding
(20996,3019,1,1,2,1647),-- Recipe: Full Plate Shield
(20996,4039,1,1,2,2198),-- Mold Glue
(20996,8725,1,1,200,1464),-- Life Stone: level 52
(20996,8735,1,1,200,457),-- Mid-Grade Life Stone: level 52
(20997,1539,1,2,-1,275438),-- Greater Healing Potion
(20997,1915,1,2,-1,109205),-- Ring of Ages Gemstone
(20997,4601,1,1,-1,2754),-- Greater Dye of Strength <Str+3 Con-3>
(20997,57,601,1220,0,700000),-- Adena
(20997,60,1,1,1,36),-- Composite Armor
(20997,1977,1,1,1,3792),-- Composite Armor Temper
(20997,1868,1,1,2,111111),-- Thread
(20997,1873,1,1,2,25000),-- Silver Nugget
(20997,1876,1,1,2,11905),-- Mithril Ore
(20997,1895,1,1,2,14493),-- Metallic Fiber
(20997,2170,1,1,2,2114),-- Recipe: Nassen\'s Earring
(20997,3012,1,1,2,1209),-- Recipe: Full Plate Helmet
(20997,4042,1,1,2,680),-- Enria
(20997,8725,1,1,200,1739),-- Life Stone: level 52
(20997,8735,1,1,200,454),-- Mid-Grade Life Stone: level 52
(20997,8755,1,1,200,59),-- Top-Grade Life Stone: level 52
(20998,1916,1,1,-1,58159),-- Necklace of Mermaid Teardrop
(20998,2169,1,1,-1,24190),-- Recipe: Necklace of Binding
(20998,2170,1,1,-1,32234),-- Recipe: Nassen\'s Earring
(20998,57,664,1343,0,700000),-- Adena
(20998,162,1,1,1,2),-- War Axe
(20998,174,1,1,1,2),-- Nirvana Axe
(20998,196,1,1,1,2),-- Stick of Eternity
(20998,197,1,1,1,2),-- Paradia Staff
(20998,199,1,1,1,2),-- Pa\'agrian Hammer
(20998,200,1,1,1,2),-- Sage\'s Staff
(20998,201,1,1,1,2),-- Club of Nature
(20998,202,1,1,1,2),-- Mace of The Underworld
(20998,2097,1,1,1,190),-- War Axe Blade
(20998,2098,1,1,1,190),-- Nirvana Axe Blade
(20998,2099,1,1,1,200),-- Stick of Eternity Shaft
(20998,2100,1,1,1,200),-- Paradia Staff Head
(20998,2102,1,1,1,192),-- Pa\'agrian Hammer Head
(20998,2103,1,1,1,200),-- Sage\'s Staff Head
(20998,2104,1,1,1,200),-- Club of Nature Head
(20998,2105,1,1,1,190),-- Mace of The Underworld Head
(20998,1832,1,1,2,58824),-- Greater Antidote
(20998,1866,1,1,2,32258),-- Suede
(20998,1871,1,1,2,55556),-- Charcoal
(20998,1877,1,1,2,2062),-- Adamantite Nugget
(20998,2231,1,1,2,271),-- Recipe: Full Plate Armor
(20998,4040,1,1,2,1805),-- Mold Lubricant
(20998,4041,1,1,2,1289),-- Mold Hardener
(20998,5155,1,1,2,126),-- Recipe: Soulshot (B) Compressed Package (100%)
(20998,6669,1,1,2,12821),-- Deluxe Chest Key - Grade 5
(20998,8725,1,1,200,1031),-- Life Stone: level 52
(20998,8735,1,1,200,515),-- Mid-Grade Life Stone: level 52
(20999,1868,1,9,-1,811841),-- Thread
(20999,1873,1,2,-1,649473),-- Silver Nugget
(20999,1918,1,2,-1,70320),-- Nassen\'s Earring Gemstone
(20999,57,787,1598,0,700000),-- Adena
(20999,228,1,1,1,10),-- Crystal Dagger
(20999,2119,1,1,1,1367),-- Crystal Dagger Blade
(20999,1806,1,1,2,226),-- Recipe: Soulshot: B-Grade
(20999,1868,1,1,2,125000),-- Thread
(20999,1873,1,1,2,22727),-- Silver Nugget
(20999,1889,1,1,2,4975),-- Compound Braid
(20999,4044,1,1,2,1605),-- Thons
(20999,5270,1,1,2,150),-- Recipe: Greater Soulshot (B) Compressed Package(100%)
(20999,5280,1,1,2,113),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(20999,5549,1,1,2,5650),-- Metallic Thread
(20999,8726,1,1,200,2146),-- Life Stone: level 55
(20999,8736,1,1,200,113),-- Mid-Grade Life Stone: level 55
(21000,1865,1,3,-1,711817),-- Varnish
(21000,1918,1,1,-1,61656),-- Nassen\'s Earring Gemstone
(21000,1986,1,1,-1,15043),-- Full Plate Armor Temper
(21000,57,688,1402,0,700000),-- Adena
(21000,299,1,1,1,3),-- Orcish Poleaxe
(21000,301,1,1,1,4),-- Scorpion
(21000,303,1,1,1,4),-- Widow Maker
(21000,2113,1,1,1,425),-- Scorpion Blade
(21000,2114,1,1,1,425),-- Widowmaker Head
(21000,2122,1,1,1,400),-- Orcish Poleaxe Blade
(21000,1865,1,1,2,45455),-- Varnish
(21000,1870,1,1,2,45455),-- Coal
(21000,1875,1,1,2,4762),-- Stone of Purity
(21000,1879,1,1,2,6061),-- Cokes
(21000,4040,1,1,2,1570),-- Mold Lubricant
(21000,4146,1,1,2,121),-- Recipe: Tunic of Zubei
(21000,4182,1,1,2,121),-- Recipe: Great Sword
(21000,5160,1,1,2,117),-- Recipe: Spiritshot (B) Compressed Package (100%)
(21000,5436,1,1,2,81),-- Recipe: Berserker Blade(100%)
(21000,8726,1,1,200,685),-- Life Stone: level 55
(21000,8736,1,1,200,201);   -- Mid-Grade Life Stone: level 55

INSERT INTO `droplist` VALUES 
(21001,1895,1,3,-1,551315),-- Metallic Fiber
(21001,1988,1,1,-1,37076),-- Divine Tunic Fabric
(21001,4042,1,1,-1,32160),-- Enria
(21001,57,933,1901,0,700000),-- Adena
(21001,286,1,1,1,12),-- Eminence Bow
(21001,2121,1,1,1,1639),-- Eminence Bow Shaft
(21001,1868,1,1,2,125000),-- Thread
(21001,1873,1,1,2,27027),-- Silver Nugget
(21001,1876,1,1,2,12987),-- Mithril Ore
(21001,1895,1,1,2,15625),-- Metallic Fiber
(21001,4042,1,1,2,1869),-- Enria
(21001,4180,1,1,2,8403),-- Recipe: Silver Arrow
(21001,4939,1,1,2,493),-- Recipe: Adamantite Necklace (70%)
(21001,4966,1,1,2,33),-- Recipe: Keshanberk (60%)
(21001,4971,1,1,2,42),-- Recipe: Dark Elven Long Bow (60%)
(21001,5165,1,1,2,67),-- Recipe: Blessed Spiritshot (B) Compressed Package (100%)
(21001,8726,1,1,200,1110),-- Life Stone: level 55
(21001,8736,1,1,200,334),-- Mid-Grade Life Stone: level 55
(21001,8746,1,1,200,25),-- High-Grade Life Stone: level 55
(21002,1874,1,1,-1,113674),-- Oriharukon Ore
(21002,2960,1,2,-1,92845),-- Drake Leather Boots Design
(21002,4613,1,1,-1,2352),-- Greater Dye of Strength <Str+4 Con-4>
(21002,57,823,1681,0,700000),-- Adena
(21002,11896,1,1,1,45),-- Common Item - Dark Elven Long Bow
(21002,4112,1,1,1,1427),-- Dark Elven Longbow Shaft
(21002,1867,1,1,2,66667),-- Animal Skin
(21002,1872,1,1,2,76923),-- Animal Bone
(21002,1875,1,1,2,3125),-- Stone of Purity
(21002,1881,1,1,2,6211),-- Coarse Bone Powder
(21002,4041,1,1,2,331),-- Mold Hardener
(21002,4949,1,1,2,483),-- Recipe: Tunic of Zubei (60%)
(21002,4992,1,1,2,358),-- Recipe: Blue Wolf Boots (60%)
(21002,8311,1,1,2,18),-- Recipe: Kaim Vanul\'s Bones (100%)
(21002,8727,1,1,200,581),-- Life Stone: level 58
(21002,8737,1,1,200,98),-- Mid-Grade Life Stone: level 58
(21002,8747,1,1,200,9),-- High-Grade Life Stone: level 58
(21003,2963,1,2,-1,85150),-- Full Plate Gauntlets Part
(21003,4049,1,1,-1,82024),-- Adamantite Earring Gemstone
(21003,4949,1,1,-1,10024),-- Recipe: Tunic of Zubei (60%)
(21003,57,755,1542,0,700000),-- Adena
(21003,12160,1,1,1,317),-- Common Item - Zubei\'s Helmet
(21003,12159,1,1,1,489),-- Common Item - Zubei\'s Shield
(21003,12171,1,1,1,875),-- Common Item - Avadon Shield
(21003,12170,1,1,1,219),-- Common Item - Avadon Circlet
(21003,4072,1,1,1,2594),-- Avadon Circlet Pattern
(21003,4101,1,1,1,3348),-- Zubei\'s Shield Fragment
(21003,4102,1,1,1,2594),-- Zubei\'s Helmet Design
(21003,4439,1,1,1,3983),-- Avadon Shield Fragment
(21003,1832,1,1,2,47619),-- Greater Antidote
(21003,1866,1,1,2,27778),-- Suede
(21003,1871,1,1,2,43478),-- Charcoal
(21003,1885,1,1,2,3704),-- High Grade Suede
(21003,4041,1,1,2,691),-- Mold Hardener
(21003,4963,1,1,2,12),-- Recipe: Great Sword (60%)
(21003,4993,1,1,2,581),-- Recipe: Doom Gloves (60%)
(21003,4999,1,1,2,288),-- Recipe: Doom Boots (60%)
(21003,5220,1,1,2,1468),-- Metal Hardener
(21003,8727,1,1,200,782),-- Life Stone: level 58
(21003,8737,1,1,200,121),-- Mid-Grade Life Stone: level 58
(21003,8747,1,1,200,12),-- High-Grade Life Stone: level 58
(21003,8757,1,1,200,10),-- Top-Grade Life Stone: level 58
(21004,1920,1,1,-1,59425),-- Necklace of Binding Chain
(21004,4129,1,1,-1,18795),-- Recipe: Earrings of Black Ore
(21004,4951,1,1,-1,7617),-- Recipe: Avadon Robe (60%)
(21004,57,840,1710,0,700000),-- Adena
(21004,12151,1,1,1,305),-- Common Item - Zubei\'s Leather Shirt
(21004,12150,1,1,1,441),-- Common Item - Zubei\'s Leather Gaiters
(21004,12165,1,1,1,184),-- Common Item - Avadon Leather Armor
(21004,4066,1,1,1,2016),-- Zubei\'s Leather Shirt Fabric
(21004,4067,1,1,1,3060),-- Zubei\'s Leather Gaiter Texture
(21004,4068,1,1,1,1549),-- Avadon Leather Armor Lining
(21004,1868,1,1,2,100000),-- Thread
(21004,1873,1,1,2,21739),-- Silver Nugget
(21004,1876,1,1,2,10417),-- Mithril Ore
(21004,1895,1,1,2,15385),-- Metallic Fiber
(21004,4042,1,1,2,1139),-- Enria
(21004,4129,1,1,2,545),-- Recipe: Earrings of Black Ore
(21004,4167,1,1,2,545),-- Recipe: Blue Wolf Boots
(21004,4197,1,1,2,64),-- Recipe: Demon Dagger
(21004,8728,1,1,200,737),-- Life Stone: level 61
(21004,8738,1,1,200,152),-- Mid-Grade Life Stone: level 61
(21004,8748,1,1,200,8),-- High-Grade Life Stone: level 61
(21004,8758,1,1,200,16),-- Top-Grade Life Stone: level 61
(21005,947,1,1,-1,762),-- Scroll: Enchant Weapon (Grade B)
(21005,1880,1,2,-1,190431),-- Steel
(21005,4050,1,2,-1,117987),-- Adamantite Ring Wire
(21005,57,920,1876,0,700000),-- Adena
(21005,11889,1,1,1,34),-- Common Item - Great Sword
(21005,11916,1,1,1,34),-- Common Item - Keshanberk
(21005,11900,1,1,1,108),-- Common Item - Sword of Valhalla
(21005,4104,1,1,1,537),-- Great Sword Blade
(21005,4107,1,1,1,529),-- Keshanberk Blade
(21005,4108,1,1,1,529),-- Sword of Valhalla Blade
(21005,1868,1,1,2,111111),-- Thread
(21005,1873,1,1,2,22222),-- Silver Nugget
(21005,1876,1,1,2,11364),-- Mithril Ore
(21005,1895,1,1,2,16949),-- Metallic Fiber
(21005,4042,1,1,2,1030),-- Enria
(21005,4129,1,1,2,549),-- Recipe: Earrings of Black Ore
(21005,4167,1,1,2,678),-- Recipe: Blue Wolf Boots
(21005,4197,1,1,2,27),-- Recipe: Demon Dagger
(21005,8728,1,1,200,935),-- Life Stone: level 61
(21005,8738,1,1,200,271),-- Mid-Grade Life Stone: level 61
(21005,8748,1,1,200,41),-- High-Grade Life Stone: level 61
(21006,948,1,1,-1,4532),-- Scroll: Enchant Armor (Grade B)
(21006,4066,1,1,-1,42738),-- Zubei\'s Leather Shirt Fabric
(21006,4130,1,1,-1,29428),-- Recipe: Ring of Black Ore
(21006,57,878,1784,0,700000),-- Adena
(21006,12161,1,1,1,176),-- Common Item - Zubei\'s Breastplate
(21006,12152,1,1,1,234),-- Common Item - Zubei\'s Gaiters
(21006,12173,1,1,1,285),-- Common Item - Avadon Breastplate
(21006,12172,1,1,1,241),-- Common Item - Avadon Gaiters
(21006,4056,1,1,1,1212),-- Zubei\'s Breastplate Part
(21006,4057,1,1,1,1872),-- Zubei\'s Gaiter Material
(21006,4064,1,1,1,1212),-- Avadon Breastplate Part
(21006,4065,1,1,1,1872),-- Avadon Gaiters Material
(21006,1864,1,1,2,111111),-- Stem
(21006,1869,1,1,2,55556),-- Iron Ore
(21006,1880,1,1,2,5556),-- Steel
(21006,4043,1,1,2,1471),-- Asofe
(21006,4173,1,1,2,417),-- Recipe: Blue Wolf Gloves
(21006,4193,1,1,2,80),-- Recipe: Lance
(21006,5003,1,1,2,15),-- Recipe: Art of Battle Axe (60%)
(21006,5550,1,1,2,797),-- Durable Metal Plate
(21006,8728,1,1,200,1229),-- Life Stone: level 61
(21006,8738,1,1,200,307),-- Mid-Grade Life Stone: level 61
(21006,8748,1,1,200,29),-- High-Grade Life Stone: level 61
(21007,948,1,1,-1,4875),-- Scroll: Enchant Armor (Grade B)
(21007,4051,1,1,-1,75618),-- Adamantite Necklace Chain
(21007,4981,1,1,-1,6055),-- Recipe: Blue Wolf Breastplate (60%)
(21007,57,946,1918,0,700000),-- Adena
(21007,11918,1,1,1,108),-- Common Item - Heavy War Axe
(21007,11908,1,1,1,108),-- Common Item - Sprite\'s Staff
(21007,4105,1,1,1,824),-- Heavy War Axe Head
(21007,4106,1,1,1,811),-- Sprite\'s Staff Head
(21007,1832,1,1,2,71429),-- Greater Antidote
(21007,1894,1,1,2,9767),-- Crafted Leather
(21007,4039,1,1,2,1953),-- Mold Glue
(21007,4041,1,1,2,1085),-- Mold Hardener
(21007,4044,1,1,2,2604),-- Thons
(21007,4166,1,1,2,325),-- Recipe: Doom Helmet
(21007,4621,1,1,2,2170),-- Greater Dye of Mental Strength <Men+4 Int-4>
(21007,4998,1,1,2,651),-- Recipe: Blue Wolf Gloves (60%)
(21007,8728,1,1,200,1302),-- Life Stone: level 61
(21007,8738,1,1,200,434),-- Mid-Grade Life Stone: level 61
(21007,8748,1,1,200,97),-- High-Grade Life Stone: level 61
(21008,4052,1,1,-1,88462),-- Earring of Black Ore Piece
(21008,4983,1,1,-1,5305),-- Recipe: Doom Plate Armor (60%)
(21008,5230,1,1,-1,3331),-- Recipe: Mithril Arrow (100%)
(21008,57,1213,2457,0,700000),-- Adena
(21008,12188,1,1,1,784),-- Common Item - Boots of Doom
(21008,12189,1,1,1,784),-- Common Item - Blue Wolf Boots
(21008,12187,1,1,1,1176),-- Common Item - Doom Gloves
(21008,12190,1,1,1,196),-- Common Item - Blue Wolf Gloves
(21008,4090,1,1,1,4141),-- Blue Wolf Boots Design
(21008,4091,1,1,1,4141),-- Doom Gloves Part
(21008,4096,1,1,1,4141),-- Blue Wolf Gloves Fabric
(21008,4099,1,1,1,4141),-- Doom Boots Part
(21008,1867,1,1,2,111111),-- Animal Skin
(21008,1872,1,1,2,111111),-- Animal Bone
(21008,1882,1,1,2,23810),-- Leather
(21008,1894,1,1,2,3333),-- Crafted Leather
(21008,4042,1,1,2,1176),-- Enria
(21008,4983,1,1,2,224),-- Recipe: Doom Plate Armor (60%)
(21008,5276,1,1,2,141),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21008,8729,1,1,200,1961),-- Life Stone: level 64
(21008,8739,1,1,200,392),-- Mid-Grade Life Stone: level 64
(21009,948,1,1,-1,6677),-- Scroll: Enchant Armor (Grade B)
(21009,4052,1,1,-1,94558),-- Earring of Black Ore Piece
(21009,4104,1,1,-1,15284),-- Great Sword Blade
(21009,57,1297,2625,0,700000),-- Adena
(21009,12204,1,1,1,31),-- Common Item - Blue Wolf Breastplate
(21009,12193,1,1,1,248),-- Common Item - Blue Wolf Gaiters
(21009,12184,1,1,1,155),-- Common Item - Doom Plate Armor
(21009,4078,1,1,1,1921),-- Blue Wolf Breastplate Part
(21009,4080,1,1,1,2705),-- Blue Wolf Gaiters Material
(21009,4081,1,1,1,1296),-- Doom Plate Armor Temper
(21009,1865,1,1,2,90909),-- Varnish
(21009,1870,1,1,2,90909),-- Coal
(21009,1875,1,1,2,7752),-- Stone of Purity
(21009,1879,1,1,2,15873),-- Cokes
(21009,4040,1,1,2,2169),-- Mold Lubricant
(21009,5004,1,1,2,86),-- Recipe: Staff of Evil Spirits (60%)
(21009,5281,1,1,2,151),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21009,8729,1,1,200,1116),-- Life Stone: level 64
(21009,8739,1,1,200,279),-- Mid-Grade Life Stone: level 64
(21009,8759,1,1,200,93),-- Top-Grade Life Stone: level 64
(21010,4621,1,1,-1,2958),-- Greater Dye of Mental Strength <Men+4 Int-4>
(21010,5230,1,1,-1,2859),-- Recipe: Mithril Arrow (100%)
(21010,57,1042,2108,0,700000),-- Adena
(21010,11933,1,1,1,50),-- Common Item - Bellion Cestus
(21010,4120,1,1,1,1385),-- Bellion Cestus Edge
(21010,1868,1,1,2,125000),-- Thread
(21010,1873,1,1,2,18182),-- Silver Nugget
(21010,1889,1,1,2,4016),-- Compound Braid
(21010,3956,1,1,2,50),-- Recipe: Blessed Spiritshot A
(21010,4044,1,1,2,2558),-- Thons
(21010,4129,1,1,2,752),-- Recipe: Earrings of Black Ore
(21010,4624,1,1,2,98),-- Greater Dye of Wit <Wit+4 Men-4>
(21010,5549,1,1,2,5464),-- Metallic Thread
(21010,6670,1,1,2,10526),-- Deluxe Chest Key - Grade 6
(21010,8729,1,1,200,1103),-- Life Stone: level 64
(21010,8739,1,1,200,150),-- Mid-Grade Life Stone: level 64
(21011,1832,1,1,-1,491646),-- Greater Antidote
(21011,1884,1,1,-1,272296),-- Cord
(21011,1900,1,1,-1,48411),-- Enchanted Necklace Chain
(21011,57,224,444,0,700000),-- Adena
(21011,46,1,1,1,484),-- Bronze Helmet
(21011,627,1,1,1,691),-- Aspis
(21011,1061,1,1,2,18319),-- Healing Potion
(21011,1864,1,1,2,60450),-- Stem
(21011,1865,1,1,2,30225),-- Varnish
(21011,1866,1,1,2,20150),-- Suede
(21011,1868,1,1,2,60450),-- Thread
(21011,1873,1,1,2,12090),-- Silver Nugget
(21011,2185,1,1,2,5950),-- Recipe: Iron Boots
(21011,2274,1,1,2,216),-- Recipe: Skull Breaker
(21012,1936,1,1,-1,44548),-- White Tunic Pattern
(21012,2153,1,1,-1,84661),-- Recipe: Tiger\'s Eye Earring
(21012,2185,1,1,-1,107160),-- Recipe: Iron Boots
(21012,57,277,550,0,700000),-- Adena
(21012,275,1,1,1,31),-- Long Bow
(21012,276,1,1,1,31),-- Elven Bow
(21012,2024,1,1,1,1413),-- Elven Bow Shaft
(21012,1833,1,1,2,113326),-- Bandage
(21012,1868,1,1,2,84994),-- Thread
(21012,1873,1,1,2,16999),-- Silver Nugget
(21012,1889,1,1,2,2834),-- Compound Braid
(21012,2264,1,1,2,440),-- Recipe: Two-handed Sword
(21012,2273,1,1,2,304),-- Recipe: Tarbar
(21012,5549,1,1,2,4250),-- Metallic Thread
(21013,1061,1,1,-1,388921),-- Healing Potion
(21013,1933,1,1,-1,75496),-- Dark Stocking Fabric
(21013,2023,1,1,-1,7952),-- Conjuror\'s Staff Head
(21013,57,324,649,0,700000),-- Adena
(21013,848,1,1,1,412),-- Enchanted Earring
(21013,879,1,1,1,618),-- Enchanted Ring
(21013,911,1,1,1,309),-- Enchanted Necklace
(21013,1900,1,1,1,9592),-- Enchanted Necklace Chain
(21013,737,1,1,2,5845),-- Scroll of Resurrection
(21013,1061,1,1,2,35422),-- Healing Potion
(21013,1866,1,1,2,38964),-- Suede
(21013,1871,1,1,2,58446),-- Charcoal
(21013,1877,1,1,2,2338),-- Adamantite Nugget
(21013,2187,1,1,2,2610),-- Recipe: Manticore Skin Shirt
(21014,1865,1,4,-1,662426),-- Varnish
(21014,1958,1,1,-1,59544),-- Mithril Boots Design
(21014,2977,1,1,-1,88678),-- Recipe: Brigandine Boots
(21014,57,336,671,0,700000),-- Adena
(21014,316,1,1,1,25),-- Blood of Saints
(21014,317,1,1,1,25),-- Tome of Blood
(21014,2030,1,1,1,1374),-- Tome of Blood Page
(21014,1865,1,1,2,58824),-- Varnish
(21014,1870,1,1,2,62500),-- Coal
(21014,1875,1,1,2,9346),-- Stone of Purity
(21014,1879,1,1,2,11236),-- Cokes
(21014,2189,1,1,2,2621),-- Recipe: Mithril Tunic
(21014,2286,1,1,2,327),-- Recipe: Bonebreaker
(21015,1869,1,2,-1,675737),-- Iron Ore
(21015,1873,1,1,-1,270295),-- Silver Nugget
(21015,1940,1,1,-1,100856),-- Iron Boots Design
(21015,57,342,683,0,700000),-- Adena
(21015,83,1,1,1,17),-- Sword of Magic
(21015,124,1,1,1,13),-- Two-Handed Sword
(21015,127,1,1,1,13),-- Crimson Sword
(21015,130,1,1,1,13),-- Elven Sword
(21015,144,1,1,1,17),-- Sword of Occult
(21015,2026,1,1,1,721),-- Two-Handed Sword Edge
(21015,1868,1,1,2,166667),-- Thread
(21015,1873,1,1,2,28571),-- Silver Nugget
(21015,1876,1,1,2,12987),-- Mithril Ore
(21015,2280,1,1,2,514),-- Recipe: Crucifix of Blood
(21015,2978,1,1,2,9091),-- Recipe: Elven Mithril Boots
(21016,1870,1,2,-1,723751),-- Coal
(21016,1882,1,1,-1,160833),-- Leather
(21016,1901,1,1,-1,99966),-- Tiger\'s Eye Earring Stone
(21016,57,367,732,0,700000),-- Adena
(21016,58,1,1,1,61),-- Mithril Breastplate
(21016,59,1,1,1,87),-- Mithril Gaiters
(21016,349,1,1,1,61),-- Compound Scale Mail
(21016,350,1,1,1,61),-- Dwarven Scale Mail
(21016,351,1,1,1,55),-- Blast Plate
(21016,378,1,1,1,87),-- Compound Scale Gaiters
(21016,379,1,1,1,87),-- Dwarven Scale Gaiters
(21016,1864,1,1,2,111111),-- Stem
(21016,1869,1,1,2,58824),-- Iron Ore
(21016,1880,1,1,2,5495),-- Steel
(21016,2191,1,1,2,6024),-- Recipe: RIP Gauntlets
(21016,2276,1,1,2,393),-- Recipe: Maingauche
(21016,2293,1,1,2,306),-- Recipe: Glaive
(21016,5550,1,1,2,733),-- Durable Metal Plate
(21017,1806,1,1,-1,2899),-- Recipe: Soulshot: B-Grade
(21017,1880,1,1,-1,144955),-- Steel
(21017,1989,1,1,-1,43688),-- Divine Stocking Pattern
(21017,57,701,1428,0,700000),-- Adena
(21017,400,1,1,1,22),-- Theca Leather Armor
(21017,401,1,1,1,12),-- Drake Leather Armor
(21017,420,1,1,1,35),-- Theca Leather Gaiters
(21017,1984,1,1,1,1913),-- Theca Leather Armor Pattern
(21017,1985,1,1,1,3077),-- Theca Leather Gaiters Pattern
(21017,1987,1,1,1,1362),-- Drake Leather Armor Texture
(21017,1866,1,1,2,34483),-- Suede
(21017,1871,1,1,2,526323),-- Charcoal
(21017,1885,1,1,2,4566),-- High Grade Suede
(21017,4041,1,1,2,694),-- Mold Hardener
(21017,4189,1,1,2,78),-- Recipe: Arthro Nail
(21017,4940,1,1,2,289),-- Recipe: Zubei\'s Breastplate (60%)
(21017,5220,1,1,2,2315),-- Metal Hardener
(21017,5280,1,1,2,231),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(21017,8726,1,1,200,1157),-- Life Stone: level 55
(21017,8736,1,1,200,174),-- Mid-Grade Life Stone: level 55
(21018,2961,1,1,-1,99537),-- Full Plate Boots Part
(21018,4049,1,1,-1,95882),-- Adamantite Earring Gemstone
(21018,4947,1,1,-1,18787),-- Recipe: Zubei\'s Leather Gaiters (60%)
(21018,57,883,1802,0,700000),-- Adena
(21018,11918,1,1,1,85),-- Common Item - Heavy War Axe
(21018,11908,1,1,1,56),-- Common Item - Sprite\'s Staff
(21018,4105,1,1,1,773),-- Heavy War Axe Head
(21018,4106,1,1,1,760),-- Sprite\'s Staff Head
(21018,1867,1,1,2,71429),-- Animal Skin
(21018,1872,1,1,2,71429),-- Animal Bone
(21018,1875,1,1,2,3401),-- Stone of Purity
(21018,1881,1,1,2,6536),-- Coarse Bone Powder
(21018,4041,1,1,2,529),-- Mold Hardener
(21018,4950,1,1,2,508),-- Recipe: Stockings of Zubei (60%)
(21018,4999,1,1,2,656),-- Recipe: Doom Boots (60%)
(21018,8309,1,1,2,85),-- Recipe: Spell Breaker (100%)
(21018,8727,1,1,200,1776),-- Life Stone: level 58
(21018,8737,1,1,200,254),-- Mid-Grade Life Stone: level 58
(21019,2964,1,1,-1,109806),-- Divine Gloves Pattern
(21019,4129,1,1,-1,21825),-- Recipe: Earrings of Black Ore
(21019,4950,1,1,-1,20726),-- Recipe: Stockings of Zubei (60%)
(21019,57,973,1988,0,700000),-- Adena
(21019,11903,1,1,1,222),-- Common Item - Arthro Nail
(21019,4111,1,1,1,1705),-- Arthro Nail Blade
(21019,1865,1,1,2,58824),-- Varnish
(21019,1870,1,1,2,55556),-- Coal
(21019,1874,1,1,2,4854),-- Oriharukon Ore
(21019,4039,1,1,2,1553),-- Mold Glue
(21019,4043,1,1,2,2198),-- Asofe
(21019,4176,1,1,2,843),-- Recipe: Doom Boots
(21019,4964,1,1,2,67),-- Recipe: Heavy War Axe (60%)
(21019,5003,1,1,2,44),-- Recipe: Art of Battle Axe (60%)
(21019,8727,1,1,200,1198),-- Life Stone: level 58
(21019,8737,1,1,200,466),-- Mid-Grade Life Stone: level 58
(21019,8747,1,1,200,44),-- High-Grade Life Stone: level 58
(21020,1987,1,1,-1,34883),-- Drake Leather Armor Texture
(21020,4050,1,1,-1,141660),-- Adamantite Ring Wire
(21020,4952,1,1,-1,29388),-- Recipe: Avadon Circlet (60%)
(21020,57,1105,2252,0,700000),-- Adena
(21020,11889,1,1,1,54),-- Common Item - Great Sword
(21020,11916,1,1,1,54),-- Common Item - Keshanberk
(21020,11900,1,1,1,54),-- Common Item - Sword of Valhalla
(21020,4104,1,1,1,645),-- Great Sword Blade
(21020,4107,1,1,1,635),-- Keshanberk Blade
(21020,4108,1,1,1,635),-- Sword of Valhalla Blade
(21020,1832,1,1,2,500000),-- Greater Antidote
(21020,4167,1,1,2,5155),-- Recipe: Blue Wolf Boots
(21021,947,1,1,-1,844),-- Scroll: Enchant Weapon (Grade B)
(21021,4040,1,1,-1,42217),-- Mold Lubricant
(21021,4613,1,1,-1,2912),-- Greater Dye of Strength <Str+4 Con-4>
(21021,57,1020,2079,0,700000),-- Adena
(21021,11896,1,1,1,95),-- Common Item - Dark Elven Long Bow
(21021,4112,1,1,1,1765),-- Dark Elven Longbow Shaft
(21021,1867,1,1,2,76923),-- Animal Skin
(21021,1869,1,1,2,58824),-- Iron Ore
(21021,1870,1,1,2,58824),-- Coal
(21021,1871,1,1,2,62500),-- Charcoal
(21021,1872,1,1,2,83333),-- Animal Bone
(21021,4130,1,1,2,893),-- Recipe: Ring of Black Ore
(21021,4998,1,1,2,893),-- Recipe: Blue Wolf Gloves (60%)
(21021,5001,1,1,2,76),-- Recipe: Lance (60%)
(21021,8728,1,1,200,722),-- Life Stone: level 61
(21021,8738,1,1,200,247),-- Mid-Grade Life Stone: level 61
(21022,947,1,1,-1,860),-- Scroll: Enchant Weapon (Grade B)
(21022,4049,1,1,-1,112794),-- Adamantite Earring Gemstone
(21022,4613,1,1,-1,2966),-- Greater Dye of Strength <Str+4 Con-4>
(21022,57,1041,2116,0,700000),-- Adena
(21022,12175,1,1,1,200),-- Common Item - Tunic of Zubei
(21022,12176,1,1,1,474),-- Common Item - Stockings of Zubei
(21022,12166,1,1,1,150),-- Common Item - Avadon Robe
(21022,4069,1,1,1,2496),-- Tunic of Zubei Fabric
(21022,4070,1,1,1,3789),-- Stockings of Zubei Fabric
(21022,4071,1,1,1,1918),-- Avadon Robe Fabric
(21022,1867,1,1,2,83333),-- Animal Skin
(21022,1869,1,1,2,66667),-- Iron Ore
(21022,1870,1,1,2,62500),-- Coal
(21022,1871,1,1,2,62500),-- Charcoal
(21022,1872,1,1,2,76923),-- Animal Bone
(21022,4130,1,1,2,1172),-- Recipe: Ring of Black Ore
(21022,4998,1,1,2,699),-- Recipe: Blue Wolf Gloves (60%)
(21022,5001,1,1,2,50),-- Recipe: Lance (60%)
(21022,8728,1,1,200,1647),-- Life Stone: level 61
(21022,8738,1,1,200,150),-- Mid-Grade Life Stone: level 61
(21022,8748,1,1,200,649),-- High-Grade Life Stone: level 61
(21023,1866,1,1,-1,245597),-- Suede
(21023,2019,1,1,-1,7922),-- Trident Edge
(21023,2263,1,1,-1,5720),-- Recipe: Dwarven Trident
(21023,57,193,367,0,700000),-- Adena
(21023,156,1,1,1,34),-- Hand Axe
(21023,167,1,1,1,34),-- Scalpel
(21023,178,1,1,1,34),-- Bone Staff
(21023,1864,1,1,2,71429),-- Stem
(21023,1865,1,1,2,55556),-- Varnish
(21023,1866,1,1,2,6024),-- Suede
(21023,1868,1,1,2,71429),-- Thread
(21023,1873,1,1,2,6024),-- Silver Nugget
(21023,2260,1,1,2,535),-- Recipe: Mace of Judgment
(21024,1899,1,1,-1,60101),-- Necklace of Devotion Chain
(21024,1934,1,1,-1,50280),-- Crafted Leather Gloves Lining
(21024,2178,1,1,-1,61543),-- Recipe: Dark Stockings
(21024,57,200,383,0,700000),-- Adena
(21024,220,1,1,1,106),-- Crafted Dagger
(21024,1864,1,1,2,83333),-- Stem
(21024,1869,1,1,2,47619),-- Iron Ore
(21024,1878,1,1,2,14925),-- Braided Hemp
(21024,1884,1,1,2,38462),-- Cord
(21024,2180,1,1,2,1153),-- Recipe: Scale Mail
(21025,1868,1,3,-1,808633),-- Thread
(21025,1871,1,1,-1,404316),-- Charcoal
(21025,2152,1,1,-1,71182),-- Recipe: Enchanted Necklace
(21025,57,208,407,0,700000),-- Adena
(21025,291,1,1,1,56),-- Trident
(21025,2019,1,1,1,2440),-- Trident Edge
(21025,1864,1,1,2,100000),-- Stem
(21025,1865,1,1,2,400009),-- Varnish
(21025,1866,1,1,2,27027),-- Suede
(21025,1868,1,1,2,90909),-- Thread
(21025,1873,1,1,2,19231),-- Silver Nugget
(21025,2180,1,1,2,2660),-- Recipe: Scale Mail
(21026,1834,1,2,-1,479579),-- Emergency Dressing
(21026,1900,1,1,-1,47223),-- Enchanted Necklace Chain
(21026,2025,1,1,-1,5978),-- Dwarven Trident Edge
(21026,57,220,435,0,700000),-- Adena
(21026,275,1,1,1,25),-- Long Bow
(21026,276,1,1,1,25),-- Elven Bow
(21026,2024,1,1,1,1121),-- Elven Bow Shaft
(21026,737,1,1,2,3236),-- Scroll of Resurrection
(21026,1867,1,1,2,58824),-- Animal Skin
(21026,1872,1,1,2,52632),-- Animal Bone
(21026,1875,1,1,2,4630),-- Stone of Purity
(21026,1881,1,1,2,5102),-- Coarse Bone Powder
(21026,2191,1,1,2,4630),-- Recipe: RIP Gauntlets
(21026,2274,1,1,2,463),-- Recipe: Skull Breaker
(21034,1956,1,1,-1,66832),-- Square Shield Fragment
(21034,2154,1,1,-1,63039),-- Recipe: Elven Earring
(21034,2302,1,1,-1,2571),-- Recipe: Silver Axe
(21034,57,296,590,0,700000),-- Adena
(21034,88,1,1,1,5),-- Morning Star
(21034,90,1,1,1,5),-- Goat Head Staff
(21034,158,1,1,1,5),-- Tarbar
(21034,169,1,1,1,5),-- Skull Breaker
(21034,172,1,1,1,5),-- Heavy Bone Club
(21034,2031,1,1,1,309),-- Morning Star Head
(21034,2032,1,1,1,282),-- Goat Head Staff Head
(21034,2035,1,1,1,309),-- Tarbar Head
(21034,2036,1,1,1,309),-- Skull Breaker Head
(21034,2037,1,1,1,309),-- Heavy Bone Club Head
(21034,1341,10,30,2,142857),-- Bone Arrow
(21034,1874,1,1,2,3165),-- Oriharukon Ore
(21034,1878,1,1,2,19608),-- Braided Hemp
(21034,1884,1,1,2,27027),-- Cord
(21034,1889,1,1,2,3401),-- Compound Braid
(21034,2190,1,1,2,1460),-- Recipe: Mithril Stockings
(21034,2279,1,1,2,272),-- Recipe: War Pick
(21034,5220,1,1,2,1217),-- Metal Hardener
(21035,1873,1,3,-1,593536),-- Silver Nugget
(21035,1905,1,1,-1,144342),-- Omen Beast\'s Eye Earring Gemstone
(21035,1952,1,1,-1,47529),-- Salamander Skin Mail Texture
(21035,57,916,1852,0,700000),-- Adena
(21035,396,1,1,1,174),-- Salamander Skin Mail
(21035,1952,1,1,1,11934),-- Salamander Skin Mail Texture
(21035,1865,1,1,2,100000),-- Varnish
(21035,1870,1,1,2,111111),-- Coal
(21035,1875,1,1,2,6536),-- Stone of Purity
(21035,1879,1,1,2,15385),-- Cokes
(21035,2205,1,1,2,3571),-- Recipe: Chain Gaiters
(21035,2308,1,1,2,132),-- Recipe: Cursed Dagger
(21035,2989,1,1,2,7143),-- Recipe: Chain Hood
(21035,2997,1,1,2,9174),-- Recipe: Karmian Gloves
(21035,6668,1,1,2,37037),-- Deluxe Chest Key - Grade 4
(21036,1895,1,3,-1,534500),-- Metallic Fiber
(21036,2070,1,1,-1,14854),-- Cursed Dagger Blade
(21036,2316,1,1,-1,6518),-- Recipe: Katana
(21036,57,1057,2156,0,700000),-- Adena
(21036,354,1,1,1,88),-- Chain Mail Shirt
(21036,381,1,1,1,140),-- Chain Gaiters
(21036,1959,1,1,1,5955),-- Chain Mail Shirt Material
(21036,1960,1,1,1,12113),-- Chain Gaiters Part
(21036,1895,1,1,2,50000),-- Metallic Fiber
(21036,2318,1,1,2,1541),-- Recipe: Spirit Sword
(21036,8743,1,1,200,1115),-- High-Grade Life Stone: level 46
(21036,8733,1,1,200,1115),-- Mid-Grade Life Stone: level 46
(21036,8723,1,1,200,10000),-- Life Stone: level 46
(21037,2166,1,1,-1,65513),-- Recipe: Earrings of Binding (100%)
(21037,4040,1,1,-1,38652),-- Mold Lubricant
(21037,5231,1,1,-1,138044),-- Recipe: Metal Hardener (100%)
(21037,57,1372,2791,0,700000),-- Adena
(21037,1126,1,1,1,89),-- Crimson Boots
(21037,1974,1,1,1,6001),-- Crimson Boot Fabric
(21037,2434,1,1,1,89),-- Rind Leather Boots
(21037,2455,1,1,1,96),-- Plated Leather Gloves
(21037,2456,1,1,1,89),-- Dwarven Chain Gloves
(21037,2457,1,1,1,89),-- Gloves of Seal
(21037,2458,1,1,1,89),-- Rind Leather Gloves
(21037,2946,1,1,1,3989),-- Plate Leather Gloves Design
(21037,2948,1,1,1,6001),-- Rind Leather Boot Lining
(21037,2949,1,1,1,6001),-- Dwarven Chain Gloves Design
(21037,2950,1,1,1,6001),-- Gloves of Seal Pattern
(21037,2951,1,1,1,6001),-- Rind Leather Gloves Design
(21037,1895,1,1,2,34483),-- Metallic Fiber
(21037,2225,1,1,2,1473),-- Recipe: Demon\'s Stockings
(21037,2329,1,1,2,249),-- Recipe: Tsurugi
(21037,2344,1,1,2,249),-- Recipe: Grace Dagger
(21037,2345,1,1,2,249),-- Recipe: Dark Screamer
(21037,4040,1,1,2,736),-- Mold Lubricant
(21037,4042,1,1,2,1473),-- Enria
(21037,4043,1,1,2,5155),-- Asofe
(21037,8724,1,1,200,4425),-- Life Stone: level 49
(21037,8734,1,1,200,1473),-- Mid-Grade Life Stone: level 49
(21038,1911,1,1,-1,142220),-- Earring of Protection Gemstone
(21038,2166,1,1,-1,64819),-- Recipe: Earrings of Binding (100%)
(21038,5231,1,1,-1,136582),-- Recipe: Metal Hardener (100%)
(21038,57,1037,2113,0,700000),-- Adena
(21038,500,1,1,1,117),-- Great Helmet
(21038,632,1,1,1,168),-- Knight\'s Shield
(21038,1971,1,1,1,8645),-- Great Helmet Material
(21038,1972,1,1,1,11297),-- Knight\'s Shield Fragment
(21038,2496,1,1,1,182),-- Dwarven Chain Shield
(21038,2947,1,1,1,10087),-- Dwarven Chain Shield Fragment
(21038,952,1,1,2,831),-- Scroll: Enchant Armor (Grade C)
(21038,1867,1,1,2,125000),-- Animal Skin
(21038,1872,1,1,2,125000),-- Animal Bone
(21038,1875,1,1,2,5814),-- Stone of Purity
(21038,1881,1,1,2,20833),-- Coarse Bone Powder
(21038,2224,1,1,2,1664),-- Recipe: Demon\'s Tunic
(21038,4041,1,1,2,1664),-- Mold Hardener
(21038,8724,1,1,200,1664),-- Life Stone: level 49
(21038,8734,1,1,200,2494),-- Mid-Grade Life Stone: level 49
(21039,1805,1,1,-1,6613),-- Recipe: Soulshot: C-Grade
(21039,1871,1,5,-1,661279),-- Charcoal
(21039,1905,1,1,-1,192980),-- Omen Beast\'s Eye Earring Gemstone
(21039,57,1039,2093,0,700000),-- Adena
(21039,70,1,1,1,28),-- Claymore
(21039,2047,1,1,1,2190),-- Claymore Blade
(21039,2499,1,1,1,28),-- Elven Long Sword
(21039,2967,1,1,1,2160),-- Elven Long Sword Blade
(21039,1866,1,1,2,100000),-- Suede
(21039,1871,1,1,2,200000),-- Charcoal
(21039,1877,1,1,2,6410),-- Adamantite Nugget
(21039,2162,1,1,2,8021),-- Recipe: Aquastone Necklace
(21039,2312,1,1,2,725),-- Recipe: Crystallized Ice Bow
(21039,5159,1,1,2,554),-- Recipe: Spiritshot (C) Compressed Package (100%)
(21040,1875,1,1,-1,204167),-- Stone of Purity
(21040,1911,1,1,-1,227781),-- Earring of Protection Gemstone
(21040,1967,1,1,-1,167031),-- Plated Leather Gaiters Material
(21040,57,1649,3351,0,700000),-- Adena
(21040,195,1,1,1,24),-- Cursed Staff
(21040,2082,1,1,1,2391),-- Cursed Staff Head
(21040,2502,1,1,1,24),-- Dwarven War Hammer
(21040,2968,1,1,1,2289),-- Dwarven Warhammer Head
(21040,952,1,1,2,2585),-- Scroll: Enchant Armor (Grade C)
(21040,1867,1,1,2,200000),-- Animal Skin
(21040,1872,1,1,2,200000),-- Animal Bone
(21040,1875,1,1,2,7194),-- Stone of Purity
(21040,1881,1,1,2,30303),-- Coarse Bone Powder
(21040,2224,1,1,2,1805),-- Recipe: Demon\'s Tunic
(21040,4041,1,1,2,7194),-- Mold Hardener
(21040,8724,1,1,200,1754),-- Life Stone: level 49
(21040,8734,1,1,200,2924),-- Mid-Grade Life Stone: level 49
(21040,8744,1,1,200,585),-- High-Grade Life Stone: level 49
(21058,4049,1,1,-1,256419),-- Adamantite Earring Gemstone
(21058,4129,1,1,-1,52907),-- Recipe: Earrings of Black Ore
(21058,4617,1,1,-1,6743),-- Greater Dye of Dexterity <Dex+4 Str-4>
(21058,57,2369,4811,0,700000),-- Adena
(21058,78,1,1,1,6),-- Great Sword
(21058,142,1,1,1,6),-- Keshanberk
(21058,148,1,1,1,6),-- Sword of Valhalla
(21058,4104,1,1,1,1378),-- Great Sword Blade
(21058,4107,1,1,1,1356),-- Keshanberk Blade
(21058,4108,1,1,1,1356),-- Sword of Valhalla Blade
(21058,1895,1,1,2,40000),-- Metallic Fiber
(21058,4040,1,1,2,5556),-- Mold Lubricant
(21058,4042,1,1,2,3439),-- Enria
(21058,4043,1,1,2,3176),-- Asofe
(21058,4163,1,1,2,1585),-- Recipe: Blue Wolf Stockings
(21058,4195,1,1,2,158),-- Recipe: Art of Battle Axe
(21058,4999,1,1,2,2566),-- Recipe: Doom Boots (60%)
(21058,8728,1,1,200,1585),-- Life Stone: level 61
(21059,4049,1,2,-1,219194),-- Adamantite Earring Gemstone
(21059,4129,1,1,-1,45227),-- Recipe: Earrings of Black Ore
(21059,4130,1,1,-1,67840),-- Recipe: Ring of Black Ore
(21059,57,2020,4116,0,700000),-- Adena
(21059,284,1,1,1,15),-- Dark Elven Long Bow
(21059,4112,1,1,1,3495),-- Dark Elven Longbow Shaft
(21059,1864,1,3,2,125000),-- Stem
(21059,1869,1,1,2,125000),-- Iron Ore
(21059,1878,1,1,2,47619),-- Braided Hemp
(21059,1884,1,1,2,83333),-- Cord
(21059,4039,1,1,2,7143),-- Mold Glue
(21059,4154,1,1,2,843),-- Recipe: Doom Shield
(21059,4194,1,1,2,118),-- Recipe: Deadman\'s Glory
(21059,4198,1,1,2,118),-- Recipe: Bellion Cestus
(21059,8728,1,1,200,843),-- Life Stone: level 61
(21059,8748,1,1,200,66),-- High-Grade Life Stone: level 61
(21060,947,1,1,-1,1672),-- Scroll: Enchant Weapon (Grade B)
(21060,1865,1,11,-1,835788),-- Varnish
(21060,4050,1,1,-1,258918),-- Adamantite Ring Wire
(21060,57,2020,4113,0,700000),-- Adena
(21060,229,1,1,1,8),-- Kris
(21060,243,1,1,1,8),-- Hell Knife
(21060,4109,1,1,1,1739),-- Kris Edge
(21060,4110,1,1,1,1739),-- Hell Knife Edge
(21060,1343,20,60,2,100000),-- Silver Arrow
(21060,1865,1,1,2,125000),-- Varnish
(21060,1870,1,1,2,125000),-- Coal
(21060,1875,1,1,2,7407),-- Stone of Purity
(21060,1879,1,1,2,23810),-- Cokes
(21060,4040,1,1,2,2882),-- Mold Lubricant
(21060,4155,1,1,2,822),-- Recipe: Blue Wolf Breastplate
(21060,8728,1,1,200,2058),-- Life Stone: level 61
(21060,8738,1,1,200,411),-- Mid-Grade Life Stone: level 61
(21060,8748,1,1,200,411),-- High-Grade Life Stone: level 61
(21061,1895,1,7,-1,790309),-- Metallic Fiber
(21061,5478,1,1,-1,69780),-- Sealed Dark Crystal Leather Armor Pattern
(21061,57,4020,8163,0,700000),-- Adena
(21061,5287,1,1,1,18),-- Sealed Dark Crystal breastplate
(21061,5288,1,1,1,29),-- Sealed Dark Crystal Gaiters
(21061,5293,1,1,1,13),-- Sealed Tallum Plate Armor
(21061,5520,1,1,1,2635),-- Sealed Dark Crystal Breastplate Pattern
(21061,5521,1,1,1,1860),-- Sealed Tallum Plate Armor Pattern
(21061,5524,1,1,1,4028),-- Sealed Dark Crystal Gaiters Pattern
(21061,730,1,1,2,796),-- Scroll: Enchant Armor (Grade A)
(21061,1868,1,3,2,250000),-- Thread
(21061,1873,1,1,2,100000),-- Silver Nugget
(21061,1876,1,1,2,47610),-- Mithril Ore
(21061,1895,1,1,2,71429),-- Metallic Fiber
(21061,4042,1,1,2,4902),-- Enria
(21061,5000,1,1,2,184),-- Recipe: Sword of Damascus (60%)
(21061,5422,1,1,2,61),-- Recipe: Sealed Majestic Plate Armor(60%)
(21061,8730,1,1,200,2755),-- Life Stone: level 67
(21061,8740,1,1,200,367),-- Mid-Grade Life Stone: level 67
(21063,1807,1,1,-1,23854),-- Recipe: Soulshot: A-Grade
(21063,5220,1,2,-1,715633),-- Metal Hardener
(21063,5478,1,1,-1,150444),-- Sealed Dark Crystal Leather Armor Pattern
(21063,57,8664,17620,0,700000),-- Adena
(21063,5287,1,1,1,39),-- Sealed Dark Crystal breastplate
(21063,5288,1,1,1,61),-- Sealed Dark Crystal Gaiters
(21063,5293,1,1,1,27),-- Sealed Tallum Plate Armor
(21063,5520,1,1,1,5681),-- Sealed Dark Crystal Breastplate Pattern
(21063,5521,1,1,1,4009),-- Sealed Tallum Plate Armor Pattern
(21063,5524,1,1,1,8683),-- Sealed Dark Crystal Gaiters Pattern
(21063,729,1,1,2,85),-- Scroll: Enchant Weapon (Grade A)
(21063,1864,4,12,2,125000),-- Stem
(21063,1865,2,6,2,111111),-- Varnish
(21063,1866,3,9,2,62500),-- Suede
(21063,1868,10,30,2,47619),-- Thread
(21063,1873,4,12,2,25000),-- Silver Nugget
(21063,8314,1,1,2,514),-- Recipe: Spiritual Eye (60%)
(21063,5281,1,1,2,252),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21063,8730,1,1,200,7042),-- Life Stone: level 67
(21063,8740,1,1,200,1511),-- Mid-Grade Life Stone: level 67
(21063,8760,1,1,200,252),-- Top-Grade Life Stone: level 67
(21064,4119,1,1,-1,40287),-- Demon Dagger Edge
(21064,5230,1,1,-1,12272),-- Recipe: Mithril Arrow (100%)
(21064,5479,1,1,-1,54461),-- Sealed Tallum Leather Armor Pattern
(21064,57,4450,9065,0,700000),-- Adena
(21064,5287,1,1,1,20),-- Sealed Dark Crystal breastplate
(21064,5288,1,1,1,32),-- Sealed Dark Crystal Gaiters
(21064,5293,1,1,1,14),-- Sealed Tallum Plate Armor
(21064,5520,1,1,1,2923),-- Sealed Dark Crystal Breastplate Pattern
(21064,5521,1,1,1,2063),-- Sealed Tallum Plate Armor Pattern
(21064,5524,1,1,1,4467),-- Sealed Dark Crystal Gaiters Pattern
(21064,1344,40,120,2,66667),-- Mithril Arrow
(21064,1867,1,3,2,125000),-- Animal Skin
(21064,1872,1,3,2,166667),-- Animal Bone
(21064,1875,1,1,2,16949),-- Stone of Purity
(21064,1881,1,1,2,35714),-- Coarse Bone Powder
(21064,4041,1,1,2,1570),-- Mold Hardener
(21064,4130,1,1,2,5882),-- Recipe: Ring of Black Ore
(21064,4403,1,1,2,392),-- Red Dimensional Stone
(21064,8730,1,1,200,1961),-- Life Stone: level 67
(21064,8740,1,1,200,784),-- Mid-Grade Life Stone: level 67
(21064,8750,1,1,200,392),-- High-Grade Life Stone: level 67
(21065,1807,1,1,-1,13828),-- Recipe: Soulshot: A-Grade
(21065,4120,1,1,-1,45319),-- Bellion Cestus Edge
(21065,57,5007,10220,0,700000),-- Adena
(21065,5290,1,1,1,67),-- Sealed Dark Crystal Gloves
(21065,5291,1,1,1,67),-- Sealed Dark Crystal Boots
(21065,5295,1,1,1,67),-- Sealed Tallum Gloves
(21065,5296,1,1,1,67),-- Sealed Tallum Boots
(21065,5496,1,1,1,8979),-- Sealed Dark Crystal Boots Lining
(21065,5497,1,1,1,8979),-- Sealed Tallum Boots Lining
(21065,5508,1,1,1,8979),-- Sealed Dark Crystal Gloves Design
(21065,5509,1,1,1,8979),-- Sealed Tallum Gloves Design
(21065,1867,1,3,2,142857),-- Animal Skin
(21065,1872,1,3,2,200000),-- Animal Bone
(21065,1875,1,1,2,20833),-- Stone of Purity
(21065,1881,1,1,2,40000),-- Coarse Bone Powder
(21065,4041,1,1,2,4255),-- Mold Hardener
(21065,4164,1,1,2,3257),-- Recipe: Stockings of Doom
(21065,4401,1,1,2,1253),-- Green Dimensional Stone
(21065,8730,1,1,200,4016),-- Life Stone: level 67
(21065,8740,1,1,200,451),-- Mid-Grade Life Stone: level 67
(21066,4081,1,1,-1,102859),-- Doom Plate Armor Temper
(21066,5230,1,1,-1,13915),-- Recipe: Mithril Arrow (100%)
(21066,5482,1,1,-1,137743),-- Sealed Dark Crystalline Leggings Design
(21066,57,5047,10270,0,700000),-- Adena
(21066,5289,1,1,1,60),-- Sealed Dark Crystal Helmet
(21066,5292,1,1,1,85),-- Sealed Dark Crystal Shield
(21066,5294,1,1,1,60),-- Sealed Tallum Helmet
(21066,5494,1,1,1,10790),-- Sealed Dark Crystal Shield Fragment
(21066,5525,1,1,1,7942),-- Sealed Dark Crystal Helmet Design
(21066,5526,1,1,1,7942),-- Sealed Tallum Helm Design
(21066,729,1,1,2,43),-- Scroll: Enchant Weapon (Grade A)
(21066,1868,3,9,2,100000),-- Thread
(21066,1873,1,1,2,125000),-- Silver Nugget
(21066,1889,1,1,2,21739),-- Compound Braid
(21066,4044,1,1,2,11494),-- Thons
(21066,4613,1,1,2,532),-- Greater Dye of Strength <Str+4 Con-4>
(21066,5549,1,1,2,34483),-- Metallic Thread
(21066,6671,1,1,2,31250),-- Deluxe Chest Key - Grade 7
(21066,8731,1,1,200,3460),-- Life Stone: level 70
(21066,8741,1,1,200,1065),-- Mid-Grade Life Stone: level 70
(21066,8751,1,1,200,266),-- High-Grade Life Stone: level 70
(21068,3035,1,1,-1,32712),-- Recipe: Spiritshot A
(21068,5479,1,1,-1,145173),-- Sealed Tallum Leather Armor Pattern
(21068,5485,1,1,-1,206309),-- Sealed Tallum Tunic Texture
(21068,57,11870,24160,0,700000),-- Adena
(21068,5287,1,1,1,53),-- Sealed Dark Crystal breastplate
(21068,5288,1,1,1,84),-- Sealed Dark Crystal Gaiters
(21068,5293,1,1,1,36),-- Sealed Tallum Plate Armor
(21068,5520,1,1,1,7790),-- Sealed Dark Crystal Breastplate Pattern
(21068,5521,1,1,1,5497),-- Sealed Tallum Plate Armor Pattern
(21068,5524,1,1,1,11907),-- Sealed Dark Crystal Gaiters Pattern
(21068,729,1,1,2,115),-- Scroll: Enchant Weapon (Grade A)
(21068,1867,12,36,2,37037),-- Animal Skin
(21068,1869,18,54,2,18182),-- Iron Ore
(21068,1870,2,6,2,142857),-- Coal
(21068,1871,3,9,2,166667),-- Charcoal
(21068,1872,7,21,2,66667),-- Animal Bone
(21068,4402,1,1,2,1003),-- Blue Dimensional Stone
(21068,8731,1,1,200,5780),-- Life Stone: level 70
(21068,8741,1,1,200,752),-- Mid-Grade Life Stone: level 70
(21068,8751,1,1,200,251),-- High-Grade Life Stone: level 70
(21068,8761,1,1,200,251),-- Top-Grade Life Stone: level 70
(21069,3035,1,1,-1,17599),-- Recipe: Spiritshot A
(21069,4041,1,1,-1,114774),-- Mold Hardener
(21069,4080,1,1,-1,271501),-- Blue Wolf Gaiters Material
(21069,57,6379,13000,0,700000),-- Adena
(21069,5290,1,1,1,85),-- Sealed Dark Crystal Gloves
(21069,5291,1,1,1,85),-- Sealed Dark Crystal Boots
(21069,5295,1,1,1,85),-- Sealed Tallum Gloves
(21069,5296,1,1,1,85),-- Sealed Tallum Boots
(21069,5496,1,1,1,11427),-- Sealed Dark Crystal Boots Lining
(21069,5497,1,1,1,11427),-- Sealed Tallum Boots Lining
(21069,5508,1,1,1,11427),-- Sealed Dark Crystal Gloves Design
(21069,5509,1,1,1,11427),-- Sealed Tallum Gloves Design
(21069,729,1,1,2,103),-- Scroll: Enchant Weapon (Grade A)
(21069,1866,1,1,2,250000),-- Suede
(21069,1871,1,3,2,200000),-- Charcoal
(21069,1877,1,1,2,16949),-- Adamantite Nugget
(21069,4040,1,1,2,10417),-- Mold Lubricant
(21069,4041,1,1,2,3279),-- Mold Hardener
(21069,5422,1,1,2,308),-- Recipe: Sealed Majestic Plate Armor(60%)
(21069,4619,1,1,2,410),-- Greater Dye of Intelligence <Int+4 Men-4>
(21069,8731,1,1,200,3484),-- Life Stone: level 70
(21069,8741,1,1,200,1026),-- Mid-Grade Life Stone: level 70
(21071,5156,1,1,-1,34020),-- Recipe: Soulshot (A) Compressed Package (100%)
(21071,5486,1,1,-1,150978),-- Sealed Dark Crystal Robe Fabric
(21071,5489,1,1,-1,336768),-- Sealed Tallum Stockings Fabric
(21071,57,12340,25140,0,700000),-- Adena
(21071,5297,1,1,1,73),-- Sealed Dark Crystal Leather Armor
(21071,5298,1,1,1,116),-- Sealed Dark Crystalline Leggings
(21071,5301,1,1,1,50),-- Sealed Tallum Leather Armor
(21071,5478,1,1,1,10562),-- Sealed Dark Crystal Leather Armor Pattern
(21071,5479,1,1,1,7432),-- Sealed Tallum Leather Armor Pattern
(21071,5482,1,1,1,16578),-- Sealed Dark Crystalline Leggings Design
(21071,729,1,1,2,229),-- Scroll: Enchant Weapon (Grade A)
(21071,1894,1,1,2,37037),-- Crafted Leather
(21071,4039,1,1,2,31250),-- Mold Glue
(21071,4041,1,1,2,6211),-- Mold Hardener
(21071,4044,1,1,2,27778),-- Thons
(21071,4403,1,1,2,1377),-- Red Dimensional Stone
(21071,8731,1,1,200,9901),-- Life Stone: level 70
(21071,8741,1,1,200,2294),-- Mid-Grade Life Stone: level 70
(21071,8761,1,1,200,102),-- Top-Grade Life Stone: level 70
(21072,4083,1,1,-1,122362),-- Leather Armor of Doom Design
(21072,5161,1,1,-1,14439),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21072,5230,1,1,-1,14439),-- Recipe: Mithril Arrow (100%)
(21072,57,5240,10670,0,700000),-- Adena
(21072,5289,1,1,1,62),-- Sealed Dark Crystal Helmet
(21072,5292,1,1,1,88),-- Sealed Dark Crystal Shield
(21072,5294,1,1,1,62),-- Sealed Tallum Helmet
(21072,5494,1,1,1,11196),-- Sealed Dark Crystal Shield Fragment
(21072,5525,1,1,1,8241),-- Sealed Dark Crystal Helmet Design
(21072,5526,1,1,1,8241),-- Sealed Tallum Helm Design
(21072,1868,3,9,2,125000),-- Thread
(21072,1873,1,1,2,142857),-- Silver Nugget
(21072,1889,1,1,2,25000),-- Compound Braid
(21072,4044,1,1,2,10526),-- Thons
(21072,4403,1,1,2,352),-- Red Dimensional Stone
(21072,4984,1,1,2,1056),-- Recipe: Blue Wolf Leather Armor (60%)
(21072,5549,1,1,2,31250),-- Metallic Thread
(21072,8731,1,1,200,2817),-- Life Stone: level 70
(21072,8741,1,1,200,1408),-- Mid-Grade Life Stone: level 70
(21073,4052,1,1,-1,167558),-- Earring of Black Ore Piece
(21073,5230,1,1,-1,6310),-- Recipe: Mithril Arrow (100%)
(21073,57,2319,4638,0,700000),-- Adena
(21073,287,1,1,1,11),-- Bow of Peril
(21073,4121,1,1,1,3045),-- Bow of Peril Shaft
(21073,1830,1,1,2,38462),-- Scroll of Escape: Castle
(21073,1864,1,3,2,111111),-- Stem
(21073,1869,1,1,2,200000),-- Iron Ore
(21073,1880,1,1,2,21739),-- Steel
(21073,4043,1,1,2,7299),-- Asofe
(21073,5276,1,1,2,267),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21073,5550,1,1,2,2663),-- Durable Metal Plate
(21074,1874,1,1,-1,411554),-- Oriharukon Ore
(21074,5478,1,1,-1,51911),-- Sealed Dark Crystal Leather Armor Pattern
(21074,5482,1,1,-1,81480),-- Sealed Dark Crystalline Leggings Design
(21074,57,2985,6083,0,700000),-- Adena
(21074,5290,1,1,1,40),-- Sealed Dark Crystal Gloves
(21074,5291,1,1,1,40),-- Sealed Dark Crystal Boots
(21074,5295,1,1,1,40),-- Sealed Tallum Gloves
(21074,5296,1,1,1,40),-- Sealed Tallum Boots
(21074,5496,1,1,1,5345),-- Sealed Dark Crystal Boots Lining
(21074,5497,1,1,1,5345),-- Sealed Tallum Boots Lining
(21074,5508,1,1,1,5345),-- Sealed Dark Crystal Gloves Design
(21074,5509,1,1,1,5345),-- Sealed Tallum Gloves Design
(21074,730,1,1,2,218),-- Scroll: Enchant Armor (Grade A)
(21074,1865,1,1,2,200000),-- Varnish
(21074,1870,1,1,2,200000),-- Coal
(21074,1874,1,1,2,14286),-- Oriharukon Ore
(21074,4039,1,1,2,7874),-- Mold Glue
(21074,4043,1,1,2,3953),-- Asofe
(21074,5166,1,1,2,348),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21074,8730,1,1,200,2370),-- Life Stone: level 67
(21075,4041,1,1,-1,38815),-- Mold Hardener
(21075,4052,1,1,-1,158037),-- Earring of Black Ore Piece
(21075,4616,1,1,-1,6157),-- Greater Dye of Constitution <Con+4 Dex-4>
(21075,57,2191,4376,0,700000),-- Adena
(21075,601,1,1,1,41),-- Boots of Doom
(21075,2439,1,1,1,41),-- Blue Wolf Boots
(21075,2475,1,1,1,41),-- Doom Gloves
(21075,2487,1,1,1,41),-- Blue Wolf Gloves
(21075,4090,1,1,1,7398),-- Blue Wolf Boots Design
(21075,4091,1,1,1,7398),-- Doom Gloves Part
(21075,4096,1,1,1,7398),-- Blue Wolf Gloves Fabric
(21075,4099,1,1,1,7398),-- Doom Boots Part
(21075,1866,1,1,2,100000),-- Suede
(21075,1871,1,1,2,142857),-- Charcoal
(21075,1877,1,1,2,1658),-- Adamantite Nugget
(21075,4040,1,1,2,1658),-- Mold Lubricant
(21075,4041,1,1,2,1658),-- Mold Hardener
(21075,5005,1,1,2,144),-- Recipe: Demon Dagger (60%)
(21075,5418,1,1,2,492),-- Recipe: Sealed Tallum Plate Armor(60%)
(21075,5276,1,1,2,252),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21075,8729,1,1,200,4975),-- Life Stone: level 64
(21076,4052,1,1,-1,155045),-- Earring of Black Ore Piece
(21076,4053,1,1,-1,187709),-- Ring of Black Ore Gemstone
(21076,5230,1,1,-1,5839),-- Recipe: Mithril Arrow (100%)
(21076,57,2130,4285,0,700000),-- Adena
(21076,2398,1,1,1,14),-- Blue Wolf Tunic
(21076,2399,1,1,1,14),-- Tunic of Doom
(21076,2403,1,1,1,22),-- Blue Wolf Stockings
(21076,2404,1,1,1,22),-- Stockings of Doom
(21076,4084,1,1,1,2809),-- Blue Wolf Tunic Fabric
(21076,4085,1,1,1,2809),-- Tunic of Doom Pattern
(21076,4086,1,1,1,3793),-- Blue Wolf Stockings Pattern
(21076,4087,1,1,1,3793),-- Stockings of Doom Pattern
(21076,1830,1,1,2,52632),-- Scroll of Escape: Castle
(21076,1866,1,1,2,125000),-- Suede
(21076,1871,1,1,2,142857),-- Charcoal
(21076,1877,1,1,2,9524),-- Adamantite Nugget
(21076,4040,1,1,2,3185),-- Mold Lubricant
(21076,4041,1,1,2,1875),-- Mold Hardener
(21076,8729,1,1,200,1203),-- Life Stone: level 64
(21076,8739,1,1,200,1590),-- Mid-Grade Life Stone: level 64
(21077,4041,1,1,-1,36223),-- Mold Hardener
(21077,4054,1,1,-1,82050),-- Necklace of Black Ore Beads
(21077,5230,1,1,-1,5554),-- Recipe: Mithril Arrow (100%)
(21077,57,2023,4088,0,700000),-- Adena
(21077,864,1,1,1,45),-- Earring of Black Ore
(21077,895,1,1,1,67),-- Ring of Black Ore
(21077,926,1,1,1,34),-- Necklace of Black Ore
(21077,4052,1,1,1,7260),-- Earring of Black Ore Piece
(21077,4053,1,1,1,8790),-- Ring of Black Ore Gemstone
(21077,4054,1,1,1,4039),-- Necklace of Black Ore Beads
(21077,1343,20,60,2,83333),-- Silver Arrow
(21077,1867,1,1,2,166667),-- Animal Skin
(21077,1872,1,1,2,166667),-- Animal Bone
(21077,1875,1,1,2,12821),-- Stone of Purity
(21077,1881,1,1,2,20833),-- Coarse Bone Powder
(21077,4041,1,1,2,3205),-- Mold Hardener
(21077,5230,1,1,2,1603),-- Recipe: Mithril Arrow (100%)
(21077,5276,1,1,2,1603),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21077,8739,1,1,200,1603),-- Mid-Grade Life Stone: level 64
(21078,3956,1,1,-1,7360),-- Recipe: Blessed Spiritshot A
(21078,4040,1,1,-1,110398),-- Mold Lubricant
(21078,5485,1,1,-1,46417),-- Sealed Tallum Tunic Texture
(21078,57,2685,5428,0,700000),-- Adena
(21078,5289,1,1,1,32),-- Sealed Dark Crystal Helmet
(21078,5292,1,1,1,45),-- Sealed Dark Crystal Shield
(21078,5294,1,1,1,32),-- Sealed Tallum Helmet
(21078,5494,1,1,1,5707),-- Sealed Dark Crystal Shield Fragment
(21078,5525,1,1,1,4201),-- Sealed Dark Crystal Helmet Design
(21078,5526,1,1,1,4201),-- Sealed Tallum Helm Design
(21078,1868,1,3,2,200000),-- Thread
(21078,1873,1,1,2,71429),-- Silver Nugget
(21078,1876,1,1,2,40000),-- Mithril Ore
(21078,1895,1,1,2,62500),-- Metallic Fiber
(21078,4042,1,1,2,3145),-- Enria
(21078,4175,1,1,2,7874),-- Recipe: Avadon Boots
(21078,8730,1,1,200,1572),-- Life Stone: level 67
(21078,8740,1,1,200,895),-- Mid-Grade Life Stone: level 67
(21079,1895,1,3,-1,750339),-- Metallic Fiber
(21079,4042,1,1,-1,87540),-- Enria
(21079,5478,1,1,-1,44167),-- Sealed Dark Crystal Leather Armor Pattern
(21079,57,2550,5165,0,700000),-- Adena
(21079,5304,1,1,1,15),-- Sealed Tallum Tunic
(21079,5305,1,1,1,24),-- Sealed Tallum Stockings
(21079,5308,1,1,1,11),-- Sealed Dark Crystal Robe
(21079,5485,1,1,1,2175),-- Sealed Tallum Tunic Texture
(21079,5486,1,1,1,1530),-- Sealed Dark Crystal Robe Fabric
(21079,5489,1,1,1,3413),-- Sealed Tallum Stockings Fabric
(21079,730,1,1,2,1639),-- Scroll: Enchant Armor (Grade A)
(21079,1868,1,3,2,142857),-- Thread
(21079,1873,1,1,2,71429),-- Silver Nugget
(21079,1876,1,1,2,27778),-- Mithril Ore
(21079,1895,1,1,2,40000),-- Metallic Fiber
(21079,4042,1,1,2,4926),-- Enria
(21079,5161,1,1,2,296),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21079,8730,1,1,200,3279),-- Life Stone: level 67
(21079,8740,1,1,200,1639),-- Mid-Grade Life Stone: level 67
(21080,1807,1,1,-1,6473),-- Recipe: Soulshot: A-Grade
(21080,3035,1,1,-1,6473),-- Recipe: Spiritshot A
(21080,5479,1,1,-1,28725),-- Sealed Tallum Leather Armor Pattern
(21080,57,2353,4764,0,700000),-- Adena
(21080,6323,1,1,1,28),-- Sealed Phoenix Necklace
(21080,6324,1,1,1,37),-- Sealed Phoenix Earring
(21080,6325,1,1,1,55),-- Sealed Phoenix Ring
(21080,6341,1,1,1,6611),-- Sealed Phoenix Earring Gemstone
(21080,6343,1,1,1,5110),-- Sealed Phoenix Necklace Beads
(21080,6345,1,1,1,9143),-- Sealed Phoenix Ring Gemstone
(21080,1864,1,3,2,142857),-- Stem
(21080,1869,1,1,2,142857),-- Iron Ore
(21080,1878,1,1,2,90909),-- Braided Hemp
(21080,1884,1,1,2,90909),-- Cord
(21080,4039,1,1,2,8065),-- Mold Glue
(21080,5166,1,1,2,274),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21080,6036,1,1,2,6452),-- Greater Magic Haste Potion
(21080,8730,1,1,200,921),-- Life Stone: level 67
(21081,4119,1,1,-1,26946),-- Demon Dagger Edge
(21081,5230,1,1,-1,8208),-- Recipe: Mithril Arrow (100%)
(21081,5478,1,1,-1,51767),-- Sealed Dark Crystal Leather Armor Pattern
(21081,57,2994,6056,0,700000),-- Adena
(21081,5297,1,1,1,18),-- Sealed Dark Crystal Leather Armor
(21081,5298,1,1,1,28),-- Sealed Dark Crystalline Leggings
(21081,5301,1,1,1,12),-- Sealed Tallum Leather Armor
(21081,5478,1,1,1,2549),-- Sealed Dark Crystal Leather Armor Pattern
(21081,5479,1,1,1,1794),-- Sealed Tallum Leather Armor Pattern
(21081,5482,1,1,1,4000),-- Sealed Dark Crystalline Leggings Design
(21081,1867,1,1,2,333333),-- Animal Skin
(21081,1872,1,3,2,166667),-- Animal Bone
(21081,1875,1,1,2,25641),-- Stone of Purity
(21081,1881,1,1,2,47619),-- Coarse Bone Powder
(21081,4041,1,1,2,2636),-- Mold Hardener
(21081,4985,1,1,2,859),-- Recipe: Leather Armor of Doom (60%)
(21081,8731,1,1,200,3663),-- Life Stone: level 70
(21082,5156,1,1,-1,7871),-- Recipe: Soulshot (A) Compressed Package (100%)
(21082,5482,1,1,-1,77916),-- Sealed Dark Crystalline Leggings Design
(21082,5486,1,1,-1,34931),-- Sealed Dark Crystal Robe Fabric
(21082,57,2884,5776,0,700000),-- Adena
(21082,5304,1,1,1,17),-- Sealed Tallum Tunic
(21082,5305,1,1,1,27),-- Sealed Tallum Stockings
(21082,5308,1,1,1,12),-- Sealed Dark Crystal Robe
(21082,5485,1,1,1,2444),-- Sealed Tallum Tunic Texture
(21082,5486,1,1,1,1720),-- Sealed Dark Crystal Robe Fabric
(21082,5489,1,1,1,3836),-- Sealed Tallum Stockings Fabric
(21082,1865,1,1,2,200000),-- Varnish
(21082,1870,1,1,2,250000),-- Coal
(21082,1875,1,1,2,17544),-- Stone of Purity
(21082,1879,1,1,2,35714),-- Cokes
(21082,4040,1,1,2,5813),-- Mold Lubricant
(21082,5281,1,1,2,388),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21083,5156,1,1,-1,8223),-- Recipe: Soulshot (A) Compressed Package (100%)
(21083,5485,1,1,-1,51860),-- Sealed Tallum Tunic Texture
(21083,5486,1,1,-1,36492),-- Sealed Dark Crystal Robe Fabric
(21083,57,2987,6015,0,700000),-- Adena
(21083,6323,1,1,1,35),-- Sealed Phoenix Necklace
(21083,6324,1,1,1,47),-- Sealed Phoenix Earring
(21083,6325,1,1,1,70),-- Sealed Phoenix Ring
(21083,6341,1,1,1,8398),-- Sealed Phoenix Earring Gemstone
(21083,6343,1,1,1,6492),-- Sealed Phoenix Necklace Beads
(21083,6345,1,1,1,11615),-- Sealed Phoenix Ring Gemstone
(21083,730,1,1,2,217),-- Scroll: Enchant Armor (Grade A)
(21083,1807,1,1,2,347),-- Recipe: Soulshot: A-Grade
(21083,1866,1,1,2,166667),-- Suede
(21083,1871,1,1,2,250000),-- Charcoal
(21083,1885,1,1,2,21739),-- High Grade Suede
(21083,4041,1,1,2,2264),-- Mold Hardener
(21083,5220,1,1,2,7194),-- Metal Hardener
(21083,8731,1,1,200,7194),-- Life Stone: level 70
(21084,4056,1,1,-1,74312),-- Zubei\'s Breastplate Part
(21084,4129,1,2,-1,44434),-- Recipe: Earrings of Black Ore
(21084,57,1989,4041,0,700000),-- Adena
(21084,78,1,1,1,5),-- Great Sword
(21084,142,1,1,1,5),-- Keshanberk
(21084,148,1,1,1,5),-- Sword of Valhalla
(21084,4104,1,1,1,1157),-- Great Sword Blade
(21084,4107,1,1,1,1139),-- Keshanberk Blade
(21084,4108,1,1,1,1139),-- Sword of Valhalla Blade
(21084,1832,1,1,2,125000),-- Greater Antidote
(21084,1868,1,1,2,200000),-- Thread
(21084,1873,1,1,2,35714),-- Silver Nugget
(21084,1889,1,1,2,6579),-- Compound Braid
(21084,4044,1,1,2,3690),-- Thons
(21084,4196,1,1,2,137),-- Recipe: Staff of Evil Spirits
(21084,4402,1,1,2,270),-- Blue Dimensional Stone
(21084,5002,1,1,2,103),-- Recipe: Deadman\'s Glory (60%)
(21084,5549,1,1,2,8197),-- Metallic Thread
(21084,8728,1,1,200,2188),-- Life Stone: level 61
(21084,8738,1,1,200,274),-- Mid-Grade Life Stone: level 61
(21085,1889,1,1,-1,636970),-- Compound Braid
(21085,5479,1,1,-1,56536),-- Sealed Tallum Leather Armor Pattern
(21085,5489,1,1,-1,126108),-- Sealed Tallum Stockings Fabric
(21085,57,4618,9398,0,700000),-- Adena
(21085,5297,1,1,1,28),-- Sealed Dark Crystal Leather Armor
(21085,5298,1,1,1,44),-- Sealed Dark Crystalline Leggings
(21085,5301,1,1,1,19),-- Sealed Tallum Leather Armor
(21085,5478,1,1,1,3955),-- Sealed Dark Crystal Leather Armor Pattern
(21085,5479,1,1,1,2783),-- Sealed Tallum Leather Armor Pattern
(21085,5482,1,1,1,6208),-- Sealed Dark Crystalline Leggings Design
(21085,1868,3,9,2,111111),-- Thread
(21085,1873,1,1,2,125000),-- Silver Nugget
(21085,1889,1,1,2,16949),-- Compound Braid
(21085,4044,1,1,2,9434),-- Thons
(21085,4131,1,1,2,2985),-- Recipe: Necklace of Black Ore
(21085,5549,1,1,2,37037),-- Metallic Thread
(21085,6036,1,1,2,5988),-- Greater Magic Haste Potion
(21085,8730,1,1,200,6452),-- Life Stone: level 67
(21085,8740,1,1,200,1493),-- Mid-Grade Life Stone: level 67
(21085,8750,1,1,200,1493),-- High-Grade Life Stone: level 67
(21086,1807,1,1,-1,12750),-- Recipe: Soulshot: A-Grade
(21086,1895,1,5,-1,910686),-- Metallic Fiber
(21086,5482,1,1,-1,126209),-- Sealed Dark Crystalline Leggings Design
(21086,57,4625,9425,0,700000),-- Adena
(21086,5287,1,1,1,21),-- Sealed Dark Crystal breastplate
(21086,5288,1,1,1,33),-- Sealed Dark Crystal Gaiters
(21086,5293,1,1,1,14),-- Sealed Tallum Plate Armor
(21086,5520,1,1,1,3037),-- Sealed Dark Crystal Breastplate Pattern
(21086,5521,1,1,1,2143),-- Sealed Tallum Plate Armor Pattern
(21086,5524,1,1,1,4641),-- Sealed Dark Crystal Gaiters Pattern
(21086,1895,1,1,2,90909),-- Metallic Fiber
(21086,4040,1,1,2,3268),-- Mold Lubricant
(21086,4042,1,1,2,8197),-- Enria
(21086,4043,1,1,2,8621),-- Asofe
(21086,4166,1,1,2,5714),-- Recipe: Doom Helmet
(21086,4401,1,1,2,942),-- Green Dimensional Stone
(21086,8730,1,1,200,3676),-- Life Stone: level 67
(21086,8740,1,1,200,362),-- Mid-Grade Life Stone: level 67
(21087,1894,1,1,-1,465485),-- Crafted Leather
(21087,4082,1,1,-1,149902),-- Blue Wolf Leather Armor Texture
(21087,5494,1,1,-1,278616),-- Sealed Dark Crystal Shield Fragment
(21087,57,6417,13060,0,700000),-- Adena
(21087,5287,1,1,1,29),-- Sealed Dark Crystal breastplate
(21087,5288,1,1,1,46),-- Sealed Dark Crystal Gaiters
(21087,5293,1,1,1,20),-- Sealed Tallum Plate Armor
(21087,5520,1,1,1,4213),-- Sealed Dark Crystal Breastplate Pattern
(21087,5521,1,1,1,2973),-- Sealed Tallum Plate Armor Pattern
(21087,5524,1,1,1,6438),-- Sealed Dark Crystal Gaiters Pattern
(21087,729,1,1,2,49),-- Scroll: Enchant Weapon (Grade A)
(21087,1864,5,15,2,76923),-- Stem
(21087,1869,2,6,2,90909),-- Iron Ore
(21087,1880,1,1,2,38462),-- Steel
(21087,4043,1,1,2,10870),-- Asofe
(21087,4614,1,1,2,105),-- Greater Dye of Strength <Str+4 Dex-4>
(21087,5550,1,1,2,5236),-- Durable Metal Plate
(21087,6037,9,27,2,41667),-- Waking Scroll
(21087,6671,1,1,2,41667),-- Deluxe Chest Key - Grade 7
(21087,8731,1,1,200,3040),-- Life Stone: level 70
(21087,8741,1,1,200,734),-- Mid-Grade Life Stone: level 70
(21087,8751,1,1,200,314),-- Mid-Grade Life Stone: level 70
(21088,4042,1,1,-1,169225),-- Enria
(21088,5230,1,1,-1,13538),-- Recipe: Mithril Arrow (100%)
(21088,5485,1,1,-1,85381),-- Sealed Tallum Tunic Texture
(21088,57,4909,10000,0,700000),-- Adena
(21088,5287,1,1,1,22),-- Sealed Dark Crystal breastplate
(21088,5288,1,1,1,35),-- Sealed Dark Crystal Gaiters
(21088,5293,1,1,1,15),-- Sealed Tallum Plate Armor
(21088,5485,1,1,1,4887),-- Sealed Tallum Tunic Texture
(21088,5486,1,1,1,3439),-- Sealed Dark Crystal Robe Fabric
(21088,5489,1,1,1,7670),-- Sealed Tallum Stockings Fabric
(21088,1868,3,9,2,125000),-- Thread
(21088,1873,1,1,2,142857),-- Silver Nugget
(21088,1876,1,1,2,62500),-- Mithril Ore
(21088,1895,1,1,2,111111),-- Metallic Fiber
(21088,4042,1,1,2,5495),-- Enria
(21088,4986,1,1,2,1321),-- Recipe: Blue Wolf Tunic (60%)
(21088,4403,1,1,2,857),-- Red Dimensional Stone
(21088,4622,1,1,2,592),-- Greater Dye of Mental Strength <Men+4 Wit-4>
(21088,10306,1,1,2,302),-- Transform Sealbook - Doll Blader
(21088,8731,1,1,200,3745),-- Life Stone: level 70
(21088,8741,1,1,200,440),-- Mid-Grade Life Stone: level 70
(21088,8751,1,1,200,220),-- High-Grade Life Stone: level 70
(21089,3035,1,1,-1,15741),-- Recipe: Spiritshot A
(21089,4116,1,1,-1,51588),-- Deadman\'s Glory Stone
(21089,5480,1,1,-1,52974),-- Sealed Leather Armor of Nightmare Fabric
(21089,57,5703,11630,0,700000),-- Adena
(21089,5304,1,1,1,34),-- Sealed Tallum Tunic
(21089,5305,1,1,1,54),-- Sealed Tallum Stockings
(21089,5308,1,1,1,23),-- Sealed Dark Crystal Robe
(21089,5485,1,1,1,4887),-- Sealed Tallum Tunic Texture
(21089,5486,1,1,1,3439),-- Sealed Dark Crystal Robe Fabric
(21089,5489,1,1,1,7670),-- Sealed Tallum Stockings Fabric
(21089,1868,3,9,2,142857),-- Thread
(21089,1873,2,6,2,45455),-- Silver Nugget
(21089,1876,1,1,2,83333),-- Mithril Ore
(21089,1895,1,1,2,125000),-- Metallic Fiber
(21089,4042,1,1,2,7353),-- Enria
(21089,4986,1,1,2,781),-- Recipe: Blue Wolf Tunic (60%)
(21089,8731,1,1,200,4425),-- Life Stone: level 70
(21089,8741,1,1,200,694),-- Mid-Grade Life Stone: level 70
(21089,8761,1,1,200,52),-- Top-Grade Life Stone: level 70
(21090,5156,1,1,-1,12101),-- Recipe: Soulshot (A) Compressed Package (100%)
(21090,5230,1,1,-1,12101),-- Recipe: Mithril Arrow (100%)
(21090,5486,1,1,-1,53701),-- Sealed Dark Crystal Robe Fabric
(21090,57,4386,8941,0,700000),-- Adena
(21090,5287,1,1,1,20),-- Sealed Dark Crystal breastplate
(21090,5288,1,1,1,31),-- Sealed Dark Crystal Gaiters
(21090,5293,1,1,1,14),-- Sealed Tallum Plate Armor
(21090,5520,1,1,1,2882),-- Sealed Dark Crystal Breastplate Pattern
(21090,5521,1,1,1,2034),-- Sealed Tallum Plate Armor Pattern
(21090,5524,1,1,1,4405),-- Sealed Dark Crystal Gaiters Pattern
(21090,730,1,1,2,552),-- Scroll: Enchant Armor (Grade A)
(21090,1344,20,60,2,200000),-- Mithril Arrow
(21090,1866,1,1,2,166667),-- Suede
(21090,1871,1,3,2,90909),-- Charcoal
(21090,1877,1,1,2,9804),-- Adamantite Nugget
(21090,4040,1,1,2,3135),-- Mold Lubricant
(21090,4041,1,1,2,1842),-- Mold Hardener
(21090,4401,1,1,2,1105),-- Green Dimensional Stone
(21090,8731,1,1,200,2950),-- Life Stone: level 70
(21090,8741,1,1,200,184),-- Mid-Grade Life Stone: level 70
(21091,3035,1,1,-1,8134),-- Recipe: Spiritshot A
(21091,5479,1,1,-1,36097),-- Sealed Tallum Leather Armor Pattern
(21091,5482,1,1,-1,80517),-- Sealed Dark Crystalline Leggings Design
(21091,57,2948,6005,0,700000),-- Adena
(21091,5297,1,1,1,18),-- Sealed Dark Crystal Leather Armor
(21091,5298,1,1,1,28),-- Sealed Dark Crystalline Leggings
(21091,5301,1,1,1,12),-- Sealed Tallum Leather Armor
(21091,5478,1,1,1,2526),-- Sealed Dark Crystal Leather Armor Pattern
(21091,5479,1,1,1,1777),-- Sealed Tallum Leather Armor Pattern
(21091,5482,1,1,1,3964),-- Sealed Dark Crystalline Leggings Design
(21091,1867,1,3,2,200000),-- Animal Skin
(21091,1872,1,3,2,142857),-- Animal Bone
(21091,1882,1,1,2,62500),-- Leather
(21091,1894,1,1,2,12821),-- Crafted Leather
(21091,3035,1,1,2,547),-- Recipe: Spiritshot A
(21091,4042,1,1,2,5848),-- Enria
(21091,10306,1,1,2,1642),-- Transform Sealbook - Doll Blader
(21092,5161,1,1,-1,7946),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21092,5478,1,1,-1,50115),-- Sealed Dark Crystal Leather Armor Pattern
(21092,5485,1,1,-1,50115),-- Sealed Tallum Tunic Texture
(21092,57,2881,5868,0,700000),-- Adena
(21092,5289,1,1,1,34),-- Sealed Dark Crystal Helmet
(21092,5292,1,1,1,49),-- Sealed Dark Crystal Shield
(21092,5294,1,1,1,34),-- Sealed Tallum Helmet
(21092,5494,1,1,1,6162),-- Sealed Dark Crystal Shield Fragment
(21092,5525,1,1,1,4536),-- Sealed Dark Crystal Helmet Design
(21092,5526,1,1,1,4536),-- Sealed Tallum Helm Design
(21092,1344,30,90,2,111111),-- Mithril Arrow
(21092,1868,4,12,2,58824),-- Thread
(21092,1873,1,1,2,100000),-- Silver Nugget
(21092,1889,1,1,2,18519),-- Compound Braid
(21092,3956,1,1,2,336),-- Recipe: Blessed Spiritshot A
(21092,4044,1,1,2,9615),-- Thons
(21092,5549,1,1,2,33333),-- Metallic Thread
(21092,10306,1,1,2,929),-- Transform Sealbook - Doll Blader
(21092,8731,1,1,200,207),-- Life Stone: level 70
(21092,8741,1,1,200,207),-- Mid-Grade Life Stone: level 70
(21093,1800,1,1,-1,83774),-- Recipe: Piece Bone Breastplate
(21093,1864,1,2,-1,532802),-- Stem
(21093,1897,1,1,-1,63429),-- Necklace of Wisdom Chain
(21093,57,125,213,0,700000),-- Adena
(21093,2,1,1,1,58),-- Long Sword
(21093,120,1,1,1,58),-- Sword of Reflection
(21093,121,1,1,1,58),-- Sword of Watershadow
(21093,2012,1,1,1,820),-- Sword of Reflection Blade
(21093,1801,1,1,2,19822),-- Recipe: Piece Bone Gaiters
(21093,1864,1,1,2,78691),-- Stem
(21093,1869,1,1,2,39346),-- Iron Ore
(21093,1878,1,1,2,15739),-- Braided Hemp
(21094,734,1,1,-1,45036),-- Haste Potion
(21094,1870,1,1,-1,270215),-- Coal
(21094,1924,1,1,-1,87166),-- Leather Helmet Design
(21094,57,125,217,0,700000),-- Adena
(21094,176,1,1,1,118),-- Journeyman\'s Staff
(21094,2501,1,1,1,118),-- Bone Club
(21094,1865,1,1,2,39909),-- Varnish
(21094,1870,1,1,2,39909),-- Coal
(21094,1879,1,1,2,6652),-- Cokes
(21094,2141,1,1,2,7982),-- Recipe: Silver Mold
(21095,1804,1,1,-1,12530),-- Recipe: Soulshot: D-Grade
(21095,1898,1,1,-1,32177),-- Blue Diamond Necklace Gem
(21095,2151,1,1,-1,87255),-- Recipe: Necklace of Devotion
(21095,57,143,255,0,700000),-- Adena
(21095,257,1,1,1,152),-- Viper\'s Fang
(21095,1804,1,1,2,1851),-- Recipe: Soulshot: D-Grade
(21095,1831,1,1,2,123370),-- Antidote
(21095,1865,1,1,2,46264),-- Varnish
(21095,1870,1,1,2,46264),-- Coal
(21096,1866,1,2,-1,205204),-- Suede
(21096,1871,1,3,-1,307807),-- Charcoal
(21096,1926,1,1,-1,31092),-- Piece Bone Breastplate Fragment
(21096,57,140,251,0,700000),-- Adena
(21096,39,1,1,1,986),-- Boots
(21096,51,1,1,1,986),-- Bracer
(21096,1930,1,1,1,15542),-- Boot Lining
(21096,1866,1,1,2,40410),-- Suede
(21096,1871,1,1,2,60615),-- Charcoal
(21096,2179,1,1,2,29003),-- Recipe: Crafted Leather Gloves
(21097,1927,1,1,-1,28206),-- Hard Leather Shirt Pattern
(21097,1928,1,1,-1,51502),-- Piece Bone Gaiters Fragment
(21097,2176,1,1,-1,169170),-- Recipe: Leather Boots
(21097,57,162,286,0,700000),-- Adena
(21097,846,1,1,1,653),-- Coral Earring
(21097,878,1,1,1,977),-- Blue Coral Ring
(21097,909,1,1,1,491),-- Blue Diamond Necklace
(21097,1898,1,1,1,5364),-- Blue Diamond Necklace Gem
(21097,734,1,1,2,6963),-- Haste Potion
(21097,1831,1,1,2,111401),-- Antidote
(21097,1865,1,1,2,41775),-- Varnish
(21097,1870,1,1,2,41775),-- Coal
(21097,2257,1,1,2,1022),-- Recipe: Trident
(21098,1879,1,1,-1,55471),-- Cokes
(21098,1929,1,1,-1,42945),-- Hard Leather Gaiters Material
(21098,2177,1,1,-1,106333),-- Recipe: Bone Helmet
(21098,57,148,273,0,700000),-- Adena
(21098,16,1,1,1,161),-- Long Spear
(21098,1865,1,1,2,49156),-- Varnish
(21098,1870,1,1,2,49156),-- Coal
(21098,1879,1,1,2,8193),-- Cokes
(21098,5158,1,1,2,1967),-- Recipe: Spiritshot (D) Compressed Package (100%)
(21099,1875,1,1,-1,23782),-- Stone of Purity
(21099,1899,1,1,-1,55740),-- Necklace of Devotion Chain
(21099,2152,1,1,-1,62805),-- Recipe: Enchanted Necklace
(21099,57,157,294,0,700000),-- Adena
(21099,45,1,1,1,427),-- Bone Helmet
(21099,626,1,1,1,610),-- Bronze Shield
(21099,1932,1,1,1,10594),-- Bone Helmet Design
(21099,1875,1,1,2,3337),-- Stone of Purity
(21099,1879,1,1,2,8343),-- Cokes
(21099,2152,1,1,2,8812),-- Recipe: Enchanted Necklace
(21099,5549,1,1,2,5006),-- Metallic Thread
(21100,1800,1,1,-1,83774),-- Recipe: Piece Bone Breastplate
(21100,1897,1,1,-1,63429),-- Necklace of Wisdom Chain
(21100,1924,1,1,-1,85936),-- Leather Helmet Design
(21100,57,125,213,0,700000),-- Adena
(21100,845,1,1,1,886),-- Cat\'s Eye Earring
(21100,877,1,1,1,1330),-- Ring of Wisdom
(21100,908,1,1,1,662),-- Necklace of Wisdom
(21100,1897,1,1,1,9369),-- Necklace of Wisdom Chain
(21100,1817,1,1,2,7870),-- Recipe: Cord
(21100,1864,1,1,2,78691),-- Stem
(21100,1869,1,1,2,39346),-- Iron Ore
(21100,1880,1,1,2,3935),-- Steel
(21101,1865,1,1,-1,332824),-- Varnish
(21101,1870,1,1,-1,332824),-- Coal
(21101,1898,1,1,-1,34188),-- Blue Diamond Necklace Gem
(21101,57,148,273,0,700000),-- Adena
(21101,39,1,1,1,1066),-- Boots
(21101,51,1,1,1,1066),-- Bracer
(21101,1930,1,1,1,16806),-- Boot Lining
(21101,1865,1,1,2,39325),-- Varnish
(21101,1870,1,1,2,39325),-- Coal
(21101,5163,1,1,2,1573),-- Recipe: Blessed Spiritshot (D) Compressed Package (100%)
(21101,6035,1,1,2,3277),-- Magic Haste Potion
(21101,6666,1,1,2,39325),-- Deluxe Chest Key - Grade 2
(21102,1900,1,1,-1,49380),-- Enchanted Necklace Chain
(21102,2153,1,1,-1,70191),-- Recipe: Tiger\'s Eye Earring
(21102,2183,1,1,-1,29693),-- Recipe: Mithril Banded Mail
(21102,57,228,457,0,700000),-- Adena
(21102,433,1,1,1,62),-- Elven Tunic
(21102,434,1,1,1,62),-- White Tunic
(21102,435,1,1,1,62),-- Mystic\'s Tunic
(21102,466,1,1,1,99),-- Elven Stockings
(21102,467,1,1,1,99),-- Dark Stockings
(21102,468,1,1,1,99),-- Mystic\'s Stockings
(21102,1933,1,1,1,3628),-- Dark Stocking Fabric
(21102,1936,1,1,1,2523),-- White Tunic Pattern
(21102,956,1,1,2,414),-- Scroll: Enchant Armor (Grade D)
(21102,1874,1,1,2,3268),-- Oriharukon Ore
(21102,1878,1,1,2,19231),-- Braided Hemp
(21102,1884,1,1,2,31250),-- Cord
(21102,1889,1,1,2,4098),-- Compound Braid
(21102,5220,1,1,2,1637),-- Metal Hardener
(21103,955,1,1,-1,1880),-- Scroll: Enchant Weapon (Grade D)
(21103,1879,1,1,-1,78342),-- Cokes
(21103,1933,1,1,-1,55300),-- Dark Stocking Fabric
(21103,57,239,476,0,700000),-- Adena
(21103,348,1,1,1,97),-- Scale Mail
(21103,377,1,1,1,155),-- Scale Gaiters
(21103,1935,1,1,1,3588),-- Scale Mail Fragment
(21103,1937,1,1,1,5517),-- Scale Gaiters Fragment
(21103,1061,1,1,2,28571),-- Healing Potion
(21103,1875,1,1,2,2252),-- Stone of Purity
(21103,1879,1,1,2,7937),-- Cokes
(21103,1885,1,1,2,4184),-- High Grade Suede
(21103,2185,1,1,2,9009),-- Recipe: Iron Boots
(21103,5549,1,1,2,4292),-- Metallic Thread
(21103,5550,1,1,2,858),-- Durable Metal Plate
(21104,1061,1,1,-1,355341),-- Healing Potion
(21104,1876,1,1,-1,117263),-- Mithril Ore
(21104,1935,1,1,-1,32755),-- Scale Mail Fragment
(21104,57,296,595,0,700000),-- Adena
(21104,605,1,1,1,962),-- Leather Gauntlets
(21104,1123,1,1,1,962),-- Blue Buckskin Boots
(21104,1876,1,1,2,10638),-- Mithril Ore
(21104,1877,1,1,2,1938),-- Adamantite Nugget
(21104,1880,1,1,2,4785),-- Steel
(21104,1881,1,1,2,6579),-- Coarse Bone Powder
(21104,1882,1,1,2,12500),-- Leather
(21104,2264,1,1,2,775),-- Recipe: Two-handed Sword
(21104,6667,1,1,2,32258),-- Deluxe Chest Key - Grade 3
(21105,1901,1,1,-1,93083),-- Tiger\'s Eye Earring Stone
(21105,2154,1,1,-1,72155),-- Recipe: Elven Earring
(21105,2155,1,1,-1,108174),-- Recipe: Elven Ring
(21105,57,342,683,0,700000),-- Adena
(21105,393,1,1,1,79),-- Mithril Banded Mail
(21105,394,1,1,1,79),-- Reinforced Leather Shirt
(21105,415,1,1,1,126),-- Mithril Banded Gaiters
(21105,416,1,1,1,126),-- Reinforced Leather Gaiters
(21105,1938,1,1,1,4016),-- Mithril Banded Mail Material
(21105,1939,1,1,1,5246),-- Mithril Banded Gaiters Material
(21105,1874,1,1,2,1805),-- Oriharukon Ore
(21105,1878,1,1,2,25000),-- Braided Hemp
(21105,1884,1,1,2,43478),-- Cord
(21105,1889,1,1,2,5848),-- Compound Braid
(21105,2154,1,1,2,5848),-- Recipe: Elven Earring
(21105,5220,1,1,2,1353),-- Metal Hardener
(21106,1885,1,1,-1,47321),-- High Grade Suede
(21106,1938,1,1,-1,38175),-- Mithril Banded Mail Material
(21106,5550,1,1,-1,7571),-- Durable Metal Plate
(21106,57,288,574,0,700000),-- Adena
(21106,47,1,1,1,397),-- Helmet
(21106,628,1,1,1,567),-- Hoplon
(21106,1341,10,30,2,125000),-- Bone Arrow
(21106,1875,1,1,2,2732),-- Stone of Purity
(21106,1879,1,1,2,5464),-- Cokes
(21106,1885,1,1,2,3559),-- High Grade Suede
(21106,2186,1,1,2,2463),-- Recipe: Brigandine Tunic
(21106,8317,1,1,2,67),-- Recipe: Titan Hammer (100%)
(21106,5549,1,1,2,3831),-- Metallic Thread
(21106,5550,1,1,2,547),-- Durable Metal Plate
(21107,1939,1,2,-1,105226),-- Mithril Banded Gaiters Material
(21107,1940,1,2,-1,178805),-- Iron Boots Design
(21107,2156,1,1,-1,95840),-- Recipe: Elven Necklace
(21107,57,610,1210,0,700000),-- Adena
(21107,260,1,1,1,132),-- Triple-Edged Jamadhr
(21107,1876,1,1,2,26316),-- Mithril Ore
(21107,1877,1,1,2,3731),-- Adamantite Nugget
(21107,1880,1,1,2,12048),-- Steel
(21107,1881,1,1,2,17857),-- Coarse Bone Powder
(21107,1882,1,1,2,24390),-- Leather
(21107,2188,1,1,2,5587),-- Recipe: Manticore Skin Gaiters
(21108,4052,1,2,-1,78329),-- Earring of Black Ore Piece
(21108,5230,1,1,-1,2950),-- Recipe: Mithril Arrow (100%)
(21108,5550,1,1,-1,29499),-- Durable Metal Plate
(21108,57,1068,2182,0,700000),-- Adena
(21108,12204,1,1,1,33),-- Common Item - Blue Wolf Breastplate
(21108,12193,1,1,1,407),-- Common Item - Blue Wolf Gaiters
(21108,12184,1,1,1,62),-- Common Item - Doom Plate Armor
(21108,4078,1,1,1,1591),-- Blue Wolf Breastplate Part
(21108,4080,1,1,1,2241),-- Blue Wolf Gaiters Material
(21108,4081,1,1,1,1074),-- Doom Plate Armor Temper
(21108,1864,1,1,2,142857),-- Stem
(21108,1869,1,1,2,71429),-- Iron Ore
(21108,1880,1,1,2,7576),-- Steel
(21108,4043,1,1,2,4115),-- Asofe
(21108,5230,1,1,2,52),-- Recipe: Mithril Arrow (100%)
(21108,8487,1,1,2,9),-- Recipe: Destroyer Hammer (60%)
(21108,5550,1,1,2,644),-- Durable Metal Plate
(21108,8730,1,1,200,934),-- Life Stone: level 67
(21108,8740,1,1,200,161),-- Mid-Grade Life Stone: level 67
(21108,8750,1,1,200,5),-- High-Grade Life Stone: level 67
(21109,1874,1,1,-1,183146),-- Oriharukon Ore
(21109,4043,1,1,-1,91573),-- Asofe
(21109,5478,1,1,-1,23101),-- Sealed Dark Crystal Leather Armor Pattern
(21109,57,1330,2704,0,700000),-- Adena
(21109,12344,1,1,1,384),-- Common Item - Sealed Phoenix Necklace
(21109,12345,1,1,1,499),-- Common Item - Sealed Phoenix Earring
(21109,12346,1,1,1,974),-- Common Item - Sealed Phoenix Ring
(21109,6341,1,1,1,3741),-- Sealed Phoenix Earring Gemstone
(21109,6343,1,1,1,2892),-- Sealed Phoenix Necklace Beads
(21109,6345,1,1,1,5175),-- Sealed Phoenix Ring Gemstone
(21109,1807,1,1,2,49),-- Recipe: Soulshot: A-Grade
(21109,1865,1,1,2,83333),-- Varnish
(21109,1870,1,1,2,83333),-- Coal
(21109,1874,1,1,2,4975),-- Oriharukon Ore
(21109,4039,1,1,2,2519),-- Mold Glue
(21109,4043,1,1,2,1529),-- Asofe
(21109,6037,1,1,2,166667),-- Waking Scroll
(21109,6670,1,1,2,16949),-- Deluxe Chest Key - Grade 6
(21109,8730,1,1,200,540),-- Life Stone: level 67
(21109,8740,1,1,200,115),-- Mid-Grade Life Stone: level 67
(21109,8750,1,1,200,16),-- High-Grade Life Stone: level 67
(21110,1807,1,1,-1,3130),-- Recipe: Soulshot: A-Grade
(21110,3035,1,1,-1,3130),-- Recipe: Spiritshot A
(21110,5479,1,1,-1,13892),-- Sealed Tallum Leather Armor Pattern
(21110,57,1137,2310,0,700000),-- Adena
(21110,12222,1,1,1,61),-- Common Item - Sealed Dark Crystal Leather Armor
(21110,12228,1,1,1,469),-- Common Item - Sealed Dark Crystalline Leggings
(21110,12230,1,1,1,31),-- Common Item - Sealed Tallum Leather Armor
(21110,5478,1,1,1,972),-- Sealed Dark Crystal Leather Armor Pattern
(21110,5479,1,1,1,684),-- Sealed Tallum Leather Armor Pattern
(21110,5482,1,1,1,1526),-- Sealed Dark Crystalline Leggings Design
(21110,1866,1,1,2,50000),-- Suede
(21110,1871,1,1,2,76923),-- Charcoal
(21110,1877,1,1,2,3745),-- Adamantite Nugget
(21110,3956,1,1,2,57),-- Recipe: Blessed Spiritshot A
(21110,8487,1,1,2,237),-- Recipe: Destroyer Hammer (60%)
(21110,4040,1,1,2,1290),-- Mold Lubricant
(21110,4041,1,1,2,964),-- Mold Hardener
(21110,8730,1,1,200,1133),-- Life Stone: level 67
(21110,8740,1,1,200,161),-- Mid-Grade Life Stone: level 67
(21110,8750,1,1,200,88),-- High-Grade Life Stone: level 67
(21111,4041,1,1,-1,21400),-- Mold Hardener
(21111,5479,1,1,-1,14562),-- Sealed Tallum Leather Armor Pattern
(21111,5482,1,1,-1,32483),-- Sealed Dark Crystalline Leggings Design
(21111,57,1191,2424,0,700000),-- Adena
(21111,12222,1,1,1,170),-- Common Item - Sealed Dark Crystal Leather Armor
(21111,12228,1,1,1,240),-- Common Item - Sealed Dark Crystalline Leggings
(21111,12230,1,1,1,99),-- Common Item - Sealed Tallum Leather Armor
(21111,5478,1,1,1,1019),-- Sealed Dark Crystal Leather Armor Pattern
(21111,5479,1,1,1,717),-- Sealed Tallum Leather Armor Pattern
(21111,5482,1,1,1,1599),-- Sealed Dark Crystalline Leggings Design
(21111,1344,10,30,2,100000),-- Mithril Arrow
(21111,1865,1,1,2,71429),-- Varnish
(21111,1870,1,1,2,76923),-- Coal
(21111,1875,1,1,2,3623),-- Stone of Purity
(21111,1879,1,1,2,11236),-- Cokes
(21111,4040,1,1,2,1160),-- Mold Lubricant
(21111,8487,1,1,2,14),-- Recipe: Destroyer Hammer (60%)
(21111,5166,1,1,2,113),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21111,8730,1,1,200,778),-- Life Stone: level 67
(21111,8740,1,1,200,85),-- Mid-Grade Life Stone: level 67
(21111,8750,1,1,200,12),-- High-Grade Life Stone: level 67
(21112,3956,1,1,-1,3987),-- Recipe: Blessed Spiritshot A
(21112,5485,1,1,-1,25148),-- Sealed Tallum Tunic Texture
(21112,5486,1,1,-1,17696),-- Sealed Dark Crystal Robe Fabric
(21112,57,1445,2465,0,700000),-- Adena
(21112,12221,1,1,1,562),-- Common Item - Sealed Dark Crystal Gloves
(21112,12224,1,1,1,745),-- Common Item - Sealed Dark Crystal Boots
(21112,12229,1,1,1,408),-- Common Item - Sealed Tallum Gloves
(21112,12231,1,1,1,450),-- Common Item - Sealed Tallum Boots
(21112,5496,1,1,1,2589),-- Sealed Dark Crystal Boots Lining
(21112,5497,1,1,1,2589),-- Sealed Tallum Boots Lining
(21112,5508,1,1,1,2589),-- Sealed Dark Crystal Gloves Design
(21112,5509,1,1,1,2589),-- Sealed Tallum Gloves Design
(21112,1868,1,1,2,200000),-- Thread
(21112,1873,1,1,2,50000),-- Silver Nugget
(21112,1876,1,1,2,20000),-- Mithril Ore
(21112,1895,1,1,2,31250),-- Metallic Fiber
(21112,4042,1,1,2,4464),-- Enria
(21112,5281,1,1,2,70),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21112,8730,1,1,200,7599),-- Life Stone: level 67
(21112,8740,1,1,200,183),-- Mid-Grade Life Stone: level 67
(21112,8750,1,1,200,28),-- High-Grade Life Stone: level 67
(21113,5156,1,1,-1,4454),-- Recipe: Soulshot (A) Compressed Package (100%)
(21113,5166,1,1,-1,4454),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21113,5489,1,1,-1,44093),-- Sealed Tallum Stockings Fabric
(21113,57,1613,3293,0,700000),-- Adena
(21113,12222,1,1,1,175),-- Common Item - Sealed Dark Crystal Leather Armor
(21113,12228,1,1,1,582),-- Common Item - Sealed Dark Crystalline Leggings
(21113,12230,1,1,1,136),-- Common Item - Sealed Tallum Leather Armor
(21113,5478,1,1,1,1383),-- Sealed Dark Crystal Leather Armor Pattern
(21113,5479,1,1,1,973),-- Sealed Tallum Leather Armor Pattern
(21113,5482,1,1,1,2171),-- Sealed Dark Crystalline Leggings Design
(21113,1344,10,30,2,142857),-- Mithril Arrow
(21113,1864,1,3,2,111111),-- Stem
(21113,1869,1,1,2,100000),-- Iron Ore
(21113,1878,1,1,2,43478),-- Braided Hemp
(21113,1884,1,1,2,66667),-- Cord
(21113,3035,1,1,2,175),-- Recipe: Spiritshot A
(21113,4039,1,1,2,2933),-- Mold Glue
(21113,8730,1,1,200,1534),-- Life Stone: level 67
(21113,8740,1,1,200,97),-- Mid-Grade Life Stone: level 67
(21113,8750,1,1,200,19),-- High-Grade Life Stone: level 67
(21113,8760,1,1,200,39),-- Top-Grade Life Stone: level 67
(21114,1875,1,1,-1,212062),-- Stone of Purity
(21114,5161,1,1,-1,4241),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21114,5494,1,1,-1,66805),-- Sealed Dark Crystal Shield Fragment
(21114,57,1538,3132,0,700000),-- Adena
(21114,12227,1,1,1,108),-- Common Item - Sealed Dark Crystal breastplate
(21114,12220,1,1,1,463),-- Common Item - Sealed Dark Crystal Gaiters
(21114,12233,1,1,1,590),-- Common Item - Sealed Tallum Plate Armor
(21114,5520,1,1,1,1011),-- Sealed Dark Crystal Breastplate Pattern
(21114,5521,1,1,1,713),-- Sealed Tallum Plate Armor Pattern
(21114,5524,1,1,1,1544),-- Sealed Dark Crystal Gaiters Pattern
(21114,1344,10,30,2,125000),-- Mithril Arrow
(21114,1865,1,1,2,100000),-- Varnish
(21114,1870,1,1,2,100000),-- Coal
(21114,1874,1,1,2,5780),-- Oriharukon Ore
(21114,4039,1,1,2,5556),-- Mold Glue
(21114,4043,1,1,2,4425),-- Asofe
(21114,5271,1,1,2,49),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(21114,8731,1,1,200,1214),-- Life Stone: level 70
(21114,8741,1,1,200,396),-- Mid-Grade Life Stone: level 70
(21114,8751,1,1,200,11),-- High-Grade Life Stone: level 70
(21115,5166,1,1,-1,3704),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21115,5496,1,1,-1,65134),-- Sealed Dark Crystal Boots Lining
(21115,5497,1,1,-1,65134),-- Sealed Tallum Boots Lining
(21115,57,1342,2736,0,700000),-- Adena
(21115,12232,1,1,1,315),-- Common Item - Sealed Tallum Tunic
(21115,12235,1,1,1,785),-- Common Item - Sealed Tallum Stockings
(21115,12223,1,1,1,489),-- Common Item - Sealed Dark Crystal Robe
(21115,5485,1,1,1,1150),-- Sealed Tallum Tunic Texture
(21115,5486,1,1,1,810),-- Sealed Dark Crystal Robe Fabric
(21115,5489,1,1,1,1805),-- Sealed Tallum Stockings Fabric
(21115,1866,1,1,2,62500),-- Suede
(21115,1871,1,1,2,90909),-- Charcoal
(21115,1885,1,1,2,7937),-- High Grade Suede
(21115,4041,1,1,2,469),-- Mold Hardener
(21115,5220,1,1,2,2625),-- Metal Hardener
(21115,5281,1,1,2,74),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21115,6671,1,1,2,10870),-- Deluxe Chest Key - Grade 7
(21115,8731,1,1,200,967),-- Life Stone: level 70
(21115,8741,1,1,200,312),-- Mid-Grade Life Stone: level 70
(21115,8751,1,1,200,8),-- High-Grade Life Stone: level 70
(21115,8761,1,1,200,3),-- Top-Grade Life Stone: level 70
(21116,729,1,1,-1,489),-- Scroll: Enchant Weapon (Grade A)
(21116,5480,1,1,-1,19737),-- Sealed Leather Armor of Nightmare Fabric
(21116,5481,1,1,-1,19737),-- Sealed Majestic Leather Armor Fabric
(21116,57,1770,3612,0,700000),-- Adena
(21116,12344,1,1,1,913),-- Common Item - Sealed Phoenix Necklace
(21116,12345,1,1,1,919),-- Common Item - Sealed Phoenix Earring
(21116,12346,1,1,1,1066),-- Common Item - Sealed Phoenix Ring
(21116,6341,1,1,1,5990),-- Sealed Phoenix Earring Gemstone
(21116,6343,1,1,1,4630),-- Sealed Phoenix Necklace Beads
(21116,6345,1,1,1,8285),-- Sealed Phoenix Ring Gemstone
(21116,729,1,1,2,10),-- Scroll: Enchant Weapon (Grade A)
(21116,1867,1,1,2,200000),-- Animal Skin
(21116,1872,1,1,2,200000),-- Animal Bone
(21116,1882,1,1,2,33333),-- Leather
(21116,1894,1,1,2,3717),-- Crafted Leather
(21116,4042,1,1,2,2500),-- Enria
(21116,4987,1,1,2,306),-- Recipe: Tunic of Doom (60%)
(21116,8731,1,1,200,963),-- Life Stone: level 70
(21116,8741,1,1,200,379),-- Mid-Grade Life Stone: level 70
(21116,8751,1,1,200,18),-- High-Grade Life Stone: level 70
(21116,8761,1,1,200,6),-- Top-Grade Life Stone: level 70
(21117,734,1,1,-1,43530),-- Haste Potion
(21117,1800,1,1,-1,82131),-- Recipe: Piece Bone Breastplate
(21117,1897,1,1,-1,62185),-- Necklace of Wisdom Chain
(21117,57,122,208,0,700000),-- Adena
(21117,38,1,1,1,1520),-- Low Boots
(21117,50,1,1,1,1520),-- Leather Gloves
(21117,1925,1,1,1,33727),-- Leather Gloves Lining
(21117,1868,1,3,2,102864),-- Thread
(21117,1873,1,1,2,20573),-- Silver Nugget
(21117,2139,1,1,2,10287),-- Recipe: Steel Mold
(21118,1801,1,1,-1,138851),-- Recipe: Piece Bone Gaiters
(21118,1924,1,1,-1,88910),-- Leather Helmet Design
(21118,1925,1,1,-1,180734),-- Leather Gloves Lining
(21118,57,127,221,0,700000),-- Adena
(21118,44,1,1,1,799),-- Leather Helmet
(21118,102,1,1,1,1146),-- Round Shield
(21118,945,1,1,1,2367),-- Skeleton Buckler
(21118,1924,1,1,1,13132),-- Leather Helmet Design
(21118,1864,1,1,2,65132),-- Stem
(21118,1869,1,1,2,32566),-- Iron Ore
(21118,1880,1,1,2,3257),-- Steel
(21118,2142,1,1,2,6514),-- Recipe: Varnish of Purity
(21118,6035,1,1,2,2714),-- Magic Haste Potion
(21119,1868,1,1,-1,569536),-- Thread
(21119,1873,1,1,-1,113907),-- Silver Nugget
(21119,1898,1,1,-1,29252),-- Blue Diamond Necklace Gem
(21119,57,130,231,0,700000),-- Adena
(21119,27,1,1,1,228),-- Hard Leather Shirt
(21119,33,1,1,1,366),-- Hard Leather Gaiters
(21119,1927,1,1,1,3356),-- Hard Leather Shirt Pattern
(21119,1929,1,1,1,5427),-- Hard Leather Gaiters Material
(21119,1868,1,1,2,168233),-- Thread
(21119,1873,1,1,2,33647),-- Silver Nugget
(21120,1804,1,1,-1,14652),-- Recipe: Soulshot: D-Grade
(21120,1926,1,1,-1,36999),-- Piece Bone Breastplate Fragment
(21120,2151,1,1,-1,102031),-- Recipe: Necklace of Devotion
(21120,57,166,299,0,700000),-- Adena
(21120,87,1,1,1,45),-- Iron Hammer
(21120,155,1,1,1,45),-- Buzdygan
(21120,177,1,1,1,45),-- Mage Staff
(21120,2014,1,1,1,622),-- Iron Hammer Head
(21120,1867,1,1,2,72131),-- Animal Skin
(21120,1872,1,1,2,72131),-- Animal Bone
(21120,1882,1,1,2,12022),-- Leather
(21120,2255,1,1,2,1323),-- Recipe: Saber
(21121,1875,1,1,-1,21428),-- Stone of Purity
(21121,1881,1,1,-1,42856),-- Coarse Bone Powder
(21121,1927,1,1,-1,25642),-- Hard Leather Shirt Pattern
(21121,57,147,260,0,700000),-- Adena
(21121,257,1,1,1,156),-- Viper\'s Fang
(21121,1867,1,1,2,63296),-- Animal Skin
(21121,1872,1,1,2,63296),-- Animal Bone
(21121,1881,1,1,2,6330),-- Coarse Bone Powder
(21121,2258,1,1,2,1161),-- Recipe: Temptation of Abyss
(21122,1928,1,1,-1,49451),-- Piece Bone Gaiters Fragment
(21122,1929,1,1,-1,43804),-- Hard Leather Gaiters Material
(21122,2176,1,1,-1,162431),-- Recipe: Leather Boots
(21122,57,151,278,0,700000),-- Adena
(21122,273,1,1,1,83),-- Composition Bow
(21122,2016,1,1,1,1175),-- Composition Bow Shaft
(21122,1868,1,1,2,133704),-- Thread
(21122,1873,1,1,2,26741),-- Silver Nugget
(21122,5268,1,1,2,2675),-- Recipe: Greater Soulshot (D) Compressed Package (100%)
(21123,1874,1,1,-1,29775),-- Oriharukon Ore
(21123,1899,1,2,-1,69786),-- Necklace of Devotion Chain
(21123,2152,1,2,-1,78632),-- Recipe: Enchanted Necklace
(21123,57,236,442,0,700000),-- Adena
(21123,220,1,1,1,123),-- Crafted Dagger
(21123,1874,1,1,2,4739),-- Oriharukon Ore
(21123,1878,1,1,2,27778),-- Braided Hemp
(21123,2178,1,1,2,8475),-- Recipe: Dark Stockings
(21123,5220,1,1,2,2564),-- Metal Hardener
(21124,735,1,1,-1,21105),-- Potion of Alacrity
(21124,1865,1,2,-1,253256),-- Varnish
(21124,1897,1,1,-1,60299),-- Necklace of Wisdom Chain
(21124,57,121,200,0,700000),-- Adena
(21124,309,1,1,1,110),-- Tears of Eva
(21124,310,1,1,1,110),-- Relic of The Saints
(21124,1800,1,1,2,11763),-- Recipe: Piece Bone Breastplate
(21124,1865,1,1,2,37404),-- Varnish
(21124,1870,1,1,2,37404),-- Coal
(21124,1879,1,1,2,6235),-- Cokes
(21125,1800,1,1,-1,82944),-- Recipe: Piece Bone Breastplate
(21125,1801,1,1,-1,132878),-- Recipe: Piece Bone Gaiters
(21125,1924,1,1,-1,85085),-- Leather Helmet Design
(21125,57,124,211,0,700000),-- Adena
(21125,15,1,1,1,115),-- Short Spear
(21125,2011,1,1,1,1778),-- Short Spear Edge
(21125,734,1,1,2,6493),-- Haste Potion
(21125,1866,1,1,2,25971),-- Suede
(21125,1871,1,1,2,38956),-- Charcoal
(21125,2140,1,1,2,7792),-- Recipe: High Grade Suede
(21126,1864,1,2,-1,540431),-- Stem
(21126,1865,1,2,-1,270215),-- Varnish
(21126,1925,1,1,-1,177190),-- Leather Gloves Lining
(21126,57,125,217,0,700000),-- Adena
(21126,24,1,1,1,394),-- Bone Breastplate
(21126,31,1,1,1,629),-- Bone Gaiters
(21126,390,1,1,1,394),-- Cotton Shirt
(21126,412,1,1,1,629),-- Cotton Pants
(21126,1864,1,1,2,53212),-- Stem
(21126,1865,1,1,2,26606),-- Varnish
(21126,1866,1,1,2,17738),-- Suede
(21126,1868,1,1,2,53212),-- Thread
(21126,1873,1,1,2,10643),-- Silver Nugget
(21126,2143,1,1,2,5322),-- Recipe: Synthetic Cokes
(21127,1804,1,1,-1,11391),-- Recipe: Soulshot: D-Grade
(21127,1898,1,1,-1,29252),-- Blue Diamond Necklace Gem
(21127,1926,1,1,-1,28764),-- Piece Bone Breastplate Fragment
(21127,57,130,231,0,700000),-- Adena
(21127,25,1,1,1,177),-- Piece Bone Breastplate
(21127,26,1,1,1,114),-- Bronze Breastplate
(21127,32,1,1,1,282),-- Piece Bone Gaiters
(21127,34,1,1,1,183),-- Bronze Gaiters
(21127,1926,1,1,1,2833),-- Piece Bone Breastplate Fragment
(21127,1928,1,1,1,4085),-- Piece Bone Gaiters Fragment
(21127,1868,1,1,2,84116),-- Thread
(21127,1873,1,1,2,16824),-- Silver Nugget
(21127,2151,1,1,2,11716),-- Recipe: Necklace of Devotion
(21127,6035,1,1,2,3505),-- Magic Haste Potion
(21128,1869,1,3,-1,301771),-- Iron Ore
(21128,1927,1,1,-1,24074),-- Hard Leather Shirt Pattern
(21128,2151,1,1,-1,84059),-- Recipe: Necklace of Devotion
(21128,57,137,246,0,700000),-- Adena
(21128,1102,1,1,1,484),-- Tunic of Magic
(21128,1105,1,1,1,775),-- Stockings of Magic
(21128,1867,1,1,2,39618),-- Animal Skin
(21128,1869,1,1,2,29713),-- Iron Ore
(21128,1870,1,1,2,29713),-- Coal
(21128,1871,1,1,2,29713),-- Charcoal
(21128,1872,1,1,2,39618),-- Animal Bone
(21128,2256,1,1,2,727),-- Recipe: Assassin Knife
(21129,1870,1,2,-1,321422),-- Coal
(21129,1928,1,1,-1,46820),-- Piece Bone Gaiters Fragment
(21129,2176,1,1,-1,153791),-- Recipe: Leather Boots
(21129,57,147,260,0,700000),-- Adena
(21129,846,1,1,1,594),-- Coral Earring
(21129,878,1,1,1,888),-- Blue Coral Ring
(21129,909,1,1,1,446),-- Blue Diamond Necklace
(21129,1898,1,1,1,4877),-- Blue Diamond Necklace Gem
(21129,1865,1,1,2,47472),-- Varnish
(21129,1870,1,1,2,47472),-- Coal
(21129,3032,1,1,2,1899),-- Recipe: Spiritshot D
(21129,6035,1,1,2,3956),-- Magic Haste Potion
(21130,1929,1,1,-1,43804),-- Hard Leather Gaiters Material
(21130,1930,1,1,-1,87046),-- Boot Lining
(21130,2177,1,1,-1,108460),-- Recipe: Bone Helmet
(21130,57,151,278,0,700000),-- Adena
(21130,39,1,1,1,1087),-- Boots
(21130,51,1,1,1,1087),-- Bracer
(21130,1930,1,1,1,17142),-- Boot Lining
(21130,734,1,1,2,8357),-- Haste Potion
(21130,1866,1,1,2,33426),-- Suede
(21130,1871,1,1,2,50139),-- Charcoal
(21130,5273,1,1,2,2006),-- Recipe: Greater Spiritshot (D) Compressed Package(100%)
(21131,1867,1,3,-1,466319),-- Animal Skin
(21131,1872,1,3,-1,466319),-- Animal Bone
(21131,1899,1,1,-1,54647),-- Necklace of Devotion Chain
(21131,57,185,345,0,700000),-- Adena
(21131,45,1,1,1,419),-- Bone Helmet
(21131,626,1,1,1,598),-- Bronze Shield
(21131,1932,1,1,1,10386),-- Bone Helmet Design
(21131,1867,1,1,2,71429),-- Animal Skin
(21131,1872,1,1,2,58824),-- Animal Bone
(21131,1875,1,1,2,6329),-- Stone of Purity
(21131,1881,1,1,2,3802),-- Coarse Bone Powder
(21131,2180,1,1,2,5076),-- Recipe: Scale Mail
(21132,1898,1,1,-1,28490),-- Blue Diamond Necklace Gem
(21132,2179,1,1,-1,132705),-- Recipe: Crafted Leather Gloves
(21132,2255,1,1,-1,6781),-- Recipe: Saber
(21132,57,123,227,0,700000),-- Adena
(21132,68,1,1,1,67),-- Falchion
(21132,5284,1,1,1,67),-- Zweihander
(21132,1868,1,1,2,109235),-- Thread
(21132,1873,1,1,2,21847),-- Silver Nugget
(21132,5278,1,1,2,2185),-- Recipe: Greater Blessed Spiritshot (D) Compressed Package(100%)
(21133,1864,1,1,-1,600386),-- Stem
(21133,1878,1,1,-1,120077),-- Braided Hemp
(21133,1899,1,1,-1,46905),-- Necklace of Devotion Chain
(21133,57,160,295,0,700000),-- Adena
(21133,156,1,1,1,28),-- Hand Axe
(21133,167,1,1,1,28),-- Scalpel
(21133,178,1,1,1,28),-- Bone Staff
(21133,1061,1,1,2,20422),-- Healing Potion
(21133,1864,1,1,2,67391),-- Stem
(21133,1869,1,1,2,33696),-- Iron Ore
(21133,1878,1,1,2,13479),-- Braided Hemp
(21133,2181,1,1,2,3373),-- Recipe: White Tunic
(21134,1931,1,1,-1,40933),-- Leather Boot Lining
(21134,1932,1,1,-1,49704),-- Bone Helmet Design
(21134,2152,1,1,-1,55130),-- Recipe: Enchanted Necklace
(21134,57,164,312,0,700000),-- Adena
(21134,258,1,1,1,86),-- Bagh-Nakh
(21134,1865,1,1,2,35149),-- Varnish
(21134,1870,1,1,2,35149),-- Coal
(21134,1875,1,1,2,2344),-- Stone of Purity
(21134,1879,1,1,2,5859),-- Cokes
(21134,2259,1,1,2,546),-- Recipe: Spinebone Sword
(21135,1934,1,1,-1,49861),-- Crafted Leather Gloves Lining
(21135,2178,1,1,-1,61030),-- Recipe: Dark Stockings
(21135,5220,1,1,-1,15258),-- Metal Hardener
(21135,57,201,368,0,700000),-- Adena
(21135,274,1,1,1,105),-- Strengthened Bow
(21135,1866,1,1,2,28544),-- Suede
(21135,1871,1,1,2,42816),-- Charcoal
(21135,1885,1,1,2,3568),-- High Grade Suede
(21135,2261,1,1,2,665),-- Recipe: Conjuror\'s Staff
(21135,5220,1,1,2,1713),-- Metal Hardener
(21136,1882,1,1,-1,74873),-- Leather
(21136,2017,1,1,-1,4477),-- Saber Blade
(21136,2180,1,1,-1,25333),-- Recipe: Scale Mail
(21136,57,174,336,0,700000),-- Adena
(21136,220,1,1,1,92),-- Crafted Dagger
(21136,1834,1,1,2,42022),-- Emergency Dressing
(21136,1867,1,1,2,50426),-- Animal Skin
(21136,1872,1,1,2,50426),-- Animal Bone
(21136,1882,1,1,2,8405),-- Leather
(21136,2152,1,1,2,6659),-- Recipe: Enchanted Necklace
(21137,1900,1,1,-1,39353),-- Enchanted Necklace Chain
(21137,1933,1,1,-1,42316),-- Dark Stocking Fabric
(21137,2153,1,1,-1,55938),-- Recipe: Tiger\'s Eye Earring
(21137,57,185,357,0,700000),-- Adena
(21137,315,1,1,1,61),-- Divine Tome
(21137,737,1,1,2,3276),-- Scroll of Resurrection
(21137,1868,1,1,2,65519),-- Thread
(21137,1873,1,1,2,13104),-- Silver Nugget
(21137,1889,1,1,2,2184),-- Compound Braid
(21137,5549,1,1,2,3276),-- Metallic Thread
(21137,6387,1,1,2,1311),-- Blessed Scroll of Resurrection for Pets
(21138,956,1,1,-1,12783),-- Scroll: Enchant Armor (Grade D)
(21138,1864,1,1,-1,766967),-- Stem
(21138,1869,1,1,-1,383484),-- Iron Ore
(21138,57,199,383,0,700000),-- Adena
(21138,292,1,1,1,65),-- Pike
(21138,1341,10,30,2,116423),-- Bone Arrow
(21138,1829,1,1,2,13971),-- Scroll of Escape: Clan Hall
(21138,1864,1,2,2,69854),-- Stem
(21138,1869,1,1,2,34927),-- Iron Ore
(21138,1880,1,1,2,3493),-- Steel
(21138,5550,1,1,2,466),-- Durable Metal Plate
(21139,1804,1,1,-1,37606),-- Recipe: Soulshot: D-Grade
(21139,1898,1,1,-1,96575),-- Blue Diamond Necklace Gem
(21139,1926,1,1,-1,94966),-- Piece Bone Breastplate Fragment
(21139,57,430,759,0,700000),-- Adena
(21139,27,1,1,1,753),-- Hard Leather Shirt
(21139,33,1,1,1,1208),-- Hard Leather Gaiters
(21139,1927,1,1,1,11078),-- Hard Leather Shirt Pattern
(21139,1929,1,1,1,17918),-- Hard Leather Gaiters Material
(21139,1865,1,1,2,142857),-- Varnish
(21139,1870,1,1,2,166667),-- Coal
(21139,3953,1,1,2,8000),-- Recipe: Blessed Spiritshot D
(21140,1871,1,3,-1,549553),-- Charcoal
(21140,1899,1,1,-1,171735),-- Necklace of Devotion Chain
(21140,2152,1,1,-1,193505),-- Recipe: Enchanted Necklace
(21140,57,477,911,0,700000),-- Adena
(21140,347,1,1,1,739),-- Ring Mail Breastplate
(21140,376,1,1,1,1182),-- Iron Plate Gaiters
(21140,1866,1,1,2,83333),-- Suede
(21140,1871,1,1,2,125000),-- Charcoal
(21140,1877,1,1,2,1639),-- Adamantite Nugget
(21140,2260,1,1,2,4098),-- Recipe: Mace of Judgment
(21141,1881,1,1,-1,169543),-- Coarse Bone Powder
(21141,1931,1,1,-1,166219),-- Leather Boot Lining
(21141,2178,1,1,-1,203452),-- Recipe: Dark Stockings
(21141,57,545,1067,0,700000),-- Adena
(21141,432,1,1,1,1140),-- Cursed Tunic
(21141,465,1,1,1,1826),-- Cursed Stockings
(21141,1867,1,1,2,125000),-- Animal Skin
(21141,1872,1,1,2,125000),-- Animal Bone
(21141,1875,1,1,2,6757),-- Stone of Purity
(21141,1881,1,1,2,10753),-- Coarse Bone Powder
(21141,2178,1,1,2,12195),-- Recipe: Dark Stockings
(21142,1900,1,1,-1,138128),-- Enchanted Necklace Chain
(21142,1933,1,1,-1,148528),-- Dark Stocking Fabric
(21142,2153,1,1,-1,196344),-- Recipe: Tiger\'s Eye Earring
(21142,57,535,1059,0,700000),-- Adena
(21142,848,1,1,1,810),-- Enchanted Earring
(21142,879,1,1,1,1215),-- Enchanted Ring
(21142,911,1,1,1,608),-- Enchanted Necklace
(21142,1900,1,1,1,18871),-- Enchanted Necklace Chain
(21142,955,1,1,2,690),-- Scroll: Enchant Weapon (Grade D)
(21142,1868,1,1,2,200000),-- Thread
(21142,1873,1,1,2,35714),-- Silver Nugget
(21142,1876,1,1,2,27778),-- Mithril Ore
(21143,955,1,1,-1,5610),-- Scroll: Enchant Weapon (Grade D)
(21143,1872,1,3,-1,934937),-- Animal Bone
(21143,1875,1,1,-1,93494),-- Stone of Purity
(21143,57,593,1182,0,700000),-- Adena
(21143,605,1,1,1,2302),-- Leather Gauntlets
(21143,1123,1,1,1,2302),-- Blue Buckskin Boots
(21143,1867,1,1,2,125000),-- Animal Skin
(21143,1872,1,1,2,125000),-- Animal Bone
(21143,1875,1,1,2,8929),-- Stone of Purity
(21143,1881,1,1,2,13514),-- Coarse Bone Powder
(21143,2250,1,1,2,23256),-- Recipe: Bone Arrow
(21143,6035,1,1,2,8333),-- Magic Haste Potion
(21144,1901,1,1,-1,229413),-- Tiger\'s Eye Earring Stone
(21144,1938,1,1,-1,111661),-- Mithril Banded Mail Material
(21144,2154,1,1,-1,177832),-- Recipe: Elven Earring
(21144,57,702,1399,0,700000),-- Adena
(21144,47,1,1,1,1160),-- Helmet
(21144,628,1,1,1,1657),-- Hoplon
(21144,1865,1,1,2,100000),-- Varnish
(21144,1870,1,1,2,111111),-- Coal
(21144,1875,1,1,2,5208),-- Stone of Purity
(21144,1879,1,1,2,20000),-- Cokes
(21144,2187,1,1,2,4651),-- Recipe: Manticore Skin Shirt
(21144,6035,1,1,2,8197),-- Magic Haste Potion
(21144,8317,1,1,2,1095),-- Recipe: Titan Hammer (100%)
(21145,1825,1,1,-1,150797),-- Recipe: Oriharukon
(21145,1885,1,1,-1,175930),-- High Grade Suede
(21145,1902,1,1,-1,213897),-- Elven Earring Beads
(21145,57,893,1781,0,700000),-- Adena
(21145,129,1,1,1,78),-- Sword of Revolution
(21145,2034,1,1,1,5766),-- Sword of Revolution Blade
(21145,1832,1,1,2,142857),-- Greater Antidote
(21145,1866,1,1,2,90909),-- Suede
(21145,1871,1,1,2,142857),-- Charcoal
(21145,1885,1,1,2,11364),-- High Grade Suede
(21145,2146,1,1,2,8547),-- Recipe: Artisan\'s frame
(21145,5220,1,1,2,4386),-- Metal Hardener
(21145,5473,1,1,2,10309),-- Recipe: Durable Metal Plate(100%)
(21146,1894,1,1,-1,79584),-- Crafted Leather
(21146,1903,1,1,-1,378338),-- Elven Ring Piece
(21146,2145,1,1,-1,162010),-- Recipe: Mithril Alloy
(21146,57,951,1898,0,700000),-- Adena
(21146,88,1,1,1,17),-- Morning Star
(21146,90,1,1,1,17),-- Goat Head Staff
(21146,158,1,1,1,17),-- Tarbar
(21146,169,1,1,1,17),-- Skull Breaker
(21146,172,1,1,1,17),-- Heavy Bone Club
(21146,2031,1,1,1,1187),-- Morning Star Head
(21146,2032,1,1,1,1084),-- Goat Head Staff Head
(21146,2035,1,1,1,1187),-- Tarbar Head
(21146,2036,1,1,1,1187),-- Skull Breaker Head
(21146,2037,1,1,1,1187),-- Heavy Bone Club Head
(21146,1867,1,1,2,200000),-- Animal Skin
(21146,1872,1,1,2,200000),-- Animal Bone
(21146,1875,1,1,2,12048),-- Stone of Purity
(21146,1881,1,1,2,20000),-- Coarse Bone Powder
(21146,1805,1,1,2,489),-- Recipe: Soulshot: C-Grade
(21146,2990,1,1,2,10753),-- Recipe: Chain Boots
(21147,1805,1,1,-1,9195),-- Recipe: Soulshot: C-Grade
(21147,1905,1,1,-1,268330),-- Omen Beast\'s Eye Earring Gemstone
(21147,1906,1,1,-1,379166),-- Mithril Ring Wire
(21147,57,1113,2260,0,700000),-- Adena
(21147,262,1,1,1,77),-- Scallop Jamadhr
(21147,2053,1,1,1,5727),-- Scallop Jamadhr Edge
(21147,1868,1,3,2,250000),-- Thread
(21147,1873,1,1,2,90909),-- Silver Nugget
(21147,1889,1,1,2,13889),-- Compound Braid
(21147,2199,1,1,2,10204),-- Recipe: Karmian Stockings
(21147,2301,1,1,2,1570),-- Recipe: Battle Axe
(21147,5549,1,1,2,22727),-- Metallic Thread
(21148,1869,1,5,-1,912739),-- Iron Ore
(21148,1880,1,1,-1,273822),-- Steel
(21148,5550,1,1,-1,36510),-- Durable Metal Plate
(21148,57,1099,2253,0,700000),-- Adena
(21148,281,1,1,1,57),-- Crystallized Ice Bow
(21148,2074,1,1,1,4583),-- Crystallized Ice Bow Shaft
(21148,1864,1,3,2,90909),-- Stem
(21148,1869,1,1,2,90909),-- Iron Ore
(21148,1880,1,1,2,7042),-- Steel
(21148,2163,1,1,2,5263),-- Recipe: Earring of Protection
(21148,2217,1,1,2,8547),-- Recipe: Knight\'s Shield
(21148,5550,1,1,2,1099),-- Durable Metal Plate
(21148,6037,1,3,2,58824),-- Waking Scroll
(21148,8723,1,1,200,3953),-- Life Stone: level 46
(21148,8733,1,1,200,1099),-- Mid-Grade Life Stone: level 46
(21148,8743,1,1,200,53),-- High-Grade Life Stone: level 46
(21149,1908,1,1,-1,299146),-- Moonstone Earring Wire
(21149,1909,1,1,-1,426601),-- Aquastone Ring Wire
(21149,2163,1,1,-1,161172),-- Recipe: Earring of Protection
(21149,57,1269,2596,0,700000),-- Adena
(21149,226,1,1,1,33),-- Cursed Dagger
(21149,232,1,1,1,33),-- Dark Elven Dagger
(21149,2070,1,1,1,2964),-- Cursed Dagger Blade
(21149,2072,1,1,1,2964),-- Dark Elven Dagger Edge
(21149,1868,1,1,2,250000),-- Thread
(21149,1873,1,1,2,50000),-- Silver Nugget
(21149,1876,1,1,2,20408),-- Mithril Ore
(21149,1895,1,1,2,37037),-- Metallic Fiber
(21149,2215,1,1,2,1988),-- Recipe: Robe of Seal
(21149,2228,1,1,2,3968),-- Recipe: Shining Circlet
(21149,4042,1,1,2,15465),-- Enria
(21149,8723,1,1,200,5525),-- Life Stone: level 46
(21149,8733,1,1,200,1546),-- Mid-Grade Life Stone: level 46
(21149,8743,1,1,200,442),-- High-Grade Life Stone: level 46
(21149,8753,1,1,200,110),-- Top-Grade Life Stone: level 46
(21150,951,1,1,-1,6524),-- Scroll: Enchant Weapon (Grade C)
(21150,1911,1,1,-1,266897),-- Earring of Protection Gemstone
(21150,2166,1,1,-1,121642),-- Recipe: Earrings of Binding (100%)
(21150,57,1445,2946,0,700000),-- Adena
(21150,325,1,1,1,56),-- Horn of Glory
(21150,2088,1,1,1,5603),-- Horn of Glory Fragment
(21150,1342,10,30,2,200000),-- Fine Steel Arrow
(21150,1866,1,1,2,66667),-- Suede
(21150,1871,1,1,2,90909),-- Charcoal
(21150,1877,1,1,2,3759),-- Adamantite Nugget
(21150,2166,1,1,2,3861),-- Recipe: Earrings of Binding (100%)
(21150,2234,1,1,2,2463),-- Recipe: Divine Stockings
(21150,3014,1,1,2,1736),-- Recipe: Full Plate Boots
(21150,4040,1,1,2,1513),-- Mold Lubricant
(21150,4041,1,1,2,784),-- Mold Hardener
(21150,8724,1,1,200,3584),-- Life Stone: level 49
(21150,8734,1,1,200,897),-- Mid-Grade Life Stone: level 49
(21150,8744,1,1,200,44),-- High-Grade Life Stone: level 49
(21150,8754,1,1,200,43),-- Top-Grade Life Stone: level 49
(21151,1869,1,9,-1,935306),-- Iron Ore
(21151,4043,1,1,-1,124707),-- Asofe
(21151,4601,1,1,-1,8314),-- Greater Dye of Strength <Str+3 Con-3>
(21151,57,1509,3068,0,700000),-- Adena
(21151,231,1,1,1,18),-- Grace Dagger
(21151,233,1,1,1,18),-- Dark Screamer
(21151,2106,1,1,1,2109),-- Grace Dagger Edge
(21151,2107,1,1,1,2109),-- Dark Screamer Edge
(21151,1864,1,3,2,125000),-- Stem
(21151,1869,1,1,2,125000),-- Iron Ore
(21151,1880,1,1,2,12500),-- Steel
(21151,2169,1,1,2,2564),-- Recipe: Necklace of Binding
(21151,4043,1,1,2,4425),-- Asofe
(21151,5550,1,1,2,1121),-- Durable Metal Plate
(21151,8725,1,1,200,3311),-- Life Stone: level 52
(21151,8735,1,1,200,374),-- Mid-Grade Life Stone: level 52
(21151,8745,1,1,200,53),-- High-Grade Life Stone: level 52
(21152,1806,1,1,-1,8007),-- Recipe: Soulshot: B-Grade
(21152,1920,1,1,-1,136989),-- Necklace of Binding Chain
(21152,4600,1,1,-1,13345),-- Greater Dye of Wit <Wit+2 Men-2>
(21152,57,1616,3283,0,700000),-- Adena
(21152,400,1,1,1,60),-- Theca Leather Armor
(21152,401,1,1,1,32),-- Drake Leather Armor
(21152,420,1,1,1,96),-- Theca Leather Gaiters
(21152,1984,1,1,1,5282),-- Theca Leather Armor Pattern
(21152,1985,1,1,1,8496),-- Theca Leather Gaiters Pattern
(21152,1987,1,1,1,3759),-- Drake Leather Armor Texture
(21152,1806,1,1,2,370),-- Recipe: Soulshot: B-Grade
(21152,1868,1,1,2,250000),-- Thread
(21152,1873,1,1,2,43478),-- Silver Nugget
(21152,1876,1,1,2,27027),-- Mithril Ore
(21152,1895,1,1,2,38462),-- Metallic Fiber
(21152,4042,1,1,2,2364),-- Enria
(21152,4941,1,1,2,1290),-- Recipe: Zubei\'s Gaiters  (60%)
(21152,4959,1,1,2,1506),-- Recipe: Avadon Boots (60%)
(21152,8726,1,1,200,2364),-- Life Stone: level 55
(21152,8736,1,1,200,645),-- Mid-Grade Life Stone: level 55
(21152,8746,1,1,200,215),-- High-Grade Life Stone: level 55
(21153,947,1,1,-1,1895),-- Scroll: Enchant Weapon (Grade B)
(21153,4049,1,1,-1,248435),-- Adamantite Earring Gemstone
(21153,4129,1,1,-1,51260),-- Recipe: Earrings of Black Ore
(21153,57,1906,3892,0,700000),-- Adena
(21153,357,1,1,1,17),-- Zubei\'s Breastplate
(21153,383,1,1,1,27),-- Zubei\'s Gaiters
(21153,2376,1,1,1,17),-- Avadon Breastplate
(21153,2379,1,1,1,27),-- Avadon Gaiters
(21153,4056,1,1,1,3165),-- Zubei\'s Breastplate Part
(21153,4057,1,1,1,4892),-- Zubei\'s Gaiter Material
(21153,4064,1,1,1,3165),-- Avadon Breastplate Part
(21153,4065,1,1,1,4892),-- Avadon Gaiters Material
(21153,947,1,1,2,80),-- Scroll: Enchant Weapon (Grade B)
(21153,1894,1,1,2,4975),-- Crafted Leather
(21153,3955,1,1,2,390),-- Recipe: Blessed Spiritshot B
(21153,4039,1,1,2,4587),-- Mold Glue
(21153,4041,1,1,2,1075),-- Mold Hardener
(21153,4044,1,1,2,5291),-- Thons
(21153,5280,1,1,2,98),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(21153,8727,1,1,200,2639),-- Life Stone: level 58
(21153,8737,1,1,200,293),-- Mid-Grade Life Stone: level 58
(21153,8747,1,1,200,195),-- High-Grade Life Stone: level 58
(21154,4049,1,1,-1,260988),-- Adamantite Earring Gemstone
(21154,4613,1,1,-1,6863),-- Greater Dye of Strength <Str+4 Con-4>
(21154,4614,1,1,-1,6863),-- Greater Dye of Strength <Str+4 Dex-4>
(21154,57,2005,4083,0,700000),-- Adena
(21154,2397,1,1,1,32),-- Tunic of Zubei
(21154,2402,1,1,1,51),-- Stockings of Zubei
(21154,2406,1,1,1,22),-- Avadon Robe
(21154,4069,1,1,1,5774),-- Tunic of Zubei Fabric
(21154,4070,1,1,1,8767),-- Stockings of Zubei Fabric
(21154,4071,1,1,1,4436),-- Avadon Robe Fabric
(21154,1832,1,1,2,166667),-- Greater Antidote
(21154,1867,1,3,2,66667),-- Animal Skin
(21154,1872,1,3,2,100000),-- Animal Bone
(21154,1875,1,1,2,10638),-- Stone of Purity
(21154,1881,1,1,2,19231),-- Coarse Bone Powder
(21154,4041,1,1,2,1103),-- Mold Hardener
(21154,948,1,1,2,331),-- Scroll: Enchant Armor (Grade B)
(21154,4162,1,1,2,1323),-- Recipe: Tunic of Doom
(21155,4050,1,1,-1,310911),-- Adamantite Ring Wire
(21155,4051,1,1,-1,194613),-- Adamantite Necklace Chain
(21155,4130,1,1,-1,81463),-- Recipe: Ring of Black Ore
(21155,57,2021,4119,0,700000),-- Adena
(21155,856,1,1,1,82),-- Adamantite Earring
(21155,887,1,1,1,123),-- Adamantite Ring
(21155,918,1,1,1,62),-- Adamantite Necklace
(21155,4049,1,1,1,12961),-- Adamantite Earring Gemstone
(21155,4050,1,1,1,15310),-- Adamantite Ring Wire
(21155,4051,1,1,1,9583),-- Adamantite Necklace Chain
(21155,1895,1,1,2,40000),-- Metallic Fiber
(21155,4040,1,1,2,3356),-- Mold Lubricant
(21155,4042,1,1,2,1838),-- Enria
(21155,4043,1,1,2,4651),-- Asofe
(21155,4154,1,1,2,2985),-- Recipe: Doom Shield
(21155,6037,1,1,2,333333),-- Waking Scroll
(21155,8728,1,1,200,1949),-- Life Stone: level 61
(21155,8738,1,1,200,866),-- Mid-Grade Life Stone: level 61
(21155,8748,1,1,200,5),-- High-Grade Life Stone: level 61
(21156,948,1,1,-1,13466),-- Scroll: Enchant Armor (Grade B)
(21156,4056,1,1,-1,97490),-- Zubei\'s Breastplate Part
(21156,4131,1,1,-1,43791),-- Recipe: Necklace of Black Ore
(21156,57,2177,4415,0,700000),-- Adena
(21156,554,1,1,1,77),-- Zubei\'s Boots
(21156,600,1,1,1,77),-- Avadon Boots
(21156,612,1,1,1,77),-- Zubei\'s Gauntlets
(21156,2464,1,1,1,77),-- Avadon Gloves
(21156,4073,1,1,1,10348),-- Avadon Gloves Part
(21156,4097,1,1,1,10348),-- Zubei\'s Boots Design
(21156,4098,1,1,1,10348),-- Avadon Boots Design
(21156,4100,1,1,1,10348),-- Zubei\'s Gauntlets Part
(21156,1343,20,60,2,100000),-- Silver Arrow
(21156,1865,1,1,2,142857),-- Varnish
(21156,1870,1,1,2,142857),-- Coal
(21156,1875,1,1,2,9259),-- Stone of Purity
(21156,1879,1,1,2,22222),-- Cokes
(21156,4040,1,1,2,2882),-- Mold Lubricant
(21156,4160,1,1,2,537),-- Recipe: Leather Armor of Doom
(21156,8728,1,1,200,2445),-- Life Stone: level 61
(21156,8738,1,1,200,586),-- Mid-Grade Life Stone: level 61
(21156,8748,1,1,200,31),-- High-Grade Life Stone: level 61
(21157,1889,1,2,-1,409609),-- Compound Braid
(21157,4052,1,1,-1,217530),-- Earring of Black Ore Piece
(21157,5549,1,2,-1,614413),-- Metallic Thread
(21157,57,2488,5032,0,700000),-- Adena
(21157,110,1,1,1,72),-- Doom Shield
(21157,2416,1,1,1,50),-- Blue Wolf Helmet
(21157,2417,1,1,1,50),-- Doom Helmet
(21157,4077,1,1,1,13606),-- Doom Shield Fragment
(21157,4088,1,1,1,11431),-- Blue Wolf Helmet Design
(21157,4089,1,1,1,11431),-- Doom Helmet Pattern
(21157,1830,1,1,2,58824),-- Scroll of Escape: Castle
(21157,1832,1,1,2,166667),-- Greater Antidote
(21157,1868,1,3,2,142857),-- Thread
(21157,1873,1,1,2,58824),-- Silver Nugget
(21157,1889,1,1,2,11111),-- Compound Braid
(21157,4044,1,1,2,4975),-- Thons
(21157,5549,1,1,2,15385),-- Metallic Thread
(21157,8487,1,1,2,109),-- Recipe: Destroyer Hammer (60%)
(21157,8729,1,1,200,2506),-- Life Stone: level 64
(21157,8739,1,1,200,472),-- Mid-Grade Life Stone: level 64
(21157,8749,1,1,200,109),-- High-Grade Life Stone: level 64
(21157,8759,1,1,200,36),-- Top-Grade Life Stone: level 64
(21158,4053,1,1,-1,298243),-- Ring of Black Ore Gemstone
(21158,5230,1,1,-1,9277),-- Recipe: Mithril Arrow (100%)
(21158,57,2798,5718,0,700000),-- Adena
(21158,2391,1,1,1,30),-- Blue Wolf Leather Armor
(21158,2392,1,1,1,30),-- Leather Armor of Doom
(21158,4082,1,1,1,5805),-- Blue Wolf Leather Armor Texture
(21158,4083,1,1,1,5805),-- Leather Armor of Doom Design
(21158,1344,30,90,2,71429),-- Mithril Arrow
(21158,1830,1,1,2,76923),-- Scroll of Escape: Castle
(21158,1864,1,3,2,166667),-- Stem
(21158,1869,2,6,2,43478),-- Iron Ore
(21158,1880,1,1,2,18519),-- Steel
(21158,4043,1,1,2,5102),-- Asofe
(21158,5550,1,1,2,2481),-- Durable Metal Plate
(21158,8487,1,1,2,130),-- Recipe: Destroyer Hammer (60%)
(21158,8730,1,1,200,1969),-- Life Stone: level 67
(21158,8740,1,1,200,438),-- Mid-Grade Life Stone: level 67
(21158,8750,1,1,200,22),-- High-Grade Life Stone: level 67
(21159,1874,1,1,-1,479284),-- Oriharukon Ore
(21159,4043,1,1,-1,239642),-- Asofe
(21159,5478,1,1,-1,60455),-- Sealed Dark Crystal Leather Armor Pattern
(21159,57,2901,5897,0,700000),-- Adena
(21159,5304,1,1,1,21),-- Sealed Tallum Tunic
(21159,5305,1,1,1,33),-- Sealed Tallum Stockings
(21159,5308,1,1,1,14),-- Sealed Dark Crystal Robe
(21159,5485,1,1,1,2976),-- Sealed Tallum Tunic Texture
(21159,5486,1,1,1,2094),-- Sealed Dark Crystal Robe Fabric
(21159,5489,1,1,1,4671),-- Sealed Tallum Stockings Fabric
(21159,1344,20,60,2,100000),-- Mithril Arrow
(21159,1865,1,1,2,166667),-- Varnish
(21159,1870,1,1,2,166667),-- Coal
(21159,1874,1,1,2,11765),-- Oriharukon Ore
(21159,3035,1,1,2,127),-- Recipe: Spiritshot A
(21159,4039,1,1,2,4926),-- Mold Glue
(21159,4043,1,1,2,5848),-- Asofe
(21159,8326,1,1,2,85),-- Recipe: Doom Crusher (60%)
(21159,5281,1,1,2,170),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21159,8730,1,1,200,1779),-- Life Stone: level 67
(21159,8740,1,1,200,720),-- Mid-Grade Life Stone: level 67
(21159,8750,1,1,200,9),-- High-Grade Life Stone: level 67
(21159,8760,1,1,200,42),-- Top-Grade Life Stone: level 67
(21160,1807,1,1,-1,9997),-- Recipe: Soulshot: A-Grade
(21160,5479,1,1,-1,44366),-- Sealed Tallum Leather Armor Pattern
(21160,5482,1,1,-1,98961),-- Sealed Dark Crystalline Leggings Design
(21160,57,3019,6154,0,700000),-- Adena
(21160,5304,1,1,1,22),-- Sealed Tallum Tunic
(21160,5305,1,1,1,35),-- Sealed Tallum Stockings
(21160,5308,1,1,1,15),-- Sealed Dark Crystal Robe
(21160,5485,1,1,1,3104),-- Sealed Tallum Tunic Texture
(21160,5486,1,1,1,2184),-- Sealed Dark Crystal Robe Fabric
(21160,5489,1,1,1,4872),-- Sealed Tallum Stockings Fabric
(21160,729,1,1,2,35),-- Scroll: Enchant Weapon (Grade A)
(21160,1807,1,1,2,1040),-- Recipe: Soulshot: A-Grade
(21160,1894,1,1,2,7042),-- Crafted Leather
(21160,4039,1,1,2,5464),-- Mold Glue
(21160,4041,1,1,2,3115),-- Mold Hardener
(21160,4044,1,1,2,6757),-- Thons
(21160,6037,1,1,2,500000),-- Waking Scroll
(21160,8730,1,1,200,2597),-- Life Stone: level 67
(21160,8740,1,1,200,1818),-- Mid-Grade Life Stone: level 67
(21161,3035,1,1,-1,10926),-- Recipe: Spiritshot A
(21161,4041,1,1,-1,71260),-- Mold Hardener
(21161,5485,1,1,-1,68911),-- Sealed Tallum Tunic Texture
(21161,57,3302,6724,0,700000),-- Adena
(21161,6323,1,1,1,47),-- Sealed Phoenix Necklace
(21161,6324,1,1,1,62),-- Sealed Phoenix Earring
(21161,6325,1,1,1,93),-- Sealed Phoenix Ring
(21161,6341,1,1,1,11159),-- Sealed Phoenix Earring Gemstone
(21161,6343,1,1,1,8627),-- Sealed Phoenix Necklace Beads
(21161,6345,1,1,1,15434),-- Sealed Phoenix Ring Gemstone
(21161,1866,1,1,2,142857),-- Suede
(21161,1871,1,1,2,200000),-- Charcoal
(21161,1877,1,1,2,7634),-- Adamantite Nugget
(21161,4040,1,1,2,3509),-- Mold Lubricant
(21161,4041,1,1,2,2004),-- Mold Hardener
(21161,5276,1,1,2,125),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21161,8731,1,1,200,2381),-- Life Stone: level 70
(21161,8741,1,1,200,63),-- Mid-Grade Life Stone: level 70
(21161,8761,1,1,200,63),-- Top-Grade Life Stone: level 70
(21162,3956,1,1,-1,10942),-- Recipe: Blessed Spiritshot A
(21162,5478,1,1,-1,69011),-- Sealed Dark Crystal Leather Armor Pattern
(21162,5486,1,1,-1,48561),-- Sealed Dark Crystal Robe Fabric
(21162,57,3305,6737,0,700000),-- Adena
(21162,5297,1,1,1,24),-- Sealed Dark Crystal Leather Armor
(21162,5298,1,1,1,38),-- Sealed Dark Crystalline Leggings
(21162,5301,1,1,1,16),-- Sealed Tallum Leather Armor
(21162,5478,1,1,1,3398),-- Sealed Dark Crystal Leather Armor Pattern
(21162,5479,1,1,1,2391),-- Sealed Tallum Leather Armor Pattern
(21162,5482,1,1,1,5333),-- Sealed Dark Crystalline Leggings Design
(21162,1807,1,1,2,462),-- Recipe: Soulshot: A-Grade
(21162,1868,1,3,2,250000),-- Thread
(21162,1873,1,1,2,90909),-- Silver Nugget
(21162,1876,1,1,2,43478),-- Mithril Ore
(21162,1895,1,1,2,62500),-- Metallic Fiber
(21162,4042,1,1,2,3067),-- Enria
(21162,6036,1,1,2,7407),-- Greater Magic Haste Potion
(21162,8731,1,1,200,1838),-- Life Stone: level 70
(21162,8741,1,1,200,322),-- Mid-Grade Life Stone: level 70
(21162,8751,1,1,200,36),-- High-Grade Life Stone: level 70
(21163,729,1,1,-1,967),-- Scroll: Enchant Weapon (Grade A)
(21163,5480,1,1,-1,39041),-- Sealed Leather Armor of Nightmare Fabric
(21163,5481,1,1,-1,39041),-- Sealed Majestic Leather Armor Fabric
(21163,57,3493,7152,0,700000),-- Adena
(21163,5290,1,1,1,56),-- Sealed Dark Crystal Gloves
(21163,5291,1,1,1,56),-- Sealed Dark Crystal Boots
(21163,5295,1,1,1,56),-- Sealed Tallum Gloves
(21163,5296,1,1,1,56),-- Sealed Tallum Boots
(21163,5496,1,1,1,7532),-- Sealed Dark Crystal Boots Lining
(21163,5497,1,1,1,7532),-- Sealed Tallum Boots Lining
(21163,5508,1,1,1,7532),-- Sealed Dark Crystal Gloves Design
(21163,5509,1,1,1,7532),-- Sealed Tallum Gloves Design
(21164,5480,1,1,-1,44804),-- Sealed Leather Armor of Nightmare Fabric
(21164,5481,1,1,-1,44804),-- Sealed Majestic Leather Armor Fabric
(21164,5487,1,1,-1,44963),-- Sealed Robe of Nightmare Fabric
(21164,57,4018,8199,0,700000),-- Adena
(21164,6326,1,1,1,38),-- Sealed Majestic Necklace
(21164,6327,1,1,1,51),-- Sealed Majestic Earring
(21164,6328,1,1,1,76),-- Sealed Majestic Ring
(21164,6342,1,1,1,11135),-- Sealed Majestic Earring Gemstone
(21164,6344,1,1,1,7206),-- Sealed Majestic Necklace Beads
(21164,6346,1,1,1,16004),-- Sealed Majestic Ring Gemstone
(21164,729,1,1,2,31),-- Scroll: Enchant Weapon (Grade A)
(21164,1866,1,1,2,250000),-- Suede
(21164,1871,1,3,2,166667),-- Charcoal
(21164,1877,1,1,2,12195),-- Adamantite Nugget
(21164,4040,1,1,2,7463),-- Mold Lubricant
(21164,8732,1,1,200,2469),-- Life Stone: level 76
(21164,8742,1,1,200,875),-- Mid-Grade Life Stone: level 76
(21164,8752,1,1,200,144),-- High-Grade Life Stone: level 76
(21164,4041,1,1,2,2865); -- Mold Hardener

INSERT INTO `droplist` VALUES 
(21165,730,1,1,-1,8591),-- Scroll: Enchant Armor (Grade A)
(21165,5488,1,1,-1,46422),-- Sealed Majestic Robe Fabric
(21165,5495,1,1,-1,156540),-- Sealed Shield of Nightmare Fragment
(21165,57,4147,8467,0,700000),-- Adena
(21165,5320,1,1,1,20),-- Sealed Leather Armor of Nightmare
(21165,5323,1,1,1,20),-- Sealed Majestic Leather Armor
(21165,5480,1,1,1,3417),-- Sealed Leather Armor of Nightmare Fabric
(21165,5481,1,1,1,3417),-- Sealed Majestic Leather Armor Fabric
(21165,729,1,1,2,49),-- Scroll: Enchant Weapon (Grade A)
(21165,730,1,1,2,236),-- Scroll: Enchant Armor (Grade A)
(21165,1895,1,1,2,90909),-- Metallic Fiber
(21165,4040,1,1,2,4739),-- Mold Lubricant
(21165,4042,1,1,2,4831),-- Enria
(21165,4043,1,1,2,10204),-- Asofe
(21165,6037,1,3,2,333333),-- Waking Scroll
(21165,8732,1,1,200,2114),-- Life Stone: level 76
(21165,8742,1,1,200,272),-- Mid-Grade Life Stone: level 76
(21165,8752,1,1,200,165),-- High-Grade Life Stone: level 76
(21166,1868,1,3,-1,886728),-- Thread
(21166,1873,1,1,-1,354691),-- Silver Nugget
(21166,1898,1,1,-1,91087),-- Blue Diamond Necklace Gem
(21166,6360,53,109,0,700000),-- Blue Seal Stone
(21166,25,1,1,1,550),-- Piece Bone Breastplate
(21166,26,1,1,1,355),-- Bronze Breastplate
(21166,32,1,1,1,878),-- Piece Bone Gaiters
(21166,34,1,1,1,569),-- Bronze Gaiters
(21166,1926,1,1,1,8819),-- Piece Bone Breastplate Fragment
(21166,1928,1,1,1,12718),-- Piece Bone Gaiters Fragment
(21166,1868,1,3,2,100000),-- Thread
(21166,1873,1,1,2,41667),-- Silver Nugget
(21166,2176,1,1,2,45455),-- Recipe: Leather Boots
(21166,6037,1,3,2,62500),-- Waking Scroll
(21166,6362,23,42,4,700000),-- Red Seal Stone
(21167,1804,1,1,-1,39326),-- Recipe: Soulshot: D-Grade
(21167,1926,1,1,-1,99308),-- Piece Bone Breastplate Fragment
(21167,2151,1,2,-1,273857),-- Recipe: Necklace of Devotion
(21167,6360,72,143,0,700000),-- Blue Seal Stone
(21167,1102,1,1,1,1574),-- Tunic of Magic
(21167,1105,1,1,1,2526),-- Stockings of Magic
(21167,735,1,1,2,12500),-- Potion of Alacrity
(21167,1867,1,1,2,166667),-- Animal Skin
(21167,1872,1,1,2,166667),-- Animal Bone
(21167,1882,1,1,2,33333),-- Leather
(21167,6362,21,39,4,700000),-- Red Seal Stone
(21168,1889,1,1,-1,78778),-- Compound Braid
(21168,1899,1,1,-1,184636),-- Necklace of Devotion Chain
(21168,5549,1,1,-1,118167),-- Metallic Thread
(21168,6360,97,190,0,700000),-- Blue Seal Stone
(21168,847,1,1,1,1233),-- Red Crescent Earring
(21168,890,1,1,1,1843),-- Ring of Devotion
(21168,910,1,1,1,924),-- Necklace of Devotion
(21168,1899,1,1,1,25907),-- Necklace of Devotion Chain
(21168,1868,1,3,2,100000),-- Thread
(21168,1873,1,1,2,38462),-- Silver Nugget
(21168,1889,1,1,2,2519),-- Compound Braid
(21168,2262,1,1,2,2519),-- Recipe: Elven Bow
(21168,5549,1,1,2,7576),-- Metallic Thread
(21168,6037,1,1,2,166667),-- Waking Scroll
(21168,6362,22,41,4,700000),-- Red Seal Stone
(21169,1061,1,4,-1,881783),-- Healing Potion
(21169,1900,1,1,-1,159184),-- Enchanted Necklace Chain
(21169,2153,1,1,-1,226274),-- Recipe: Tiger\'s Eye Earring
(21169,6360,128,259,0,700000),-- Blue Seal Stone
(21169,605,1,1,1,2388),-- Leather Gauntlets
(21169,1123,1,1,1,2388),-- Blue Buckskin Boots
(21169,1864,1,3,2,125000),-- Stem
(21169,1869,1,1,2,125000),-- Iron Ore
(21169,1880,1,1,2,13514),-- Steel
(21169,2153,1,1,2,18519),-- Recipe: Tiger\'s Eye Earring
(21169,5550,1,1,2,2141),-- Durable Metal Plate
(21169,6362,24,44,4,700000),-- Red Seal Stone
(21170,1901,1,2,-1,222980),-- Tiger\'s Eye Earring Stone
(21170,1938,1,1,-1,108529),-- Mithril Banded Mail Material
(21170,2154,1,1,-1,172845),-- Recipe: Elven Earring
(21170,6360,108,211,0,700000),-- Blue Seal Stone
(21170,47,1,1,1,1127),-- Helmet
(21170,628,1,1,1,1611),-- Hoplon
(21170,1864,1,1,2,250000),-- Stem
(21170,1869,1,1,2,125000),-- Iron Ore
(21170,1880,1,1,2,12346),-- Steel
(21170,2155,1,1,2,19231),-- Recipe: Elven Ring
(21170,5550,1,1,2,1558),-- Durable Metal Plate
(21170,6035,1,1,2,9901),-- Magic Haste Potion
(21170,6361,33,63,3,700000),-- Green Seal Stone
(21170,6362,22,41,4,700000),-- Red Seal Stone
(21171,1865,1,5,-1,684631),-- Varnish
(21171,1870,1,5,-1,684631),-- Coal
(21171,1939,1,1,-1,180403),-- Mithril Banded Gaiters Material
(21171,6360,96,192,0,700000),-- Blue Seal Stone
(21171,83,1,1,1,51),-- Sword of Magic
(21171,124,1,1,1,38),-- Two-Handed Sword
(21171,127,1,1,1,38),-- Crimson Sword
(21171,130,1,1,1,38),-- Elven Sword
(21171,144,1,1,1,51),-- Sword of Occult
(21171,2026,1,1,1,1215),-- Two-Handed Sword Edge
(21171,1865,1,1,2,250000),-- Varnish
(21171,1870,1,3,2,111111),-- Coal
(21171,1874,1,1,2,14706),-- Oriharukon Ore
(21171,2191,1,1,2,32258),-- Recipe: RIP Gauntlets
(21171,6361,68,136,3,700000),-- Green Seal Stone
(21171,6362,24,44,4,700000),-- Red Seal Stone
(21172,1825,1,1,-1,172903),-- Recipe: Oriharukon
(21172,1902,1,1,-1,245252),-- Elven Earring Beads
(21172,2145,1,1,-1,172903),-- Recipe: Mithril Alloy
(21172,6360,89,178,0,700000),-- Blue Seal Stone
(21172,88,1,1,1,18),-- Morning Star
(21172,90,1,1,1,18),-- Goat Head Staff
(21172,158,1,1,1,18),-- Tarbar
(21172,169,1,1,1,18),-- Skull Breaker
(21172,172,1,1,1,18),-- Heavy Bone Club
(21172,2031,1,1,1,1267),-- Morning Star Head
(21172,2032,1,1,1,1157),-- Goat Head Staff Head
(21172,2035,1,1,1,1267),-- Tarbar Head
(21172,2036,1,1,1,1267),-- Skull Breaker Head
(21172,2037,1,1,1,1267),-- Heavy Bone Club Head
(21172,1341,60,180,2,111111),-- Bone Arrow
(21172,1866,1,1,2,142857),-- Suede
(21172,1871,2,6,2,55556),-- Charcoal
(21172,1877,1,1,2,6410),-- Adamantite Nugget
(21172,2147,1,1,2,15385),-- Recipe: Blacksmith\'s frame
(21172,6361,105,206,3,700000),-- Green Seal Stone
(21172,6362,24,45,4,700000),-- Red Seal Stone
(21173,1875,1,1,-1,170540),-- Stone of Purity
(21173,1881,1,1,-1,341080),-- Coarse Bone Powder
(21173,1905,1,1,-1,248842),-- Omen Beast\'s Eye Earring Gemstone
(21173,6360,56,111,0,700000),-- Blue Seal Stone
(21173,262,1,1,1,72),-- Scallop Jamadhr
(21173,2053,1,1,1,5311),-- Scallop Jamadhr Edge
(21173,1805,1,1,2,472),-- Recipe: Soulshot: C-Grade
(21173,1867,1,1,2,166667),-- Animal Skin
(21173,1872,1,1,2,166667),-- Animal Bone
(21173,1875,1,1,2,8621),-- Stone of Purity
(21173,1881,1,1,2,16667),-- Coarse Bone Powder
(21173,2990,1,1,2,9524),-- Recipe: Chain Boots
(21173,5164,1,1,2,440),-- Recipe: Blessed Spiritshot (C) Compressed Package (100%)
(21173,6036,1,1,2,3984),-- Greater Magic Haste Potion
(21173,6361,137,274,3,700000),-- Green Seal Stone
(21173,6362,22,41,4,700000),-- Red Seal Stone
(21174,1873,1,3,-1,654639),-- Silver Nugget
(21174,1908,1,1,-1,309962),-- Moonstone Earring Wire
(21174,2163,1,1,-1,167000),-- Recipe: Earring of Protection
(21174,6360,33,62,0,700000),-- Blue Seal Stone
(21174,281,1,1,1,68),-- Crystallized Ice Bow
(21174,2074,1,1,1,5479),-- Crystallized Ice Bow Shaft
(21174,1868,1,3,2,200000),-- Thread
(21174,1873,1,1,2,76923),-- Silver Nugget
(21174,1876,1,1,2,40000),-- Mithril Ore
(21174,1895,1,1,2,55556),-- Metallic Fiber
(21174,2164,1,1,2,13699),-- Recipe: Ring of Protection
(21174,3002,1,1,2,11905),-- Recipe: Dwarven Chain Gloves
(21174,4042,1,1,2,3509),-- Enria
(21174,6361,211,425,3,700000),-- Green Seal Stone
(21174,6362,25,45,4,700000),-- Red Seal Stone
(21175,440,1,1,-1,1053),-- Robe of Seal
(21175,1912,1,1,-1,401114),-- Ring of Protection Gemstone
(21175,4590,1,1,-1,12408),-- Greater Dye of Strength <Str+2 Dex-2>
(21175,227,1,1,1,29),-- Stiletto
(21175,242,1,1,1,29),-- Soulfire Dirk
(21175,2083,1,1,1,2740),-- Stiletto Edge
(21175,2084,1,1,1,2740),-- Soulfire Dirk Blade
(21175,951,1,1,2,423),-- Scroll: Enchant Weapon (Grade C)
(21175,1866,1,1,2,166667),-- Suede
(21175,1871,1,3,2,83333),-- Charcoal
(21175,1877,1,1,2,9346),-- Adamantite Nugget
(21175,4040,1,1,2,5714),-- Mold Lubricant
(21175,4041,1,1,2,2114),-- Mold Hardener
(21175,6037,1,3,2,250000),-- Waking Scroll
(21175,6361,268,532,3,700000),-- Green Seal Stone
(21175,6362,25,45,4,700000),-- Red Seal Stone
(21176,1872,1,11,-1,842436),-- Animal Bone
(21176,4041,1,1,-1,32965),-- Mold Hardener
(21176,4601,1,1,-1,8424),-- Greater Dye of Strength <Str+3 Con-3>
(21176,6360,133,267,0,700000),-- Blue Seal Stone
(21176,326,1,1,1,37),-- Heathen\'s Book
(21176,2110,1,1,1,4419),-- Heathen\'s Book Page
(21176,1864,4,12,2,50000),-- Stem
(21176,1865,2,6,2,38462),-- Varnish
(21176,1866,3,9,2,23256),-- Suede
(21176,1868,1,3,2,142857),-- Thread
(21176,1873,1,1,2,76923),-- Silver Nugget
(21176,2232,1,1,2,1188),-- Recipe: Drake Leather Armor
(21176,3012,1,1,2,3571),-- Recipe: Full Plate Helmet
(21176,6361,219,437,3,700000),-- Green Seal Stone
(21176,6362,11,22,4,700000),-- Red Seal Stone
(21177,1806,1,1,-1,10342),-- Recipe: Soulshot: B-Grade
(21177,1920,1,1,-1,176944),-- Necklace of Binding Chain
(21177,4594,1,1,-1,17237),-- Greater Dye of Dexterity <Dex+2 Con-2>
(21177,6360,148,301,0,700000),-- Blue Seal Stone
(21177,299,1,1,1,11),-- Orcish Poleaxe
(21177,301,1,1,1,14),-- Scorpion
(21177,303,1,1,1,14),-- Widow Maker
(21177,2113,1,1,1,1541),-- Scorpion Blade
(21177,2114,1,1,1,1541),-- Widowmaker Head
(21177,2122,1,1,1,1453),-- Orcish Poleaxe Blade
(21177,1868,1,3,2,250000),-- Thread
(21177,1873,1,1,2,100000),-- Silver Nugget
(21177,1876,1,1,2,45455),-- Mithril Ore
(21177,1895,1,1,2,58824),-- Metallic Fiber
(21177,3034,1,1,2,188),-- Recipe: Spiritshot B
(21177,4042,1,1,2,5464),-- Enria
(21177,4944,1,1,2,1506),-- Recipe: Avadon Breastplate (60%)
(21177,4968,1,1,2,275),-- Recipe: Kris (60%)
(21177,6361,314,633,3,700000),-- Green Seal Stone
(21177,6362,13,27,4,700000),-- Red Seal Stone
(21178,947,1,1,-1,2218),-- Scroll: Enchant Weapon (Grade B)
(21178,4129,1,1,-1,59998),-- Recipe: Earrings of Black Ore
(21178,4610,1,1,-1,12320),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(21178,6360,73,146,0,700000),-- Blue Seal Stone
(21178,2384,1,1,1,35),-- Zubei\'s Leather Shirt
(21178,2388,1,1,1,57),-- Zubei\'s Leather Gaiters
(21178,2390,1,1,1,24),-- Avadon Leather Armor
(21178,4066,1,1,1,6434),-- Zubei\'s Leather Shirt Fabric
(21178,4067,1,1,1,9768),-- Zubei\'s Leather Gaiter Texture
(21178,4068,1,1,1,4943),-- Avadon Leather Armor Lining
(21178,948,1,1,2,1071),-- Scroll: Enchant Armor (Grade B)
(21178,1867,1,1,2,250000),-- Animal Skin
(21178,1872,1,3,2,142857),-- Animal Bone
(21178,1875,1,1,2,12048),-- Stone of Purity
(21178,1881,1,1,2,27778),-- Coarse Bone Powder
(21178,4041,1,1,2,1565),-- Mold Hardener
(21178,4162,1,1,2,988),-- Recipe: Tunic of Doom
(21178,5003,1,1,2,82),-- Recipe: Art of Battle Axe (60%)
(21178,6361,391,782,3,700000),-- Green Seal Stone
(21178,6362,13,26,4,700000),-- Red Seal Stone
(21179,947,1,1,-1,2132),-- Scroll: Enchant Weapon (Grade B)
(21179,1872,1,15,-1,888172),-- Animal Bone
(21179,4613,1,1,-1,7350),-- Greater Dye of Strength <Str+4 Con-4>
(21179,6360,7,15,0,700000),-- Blue Seal Stone
(21179,357,1,1,1,19),-- Zubei\'s Breastplate
(21179,383,1,1,1,31),-- Zubei\'s Gaiters
(21179,2376,1,1,1,19),-- Avadon Breastplate
(21179,2379,1,1,1,31),-- Avadon Gaiters
(21179,4056,1,1,1,3561),-- Zubei\'s Breastplate Part
(21179,4057,1,1,1,5504),-- Zubei\'s Gaiter Material
(21179,4064,1,1,1,3561),-- Avadon Breastplate Part
(21179,4065,1,1,1,5504),-- Avadon Gaiters Material
(21179,1343,20,60,2,166667),-- Silver Arrow
(21179,1867,2,6,2,71429),-- Animal Skin
(21179,1872,1,3,2,100000),-- Animal Bone
(21179,1875,1,1,2,13699),-- Stone of Purity
(21179,1881,1,1,2,29412),-- Coarse Bone Powder
(21179,4041,1,1,2,1916),-- Mold Hardener
(21179,4155,1,1,2,667),-- Recipe: Blue Wolf Breastplate
(21179,6361,415,830,3,700000),-- Green Seal Stone
(21179,6362,11,22,4,700000),-- Red Seal Stone
(21180,4052,1,1,-1,266218),-- Earring of Black Ore Piece
(21180,4053,1,1,-1,322303),-- Ring of Black Ore Gemstone
(21180,5230,1,1,-1,10026),-- Recipe: Mithril Arrow (100%)
(21180,2398,1,1,1,24),-- Blue Wolf Tunic
(21180,2399,1,1,1,24),-- Tunic of Doom
(21180,2403,1,1,1,37),-- Blue Wolf Stockings
(21180,2404,1,1,1,37),-- Stockings of Doom
(21180,4084,1,1,1,4824),-- Blue Wolf Tunic Fabric
(21180,4085,1,1,1,4824),-- Tunic of Doom Pattern
(21180,4086,1,1,1,6513),-- Blue Wolf Stockings Pattern
(21180,4087,1,1,1,6513),-- Stockings of Doom Pattern
(21180,1830,1,1,2,142857),-- Scroll of Escape: Castle
(21180,1866,1,3,2,76923),-- Suede
(21180,1871,1,3,2,200000),-- Charcoal
(21180,1877,1,1,2,13333),-- Adamantite Nugget
(21180,4040,1,1,2,7407),-- Mold Lubricant
(21180,4041,1,1,2,2967),-- Mold Hardener
(21180,6361,574,1144,3,700000),-- Green Seal Stone
(21180,6362,14,28,4,700000),-- Red Seal Stone
(21181,4041,1,1,-1,69372),-- Mold Hardener
(21181,4054,1,2,-1,157136),-- Necklace of Black Ore Beads
(21181,5230,1,1,-1,10637),-- Recipe: Mithril Arrow (100%)
(21181,864,1,1,1,86),-- Earring of Black Ore
(21181,895,1,1,1,128),-- Ring of Black Ore
(21181,926,1,1,1,64),-- Necklace of Black Ore
(21181,4052,1,1,1,13904),-- Earring of Black Ore Piece
(21181,4053,1,1,1,16833),-- Ring of Black Ore Gemstone
(21181,4054,1,1,1,7735),-- Necklace of Black Ore Beads
(21181,1867,1,3,2,142857),-- Animal Skin
(21181,1872,1,3,2,250000),-- Animal Bone
(21181,1875,1,1,2,22727),-- Stone of Purity
(21181,1881,1,1,2,43478),-- Coarse Bone Powder
(21181,4041,1,1,2,2695),-- Mold Hardener
(21181,5230,1,1,2,408),-- Recipe: Mithril Arrow (100%)
(21181,5276,1,1,2,453),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21181,6361,578,1160,3,700000),-- Green Seal Stone
(21181,6362,14,27,4,700000),-- Red Seal Stone
(21182,1876,1,3,-1,721648),-- Mithril Ore
(21182,5478,1,2,-1,60684),-- Sealed Dark Crystal Leather Armor Pattern
(21182,5290,1,1,1,46),-- Sealed Dark Crystal Gloves
(21182,5291,1,1,1,46),-- Sealed Dark Crystal Boots
(21182,5295,1,1,1,46),-- Sealed Tallum Gloves
(21182,5296,1,1,1,46),-- Sealed Tallum Boots
(21182,5496,1,1,1,6248),-- Sealed Dark Crystal Boots Lining
(21182,5497,1,1,1,6248),-- Sealed Tallum Boots Lining
(21182,5508,1,1,1,6248),-- Sealed Dark Crystal Gloves Design
(21182,5509,1,1,1,6248),-- Sealed Tallum Gloves Design
(21182,1344,30,90,2,125000),-- Mithril Arrow
(21182,1868,4,12,2,76923),-- Thread
(21182,1873,1,1,2,125000),-- Silver Nugget
(21182,1876,1,1,2,58824),-- Mithril Ore
(21182,1895,1,1,2,83333),-- Metallic Fiber
(21182,4042,1,1,2,5556),-- Enria
(21182,5156,1,1,2,435),-- Recipe: Soulshot (A) Compressed Package (100%)
(21182,6361,512,1023,3,700000),-- Green Seal Stone
(21182,6362,11,22,4,700000),-- Red Seal Stone
(21183,1807,1,1,-1,13420),-- Recipe: Soulshot: A-Grade
(21183,3035,1,1,-1,13420),-- Recipe: Spiritshot A
(21183,5479,1,1,-1,59556),-- Sealed Tallum Leather Armor Pattern
(21183,5289,1,1,1,57),-- Sealed Dark Crystal Helmet
(21183,5292,1,1,1,82),-- Sealed Dark Crystal Shield
(21183,5294,1,1,1,57),-- Sealed Tallum Helmet
(21183,5494,1,1,1,10406),-- Sealed Dark Crystal Shield Fragment
(21183,5525,1,1,1,7660),-- Sealed Dark Crystal Helmet Design
(21183,5526,1,1,1,7660),-- Sealed Tallum Helm Design
(21183,1894,1,1,2,17857),-- Crafted Leather
(21183,3956,1,1,2,481),-- Recipe: Blessed Spiritshot A
(21183,4039,1,1,2,17857),-- Mold Glue
(21183,4041,1,1,2,4149),-- Mold Hardener
(21183,4044,1,1,2,18868),-- Thons
(21183,6361,687,1380,3,700000),-- Green Seal Stone
(21183,6362,14,28,4,700000),-- Red Seal Stone
(21184,729,1,1,-1,1173),-- Scroll: Enchant Weapon (Grade A)
(21184,1895,1,5,-1,753968),-- Metallic Fiber
(21184,5480,1,1,-1,47365),-- Sealed Leather Armor of Nightmare Fabric
(21184,5304,1,1,1,30),-- Sealed Tallum Tunic
(21184,5305,1,1,1,48),-- Sealed Tallum Stockings
(21184,5308,1,1,1,21),-- Sealed Dark Crystal Robe
(21184,5485,1,1,1,4370),-- Sealed Tallum Tunic Texture
(21184,5486,1,1,1,3075),-- Sealed Dark Crystal Robe Fabric
(21184,5489,1,1,1,6858),-- Sealed Tallum Stockings Fabric
(21184,6361,700,1400,3,700000),-- Green Seal Stone
(21184,6362,14,27,4,700000),-- Red Seal Stone
(21185,1875,1,2,-1,629418),-- Stone of Purity
(21185,5481,1,1,-1,42365),-- Sealed Majestic Leather Armor Fabric
(21185,5487,1,1,-1,42515),-- Sealed Robe of Nightmare Fabric
(21185,5304,1,1,1,27),-- Sealed Tallum Tunic
(21185,5305,1,1,1,43),-- Sealed Tallum Stockings
(21185,5308,1,1,1,19),-- Sealed Dark Crystal Robe
(21185,5485,1,1,1,3909),-- Sealed Tallum Tunic Texture
(21185,5486,1,1,1,2750),-- Sealed Dark Crystal Robe Fabric
(21185,5489,1,1,1,6135),-- Sealed Tallum Stockings Fabric
(21185,6361,612,1227,3,700000),-- Green Seal Stone
(21185,6362,11,22,4,700000),-- Red Seal Stone
(21186,730,1,1,-1,10564),-- Scroll: Enchant Armor (Grade A)
(21186,5480,1,1,-1,56883),-- Sealed Leather Armor of Nightmare Fabric
(21186,5488,1,1,-1,57085),-- Sealed Majestic Robe Fabric
(21186,5312,1,1,1,48),-- Sealed Helm of Nightmare
(21186,5315,1,1,1,68),-- Sealed Shield of Nightmare
(21186,5317,1,1,1,48),-- Sealed Majestic Circlet
(21186,5495,1,1,1,9477),-- Sealed Shield of Nightmare Fragment
(21186,5527,1,1,1,7462),-- Sealed Helm of Nightmare Design
(21186,5528,1,1,1,7462),-- Sealed Majestic Circlet Design
(21186,729,1,1,2,173),-- Scroll: Enchant Weapon (Grade A)
(21186,1866,1,3,2,142857),-- Suede
(21186,1871,4,12,2,76923),-- Charcoal
(21186,1877,1,1,2,23810),-- Adamantite Nugget
(21186,4040,1,1,2,15152),-- Mold Lubricant
(21186,4041,1,1,2,4975),-- Mold Hardener
(21186,6361,805,1610,3,700000),-- Green Seal Stone
(21186,6362,14,28,4,700000),-- Red Seal Stone
(21187,1868,1,3,-1,886728),-- Thread
(21187,1873,1,1,-1,354691),-- Silver Nugget
(21187,1898,1,1,-1,91087),-- Blue Diamond Necklace Gem
(21187,6360,53,109,0,700000),-- Blue Seal Stone
(21187,25,1,1,1,550),-- Piece Bone Breastplate
(21187,26,1,1,1,355),-- Bronze Breastplate
(21187,32,1,1,1,878),-- Piece Bone Gaiters
(21187,34,1,1,1,569),-- Bronze Gaiters
(21187,1926,1,1,1,8819),-- Piece Bone Breastplate Fragment
(21187,1928,1,1,1,12718),-- Piece Bone Gaiters Fragment
(21187,1868,1,1,2,200000),-- Thread
(21187,1873,1,1,2,43478),-- Silver Nugget
(21187,2176,1,1,2,41667),-- Recipe: Leather Boots
(21187,6035,1,1,2,6897),-- Magic Haste Potion
(21187,6362,23,42,4,700000),-- Red Seal Stone
(21188,1804,1,1,-1,39326),-- Recipe: Soulshot: D-Grade
(21188,1926,1,1,-1,99308),-- Piece Bone Breastplate Fragment
(21188,2151,1,1,-1,273857),-- Recipe: Necklace of Devotion
(21188,6360,72,143,0,700000),-- Blue Seal Stone
(21188,1102,1,1,1,1574),-- Tunic of Magic
(21188,1105,1,1,1,2526),-- Stockings of Magic
(21188,735,1,1,2,6494),-- Potion of Alacrity
(21188,1867,1,1,2,200000),-- Animal Skin
(21188,1872,1,1,2,166667),-- Animal Bone
(21188,1882,1,1,2,32258),-- Leather
(21188,6362,21,39,4,700000),-- Red Seal Stone
(21189,1889,1,1,-1,78778),-- Compound Braid
(21189,1899,1,1,-1,184636),-- Necklace of Devotion Chain
(21189,5549,1,1,-1,118167),-- Metallic Thread
(21189,6360,97,190,0,700000),-- Blue Seal Stone
(21189,847,1,1,1,1233),-- Red Crescent Earring
(21189,890,1,1,1,1843),-- Ring of Devotion
(21189,910,1,1,1,924),-- Necklace of Devotion
(21189,1899,1,1,1,25907),-- Necklace of Devotion Chain
(21189,1868,1,1,2,200000),-- Thread
(21189,1873,1,1,2,43478),-- Silver Nugget
(21189,1889,1,1,2,7407),-- Compound Braid
(21189,2262,1,1,2,3704),-- Recipe: Elven Bow
(21189,5549,1,1,2,7407),-- Metallic Thread
(21189,6362,22,41,4,700000),-- Red Seal Stone
(21190,1061,1,1,-1,881783),-- Healing Potion
(21190,1900,1,1,-1,159184),-- Enchanted Necklace Chain
(21190,2153,1,1,-1,226274),-- Recipe: Tiger\'s Eye Earring
(21190,6360,128,259,0,700000),-- Blue Seal Stone
(21190,605,1,1,1,2388),-- Leather Gauntlets
(21190,1123,1,1,1,2388),-- Blue Buckskin Boots
(21190,1864,1,1,2,200000),-- Stem
(21190,1869,1,1,2,111111),-- Iron Ore
(21190,1880,1,1,2,10870),-- Steel
(21190,2153,1,1,2,17241),-- Recipe: Tiger\'s Eye Earring
(21190,5550,1,1,2,1558),-- Durable Metal Plate
(21190,6035,1,1,2,8696),-- Magic Haste Potion
(21190,6362,24,44,4,700000),-- Red Seal Stone
(21191,1901,1,1,-1,222980),-- Tiger\'s Eye Earring Stone
(21191,1938,1,1,-1,108529),-- Mithril Banded Mail Material
(21191,2154,1,1,-1,172845),-- Recipe: Elven Earring
(21191,6360,108,211,0,700000),-- Blue Seal Stone
(21191,47,1,1,1,1127),-- Helmet
(21191,628,1,1,1,1611),-- Hoplon
(21191,1864,1,3,2,142857),-- Stem
(21191,1869,1,1,2,166667),-- Iron Ore
(21191,1880,1,1,2,16667),-- Steel
(21191,2155,1,1,2,24390),-- Recipe: Elven Ring
(21191,5550,1,1,2,2119),-- Durable Metal Plate
(21191,6361,33,63,3,700000),-- Green Seal Stone
(21191,6362,22,41,4,700000),-- Red Seal Stone
(21192,1865,1,5,-1,684631),-- Varnish
(21192,1870,1,5,-1,684631),-- Coal
(21192,1939,1,1,-1,180403),-- Mithril Banded Gaiters Material
(21192,6360,96,192,0,700000),-- Blue Seal Stone
(21192,83,1,1,1,51),-- Sword of Magic
(21192,124,1,1,1,38),-- Two-Handed Sword
(21192,127,1,1,1,38),-- Crimson Sword
(21192,130,1,1,1,38),-- Elven Sword
(21192,144,1,1,1,51),-- Sword of Occult
(21192,2026,1,1,1,2191),-- Two-Handed Sword Edge
(21192,1865,1,1,2,166667),-- Varnish
(21192,1870,1,1,2,166667),-- Coal
(21192,1874,1,1,2,10417),-- Oriharukon Ore
(21192,2191,1,1,2,21277),-- Recipe: RIP Gauntlets
(21192,8317,1,1,2,1481),-- Recipe: Titan Hammer (100%)
(21192,6361,68,136,3,700000),-- Green Seal Stone
(21192,6362,24,44,4,700000),-- Red Seal Stone
(21193,1825,1,1,-1,172903),-- Recipe: Oriharukon
(21193,1902,1,1,-1,245252),-- Elven Earring Beads
(21193,2145,1,1,-1,172903),-- Recipe: Mithril Alloy
(21193,6360,89,178,0,700000),-- Blue Seal Stone
(21193,88,1,1,1,18),-- Morning Star
(21193,90,1,1,1,18),-- Goat Head Staff
(21193,158,1,1,1,18),-- Tarbar
(21193,169,1,1,1,18),-- Skull Breaker
(21193,172,1,1,1,18),-- Heavy Bone Club
(21193,2031,1,1,1,1267),-- Morning Star Head
(21193,2032,1,1,1,1157),-- Goat Head Staff Head
(21193,2035,1,1,1,1267),-- Tarbar Head
(21193,2036,1,1,1,1267),-- Skull Breaker Head
(21193,2037,1,1,1,1267),-- Heavy Bone Club Head
(21193,1866,1,1,2,166667),-- Suede
(21193,1871,1,3,2,125000),-- Charcoal
(21193,1877,1,1,2,10938),-- Adamantite Nugget
(21193,2147,1,1,2,16949),-- Recipe: Blacksmith\'s frame
(21193,6361,105,206,3,700000),-- Green Seal Stone
(21193,6362,24,45,4,700000),-- Red Seal Stone
(21194,1875,1,1,-1,170540),-- Stone of Purity
(21194,1881,1,1,-1,341080),-- Coarse Bone Powder
(21194,1905,1,1,-1,248842),-- Omen Beast\'s Eye Earring Gemstone
(21194,6360,56,111,0,700000),-- Blue Seal Stone
(21194,262,1,1,1,72),-- Scallop Jamadhr
(21194,2053,1,1,1,5311),-- Scallop Jamadhr Edge
(21194,1805,1,1,2,466),-- Recipe: Soulshot: C-Grade
(21194,1867,2,6,2,52632),-- Animal Skin
(21194,1872,1,3,2,66667),-- Animal Bone
(21194,1875,1,1,2,11364),-- Stone of Purity
(21194,1881,1,1,2,19231),-- Coarse Bone Powder
(21194,2994,1,1,2,13514),-- Recipe: Boots of Seal
(21194,6037,1,3,2,142857),-- Waking Scroll
(21194,6361,137,274,3,700000),-- Green Seal Stone
(21194,6362,22,41,4,700000),-- Red Seal Stone
(21195,440,1,1,-1,926),-- Robe of Seal
(21195,1908,1,2,-1,309962),-- Moonstone Earring Wire
(21195,6360,33,62,0,700000),-- Blue Seal Stone
(21195,281,1,1,1,68),-- Crystallized Ice Bow
(21195,2074,1,1,1,5479),-- Crystallized Ice Bow Shaft
(21195,1868,1,3,2,200000),-- Thread
(21195,1873,1,1,2,76923),-- Silver Nugget
(21195,1876,1,1,2,37037),-- Mithril Ore
(21195,1895,1,1,2,52632),-- Metallic Fiber
(21195,2164,1,1,2,13158),-- Recipe: Ring of Protection
(21195,2228,1,1,2,5263),-- Recipe: Shining Circlet
(21195,4042,1,1,2,3390),-- Enria
(21195,6036,1,1,2,5525),-- Greater Magic Haste Potion
(21195,6361,211,425,3,700000),-- Green Seal Stone
(21195,6362,25,45,4,700000),-- Red Seal Stone
(21196,1911,1,1,-1,276857),-- Earring of Protection Gemstone
(21196,2166,1,1,-1,126181),-- Recipe: Earrings of Binding (100%)
(21196,2167,1,1,-1,189915),-- Recipe: Ring of Ages
(21196,227,1,1,1,29),-- Stiletto
(21196,242,1,1,1,29),-- Soulfire Dirk
(21196,2083,1,1,1,2740),-- Stiletto Edge
(21196,2084,1,1,1,2740),-- Soulfire Dirk Blade
(21196,951,1,1,2,500),-- Scroll: Enchant Weapon (Grade C)
(21196,1866,1,1,2,166667),-- Suede
(21196,1871,1,1,2,250000),-- Charcoal
(21196,1877,1,1,2,13889),-- Adamantite Nugget
(21196,4040,1,1,2,4444),-- Mold Lubricant
(21196,4041,1,1,2,2967),-- Mold Hardener
(21196,6361,268,532,3,700000),-- Green Seal Stone
(21196,6362,25,45,4,700000),-- Red Seal Stone
(21197,1872,1,11,-1,917243),-- Animal Bone
(21197,4041,1,1,-1,35892),-- Mold Hardener
(21197,4603,1,1,-1,9172),-- Greater Dye of Constitution <Con+3 Str-3>
(21197,6360,145,291,0,700000),-- Blue Seal Stone
(21197,326,1,1,1,40),-- Heathen\'s Book
(21197,2110,1,1,1,4811),-- Heathen\'s Book Page
(21197,1867,1,3,2,111111),-- Animal Skin
(21197,1869,2,6,2,62500),-- Iron Ore
(21197,1870,2,6,2,50000),-- Coal
(21197,1871,3,9,2,43478),-- Charcoal
(21197,1872,7,21,2,23810),-- Animal Bone
(21197,2232,1,1,2,1235),-- Recipe: Drake Leather Armor
(21197,6361,238,476,3,700000),-- Green Seal Stone
(21197,6362,12,24,4,700000),-- Red Seal Stone
(21198,1806,1,1,-1,10342),-- Recipe: Soulshot: B-Grade
(21198,1920,1,1,-1,176944),-- Necklace of Binding Chain
(21198,4593,1,1,-1,17237),-- Greater Dye of Dexterity <Dex+2 Str-2>
(21198,6360,148,301,0,700000),-- Blue Seal Stone
(21198,299,1,1,1,11),-- Orcish Poleaxe
(21198,301,1,1,1,14),-- Scorpion
(21198,303,1,1,1,14),-- Widow Maker
(21198,2113,1,1,1,1541),-- Scorpion Blade
(21198,2114,1,1,1,1541),-- Widowmaker Head
(21198,2122,1,1,1,1453),-- Orcish Poleaxe Blade
(21198,1868,1,3,2,200000),-- Thread
(21198,1873,1,1,2,90909),-- Silver Nugget
(21198,1876,1,1,2,41667),-- Mithril Ore
(21198,1895,1,1,2,83333),-- Metallic Fiber
(21198,3034,1,1,2,478),-- Recipe: Spiritshot B
(21198,4042,1,1,2,3610),-- Enria
(21198,4944,1,1,2,1148),-- Recipe: Avadon Breastplate (60%)
(21198,4968,1,1,2,275),-- Recipe: Kris (60%)
(21198,6361,314,633,3,700000),-- Green Seal Stone
(21198,6362,13,27,4,700000),-- Red Seal Stone
(21199,947,1,1,-1,2218),-- Scroll: Enchant Weapon (Grade B)
(21199,4129,1,1,-1,59998),-- Recipe: Earrings of Black Ore
(21199,4611,1,1,-1,12320),-- Greater Dye of Wit <Wit+3 Int-3>
(21199,6360,73,146,0,700000),-- Blue Seal Stone
(21199,2384,1,1,1,35),-- Zubei\'s Leather Shirt
(21199,2388,1,1,1,57),-- Zubei\'s Leather Gaiters
(21199,2390,1,1,1,24),-- Avadon Leather Armor
(21199,4066,1,1,1,6434),-- Zubei\'s Leather Shirt Fabric
(21199,4067,1,1,1,9768),-- Zubei\'s Leather Gaiter Texture
(21199,4068,1,1,1,4943),-- Avadon Leather Armor Lining
(21199,948,1,1,2,1422),-- Scroll: Enchant Armor (Grade B)
(21199,1867,1,1,2,250000),-- Animal Skin
(21199,1872,1,3,2,142857),-- Animal Bone
(21199,1875,1,1,2,15873),-- Stone of Purity
(21199,1881,1,1,2,27778),-- Coarse Bone Powder
(21199,4041,1,1,2,2070),-- Mold Hardener
(21199,4163,1,1,2,1035),-- Recipe: Blue Wolf Stockings
(21199,8311,1,1,2,259),-- Recipe: Kaim Vanul\'s Bones (100%)
(21199,6361,391,782,3,700000),-- Green Seal Stone
(21199,6362,13,26,4,700000),-- Red Seal Stone
(21200,4049,1,1,-1,304340),-- Adamantite Earring Gemstone
(21200,4129,1,1,-1,62795),-- Recipe: Earrings of Black Ore
(21200,4613,1,1,-1,8003),-- Greater Dye of Strength <Str+4 Con-4>
(21200,6360,8,16,0,700000),-- Blue Seal Stone
(21200,357,1,1,1,21),-- Zubei\'s Breastplate
(21200,383,1,1,1,33),-- Zubei\'s Gaiters
(21200,2376,1,1,1,21),-- Avadon Breastplate
(21200,2379,1,1,1,33),-- Avadon Gaiters
(21200,4056,1,1,1,3878),-- Zubei\'s Breastplate Part
(21200,4057,1,1,1,5993),-- Zubei\'s Gaiter Material
(21200,4064,1,1,1,3878),-- Avadon Breastplate Part
(21200,4065,1,1,1,5993),-- Avadon Gaiters Material
(21200,1343,20,60,2,166666),-- Silver Arrow
(21200,1867,2,6,2,76923),-- Animal Skin
(21200,1872,1,3,2,100000),-- Animal Bone
(21200,1875,1,1,2,15625),-- Stone of Purity
(21200,1881,1,1,2,31250),-- Coarse Bone Powder
(21200,4041,1,1,2,2703),-- Mold Hardener
(21200,4155,1,1,2,541),-- Recipe: Blue Wolf Breastplate
(21200,6361,452,904,3,700000),-- Green Seal Stone
(21200,6362,12,24,4,700000),-- Red Seal Stone
(21201,4052,1,1,-1,266218),-- Earring of Black Ore Piece
(21201,4053,1,1,-1,322303),-- Ring of Black Ore Gemstone
(21201,5230,1,1,-1,10026),-- Recipe: Mithril Arrow (100%)
(21201,2398,1,1,1,24),-- Blue Wolf Tunic
(21201,2399,1,1,1,24),-- Tunic of Doom
(21201,2403,1,1,1,37),-- Blue Wolf Stockings
(21201,2404,1,1,1,37),-- Stockings of Doom
(21201,4084,1,1,1,4824),-- Blue Wolf Tunic Fabric
(21201,4085,1,1,1,4824),-- Tunic of Doom Pattern
(21201,4086,1,1,1,6513),-- Blue Wolf Stockings Pattern
(21201,4087,1,1,1,6513),-- Stockings of Doom Pattern
(21201,1830,1,1,2,125000),-- Scroll of Escape: Castle
(21201,1866,1,3,2,76923),-- Suede
(21201,1871,1,3,2,166667),-- Charcoal
(21201,1877,1,1,2,14286),-- Adamantite Nugget
(21201,4040,1,1,2,10000),-- Mold Lubricant
(21201,4041,1,1,2,1348),-- Mold Hardener
(21201,6036,1,1,2,9174),-- Greater Magic Haste Potion
(21201,6361,574,1144,3,700000),-- Green Seal Stone
(21201,6362,14,28,4,700000),-- Red Seal Stone
(21202,4041,1,1,-1,69372),-- Mold Hardener
(21202,4054,1,1,-1,157136),-- Necklace of Black Ore Beads
(21202,5230,1,1,-1,10637),-- Recipe: Mithril Arrow (100%)
(21202,864,1,1,1,86),-- Earring of Black Ore
(21202,895,1,1,1,128),-- Ring of Black Ore
(21202,926,1,1,1,64),-- Necklace of Black Ore
(21202,4052,1,1,1,13904),-- Earring of Black Ore Piece
(21202,4053,1,1,1,16833),-- Ring of Black Ore Gemstone
(21202,4054,1,1,1,7735),-- Necklace of Black Ore Beads
(21202,1867,1,3,2,166667),-- Animal Skin
(21202,1872,4,12,2,66667),-- Animal Bone
(21202,1875,1,1,2,25000),-- Stone of Purity
(21202,1881,1,1,2,52632),-- Coarse Bone Powder
(21202,4041,1,1,2,2915),-- Mold Hardener
(21202,5230,1,1,2,525),-- Recipe: Mithril Arrow (100%)
(21202,6361,578,1160,3,700000),-- Green Seal Stone
(21202,6362,14,27,4,700000),-- Red Seal Stone
(21203,1876,1,3,-1,785729),-- Mithril Ore
(21203,4042,1,1,-1,130955),-- Enria
(21203,5478,1,1,-1,66072),-- Sealed Dark Crystal Leather Armor Pattern
(21203,5290,1,1,1,51),-- Sealed Dark Crystal Gloves
(21203,5291,1,1,1,51),-- Sealed Dark Crystal Boots
(21203,5295,1,1,1,51),-- Sealed Tallum Gloves
(21203,5296,1,1,1,51),-- Sealed Tallum Boots
(21203,5496,1,1,1,6802),-- Sealed Dark Crystal Boots Lining
(21203,5497,1,1,1,6802),-- Sealed Tallum Boots Lining
(21203,5508,1,1,1,6802),-- Sealed Dark Crystal Gloves Design
(21203,5509,1,1,1,6802),-- Sealed Tallum Gloves Design
(21203,1344,150,450,2,27778),-- Mithril Arrow
(21203,1868,4,12,2,83333),-- Thread
(21203,1873,1,1,2,125000),-- Silver Nugget
(21203,1876,1,1,2,62500),-- Mithril Ore
(21203,1895,1,1,2,100000),-- Metallic Fiber
(21203,4042,1,1,2,5236),-- Enria
(21203,5156,1,1,2,158),-- Recipe: Soulshot (A) Compressed Package (100%)
(21203,6361,557,1114,3,700000),-- Green Seal Stone
(21203,6362,12,24,4,700000),-- Red Seal Stone
(21204,1807,1,1,-1,13420),-- Recipe: Soulshot: A-Grade
(21204,3035,1,1,-1,13420),-- Recipe: Spiritshot A
(21204,5479,1,1,-1,59556),-- Sealed Tallum Leather Armor Pattern
(21204,5289,1,1,1,57),-- Sealed Dark Crystal Helmet
(21204,5292,1,1,1,82),-- Sealed Dark Crystal Shield
(21204,5294,1,1,1,57),-- Sealed Tallum Helmet
(21204,5494,1,1,1,10406),-- Sealed Dark Crystal Shield Fragment
(21204,5525,1,1,1,7660),-- Sealed Dark Crystal Helmet Design
(21204,5526,1,1,1,7660),-- Sealed Tallum Helm Design
(21204,1894,1,1,2,29412),-- Crafted Leather
(21204,3956,1,1,2,328),-- Recipe: Blessed Spiritshot A
(21204,4039,1,1,2,15625),-- Mold Glue
(21204,4041,1,1,2,5405),-- Mold Hardener
(21204,4044,1,1,2,17241),-- Thons
(21204,6361,687,1380,3,700000),-- Green Seal Stone
(21204,6362,14,28,4,700000),-- Red Seal Stone
(21205,729,1,1,-1,1173),-- Scroll: Enchant Weapon (Grade A)
(21205,1895,1,5,-1,753968),-- Metallic Fiber
(21205,5480,1,1,-1,47365),-- Sealed Leather Armor of Nightmare Fabric
(21205,5297,1,1,1,30),-- Sealed Dark Crystal Leather Armor
(21205,5298,1,1,1,48),-- Sealed Dark Crystalline Leggings
(21205,5301,1,1,1,21),-- Sealed Tallum Leather Armor
(21205,5478,1,1,1,4370),-- Sealed Dark Crystal Leather Armor Pattern
(21205,5479,1,1,1,3075),-- Sealed Tallum Leather Armor Pattern
(21205,5482,1,1,1,6858),-- Sealed Dark Crystalline Leggings Design
(21205,730,1,1,2,464),-- Scroll: Enchant Armor (Grade A)
(21205,1895,1,1,2,200000),-- Metallic Fiber
(21205,4040,1,1,2,14085),-- Mold Lubricant
(21205,4042,1,1,2,11628),-- Enria
(21205,4043,1,1,2,21277),-- Asofe
(21205,6361,700,1400,3,700000),-- Green Seal Stone
(21205,6362,14,27,4,700000),-- Red Seal Stone
(21206,1875,1,1,-1,685308),-- Stone of Purity
(21206,5481,1,1,-1,46127),-- Sealed Majestic Leather Armor Fabric
(21206,5487,1,1,-1,46290),-- Sealed Robe of Nightmare Fabric
(21206,5287,1,1,1,22),-- Sealed Dark Crystal breastplate
(21206,5288,1,1,1,36),-- Sealed Dark Crystal Gaiters
(21206,5293,1,1,1,15),-- Sealed Tallum Plate Armor
(21206,5520,1,1,1,3264),-- Sealed Dark Crystal Breastplate Pattern
(21206,5521,1,1,1,2304),-- Sealed Tallum Plate Armor Pattern
(21206,5524,1,1,1,4989),-- Sealed Dark Crystal Gaiters Pattern
(21206,6361,666,1336,3,700000),-- Green Seal Stone
(21206,6362,12,24,4,700000),-- Red Seal Stone
(21207,730,1,1,-1,10564),-- Scroll: Enchant Armor (Grade A)
(21207,5480,1,1,-1,56883),-- Sealed Leather Armor of Nightmare Fabric
(21207,5488,1,1,-1,57085),-- Sealed Majestic Robe Fabric
(21207,5312,1,1,1,48),-- Sealed Helm of Nightmare
(21207,5315,1,1,1,68),-- Sealed Shield of Nightmare
(21207,5317,1,1,1,48),-- Sealed Majestic Circlet
(21207,5495,1,1,1,9477),-- Sealed Shield of Nightmare Fragment
(21207,5527,1,1,1,7462),-- Sealed Helm of Nightmare Design
(21207,5528,1,1,1,7462),-- Sealed Majestic Circlet Design
(21207,729,1,1,2,973),-- Scroll: Enchant Weapon (Grade A)
(21207,1866,1,3,2,125000),-- Suede
(21207,1871,4,12,2,71429),-- Charcoal
(21207,1877,1,1,2,31250),-- Adamantite Nugget
(21207,4040,1,1,2,12658),-- Mold Lubricant
(21207,4041,1,1,2,4065),-- Mold Hardener
(21207,6361,805,1610,3,700000),-- Green Seal Stone
(21207,6362,14,28,4,700000),-- Red Seal Stone
(21208,734,1,1,-1,131730),-- Haste Potion
(21208,1871,1,2,-1,790380),-- Charcoal
(21208,1897,1,1,-1,188186),-- Necklace of Wisdom Chain
(21208,57,365,635,0,700000),-- Adena
(21208,38,1,1,1,4599),-- Low Boots
(21208,50,1,1,1,4599),-- Leather Gloves
(21208,1925,1,1,1,102062),-- Leather Gloves Lining
(21208,734,1,1,2,20833),-- Haste Potion
(21208,1866,1,1,2,76923),-- Suede
(21208,1871,1,1,2,125000),-- Charcoal
(21209,1804,1,1,-1,36013),-- Recipe: Soulshot: D-Grade
(21209,1898,1,1,-1,92484),-- Blue Diamond Necklace Gem
(21209,2151,1,1,-1,250789),-- Recipe: Necklace of Devotion
(21209,57,409,734,0,700000),-- Adena
(21209,625,1,1,1,4124),-- Bone Shield
(21209,1148,1,1,1,2891),-- Hard Leather Helmet
(21209,734,1,1,2,18182),-- Haste Potion
(21209,1865,1,1,2,142857),-- Varnish
(21209,1870,1,1,2,125000),-- Coal
(21210,1866,1,2,-1,733287),-- Suede
(21210,1877,1,1,-1,43997),-- Adamantite Nugget
(21210,1899,1,1,-1,171864),-- Necklace of Devotion Chain
(21210,57,484,907,0,700000),-- Adena
(21210,123,1,1,1,76),-- Saber
(21210,2017,1,1,1,2051),-- Saber Blade
(21210,5285,1,1,1,76),-- Heavy Sword
(21210,5531,1,1,1,2592),-- Heavy Sword Edge
(21210,1866,1,1,2,90909),-- Suede
(21210,1871,1,1,2,125000),-- Charcoal
(21210,1877,1,1,2,4975),-- Adamantite Nugget
(21210,2182,1,1,2,14706),-- Recipe: Scale Gaiters
(21211,1931,1,1,-1,150011),-- Leather Boot Lining
(21211,1932,1,1,-1,182156),-- Bone Helmet Design
(21211,2152,1,1,-1,202039),-- Recipe: Enchanted Necklace
(21211,57,495,955,0,700000),-- Adena
(21211,156,1,1,1,105),-- Hand Axe
(21211,167,1,1,1,105),-- Scalpel
(21211,178,1,1,1,106),-- Bone Staff
(21211,1867,1,1,2,166667),-- Animal Skin
(21211,1872,1,1,2,142857),-- Animal Bone
(21211,1875,1,1,2,6849),-- Stone of Purity
(21211,1881,1,1,2,15385),-- Coarse Bone Powder
(21211,5437,1,1,2,4651),-- Recipe: Heavy Sword(100%)
(21213,1900,1,2,-1,162437),-- Enchanted Necklace Chain
(21213,2153,1,2,-1,230897),-- Recipe: Tiger\'s Eye Earring
(21213,57,626,1255,0,700000),-- Adena
(21213,275,1,1,1,84),-- Long Bow
(21213,276,1,1,1,84),-- Elven Bow
(21213,2024,1,1,1,3853),-- Elven Bow Shaft
(21213,1061,1,1,2,90909),-- Healing Potion
(21213,1864,3,9,2,50000),-- Stem
(21213,1869,1,1,2,142857),-- Iron Ore
(21213,1878,1,1,2,58824),-- Braided Hemp
(21213,1884,1,1,2,83333),-- Cord
(21214,1865,1,3,-1,924982),-- Varnish
(21214,1875,1,1,-1,123331),-- Stone of Purity
(21214,1901,1,1,-1,255520),-- Tiger\'s Eye Earring Stone
(21214,57,781,1559,0,700000),-- Adena
(21214,223,1,1,1,41),-- Kukuri
(21214,238,1,1,1,41),-- Dagger of Mana
(21214,239,1,1,1,41),-- Mystic Knife
(21214,241,1,1,1,41),-- Shilen Knife
(21214,2028,1,1,1,2338),-- Shilen Knife Edge
(21214,1865,1,1,2,166667),-- Varnish
(21214,1870,1,1,2,166667),-- Coal
(21214,1875,1,1,2,12346),-- Stone of Purity
(21214,1879,1,1,2,29412),-- Cokes
(21214,2189,1,1,2,7246),-- Recipe: Mithril Tunic
(21215,1825,1,1,-1,139345),-- Recipe: Oriharukon
(21215,1902,1,1,-1,197653),-- Elven Earring Beads
(21215,2145,1,1,-1,139345),-- Recipe: Mithril Alloy
(21215,57,824,1645,0,700000),-- Adena
(21215,318,1,1,1,73),-- Crucifix of Blood
(21215,2042,1,1,1,4659),-- Crucifix of Blood Piece
(21215,1825,1,1,2,10417),-- Recipe: Oriharukon
(21215,1866,1,1,2,100000),-- Suede
(21215,1871,1,1,2,166667),-- Charcoal
(21215,1885,1,1,2,14286),-- High Grade Suede
(21215,5220,1,1,2,4854),-- Metal Hardener
(21215,5473,1,1,2,10753),-- Recipe: Durable Metal Plate(100%)
(21217,1868,1,11,-1,831781),-- Thread
(21217,1889,1,1,-1,166356),-- Compound Braid
(21217,1905,1,1,-1,242738),-- Omen Beast\'s Eye Earring Gemstone
(21217,57,1020,2061,0,700000),-- Adena
(21217,396,1,1,1,293),-- Salamander Skin Mail
(21217,1952,1,1,1,20068),-- Salamander Skin Mail Texture
(21217,1868,1,3,2,142857),-- Thread
(21217,1873,1,1,2,62500),-- Silver Nugget
(21217,1889,1,1,2,8547),-- Compound Braid
(21217,2161,1,1,2,16667),-- Recipe: Aquastone Ring
(21217,2993,1,1,2,11905),-- Recipe: Dwarven Chain Boots
(21217,5549,1,1,2,17241),-- Metallic Thread
(21218,1908,1,1,-1,286241),-- Moonstone Earring Wire
(21218,1909,1,1,-1,408198),-- Aquastone Ring Wire
(21218,2163,1,1,-1,154220),-- Recipe: Earring of Protection
(21218,57,1220,2478,0,700000),-- Adena
(21218,354,1,1,1,142),-- Chain Mail Shirt
(21218,381,1,1,1,227),-- Chain Gaiters
(21218,1959,1,1,1,9622),-- Chain Mail Shirt Material
(21218,1960,1,1,1,19572),-- Chain Gaiters Part
(21218,1864,1,1,2,250000),-- Stem
(21218,1869,1,1,2,125000),-- Iron Ore
(21218,1880,1,1,2,16949),-- Steel
(21218,2213,1,1,2,1211),-- Recipe: Dwarven Chain Mail Shirt
(21218,4043,1,1,2,6061),-- Asofe
(21218,5550,1,1,2,1211),-- Durable Metal Plate
(21218,8723,1,1,200,3802),-- Life Stone: level 46
(21218,8733,1,1,200,1267),-- Mid-Grade Life Stone: level 46
(21218,8743,1,1,200,422),-- High-Grade Life Stone: level 46 
(21219,1911,1,1,-1,232505),-- Earring of Protection Gemstone
(21219,1912,1,1,-1,336857),-- Ring of Protection Gemstone
(21219,2166,1,1,-1,105967),-- Recipe: Earrings of Binding (100%)
(21219,57,1258,2567,0,700000),-- Adena
(21219,440,1,1,1,196),-- Robe of Seal
(21219,1970,1,1,1,15466),-- Robe of Seal Fabric
(21219,1866,1,1,2,125000),-- Suede
(21219,1871,1,1,2,200000),-- Charcoal
(21219,1877,1,1,2,8929),-- Adamantite Nugget
(21219,4040,1,1,2,2237),-- Mold Lubricant
(21219,4041,1,1,2,2237),-- Mold Hardener
(21219,951,1,1,2,186),-- Scroll: Enchant Weapon (Grade C)
(21221,1867,1,11,-1,854850),-- Animal Skin
(21221,1872,1,11,-1,854850),-- Animal Bone
(21221,4605,1,1,-1,8548),-- Greater Dye of Dexterity <Dex+3 Str-3>
(21221,57,1558,3150,0,700000),-- Adena
(21221,64,1,1,1,325),-- Composite Boots
(21221,608,1,1,1,325),-- Mithril Gauntlets
(21221,1981,1,1,1,23394),-- Mithril Gauntlets Design
(21221,2956,1,1,1,23394),-- Composite Boots Part
(21221,1342,30,90,2,76923),-- Fine Steel Arrow
(21221,1867,1,1,2,142857),-- Animal Skin
(21221,1872,2,6,2,40000),-- Animal Bone
(21221,1875,1,1,2,4115),-- Stone of Purity
(21221,1881,1,1,2,14706),-- Coarse Bone Powder
(21221,2234,1,1,2,2371),-- Recipe: Divine Stockings
(21221,3955,1,1,2,319),-- Recipe: Blessed Spiritshot B
(21221,4041,1,1,2,1642),-- Mold Hardener
(21221,5280,1,1,2,319),-- Recipe: Greater Blessed Spiritshot (B) Compressed Package(100%)
(21221,6037,1,3,2,100000),-- Waking Scroll
(21221,8725,1,1,200,2463),-- Life Stone: level 52
(21221,8735,1,1,200,822),-- Mid-Grade Life Stone: level 52
(21222,1806,1,1,-1,9118),-- Recipe: Soulshot: B-Grade
(21222,1920,1,1,-1,155991),-- Necklace of Binding Chain
(21222,4126,1,1,-1,75228),-- Recipe: Adamantite Earrings
(21222,57,1837,3743,0,700000),-- Adena
(21222,2414,1,1,1,158),-- Full Plate Helmet
(21222,2497,1,1,1,225),-- Full Plate Shield
(21222,2959,1,1,1,14556),-- Full Plate Helmet Design
(21222,2966,1,1,1,23027),-- Full Plate Shield Fragment
(21222,1868,1,1,2,333333),-- Thread
(21222,1873,1,1,2,58824),-- Silver Nugget
(21222,1876,1,1,2,27778),-- Mithril Ore
(21222,1895,1,1,2,47619),-- Metallic Fiber
(21222,4042,1,1,2,3717),-- Enria
(21222,4126,1,1,2,3472),-- Recipe: Adamantite Earrings
(21222,4183,1,1,2,243),-- Recipe: Heavy War Axe
(21222,4965,1,1,2,243),-- Recipe: Sprite\'s Staff (60%)
(21222,8726,1,1,200,4444),-- Life Stone: level 55
(21222,8736,1,1,200,742),-- Mid-Grade Life Stone: level 55
(21223,4049,1,1,-1,244718),-- Adamantite Earring Gemstone
(21223,4129,1,1,-1,50493),-- Recipe: Earrings of Black Ore
(21223,4603,1,1,-1,10368),-- Greater Dye of Constitution <Con+3 Str-3>
(21223,57,1877,3833,0,700000),-- Adena
(21223,78,1,1,1,6),-- Great Sword
(21223,142,1,1,1,6),-- Keshanberk
(21223,148,1,1,1,6),-- Sword of Valhalla
(21223,4104,1,1,1,1314),-- Great Sword Blade
(21223,4107,1,1,1,1294),-- Keshanberk Blade
(21223,4108,1,1,1,1294),-- Sword of Valhalla Blade
(21223,1868,1,1,2,250000),-- Thread
(21223,1873,1,1,2,55556),-- Silver Nugget
(21223,1876,1,1,2,25641),-- Mithril Ore
(21223,1895,1,1,2,34483),-- Metallic Fiber
(21223,4042,1,1,2,1938),-- Enria
(21223,4130,1,1,2,1357),-- Recipe: Ring of Black Ore
(21223,4182,1,1,2,227),-- Recipe: Great Sword
(21223,8727,1,1,200,2132),-- Life Stone: level 58
(21223,8737,1,1,200,775),-- Mid-Grade Life Stone: level 58
(21223,8747,1,1,200,77),-- Mid-Grade Life Stone: level 58
(21224,1894,1,1,-1,178820),-- Crafted Leather
(21224,4039,1,1,-1,169879),-- Mold Glue
(21224,4613,1,1,-1,7029),-- Greater Dye of Strength <Str+4 Con-4>
(21224,57,2056,4180,0,700000),-- Adena
(21224,91,1,1,1,9),-- Heavy War Axe
(21224,92,1,1,1,9),-- Sprite\'s Staff
(21224,4105,1,1,1,2154),-- Heavy War Axe Head
(21224,4106,1,1,1,2119),-- Sprite\'s Staff Head
(21224,1343,10,30,2,250000),-- Silver Arrow
(21224,1894,1,1,2,4808),-- Crafted Leather
(21224,4039,1,1,2,4695),-- Mold Glue
(21224,4041,1,1,2,880),-- Mold Hardener
(21224,4044,1,1,2,5435),-- Thons
(21224,4158,1,1,2,237),-- Recipe: Doom Plate Armor
(21224,8728,1,1,200,2710),-- Life Stone: level 61
(21224,8738,1,1,200,575),-- Mid-Grade Life Stone: level 61
(21224,8748,1,1,200,19),-- High-Grade Life Stone: level 61
(21224,8758,1,1,200,19),-- Top-Grade Life Stone: level 61
(21225,4052,1,1,-1,197496),-- Earring of Black Ore Piece
(21225,4053,1,1,-1,239104),-- Ring of Black Ore Gemstone
(21225,5230,1,1,-1,7438),-- Recipe: Mithril Arrow (100%)
(21225,57,2256,4571,0,700000),-- Adena
(21225,2391,1,1,1,24),-- Blue Wolf Leather Armor
(21225,2392,1,1,1,24),-- Leather Armor of Doom
(21225,4082,1,1,1,4654),-- Blue Wolf Leather Armor Texture
(21225,4083,1,1,1,4654),-- Leather Armor of Doom Design
(21225,1830,1,1,2,111111),-- Scroll of Escape: Castle
(21225,1866,1,1,2,200000),-- Suede
(21225,1871,1,3,2,142857),-- Charcoal
(21225,1877,1,1,2,9524),-- Adamantite Nugget
(21225,4040,1,1,2,4566),-- Mold Lubricant
(21225,4041,1,1,2,2217),-- Mold Hardener
(21226,4052,1,1,-1,230702),-- Earring of Black Ore Piece
(21226,4054,1,1,-1,128347),-- Necklace of Black Ore Beads
(21226,5230,1,1,-1,8688),-- Recipe: Mithril Arrow (100%)
(21226,57,2634,5340,0,700000),-- Adena
(21226,79,1,1,1,15),-- Sword of Damascus
(21226,7889,1,1,1,15),-- Wizard\'s Tear
(21226,7883,1,1,1,15),-- Guardian Sword
(21226,4114,1,1,1,4213),-- Sword of Damascus Blade
(21226,8336,1,1,1,1321),-- Blade of the Wizard\'s Tear
(21226,8330,1,1,1,1288),-- Guardian Sword Blade
(21226,8314,1,1,2,311),-- Recipe: Spiritual Eye (60%)
(21226,1867,1,3,2,250000),-- Animal Skin
(21226,1872,1,1,2,125000),-- Animal Bone
(21226,1882,1,1,2,40000),-- Leather
(21226,1894,1,1,2,8264),-- Crafted Leather
(21226,4042,1,1,2,3322),-- Enria
(21226,5230,1,1,2,104),-- Recipe: Mithril Arrow (100%)
(21226,8729,1,1,200,3115),-- Life Stone: level 64
(21226,8739,1,1,200,311),-- Mid-Grade Life Stone: level 64
(21226,8749,1,1,200,104),-- High-Grade Life Stone: level 64
(21227,4077,1,1,-1,314447),-- Doom Shield Fragment
(21227,5230,1,1,-1,9320),-- Recipe: Mithril Arrow (100%)
(21227,5549,1,1,-1,699015),-- Metallic Thread
(21227,57,2811,5744,0,700000),-- Adena
(21227,287,1,1,1,16),-- Bow of Peril
(21227,4121,1,1,1,4497),-- Bow of Peril Shaft
(21227,1868,1,3,2,200000),-- Thread
(21227,1873,1,1,2,76923),-- Silver Nugget
(21227,1889,1,1,2,9434),-- Compound Braid
(21227,4044,1,1,2,8065),-- Thons
(21227,8314,1,1,2,168),-- Recipe: Spiritual Eye (60%)
(21227,5230,1,1,2,168),-- Recipe: Mithril Arrow (100%)
(21227,5549,1,1,2,20833),-- Metallic Thread
(21227,8730,1,1,200,2519),-- Life Stone: level 67
(21227,8740,1,1,200,168),-- Mid-Grade Life Stone: level 67
(21227,8760,1,1,200,297),-- Top-Grade Life Stone: level 67
(21228,1807,1,1,-1,9504),-- Recipe: Soulshot: A-Grade
(21228,3035,1,1,-1,9504),-- Recipe: Spiritshot A
(21228,5478,1,1,-1,59940),-- Sealed Dark Crystal Leather Armor Pattern
(21228,57,2875,5849,0,700000),-- Adena
(21228,5287,1,1,1,16),-- Sealed Dark Crystal breastplate
(21228,5288,1,1,1,25),-- Sealed Dark Crystal Gaiters
(21228,5293,1,1,1,11),-- Sealed Tallum Plate Armor
(21228,5520,1,1,1,2264),-- Sealed Dark Crystal Breastplate Pattern
(21228,5521,1,1,1,1597),-- Sealed Tallum Plate Armor Pattern
(21228,5524,1,1,1,3460),-- Sealed Dark Crystal Gaiters Pattern
(21228,1866,1,1,2,142857),-- Suede
(21228,1871,1,1,2,200000),-- Charcoal
(21228,1885,1,1,2,14493),-- High Grade Suede
(21228,4041,1,1,2,3390),-- Mold Hardener
(21228,5220,1,1,2,13514),-- Metal Hardener
(21228,5271,1,1,2,231),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(21228,8730,1,1,200,2625),-- Life Stone: level 67
(21228,8740,1,1,200,772),-- Mid-Grade Life Stone: level 67
(21229,4041,1,1,-1,67208),-- Mold Hardener
(21229,4044,1,1,-1,257630),-- Thons
(21229,5479,1,1,-1,45733),-- Sealed Tallum Leather Armor Pattern
(21229,57,3110,6350,0,700000),-- Adena
(21229,6323,1,1,1,44),-- Sealed Phoenix Necklace
(21229,6324,1,1,1,59),-- Sealed Phoenix Earring
(21229,6325,1,1,1,88),-- Sealed Phoenix Ring
(21229,6341,1,1,1,10525),-- Sealed Phoenix Earring Gemstone
(21229,6343,1,1,1,8136),-- Sealed Phoenix Necklace Beads
(21229,6345,1,1,1,14557),-- Sealed Phoenix Ring Gemstone
(21229,1894,1,1,2,7246),-- Crafted Leather
(21229,4039,1,1,2,7874),-- Mold Glue
(21229,4041,1,1,2,1537),-- Mold Hardener
(21229,4044,1,1,2,10870),-- Thons
(21229,730,1,1,2,89),-- Scroll: Enchant Armor (Grade A)
(21229,8731,1,1,200,3106),-- Life Stone: level 70
(21229,8741,1,1,200,461),-- Mid-Grade Life Stone: level 70
(21229,8751,1,1,200,38),-- High-Grade Life Stone: level 70
(21229,8761,1,1,200,22),-- Top-Grade Life Stone: level 70
(21230,5480,1,1,-1,39987),-- Sealed Leather Armor of Nightmare Fabric
(21230,5481,1,1,-1,39987),-- Sealed Majestic Leather Armor Fabric
(21230,5487,1,1,-1,40128),-- Sealed Robe of Nightmare Fabric
(21230,57,3587,7319,0,700000),-- Adena
(21230,5289,1,1,1,51),-- Sealed Dark Crystal Helmet
(21230,5292,1,1,1,72),-- Sealed Dark Crystal Shield
(21230,5294,1,1,1,51),-- Sealed Tallum Helmet
(21230,5494,1,1,1,9213),-- Sealed Dark Crystal Shield Fragment
(21230,5525,1,1,1,6782),-- Sealed Dark Crystal Helmet Design
(21230,5526,1,1,1,6782),-- Sealed Tallum Helm Design
(21230,730,1,1,2,236),-- Scroll: Enchant Armor (Grade A)
(21230,1895,1,1,2,83333),-- Metallic Fiber
(21230,4040,1,1,2,5556),-- Mold Lubricant
(21230,4042,1,1,2,4762),-- Enria
(21230,4043,1,1,2,10204),-- Asofe
(21230,8730,1,1,200,1876),-- Life Stone: level 67
(21230,8740,1,1,200,863),-- Mid-Grade Life Stone: level 67
(21230,8750,1,1,200,30),-- High-Grade Life Stone: level 67
(21231,729,1,1,-1,995),-- Scroll: Enchant Weapon (Grade A)
(21231,1875,1,1,-1,597002),-- Stone of Purity
(21231,5480,1,1,-1,40183),-- Sealed Leather Armor of Nightmare Fabric
(21231,57,3602,7359,0,700000),-- Adena
(21231,5289,1,1,1,51),-- Sealed Dark Crystal Helmet
(21231,5292,1,1,1,73),-- Sealed Dark Crystal Shield
(21231,5294,1,1,1,51),-- Sealed Tallum Helmet
(21231,5494,1,1,1,9259),-- Sealed Dark Crystal Shield Fragment
(21231,5525,1,1,1,6815),-- Sealed Dark Crystal Helmet Design
(21231,5526,1,1,1,6815),-- Sealed Tallum Helm Design
(21231,729,1,1,2,49),-- Scroll: Enchant Weapon (Grade A)
(21231,1867,1,3,2,200000),-- Animal Skin
(21231,1872,1,3,2,125000),-- Animal Bone
(21231,1875,1,1,2,20000),-- Stone of Purity
(21231,1881,1,1,2,40000),-- Coarse Bone Powder
(21231,4041,1,1,2,3165),-- Mold Hardener
(21231,8732,1,1,200,994),-- Life Stone: level 76
(21231,8742,1,1,200,979),-- Mid-Grade Life Stone: level 76
(21231,8752,1,1,200,15),-- High-Grade Life Stone: level 76
(21236,1061,1,2,-1,784398),-- Healing Potion
(21236,1900,1,1,-1,141604),-- Enchanted Necklace Chain
(21236,2153,1,2,-1,201284),-- Recipe: Tiger\'s Eye Earring
(21236,57,546,1092,0,700000),-- Adena
(21236,69,1,1,1,44),-- Bastard Sword
(21236,125,1,1,1,44),-- Spinebone Sword
(21236,126,1,1,1,44),-- Artisan\'s Sword
(21236,128,1,1,1,44),-- Knight\'s Sword
(21236,2021,1,1,1,1787),-- Spinebone Sword Blade
(21236,1867,1,3,2,111111),-- Animal Skin
(21236,1872,1,1,2,200000),-- Animal Bone
(21236,1882,1,1,2,34483),-- Leather
(21236,2184,1,1,2,19231),-- Recipe: Mithril Banded Gaiters
(21237,1933,1,1,-1,174667),-- Dark Stocking Fabric
(21237,1935,1,1,-1,82942),-- Scale Mail Fragment
(21237,2183,1,1,-1,97676),-- Recipe: Mithril Banded Mail
(21237,57,626,1255,0,700000),-- Adena
(21237,86,1,1,1,26),-- Tomahawk
(21237,179,1,1,1,26),-- Mace of Prayer
(21237,180,1,1,1,26),-- Mace of Judgment
(21237,181,1,1,1,26),-- Mace of Miracle
(21237,182,1,1,1,26),-- Doom Hammer
(21237,183,1,1,1,26),-- Mystic Staff
(21237,184,1,1,1,26),-- Conjuror\'s Staff
(21237,185,1,1,1,26),-- Staff of Mana
(21237,2022,1,1,1,1264),-- Mace of Judgment Head
(21237,2023,1,1,1,1006),-- Conjuror\'s Staff Head
(21237,1868,1,3,2,125000),-- Thread
(21237,1873,1,1,2,50000),-- Silver Nugget
(21237,1889,1,1,2,7634),-- Compound Braid
(21237,2265,1,1,2,1218),-- Recipe: Spiked Club
(21237,5549,1,1,2,12048),-- Metallic Thread
(21237,6037,1,1,2,250000),-- Waking Scroll
(21238,1878,1,2,-1,739986),-- Braided Hemp
(21238,1884,1,3,-1,569220),-- Cord
(21238,1901,1,2,-1,255520),-- Tiger\'s Eye Earring Stone
(21238,57,781,1559,0,700000),-- Adena
(21238,260,1,1,1,203),-- Triple-Edged Jamadhr
(21238,1341,20,60,2,200000),-- Bone Arrow
(21238,1874,1,1,2,6944),-- Oriharukon Ore
(21238,1878,1,1,2,47619),-- Braided Hemp
(21238,1884,1,1,2,71429),-- Cord
(21238,1889,1,1,2,6757),-- Compound Braid
(21238,2190,1,1,2,8264),-- Recipe: Mithril Stockings
(21238,5220,1,1,2,4566),-- Metal Hardener
(21239,1825,1,1,-1,139345),-- Recipe: Oriharukon
(21239,1902,1,1,-1,197653),-- Elven Earring Beads
(21239,2145,1,1,-1,139345),-- Recipe: Mithril Alloy
(21239,57,824,1645,0,700000),-- Adena
(21239,279,1,1,1,73),-- Strengthened Long Bow
(21239,2040,1,1,1,5834),-- Strengthened Long Bow Shaft
(21239,1865,1,1,2,142857),-- Varnish
(21239,1870,1,1,2,166667),-- Coal
(21239,1874,1,1,2,9709),-- Oriharukon Ore
(21239,2145,1,1,2,9901),-- Recipe: Mithril Alloy
(21239,2208,1,1,2,14706),-- Recipe: Ogre Power Gauntlets
(21239,6036,1,1,2,7752),-- Greater Magic Haste Potion
(21240,1866,1,3,-1,741221),-- Suede
(21240,1871,1,5,-1,741221),-- Charcoal
(21240,1903,1,1,-1,370920),-- Elven Ring Piece
(21240,57,932,1861,0,700000),-- Adena
(21240,224,1,1,1,42),-- Maingauche
(21240,1660,1,1,1,42),-- Cursed Maingauche
(21240,2038,1,1,1,3037),-- Maingauche Edge
(21240,2044,1,1,1,3037),-- Cursed Maingauche Edge
(21240,1341,60,180,2,142857),-- Bone Arrow
(21240,1866,1,1,2,166667),-- Suede
(21240,1871,2,6,2,58824),-- Charcoal
(21240,1877,1,1,2,8065),-- Adamantite Nugget
(21241,1805,1,1,-1,8318),-- Recipe: Soulshot: C-Grade
(21241,1905,1,1,-1,242738),-- Omen Beast\'s Eye Earring Gemstone
(21241,4469,1,1,-1,14259),-- Dye of Strength <Str+1 Con-1>
(21241,57,1020,2061,0,700000),-- Adena
(21241,396,1,1,1,293),-- Salamander Skin Mail
(21241,1952,1,1,1,20068),-- Salamander Skin Mail Texture
(21241,1867,1,1,2,200000),-- Animal Skin
(21241,1872,1,3,2,100000),-- Animal Bone
(21241,1875,1,1,2,12195),-- Stone of Purity
(21241,1881,1,1,2,21277),-- Coarse Bone Powder
(21241,2162,1,1,2,7576),-- Recipe: Aquastone Necklace
(21241,2206,1,1,2,4049),-- Recipe: Mithril Shirt
(21241,2990,1,1,2,12346),-- Recipe: Chain Boots
(21242,1908,1,1,-1,286241),-- Moonstone Earring Wire
(21242,1909,1,1,-1,408198),-- Aquastone Ring Wire
(21242,2163,1,1,-1,154220),-- Recipe: Earring of Protection
(21242,57,1220,2478,0,700000),-- Adena
(21242,96,1,1,1,21),-- Scythe
(21242,298,1,1,1,21),-- Orcish Glaive
(21242,302,1,1,1,21),-- Body Slasher
(21242,2062,1,1,1,1868),-- Scythe Edge
(21242,2075,1,1,1,1868),-- Orcish Glaive Blade
(21242,2076,1,1,1,1868),-- Body Slasher Blade
(21242,1868,1,1,2,250000),-- Thread
(21242,1873,1,1,2,58824),-- Silver Nugget
(21242,1876,1,1,2,29412),-- Mithril Ore
(21242,1895,1,1,2,40000),-- Metallic Fiber
(21242,2214,1,1,2,3279),-- Recipe: Dwarven Chain Gaiters
(21242,4042,1,1,2,2618),-- Enria
(21242,8723,1,1,200,4854),-- Life Stone: level 46
(21242,8733,1,1,200,1217),-- Mid-Grade Life Stone: level 46
(21242,8743,1,1,200,20),-- High-Grade Life Stone: level 46
(21243,951,1,1,-1,5684),-- Scroll: Enchant Weapon (Grade C)
(21243,1911,1,1,-1,232505),-- Earring of Protection Gemstone
(21243,2166,1,1,-1,105967),-- Recipe: Earrings of Binding (100%)
(21243,57,1258,2567,0,700000),-- Adena
(21243,398,1,1,1,78),-- Plated Leather
(21243,399,1,1,1,72),-- Rind Leather Armor
(21243,418,1,1,1,125),-- Plated Leather Gaiters
(21243,419,1,1,1,115),-- Rind Leather Gaiters
(21243,1966,1,1,1,6321),-- Plated Leather Fragment
(21243,1967,1,1,1,9442),-- Plated Leather Gaiters Material
(21243,1975,1,1,1,5687),-- Rind Leather Armor Design
(21243,1976,1,1,1,10527),-- Rind Leather Gaiters Material
(21243,951,1,1,2,268),-- Scroll: Enchant Weapon (Grade C)
(21243,1868,1,1,2,250000),-- Thread
(21243,1873,1,1,2,52632),-- Silver Nugget
(21243,1876,1,1,2,28571),-- Mithril Ore
(21243,1895,1,1,2,40000),-- Metallic Fiber
(21243,4042,1,1,2,2232),-- Enria
(21243,8724,1,1,200,3937),-- Life Stone: level 49
(21243,8734,1,1,200,894),-- Mid-Grade Life Stone: level 49
(21243,8744,1,1,200,179),-- High-Grade Life Stone: level 49
(21244,1539,1,1,-1,773226),-- Greater Healing Potion
(21244,1914,1,1,-1,195478),-- Earring of Binding Gemstone
(21244,4601,1,1,-1,7732),-- Greater Dye of Strength <Str+3 Con-3>
(21244,57,1404,2854,0,700000),-- Adena
(21244,60,1,1,1,100),-- Composite Armor
(21244,1977,1,1,1,10643),-- Composite Armor Temper
(21244,1865,1,1,2,125000),-- Varnish
(21244,1870,1,1,2,125000),-- Coal
(21244,1875,1,1,2,8197),-- Stone of Purity
(21244,1879,1,1,2,21277),-- Cokes
(21244,2170,1,1,2,2421),-- Recipe: Nassen\'s Earring
(21244,2970,1,1,2,4425),-- Recipe: Ring of Binding
(21244,4040,1,1,2,3413),-- Mold Lubricant
(21244,8725,1,1,200,3058),-- Life Stone: level 52
(21244,8735,1,1,200,357),-- Mid-Grade Life Stone: level 52
(21244,8745,1,1,200,127),-- High-Grade Life Stone: level 52
(21244,8755,1,1,200,76),-- Top-Grade Life Stone: level 52
(21245,1915,1,2,-1,338927),-- Ring of Ages Gemstone
(21245,2169,1,1,-1,68085),-- Recipe: Necklace of Binding
(21245,2170,1,1,-1,90727),-- Recipe: Nassen\'s Earring
(21245,57,1558,3150,0,700000),-- Adena
(21245,60,1,1,1,111),-- Composite Armor
(21245,1977,1,1,1,11766),-- Composite Armor Temper
(21245,1866,1,1,2,83333),-- Suede
(21245,1871,1,1,2,125000),-- Charcoal
(21245,1877,1,1,2,5319),-- Adamantite Nugget
(21245,2353,1,1,2,88),-- Recipe: Samurai Longsword
(21245,4040,1,1,2,3279),-- Mold Lubricant
(21245,4041,1,1,2,1757),-- Mold Hardener
(21245,4122,1,1,2,5814),-- Recipe: Maestro Holder
(21245,8309,1,1,2,110),-- Recipe: Spell Breaker (100%)
(21245,8725,1,1,200,3344),-- Life Stone: level 52
(21245,8735,1,1,200,1319),-- Mid-Grade Life Stone: level 52
(21245,8745,1,1,200,220),-- High-Grade Life Stone: level 52
(21246,1867,1,13,-1,868351),-- Animal Skin
(21246,1918,1,1,-1,197438),-- Nassen\'s Earring Gemstone
(21246,4042,1,1,-1,75981),-- Enria
(21246,57,1837,3743,0,700000),-- Adena
(21246,119,1,1,1,100),-- Necklace of Binding
(21246,855,1,1,1,133),-- Nassen\'s Earring
(21246,886,1,1,1,199),-- Ring of Binding
(21246,1918,1,1,1,12150),-- Nassen\'s Earring Gemstone
(21246,1920,1,1,1,9599),-- Necklace of Binding Chain
(21246,2917,1,1,1,16113),-- Ring of Binding Gemstone
(21246,1867,1,1,2,166667),-- Animal Skin
(21246,1872,1,1,2,166667),-- Animal Bone
(21246,1882,1,1,2,32258),-- Leather
(21246,1894,1,1,2,3125),-- Crafted Leather
(21246,4042,1,1,2,1873),-- Enria
(21246,4127,1,1,2,3745),-- Recipe: Adamantite Ring
(21246,4183,1,1,2,625),-- Recipe: Heavy War Axe
(21246,4965,1,1,2,312),-- Recipe: Sprite\'s Staff (60%)
(21246,8726,1,1,200,5319),-- Life Stone: level 55
(21246,8736,1,1,200,937),-- Mid-Grade Life Stone: level 55
(21246,8746,1,1,200,39),-- High-Grade Life Stone: level 55
(21247,4049,1,1,-1,244718),-- Adamantite Earring Gemstone
(21247,4129,1,1,-1,50493),-- Recipe: Earrings of Black Ore
(21247,4613,1,1,-1,6435),-- Greater Dye of Strength <Str+4 Con-4>
(21247,57,1877,3833,0,700000),-- Adena
(21247,554,1,1,1,67),-- Zubei\'s Boots
(21247,600,1,1,1,67),-- Avadon Boots
(21247,612,1,1,1,67),-- Zubei\'s Gauntlets
(21247,2464,1,1,1,67),-- Avadon Gloves
(21247,4073,1,1,1,8963),-- Avadon Gloves Part
(21247,4097,1,1,1,8963),-- Zubei\'s Boots Design
(21247,4098,1,1,1,8963),-- Avadon Boots Design
(21247,4100,1,1,1,8963),-- Zubei\'s Gauntlets Part
(21247,1867,1,1,2,166667),-- Animal Skin
(21247,1872,1,1,2,166667),-- Animal Bone
(21247,1875,1,1,2,6711),-- Stone of Purity
(21247,1881,1,1,2,13699),-- Coarse Bone Powder
(21247,4041,1,1,2,319),-- Mold Hardener
(21247,4131,1,1,2,319),-- Recipe: Necklace of Black Ore
(21247,4173,1,1,2,637),-- Recipe: Blue Wolf Gloves
(21247,8727,1,1,200,3185),-- Life Stone: level 58
(21247,8737,1,1,200,637),-- Mid-Grade Life Stone: level 58
(21248,4050,1,1,-1,315761),-- Adamantite Ring Wire
(21248,4051,1,1,-1,197649),-- Adamantite Necklace Chain
(21248,4130,1,1,-1,82733),-- Recipe: Ring of Black Ore
(21248,57,2056,4180,0,700000),-- Adena
(21248,503,1,1,1,49),-- Zubei\'s Helmet
(21248,633,1,1,1,70),-- Zubei\'s Shield
(21248,673,1,1,1,70),-- Avadon Shield
(21248,2415,1,1,1,49),-- Avadon Circlet
(21248,4072,1,1,1,8454),-- Avadon Circlet Pattern
(21248,4101,1,1,1,10911),-- Zubei\'s Shield Fragment
(21248,4102,1,1,1,8453),-- Zubei\'s Helmet Design
(21248,4439,1,1,1,12980),-- Avadon Shield Fragment
(21248,1864,1,1,2,250000),-- Stem
(21248,1869,1,1,2,125000),-- Iron Ore
(21248,1880,1,1,2,12346),-- Steel
(21248,4043,1,1,2,4132),-- Asofe
(21248,4159,1,1,2,852),-- Recipe: Blue Wolf Leather Armor
(21248,5550,1,1,2,1048),-- Durable Metal Plate
(21248,6036,1,1,2,4525),-- Greater Magic Haste Potion
(21248,8728,1,1,200,2096),-- Life Stone: level 61
(21248,8738,1,1,200,655),-- Mid-Grade Life Stone: level 61
(21248,8748,1,1,200,131),-- High-Grade Life Stone: level 61
(21249,4052,1,1,-1,197496),-- Earring of Black Ore Piece
(21249,4053,1,1,-1,239104),-- Ring of Black Ore Gemstone
(21249,5230,1,1,-1,7438),-- Recipe: Mithril Arrow (100%)
(21249,57,2256,4571,0,700000),-- Adena
(21249,358,1,1,1,18),-- Blue Wolf Breastplate
(21249,2380,1,1,1,28),-- Blue Wolf Gaiters
(21249,2381,1,1,1,12),-- Doom Plate Armor
(21249,4078,1,1,1,4012),-- Blue Wolf Breastplate Part
(21249,4080,1,1,1,5649),-- Blue Wolf Gaiters Material
(21249,4081,1,1,1,2707),-- Doom Plate Armor Temper
(21249,1866,1,1,2,125000),-- Suede
(21249,1871,1,1,2,142857),-- Charcoal
(21249,1877,1,1,2,3774),-- Adamantite Nugget
(21249,4040,1,1,2,4016),-- Mold Lubricant
(21249,4041,1,1,2,473),-- Mold Hardener
(21249,8729,1,1,200,3311),-- Life Stone: level 64
(21249,8739,1,1,200,268),-- Mid-Grade Life Stone: level 64
(21249,8749,1,1,200,134),-- High-Grade Life Stone: level 64
(21250,4052,1,1,-1,230702),-- Earring of Black Ore Piece
(21250,4053,1,1,-1,279304),-- Ring of Black Ore Gemstone
(21250,5230,1,1,-1,8688),-- Recipe: Mithril Arrow (100%)
(21250,57,2634,5340,0,700000),-- Adena
(21250,171,1,1,1,5),-- Deadman\'s Glory
(21250,175,1,1,1,5),-- Art of Battle Axe
(21250,210,1,1,1,5),-- Staff of Evil Spirits
(21250,4116,1,1,1,1402),-- Deadman\'s Glory Stone
(21250,4117,1,1,1,1402),-- Art of Battle Axe Blade
(21250,4118,1,1,1,1329),-- Evil Spirit Head
(21250,1344,20,60,2,111111),-- Mithril Arrow
(21250,1830,1,1,2,62500),-- Scroll of Escape: Castle
(21250,1866,1,1,2,125000),-- Suede
(21250,1871,1,1,2,166667),-- Charcoal
(21250,1877,1,1,2,7519),-- Adamantite Nugget
(21250,4040,1,1,2,2740),-- Mold Lubricant
(21250,4041,1,1,2,1370),-- Mold Hardener
(21250,8729,1,1,200,2801),-- Life Stone: level 64
(21250,8739,1,1,200,596),-- Mid-Grade Life Stone: level 64
(21250,8749,1,1,200,60),-- High-Grade Life Stone: level 64
(21250,8759,1,1,200,21),-- Top-Grade Life Stone: level 64
(21251,4053,1,1,-1,299621),-- Ring of Black Ore Gemstone
(21251,4054,1,1,-1,137683),-- Necklace of Black Ore Beads
(21251,5230,1,1,-1,9320),-- Recipe: Mithril Arrow (100%)
(21251,57,2811,5744,0,700000),-- Adena
(21251,2398,1,1,1,22),-- Blue Wolf Tunic
(21251,2399,1,1,1,22),-- Tunic of Doom
(21251,2403,1,1,1,35),-- Blue Wolf Stockings
(21251,2404,1,1,1,35),-- Stockings of Doom
(21251,4084,1,1,1,4484),-- Blue Wolf Tunic Fabric
(21251,4085,1,1,1,4484),-- Tunic of Doom Pattern
(21251,4086,1,1,1,6054),-- Blue Wolf Stockings Pattern
(21251,4087,1,1,1,6054),-- Stockings of Doom Pattern
(21251,1344,20,60,2,125000),-- Mithril Arrow
(21251,1830,1,1,2,76923),-- Scroll of Escape: Castle
(21251,1866,1,1,2,125000),-- Suede
(21251,1871,1,3,2,62500),-- Charcoal
(21251,1885,1,1,2,15873),-- High Grade Suede
(21251,4041,1,1,2,1838),-- Mold Hardener
(21251,5220,1,1,2,6329),-- Metal Hardener
(21251,8730,1,1,200,1961),-- Life Stone: level 67
(21251,8740,1,1,200,674),-- Mid-Grade Life Stone: level 67
(21252,4053,1,1,-1,305530),-- Ring of Black Ore Gemstone
(21252,5230,1,1,-1,9504),-- Recipe: Mithril Arrow (100%)
(21252,5478,1,1,-1,59940),-- Sealed Dark Crystal Leather Armor Pattern
(21252,57,2875,5849,0,700000),-- Adena
(21252,5297,1,1,1,21),-- Sealed Dark Crystal Leather Armor
(21252,5298,1,1,1,33),-- Sealed Dark Crystalline Leggings
(21252,5301,1,1,1,14),-- Sealed Tallum Leather Armor
(21252,5478,1,1,1,2951),-- Sealed Dark Crystal Leather Armor Pattern
(21252,5479,1,1,1,2077),-- Sealed Tallum Leather Armor Pattern
(21252,5482,1,1,1,4631),-- Sealed Dark Crystalline Leggings Design
(21252,10608,1,1,3,10638),-- Forgotten Scroll - Chant of Combat
(21252,10579,1,1,3,15152),-- Forgotten Scroll - Improve Combat
(21252,10583,1,1,3,15152),-- Forgotten Scroll - Improve Shield Defense
(21252,14214,1,1,3,15152),-- Forgotten Scroll - Chant of Movement
(21252,5276,1,1,2,402),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21252,8730,1,1,200,70),-- Life Stone: level 67
(21253,1807,1,1,-1,10305),-- Recipe: Soulshot: A-Grade
(21253,3035,1,1,-1,10305),-- Recipe: Spiritshot A
(21253,5479,1,2,-1,45733),-- Sealed Tallum Leather Armor Pattern
(21253,57,3110,6350,0,700000),-- Adena
(21253,5304,1,1,1,22),-- Sealed Tallum Tunic
(21253,5305,1,1,1,36),-- Sealed Tallum Stockings
(21253,5308,1,1,1,15),-- Sealed Dark Crystal Robe
(21253,5485,1,1,1,3200),-- Sealed Tallum Tunic Texture
(21253,5486,1,1,1,2252),-- Sealed Dark Crystal Robe Fabric
(21253,5489,1,1,1,5022),-- Sealed Tallum Stockings Fabric
(21253,10610,1,1,3,16129),-- Forgotten Scroll - Chant of Blood Awakening
(21253,14215,1,1,3,16129),-- Forgotten Scroll - Combat of Paagrio
(21253,10582,1,1,3,15152),-- Forgotten Scroll - Improve Critical Attack
(21253,10580,1,1,3,16129),-- Forgotten Scroll - Improve Magic
(21254,729,1,1,-1,990),-- Scroll: Enchant Weapon (Grade A)
(21254,1877,1,1,-1,356449),-- Adamantite Nugget
(21254,5480,1,1,-1,39987),-- Sealed Leather Armor of Nightmare Fabric
(21254,57,3587,7319,0,700000),-- Adena
(21254,6323,1,1,1,51),-- Sealed Phoenix Necklace
(21254,6324,1,1,1,67),-- Sealed Phoenix Earring
(21254,6325,1,1,1,101),-- Sealed Phoenix Ring
(21254,6341,1,1,1,12134),-- Sealed Phoenix Earring Gemstone
(21254,6343,1,1,1,9381),-- Sealed Phoenix Necklace Beads
(21254,6345,1,1,1,16783),-- Sealed Phoenix Ring Gemstone
(21254,14216,1,1,3,16129),-- Forgotten Scroll - Critical of Paagrio
(21255,4041,1,1,-1,82464),-- Mold Hardener
(21255,5481,1,1,-1,42554),-- Sealed Majestic Leather Armor Fabric
(21255,5487,1,1,-1,42704),-- Sealed Robe of Nightmare Fabric
(21255,57,3814,7793,0,700000),-- Adena
(21255,5297,1,1,1,27),-- Sealed Dark Crystal Leather Armor
(21255,5298,1,1,1,44),-- Sealed Dark Crystalline Leggings
(21255,5301,1,1,1,19),-- Sealed Tallum Leather Armor
(21255,5478,1,1,1,3926),-- Sealed Dark Crystal Leather Armor Pattern
(21255,5479,1,1,1,2763),-- Sealed Tallum Leather Armor Pattern
(21255,5482,1,1,1,6162),-- Sealed Dark Crystalline Leggings Design
(21256,1804,1,1,-1,10820),-- Recipe: Soulshot: D-Grade
(21256,1872,1,1,-1,360672),-- Animal Bone
(21256,1898,1,1,-1,27787),-- Blue Diamond Necklace Gem
(21256,57,124,218,0,700000),-- Adena
(21256,27,1,1,1,217),-- Hard Leather Shirt
(21256,33,1,1,1,348),-- Hard Leather Gaiters
(21256,1927,1,1,1,3188),-- Hard Leather Shirt Pattern
(21256,1929,1,1,1,5155),-- Hard Leather Gaiters Material
(21256,735,1,1,2,2664),-- Potion of Alacrity
(21256,1867,1,1,2,42615),-- Animal Skin
(21256,1872,1,1,2,42615),-- Animal Bone
(21256,1882,1,1,2,7103),-- Leather
(21256,5153,1,1,2,1279),-- Recipe: Soulshot (D) Compressed Package (100%)
(21257,734,1,1,-1,42627),-- Haste Potion
(21257,1868,1,1,-1,511527),-- Thread
(21257,1897,1,1,-1,60896),-- Necklace of Wisdom Chain
(21257,57,122,202,0,700000),-- Adena
(21257,217,1,1,1,111),-- Shining Knife
(21257,218,1,1,1,111),-- Throwing Knife
(21257,735,1,1,2,4198),-- Potion of Alacrity
(21257,1868,1,1,2,100732),-- Thread
(21257,1873,1,1,2,20147),-- Silver Nugget
(21259,4049,1,1,-1,226242),-- Adamantite Earring Gemstone
(21259,4129,1,1,-1,46681),-- Recipe: Earrings of Black Ore
(21259,4613,1,1,-1,5949),-- Greater Dye of Strength <Str+4 Con-4>
(21259,57,2085,4248,0,700000),-- Adena
(21259,11915,1,1,1,227),-- Common Item - Kris
(21259,11919,1,1,1,313),-- Common Item - Hell Knife
(21259,4109,1,1,1,1795),-- Kris Edge
(21259,4110,1,1,1,1795),-- Hell Knife Edge
(21259,947,1,1,2,85),-- Scroll: Enchant Weapon (Grade B)
(21259,1868,1,1,2,333333),-- Thread
(21259,1873,1,1,2,62500),-- Silver Nugget
(21259,1876,1,1,2,30303),-- Mithril Ore
(21259,1895,1,1,2,41667),-- Metallic Fiber
(21259,4042,1,1,2,2105),-- Enria
(21259,8728,1,1,200,3125),-- Life Stone: level 61
(21259,8738,1,1,200,597),-- Mid-Grade Life Stone: level 61
(21259,8748,1,1,200,8),-- High-Grade Life Stone: level 61
(21259,8758,1,1,200,28),-- Top-Grade Life Stone: level 61
(21261,1914,1,1,-1,78741),-- Earring of Binding Gemstone
(21261,2169,1,1,-1,24807),-- Recipe: Necklace of Binding
(21261,2170,1,1,-1,33056),-- Recipe: Nassen\'s Earring
(21261,57,654,1329,0,700000),-- Adena
(21261,231,1,1,1,7),-- Grace Dagger
(21261,233,1,1,1,7),-- Dark Screamer
(21261,2106,1,1,1,790),-- Grace Dagger Edge
(21261,2107,1,1,1,790),-- Dark Screamer Edge
(21261,1539,1,1,2,14493),-- Greater Healing Potion
(21261,1864,1,1,2,125000),-- Stem
(21261,1869,1,1,2,66667),-- Iron Ore
(21261,1880,1,1,2,7246),-- Steel
(21261,2231,1,1,2,320),-- Recipe: Full Plate Armor
(21261,4043,1,1,2,2558),-- Asofe
(21261,5550,1,1,2,879),-- Durable Metal Plate
(21261,8725,1,1,200,1919),-- Life Stone: level 52
(21261,8735,1,1,200,400),-- Mid-Grade Life Stone: level 52
(21261,8745,1,1,200,80),-- High-Grade Life Stone: level 52
(21262,1914,1,1,-1,142677),-- Earring of Binding Gemstone
(21262,2169,1,1,-1,44949),-- Recipe: Necklace of Binding
(21262,2170,1,1,-1,59897),-- Recipe: Nassen\'s Earring
(21262,57,879,1786,0,700000),-- Adena
(21262,228,1,1,1,16),-- Crystal Dagger
(21262,2119,1,1,1,2137),-- Crystal Dagger Blade
(21262,1539,1,1,2,20000),-- Greater Healing Potion
(21262,1864,1,1,2,142857),-- Stem
(21262,1869,1,1,2,76923),-- Iron Ore
(21262,1880,1,1,2,7634),-- Steel
(21262,2231,1,1,2,206),-- Recipe: Full Plate Armor
(21262,4043,1,1,2,3300),-- Asofe
(21262,5550,1,1,2,206),-- Durable Metal Plate
(21262,8726,1,1,200,1650),-- Life Stone: level 55
(21262,8736,1,1,200,206),-- Mid-Grade Life Stone: level 55
(21263,1914,1,1,-1,218248),-- Earring of Binding Gemstone
(21263,2169,1,1,-1,68758),-- Recipe: Necklace of Binding
(21263,2170,1,1,-1,91623),-- Recipe: Nassen\'s Earring
(21263,57,1233,2511,0,700000),-- Adena
(21263,228,1,1,1,24),-- Crystal Dagger
(21263,2119,1,1,1,3269),-- Crystal Dagger Blade
(21263,1539,1,1,2,24390),-- Greater Healing Potion
(21263,1864,1,1,2,200000),-- Stem
(21263,1869,1,1,2,100000),-- Iron Ore
(21263,1880,1,1,2,12987),-- Steel
(21263,2231,1,1,2,500),-- Recipe: Full Plate Armor
(21263,4043,1,1,2,1499),-- Asofe
(21263,5550,1,1,2,1000),-- Durable Metal Plate
(21263,8726,1,1,200,2000),-- Life Stone: level 55
(21263,8736,1,1,200,1000),-- Mid-Grade Life Stone: level 55
(21264,1914,1,1,-1,355312),-- Earring of Binding Gemstone
(21264,2169,1,1,-1,111939),-- Recipe: Necklace of Binding
(21264,2170,1,1,-1,149164),-- Recipe: Nassen\'s Earring
(21264,57,1713,3458,0,700000),-- Adena
(21264,11915,1,1,1,11),-- Common Item - Kris (no drop info)
(21264,11919,1,1,1,11),-- Common Item - Hell Knife
(21264,4109,1,1,1,2631),-- Kris Edge
(21264,4110,1,1,1,2631),-- Hell Knife Edge
(21264,1539,1,1,2,25000),-- Greater Healing Potion
(21264,1864,1,1,2,250000),-- Stem
(21264,1869,1,1,2,111111),-- Iron Ore
(21264,1880,1,1,2,2410),-- Steel
(21264,2231,1,1,2,1205),-- Recipe: Full Plate Armor
(21264,4043,1,1,2,4831),-- Asofe
(21264,5550,1,1,2,1205),-- Durable Metal Plate
(21264,8727,1,1,200,1205),-- Life Stone: level 58
(21264,8737,1,1,200,1205),-- Mid-Grade Life Stone: level 58
(21265,1864,1,3,-1,522665),-- Stem
(21265,1869,1,1,-1,522665),-- Iron Ore
(21265,1901,1,1,-1,72191),-- Tiger\'s Eye Earring Stone
(21265,57,264,530,0,700000),-- Adena
(21265,83,1,1,1,13),-- Sword of Magic
(21265,124,1,1,1,10),-- Two-Handed Sword
(21265,127,1,1,1,10),-- Crimson Sword
(21265,130,1,1,1,10),-- Elven Sword
(21265,144,1,1,1,13),-- Sword of Occult
(21265,2026,1,1,1,558),-- Two-Handed Sword Edge
(21265,1864,1,1,2,142857),-- Stem
(21265,1869,1,1,2,76923),-- Iron Ore
(21265,1878,1,1,2,20833),-- Braided Hemp
(21265,1884,1,1,2,33333),-- Cord
(21265,2154,1,1,2,8621),-- Recipe: Elven Earring
(21268,1940,1,1,-1,145098),-- Iron Boots Design
(21268,2026,1,1,-1,11701),-- Two-Handed Sword Edge
(21268,2156,1,1,-1,77773),-- Recipe: Elven Necklace
(21268,57,493,986,0,700000),-- Adena
(21268,278,1,1,1,54),-- Gastraphetes
(21268,2029,1,1,1,3083),-- Gastraphetes Shaft
(21268,1341,20,60,2,142857),-- Bone Arrow
(21268,1867,1,1,2,125000),-- Animal Skin
(21268,1872,1,1,2,100000),-- Animal Bone
(21268,1882,1,1,2,15385),-- Leather
(21268,1894,1,1,2,2564),-- Crafted Leather
(21268,2186,1,1,2,2564),-- Recipe: Brigandine Tunic
(21268,6037,1,3,2,62500),-- Waking Scroll
(21269,1825,1,1,-1,79006),-- Recipe: Oriharukon
(21269,1902,1,1,-1,112065),-- Elven Earring Beads
(21269,2145,1,1,-1,79006),-- Recipe: Mithril Alloy
(21269,57,561,1118,0,700000),-- Adena
(21269,129,1,1,1,41),-- Sword of Revolution
(21269,2034,1,1,1,3021),-- Sword of Revolution Blade
(21269,1825,1,1,2,7407),-- Recipe: Oriharukon
(21269,1867,1,1,2,200000),-- Animal Skin
(21269,1872,1,1,2,166667),-- Animal Bone
(21269,1875,1,1,2,8696),-- Stone of Purity
(21269,1881,1,1,2,20000),-- Coarse Bone Powder
(21270,1902,1,1,-1,129174),-- Elven Earring Beads
(21270,1903,1,1,-1,212668),-- Elven Ring Piece
(21270,6036,1,1,-1,42498),-- Greater Magic Haste Potion
(21270,57,648,1289,0,700000),-- Adena
(21270,88,1,1,1,10),-- Morning Star
(21270,90,1,1,1,10),-- Goat Head Staff
(21270,158,1,1,1,10),-- Tarbar
(21270,169,1,1,1,10),-- Skull Breaker
(21270,172,1,1,1,10),-- Heavy Bone Club
(21270,2031,1,1,1,668),-- Morning Star Head
(21270,2032,1,1,1,609),-- Goat Head Staff Head
(21270,2035,1,1,1,668),-- Tarbar Head
(21270,2036,1,1,1,668),-- Skull Breaker Head
(21270,2037,1,1,1,668),-- Heavy Bone Club Head
(21270,1868,1,1,2,250000),-- Thread
(21270,1873,1,1,2,76923),-- Silver Nugget
(21270,1889,1,1,2,6579),-- Compound Braid
(21270,2145,1,1,2,6579),-- Recipe: Mithril Alloy
(21270,5549,1,1,2,14493),-- Metallic Thread
(21271,1864,1,3,-1,754767),-- Stem
(21271,1869,1,1,-1,754767),-- Iron Ore
(21271,1901,1,1,-1,104250),-- Tiger\'s Eye Earring Stone
(21271,57,385,758,0,700000),-- Adena
(21271,129,1,1,1,28),-- Sword of Revolution
(21271,2034,1,1,1,2062),-- Sword of Revolution Blade
(21271,1864,1,1,2,250000),-- Stem
(21271,1869,1,1,2,83333),-- Iron Ore
(21271,1878,1,1,2,16667),-- Braided Hemp
(21271,1884,1,1,2,76923),-- Cord
(21271,2154,1,1,2,16667),-- Recipe: Elven Earring
(21272,1864,1,7,-1,892175),-- Stem
(21272,1869,1,3,-1,669131),-- Iron Ore
(21272,1901,1,1,-1,184843),-- Tiger\'s Eye Earring Stone
(21272,57,696,1334,0,700000),-- Adena
(21272,129,1,1,1,50),-- Sword of Revolution
(21272,2034,1,1,1,3655),-- Sword of Revolution Blade
(21272,1864,3,9,2,90909),-- Stem
(21272,1869,1,1,2,200000),-- Iron Ore
(21272,1878,1,1,2,52632),-- Braided Hemp
(21272,1884,1,1,2,52632),-- Cord
(21272,2154,1,1,2,14814),-- Recipe: Elven Earring
(21273,1864,1,16,-1,861782),-- Stem
(21273,1869,1,7,-1,775604),-- Iron Ore
(21273,1901,1,1,-1,535638),-- Tiger\'s Eye Earring Stone
(21273,57,2003,3917,0,700000),-- Adena
(21273,129,1,1,1,144),-- Sword of Revolution
(21273,2034,1,1,1,10591),-- Sword of Revolution Blade
(21273,1864,3,9,2,133643),-- Stem
(21273,1869,2,6,2,100233),-- Iron Ore
(21273,1878,1,1,2,160371),-- Braided Hemp
(21273,1884,2,6,2,49345),-- Cord
(21273,2154,1,1,2,42926),-- Recipe: Elven Earring
(21274,4087,1,1,-1,33226),-- Stockings of Doom Pattern
(21274,4088,1,1,-1,53512),-- Blue Wolf Helmet Design
(21274,6333,1,1,-1,16256),-- Recipe: Sealed Phoenix Ring(70%)
(21274,57,1223,2498,0,700000),-- Adena
(21274,12341,1,1,1,864),-- Common Item - Earring of Black Ore
(21274,12343,1,1,1,1008),-- Common Item - Ring of Black Ore
(21274,12342,1,1,1,576),-- Common Item - Necklace of Black Ore
(21274,4052,1,1,1,2468),-- Earring of Black Ore Piece
(21274,4053,1,1,1,2988),-- Ring of Black Ore Gemstone
(21274,4054,1,1,1,1373),-- Necklace of Black Ore Beads
(21274,1343,10,30,2,125000),-- Silver Arrow
(21274,1864,1,1,2,166667),-- Stem
(21274,1869,1,1,2,83333),-- Iron Ore
(21274,1878,1,1,2,32258),-- Braided Hemp
(21274,1884,1,1,2,50000),-- Cord
(21274,4039,1,1,2,2415),-- Mold Glue
(21274,5456,1,1,2,29),-- Recipe: Tallum Glaive(60%)
(21274,8730,1,1,200,691),-- Life Stone: level 67
(21274,8740,1,1,200,403),-- Mid-Grade Life Stone: level 67
(21275,4087,1,1,-1,33226),-- Stockings of Doom Pattern
(21275,4088,1,1,-1,53512),-- Blue Wolf Helmet Design
(21275,6333,1,1,-1,16256),-- Recipe: Sealed Phoenix Ring(70%)
(21275,57,1230,2515,0,700000),-- Adena
(21275,12341,1,1,1,708),-- Common Item - Earring of Black Ore
(21275,12343,1,1,1,1048),-- Common Item - Ring of Black Ore
(21275,12342,1,1,1,623),-- Common Item - Necklace of Black Ore
(21275,4052,1,1,1,2468),-- Earring of Black Ore Piece
(21275,4053,1,1,1,2988),-- Ring of Black Ore Gemstone
(21275,4054,1,1,1,1373),-- Necklace of Black Ore Beads
(21275,1343,10,30,2,125000),-- Silver Arrow
(21275,1864,1,1,2,166667),-- Stem
(21275,1869,1,1,2,83333),-- Iron Ore
(21275,1878,1,1,2,35714),-- Braided Hemp
(21275,1884,1,1,2,50000),-- Cord
(21275,4039,1,1,2,3058),-- Mold Glue
(21275,5456,1,1,2,23),-- Recipe: Tallum Glaive(60%)
(21275,8730,1,1,200,822),-- Life Stone: level 67
(21275,8740,1,1,200,227),-- Mid-Grade Life Stone: level 67
(21275,8750,1,1,200,57),-- High-Grade Life Stone: level 67
(21275,8760,1,1,200,113),-- Top-Grade Life Stone: level 67
(21276,4087,1,1,-1,33226),-- Stockings of Doom Pattern
(21276,4088,1,1,-1,53512),-- Blue Wolf Helmet Design
(21276,6333,1,1,-1,16256),-- Recipe: Sealed Phoenix Ring(70%)
(21276,57,1216,2484,0,700000),-- Adena
(21276,12341,1,1,1,473),-- Common Item - Earring of Black Ore
(21276,12343,1,1,1,946),-- Common Item - Ring of Black Ore
(21276,12342,1,1,1,697),-- Common Item - Necklace of Black Ore
(21276,4052,1,1,1,2468),-- Earring of Black Ore Piece
(21276,4053,1,1,1,2988),-- Ring of Black Ore Gemstone
(21276,4054,1,1,1,1373),-- Necklace of Black Ore Beads
(21276,1343,10,30,2,111111),-- Silver Arrow
(21276,1864,1,1,2,166667),-- Stem
(21276,1869,1,1,2,83333),-- Iron Ore
(21276,1878,1,1,2,32258),-- Braided Hemp
(21276,1884,1,1,2,50000),-- Cord
(21276,4039,1,1,2,2688),-- Mold Glue
(21276,5456,1,1,2,23),-- Recipe: Tallum Glaive(60%)
(21276,8730,1,1,200,1295),-- Life Stone: level 67
(21276,8740,1,1,200,349),-- Mid-Grade Life Stone: level 67
(21276,8750,1,1,200,100),-- High-Grade Life Stone: level 67
(21278,729,1,1,-1,163),-- Scroll: Enchant Weapon (Grade A)
(21278,4040,1,1,-1,29260),-- Mold Lubricant
(21278,5478,1,1,-1,12302),-- Sealed Dark Crystal Leather Armor Pattern
(21278,57,1268,2577,0,700000),-- Adena
(21278,11957,1,1,1,74),-- Common Item - Tallum Blade
(21278,11952,1,1,1,74),-- Common Item - Elemental Sword
(21278,11954,1,1,1,74),-- Common Item - Infernal Master
(21278,8331,1,1,1,373),-- Infernal Master Blade
(21278,5547,1,1,1,368),-- Elemental Sword Edge
(21278,5548,1,1,1,368),-- Tallum Blade Edge
(21278,1865,1,1,2,100000),-- Varnish
(21278,1870,1,1,2,90909),-- Coal
(21278,1875,1,1,2,6452),-- Stone of Purity
(21278,1879,1,1,2,16667),-- Cokes
(21278,4040,1,1,2,2732),-- Mold Lubricant
(21278,6339,1,1,2,948),-- Recipe: Sealed Majestic Ring(70%)
(21278,8730,1,1,200,948),-- Life Stone: level 67
(21278,8740,1,1,200,223),-- Mid-Grade Life Stone: level 67
(21278,8750,1,1,200,112),-- High-Grade Life Stone: level 67
(21279,729,1,1,-1,163),-- Scroll: Enchant Weapon (Grade A)
(21279,4040,1,1,-1,29260),-- Mold Lubricant
(21279,5478,1,1,-1,12302),-- Sealed Dark Crystal Leather Armor Pattern
(21279,57,1260,2562,0,700000),-- Adena
(21279,11957,1,1,1,189),-- Common Item - Tallum Blade
(21279,11952,1,1,1,189),-- Common Item - Elemental Sword
(21279,11954,1,1,1,189),-- Common Item - Infernal Master
(21279,8331,1,1,1,373),-- Infernal Master Blade
(21279,5547,1,1,1,368),-- Elemental Sword Edge
(21279,5548,1,1,1,368),-- Tallum Blade Edge
(21279,1865,1,1,2,90909),-- Varnish
(21279,1870,1,1,2,90909),-- Coal
(21279,1875,1,1,2,5988),-- Stone of Purity
(21279,1879,1,1,2,16667),-- Cokes
(21279,4040,1,1,2,1931),-- Mold Lubricant
(21279,6339,1,1,2,704),-- Recipe: Sealed Majestic Ring(70%)
(21279,8730,1,1,200,1112),-- Life Stone: level 67
(21279,8740,1,1,200,341),-- Mid-Grade Life Stone: level 67
(21280,729,1,1,-1,163),-- Scroll: Enchant Weapon (Grade A)
(21280,4040,1,1,-1,29260),-- Mold Lubricant
(21280,5478,1,1,-1,12302),-- Sealed Dark Crystal Leather Armor Pattern
(21280,57,1276,2593,0,700000),-- Adena
(21280,11957,1,1,1,12),-- Common Item - Tallum Blade
(21280,11952,1,1,1,12),-- Common Item - Elemental Sword
(21280,11954,1,1,1,12),-- Common Item - Infernal Master
(21280,8331,1,1,1,373),-- Infernal Master Blade
(21280,5547,1,1,1,368),-- Elemental Sword Edge
(21280,5548,1,1,1,368),-- Tallum Blade Edge
(21280,1865,1,1,2,100000),-- Varnish
(21280,1870,1,1,2,100000),-- Coal
(21280,1875,1,1,2,6173),-- Stone of Purity
(21280,1879,1,1,2,16129),-- Cokes
(21280,4040,1,1,2,2288),-- Mold Lubricant
(21280,6339,1,1,2,786),-- Recipe: Sealed Majestic Ring(70%)
(21280,8730,1,1,200,1000),-- Life Stone: level 67
(21280,8740,1,1,200,250),-- Mid-Grade Life Stone: level 67
(21280,8750,1,1,200,36),-- High-Grade Life Stone: level 67
(21282,729,1,1,-1,171),-- Scroll: Enchant Weapon (Grade A)
(21282,4040,1,1,-1,30805),-- Mold Lubricant
(21282,5536,1,1,-1,5240),-- Bloody Orchid Head
(21282,57,1309,2658,0,700000),-- Adena
(21282,11957,1,1,1,17),-- Common Item - Tallum Blade
(21282,11952,1,1,1,17),-- Common Item - Elemental Sword
(21282,11954,1,1,1,17),-- Common Item - Infernal Master
(21282,8331,1,1,1,800),-- Infernal Master Blade
(21282,5547,1,1,1,387),-- Elemental Sword Edge
(21282,5548,1,1,1,387),-- Tallum Blade Edge
(21282,730,1,1,2,64),-- Scroll: Enchant Armor (Grade A)
(21282,1865,1,1,2,100000),-- Varnish
(21282,1870,1,1,2,100000),-- Coal
(21282,1875,1,1,2,5988),-- Stone of Purity
(21282,1879,1,1,2,17857),-- Cokes
(21282,4040,1,1,2,2037),-- Mold Lubricant
(21282,5456,1,1,2,26),-- Recipe: Tallum Glaive(60%)
(21282,8730,1,1,200,1495),-- Life Stone: level 67
(21282,8740,1,1,200,747),-- Mid-Grade Life Stone: level 67
(21282,8750,1,1,200,26),-- High-Grade Life Stone: level 67
(21283,729,1,1,-1,171),-- Scroll: Enchant Weapon (Grade A)
(21283,4040,1,1,-1,30805),-- Mold Lubricant
(21283,5536,1,1,-1,5240),-- Bloody Orchid Head
(21283,57,1318,2676,0,700000),-- Adena
(21283,11957,1,1,1,18),-- Common Item - Tallum Blade
(21283,11952,1,1,1,18),-- Common Item - Elemental Sword
(21283,11954,1,1,1,230),-- Common Item - Infernal Master
(21283,8331,1,1,1,855),-- Infernal Master Blade
(21283,5547,1,1,1,387),-- Elemental Sword Edge
(21283,5548,1,1,1,387),-- Tallum Blade Edge
(21283,730,1,1,2,64),-- Scroll: Enchant Armor (Grade A)
(21283,1865,1,1,2,100000),-- Varnish
(21283,1870,1,1,2,90909),-- Coal
(21283,1875,1,1,2,5882),-- Stone of Purity
(21283,1879,1,1,2,15625),-- Cokes
(21283,4040,1,1,2,2778),-- Mold Lubricant
(21283,5456,1,1,2,53),-- Recipe: Tallum Glaive(60%)
(21283,8730,1,1,200,1362),-- Life Stone: level 67
(21283,8740,1,1,200,407),-- Mid-Grade Life Stone: level 67
(21283,8750,1,1,200,35),-- High-Grade Life Stone: level 67
(21284,729,1,1,-1,171),-- Scroll: Enchant Weapon (Grade A)
(21284,4040,1,1,-1,30805),-- Mold Lubricant
(21284,5536,1,1,-1,5240),-- Bloody Orchid Head
(21284,57,1302,2643,0,700000),-- Adena
(21284,11957,1,1,1,83),-- Common Item - Tallum Blade
(21284,11952,1,1,1,14),-- Common Item - Elemental Sword
(21284,11954,1,1,1,28),-- Common Item - Infernal Master
(21284,8331,1,1,1,206),-- Infernal Master Blade
(21284,5547,1,1,1,387),-- Elemental Sword Edge
(21284,5548,1,1,1,387),-- Tallum Blade Edge
(21284,730,1,1,2,64),-- Scroll: Enchant Armor (Grade A)
(21284,1865,1,1,2,100000),-- Varnish
(21284,1870,1,1,2,100000),-- Coal
(21284,1875,1,1,2,5747),-- Stone of Purity
(21284,1879,1,1,2,15873),-- Cokes
(21284,4040,1,1,2,1623),-- Mold Lubricant
(21284,5456,1,1,2,55),-- Recipe: Tallum Glaive(60%)
(21284,8730,1,1,200,922),-- Life Stone: level 67
(21284,8740,1,1,200,289),-- Mid-Grade Life Stone: level 67
(21284,8750,1,1,200,14),-- High-Grade Life Stone: level 67
(21286,1889,1,1,-1,103976),-- Compound Braid
(21286,5549,1,1,-1,155963),-- Metallic Thread
(21286,6343,1,1,-1,33347),-- Sealed Phoenix Necklace Beads
(21286,57,1351,2748,0,700000),-- Adena
(21286,12221,1,1,1,348),-- Common Item - Sealed Dark Crystal Gloves
(21286,12224,1,1,1,560),-- Common Item - Sealed Dark Crystal Boots
(21286,12229,1,1,1,602),-- Common Item - Sealed Tallum Gloves
(21286,12231,1,1,1,665),-- Common Item - Sealed Tallum Boots
(21286,5496,1,1,1,1351),-- Sealed Dark Crystal Boots Lining
(21286,5497,1,1,1,1351),-- Sealed Tallum Boots Lining
(21286,5508,1,1,1,1351),-- Sealed Dark Crystal Gloves Design
(21286,5509,1,1,1,1351),-- Sealed Tallum Gloves Design
(21286,1868,1,1,2,200000),-- Thread
(21286,1873,1,1,2,38462),-- Silver Nugget
(21286,1889,1,1,2,6061),-- Compound Braid
(21286,4044,1,1,2,3546),-- Thons
(21286,5440,1,1,2,11),-- Recipe: Elysian(60%)
(21286,5549,1,1,2,9901),-- Metallic Thread
(21286,8730,1,1,200,1245),-- Life Stone: level 67
(21286,8740,1,1,200,317),-- Mid-Grade Life Stone: level 67
(21286,8750,1,1,200,32),-- High-Grade Life Stone: level 67
(21286,8760,1,1,200,11),-- Top-Grade Life Stone: level 67
(21287,1889,1,1,-1,103976),-- Compound Braid
(21287,5549,1,2,-1,155963),-- Metallic Thread
(21287,6343,1,1,-1,33347),-- Sealed Phoenix Necklace Beads
(21287,57,1343,2732,0,700000),-- Adena
(21287,12221,1,1,1,543),-- Common Item - Sealed Dark Crystal Gloves
(21287,12224,1,1,1,572),-- Common Item - Sealed Dark Crystal Boots
(21287,12229,1,1,1,572),-- Common Item - Sealed Tallum Gloves
(21287,12231,1,1,1,514),-- Common Item - Sealed Tallum Boots
(21287,5496,1,1,1,1351),-- Sealed Dark Crystal Boots Lining
(21287,5497,1,1,1,1351),-- Sealed Tallum Boots Lining
(21287,5508,1,1,1,1351),-- Sealed Dark Crystal Gloves Design
(21287,5509,1,1,1,1351),-- Sealed Tallum Gloves Design
(21287,1868,1,1,2,200000),-- Thread
(21287,1873,1,1,2,41667),-- Silver Nugget
(21287,1889,1,1,2,6452),-- Compound Braid
(21287,4044,1,1,2,2985),-- Thons
(21287,5440,1,1,2,51),-- Recipe: Elysian(60%)
(21287,5549,1,1,2,10417),-- Metallic Thread
(21287,8730,1,1,200,1130),-- Life Stone: level 67
(21287,8740,1,1,200,242),-- Mid-Grade Life Stone: level 67
(21287,8750,1,1,200,15),-- High-Grade Life Stone: level 67
(21287,8760,1,1,200,7),-- Top-Grade Life Stone: level 67
(21288,1889,1,1,-1,103976),-- Compound Braid
(21288,5549,1,1,-1,155963),-- Metallic Thread
(21288,6343,1,1,-1,33347),-- Sealed Phoenix Necklace Beads
(21288,57,1360,2766,0,700000),-- Adena
(21288,12221,1,1,1,609),-- Common Item - Sealed Dark Crystal Gloves
(21288,12224,1,1,1,591),-- Common Item - Sealed Dark Crystal Boots
(21288,12229,1,1,1,711),-- Common Item - Sealed Tallum Gloves
(21288,12231,1,1,1,369),-- Common Item - Sealed Tallum Boots
(21288,5496,1,1,1,1351),-- Sealed Dark Crystal Boots Lining
(21288,5497,1,1,1,1351),-- Sealed Tallum Boots Lining
(21288,5508,1,1,1,1351),-- Sealed Dark Crystal Gloves Design
(21288,5509,1,1,1,1351),-- Sealed Tallum Gloves Design
(21288,1868,1,1,2,200000),-- Thread
(21288,1873,1,1,2,41667),-- Silver Nugget
(21288,1889,1,1,2,6211),-- Compound Braid
(21288,4044,1,1,2,2915),-- Thons
(21288,5440,1,1,2,289),-- Recipe: Elysian(60%)
(21288,5549,1,1,2,10638),-- Metallic Thread
(21288,8730,1,1,200,1238),-- Life Stone: level 67
(21288,8740,1,1,200,240),-- Mid-Grade Life Stone: level 67
(21288,8750,1,1,200,9),-- High-Grade Life Stone: level 67
(21288,8760,1,1,200,18),-- Top-Grade Life Stone: level 67
(21290,730,1,1,-1,1367),-- Scroll: Enchant Armor (Grade A)
(21290,1875,1,1,-1,109369),-- Stone of Purity
(21290,4041,1,1,-1,14266),-- Mold Hardener
(21290,57,1391,2835,0,700000),-- Adena
(21290,11957,1,1,1,20),-- Common Item - Tallum Blade
(21290,11952,1,1,1,20),-- Common Item - Elemental Sword
(21290,11954,1,1,1,20),-- Common Item - Infernal Master
(21290,8331,1,1,1,274),-- Infernal Master Blade
(21290,5547,1,1,1,413),-- Elemental Sword Edge
(21290,5548,1,1,1,413),-- Tallum Blade Edge
(21290,1808,1,1,2,36),-- Recipe: Soulshot: S Grade
(21290,1867,1,1,2,142857),-- Animal Skin
(21290,1872,1,1,2,142857),-- Animal Bone
(21290,1875,1,1,2,5495),-- Stone of Purity
(21290,1881,1,1,2,11364),-- Coarse Bone Powder
(21290,4041,1,1,2,1374),-- Mold Hardener
(21290,8730,1,1,200,1433),-- Life Stone: level 67
(21290,8740,1,1,200,209),-- Mid-Grade Life Stone: level 67
(21291,730,1,1,-1,1367),-- Scroll: Enchant Armor (Grade A)
(21291,1875,1,1,-1,109369),-- Stone of Purity
(21291,4041,1,1,-1,14266),-- Mold Hardener
(21291,57,1383,2819,0,700000),-- Adena
(21291,11957,1,1,1,20),-- Common Item - Tallum Blade
(21291,11952,1,1,1,20),-- Common Item - Elemental Sword
(21291,11954,1,1,1,20),-- Common Item - Infernal Master
(21291,8331,1,1,1,274),-- Infernal Master Blade
(21291,5547,1,1,1,413),-- Elemental Sword Edge
(21291,5548,1,1,1,413),-- Tallum Blade Edge
(21291,1808,1,1,2,36),-- Recipe: Soulshot: S Grade
(21291,1867,1,1,2,142857),-- Animal Skin
(21291,1872,1,1,2,142857),-- Animal Bone
(21291,1875,1,1,2,7692),-- Stone of Purity
(21291,1881,1,1,2,12821),-- Coarse Bone Powder
(21291,4041,1,1,2,8013),-- Mold Hardener
(21291,8730,1,1,200,771),-- Life Stone: level 67
(21291,8740,1,1,200,297),-- Mid-Grade Life Stone: level 67
(21291,8750,1,1,200,59),-- High-Grade Life Stone: level 67
(21292,730,1,1,-1,1367),-- Scroll: Enchant Armor (Grade A)
(21292,1875,1,1,-1,109369),-- Stone of Purity
(21292,4041,1,1,-1,14266),-- Mold Hardener
(21292,57,1400,2853,0,700000),-- Adena
(21292,11957,1,1,1,128),-- Common Item - Tallum Blade
(21292,11952,1,1,1,26),-- Common Item - Elemental Sword
(21292,11954,1,1,1,26),-- Common Item - Infernal Master
(21292,8331,1,1,1,274),-- Infernal Master Blade
(21292,5547,1,1,1,413),-- Elemental Sword Edge
(21292,5548,1,1,1,413),-- Tallum Blade Edge
(21292,1808,1,1,2,36),-- Recipe: Soulshot: S Grade
(21292,1867,1,1,2,142857),-- Animal Skin
(21292,1872,1,1,2,142857),-- Animal Bone
(21292,1875,1,1,2,7246),-- Stone of Purity
(21292,1881,1,1,2,12821),-- Coarse Bone Powder
(21292,4041,1,1,2,638),-- Mold Hardener
(21292,5456,1,1,2,77),-- Recipe: Tallum Glaive(60%)
(21292,8730,1,1,200,1225),-- Life Stone: level 67
(21292,8740,1,1,200,485),-- Mid-Grade Life Stone: level 67
(21292,8750,1,1,200,77),-- High-Grade Life Stone: level 67
(21292,8760,1,1,200,26),-- Top-Grade Life Stone: level 67
(21294,5442,1,1,-1,1736),-- Recipe: Soul Bow(60%)
(21294,5486,1,1,-1,27729),-- Sealed Dark Crystal Robe Fabric
(21294,5489,1,1,-1,61851),-- Sealed Tallum Stockings Fabric
(21294,57,2270,4611,0,700000),-- Adena
(21294,11957,1,1,1,127),-- Common Item - Tallum Blade
(21294,11952,1,1,1,127),-- Common Item - Elemental Sword
(21294,11954,1,1,1,127),-- Common Item - Infernal Master
(21294,8331,1,1,1,373),-- Infernal Master Blade
(21294,5547,1,1,1,1178),-- Elemental Sword Edge
(21294,5548,1,1,1,1178),-- Tallum Blade Edge
(21294,1864,1,3,2,166667),-- Stem
(21294,1869,1,1,2,166667),-- Iron Ore
(21294,1880,1,1,2,18182),-- Steel
(21294,4043,1,1,2,5556),-- Asofe
(21294,5440,1,1,2,126),-- Recipe: Elysian(60%)
(21294,5550,1,1,2,2272),-- Durable Metal Plate
(21294,8730,1,1,200,2899),-- Life Stone: level 67
(21294,8740,1,1,200,252),-- Mid-Grade Life Stone: level 67
(21294,8750,1,1,200,126),-- High-Grade Life Stone: level 67
(21295,5442,1,1,-1,1845),-- Recipe: Soul Bow(60%)
(21295,5547,1,1,-1,16952),-- Elemental Sword Edge
(21295,5548,1,1,-1,16952),-- Tallum Blade Edge
(21295,57,2414,4902,0,700000),-- Adena
(21295,11957,1,1,1,63),-- Common Item - Tallum Blade
(21295,11952,1,1,1,63),-- Common Item - Elemental Sword
(21295,11954,1,1,1,63),-- Common Item - Infernal Master
(21295,8331,1,1,1,373),-- Infernal Master Blade
(21295,5547,1,1,1,1252),-- Elemental Sword Edge
(21295,5548,1,1,1,1252),-- Tallum Blade Edge
(21295,1864,1,3,2,333333),-- Stem
(21295,1869,1,3,2,100000),-- Iron Ore
(21295,1880,1,1,2,28571),-- Steel
(21295,4043,1,1,2,10309),-- Asofe
(21295,5448,1,1,2,91),-- Recipe: Soul Separator(60%)
(21295,5550,1,1,2,5000),-- Durable Metal Plate
(21295,10304,1,1,1,945),-- Transform Sealbook - Saber Tooth Tiger
(21296,5440,1,1,-1,2753),-- Recipe: Elysian(60%)
(21296,5482,1,1,-1,98095),-- Sealed Dark Crystalline Leggings Design
(21296,5485,1,1,-1,62497),-- Sealed Tallum Tunic Texture
(21296,57,3598,7319,0,700000),-- Adena
(21296,11955,1,1,1,250),-- Common Item - Carnage Bow
(21296,5535,1,1,1,3746),-- Carnage Bow Stave
(21296,1867,1,3,2,166667),-- Animal Skin
(21296,1872,4,12,2,71429),-- Animal Bone
(21296,1875,1,1,2,24390),-- Stone of Purity
(21296,1881,1,1,2,55556),-- Coarse Bone Powder
(21296,4041,1,1,2,3257),-- Mold Hardener
(21296,5456,1,1,2,136),-- Recipe: Tallum Glaive(60%)
(21296,10304,1,1,1,1418),-- Transform Sealbook - Saber Tooth Tiger
(21297,5450,1,1,-1,2771),-- Recipe: Dragon Grinder(60%)
(21297,5456,1,1,-1,2771),-- Recipe: Tallum Glaive(60%)
(21297,6345,1,1,-1,286205),-- Sealed Phoenix Ring Gemstone
(21297,57,3616,7368,0,700000),-- Adena
(21297,11951,1,1,1,496),-- Common Item - Bloody Orchid
(21297,5536,1,1,1,3759),-- Bloody Orchid Head
(21297,1868,3,9,2,125000),-- Thread
(21297,1873,2,6,2,40000),-- Silver Nugget
(21297,1876,1,1,2,83333),-- Mithril Ore
(21297,1895,1,1,2,111111),-- Metallic Fiber
(21297,4042,1,1,2,6579),-- Enria
(21297,5162,1,1,2,163),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21297,10304,1,1,1,1770),-- Transform Sealbook - Saber Tooth Tiger
(21298,730,1,1,-1,4013),-- Scroll: Enchant Armor (Grade A)
(21298,1895,1,3,-1,687861),-- Metallic Fiber
(21298,5478,1,1,-1,40490),-- Sealed Dark Crystal Leather Armor Pattern
(21298,57,2327,4743,0,700000),-- Adena
(21298,12221,1,1,1,501),-- Common Item - Sealed Dark Crystal Gloves
(21298,12224,1,1,1,903),-- Common Item - Sealed Dark Crystal Boots
(21298,12229,1,1,1,1203),-- Common Item - Sealed Tallum Gloves
(21298,12231,1,1,1,802),-- Common Item - Sealed Tallum Boots
(21298,5496,1,1,1,4169),-- Sealed Dark Crystal Boots Lining
(21298,5497,1,1,1,4169),-- Sealed Tallum Boots Lining
(21298,5508,1,1,1,4169),-- Sealed Dark Crystal Gloves Design
(21298,5509,1,1,1,4169),-- Sealed Tallum Gloves Design
(21298,1344,10,30,2,166667),-- Mithril Arrow
(21298,1868,1,3,2,142857),-- Thread
(21298,1873,1,1,2,55556),-- Silver Nugget
(21298,1876,1,1,2,29412),-- Mithril Ore
(21298,1895,1,1,2,40000),-- Metallic Fiber
(21298,4042,1,1,2,2604),-- Enria
(21298,5380,1,1,2,501),-- Recipe: Sealed Boots of Nightmare(60%)
(21298,5167,1,1,2,91),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21298,8730,1,1,200,903),-- Life Stone: level 67
(21298,8740,1,1,200,200),-- Mid-Grade Life Stone: level 67
(21299,729,1,1,-1,857),-- Scroll: Enchant Weapon (Grade A)
(21299,4041,1,1,-1,67061),-- Mold Hardener
(21299,5508,1,1,-1,180821),-- Sealed Dark Crystal Gloves Design
(21299,57,3724,7602,0,700000),-- Adena
(21299,11957,1,1,1,102),-- Common Item - Tallum Blade
(21299,11952,1,1,1,102),-- Common Item - Elemental Sword
(21299,11954,1,1,1,102),-- Common Item - Infernal Master
(21299,5547,1,1,1,1938),-- Elemental Sword Edge
(21299,5548,1,1,1,1938),-- Tallum Blade Edge
(21299,8331,1,1,1,1114),-- Infernal Master Blade
(21299,1344,40,120,2,71429),-- Mithril Arrow
(21299,1867,1,3,2,100000),-- Animal Skin
(21299,1872,1,3,2,142857),-- Animal Bone
(21299,1875,1,1,2,16393),-- Stone of Purity
(21299,1881,1,1,2,30303),-- Coarse Bone Powder
(21299,4041,1,1,2,1634),-- Mold Hardener
(21299,5382,1,1,2,1326),-- Recipe: Sealed Majestic Boots(60%)
(21299,5157,1,1,2,145),-- Recipe: Soulshot (S) Compressed Package (100%)
(21299,8730,1,1,200,2551),-- Life Stone: level 67
(21299,8740,1,1,200,715),-- Mid-Grade Life Stone: level 67
(21300,1895,1,5,-1,722569),-- Metallic Fiber
(21300,5434,1,1,-1,2810),-- Recipe: Dragon Slayer(60%)
(21300,5509,1,2,-1,177889),-- Sealed Tallum Gloves Design
(21300,57,3664,7480,0,700000),-- Adena
(21300,11946,1,1,1,99),-- Common Item - Dasparion\'s Staff
(21300,11949,1,1,1,99),-- Common Item - Meteor Shower
(21300,8346,1,1,1,1243),-- Hammer Piece of Destroyer
(21300,11958,1,1,1,99),-- Common Item - Destroyer Hammer
(21300,8341,1,1,1,746),-- Spiritual Eye Piece
(21300,11953,1,1,1,99),-- Common Item - Spiritual Eye
(21300,5532,1,1,1,1889),-- Meteor Shower Head
(21300,5543,1,1,1,1907),-- Dasparion\'s Staff Edge
(21300,1895,1,1,2,90909),-- Metallic Fiber
(21300,4040,1,1,2,7407),-- Mold Lubricant
(21300,4042,1,1,2,4762),-- Enria
(21300,4043,1,1,2,10417),-- Asofe
(21300,5162,1,1,2,200),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21300,8730,1,1,200,2907),-- Life Stone: level 67
(21300,8740,1,1,200,529),-- Mid-Grade Life Stone: level 67
(21300,8760,1,1,200,132),-- Top-Grade Life Stone: level 67
(21301,5440,1,1,-1,3372),-- Recipe: Elysian(60%)
(21301,5520,1,1,-1,58709),-- Sealed Dark Crystal Breastplate Pattern
(21301,5521,1,2,-1,41428),-- Sealed Tallum Plate Armor Pattern
(21301,57,4398,8975,0,700000),-- Adena
(21301,11950,1,1,1,550),-- Common Item - Blood Tornado
(21301,5539,1,1,1,4536),-- Blood Tornado Edge
(21301,1344,30,90,2,111111),-- Mithril Arrow
(21301,1868,4,12,2,66667),-- Thread
(21301,1873,1,1,2,111111),-- Silver Nugget
(21301,1889,1,1,2,17857),-- Compound Braid
(21301,4044,1,1,2,7692),-- Thons
(21301,5404,1,1,2,1238),-- Recipe: Sealed Gloves of Nightmare(60%)
(21301,5167,1,1,2,138),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21301,5549,1,1,2,31250),-- Metallic Thread
(21301,8730,1,1,200,3165),-- Life Stone: level 67
(21301,8740,1,1,200,688),-- Mid-Grade Life Stone: level 67
(21302,729,1,1,-1,816),-- Scroll: Enchant Weapon (Grade A)
(21302,1875,1,3,-1,489683),-- Stone of Purity
(21302,4041,1,2,-1,63872),-- Mold Hardener
(21302,57,3552,7231,0,700000),-- Adena
(21302,11957,1,1,1,56),-- Common Item - Tallum Blade
(21302,11952,1,1,1,56),-- Common Item - Elemental Sword
(21302,11954,1,1,1,56),-- Common Item - Infernal Master
(21302,5547,1,1,1,1846),-- Elemental Sword Edge
(21302,5548,1,1,1,1846),-- Tallum Blade Edge
(21302,8331,1,1,1,1206),-- Infernal Master Blade
(21302,1867,1,3,2,111111),-- Animal Skin
(21302,1872,1,3,2,166667),-- Animal Bone
(21302,1875,1,1,2,14925),-- Stone of Purity
(21302,1881,1,1,2,32258),-- Coarse Bone Powder
(21302,4041,1,1,2,2252),-- Mold Hardener
(21302,5382,1,1,2,918),-- Recipe: Sealed Majestic Boots(60%)
(21302,5162,1,1,2,501),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21302,8731,1,1,200,1838),-- Life Stone: level 70
(21302,8741,1,1,200,918),-- Mid-Grade Life Stone: level 70
(21303,5466,1,1,-1,2751),-- Recipe: Sword of Miracles(60%)
(21303,5547,1,1,-1,25271),-- Elemental Sword Edge
(21303,5548,1,1,-1,25271),-- Tallum Blade Edge
(21303,57,3594,7313,0,700000),-- Adena
(21303,11946,1,1,1,88),-- Common Item - Dasparion\'s Staff
(21303,11949,1,1,1,88),-- Common Item - Meteor Shower
(21303,5532,1,1,1,1850),-- Meteor Shower Head
(21303,5543,1,1,1,1867),-- Dasparion\'s Staff Edge
(21303,11958,1,1,1,88),-- Common Item - Destroyer Hammer
(21303,11953,1,1,1,88),-- Common Item - Spiritual Eye
(21303,8341,1,1,1,901),-- Spiritual Eye Piece
(21303,8346,1,1,1,1102),-- Hammer Piece of Destroyer 
(21303,1344,40,120,2,66667),-- Mithril Arrow
(21303,1867,1,3,2,90909),-- Animal Skin
(21303,1872,1,3,2,142857),-- Animal Bone
(21303,1875,1,1,2,15625),-- Stone of Purity
(21303,1881,1,1,2,27027),-- Coarse Bone Powder
(21303,4041,1,1,2,1768),-- Mold Hardener
(21303,5406,1,1,2,1946),-- Recipe: Sealed Majestic Gloves(60%)
(21303,5167,1,1,2,88),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21303,8731,1,1,200,1502),-- Life Stone: level 70
(21303,8741,1,1,200,796),-- Mid-Grade Life Stone: level 70
(21303,8751,1,1,200,88),-- High-Grade Life Stone: level 70
(21304,5440,1,1,-1,4052),-- Recipe: Elysian(60%)
(21304,5542,1,1,-1,36893),-- Halberd Edge
(21304,5543,1,1,-1,37241),-- Dasparion\'s Staff Edge
(21304,57,5287,10780,0,700000),-- Adena
(21304,11946,1,1,1,172),-- Common Item - Dasparion\'s Staff
(21304,11949,1,1,1,172),-- Common Item - Meteor Shower
(21304,8346,1,1,1,1663),-- Hammer Piece of Destroyer
(21304,11958,1,1,1,172),-- Common Item - Destroyer Hammer
(21304,8341,1,1,1,1261),-- Spiritual Eye Piece
(21304,11953,1,1,1,172),-- Common Item - Spiritual Eye
(21304,5532,1,1,1,2724),-- Meteor Shower Head
(21304,5543,1,1,1,2750),-- Dasparion\'s Staff Edge
(21304,1344,20,60,2,250000),-- Mithril Arrow
(21304,1894,1,1,2,12987),-- Crafted Leather
(21304,4039,1,1,2,12195),-- Mold Glue
(21304,4041,1,1,2,2584),-- Mold Hardener
(21304,4044,1,1,2,13699),-- Thons
(21304,5277,1,1,2,240),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21304,8731,1,1,200,2066),-- Life Stone: level 70
(21304,8741,1,1,200,172),-- Mid-Grade Life Stone: level 70
(21304,8751,1,1,200,172),-- High-Grade Life Stone: level 70
(21305,730,1,1,-1,6713),-- Scroll: Enchant Armor (Grade A)
(21305,1877,1,3,-1,322245),-- Adamantite Nugget
(21305,4040,1,2,-1,161122),-- Mold Lubricant
(21305,57,3893,7933,0,700000),-- Adena
(21305,11946,1,1,1,174),-- Common Item - Dasparion\'s Staff
(21305,11949,1,1,1,174),-- Common Item - Meteor Shower
(21305,8346,1,1,1,829),-- Hammer Piece of Destroyer
(21305,11958,1,1,1,174),-- Common Item - Destroyer Hammer
(21305,8341,1,1,1,737),-- Spiritual Eye Piece
(21305,11953,1,1,1,174),-- Common Item - Spiritual Eye
(21305,5532,1,1,1,2006),-- Meteor Shower Head
(21305,5543,1,1,1,2025),-- Dasparion\'s Staff Edge
(21305,1866,1,1,2,200000),-- Suede
(21305,1871,1,3,2,142857),-- Charcoal
(21305,1877,1,1,2,12821),-- Adamantite Nugget
(21305,4040,1,1,2,7299),-- Mold Lubricant
(21305,4041,1,1,2,2611),-- Mold Hardener
(21305,5282,1,1,2,348),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21305,8731,1,1,200,3484),-- Life Stone: level 70
(21305,8741,1,1,200,1045),-- Mid-Grade Life Stone: level 70
(21305,8751,1,1,200,174),-- High-Grade Life Stone: level 70
(21306,5442,1,1,-1,5399),-- Recipe: Soul Bow(60%)
(21306,5547,1,1,-1,49597),-- Elemental Sword Edge
(21306,5548,1,1,-1,49597),-- Tallum Blade Edge
(21306,57,7045,14360,0,700000),-- Adena
(21306,11950,1,1,1,998),-- Common Item - Blood Tornado
(21306,5539,1,1,1,7262),-- Blood Tornado Edge
(21306,1344,100,300,2,55556),-- Mithril Arrow
(21306,1865,1,3,2,142857),-- Varnish
(21306,1870,2,6,2,111111),-- Coal
(21306,1875,1,1,2,31250),-- Stone of Purity
(21306,1879,1,1,2,90909),-- Cokes
(21306,4040,1,1,2,8000),-- Mold Lubricant
(21306,6901,1,1,2,713),-- Recipe: Shining Arrow (100%)
(21306,8731,1,1,200,2994),-- Life Stone: level 70
(21306,8741,1,1,200,2710),-- Mid-Grade Life Stone: level 70
(21307,1808,1,1,-1,4553),-- Recipe: Soulshot: S Grade
(21307,5480,1,1,-1,45971),-- Sealed Leather Armor of Nightmare Fabric
(21307,5481,1,2,-1,45971),-- Sealed Majestic Leather Armor Fabric
(21307,57,4948,10100,0,700000),-- Adena
(21307,11957,1,1,1,121),-- Common Item - Tallum Blade
(21307,11952,1,1,1,241),-- Common Item - Elemental Sword
(21307,11954,1,1,1,121),-- Common Item - Infernal Master
(21307,8331,1,1,1,1628),-- Infernal Master Blade
(21307,5547,1,1,1,2574),-- Elemental Sword Edge
(21307,5548,1,1,1,2574),-- Tallum Blade Edge
(21307,1868,3,9,2,125000),-- Thread
(21307,1873,1,1,2,142857),-- Silver Nugget
(21307,1889,1,1,2,25641),-- Compound Braid
(21307,4044,1,1,2,9901),-- Thons
(21307,5277,1,1,2,121),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21307,5549,1,1,2,37037),-- Metallic Thread
(21307,8731,1,1,200,2890),-- Life Stone: level 70
(21307,8741,1,1,200,362),-- Mid-Grade Life Stone: level 70
(21307,8751,1,1,200,723),-- High-Grade Life Stone: level 70
(21308,1875,1,2,-1,907479),-- Stone of Purity
(21308,3036,1,1,-1,6050),-- Recipe: Spiritshot S
(21308,5487,1,1,-1,61297),-- Sealed Robe of Nightmare Fabric
(21308,57,6577,13410,0,700000),-- Adena
(21308,11946,1,1,1,231),-- Common Item - Dasparion\'s Staff
(21308,11949,1,1,1,231),-- Common Item - Meteor Shower
(21308,8346,1,1,1,1760),-- Hammer Piece of Destroyer
(21308,11958,1,1,1,231),-- Common Item - Destroyer Hammer
(21308,8341,1,1,1,2183),-- Spiritual Eye Piece
(21308,11953,1,1,1,231),-- Common Item - Spiritual Eye
(21308,5532,1,1,1,3390),-- Meteor Shower Head
(21308,5543,1,1,1,3421),-- Dasparion\'s Staff Edge
(21308,1344,40,120,2,142857),-- Mithril Arrow
(21308,1867,1,3,2,200000),-- Animal Skin
(21308,1872,5,15,2,62500),-- Animal Bone
(21308,1875,1,1,2,33333),-- Stone of Purity
(21308,1881,1,1,2,52632),-- Coarse Bone Powder
(21308,4041,1,1,2,3390),-- Mold Hardener
(21308,5282,1,1,2,308),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21308,8731,1,1,200,3086),-- Life Stone: level 70
(21308,8741,1,1,200,925),-- Mid-Grade Life Stone: level 70
(21308,8761,1,1,200,154),-- Top-Grade Life Stone: level 70
(21309,1875,1,2,-1,945811),-- Stone of Purity
(21309,3036,1,1,-1,6305),-- Recipe: Spiritshot S
(21309,5487,1,1,-1,63886),-- Sealed Robe of Nightmare Fabric
(21309,57,6856,13980,0,700000),-- Adena
(21309,11946,1,1,1,41),-- Common Item - Dasparion\'s Staff
(21309,11949,1,1,1,41),-- Common Item - Meteor Shower
(21309,8346,1,1,1,2197),-- Hammer Piece of Destroyer
(21309,11958,1,1,1,41),-- Common Item - Destroyer Hammer
(21309,8341,1,1,1,1757),-- Spiritual Eye Piece
(21309,11953,1,1,1,41),-- Common Item - Spiritual Eye
(21309,5532,1,1,1,3533),-- Meteor Shower Head
(21309,5543,1,1,1,3566),-- Dasparion\'s Staff Edge
(21309,1344,40,120,2,1428574),-- Mithril Arrow
(21309,1867,9,27,2,38462),-- Animal Skin
(21309,1872,5,15,2,58824),-- Animal Bone
(21309,1875,1,1,2,33333),-- Stone of Purity
(21309,1881,1,1,2,62500),-- Coarse Bone Powder
(21309,4041,1,1,2,3745),-- Mold Hardener
(21309,8731,1,1,200,3906),-- Life Stone: level 70
(21309,8741,1,1,200,976),-- Mid-Grade Life Stone: level 70
(21309,8751,1,1,200,488),-- High-Grade Life Stone: level 70
(21309,5282,1,1,2,163); -- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)

INSERT INTO `droplist` VALUES 
(21310,5162,1,1,-1,6226),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21310,5546,1,1,-1,42217),-- Sword of Miracles Edge
(21310,5549,1,3,-1,700478),-- Metallic Thread
(21310,57,5625,11517,0,700000),-- Adena
(21310,11957,1,1,1,76),-- Common Item - Tallum Blade
(21310,11952,1,1,1,76),-- Common Item - Elemental Sword
(21310,11954,1,1,1,152),-- Common Item - Infernal Master
(21310,8331,1,1,1,3134),-- Infernal Master Blade
(21310,5547,1,1,1,3520),-- Elemental Sword Edge
(21310,5548,1,1,1,3520),-- Tallum Blade Edge
(21310,1868,3,9,2,166667),-- Thread
(21310,1873,1,1,2,200000),-- Silver Nugget
(21310,1889,1,1,2,30303),-- Compound Braid
(21310,4044,1,1,2,15625),-- Thons
(21310,5157,1,1,2,76),-- Recipe: Soulshot (S) Compressed Package (100%)
(21310,5549,1,1,2,476191),-- Metallic Thread
(21310,8732,1,1,200,3559),-- Life Stone: level 76
(21310,8742,1,1,200,985),-- Mid-Grade Life Stone: level 76
(21310,8752,1,1,200,76),-- High-Grade Life Stone: level 76
(21311,5162,1,1,-1,5741),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21311,5546,1,1,-1,38925),-- Sword of Miracles Edge
(21311,5549,1,3,-1,645849),-- Metallic Thread
(21311,57,5186,10619,0,700000),-- Adena
(21311,11957,1,1,1,168),-- Common Item - Tallum Blade
(21311,11952,1,1,1,420),-- Common Item - Elemental Sword
(21311,11954,1,1,1,84),-- Common Item - Infernal Master
(21311,8331,1,1,1,3246),-- Infernal Master Blade
(21311,5547,1,1,1,3246),-- Elemental Sword Edge
(21311,5548,1,1,1,3246),-- Tallum Blade Edge
(21311,1868,3,9,2,142857),-- Thread
(21311,1873,1,1,2,166667),-- Silver Nugget
(21311,1889,1,1,2,30303),-- Compound Braid
(21311,4044,1,1,2,15625),-- Thons
(21311,5157,1,1,2,252),-- Recipe: Soulshot (S) Compressed Package (100%)
(21311,5549,1,1,2,43478),-- Metallic Thread
(21311,8732,1,1,200,3610),-- Life Stone: level 76
(21311,8742,1,1,200,841),-- Mid-Grade Life Stone: level 76
(21311,8752,1,1,200,168),-- High-Grade Life Stone: level 76
(21312,729,1,1,-1,1238),-- Scroll: Enchant Weapon (Grade A)
(21312,5448,1,1,-1,4126),-- Recipe: Soul Separator(60%)
(21312,6341,1,1,-1,308150),-- Sealed Phoenix Earring Gemstone
(21312,57,5397,10970,0,700000),-- Adena
(21312,11955,1,1,1,18),-- Common Item - Carnage Bow (no drop info)
(21312,5535,1,1,1,5615),-- Carnage Bow Stave
(21312,1808,1,1,2,513),-- Recipe: Soulshot: S Grade
(21312,1866,1,3,2,90909),-- Suede
(21312,1871,1,3,2,200000),-- Charcoal
(21312,1885,1,1,2,41667),-- High Grade Suede
(21312,4041,1,1,2,3077),-- Mold Hardener
(21312,5220,1,1,2,17544),-- Metal Hardener
(21312,8732,1,1,200,2053),-- Life Stone: level 76
(21312,8742,1,1,200,1538),-- Mid-Grade Life Stone: level 76
(21312,8761,1,1,200,533),-- Top-Grade Life Stone: level 70
(21313,730,1,1,-1,8843),-- Scroll: Enchant Armor (Grade A)
(21313,1895,1,7,-1,757976),-- Metallic Fiber
(21313,4042,1,2,-1,176861),-- Enria
(21313,57,5125,10460,0,700000),-- Adena
(21313,11955,1,1,1,534),-- Common Item - Carnage Bow
(21313,5535,1,1,1,5347),-- Carnage Bow Stave
(21313,1868,3,9,2,111111),-- Thread
(21313,1873,1,1,2,125000),-- Silver Nugget
(21313,1876,1,1,2,66667),-- Mithril Ore
(21313,1895,1,1,2,100000),-- Metallic Fiber
(21313,4042,1,1,2,5076),-- Enria
(21313,5380,1,1,2,2137),-- Recipe: Sealed Boots of Nightmare(60%)
(21313,5272,1,1,2,267),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21313,8730,1,1,200,4274),-- Life Stone: level 67
(21313,8740,1,1,200,1603),-- Mid-Grade Life Stone: level 67
(21314,730,1,1,-1,4944),-- Scroll: Enchant Armor (Grade A)
(21314,6335,1,1,-1,22819),-- Recipe: Sealed Majestic Necklace(70%)
(21314,6341,1,1,-1,164100),-- Sealed Phoenix Earring Gemstone
(21314,57,2869,5842,0,700000),-- Adena
(21314,11950,1,1,1,234),-- Common Item - Blood Tornado
(21314,5539,1,1,1,2956),-- Blood Tornado Edge
(21314,1895,1,1,2,66667),-- Metallic Fiber
(21314,4040,1,1,2,4292),-- Mold Lubricant
(21314,4042,1,1,2,4464),-- Enria
(21314,4043,1,1,2,8065),-- Asofe
(21314,5272,1,1,2,23),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21314,8731,1,1,200,1961),-- Life Stone: level 70
(21314,8741,1,1,200,607),-- Mid-Grade Life Stone: level 70
(21314,8751,1,1,200,47),-- High-Grade Life Stone: level 70
(21315,1889,1,1,-1,412195),-- Compound Braid
(21315,5549,1,2,-1,618292),-- Metallic Thread
(21315,6343,1,2,-1,132198),-- Sealed Phoenix Necklace Beads
(21315,57,2990,6088,0,700000),-- Adena
(21315,11955,1,1,1,332),-- Common Item - Carnage Bow
(21315,5535,1,1,1,3117),-- Carnage Bow Stave
(21315,1344,30,90,2,83333),-- Mithril Arrow
(21315,1868,1,3,2,200000),-- Thread
(21315,1873,1,1,2,83333),-- Silver Nugget
(21315,1889,1,1,2,13699),-- Compound Braid
(21315,4044,1,1,2,6711),-- Thons
(21315,5277,1,1,2,39),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21315,8731,1,1,200,1486),-- Life Stone: level 70
(21315,8741,1,1,200,254),-- Mid-Grade Life Stone: level 70
(21315,8751,1,1,200,117),-- High-Grade Life Stone: level 70
(21315,5549,1,1,2,20833),-- Metallic Thread
(21316,4043,1,2,-1,215511),-- Asofe
(21316,5550,1,1,-1,86204),-- Durable Metal Plate
(21316,6343,1,2,-1,138237),-- Sealed Phoenix Necklace Beads
(21316,57,3124,6369,0,700000),-- Adena
(21316,11951,1,1,1,151),-- Common Item - Bloody Orchid
(21316,5536,1,1,1,3249),-- Bloody Orchid Head
(21316,1344,30,90,2,83333),-- Mithril Arrow
(21316,1864,1,3,2,200000),-- Stem
(21316,1869,2,6,2,52632),-- Iron Ore
(21316,1880,1,1,2,22727),-- Steel
(21316,3036,1,1,2,151),-- Recipe: Spiritshot S
(21316,4043,1,1,2,7042),-- Asofe
(21316,5550,1,1,2,3356),-- Durable Metal Plate
(21316,8731,1,1,200,2088),-- Life Stone: level 70
(21316,8741,1,1,200,424),-- Mid-Grade Life Stone: level 70
(21316,8751,1,1,200,30),-- High-Grade Life Stone: level 70
(21317,5450,1,1,-1,2263),-- Recipe: Dragon Grinder(60%)
(21317,5456,1,1,-1,2263),-- Recipe: Tallum Glaive(60%)
(21317,6345,1,1,-1,233778),-- Sealed Phoenix Ring Gemstone
(21317,57,2953,6019,0,700000),-- Adena
(21317,11950,1,1,1,786),-- Common Item - Blood Tornado
(21317,5539,1,1,1,3045),-- Blood Tornado Edge
(21317,1867,1,3,2,142857),-- Animal Skin
(21317,1872,1,3,2,100000),-- Animal Bone
(21317,1882,1,1,2,47619),-- Leather
(21317,1894,1,1,2,7407),-- Crafted Leather
(21317,3957,1,1,2,124),-- Recipe: Blessed Spiritshot S
(21317,4042,1,1,2,3968),-- Enria
(21317,8731,1,1,200,2481),-- Life Stone: level 70
(21317,8741,1,1,200,455),-- Mid-Grade Life Stone: level 70
(21317,8751,1,1,200,29),-- High-Grade Life Stone: level 70
(21317,8761,1,1,200,29),-- Top-Grade Life Stone: level 70
(21318,730,1,1,-1,5323),-- Scroll: Enchant Armor (Grade A)
(21318,5478,1,1,-1,53713),-- Sealed Dark Crystal Leather Armor Pattern
(21318,5549,1,3,-1,638760),-- Metallic Thread
(21318,57,3087,6292,0,700000),-- Adena
(21318,12221,1,1,1,1321),-- Common Item - Sealed Dark Crystal Gloves
(21318,12224,1,1,1,1098),-- Common Item - Sealed Dark Crystal Boots
(21318,12229,1,1,1,1149),-- Common Item - Sealed Tallum Gloves
(21318,12231,1,1,1,1425),-- Common Item - Sealed Tallum Boots
(21318,5496,1,1,1,5530),-- Sealed Dark Crystal Boots Lining
(21318,5497,1,1,1,5530),-- Sealed Tallum Boots Lining
(21318,5508,1,1,1,5530),-- Sealed Dark Crystal Gloves Design
(21318,5509,1,1,1,5530),-- Sealed Tallum Gloves Design
(21318,1344,30,90,2,90909),-- Mithril Arrow
(21318,1868,1,3,2,200000),-- Thread
(21318,1873,1,1,2,83333),-- Silver Nugget
(21318,1889,1,1,2,12821),-- Compound Braid
(21318,4044,1,1,2,7353),-- Thons
(21318,5157,1,1,2,120),-- Recipe: Soulshot (S) Compressed Package (100%)
(21318,5549,1,1,2,22727),-- Metallic Thread
(21318,8731,1,1,200,1527),-- Life Stone: level 70
(21318,8741,1,1,200,583),-- Mid-Grade Life Stone: level 70
(21318,8751,1,1,200,51),-- High-Grade Life Stone: level 70
(21319,5157,1,1,-1,3204),-- Recipe: Soulshot (S) Compressed Package (100%)
(21319,5495,1,1,-1,109466),-- Sealed Shield of Nightmare Fragment
(21319,5502,1,1,-1,121751),-- Sealed Boots of Nightmare Lining
(21319,57,3482,7105,0,700000),-- Adena
(21319,11955,1,1,1,416),-- Common Item - Carnage Bow
(21319,5535,1,1,1,3633),-- Carnage Bow Stave
(21319,1344,20,60,2,250000),-- Mithril Arrow
(21320,1894,1,1,-1,230170),-- Crafted Leather
(21320,4042,1,1,-1,109331),-- Enria
(21320,5503,1,1,-1,110790),-- Sealed Majestic Boots Lining
(21320,57,3168,6465,0,700000),-- Adena
(21320,11951,1,1,1,360),-- Common Item - Bloody Orchid
(21320,5536,1,1,1,3297),-- Bloody Orchid Head
(21320,1867,1,3,2,166667),-- Animal Skin
(21320,1872,1,3,2,111111),-- Animal Bone
(21320,1882,1,1,2,52632),-- Leather
(21320,1894,1,1,2,8621),-- Crafted Leather
(21320,3036,1,1,2,114),-- Recipe: Spiritshot S
(21320,4042,1,1,2,4065),-- Enria
(21320,8731,1,1,200,1953),-- Life Stone: level 70
(21320,8741,1,1,200,474),-- Mid-Grade Life Stone: level 70
(21320,8751,1,1,200,57),-- High-Grade Life Stone: level 70
(21320,8761,1,1,200,19),-- Top-Grade Life Stone: level 70
(21321,1876,1,3,-1,685417),-- Mithril Ore
(21321,1895,1,3,-1,652778),-- Metallic Fiber
(21321,6346,1,2,-1,223154),-- Sealed Majestic Ring Gemstone
(21321,57,2752,5635,0,700000),-- Adena
(21321,11950,1,1,1,180),-- Common Item - Blood Tornado
(21321,5539,1,1,1,3414),-- Blood Tornado Edge
(21321,1868,1,3,2,250000),-- Thread
(21321,1873,1,1,2,90909),-- Silver Nugget
(21321,1876,1,1,2,45455),-- Mithril Ore
(21321,1895,1,1,2,66667),-- Metallic Fiber
(21321,4042,1,1,2,3937),-- Enria
(21321,5167,1,1,2,150),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21321,8732,1,1,200,1473),-- Life Stone: level 76
(21321,8742,1,1,200,241),-- Mid-Grade Life Stone: level 76
(21321,8752,1,1,200,60),-- High-Grade Life Stone: level 76
(21322,5272,1,1,-1,3046),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21322,5277,1,1,-1,3046),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21322,5480,1,1,-1,30756),-- Sealed Leather Armor of Nightmare Fabric
(21322,57,2752,5635,0,700000),-- Adena
(21322,11955,1,1,1,429),-- Common Item - Carnage Bow
(21322,5535,1,1,1,3454),-- Carnage Bow Stave
(21322,1344,20,60,2,125000),-- Mithril Arrow
(21322,1866,1,1,2,142857),-- Suede
(21322,1871,1,1,2,200000),-- Charcoal
(21322,1877,1,1,2,6667),-- Adamantite Nugget
(21322,4040,1,1,2,3817),-- Mold Lubricant
(21322,4041,1,1,2,2817),-- Mold Hardener
(21322,5272,1,1,2,48),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21322,8732,1,1,200,1383),-- Life Stone: level 76
(21322,8742,1,1,200,382),-- Mid-Grade Life Stone: level 76
(21322,8752,1,1,200,15),-- High-Grade Life Stone: level 76
(21323,1894,1,1,-1,237606),-- Crafted Leather
(21323,4042,1,1,-1,112863),-- Enria
(21323,5481,1,1,-1,30386),-- Sealed Majestic Leather Armor Fabric
(21323,57,2719,5567,0,700000),-- Adena
(21323,11951,1,1,1,302),-- Common Item - Bloody Orchid
(21323,5536,1,1,1,3404),-- Bloody Orchid Head
(21323,1867,1,3,2,142857),-- Animal Skin
(21323,1872,1,3,2,100000),-- Animal Bone
(21323,1882,1,1,2,50000),-- Leather
(21323,1894,1,1,2,7407),-- Crafted Leather
(21323,4042,1,1,2,3497),-- Enria
(21323,5277,1,1,2,43),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21323,8732,1,1,200,1859),-- Life Stone: level 76
(21323,8742,1,1,200,475),-- Mid-Grade Life Stone: level 76
(21324,1808,1,1,-1,2885),-- Recipe: Soulshot: S Grade
(21324,4042,1,1,-1,108175),-- Enria
(21324,5502,1,1,-1,109618),-- Sealed Boots of Nightmare Lining
(21324,57,2610,5333,0,700000),-- Adena
(21324,11957,1,1,1,87),-- Common Item - Tallum Blade
(21324,11952,1,1,1,87),-- Common Item - Elemental Sword
(21324,11954,1,1,1,87),-- Common Item - Infernal Master
(21324,8331,1,1,1,794),-- Infernal Master Blade
(21324,5547,1,1,1,1631),-- Elemental Sword Edge
(21324,5548,1,1,1,1631),-- Tallum Blade Edge
(21324,1344,30,90,2,58824),-- Mithril Arrow
(21324,1808,1,1,2,87),-- Recipe: Soulshot: S Grade
(21324,1868,1,3,2,142857),-- Thread
(21324,1873,1,1,2,58824),-- Silver Nugget
(21324,1876,1,1,2,31250),-- Mithril Ore
(21324,1895,1,1,2,43478),-- Metallic Fiber
(21324,4042,1,1,2,2611),-- Enria
(21324,8732,1,1,200,1131),-- Life Stone: level 76
(21324,8742,1,1,200,696),-- Mid-Grade Life Stone: level 76
(21324,8752,1,1,200,87),-- High-Grade Life Stone: level 76
(21325,3036,1,1,-1,2789),-- Recipe: Spiritshot S
(21325,5503,1,1,-1,105999),-- Sealed Majestic Boots Lining
(21325,5514,1,1,-1,105999),-- Sealed Gloves of Nightmare Design
(21325,57,2524,5157,0,700000),-- Adena
(21325,11946,1,1,1,42),-- Common Item - Dasparion\'s Staff
(21325,11949,1,1,1,42),-- Common Item - Meteor Shower
(21325,8346,1,1,1,831),-- Hammer Piece of Destroyer
(21325,11958,1,1,1,42),-- Common Item - Destroyer Hammer
(21325,8341,1,1,1,787),-- Spiritual Eye Piece
(21325,11953,1,1,1,42),-- Common Item - Spiritual Eye
(21325,5532,1,1,1,1563),-- Meteor Shower Head
(21325,5543,1,1,1,1578),-- Dasparion\'s Staff Edge
(21325,1894,1,1,2,5495),-- Crafted Leather
(21325,3036,1,1,2,168),-- Recipe: Spiritshot S
(21325,4039,1,1,2,6173),-- Mold Glue
(21325,4041,1,1,2,1634),-- Mold Hardener
(21325,4044,1,1,2,6211),-- Thons
(21325,8732,1,1,200,1091),-- Life Stone: level 76
(21325,8742,1,1,200,293),-- Mid-Grade Life Stone: level 76
(21325,8752,1,1,200,42),-- High-Grade Life Stone: level 76
(21326,1877,1,1,-1,251047),-- Adamantite Nugget
(21326,4041,1,1,-1,54575),-- Mold Hardener
(21326,5515,1,1,-1,105999),-- Sealed Majestic Gloves Design
(21326,57,2524,5157,0,700000),-- Adena
(21326,11950,1,1,1,134),-- Common Item - Blood Tornado
(21326,5539,1,1,1,3126),-- Blood Tornado Edge
(21326,1344,20,60,2,76923),-- Mithril Arrow
(21326,1866,1,1,2,90909),-- Suede
(21326,1871,1,1,2,125000),-- Charcoal
(21326,1877,1,1,2,4630),-- Adamantite Nugget
(21326,3957,1,1,2,118),-- Recipe: Blessed Spiritshot S
(21326,4040,1,1,2,3165),-- Mold Lubricant
(21326,4041,1,1,2,678),-- Mold Hardener
(21326,8732,1,1,200,791),-- Life Stone: level 76
(21326,8742,1,1,200,113),-- Mid-Grade Life Stone: level 76
(21326,8752,1,1,200,113),-- High-Grade Life Stone: level 76
(21327,4039,1,1,-1,238154),-- Mold Glue
(21327,5272,1,1,-1,3175),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21327,5533,1,1,-1,19597),-- Elysian Head
(21327,57,2875,5868,0,700000),-- Adena
(21327,11957,1,1,1,192),-- Common Item - Tallum Blade
(21327,11952,1,1,1,192),-- Common Item - Elemental Sword
(21327,11954,1,1,1,192),-- Common Item - Infernal Master
(21327,8331,1,1,1,1666),-- Infernal Master Blade
(21327,5547,1,1,1,1796),-- Elemental Sword Edge
(21327,5548,1,1,1,1796),-- Tallum Blade Edge
(21327,1894,1,1,2,9901),-- Crafted Leather
(21327,4039,1,1,2,8130),-- Mold Glue
(21327,4041,1,1,2,2155),-- Mold Hardener
(21327,4044,1,1,2,8333),-- Thons
(21327,5277,1,1,2,144),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21327,8732,1,1,200,2083),-- Life Stone: level 76
(21327,8742,1,1,200,719),-- Mid-Grade Life Stone: level 76
(21328,5277,1,1,-1,3044),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21328,5534,1,1,-1,18804),-- Soul Bow Stave
(21328,5537,1,1,-1,18791),-- Soul Separator Head
(21328,57,2756,5626,0,700000),-- Adena
(21328,11946,1,1,1,42),-- Common Item - Dasparion\'s Staff
(21328,11949,1,1,1,42),-- Common Item - Meteor Shower
(21328,8346,1,1,1,678),-- Hammer Piece of Destroyer
(21328,11958,1,1,1,42),-- Common Item - Destroyer Hammer
(21328,8341,1,1,1,814),-- Spiritual Eye Piece
(21328,11953,1,1,1,42),-- Common Item - Spiritual Eye
(21328,5532,1,1,1,1706),-- Meteor Shower Head
(21328,5543,1,1,1,1722),-- Dasparion\'s Staff Edge
(21328,1344,30,90,2,62500),-- Mithril Arrow
(21328,1868,1,3,2,142857),-- Thread
(21328,1873,1,1,2,58824),-- Silver Nugget
(21328,1876,1,1,2,27778),-- Mithril Ore
(21328,1895,1,1,2,38462),-- Metallic Fiber
(21328,4042,1,1,2,2146),-- Enria
(21328,5282,1,1,2,170),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21328,6671,1,1,2,16949),-- Deluxe Chest Key - Grade 7
(21328,8732,1,1,200,2033),-- Life Stone: level 76
(21328,8742,1,1,200,170),-- Mid-Grade Life Stone: level 76
(21328,8752,1,1,200,57),-- High-Grade Life Stone: level 76
(21329,5162,1,1,-1,3095),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21329,5167,1,1,-1,3095),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21329,5487,1,1,-1,31362),-- Sealed Robe of Nightmare Fabric
(21329,57,2802,5719,0,700000),-- Adena
(21329,11964,1,1,1,20),-- Common Item - Dark Legion's Edge
(21329,11971,1,1,1,20),-- Common Item - Sword of Miracles 
(21329,11968,1,1,1,20),-- Common Item - Dragon Slayer
(21329,5529,1,1,1,941),-- Dragon Slayer Edge
(21329,5545,1,1,1,941),-- Dark Legion\'s Edge Blade
(21329,5546,1,1,1,1034),-- Sword of Miracles Edge
(21330,1885,1,2,-1,553215),-- High Grade Suede
(21330,4041,1,1,-1,57727),-- Mold Hardener
(21330,5488,1,1,-1,29894),-- Sealed Majestic Robe Fabric
(21330,57,2671,5451,0,700000),-- Adena
(21330,11970,1,1,1,22),-- Common Item - Branch of The Mother Tree
(21330,11974,1,1,1,22),-- Common Item - Elysian
(21330,11969,1,1,1,43),-- Common Item - Flaming Dragon Skull
(21330,11966,1,1,1,22),-- Common Item - Doom Crusher
(21330,8342,1,1,1,782),-- Flaming Dragon Skull Piece
(21330,8349,1,1,1,684),-- Doom Crusher Head
(21330,5533,1,1,1,1346),-- Elysian Head
(21330,5544,1,1,1,1344),-- Branch of The Mother Tree Head
(21330,960,1,1,2,43),-- Scroll: Enchant Armor (Grade S)
(21330,1344,20,60,2,111111),-- Mithril Arrow
(21330,1866,1,1,2,111111),-- Suede
(21330,1871,1,1,2,166667),-- Charcoal
(21330,1885,1,1,2,17085),-- High Grade Suede
(21330,4041,1,1,2,1314),-- Mold Hardener
(21330,5220,1,1,2,5848),-- Metal Hardener
(21330,8732,1,1,200,1443),-- Life Stone: level 76
(21330,8742,1,1,200,259),-- Mid-Grade Life Stone: level 76
(21330,8752,1,1,200,22),-- High-Grade Life Stone: level 76
(21331,1895,1,3,-1,931857),-- Metallic Fiber
(21331,4042,1,1,-1,108717),-- Enria
(21331,5546,1,1,-1,19657),-- Sword of Miracles Edge
(21331,57,2624,5357,0,700000),-- Adena
(21331,11964,1,1,1,46),-- Common Item - Dark Legion's Edge
(21331,11971,1,1,1,46),-- Common Item - Sword of Miracles 
(21331,11968,1,1,1,46),-- Common Item - Dragon Slayer
(21331,5529,1,1,1,882),-- Dragon Slayer Edge
(21331,5545,1,1,1,882),-- Dark Legion\'s Edge Blade
(21331,5546,1,1,1,968),-- Sword of Miracles Edge
(21331,960,1,1,2,92),-- Scroll: Enchant Armor (Grade S)
(21331,1344,30,90,2,76923),-- Mithril Arrow
(21331,1868,1,3,2,166667),-- Thread
(21331,1873,1,1,2,76923),-- Silver Nugget
(21331,1876,1,1,2,33333),-- Mithril Ore
(21331,1895,1,1,2,47619),-- Metallic Fiber
(21331,4042,1,1,2,2681),-- Enria
(21331,8732,1,1,200,1156),-- Life Stone: level 76
(21331,8742,1,1,200,324),-- Mid-Grade Life Stone: level 76
(21331,8752,1,1,200,37),-- High-Grade Life Stone: level 76
(21332,5282,1,1,-1,3247),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21332,6342,1,1,-1,165490),-- Sealed Majestic Earring Gemstone
(21332,6344,1,1,-1,107096),-- Sealed Majestic Necklace Beads
(21332,57,2939,6000,0,700000),-- Adena
(21332,11970,1,1,1,22),-- Common Item - Branch of The Mother Tree
(21332,11974,1,1,1,22),-- Common Item - Elysian
(21332,11969,1,1,1,22),-- Common Item - Flaming Dragon Skull
(21332,11966,1,1,1,22),-- Common Item - Doom Crusher
(21332,8342,1,1,1,594),-- Flaming Dragon Skull Piece
(21332,8349,1,1,1,594),-- Doom Crusher Head
(21332,5533,1,1,1,1481),-- Elysian Head
(21332,5544,1,1,1,1479),-- Branch of The Mother Tree Head
(21332,959,1,1,2,18),-- Scroll: Enchant Weapon (Grade S)
(21332,1894,1,1,2,8696),-- Crafted Leather
(21332,4039,1,1,2,7874),-- Mold Glue
(21332,4041,1,1,2,2375),-- Mold Hardener
(21332,4044,1,1,2,9009),-- Thons
(21332,8732,1,1,200,1894),-- Life Stone: level 76
(21332,8742,1,1,200,1056),-- Mid-Grade Life Stone: level 76
(21332,8752,1,1,200,44),-- High-Grade Life Stone: level 76
(21333,1808,1,1,-1,3114),-- Recipe: Soulshot: S Grade
(21333,3036,1,1,-1,3114),-- Recipe: Spiritshot S
(21333,5503,1,1,-1,118333),-- Sealed Majestic Boots Lining
(21333,57,2813,5760,0,700000),-- Adena
(21333,11970,1,1,1,43),-- Common Item - Branch of The Mother Tree
(21333,11974,1,1,1,43),-- Common Item - Elysian
(21333,11969,1,1,1,43),-- Common Item - Flaming Dragon Skull
(21333,11966,1,1,1,43),-- Common Item - Doom Crusher
(21333,8342,1,1,1,466),-- Flaming Dragon Skull Piece
(21333,8349,1,1,1,622),-- Doom Crusher Head
(21333,5533,1,1,1,1420),-- Elysian Head
(21333,5544,1,1,1,1418),-- Branch of The Mother Tree Head
(21333,959,1,1,2,15),-- Scroll: Enchant Weapon (Grade S)
(21333,1864,4,12,2,58824),-- Stem
(21333,1865,2,6,2,45455),-- Varnish
(21333,1866,3,9,2,26316),-- Suede
(21333,1868,1,3,2,142857),-- Thread
(21333,1873,1,1,2,90909),-- Silver Nugget
(21333,8732,1,1,200,1538),-- Life Stone: level 76
(21333,8742,1,1,200,684),-- Mid-Grade Life Stone: level 76
(21333,8752,1,1,200,57),-- High-Grade Life Stone: level 76
(21334,5480,1,1,-1,42540),-- Sealed Leather Armor of Nightmare Fabric
(21334,5514,1,1,-1,160113),-- Sealed Gloves of Nightmare Design
(21334,57,3806,7794,0,700000),-- Adena
(21334,11967,1,1,1,187),-- Common Item - Dragon Grinder
(21334,5538,1,1,1,3841),-- Dragon Grinder Edge
(21334,960,1,1,2,187),-- Scroll: Enchant Armor (Grade S)
(21334,1344,40,120,2,76923),-- Mithril Arrow
(21334,1867,1,3,2,111111),-- Animal Skin
(21334,1869,2,6,2,55556),-- Iron Ore
(21334,1870,3,9,2,41667),-- Coal
(21334,1871,3,9,2,32258),-- Charcoal
(21334,1872,8,24,2,22222),-- Animal Bone
(21334,6672,1,1,2,23810),-- Deluxe Chest Key - Grade 8
(21334,8732,1,1,200,2618),-- Life Stone: level 76
(21334,8742,1,1,200,374),-- Mid-Grade Life Stone: level 76
(21335,1889,1,1,-1,451301),-- Compound Braid
(21335,5272,1,1,-1,3009),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21335,5495,1,1,-1,102794),-- Sealed Shield of Nightmare Fragment
(21335,57,2718,5566,0,700000),-- Adena
(21335,11976,1,1,1,227),-- Common Item - Tallum Glaive
(21335,5541,1,1,1,2742),-- Tallum Glaive Edge
(21335,959,1,1,2,113),-- Scroll: Enchant Weapon (Grade S)
(21335,1868,1,3,2,200000),-- Thread
(21335,1873,1,1,2,90909),-- Silver Nugget
(21335,1889,1,1,2,12821),-- Compound Braid
(21335,4044,1,1,2,7042),-- Thons
(21335,5549,1,1,2,23256),-- Metallic Thread
(21335,8732,1,1,200,2494),-- Life Stone: level 76
(21335,8742,1,1,200,227),-- Mid-Grade Life Stone: level 76
(21335,8752,1,1,200,57),-- High-Grade Life Stone: level 76
(21336,5277,1,1,-1,3537),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21336,5502,1,1,-1,134421),-- Sealed Boots of Nightmare Lining
(21336,5503,1,2,-1,134421),-- Sealed Majestic Boots Lining
(21336,57,3195,6545,0,700000),-- Adena
(21336,12265,1,1,1,570),-- Common Item - Sealed Leather Armor of Nightmare
(21336,12258,1,1,1,443),-- Common Item - Sealed Majestic Leather Armor
(21336,5480,1,1,1,2638),-- Sealed Leather Armor of Nightmare Fabric
(21336,5481,1,1,1,2638),-- Sealed Majestic Leather Armor Fabric
(21336,960,1,1,2,127),-- Scroll: Enchant Armor (Grade S)
(21336,1344,40,120,2,66667),-- Mithril Arrow
(21336,1864,5,15,2,50000),-- Stem
(21336,1865,3,9,2,43478),-- Varnish
(21336,1866,3,9,2,25000),-- Suede
(21336,1868,1,3,2,166667),-- Thread
(21336,1873,1,1,2,83333),-- Silver Nugget
(21336,8732,1,1,200,1901),-- Life Stone: level 76
(21336,8742,1,1,200,380),-- Mid-Grade Life Stone: level 76
(21336,8752,1,1,200,127),-- High-Grade Life Stone: level 76
(21337,959,1,1,-1,295),-- Scroll: Enchant Weapon (Grade S)
(21337,1885,1,1,-1,614531),-- High Grade Suede
(21337,5220,1,1,-1,294975),-- Metal Hardener
(21337,57,2968,6057,0,700000),-- Adena
(21337,12266,1,1,1,215),-- Common Item - Sealed Robe of Nightmare
(21337,12259,1,1,1,215),-- Common Item - Sealed Majestic Robe
(21337,5487,1,1,1,2453),-- Sealed Robe of Nightmare Fabric
(21337,5488,1,1,1,2453),-- Sealed Majestic Robe Fabric
(21337,959,1,1,2,15),-- Scroll: Enchant Weapon (Grade S)
(21337,1866,1,1,2,166667),-- Suede
(21337,1871,1,1,2,250000),-- Charcoal
(21337,1885,1,1,2,21739),-- High Grade Suede
(21337,4041,1,1,2,2584),-- Mold Hardener
(21337,5220,1,1,2,10753),-- Metal Hardener
(21337,8732,1,1,200,4310),-- Life Stone: level 76
(21337,8742,1,1,200,431),-- Mid-Grade Life Stone: level 76
(21338,6688,1,1,-1,20974),-- Forgotten Blade Edge
(21338,6689,1,1,-1,20579),-- Basalt Battlehammer Head
(21338,6690,1,1,-1,21014),-- Imperial Staff Head
(21338,57,3905,7970,0,700000),-- Adena
(21338,12350,1,1,1,356),-- Common Item - Sealed Majestic Necklace
(21338,12352,1,1,1,702),-- Common Item - Sealed Majestic Earring
(21338,12351,1,1,1,1404),-- Common Item - Sealed Majestic Ring
(21338,6342,1,1,1,10822),-- Sealed Majestic Earring Gemstone
(21338,6344,1,1,1,7003),-- Sealed Majestic Necklace Beads
(21338,6346,1,1,1,15553),-- Sealed Majestic Ring Gemstone
(21338,960,1,1,2,164),-- Scroll: Enchant Armor (Grade S)
(21338,1344,30,90,2,111111),-- Mithril Arrow
(21338,1864,5,15,2,66667),-- Stem
(21338,1869,2,6,2,66667),-- Iron Ore
(21338,1880,1,1,2,32258),-- Steel
(21338,4043,1,1,2,10526),-- Asofe
(21338,5550,1,1,2,1404),-- Durable Metal Plate
(21338,8732,1,1,200,2809),-- Life Stone: level 76
(21338,8752,1,1,200,480),-- High-Grade Life Stone: level 76
(21339,6691,1,1,-1,15980),-- Angel Slayer Blade
(21339,6693,1,1,-1,16269),-- Dragon Hunter Axe Blade
(21339,6694,1,1,-1,16269),-- Saint Spear Blade
(21339,57,2975,6071,0,700000),-- Adena
(21339,11973,1,1,1,135),-- Common Item - Soul Separator
(21339,5537,1,1,1,2995),-- Soul Separator Head
(21339,960,1,1,2,90),-- Scroll: Enchant Armor (Grade S)
(21339,1344,40,120,2,71429),-- Mithril Arrow
(21339,1867,1,3,2,142857),-- Animal Skin
(21339,1872,1,3,2,100000),-- Animal Bone
(21339,1882,1,1,2,47619),-- Leather
(21339,1894,1,1,2,6897),-- Crafted Leather
(21339,4042,1,1,2,4329),-- Enria
(21339,8732,1,1,200,1305),-- Life Stone: level 76
(21339,8742,1,1,200,315),-- Mid-Grade Life Stone: level 76
(21339,8752,1,1,200,45),-- High-Grade Life Stone: level 76
(21340,960,1,1,-1,2957),-- Scroll: Enchant Armor (Grade S)
(21340,1889,1,2,-1,492862),-- Compound Braid
(21340,5549,1,2,-1,739293),-- Metallic Thread
(21340,57,2975,6071,0,700000),-- Adena
(21340,12269,1,1,1,1367),-- Common Item - Sealed Helm of Nightmare
(21340,12268,1,1,1,1095),-- Common Item - Sealed Shield of Nightmare
(21340,12261,1,1,1,569),-- Common Item - Sealed Majestic Circlet
(21340,5495,1,1,1,5527),-- Sealed Shield of Nightmare Fragment
(21340,5527,1,1,1,4352),-- Sealed Helm of Nightmare Design
(21340,5528,1,1,1,4352),-- Sealed Majestic Circlet Design
(21340,959,1,1,2,13),-- Scroll: Enchant Weapon (Grade S)
(21340,1868,1,3,2,200000),-- Thread
(21340,1873,1,1,2,83333),-- Silver Nugget
(21340,1889,1,1,2,12658),-- Compound Braid
(21340,4044,1,1,2,6289),-- Thons
(21340,5549,1,1,2,25641),-- Metallic Thread
(21340,6672,1,1,2,17544),-- Deluxe Chest Key - Grade 8
(21340,8732,1,1,200,1883),-- Life Stone: level 76
(21340,8742,1,1,200,44),-- Mid-Grade Life Stone: level 76
(21341,960,1,1,-1,3097),-- Scroll: Enchant Armor (Grade S)
(21341,1876,1,3,-1,774226),-- Mithril Ore
(21341,4042,1,1,-1,129038),-- Enria
(21341,57,3115,6359,0,700000),-- Adena
(21341,11968,1,1,1,309),-- Common Item - Dragon Slayer
(21341,11971,1,1,1,309),-- Common Item - Sword of Miracles 
(21341,11964,1,1,1,309),-- Common Item - Dark Legion\'s Edge
(21341,5529,1,1,1,1046),-- Dragon Slayer Edge
(21341,5545,1,1,1,1046),-- Dark Legion\'s Edge Blade
(21341,5546,1,1,1,1149),-- Sword of Miracles Edge
(21341,959,1,1,2,16),-- Scroll: Enchant Weapon (Grade S)
(21341,1868,1,3,2,250000),-- Thread
(21341,1873,1,1,2,111111),-- Silver Nugget
(21341,1876,1,1,2,52632),-- Mithril Ore
(21341,1895,1,1,2,76923),-- Metallic Fiber
(21341,4042,1,1,2,5495),-- Enria
(21341,8732,1,1,200,3367),-- Life Stone: level 76
(21341,8742,1,1,200,406),-- Mid-Grade Life Stone: level 76
(21342,6707,1,1,-1,50478),-- Sealed Draconic Leather Armor Part
(21342,6708,1,1,-1,145051),-- Sealed Draconic Leather Gloves Fabric
(21342,6709,1,1,-1,145051),-- Sealed Draconic Leather Boots Design
(21342,57,4098,8367,0,700000),-- Adena
(21342,11974,1,1,1,13),-- Common Item - Elysian
(21342,11970,1,1,1,13),-- Common Item - Branch of The Mother Tree
(21342,8342,1,1,1,825),-- Flaming Dragon Skull Piece
(21342,11969,1,1,1,13),-- Common Item - Flaming Dragon Skull
(21342,8349,1,1,1,891),-- Doom Crusher Head
(21342,11966,1,1,1,13),-- Common Item - Doom Crusher
(21342,5533,1,1,1,2064),-- Elysian Head
(21342,5544,1,1,1,2062),-- Branch of The Mother Tree Head
(21342,960,1,1,2,74),-- Scroll: Enchant Armor (Grade S)
(21342,1344,20,60,2,200000),-- Mithril Arrow
(21342,1866,2,6,2,52632),-- Suede
(21342,1871,1,3,2,100000),-- Charcoal
(21342,1877,1,1,2,12987),-- Adamantite Nugget
(21342,4040,1,1,2,5025),-- Mold Lubricant
(21342,4041,1,1,2,2950),-- Mold Hardener
(21342,8732,1,1,200,2660),-- Life Stone: level 76
(21342,8742,1,1,200,1403),-- Mid-Grade Life Stone: level 76
(21343,3957,1,1,-1,5119),-- Recipe: Blessed Spiritshot S
(21343,5515,1,1,-1,194527),-- Sealed Majestic Gloves Design
(21343,5522,1,1,-1,39225),-- Sealed Armor of Nightmare Pattern
(21343,57,4624,9470,0,700000),-- Adena
(21343,11975,1,1,1,452),-- Common Item - Soul Bow
(21343,5534,1,1,1,4671),-- Soul Bow Stave
(21343,959,1,1,2,28),-- Scroll: Enchant Weapon (Grade S)
(21343,1894,1,1,2,9709),-- Crafted Leather
(21343,4039,1,1,2,9009),-- Mold Glue
(21343,4041,1,1,2,7246),-- Mold Hardener
(21343,4044,1,1,2,14493),-- Thons
(21343,8732,1,1,200,3165),-- Life Stone: level 76
(21343,8742,1,1,200,903),-- Mid-Grade Life Stone: level 76
(21343,8752,1,1,200,452),-- High-Grade Life Stone: level 76
(21344,4040,1,1,-1,156171),-- Mold Lubricant
(21344,6346,1,1,-1,254226),-- Sealed Majestic Ring Gemstone
(21344,6901,1,1,-1,7809),-- Recipe: Shining Arrow (100%)
(21344,57,3141,6413,0,700000),-- Adena
(21344,11967,1,1,1,116),-- Common Item - Dragon Grinder
(21344,5538,1,1,1,3164),-- Dragon Grinder Edge
(21344,960,1,1,2,232),-- Scroll: Enchant Armor (Grade S)
(21344,1344,20,60,2,142857),-- Mithril Arrow
(21344,1866,1,1,2,142857),-- Suede
(21344,1871,1,3,2,83333),-- Charcoal
(21344,1877,1,1,2,8130),-- Adamantite Nugget
(21344,4040,1,1,2,4878),-- Mold Lubricant
(21344,4041,1,1,2,2899),-- Mold Hardener
(21344,8732,1,1,200,2320),-- Life Stone: level 76
(21344,8742,1,1,200,348),-- Mid-Grade Life Stone: level 76
(21344,8752,1,1,200,232),-- High-Grade Life Stone: level 76
(21345,959,1,1,-1,348),-- Scroll: Enchant Weapon (Grade S)
(21345,1894,1,1,-1,305635),-- Crafted Leather
(21345,4042,1,2,-1,145177),-- Enria
(21345,57,3504,7154,0,700000),-- Adena
(21345,11967,1,1,1,118),-- Common Item - Dragon Grinder
(21345,5538,1,1,1,3530),-- Dragon Grinder Edge
(21345,959,1,1,2,39),-- Scroll: Enchant Weapon (Grade S)
(21345,1867,1,3,2,200000),-- Animal Skin
(21345,1872,1,3,2,125000),-- Animal Bone
(21345,1882,1,1,2,62500),-- Leather
(21345,1894,1,1,2,10753),-- Crafted Leather
(21345,4042,1,1,2,4630),-- Enria
(21345,8732,1,1,200,2439),-- Life Stone: level 76
(21345,8742,1,1,200,354),-- Mid-Grade Life Stone: level 76
(21345,8752,1,1,200,236),-- High-Grade Life Stone: level 76
(21346,6695,1,1,-1,18346),-- Demon Splinter Blade
(21346,6696,1,1,-1,18995),-- Heavens Divider Edge
(21346,6697,1,1,-1,18346),-- Arcana Mace Head
(21346,57,3481,7103,0,700000),-- Adena
(21346,11976,1,1,1,215),-- Common Item - Tallum Glaive
(21346,5541,1,1,1,3504),-- Tallum Glaive Edge
(21346,960,1,1,2,254),-- Scroll: Enchant Armor (Grade S)
(21346,1344,40,120,2,76923),-- Mithril Arrow
(21346,1864,5,15,2,50000),-- Stem
(21346,1865,3,9,2,41667),-- Varnish
(21346,1866,3,9,2,24390),-- Suede
(21346,1868,1,3,2,166667),-- Thread
(21346,1873,1,1,2,100000),-- Silver Nugget
(21346,8732,1,1,200,2169),-- Life Stone: level 76
(21346,8742,1,1,200,567),-- Mid-Grade Life Stone: level 76
(21346,8752,1,1,200,20),-- High-Grade Life Stone: level 76
(21347,6710,1,1,-1,214664),-- Sealed Draconic Leather Helmet Pattern
(21347,6711,1,1,-1,65566),-- Sealed Major Arcana Robe Part
(21347,6712,1,1,-1,188410),-- Sealed Major Arcana Gloves fabric
(21347,57,5309,10884,0,700000),-- Adena
(21347,12263,1,1,1,523),-- Common Item - Sealed Armor of Nightmare
(21347,12262,1,1,1,75),-- Common Item - Sealed Majestic Plate Armor
(21347,5522,1,1,1,3328),-- Sealed Armor of Nightmare Pattern
(21347,5523,1,1,1,3328),-- Sealed Majestic Plate Armor Pattern
(21347,960,1,1,2,75),-- Scroll: Enchant Armor (Grade S)
(21347,1867,4,12,2,71429),-- Animal Skin
(21347,1872,1,3,2,200000),-- Animal Bone
(21347,1882,1,1,2,90909),-- Leather
(21347,1894,1,1,2,13514),-- Crafted Leather
(21347,4042,1,1,2,7634),-- Enria
(21347,8732,1,1,200,2092),-- Life Stone: level 76
(21347,8742,1,1,200,1269),-- Mid-Grade Life Stone: level 76
(21348,6704,1,1,-1,129163),-- Sealed Imperial Crusader Boots Design
(21348,6705,1,1,-1,126515),-- Sealed Imperial Crusader Shield Part
(21348,6706,1,1,-1,147161),-- Sealed Imperial Crusader Helmet Pattern
(21348,57,3643,7457,0,700000),-- Adena
(21348,11973,1,1,1,75),-- Common Item - Soul Separator
(21348,5537,1,1,1,3676),-- Soul Separator Head
(21348,960,1,1,2,179),-- Scroll: Enchant Armor (Grade S)
(21348,1868,1,3,2,333333),-- Thread
(21348,1873,1,1,2,125000),-- Silver Nugget
(21348,1889,1,1,2,19608),-- Compound Braid
(21348,4044,1,1,2,10870),-- Thons
(21348,5549,1,1,2,28571),-- Metallic Thread
(21348,8732,1,1,200,2849),-- Life Stone: level 76
(21348,8742,1,1,200,150),-- Mid-Grade Life Stone: level 76
(21349,959,1,1,-1,369),-- Scroll: Enchant Weapon (Grade S)
(21349,1875,1,1,-1,614183),-- Stone of Purity
(21349,4041,1,1,-1,80111),-- Mold Hardener
(21349,57,3700,7573,0,700000),-- Adena
(21349,11967,1,1,1,69),-- Common Item - Dragon Grinder
(21349,5538,1,1,1,3733),-- Dragon Grinder Edge
(21349,959,1,1,2,19),-- Scroll: Enchant Weapon (Grade S)
(21349,1867,1,3,2,125000),-- Animal Skin
(21349,1872,1,3,2,200000),-- Animal Bone
(21349,1875,1,1,2,21739),-- Stone of Purity
(21349,1881,1,1,2,34483),-- Coarse Bone Powder
(21349,4041,1,1,2,2500),-- Mold Hardener
(21349,8732,1,1,200,3185),-- Life Stone: level 76
(21349,8742,1,1,200,76),-- Mid-Grade Life Stone: level 76
(21350,3957,1,1,-1,3048),-- Recipe: Blessed Spiritshot S
(21350,5157,1,1,-1,3048),-- Recipe: Soulshot (S) Compressed Package (100%)
(21350,5522,1,1,-1,23355),-- Sealed Armor of Nightmare Pattern
(21350,57,2758,5635,0,700000),-- Adena
(21350,11955,1,1,1,384),-- Common Item - Carnage Bow
(21350,5535,1,1,1,3456),-- Carnage Bow Stave
(21350,1865,1,3,2,125000),-- Varnish
(21350,1870,1,1,2,250000),-- Coal
(21350,1875,1,1,2,17241),-- Stone of Purity
(21350,1879,1,1,2,40000),-- Cokes
(21350,4040,1,1,2,4149),-- Mold Lubricant
(21350,5157,1,1,2,51),-- Recipe: Soulshot (S) Compressed Package (100%)
(21350,8732,1,1,200,1866),-- Life Stone: level 76
(21350,8742,1,1,200,230),-- Mid-Grade Life Stone: level 76
(21350,8752,1,1,200,102),-- High-Grade Life Stone: level 76
(21351,4041,1,1,-1,51381),-- Mold Hardener
(21351,5220,1,2,-1,236352),-- Metal Hardener
(21351,5523,1,1,-1,20123),-- Sealed Majestic Plate Armor Pattern
(21351,57,2376,4855,0,700000),-- Adena
(21351,11951,1,1,1,299),-- Common Item - Bloody Orchid
(21351,5536,1,1,1,2970),-- Bloody Orchid Head
(21351,1344,20,60,2,111111),-- Mithril Arrow
(21351,1866,1,1,2,125000),-- Suede
(21351,1871,1,1,2,200000),-- Charcoal
(21351,1885,1,1,2,16129),-- High Grade Suede
(21351,4041,1,1,2,1520),-- Mold Hardener
(21351,5162,1,1,2,75),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21351,5220,1,1,2,7874),-- Metal Hardener
(21351,8732,1,1,200,1842),-- Life Stone: level 76
(21351,8742,1,1,200,299),-- Mid-Grade Life Stone: level 76
(21351,8752,1,1,200,199),-- High-Grade Life Stone: level 76
(21352,5162,1,1,-1,2789),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21352,5527,1,1,-1,75038),-- Sealed Helm of Nightmare Design
(21352,5528,1,1,-1,75038),-- Sealed Majestic Circlet Design
(21352,57,1553,3172,0,700000),-- Adena
(21352,12222,1,1,1,200),-- Common Item - Sealed Dark Crystal Leather Armor
(21352,12228,1,1,1,171),-- Common Item - Sealed Dark Crystalline Leggings
(21352,12230,1,1,1,342),-- Common Item - Sealed Tallum Leather Armor
(21352,5478,1,1,1,2598),-- Sealed Dark Crystal Leather Armor Pattern
(21352,5479,1,1,1,1829),-- Sealed Tallum Leather Armor Pattern
(21352,5482,1,1,1,4078),-- Sealed Dark Crystalline Leggings Design
(21352,1864,1,1,2,250000),-- Stem
(21352,1869,1,1,2,125000),-- Iron Ore
(21352,1880,1,1,2,12821),-- Steel
(21352,4043,1,1,2,4082),-- Asofe
(21352,5167,1,1,2,228),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21352,5550,1,1,2,1626),-- Durable Metal Plate
(21352,8732,1,1,200,1597),-- Life Stone: level 76
(21352,8742,1,1,200,257),-- Mid-Grade Life Stone: level 76
(21352,8752,1,1,200,57),-- High-Grade Life Stone: level 76
(21353,1875,1,2,-1,460775),-- Stone of Purity
(21353,4041,1,1,-1,60101),-- Mold Hardener
(21353,5538,1,1,-1,18958),-- Dragon Grinder Edge
(21353,57,2782,5677,0,700000),-- Adena
(21353,11950,1,1,1,365),-- Common Item - Blood Tornado
(21353,5539,1,1,1,3443),-- Blood Tornado Edge
(21353,1867,1,3,2,125000),-- Animal Skin
(21353,1872,1,3,2,200000),-- Animal Bone
(21353,1875,1,1,2,18868),-- Stone of Purity
(21353,1881,1,1,2,38462),-- Coarse Bone Powder
(21353,4041,1,1,2,2481),-- Mold Hardener
(21353,6901,1,1,2,365),-- Recipe: Shining Arrow (100%)
(21353,8732,1,1,200,2336),-- Life Stone: level 76
(21353,8742,1,1,200,365),-- Mid-Grade Life Stone: level 76
(21353,8752,1,1,200,73),-- High-Grade Life Stone: level 76
(21354,5282,1,1,-1,3044),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21354,5541,1,1,-1,18784),-- Tallum Glaive Edge
(21354,6901,1,1,-1,6849),-- Recipe: Shining Arrow (100%)
(21354,57,2756,5626,0,700000),-- Adena
(21354,11955,1,1,1,438),-- Common Item - Carnage Bow
(21354,5535,1,1,1,3451),-- Carnage Bow Stave
(21354,1344,30,90,2,90909),-- Mithril Arrow
(21354,1808,1,1,2,110),-- Recipe: Soulshot: S Grade
(21354,1868,4,12,2,55556),-- Thread
(21354,1873,1,1,2,90909),-- Silver Nugget
(21354,1889,1,1,2,12987),-- Compound Braid
(21354,4044,1,1,2,6250),-- Thons
(21354,5549,1,1,2,23256),-- Metallic Thread
(21354,6671,1,1,2,24390),-- Deluxe Chest Key - Grade 7
(21354,8732,1,1,200,2083),-- Life Stone: level 76
(21354,8742,1,1,200,219),-- Mid-Grade Life Stone: level 76
(21354,8752,1,1,200,55),-- High-Grade Life Stone: level 76
(21355,5272,1,1,-1,2951),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21355,5495,1,1,-1,100838),-- Sealed Shield of Nightmare Fragment
(21355,5502,1,1,-1,112155),-- Sealed Boots of Nightmare Lining
(21355,57,2672,5453,0,700000),-- Adena
(21355,11967,1,1,1,211),-- Common Item - Dragon Grinder
(21355,5538,1,1,1,2691),-- Dragon Grinder Edge
(21355,959,1,1,2,14),-- Scroll: Enchant Weapon (Grade S)
(21355,1864,1,3,2,250000),-- Stem
(21355,1869,1,3,2,100000),-- Iron Ore
(21355,1880,1,1,2,28571),-- Steel
(21355,4043,1,1,2,10870),-- Asofe
(21355,5550,1,1,2,3030),-- Durable Metal Plate
(21355,8732,1,1,200,1761),-- Life Stone: level 76
(21355,8742,1,1,200,986),-- Mid-Grade Life Stone: level 76
(21355,8752,1,1,200,70),-- High-Grade Life Stone: level 76
(21356,4042,1,1,-1,110643),-- Enria
(21356,5277,1,1,-1,2950),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21356,5503,1,1,-1,112119),-- Sealed Majestic Boots Lining
(21356,57,2254,4600,0,700000),-- Adena
(21356,11975,1,1,1,72),-- Common Item - Soul Bow
(21356,5534,1,1,1,2692),-- Soul Bow Stave
(21356,960,1,1,2,216),-- Scroll: Enchant Armor (Grade S)
(21356,1344,10,30,2,200000),-- Mithril Arrow
(21356,1867,1,3,2,111111),-- Animal Skin
(21356,1872,1,3,2,71429),-- Animal Bone
(21356,1882,1,1,2,40000),-- Leather
(21356,1894,1,1,2,6329),-- Crafted Leather
(21356,4042,1,1,2,4950),-- Enria
(21356,8732,1,1,200,1149),-- Life Stone: level 76
(21356,8742,1,1,200,144),-- Mid-Grade Life Stone: level 76
(21356,8752,1,1,200,72),-- High-Grade Life Stone: level 76
(21357,1808,1,1,-1,3202),-- Recipe: Soulshot: S Grade
(21357,4040,1,1,-1,144105),-- Mold Lubricant
(21357,5480,1,1,-1,32332),-- Sealed Leather Armor of Nightmare Fabric
(21357,57,2899,5918,0,700000),-- Adena
(21357,11975,1,1,1,177),-- Common Item - Soul Bow
(21357,5534,1,1,1,2922),-- Soul Bow Stave
(21357,959,1,1,2,15),-- Scroll: Enchant Weapon (Grade S)
(21357,1865,1,3,2,142857),-- Varnish
(21357,1870,1,3,2,111111),-- Coal
(21357,1875,1,1,2,20833),-- Stone of Purity
(21357,1879,1,1,2,50000),-- Cokes
(21357,4040,1,1,2,6494),-- Mold Lubricant
(21357,8732,1,1,200,2316),-- Life Stone: level 76
(21357,8742,1,1,200,838),-- Mid-Grade Life Stone: level 76
(21357,8752,1,1,200,22),-- High-Grade Life Stone: level 76
(21358,3036,1,1,-1,3356),-- Recipe: Spiritshot S
(21358,5481,1,1,-1,33887),-- Sealed Majestic Leather Armor Fabric
(21358,5487,1,1,-1,34007),-- Sealed Robe of Nightmare Fabric
(21358,57,3038,6202,0,700000),-- Adena
(21358,11973,1,1,1,352),-- Common Item - Soul Separator
(21358,5537,1,1,1,3060),-- Soul Separator Head
(21358,960,1,1,2,201),-- Scroll: Enchant Armor (Grade S)
(21358,1344,20,60,2,166667),-- Mithril Arrow
(21358,1866,1,1,2,200000),-- Suede
(21358,1871,1,3,2,90909),-- Charcoal
(21358,1885,1,1,2,22727),-- High Grade Suede
(21358,4041,1,1,2,2037),-- Mold Hardener
(21358,5220,1,1,2,9709),-- Metal Hardener
(21358,8732,1,1,200,2315),-- Life Stone: level 76
(21358,8742,1,1,200,730),-- Mid-Grade Life Stone: level 76
(21358,8752,1,1,200,50),-- High-Grade Life Stone: level 76
(21359,4042,1,1,-1,116775),-- Enria
(21359,5157,1,1,-1,3114),-- Recipe: Soulshot (S) Compressed Package (100%)
(21359,5523,1,1,-1,23861),-- Sealed Majestic Plate Armor Pattern
(21359,57,2396,4907,0,700000),-- Adena
(21359,11973,1,1,1,319),-- Common Item - Soul Separator
(21359,5537,1,1,1,2839),-- Soul Separator Head
(21359,960,1,1,2,145),-- Scroll: Enchant Armor (Grade S)
(21359,1344,30,90,2,76923),-- Mithril Arrow
(21359,1868,1,3,2,166667),-- Thread
(21359,1873,1,1,2,71429),-- Silver Nugget
(21359,1876,1,1,2,35714),-- Mithril Ore
(21359,1895,1,1,2,52632),-- Metallic Fiber
(21359,4042,1,1,2,3367),-- Enria
(21359,8732,1,1,200,1276),-- Life Stone: level 76
(21359,8742,1,1,200,522),-- Mid-Grade Life Stone: level 76
(21359,8752,1,1,200,29),-- High-Grade Life Stone: level 76
(21360,1875,1,2,-1,676905),-- Stone of Purity
(21360,5162,1,1,-1,4513),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21360,5527,1,1,-1,121396),-- Sealed Helm of Nightmare Design
(21360,57,4077,8348,0,700000),-- Adena
(21360,11973,1,1,1,536),-- Common Item - Soul Separator
(21360,5537,1,1,1,4114),-- Soul Separator Head
(21360,959,1,1,2,21),-- Scroll: Enchant Weapon (Grade S)
(21360,1867,1,3,2,200000),-- Animal Skin
(21360,1872,4,12,2,66667),-- Animal Bone
(21360,1875,1,1,2,27778),-- Stone of Purity
(21360,1881,1,1,2,55556),-- Coarse Bone Powder
(21360,4041,1,1,2,3584),-- Mold Hardener
(21360,8732,1,1,200,3356),-- Life Stone: level 76
(21360,8742,1,1,200,1101),-- Mid-Grade Life Stone: level 76
(21360,8752,1,1,200,198),-- High-Grade Life Stone: level 76
(21361,5481,1,1,-1,30376),-- Sealed Majestic Leather Armor Fabric
(21361,5514,1,1,-1,114331),-- Sealed Gloves of Nightmare Design
(21361,6901,1,1,-1,6770),-- Recipe: Shining Arrow (100%)
(21361,57,2718,5566,0,700000),-- Adena
(21361,12263,1,1,1,309),-- Common Item - Sealed Armor of Nightmare
(21361,12262,1,1,1,154),-- Common Item - Sealed Majestic Plate Armor
(21361,5522,1,1,1,1703),-- Sealed Armor of Nightmare Pattern
(21361,5523,1,1,1,1703),-- Sealed Majestic Plate Armor Pattern
(21361,959,1,1,2,77),-- Scroll: Enchant Weapon (Grade S)
(21361,1867,1,3,2,125000),-- Animal Skin
(21361,1869,2,6,2,71429),-- Iron Ore
(21361,1870,2,6,2,50000),-- Coal
(21361,1871,3,9,2,43478),-- Charcoal
(21361,1872,1,3,2,166667),-- Animal Bone
(21361,6672,1,1,2,21277),-- Deluxe Chest Key - Grade 8
(21361,8732,1,1,200,1312),-- Life Stone: level 76
(21361,8742,1,1,200,695),-- Mid-Grade Life Stone: level 76
(21362,5282,1,1,-1,3537),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21362,5515,1,1,-1,134421),-- Sealed Majestic Gloves Design
(21362,6901,1,1,-1,7959),-- Recipe: Shining Arrow (100%)
(21362,57,3195,6545,0,700000),-- Adena
(21362,12266,1,1,1,945),-- Common Item - Sealed Robe of Nightmare
(21362,12259,1,1,1,315),-- Common Item - Sealed Majestic Robe
(21362,5487,1,1,1,2648),-- Sealed Robe of Nightmare Fabric
(21362,5488,1,1,1,2648),-- Sealed Majestic Robe Fabric
(21362,960,1,1,2,79),-- Scroll: Enchant Armor (Grade S)
(21362,1344,10,30,2,500000),-- Mithril Arrow
(21362,1894,1,1,2,11765),-- Crafted Leather
(21362,4039,1,1,2,9174),-- Mold Glue
(21362,4041,1,1,2,2915),-- Mold Hardener
(21362,4044,1,1,2,10753),-- Thons
(21362,8732,1,1,200,3390),-- Life Stone: level 76
(21362,8742,1,1,200,473),-- Mid-Grade Life Stone: level 76
(21363,960,1,1,-1,2950),-- Scroll: Enchant Armor (Grade S)
(21363,1894,1,1,-1,258750),-- Crafted Leather
(21363,4042,1,1,-1,122906),-- Enria
(21363,57,2968,6057,0,700000),-- Adena
(21363,12264,1,1,1,787),-- Common Item - Sealed Gloves of Nightmare
(21363,12267,1,1,1,629),-- Common Item - Sealed Boots of Nightmare
(21363,12257,1,1,1,943),-- Common Item - Sealed Majestic Gloves
(21363,12260,1,1,1,1101),-- Common Item - Sealed Majestic Boots
(21363,5502,1,1,1,4599),-- Sealed Boots of Nightmare Lining
(21363,5503,1,1,1,4599),-- Sealed Majestic Boots Lining
(21363,5514,1,1,1,4599),-- Sealed Gloves of Nightmare Design
(21363,5515,1,1,1,4599),-- Sealed Majestic Gloves Design
(21363,959,1,1,2,15),-- Scroll: Enchant Weapon (Grade S)
(21363,1867,1,3,2,200000),-- Animal Skin
(21363,1872,1,3,2,142857),-- Animal Bone
(21363,1882,1,1,2,70000),-- Leather
(21363,1894,1,1,2,11111),-- Crafted Leather
(21363,4042,1,1,2,5348),-- Enria
(21363,8732,1,1,200,3145),-- Life Stone: level 76
(21363,8742,1,1,200,157),-- Mid-Grade Life Stone: level 76
(21364,6691,1,1,-1,20197),-- Angel Slayer Blade
(21364,6693,1,1,-1,20563),-- Dragon Hunter Axe Blade
(21364,6694,1,1,-1,20563),-- Saint Spear Blade
(21364,57,2916,5951,0,700000),-- Adena
(21364,12269,1,1,1,704),-- Common Item - Sealed Helm of Nightmare
(21364,12268,1,1,1,704),-- Common Item - Sealed Shield of Nightmare
(21364,12261,1,1,1,391),-- Common Item - Sealed Majestic Circlet
(21364,5495,1,1,1,6985),-- Sealed Shield of Nightmare Fragment
(21364,5527,1,1,1,5500),-- Sealed Helm of Nightmare Design
(21364,5528,1,1,1,5500),-- Sealed Majestic Circlet Design
(21364,960,1,1,2,158),-- Scroll: Enchant Armor (Grade S)
(21364,1344,30,90,2,90909),-- Mithril Arrow
(21364,1868,1,3,2,200000),-- Thread
(21364,1873,1,1,2,90909),-- Silver Nugget
(21364,1889,1,1,2,14706),-- Compound Braid
(21364,4044,1,1,2,8065),-- Thons
(21364,5549,1,1,2,22727),-- Metallic Thread
(21364,8732,1,1,200,1408),-- Life Stone: level 76
(21364,8742,1,1,200,704),-- Mid-Grade Life Stone: level 76
(21365,959,1,1,-1,325),-- Scroll: Enchant Weapon (Grade S)
(21365,6688,1,1,-1,17550),-- Forgotten Blade Edge
(21365,6689,1,2,-1,17219),-- Basalt Battlehammer Head
(21365,57,3267,6667,0,700000),-- Adena
(21365,12265,1,1,1,583),-- Common Item - Sealed Leather Armor of Nightmare
(21365,12258,1,1,1,269),-- Common Item - Sealed Majestic Leather Armor
(21365,5480,1,1,1,2691),-- Sealed Leather Armor of Nightmare Fabric
(21365,5481,1,1,1,2691),-- Sealed Majestic Leather Armor Fabric
(21365,959,1,1,2,17),-- Scroll: Enchant Weapon (Grade S)
(21365,1867,1,3,2,142857),-- Animal Skin
(21365,1869,2,6,2,83333),-- Iron Ore
(21365,1870,2,6,2,66667),-- Coal
(21365,1871,3,9,2,55556),-- Charcoal
(21365,1872,7,21,2,33333),-- Animal Bone
(21365,8732,1,1,200,2915),-- Life Stone: level 76
(21365,8742,1,1,200,1167),-- Mid-Grade Life Stone: level 76
(21365,8752,1,1,200,224),-- High-Grade Life Stone: level 76
(21366,6698,1,2,-1,175995),-- Sealed Tateossian Earring  Part
(21366,6699,1,2,-1,195290),-- Sealed Tateossian Ring Gem
(21366,6700,1,1,-1,107266),-- Sealed Tateossian Ring Wire
(21366,57,3605,7357,0,700000),-- Adena
(21366,12263,1,1,1,318),-- Common Item - Sealed Armor of Nightmare
(21366,12262,1,1,1,212),-- Common Item - Sealed Majestic Plate Armor
(21366,5522,1,1,1,2254),-- Sealed Armor of Nightmare Pattern
(21366,5523,1,1,1,2254),-- Sealed Majestic Plate Armor Pattern
(21366,960,1,1,2,212),-- Scroll: Enchant Armor (Grade S)
(21366,1344,20,60,2,250000),-- Mithril Arrow
(21366,1894,1,1,2,14085),-- Crafted Leather
(21366,4039,1,1,2,11111),-- Mold Glue
(21366,4041,1,1,2,2331),-- Mold Hardener
(21366,4044,1,1,2,12658),-- Thons
(21366,8732,1,1,200,2227),-- Life Stone: level 76
(21366,8742,1,1,200,477),-- Mid-Grade Life Stone: level 76
(21366,8752,1,1,200,265),-- High-Grade Life Stone: level 76
(21367,6710,1,1,-1,125594),-- Sealed Draconic Leather Helmet Pattern
(21367,6711,1,1,-1,38361),-- Sealed Major Arcana Robe Part
(21367,6712,1,1,-1,110234),-- Sealed Major Arcana Gloves fabric
(21367,57,2654,5416,0,700000),-- Adena
(21367,11975,1,1,1,113),-- Common Item - Soul Bow
(21367,5534,1,1,1,3140),-- Soul Bow Stave
(21367,960,1,1,2,153),-- Scroll: Enchant Armor (Grade S)
(21367,1344,10,30,2,250000),-- Mithril Arrow
(21367,1895,1,1,2,62500),-- Metallic Fiber
(21367,4040,1,1,2,3663),-- Mold Lubricant
(21367,4042,1,1,2,3448),-- Enria
(21367,4043,1,1,2,6993),-- Asofe
(21367,8732,1,1,200,1580),-- Life Stone: level 76
(21367,8742,1,1,200,395),-- Mid-Grade Life Stone: level 76
(21367,8752,1,1,200,45),-- High-Grade Life Stone: level 76
(21368,960,1,1,-1,4337),-- Scroll: Enchant Armor (Grade S)
(21368,1895,1,7,-1,774442),-- Metallic Fiber
(21368,4042,1,1,-1,180703),-- Enria
(21368,57,4362,8904,0,700000),-- Adena
(21368,11973,1,1,1,581),-- Common Item - Soul Separator
(21368,5537,1,1,1,4394),-- Soul Separator Head
(21368,959,1,1,2,19),-- Scroll: Enchant Weapon (Grade S)
(21368,1868,3,9,2,142857),-- Thread
(21368,1873,2,6,2,37037),-- Silver Nugget
(21368,1876,1,1,2,83333),-- Mithril Ore
(21368,1895,1,1,2,111111),-- Metallic Fiber
(21368,4042,1,1,2,6098),-- Enria
(21368,6672,1,1,2,30303),-- Deluxe Chest Key - Grade 8
(21368,8732,1,1,200,2410),-- Life Stone: level 76
(21368,8742,1,1,200,954),-- Mid-Grade Life Stone: level 76
(21368,8752,1,1,200,83),-- High-Grade Life Stone: level 76
(21369,5167,1,1,-1,4980),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21369,5528,1,1,-1,133959),-- Sealed Majestic Circlet Design
(21369,5529,1,1,-1,30739),-- Dragon Slayer Edge
(21369,57,4498,9212,0,700000),-- Adena
(21369,11976,1,1,1,70),-- Common Item - Tallum Glaive
(21369,5541,1,1,1,4538),-- Tallum Glaive Edge
(21369,960,1,1,2,189),-- Scroll: Enchant Armor (Grade S)
(21369,1344,30,90,2,142857),-- Mithril Arrow
(21369,1868,4,12,2,90909),-- Thread
(21369,1873,1,1,2,142857),-- Silver Nugget
(21369,1889,1,1,2,26316),-- Compound Braid
(21369,4044,1,1,2,12987),-- Thons
(21369,5549,1,1,2,34483),-- Metallic Thread
(21369,8732,1,1,200,3344),-- Life Stone: level 76
(21369,8742,1,1,200,557),-- Mid-Grade Life Stone: level 76
(21370,4043,1,2,-1,270265),-- Asofe
(21370,5488,1,1,-1,36511),-- Sealed Majestic Robe Fabric
(21370,5550,1,1,-1,108106),-- Durable Metal Plate
(21370,57,3262,6659,0,700000),-- Adena
(21370,12266,1,1,1,525),-- Common Item - Sealed Robe of Nightmare
(21370,12259,1,1,1,596),-- Common Item - Sealed Majestic Robe
(21370,5487,1,1,1,2697),-- Sealed Robe of Nightmare Fabric
(21370,5488,1,1,1,2697),-- Sealed Majestic Robe Fabric
(21370,959,1,1,2,16),-- Scroll: Enchant Weapon (Grade S)
(21370,1864,1,3,2,333333),-- Stem
(21370,1869,1,3,2,111111),-- Iron Ore
(21370,1880,1,1,2,30303),-- Steel
(21370,4043,1,1,2,10989),-- Asofe
(21370,5550,1,1,2,4545),-- Durable Metal Plate
(21370,8732,1,1,200,2695),-- Life Stone: level 76
(21370,8742,1,1,200,525),-- Mid-Grade Life Stone: level 76
(21370,8752,1,1,200,315),-- High-Grade Life Stone: level 76
(21371,6713,1,1,-1,124021),-- Sealed Major Arcana Boots Design
(21371,6714,1,1,-1,141303),-- Sealed Major Arcana Circlet Pattern
(21371,7579,1,1,-1,19046),-- Draconic Bow Shaft
(21371,57,3504,7154,0,700000),-- Adena
(21371,11973,1,1,1,602),-- Common Item - Soul Separator
(21371,5537,1,1,1,3530),-- Soul Separator Head
(21371,960,1,1,2,334),-- Scroll: Enchant Armor (Grade S)
(21371,1344,10,30,2,500000),-- Mithril Arrow
(21371,1894,1,1,2,11364),-- Crafted Leather
(21371,4039,1,1,2,10989),-- Mold Glue
(21371,4041,1,1,2,3205),-- Mold Hardener
(21371,4044,1,1,2,11905),-- Thons
(21371,8732,1,1,200,2809),-- Life Stone: level 76
(21371,8742,1,1,200,1136),-- Mid-Grade Life Stone: level 76
(21372,960,1,1,-1,3460),-- Scroll: Enchant Armor (Grade S)
(21372,1876,1,3,-1,865067),-- Mithril Ore
(21372,1895,1,3,-1,823874),-- Metallic Fiber
(21372,57,2763,5640,0,700000),-- Adena
(21372,12266,1,1,1,443),-- Common Item - Sealed Robe of Nightmare
(21372,12259,1,1,1,273),-- Common Item - Sealed Majestic Robe
(21372,5487,1,1,1,2877),-- Sealed Robe of Nightmare Fabric
(21372,5488,1,1,1,2877),-- Sealed Majestic Robe Fabric
(21372,959,1,1,2,18),-- Scroll: Enchant Weapon (Grade S)
(21372,1868,1,3,2,250000),-- Thread
(21372,1873,1,1,2,90909),-- Silver Nugget
(21372,1876,1,1,2,45455),-- Mithril Ore
(21372,1895,1,1,2,66667),-- Metallic Fiber
(21372,4042,1,1,2,3610),-- Enria
(21372,8732,1,1,200,1706),-- Life Stone: level 76
(21372,8742,1,1,200,341),-- Mid-Grade Life Stone: level 76
(21372,8752,1,1,200,102),-- High-Grade Life Stone: level 76
(21373,959,1,1,-1,510),-- Scroll: Enchant Weapon (Grade S)
(21373,1895,1,7,-1,910171),-- Metallic Fiber
(21373,4040,1,1,-1,254848),-- Mold Lubricant
(21373,57,5113,10480,0,700000),-- Adena
(21373,12266,1,1,1,442),-- Common Item - Sealed Robe of Nightmare
(21373,12259,1,1,1,442),-- Common Item - Sealed Majestic Robe
(21373,5487,1,1,1,4239),-- Sealed Robe of Nightmare Fabric
(21373,5488,1,1,1,4239),-- Sealed Majestic Robe Fabric
(21373,959,1,1,2,30),-- Scroll: Enchant Weapon (Grade S)
(21373,1895,1,1,2,142857),-- Metallic Fiber
(21373,4040,1,1,2,9804),-- Mold Lubricant
(21373,4042,1,1,2,8547),-- Enria
(21373,4043,1,1,2,16393),-- Asofe
(21373,8732,1,1,200,3846),-- Life Stone: level 76
(21373,8742,1,1,200,1010),-- Mid-Grade Life Stone: level 76
(21373,8752,1,1,200,63),-- High-Grade Life Stone: level 76
(21374,6707,1,1,-1,44949),-- Sealed Draconic Leather Armor Part
(21374,6708,1,1,-1,129163),-- Sealed Draconic Leather Gloves Fabric
(21374,6709,1,1,-1,129163),-- Sealed Draconic Leather Boots Design
(21374,57,2916,5970,0,700000),-- Adena
(21374,11976,1,1,1,64),-- Common Item - Tallum Glaive
(21374,5541,1,1,1,3675),-- Tallum Glaive Edge
(21374,960,1,1,2,128),-- Scroll: Enchant Armor (Grade S)
(21374,1868,1,3,2,250000),-- Thread
(21374,1873,1,1,2,100000),-- Silver Nugget
(21374,1876,1,1,2,45455),-- Mithril Ore
(21374,1895,1,1,2,71429),-- Metallic Fiber
(21374,4042,1,1,2,5025),-- Enria
(21374,8732,1,1,200,701),-- Life Stone: level 76
(21374,8742,1,1,200,255),-- Mid-Grade Life Stone: level 76
(21375,960,1,1,-1,3497),-- Scroll: Enchant Armor (Grade S)
(21375,1877,1,1,-1,349708),-- Adamantite Nugget
(21375,4040,1,1,-1,174854),-- Mold Lubricant
(21375,57,2811,5753,0,700000),-- Adena
(21375,12265,1,1,1,188),-- Common Item - Sealed Leather Armor of Nightmare
(21375,12258,1,1,1,688),-- Common Item - Sealed Majestic Leather Armor
(21375,5480,1,1,1,2898),-- Sealed Leather Armor of Nightmare Fabric
(21375,5481,1,1,1,2898),-- Sealed Majestic Leather Armor Fabric
(21375,959,1,1,2,18),-- Scroll: Enchant Weapon (Grade S)
(21375,1866,1,1,2,142857),-- Suede
(21375,1871,1,1,2,250000),-- Charcoal
(21375,1877,1,1,2,7874),-- Adamantite Nugget
(21375,4040,1,1,2,4878),-- Mold Lubricant
(21375,4041,1,1,2,2439),-- Mold Hardener
(21375,8732,1,1,200,2564),-- Life Stone: level 76
(21375,8742,1,1,200,250),-- Mid-Grade Life Stone: level 76
(21376,1875,1,1,-1,820385),-- Stone of Purity
(21376,4041,1,1,-1,107007),-- Mold Hardener
(21376,5544,1,1,-1,33721),-- Branch of The Mother Tree Head
(21376,57,4952,10108,0,700000),-- Adena
(21376,11951,1,1,1,400),-- Common Item - Bloody Orchid
(21376,5536,1,1,1,6184),-- Bloody Orchid Head
(21376,1867,1,3,2,200000),-- Animal Skin
(21376,1872,4,12,2,76963),-- Animal Bone
(21376,1875,1,1,2,25000),-- Stone of Purity
(21376,1881,1,1,2,55556),-- Coarse Bone Powder
(21376,3036,1,1,2,270),-- Recipe: Spiritshot S
(21376,4041,1,1,2,3610),-- Mold Hardener
(21376,8732,1,1,200,3205),-- Life Stone: level 76
(21376,8742,1,1,200,801),-- Mid-Grade Life Stone: level 76
(21377,1808,1,1,-1,5353),-- Recipe: Soulshot: S Grade
(21377,5545,1,1,-1,33041),-- Dark Legion\'s Edge Blade
(21377,5546,1,1,-1,36292),-- Sword of Miracles Edge
(21377,57,4847,9892,0,700000),-- Adena
(21377,12227,1,1,1,782),-- Common Item - Sealed Dark Crystal breastplate
(21377,12220,1,1,1,1565),-- Common Item - Sealed Dark Crystal Gaiters
(21377,12233,1,1,1,391),-- Common Item - Sealed Tallum Plate Armor
(21377,5520,1,1,1,3824),-- Sealed Dark Crystal Breastplate Pattern
(21377,5521,1,1,1,2699),-- Sealed Tallum Plate Armor Pattern
(21377,5524,1,1,1,5845),-- Sealed Dark Crystal Gaiters Pattern
(21377,1344,150,450,2,28571),-- Mithril Arrow
(21377,1868,4,12,2,83333),-- Thread
(21377,1873,1,1,2,142857),-- Silver Nugget
(21377,1876,1,1,2,71429),-- Mithril Ore
(21377,1895,1,1,2,100000),-- Metallic Fiber
(21377,3957,1,1,2,226),-- Recipe: Blessed Spiritshot S
(21377,4042,1,1,2,4695),-- Enria
(21377,8732,1,1,200,2347),-- Life Stone: level 76
(21377,8742,1,1,200,782),-- Mid-Grade Life Stone: level 76
(21377,8752,1,1,200,27),-- High-Grade Life Stone: level 76
(21378,1889,1,2,-1,492955),-- Compound Braid
(21378,5282,1,1,-1,6573),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21378,5514,1,1,-1,249766),-- Sealed Gloves of Nightmare Design
(21378,57,5950,12144,0,700000),-- Adena
(21378,11973,1,1,1,613),-- Common Item - Soul Separator
(21378,5537,1,1,1,5992),-- Soul Separator Head
(21378,959,1,1,2,29),-- Scroll: Enchant Weapon (Grade S)
(21378,1868,3,9,2,166667),-- Thread
(21378,1873,1,1,2,200000),-- Silver Nugget
(21378,1889,1,1,2,30303),-- Compound Braid
(21378,4044,1,1,2,14706),-- Thons
(21378,5549,1,1,2,47619),-- Metallic Thread
(21378,8732,1,1,200,4292),-- Life Stone: level 76
(21378,8742,1,1,200,245),-- Mid-Grade Life Stone: level 76
(21378,8752,1,1,200,123),-- High-Grade Life Stone: level 76
(21379,5515,1,1,-1,209654),-- Sealed Majestic Gloves Design
(21379,5522,1,1,-1,42275),-- Sealed Armor of Nightmare Pattern
(21379,6901,1,1,-1,12414),-- Recipe: Shining Arrow (100%)
(21379,57,4995,10193,0,700000),-- Adena
(21379,12263,1,1,1,18),-- Common Item - Sealed Armor of Nightmare (no drop info)
(21379,12262,1,1,1,18),-- Common Item - Sealed Majestic Plate Armor
(21379,5522,1,1,1,3122),-- Sealed Armor of Nightmare Pattern
(21379,5523,1,1,1,3122),-- Sealed Majestic Plate Armor Pattern
(21379,960,1,1,2,210),-- Scroll: Enchant Armor (Grade S)
(21379,1344,40,120,2,111111),-- Mithril Arrow
(21379,1864,5,15,2,71429),-- Stem
(21379,1865,3,9,2,58824),-- Varnish
(21379,1866,3,9,2,38462),-- Suede
(21379,1868,12,36,2,32258),-- Thread
(21379,1873,1,1,2,166667),-- Silver Nugget
(21379,8732,1,1,200,2020),-- Life Stone: level 76
(21379,8742,1,1,200,782),-- Mid-Grade Life Stone: level 76
(21380,1808,1,1,-1,5625),-- Recipe: Soulshot: S Grade
(21380,5481,1,1,-1,56792),-- Sealed Majestic Leather Armor Fabric
(21380,5523,1,1,-1,43102),-- Sealed Majestic Plate Armor Pattern
(21380,57,5093,10393,0,700000),-- Adena
(21380,11974,1,1,1,7),-- Common Item - Elysian (no drop info)
(21380,11970,1,1,1,7),-- Common Item - Branch of The Mother Tree
(21380,8342,1,1,1,1828),-- Flaming Dragon Skull Piece
(21380,11969,1,1,1,7),-- Common Item - Flaming Dragon Skull
(21380,8349,1,1,1,914),-- Doom Crusher Head
(21380,11966,1,1,1,7),-- Common Item - Doom Crusher
(21380,5533,1,1,1,2565),-- Elysian Head
(21380,5544,1,1,1,2562),-- Branch of The Mother Tree Head
(21380,959,1,1,2,25),-- Scroll: Enchant Weapon (Grade S)
(21380,1867,1,3,2,166667),-- Animal Skin
(21380,1869,2,6,2,111111),-- Iron Ore
(21380,1870,2,6,2,76923),-- Coal
(21380,1871,3,9,2,47619),-- Charcoal
(21380,1872,7,21,2,43478),-- Animal Bone
(21380,8732,1,1,200,3521),-- Life Stone: level 76
(21381,3957,1,1,-1,6149),-- Recipe: Blessed Spiritshot S
(21381,5157,1,1,-1,6149),-- Recipe: Soulshot (S) Compressed Package (100%)
(21381,5495,1,1,-1,210102),-- Sealed Shield of Nightmare Fragment
(21381,57,5566,11364,0,700000),-- Adena
(21381,11976,1,1,1,318),-- Common Item - Tallum Glaive
(21381,5541,1,1,1,5604),-- Tallum Glaive Edge
(21381,960,1,1,2,318),-- Scroll: Enchant Armor (Grade S)
(21381,1344,40,120,2,142857),-- Mithril Arrow
(21381,1867,5,15,2,55556),-- Animal Skin
(21381,1872,1,3,2,200000),-- Animal Bone
(21381,1882,1,1,2,90909),-- Leather
(21381,1894,1,1,2,16949),-- Crafted Leather
(21381,4042,1,1,2,7634),-- Enria
(21381,8732,1,1,200,2865),-- Life Stone: level 76
(21381,8742,1,1,200,1274),-- Mid-Grade Life Stone: level 76
(21382,1889,1,1,-1,491264),-- Compound Braid
(21382,5502,1,1,-1,248909),-- Sealed Boots of Nightmare Lining
(21382,5549,1,3,-1,736897),-- Metallic Thread
(21382,57,5929,12104,0,700000),-- Adena
(21382,12265,1,1,1,710),-- Common Item - Sealed Leather Armor of Nightmare
(21382,12258,1,1,1,627),-- Common Item - Sealed Majestic Leather Armor
(21382,5480,1,1,1,4885),-- Sealed Leather Armor of Nightmare Fabric
(21382,5481,1,1,1,4885),-- Sealed Majestic Leather Armor Fabric
(21382,959,1,1,2,29),-- Scroll: Enchant Weapon (Grade S)
(21382,1868,3,9,2,166667),-- Thread
(21382,1873,1,1,2,200000),-- Silver Nugget
(21382,1889,1,1,2,32258),-- Compound Braid
(21382,4044,1,1,2,16667),-- Thons
(21382,5549,1,1,2,50000),-- Metallic Thread
(21382,8732,1,1,200,4464),-- Life Stone: level 76
(21382,8742,1,1,200,835),-- Mid-Grade Life Stone: level 76
(21382,8752,1,1,200,84),-- High-Grade Life Stone: level 76
(21383,5162,1,1,-1,7001),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21383,5503,1,1,-1,266055),-- Sealed Majestic Boots Lining
(21383,5514,1,1,-1,266055),-- Sealed Gloves of Nightmare Design
(21383,57,6337,12938,0,700000),-- Adena
(21383,12263,1,1,1,591),-- Common Item - Sealed Armor of Nightmare
(21383,12262,1,1,1,422),-- Common Item - Sealed Majestic Plate Armor
(21383,5522,1,1,1,3962),-- Sealed Armor of Nightmare Pattern
(21383,5523,1,1,1,3962),-- Sealed Majestic Plate Armor Pattern
(21383,960,1,1,2,253),-- Scroll: Enchant Armor (Grade S)
(21383,1344,40,120,2,142857),-- Mithril Arrow
(21383,1864,5,15,2,90909),-- Stem
(21383,1865,3,9,2,76923),-- Varnish
(21383,1866,3,9,2,45455),-- Suede
(21383,1868,12,36,2,37037),-- Thread
(21383,1873,4,12,2,20000),-- Silver Nugget
(21383,8732,1,1,200,4386),-- Life Stone: level 76
(21383,8742,1,1,200,760),-- Mid-Grade Life Stone: level 76
(21384,1875,1,1,-1,488972),-- Stone of Purity
(21384,4041,1,1,-1,127558),-- Mold Hardener
(21384,5533,1,1,-1,40237),-- Elysian Head
(21384,57,5890,12060,0,700000),-- Adena
(21384,12265,1,1,1,712),-- Common Item - Sealed Leather Armor of Nightmare
(21384,12258,1,1,1,559),-- Common Item - Sealed Majestic Leather Armor
(21384,5480,1,1,1,4862),-- Sealed Leather Armor of Nightmare Fabric
(21384,5481,1,1,1,4862),-- Sealed Majestic Leather Armor Fabric
(21384,959,1,1,2,51),-- Scroll: Enchant Weapon (Grade S)
(21384,1867,1,3,2,200000),-- Animal Skin
(21384,1872,4,12,2,83333),-- Animal Bone
(21384,1875,1,1,2,32258),-- Stone of Purity
(21384,1881,1,1,2,66667),-- Coarse Bone Powder
(21384,4041,1,1,2,4202),-- Mold Hardener
(21384,8732,1,1,200,3817),-- Life Stone: level 76
(21384,8742,1,1,200,1067),-- Mid-Grade Life Stone: level 76
(21384,8752,1,1,200,153),-- High-Grade Life Stone: level 76
(21385,5272,1,1,-1,6166),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21385,5277,1,1,-1,6166),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21385,5534,1,1,-1,38091),-- Soul Bow Stave
(21385,57,5571,11407,0,700000),-- Adena
(21385,12263,1,1,1,1311),-- Common Item - Sealed Armor of Nightmare
(21385,12262,1,1,1,1311),-- Common Item - Sealed Majestic Plate Armor
(21385,5522,1,1,1,3490),-- Sealed Armor of Nightmare Pattern
(21385,5523,1,1,1,3490),-- Sealed Majestic Plate Armor Pattern
(21385,960,1,1,2,235),-- Scroll: Enchant Armor (Grade S)
(21385,1344,150,450,2,35714),-- Mithril Arrow
(21385,1868,4,12,2,100000),-- Thread
(21385,1873,3,9,2,34483),-- Silver Nugget
(21385,1876,1,1,2,66667),-- Mithril Ore
(21385,1895,1,1,2,125000),-- Metallic Fiber
(21385,4042,1,1,2,3937),-- Enria
(21385,8732,1,1,200,3937),-- Life Stone: level 76
(21385,8742,1,1,200,1311),-- Mid-Grade Life Stone: level 76
(21386,1877,1,1,-1,572799),-- Adamantite Nugget
(21386,4041,1,1,-1,124522),-- Mold Hardener
(21386,5537,1,1,-1,39287),-- Soul Separator Head
(21386,57,5749,11773,0,700000),-- Adena
(21386,12266,1,1,1,1416),-- Common Item - Sealed Robe of Nightmare
(21386,12259,1,1,1,1416),-- Common Item - Sealed Majestic Robe
(21386,5487,1,1,1,4763),-- Sealed Robe of Nightmare Fabric
(21386,5488,1,1,1,4763),-- Sealed Majestic Robe Fabric
(21386,959,1,1,2,29),-- Scroll: Enchant Weapon (Grade S)
(21386,1866,1,3,2,100000),-- Suede
(21386,1871,1,3,2,250000),-- Charcoal
(21386,1877,1,1,2,27027),-- Adamantite Nugget
(21386,4040,1,1,2,5682),-- Mold Lubricant
(21386,4041,1,1,2,2833),-- Mold Hardener
(21386,8732,1,1,200,5682),-- Life Stone: level 76
(21386,8752,1,1,200,1416),-- High-Grade Life Stone: level 76
(21387,1895,1,7,-1,918637),-- Metallic Fiber
(21387,4042,1,1,-1,214349),-- Enria
(21387,5522,1,1,-1,43798),-- Sealed Armor of Nightmare Pattern
(21387,57,5163,10575,0,700000),-- Adena
(21387,12350,1,1,1,1355),-- Common Item - Sealed Majestic Necklace
(21387,12352,1,1,1,3610),-- Common Item - Sealed Majestic Earring
(21387,12351,1,1,1,1355),-- Common Item - Sealed Majestic Ring
(21387,6342,1,1,1,14343),-- Sealed Majestic Earring Gemstone
(21387,6344,1,1,1,9282),-- Sealed Majestic Necklace Beads
(21387,6346,1,1,1,20614),-- Sealed Majestic Ring Gemstone
(21387,959,1,1,2,25),-- Scroll: Enchant Weapon (Grade S)
(21387,1868,3,9,2,142857),-- Thread
(21387,1873,2,6,2,47629),-- Silver Nugget
(21387,1876,1,1,2,90909),-- Mithril Ore
(21387,1895,1,1,2,125000),-- Metallic Fiber
(21387,4042,1,1,2,9009),-- Enria
(21387,8732,1,1,200,3610),-- Life Stone: level 76
(21387,8742,1,1,200,1355),-- Mid-Grade Life Stone: level 76
(21388,1808,1,1,-1,8647),-- Recipe: Soulshot: S Grade
(21388,5523,1,1,-1,66254),-- Sealed Majestic Plate Armor Pattern
(21388,5527,1,1,-1,232602),-- Sealed Helm of Nightmare Design
(21388,57,7810,15997,0,700000),-- Adena
(21388,12264,1,1,1,327),-- Common Item - Sealed Gloves of Nightmare
(21388,12267,1,1,1,327),-- Common Item - Sealed Boots of Nightmare
(21388,12257,1,1,1,327),-- Common Item - Sealed Majestic Gloves
(21388,12260,1,1,1,327),-- Common Item - Sealed Majestic Boots
(21388,5502,1,1,1,12131),-- Sealed Boots of Nightmare Lining
(21388,5503,1,1,1,12131),-- Sealed Majestic Boots Lining
(21388,5514,1,1,1,12131),-- Sealed Gloves of Nightmare Design
(21388,5515,1,1,1,12131),-- Sealed Majestic Gloves Design
(21388,960,1,1,2,329),-- Scroll: Enchant Armor (Grade S)
(21388,1344,40,120,2,200000),-- Mithril Arrow
(21388,1867,9,27,2,45455),-- Animal Skin
(21388,1872,5,15,2,83333),-- Animal Bone
(21388,1875,1,1,2,43478),-- Stone of Purity
(21388,1881,1,1,2,76923),-- Coarse Bone Powder
(21388,4041,1,1,2,1309),-- Mold Hardener
(21388,8732,1,1,200,3922),-- Life Stone: level 76
(21388,8742,1,1,200,1309),-- Mid-Grade Life Stone: level 76
(21389,959,1,1,-1,711),-- Scroll: Enchant Weapon (Grade S)
(21389,6690,1,1,-1,38470),-- Imperial Staff Head
(21389,6691,1,1,-1,38397),-- Angel Slayer Blade
(21389,57,7149,14591,0,700000),-- Adena
(21389,11964,1,1,1,371),-- Common Item - Dark Legion's Edge
(21389,11971,1,1,1,371),-- Common Item - Sword of Miracles 
(21389,11968,1,1,1,371),-- Common Item - Dragon Slayer
(21389,5529,1,1,1,2400),-- Dragon Slayer Edge
(21389,5545,1,1,1,2400),-- Dark Legion\'s Edge Blade
(21389,5546,1,1,1,2636),-- Sword of Miracles Edge
(21389,959,1,1,2,36),-- Scroll: Enchant Weapon (Grade S)
(21389,1864,4,12,2,142857),-- Stem
(21389,1865,2,6,2,125000),-- Varnish
(21389,1866,3,9,2,58824),-- Suede
(21389,1868,10,30,2,50000),-- Thread
(21389,1873,4,12,2,23256),-- Silver Nugget
(21389,8732,1,1,200,1437),-- Life Stone: level 76
(21389,8742,1,1,200,1437),-- Mid-Grade Life Stone: level 76
(21390,6701,1,1,-1,110268),-- Sealed Imperial Crusader Breastplate Part
(21390,6702,1,1,-1,141944),-- Sealed Imperial Crusader Gaiters Pattern
(21390,6703,1,1,-1,294882),-- Sealed Imperial Crusader Gauntlets Design
(21390,57,8333,17007,0,700000),-- Adena
(21390,12350,1,1,1,970),-- Common Item - Sealed Majestic Necklace
(21390,12352,1,1,1,3876),-- Common Item - Sealed Majestic Earring
(21390,12351,1,1,1,2907),-- Common Item - Sealed Majestic Ring
(21390,6342,1,1,1,23097),-- Sealed Majestic Earring Gemstone
(21390,6344,1,1,1,14947),-- Sealed Majestic Necklace Beads
(21390,6346,1,1,1,33196),-- Sealed Majestic Ring Gemstone
(21390,960,1,1,2,350),-- Scroll: Enchant Armor (Grade S)
(21390,1344,40,120,2,200000),-- Mithril Arrow
(21390,1867,9,27,2,40000),-- Animal Skin
(21390,1872,5,15,2,90909),-- Animal Bone
(21390,1875,1,1,2,37037),-- Stone of Purity
(21390,1881,1,1,2,71429),-- Coarse Bone Powder
(21390,4041,1,1,2,3876),-- Mold Hardener
(21390,8732,1,1,200,4367),-- Life Stone: level 76
(21390,8742,1,1,200,485),-- Mid-Grade Life Stone: level 76
(21391,6695,1,1,-1,35325),-- Demon Splinter Blade
(21391,6696,1,1,-1,36575),-- Heavens Divider Edge
(21391,6697,1,1,-1,35325),-- Arcana Mace Head
(21391,57,6703,13681,0,700000),-- Adena
(21391,11970,1,1,1,156),-- Common Item - Branch of The Mother Tree
(21391,11974,1,1,1,156),-- Common Item - Elysian
(21391,11969,1,1,1,156),-- Common Item - Flaming Dragon Skull
(21391,11966,1,1,1,156),-- Common Item - Doom Crusher
(21391,8342,1,1,1,2024),-- Flaming Dragon Skull Piece
(21391,8349,1,1,1,505),-- Doom Crusher Head
(21391,5533,1,1,1,3375),-- Elysian Head
(21391,5544,1,1,1,3372),-- Branch of The Mother Tree Head
(21391,960,1,1,2,208),-- Scroll: Enchant Armor (Grade S)
(21391,1344,40,120,2,166667),-- Mithril Arrow
(21391,1867,13,39,2,22222),-- Animal Skin
(21391,1869,2,6,2,100000),-- Iron Ore
(21391,1870,3,9,2,76923),-- Coal
(21391,1871,3,9,2,76923),-- Charcoal
(21391,1872,8,24,2,37037),-- Animal Bone
(21391,8732,1,1,200,3745),-- Life Stone: level 76
(21391,8742,1,1,200,624),-- Mid-Grade Life Stone: level 76
(21392,959,1,1,-1,822),-- Scroll: Enchant Weapon (Grade S)
(21392,4044,1,1,-1,684717),-- Thons
(21392,5549,1,3,-1,684717),-- Metallic Thread
(21392,57,8265,16868,0,700000),-- Adena
(21392,12264,1,1,1,2457),-- Common Item - Sealed Gloves of Nightmare
(21392,12267,1,1,1,1754),-- Common Item - Sealed Boots of Nightmare
(21392,12257,1,1,1,2105),-- Common Item - Sealed Majestic Gloves
(21392,12260,1,1,1,1053),-- Common Item - Sealed Majestic Boots
(21392,5502,1,1,1,12809),-- Sealed Boots of Nightmare Lining
(21392,5503,1,1,1,12809),-- Sealed Majestic Boots Lining
(21392,5514,1,1,1,12809),-- Sealed Gloves of Nightmare Design
(21392,5515,1,1,1,12809),-- Sealed Majestic Gloves Design
(21392,959,1,1,2,41),-- Scroll: Enchant Weapon (Grade S)
(21392,1868,3,9,2,250000),-- Thread
(21392,1873,2,6,2,62500),-- Silver Nugget
(21392,1889,1,1,2,50000),-- Compound Braid
(21392,4044,1,1,2,25641),-- Thons
(21392,5549,1,1,2,66667),-- Metallic Thread
(21392,8732,1,1,200,7874),-- Life Stone: level 76
(21392,8742,1,1,200,1053),-- Mid-Grade Life Stone: level 76
(21392,8752,1,1,200,175),-- High-Grade Life Stone: level 76
(21393,959,1,1,-1,740),-- Scroll: Enchant Weapon (Grade S)
(21393,1877,1,1,-1,740087),-- Adamantite Nugget
(21393,4041,1,1,-1,160889),-- Mold Hardener
(21393,57,7443,15195,0,700000),-- Adena
(21393,11976,1,1,1,664),-- Common Item - Tallum Glaive
(21393,5541,1,1,1,7494),-- Tallum Glaive Edge
(21393,959,1,1,2,111),-- Scroll: Enchant Weapon (Grade S)
(21393,1866,1,3,2,142857),-- Suede
(21393,1871,1,3,2,333333),-- Charcoal
(21393,1877,1,1,2,25641),-- Adamantite Nugget
(21393,4040,1,1,2,12658),-- Mold Lubricant
(21393,4041,1,1,2,5208),-- Mold Hardener
(21393,8732,1,1,200,4854),-- Life Stone: level 76
(21393,8742,1,1,200,1106),-- Mid-Grade Life Stone: level 76
(21393,8752,1,1,200,111),-- High-Grade Life Stone: level 76
(21394,1808,1,1,-1,6211),-- Recipe: Soulshot: S Grade
(21394,3036,1,1,-1,6211),-- Recipe: Spiritshot S
(21394,5527,1,1,-1,167086),-- Sealed Helm of Nightmare Design
(21394,57,5623,11476,0,700000),-- Adena
(21394,11967,1,1,1,187),-- Common Item - Dragon Grinder
(21394,5538,1,1,1,5662),-- Dragon Grinder Edge
(21394,960,1,1,2,276),-- Scroll: Enchant Armor (Grade S)
(21394,1344,20,60,2,333333),-- Mithril Arrow
(21394,1894,1,1,2,18182),-- Crafted Leather
(21394,4039,1,1,2,21739),-- Mold Glue
(21394,4041,1,1,2,9091),-- Mold Hardener
(21394,4044,1,1,2,10870),-- Thons
(21394,8732,1,1,200,1495),-- Life Stone: level 76
(21394,8742,1,1,200,1495),-- Mid-Grade Life Stone: level 76
(21395,960,1,1,-1,7328),-- Scroll: Enchant Armor (Grade S)
(21395,1894,1,1,-1,642786),-- Crafted Leather
(21395,4039,1,1,-1,610647),-- Mold Glue
(21395,57,7372,15047,0,700000),-- Adena
(21395,11967,1,1,1,21),-- Common Item - Dragon Grinder (no drop info)
(21395,5538,1,1,1,7422),-- Dragon Grinder Edge
(21395,959,1,1,2,44),-- Scroll: Enchant Weapon (Grade S)
(21395,1894,1,1,2,22222),-- Crafted Leather
(21395,4039,1,1,2,19608),-- Mold Glue
(21395,4041,1,1,2,4167),-- Mold Hardener
(21395,4044,1,1,2,26316),-- Thons
(21395,8732,1,1,200,11111),-- Life Stone: level 76
(21395,8742,1,1,200,2786),-- Mid-Grade Life Stone: level 76
(21396,3036,1,1,-1,3389),-- Recipe: Spiritshot S (this one should have no spawn.. no drop info)
(21396,4040,1,1,-1,152522),-- Mold Lubricant
(21396,6342,1,1,-1,172751),-- Sealed Majestic Earring Gemstone
(21396,57,3069,6264,0,700000),-- Adena
(21396,11946,1,1,1,7),-- Common Item - Dasparion\'s Staff
(21396,11949,1,1,1,7),-- Common Item - Meteor Shower
(21396,11958,1,1,1,7),-- Common Item - Destroyer Hammer
(21396,11953,1,1,1,7),-- Common Item - Spiritual Eye
(21396,5532,1,1,1,1899),-- Meteor Shower Head
(21396,5543,1,1,1,1917),-- Dasparion\'s Staff Edge
(21396,1866,1,1,2,250293),-- Suede
(21396,1871,1,3,2,187720),-- Charcoal
(21396,1877,1,1,2,15018),-- Adamantite Nugget
(21396,4040,1,1,2,7509),-- Mold Lubricant
(21396,4041,1,1,2,3265),-- Mold Hardener
(21396,5157,1,1,2,167),-- Recipe: Soulshot (S) Compressed Package (100%)
(21397,3036,1,1,-1,3304),-- Recipe: Spiritshot S
(21397,4040,1,1,-1,148684),-- Mold Lubricant
(21397,6342,1,1,-1,168404),-- Sealed Majestic Earring Gemstone
(21397,57,2992,6106,0,700000),-- Adena
(21397,11946,1,1,1,92),-- Common Item - Dasparion\'s Staff
(21397,11949,1,1,1,92),-- Common Item - Meteor Shower
(21397,8341,1,1,1,938),-- Spiritual Eye Piece
(21397,11958,1,1,1,92),-- Common Item - Destroyer Hammer
(21397,11953,1,1,1,92),-- Common Item - Spiritual Eye
(21397,5532,1,1,1,1851),-- Meteor Shower Head
(21397,5543,1,1,1,1869),-- Dasparion\'s Staff Edge
(21397,1866,1,1,2,142857),-- Suede
(21397,1871,1,1,2,250000),-- Charcoal
(21397,1877,1,1,2,10989),-- Adamantite Nugget
(21397,4040,1,1,2,2577),-- Mold Lubricant
(21397,4041,1,1,2,368),-- Mold Hardener
(21397,5157,1,1,2,368),-- Recipe: Soulshot (S) Compressed Package (100%)
(21397,8732,1,1,200,2577),-- Life Stone: level 76
(21398,5157,1,1,-1,6274),-- Recipe: Soulshot (S) Compressed Package (100%)
(21398,5480,1,1,-1,63343),-- Sealed Leather Armor of Nightmare Fabric
(21398,6346,1,2,-1,459587),-- Sealed Majestic Ring Gemstone
(21398,57,5681,11595,0,700000),-- Adena
(21398,11960,1,1,1,465),-- Common Item - Halberd
(21398,5542,1,1,1,7032),-- Halberd Edge
(21398,1895,1,1,2,142857),-- Metallic Fiber
(21398,4040,1,1,2,10989),-- Mold Lubricant
(21398,4042,1,1,2,8333),-- Enria
(21398,4043,1,1,2,16949),-- Asofe
(21398,5167,1,1,2,232),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21398,8732,1,1,200,4950),-- Life Stone: level 76
(21398,8742,1,1,200,1239),-- Mid-Grade Life Stone: level 76
(21399,5157,1,1,-1,5506),-- Recipe: Soulshot (S) Compressed Package (100%)
(21399,5480,1,1,-1,55588),-- Sealed Leather Armor of Nightmare Fabric
(21399,6346,1,1,-1,403326),-- Sealed Majestic Ring Gemstone
(21399,57,4986,10175,0,700000),-- Adena
(21399,11960,1,1,1,639),-- Common Item - Halberd
(21399,5542,1,1,1,6172),-- Halberd Edge
(21399,1895,1,1,2,125000),-- Metallic Fiber
(21399,4040,1,1,2,5102),-- Mold Lubricant
(21399,4042,1,1,2,4484),-- Enria
(21399,4043,1,1,2,12500),-- Asofe
(21399,5167,1,1,2,1279),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21399,8732,1,1,200,6410),-- Life Stone: level 76
(21399,8742,1,1,200,320),-- Mid-Grade Life Stone: level 76
(21399,8752,1,1,200,320),-- High-Grade Life Stone: level 76
(21400,1876,1,5,-1,770779),-- Mithril Ore
(21400,4042,1,1,-1,192695),-- Enria
(21400,5528,1,1,-1,138232),-- Sealed Majestic Circlet Design
(21400,57,4652,9494,0,700000),-- Adena
(21400,11976,1,1,1,450),-- Common Item - Tallum Glaive
(21400,5541,1,1,1,4683),-- Tallum Glaive Edge
(21400,959,1,1,2,23),-- Scroll: Enchant Weapon (Grade S)
(21400,1868,3,9,2,142857),-- Thread
(21400,1873,1,1,2,166667),-- Silver Nugget
(21400,1876,1,1,2,83333),-- Mithril Ore
(21400,1895,1,1,2,100000),-- Metallic Fiber
(21400,4042,1,1,2,6061),-- Enria
(21400,8732,1,1,200,4717),-- Life Stone: level 76
(21400,8742,1,1,200,225),-- Mid-Grade Life Stone: level 76
(21400,8752,1,1,200,225),-- High-Grade Life Stone: level 76
(21401,1876,1,5,-1,678250),-- Mithril Ore
(21401,4042,1,1,-1,169563),-- Enria
(21401,5528,1,1,-1,121637),-- Sealed Majestic Circlet Design
(21401,57,4094,8354,0,700000),-- Adena
(21401,11976,1,1,1,12),-- Common Item - Tallum Glaive (no drop info, low material info)
(21401,5541,1,1,1,4121),-- Tallum Glaive Edge
(21401,959,1,1,2,21),-- Scroll: Enchant Weapon (Grade S)
(21401,1868,3,9,2,111111),-- Thread
(21401,1873,1,1,2,142857),-- Silver Nugget
(21401,1876,1,1,2,100172),-- Mithril Ore
(21401,1895,1,1,2,100000),-- Metallic Fiber
(21401,4042,1,1,2,8348),-- Enria
(21402,4044,1,1,-1,349920),-- Thons
(21402,5157,1,1,-1,4666),-- Recipe: Soulshot (S) Compressed Package (100%)
(21402,5534,1,1,-1,28820),-- Soul Bow Stave
(21402,57,4224,8620,0,700000),-- Adena
(21402,11975,1,1,1,360),-- Common Item - Soul Bow
(21402,5534,1,1,1,4257),-- Soul Bow Stave
(21402,959,1,1,2,21),-- Scroll: Enchant Weapon (Grade S)
(21402,1868,3,9,2,125000),-- Thread
(21402,1873,1,1,2,142857),-- Silver Nugget
(21402,1889,1,1,2,20000),-- Compound Braid
(21402,4044,1,1,2,10753),-- Thons
(21402,5549,1,1,2,30303),-- Metallic Thread
(21402,8732,1,1,200,1080),-- Life Stone: level 76
(21402,8742,1,1,200,360),-- Mid-Grade Life Stone: level 76
(21403,4044,1,1,-1,403892),-- Thons
(21403,5157,1,1,-1,5385),-- Recipe: Soulshot (S) Compressed Package (100%)
(21403,5534,1,1,-1,33265),-- Soul Bow Stave
(21403,57,4875,9950,0,700000),-- Adena
(21403,11975,1,1,1,101),-- Common Item - Soul Bow
(21403,5534,1,1,1,4913),-- Soul Bow Stave
(21403,959,1,1,2,24),-- Scroll: Enchant Weapon (Grade S)
(21403,1868,3,9,2,125000),-- Thread
(21403,1873,1,1,2,166667),-- Silver Nugget
(21403,1889,1,1,2,29412),-- Compound Braid
(21403,4044,1,1,2,11628),-- Thons
(21403,5549,1,1,2,38462),-- Metallic Thread
(21403,8732,1,1,200,2415),-- Life Stone: level 76
(21403,8742,1,1,200,805),-- Mid-Grade Life Stone: level 76
(21403,8752,1,1,200,147),-- High-Grade Life Stone: level 76
(21404,5167,1,1,-1,6233),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21404,5515,1,1,-1,236857),-- Sealed Majestic Gloves Design
(21404,6901,1,1,-1,14024),-- Recipe: Shining Arrow (100%)
(21404,57,5642,11518,0,700000),-- Adena
(21404,12266,1,1,1,962),-- Common Item - Sealed Robe of Nightmare
(21404,12259,1,1,1,962),-- Common Item - Sealed Majestic Robe
(21404,5487,1,1,1,4665),-- Sealed Robe of Nightmare Fabric
(21404,5488,1,1,1,4665),-- Sealed Majestic Robe Fabric
(21404,959,1,1,2,28),-- Scroll: Enchant Weapon (Grade S)
(21404,1867,12,36,2,31250),-- Animal Skin
(21404,1869,2,6,2,125000),-- Iron Ore
(21404,1870,2,6,2,90909),-- Coal
(21404,1871,3,9,2,100000),-- Charcoal
(21404,1872,7,21,2,50000),-- Animal Bone
(21404,8732,1,1,200,5000),-- Life Stone: level 76
(21404,8742,1,1,200,385),-- Mid-Grade Life Stone: level 76
(21405,5167,1,1,-1,4454),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21405,5515,1,1,-1,169243),-- Sealed Majestic Gloves Design
(21405,6901,1,1,-1,10021),-- Recipe: Shining Arrow (100%)
(21405,57,4031,8230,0,700000),-- Adena
(21405,12266,1,1,1,121),-- Common Item - Sealed Robe of Nightmare
(21405,12259,1,1,1,121),-- Common Item - Sealed Majestic Robe
(21405,5487,1,1,1,3333),-- Sealed Robe of Nightmare Fabric
(21405,5488,1,1,1,3333),-- Sealed Majestic Robe Fabric
(21405,959,1,1,2,970),-- Scroll: Enchant Weapon (Grade S)
(21405,1867,1,3,2,125000),-- Animal Skin
(21405,1869,2,6,2,90909),-- Iron Ore
(21405,1870,2,6,2,83333),-- Coal
(21405,1871,3,9,2,66667),-- Charcoal
(21405,1872,7,21,2,27778),-- Animal Bone
(21405,8732,1,1,200,3636),-- Life Stone: level 76
(21405,8742,1,1,200,970),-- Mid-Grade Life Stone: level 76
(21406,5277,1,1,-1,6066),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21406,5523,1,1,-1,46482),-- Sealed Majestic Plate Armor Pattern
(21406,5527,1,2,-1,163187),-- Sealed Helm of Nightmare Design
(21406,57,5491,11210,0,700000),-- Adena
(21406,12264,1,1,1,989),-- Common Item - Sealed Gloves of Nightmare
(21406,12267,1,1,1,1647),-- Common Item - Sealed Boots of Nightmare
(21406,12257,1,1,1,989),-- Common Item - Sealed Majestic Gloves
(21406,12260,1,1,1,1647),-- Common Item - Sealed Majestic Boots
(21406,5502,1,1,1,8511),-- Sealed Boots of Nightmare Lining
(21406,5503,1,1,1,8511),-- Sealed Majestic Boots Lining
(21406,5514,1,1,1,8511),-- Sealed Gloves of Nightmare Design
(21406,5515,1,1,1,8511),-- Sealed Majestic Gloves Design
(21406,959,1,1,2,27),-- Scroll: Enchant Weapon (Grade S)
(21406,1868,3,9,2,166667),-- Thread
(21406,1873,2,6,2,47619),-- Silver Nugget
(21406,1876,1,1,2,83333),-- Mithril Ore
(21406,1895,1,1,2,142857),-- Metallic Fiber
(21406,4042,1,1,2,10204),-- Enria
(21406,8732,1,1,200,4608),-- Life Stone: level 76
(21406,8742,1,1,200,470),-- Mid-Grade Life Stone: level 76
(21407,5277,1,1,-1,3888),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21407,5523,1,1,-1,29795),-- Sealed Majestic Plate Armor Pattern
(21407,5527,1,1,-1,104602),-- Sealed Helm of Nightmare Design
(21407,57,3520,7185,0,700000),-- Adena
(21407,12264,1,1,1,1333),-- Common Item - Sealed Gloves of Nightmare
(21407,12267,1,1,1,1143),-- Common Item - Sealed Boots of Nightmare
(21407,12257,1,1,1,1269),-- Common Item - Sealed Majestic Gloves
(21407,12260,1,1,1,825),-- Common Item - Sealed Majestic Boots
(21407,5502,1,1,1,5456),-- Sealed Boots of Nightmare Lining
(21407,5503,1,1,1,5456),-- Sealed Majestic Boots Lining
(21407,5514,1,1,1,5456),-- Sealed Gloves of Nightmare Design
(21407,5515,1,1,1,5456),-- Sealed Majestic Gloves Design
(21407,959,1,1,2,63),-- Scroll: Enchant Weapon (Grade S)
(21407,1868,3,9,2,90909),-- Thread
(21407,1873,1,1,2,111111),-- Silver Nugget
(21407,1876,1,1,2,58824),-- Mithril Ore
(21407,1895,1,1,2,90909),-- Metallic Fiber
(21407,4042,1,1,2,5714),-- Enria
(21407,8732,1,1,200,3623),-- Life Stone: level 76
(21407,8742,1,1,200,571),-- Mid-Grade Life Stone: level 76
(21408,5282,1,1,-1,5732),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21408,5529,1,1,-1,35381),-- Dragon Slayer Edge
(21408,6901,1,1,-1,12896),-- Recipe: Shining Arrow (100%)
(21408,57,5188,10592,0,700000),-- Adena
(21408,11964,1,1,1,231),-- Common Item - Dark Legion's Edge
(21408,11971,1,1,1,231),-- Common Item - Sword of Miracles 
(21408,11968,1,1,1,231),-- Common Item - Dragon Slayer
(21408,5529,1,1,1,1743),-- Dragon Slayer Edge
(21408,5545,1,1,1,1743),-- Dark Legion\'s Edge Blade
(21408,5546,1,1,1,1914),-- Sword of Miracles Edge
(21408,959,1,1,2,25),-- Scroll: Enchant Weapon (Grade S)
(21408,1868,3,9,2,142857),-- Thread
(21408,1873,1,1,2,200000),-- Silver Nugget
(21408,1889,1,1,2,26316),-- Compound Braid
(21408,4044,1,1,2,13158),-- Thons
(21408,5549,1,1,2,45455),-- Metallic Thread
(21408,8732,1,1,200,5525),-- Life Stone: level 76
(21408,8742,1,1,200,346),-- Mid-Grade Life Stone: level 76
(21408,8752,1,1,200,115),-- High-Grade Life Stone: level 76
(21409,5282,1,1,-1,6410),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21409,5529,1,1,-1,39565),-- Dragon Slayer Edge
(21409,6901,1,1,-1,14422),-- Recipe: Shining Arrow (100%)
(21409,57,5802,11844,0,700000),-- Adena
(21409,11964,1,1,1,354),-- Common Item - Dark Legion's Edge
(21409,11971,1,1,1,354),-- Common Item - Sword of Miracles 
(21409,11968,1,1,1,354),-- Common Item - Dragon Slayer
(21409,5529,1,1,1,1949),-- Dragon Slayer Edge
(21409,5545,1,1,1,1949),-- Dark Legion\'s Edge Blade
(21409,5546,1,1,1,2140),-- Sword of Miracles Edge
(21409,959,1,1,2,28),-- Scroll: Enchant Weapon (Grade S)
(21409,1868,3,9,2,166667),-- Thread
(21409,1873,1,1,2,200000),-- Silver Nugget
(21409,1889,1,1,2,27027),-- Compound Braid
(21409,4044,1,1,2,16393),-- Thons
(21409,5549,1,1,2,40000),-- Metallic Thread
(21409,8732,1,1,200,4237),-- Life Stone: level 76
(21409,8742,1,1,200,943),-- Mid-Grade Life Stone: level 76
(21409,8752,1,1,200,236),-- High-Grade Life Stone: level 76
(21410,5282,1,1,-1,5808),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21410,5538,1,1,-1,35845),-- Dragon Grinder Edge
(21410,5541,1,1,-1,35838),-- Tallum Glaive Edge
(21410,57,5247,10744,0,700000),-- Adena
(21410,12350,1,1,1,1340),-- Common Item - Sealed Majestic Necklace
(21410,12352,1,1,1,2105),-- Common Item - Sealed Majestic Earring
(21410,12351,1,1,1,3831),-- Common Item - Sealed Majestic Ring
(21410,6342,1,1,1,14574),-- Sealed Majestic Earring Gemstone
(21410,6344,1,1,1,9432),-- Sealed Majestic Necklace Beads
(21410,6346,1,1,1,20946),-- Sealed Majestic Ring Gemstone
(21410,960,1,1,2,192),-- Scroll: Enchant Armor (Grade S)
(21410,1344,40,120,2,125000),-- Mithril Arrow
(21410,1867,5,15,2,50000),-- Animal Skin
(21410,1872,1,3,2,100000),-- Animal Bone
(21410,1882,1,1,2,76923),-- Leather
(21410,1894,1,1,2,16949),-- Crafted Leather
(21410,4042,1,1,2,7299),-- Enria
(21410,8732,1,1,200,3067),-- Life Stone: level 76
(21410,8742,1,1,200,958),-- Mid-Grade Life Stone: level 76
(21411,5282,1,1,-1,4735),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21411,5538,1,1,-1,29220),-- Dragon Grinder Edge
(21411,5541,1,1,-1,29214),-- Tallum Glaive Edge
(21411,57,4277,8758,0,700000),-- Adena
(21411,12350,1,1,1,1404),-- Common Item - Sealed Majestic Necklace
(21411,12352,1,1,1,1404),-- Common Item - Sealed Majestic Earring
(21411,12351,1,1,1,1686),-- Common Item - Sealed Majestic Ring
(21411,6342,1,1,1,11880),-- Sealed Majestic Earring Gemstone
(21411,6344,1,1,1,7688),-- Sealed Majestic Necklace Beads
(21411,6346,1,1,1,17074),-- Sealed Majestic Ring Gemstone
(21411,960,1,1,2,180),-- Scroll: Enchant Armor (Grade S)
(21411,1344,40,120,2,100000),-- Mithril Arrow
(21411,1867,5,15,2,40000),-- Animal Skin
(21411,1872,1,3,2,142857),-- Animal Bone
(21411,1882,1,1,2,76923),-- Leather
(21411,1894,1,1,2,10417),-- Crafted Leather
(21411,4042,1,1,2,4878),-- Enria
(21411,8732,1,1,200,3279),-- Life Stone: level 76
(21411,8742,1,1,200,375),-- Mid-Grade Life Stone: level 76
(21411,8752,1,1,200,81),-- High-Grade Life Stone: level 76
(21412,1808,1,1,-1,7556),-- Recipe: Soulshot: S Grade
(21412,1876,1,5,-1,850033),-- Mithril Ore
(21412,5545,1,1,-1,46641),-- Dark Legion\'s Edge Blade
(21412,57,6826,13977,0,700000),-- Adena
(21412,12269,1,1,1,1183),-- Common Item - Sealed Helm of Nightmare
(21412,12268,1,1,1,1776),-- Common Item - Sealed Shield of Nightmare
(21412,12261,1,1,1,2364),-- Common Item - Sealed Majestic Circlet
(21412,5495,1,1,1,12709),-- Sealed Shield of Nightmare Fragment
(21412,5527,1,1,1,10007),-- Sealed Helm of Nightmare Design
(21412,5528,1,1,1,10007),-- Sealed Majestic Circlet Design
(21412,960,1,1,2,287),-- Scroll: Enchant Armor (Grade S)
(21412,1344,150,450,2,43478),-- Mithril Arrow
(21412,1868,4,12,2,125000),-- Thread
(21412,1873,3,9,2,23810),-- Silver Nugget
(21412,1876,1,1,2,111111),-- Mithril Ore
(21412,1895,1,1,2,166667),-- Metallic Fiber
(21412,4042,1,1,2,11236),-- Enria
(21412,8732,1,1,200,2959),-- Life Stone: level 76
(21412,8742,1,1,200,592),-- Mid-Grade Life Stone: level 76
(21413,1808,1,1,-1,5810),-- Recipe: Soulshot: S Grade
(21413,1876,1,5,-1,871537),-- Mithril Ore
(21413,5545,1,1,-1,35866),-- Dark Legion\'s Edge Blade
(21413,57,5249,10748,0,700000),-- Adena
(21413,12269,1,1,1,1692),-- Common Item - Sealed Helm of Nightmare
(21413,12268,1,1,1,1692),-- Common Item - Sealed Shield of Nightmare
(21413,12261,1,1,1,1692),-- Common Item - Sealed Majestic Circlet
(21413,5495,1,1,1,9773),-- Sealed Shield of Nightmare Fragment
(21413,5527,1,1,1,7695),-- Sealed Helm of Nightmare Design
(21413,5528,1,1,1,7695),-- Sealed Majestic Circlet Design
(21413,960,1,1,2,221),-- Scroll: Enchant Armor (Grade S)
(21413,1344,150,450,2,28571),-- Mithril Arrow
(21413,1868,4,12,2,90909),-- Thread
(21413,1873,1,1,2,142857),-- Silver Nugget
(21413,1876,1,1,2,66667),-- Mithril Ore
(21413,1895,1,1,2,125000),-- Metallic Fiber
(21413,4042,1,1,2,7874),-- Enria
(21413,8732,1,1,200,1692),-- Life Stone: level 76
(21413,8742,1,1,200,1692),-- Mid-Grade Life Stone: level 76
(21414,1877,1,1,-1,508201),-- Adamantite Nugget
(21414,4040,1,1,-1,254101),-- Mold Lubricant
(21414,6344,1,1,-1,186250),-- Sealed Majestic Necklace Beads
(21414,57,5101,10446,0,700000),-- Adena
(21414,11970,1,1,1,32),-- Common Item - Branch of The Mother Tree
(21414,11974,1,1,1,32),-- Common Item - Elysian
(21414,11969,1,1,1,32),-- Common Item - Flaming Dragon Skull
(21414,11966,1,1,1,32),-- Common Item - Doom Crusher
(21414,5533,1,1,1,2574),-- Elysian Head
(21414,5544,1,1,1,2572),-- Branch of The Mother Tree Head
(21414,8349,1,1,1,605),-- Doom Crusher Head
(21414,8342,1,1,1,1818),-- Flaming Dragon Skull Piece
(21414,960,1,1,2,128),-- Scroll: Enchant Armor (Grade S)
(21414,1344,20,60,2,250000),-- Mithril Arrow
(21414,1866,2,6,2,66667),-- Suede
(21414,1871,1,3,2,125000),-- Charcoal
(21414,1877,1,1,2,14085),-- Adamantite Nugget
(21414,4040,1,1,2,7299),-- Mold Lubricant
(21414,4041,1,1,2,3448),-- Mold Hardener
(21414,8732,1,1,200,3257),-- Life Stone: level 76
(21414,8742,1,1,200,512),-- Mid-Grade Life Stone: level 76
(21414,8752,1,1,200,64),-- High-Grade Life Stone: level 76
(21415,1877,1,2,-1,512669),-- Adamantite Nugget
(21415,4040,1,1,-1,256334),-- Mold Lubricant
(21415,6344,1,1,-1,187887),-- Sealed Majestic Necklace Beads
(21415,57,5146,10537,0,700000),-- Adena
(21415,11970,1,1,1,77),-- Common Item - Branch of The Mother Tree
(21415,11974,1,1,1,77),-- Common Item - Elysian
(21415,11969,1,1,1,77),-- Common Item - Flaming Dragon Skull
(21415,11966,1,1,1,77),-- Common Item - Doom Crusher
(21415,8342,1,1,1,1818),-- Flaming Dragon Skull Piece
(21415,8349,1,1,1,605),-- Doom Crusher Head
(21415,5533,1,1,1,2597),-- Elysian Head
(21415,5544,1,1,1,2595),-- Branch of The Mother Tree Head
(21415,960,1,1,2,315),-- Scroll: Enchant Armor (Grade S)
(21415,1344,20,60,2,250000),-- Mithril Arrow
(21415,1866,2,6,2,58824),-- Suede
(21415,1871,1,3,2,111111),-- Charcoal
(21415,1877,1,1,2,17241),-- Adamantite Nugget
(21415,4040,1,1,2,9804),-- Mold Lubricant
(21415,4041,1,1,2,3460),-- Mold Hardener
(21415,8732,1,1,200,1887),-- Life Stone: level 76
(21416,1885,1,3,-1,547540),-- High Grade Suede
(21416,4041,1,1,-1,114269),-- Mold Hardener
(21416,5480,1,1,-1,58966),-- Sealed Leather Armor of Nightmare Fabric
(21416,57,5276,10804,0,700000),-- Adena
(21416,11975,1,1,1,15),-- Common Item - Soul Bow (no drop info)
(21416,5534,1,1,1,5328),-- Soul Bow Stave
(21416,960,1,1,2,222),-- Scroll: Enchant Armor (Grade S)
(21416,1344,100,300,2,52632),-- Mithril Arrow
(21416,1866,2,6,2,100000),-- Suede
(21416,1871,1,3,2,111111),-- Charcoal
(21416,1885,1,1,2,47619),-- High Grade Suede
(21416,4041,1,1,2,4822),-- Mold Hardener
(21416,5220,1,1,2,9346),-- Metal Hardener
(21416,8732,1,1,200,1408),-- Life Stone: level 76
(21416,8752,1,1,200,469),-- High-Grade Life Stone: level 76
(21417,1885,1,3,-1,853217),-- High Grade Suede
(21417,4041,1,1,-1,178063),-- Mold Hardener
(21417,5480,1,1,-1,91886),-- Sealed Leather Armor of Nightmare Fabric
(21417,57,8221,16836,0,700000),-- Adena
(21417,11975,1,1,1,23),-- Common Item - Soul Bow (no drop info, no material info)
(21417,5534,1,1,1,8303),-- Soul Bow Stave
(21417,960,1,1,2,346),-- Scroll: Enchant Armor (Grade S)
(21417,1344,100,300,2,108012),-- Mithril Arrow
(21417,1866,2,6,2,144016),-- Suede
(21417,1871,1,3,2,250000),-- Charcoal
(21417,1885,1,1,2,72008),-- High Grade Suede
(21417,4041,1,1,2,7514),-- Mold Hardener
(21417,5220,1,1,2,34564),-- Metal Hardener
(21418,3036,1,1,-1,4837),-- Recipe: Spiritshot S
(21418,4044,1,1,-1,362779),-- Thons
(21418,5528,1,2,-1,130122),-- Sealed Majestic Circlet Design
(21418,57,4369,8949,0,700000),-- Adena
(21418,12269,1,1,1,575),-- Common Item - Sealed Helm of Nightmare
(21418,12268,1,1,1,1724),-- Common Item - Sealed Shield of Nightmare
(21418,12261,1,1,1,627),-- Common Item - Sealed Majestic Circlet
(21418,5495,1,1,1,8136),-- Sealed Shield of Nightmare Fragment
(21418,5527,1,1,1,6406),-- Sealed Helm of Nightmare Design
(21418,5528,1,1,1,6406),-- Sealed Majestic Circlet Design
(21418,959,1,1,2,22),-- Scroll: Enchant Weapon (Grade S)
(21418,1868,3,9,2,125000),-- Thread
(21418,1873,1,1,2,142857),-- Silver Nugget
(21418,1889,1,1,2,23810),-- Compound Braid
(21418,4044,1,1,2,12346),-- Thons
(21418,5549,1,1,2,38462),-- Metallic Thread
(21418,8732,1,1,200,2874),-- Life Stone: level 76
(21418,8742,1,1,200,941),-- Mid-Grade Life Stone: level 76
(21418,8752,1,1,200,157),-- High-Grade Life Stone: level 76
(21419,3036,1,1,-1,4926),-- Recipe: Spiritshot S
(21419,4044,1,1,-1,369466),-- Thons
(21419,5528,1,1,-1,132520),-- Sealed Majestic Circlet Design
(21419,57,4450,9114,0,700000),-- Adena (no drop info, no material info, no spawn)
(21419,12269,1,1,1,42),-- Common Item - Sealed Helm of Nightmare
(21419,12268,1,1,1,59),-- Common Item - Sealed Shield of Nightmare
(21419,12261,1,1,1,42),-- Common Item - Sealed Majestic Circlet
(21419,5495,1,1,1,8286),-- Sealed Shield of Nightmare Fragment
(21419,5527,1,1,1,6524),-- Sealed Helm of Nightmare Design
(21419,5528,1,1,1,6524),-- Sealed Majestic Circlet Design
(21419,959,1,1,2,22),-- Scroll: Enchant Weapon (Grade S)
(21419,1868,3,9,2,181891),-- Thread
(21419,1873,2,6,2,54568),-- Silver Nugget
(21419,1889,1,1,2,36379),-- Compound Braid
(21419,4044,1,1,2,18189),-- Thons
(21419,5549,1,1,2,54568),-- Metallic Thread
(21420,5157,1,1,-1,6020),-- Recipe: Soulshot (S) Compressed Package (100%)
(21420,5533,1,1,-1,37153),-- Elysian Head
(21420,5534,1,1,-1,37186),-- Soul Bow Stave
(21420,57,5438,11138,0,700000),-- Adena
(21420,11970,1,1,1,151),-- Common Item - Branch of The Mother Tree
(21420,11974,1,1,1,151),-- Common Item - Elysian
(21420,11969,1,1,1,151),-- Common Item - Flaming Dragon Skull
(21420,11966,1,1,1,151),-- Common Item - Doom Crushe
(21420,8342,1,1,1,1639),-- Flaming Dragon Skull Piece
(21420,8349,1,1,1,1474),-- Doom Crusher Head
(21420,5533,1,1,1,2744),-- Elysian Head
(21420,5544,1,1,1,2742),-- Branch of The Mother Tree Head
(21420,959,1,1,2,27),-- Scroll: Enchant Weapon (Grade S)
(21420,1868,3,9,2,142587),-- Thread
(21420,1873,2,6,2,47619),-- Silver Nugget
(21420,1876,1,1,2,90909),-- Mithril Ore
(21420,1895,1,1,2,125000),-- Metallic Fiber
(21420,4042,1,1,2,8403),-- Enria
(21420,8732,1,1,200,2410),-- Life Stone: level 76
(21420,8742,1,1,200,602),-- Mid-Grade Life Stone: level 76
(21421,5157,1,1,-1,5651),-- Recipe: Soulshot (S) Compressed Package (100%)
(21421,5533,1,1,-1,34876),-- Elysian Head
(21421,5534,1,1,-1,34907),-- Soul Bow Stave
(21421,57,5104,10455,0,700000),-- Adena
(21421,11970,1,1,1,114),-- Common Item - Branch of The Mother Tree
(21421,11974,1,1,1,114),-- Common Item - Elysian
(21421,11969,1,1,1,114),-- Common Item - Flaming Dragon Skull
(21421,11966,1,1,1,114),-- Common Item - Doom Crushe
(21421,8342,1,1,1,1375),-- Flaming Dragon Skull Piece
(21421,8349,1,1,1,1547),-- Doom Crusher Head
(21421,5533,1,1,1,2576),-- Elysian Head
(21421,5544,1,1,1,2574),-- Branch of The Mother Tree Head
(21421,959,1,1,2,25),-- Scroll: Enchant Weapon (Grade S)
(21421,1868,3,9,2,125000),-- Thread
(21421,1873,2,6,2,45455),-- Silver Nugget
(21421,1876,1,1,2,83333),-- Mithril Ore
(21421,1895,1,1,2,111111),-- Metallic Fiber
(21421,4042,1,1,2,9091),-- Enria
(21421,8732,1,1,200,5435),-- Life Stone: level 76
(21421,8742,1,1,200,1209),-- Mid-Grade Life Stone: level 76
(21422,5162,1,1,-1,6483),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21422,5167,1,1,-1,6483),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21422,5538,1,1,-1,40009),-- Dragon Grinder Edge
(21422,57,5855,11994,0,700000),-- Adena
(21422,11975,1,1,1,643),-- Common Item - Soul Bow
(21422,5534,1,1,1,5915),-- Soul Bow Stave
(21422,959,1,1,2,29),-- Scroll: Enchant Weapon (Grade S)
(21422,1867,4,12,2,83333),-- Animal Skin
(21422,1872,1,3,2,200000),-- Animal Bone
(21422,1882,1,1,2,111111),-- Leather
(21422,1894,1,1,2,17241),-- Crafted Leather
(21422,4042,1,1,2,7634),-- Enria
(21422,8732,1,1,200,4975),-- Life Stone: level 76
(21422,8742,1,1,200,1178),-- Mid-Grade Life Stone: level 76
(21422,8752,1,1,200,214),-- High-Grade Life Stone: level 76
(21423,5162,1,1,-1,6872),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21423,5167,1,1,-1,6872),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21423,5538,1,1,-1,42413),-- Dragon Grinder Edge
(21423,57,6207,12714,0,700000),-- Adena
(21423,11975,1,1,1,276),-- Common Item - Soul Bow
(21423,5534,1,1,1,6270),-- Soul Bow Stave
(21423,959,1,1,2,31),-- Scroll: Enchant Weapon (Grade S)
(21423,1867,4,12,2,83333),-- Animal Skin
(21423,1872,1,3,2,250000),-- Animal Bone
(21423,1882,1,1,2,111111),-- Leather
(21423,1894,1,1,2,15625),-- Crafted Leather
(21423,4042,1,1,2,9434),-- Enria
(21423,8732,1,1,200,4405),-- Life Stone: level 76
(21423,8742,1,1,200,1214),-- Mid-Grade Life Stone: level 76
(21423,8752,1,1,200,110),-- High-Grade Life Stone: level 76
(21424,5272,1,1,-1,8710),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21424,5544,1,1,-1,53702),-- Branch of The Mother Tree Head
(21424,5545,1,1,-1,53765),-- Dark Legion\'s Edge Blade
(21424,57,7867,16114,0,700000),-- Adena
(21424,11976,1,1,1,392),-- Common Item - Tallum Glaive
(21424,5541,1,1,1,7937),-- Tallum Glaive Edge
(21424,959,1,1,2,34),-- Scroll: Enchant Weapon (Grade S)
(21424,1868,15,45,2,40000),-- Thread
(21424,1873,2,6,2,55556),-- Silver Nugget
(21424,1876,1,1,2,111111),-- Mithril Ore
(21424,1895,1,1,2,166667),-- Metallic Fiber
(21424,4042,1,1,2,10638),-- Enria
(21424,6672,1,1,2,47619),-- Deluxe Chest Key - Grade 8
(21424,8732,1,1,200,4444),-- Life Stone: level 76
(21424,8742,1,1,200,871),-- Mid-Grade Life Stone: level 76
(21424,8752,1,1,200,44),-- High-Grade Life Stone: level 76
(21425,5272,1,1,-1,8856),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21425,5544,1,1,-1,54603),-- Branch of The Mother Tree Head
(21425,5545,1,1,-1,54667),-- Dark Legion\'s Edge Blade
(21425,57,7999,16385,0,700000),-- Adena
(21425,11976,1,1,1,303),-- Common Item - Tallum Glaive
(21425,5541,1,1,1,8071),-- Tallum Glaive Edge
(21425,959,1,1,2,130),-- Scroll: Enchant Weapon (Grade S)
(21425,1868,15,45,2,43478),-- Thread
(21425,1873,2,6,2,66667),-- Silver Nugget
(21425,1876,1,1,2,125000),-- Mithril Ore
(21425,1895,1,1,2,200000),-- Metallic Fiber
(21425,4042,1,1,2,11765),-- Enria
(21425,8732,1,1,200,5291),-- Life Stone: level 76
(21425,8742,1,1,200,1689),-- Mid-Grade Life Stone: level 76
(21425,8752,1,1,200,43),-- High-Grade Life Stone: level 76
(21426,6698,1,2,-1,359685),-- Sealed Tateossian Earring  Part
(21426,6699,1,1,-1,399119),-- Sealed Tateossian Ring Gem
(21426,6700,1,1,-1,219221),-- Sealed Tateossian Ring Wire
(21426,57,7369,15040,0,700000),-- Adena
(21426,11975,1,1,1,518),-- Common Item - Soul Bow
(21426,5534,1,1,1,7425),-- Soul Bow Stave
(21426,960,1,1,2,194),-- Scroll: Enchant Armor (Grade S)
(21426,1344,150,450,2,45455),-- Mithril Arrow
(21426,1868,4,12,2,142857),-- Thread
(21426,1873,3,9,2,37037),-- Silver Nugget
(21426,1876,1,1,2,111111),-- Mithril Ore
(21426,1895,3,9,2,20000),-- Metallic Fiber
(21426,4042,1,1,2,10526),-- Enria
(21426,8732,1,1,200,5376),-- Life Stone: level 76
(21426,8742,1,1,200,1488),-- Mid-Grade Life Stone: level 76
(21426,8752,1,1,200,129),-- High-Grade Life Stone: level 76
(21427,6698,1,1,-1,244891),-- Sealed Tateossian Earring  Part
(21427,6699,1,1,-1,271740),-- Sealed Tateossian Ring Gem
(21427,6700,1,1,-1,149257),-- Sealed Tateossian Ring Wire
(21427,57,5017,10240,0,700000),-- Adena
(21427,11975,1,1,1,327),-- Common Item - Soul Bow
(21427,5534,1,1,1,5056),-- Soul Bow Stave
(21427,960,1,1,2,164),-- Scroll: Enchant Armor (Grade S)
(21427,1344,150,450,2,32258),-- Mithril Arrow
(21427,1868,4,12,2,90909),-- Thread
(21427,1873,1,1,2,142857),-- Silver Nugget
(21427,1876,1,1,2,71429),-- Mithril Ore
(21427,1895,1,1,2,111111),-- Metallic Fiber
(21427,4042,1,1,2,5556),-- Enria
(21427,8732,1,1,200,2451),-- Life Stone: level 76
(21427,8742,1,1,200,1391),-- Mid-Grade Life Stone: level 76
(21427,8752,1,1,200,164),-- High-Grade Life Stone: level 76
(21428,6701,1,1,-1,68629),-- Sealed Imperial Crusader Breastplate Part
(21428,6702,1,1,-1,88343),-- Sealed Imperial Crusader Gaiters Pattern
(21428,6703,1,1,-1,183529),-- Sealed Imperial Crusader Gauntlets Design
(21428,57,5187,10587,0,700000),-- Adena
(21428,11973,1,1,1,630),-- Common Item - Soul Separator
(21428,5537,1,1,1,5223),-- Soul Separator Head
(21428,960,1,1,2,126),-- Scroll: Enchant Armor (Grade S)
(21428,1344,30,90,2,166667),-- Mithril Arrow
(21428,1868,4,12,2,100000),-- Thread
(21428,1873,1,1,2,166667),-- Silver Nugget
(21428,1889,1,1,2,25000),-- Compound Braid
(21428,4044,1,1,2,12500),-- Thons
(21428,5549,1,1,2,43478),-- Metallic Thread
(21428,8732,1,1,200,3401),-- Life Stone: level 76
(21428,8742,1,1,200,1261),-- Mid-Grade Life Stone: level 76
(21428,8752,1,1,200,126),-- High-Grade Life Stone: level 76
(21429,6701,1,1,-1,71661),-- Sealed Imperial Crusader Breastplate Part
(21429,6702,1,1,-1,92247),-- Sealed Imperial Crusader Gaiters Pattern
(21429,6703,1,1,-1,191638),-- Sealed Imperial Crusader Gauntlets Design
(21429,57,5417,11055,0,700000),-- Adena
(21429,11973,1,1,1,678),-- Common Item - Soul Separator
(21429,5537,1,1,1,5454),-- Soul Separator Head
(21429,960,1,1,2,170),-- Scroll: Enchant Armor (Grade S)
(21429,1344,150,450,2,34483),-- Mithril Arrow
(21429,1868,4,12,2,111111),-- Thread
(21429,1873,1,1,2,166667),-- Silver Nugget
(21429,1889,1,1,2,24390),-- Compound Braid
(21429,4044,1,1,2,10870),-- Thons
(21429,5549,1,1,2,41667),-- Metallic Thread
(21429,8732,1,1,200,3559),-- Life Stone: level 76
(21429,8742,1,1,200,339),-- Mid-Grade Life Stone: level 76
(21430,6704,1,1,-1,264663),-- Sealed Imperial Crusader Boots Design
(21430,6705,1,1,-1,259237),-- Sealed Imperial Crusader Shield Part
(21430,6706,1,1,-1,301543),-- Sealed Imperial Crusader Helmet Pattern
(21430,57,7481,15268,0,700000),-- Adena
(21430,12265,1,1,1,431),-- Common Item - Sealed Leather Armor of Nightmare
(21430,12258,1,1,1,1007),-- Common Item - Sealed Majestic Leather Armor
(21430,5480,1,1,1,6161),-- Sealed Leather Armor of Nightmare Fabric
(21430,5481,1,1,1,6161),-- Sealed Majestic Leather Armor Fabric
(21430,960,1,1,2,431),-- Scroll: Enchant Armor (Grade S)
(21430,1344,150,450,2,43478),-- Mithril Arrow
(21430,1868,4,12,2,142857),-- Thread
(21430,1873,3,9,2,38462),-- Silver Nugget
(21430,1876,1,1,2,111111),-- Mithril Ore
(21430,1895,3,9,2,23810),-- Metallic Fiber
(21430,4042,1,1,2,8621),-- Enria
(21430,8732,1,1,200,4310),-- Life Stone: level 76
(21430,8742,1,1,200,1439),-- Mid-Grade Life Stone: level 76
(21430,8752,1,1,200,144),-- High-Grade Life Stone: level 76
(21431,6704,1,1,-1,225750),-- Sealed Imperial Crusader Boots Design
(21431,6705,1,1,-1,221122),-- Sealed Imperial Crusader Shield Part
(21431,6706,1,1,-1,257207),-- Sealed Imperial Crusader Helmet Pattern
(21431,57,6381,13023,0,700000),-- Adena
(21431,12265,1,1,1,674),-- Common Item - Sealed Leather Armor of Nightmare
(21431,12258,1,1,1,674),-- Common Item - Sealed Majestic Leather Armor
(21431,5480,1,1,1,5255),-- Sealed Leather Armor of Nightmare Fabric
(21431,5481,1,1,1,5255),-- Sealed Majestic Leather Armor Fabric
(21431,960,1,1,2,268),-- Scroll: Enchant Armor (Grade S)
(21431,1344,150,450,2,34483),-- Mithril Arrow
(21431,1868,4,12,2,125000),-- Thread
(21431,1873,3,9,2,33333),-- Silver Nugget
(21431,1876,1,1,2,111111),-- Mithril Ore
(21431,1895,1,1,2,142857),-- Metallic Fiber
(21431,4042,1,1,2,5051),-- Enria
(21431,8732,1,1,200,4386),-- Life Stone: level 76
(21431,8742,1,1,200,674),-- Mid-Grade Life Stone: level 76
(21431,8752,1,1,200,337),-- High-Grade Life Stone: level 76
(21432,1808,1,1,-1,9332),-- Recipe: Soulshot: S Grade
(21432,5534,1,1,-1,57647),-- Soul Bow Stave
(21432,5537,1,1,-1,57607),-- Soul Separator Head
(21432,57,8447,17245,0,700000),-- Adena
(21432,11967,1,1,1,23),-- Common Item - Dragon Grinder (no drop info, no material info)
(21432,5538,1,1,1,8508),-- Dragon Grinder Edge
(21432,959,1,1,2,42),-- Scroll: Enchant Weapon (Grade S)
(21432,1868,15,45,2,68916),-- Thread
(21432,1873,2,6,2,103374),-- Silver Nugget
(21432,1876,1,1,2,206747),-- Mithril Ore
(21432,1895,3,9,2,49226),-- Metallic Fiber
(21432,4042,1,1,2,17229),-- Enria
(21433,5167,1,1,-1,9039),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21433,5538,1,1,-1,55789),-- Dragon Grinder Edge
(21433,5541,1,1,-1,55777),-- Tallum Glaive Edge
(21433,57,8184,16701,0,700000),-- Adena
(21433,12263,1,1,1,30),-- Common Item - Sealed Armor of Nightmare (no drop info, no material info)
(21433,12262,1,1,1,30),-- Common Item - Sealed Majestic Plate Armor
(21433,5522,1,1,1,5115),-- Sealed Armor of Nightmare Pattern
(21433,5523,1,1,1,5115),-- Sealed Majestic Plate Armor Pattern
(21433,959,1,1,2,41),-- Scroll: Enchant Weapon (Grade S)
(21433,1868,15,45,2,66754),-- Thread
(21433,1873,2,6,2,100130),-- Silver Nugget
(21433,1876,1,1,2,200259),-- Mithril Ore
(21433,1895,3,9,2,47681),-- Metallic Fiber
(21433,4042,1,1,2,16689),-- Enria
(21434,1877,1,3,-1,773988),-- Adamantite Nugget
(21434,5282,1,1,-1,17200),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21434,6342,1,1,-1,876643),-- Sealed Majestic Earring Gemstone
(21434,57,15536,31822,0,700000),-- Adena
(21434,12265,1,1,1,405),-- Common Item - Sealed Leather Armor of Nightmare
(21434,12258,1,1,1,405),-- Common Item - Sealed Majestic Leather Armor
(21434,5480,1,1,1,12826),-- Sealed Leather Armor of Nightmare Fabric
(21434,5481,1,1,1,12826),-- Sealed Majestic Leather Armor Fabric
(21434,959,1,1,2,77),-- Scroll: Enchant Weapon (Grade S)
(21434,1866,7,21,2,66667),-- Suede
(21434,1871,4,12,2,166667),-- Charcoal
(21434,1877,1,1,2,50000),-- Adamantite Nugget
(21434,4040,1,1,2,32258),-- Mold Lubricant
(21434,4041,1,1,2,9259),-- Mold Hardener
(21434,8732,1,1,200,10638),-- Life Stone: level 76
(21434,8742,1,1,200,2660),-- Mid-Grade Life Stone: level 76
(21435,1877,1,1,-1,501519),-- Adamantite Nugget
(21435,3036,1,1,-1,11145),-- Recipe: Spiritshot S
(21435,5538,1,1,-1,68782),-- Dragon Grinder Edge
(21435,57,10088,20595,0,700000),-- Adena
(21435,11975,1,1,1,28),-- Common Item - Soul Bow (no drop info)
(21435,5534,1,1,1,10168),-- Soul Bow Stave
(21435,960,1,1,2,2639),-- Scroll: Enchant Armor (Grade S)
(21435,1344,100,300,2,111111),-- Mithril Arrow
(21435,1866,2,6,2,100000),-- Suede
(21435,1871,9,27,2,40000),-- Charcoal
(21435,1877,1,1,1,38462),-- Adamantite Nugget
(21435,4040,1,1,2,17241),-- Mold Lubricant
(21435,4041,1,1,2,6623),-- Mold Hardener
(21435,8732,1,1,200,3968),-- Life Stone: level 76
(21435,8742,1,1,200,2639),-- Mid-Grade Life Stone: level 76
(21436,1894,1,1,-1,912701),-- Crafted Leather
(21436,3957,1,1,-1,11561),-- Recipe: Blessed Spiritshot S
(21436,5541,1,1,-1,71335),-- Tallum Glaive Edge
(21436,57,10464,21364,0,700000),-- Adena
(21436,11973,1,1,1,337),-- Common Item - Soul Separator
(21436,5537,1,1,1,10539),-- Soul Separator Head
(21436,959,1,1,2,51),-- Scroll: Enchant Weapon (Grade S)
(21436,1867,4,12,2,142857),-- Animal Skin
(21436,1872,7,21,2,83333),-- Animal Bone
(21436,1882,1,1,2,200000),-- Leather
(21436,1894,1,1,2,30303),-- Crafted Leather
(21436,4042,1,1,2,17241),-- Enria
(21436,8732,1,1,200,6757),-- Life Stone: level 76
(21436,8742,1,1,200,1350),-- Mid-Grade Life Stone: level 76
(21437,1877,1,1,-1,419120),-- Adamantite Nugget
(21437,4041,1,1,-1,91113),-- Mold Hardener
(21437,5544,1,1,-1,28712),-- Branch of The Mother Tree Head
(21437,57,4216,8604,0,700000),-- Adena
(21437,11970,1,1,1,6),-- Common Item - Branch of The Mother Tree (no drop info, no spawn)
(21437,11974,1,1,1,6),-- Common Item - Elysian
(21437,5533,1,1,1,2123),-- Elysian Head
(21437,5544,1,1,1,2121),-- Branch of The Mother Tree Head
(21437,960,1,1,2,177),-- Scroll: Enchant Armor (Grade S)
(21437,1344,100,300,2,55269),-- Mithril Arrow
(21437,1866,2,6,2,73692),-- Suede
(21437,1871,1,3,2,147384),-- Charcoal
(21437,1877,1,1,2,17686),-- Adamantite Nugget
(21437,4040,1,1,2,8843),-- Mold Lubricant
(21437,4041,1,1,2,3845),-- Mold Hardener
(21438,4077,1,1,-1,22626),-- Doom Shield Fragment
(21438,4078,1,1,-1,7347),-- Blue Wolf Breastplate Part
(21438,6333,1,1,-1,5775),-- Recipe: Sealed Phoenix Ring(70%)
(21438,57,270,442,0,700000),-- Adena
(21438,11930,1,1,1,2),-- Common Item - Sword of Damascus (no drop info, no material info)
(21438,4114,1,1,1,325),-- Sword of Damascus Blade
(21438,729,1,1,2,3),-- Scroll: Enchant Weapon (Grade A)
(21438,1868,1,1,2,49524),-- Thread
(21438,1873,1,1,2,9905),-- Silver Nugget
(21438,1876,1,1,2,4953),-- Mithril Ore
(21438,1895,1,1,2,7075),-- Metallic Fiber
(21438,4042,1,1,2,413),-- Enria
(21439,4116,1,1,-1,2272),-- Deadman\'s Glory Stone
(21439,4117,1,1,-1,2272),-- Art of Battle Axe Blade
(21439,5276,1,1,-1,693),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21439,57,255,464,0,700000),-- Adena
(21439,11932,1,1,1,2),-- Common Item - Lance (no drop info, no material info)
(21439,4115,1,1,1,335),-- Lance Blade
(21439,1343,10,30,2,31340),-- Silver Arrow
(21439,1864,1,1,2,43875),-- Stem
(21439,1865,1,1,2,21938),-- Varnish
(21439,1866,1,1,2,14625),-- Suede
(21439,1868,1,1,2,43875),-- Thread
(21439,1873,1,1,2,8776),-- Silver Nugget
(21439,5456,1,1,2,9),-- Recipe: Tallum Glaive(60%)
(21440,1875,1,1,-1,38040),-- Stone of Purity
(21440,4040,1,1,-1,11412),-- Mold Lubricant
(21440,4099,1,1,-1,25616),-- Doom Boots Part
(21440,57,283,529,0,700000),-- Adena
(21440,12201,1,1,1,2),-- Common Item - Blue Wolf Tunic (no drop info, no material info)
(21440,12192,1,1,1,2),-- Common Item - Tunic of Doom
(21440,12203,1,1,1,3),-- Common Item - Blue Wolf Stockings
(21440,12205,1,1,1,3),-- Common Item - Stockings of Doom
(21440,4084,1,1,1,366),-- Blue Wolf Tunic Fabric
(21440,4085,1,1,1,366),-- Tunic of Doom Pattern
(21440,4086,1,1,1,495),-- Blue Wolf Stockings Pattern
(21440,4087,1,1,1,495),-- Stockings of Doom Pattern
(21440,1343,10,30,2,34398),-- Silver Arrow
(21440,1865,1,1,2,24079),-- Varnish
(21440,1870,1,1,2,24079),-- Coal
(21440,1875,1,1,2,1606),-- Stone of Purity
(21440,1879,1,1,2,4014),-- Cokes
(21440,4040,1,1,2,482),-- Mold Lubricant
(21440,6335,1,1,2,93),-- Recipe: Sealed Majestic Necklace(70%)
(21441,729,1,1,-1,62),-- Scroll: Enchant Weapon (Grade A)
(21441,5448,1,1,-1,206),-- Recipe: Soul Separator(60%)
(21441,5494,1,1,-1,11655),-- Sealed Dark Crystal Shield Fragment
(21441,57,274,511,0,700000),-- Adena
(21441,11946,1,1,1,1),-- Common Item - Dasparion\'s Staff (no drop info, no material info)
(21441,11949,1,1,1,1),-- Common Item - Meteor Shower
(21441,5532,1,1,1,139),-- Meteor Shower Head
(21441,5543,1,1,1,140),-- Dasparion\'s Staff Edge
(21441,1343,10,30,2,33454),-- Silver Arrow
(21441,1865,1,1,2,23418),-- Varnish
(21441,1870,1,1,2,23418),-- Coal
(21441,1874,1,1,2,1562),-- Oriharukon Ore
(21441,4039,1,1,2,781),-- Mold Glue
(21441,4043,1,1,2,781),-- Asofe
(21441,5442,1,1,2,9),-- Recipe: Soul Bow(60%)
(21442,729,1,1,-1,64),-- Scroll: Enchant Weapon (Grade A)
(21442,5448,1,1,-1,212),-- Recipe: Soul Separator(60%)
(21442,6341,1,1,-1,15842),-- Sealed Phoenix Earring Gemstone
(21442,57,284,553,0,700000),-- Adena
(21442,11946,1,1,1,1),-- Common Item - Dasparion\'s Staff (no drop info, no material info)
(21442,11949,1,1,1,1),-- Common Item - Meteor Shower
(21442,5532,1,1,1,143),-- Meteor Shower Head
(21442,5543,1,1,1,144),-- Dasparion\'s Staff Edge
(21442,1343,10,30,2,34529),-- Silver Arrow
(21442,1865,1,1,2,24171),-- Varnish
(21442,1870,1,1,2,24171),-- Coal
(21442,1874,1,1,2,1612),-- Oriharukon Ore
(21442,4039,1,1,2,806),-- Mold Glue
(21442,4043,1,1,2,806),-- Asofe
(21442,5450,1,1,2,9),-- Recipe: Dragon Grinder(60%)
(21443,3956,1,1,-1,3132),-- Recipe: Blessed Spiritshot A
(21443,4077,1,1,-1,105674),-- Doom Shield Fragment
(21443,4078,1,1,-1,34314),-- Blue Wolf Breastplate Part
(21443,57,1140,2308,0,700000),-- Adena
(21443,12204,1,1,1,140),-- Common Item - Blue Wolf Breastplate
(21443,12193,1,1,1,210),-- Common Item - Blue Wolf Gaiters
(21443,12184,1,1,1,88),-- Common Item - Doom Plate Armor
(21443,4078,1,1,1,1690),-- Blue Wolf Breastplate Part
(21443,4080,1,1,1,2380),-- Blue Wolf Gaiters Material
(21443,4081,1,1,1,1140),-- Doom Plate Armor Temper
(21443,1343,10,30,2,100000),-- Silver Arrow
(21443,1867,1,1,2,90909),-- Animal Skin
(21443,1872,1,1,2,90909),-- Animal Bone
(21443,1875,1,1,2,4831),-- Stone of Purity
(21443,1881,1,1,2,10101),-- Coarse Bone Powder
(21443,4041,1,1,2,1261),-- Mold Hardener
(21443,5156,1,1,2,210),-- Recipe: Soulshot (A) Compressed Package (100%)
(21443,8729,1,1,200,1261),-- Life Stone: level 64
(21443,8739,1,1,200,420),-- Mid-Grade Life Stone: level 64
(21444,3956,1,1,-1,2632),-- Recipe: Blessed Spiritshot A
(21444,4077,1,1,-1,88802),-- Doom Shield Fragment
(21444,4078,1,1,-1,28835),-- Blue Wolf Breastplate Part
(21444,57,958,1940,0,700000),-- Adena
(21444,12204,1,1,1,165),-- Common Item - Blue Wolf Breastplate
(21444,12193,1,1,1,494),-- Common Item - Blue Wolf Gaiters
(21444,12184,1,1,1,165),-- Common Item - Doom Plate Armor
(21444,4078,1,1,1,1420),-- Blue Wolf Breastplate Part
(21444,4080,1,1,1,1999),-- Blue Wolf Gaiters Material
(21444,4081,1,1,1,958),-- Doom Plate Armor Temper
(21444,1343,10,30,2,83333),-- Silver Arrow
(21444,1867,1,1,2,83333),-- Animal Skin
(21444,1872,1,1,2,90909),-- Animal Bone
(21444,1875,1,1,2,3953),-- Stone of Purity
(21444,1881,1,1,2,8403),-- Coarse Bone Powder
(21444,4041,1,1,2,1153),-- Mold Hardener
(21444,5156,1,1,2,165),-- Recipe: Soulshot (A) Compressed Package (100%)
(21444,8729,1,1,200,1318),-- Life Stone: level 64
(21445,1875,1,2,-1,136158),-- Stone of Purity
(21445,4118,1,1,-1,8456),-- Evil Spirit Head
(21445,5460,1,1,-1,1116),-- Recipe: Dasparion\'s Staff(60%)
(21445,57,992,2007,0,700000),-- Adena
(21445,11930,1,1,1,16),-- Common Item - Sword of Damascus
(21445,11943,1,1,1,16),-- Common Item - Wizard\'s Tear
(21445,11929,1,1,1,16),-- Common Item - Guardian Sword
(21445,4114,1,1,1,1321),-- Sword of Damascus Blade
(21445,8336,1,1,1,417),-- Blade of the Wizard\'s Tear
(21445,8330,1,1,1,479),-- Guardian Sword Blade
(21445,1865,1,1,2,71429),-- Varnish
(21445,1870,1,1,2,76923),-- Coal
(21445,1875,1,1,2,3922),-- Stone of Purity
(21445,1879,1,1,2,12346),-- Cokes
(21445,4040,1,1,2,1166),-- Mold Lubricant
(21445,6329,1,1,2,186),-- Recipe: Sealed Phoenix Necklace(70%)
(21445,8729,1,1,200,1072),-- Life Stone: level 64
(21445,8739,1,1,200,513),-- Mid-Grade Life Stone: level 64
(21446,1885,1,1,-1,209411),-- High Grade Suede
(21446,4119,1,1,-1,10999),-- Demon Dagger Edge
(21446,5468,1,1,-1,1373),-- Recipe: Elemental Sword(60%)
(21446,57,1221,2470,0,700000),-- Adena
(21446,11935,1,1,1,40),-- Common Item - Deadman\'s Glory
(21446,11938,1,1,1,40),-- Common Item - Art of Battle Axe
(21446,11934,1,1,1,40),-- Common Item - Staff of Evil Spirits
(21446,4116,1,1,1,541),-- Deadman\'s Glory Stone
(21446,4117,1,1,1,541),-- Art of Battle Axe Blade
(21446,4118,1,1,1,513),-- Evil Spirit Head
(21446,1343,10,30,2,111111),-- Silver Arrow
(21446,1866,1,1,2,50000),-- Suede
(21446,1871,1,1,2,76923),-- Charcoal
(21446,1885,1,1,2,4762),-- High Grade Suede
(21446,4041,1,1,2,476),-- Mold Hardener
(21446,5220,1,1,2,2976),-- Metal Hardener
(21446,6331,1,1,2,476),-- Recipe: Sealed Phoenix Earring(70%)
(21446,8729,1,1,200,1309),-- Life Stone: level 64
(21446,8739,1,1,200,238),-- Mid-Grade Life Stone: level 64
(21447,4052,1,1,-1,92029),-- Earring of Black Ore Piece
(21447,4118,1,1,-1,10762),-- Evil Spirit Head
(21447,5281,1,1,-1,3466),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21447,57,1260,2556,0,700000),-- Adena
(21447,12183,1,1,1,736),-- Common Item - Doom Shield
(21447,12202,1,1,1,1350),-- Common Item - Blue Wolf Helmet
(21447,12185,1,1,1,245),-- Common Item - Doom Helmet
(21447,4077,1,1,1,5757),-- Doom Shield Fragment
(21447,4088,1,1,1,4837),-- Blue Wolf Helmet Design
(21447,4089,1,1,1,4837),-- Doom Helmet Pattern
(21447,1867,1,1,2,125000),-- Animal Skin
(21447,1869,1,1,2,100000),-- Iron Ore
(21447,1870,1,1,2,90909),-- Coal
(21447,1871,1,1,2,90909),-- Charcoal
(21447,1872,1,3,2,55556),-- Animal Bone
(21447,5462,1,1,2,48),-- Recipe: Branch of The Mother Tree(60%)
(21447,8729,1,1,200,1227),-- Life Stone: level 64
(21447,8739,1,1,200,245),-- Mid-Grade Life Stone: level 64
(21447,8749,1,1,200,123),-- High-Grade Life Stone: level 64
(21448,4118,1,1,-1,13565),-- Evil Spirit Head
(21448,5281,1,1,-1,4369),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21448,57,1060,2147,0,700000),-- Adena
(21448,12183,1,1,1,1092),-- Common Item - Doom Shield
(21448,12202,1,1,1,364),-- Common Item - Blue Wolf Helmet
(21448,12185,1,1,1,364),-- Common Item - Doom Helmet
(21448,4077,1,1,1,4838),-- Doom Shield Fragment
(21448,4088,1,1,1,4064),-- Blue Wolf Helmet Design
(21448,4089,1,1,1,4064),-- Doom Helmet Pattern
(21448,1867,1,1,2,111111),-- Animal Skin
(21448,1869,1,1,2,83333),-- Iron Ore
(21448,1870,1,1,2,83333),-- Coal
(21448,1871,1,1,2,71429),-- Charcoal
(21448,1872,1,1,2,125000),-- Animal Bone
(21448,5462,1,1,2,41),-- Recipe: Branch of The Mother Tree(60%)
(21448,8729,1,1,200,1092),-- Life Stone: level 64
(21449,4040,1,1,-1,57377),-- Mold Lubricant
(21449,4041,1,1,-1,24947),-- Mold Hardener
(21449,5496,1,1,-1,67265),-- Sealed Dark Crystal Boots Lining
(21449,57,1389,2824,0,700000),-- Adena
(21449,11950,1,1,1,293),-- Common Item - Blood Tornado
(21449,5539,1,1,1,1430),-- Blood Tornado Edge
(21449,1866,1,1,2,66667),-- Suede
(21449,1871,1,1,2,100000),-- Charcoal
(21449,1877,1,1,2,4149),-- Adamantite Nugget
(21449,4040,1,1,2,2004),-- Mold Lubricant
(21449,4041,1,1,2,733),-- Mold Hardener
(21449,5448,1,1,2,98),-- Recipe: Soul Separator(60%)
(21449,8730,1,1,200,1515),-- Life Stone: level 67
(21449,8740,1,1,200,98),-- Mid-Grade Life Stone: level 67
(21449,8760,1,1,200,49),-- Top-Grade Life Stone: level 67
(21450,5450,1,1,-1,1028),-- Recipe: Dragon Grinder(60%)
(21450,5456,1,1,-1,1028),-- Recipe: Tallum Glaive(60%)
(21450,5497,1,2,-1,65057),-- Sealed Tallum Boots Lining
(21450,57,1343,2731,0,700000),-- Adena
(21450,11955,1,1,1,222),-- Common Item - Carnage Bow
(21450,5535,1,1,1,1399),-- Carnage Bow Stave
(21450,1343,10,30,2,125000),-- Silver Arrow
(21450,1867,1,1,2,111111),-- Animal Skin
(21450,1872,1,1,2,111111),-- Animal Bone
(21450,1875,1,1,2,6098),-- Stone of Purity
(21450,1881,1,1,2,12346),-- Coarse Bone Powder
(21450,4041,1,1,2,1054),-- Mold Hardener
(21450,5450,1,1,2,44),-- Recipe: Dragon Grinder(60%)
(21450,8730,1,1,200,832),-- Life Stone: level 67
(21450,8740,1,1,200,333),-- Mid-Grade Life Stone: level 67
(21451,1807,1,1,-1,1243),-- Recipe: Soulshot: A-Grade
(21451,4041,1,1,-1,8107),-- Mold Hardener
(21451,4080,1,1,-1,19178),-- Blue Wolf Gaiters Material
(21451,57,452,917,0,700000),-- Adena
(21451,11935,1,1,1,8),-- Common Item - Deadman\'s Glory
(21451,11938,1,1,1,8),-- Common Item - Art of Battle Axe
(21451,11934,1,1,1,8),-- Common Item - Staff of Evil Spirits
(21451,4116,1,1,1,201),-- Deadman\'s Glory Stone
(21451,4117,1,1,1,201),-- Art of Battle Axe Blade
(21451,4118,1,1,1,190),-- Evil Spirit Head
(21451,730,1,1,2,47),-- Scroll: Enchant Armor (Grade A)
(21451,1343,10,30,2,45455),-- Silver Arrow
(21451,1866,1,1,2,20000),-- Suede
(21451,1871,1,1,2,29412),-- Charcoal
(21451,1885,1,1,2,2326),-- High Grade Suede
(21451,4041,1,1,2,332),-- Mold Hardener
(21451,5220,1,1,2,1151),-- Metal Hardener
(21451,8729,1,1,200,451),-- Life Stone: level 64
(21451,8739,1,1,200,95),-- Mid-Grade Life Stone: level 64
(21451,8749,1,1,200,24),-- High-Grade Life Stone: level 64
(21452,1875,1,1,-1,164372),-- Stone of Purity
(21452,3035,1,1,-1,3287),-- Recipe: Spiritshot A
(21452,4081,1,1,-1,24301),-- Doom Plate Armor Temper
(21452,57,986,1982,0,700000); -- Adena

INSERT INTO `droplist` VALUES 
(21453,3956,1,1,-1,2763),-- Recipe: Blessed Spiritshot A
(21453,4082,1,1,-1,23411),-- Blue Wolf Leather Armor Texture
(21453,4083,1,1,-1,23411),-- Leather Armor of Doom Design
(21453,57,825,1667,0,700000),-- Adena
(21454,1884,1,3,-1,758639),-- Cord
(21454,1889,1,1,-1,164372),-- Compound Braid
(21454,4084,1,1,-1,42850),-- Blue Wolf Tunic Fabric
(21454,57,984,1984,0,700000),-- Adena
(21455,4085,1,1,-1,36008),-- Tunic of Doom Pattern
(21455,5156,1,1,-1,2763),-- Recipe: Soulshot (A) Compressed Package (100%)
(21455,5161,1,1,-1,2763),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21455,57,828,1666,0,700000),-- Adena
(21456,1884,1,3,-1,758639),-- Cord
(21456,4039,1,1,-1,82186),-- Mold Glue
(21456,4086,1,1,-1,57857),-- Blue Wolf Stockings Pattern
(21456,11935,1,1,1,2),-- Common Item - Deadman\'s Glory (unknown chances)
(21456,11938,1,1,1,2),-- Common Item - Art of Battle Axe
(21456,11934,1,1,1,2),-- Common Item - Staff of Evil Spirits
(21456,4116,1,1,1,531),-- Deadman\'s Glory Stone
(21456,4117,1,1,1,531),-- Art of Battle Axe Blade
(21456,4118,1,1,1,503),-- Evil Spirit Head
(21456,1864,1,1,2,242765),-- Stem
(21456,1869,1,1,2,121383),-- Iron Ore
(21456,1878,1,1,2,48554),-- Braided Hemp
(21456,1884,1,1,2,74697),-- Cord
(21456,4039,1,1,2,4046),-- Mold Glue
(21456,5450,1,1,2,46),-- Recipe: Dragon Grinder(60%)
(21457,4087,1,1,-1,48619),-- Stockings of Doom Pattern
(21457,4088,1,1,-1,78304),-- Blue Wolf Helmet Design
(21457,5166,1,1,-1,2763),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21457,12341,1,1,1,23),-- Common Item - Earring of Black Ore (unknown chances)
(21457,12343,1,1,1,34),-- Common Item - Ring of Black Ore
(21457,12342,1,1,1,17),-- Common Item - Necklace of Black Ore
(21457,4052,1,1,1,3611),-- Earring of Black Ore Piece
(21457,4053,1,1,1,4372),-- Ring of Black Ore Gemstone
(21457,4054,1,1,1,2009),-- Necklace of Black Ore Beads
(21457,1343,10,30,2,124901),-- Silver Arrow
(21457,1866,1,1,2,58287),-- Suede
(21457,1871,1,1,2,87431),-- Charcoal
(21457,1885,1,1,2,7286),-- High Grade Suede
(21457,4041,1,1,2,761),-- Mold Hardener
(21457,5220,1,1,2,3498),-- Metal Hardener
(21457,5456,1,1,2,33),-- Recipe: Tallum Glaive(60%)
(21458,4041,1,1,-1,21440),-- Mold Hardener
(21458,4089,1,1,-1,93181),-- Doom Helmet Pattern
(21458,5230,1,1,-1,3287),-- Recipe: Mithril Arrow (100%)
(21458,12188,1,1,1,23),-- Common Item - Boots of Doom (unknown chances)
(21458,12189,1,1,1,23),-- Common Item - Blue Wolf Boots
(21458,12187,1,1,1,23),-- Common Item - Doom Gloves
(21458,12190,1,1,1,23),-- Common Item - Blue Wolf Gloves
(21458,4090,1,1,1,4086),-- Blue Wolf Boots Design
(21458,4091,1,1,1,4086),-- Doom Gloves Part
(21458,4096,1,1,1,4086),-- Blue Wolf Gloves Fabric
(21458,4099,1,1,1,4086),-- Doom Boots Part
(21458,1866,1,1,2,80922),-- Suede
(21458,1871,1,1,2,121383),-- Charcoal
(21458,1877,1,1,2,4856),-- Adamantite Nugget
(21458,4040,1,1,2,2428),-- Mold Lubricant
(21458,4041,1,1,2,1056),-- Mold Hardener
(21458,5462,1,1,2,45),-- Recipe: Branch of The Mother Tree(60%)
(21459,1894,1,2,-1,72699),-- Crafted Leather
(21459,4090,1,1,-1,93015),-- Blue Wolf Boots Design
(21459,5271,1,1,-1,2763),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(21459,11933,1,1,1,5),-- Common Item - Bellion Cestus (unknown chances)
(21459,4120,1,1,1,1338),-- Bellion Cestus Edge
(21459,1343,10,30,2,124900),-- Silver Arrow
(21459,1867,1,1,2,116574),-- Animal Skin
(21459,1872,1,1,2,116574),-- Animal Bone
(21459,1882,1,1,2,19430),-- Leather
(21459,1894,1,1,2,3068),-- Crafted Leather
(21459,4042,1,1,2,1458),-- Enria
(21459,5464,1,1,2,33),-- Recipe: Dark Legion\'s Edge(60%)
(21460,4091,1,2,-1,231933),-- Doom Gloves Part
(21460,4096,1,1,-1,231933),-- Blue Wolf Gloves Fabric
(21460,5276,1,1,-1,6888),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21460,57,2056,4157,0,700000),-- Adena
(21461,1876,1,3,-1,537011),-- Mithril Ore
(21461,4042,1,1,-1,89502),-- Enria
(21461,4099,1,1,-1,241083),-- Doom Boots Part
(21461,57,2137,4321,0,700000),-- Adena
(21462,4114,1,1,-1,22614),-- Sword of Damascus Blade
(21462,5281,1,1,-1,6888),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21462,5438,1,1,-1,2823),-- Recipe: Meteor Shower(60%)
(21462,57,2059,4152,0,700000),-- Adena
(21463,1875,1,2,-1,331238),-- Stone of Purity
(21463,4041,1,1,-1,43205),-- Mold Hardener
(21463,4115,1,1,-1,21675),-- Lance Blade
(21463,57,1976,3999,0,700000),-- Adena
(21464,4116,1,1,-1,22576),-- Deadman\'s Glory Stone
(21464,4117,1,1,-1,22576),-- Art of Battle Axe Blade
(21464,5444,1,1,-1,2823),-- Recipe: Carnage Bow(60%)
(21464,11935,1,1,1,4),-- Common Item - Deadman\'s Glory (unknown chances)
(21464,11938,1,1,1,4),-- Common Item - Art of Battle Axe
(21464,11934,1,1,1,4),-- Common Item - Staff of Evil Spirits
(21464,4116,1,1,1,1112),-- Deadman\'s Glory Stone
(21464,4117,1,1,1,1112),-- Art of Battle Axe Blade
(21464,4118,1,1,1,1054),-- Evil Spirit Head
(21464,729,1,1,2,29),-- Scroll: Enchant Weapon (Grade A)
(21464,1867,1,3,2,169561),-- Animal Skin
(21464,1872,1,3,2,113041),-- Animal Bone
(21464,1882,1,1,2,56521),-- Leather
(21464,1894,1,1,2,8925),-- Crafted Leather
(21464,4042,1,1,2,4240),-- Enria
(21465,4118,1,1,-1,22233),-- Evil Spirit Head
(21465,5220,1,4,-1,214805),-- Metal Hardener
(21465,5446,1,1,-1,2934),-- Recipe: Bloody Orchid(60%)
(21465,5370,1,1,2,3534),-- Recipe: Sealed Tallum Boots(60%)
(21465,12341,1,1,1,58),-- Common Item - Earring of Black Ore (unknown chances)
(21465,12343,1,1,1,86),-- Common Item - Ring of Black Ore
(21465,12342,1,1,1,43),-- Common Item - Necklace of Black Ore
(21465,4052,1,1,1,9359),-- Earring of Black Ore Piece
(21465,4053,1,1,1,11331),-- Ring of Black Ore Gemstone
(21465,4054,1,1,1,5207),-- Necklace of Black Ore Beads
(21465,730,1,1,2,189),-- Scroll: Enchant Armor (Grade A)
(21465,1343,20,60,2,161863),-- Silver Arrow
(21465,1874,1,1,2,15108),-- Oriharukon Ore
(21465,1878,1,1,2,90643),-- Braided Hemp
(21465,1884,1,1,2,139451),-- Cord
(21465,1889,1,1,2,15108),-- Compound Braid
(21465,5220,1,1,2,9065),-- Metal Hardener
(21466,4042,1,2,-1,86105),-- Enria
(21466,4119,1,1,-1,22614),-- Demon Dagger Edge
(21466,5452,1,1,-1,2823),-- Recipe: Blood Tornado(60%)
(21466,5368,1,1,2,1244),-- Recipe: Sealed Dark Crystal Boots(60%)
(21466,12188,1,1,1,48),-- Common Item - Boots of Doom (unknown chances)
(21466,12189,1,1,1,48),-- Common Item - Blue Wolf Boots
(21466,12187,1,1,1,48),-- Common Item - Doom Gloves
(21466,12190,1,1,1,48),-- Common Item - Blue Wolf Gloves
(21466,4090,1,1,1,8562),-- Blue Wolf Boots Design
(21466,4091,1,1,1,8562),-- Doom Gloves Part
(21466,4096,1,1,1,8562),-- Blue Wolf Gloves Fabric
(21466,4099,1,1,1,8562),-- Doom Boots Part
(21466,1867,1,3,2,169561),-- Animal Skin
(21466,1872,1,3,2,113041),-- Animal Bone
(21466,1882,1,1,2,56521),-- Leather
(21466,1894,1,1,2,8925),-- Crafted Leather
(21466,4042,1,1,2,4240),-- Enria
(21466,5434,1,1,2,95),-- Recipe: Dragon Slayer(60%)
(21467,4120,1,1,-1,21712),-- Bellion Cestus Edge
(21467,4121,1,1,-1,21639),-- Bow of Peril Shaft
(21467,5458,1,1,-1,2715),-- Recipe: Halberd(60%)
(21467,11933,1,1,1,12),-- Common Item - Bellion Cestus (unknown chances)
(21467,4120,1,1,1,3207),-- Bellion Cestus Edge
(21467,1343,30,90,2,99840),-- Silver Arrow
(21467,1864,4,12,2,52416),-- Stem
(21467,1869,2,6,2,41933),-- Iron Ore
(21467,1878,1,1,2,83866),-- Braided Hemp
(21467,1884,1,1,2,129024),-- Cord
(21467,4039,1,1,2,6989),-- Mold Glue
(21467,5440,1,1,2,78),-- Recipe: Elysian(60%)
(21467,5368,1,1,2,3049),-- Recipe: Sealed Dark Crystal Boots(60%)
(21468,4052,1,1,-1,308357),-- Earring of Black Ore Piece
(21468,5220,1,1,-1,348382),-- Metal Hardener
(21468,5370,1,1,-1,40000),-- Recipe: Sealed Tallum Boots(60%)
(21468,57,3500,7005,0,700000),-- Adena
(21469,4053,1,1,-1,356108),-- Ring of Black Ore Gemstone
(21469,5460,1,1,-1,4540),-- Recipe: Dasparion\'s Staff(60%)
(21469,5468,1,1,-1,4540),-- Recipe: Elemental Sword(60%)
(21469,57,3309,6685,0,700000),-- Adena
(21470,4114,1,1,-1,4358),-- Sword of Damascus Blade
(21470,5156,1,1,-1,1327),-- Recipe: Soulshot (A) Compressed Package (100%)
(21470,5161,1,1,-1,1327),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21470,57,480,982,0,700000),-- Adena
(21470,12188,1,1,1,180),-- Common Item - Boots of Doom
(21470,12189,1,1,1,283),-- Common Item - Blue Wolf Boots
(21470,12187,1,1,1,257),-- Common Item - Doom Gloves
(21470,12190,1,1,1,412),-- Common Item - Blue Wolf Gloves
(21470,4090,1,1,1,1650),-- Blue Wolf Boots Design
(21470,4091,1,1,1,1650),-- Doom Gloves Part
(21470,4096,1,1,1,1650),-- Blue Wolf Gloves Fabric
(21470,4099,1,1,1,1650),-- Doom Boots Part
(21470,1866,1,1,2,21739),-- Suede
(21470,1871,1,1,2,35714),-- Charcoal
(21470,1885,1,1,2,2625),-- High Grade Suede
(21470,4041,1,1,2,231),-- Mold Hardener
(21470,5220,1,1,2,1106),-- Metal Hardener
(21470,6337,1,1,2,103),-- Recipe: Sealed Majestic Earring(70%)
(21470,5364,1,1,2,257),-- Recipe: Sealed Dark Crystal Shield(60%)
(21470,8730,1,1,200,309),-- Life Stone: level 67
(21470,8740,1,1,200,77),-- Mid-Grade Life Stone: level 67
(21470,8750,1,1,200,26),-- High-Grade Life Stone: level 67
(21471,1894,1,1,-1,92378),-- Crafted Leather
(21471,4042,1,1,-1,43879),-- Enria
(21471,4115,1,1,-1,11485),-- Lance Blade
(21471,57,1052,2114,0,700000),-- Adena
(21472,4116,1,1,-1,9668),-- Deadman\'s Glory Stone
(21472,4117,1,1,-1,9668),-- Art of Battle Axe Blade
(21472,5166,1,1,-1,2950),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21472,57,881,1760,0,700000),-- Adena
(21473,4043,1,1,-1,87759),-- Asofe
(21473,4118,1,1,-1,10900),-- Evil Spirit Head
(21473,5230,1,1,-1,3510),-- Recipe: Mithril Arrow (100%)
(21473,57,1047,2118,0,700000),-- Adena
(21474,4039,1,1,-1,73747),-- Mold Glue
(21474,4119,1,1,-1,9684),-- Demon Dagger Edge
(21474,5271,1,1,-1,2950),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(21474,57,881,1779,0,700000),-- Adena
(21475,4120,1,1,-1,11505),-- Bellion Cestus Edge
(21475,4121,1,1,-1,11466),-- Bow of Peril Shaft
(21475,5276,1,1,-1,3510),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21475,12201,1,1,1,9),-- Common Item - Blue Wolf Tunic (unknown chances)
(21475,12192,1,1,1,9),-- Common Item - Tunic of Doom
(21475,12203,1,1,1,13),-- Common Item - Blue Wolf Stockings
(21475,12205,1,1,1,13),-- Common Item - Stockings of Doom
(21475,4084,1,1,1,1689),-- Blue Wolf Tunic Fabric
(21475,4085,1,1,1,1689),-- Tunic of Doom Pattern
(21475,4086,1,1,1,2280),-- Blue Wolf Stockings Pattern
(21475,4087,1,1,1,2280),-- Stockings of Doom Pattern
(21475,1343,10,30,2,158710),-- Silver Arrow
(21475,1866,1,1,2,74065),-- Suede
(21475,1871,1,1,2,111097),-- Charcoal
(21475,1877,1,1,2,4444),-- Adamantite Nugget
(21475,4040,1,1,2,2223),-- Mold Lubricant
(21475,4041,1,1,2,967),-- Mold Hardener
(21475,5440,1,1,2,42),-- Recipe: Elysian(60%)
(21476,1875,1,1,-1,147494),-- Stone of Purity
(21476,4041,1,1,-1,19238),-- Mold Hardener
(21476,4052,1,1,-1,78329),-- Earring of Black Ore Piece
(21476,12204,1,1,1,7),-- Common Item - Blue Wolf Breastplate (unknown chances)
(21476,12193,1,1,1,11),-- Common Item - Blue Wolf Gaiters
(21476,12184,1,1,1,5),-- Common Item - Doom Plate Armor
(21476,4078,1,1,1,1591),-- Blue Wolf Breastplate Part
(21476,4080,1,1,1,2241),-- Blue Wolf Gaiters Material
(21476,4081,1,1,1,1074),-- Doom Plate Armor Temper
(21476,1867,1,1,2,145225),-- Animal Skin
(21476,1872,1,1,2,145225),-- Animal Bone
(21476,1875,1,1,2,7262),-- Stone of Purity
(21476,1881,1,1,2,14523),-- Coarse Bone Powder
(21476,4041,1,1,2,948),-- Mold Hardener
(21476,5442,1,1,2,41),-- Recipe: Soul Bow(60%)
(21477,4053,1,1,-1,112849),-- Ring of Black Ore Gemstone
(21477,5281,1,1,-1,3510),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21477,5438,1,1,-1,1439),-- Recipe: Meteor Shower(60%)
(21477,12201,1,1,1,9),-- Common Item - Blue Wolf Tunic (unknown chances)
(21477,12192,1,1,1,9),-- Common Item - Tunic of Doom
(21477,12203,1,1,1,13),-- Common Item - Blue Wolf Stockings
(21477,12205,1,1,1,13),-- Common Item - Stockings of Doom
(21477,4084,1,1,1,1689),-- Blue Wolf Tunic Fabric
(21477,4085,1,1,1,1689),-- Tunic of Doom Pattern
(21477,4086,1,1,1,2280),-- Blue Wolf Stockings Pattern
(21477,4087,1,1,1,2280),-- Stockings of Doom Pattern
(21477,1343,10,30,2,158710),-- Silver Arrow
(21477,1865,1,1,2,111097),-- Varnish
(21477,1870,1,1,2,111097),-- Coal
(21477,1875,1,1,2,7407),-- Stone of Purity
(21477,1879,1,1,2,18517),-- Cokes
(21477,4040,1,1,2,2222),-- Mold Lubricant
(21477,5448,1,1,2,42),-- Recipe: Soul Separator(60%)
(21478,1885,1,1,-1,184367),-- High Grade Suede
(21478,4041,1,1,-1,19238),-- Mold Hardener
(21478,4054,1,1,-1,43577),-- Necklace of Black Ore Beads
(21478,12341,1,1,1,24),-- Common Item - Earring of Black Ore (unknown chances)
(21478,12343,1,1,1,36),-- Common Item - Ring of Black Ore
(21478,12342,1,1,1,18),-- Common Item - Necklace of Black Ore
(21478,4052,1,1,1,3856),-- Earring of Black Ore Piece
(21478,4053,1,1,1,4668),-- Ring of Black Ore Gemstone
(21478,4054,1,1,1,2145),-- Necklace of Black Ore Beads
(21478,1866,1,1,2,72613),-- Suede
(21478,1871,1,1,2,108919),-- Charcoal
(21478,1885,1,1,2,9077),-- High Grade Suede
(21478,4041,1,1,2,948),-- Mold Hardener
(21478,5220,1,1,2,4357),-- Metal Hardener
(21478,5450,1,1,2,41),-- Recipe: Dragon Grinder(60%)
(21479,4077,1,1,-1,248161),-- Doom Shield Fragment
(21479,4078,1,1,-1,80582),-- Blue Wolf Breastplate Part
(21479,5444,1,1,-1,3015),-- Recipe: Carnage Bow(60%)
(21479,57,2203,4437,0,700000),-- Adena
(21480,1889,1,1,-1,382283),-- Compound Braid
(21480,4080,1,1,-1,117952),-- Blue Wolf Gaiters Material
(21480,5446,1,1,-1,3133),-- Recipe: Bloody Orchid(60%)
(21480,57,2297,4608,0,700000),-- Adena
(21481,4081,1,1,-1,54372),-- Doom Plate Armor Temper
(21481,5452,1,1,-1,3015),-- Recipe: Blood Tornado(60%)
(21481,5550,1,1,-1,73555),-- Durable Metal Plate
(21481,57,2195,4427,0,700000),-- Adena
(21482,4082,1,1,-1,59949),-- Blue Wolf Leather Armor Texture
(21482,4083,1,1,-1,59949),-- Leather Armor of Doom Design
(21482,5458,1,1,-1,2899),-- Recipe: Halberd(60%)
(21482,57,2122,4264,0,700000),-- Adena
(21483,1877,1,2,-1,220665),-- Adamantite Nugget
(21483,4040,1,2,-1,110332),-- Mold Lubricant
(21483,5364,1,1,-1,36232),-- Recipe: Sealed Dark Crystal Shield(60%)
(21483,11944,1,1,1,13),-- Common Item - Bow of Peril (unknown chances)
(21483,4121,1,1,1,3549),-- Bow of Peril Shaft
(21483,1343,20,60,2,166279),-- Silver Arrow
(21483,1866,1,1,2,155194),-- Suede
(21483,1871,1,3,2,77597),-- Charcoal
(21483,1877,1,1,2,9312),-- Adamantite Nugget
(21483,4040,1,1,2,4656),-- Mold Lubricant
(21483,4041,1,1,2,2025),-- Mold Hardener
(21483,6335,1,1,2,896),-- Recipe: Sealed Majestic Necklace(70%)
(21484,4085,1,1,-1,99657),-- Tunic of Doom Pattern
(21484,5460,1,1,-1,3133),-- Recipe: Dasparion\'s Staff(60%)
(21484,5468,1,1,-1,3133),-- Recipe: Elemental Sword(60%)
(21484,12201,1,1,1,18),-- Common Item - Blue Wolf Tunic (unknown chances)
(21484,12192,1,1,1,18),-- Common Item - Tunic of Doom
(21484,12203,1,1,1,29),-- Common Item - Blue Wolf Stockings
(21484,12205,1,1,1,29),-- Common Item - Stockings of Doom
(21484,4084,1,1,1,3678),-- Blue Wolf Tunic Fabric
(21484,4085,1,1,1,3678),-- Tunic of Doom Pattern
(21484,4086,1,1,1,4967),-- Blue Wolf Stockings Pattern
(21484,4087,1,1,1,4966),-- Stockings of Doom Pattern
(21484,1867,1,3,2,125468),-- Animal Skin
(21484,1872,1,3,2,188201),-- Animal Bone
(21484,1875,1,1,2,18821),-- Stone of Purity
(21484,1881,1,1,2,37641),-- Coarse Bone Powder
(21484,4041,1,1,2,2455),-- Mold Hardener
(21484,6337,1,1,2,1448),-- Recipe: Sealed Majestic Earring(70%)
(21485,4040,1,1,-1,110332),-- Mold Lubricant
(21485,4086,1,1,-1,129452),-- Blue Wolf Stockings Pattern
(21485,5368,1,1,-1,43478),-- Recipe: Sealed Dark Crystal Boots(60%)
(21485,12341,1,1,1,59),-- Common Item - Earring of Black Ore (unknown chances)
(21485,12343,1,1,1,89),-- Common Item - Ring of Black Ore
(21485,12342,1,1,1,45),-- Common Item - Necklace of Black Ore
(21485,4052,1,1,1,9615),-- Earring of Black Ore Piece
(21485,4053,1,1,1,11640),-- Ring of Black Ore Gemstone
(21485,4054,1,1,1,5349),-- Necklace of Black Ore Beads
(21485,1343,20,60,2,166278),-- Silver Arrow
(21485,1865,1,3,2,77597),-- Varnish
(21485,1870,2,6,2,58198),-- Coal
(21485,1875,1,1,2,15520),-- Stone of Purity
(21485,1879,1,1,2,38799),-- Cokes
(21485,4040,1,1,2,4656),-- Mold Lubricant
(21485,6339,1,1,2,1791),-- Recipe: Sealed Majestic Ring(70%)
(21486,4087,1,1,-1,124498),-- Stockings of Doom Pattern
(21486,4088,1,1,-1,200510),-- Blue Wolf Helmet Design
(21486,5470,1,1,-1,2899),-- Recipe: Tallum Blade(60%)
(21486,12204,1,1,1,17),-- Common Item - Blue Wolf Breastplate (unknown chances)
(21486,12193,1,1,1,26),-- Common Item - Blue Wolf Gaiters
(21486,12184,1,1,1,12),-- Common Item - Doom Plate Armor
(21486,4078,1,1,1,3816),-- Blue Wolf Breastplate Part
(21486,4080,1,1,1,5373),-- Blue Wolf Gaiters Material
(21486,4081,1,1,1,2575),-- Doom Plate Armor Temper
(21486,729,1,1,2,30),-- Scroll: Enchant Weapon (Grade A)
(21486,1866,1,1,2,174129),-- Suede
(21486,1871,1,1,2,261194),-- Charcoal
(21486,1885,1,1,2,21767),-- High Grade Suede
(21486,4041,1,1,2,2272),-- Mold Hardener
(21486,5220,1,1,2,10448),-- Metal Hardener
(21487,1894,1,1,-1,326320),-- Crafted Leather
(21487,4089,1,1,-1,351478),-- Doom Helmet Pattern
(21487,5428,1,1,-1,5464),-- Recipe: Sealed Tallum Helmet(60%)
(21487,57,3703,7484,0,700000),-- Adena
(21488,4044,1,1,-1,295712),-- Thons
(21488,4090,1,1,-1,398265),-- Blue Wolf Boots Design
(21488,5426,1,1,-1,6645),-- Recipe: Sealed Dark Crystal Helmet(60%)
(21488,57,3529,7137,0,700000),-- Adena
(21489,1877,1,1,-1,42470),-- Adamantite Nugget
(21489,4040,1,1,-1,21235),-- Mold Lubricant
(21489,6343,1,1,-1,22702),-- Sealed Phoenix Necklace Beads
(21489,57,514,1045,0,700000),-- Adena
(21489,11950,1,1,1,37),-- Common Item - Blood Tornado
(21489,5539,1,1,1,530),-- Blood Tornado Edge
(21489,1866,1,1,2,25641),-- Suede
(21489,1871,1,1,2,41667),-- Charcoal
(21489,1877,1,1,2,1499),-- Adamantite Nugget
(21489,4040,1,1,2,900),-- Mold Lubricant
(21489,4041,1,1,2,487),-- Mold Hardener
(21489,5456,1,1,2,20),-- Recipe: Tallum Glaive(60%)
(21489,8730,1,1,200,300),-- Life Stone: level 67
(21489,8740,1,1,200,75),-- Mid-Grade Life Stone: level 67
(21490,5450,1,1,-1,1040),-- Recipe: Dragon Grinder(60%)
(21490,5456,1,1,-1,1040),-- Recipe: Tallum Glaive(60%)
(21490,6345,1,1,-1,107412),-- Sealed Phoenix Ring Gemstone
(21490,57,1117,2256,0,700000),-- Adena
(21491,730,1,1,-1,1966),-- Scroll: Enchant Armor (Grade A)
(21491,4041,1,1,-1,20517),-- Mold Hardener
(21491,5478,1,1,-1,19841),-- Sealed Dark Crystal Leather Armor Pattern
(21491,57,946,1894,0,700000),-- Adena
(21492,4040,1,1,-1,56155),-- Mold Lubricant
(21492,5462,1,1,-1,1040),-- Recipe: Branch of The Mother Tree(60%)
(21492,5479,1,1,-1,16614),-- Sealed Tallum Leather Armor Pattern
(21492,57,1123,2259,0,700000),-- Adena
(21493,5464,1,1,-1,874),-- Recipe: Dark Legion\'s Edge(60%)
(21493,5482,1,1,-1,31142),-- Sealed Dark Crystalline Leggings Design
(21493,5485,1,1,-1,19841),-- Sealed Tallum Tunic Texture
(21493,57,946,1859,0,700000),-- Adena
(21494,729,1,1,-1,312),-- Scroll: Enchant Weapon (Grade A)
(21494,1876,1,1,-1,561552),-- Mithril Ore
(21494,4042,1,1,-1,46796),-- Enria
(21494,11955,1,1,1,5),-- Common Item - Carnage Bow (unknown chances)
(21494,5535,1,1,1,1415),-- Carnage Bow Stave
(21494,1343,10,30,2,169260),-- Silver Arrow
(21494,1868,1,3,2,118482),-- Thread
(21494,1873,1,1,2,47393),-- Silver Nugget
(21494,1876,1,1,2,23697),-- Mithril Ore
(21494,1895,1,1,2,33852),-- Metallic Fiber
(21494,4042,1,1,2,1975),-- Enria
(21494,6337,1,1,2,608),-- Recipe: Sealed Majestic Earring(70%)
(21495,5466,1,1,-1,874),-- Recipe: Sword of Miracles(60%)
(21495,5486,1,1,-1,13961),-- Sealed Dark Crystal Robe Fabric
(21495,5489,1,1,-1,31142),-- Sealed Tallum Stockings Fabric
(21495,11951,1,1,1,4),-- Common Item - Bloody Orchid (unknown chances)
(21495,5536,1,1,1,1186),-- Bloody Orchid Head
(21495,1868,1,1,2,232316),-- Thread
(21495,1873,1,1,2,46464),-- Silver Nugget
(21495,1876,1,1,2,23232),-- Mithril Ore
(21495,1895,1,1,2,33188),-- Metallic Fiber
(21495,4042,1,1,2,1936),-- Enria
(21495,6339,1,1,2,894),-- Recipe: Sealed Majestic Ring(70%)
(21496,730,1,1,-1,2340),-- Scroll: Enchant Armor (Grade A)
(21496,5494,1,1,-1,58968),-- Sealed Dark Crystal Shield Fragment
(21496,6335,1,1,-1,10799),-- Recipe: Sealed Majestic Necklace(70%)
(21496,11946,1,1,1,3),-- Common Item - Dasparion\'s Staff
(21496,11949,1,1,1,3),-- Common Item - Meteor Shower
(21496,11958,1,1,1,3),-- Common Item - Destroyer Hammer
(21496,11953,1,1,1,3),-- Common Item - Spiritual Eye
(21496,5532,1,1,1,699),-- Meteor Shower Head
(21496,5543,1,1,1,706),-- Dasparion\'s Staff Edge
(21496,729,1,1,2,14),-- Scroll: Enchant Weapon (Grade A)
(21496,1343,10,30,2,169259),-- Silver Arrow
(21496,1868,1,3,2,118481),-- Thread
(21496,1873,1,1,2,47393),-- Silver Nugget
(21496,1876,1,1,2,23697),-- Mithril Ore
(21496,1895,1,1,2,33852),-- Metallic Fiber
(21496,4042,1,1,2,1975),-- Enria
(21497,1884,1,3,-1,725988),-- Cord
(21497,4039,1,1,-1,78649),-- Mold Glue
(21497,5496,1,1,-1,55321),-- Sealed Dark Crystal Boots Lining
(21497,11957,1,1,1,2),-- Common Item - Tallum Blade
(21497,11952,1,1,1,2),-- Common Item - Elemental Sword
(21497,11954,1,1,1,2),-- Common Item - Infernal Master
(21497,5547,1,1,1,593),-- Elemental Sword Edge
(21497,5548,1,1,1,593),-- Tallum Blade Edge
(21497,730,1,1,2,97),-- Scroll: Enchant Armor (Grade A)
(21497,1864,1,1,2,232317),-- Stem
(21497,1869,1,1,2,116159),-- Iron Ore
(21497,1878,1,1,2,46464),-- Braided Hemp
(21497,1884,1,1,2,71483),-- Cord
(21497,4039,1,1,2,3872),-- Mold Glue
(21497,5366,1,1,2,342),-- Recipe: Sealed Shield of Nightmare(60%)
(21498,5497,1,1,-1,137944),-- Sealed Tallum Boots Lining
(21498,6337,1,1,-1,30171),-- Recipe: Sealed Majestic Earring(70%)
(21498,6339,1,1,-1,45256),-- Recipe: Sealed Majestic Ring(70%)
(21498,57,2355,4724,0,700000),-- Adena
(21499,729,1,1,-1,679),-- Scroll: Enchant Weapon (Grade A)
(21499,1895,1,3,-1,873628),-- Metallic Fiber
(21499,5508,1,1,-1,143386),-- Sealed Dark Crystal Gloves Design
(21499,57,2436,4920,0,700000),-- Adena
(21500,4042,1,1,-1,98055),-- Enria
(21500,5434,1,1,-1,2179),-- Recipe: Dragon Slayer(60%)
(21500,5509,1,1,-1,137944),-- Sealed Tallum Gloves Design
(21500,57,2340,4734,0,700000),-- Adena
(21501,5440,1,1,-1,2096),-- Recipe: Elysian(60%)
(21501,5520,1,1,-1,36486),-- Sealed Dark Crystal Breastplate Pattern
(21501,5521,1,1,-1,25746),-- Sealed Tallum Plate Armor Pattern
(21501,57,2250,4554,0,700000),-- Adena
(21502,730,1,1,-1,4903),-- Scroll: Enchant Armor (Grade A)
(21502,1885,1,3,-1,490275),-- High Grade Suede
(21502,5432,1,1,-1,28373),-- Recipe: Sealed Majestic Circlet(60%)
(21502,1343,560,1680,2,79166),-- Silver Arrow
(21502,1866,20,60,2,51722),-- Suede
(21502,1871,9,27,2,172405),-- Charcoal
(21502,1885,4,12,2,28734),-- High Grade Suede
(21502,4041,1,1,2,26986),-- Mold Hardener
(21502,5220,1,1,2,124132),-- Metal Hardener
(21502,5450,1,1,2,1150),-- Recipe: Dragon Grinder(60%)
(21503,5442,1,1,-1,2265),-- Recipe: Soul Bow(60%)
(21503,5524,1,1,-1,60274),-- Sealed Dark Crystal Gaiters Pattern
(21503,5525,1,1,-1,94534),-- Sealed Dark Crystal Helmet Design
(21503,11960,1,1,1,10),-- Common Item - Halberd (unknown chances)
(21503,5542,1,1,1,3047),-- Halberd Edge
(21503,1867,1,3,2,200711),-- Animal Skin
(21503,1872,1,3,2,133807),-- Animal Bone
(21503,1882,1,1,2,66904),-- Leather
(21503,1894,1,1,2,10564),-- Crafted Leather
(21503,4042,1,1,2,5018),-- Enria
(21503,5456,1,1,2,112),-- Recipe: Tallum Glaive(60%)
(21504,729,1,1,-1,654),-- Scroll: Enchant Weapon (Grade A)
(21504,5448,1,1,-1,2179),-- Recipe: Soul Separator(60%)
(21504,5526,1,1,-1,90946),-- Sealed Tallum Helm Design
(21504,11955,1,1,1,10),-- Common Item - Carnage Bow (unknown chances)
(21504,5535,1,1,1,2965),-- Carnage Bow Stave
(21504,1343,30,90,2,118221),-- Silver Arrow
(21504,1868,4,12,2,62066),-- Thread
(21504,1873,1,1,2,99306),-- Silver Nugget
(21504,1889,1,1,2,16551),-- Compound Braid
(21504,4044,1,1,2,8276),-- Thons
(21504,5462,1,1,2,92),-- Recipe: Branch of The Mother Tree(60%)
(21504,5432,1,1,2,541),-- Recipe: Sealed Majestic Circlet(60%)
(21504,5549,1,1,2,24827),-- Metallic Thread
(21505,4043,1,2,-1,188604),-- Asofe
(21505,5532,1,1,-1,19080),-- Meteor Shower Head
(21505,5366,1,1,-1,40323),-- Recipe: Sealed Shield of Nightmare(60%)
(21505,11951,1,1,1,10),-- Common Item - Bloody Orchid (unknown chances)
(21505,5536,1,1,1,2843),-- Bloody Orchid Head
(21505,1864,1,3,2,278555),-- Stem
(21505,1869,1,3,2,92852),-- Iron Ore
(21505,1880,1,1,2,27856),-- Steel
(21505,4043,1,1,2,9286),-- Asofe
(21505,5464,1,1,2,103),-- Recipe: Dark Legion\'s Edge(60%)
(21505,5430,1,1,2,68),-- Recipe: Sealed Helm of Nightmare(60%)
(21505,5550,1,1,2,3715),-- Durable Metal Plate
(21506,5450,1,1,-1,3673),-- Recipe: Dragon Grinder(60%)
(21506,5456,1,1,-1,3673),-- Recipe: Tallum Glaive(60%)
(21506,5535,1,1,-1,33842),-- Carnage Bow Stave
(21506,57,3990,7934,0,700000),-- Adena
(21507,4041,1,1,-1,82270),-- Mold Hardener
(21507,5536,1,1,-1,32189),-- Bloody Orchid Head
(21507,5430,1,1,-1,15152),-- Recipe: Sealed Helm of Nightmare(60%)
(21507,57,3792,7613,0,700000),-- Adena
(21508,4120,1,1,-1,11696),-- Bellion Cestus Edge
(21508,4121,1,1,-1,11657),-- Bow of Peril Shaft
(21508,5444,1,1,-1,1463),-- Recipe: Carnage Bow(60%)
(21508,57,1303,2629,0,700000),-- Adena
(21508,12341,1,1,1,993),-- Common Item - Earring of Black Ore
(21508,12343,1,1,1,1473),-- Common Item - Ring of Black Ore
(21508,12342,1,1,1,754),-- Common Item - Necklace of Black Ore
(21508,4052,1,1,1,4665),-- Earring of Black Ore Piece
(21508,4053,1,1,1,5648),-- Ring of Black Ore Gemstone
(21508,4054,1,1,1,2596),-- Necklace of Black Ore Beads
(21508,1875,1,1,2,7353),-- Stone of Purity
(21508,1879,1,1,2,16129),-- Cokes
(21508,1885,1,1,2,4425),-- High Grade Suede
(21508,5466,1,1,2,42),-- Recipe: Sword of Miracles(60%)
(21508,5549,1,1,2,13333),-- Metallic Thread
(21508,5550,1,1,2,1473),-- Durable Metal Plate
(21508,6670,1,1,2,22222),-- Deluxe Chest Key - Grade 6
(21508,8729,1,1,200,2950),-- Life Stone: level 64
(21509,4091,1,1,-1,104895),-- Doom Gloves Part
(21509,4096,1,2,-1,104895),-- Blue Wolf Gloves Fabric
(21509,6333,1,1,-1,26826),-- Recipe: Sealed Phoenix Ring(70%)
(21509,57,1128,2304,0,700000),-- Adena
(21509,12194,1,1,1,10),-- Common Item - Blue Wolf Leather Armor (no drop info)
(21509,12186,1,1,1,10),-- Common Item - Leather Armor of Doom
(21509,4082,1,1,1,1950),-- Blue Wolf Leather Armor Texture
(21509,4083,1,1,1,1950),-- Leather Armor of Doom Design
(21509,1343,10,30,2,111111),-- Silver Arrow
(21509,1864,1,1,2,142857),-- Stem
(21509,1869,1,1,2,66667),-- Iron Ore
(21509,1880,1,1,2,17241),-- Steel
(21509,4043,1,1,2,1441),-- Asofe
(21509,5440,1,1,2,37),-- Recipe: Elysian(60%)
(21509,5550,1,1,2,1441),-- Durable Metal Plate
(21510,730,1,1,-1,2381),-- Scroll: Enchant Armor (Grade A)
(21510,1875,1,1,-1,190480),-- Stone of Purity
(21510,5508,1,1,-1,66992),-- Sealed Dark Crystal Gloves Design
(21510,57,1383,2802,0,700000),-- Adena
(21510,11951,1,1,1,5),-- Common Item - Bloody Orchid (no drop info)
(21510,5536,1,1,1,1436),-- Bloody Orchid Head
(21510,1865,1,1,2,142857),-- Varnish
(21510,1870,1,1,2,100000),-- Coal
(21510,1875,1,1,2,5682),-- Stone of Purity
(21510,1879,1,1,2,5682),-- Cokes
(21510,4040,1,1,2,4255),-- Mold Lubricant
(21510,5456,1,1,2,53),-- Recipe: Tallum Glaive(60%)
(21510,8730,1,1,200,2841),-- Life Stone: level 67
(21510,8760,1,1,200,934),-- Top-Grade Life Stone: level 67
(21511,1875,1,1,-1,246682),-- Stone of Purity
(21511,5462,1,1,-1,1370),-- Recipe: Branch of The Mother Tree(60%)
(21511,5539,1,1,-1,12478),-- Blood Tornado Edge
(21511,57,1794,3639,0,700000),-- Adena
(21511,11946,1,1,1,116),-- Common Item - Dasparion\'s Staff
(21511,11949,1,1,1,116),-- Common Item - Meteor Shower
(21511,8346,1,1,1,671),-- Hammer Piece of Destroyer
(21511,11958,1,1,1,116),-- Common Item - Destroyer Hammer
(21511,8341,1,1,1,335),-- Spiritual Eye Piece
(21511,11953,1,1,1,116),-- Common Item - Spiritual Eye
(21511,5532,1,1,1,922),-- Meteor Shower Head
(21511,5543,1,1,1,930),-- Dasparion\'s Staff Edge
(21511,1343,20,60,2,90909),-- Silver Arrow
(21511,1867,1,1,2,166667),-- Animal Skin
(21511,1872,1,1,2,166667),-- Animal Bone
(21511,1875,1,1,2,7407),-- Stone of Purity
(21511,1881,1,1,2,23810),-- Coarse Bone Powder
(21511,4041,1,1,2,1358),-- Mold Hardener
(21511,6337,1,1,2,801),-- Recipe: Sealed Majestic Earring(70%)
(21511,8740,1,1,200,1041),-- Mid-Grade Life Stone: level 67
(21511,8750,1,1,200,1041),-- High-Grade Life Stone: level 67
(21512,730,1,1,-1,3601),-- Scroll: Enchant Armor (Grade A)
(21512,1875,1,1,-1,288061),-- Stone of Purity
(21512,4041,1,1,-1,37573),-- Mold Hardener
(21512,57,2125,3872,0,700000),-- Adena
(21512,11951,1,1,1,7),-- Common Item - Bloody Orchid (too low drop info, no material info)
(21512,5536,1,1,1,2172),-- Bloody Orchid Head
(21512,1343,20,60,2,130239),-- Silver Arrow
(21512,1867,1,1,2,243112),-- Animal Skin
(21512,1872,1,3,2,81037),-- Animal Bone
(21512,1875,1,1,2,12156),-- Stone of Purity
(21512,1881,1,1,2,24312),-- Coarse Bone Powder
(21512,4041,1,1,2,1586),-- Mold Hardener
(21512,5462,1,1,2,68),-- Recipe: Branch of The Mother Tree(60%)
(21512,8730,1,1,200,133),-- Life Stone: level 67
(21513,5442,1,1,-1,1128),-- Recipe: Soul Bow(60%)
(21513,5486,1,1,-1,18023),-- Sealed Dark Crystal Robe Fabric
(21513,5489,1,1,-1,40202),-- Sealed Tallum Stockings Fabric
(21513,57,1485,2997,0,700000),-- Adena
(21513,12221,1,1,1,20),-- Common Item - Sealed Dark Crystal Gloves (no drop info)
(21513,12224,1,1,1,20),-- Common Item - Sealed Dark Crystal Boots
(21513,12229,1,1,1,20),-- Common Item - Sealed Tallum Gloves
(21513,12231,1,1,1,20),-- Common Item - Sealed Tallum Boots
(21513,5496,1,1,1,2637),-- Sealed Dark Crystal Boots Lining
(21513,5497,1,1,1,2637),-- Sealed Tallum Boots Lining
(21513,5508,1,1,1,2637),-- Sealed Dark Crystal Gloves Design
(21513,5509,1,1,1,2637),-- Sealed Tallum Gloves Design
(21513,1895,1,1,2,38462),-- Metallic Fiber
(21513,4040,1,1,2,1403),-- Mold Lubricant
(21513,4042,1,1,2,2809),-- Enria
(21513,4043,1,1,2,2809),-- Asofe
(21513,5464,1,1,2,67),-- Recipe: Dark Legion\'s Edge(60%)
(21514,4039,1,1,-1,88470),-- Mold Glue
(21514,5462,1,1,-1,983),-- Recipe: Branch of The Mother Tree(60%)
(21514,5479,1,1,-1,15705),-- Sealed Tallum Leather Armor Pattern
(21514,57,1284,2613,0,700000),-- Adena
(21514,11946,1,1,1,3),-- Common Item - Dasparion\'s Staff (no drop info)
(21514,11949,1,1,1,3),-- Common Item - Meteor Shower
(21514,8341,1,1,1,1512),-- Spiritual Eye Piece
(21514,11953,1,1,1,3),-- Common Item - Spiritual Eye
(21514,11958,1,1,1,3),-- Common Item - Destroyer Hammer
(21514,5532,1,1,1,661),-- Meteor Shower Head
(21514,5543,1,1,1,667),-- Dasparion\'s Staff Edge
(21514,1864,1,1,2,200000),-- Stem
(21514,1869,1,1,2,100000),-- Iron Ore
(21514,1878,1,1,2,34483),-- Braided Hemp
(21514,1884,1,1,2,55556),-- Cord
(21514,4039,1,1,2,4000),-- Mold Glue
(21514,5272,1,1,2,58),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21514,8730,1,1,200,2674),-- Life Stone: level 67
(21514,8740,1,1,200,2674),-- Mid-Grade Life Stone: level 67
(21515,5442,1,1,-1,1199),-- Recipe: Soul Bow(60%)
(21515,5524,1,1,-1,31914),-- Sealed Dark Crystal Gaiters Pattern
(21515,5525,1,1,-1,50053),-- Sealed Dark Crystal Helmet Design
(21515,57,1566,3183,0,700000),-- Adena
(21515,11951,1,1,1,151),-- Common Item - Bloody Orchid (check this drop - 10x lower now, not enough info)
(21515,5536,1,1,1,1627),-- Bloody Orchid Head
(21515,1344,10,30,2,142857),-- Mithril Arrow
(21515,1868,1,1,2,200000),-- Thread
(21515,1873,1,1,2,47619),-- Silver Nugget
(21515,1876,1,1,2,21277),-- Mithril Ore
(21515,1895,1,1,2,43478),-- Metallic Fiber
(21515,4042,1,1,2,1993),-- Enria
(21515,5277,1,1,2,54),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21515,6670,1,1,2,9091),-- Deluxe Chest Key - Grade 6
(21515,8730,1,1,200,1508),-- Life Stone: level 67
(21515,8750,1,1,200,151),-- High-Grade Life Stone: level 67
(21516,6337,1,1,-1,20081),-- Recipe: Sealed Majestic Earring(70%)
(21516,6339,1,1,-1,30122),-- Recipe: Sealed Majestic Ring(70%)
(21516,6345,1,1,-1,149802),-- Sealed Phoenix Ring Gemstone
(21516,57,1902,3855,0,700000),-- Adena
(21516,11951,1,1,1,7),-- Common Item - Bloody Orchid (no drop info)
(21516,5536,1,1,1,1968),-- Bloody Orchid Head
(21516,1868,1,1,2,333333),-- Thread
(21516,1873,1,1,2,76923),-- Silver Nugget
(21516,1876,1,1,2,27778),-- Mithril Ore
(21516,1895,1,1,2,29412),-- Metallic Fiber
(21516,4042,1,1,2,2959),-- Enria
(21516,5282,1,1,2,86),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21517,5462,1,1,-1,1806),-- Recipe: Branch of The Mother Tree(60%)
(21517,5478,1,1,-1,41014),-- Sealed Dark Crystal Leather Armor Pattern
(21517,5479,1,1,-1,28860),-- Sealed Tallum Leather Armor Pattern
(21517,57,1964,4445,0,700000),-- Adena
(21517,12222,1,1,1,14),-- Common Item - Sealed Dark Crystal Leather Armor (no drop info, no material info)
(21517,12228,1,1,1,23),-- Common Item - Sealed Dark Crystalline Leggings
(21517,12230,1,1,1,10),-- Common Item - Sealed Tallum Leather Armor
(21517,5478,1,1,1,2019),-- Sealed Dark Crystal Leather Armor Pattern
(21517,5479,1,1,1,1421),-- Sealed Tallum Leather Armor Pattern
(21517,5482,1,1,1,3169),-- Sealed Dark Crystalline Leggings Design
(21517,1864,4,12,2,60029),-- Stem
(21517,1865,2,6,2,48023),-- Varnish
(21517,1866,3,9,2,26680),-- Suede
(21517,1868,1,3,2,160077),-- Thread
(21517,1873,1,1,2,96046),-- Silver Nugget
(21517,5162,1,1,2,107),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21518,4041,1,1,-1,23654),-- Mold Hardener
(21518,5462,1,1,-1,1008),-- Recipe: Branch of The Mother Tree(60%)
(21518,5509,1,1,-1,63781),-- Sealed Tallum Gloves Design
(21518,57,1445,2670,0,700000),-- Adena
(21518,11955,1,1,1,5),-- Common Item - Carnage Bow (no drop info, no material info)
(21518,5535,1,1,1,1371),-- Carnage Bow Stave
(21518,1343,10,30,2,163985),-- Silver Arrow
(21518,1866,1,1,2,76526),-- Suede
(21518,1871,1,1,2,114790),-- Charcoal
(21518,1885,1,1,2,9566),-- High Grade Suede
(21518,4041,1,1,2,999),-- Mold Hardener
(21518,5220,1,1,2,4592),-- Metal Hardener
(21518,5462,1,1,2,43),-- Recipe: Branch of The Mother Tree(60%)
(21518,8730,1,1,200,266),-- Life Stone: level 67
(21518,8740,1,1,200,76),-- Mid-Grade Life Stone: level 67
(21519,729,1,1,-1,343),-- Scroll: Enchant Weapon (Grade A)
(21519,5448,1,1,-1,1142),-- Recipe: Soul Separator(60%)
(21519,5526,1,1,-1,47654),-- Sealed Tallum Helm Design
(21519,57,1495,2999,0,700000),-- Adena
(21519,12229,1,1,1,20),-- Common Item - Sealed Tallum Gloves (no drop info, no material info)
(21519,12231,1,1,1,20),-- Common Item - Sealed Tallum Boots
(21519,12221,1,1,1,20),-- Common Item - Sealed Dark Crystal Gloves
(21519,12224,1,1,1,20),-- Common Item - Sealed Dark Crystal Boots
(21519,5496,1,1,1,2669),-- Sealed Dark Crystal Boots Lining
(21519,5497,1,1,1,2669),-- Sealed Tallum Boots Lining
(21519,5508,1,1,1,2669),-- Sealed Dark Crystal Gloves Design
(21519,5509,1,1,1,2669),-- Sealed Tallum Gloves Design
(21519,1868,1,1,2,303532),-- Thread
(21519,1873,1,1,2,60707),-- Silver Nugget
(21519,1876,1,1,2,30354),-- Mithril Ore
(21519,1895,1,1,2,43362),-- Metallic Fiber
(21519,4042,1,1,2,2530),-- Enria
(21519,5282,1,1,2,68),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21520,4049,1,1,-1,130322),-- Adamantite Earring Gemstone
(21520,4050,1,1,-1,153940),-- Adamantite Ring Wire
(21520,4129,1,1,-1,26890),-- Recipe: Earrings of Black Ore
(21520,57,1203,2445,0,700000),-- Adena
(21520,12338,1,1,1,1119),-- Common Item - Adamantite Earring
(21520,12340,1,1,1,1701),-- Common Item - Adamantite Ring
(21520,12339,1,1,1,924),-- Common Item - Adamantite Necklace
(21520,4049,1,1,1,6418),-- Adamantite Earring Gemstone
(21520,4050,1,1,1,7581),-- Adamantite Ring Wire
(21520,4051,1,1,1,4745),-- Adamantite Necklace Chain
(21520,947,1,1,2,49),-- Scroll: Enchant Weapon (Grade B)
(21520,1832,1,1,2,100000),-- Greater Antidote
(21520,1895,1,1,2,25000),-- Metallic Fiber
(21520,4040,1,1,2,1799),-- Mold Lubricant
(21520,4042,1,1,2,1605),-- Enria
(21520,4043,1,1,2,3597),-- Asofe
(21520,8728,1,1,200,1070),-- Life Stone: level 61
(21520,8738,1,1,200,146),-- Mid-Grade Life Stone: level 61
(21520,8748,1,1,200,49),-- High-Grade Life Stone: level 61
(21521,947,1,1,-1,977),-- Scroll: Enchant Weapon (Grade B)
(21521,4051,1,1,-1,94736),-- Adamantite Necklace Chain
(21521,4130,1,1,-1,39655),-- Recipe: Ring of Black Ore
(21521,57,1183,2404,0,700000),-- Adena
(21521,11889,1,1,1,152),-- Common Item - Great Sword
(21521,11916,1,1,1,152),-- Common Item - Keshanberk
(21521,11900,1,1,1,152),-- Common Item - Sword of Valhalla
(21521,4104,1,1,1,689),-- Great Sword Blade
(21521,4107,1,1,1,678),-- Keshanberk Blade
(21521,4108,1,1,1,678),-- Sword of Valhalla Blade
(21521,948,1,1,2,258),-- Scroll: Enchant Armor (Grade B)
(21521,1832,1,1,2,90909),-- Greater Antidote
(21521,1868,1,1,2,166667),-- Thread
(21521,1873,1,1,2,32258),-- Silver Nugget
(21521,1889,1,1,2,5525),-- Compound Braid
(21521,4044,1,1,2,2959),-- Thons
(21521,5549,1,1,2,7752),-- Metallic Thread
(21521,8728,1,1,200,1289),-- Life Stone: level 61
(21521,8738,1,1,200,758),-- Mid-Grade Life Stone: level 61
(21521,8748,1,1,200,76),-- High-Grade Life Stone: level 61
(21522,947,1,1,-1,1165),-- Scroll: Enchant Weapon (Grade B)
(21522,4051,1,1,-1,112905),-- Adamantite Necklace Chain
(21522,4130,1,1,-1,47261),-- Recipe: Ring of Black Ore
(21522,57,1409,2864,0,700000),-- Adena
(21522,11889,1,1,1,156),-- Common Item - Great Sword
(21522,11916,1,1,1,156),-- Common Item - Keshanberk
(21522,11900,1,1,1,156),-- Common Item - Sword of Valhalla
(21522,4104,1,1,1,820),-- Great Sword Blade
(21522,4107,1,1,1,808),-- Keshanberk Blade
(21522,4108,1,1,1,808),-- Sword of Valhalla Blade
(21522,948,1,1,2,468),-- Scroll: Enchant Armor (Grade B)
(21522,1832,1,1,2,111111),-- Greater Antidote
(21522,1868,1,1,2,200000),-- Thread
(21522,1873,1,1,2,31250),-- Silver Nugget
(21522,1889,1,1,2,7042),-- Compound Braid
(21522,4044,1,1,2,3745),-- Thons
(21522,5549,1,1,2,8403),-- Metallic Thread
(21522,8728,1,1,200,1873),-- Life Stone: level 61
(21522,8738,1,1,200,468),-- Mid-Grade Life Stone: level 61
(21523,948,1,1,-1,6279),-- Scroll: Enchant Armor (Grade B)
(21523,4070,1,1,-1,89915),-- Stockings of Zubei Fabric
(21523,4159,1,1,-1,7116),-- Recipe: Blue Wolf Leather Armor
(21523,57,1218,2471,0,700000),-- Adena
(21523,12161,1,1,1,219),-- Common Item - Zubei\'s Breastplate
(21523,12152,1,1,1,281),-- Common Item - Zubei\'s Gaiters
(21523,12173,1,1,1,187),-- Common Item - Avadon Breastplate
(21523,12172,1,1,1,281),-- Common Item - Avadon Gaiters
(21523,4056,1,1,1,1679),-- Zubei\'s Breastplate Part
(21523,4057,1,1,1,2594),-- Zubei\'s Gaiter Material
(21523,4064,1,1,1,1679),-- Avadon Breastplate Part
(21523,4065,1,1,1,2594),-- Avadon Gaiters Material
(21523,1832,1,1,2,83333),-- Greater Antidote
(21523,1864,1,3,2,76923),-- Stem
(21523,1865,1,1,2,76923),-- Varnish
(21523,1866,1,1,2,50000),-- Suede
(21523,1868,1,1,2,142857),-- Thread
(21523,1873,1,1,2,30303),-- Silver Nugget
(21523,4160,1,1,2,203),-- Recipe: Leather Armor of Doom
(21523,6670,1,1,2,12987),-- Deluxe Chest Key - Grade 6
(21523,8728,1,1,200,1406),-- Life Stone: level 61
(21523,8738,1,1,200,203),-- Mid-Grade Life Stone: level 61
(21523,8748,1,1,200,47),-- High-Grade Life Stone: level 61
(21523,8758,1,1,200,16),-- Top-Grade Life Stone: level 61
(21524,948,1,1,-1,6696),-- Scroll: Enchant Armor (Grade B)
(21524,4070,1,1,-1,95874),-- Stockings of Zubei Fabric
(21524,4159,1,1,-1,7587),-- Recipe: Blue Wolf Leather Armor
(21524,57,1299,2634,0,700000),-- Adena
(21524,12151,1,1,1,559),-- Common Item - Zubei\'s Leather Shirt
(21524,12150,1,1,1,1000),-- Common Item - Zubei\'s Leather Gaiters
(21524,12165,1,1,1,441),-- Common Item - Avadon Leather Armor
(21524,4066,1,1,1,3108),-- Zubei\'s Leather Shirt Fabric
(21524,4067,1,1,1,4719),-- Zubei\'s Leather Gaiter Texture
(21524,4068,1,1,1,2388),-- Avadon Leather Armor Lining
(21524,1343,20,60,2,62500),-- Silver Arrow
(21524,1867,1,1,2,111111),-- Animal Skin
(21524,1869,1,1,2,90909),-- Iron Ore
(21524,1870,1,1,2,90909),-- Coal
(21524,1871,1,1,2,83333),-- Charcoal
(21524,1872,1,3,2,38462),-- Animal Bone
(21524,4161,1,1,2,373),-- Recipe: Blue Wolf Tunic
(21524,8728,1,1,200,1458),-- Life Stone: level 61
(21524,8738,1,1,200,271),-- Mid-Grade Life Stone: level 61
(21524,8748,1,1,200,34),-- High-Grade Life Stone: level 61
(21525,4049,1,1,-1,140479),-- Adamantite Earring Gemstone
(21525,4050,1,1,-1,165938),-- Adamantite Ring Wire
(21525,4129,1,1,-1,28985),-- Recipe: Earrings of Black Ore
(21525,57,1299,2634,0,700000),-- Adena
(21525,11915,1,1,1,71),-- Common Item - Kris
(21525,11919,1,1,1,247),-- Common Item - Hell Knife
(21525,4109,1,1,1,1114),-- Kris Edge
(21525,4110,1,1,1,1114),-- Hell Knife Edge
(21525,947,1,1,2,53),-- Scroll: Enchant Weapon (Grade B)
(21525,1343,10,30,2,142857),-- Silver Arrow
(21525,1894,1,1,2,3390),-- Crafted Leather
(21525,4039,1,1,2,3378),-- Mold Glue
(21525,4041,1,1,2,918),-- Mold Hardener
(21525,4044,1,1,2,3300),-- Thons
(21525,8728,1,1,200,1572),-- Life Stone: level 61
(21525,8738,1,1,200,618),-- Mid-Grade Life Stone: level 61
(21525,8748,1,1,200,18),-- High-Grade Life Stone: level 61
(21526,1889,1,1,-1,198975),-- Compound Braid
(21526,4052,1,1,-1,105669),-- Earring of Black Ore Piece
(21526,5549,1,1,-1,298462),-- Metallic Thread
(21526,57,1449,2935,0,700000),-- Adena
(21526,11930,1,1,1,45),-- Common Item - Sword of Damascus
(21526,11943,1,1,1,67),-- Common Item - Wizard\'s Tear
(21526,11929,1,1,1,90),-- Common Item - Guardian Sword
(21526,8336,1,1,1,569),-- Blade of the Wizard\'s Tear
(21526,8330,1,1,1,642),-- Guardian Sword Blade
(21526,4114,1,1,1,1930),-- Sword of Damascus Blade
(21526,1807,1,1,2,135),-- Recipe: Soulshot: A-Grade
(21526,1868,1,1,2,200000),-- Thread
(21526,1873,1,1,2,41667),-- Silver Nugget
(21526,1889,1,1,2,7042),-- Compound Braid
(21526,4044,1,1,2,4878),-- Thons
(21526,5549,1,1,2,11111),-- Metallic Thread
(21526,8729,1,1,200,1730),-- Life Stone: level 64
(21526,8739,1,1,200,517),-- Mid-Grade Life Stone: level 64
(21526,8749,1,1,200,45),-- High-Grade Life Stone: level 64
(21526,8759,1,1,200,22),-- Top-Grade Life Stone: level 64
(21527,4040,1,1,-1,52463),-- Mold Lubricant
(21527,4081,1,1,-1,25854),-- Doom Plate Armor Temper
(21527,5161,1,1,-1,3498),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21527,57,1273,2578,0,700000),-- Adena
(21527,12183,1,1,1,397),-- Common Item - Doom Shield
(21527,12202,1,1,1,397),-- Common Item - Blue Wolf Helmet
(21527,12185,1,1,1,397),-- Common Item - Doom Helmet
(21527,4077,1,1,1,5810),-- Doom Shield Fragment
(21527,4088,1,1,1,4881),-- Blue Wolf Helmet Design
(21527,4089,1,1,1,4881),-- Doom Helmet Pattern
(21527,1343,10,30,2,125000),-- Silver Arrow
(21527,1866,1,1,2,62500),-- Suede
(21527,1871,1,1,2,83333),-- Charcoal
(21527,1877,1,1,2,6579),-- Adamantite Nugget
(21527,4040,1,1,2,2387),-- Mold Lubricant
(21527,4041,1,1,2,1789),-- Mold Hardener
(21527,5166,1,1,2,148),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21527,8729,1,1,200,596),-- Life Stone: level 64
(21528,4040,1,1,-1,64675),-- Mold Lubricant
(21528,4081,1,1,-1,31872),-- Doom Plate Armor Temper
(21528,5161,1,1,-1,4312),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21528,57,1586,3176,0,700000),-- Adena
(21528,12183,1,1,1,1389),-- Common Item - Doom Shield
(21528,12202,1,1,1,1389),-- Common Item - Blue Wolf Helmet
(21528,12185,1,1,1,1389),-- Common Item - Doom Helmet
(21528,4077,1,1,1,7162),-- Doom Shield Fragment
(21528,4088,1,1,1,6017),-- Blue Wolf Helmet Design
(21528,4089,1,1,1,6017),-- Doom Helmet Pattern
(21528,1343,10,30,2,125000),-- Silver Arrow
(21528,1866,1,1,2,90909),-- Suede
(21528,1871,1,1,2,125000),-- Charcoal
(21528,1877,1,1,2,5459),-- Adamantite Nugget
(21528,4040,1,1,2,2083),-- Mold Lubricant
(21528,4041,1,1,2,2083),-- Mold Hardener
(21528,5166,1,1,2,182),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21529,4082,1,1,-1,27949),-- Blue Wolf Leather Armor Texture
(21529,4083,1,1,-1,27949),-- Leather Armor of Doom Design
(21529,5166,1,1,-1,3298),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21529,57,1202,2430,0,700000),-- Adena
(21529,12201,1,1,1,143),-- Common Item - Blue Wolf Tunic
(21529,12192,1,1,1,143),-- Common Item - Tunic of Doom
(21529,12203,1,1,1,246),-- Common Item - Blue Wolf Stockings
(21529,12205,1,1,1,246),-- Common Item - Stockings of Doom
(21529,4084,1,1,1,1587),-- Blue Wolf Tunic Fabric
(21529,4085,1,1,1,1587),-- Tunic of Doom Pattern
(21529,4086,1,1,1,2143),-- Blue Wolf Stockings Pattern
(21529,4087,1,1,1,2143),-- Stockings of Doom Pattern
(21529,1832,1,1,2,83333),-- Greater Antidote
(21529,1867,1,1,2,111111),-- Animal Skin
(21529,1872,1,1,2,100000),-- Animal Bone
(21529,1882,1,1,2,17857),-- Leather
(21529,1894,1,1,2,2950),-- Crafted Leather
(21529,4042,1,1,2,492),-- Enria
(21529,5230,1,1,2,246),-- Recipe: Mithril Arrow (100%)
(21529,8729,1,1,200,1969),-- Life Stone: level 64
(21529,8739,1,1,200,246),-- Mid-Grade Life Stone: level 64
(21530,4120,1,1,-1,11550),-- Bellion Cestus Edge
(21530,4121,1,1,-1,11511),-- Bow of Peril Shaft
(21530,5470,1,1,-1,1444),-- Recipe: Tallum Blade(60%)
(21530,57,1284,2598,0,700000),-- Adena
(21530,11933,1,1,1,65),-- Common Item - Bellion Cestus
(21530,4120,1,1,1,1707),-- Bellion Cestus Edge
(21530,1874,1,1,2,5025),-- Oriharukon Ore
(21530,1878,1,1,2,35714),-- Braided Hemp
(21530,1884,1,1,2,52632),-- Cord
(21530,1889,1,1,2,5348),-- Compound Braid
(21530,5220,1,1,2,3717),-- Metal Hardener
(21530,6333,1,1,2,1043),-- Recipe: Sealed Phoenix Ring(70%)
(21530,6670,1,1,2,14085),-- Deluxe Chest Key - Grade 6
(21530,8729,1,1,200,978),-- Life Stone: level 64
(21530,8739,1,1,200,130),-- Mid-Grade Life Stone: level 64
(21531,1876,1,1,-1,594082),-- Mithril Ore
(21531,1895,1,3,-1,848688),-- Metallic Fiber
(21531,4052,1,1,-1,105166),-- Earring of Black Ore Piece
(21531,57,1443,2919,0,700000),-- Adena
(21531,11944,1,1,1,80),-- Common Item - Bow of Peril
(21531,4121,1,1,1,1911),-- Bow of Peril Shaft
(21531,1343,10,30,2,142857),-- Silver Arrow
(21531,1807,1,1,2,241),-- Recipe: Soulshot: A-Grade
(21531,5392,1,1,2,600),-- Recipe: Sealed Dark Crystal Gloves(60%)
(21531,1868,1,1,2,200000),-- Thread
(21531,1873,1,1,2,37037),-- Silver Nugget
(21531,1876,1,1,2,18519),-- Mithril Ore
(21531,1895,1,1,2,27778),-- Metallic Fiber
(21531,4042,1,1,2,1887),-- Enria
(21531,8729,1,1,200,1285),-- Life Stone: level 64
(21531,8739,1,1,200,482),-- Mid-Grade Life Stone: level 64
(21531,8749,1,1,200,40),-- High-Grade Life Stone: level 64
(21532,1894,1,1,-1,107921),-- Crafted Leather
(21532,4042,1,1,-1,51262),-- Enria
(21532,4054,1,1,-1,60582),-- Necklace of Black Ore Beads
(21532,57,1492,3025,0,700000),-- Adena
(21532,11935,1,1,1,195),-- Common Item - Deadman\'s Glory
(21532,11938,1,1,1,49),-- Common Item - Art of Battle Axe
(21532,11934,1,1,1,98),-- Common Item - Staff of Evil Spirits
(21532,4116,1,1,1,662),-- Deadman\'s Glory Stone
(21532,4117,1,1,1,662),-- Art of Battle Axe Blade
(21532,4118,1,1,1,627),-- Evil Spirit Head
(21532,1867,1,1,2,142857),-- Animal Skin
(21532,1872,1,1,2,142857),-- Animal Bone
(21532,1882,1,1,2,27778),-- Leather
(21532,1894,1,1,2,3817),-- Crafted Leather
(21532,4042,1,1,2,1905),-- Enria
(21532,5450,1,1,2,49),-- Recipe: Dragon Grinder(60%)
(21532,8729,1,1,200,1416),-- Life Stone: level 64
(21532,8739,1,1,200,147),-- Mid-Grade Life Stone: level 64
(21532,8749,1,1,200,8),-- High-Grade Life Stone: level 64
(21533,4077,1,1,-1,117809),-- Doom Shield Fragment
(21533,4078,1,1,-1,38254),-- Blue Wolf Breastplate Part
(21533,5470,1,1,-1,1431),-- Recipe: Tallum Blade(60%)
(21533,5348,1,1,-1,3099),-- Recipe: Sealed Dark Crystal Robe(60%)
(21533,57,1270,2576,0,700000),-- Adena
(21533,12341,1,1,1,859),-- Common Item - Earring of Black Ore
(21533,12343,1,1,1,1181),-- Common Item - Ring of Black Ore
(21533,12342,1,1,1,644),-- Common Item - Necklace of Black Ore
(21533,4052,1,1,1,4565),-- Earring of Black Ore Piece
(21533,4053,1,1,1,5526),-- Ring of Black Ore Gemstone
(21533,4054,1,1,1,2540),-- Necklace of Black Ore Beads
(21533,1343,10,30,2,125000),-- Silver Arrow
(21533,1866,1,1,2,58824),-- Suede
(21533,1871,1,1,2,83333),-- Charcoal
(21533,1877,1,1,2,3861),-- Adamantite Nugget
(21533,4040,1,1,2,1718),-- Mold Lubricant
(21533,4041,1,1,2,698),-- Mold Hardener
(21533,5456,1,1,2,41),-- Recipe: Tallum Glaive(60%)
(21533,8729,1,1,200,1558),-- Life Stone: level 64
(21533,8739,1,1,200,322),-- Mid-Grade Life Stone: level 64
(21534,4077,1,1,-1,155732),-- Doom Shield Fragment
(21534,5470,1,1,-1,1892),-- Recipe: Tallum Blade(60%)
(21534,5348,1,1,-1,3099),-- Recipe: Sealed Dark Crystal Robe(60%)
(21534,57,1679,3404,0,700000),-- Adena
(21534,12341,1,1,1,1610),-- Common Item - Earring of Black Ore
(21534,12343,1,1,1,806),-- Common Item - Ring of Black Ore
(21534,12342,1,1,1,1208),-- Common Item - Necklace of Black Ore
(21534,4052,1,1,1,6034),-- Earring of Black Ore Piece
(21534,4053,1,1,1,7305),-- Ring of Black Ore Gemstone
(21534,4054,1,1,1,3357),-- Necklace of Black Ore Beads
(21534,1343,10,30,2,142857),-- Silver Arrow
(21534,1866,1,1,2,76923),-- Suede
(21534,1871,1,1,2,111111),-- Charcoal
(21534,1877,1,1,2,3623),-- Adamantite Nugget
(21534,4040,1,1,2,3226),-- Mold Lubricant
(21534,4041,1,1,2,403),-- Mold Hardener
(21534,5456,1,1,2,55),-- Recipe: Tallum Glaive(60%)
(21534,8729,1,1,200,806),-- Life Stone: level 64
(21534,8739,1,1,200,120),-- Mid-Grade Life Stone: level 64
(21535,1876,1,2,-1,620351),-- Mithril Ore
(21535,1877,1,1,-1,124070),-- Adamantite Nugget
(21535,4052,1,1,-1,109816),-- Earring of Black Ore Piece
(21535,57,1504,3050,0,700000),-- Adena
(21535,11930,1,1,1,20),-- Common Item - Sword of Damascus
(21535,11943,1,1,1,20),-- Common Item - Wizard\'s Tear
(21535,11929,1,1,1,20),-- Common Item - Guardian Sword
(21535,8336,1,1,1,672),-- Blade of the Wizard\'s Tear
(21535,8330,1,1,1,636),-- Guardian Sword Blade
(21535,4114,1,1,1,2006),-- Sword of Damascus Blade
(21535,1343,10,30,2,142857),-- Silver Arrow
(21535,1876,1,1,2,24390),-- Mithril Ore
(21535,1877,1,1,2,3846),-- Adamantite Nugget
(21535,1880,1,1,2,12658),-- Steel
(21535,1881,1,1,2,14493),-- Coarse Bone Powder
(21535,1882,1,1,2,20833),-- Leather
(21535,6335,1,1,2,531),-- Recipe: Sealed Majestic Necklace(70%)
(21535,8729,1,1,200,1062),-- Life Stone: level 64
(21535,8739,1,1,200,398),-- Mid-Grade Life Stone: level 64
(21536,4053,1,1,-1,128534),-- Ring of Black Ore Gemstone
(21536,5446,1,1,-1,1639),-- Recipe: Bloody Orchid(60%)
(21536,5452,1,1,-1,1639),-- Recipe: Blood Tornado(60%)
(21536,57,1454,2948,0,700000),-- Adena
(21536,11939,1,1,1,306),-- Common Item - Demon Dagger
(21536,4119,1,1,1,1939),-- Demon Dagger Edge
(21536,1894,1,1,2,3676),-- Crafted Leather
(21536,4039,1,1,2,4292),-- Mold Glue
(21536,4041,1,1,2,766),-- Mold Hardener
(21536,4044,1,1,2,2755),-- Thons
(21536,6337,1,1,2,153),-- Recipe: Sealed Majestic Earring(70%)
(21536,6670,1,1,2,18868),-- Deluxe Chest Key - Grade 6
(21536,8729,1,1,200,766),-- Life Stone: level 64
(21536,8739,1,1,200,460),-- Mid-Grade Life Stone: level 64
(21536,8749,1,1,200,156),-- High-Grade Life Stone: level 64
(21537,1874,1,1,-1,201181),-- Oriharukon Ore
(21537,4043,1,1,-1,100591),-- Asofe
(21537,4099,1,1,-1,135476),-- Doom Boots Part
(21537,57,1456,2976,0,700000),-- Adena
(21537,11944,1,1,1,76),-- Common Item - Bow of Peril
(21537,4121,1,1,1,1942),-- Bow of Peril Shaft
(21537,1865,1,1,2,100000),-- Varnish
(21537,1870,1,1,2,100000),-- Coal
(21537,1874,1,1,2,6494),-- Oriharukon Ore
(21537,4039,1,1,2,3135),-- Mold Glue
(21537,4043,1,1,2,2652),-- Asofe
(21537,5442,1,1,2,55),-- Recipe: Soul Bow(60%)
(21537,8730,1,1,200,482),-- Life Stone: level 67
(21537,8740,1,1,200,362),-- Mid-Grade Life Stone: level 67
(21538,1874,1,1,-1,246445),-- Oriharukon Ore
(21538,4043,1,1,-1,123222),-- Asofe
(21538,4099,1,1,-1,165956),-- Doom Boots Part
(21538,57,1787,3644,0,700000),-- Adena
(21538,11944,1,1,1,594),-- Common Item - Bow of Peril
(21538,4121,1,1,1,2378),-- Bow of Peril Shaft
(21538,1865,1,1,2,142857),-- Varnish
(21538,1870,1,1,2,125000),-- Coal
(21538,1874,1,1,2,6536),-- Oriharukon Ore
(21538,4039,1,1,2,2967),-- Mold Glue
(21538,4043,1,1,2,1188),-- Asofe
(21538,5442,1,1,2,68),-- Recipe: Soul Bow(60%)
(21538,8730,1,1,200,1783),-- Life Stone: level 67
(21538,8740,1,1,200,594),-- Mid-Grade Life Stone: level 67
(21539,1807,1,1,-1,4376),-- Recipe: Soulshot: A-Grade
(21539,3035,1,1,-1,4376),-- Recipe: Spiritshot A
(21539,4114,1,1,-1,14366),-- Sword of Damascus Blade
(21539,57,1584,3236,0,700000),-- Adena
(21539,12201,1,1,1,368),-- Common Item - Blue Wolf Tunic
(21539,12192,1,1,1,245),-- Common Item - Tunic of Doom
(21539,12203,1,1,1,736),-- Common Item - Blue Wolf Stockings
(21539,12205,1,1,1,613),-- Common Item - Stockings of Doom
(21539,4084,1,1,1,2106),-- Blue Wolf Tunic Fabric
(21539,4085,1,1,1,2106),-- Tunic of Doom Pattern
(21539,4086,1,1,1,2843),-- Blue Wolf Stockings Pattern
(21539,4087,1,1,1,2843),-- Stockings of Doom Pattern
(21539,1343,10,30,2,142857),-- Silver Arrow
(21539,1866,1,1,2,71429),-- Suede
(21539,1871,1,1,2,100000),-- Charcoal
(21539,1877,1,1,2,4167),-- Adamantite Nugget
(21539,4040,1,1,2,1473),-- Mold Lubricant
(21539,4041,1,1,2,1473),-- Mold Hardener
(21539,5448,1,1,2,52),-- Recipe: Soul Separator(60%)
(21539,5394,1,1,2,786),-- Recipe: Sealed Tallum Gloves(60%)
(21539,8730,1,1,200,1227),-- Life Stone: level 67
(21539,8740,1,1,200,123),-- Mid-Grade Life Stone: level 67
(21539,8750,1,1,200,11),-- High-Grade Life Stone: level 67
(21540,1807,1,1,-1,4376),-- Recipe: Soulshot: A-Grade
(21540,3035,1,1,-1,4376),-- Recipe: Spiritshot A
(21540,4053,1,1,-1,140680),-- Ring of Black Ore Gemstone
(21540,57,1586,3236,0,700000),-- Adena
(21540,11935,1,1,1,130),-- Common Item - Deadman\'s Glory
(21540,11938,1,1,1,130),-- Common Item - Art of Battle Axe
(21540,11934,1,1,1,130),-- Common Item - Staff of Evil Spirits
(21540,4116,1,1,1,707),-- Deadman\'s Glory Stone
(21540,4117,1,1,1,707),-- Art of Battle Axe Blade
(21540,4118,1,1,1,670),-- Evil Spirit Head
(21540,729,1,1,2,18),-- Scroll: Enchant Weapon (Grade A)
(21540,1343,10,30,2,142857),-- Silver Arrow
(21540,1894,1,1,2,3390),-- Crafted Leather
(21540,4039,1,1,2,3125),-- Mold Glue
(21540,4041,1,1,2,521),-- Mold Hardener
(21540,4044,1,1,2,2083),-- Thons
(21540,5394,1,1,2,1042),-- Recipe: Sealed Tallum Gloves(60%)
(21540,8730,1,1,200,1433),-- Life Stone: level 67
(21540,8740,1,1,200,651),-- Mid-Grade Life Stone: level 67
(21541,1874,1,1,-1,222264),-- Oriharukon Ore
(21541,4043,1,1,-1,111132),-- Asofe
(21541,4084,1,1,-1,57942),-- Blue Wolf Tunic Fabric
(21541,57,1620,3277,0,700000),-- Adena
(21541,11930,1,1,1,8),-- Common Item - Sword of Damascus (no drop info)
(21541,11943,1,1,1,8),-- Common Item - Wizard\'s Tear
(21541,11929,1,1,1,8),-- Common Item - Guardian Sword
(21541,8336,1,1,1,553),-- Blade of the Wizard\'s Tear
(21541,8330,1,1,1,738),-- Guardian Sword Blade
(21541,4114,1,1,1,2156),-- Sword of Damascus Blade
(21541,1343,10,30,2,142857),-- Silver Arrow
(21541,1865,1,1,2,90909),-- Varnish
(21541,1870,1,1,2,111111),-- Coal
(21541,1874,1,1,2,9804),-- Oriharukon Ore
(21541,4039,1,1,2,4926),-- Mold Glue
(21541,4043,1,1,2,3690),-- Asofe
(21541,5392,1,1,2,520),-- Recipe: Sealed Dark Crystal Gloves(60%)
(21541,5271,1,1,2,188),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(21541,8729,1,1,200,1230),-- Life Stone: level 64
(21541,8739,1,1,200,122),-- Mid-Grade Life Stone: level 64
(21542,1807,1,1,-1,3091),-- Recipe: Soulshot: A-Grade
(21542,3035,1,1,-1,3091),-- Recipe: Spiritshot A
(21542,4053,1,1,-1,99376),-- Ring of Black Ore Gemstone
(21542,57,1128,2280,0,700000),-- Adena
(21542,11939,1,1,1,6),-- Common Item - Demon Dagger (no drop info)
(21542,4119,1,1,1,1499),-- Demon Dagger Edge
(21542,1343,10,30,2,125000),-- Silver Arrow
(21542,1864,1,1,2,142857),-- Stem
(21542,1869,1,1,2,70000),-- Iron Ore
(21542,1880,1,1,2,9784),-- Steel
(21542,3035,1,1,2,131),-- Recipe: Spiritshot A
(21542,4043,1,1,2,2639),-- Asofe
(21542,5550,1,1,2,1321),-- Durable Metal Plate
(21542,8729,1,1,200,1321),-- Life Stone: level 64
(21543,1877,1,1,-1,105814),-- Adamantite Nugget
(21543,4041,1,1,-1,23003),-- Mold Hardener
(21543,4054,1,1,-1,52104),-- Necklace of Black Ore Beads
(21543,57,1284,2167,0,700000),-- Adena
(21543,11935,1,1,1,2),-- Common Item - Deadman\'s Glory (no drop info)
(21543,11938,1,1,1,2),-- Common Item - Art of Battle Axe
(21543,11934,1,1,1,2),-- Common Item - Staff of Evil Spirits
(21543,4116,1,1,1,570),-- Deadman\'s Glory Stone
(21543,4117,1,1,1,570),-- Art of Battle Axe Blade
(21543,4118,1,1,1,540),-- Evil Spirit Head
(21543,1866,1,1,2,83333),-- Suede
(21543,1871,1,1,2,90909),-- Charcoal
(21543,1877,1,1,2,6579),-- Adamantite Nugget
(21543,3956,1,1,2,174),-- Recipe: Blessed Spiritshot A
(21543,4040,1,1,2,2605),-- Mold Lubricant
(21543,4041,1,1,2,1318),-- Mold Hardener
(21543,8729,1,1,200,251),-- Life Stone: level 64
(21543,8739,1,1,200,251),-- Mid-Grade Life Stone: level 64
(21544,1875,1,1,-1,246302),-- Stone of Purity
(21544,4041,1,1,-1,32126),-- Mold Hardener
(21544,4115,1,1,-1,16117),-- Lance Blade
(21544,57,1783,3643,0,700000),-- Adena
(21544,12341,1,1,1,1054),-- Common Item - Earring of Black Ore
(21544,12343,1,1,1,1318),-- Common Item - Ring of Black Ore
(21544,12342,1,1,1,791),-- Common Item - Necklace of Black Ore
(21544,4052,1,1,1,6439),-- Earring of Black Ore Piece
(21544,4053,1,1,1,7796),-- Ring of Black Ore Gemstone
(21544,4054,1,1,1,3583),-- Necklace of Black Ore Beads
(21544,1867,1,1,2,166667),-- Animal Skin
(21544,1872,1,1,2,166667),-- Animal Bone
(21544,1875,1,1,2,13514),-- Stone of Purity
(21544,1881,1,1,2,14925),-- Coarse Bone Powder
(21544,4041,1,1,2,1054),-- Mold Hardener
(21544,5450,1,1,2,68),-- Recipe: Dragon Grinder(60%)
(21544,5392,1,1,2,791),-- Recipe: Sealed Dark Crystal Gloves(60%)
(21544,8730,1,1,200,1054),-- Life Stone: level 67
(21544,8740,1,1,200,527),-- Mid-Grade Life Stone: level 67
(21545,4052,1,1,-1,94015),-- Earring of Black Ore Piece
(21545,4054,1,1,-1,52304),-- Necklace of Black Ore Beads
(21545,5230,1,1,-1,3541),-- Recipe: Mithril Arrow (100%)
(21545,57,1289,2611,0,700000),-- Adena
(21545,12204,1,1,1,180),-- Common Item - Blue Wolf Breastplate
(21545,12193,1,1,1,280),-- Common Item - Blue Wolf Gaiters
(21545,12184,1,1,1,120),-- Common Item - Doom Plate Armor
(21545,4078,1,1,1,1910),-- Blue Wolf Breastplate Part
(21545,4080,1,1,1,2690),-- Blue Wolf Gaiters Material
(21545,4081,1,1,1,1289),-- Doom Plate Armor Temper
(21545,1343,10,30,2,142857),-- Silver Arrow
(21545,1864,1,3,2,90909),-- Stem
(21545,1865,1,1,2,90909),-- Varnish
(21545,1866,1,1,2,58824),-- Suede
(21545,1868,1,3,2,55556),-- Thread
(21545,1873,1,1,2,34483),-- Silver Nugget
(21545,6339,1,1,2,561),-- Recipe: Sealed Majestic Ring(70%)
(21545,8729,1,1,200,840),-- Life Stone: level 64
(21545,8739,1,1,200,124),-- Mid-Grade Life Stone: level 64
(21546,4077,1,1,-1,136297),-- Doom Shield Fragment
(21546,4078,1,1,-1,44258),-- Blue Wolf Breastplate Part
(21546,5458,1,1,-1,1656),-- Recipe: Halberd(60%)
(21546,57,1469,2980,0,700000),-- Adena
(21546,12201,1,1,1,375),-- Common Item - Blue Wolf Tunic
(21546,12192,1,1,1,375),-- Common Item - Tunic of Doom
(21546,12203,1,1,1,375),-- Common Item - Blue Wolf Stockings
(21546,12205,1,1,1,375),-- Common Item - Stockings of Doom
(21546,4084,1,1,1,1944),-- Blue Wolf Tunic Fabric
(21546,4085,1,1,1,1944),-- Tunic of Doom Pattern
(21546,4086,1,1,1,2625),-- Blue Wolf Stockings Pattern
(21546,4087,1,1,1,2625),-- Stockings of Doom Pattern
(21546,729,1,1,2,17),-- Scroll: Enchant Weapon (Grade A)
(21546,1867,1,3,2,52632),-- Animal Skin
(21546,1869,1,1,2,100000),-- Iron Ore
(21546,1870,1,1,2,100000),-- Coal
(21546,1871,1,1,2,100000),-- Charcoal
(21546,1872,1,3,2,71429),-- Animal Bone
(21546,5394,1,1,2,1124),-- Recipe: Sealed Tallum Gloves(60%)
(21546,8729,1,1,200,2809),-- Life Stone: level 64
(21546,8739,1,1,200,83),-- Mid-Grade Life Stone: level 64
(21547,4085,1,1,-1,43995),-- Tunic of Doom Pattern
(21547,5230,1,1,-1,3375),-- Recipe: Mithril Arrow (100%)
(21547,5271,1,1,-1,3375),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(21547,57,1229,2488,0,700000),-- Adena
(21547,11935,1,1,1,2),-- Common Item - Deadman\'s Glory (no drop info)
(21547,11938,1,1,1,2),-- Common Item - Art of Battle Axe
(21547,11934,1,1,1,2),-- Common Item - Staff of Evil Spirits
(21547,4116,1,1,1,545),-- Deadman\'s Glory Stone
(21547,4117,1,1,1,545),-- Art of Battle Axe Blade
(21547,4118,1,1,1,517),-- Evil Spirit Head
(21547,1895,1,1,2,26316),-- Metallic Fiber
(21547,4040,1,1,2,2137),-- Mold Lubricant
(21547,4042,1,1,2,1524),-- Enria
(21547,4043,1,1,2,2865),-- Asofe
(21547,5346,1,1,2,183),-- Recipe: Sealed Tallum Tunic(60%)
(21547,5276,1,1,2,61),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21547,8729,1,1,200,1647),-- Life Stone: level 64
(21547,8739,1,1,200,244),-- Mid-Grade Life Stone: level 64
(21548,4044,1,1,-1,72124),-- Thons
(21548,4086,1,1,-1,50774),-- Blue Wolf Stockings Pattern
(21548,5549,1,1,-1,216373),-- Metallic Thread
(21548,57,1050,2127,0,700000),-- Adena
(21548,11933,1,1,1,131),-- Common Item - Bellion Cestus
(21548,4120,1,1,1,1397),-- Bellion Cestus Edge
(21548,1343,10,30,2,83333),-- Silver Arrow
(21548,1868,1,1,2,111111),-- Thread
(21548,1873,1,1,2,25641),-- Silver Nugget
(21548,1889,1,1,2,3610),-- Compound Braid
(21548,4044,1,1,2,1642),-- Thons
(21548,5346,1,1,2,131),-- Recipe: Sealed Tallum Tunic(60%)
(21548,5281,1,1,2,107),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21548,5549,1,1,2,6897),-- Metallic Thread
(21548,6670,1,1,2,9524),-- Deluxe Chest Key - Grade 6
(21548,8729,1,1,200,722),-- Life Stone: level 64
(21548,8739,1,1,200,263),-- Mid-Grade Life Stone: level 64
(21549,4053,1,1,-1,112261),-- Ring of Black Ore Gemstone
(21549,6329,1,1,-1,15052),-- Recipe: Sealed Phoenix Necklace(70%)
(21549,6331,1,1,-1,19993),-- Recipe: Sealed Phoenix Earring(70%)
(21549,57,1273,2574,0,700000),-- Adena
(21549,12183,1,1,1,530),-- Common Item - Doom Shield
(21549,12202,1,1,1,371),-- Common Item - Blue Wolf Helmet
(21549,12185,1,1,1,847),-- Common Item - Doom Helmet
(21549,4077,1,1,1,5800),-- Doom Shield Fragment
(21549,4088,1,1,1,4873),-- Blue Wolf Helmet Design
(21549,4089,1,1,1,4873),-- Doom Helmet Pattern
(21549,1864,1,3,2,90909),-- Stem
(21549,1865,1,1,2,90909),-- Varnish
(21549,1866,1,1,2,55556),-- Suede
(21549,1868,1,1,2,166667),-- Thread
(21549,1873,1,1,2,35714),-- Silver Nugget
(21549,5354,1,1,2,530),-- Recipe: Sealed Tallum Stockings(60%)
(21549,3035,1,1,2,212),-- Recipe: Spiritshot A
(21549,8729,1,1,200,1112),-- Life Stone: level 64
(21549,8739,1,1,200,424),-- Mid-Grade Life Stone: level 64
(21549,8749,1,1,200,212),-- High-Grade Life Stone: level 64
(21550,4053,1,1,-1,114506),-- Ring of Black Ore Gemstone
(21550,6329,1,1,-1,15353),-- Recipe: Sealed Phoenix Necklace(70%)
(21550,6331,1,1,-1,20393),-- Recipe: Sealed Phoenix Earring(70%)
(21550,57,1298,2625,0,700000),-- Adena
(21550,12183,1,1,1,929),-- Common Item - Doom Shield
(21550,12202,1,1,1,929),-- Common Item - Blue Wolf Helmet
(21550,12185,1,1,1,929),-- Common Item - Doom Helmet
(21550,4077,1,1,1,5916),-- Doom Shield Fragment
(21550,4088,1,1,1,4970),-- Blue Wolf Helmet Design
(21550,4089,1,1,1,4970),-- Doom Helmet Pattern
(21550,1864,1,3,2,90909),-- Stem
(21550,1865,1,1,2,83333),-- Varnish
(21550,1866,1,1,2,62500),-- Suede
(21550,1868,1,1,2,166667),-- Thread
(21550,1873,1,1,2,37037),-- Silver Nugget
(21550,3035,1,1,2,186),-- Recipe: Spiritshot A
(21550,5354,1,1,2,434),-- Recipe: Sealed Tallum Stockings(60%)
(21550,8729,1,1,200,1116),-- Life Stone: level 64
(21550,8739,1,1,200,372),-- Mid-Grade Life Stone: level 64
(21551,1884,1,5,-1,927765),-- Cord
(21551,4080,1,1,-1,62023),-- Blue Wolf Gaiters Material
(21551,5460,1,1,-1,1648),-- Recipe: Dasparion\'s Staff(60%)
(21551,57,1463,2964,0,700000),-- Adena
(21551,12188,1,1,1,554),-- Common Item - Boots of Doom
(21551,12189,1,1,1,554),-- Common Item - Blue Wolf Boots
(21551,12187,1,1,1,554),-- Common Item - Doom Gloves
(21551,12190,1,1,1,554),-- Common Item - Blue Wolf Gloves
(21551,4090,1,1,1,4997),-- Blue Wolf Boots Design
(21551,4091,1,1,1,4997),-- Doom Gloves Part
(21551,4096,1,1,1,4997),-- Blue Wolf Gloves Fabric
(21551,4099,1,1,1,4997),-- Doom Boots Part
(21551,730,1,1,2,739),-- Scroll: Enchant Armor (Grade A)
(21551,1343,10,30,2,142857),-- Silver Arrow
(21551,1874,1,1,2,5917),-- Oriharukon Ore
(21551,1878,1,1,2,34483),-- Braided Hemp
(21551,1884,1,1,2,71429),-- Cord
(21551,1889,1,1,2,7407),-- Compound Braid
(21551,5220,1,1,2,5090),-- Metal Hardener
(21551,8729,1,1,200,4444),-- Life Stone: level 64
(21552,1884,1,5,-1,879110),-- Cord
(21552,4080,1,1,-1,58770),-- Blue Wolf Gaiters Material
(21552,5460,1,1,-1,1561),-- Recipe: Dasparion\'s Staff(60%)
(21552,57,1385,2810,0,700000),-- Adena
(21552,12188,1,1,1,443),-- Common Item - Boots of Doom
(21552,12189,1,1,1,997),-- Common Item - Blue Wolf Boots
(21552,12187,1,1,1,443),-- Common Item - Doom Gloves
(21552,12190,1,1,1,775),-- Common Item - Blue Wolf Gloves
(21552,4090,1,1,1,4735),-- Blue Wolf Boots Design
(21552,4091,1,1,1,4735),-- Doom Gloves Part
(21552,4096,1,1,1,4735),-- Blue Wolf Gloves Fabric
(21552,4099,1,1,1,4735),-- Doom Boots Part
(21552,730,1,1,2,101),-- Scroll: Enchant Armor (Grade A)
(21552,1343,10,30,2,142857),-- Silver Arrow
(21552,1874,1,1,2,6329),-- Oriharukon Ore
(21552,1878,1,1,2,34483),-- Braided Hemp
(21552,1884,1,1,2,58824),-- Cord
(21552,1889,1,1,2,5747),-- Compound Braid
(21552,5220,1,1,2,4310),-- Metal Hardener
(21552,8729,1,1,200,1218),-- Life Stone: level 64
(21552,8739,1,1,200,332),-- Mid-Grade Life Stone: level 64
(21553,1895,1,2,-1,603815),-- Metallic Fiber
(21553,4080,1,1,-1,43471),-- Blue Wolf Gaiters Material
(21553,5346,1,1,-1,14286),-- Recipe: Sealed Tallum Tunic(60%)
(21553,57,854,1732,0,700000),-- Adena
(21553,12188,1,1,1,425),-- Common Item - Boots of Doom
(21553,12189,1,1,1,567),-- Common Item - Blue Wolf Boots
(21553,12187,1,1,1,283),-- Common Item - Doom Gloves
(21553,12190,1,1,1,425),-- Common Item - Blue Wolf Gloves
(21553,4090,1,1,1,3503),-- Blue Wolf Boots Design
(21553,4091,1,1,1,3503),-- Doom Gloves Part
(21553,4096,1,1,1,3503),-- Blue Wolf Gloves Fabric
(21553,4099,1,1,1,3503),-- Doom Boots Part
(21553,1868,1,1,2,125000),-- Thread
(21553,1873,1,1,2,31250),-- Silver Nugget
(21553,1876,1,1,2,11628),-- Mithril Ore
(21553,1895,1,1,2,18868),-- Metallic Fiber
(21553,4042,1,1,2,709),-- Enria
(21553,5348,1,1,2,84),-- Recipe: Sealed Dark Crystal Robe(60%)
(21553,5462,1,1,2,39),-- Recipe: Branch of The Mother Tree(60%)
(21553,8729,1,1,200,567),-- Life Stone: level 64
(21553,8739,1,1,200,850),-- Mid-Grade Life Stone: level 64
(21554,1895,1,1,-1,603815),-- Metallic Fiber
(21554,4080,1,1,-1,43471),-- Blue Wolf Gaiters Material
(21554,6329,1,1,-1,12146),-- Recipe: Sealed Phoenix Necklace(70%)
(21554,5346,1,1,-1,14286),-- Recipe: Sealed Tallum Tunic(60%)
(21554,57,1025,2078,0,700000),-- Adena
(21554,12188,1,1,1,738),-- Common Item - Boots of Doom
(21554,12189,1,1,1,615),-- Common Item - Blue Wolf Boots
(21554,12187,1,1,1,369),-- Common Item - Doom Gloves
(21554,12190,1,1,1,492),-- Common Item - Blue Wolf Gloves
(21554,4090,1,1,1,3503),-- Blue Wolf Boots Design
(21554,4091,1,1,1,3503),-- Doom Gloves Part
(21554,4096,1,1,1,3503),-- Blue Wolf Gloves Fabric
(21554,4099,1,1,1,3503),-- Doom Boots Part
(21554,1868,1,1,2,142857),-- Thread
(21554,1873,1,1,2,27778),-- Silver Nugget
(21554,1876,1,1,2,13699),-- Mithril Ore
(21554,1895,1,1,2,20000),-- Metallic Fiber
(21554,4042,1,1,2,738),-- Enria
(21554,5462,1,1,2,39),-- Recipe: Branch of The Mother Tree(60%)
(21554,5348,1,1,2,246),-- Recipe: Sealed Dark Crystal Robe(60%)
(21554,8729,1,1,200,861),-- Life Stone: level 64
(21554,8739,1,1,200,246),-- Mid-Grade Life Stone: level 64
(21555,4082,1,1,-1,29030),-- Blue Wolf Leather Armor Texture
(21555,4083,1,1,-1,29030),-- Leather Armor of Doom Design
(21555,5354,1,1,-1,28571),-- Recipe: Sealed Tallum Stockings(60%)
(21555,57,1246,2527,0,700000),-- Adena
(21555,11933,1,1,1,144),-- Common Item - Bellion Cestus
(21555,4120,1,1,1,1659),-- Bellion Cestus Edge
(21555,1867,1,1,2,100000),-- Animal Skin
(21555,1872,1,1,2,111111),-- Animal Bone
(21555,1875,1,1,2,5747),-- Stone of Purity
(21555,1881,1,1,2,15625),-- Coarse Bone Powder
(21555,4041,1,1,2,576),-- Mold Hardener
(21555,5466,1,1,2,47),-- Recipe: Sword of Miracles(60%)
(21555,8729,1,1,200,864),-- Life Stone: level 64
(21555,8739,1,1,200,89),-- Mid-Grade Life Stone: level 64
(21556,4082,1,1,-1,28328),-- Blue Wolf Leather Armor Texture
(21556,4083,1,1,-1,28328),-- Leather Armor of Doom Design
(21556,5354,1,1,-1,28571),-- Recipe: Sealed Tallum Stockings(60%)
(21556,57,1216,2466,0,700000),-- Adena
(21556,11933,1,1,1,110),-- Common Item - Bellion Cestus
(21556,4120,1,1,1,1618),-- Bellion Cestus Edge
(21556,1867,1,1,2,111111),-- Animal Skin
(21556,1872,1,1,2,125000),-- Animal Bone
(21556,1875,1,1,2,6024),-- Stone of Purity
(21556,1881,1,1,2,13158),-- Coarse Bone Powder
(21556,4041,1,1,2,657),-- Mold Hardener
(21556,5466,1,1,2,46),-- Recipe: Sword of Miracles(60%)
(21556,8729,1,1,200,876),-- Life Stone: level 64
(21556,8739,1,1,200,219),-- Mid-Grade Life Stone: level 64
(21557,4082,1,1,-1,33514),-- Blue Wolf Leather Armor Texture
(21557,4083,1,1,-1,33514),-- Leather Armor of Doom Design
(21557,5470,1,1,-1,1621),-- Recipe: Tallum Blade(60%)
(21557,57,1438,2917,0,700000),-- Adena
(21557,12204,1,1,1,478),-- Common Item - Blue Wolf Breastplate
(21557,12193,1,1,1,478),-- Common Item - Blue Wolf Gaiters
(21557,12184,1,1,1,478),-- Common Item - Doom Plate Armor
(21557,4078,1,1,1,2134),-- Blue Wolf Breastplate Part
(21557,4080,1,1,1,3004),-- Blue Wolf Gaiters Material
(21557,4081,1,1,1,1440),-- Doom Plate Armor Temper
(21557,1343,10,30,2,142857),-- Silver Arrow
(21557,1876,1,1,2,16667),-- Mithril Ore
(21557,1877,1,1,2,4785),-- Adamantite Nugget
(21557,1880,1,1,2,9434),-- Steel
(21557,1881,1,1,2,14925),-- Coarse Bone Powder
(21557,1882,1,1,2,20833),-- Leather
(21557,5440,1,1,2,47),-- Recipe: Elysian(60%)
(21557,8729,1,1,200,1115),-- Life Stone: level 64
(21557,8739,1,1,200,319),-- Mid-Grade Life Stone: level 64
(21557,8759,1,1,200,159),-- Top-Grade Life Stone: level 64
(21558,4082,1,1,-1,33514),-- Blue Wolf Leather Armor Texture
(21558,4083,1,1,-1,33514),-- Leather Armor of Doom Design
(21558,5470,1,1,-1,1621),-- Recipe: Tallum Blade(60%)
(21558,57,1438,2917,0,700000),-- Adena
(21558,12204,1,1,1,301),-- Common Item - Blue Wolf Breastplate
(21558,12193,1,1,1,452),-- Common Item - Blue Wolf Gaiters
(21558,12184,1,1,1,211),-- Common Item - Doom Plate Armor
(21558,4078,1,1,1,2134),-- Blue Wolf Breastplate Part
(21558,4080,1,1,1,3004),-- Blue Wolf Gaiters Material
(21558,4081,1,1,1,1440),-- Doom Plate Armor Temper
(21558,1343,10,30,2,142857),-- Silver Arrow
(21558,1876,1,1,2,18868),-- Mithril Ore
(21558,1877,1,1,2,3984),-- Adamantite Nugget
(21558,1880,1,1,2,9709),-- Steel
(21558,1881,1,1,2,12500),-- Coarse Bone Powder
(21558,1882,1,1,2,23256),-- Leather
(21558,5440,1,1,2,47),-- Recipe: Elysian(60%)
(21558,8729,1,1,200,1808),-- Life Stone: level 64
(21558,8739,1,1,200,301),-- Mid-Grade Life Stone: level 64
(21559,4085,1,1,-1,56374),-- Tunic of Doom Pattern
(21559,6329,1,1,-1,18642),-- Recipe: Sealed Phoenix Necklace(70%)
(21559,6331,1,1,-1,24761),-- Recipe: Sealed Phoenix Earring(70%)
(21559,57,1573,3190,0,700000),-- Adena
(21559,12188,1,1,1,959),-- Common Item - Boots of Doom
(21559,12189,1,1,1,754),-- Common Item - Blue Wolf Boots
(21559,12187,1,1,1,617),-- Common Item - Doom Gloves
(21559,12190,1,1,1,617),-- Common Item - Blue Wolf Gloves
(21559,4090,1,1,1,5376),-- Blue Wolf Boots Design
(21559,4091,1,1,1,5376),-- Doom Gloves Part
(21559,4096,1,1,1,5376),-- Blue Wolf Gloves Fabric
(21559,4099,1,1,1,5376),-- Doom Boots Part
(21559,1343,10,30,2,142857),-- Silver Arrow
(21559,1864,1,3,2,111111),-- Stem
(21559,1869,1,1,2,100000),-- Iron Ore
(21559,1878,1,1,2,45455),-- Braided Hemp
(21559,1884,1,1,2,66667),-- Cord
(21559,4039,1,1,2,3155),-- Mold Glue
(21559,5448,1,1,2,68),-- Recipe: Soul Separator(60%)
(21559,8729,1,1,200,1164),-- Life Stone: level 64
(21559,8739,1,1,200,68),-- Mid-Grade Life Stone: level 64
(21559,8749,1,1,200,68),-- High-Grade Life Stone: level 64
(21560,1875,1,1,-1,216249),-- Stone of Purity
(21560,4040,1,1,-1,64875),-- Mold Lubricant
(21560,4086,1,1,-1,76117),-- Blue Wolf Stockings Pattern
(21560,57,1574,3189,0,700000),-- Adena
(21560,12201,1,1,1,808),-- Common Item - Blue Wolf Tunic
(21560,12192,1,1,1,808),-- Common Item - Tunic of Doom
(21560,12203,1,1,1,808),-- Common Item - Blue Wolf Stockings
(21560,12205,1,1,1,808),-- Common Item - Stockings of Doom
(21560,4084,1,1,1,2081),-- Blue Wolf Tunic Fabric
(21560,4085,1,1,1,2081),-- Tunic of Doom Pattern
(21560,4086,1,1,1,2810),-- Blue Wolf Stockings Pattern
(21560,4087,1,1,1,2810),-- Stockings of Doom Pattern
(21560,1865,1,1,2,125000),-- Varnish
(21560,1870,1,1,2,100000),-- Coal
(21560,1875,1,1,2,4854),-- Stone of Purity
(21560,1879,1,1,2,27778),-- Cokes
(21560,4040,1,1,2,2695),-- Mold Lubricant
(21560,5450,1,1,2,60),-- Recipe: Dragon Grinder(60%)
(21560,8729,1,1,200,1616),-- Life Stone: level 64
(21561,3956,1,1,-1,3641),-- Recipe: Blessed Spiritshot A
(21561,4116,1,1,-1,11932),-- Deadman\'s Glory Stone
(21561,4117,1,1,-1,11932),-- Art of Battle Axe Blade
(21561,57,1318,2693,0,700000),-- Adena
(21561,12204,1,1,1,183),-- Common Item - Blue Wolf Breastplate
(21561,12193,1,1,1,548),-- Common Item - Blue Wolf Gaiters
(21561,12184,1,1,1,91),-- Common Item - Doom Plate Armor
(21561,4078,1,1,1,1964),-- Blue Wolf Breastplate Part
(21561,4080,1,1,1,2766),-- Blue Wolf Gaiters Material
(21561,4081,1,1,1,1325),-- Doom Plate Armor Temper
(21561,1343,10,30,2,111111),-- Silver Arrow
(21561,1865,1,1,2,83333),-- Varnish
(21561,1870,1,1,2,76923),-- Coal
(21561,1875,1,1,2,4292),-- Stone of Purity
(21561,1879,1,1,2,14925),-- Cokes
(21561,4040,1,1,2,1279),-- Mold Lubricant
(21561,5456,1,1,2,183),-- Recipe: Tallum Glaive(60%)
(21561,6670,1,1,2,10989),-- Deluxe Chest Key - Grade 6
(21561,8730,1,1,200,1370),-- Life Stone: level 67
(21561,8740,1,1,200,35),-- Mid-Grade Life Stone: level 67
(21562,4118,1,1,-1,11706),-- Evil Spirit Head
(21562,5156,1,1,-1,3770),-- Recipe: Soulshot (A) Compressed Package (100%)
(21562,5220,1,1,-1,113103),-- Metal Hardener
(21562,57,1364,2788,0,700000),-- Adena
(21562,12201,1,1,1,416),-- Common Item - Blue Wolf Tunic
(21562,12192,1,1,1,625),-- Common Item - Tunic of Doom
(21562,12203,1,1,1,416),-- Common Item - Blue Wolf Stockings
(21562,12205,1,1,1,625),-- Common Item - Stockings of Doom
(21562,4084,1,1,1,1814),-- Blue Wolf Tunic Fabric
(21562,4085,1,1,1,1814),-- Tunic of Doom Pattern
(21562,4086,1,1,1,2449),-- Blue Wolf Stockings Pattern
(21562,4087,1,1,1,2449),-- Stockings of Doom Pattern
(21562,1866,1,1,2,62500),-- Suede
(21562,1871,1,1,2,100000),-- Charcoal
(21562,1885,1,1,2,9615),-- High Grade Suede
(21562,4041,1,1,2,416),-- Mold Hardener
(21562,5220,1,1,2,3534),-- Metal Hardener
(21562,5462,1,1,2,52),-- Recipe: Branch of The Mother Tree(60%)
(21562,8730,1,1,200,1458),-- Life Stone: level 67
(21562,8740,1,1,200,416),-- Mid-Grade Life Stone: level 67
(21563,4042,1,1,-1,54757),-- Enria
(21563,4119,1,1,-1,14381),-- Demon Dagger Edge
(21563,5161,1,1,-1,4381),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21563,57,1585,3240,0,700000),-- Adena
(21563,12204,1,1,1,140),-- Common Item - Blue Wolf Breastplate
(21563,12193,1,1,1,280),-- Common Item - Blue Wolf Gaiters
(21563,12184,1,1,1,140),-- Common Item - Doom Plate Armor
(21563,4078,1,1,1,2363),-- Blue Wolf Breastplate Part
(21563,4080,1,1,1,3328),-- Blue Wolf Gaiters Material
(21563,4081,1,1,1,1595),-- Doom Plate Armor Temper
(21563,1343,10,30,2,166667),-- Silver Arrow
(21563,1867,1,1,2,142857),-- Animal Skin
(21563,1872,1,1,2,142857),-- Animal Bone
(21563,1882,1,1,2,23256),-- Leather
(21563,1894,1,1,2,4425),-- Crafted Leather
(21563,4042,1,1,2,1818),-- Enria
(21563,5464,1,1,2,52),-- Recipe: Dark Legion\'s Edge(60%)
(21563,8730,1,1,200,1025),-- Life Stone: level 67
(21563,8740,1,1,200,186),-- Mid-Grade Life Stone: level 67
(21564,4120,1,1,-1,13658),-- Bellion Cestus Edge
(21564,4121,1,1,-1,13612),-- Bow of Peril Shaft
(21564,5166,1,1,-1,4167),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21564,57,1508,3081,0,700000),-- Adena
(21564,12194,1,1,1,347),-- Common Item - Blue Wolf Leather Armor
(21564,12186,1,1,1,347),-- Common Item - Leather Armor of Doom
(21564,4082,1,1,1,2608),-- Blue Wolf Leather Armor Texture
(21564,4083,1,1,1,2608),-- Leather Armor of Doom Design
(21564,1868,1,1,2,200000),-- Thread
(21564,1873,1,1,2,45455),-- Silver Nugget
(21564,1889,1,1,2,7813),-- Compound Braid
(21564,4044,1,1,2,2427),-- Thons
(21564,5466,1,1,2,58),-- Recipe: Sword of Miracles(60%)
(21564,5549,1,1,2,10204),-- Metallic Thread
(21564,8730,1,1,200,1563),-- Life Stone: level 67
(21564,8740,1,1,200,347),-- Mid-Grade Life Stone: level 67
(21565,5464,1,1,-1,1173),-- Recipe: Dark Legion\'s Edge(60%)
(21565,5520,1,1,-1,20416),-- Sealed Dark Crystal Breastplate Pattern
(21565,5521,1,1,-1,14406),-- Sealed Tallum Plate Armor Pattern
(21565,57,1533,3116,0,700000),-- Adena
(21565,11960,1,1,1,72),-- Common Item - Halberd
(21565,5542,1,1,1,1577),-- Halberd Edge
(21565,1867,1,1,2,166667),-- Animal Skin
(21565,1872,1,1,2,142857),-- Animal Bone
(21565,1882,1,1,2,24390),-- Leather
(21565,1894,1,1,2,3236),-- Crafted Leather
(21565,4042,1,1,2,2519),-- Enria
(21565,5464,1,1,2,72),-- Recipe: Dark Legion\'s Edge(60%)
(21565,8730,1,1,200,1656),-- Life Stone: level 67
(21565,8740,1,1,200,360),-- Mid-Grade Life Stone: level 67
(21565,8750,1,1,200,144),-- High-Grade Life Stone: level 67
(21566,729,1,1,-1,385),-- Scroll: Enchant Weapon (Grade A)
(21566,4044,1,1,-1,115417),-- Thons
(21566,5549,1,2,-1,346250),-- Metallic Thread
(21566,57,1676,3408,0,700000),-- Adena
(21566,11955,1,1,1,560),-- Common Item - Carnage Bow
(21566,5535,1,1,1,1745),-- Carnage Bow Stave
(21566,1343,10,30,2,166667),-- Silver Arrow
(21566,1868,1,1,2,250000),-- Thread
(21566,1873,1,1,2,50000),-- Silver Nugget
(21566,1889,1,1,2,5587),-- Compound Braid
(21566,4044,1,1,2,3497),-- Thons
(21566,5466,1,1,2,140),-- Recipe: Sword of Miracles(60%)
(21566,5549,1,1,2,9091),-- Metallic Thread
(21566,8730,1,1,200,840),-- Life Stone: level 67
(21566,8740,1,1,200,140),-- Mid-Grade Life Stone: level 67
(21566,8750,1,1,200,140),-- High-Grade Life Stone: level 67
(21567,5466,1,1,-1,1533),-- Recipe: Sword of Miracles(60%)
(21567,5524,1,1,-1,40791),-- Sealed Dark Crystal Gaiters Pattern
(21567,5525,1,1,-1,63977),-- Sealed Dark Crystal Helmet Design
(21567,57,2004,4074,0,700000),-- Adena
(21567,11951,1,1,1,324),-- Common Item - Bloody Orchid
(21567,5536,1,1,1,2080),-- Bloody Orchid Head
(21567,1864,1,1,2,333333),-- Stem
(21567,1869,1,1,2,142857),-- Iron Ore
(21567,1880,1,1,2,13333),-- Steel
(21567,4043,1,1,2,4405),-- Asofe
(21567,5550,1,1,2,2653),-- Durable Metal Plate
(21567,6335,1,1,2,583),-- Recipe: Sealed Majestic Necklace(70%)
(21567,8730,1,1,200,1425),-- Life Stone: level 67
(21567,8740,1,1,200,453),-- Mid-Grade Life Stone: level 67
(21567,8750,1,1,200,65),-- High-Grade Life Stone: level 67
(21568,730,1,1,-1,2722),-- Scroll: Enchant Armor (Grade A)
(21568,5526,1,1,-1,50497),-- Sealed Tallum Helm Design
(21568,6335,1,1,-1,12564),-- Recipe: Sealed Majestic Necklace(70%)
(21568,57,1583,3214,0,700000),-- Adena
(21568,11951,1,1,1,6),-- Common Item - Bloody Orchid (no drop info)
(21568,5536,1,1,1,1642),-- Bloody Orchid Head
(21568,1343,10,30,2,166667),-- Silver Arrow
(21568,1865,1,1,2,100000),-- Varnish
(21568,1870,1,1,2,111111),-- Coal
(21568,1874,1,1,2,7042),-- Oriharukon Ore
(21568,4039,1,1,2,3356),-- Mold Glue
(21568,4043,1,1,2,3663),-- Asofe
(21568,6337,1,1,2,306),-- Recipe: Sealed Majestic Earring(70%)
(21568,8730,1,1,200,917),-- Life Stone: level 67
(21568,8740,1,1,200,611),-- Mid-Grade Life Stone: level 67
(21569,730,1,1,-1,2062),-- Scroll: Enchant Armor (Grade A)
(21569,5526,1,1,-1,38242),-- Sealed Tallum Helm Design
(21569,6335,1,1,-1,9515),-- Recipe: Sealed Majestic Necklace(70%)
(21569,57,1199,2435,0,700000),-- Adena
(21569,11951,1,1,1,856),-- Common Item - Bloody Orchid
(21569,5536,1,1,1,1244),-- Bloody Orchid Head
(21569,1343,10,30,2,111111),-- Silver Arrow
(21569,1865,1,1,2,66667),-- Varnish
(21569,1870,1,1,2,83333),-- Coal
(21569,1874,1,1,2,4717),-- Oriharukon Ore
(21569,4039,1,1,2,2857),-- Mold Glue
(21569,4043,1,1,2,2000),-- Asofe
(21569,6337,1,1,2,571),-- Recipe: Sealed Majestic Earring(70%)
(21569,8730,1,1,200,428),-- Life Stone: level 67
(21569,8740,1,1,200,143),-- Mid-Grade Life Stone: level 67
(21570,5464,1,1,-1,954),-- Recipe: Dark Legion\'s Edge(60%)
(21570,5542,1,1,-1,8684),-- Halberd Edge
(21570,5543,1,1,-1,8766),-- Dasparion\'s Staff Edge
(21570,57,1248,2111,0,700000),-- Adena
(21570,11950,1,1,1,43),-- Common Item - Blood Tornado
(21570,5539,1,1,1,1283),-- Blood Tornado Edge
(21570,1867,1,1,2,125000),-- Animal Skin
(21570,1872,1,1,2,125000),-- Animal Bone
(21570,1875,1,1,2,6711),-- Stone of Purity
(21570,1881,1,1,2,11236),-- Coarse Bone Powder
(21570,4041,1,1,2,576),-- Mold Hardener
(21570,6339,1,1,2,672),-- Recipe: Sealed Majestic Ring(70%)
(21570,8730,1,1,200,1441),-- Life Stone: level 67
(21570,8740,1,1,200,288),-- Mid-Grade Life Stone: level 67
(21571,729,1,1,-1,377),-- Scroll: Enchant Weapon (Grade A)
(21571,4042,1,1,-1,56507),-- Enria
(21571,4043,1,1,-1,113015),-- Asofe
(21571,57,1644,3334,0,700000),-- Adena
(21571,11955,1,1,1,392),-- Common Item - Carnage Bow
(21571,5535,1,1,1,1709),-- Carnage Bow Stave
(21571,729,1,1,2,19),-- Scroll: Enchant Weapon (Grade A)
(21571,1343,10,30,2,166667),-- Silver Arrow
(21571,1895,1,1,2,23810),-- Metallic Fiber
(21571,4040,1,1,2,1957),-- Mold Lubricant
(21571,4042,1,1,2,3521),-- Enria
(21571,4043,1,1,2,7463),-- Asofe
(21571,8730,1,1,200,1957),-- Life Stone: level 67
(21572,5466,1,1,-1,1298),-- Recipe: Sword of Miracles(60%)
(21572,5547,1,1,-1,11921),-- Elemental Sword Edge
(21572,5548,1,1,-1,11921),-- Tallum Blade Edge
(21572,57,1698,3447,0,700000),-- Adena
(21572,11951,1,1,1,227),-- Common Item - Bloody Orchid
(21572,5536,1,1,1,1761),-- Bloody Orchid Head
(21572,730,1,1,2,227),-- Scroll: Enchant Armor (Grade A)
(21572,1868,1,1,2,250000),-- Thread
(21572,1873,1,1,2,43478),-- Silver Nugget
(21572,1889,1,1,2,9346),-- Compound Braid
(21572,4044,1,1,2,4098),-- Thons
(21572,5549,1,1,2,10000),-- Metallic Thread
(21572,5352,1,1,2,223),-- Recipe: Sealed Majestic Robe(60%)
(21572,8730,1,1,200,910),-- Life Stone: level 67
(21572,8740,1,1,200,1138),-- Mid-Grade Life Stone: level 67
(21573,730,1,1,-1,3010),-- Scroll: Enchant Armor (Grade A)
(21573,6335,1,1,-1,13894),-- Recipe: Sealed Majestic Necklace(70%)
(21573,6341,1,1,-1,99914),-- Sealed Phoenix Earring Gemstone
(21573,57,1751,3554,0,700000),-- Adena
(21573,12226,1,1,1,876),-- Common Item - Sealed Dark Crystal Helmet
(21573,12225,1,1,1,375),-- Common Item - Sealed Dark Crystal Shield
(21573,12234,1,1,1,751),-- Common Item - Sealed Tallum Helmet
(21573,5494,1,1,1,3735),-- Sealed Dark Crystal Shield Fragment
(21573,5525,1,1,1,2750),-- Sealed Dark Crystal Helmet Design
(21573,5526,1,1,1,2750),-- Sealed Tallum Helm Design
(21573,1343,10,30,2,166667),-- Silver Arrow
(21573,1868,1,3,2,125000),-- Thread
(21573,1873,1,1,2,45455),-- Silver Nugget
(21573,1876,1,1,2,24390),-- Mithril Ore
(21573,1895,1,1,2,34483),-- Metallic Fiber
(21573,4042,1,1,2,2252),-- Enria
(21573,5434,1,1,2,57),-- Recipe: Dragon Slayer(60%)
(21573,8730,1,1,200,1252),-- Life Stone: level 67
(21573,8740,1,1,200,28),-- Mid-Grade Life Stone: level 67
(21573,8750,1,1,200,125),-- High-Grade Life Stone: level 67
(21574,729,1,1,-1,490),-- Scroll: Enchant Weapon (Grade A)
(21574,5448,1,1,-1,1634),-- Recipe: Soul Separator(60%)
(21574,5494,1,1,-1,92661),-- Sealed Dark Crystal Shield Fragment
(21574,57,2136,4344,0,700000),-- Adena
(21574,11960,1,1,1,250),-- Common Item - Halberd
(21574,5542,1,1,1,2198),-- Halberd Edge
(21574,1343,30,90,2,83333),-- Silver Arrow
(21574,1868,1,3,2,200000),-- Thread
(21574,1873,1,1,2,76923),-- Silver Nugget
(21574,1889,1,1,2,11628),-- Compound Braid
(21574,4044,1,1,2,6757),-- Thons
(21574,5466,1,1,2,69),-- Recipe: Sword of Miracles(60%)
(21574,5549,1,1,2,20408),-- Metallic Thread
(21574,10305,1,1,1,749),-- Transform Sealbook - Oel Mahum
(21575,729,1,1,-1,276),-- Scroll: Enchant Weapon (Grade A)
(21575,5448,1,1,-1,921),-- Recipe: Soul Separator(60%)
(21575,5494,1,1,-1,52203),-- Sealed Dark Crystal Shield Fragment
(21575,57,1203,2448,0,700000),-- Adena
(21575,11960,1,1,1,5),-- Common Item - Halberd (no drop info)
(21575,5542,1,1,1,1238),-- Halberd Edge
(21575,1343,10,30,2,166667),-- Silver Arrow
(21575,1868,1,1,2,250000),-- Thread
(21575,1873,1,1,2,45455),-- Silver Nugget
(21575,1889,1,1,2,6803),-- Compound Braid
(21575,4044,1,1,2,3413),-- Thons
(21575,5466,1,1,2,39),-- Recipe: Sword of Miracles(60%)
(21575,5549,1,1,2,8850),-- Metallic Thread
(21575,10305,1,1,1,620),-- Transform Sealbook - Oel Mahum
(21576,5450,1,1,-1,1075),-- Recipe: Dragon Grinder(60%)
(21576,5456,1,1,-1,1075),-- Recipe: Tallum Glaive(60%)
(21576,5497,1,1,-1,68051),-- Sealed Tallum Boots Lining
(21576,57,1406,2858,0,700000),-- Adena
(21576,11951,1,1,1,196),-- Common Item - Bloody Orchid
(21576,5536,1,1,1,1459),-- Bloody Orchid Head
(21576,1343,10,30,2,125000),-- Silver Arrow
(21576,1868,1,1,2,200000),-- Thread
(21576,1873,1,1,2,37037),-- Silver Nugget
(21576,1876,1,1,2,16949),-- Mithril Ore
(21576,1895,1,1,2,25641),-- Metallic Fiber
(21576,4042,1,1,2,1186),-- Enria
(21576,6337,1,1,2,297),-- Recipe: Sealed Majestic Earring(70%)
(21576,8730,1,1,200,297),-- Life Stone: level 67
(21576,8740,1,1,200,297),-- Mid-Grade Life Stone: level 67
(21577,5450,1,1,-1,1038),-- Recipe: Dragon Grinder(60%)
(21577,5456,1,1,-1,1038),-- Recipe: Tallum Glaive(60%)
(21577,5497,1,1,-1,65708),-- Sealed Tallum Boots Lining
(21577,57,1357,2750,0,700000),-- Adena
(21577,11951,1,1,1,5),-- Common Item - Bloody Orchid (no drop info)
(21577,5536,1,1,1,1409),-- Bloody Orchid Head
(21577,1343,10,30,2,125000),-- Silver Arrow
(21577,1868,1,1,2,142857),-- Thread
(21577,1873,1,1,2,38462),-- Silver Nugget
(21577,1876,1,1,2,18868),-- Mithril Ore
(21577,1895,1,1,2,26316),-- Metallic Fiber
(21577,4042,1,1,2,2353),-- Enria
(21577,6337,1,1,2,607),-- Recipe: Sealed Majestic Earring(70%)
(21577,8730,1,1,200,784),-- Life Stone: level 67
(21578,5464,1,1,-1,1217),-- Recipe: Dark Legion\'s Edge(60%)
(21578,5482,1,1,-1,43386),-- Sealed Dark Crystalline Leggings Design
(21578,5485,1,1,-1,27641),-- Sealed Tallum Tunic Texture
(21578,57,1588,3238,0,700000),-- Adena
(21578,11950,1,1,1,311),-- Common Item - Blood Tornado
(21578,5539,1,1,1,1638),-- Blood Tornado Edge
(21578,1344,10,30,2,125000),-- Mithril Arrow
(21578,1866,1,1,2,71429),-- Suede
(21578,1871,1,1,2,111111),-- Charcoal
(21578,1877,1,1,2,4149),-- Adamantite Nugget
(21578,4040,1,1,2,2695),-- Mold Lubricant
(21578,4041,1,1,2,830),-- Mold Hardener
(21578,5277,1,1,2,62),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21578,8730,1,1,200,1451),-- Life Stone: level 67
(21578,8740,1,1,200,207),-- Mid-Grade Life Stone: level 67
(21579,729,1,1,-1,381),-- Scroll: Enchant Weapon (Grade A)
(21579,1876,1,1,-1,686571),-- Mithril Ore
(21579,4042,1,1,-1,57214),-- Enria
(21579,57,1660,3380,0,700000),-- Adena
(21579,11955,1,1,1,6),-- Common Item - Carnage Bow (no drop info)
(21579,5535,1,1,1,1730),-- Carnage Bow Stave
(21579,1868,1,1,2,250000),-- Thread
(21579,1873,1,1,2,38462),-- Silver Nugget
(21579,1876,1,1,2,26316),-- Mithril Ore
(21579,1895,1,1,2,41667),-- Metallic Fiber
(21579,4042,1,1,2,1307),-- Enria
(21579,5282,1,1,2,76),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21579,8730,1,1,200,1742),-- Life Stone: level 67
(21579,8740,1,1,200,436),-- Mid-Grade Life Stone: level 67
(21580,5466,1,1,-1,1289),-- Recipe: Sword of Miracles(60%)
(21580,5486,1,1,-1,20595),-- Sealed Dark Crystal Robe Fabric
(21580,5489,1,1,-1,45940),-- Sealed Tallum Stockings Fabric
(21580,57,1682,3428,0,700000),-- Adena
(21580,11951,1,1,1,358),-- Common Item - Bloody Orchid
(21580,5536,1,1,1,1750),-- Bloody Orchid Head
(21580,1344,10,30,2,142857),-- Mithril Arrow
(21580,1868,1,1,2,250000),-- Thread
(21580,1873,1,1,2,41667),-- Silver Nugget
(21580,1876,1,1,2,25000),-- Mithril Ore
(21580,1895,1,1,2,32258),-- Metallic Fiber
(21580,4042,1,1,2,1672),-- Enria
(21580,6671,1,1,2,14280),-- Deluxe Chest Key - Grade 7
(21580,6901,1,1,2,129),-- Recipe: Shining Arrow (100%)
(21580,8730,1,1,200,1314),-- Life Stone: level 67
(21580,8740,1,1,200,119),-- Mid-Grade Life Stone: level 67
(21581,730,1,1,-1,3172),-- Scroll: Enchant Armor (Grade A)
(21581,5494,1,1,-1,79945),-- Sealed Dark Crystal Shield Fragment
(21581,6335,1,1,-1,14641),-- Recipe: Sealed Majestic Necklace(70%)
(21581,57,1839,3749,0,700000),-- Adena
(21581,11950,1,1,1,281),-- Common Item - Blood Tornado
(21581,5539,1,1,1,1896),-- Blood Tornado Edge
(21581,1808,1,1,2,113),-- Recipe: Soulshot: S Grade
(21581,1865,1,1,2,142857),-- Varnish
(21581,1870,1,1,2,142857),-- Coal
(21581,1875,1,1,2,9009),-- Stone of Purity
(21581,1879,1,1,2,22727),-- Cokes
(21581,4040,1,1,2,2985),-- Mold Lubricant
(21581,8730,1,1,200,1689),-- Life Stone: level 67
(21581,8740,1,1,200,507),-- Mid-Grade Life Stone: level 67
(21581,8750,1,1,200,22),-- High-Grade Life Stone: level 67
(21581,8760,1,1,200,22),-- Top-Grade Life Stone: level 67
(21582,4041,1,1,-1,32801),-- Mold Hardener
(21582,5220,1,1,-1,150885),-- Metal Hardener
(21582,5496,1,1,-1,88444),-- Sealed Dark Crystal Boots Lining
(21582,57,1824,3716,0,700000),-- Adena
(21582,11955,1,1,1,487),-- Common Item - Carnage Bow
(21582,5535,1,1,1,1901),-- Carnage Bow Stave
(21582,1344,10,30,2,142857),-- Mithril Arrow
(21582,1866,1,1,2,90909),-- Suede
(21582,1871,1,1,2,111111),-- Charcoal
(21582,1885,1,1,2,9091),-- High Grade Suede
(21582,3036,1,1,2,71),-- Recipe: Spiritshot S
(21582,4041,1,1,2,975),-- Mold Hardener
(21582,5220,1,1,2,6993),-- Metal Hardener
(21582,8730,1,1,200,1300),-- Life Stone: level 67
(21582,8740,1,1,200,248),-- Mid-Grade Life Stone: level 67
(21582,8750,1,1,200,124),-- High-Grade Life Stone: level 67
(21583,1884,1,5,-1,789121),-- Cord
(21583,4039,1,1,-1,128232),-- Mold Glue
(21583,5532,1,1,-1,12973),-- Meteor Shower Head
(21583,57,1857,3793,0,700000),-- Adena
(21583,11946,1,1,1,57),-- Common Item - Dasparion\'s Staff
(21583,11949,1,1,1,57),-- Common Item - Meteor Shower
(21583,8346,1,1,1,725),-- Hammer Piece of Destroyer
(21583,11958,1,1,1,57),-- Common Item - Destroyer Hammer
(21583,8341,1,1,1,571),-- Spiritual Eye Piece
(21583,11953,1,1,1,57),-- Common Item - Spiritual Eye
(21583,5532,1,1,1,958),-- Meteor Shower Head
(21583,5543,1,1,1,967),-- Dasparion\'s Staff Edge
(21583,3036,1,1,2,46),-- Recipe: Spiritshot S
(21583,1344,30,90,2,71429),-- Mithril Arrow
(21583,1864,4,12,2,47619),-- Stem
(21583,1869,1,1,2,166667),-- Iron Ore
(21583,1878,1,1,2,71429),-- Braided Hemp
(21583,1884,1,1,2,111111),-- Cord
(21583,4039,1,1,2,6289),-- Mold Glue
(21583,6901,1,1,2,163),-- Recipe: Shining Arrow (100%)
(21583,10305,1,1,1,504),-- Transform Sealbook - Oel Mahum
(21584,1884,1,5,-1,735982),-- Cord
(21584,4039,1,1,-1,119597),-- Mold Glue
(21584,5532,1,1,-1,12099),-- Meteor Shower Head
(21584,57,1732,3537,0,700000),-- Adena
(21584,11946,1,1,1,26),-- Common Item - Dasparion\'s Staff
(21584,11949,1,1,1,26),-- Common Item - Meteor Shower
(21584,8346,1,1,1,682),-- Hammer Piece of Destroyer
(21584,11958,1,1,1,26),-- Common Item - Destroyer Hammer
(21584,8341,1,1,1,682),-- Spiritual Eye Piece
(21584,11953,1,1,1,26),-- Common Item - Spiritual Eye
(21584,5532,1,1,1,894),-- Meteor Shower Head
(21584,5543,1,1,1,902),-- Dasparion\'s Staff Edge
(21584,3036,1,1,2,103),-- Recipe: Spiritshot S
(21584,1344,30,90,2,66667),-- Mithril Arrow
(21584,1864,4,12,2,41667),-- Stem
(21584,1869,1,1,2,166667),-- Iron Ore
(21584,1878,1,1,2,66667),-- Braided Hemp
(21584,1884,1,1,2,100000),-- Cord
(21584,4039,1,1,2,6211),-- Mold Glue
(21584,6901,1,1,2,152),-- Recipe: Shining Arrow (100%)
(21584,10305,1,1,1,310),-- Transform Sealbook - Oel Mahum
(21585,730,1,1,-1,2490),-- Scroll: Enchant Armor (Grade A)
(21585,1876,1,1,-1,597702),-- Mithril Ore
(21585,5536,1,1,-1,10168),-- Bloody Orchid Head
(21585,57,1443,2946,0,700000),-- Adena
(21585,11955,1,1,1,228),-- Common Item - Carnage Bow
(21585,5535,1,1,1,1507),-- Carnage Bow Stave
(21585,1344,10,30,2,125000),-- Mithril Arrow
(21585,1868,1,1,2,200000),-- Thread
(21585,1873,1,1,2,34483),-- Silver Nugget
(21585,1876,1,1,2,19608),-- Mithril Ore
(21585,1895,1,1,2,27027),-- Metallic Fiber
(21585,3036,1,1,2,57),-- Recipe: Spiritshot S
(21585,4042,1,1,2,912),-- Enria
(21585,8730,1,1,200,1368),-- Life Stone: level 67
(21585,8740,1,1,200,228),-- Mid-Grade Life Stone: level 67
(21586,1895,1,3,-1,512618),-- Metallic Fiber
(21586,5462,1,1,-1,1329),-- Recipe: Branch of The Mother Tree(60%)
(21586,5539,1,1,-1,12101),-- Blood Tornado Edge
(21586,57,1733,3538,0,700000),-- Adena
(21586,11951,1,1,1,305),-- Common Item - Bloody Orchid
(21586,5536,1,1,1,1804),-- Bloody Orchid Head
(21586,1868,1,1,2,250000),-- Thread
(21586,1873,1,1,2,52632),-- Silver Nugget
(21586,1876,1,1,2,28571),-- Mithril Ore
(21586,1895,1,1,2,37037),-- Metallic Fiber
(21586,3957,1,1,2,79),-- Recipe: Blessed Spiritshot S
(21586,4042,1,1,2,1905),-- Enria
(21586,8730,1,1,200,1295),-- Life Stone: level 67
(21586,8740,1,1,200,152),-- Mid-Grade Life Stone: level 67
(21586,8750,1,1,200,76),-- High-Grade Life Stone: level 67
(21587,5464,1,1,-1,1392),-- Recipe: Dark Legion\'s Edge(60%)
(21587,5542,1,1,-1,12676),-- Halberd Edge
(21587,5543,1,1,-1,12795),-- Dasparion\'s Staff Edge
(21587,57,1815,3706,0,700000),-- Adena
(21587,11950,1,1,1,102),-- Common Item - Blood Tornado
(21587,5539,1,1,1,1872),-- Blood Tornado Edge
(21587,1344,10,30,2,142857),-- Mithril Arrow
(21587,1865,1,1,2,125000),-- Varnish
(21587,1870,1,1,2,125000),-- Coal
(21587,1875,1,1,2,8130),-- Stone of Purity
(21587,1879,1,1,2,20408),-- Cokes
(21587,4040,1,1,2,2217),-- Mold Lubricant
(21587,5157,1,1,2,102),-- Recipe: Soulshot (S) Compressed Package (100%)
(21587,8730,1,1,200,1603),-- Life Stone: level 67
(21587,8740,1,1,200,443),-- Mid-Grade Life Stone: level 67
(21588,729,1,1,-1,345),-- Scroll: Enchant Weapon (Grade A)
(21588,4042,1,1,-1,51677),-- Enria
(21588,5478,1,1,-1,26073),-- Sealed Dark Crystal Leather Armor Pattern
(21588,57,1499,3053,0,700000),-- Adena
(21588,11957,1,1,1,39),-- Common Item - Tallum Blade
(21588,11952,1,1,1,39),-- Common Item - Elemental Sword
(21588,11954,1,1,1,39),-- Common Item - Infernal Master
(21588,8331,1,1,1,593),-- Infernal Master Blade
(21588,5547,1,1,1,779),-- Elemental Sword Edge
(21588,5548,1,1,1,779),-- Tallum Blade Edge
(21588,1344,10,30,2,111111),-- Mithril Arrow
(21588,1868,1,1,2,200000),-- Thread
(21588,1873,1,1,2,43478),-- Silver Nugget
(21588,1876,1,1,2,21739),-- Mithril Ore
(21588,1895,1,1,2,30303),-- Metallic Fiber
(21588,4042,1,1,2,946),-- Enria
(21588,6901,1,1,2,131),-- Recipe: Shining Arrow (100%)
(21588,8731,1,1,200,709),-- Life Stone: level 70
(21588,8741,1,1,200,118),-- Mid-Grade Life Stone: level 70
(21589,729,1,1,-1,353),-- Scroll: Enchant Weapon (Grade A)
(21589,4042,1,1,-1,53012),-- Enria
(21589,5478,1,1,-1,26747),-- Sealed Dark Crystal Leather Armor Pattern
(21589,57,1539,3132,0,700000),-- Adena
(21589,11957,1,1,1,245),-- Common Item - Tallum Blade
(21589,11952,1,1,1,245),-- Common Item - Elemental Sword
(21589,11954,1,1,1,245),-- Common Item - Infernal Master
(21589,8331,1,1,1,435),-- Infernal Master Blade
(21589,5547,1,1,1,799),-- Elemental Sword Edge
(21589,5548,1,1,1,799),-- Tallum Blade Edge
(21589,1344,10,30,2,125000),-- Mithril Arrow
(21589,1868,1,1,2,200000),-- Thread
(21589,1873,1,1,2,41667),-- Silver Nugget
(21589,1876,1,1,2,20408),-- Mithril Ore
(21589,1895,1,1,2,34483),-- Metallic Fiber
(21589,4042,1,1,2,1468),-- Enria
(21589,6901,1,1,2,183),-- Recipe: Shining Arrow (100%)
(21589,8731,1,1,200,550),-- Life Stone: level 70
(21589,8741,1,1,200,367),-- Mid-Grade Life Stone: level 70
(21590,5440,1,1,-1,1148),-- Recipe: Elysian(60%)
(21590,5482,1,1,-1,40924),-- Sealed Dark Crystalline Leggings Design
(21590,5485,1,1,-1,26073),-- Sealed Tallum Tunic Texture
(21590,57,1499,3053,0,700000),-- Adena
(21590,11950,1,1,1,189),-- Common Item - Blood Tornado
(21590,5539,1,1,1,1544),-- Blood Tornado Edge
(21590,1344,10,30,2,125000),-- Mithril Arrow
(21590,1864,1,3,2,111111),-- Stem
(21590,1869,1,1,2,111111),-- Iron Ore
(21590,1878,1,1,2,40000),-- Braided Hemp
(21590,1884,1,1,2,62500),-- Cord
(21590,3036,1,1,2,189),-- Recipe: Spiritshot S
(21590,4039,1,1,2,4065),-- Mold Glue
(21590,8731,1,1,200,946),-- Life Stone: level 70
(21590,8741,1,1,200,95),-- Mid-Grade Life Stone: level 70
(21590,8751,1,1,200,95),-- High-Grade Life Stone: level 70
(21591,5440,1,1,-1,1343),-- Recipe: Elysian(60%)
(21591,5482,1,1,-1,47868),-- Sealed Dark Crystalline Leggings Design
(21591,5485,1,1,-1,30497),-- Sealed Tallum Tunic Texture
(21591,57,1756,3570,0,700000),-- Adena
(21591,11950,1,1,1,6),-- Common Item - Blood Tornado (no drop info)
(21591,5539,1,1,1,1807),-- Blood Tornado Edge
(21591,1344,30,90,2,47619),-- Mithril Arrow
(21591,1864,1,3,2,125000),-- Stem
(21591,1869,1,1,2,125000),-- Iron Ore
(21591,1878,1,1,2,47619),-- Braided Hemp
(21591,1884,1,1,2,66667),-- Cord
(21591,3036,1,1,2,69),-- Recipe: Spiritshot S
(21591,4039,1,1,2,2985),-- Mold Glue
(21591,8731,1,1,200,2320),-- Life Stone: level 70
(21591,8741,1,1,200,331),-- Mid-Grade Life Stone: level 70
(21592,5440,1,1,-1,989),-- Recipe: Elysian(60%)
(21592,5482,1,1,-1,35249),-- Sealed Dark Crystalline Leggings Design
(21592,5485,1,1,-1,22457),-- Sealed Tallum Tunic Texture
(21592,57,1293,2191,0,700000),-- Adena
(21592,11950,1,1,1,5),-- Common Item - Blood Tornado (no drop info)
(21592,5539,1,1,1,1331),-- Blood Tornado Edge
(21592,1344,10,30,2,100000),-- Mithril Arrow
(21592,1864,1,1,2,166667),-- Stem
(21592,1869,1,1,2,90909),-- Iron Ore
(21592,1878,1,1,2,38462),-- Braided Hemp
(21592,1884,1,1,2,55556),-- Cord
(21592,3036,1,1,2,50),-- Recipe: Spiritshot S
(21592,4039,1,1,2,3546),-- Mold Glue
(21592,8731,1,1,200,2358),-- Life Stone: level 70
(21593,729,1,1,-1,475),-- Scroll: Enchant Weapon (Grade A)
(21593,5448,1,1,-1,1584),-- Recipe: Soul Separator(60%)
(21593,5494,1,1,-1,89797),-- Sealed Dark Crystal Shield Fragment
(21593,57,2068,4210,0,700000),-- Adena
(21593,11950,1,1,1,75),-- Common Item - Blood Tornado
(21593,5539,1,1,1,2131),-- Blood Tornado Edge
(21593,1868,1,1,2,333333),-- Thread
(21593,1873,1,1,2,62500),-- Silver Nugget
(21593,1876,1,1,2,33333),-- Mithril Ore
(21593,1895,1,1,2,43478),-- Metallic Fiber
(21593,4042,1,1,2,2358),-- Enria
(21593,5162,1,1,2,75),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21593,8731,1,1,200,1161),-- Life Stone: level 70
(21593,8741,1,1,200,337),-- Mid-Grade Life Stone: level 70
(21593,8751,1,1,200,75),-- High-Grade Life Stone: level 70
(21593,8761,1,1,200,37),-- Top-Grade Life Stone: level 70
(21594,729,1,1,-1,445),-- Scroll: Enchant Weapon (Grade A)
(21594,5448,1,1,-1,1485),-- Recipe: Soul Separator(60%)
(21594,5494,1,1,-1,84188),-- Sealed Dark Crystal Shield Fragment
(21594,57,1939,3947,0,700000),-- Adena
(21594,11950,1,1,1,98),-- Common Item - Blood Tornado
(21594,5539,1,1,1,1997),-- Blood Tornado Edge
(21594,1868,1,1,2,333333),-- Thread
(21594,1873,1,1,2,55556),-- Silver Nugget
(21594,1876,1,1,2,27027),-- Mithril Ore
(21594,1895,1,1,2,41667),-- Metallic Fiber
(21594,4042,1,1,2,2849),-- Enria
(21594,5162,1,1,2,98),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21594,8731,1,1,200,1376),-- Life Stone: level 70
(21594,8741,1,1,200,196),-- Mid-Grade Life Stone: level 70
(21595,729,1,1,-1,315),-- Scroll: Enchant Weapon (Grade A)
(21595,5448,1,1,-1,1049),-- Recipe: Soul Separator(60%)
(21595,5494,1,1,-1,59510),-- Sealed Dark Crystal Shield Fragment
(21595,57,1373,2786,0,700000),-- Adena
(21595,11950,1,1,1,110),-- Common Item - Blood Tornado (check this drop - now 10x lower)
(21595,5539,1,1,1,1412),-- Blood Tornado Edge
(21595,1868,1,1,2,200000),-- Thread
(21595,1873,1,1,2,47619),-- Silver Nugget
(21595,1876,1,1,2,34483),-- Mithril Ore
(21595,1895,1,1,2,34483),-- Metallic Fiber
(21595,4042,1,1,2,2193),-- Enria
(21595,5162,1,1,2,62),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21596,5535,1,1,-1,13962),-- Carnage Bow Stave
(21596,6337,1,1,-1,20985),-- Recipe: Sealed Majestic Earring(70%)
(21596,6339,1,1,-1,31477),-- Recipe: Sealed Majestic Ring(70%)
(21596,57,1982,4028,0,700000),-- Adena
(21596,11960,1,1,1,488),-- Common Item - Halberd
(21596,5542,1,1,1,2039),-- Halberd Edge
(21596,729,1,1,2,20),-- Scroll: Enchant Weapon (Grade A)
(21596,1343,20,60,2,83333),-- Silver Arrow
(21596,1867,1,1,2,166667),-- Animal Skin
(21596,1872,1,1,2,166667),-- Animal Bone
(21596,1875,1,1,2,8065),-- Stone of Purity
(21596,1881,1,1,2,15385),-- Coarse Bone Powder
(21596,4041,1,1,2,976),-- Mold Hardener
(21596,5350,1,1,2,247),-- Recipe: Sealed Robe of Nightmare(60%) 
(21596,8730,1,1,200,2193),-- Life Stone: level 67
(21596,8740,1,1,200,976),-- Mid-Grade Life Stone: level 67
(21597,4043,1,1,-1,86136),-- Asofe
(21597,4052,1,1,-1,91488),-- Earring of Black Ore Piece
(21597,5550,1,1,-1,34455),-- Durable Metal Plate
(21597,57,1247,2547,0,700000),-- Adena
(21597,11944,1,1,1,6),-- Common Item - Bow of Peril (no drop info)
(21597,4121,1,1,1,1663),-- Bow of Peril Shaft
(21597,1343,10,30,2,125000),-- Silver Arrow
(21597,1864,1,1,2,166667),-- Stem
(21597,1869,1,1,2,83333),-- Iron Ore
(21597,1880,1,1,2,9709),-- Steel
(21597,4043,1,1,2,1538),-- Asofe
(21597,5550,1,1,2,2825),-- Durable Metal Plate
(21597,6335,1,1,2,420),-- Recipe: Sealed Majestic Necklace(70%)
(21597,8730,1,1,200,257),-- Life Stone: level 67
(21597,8740,1,1,200,513),-- Mid-Grade Life Stone: level 67
(21598,4053,1,1,-1,124039),-- Ring of Black Ore Gemstone
(21598,5230,1,1,-1,3858),-- Recipe: Mithril Arrow (100%)
(21598,5271,1,1,-1,3858),-- Recipe: Greater Soulshot (A) Compressed Package(100%)
(21598,57,1396,2378,0,700000),-- Adena
(21598,12201,1,1,1,255),-- Common Item - Blue Wolf Tunic
(21598,12192,1,1,1,255),-- Common Item - Tunic of Doom
(21598,12203,1,1,1,255),-- Common Item - Blue Wolf Stockings
(21598,12205,1,1,1,255),-- Common Item - Stockings of Doom
(21598,4084,1,1,1,1857),-- Blue Wolf Tunic Fabric
(21598,4085,1,1,1,1857),-- Tunic of Doom Pattern
(21598,4086,1,1,1,2507),-- Blue Wolf Stockings Pattern
(21598,4087,1,1,1,2507),-- Stockings of Doom Pattern
(21598,1865,1,1,2,100000),-- Varnish
(21598,1870,1,1,2,100000),-- Coal
(21598,1874,1,1,2,6369),-- Oriharukon Ore
(21598,4039,1,1,2,4587),-- Mold Glue
(21598,4043,1,1,2,3571),-- Asofe
(21598,6337,1,1,2,766),-- Recipe: Sealed Majestic Earring(70%)
(21598,8730,1,1,200,1531),-- Life Stone: level 67
(21598,8740,1,1,200,255),-- Mid-Grade Life Stone: level 67
(21599,5497,1,1,-1,102474),-- Sealed Tallum Boots Lining
(21599,6337,1,1,-1,22413),-- Recipe: Sealed Majestic Earring(70%)
(21599,6339,1,1,-1,33619),-- Recipe: Sealed Majestic Ring(70%)
(21599,57,2112,4305,0,700000),-- Adena
(21599,11951,1,1,1,8),-- Common Item - Bloody Orchid (no drop info)
(21599,5536,1,1,1,2196),-- Bloody Orchid Head
(21599,1867,1,1,2,200000),-- Animal Skin
(21599,1872,1,1,2,200000),-- Animal Bone
(21599,1882,1,1,2,35714),-- Leather
(21599,1894,1,1,2,5263),-- Crafted Leather
(21599,3957,1,1,2,96),-- Recipe: Blessed Spiritshot S
(21599,4042,1,1,2,2481),-- Enria
(21599,8730,1,1,200,2070),-- Life Stone: level 67
(21599,8740,1,1,200,517),-- Mid-Grade Life Stone: level 67
(21599,8750,1,1,200,207),-- High-Grade Life Stone: level 67
(21600,1884,1,5,-1,875794),-- Cord
(21600,5462,1,1,-1,1054),-- Recipe: Branch of The Mother Tree(60%)
(21600,5509,1,1,-1,66737),-- Sealed Tallum Gloves Design
(21600,57,1378,2803,0,700000),-- Adena
(21600,11957,1,1,1,15),-- Common Item - Tallum Blade
(21600,11952,1,1,1,15),-- Common Item - Elemental Sword
(21600,11954,1,1,1,15),-- Common Item - Infernal Master
(21600,8331,1,1,1,454),-- Infernal Master Blade
(21600,5547,1,1,1,716),-- Elemental Sword Edge
(21600,5548,1,1,1,716),-- Tallum Blade Edge
(21600,729,1,1,2,109),-- Scroll: Enchant Weapon (Grade A)
(21600,1343,10,30,2,142857),-- Silver Arrow
(21600,1864,1,3,2,90909),-- Stem
(21600,1869,1,1,2,90909),-- Iron Ore
(21600,1878,1,1,2,32258),-- Braided Hemp
(21600,1884,1,1,2,62500),-- Cord
(21600,4039,1,1,2,3472),-- Mold Glue
(21600,8730,1,1,200,1304),-- Life Stone: level 67
(21601,5464,1,1,-1,1271),-- Recipe: Dark Legion\'s Edge(60%)
(21601,5520,1,1,-1,22126),-- Sealed Dark Crystal Breastplate Pattern
(21601,5521,1,1,-1,15613),-- Sealed Tallum Plate Armor Pattern
(21601,57,1661,3379,0,700000),-- Adena
(21601,11946,1,1,1,109),-- Common Item - Dasparion\'s Staff
(21601,11949,1,1,1,109),-- Common Item - Meteor Shower
(21601,11958,1,1,1,109),-- Common Item - Destroyer Hammer
(21601,11953,1,1,1,109),-- Common Item - Spiritual Eye
(21601,8346,1,1,1,724),-- Hammer Piece of Destroyer
(21601,8341,1,1,1,305),-- Spiritual Eye Piece
(21601,5532,1,1,1,855),-- Meteor Shower Head
(21601,5543,1,1,1,863),-- Dasparion\'s Staff Edge
(21601,730,1,1,2,109),-- Scroll: Enchant Armor (Grade A)
(21601,1866,1,1,2,83333),-- Suede
(21601,1871,1,1,2,125000),-- Charcoal
(21601,1877,1,1,2,4587),-- Adamantite Nugget
(21601,4040,1,1,2,3049),-- Mold Lubricant
(21601,4041,1,1,2,1634),-- Mold Hardener
(21601,8730,1,1,200,763),-- Life Stone: level 67
(21602,1825,1,1,-1,86056),-- Recipe: Oriharukon
(21602,1902,1,1,-1,122065),-- Elven Earring Beads
(21602,5231,1,1,-1,86056),-- Recipe: Metal Hardener (100%)
(21602,57,583,1165,0,700000),-- Adena
(21602,61,1,1,1,104),-- Mithril Gloves
(21602,607,1,1,1,104),-- Ogre Power Gauntlets
(21602,1125,1,1,1,104),-- Assault Boots
(21602,1949,1,1,1,5627),-- Mithril Gloves Design
(21602,1957,1,1,1,5627),-- Assault Boots Part
(21602,1963,1,1,1,5627),-- Ogre Power Gauntlets Part
(21602,2427,1,1,1,104),-- Salamander Skin Boots
(21602,2428,1,1,1,104),-- Plate Boots
(21602,2451,1,1,1,104),-- Sage\'s Worn Gloves
(21602,2931,1,1,1,5627),-- Salamander Skin Boot Lining
(21602,2932,1,1,1,5627),-- Plate Boots Design
(21602,2933,1,1,1,5627),-- Sage\'s Worn Gloves Lining
(21602,1865,1,1,2,111111),-- Varnish
(21602,1870,1,1,2,83333),-- Coal
(21602,1875,1,1,2,5762),-- Stone of Purity
(21602,1879,1,1,2,14405),-- Cokes
(21602,2162,1,1,2,4176),-- Recipe: Aquastone Necklace
(21602,2312,1,1,2,378),-- Recipe: Crystallized Ice Bow
(21602,5159,1,1,2,289),-- Recipe: Spiritshot (C) Compressed Package (100%)
(21603,1825,1,1,-1,121859),-- Recipe: Oriharukon
(21603,1902,1,1,-1,172850),-- Elven Earring Beads
(21603,5231,1,1,-1,121859),-- Recipe: Metal Hardener (100%)
(21603,57,963,1590,0,700000),-- Adena
(21603,61,1,1,1,146),-- Mithril Gloves
(21603,607,1,1,1,146),-- Ogre Power Gauntlets
(21603,1125,1,1,1,146),-- Assault Boots
(21603,1949,1,1,1,7968),-- Mithril Gloves Design
(21603,1957,1,1,1,7968),-- Assault Boots Part
(21603,1963,1,1,1,7968),-- Ogre Power Gauntlets Part
(21603,2427,1,1,1,146),-- Salamander Skin Boots
(21603,2428,1,1,1,146),-- Plate Boots
(21603,2451,1,1,1,146),-- Sage\'s Worn Gloves
(21603,2931,1,1,1,7968),-- Salamander Skin Boot Lining
(21603,2932,1,1,1,7968),-- Plate Boots Design
(21603,2933,1,1,1,7968),-- Sage\'s Worn Gloves Lining
(21603,1865,1,1,2,122385),-- Varnish
(21603,1870,1,1,2,122385),-- Coal
(21603,1875,1,1,2,8159),-- Stone of Purity
(21603,1879,1,1,2,20398),-- Cokes
(21603,2162,1,1,2,5913),-- Recipe: Aquastone Necklace
(21603,2312,1,1,2,535),-- Recipe: Crystallized Ice Bow
(21603,5159,1,1,2,408),-- Recipe: Spiritshot (C) Compressed Package (100%)
(21604,1805,1,1,-1,9195),-- Recipe: Soulshot: C-Grade
(21604,1905,1,1,-1,268330),-- Omen Beast\'s Eye Earring Gemstone
(21604,4469,1,1,-1,15762),-- Dye of Strength <Str+1 Con-1>
(21604,57,1382,2703,0,700000),-- Adena
(21604,353,1,1,1,178),-- Half Plate Armor
(21604,380,1,1,1,284),-- Plate Gaiters
(21604,1950,1,1,1,11240),-- Half Plate Temper
(21604,1951,1,1,1,17481),-- Plate Gaiters Part
(21604,1805,1,1,2,924),-- Recipe: Soulshot: C-Grade
(21604,1867,1,3,2,200000),-- Animal Skin
(21604,1872,1,3,2,125000),-- Animal Bone
(21604,1882,1,1,2,66667),-- Leather
(21604,1894,1,1,2,9721),-- Crafted Leather
(21605,1874,1,1,-1,89292),-- Oriharukon Ore
(21605,1875,1,1,-1,89292),-- Stone of Purity
(21605,1905,1,1,-1,130289),-- Omen Beast\'s Eye Earring Gemstone
(21605,57,648,1314,0,700000),-- Adena
(21605,281,1,1,1,28),-- Crystallized Ice Bow
(21605,2074,1,1,1,2242),-- Crystallized Ice Bow Shaft
(21605,1867,1,1,2,120574),-- Animal Skin
(21605,1872,1,1,2,120574),-- Animal Bone
(21605,1875,1,1,2,6029),-- Stone of Purity
(21605,1881,1,1,2,12058),-- Coarse Bone Powder
(21605,2205,1,1,2,2862),-- Recipe: Chain Gaiters
(21605,2209,1,1,2,6800),-- Recipe: Eldarake
(21605,2299,1,1,2,395),-- Recipe: Big Hammer
(21606,1874,1,1,-1,126442),-- Oriharukon Ore
(21606,1875,1,1,-1,126442),-- Stone of Purity
(21606,1905,1,1,-1,184497),-- Omen Beast\'s Eye Earring Gemstone
(21606,57,943,1789,0,700000),-- Adena
(21606,281,1,1,1,40),-- Crystallized Ice Bow
(21606,2074,1,1,1,3174),-- Crystallized Ice Bow Shaft
(21606,1867,1,1,2,170738),-- Animal Skin
(21606,1872,1,1,2,170738),-- Animal Bone
(21606,1875,1,1,2,8537),-- Stone of Purity
(21606,1881,1,1,2,17074),-- Coarse Bone Powder
(21606,2205,1,1,2,4053),-- Recipe: Chain Gaiters
(21606,2209,1,1,2,9629),-- Recipe: Eldarake
(21606,2299,1,1,2,560),-- Recipe: Big Hammer
(21607,1908,1,1,-1,280642),-- Moonstone Earring Wire
(21607,1909,1,1,-1,400213),-- Aquastone Ring Wire
(21607,2163,1,1,-1,151203),-- Recipe: Earring of Protection
(21607,57,1429,2915,0,700000),-- Adena
(21607,354,1,1,1,139),-- Chain Mail Shirt
(21607,381,1,1,1,222),-- Chain Gaiters
(21607,1959,1,1,1,9433),-- Chain Mail Shirt Material
(21607,1960,1,1,1,19189),-- Chain Gaiters Part
(21607,1875,1,1,2,15563),-- Stone of Purity
(21607,1879,1,1,2,38907),-- Cokes
(21607,1885,1,1,2,19454),-- High Grade Suede
(21607,2163,1,1,2,11911),-- Recipe: Earring of Protection
(21607,5549,1,1,2,23344),-- Metallic Thread
(21607,5550,1,1,2,3113),-- Durable Metal Plate
(21607,8723,1,1,200,2703),-- Life Stone: level 46
(21608,1871,1,3,-1,549216),-- Charcoal
(21608,1879,1,1,-1,183072),-- Cokes
(21608,1906,1,1,-1,150987),-- Mithril Ring Wire
(21608,57,549,1079,0,700000),-- Adena
(21608,439,1,1,1,69),-- Karmian Tunic
(21608,471,1,1,1,110),-- Karmian Stockings
(21608,1954,1,1,1,6900),-- Karmian Stocking Design
(21608,1962,1,1,1,4154),-- Karmian Tunic Pattern
(21608,1865,1,1,2,51914),-- Varnish
(21608,1870,1,1,2,51914),-- Coal
(21608,1875,1,1,2,3461),-- Stone of Purity
(21608,1879,1,1,2,8653),-- Cokes
(21608,2303,1,1,2,227),-- Recipe: Skull Graver
(21608,2990,1,1,2,4121),-- Recipe: Chain Boots
(21608,3000,1,1,2,3328),-- Recipe: Dwarven Chain Shield
(21608,3033,1,1,2,173),-- Recipe: Spiritshot C
(21608,3954,1,1,2,174),-- Recipe: Blessed Spiritshot C
(21608,4040,1,1,2,1039),-- Mold Lubricant
(21609,1871,1,5,-1,777719),-- Charcoal
(21609,1879,1,1,-1,259240),-- Cokes
(21609,1906,1,1,-1,213806),-- Mithril Ring Wire
(21609,57,835,1369,0,700000),-- Adena
(21609,439,1,1,1,97),-- Karmian Tunic
(21609,471,1,1,1,156),-- Karmian Stockings
(21609,1954,1,1,1,9771),-- Karmian Stocking Design
(21609,1962,1,1,1,5882),-- Karmian Tunic Pattern
(21609,1865,1,1,2,73513),-- Varnish
(21609,1870,1,1,2,73513),-- Coal
(21609,1875,1,1,2,4901),-- Stone of Purity
(21609,1879,1,1,2,12253),-- Cokes
(21609,2303,1,1,2,322),-- Recipe: Skull Graver
(21609,2990,1,1,2,5835),-- Recipe: Chain Boots
(21609,3000,1,1,2,4713),-- Recipe: Dwarven Chain Shield
(21609,3033,1,1,2,245),-- Recipe: Spiritshot C
(21609,3954,1,1,2,246),-- Recipe: Blessed Spiritshot C
(21609,4040,1,1,2,1471); -- Mold Lubricant

INSERT INTO `droplist` VALUES 
(21610,1869,1,7,-1,946366),-- Iron Ore
(21610,2163,1,1,-1,144852),-- Recipe: Earring of Protection
(21610,4043,1,1,-1,94637),-- Asofe
(21610,57,1392,2790,0,700000),-- Adena
(21610,397,1,1,1,178),-- Mithril Shirt
(21610,1961,1,1,1,10735),-- Mithril Shirt Fabric
(21610,2387,1,1,1,284),-- Tempered Mithril Gaiters
(21610,2935,1,1,1,17835),-- Tempered Mithril Gaiters Fragment
(21610,1864,1,3,2,111111),-- Stem
(21610,1869,1,1,2,200000),-- Iron Ore
(21610,1880,1,1,2,24390),-- Steel
(21610,2164,1,1,2,8065),-- Recipe: Ring of Protection
(21610,4043,1,1,2,7455),-- Asofe
(21610,5550,1,1,2,2982),-- Durable Metal Plate
(21610,8733,1,1,200,8065),-- Mid-Grade Life Stone: level 46
(21611,1908,1,1,-1,120774),-- Moonstone Earring Wire
(21611,2066,1,1,-1,10003),-- Heavy Doom Hammer Head
(21611,2320,1,1,-1,4444),-- Recipe: Cursed Staff
(21611,57,643,1220,0,700000),-- Adena
(21611,631,1,1,1,152),-- Eldarake
(21611,1964,1,1,1,8118),-- Eldarake Temper
(21611,2413,1,1,1,107),-- Chain Hood
(21611,2495,1,1,1,152),-- Chain Shield
(21611,2936,1,1,1,7374),-- Chain Hood Pattern
(21611,2945,1,1,1,8118),-- Chain Shield Fragment
(21611,1867,1,1,2,100461),-- Animal Skin
(21611,1872,1,1,2,100461),-- Animal Bone
(21611,1882,1,1,2,16744),-- Leather
(21611,1894,1,1,2,2644),-- Crafted Leather
(21611,2230,1,1,2,1461),-- Recipe: Theca Leather Gaiters
(21611,2326,1,1,2,263),-- Recipe: Horn of Glory
(21611,4042,1,1,2,1256),-- Enria
(21611,6668,1,1,2,26909),-- Deluxe Chest Key - Grade 4
(21612,1908,1,1,-1,171022),-- Moonstone Earring Wire
(21612,2066,1,1,-1,14165),-- Heavy Doom Hammer Head
(21612,2320,1,1,-1,6293),-- Recipe: Cursed Staff
(21612,57,934,1570,0,700000),-- Adena
(21612,631,1,1,1,214),-- Eldarake
(21612,1964,1,1,1,11496),-- Eldarake Temper
(21612,2413,1,1,1,151),-- Chain Hood
(21612,2495,1,1,1,214),-- Chain Shield
(21612,2936,1,1,1,10441),-- Chain Hood Pattern
(21612,2945,1,1,1,11496),-- Chain Shield Fragment
(21612,1867,1,1,2,142257),-- Animal Skin
(21612,1872,1,1,2,142257),-- Animal Bone
(21612,1882,1,1,2,23710),-- Leather
(21612,1894,1,1,2,3744),-- Crafted Leather
(21612,2230,1,1,2,2068),-- Recipe: Theca Leather Gaiters
(21612,2326,1,1,2,372),-- Recipe: Horn of Glory
(21612,4042,1,1,2,1779),-- Enria
(21612,6668,1,1,2,38105),-- Deluxe Chest Key - Grade 4
(21613,1864,1,14,-1,880611),-- Stem
(21613,2062,1,1,-1,24174),-- Scythe Edge
(21613,2219,1,1,-1,191437),-- Recipe: Crimson Boots
(21613,57,1521,3011,0,700000),-- Adena
(21613,226,1,1,1,32),-- Cursed Dagger
(21613,232,1,1,1,32),-- Dark Elven Dagger
(21613,2070,1,1,1,2892),-- Cursed Dagger Blade
(21613,2072,1,1,1,2892),-- Dark Elven Dagger Edge
(21613,1864,1,3,2,242778),-- Stem
(21613,1869,1,3,2,80926),-- Iron Ore
(21613,1880,1,1,2,24278),-- Steel
(21613,2251,1,1,2,40464),-- Recipe: Steel Arrow
(21613,4043,1,1,2,8093),-- Asofe
(21613,5550,1,1,2,3237),-- Durable Metal Plate
(21613,8723,1,1,200,4464),-- Life Stone: level 46
(21613,8743,1,1,200,8929),-- High-Grade Life Stone: level 46
(21614,1967,1,1,-1,71784),-- Plated Leather Gaiters Material
(21614,2166,1,1,-1,44616),-- Recipe: Earrings of Binding (100%)
(21614,2328,1,1,-1,3061),-- Recipe: Sword of Delusion
(21614,57,716,1240,0,700000),-- Adena
(21614,227,1,1,1,11),-- Stiletto
(21614,242,1,1,1,11),-- Soulfire Dirk
(21614,2083,1,1,1,969),-- Stiletto Edge
(21614,2084,1,1,1,969),-- Soulfire Dirk Blade
(21614,952,1,1,2,1111),-- Scroll: Enchant Armor (Grade C)
(21614,1868,1,1,2,166618),-- Thread
(21614,1873,1,1,2,33324),-- Silver Nugget
(21614,1889,1,1,2,5554),-- Compound Braid
(21614,2229,1,1,2,1010),-- Recipe: Theca Leather Armor
(21614,4044,1,1,2,2777),-- Thons
(21614,5549,1,1,2,8331),-- Metallic Thread
(21615,1967,1,1,-1,101650),-- Plated Leather Gaiters Material
(21615,2166,1,1,-1,63178),-- Recipe: Earrings of Binding (100%)
(21615,2328,1,1,-1,4334),-- Recipe: Sword of Delusion
(21615,57,750,1531,0,700000),-- Adena
(21615,227,1,1,1,15),-- Stiletto
(21615,242,1,1,1,15),-- Soulfire Dirk
(21615,2083,1,1,1,1373),-- Stiletto Edge
(21615,2084,1,1,1,1373),-- Soulfire Dirk Blade
(21615,952,1,1,2,1573),-- Scroll: Enchant Armor (Grade C)
(21615,1868,1,1,2,235939),-- Thread
(21615,1873,1,1,2,47188),-- Silver Nugget
(21615,1889,1,1,2,7865),-- Compound Braid
(21615,2229,1,1,2,1430),-- Recipe: Theca Leather Armor
(21615,4044,1,1,2,3933),-- Thons
(21615,5549,1,1,2,11798),-- Metallic Thread
(21616,1873,1,3,-1,680372),-- Silver Nugget
(21616,1911,1,1,-1,253020),-- Earring of Protection Gemstone
(21616,1966,1,1,-1,124201),-- Plated Leather Fragment
(21616,57,1834,3326,0,700000),-- Adena
(21616,500,1,1,1,208),-- Great Helmet
(21616,632,1,1,1,298),-- Knight\'s Shield
(21616,1971,1,1,1,15380),-- Great Helmet Material
(21616,1972,1,1,1,20098),-- Knight\'s Shield Fragment
(21616,2496,1,1,1,323),-- Dwarven Chain Shield
(21616,2947,1,1,1,17945),-- Dwarven Chain Shield Fragment
(21616,951,1,1,2,457),-- Scroll: Enchant Weapon (Grade C)
(21616,1868,1,3,2,251215),-- Thread
(21616,1873,1,1,2,100486),-- Silver Nugget
(21616,1876,1,1,2,50243),-- Mithril Ore
(21616,1895,1,1,2,71776),-- Metallic Fiber
(21616,4042,1,1,2,4188),-- Enria
(21617,951,1,1,-1,2435),-- Scroll: Enchant Weapon (Grade C)
(21617,1911,1,1,-1,99629),-- Earring of Protection Gemstone
(21617,4438,1,1,-1,9553),-- Knuckle Duster Edge
(21617,57,666,1313,0,700000),-- Adena
(21617,282,1,1,1,11),-- Elemental Bow
(21617,285,1,1,1,11),-- Noble Elven Bow
(21617,2085,1,1,1,1027),-- Crossbow Shaft
(21617,2086,1,1,1,1027),-- Elven Bow of Nobility Shaft
(21617,952,1,1,2,990),-- Scroll: Enchant Armor (Grade C)
(21617,1864,1,1,2,148377),-- Stem
(21617,1869,1,1,2,74189),-- Iron Ore
(21617,1880,1,1,2,7419),-- Steel
(21617,2348,1,1,2,173),-- Recipe: Heathen\'s Book
(21617,2351,1,1,2,156),-- Recipe: Scorpion
(21617,4043,1,1,2,2473),-- Asofe
(21617,5550,1,1,2,990),-- Durable Metal Plate
(21618,951,1,1,-1,3449),-- Scroll: Enchant Weapon (Grade C)
(21618,1911,1,1,-1,141079),-- Earring of Protection Gemstone
(21618,4438,1,1,-1,13527),-- Knuckle Duster Edge
(21618,57,985,1865,0,700000),-- Adena
(21618,282,1,1,1,15),-- Elemental Bow
(21618,285,1,1,1,15),-- Noble Elven Bow
(21618,2085,1,1,1,1454),-- Crossbow Shaft
(21618,2086,1,1,1,1454),-- Elven Bow of Nobility Shaft
(21618,952,1,1,2,1401),-- Scroll: Enchant Armor (Grade C)
(21618,1864,1,1,2,210109),-- Stem
(21618,1869,1,1,2,105055),-- Iron Ore
(21618,1880,1,1,2,10506),-- Steel
(21618,2348,1,1,2,245),-- Recipe: Heathen\'s Book
(21618,2351,1,1,2,220),-- Recipe: Scorpion
(21618,4043,1,1,2,3502),-- Asofe
(21618,5550,1,1,2,1401),-- Durable Metal Plate
(21619,1912,1,1,-1,373082),-- Ring of Protection Gemstone
(21619,2166,1,1,-1,117363),-- Recipe: Earrings of Binding (100%)
(21619,2167,1,1,-1,176643),-- Recipe: Ring of Ages
(21619,57,1743,3406,0,700000),-- Adena
(21619,282,1,1,1,27),-- Elemental Bow
(21619,285,1,1,1,27),-- Noble Elven Bow
(21619,2085,1,1,1,2653),-- Crossbow Shaft
(21619,2086,1,1,1,2653),-- Elven Bow of Nobility Shaft
(21619,952,1,1,2,3410),-- Scroll: Enchant Armor (Grade C)
(21619,1867,1,3,2,71429),-- Animal Skin
(21619,1869,2,6,2,62500),-- Iron Ore
(21619,1870,2,6,2,71429),-- Coal
(21619,1871,1,1,2,125000),-- Charcoal
(21619,1872,1,3,2,142857),-- Animal Bone
(21620,1976,1,1,-1,78518),-- Rind Leather Gaiters Material
(21620,2167,1,1,-1,65879),-- Recipe: Ring of Ages
(21620,2335,1,1,-1,3003),-- Recipe: War Axe
(21620,57,640,1251,0,700000),-- Adena
(21620,853,1,1,1,98),-- Earring of Protection
(21620,884,1,1,1,147),-- Ring of Protection
(21620,916,1,1,1,74),-- Necklace of Protection
(21620,1911,1,1,1,7093),-- Earring of Protection Gemstone
(21620,1912,1,1,1,10277),-- Ring of Protection Gemstone
(21620,1913,1,1,1,5799),-- Necklace of Protection Gemstone
(21620,952,1,1,2,1272),-- Scroll: Enchant Armor (Grade C)
(21620,1865,1,1,2,95353),-- Varnish
(21620,1870,1,1,2,95353),-- Coal
(21620,1874,1,1,2,6357),-- Oriharukon Ore
(21620,4039,1,1,2,3179),-- Mold Glue
(21620,4043,1,1,2,3179),-- Asofe
(21621,1976,1,1,-1,111186),-- Rind Leather Gaiters Material
(21621,2167,1,1,-1,93288),-- Recipe: Ring of Ages
(21621,2335,1,1,-1,4252),-- Recipe: War Axe
(21621,57,736,1501,0,700000),-- Adena
(21621,853,1,1,1,138),-- Earring of Protection
(21621,884,1,1,1,208),-- Ring of Protection
(21621,916,1,1,1,104),-- Necklace of Protection
(21621,1911,1,1,1,10044),-- Earring of Protection Gemstone
(21621,1912,1,1,1,14552),-- Ring of Protection Gemstone
(21621,1913,1,1,1,8212),-- Necklace of Protection Gemstone
(21621,952,1,1,2,1800),-- Scroll: Enchant Armor (Grade C)
(21621,1865,1,1,2,135024),-- Varnish
(21621,1870,1,1,2,135024),-- Coal
(21621,1874,1,1,2,9002),-- Oriharukon Ore
(21621,4039,1,1,2,4501),-- Mold Glue
(21621,4043,1,1,2,4501),-- Asofe
(21622,951,1,1,-1,6068),-- Scroll: Enchant Weapon (Grade C)
(21622,1865,1,7,-1,834354),-- Varnish
(21622,1913,1,1,-1,202944),-- Necklace of Protection Gemstone
(21622,57,1628,3287,0,700000),-- Adena
(21622,853,1,1,1,252),-- Earring of Protection
(21622,884,1,1,1,380),-- Ring of Protection
(21622,916,1,1,1,189),-- Necklace of Protection
(21622,1911,1,1,1,18333),-- Earring of Protection Gemstone
(21622,1912,1,1,1,26561),-- Ring of Protection Gemstone
(21622,1913,1,1,1,14989),-- Necklace of Protection Gemstone
(21622,1342,40,120,2,38462),-- Fine Steel Arrow
(21622,1864,5,15,2,25641),-- Stem
(21622,1865,3,9,2,35208),-- Varnish
(21622,1866,1,1,2,111111),-- Suede
(21622,1868,1,3,2,100000),-- Thread
(21622,1873,1,1,2,66667),-- Silver Nugget
(21622,2166,1,1,2,7162),-- Recipe: Earrings of Binding (100%)
(21622,8724,1,1,200,12987),-- Life Stone: level 49
(21622,8734,1,1,200,8403),-- Mid-Grade Life Stone: level 49
(21623,1539,1,1,-1,363117),-- Greater Healing Potion
(21623,1914,1,1,-1,91799),-- Earring of Binding Gemstone
(21623,4601,1,1,-1,3631),-- Greater Dye of Strength <Str+3 Con-3>
(21623,57,912,1457,0,700000),-- Adena
(21623,283,1,1,1,16),-- Akat Long Bow
(21623,2109,1,1,1,1895),-- Akat Longbow Shaft
(21623,1866,1,1,2,56312),-- Suede
(21623,1871,1,1,2,84467),-- Charcoal
(21623,1877,1,1,2,3379),-- Adamantite Nugget
(21623,2357,1,1,2,138),-- Recipe: Crystal Dagger
(21623,3014,1,1,2,2360),-- Recipe: Full Plate Boots
(21623,4040,1,1,2,1690),-- Mold Lubricant
(21623,4041,1,1,2,735),-- Mold Hardener
(21623,4188,1,1,2,98),-- Recipe: Hell Knife
(21623,8735,1,1,200,3333),-- Mid-Grade Life Stone: level 52
(21624,1539,1,1,-1,514192),-- Greater Healing Potion
(21624,1914,1,1,-1,129992),-- Earring of Binding Gemstone
(21624,4601,1,1,-1,5142),-- Greater Dye of Strength <Str+3 Con-3>
(21624,57,1153,2145,0,700000),-- Adena
(21624,283,1,1,1,23),-- Akat Long Bow
(21624,2109,1,1,1,2683),-- Akat Longbow Shaft
(21624,1866,1,1,2,79740),-- Suede
(21624,1871,1,1,2,119610),-- Charcoal
(21624,1877,1,1,2,4785),-- Adamantite Nugget
(21624,2357,1,1,2,196),-- Recipe: Crystal Dagger
(21624,3014,1,1,2,3341),-- Recipe: Full Plate Boots
(21624,4040,1,1,2,2393),-- Mold Lubricant
(21624,4041,1,1,2,1041),-- Mold Hardener
(21624,4188,1,1,2,138),-- Recipe: Hell Knife
(21625,1539,1,1,-1,831383),-- Greater Healing Potion
(21625,1882,1,2,-1,831383),-- Leather
(21625,4601,1,1,-1,8314),-- Greater Dye of Strength <Str+3 Con-3>
(21625,57,1896,3653,0,700000),-- Adena (no drop info, no material info)
(21625,1867,4,12,2,165765),-- Animal Skin
(21625,1872,7,21,2,88408),-- Animal Bone
(21625,1882,3,9,2,36837),-- Leather
(21625,1894,1,1,2,34898),-- Crafted Leather
(21625,2169,1,1,2,17604),-- Recipe: Necklace of Binding
(21625,4042,1,1,2,16577),-- Enria
(21625,8725,1,1,200,2950),-- Life Stone: level 52
(21625,8735,1,1,200,1477),-- Mid-Grade Life Stone: level 52
(21626,1539,1,1,-1,324297),-- Greater Healing Potion
(21626,4041,1,1,-1,12690),-- Mold Hardener
(21626,4603,1,1,-1,3243),-- Greater Dye of Constitution <Con+3 Str-3>
(21626,57,589,1197,0,700000),-- Adena
(21626,854,1,1,1,69),-- Earring of Binding
(21626,885,1,1,1,103),-- Ring of Ages
(21626,917,1,1,1,52),-- Necklace of Mermaid
(21626,1914,1,1,1,5652),-- Earring of Binding Gemstone
(21626,1915,1,1,1,8863),-- Ring of Ages Gemstone
(21626,1916,1,1,1,4281),-- Necklace of Mermaid Teardrop
(21626,1867,1,1,2,100583),-- Animal Skin
(21626,1872,1,1,2,100583),-- Animal Bone
(21626,1875,1,1,2,5030),-- Stone of Purity
(21626,1881,1,1,2,10059),-- Coarse Bone Powder
(21626,2355,1,1,2,124),-- Recipe: Ghoul\'s Staff
(21626,4041,1,1,2,657),-- Mold Hardener
(21626,4601,1,1,2,168),-- Greater Dye of Strength <Str+3 Con-3>
(21626,4609,1,1,2,168),-- Greater Dye of Mental Strength <Men+3 Int-3>
(21627,1539,1,1,-1,459222),-- Greater Healing Potion
(21627,4041,1,1,-1,17970),-- Mold Hardener
(21627,4603,1,1,-1,4592),-- Greater Dye of Constitution <Con+3 Str-3>
(21627,57,835,1695,0,700000),-- Adena
(21627,854,1,1,1,97),-- Earring of Binding
(21627,885,1,1,1,146),-- Ring of Ages
(21627,917,1,1,1,73),-- Necklace of Mermaid
(21627,1914,1,1,1,8003),-- Earring of Binding Gemstone
(21627,1915,1,1,1,12550),-- Ring of Ages Gemstone
(21627,1916,1,1,1,6062),-- Necklace of Mermaid Teardrop
(21627,1867,1,1,2,142430),-- Animal Skin
(21627,1872,1,1,2,142430),-- Animal Bone
(21627,1875,1,1,2,7122),-- Stone of Purity
(21627,1881,1,1,2,14243),-- Coarse Bone Powder
(21627,2355,1,1,2,175),-- Recipe: Ghoul\'s Staff
(21627,4041,1,1,2,929),-- Mold Hardener
(21627,4601,1,1,2,238),-- Greater Dye of Strength <Str+3 Con-3>
(21627,4609,1,1,2,238),-- Greater Dye of Mental Strength <Men+3 Int-3>
(21628,1539,1,1,-1,838205),-- Greater Healing Potion
(21628,1874,1,1,-1,251462),-- Oriharukon Ore
(21628,4604,1,1,-1,8382),-- Greater Dye of Constitution <Con+3 Dex-3>
(21628,57,1932,3513,0,700000),-- Adena
(21628,162,1,1,1,5),-- War Axe
(21628,174,1,1,1,5),-- Nirvana Axe
(21628,196,1,1,1,5),-- Stick of Eternity
(21628,197,1,1,1,5),-- Paradia Staff
(21628,199,1,1,1,5),-- Pa\'agrian Hammer
(21628,200,1,1,1,5),-- Sage\'s Staff
(21628,201,1,1,1,5),-- Club of Nature
(21628,202,1,1,1,5),-- Mace of The Underworld
(21628,2097,1,1,1,523),-- War Axe Blade
(21628,2098,1,1,1,523),-- Nirvana Axe Blade
(21628,2099,1,1,1,550),-- Stick of Eternity Shaft
(21628,2100,1,1,1,550),-- Paradia Staff Head
(21628,2102,1,1,1,530),-- Pa\'agrian Hammer Head
(21628,2103,1,1,1,550),-- Sage\'s Staff Head
(21628,2104,1,1,1,550),-- Club of Nature Head
(21628,2105,1,1,1,523),-- Mace of The Underworld Head
(21628,1874,1,1,2,14856),-- Oriharukon Ore
(21628,1878,1,1,2,89133),-- Braided Hemp
(21628,1884,1,1,2,137128),-- Cord
(21628,1889,1,1,2,14856),-- Compound Braid
(21628,2356,1,1,2,364),-- Recipe: Demon\'s Staff
(21628,3012,1,1,2,4158),-- Recipe: Full Plate Helmet
(21628,5220,1,1,2,8914),-- Metal Hardener
(21629,2100,1,1,-1,11209),-- Paradia Staff Head
(21629,2354,1,1,-1,3244),-- Recipe: Deadman\'s Staff
(21629,4042,1,1,-1,33140),-- Enria
(21629,57,1154,1934,0,700000),-- Adena
(21629,75,1,1,1,4),-- Caliburs
(21629,76,1,1,1,4),-- Sword of Delusion
(21629,77,1,1,1,4),-- Tsurugi
(21629,84,1,1,1,4),-- Homunkulus\'s Sword
(21629,134,1,1,1,4),-- Sword of Nightmare
(21629,145,1,1,1,4),-- Sword of Whispering Death
(21629,2089,1,1,1,374),-- Caliburs Edge
(21629,2090,1,1,1,374),-- Delusional Blade
(21629,2091,1,1,1,374),-- Tsurugi Blade
(21629,2092,1,1,1,374),-- Homunkulus\'s Sword Blade
(21629,2095,1,1,1,374),-- Sword of Nightmare Blade
(21629,2096,1,1,1,374),-- Sword of Whispering Death Blade
(21629,1868,1,1,2,164455),-- Thread
(21629,1873,1,1,2,32891),-- Silver Nugget
(21629,1876,1,1,2,16446),-- Mithril Ore
(21629,1895,1,1,2,23494),-- Metallic Fiber
(21629,3017,1,1,2,2297),-- Recipe: Divine Gloves
(21629,4042,1,1,2,1371),-- Enria
(21629,4608,1,1,2,183),-- Greater Dye of Intelligence <Int+3 Wit-3>
(21629,5155,1,1,2,165),-- Recipe: Soulshot (B) Compressed Package (100%)
(21629,6669,1,1,2,19123),-- Deluxe Chest Key - Grade 5
(21630,2100,1,1,-1,15872),-- Paradia Staff Head
(21630,2354,1,1,-1,4593),-- Recipe: Deadman\'s Staff
(21630,4042,1,1,-1,46928),-- Enria
(21630,57,1461,2742,0,700000),-- Adena
(21630,75,1,1,1,5),-- Caliburs
(21630,76,1,1,1,5),-- Sword of Delusion
(21630,77,1,1,1,5),-- Tsurugi
(21630,84,1,1,1,5),-- Homunkulus\'s Sword
(21630,134,1,1,1,5),-- Sword of Nightmare
(21630,145,1,1,1,5),-- Sword of Whispering Death
(21630,2089,1,1,1,529),-- Caliburs Edge
(21630,2090,1,1,1,529),-- Delusional Blade
(21630,2091,1,1,1,529),-- Tsurugi Blade
(21630,2092,1,1,1,529),-- Homunkulus\'s Sword Blade
(21630,2095,1,1,1,529),-- Sword of Nightmare Blade
(21630,2096,1,1,1,529),-- Sword of Whispering Death Blade
(21630,1868,1,1,2,232877),-- Thread
(21630,1873,1,1,2,46576),-- Silver Nugget
(21630,1876,1,1,2,23288),-- Mithril Ore
(21630,1895,1,1,2,33269),-- Metallic Fiber
(21630,3017,1,1,2,3253),-- Recipe: Divine Gloves
(21630,4042,1,1,2,1941),-- Enria
(21630,4608,1,1,2,259),-- Greater Dye of Intelligence <Int+3 Wit-3>
(21630,5155,1,1,2,233),-- Recipe: Soulshot (B) Compressed Package (100%)
(21630,6669,1,1,2,27079),-- Deluxe Chest Key - Grade 5
(21631,2091,1,1,-1,25159),-- Tsurugi Blade
(21631,2092,1,1,-1,25159),-- Homunkulus\'s Sword Blade
(21631,2353,1,1,-1,7531),-- Recipe: Samurai Longsword
(21631,57,2300,4439,0,700000),-- Adena
(21631,103,1,1,1,185),-- Tower Shield
(21631,107,1,1,1,185),-- Composite Shield
(21631,517,1,1,1,130),-- Composite Helmet
(21631,1149,1,1,1,130),-- Shining Circlet
(21631,1978,1,1,1,15297),-- Tower Shield Fragment
(21631,1983,1,1,1,10467),-- Shining Circlet Pattern
(21631,2957,1,1,1,10467),-- Composite Helmet Design
(21631,4055,1,1,1,15297),-- Composite Shield Fragment
(21631,1868,1,3,2,142857),-- Thread
(21631,1873,1,1,2,140000),-- Silver Nugget
(21631,1876,1,1,2,62500),-- Mithril Ore
(21631,1895,1,1,2,38462),-- Metallic Fiber
(21631,2357,1,1,2,445),-- Recipe: Crystal Dagger
(21631,3012,1,1,2,5089),-- Recipe: Full Plate Helmet
(21631,4042,1,1,2,4546),-- Enria
(21631,8725,1,1,200,2202),-- Life Stone: level 52
(21632,1918,1,1,-1,75951),-- Nassen\'s Earring Gemstone
(21632,2955,1,1,-1,110994),-- Theca Leather Gloves Texture
(21632,4042,1,1,-1,29228),-- Enria
(21632,57,995,1654,0,700000),-- Adena
(21632,400,1,1,1,27),-- Theca Leather Armor
(21632,401,1,1,1,14),-- Drake Leather Armor
(21632,420,1,1,1,42),-- Theca Leather Gaiters
(21632,1984,1,1,1,2314),-- Theca Leather Armor Pattern
(21632,1985,1,1,1,3722),-- Theca Leather Gaiters Pattern
(21632,1987,1,1,1,1647),-- Drake Leather Armor Texture
(21632,1865,1,1,2,80941),-- Varnish
(21632,1870,1,1,2,80941),-- Coal
(21632,1874,1,1,2,5396),-- Oriharukon Ore
(21632,4039,1,1,2,2699),-- Mold Glue
(21632,4043,1,1,2,2699),-- Asofe
(21632,4133,1,1,2,390),-- Recipe: Zubei\'s Breastplate
(21632,4941,1,1,2,623),-- Recipe: Zubei\'s Gaiters  (60%)
(21632,4959,1,1,2,1560),-- Recipe: Avadon Boots (60%)
(21633,1918,1,1,-1,107551),-- Nassen\'s Earring Gemstone
(21633,2955,1,1,-1,157174),-- Theca Leather Gloves Texture
(21633,4042,1,1,-1,41389),-- Enria
(21633,57,1003,2037,0,700000),-- Adena
(21633,400,1,1,1,38),-- Theca Leather Armor
(21633,401,1,1,1,20),-- Drake Leather Armor
(21633,420,1,1,1,60),-- Theca Leather Gaiters
(21633,1984,1,1,1,3276),-- Theca Leather Armor Pattern
(21633,1985,1,1,1,5270),-- Theca Leather Gaiters Pattern
(21633,1987,1,1,1,2332),-- Drake Leather Armor Texture
(21633,1865,1,1,2,114616),-- Varnish
(21633,1870,1,1,2,114616),-- Coal
(21633,1874,1,1,2,7641),-- Oriharukon Ore
(21633,4039,1,1,2,3821),-- Mold Glue
(21633,4043,1,1,2,3821),-- Asofe
(21633,4133,1,1,2,552),-- Recipe: Zubei\'s Breastplate
(21633,4941,1,1,2,882),-- Recipe: Zubei\'s Gaiters  (60%)
(21633,4959,1,1,2,2209),-- Recipe: Avadon Boots (60%)
(21634,1806,1,1,-1,9066),-- Recipe: Soulshot: B-Grade
(21634,1918,1,1,-1,196309),-- Nassen\'s Earring Gemstone
(21634,4039,1,1,-1,151093),-- Mold Glue
(21634,57,2230,4448,0,700000),-- Adena
(21634,135,1,1,1,14),-- Samurai Longsword
(21634,2115,1,1,1,1918),-- Samurai Longsword Blade
(21634,5286,1,1,1,14),-- Berserker Blade
(21634,5530,1,1,1,1920),-- Berserker Blade Edge
(21634,1806,1,1,2,558),-- Recipe: Soulshot: B-Grade
(21634,1894,1,1,2,9788),-- Crafted Leather
(21634,4039,1,1,2,9299),-- Mold Glue
(21634,4041,1,1,2,2426),-- Mold Hardener
(21634,4044,1,1,2,9299),-- Thons
(21634,4134,1,1,2,2146),-- Recipe: Zubei\'s Gaiters
(21634,8726,1,1,200,6667),-- Life Stone: level 55
(21635,1806,1,1,-1,3352),-- Recipe: Soulshot: B-Grade
(21635,1985,1,1,-1,57799),-- Theca Leather Gaiters Pattern
(21635,4937,1,1,-1,27660),-- Recipe: Adamantite Earrings (70%)
(21635,57,675,1375,0,700000),-- Adena
(21635,2436,1,1,1,33),-- Theca Leather Boots
(21635,2437,1,1,1,25),-- Drake Leather Boots
(21635,2438,1,1,1,25),-- Full Plate Boots
(21635,2460,1,1,1,33),-- Theca Leather Gloves
(21635,2461,1,1,1,25),-- Drake Leather Gloves
(21635,2462,1,1,1,25),-- Full Plate Gauntlets
(21635,2463,1,1,1,25),-- Divine Gloves
(21635,2954,1,1,1,2799),-- Theca Leather Boots Texture
(21635,2955,1,1,1,2799),-- Theca Leather Gloves Texture
(21635,2960,1,1,1,2408),-- Drake Leather Boots Design
(21635,2961,1,1,1,2408),-- Full Plate Boots Part
(21635,2962,1,1,1,2408),-- Drake Leather Gloves Design
(21635,2963,1,1,1,2408),-- Full Plate Gauntlets Part
(21635,2964,1,1,1,2408),-- Divine Gloves Pattern
(21635,1864,1,1,2,154723),-- Stem
(21635,1869,1,1,2,77362),-- Iron Ore
(21635,1878,1,1,2,30945),-- Braided Hemp
(21635,1884,1,1,2,47608),-- Cord
(21635,4039,1,1,2,2579),-- Mold Glue
(21635,4178,1,1,2,1420),-- Recipe: Zubei\'s Shield (100%)
(21635,4184,1,1,2,90),-- Recipe: Sprite\'s Staff
(21635,5160,1,1,2,155),-- Recipe: Spiritshot (B) Compressed Package (100%)
(21636,1806,1,1,-1,4747),-- Recipe: Soulshot: B-Grade
(21636,1985,1,1,-1,81846),-- Theca Leather Gaiters Pattern
(21636,4937,1,1,-1,39167),-- Recipe: Adamantite Earrings (70%)
(21636,57,956,1948,0,700000),-- Adena
(21636,2436,1,1,1,46),-- Theca Leather Boots
(21636,2437,1,1,1,35),-- Drake Leather Boots
(21636,2438,1,1,1,35),-- Full Plate Boots
(21636,2460,1,1,1,46),-- Theca Leather Gloves
(21636,2461,1,1,1,35),-- Drake Leather Gloves
(21636,2462,1,1,1,35),-- Full Plate Gauntlets
(21636,2463,1,1,1,35),-- Divine Gloves
(21636,2954,1,1,1,3963),-- Theca Leather Boots Texture
(21636,2955,1,1,1,3963),-- Theca Leather Gloves Texture
(21636,2960,1,1,1,3410),-- Drake Leather Boots Design
(21636,2961,1,1,1,3410),-- Full Plate Boots Part
(21636,2962,1,1,1,3410),-- Drake Leather Gloves Design
(21636,2963,1,1,1,3410),-- Full Plate Gauntlets Part
(21636,2964,1,1,1,3410),-- Divine Gloves Pattern
(21636,1864,1,1,2,219096),-- Stem
(21636,1869,1,1,2,109548),-- Iron Ore
(21636,1878,1,1,2,43820),-- Braided Hemp
(21636,1884,1,1,2,67415),-- Cord
(21636,4039,1,1,2,3652),-- Mold Glue
(21636,4178,1,1,2,2011),-- Recipe: Zubei\'s Shield (100%)
(21636,4184,1,1,2,127),-- Recipe: Sprite\'s Staff
(21636,5160,1,1,2,219),-- Recipe: Spiritshot (B) Compressed Package (100%)
(21637,2115,1,1,-1,19848),-- Samurai Longsword Blade
(21637,2116,1,1,-1,20383),-- Deadman\'s Staff Head
(21637,4141,1,1,-1,20829),-- Recipe: Avadon Breastplate
(21637,57,1746,3555,0,700000),-- Adena
(21637,135,1,1,1,14),-- Samurai Longsword
(21637,2115,1,1,1,1833),-- Samurai Longsword Blade
(21637,5286,1,1,1,14),-- Berserker Blade
(21637,5530,1,1,1,1835),-- Berserker Blade Edge
(21637,1864,4,12,2,57130),-- Stem
(21637,1865,2,6,2,45704),-- Varnish
(21637,1866,3,9,2,25392),-- Suede
(21637,1868,1,3,2,152347),-- Thread
(21637,1873,1,1,2,91408),-- Silver Nugget
(21637,4142,1,1,2,1758),-- Recipe: Avadon Gaiters
(21637,4936,1,1,2,4194),-- Recipe: Avadon Shield (60%)
(21637,8726,1,1,200,2849),-- Life Stone: level 55
(21638,1900,1,2,-1,41596),-- Enchanted Necklace Chain
(21638,1935,1,1,-1,21240),-- Scale Mail Fragment
(21638,2203,1,1,-1,82830),-- Recipe: Mithril Boots
(21638,57,193,384,0,700000),-- Adena
(21638,316,1,1,1,14),-- Blood of Saints
(21638,317,1,1,1,14),-- Tome of Blood
(21638,2030,1,1,1,789),-- Tome of Blood Page
(21638,1874,1,1,2,1661),-- Oriharukon Ore
(21638,1878,1,1,2,10000),-- Braided Hemp
(21638,1884,1,1,2,23256),-- Cord
(21638,1889,1,1,2,3322),-- Compound Braid
(21638,2153,1,1,2,5814),-- Recipe: Tiger\'s Eye Earring
(21638,2977,1,1,2,1661),-- Recipe: Brigandine Boots
(21638,5220,1,1,2,4975),-- Metal Hardener
(21639,2021,1,1,-1,5670),-- Spinebone Sword Blade
(21639,2153,1,1,-1,69838),-- Recipe: Tiger\'s Eye Earring
(21639,2184,1,1,-1,47170),-- Recipe: Mithril Banded Gaiters
(21639,57,227,455,0,700000),-- Adena
(21639,83,1,1,1,12),-- Sword of Magic
(21639,124,1,1,1,9),-- Two-Handed Sword
(21639,127,1,1,1,9),-- Crimson Sword
(21639,130,1,1,1,9),-- Elven Sword
(21639,144,1,1,1,12),-- Sword of Occult
(21639,2026,1,1,1,480),-- Two-Handed Sword Edge
(21639,1875,1,1,2,3106),-- Stone of Purity
(21639,1879,1,1,2,7463),-- Cokes
(21639,1885,1,1,2,5587),-- High Grade Suede
(21639,2250,1,1,2,12987),-- Recipe: Bone Arrow
(21639,5549,1,1,2,3311),-- Metallic Thread
(21639,5550,1,1,2,207),-- Durable Metal Plate
(21640,6036,1,1,-1,15233),-- Greater Magic Haste Potion
(21640,57,155,308,0,700000),-- Adena
(21640,1825,1,1,2,22727),-- Recipe: Oriharukon
(21640,6667,1,1,2,200000),-- Deluxe Chest Key - Grade 3
(21641,1956,1,1,-1,66115),-- Square Shield Fragment
(21641,2154,1,1,-1,62363),-- Recipe: Elven Earring
(21641,2302,1,1,-1,2544),-- Recipe: Silver Axe
(21641,57,296,590,0,700000),-- Adena
(21641,88,1,1,1,5),-- Morning Star
(21641,90,1,1,1,5),-- Goat Head Staff
(21641,158,1,1,1,5),-- Tarbar
(21641,169,1,1,1,5),-- Skull Breaker
(21641,172,1,1,1,5),-- Heavy Bone Club
(21641,2031,1,1,1,305),-- Morning Star Head
(21641,2032,1,1,1,279),-- Goat Head Staff Head
(21641,2035,1,1,1,305),-- Tarbar Head
(21641,2036,1,1,1,305),-- Skull Breaker Head
(21641,2037,1,1,1,305),-- Heavy Bone Club Head
(21641,1341,10,30,2,142857),-- Bone Arrow
(21641,1874,1,1,2,2890),-- Oriharukon Ore
(21641,1878,1,1,2,19231),-- Braided Hemp
(21641,1884,1,1,2,27027),-- Cord
(21641,1889,1,1,2,3333),-- Compound Braid
(21641,2190,1,1,2,1779),-- Recipe: Mithril Stockings
(21641,2279,1,1,2,269),-- Recipe: War Pick
(21641,5220,1,1,2,1558),-- Metal Hardener
(21642,1868,1,3,-1,534538),-- Thread
(21642,1904,1,1,-1,49887),-- Elven Necklace Beads
(21642,2145,1,1,-1,38181),-- Recipe: Mithril Alloy
(21642,57,271,541,0,700000),-- Adena
(21642,279,1,1,1,20),-- Strengthened Long Bow
(21642,2040,1,1,1,1599),-- Strengthened Long Bow Shaft
(21642,1868,1,1,2,100000),-- Thread
(21642,1873,1,1,2,22222),-- Silver Nugget
(21642,1876,1,1,2,10309),-- Mithril Ore
(21642,1895,1,1,2,18182),-- Metallic Fiber
(21642,2146,1,1,2,6211),-- Recipe: Artisan\'s frame
(21642,2285,1,1,2,256),-- Recipe: Claymore
(21643,1868,1,3,-1,679213),-- Thread
(21643,1904,1,1,-1,63389),-- Elven Necklace Beads
(21643,2145,1,1,-1,48515),-- Recipe: Mithril Alloy
(21643,57,345,686,0,700000),-- Adena
(21643,279,1,1,1,26),-- Strengthened Long Bow
(21643,2040,1,1,1,2031),-- Strengthened Long Bow Shaft
(21643,1868,1,1,2,142857),-- Thread
(21643,1873,1,1,2,38462),-- Silver Nugget
(21643,1876,1,1,2,13158),-- Mithril Ore
(21643,1895,1,1,2,22727),-- Metallic Fiber
(21643,2146,1,1,2,3759),-- Recipe: Artisan\'s frame
(21643,2285,1,1,2,326),-- Recipe: Claymore
(21644,2026,1,1,-1,3925),-- Two-Handed Sword Edge
(21644,2155,1,1,-1,52351),-- Recipe: Elven Ring
(21644,5550,1,1,-1,4349),-- Durable Metal Plate
(21644,57,166,330,0,700000),-- Adena
(21644,395,1,1,1,38),-- Manticore Skin Shirt
(21644,417,1,1,1,61),-- Manticore Skin Gaiters
(21644,1942,1,1,1,2150),-- Manticore Skin Shirt Texture
(21644,1943,1,1,1,3348),-- Manticore Skin Gaiters Pattern
(21644,1875,1,1,2,2198),-- Stone of Purity
(21644,1879,1,1,2,4878),-- Cokes
(21644,1885,1,1,2,786),-- High Grade Suede
(21644,2278,1,1,2,157),-- Recipe: Strengthened Long Bow
(21644,2286,1,1,2,157),-- Recipe: Bonebreaker
(21644,2977,1,1,2,2358),-- Recipe: Brigandine Boots
(21644,5549,1,1,2,1414),-- Metallic Thread
(21644,5550,1,1,2,281),-- Durable Metal Plate
(21645,1870,1,2,-1,597803),-- Coal
(21645,2029,1,1,-1,7131),-- Gastraphetes Shaft
(21645,5549,1,1,-1,59780),-- Metallic Thread
(21645,57,301,600,0,700000),-- Adena
(21645,850,1,1,1,111),-- Elven Earring
(21645,881,1,1,1,166),-- Elven Ring
(21645,913,1,1,1,83),-- Elven Necklace
(21645,1902,1,1,1,5219),-- Elven Earring Beads
(21645,1903,1,1,1,8592),-- Elven Ring Piece
(21645,1904,1,1,1,4807),-- Elven Necklace Beads
(21645,1341,10,30,2,142857),-- Bone Arrow
(21645,1832,1,1,2,45455),-- Greater Antidote
(21645,1875,1,1,2,3788),-- Stone of Purity
(21645,1879,1,1,2,6803),-- Cokes
(21645,1885,1,1,2,4237),-- High Grade Suede
(21645,2188,1,1,2,2020),-- Recipe: Manticore Skin Gaiters
(21645,5473,1,1,2,2174),-- Recipe: Durable Metal Plate(100%)
(21645,5549,1,1,2,3937),-- Metallic Thread
(21645,5550,1,1,2,1212),-- Durable Metal Plate
(21646,730,1,1,-1,2931),-- Scroll: Enchant Armor (Grade A)
(21646,1894,1,1,-1,123396),-- Crafted Leather
(21646,5508,1,1,-1,82457),-- Sealed Dark Crystal Gloves Design
(21646,57,1701,3463,0,700000),-- Adena
(21646,12226,1,1,1,453),-- Common Item - Sealed Dark Crystal Helmet
(21646,12225,1,1,1,793),-- Common Item - Sealed Dark Crystal Shield
(21646,12234,1,1,1,974),-- Common Item - Sealed Tallum Helmet
(21646,5494,1,1,1,3636),-- Sealed Dark Crystal Shield Fragment
(21646,5525,1,1,1,2677),-- Sealed Dark Crystal Helmet Design
(21646,5526,1,1,1,2677),-- Sealed Tallum Helm Design
(21646,1344,10,30,2,142857),-- Mithril Arrow
(21646,1867,1,1,2,166667),-- Animal Skin
(21646,1872,1,1,2,166667),-- Animal Bone
(21646,1882,1,1,2,25000),-- Leather
(21646,1894,1,1,2,4785),-- Crafted Leather
(21646,4042,1,1,2,1675),-- Enria
(21646,5277,1,1,2,91),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21646,8731,1,1,200,679),-- Life Stone: level 70
(21646,8741,1,1,200,385),-- Mid-Grade Life Stone: level 70
(21647,4044,1,1,-1,98893),-- Thons
(21647,5462,1,1,-1,1099),-- Recipe: Branch of The Mother Tree(60%)
(21647,5509,1,1,-1,69561),-- Sealed Tallum Gloves Design
(21647,57,1435,2921,0,700000),-- Adena
(21647,12227,1,1,1,127),-- Common Item - Sealed Dark Crystal breastplate
(21647,12220,1,1,1,652),-- Common Item - Sealed Dark Crystal Gaiters
(21647,12233,1,1,1,16),-- Common Item - Sealed Tallum Plate Armor
(21647,5520,1,1,1,942),-- Sealed Dark Crystal Breastplate Pattern
(21647,5521,1,1,1,665),-- Sealed Tallum Plate Armor Pattern
(21647,5524,1,1,1,1440),-- Sealed Dark Crystal Gaiters Pattern
(21647,1868,1,1,2,200000),-- Thread
(21647,1873,1,1,2,43478),-- Silver Nugget
(21647,1889,1,1,2,7752),-- Compound Braid
(21647,4044,1,1,2,3534),-- Thons
(21647,5282,1,1,2,16),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21647,5549,1,1,2,10101),-- Metallic Thread
(21647,8731,1,1,200,1114),-- Life Stone: level 70
(21647,8741,1,1,200,238),-- Mid-Grade Life Stone: level 70
(21648,5464,1,1,-1,1200),-- Recipe: Dark Legion\'s Edge(60%)
(21648,5482,1,1,-1,42777),-- Sealed Dark Crystalline Leggings Design
(21648,5485,1,1,-1,27253),-- Sealed Tallum Tunic Texture
(21648,57,1566,3192,0,700000),-- Adena
(21648,12226,1,1,1,829),-- Common Item - Sealed Dark Crystal Helmet
(21648,12225,1,1,1,926),-- Common Item - Sealed Dark Crystal Shield
(21648,12234,1,1,1,569),-- Common Item - Sealed Tallum Helmet
(21648,5494,1,1,1,3351),-- Sealed Dark Crystal Shield Fragment
(21648,5525,1,1,1,2467),-- Sealed Dark Crystal Helmet Design
(21648,5526,1,1,1,2467),-- Sealed Tallum Helm Design
(21648,1344,40,120,2,33333),-- Mithril Arrow
(21648,1867,1,3,2,47619),-- Animal Skin
(21648,1869,1,1,2,111111),-- Iron Ore
(21648,1870,1,1,2,111111),-- Coal
(21648,1871,1,1,2,100000),-- Charcoal
(21648,1872,1,3,2,71429),-- Animal Bone
(21648,5167,1,1,2,65),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21648,8731,1,1,200,682),-- Life Stone: level 70
(21648,8741,1,1,200,455),-- Mid-Grade Life Stone: level 70
(21648,8751,1,1,200,14),-- High-Grade Life Stone: level 70
(21649,729,1,1,-1,426),-- Scroll: Enchant Weapon (Grade A)
(21649,1894,1,1,-1,134512),-- Crafted Leather
(21649,4044,1,1,-1,127787),-- Thons
(21649,57,1853,3776,0,700000),-- Adena
(21649,12227,1,1,1,63),-- Common Item - Sealed Dark Crystal breastplate
(21649,12220,1,1,1,313),-- Common Item - Sealed Dark Crystal Gaiters
(21649,12233,1,1,1,94),-- Common Item - Sealed Tallum Plate Armor
(21649,5520,1,1,1,1218),-- Sealed Dark Crystal Breastplate Pattern
(21649,5521,1,1,1,859),-- Sealed Tallum Plate Armor Pattern
(21649,5524,1,1,1,1861),-- Sealed Dark Crystal Gaiters Pattern
(21649,1894,1,1,2,5236),-- Crafted Leather
(21649,4039,1,1,2,4484),-- Mold Glue
(21649,4041,1,1,2,1047),-- Mold Hardener
(21649,4044,1,1,2,5102),-- Thons
(21649,5272,1,1,2,101),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21649,8731,1,1,200,1815),-- Life Stone: level 70
(21649,8741,1,1,200,313),-- Mid-Grade Life Stone: level 70
(21649,8751,1,1,200,63),-- High-Grade Life Stone: level 70
(21650,5162,1,1,-1,1562),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21650,5167,1,1,-1,1562),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21650,5514,1,1,-1,59368),-- Sealed Gloves of Nightmare Design
(21650,57,1698,3464,0,700000),-- Adena
(21650,11951,1,1,1,143),-- Common Item - Bloody Orchid
(21650,5536,1,1,1,1767),-- Bloody Orchid Head
(21650,1344,10,30,2,166667),-- Mithril Arrow
(21650,1895,1,1,2,38462),-- Metallic Fiber
(21650,3957,1,1,2,36),-- Recipe: Blessed Spiritshot S
(21650,4040,1,1,2,2304),-- Mold Lubricant
(21650,4042,1,1,2,2433),-- Enria
(21650,4043,1,1,2,4739),-- Asofe
(21650,8731,1,1,200,1001),-- Life Stone: level 70
(21650,8741,1,1,200,268),-- Mid-Grade Life Stone: level 70
(21651,1889,1,1,-1,223871),-- Compound Braid
(21651,4044,1,1,-1,111935),-- Thons
(21651,5515,1,1,-1,56714),-- Sealed Majestic Gloves Design
(21651,57,1622,3309,0,700000),-- Adena
(21651,11946,1,1,1,20),-- Common Item - Dasparion\'s Staff
(21651,11949,1,1,1,20),-- Common Item - Meteor Shower
(21651,8346,1,1,1,677),-- Hammer Piece of Destroyer
(21651,11958,1,1,1,20),-- Common Item - Destroyer Hammer
(21651,8341,1,1,1,351),-- Spiritual Eye Piece
(21651,11953,1,1,1,20),-- Common Item - Spiritual Eye
(21651,5532,1,1,1,836),-- Meteor Shower Head
(21651,5543,1,1,1,844),-- Dasparion\'s Staff Edge
(21651,1868,1,1,2,250000),-- Thread
(21651,1873,1,1,2,50000),-- Silver Nugget
(21651,1889,1,1,2,10000),-- Compound Braid
(21651,4044,1,1,2,3663),-- Thons
(21651,5157,1,1,2,74),-- Recipe: Soulshot (S) Compressed Package (100%)
(21651,5549,1,1,2,10417),-- Metallic Thread
(21651,8731,1,1,200,880),-- Life Stone: level 70
(21651,8741,1,1,200,123),-- Mid-Grade Life Stone: level 70
(21652,5272,1,1,-1,6225),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21652,5277,1,1,-1,6225),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21652,5545,1,1,-1,38425),-- Dark Legion\'s Edge Blade
(21652,57,5636,11501,0,700000),-- Adena
(21652,11976,1,1,1,238),-- Common Item - Tallum Glaive
(21652,5541,1,1,1,5673),-- Tallum Glaive Edge
(21652,959,1,1,2,28),-- Scroll: Enchant Weapon (Grade S)
(21652,1867,4,12,2,71429),-- Animal Skin
(21652,1872,1,3,2,200000),-- Animal Bone
(21652,1882,1,1,2,111111),-- Leather
(21652,1894,1,1,2,19608),-- Crafted Leather
(21652,4042,1,1,2,8065),-- Enria
(21652,8732,1,1,200,4292),-- Life Stone: level 76
(21652,8742,1,1,200,1190),-- Mid-Grade Life Stone: level 76
(21653,5157,1,1,-1,5300),-- Recipe: Soulshot (S) Compressed Package (100%)
(21653,5544,1,1,-1,32679),-- Branch of The Mother Tree Head
(21653,5545,1,1,-1,32718),-- Dark Legion\'s Edge Blade
(21653,57,4798,9794,0,700000),-- Adena
(21653,11967,1,1,1,527),-- Common Item - Dragon Grinder
(21653,5538,1,1,1,4832),-- Dragon Grinder Edge
(21653,960,1,1,2,235),-- Scroll: Enchant Armor (Grade S)
(21653,1344,20,60,2,250000),-- Mithril Arrow
(21653,1895,1,1,2,125000),-- Metallic Fiber
(21653,4040,1,1,2,9615),-- Mold Lubricant
(21653,4042,1,1,2,7692),-- Enria
(21653,4043,1,1,2,15385),-- Asofe
(21653,8732,1,1,200,3861),-- Life Stone: level 76
(21653,8742,1,1,200,98),-- Mid-Grade Life Stone: level 76
(21654,4042,1,1,-1,219733),-- Enria
(21654,5167,1,1,-1,5860),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21654,5488,1,1,-1,59369),-- Sealed Majestic Robe Fabric
(21654,57,5293,10839,0,700000),-- Adena
(21654,11968,1,1,1,112),-- Common Item - Dragon Slayer
(21654,11971,1,1,1,168),-- Common Item - Sword of Miracles 
(21654,11964,1,1,1,112),-- Common Item - Dark Legion\'s Edge
(21654,5529,1,1,1,1782),-- Dragon Slayer Edge
(21654,5545,1,1,1,1782),-- Dark Legion\'s Edge Blade
(21654,5546,1,1,1,1957),-- Sword of Miracles Edge
(21654,960,1,1,2,140),-- Scroll: Enchant Armor (Grade S)
(21654,1344,40,120,2,125000),-- Mithril Arrow
(21654,1867,5,15,2,52632),-- Animal Skin
(21654,1872,1,3,2,166667),-- Animal Bone
(21654,1882,1,1,2,90909),-- Leather
(21654,1894,1,1,2,14925),-- Crafted Leather
(21654,4042,1,1,2,72463),-- Enria
(21654,8732,1,1,200,3195),-- Life Stone: level 76
(21654,8742,1,1,200,982),-- Mid-Grade Life Stone: level 76
(21654,8752,1,1,200,112),-- High-Grade Life Stone: level 76
(21655,6344,1,1,-1,184250),-- Sealed Majestic Necklace Beads
(21655,6346,1,1,-1,409201),-- Sealed Majestic Ring Gemstone
(21655,6901,1,1,-1,12569),-- Recipe: Shining Arrow (100%)
(21655,57,5046,10335,0,700000),-- Adena
(21655,12263,1,1,1,19),-- Common Item - Sealed Armor of Nightmare (no drop info)
(21655,12262,1,1,1,19),-- Common Item - Sealed Majestic Plate Armor
(21655,5522,1,1,1,3161),-- Sealed Armor of Nightmare Pattern
(21655,5523,1,1,1,3161),-- Sealed Majestic Plate Armor Pattern
(21655,960,1,1,2,213),-- Scroll: Enchant Armor (Grade S)
(21655,1344,100,300,2,33333),-- Mithril Arrow
(21655,1865,1,3,2,125000),-- Varnish
(21655,1870,2,6,2,111111),-- Coal
(21655,1875,1,1,2,21739),-- Stone of Purity
(21655,1879,1,1,2,66667),-- Cokes
(21655,4040,1,1,2,9259),-- Mold Lubricant
(21655,8732,1,1,200,2532),-- Life Stone: level 76
(21655,8742,1,1,200,593),-- Mid-Grade Life Stone: level 76
(21656,6704,1,1,-1,268758),-- Sealed Imperial Crusader Boots Design
(21656,6705,1,1,-1,263248),-- Sealed Imperial Crusader Shield Part
(21656,6706,1,1,-1,306208),-- Sealed Imperial Crusader Helmet Pattern
(21656,57,7595,15500,0,700000),-- Adena
(21656,12269,1,1,1,834),-- Common Item - Sealed Helm of Nightmare
(21656,12268,1,1,1,834),-- Common Item - Sealed Shield of Nightmare
(21656,12261,1,1,1,834),-- Common Item - Sealed Majestic Circlet
(21656,5495,1,1,1,14111),-- Sealed Shield of Nightmare Fragment
(21656,5527,1,1,1,11111),-- Sealed Helm of Nightmare Design
(21656,5528,1,1,1,11111),-- Sealed Majestic Circlet Design
(21656,960,1,1,2,319),-- Scroll: Enchant Armor (Grade S)
(21656,1344,40,120,2,166667),-- Mithril Arrow
(21656,1867,9,27,2,45455),-- Animal Skin
(21656,1872,5,15,2,66667),-- Animal Bone
(21656,1875,1,1,2,45455),-- Stone of Purity
(21656,1881,1,1,2,71429),-- Coarse Bone Powder
(21656,4041,1,1,2,2506),-- Mold Hardener
(21656,8732,1,1,200,3759),-- Life Stone: level 76
(21656,8742,1,1,200,1252),-- Mid-Grade Life Stone: level 76
(21656,8752,1,1,200,1252),-- High-Grade Life Stone: level 76
(21657,6688,1,1,-1,38019),-- Forgotten Blade Edge
(21657,6689,1,1,-1,37303),-- Basalt Battlehammer Head
(21657,6690,1,1,-1,38092),-- Imperial Staff Head
(21657,57,7076,14446,0,700000),-- Adena
(21657,12265,1,1,1,886),-- Common Item - Sealed Leather Armor of Nightmare
(21657,12258,1,1,1,443),-- Common Item - Sealed Majestic Leather Armor
(21657,5480,1,1,1,5830),-- Sealed Leather Armor of Nightmare Fabric
(21657,5481,1,1,1,5830),-- Sealed Majestic Leather Armor Fabric
(21657,960,1,1,2,297),-- Scroll: Enchant Armor (Grade S)
(21657,1344,100,300,2,588249),-- Mithril Arrow
(21657,1865,9,27,2,28571),-- Varnish
(21657,1870,2,6,2,125000),-- Coal
(21657,1875,1,1,2,35714),-- Stone of Purity
(21657,1879,1,1,2,90909),-- Cokes
(21657,4040,1,1,2,9709),-- Mold Lubricant
(21657,8732,1,1,200,8000),-- Life Stone: level 76
(21657,8742,1,1,200,886),-- Mid-Grade Life Stone: level 76
(21658,1876,1,1,-1,594082),-- Mithril Ore
(21658,4052,1,2,-1,105166),-- Earring of Black Ore Piece
(21658,57,1443,2919,0,700000),-- Adena
(21658,11930,1,1,1,14),-- Common Item - Sword of Damascus
(21658,11943,1,1,1,14),-- Common Item - Wizard\'s Tear
(21658,11929,1,1,1,14),-- Common Item - Guardian Sword
(21658,8336,1,1,1,634),-- Blade of the Wizard\'s Tear
(21658,8330,1,1,1,714),-- Guardian Sword Blade
(21658,4114,1,1,1,1921),-- Sword of Damascus Blade
(21658,1343,10,30,2,125000),-- Silver Arrow
(21658,1807,1,1,2,165),-- Recipe: Soulshot: A-Grade
(21658,5392,1,1,2,661),-- Recipe: Sealed Dark Crystal Gloves(60%)
(21658,1876,1,1,2,17857),-- Mithril Ore
(21658,1877,1,1,2,3311),-- Adamantite Nugget
(21658,1880,1,1,2,8621),-- Steel
(21658,1881,1,1,2,11111),-- Coarse Bone Powder
(21658,1882,1,1,2,20408),-- Leather
(21658,8729,1,1,200,909),-- Life Stone: level 64
(21658,8739,1,1,200,165),-- Mid-Grade Life Stone: level 64
(21658,8759,1,1,200,41),-- Top-Grade Life Stone: level 64
(21659,1061,1,1,-1,770744),-- Healing Potion
(21659,1869,1,3,-1,635864),-- Iron Ore
(21659,1900,1,1,-1,139139),-- Enchanted Necklace Chain
(21659,57,536,1073,0,700000),-- Adena
(21659,46,1,1,1,1390),-- Bronze Helmet
(21659,627,1,1,1,1986),-- Aspis
(21659,737,1,1,2,11583),-- Scroll of Resurrection
(21659,1864,1,1,2,231650),-- Stem
(21659,1869,1,1,2,115825),-- Iron Ore
(21659,1880,1,1,2,11583),-- Steel
(21659,5550,1,1,2,1545),-- Durable Metal Plate
(21659,6387,1,1,2,4633),-- Blessed Scroll of Resurrection for Pets
(21660,1870,1,3,-1,887208),-- Coal
(21660,1940,1,1,-1,264838),-- Iron Boots Design
(21660,2190,1,1,-1,126744),-- Recipe: Mithril Stockings
(21660,57,749,1495,0,700000),-- Adena
(21660,63,1,1,1,531),-- Gauntlets
(21660,553,1,1,1,531),-- Iron Boots
(21660,1940,1,1,1,20116),-- Iron Boots Design
(21660,2422,1,1,1,531),-- Reinforced Leather Boots
(21660,2423,1,1,1,531),-- Boots of Knowledge
(21660,2446,1,1,1,531),-- Reinforced Leather Gloves
(21660,2447,1,1,1,531),-- Gloves of Knowledge
(21660,1865,1,1,2,235862),-- Varnish
(21660,1870,1,3,2,117931),-- Coal
(21660,1874,1,1,2,15725),-- Oriharukon Ore
(21660,2192,1,1,2,30085),-- Recipe: Kite Shield
(21661,1875,1,1,-1,91332),-- Stone of Purity
(21661,1879,1,1,-1,228329),-- Cokes
(21661,1901,1,1,-1,189223),-- Tiger\'s Eye Earring Stone
(21661,57,579,1158,0,700000),-- Adena
(21661,278,1,1,1,76),-- Gastraphetes
(21661,2029,1,1,1,4345),-- Gastraphetes Shaft
(21661,1875,1,1,2,8094),-- Stone of Purity
(21661,1879,1,1,2,20234),-- Cokes
(21661,1885,1,1,2,10117),-- High Grade Suede
(21661,2154,1,1,2,12998),-- Recipe: Elven Earring
(21661,5549,1,1,2,12141),-- Metallic Thread
(21661,5550,1,1,2,1619),-- Durable Metal Plate
(21662,1876,1,1,-1,304870),-- Mithril Ore
(21662,2026,1,1,-1,18347),-- Two-Handed Sword Edge
(21662,2191,1,1,-1,204063),-- Recipe: RIP Gauntlets
(21662,57,644,1285,0,700000),-- Adena
(21662,849,1,1,1,631),-- Tiger\'s Eye Earring
(21662,880,1,1,1,947),-- Black Pearl Ring
(21662,912,1,1,1,473),-- Near Forest Necklace
(21662,1901,1,1,1,27987),-- Tiger\'s Eye Earring Stone
(21662,1876,1,1,2,27017),-- Mithril Ore
(21662,1877,1,1,2,5404),-- Adamantite Nugget
(21662,1880,1,1,2,13509),-- Steel
(21662,1881,1,1,2,18011),-- Coarse Bone Powder
(21662,1882,1,1,2,30018),-- Leather
(21662,2193,1,1,2,18084),-- Recipe: Boots of Power
(21663,1938,1,1,-1,100705),-- Mithril Banded Mail Material
(21663,1939,1,1,-1,131576),-- Mithril Banded Gaiters Material
(21663,2154,1,1,-1,160385),-- Recipe: Elven Earring
(21663,57,633,1266,0,700000),-- Adena
(21663,849,1,1,1,620),-- Tiger\'s Eye Earring
(21663,880,1,1,1,931),-- Black Pearl Ring
(21663,912,1,1,1,465),-- Near Forest Necklace
(21663,1901,1,1,1,27502),-- Tiger\'s Eye Earring Stone
(21663,1874,1,1,2,8850),-- Oriharukon Ore
(21663,1878,1,1,2,53098),-- Braided Hemp
(21663,1884,1,1,2,81689),-- Cord
(21663,1889,1,1,2,8850),-- Compound Braid
(21663,2155,1,1,2,21308),-- Recipe: Elven Ring
(21663,5220,1,1,2,5310),-- Metal Hardener
(21664,1933,1,1,-1,96826),-- Dark Stocking Fabric
(21664,2153,1,1,-1,127997),-- Recipe: Tiger\'s Eye Earring
(21664,2183,1,1,-1,54146),-- Recipe: Mithril Banded Mail
(21664,57,347,694,0,700000),-- Adena
(21664,848,1,1,1,528),-- Enchanted Earring
(21664,879,1,1,1,792),-- Enchanted Ring
(21664,911,1,1,1,396),-- Enchanted Necklace
(21664,1900,1,1,1,12302),-- Enchanted Necklace Chain
(21664,955,1,1,2,360),-- Scroll: Enchant Weapon (Grade D)
(21664,1864,1,1,2,179900),-- Stem
(21664,1869,1,1,2,89950),-- Iron Ore
(21664,1880,1,1,2,8996),-- Steel
(21664,5550,1,1,2,1200),-- Durable Metal Plate
(21665,2027,1,1,-1,17685),-- Spiked Club Head
(21665,2028,1,1,-1,16040),-- Shilen Knife Edge
(21665,2192,1,1,-1,172191),-- Recipe: Kite Shield
(21665,57,570,1138,0,700000),-- Adena
(21665,436,1,1,1,473),-- Tunic of Knowledge
(21665,469,1,1,1,754),-- Stockings of Knowledge
(21665,1341,10,30,2,341801),-- Bone Arrow
(21665,1875,1,1,2,6836),-- Stone of Purity
(21665,1879,1,1,2,17090),-- Cokes
(21665,1885,1,1,2,8546),-- High Grade Suede
(21665,2269,1,1,2,733),-- Recipe: Morning Star
(21665,5549,1,1,2,10255),-- Metallic Thread
(21665,5550,1,1,2,1368),-- Durable Metal Plate
(21666,955,1,1,-1,4130),-- Scroll: Enchant Weapon (Grade D)
(21666,1872,1,3,-1,688304),-- Animal Bone
(21666,1935,1,1,-1,57679),-- Scale Mail Fragment
(21666,57,436,871,0,700000),-- Adena
(21666,69,1,1,1,36),-- Bastard Sword
(21666,125,1,1,1,36),-- Spinebone Sword
(21666,126,1,1,1,36),-- Artisan\'s Sword
(21666,128,1,1,1,36),-- Knight\'s Sword
(21666,2021,1,1,1,1425),-- Spinebone Sword Blade
(21666,956,1,1,2,3762),-- Scroll: Enchant Armor (Grade D)
(21666,1867,1,1,2,150453),-- Animal Skin
(21666,1872,1,1,2,150453),-- Animal Bone
(21666,1875,1,1,2,7523),-- Stone of Purity
(21666,1881,1,1,2,15046),-- Coarse Bone Powder
(21667,1874,1,1,-1,80757),-- Oriharukon Ore
(21667,1940,1,1,-1,180800),-- Iron Boots Design
(21667,2155,1,1,-1,194440),-- Recipe: Elven Ring
(21667,57,512,1024,0,700000),-- Adena
(21667,47,1,1,1,846),-- Helmet
(21667,628,1,1,1,1209),-- Hoplon
(21667,1874,1,1,2,7157),-- Oriharukon Ore
(21667,1878,1,1,2,42939),-- Braided Hemp
(21667,1884,1,1,2,66060),-- Cord
(21667,1889,1,1,2,7157),-- Compound Braid
(21667,2156,1,1,2,8588),-- Recipe: Elven Necklace
(21667,5220,1,1,2,4294),-- Metal Hardener
(21668,1832,1,3,-1,603864),-- Greater Antidote
(21668,1864,1,5,-1,724636),-- Stem
(21668,1936,1,1,-1,88949),-- White Tunic Pattern
(21668,57,459,917,0,700000),-- Adena
(21668,348,1,1,1,224),-- Scale Mail
(21668,377,1,1,1,357),-- Scale Gaiters
(21668,1935,1,1,1,8296),-- Scale Mail Fragment
(21668,1937,1,1,1,12758),-- Scale Gaiters Fragment
(21668,1061,1,1,2,59999),-- Healing Potion
(21668,1829,1,1,2,39599),-- Scroll of Escape: Clan Hall
(21668,1864,1,1,2,197994),-- Stem
(21668,1869,1,1,2,98997),-- Iron Ore
(21668,1880,1,1,2,9900),-- Steel
(21668,5550,1,1,2,1321),-- Durable Metal Plate
(21669,2026,1,1,-1,15353),-- Two-Handed Sword Edge
(21669,2156,1,1,-1,102046),-- Recipe: Elven Necklace
(21669,5550,1,1,-1,17008),-- Durable Metal Plate
(21669,57,539,1078,0,700000),-- Adena
(21669,316,1,1,1,47),-- Blood of Saints
(21669,317,1,1,1,47),-- Tome of Blood
(21669,2030,1,1,1,2645),-- Tome of Blood Page
(21669,1341,20,60,2,188394),-- Bone Arrow
(21669,1864,1,3,2,75358),-- Stem
(21669,1869,1,1,2,113036),-- Iron Ore
(21669,1880,1,1,2,11304),-- Steel
(21669,2186,1,1,2,3781),-- Recipe: Brigandine Tunic
(21669,5550,1,1,2,1508),-- Durable Metal Plate
(21670,1878,1,1,-1,601988),-- Braided Hemp
(21670,1884,1,1,-1,926136),-- Cord
(21670,2029,1,1,-1,17952),-- Gastraphetes Shaft
(21670,57,636,1268,0,700000),-- Adena
(21670,393,1,1,1,176),-- Mithril Banded Mail
(21670,394,1,1,1,176),-- Reinforced Leather Shirt
(21670,415,1,1,1,281),-- Mithril Banded Gaiters
(21670,416,1,1,1,281),-- Reinforced Leather Gaiters
(21670,1938,1,1,1,8967),-- Mithril Banded Mail Material
(21670,1939,1,1,1,11715),-- Mithril Banded Gaiters Material
(21670,1874,1,1,2,8891),-- Oriharukon Ore
(21670,1878,1,1,2,53346),-- Braided Hemp
(21670,1884,1,1,2,82070),-- Cord
(21670,1889,1,1,2,8891),-- Compound Braid
(21670,2270,1,1,2,953),-- Recipe: Goat Head Staff
(21670,5220,1,1,2,5335),-- Metal Hardener
(21671,1937,1,1,-1,38014),-- Scale Gaiters Fragment
(21671,2184,1,1,-1,46479),-- Recipe: Mithril Banded Gaiters
(21671,2185,1,1,-1,87103),-- Recipe: Iron Boots
(21671,57,187,373,0,700000),-- Adena
(21671,605,1,1,1,726),-- Leather Gauntlets
(21671,1123,1,1,1,726),-- Blue Buckskin Boots
(21671,1830,1,1,2,13817),-- Scroll of Escape: Castle
(21671,1876,1,1,2,6909),-- Mithril Ore
(21671,1877,1,1,2,1382),-- Adamantite Nugget
(21671,1880,1,1,2,3455),-- Steel
(21671,1881,1,1,2,4606),-- Coarse Bone Powder
(21671,1882,1,1,2,7677),-- Leather
(21671,6667,1,1,2,21589),-- Deluxe Chest Key - Grade 3
(21672,2030,1,1,-1,37052),-- Tome of Blood Page
(21672,2193,1,1,-1,424088),-- Recipe: Boots of Power
(21672,2269,1,1,-1,22628),-- Recipe: Morning Star
(21672,57,1338,2670,0,700000),-- Adena
(21672,157,1,1,1,117),-- Spiked Club
(21672,186,1,1,1,117),-- Staff of Magic
(21672,2027,1,1,1,7355),-- Spiked Club Head
(21672,1864,4,12,2,62500),-- Stem
(21672,1865,2,6,2,76923),-- Varnish
(21672,1866,3,9,2,31250),-- Suede
(21672,1868,1,3,2,200000),-- Thread
(21672,1873,1,1,2,83333),-- Silver Nugget
(21672,2271,1,1,2,2006),-- Recipe: Winged Spear
(21673,1889,1,1,-1,215196),-- Compound Braid
(21673,1901,1,1,-1,445848),-- Tiger\'s Eye Earring Stone
(21673,5220,1,1,-1,129118),-- Metal Hardener
(21673,57,1363,2720,0,700000),-- Adena
(21673,393,1,1,1,377),-- Mithril Banded Mail
(21673,394,1,1,1,377),-- Reinforced Leather Shirt
(21673,415,1,1,1,601),-- Mithril Banded Gaiters
(21673,416,1,1,1,601),-- Reinforced Leather Gaiters
(21673,1938,1,1,1,19232),-- Mithril Banded Mail Material
(21673,1939,1,1,1,25127),-- Mithril Banded Gaiters Material
(21673,1874,1,1,2,20408),-- Oriharukon Ore
(21673,1878,1,1,2,100000),-- Braided Hemp
(21673,1884,1,1,2,200000),-- Cord
(21673,1889,1,1,2,15873),-- Compound Braid
(21673,2272,1,1,2,1447),-- Recipe: Sword of Revolution
(21673,5220,1,1,2,7246),-- Metal Hardener
(21674,1938,1,1,-1,8463),-- Mithril Banded Mail Material
(21674,1939,1,1,-1,11057),-- Mithril Banded Gaiters Material
(21674,2270,1,1,-1,899),-- Recipe: Goat Head Staff
(21674,57,53,106,0,700000),-- Adena
(21674,260,1,1,1,14),-- Triple-Edged Jamadhr
(21674,1875,1,1,2,638),-- Stone of Purity
(21674,1879,1,1,2,1594),-- Cokes
(21674,1885,1,1,2,797),-- High Grade Suede
(21674,2273,1,1,2,69),-- Recipe: Tarbar
(21674,5549,1,1,2,957),-- Metallic Thread
(21674,5550,1,1,2,128),-- Durable Metal Plate
(21674,6036,1,1,2,319),-- Greater Magic Haste Potion
(21675,956,1,1,-1,8761),-- Scroll: Enchant Armor (Grade D)
(21675,1880,1,1,-1,26283),-- Steel
(21675,2021,1,1,-1,3319),-- Spinebone Sword Blade
(21675,57,111,222,0,700000),-- Adena
(21675,292,1,1,1,45),-- Pike
(21675,1864,1,1,2,57452),-- Stem
(21675,1869,1,1,2,28726),-- Iron Ore
(21675,1880,1,1,2,2873),-- Steel
(21675,2153,1,1,2,4468),-- Recipe: Tiger\'s Eye Earring
(21675,5550,1,1,2,384),-- Durable Metal Plate
(21676,1865,1,7,-1,726143),-- Varnish
(21676,1870,1,6,-1,726143),-- Coal
(21676,2029,1,1,-1,34646),-- Gastraphetes Shaft
(21676,57,1227,2454,0,700000),-- Adena
(21676,849,1,1,1,1201),-- Tiger\'s Eye Earring
(21676,880,1,1,1,1805),-- Black Pearl Ring
(21676,912,1,1,1,901),-- Near Forest Necklace
(21676,1901,1,1,1,53326),-- Tiger\'s Eye Earring Stone
(21676,1865,1,3,2,200000),-- Varnish
(21676,1870,1,3,2,76923),-- Coal
(21676,1875,1,1,2,52632),-- Stone of Purity
(21676,1879,1,1,2,66667),-- Cokes
(21676,2188,1,1,2,31250),-- Recipe: Manticore Skin Gaiters
(21677,1900,1,1,-1,258453),-- Enchanted Necklace Chain
(21677,2250,1,1,-1,590566),-- Recipe: Bone Arrow
(21677,6667,1,3,-1,738208),-- Deluxe Chest Key - Grade 3
(21677,57,996,1993,0,700000),-- Adena
(21677,6667,3,9,2,570000),-- Deluxe Chest Key - Grade 3
(21678,1901,1,1,-1,389212),-- Tiger\'s Eye Earring Stone
(21678,2154,1,1,-1,301702),-- Recipe: Elven Earring
(21678,6667,1,3,-1,880592),-- Deluxe Chest Key - Grade 3
(21678,57,1191,2381,0,700000),-- Adena
(21678,6667,12,36,2,166667),-- Deluxe Chest Key - Grade 3
(21679,955,1,1,-1,9315),-- Scroll: Enchant Weapon (Grade D)
(21679,956,1,1,-1,77629),-- Scroll: Enchant Armor (Grade D)
(21679,6387,1,1,-1,93155),-- Blessed Scroll of Resurrection for Pets
(21679,57,982,1965,0,700000),-- Adena
(21679,605,1,1,1,3822),-- Leather Gauntlets
(21679,1123,1,1,1,3822),-- Blue Buckskin Boots
(21679,955,1,1,2,2546),-- Scroll: Enchant Weapon (Grade D)
(21679,956,1,1,2,34483),-- Scroll: Enchant Armor (Grade D)
(21680,1901,1,1,-1,427616),-- Tiger\'s Eye Earring Stone
(21680,1938,1,1,-1,208130),-- Mithril Banded Mail Material
(21680,2154,1,1,-1,331471),-- Recipe: Elven Earring
(21680,57,1308,2609,0,700000),-- Adena
(21680,316,1,1,1,114),-- Blood of Saints
(21680,317,1,1,1,114),-- Tome of Blood
(21680,2030,1,1,1,6418),-- Tome of Blood Page
(21680,955,1,1,2,3293),-- Scroll: Enchant Weapon (Grade D)
(21680,956,1,1,2,50000),-- Scroll: Enchant Armor (Grade D)
(21681,1939,1,1,-1,235980),-- Mithril Banded Gaiters Material
(21681,2155,1,1,-1,431241),-- Recipe: Elven Ring
(21681,2156,1,1,-1,214931),-- Recipe: Elven Necklace
(21681,57,1135,2270,0,700000),-- Adena
(21681,47,1,1,1,1875),-- Helmet
(21681,628,1,1,1,2681),-- Hoplon
(21681,955,1,1,2,2857),-- Scroll: Enchant Weapon (Grade D)
(21681,956,1,1,2,9174),-- Scroll: Enchant Armor (Grade D)
(21682,1944,1,1,-1,111449),-- Mithril Tunic Fabric
(21682,2148,1,1,-1,156068),-- Recipe: Crafted Leather
(21682,6036,1,1,-1,72832),-- Greater Magic Haste Potion
(21682,57,916,1829,0,700000),-- Adena
(21682,437,1,1,1,253),-- Mithril Tunic
(21682,470,1,1,1,404),-- Mithril Stockings
(21682,1944,1,1,1,14402),-- Mithril Tunic Fabric
(21682,1945,1,1,1,22426),-- Mithril Stocking Design
(21682,1867,1,3,2,150594),-- Animal Skin
(21682,1872,1,3,2,100396),-- Animal Bone
(21682,1882,1,1,2,50198),-- Leather
(21682,1894,1,1,2,7927),-- Crafted Leather
(21682,2157,1,1,2,18669),-- Recipe: Omen Beast\'s Eye Earring
(21683,1910,1,1,-1,218318),-- Aquastone Necklace Chain
(21683,1954,1,1,-1,154134),-- Karmian Stocking Design
(21683,2164,1,1,-1,223021),-- Recipe: Ring of Protection
(21683,57,1170,2376,0,700000),-- Adena
(21683,263,1,1,1,60),-- Chakram
(21683,2073,1,1,1,5296),-- Chakram Edge
(21683,1868,1,3,2,228373),-- Thread
(21683,1873,1,1,2,91350),-- Silver Nugget
(21683,1876,1,1,2,45675),-- Mithril Ore
(21683,1895,1,1,2,65250),-- Metallic Fiber
(21683,2165,1,1,2,8750),-- Recipe: Necklace of Protection
(21683,4042,1,1,2,3807),-- Enria
(21683,8723,1,1,200,3185),-- Life Stone: level 46
(21683,8733,1,1,200,1124),-- Mid-grade Life Stone: level 46
(21684,1906,1,1,-1,312918),-- Mithril Ring Wire
(21684,1907,1,1,-1,158364),-- Necklace of Darkness Gem
(21684,2160,1,1,-1,146870),-- Recipe: Moonstone Earring
(21684,57,919,1865,0,700000),-- Adena
(21684,353,1,1,1,147),-- Half Plate Armor
(21684,380,1,1,1,235),-- Plate Gaiters
(21684,1950,1,1,1,9276),-- Half Plate Temper
(21684,1951,1,1,1,14427),-- Plate Gaiters Part
(21684,1867,1,3,2,76923),-- Animal Skin
(21684,1872,1,3,2,166667),-- Animal Bone
(21684,1875,1,1,2,15243),-- Stone of Purity
(21684,1881,1,1,2,30484),-- Coarse Bone Powder
(21684,2160,1,1,2,14751),-- Recipe: Moonstone Earring
(21685,1873,1,3,-1,498136),-- Silver Nugget
(21685,1958,1,1,-1,223881),-- Mithril Boots Design
(21685,2165,1,1,-1,95428),-- Recipe: Necklace of Protection
(21685,57,1005,2042,0,700000),-- Adena
(21685,439,1,1,1,156),-- Karmian Tunic
(21685,471,1,1,1,249),-- Karmian Stockings
(21685,1954,1,1,1,15646),-- Karmian Stocking Design
(21685,1962,1,1,1,9418),-- Karmian Tunic Pattern
(21685,1868,1,3,2,196190),-- Thread
(21685,1873,1,1,2,78476),-- Silver Nugget
(21685,1889,1,1,2,13080),-- Compound Braid
(21685,2213,1,1,2,3047),-- Recipe: Dwarven Chain Mail Shirt
(21685,4044,1,1,2,6540),-- Thons
(21685,5549,1,1,2,19619),-- Metallic Thread
(21685,8723,1,1,200,2786),-- Life Stone: level 46
(21686,1864,1,11,-1,829734),-- Stem
(21686,1869,1,5,-1,829734),-- Iron Ore
(21686,4470,1,1,-1,14224),-- Dye of Strength <Str+1 Dex-1>
(21686,57,1004,2040,0,700000),-- Adena
(21686,280,1,1,1,70),-- Light Crossbow
(21686,2054,1,1,1,5320),-- Light Crossbow Shaft
(21686,1864,3,9,2,83331),-- Stem
(21686,1869,2,6,2,62499),-- Iron Ore
(21686,1878,1,1,2,99998),-- Braided Hemp
(21686,1884,1,1,2,153842),-- Cord
(21686,2161,1,1,2,24272),-- Recipe: Aquastone Ring
(21687,1945,1,1,-1,112314),-- Mithril Stocking Design
(21687,1946,1,1,-1,191085),-- Rip Gauntlets Pattern
(21687,2149,1,1,-1,101002),-- Recipe: Metallic Fiber
(21687,57,593,1184,0,700000),-- Adena
(21687,606,1,1,1,123),-- Rip Gauntlets
(21687,1124,1,1,1,123),-- Boots of Power
(21687,1946,1,1,1,6175),-- Rip Gauntlets Pattern
(21687,1948,1,1,1,6175),-- Boots of Power Pattern
(21687,2424,1,1,1,123),-- Manticore Skin Boots
(21687,2425,1,1,1,123),-- Brigandine Boots
(21687,2426,1,1,1,123),-- Elven Mithril Boots
(21687,2448,1,1,1,123),-- Manticore Skin Gloves
(21687,2449,1,1,1,123),-- Brigandine Gauntlets
(21687,2450,1,1,1,123),-- Elven Mithril Gloves
(21687,2923,1,1,1,6175),-- Manticore Skin Boot Lining
(21687,2924,1,1,1,6175),-- Brigandine Boots Design
(21687,2925,1,1,1,6175),-- Elven Mithril Boots Design
(21687,2926,1,1,1,6175),-- Manticore Skin Gloves Lining
(21687,2927,1,1,1,6175),-- Brigandine Gauntlets Pattern
(21687,2928,1,1,1,6175),-- Elven Mithril Gloves Design
(21687,1867,1,1,2,162433),-- Animal Skin
(21687,1872,1,1,2,162433),-- Animal Bone
(21687,1875,1,1,2,8122),-- Stone of Purity
(21687,1881,1,1,2,16244),-- Coarse Bone Powder
(21687,2158,1,1,2,15078),-- Recipe: Mithril Ring
(21687,6036,1,1,2,4061),-- Greater Magic Haste Potion
(21688,1908,1,1,-1,208879),-- Moonstone Earring Wire
(21688,1959,1,1,-1,59414),-- Chain Mail Shirt Material
(21688,4042,1,1,-1,36763),-- Enria
(21688,57,890,1808,0,700000),-- Adena
(21688,281,1,1,1,46),-- Crystallized Ice Bow
(21688,2074,1,1,1,3692),-- Crystallized Ice Bow Shaft
(21688,1868,1,3,2,173746),-- Thread
(21688,1873,1,1,2,69499),-- Silver Nugget
(21688,1876,1,1,2,34750),-- Mithril Ore
(21688,1895,1,1,2,49642),-- Metallic Fiber
(21688,2214,1,1,2,4312),-- Recipe: Dwarven Chain Gaiters
(21688,4042,1,1,2,2896),-- Enria
(21688,8723,1,1,200,3497),-- Life Stone: level 46
(21689,1867,1,5,-1,788384),-- Animal Skin
(21689,1872,1,4,-1,788384),-- Animal Bone
(21689,6036,1,1,-1,59129),-- Greater Magic Haste Potion
(21689,57,744,1485,0,700000),-- Adena
(21689,352,1,1,1,103),-- Brigandine Tunic
(21689,1941,1,1,1,6078),-- Brigandine Temper
(21689,2377,1,1,1,164),-- Mithril Scale Gaiters
(21689,2378,1,1,1,164),-- Brigandine Gaiters
(21689,2918,1,1,1,9066),-- Mithril Scale Gaiters Material
(21689,2919,1,1,1,9066),-- Brigandine Gaiters Material
(21689,1867,1,3,2,122260),-- Animal Skin
(21689,1872,1,1,2,244521),-- Animal Bone
(21689,1882,1,1,2,40754),-- Leather
(21689,1894,1,1,2,6435),-- Crafted Leather
(21689,2159,1,1,2,11321),-- Recipe: Necklace of Darkness
(21690,1949,1,1,-1,224655),-- Mithril Gloves Design
(21690,1950,1,1,-1,65333),-- Half Plate Temper
(21690,2161,1,1,-1,195428),-- Recipe: Aquastone Ring
(21690,57,812,1649,0,700000),-- Adena
(21690,61,1,1,1,173),-- Mithril Gloves
(21690,607,1,1,1,173),-- Ogre Power Gauntlets
(21690,1125,1,1,1,173),-- Assault Boots
(21690,1949,1,1,1,9401),-- Mithril Gloves Design
(21690,1957,1,1,1,9401),-- Assault Boots Part
(21690,1963,1,1,1,9401),-- Ogre Power Gauntlets Part
(21690,2427,1,1,1,173),-- Salamander Skin Boots
(21690,2428,1,1,1,173),-- Plate Boots
(21690,2451,1,1,1,173),-- Sage\'s Worn Gloves
(21690,2931,1,1,1,9401),-- Salamander Skin Boot Lining
(21690,2932,1,1,1,9401),-- Plate Boots Design
(21690,2933,1,1,1,9401),-- Sage\'s Worn Gloves Lining
(21690,1868,1,3,2,202158),-- Thread
(21690,1873,1,1,2,80864),-- Silver Nugget
(21690,1889,1,1,2,13478),-- Compound Braid
(21690,2162,1,1,2,9767),-- Recipe: Aquastone Necklace
(21690,5549,1,1,2,20216),-- Metallic Thread
(21691,1947,1,1,-1,268898),-- Kite Shield Fragment
(21691,1948,1,1,-1,252365),-- Boots of Power Pattern
(21691,2157,1,1,-1,154339),-- Recipe: Omen Beast\'s Eye Earring
(21691,57,783,1563,0,700000),-- Adena
(21691,279,1,1,1,69),-- Strengthened Long Bow
(21691,2040,1,1,1,5585),-- Strengthened Long Bow Shaft
(21691,1865,1,1,2,193071),-- Varnish
(21691,1870,1,1,2,193071),-- Coal
(21691,1875,1,1,2,12872),-- Stone of Purity
(21691,1879,1,1,2,32179),-- Cokes
(21691,2194,1,1,2,19741),-- Recipe: Mithril Gloves
(21692,1866,1,3,-1,706540),-- Suede
(21692,1951,1,1,-1,106997),-- Plate Gaiters Part
(21692,2162,1,1,-1,102397),-- Recipe: Aquastone Necklace
(21692,57,855,1737,0,700000),-- Adena
(21692,159,1,1,1,12),-- Bonebreaker
(21692,187,1,1,1,12),-- Atuba Hammer
(21692,188,1,1,1,12),-- Ghost Staff
(21692,189,1,1,1,12),-- Staff of Life
(21692,190,1,1,1,12),-- Atuba Mace
(21692,2048,1,1,1,881),-- Bonebreaker Head
(21692,2049,1,1,1,881),-- Atuba Hammer Head
(21692,2050,1,1,1,934),-- Ghost Staff Head
(21692,2051,1,1,1,934),-- Staff of Life Shaft
(21692,2058,1,1,1,881),-- Atuba Mace Head
(21692,1866,1,1,2,141917),-- Suede
(21692,1871,1,1,2,212876),-- Charcoal
(21692,1885,1,1,2,17740),-- High Grade Suede
(21692,2199,1,1,2,8982),-- Recipe: Karmian Stockings
(21692,5220,1,1,2,8516),-- Metal Hardener
(21693,1960,1,1,-1,134741),-- Chain Gaiters Part
(21693,2213,1,1,-1,38183),-- Recipe: Dwarven Chain Mail Shirt
(21693,2214,1,1,-1,61018),-- Recipe: Dwarven Chain Gaiters
(21693,57,992,2016,0,700000),-- Adena
(21693,631,1,1,1,292),-- Eldarake
(21693,1964,1,1,1,15652),-- Eldarake Temper
(21693,2413,1,1,1,205),-- Chain Hood
(21693,2495,1,1,1,292),-- Chain Shield
(21693,2936,1,1,1,14216),-- Chain Hood Pattern
(21693,2945,1,1,1,15652),-- Chain Shield Fragment
(21693,1868,1,3,2,193696),-- Thread
(21693,1873,1,1,2,77478),-- Silver Nugget
(21693,1876,1,1,2,38739),-- Mithril Ore
(21693,1895,1,1,2,55342),-- Metallic Fiber
(21693,2215,1,1,2,2468),-- Recipe: Robe of Seal
(21693,4042,1,1,2,3229),-- Enria
(21694,1895,1,1,-1,217208),-- Metallic Fiber
(21694,2031,1,1,-1,7696),-- Morning Star Head
(21694,2158,1,1,-1,94088),-- Recipe: Mithril Ring
(21694,57,319,636,0,700000),-- Adena
(21694,88,1,1,1,6),-- Morning Star
(21694,90,1,1,1,6),-- Goat Head Staff
(21694,158,1,1,1,6),-- Tarbar
(21694,169,1,1,1,6),-- Skull Breaker
(21694,172,1,1,1,6),-- Heavy Bone Club
(21694,2031,1,1,1,398),-- Morning Star Head
(21694,2032,1,1,1,364),-- Goat Head Staff Head
(21694,2035,1,1,1,398),-- Tarbar Head
(21694,2036,1,1,1,398),-- Skull Breaker Head
(21694,2037,1,1,1,398),-- Heavy Bone Club Head
(21694,1895,1,1,2,28070),-- Metallic Fiber
(21694,2195,1,1,2,2513),-- Recipe: Half Plate Armor
(21694,5473,1,1,2,7018),-- Recipe: Durable Metal Plate(100%)
(21694,6668,1,1,2,35088),-- Deluxe Chest Key - Grade 4
(21695,1872,2,15,-1,862698),-- Animal Bone
(21695,1961,1,1,-1,165638),-- Mithril Shirt Fabric
(21695,4041,1,1,-1,45010),-- Mold Hardener
(21695,57,2089,4243,0,700000),-- Adena
(21695,96,1,1,1,36),-- Scythe
(21695,298,1,1,1,36),-- Orcish Glaive
(21695,302,1,1,1,36),-- Body Slasher
(21695,2062,1,1,1,3198),-- Scythe Edge
(21695,2075,1,1,1,3198),-- Orcish Glaive Blade
(21695,2076,1,1,1,3198),-- Body Slasher Blade
(21695,1867,1,3,2,111111),-- Animal Skin
(21695,1872,1,3,2,200000),-- Animal Bone
(21695,1875,1,1,2,15783),-- Stone of Purity
(21695,1881,1,1,2,32258),-- Coarse Bone Powder
(21695,2216,1,1,2,13889),-- Recipe: Great Helmet
(21695,4041,1,1,2,1733),-- Mold Hardener
(21695,8723,1,1,200,6250),-- Life Stone: level 46
(21695,8733,1,1,200,5208),-- Mid-Grade Life Stone: level 46
(21696,1894,1,2,-1,185060),-- Crafted Leather
(21696,1962,1,1,-1,168775),-- Karmian Tunic Pattern
(21696,2215,1,1,-1,67188),-- Recipe: Robe of Seal
(21696,57,2128,4323,0,700000),-- Adena
(21696,281,1,1,1,109),-- Crystallized Ice Bow
(21696,2074,1,1,1,8828),-- Crystallized Ice Bow Shaft
(21696,1894,1,1,2,10526),-- Crafted Leather
(21696,2217,1,1,2,18519),-- Recipe: Knight\'s Shield
(21696,4039,1,1,2,8197),-- Mold Glue
(21696,4041,1,1,2,2632),-- Mold Hardener
(21696,4044,1,1,2,6250),-- Thons
(21696,8723,1,1,200,11494),-- Life Stone: level 46
(21696,8733,1,1,200,1645),-- Mid-Grade Life Stone: level 46
(21696,8743,1,1,200,329),-- High-Grade Life Stone: level 46
(21697,1964,1,1,-1,16621),-- Eldarake Temper
(21697,2059,1,1,-1,1528),-- Flamberge Blade
(21697,2216,1,1,-1,8499),-- Recipe: Great Helmet
(21697,57,83,169,0,700000),-- Adena
(21697,263,1,1,1,5),-- Chakram
(21697,2073,1,1,1,376),-- Chakram Edge
(21697,1864,1,1,2,32403),-- Stem
(21697,1869,1,1,2,16202),-- Iron Ore
(21697,1878,1,1,2,6481),-- Braided Hemp
(21697,1884,1,1,2,9970),-- Cord
(21697,2219,1,1,2,1007),-- Recipe: Crimson Boots
(21697,4039,1,1,2,541),-- Mold Glue
(21697,8733,1,1,200,1372),-- Mid-Grade Life Stone: level 46
(21698,1871,1,1,-1,451575),-- Charcoal
(21698,2032,1,1,-1,4173),-- Goat Head Staff Head
(21698,6036,1,1,-1,15052),-- Greater Magic Haste Potion
(21698,57,189,378,0,700000),-- Adena
(21698,395,1,1,1,53),-- Manticore Skin Shirt
(21698,417,1,1,1,84),-- Manticore Skin Gaiters
(21698,1942,1,1,1,2977),-- Manticore Skin Shirt Texture
(21698,1943,1,1,1,4636),-- Manticore Skin Gaiters Pattern
(21698,1341,10,30,2,155621),-- Bone Arrow
(21698,1866,1,1,2,31124),-- Suede
(21698,1871,1,1,2,46686),-- Charcoal
(21698,1877,1,1,2,1868),-- Adamantite Nugget
(21698,2196,1,1,2,1914),-- Recipe: Plate Gaiters
(21699,1953,1,1,-1,154597),-- Sage\'s Rag Lining
(21699,2199,1,1,-1,203650),-- Recipe: Karmian Stockings
(21699,2203,1,3,-1,525763),-- Recipe: Mithril Boots
(21699,57,1947,3955,0,700000),-- Adena
(21699,70,1,1,1,68),-- Claymore
(21699,2047,1,1,1,5327),-- Claymore Blade
(21699,2499,1,1,1,68),-- Elven Long Sword
(21699,2967,1,1,1,5255),-- Elven Long Sword Blade
(21699,1864,4,12,2,121183),-- Stem
(21699,1869,1,3,2,161577),-- Iron Ore
(21699,1880,1,1,2,48473),-- Steel
(21699,2204,1,1,2,9599),-- Recipe: Chain Mail Shirt
(21699,5550,1,1,2,6464),-- Durable Metal Plate
(21700,1902,1,1,-1,411207),-- Elven Earring Beads
(21700,6036,1,1,-1,135287),-- Greater Magic Haste Potion
(21700,6668,1,3,-1,724752),-- Deluxe Chest Key - Grade 4
(21700,57,1702,3397,0,700000),-- Adena
(21700,6668,3,9,2,500000),-- Deluxe Chest Key - Grade 4
(21701,1905,1,1,-1,455493),-- Omen Beast\'s Eye Earring Gemstone
(21701,2160,1,1,-1,302095),-- Recipe: Moonstone Earring
(21701,6668,1,3,-1,836156),-- Deluxe Chest Key - Grade 4
(21701,57,1889,3837,0,700000),-- Adena
(21701,6668,3,9,2,500000),-- Deluxe Chest Key - Grade 4
(21702,1902,1,1,-1,405393),-- Elven Earring Beads
(21702,1903,1,1,-1,667427),-- Elven Ring Piece
(21702,2157,1,1,-1,330680),-- Recipe: Omen Beast\'s Eye Earring
(21702,57,1678,3349,0,700000),-- Adena
(21702,395,1,1,1,462),-- Manticore Skin Shirt
(21702,417,1,1,1,739),-- Manticore Skin Gaiters
(21702,1942,1,1,1,26373),-- Manticore Skin Shirt Texture
(21702,1943,1,1,1,41068),-- Manticore Skin Gaiters Pattern
(21702,951,1,1,2,1236),-- Scroll: Enchant Weapon (Grade C)
(21702,952,1,1,2,12346),-- Scroll: Enchant Armor (Grade C)
(21703,951,1,1,-1,9197),-- Scroll: Enchant Weapon (Grade C)
(21703,952,1,1,-1,67447),-- Scroll: Enchant Armor (Grade C)
(21703,2163,1,1,-1,258089),-- Recipe: Earring of Protection
(21703,57,2041,4146,0,700000),-- Adena
(21703,631,1,1,1,600),-- Eldarake
(21703,1964,1,1,1,32198),-- Eldarake Temper
(21703,2413,1,1,1,422),-- Chain Hood
(21703,2495,1,1,1,600),-- Chain Shield
(21703,2936,1,1,1,29245),-- Chain Hood Pattern
(21703,2945,1,1,1,32198),-- Chain Shield Fragment
(21703,951,1,1,2,2024),-- Scroll: Enchant Weapon (Grade C)
(21703,952,1,1,2,15939),-- Scroll: Enchant Armor (Grade C)
(21703,8723,1,1,200,11111),-- Life Stone: level 46
(21703,8733,1,1,200,2024),-- Mid-grade Life Stone: level 46
(21703,8743,1,1,200,1013),-- High-grade Life Stone: level 46
(21704,951,1,1,-1,8117),-- Scroll: Enchant Weapon (Grade C)
(21704,952,1,1,-1,59525),-- Scroll: Enchant Armor (Grade C)
(21704,1905,1,1,-1,434276),-- Omen Beast\'s Eye Earring Gemstone
(21704,57,1801,3658,0,700000),-- Adena
(21704,61,1,1,1,383),-- Mithril Gloves
(21704,607,1,1,1,383),-- Ogre Power Gauntlets
(21704,1125,1,1,1,383),-- Assault Boots
(21704,1949,1,1,1,20850),-- Mithril Gloves Design
(21704,1957,1,1,1,20850),-- Assault Boots Part
(21704,1963,1,1,1,20850),-- Ogre Power Gauntlets Part
(21704,2427,1,1,1,383),-- Salamander Skin Boots
(21704,2428,1,1,1,383),-- Plate Boots
(21704,2451,1,1,1,383),-- Sage\'s Worn Gloves
(21704,2931,1,1,1,20850),-- Salamander Skin Boot Lining
(21704,2932,1,1,1,20850),-- Plate Boots Design
(21704,2933,1,1,1,20850),-- Sage\'s Worn Gloves Lining
(21704,951,1,1,2,2564),-- Scroll: Enchant Weapon (Grade C)
(21704,952,1,1,2,20408),-- Scroll: Enchant Armor (Grade C)
(21705,1914,1,1,-1,192076),-- Earring of Binding Gemstone
(21705,1915,1,1,-1,301229),-- Ring of Ages Gemstone
(21705,2169,1,1,-1,60512),-- Recipe: Necklace of Binding
(21705,57,1379,2804,0,700000),-- Adena
(21705,75,1,1,1,6),-- Caliburs
(21705,76,1,1,1,6),-- Sword of Delusion
(21705,77,1,1,1,6),-- Tsurugi
(21705,84,1,1,1,6),-- Homunkulus\'s Sword
(21705,134,1,1,1,6),-- Sword of Nightmare
(21705,145,1,1,1,6),-- Sword of Whispering Death
(21705,2089,1,1,1,643),-- Caliburs Edge
(21705,2090,1,1,1,643),-- Delusional Blade
(21705,2091,1,1,1,643),-- Tsurugi Blade
(21705,2092,1,1,1,643),-- Homunkulus\'s Sword Blade
(21705,2095,1,1,1,643),-- Sword of Nightmare Blade
(21705,2096,1,1,1,643),-- Sword of Whispering Death Blade
(21705,1867,1,3,2,104732),-- Animal Skin
(21705,1872,1,3,2,157097),-- Animal Bone
(21705,1875,1,1,2,15710),-- Stone of Purity
(21705,1881,1,1,2,31420),-- Coarse Bone Powder
(21705,2170,1,1,2,5558),-- Recipe: Nassen\'s Earring
(21705,4041,1,1,2,2050),-- Mold Hardener
(21705,8725,1,1,200,18182),-- Life Stone: level 52
(21706,1870,1,9,-1,874534),-- Coal
(21706,2117,1,1,-1,20573),-- Ghoul\'s Staff Head
(21706,4039,1,1,-1,145756),-- Mold Glue
(21706,57,1762,3590,0,700000),-- Adena
(21706,135,1,1,1,14),-- Samurai Longsword
(21706,2115,1,1,1,1850),-- Samurai Longsword Blade
(21706,5286,1,1,1,14),-- Berserker Blade
(21706,5530,1,1,1,1852),-- Berserker Blade Edge
(21706,1865,1,1,2,230647),-- Varnish
(21706,1870,1,1,2,230647),-- Coal
(21706,1874,1,1,2,15377),-- Oriharukon Ore
(21706,4039,1,1,2,7689),-- Mold Glue
(21706,4043,1,1,2,7689),-- Asofe
(21706,4143,1,1,2,1479),-- Recipe: Zubei\'s Leather Shirt
(21706,4971,1,1,2,266),-- Recipe: Dark Elven Long Bow (60%)
(21706,8736,1,1,200,3067),-- Mid-Grade Life Stone: level 55
(21707,1885,1,1,-1,290240),-- High Grade Suede
(21707,1920,1,1,-1,119175),-- Necklace of Binding Chain
(21707,3034,1,1,-1,6966),-- Recipe: Spiritshot B
(21707,57,1406,2856,0,700000),-- Adena
(21707,286,1,1,1,21),-- Eminence Bow
(21707,2121,1,1,1,2958),-- Eminence Bow Shaft
(21707,1875,1,1,2,14289),-- Stone of Purity
(21707,1879,1,1,2,35722),-- Cokes
(21707,1885,1,1,2,17861),-- High Grade Suede
(21707,4134,1,1,2,1649),-- Recipe: Zubei\'s Gaiters
(21707,5549,1,1,2,21434),-- Metallic Thread
(21707,5550,1,1,2,2858),-- Durable Metal Plate
(21707,8726,1,1,200,1976),-- Life Stone: level 55
(21707,8736,1,1,200,1976),-- Mid-Grade Life Stone: level 55
(21708,2118,1,1,-1,17674),-- Demon\'s Staff Head
(21708,4142,1,1,-1,28896),-- Recipe: Avadon Gaiters
(21708,4143,1,1,-1,24080),-- Recipe: Zubei\'s Leather Shirt
(21708,57,1514,3084,0,700000),-- Adena
(21708,286,1,1,1,23),-- Eminence Bow
(21708,2121,1,1,1,3190),-- Eminence Bow Shaft
(21708,1864,3,9,2,66048),-- Stem
(21708,1869,1,1,2,198143),-- Iron Ore
(21708,1878,1,1,2,79257),-- Braided Hemp
(21708,1884,1,1,2,121934),-- Cord
(21708,4039,1,1,2,6605),-- Mold Glue
(21708,4144,1,1,2,2037),-- Recipe: Zubei\'s Leather Gaiters
(21708,4972,1,1,2,228),-- Recipe: Great Axe (60%)
(21708,8736,1,1,200,5556),-- Mid-Grade Life Stone: level 55
(21709,1984,1,1,-1,81636),-- Theca Leather Armor Pattern
(21709,1985,1,1,-1,131322),-- Theca Leather Gaiters Pattern
(21709,3955,1,1,-1,7617),-- Recipe: Blessed Spiritshot B
(21709,57,1538,3123,0,700000),-- Adena
(21709,203,1,1,1,6),-- Pa\'agrian Axe
(21709,204,1,1,1,5),-- Deadman\'s Staff
(21709,205,1,1,1,5),-- Ghoul\'s Staff
(21709,206,1,1,1,5),-- Demon\'s Staff
(21709,2112,1,1,1,682),-- Pa\'agrian Axe Blade
(21709,2116,1,1,1,662),-- Deadman\'s Staff Head
(21709,2117,1,1,1,662),-- Ghoul\'s Staff Head
(21709,2118,1,1,1,662),-- Demon\'s Staff Head
(21709,2503,1,1,1,5),-- Yaksa Mace
(21709,2969,1,1,1,676),-- Yaksa Mace Head
(21709,1895,1,1,2,80352),-- Metallic Fiber
(21709,3955,1,1,2,563),-- Recipe: Blessed Spiritshot B
(21709,4040,1,1,2,5625),-- Mold Lubricant
(21709,4042,1,1,2,4688),-- Enria
(21709,4043,1,1,2,9375),-- Asofe
(21709,8726,1,1,200,4292),-- Life Stone: level 55
(21709,8736,1,1,200,8621),-- Mid-Grade Life Stone: level 55
(21710,1866,1,3,-1,737542),-- Suede
(21710,1916,1,1,-1,94154),-- Necklace of Mermaid Teardrop
(21710,2170,1,1,-1,52185),-- Recipe: Nassen\'s Earring
(21710,57,893,1815,0,700000),-- Adena
(21710,64,1,1,1,187),-- Composite Boots
(21710,608,1,1,1,187),-- Mithril Gauntlets
(21710,1981,1,1,1,13456),-- Mithril Gauntlets Design
(21710,2956,1,1,1,13456),-- Composite Boots Part
(21710,1539,1,1,2,29049),-- Greater Healing Potion
(21710,1866,1,1,2,87144),-- Suede
(21710,1871,1,1,2,130716),-- Charcoal
(21710,1877,1,1,2,5229),-- Adamantite Nugget
(21710,2231,1,1,2,564),-- Recipe: Full Plate Armor
(21710,4040,1,1,2,2615),-- Mold Lubricant
(21710,4041,1,1,2,1137),-- Mold Hardener
(21710,8725,1,1,200,5181),-- Life Stone: level 52
(21711,2119,1,1,-1,15159),-- Crystal Dagger Blade
(21711,4040,1,1,-1,66534),-- Mold Lubricant
(21711,4041,1,1,-1,28928),-- Mold Hardener
(21711,57,1341,2731,0,700000),-- Adena
(21711,203,1,1,1,6),-- Pa\'agrian Axe
(21711,204,1,1,1,5),-- Deadman\'s Staff
(21711,205,1,1,1,5),-- Ghoul\'s Staff
(21711,206,1,1,1,5),-- Demon\'s Staff
(21711,2112,1,1,1,595),-- Pa\'agrian Axe Blade
(21711,2116,1,1,1,578),-- Deadman\'s Staff Head
(21711,2117,1,1,1,578),-- Ghoul\'s Staff Head
(21711,2118,1,1,1,578),-- Demon\'s Staff Head
(21711,2503,1,1,1,5),-- Yaksa Mace
(21711,2969,1,1,1,591),-- Yaksa Mace Head
(21711,1866,1,1,2,116984),-- Suede
(21711,1871,1,1,2,175476),-- Charcoal
(21711,1877,1,1,2,7020),-- Adamantite Nugget
(21711,4040,1,1,2,3510),-- Mold Lubricant
(21711,4041,1,1,2,1526),-- Mold Hardener
(21711,4145,1,1,2,770),-- Recipe: Avadon Leather Armor
(21711,4972,1,1,2,203),-- Recipe: Great Axe (60%)
(21711,8726,1,1,200,3650),-- Life Stone: level 55
(21712,1539,1,1,-1,616818),-- Greater Healing Potion
(21712,4602,1,1,-1,6168),-- Greater Dye of Strength <Str+3 Dex-3>
(21712,5549,1,1,-1,277568),-- Metallic Thread
(21712,57,1120,2277,0,700000),-- Adena
(21712,854,1,1,1,130),-- Earring of Binding
(21712,885,1,1,1,196),-- Ring of Ages
(21712,917,1,1,1,98),-- Necklace of Mermaid
(21712,1914,1,1,1,10749),-- Earring of Binding Gemstone
(21712,1915,1,1,1,16857),-- Ring of Ages Gemstone
(21712,1916,1,1,1,8142),-- Necklace of Mermaid Teardrop
(21712,1868,1,1,2,382618),-- Thread
(21712,1873,1,1,2,76524),-- Silver Nugget
(21712,1889,1,1,2,12754),-- Compound Braid
(21712,2232,1,1,2,1100),-- Recipe: Drake Leather Armor
(21712,4044,1,1,2,6377),-- Thons
(21712,5549,1,1,2,19132),-- Metallic Thread
(21712,8725,1,1,200,3690),-- Life Stone: level 52
(21713,1874,1,1,-1,205309),-- Oriharukon Ore
(21713,1986,1,1,-1,32542),-- Full Plate Armor Temper
(21713,4043,1,1,-1,102654),-- Asofe
(21713,57,1243,2526,0,700000),-- Adena
(21713,135,1,1,1,10),-- Samurai Longsword
(21713,2115,1,1,1,1303),-- Samurai Longsword Blade
(21713,5286,1,1,1,10),-- Berserker Blade
(21713,5530,1,1,1,1305),-- Berserker Blade Edge
(21713,1865,1,1,2,189516),-- Varnish
(21713,1870,1,1,2,189516),-- Coal
(21713,1874,1,1,2,12635),-- Oriharukon Ore
(21713,4039,1,1,2,6318),-- Mold Glue
(21713,4043,1,1,2,6318),-- Asofe
(21713,4126,1,1,2,3128),-- Recipe: Adamantite Earrings
(21714,1977,1,1,-1,43228),-- Composite Armor Temper
(21714,1978,1,1,-1,187320),-- Tower Shield Fragment
(21714,2231,1,1,-1,12596),-- Recipe: Full Plate Armor
(21714,57,1179,2397,0,700000),-- Adena
(21714,64,1,1,1,247),-- Composite Boots
(21714,608,1,1,1,247),-- Mithril Gauntlets
(21714,1981,1,1,1,17771),-- Mithril Gauntlets Design
(21714,2956,1,1,1,17771),-- Composite Boots Part
(21714,1895,1,1,2,57546),-- Metallic Fiber
(21714,2233,1,1,2,1883),-- Recipe: Divine Tunic
(21714,2970,1,1,2,7143),-- Recipe: Ring of Binding
(21714,4040,1,1,2,4029),-- Mold Lubricant
(21714,4042,1,1,2,3357),-- Enria
(21714,4043,1,1,2,6714),-- Asofe
(21714,8735,1,1,200,3096),-- Mid-Grade Life Stone: level 52
(21715,1987,1,1,-1,49476),-- Drake Leather Armor Texture
(21715,4126,1,1,-1,53513),-- Recipe: Adamantite Earrings
(21715,4127,1,1,-1,80270),-- Recipe: Adamantite Ring
(21715,57,1309,2660,0,700000),-- Adena
(21715,442,1,1,1,56),-- Divine Tunic
(21715,473,1,1,1,90),-- Divine Stockings
(21715,1988,1,1,1,5753),-- Divine Tunic Fabric
(21715,1989,1,1,1,9024),-- Divine Stocking Pattern
(21715,1867,1,3,2,133042),-- Animal Skin
(21715,1872,1,1,2,266085),-- Animal Bone
(21715,1882,1,1,2,44348),-- Leather
(21715,1894,1,1,2,7003),-- Crafted Leather
(21715,4042,1,1,2,3326),-- Enria
(21715,4127,1,1,2,4940),-- Recipe: Adamantite Ring
(21715,8726,1,1,200,7752),-- Life Stone: level 55
(21716,2120,1,1,-1,17815),-- Great Pata Blade
(21716,2121,1,1,-1,17055),-- Eminence Bow Shaft
(21716,4144,1,1,-1,38116),-- Recipe: Zubei\'s Leather Gaiters
(21716,57,1495,3045,0,700000),-- Adena
(21716,135,1,1,1,12),-- Samurai Longsword
(21716,2115,1,1,1,1569),-- Samurai Longsword Blade
(21716,5286,1,1,1,12),-- Berserker Blade
(21716,5530,1,1,1,1571),-- Berserker Blade Edge
(21716,1874,1,1,2,15216),-- Oriharukon Ore
(21716,1878,1,1,2,91291),-- Braided Hemp
(21716,1884,1,1,2,140448),-- Cord
(21716,1889,1,1,2,15216),-- Compound Braid
(21716,4146,1,1,2,1464),-- Recipe: Tunic of Zubei
(21716,5220,1,1,2,9130),-- Metal Hardener
(21716,8726,1,1,200,2387),-- Life Stone: level 55
(21717,1877,1,1,-1,47583),-- Adamantite Nugget
(21717,1981,1,1,-1,69975),-- Mithril Gauntlets Design
(21717,2232,1,1,-1,6837),-- Recipe: Drake Leather Armor
(21717,57,480,976,0,700000),-- Adena
(21717,103,1,1,1,48),-- Tower Shield
(21717,107,1,1,1,48),-- Composite Shield
(21717,517,1,1,1,34),-- Composite Helmet
(21717,1149,1,1,1,34),-- Shining Circlet
(21717,1978,1,1,1,3942),-- Tower Shield Fragment
(21717,1983,1,1,1,2697),-- Shining Circlet Pattern
(21717,2957,1,1,1,2697),-- Composite Helmet Design
(21717,4055,1,1,1,3942),-- Composite Shield Fragment
(21717,1342,10,30,2,58824),-- Fine Steel Arrow
(21717,1866,1,1,2,32258),-- Suede
(21717,1871,1,1,2,45455),-- Charcoal
(21717,1877,1,1,2,5650),-- Adamantite Nugget
(21717,2234,1,1,2,815),-- Recipe: Divine Stockings
(21717,3012,1,1,2,1020),-- Recipe: Full Plate Helmet
(21717,4040,1,1,2,629),-- Mold Lubricant
(21717,4041,1,1,2,476),-- Mold Hardener
(21717,6669,1,1,2,15625),-- Deluxe Chest Key - Grade 5
(21717,8725,1,1,200,1259),-- Life Stone: level 52
(21717,8735,1,1,200,629),-- Mid-Grade Life Stone: level 52
(21718,2122,1,1,-1,35632),-- Orcish Poleaxe Blade
(21718,4044,1,1,-1,260224),-- Thons
(21718,4145,1,1,-1,34240),-- Recipe: Avadon Leather Armor
(21718,57,3146,6410,0,700000),-- Adena
(21718,442,1,1,1,135),-- Divine Tunic
(21718,473,1,1,1,215),-- Divine Stockings
(21718,1988,1,1,1,13848),-- Divine Tunic Fabric
(21718,1989,1,1,1,21722),-- Divine Stocking Pattern
(21718,1868,1,3,2,250000),-- Thread
(21718,1873,1,1,2,111111),-- Silver Nugget
(21718,1889,1,1,2,16667),-- Compound Braid
(21718,4044,1,1,2,8130),-- Thons
(21718,4147,1,1,2,2899),-- Recipe: Stockings of Zubei
(21718,8311,1,1,2,723),-- Recipe: Kaim Vanul\'s Bones (100%)
(21718,5549,1,1,2,27778),-- Metallic Thread
(21718,8726,1,1,200,6579),-- Life Stone: level 55
(21718,8736,1,1,200,2128),-- Mid-Grade Life Stone: level 55
(21718,8746,1,1,200,387),-- High-Grade Life Stone: level 55
(21719,1876,1,3,-1,795456),-- Mithril Ore
(21719,2917,1,2,-1,456896),-- Ring of Binding Gemstone
(21719,4146,1,1,-1,50991),-- Recipe: Tunic of Zubei
(21719,57,3206,6531,0,700000),-- Adena
(21719,2436,1,1,1,153),-- Theca Leather Boots
(21719,2437,1,1,1,118),-- Drake Leather Boots
(21719,2438,1,1,1,118),-- Full Plate Boots
(21719,2460,1,1,1,153),-- Theca Leather Gloves
(21719,2461,1,1,1,118),-- Drake Leather Gloves
(21719,2462,1,1,1,118),-- Full Plate Gauntlets
(21719,2463,1,1,1,118),-- Divine Gloves
(21719,2954,1,1,1,13280),-- Theca Leather Boots Texture
(21719,2955,1,1,1,13280),-- Theca Leather Gloves Texture
(21719,2960,1,1,1,11426),-- Drake Leather Boots Design
(21719,2961,1,1,1,11426),-- Full Plate Boots Part
(21719,2962,1,1,1,11426),-- Drake Leather Gloves Design
(21719,2963,1,1,1,11426),-- Full Plate Gauntlets Part
(21719,2964,1,1,1,11426),-- Divine Gloves Pattern
(21719,1868,3,9,2,100000),-- Thread
(21719,1873,1,1,2,125000),-- Silver Nugget
(21719,1876,1,1,2,55556),-- Mithril Ore
(21719,1895,1,1,2,83333),-- Metallic Fiber
(21719,4042,1,1,2,3344),-- Enria
(21719,4148,1,1,2,1672),-- Recipe: Avadon Robe
(21719,8726,1,1,200,4464),-- Life Stone: level 55
(21719,8736,1,1,200,1859),-- Mid-Grade Life Stone: level 55
(21720,2954,1,1,-1,19633),-- Theca Leather Boots Texture
(21720,2955,1,1,-1,19633),-- Theca Leather Gloves Texture
(21720,4147,1,1,-1,3188),-- Recipe: Stockings of Zubei
(21720,57,125,255,0,700000),-- Adena
(21720,299,1,1,1,1),-- Orcish Poleaxe
(21720,301,1,1,1,1),-- Scorpion
(21720,303,1,1,1,1),-- Widow Maker
(21720,2113,1,1,1,93),-- Scorpion Blade
(21720,2114,1,1,1,93),-- Widowmaker Head
(21720,2122,1,1,1,88),-- Orcish Poleaxe Blade
(21720,1866,1,1,2,12727),-- Suede
(21720,1871,1,1,2,19090),-- Charcoal
(21720,1885,1,1,2,1591),-- High Grade Suede
(21720,4041,1,1,2,166),-- Mold Hardener
(21720,4149,1,1,2,246),-- Recipe: Avadon Circlet
(21720,5220,1,1,2,764),-- Metal Hardener
(21721,1539,1,1,-1,157024),-- Greater Healing Potion
(21721,1865,1,1,-1,706609),-- Varnish
(21721,4603,1,1,-1,1570),-- Greater Dye of Constitution <Con+3 Str-3>
(21721,57,285,580,0,700000),-- Adena
(21721,231,1,1,1,4),-- Grace Dagger
(21721,233,1,1,1,4),-- Dark Screamer
(21721,2106,1,1,1,399),-- Grace Dagger Edge
(21721,2107,1,1,1,399),-- Dark Screamer Edge
(21721,1865,1,1,2,30303),-- Varnish
(21721,1870,1,1,2,34483),-- Coal
(21721,1874,1,1,2,2203),-- Oriharukon Ore
(21721,2353,1,1,2,80),-- Recipe: Samurai Longsword
(21721,4039,1,1,2,440),-- Mold Glue
(21721,4043,1,1,2,1761),-- Asofe
(21722,1989,1,1,-1,222552),-- Divine Stocking Pattern
(21722,2112,1,1,-1,35759),-- Pa\'agrian Axe Blade
(21722,4128,1,1,-1,91389),-- Recipe: Adamantite Necklace
(21722,57,2981,6056,0,700000),-- Adena
(21722,299,1,1,1,15),-- Orcish Poleaxe
(21722,301,1,1,1,20),-- Scorpion
(21722,303,1,1,1,20),-- Widow Maker
(21722,2113,1,1,1,2201),-- Scorpion Blade
(21722,2114,1,1,1,2201),-- Widowmaker Head
(21722,2122,1,1,1,2074),-- Orcish Poleaxe Blade
(21722,1865,1,3,2,142857),-- Varnish
(21722,1870,1,1,2,250000),-- Coal
(21722,1875,1,1,2,16393),-- Stone of Purity
(21722,1879,1,1,2,47619),-- Cokes
(21722,4040,1,1,2,4464),-- Mold Lubricant
(21722,4133,1,1,2,992),-- Recipe: Zubei\'s Breastplate
(21722,8726,1,1,200,4464),-- Life Stone: level 55
(21723,1539,1,3,-1,705642),-- Greater Healing Potion
(21723,4601,1,1,-1,14113),-- Greater Dye of Strength <Str+3 Con-3>
(21723,6669,1,3,-1,738462),-- Deluxe Chest Key - Grade 5
(21723,57,2562,5209,0,700000),-- Adena
(21723,6669,1,3,2,250000),-- Deluxe Chest Key - Grade 5
(21723,8725,1,1,200,58824),-- Life Stone: level 52
(21723,8735,1,1,200,12821),-- Mid-Grade Life Stone: level 52
(21723,8745,1,1,200,930),-- High-Grade Life Stone: level 52
(21724,1984,1,2,-1,153567),-- Theca Leather Armor Pattern
(21724,4126,1,1,-1,118216),-- Recipe: Adamantite Earrings
(21724,6669,1,3,-1,833012),-- Deluxe Chest Key - Grade 5
(21724,57,2892,5875,0,700000),-- Adena
(21724,6669,1,3,2,250000),-- Deluxe Chest Key - Grade 5
(21724,8726,1,1,200,50000),-- Life Stone: level 55
(21724,8736,1,1,200,14286),-- Mid-Grade Life Stone: level 55
(21724,8746,1,1,200,1456),-- High-Grade Life Stone: level 55
(21724,8756,1,1,200,208),-- Top-Grade Life Stone: level 55
(21725,951,1,1,-1,11384),-- Scroll: Enchant Weapon (Grade C)
(21725,952,1,1,-1,83480),-- Scroll: Enchant Armor (Grade C)
(21725,1914,1,1,-1,351741),-- Earring of Binding Gemstone
(21725,57,2526,5135,0,700000),-- Adena
(21725,162,1,1,1,8),-- War Axe
(21725,174,1,1,1,8),-- Nirvana Axe
(21725,196,1,1,1,8),-- Stick of Eternity
(21725,197,1,1,1,8),-- Paradia Staff
(21725,199,1,1,1,8),-- Pa\'agrian Hammer
(21725,200,1,1,1,8),-- Sage\'s Staff
(21725,201,1,1,1,8),-- Club of Nature
(21725,202,1,1,1,8),-- Mace of The Underworld
(21725,2097,1,1,1,867),-- War Axe Blade
(21725,2098,1,1,1,867),-- Nirvana Axe Blade
(21725,2099,1,1,1,913),-- Stick of Eternity Shaft
(21725,2100,1,1,1,913),-- Paradia Staff Head
(21725,2102,1,1,1,880),-- Pa\'agrian Hammer Head
(21725,2103,1,1,1,913),-- Sage\'s Staff Head
(21725,2104,1,1,1,913),-- Club of Nature Head
(21725,2105,1,1,1,867),-- Mace of The Underworld Head
(21725,951,1,1,2,909),-- Scroll: Enchant Weapon (Grade C)
(21725,952,1,1,2,1364),-- Scroll: Enchant Armor (Grade C)
(21725,1894,1,1,2,10417),-- Crafted Leather
(21725,4039,1,1,2,9524),-- Mold Glue
(21725,4041,1,1,2,1818),-- Mold Hardener
(21725,4044,1,1,2,10417),-- Thons
(21725,8725,1,1,200,5917),-- Life Stone: level 52
(21725,8735,1,1,200,1818),-- Mid-Grade Life Stone: level 52
(21726,951,1,1,-1,13871),-- Scroll: Enchant Weapon (Grade C)
(21726,952,1,1,-1,101724),-- Scroll: Enchant Armor (Grade C)
(21726,4127,1,1,-1,188844),-- Recipe: Adamantite Ring
(21726,57,3075,6264,0,700000),-- Adena
(21726,119,1,1,1,167),-- Necklace of Binding
(21726,855,1,1,1,222),-- Nassen\'s Earring
(21726,886,1,1,1,333),-- Ring of Binding
(21726,1918,1,1,1,20332),-- Nassen\'s Earring Gemstone
(21726,1920,1,1,1,16064),-- Necklace of Binding Chain
(21726,2917,1,1,1,26965),-- Ring of Binding Gemstone
(21726,951,1,1,2,2561),-- Scroll: Enchant Weapon (Grade C)
(21726,952,1,1,2,10101),-- Scroll: Enchant Armor (Grade C)
(21726,8726,1,1,200,8130),-- Life Stone: level 55
(21726,8736,1,1,200,811),-- Mid-Grade Life Stone: level 55
(21726,8756,1,1,200,406),-- Top-Grade Life Stone: level 55
(21727,951,1,1,-1,12419),-- Scroll: Enchant Weapon (Grade C)
(21727,952,1,1,-1,91069),-- Scroll: Enchant Armor (Grade C)
(21727,1984,1,1,-1,146414),-- Theca Leather Armor Pattern
(21727,57,2758,5602,0,700000),-- Adena
(21727,119,1,1,1,149),-- Necklace of Binding
(21727,855,1,1,1,199),-- Nassen\'s Earring
(21727,886,1,1,1,299),-- Ring of Binding
(21727,1918,1,1,1,18202),-- Nassen\'s Earring Gemstone
(21727,1920,1,1,1,14381),-- Necklace of Binding Chain
(21727,2917,1,1,1,24141),-- Ring of Binding Gemstone
(21727,951,1,1,2,436),-- Scroll: Enchant Weapon (Grade C)
(21727,952,1,1,2,3484),-- Scroll: Enchant Armor (Grade C)
(21727,8726,1,1,200,6098),-- Life Stone: level 55
(21727,8736,1,1,200,6098),-- Mid-Grade Life Stone: level 55
(21727,8746,1,1,200,436),-- High-Grade Life Stone: level 55
(21728,1867,1,13,-1,834610),-- Animal Skin
(21728,1881,1,1,-1,667688),-- Coarse Bone Powder
(21728,4614,1,1,-1,6907),-- Greater Dye of Strength <Str+4 Dex-4>
(21728,57,2020,4107,0,700000),-- Adena
(21728,357,1,1,1,18),-- Zubei\'s Breastplate
(21728,383,1,1,1,29),-- Zubei\'s Gaiters
(21728,2376,1,1,1,18),-- Avadon Breastplate
(21728,2379,1,1,1,29),-- Avadon Gaiters
(21728,4056,1,1,1,3347),-- Zubei\'s Breastplate Part
(21728,4057,1,1,1,5172),-- Zubei\'s Gaiter Material
(21728,4064,1,1,1,3347),-- Avadon Breastplate Part
(21728,4065,1,1,1,5172),-- Avadon Gaiters Material
(21728,1867,1,3,2,109570),-- Animal Skin
(21728,1872,1,3,2,164355),-- Animal Bone
(21728,1875,1,1,2,16436),-- Stone of Purity
(21728,1881,1,1,2,32871),-- Coarse Bone Powder
(21728,4041,1,1,2,2144),-- Mold Hardener
(21728,4129,1,1,2,2668),-- Recipe: Earrings of Black Ore
(21728,8728,1,1,200,2591),-- Life Stone: level 61
(21728,8738,1,1,200,2591),-- Mid-Grade Life Stone: level 61
(21729,1807,1,1,-1,8333),-- Recipe: Soulshot: A-Grade
(21729,3035,1,1,-1,8333),-- Recipe: Spiritshot A
(21729,4085,1,1,-1,108621),-- Tunic of Doom Pattern
(21729,57,2526,5122,0,700000),-- Adena
(21729,97,1,1,1,15),-- Lance
(21729,4115,1,1,1,4028),-- Lance Blade
(21729,1874,1,1,2,20514),-- Oriharukon Ore
(21729,1878,1,1,2,123079),-- Braided Hemp
(21729,1884,1,1,2,189352),-- Cord
(21729,1889,1,1,2,20514),-- Compound Braid
(21729,5220,1,1,2,12308),-- Metal Hardener
(21729,6337,1,1,2,1578),-- Recipe: Sealed Majestic Earring(70%)
(21729,8729,1,1,200,1001),-- Life Stone: level 64
(21730,4087,1,1,-1,117875),-- Stockings of Doom Pattern
(21730,4088,1,1,-1,189843),-- Blue Wolf Helmet Design
(21730,5276,1,1,-1,6698),-- Recipe: Greater Spiritshot (A) Compressed Package(100%)
(21730,57,2032,4115,0,700000),-- Adena
(21730,287,1,1,1,12),-- Bow of Peril
(21730,4121,1,1,1,3232),-- Bow of Peril Shaft
(21730,1865,1,1,2,142857),-- Varnish
(21730,1870,1,1,2,166667),-- Coal
(21730,1874,1,1,2,10526),-- Oriharukon Ore
(21730,4039,1,1,2,4651),-- Mold Glue
(21730,4043,1,1,2,2331),-- Asofe
(21730,5438,1,1,2,136),-- Recipe: Meteor Shower(60%)
(21730,8729,1,1,200,5814),-- Life Stone: level 64
(21730,8739,1,1,200,1164),-- Mid-Grade Life Stone: level 64
(21731,1894,1,1,-1,188395),-- Crafted Leather
(21731,4042,1,1,-1,89488),-- Enria
(21731,4086,1,1,-1,125994),-- Blue Wolf Stockings Pattern
(21731,57,2170,4400,0,700000),-- Adena
(21731,110,1,1,1,63),-- Doom Shield
(21731,2416,1,1,1,44),-- Blue Wolf Helmet
(21731,2417,1,1,1,44),-- Doom Helmet
(21731,4077,1,1,1,11890),-- Doom Shield Fragment
(21731,4088,1,1,1,9990),-- Blue Wolf Helmet Design
(21731,4089,1,1,1,9990),-- Doom Helmet Pattern
(21731,1343,20,60,2,100000),-- Silver Arrow
(21731,1867,1,3,2,62500),-- Animal Skin
(21731,1872,1,1,2,166667),-- Animal Bone
(21731,1882,1,1,2,23810),-- Leather
(21731,1894,1,1,2,5587),-- Crafted Leather
(21731,4042,1,1,2,3777),-- Enria
(21731,6339,1,1,2,1393),-- Recipe: Sealed Majestic Ring(70%)
(21731,8729,1,1,200,2786),-- Life Stone: level 64
(21732,4039,1,1,-1,183088),-- Mold Glue
(21732,4089,1,1,-1,207583),-- Doom Helmet Pattern
(21732,5281,1,1,-1,7324),-- Recipe: Greater Blessed Spiritshot (A) Compressed Package(100%)
(21732,57,2222,4499,0,700000),-- Adena
(21732,234,1,1,1,13),-- Demon Dagger
(21732,4119,1,1,1,3551),-- Demon Dagger Edge
(21732,1343,30,90,2,110370),-- Silver Arrow
(21732,1864,4,12,2,57945),-- Stem
(21732,1869,2,6,2,46356),-- Iron Ore
(21732,1878,1,1,2,92711),-- Braided Hemp
(21732,1884,1,1,2,142633),-- Cord
(21732,4039,1,1,2,7727),-- Mold Glue
(21732,5444,1,1,2,127),-- Recipe: Carnage Bow(60%)
(21732,8739,1,1,200,385),-- Mid-Grade Life Stone: level 64
(21732,8749,1,1,200,385),-- High-Grade Life Stone: level 64
(21733,4056,1,1,-1,58657),-- Zubei\'s Breastplate Part
(21733,4057,1,1,-1,90651),-- Zubei\'s Gaiter Material
(21733,4131,1,1,-1,26348),-- Recipe: Necklace of Black Ore
(21733,57,1308,2658,0,700000),-- Adena
(21733,91,1,1,1,6),-- Heavy War Axe
(21733,92,1,1,1,6),-- Sprite\'s Staff
(21733,4105,1,1,1,1370),-- Heavy War Axe Head
(21733,4106,1,1,1,1347),-- Sprite\'s Staff Head
(21733,1343,20,60,2,97682),-- Silver Arrow
(21733,1867,1,1,2,182340),-- Animal Skin
(21733,1872,1,1,2,182340),-- Animal Bone
(21733,1875,1,1,2,9117),-- Stone of Purity
(21733,1881,1,1,2,18235),-- Coarse Bone Powder
(21733,4041,1,1,2,1190),-- Mold Hardener
(21733,4130,1,1,2,2221),-- Recipe: Ring of Black Ore
(21733,8738,1,1,200,3125),-- Mid-Grade Life Stone: level 61
(21734,3956,1,1,-1,6340),-- Recipe: Blessed Spiritshot A
(21734,4087,1,1,-1,111581),-- Stockings of Doom Pattern
(21734,4088,1,1,-1,179707),-- Blue Wolf Helmet Design
(21734,57,1922,3897,0,700000),-- Adena
(21734,2391,1,1,1,20),-- Blue Wolf Leather Armor
(21734,2392,1,1,1,20),-- Leather Armor of Doom
(21734,4082,1,1,1,3968),-- Blue Wolf Leather Armor Texture
(21734,4083,1,1,1,3968),-- Leather Armor of Doom Design
(21734,729,1,1,2,26),-- Scroll: Enchant Weapon (Grade A)
(21734,1864,3,9,2,78032),-- Stem
(21734,1869,2,6,2,58524),-- Iron Ore
(21734,1878,1,1,2,93638),-- Braided Hemp
(21734,1884,1,1,2,144058),-- Cord
(21734,4039,1,1,2,7804),-- Mold Glue
(21734,8739,1,1,200,3559),-- Mid-Grade Life Stone: level 64
(21735,948,1,1,-1,10164),-- Scroll: Enchant Armor (Grade B)
(21735,4064,1,1,-1,73583),-- Avadon Breastplate Part
(21735,4154,1,1,-1,48112),-- Recipe: Doom Shield
(21735,57,1640,3334,0,700000),-- Adena
(21735,300,1,1,1,14),-- Great Axe
(21735,4113,1,1,1,3090),-- Great Axe Head
(21735,1894,1,1,2,8428),-- Crafted Leather
(21735,4039,1,1,2,8006),-- Mold Glue
(21735,4041,1,1,2,2089),-- Mold Hardener
(21735,4044,1,1,2,8006),-- Thons
(21735,4131,1,1,2,1953),-- Recipe: Necklace of Black Ore
(21735,8728,1,1,200,7937),-- Life Stone: level 61
(21736,4090,1,1,-1,199401),-- Blue Wolf Boots Design
(21736,5220,1,1,-1,177666),-- Metal Hardener
(21736,5438,1,1,-1,2427),-- Recipe: Meteor Shower(60%)
(21736,57,1797,3638,0,700000),-- Adena
(21736,97,1,1,1,11),-- Lance
(21736,4115,1,1,1,2862),-- Lance Blade
(21736,1866,1,1,2,145778),-- Suede
(21736,1871,1,1,2,218667),-- Charcoal
(21736,1885,1,1,2,18223),-- High Grade Suede
(21736,4041,1,1,2,1902),-- Mold Hardener
(21736,5220,1,1,2,8747),-- Metal Hardener
(21736,5446,1,1,2,120),-- Recipe: Bloody Orchid(60%)
(21736,8729,1,1,200,943),-- Life Stone: level 64
(21737,1880,1,1,-1,428009),-- Steel
(21737,4615,1,1,-1,5904),-- Greater Dye of Constitution <Con+4 Str-4>
(21737,5550,1,1,-1,57068),-- Durable Metal Plate
(21737,57,1727,3510,0,700000),-- Adena
(21737,229,1,1,1,8),-- Kris
(21737,243,1,1,1,8),-- Hell Knife
(21737,4109,1,1,1,1781),-- Kris Edge
(21737,4110,1,1,1,1781),-- Hell Knife Edge
(21737,1343,30,90,2,86005),-- Silver Arrow
(21737,1864,1,3,2,180611),-- Stem
(21737,1869,1,1,2,180611),-- Iron Ore
(21737,1880,1,1,2,18061),-- Steel
(21737,4043,1,1,2,6021),-- Asofe
(21737,4154,1,1,2,2138),-- Recipe: Doom Shield
(21737,5550,1,1,2,2409),-- Durable Metal Plate
(21737,8728,1,1,200,2740),-- Life Stone: level 61
(21737,8738,1,1,200,1370),-- Mid-Grade Life Stone: level 61
(21738,4091,1,1,-1,209972),-- Doom Gloves Part
(21738,4096,1,1,-1,209972),-- Blue Wolf Gloves Fabric
(21738,5444,1,1,-1,2556),-- Recipe: Carnage Bow(60%)
(21738,57,1892,3831,0,700000),-- Adena
(21738,79,1,1,1,11),-- Sword of Damascus
(21738,7889,1,1,1,11),-- Wizard\'s Tear
(21738,7883,1,1,1,11),-- Guardian Sword
(21738,8336,1,1,1,1119),-- Blade of the Wizard\'s Tear
(21738,8330,1,1,1,959),-- Guardian Sword Blade
(21738,4114,1,1,1,3024),-- Sword of Damascus Blade
(21738,1343,10,30,2,166667),-- Silver Arrow
(21738,1865,1,1,2,166667),-- Varnish
(21738,1870,1,1,2,111111),-- Coal
(21738,1875,1,1,2,15385),-- Stone of Purity
(21738,1879,1,1,2,34483),-- Cokes
(21738,4040,1,1,2,4202),-- Mold Lubricant
(21738,5452,1,1,2,108),-- Recipe: Blood Tornado(60%)
(21738,8729,1,1,200,2801),-- Life Stone: level 64
(21739,4089,1,1,-1,200340),-- Doom Helmet Pattern
(21739,5156,1,1,-1,7068),-- Recipe: Soulshot (A) Compressed Package (100%)
(21739,5220,1,1,-1,212040),-- Metal Hardener
(21739,57,2143,4344,0,700000),-- Adena
(21739,864,1,1,1,57),-- Earring of Black Ore
(21739,895,1,1,1,85),-- Ring of Black Ore
(21739,926,1,1,1,43),-- Necklace of Black Ore
(21739,4052,1,1,1,9239),-- Earring of Black Ore Piece
(21739,4053,1,1,1,11185),-- Ring of Black Ore Gemstone
(21739,4054,1,1,1,5140),-- Necklace of Black Ore Beads
(21739,730,1,1,2,187),-- Scroll: Enchant Armor (Grade A)
(21739,1343,10,30,2,166667),-- Silver Arrow
(21739,1866,1,1,2,111111),-- Suede
(21739,1871,1,1,2,142857),-- Charcoal
(21739,1885,1,1,2,12048),-- High Grade Suede
(21739,4041,1,1,2,1344),-- Mold Hardener
(21739,5220,1,1,2,13514),-- Metal Hardener
(21739,8729,1,1,200,1344),-- Life Stone: level 64
(21740,4065,1,1,-1,48737),-- Avadon Gaiters Material
(21740,4066,1,1,-1,41079),-- Zubei\'s Leather Shirt Fabric
(21740,4155,1,1,-1,5411),-- Recipe: Blue Wolf Breastplate
(21740,57,703,1429,0,700000),-- Adena
(21740,357,1,1,1,7),-- Zubei\'s Breastplate
(21740,383,1,1,1,10),-- Zubei\'s Gaiters
(21740,2376,1,1,1,7),-- Avadon Breastplate
(21740,2379,1,1,1,10),-- Avadon Gaiters
(21740,4056,1,1,1,1165),-- Zubei\'s Breastplate Part
(21740,4057,1,1,1,1800),-- Zubei\'s Gaiter Material
(21740,4064,1,1,1,1165),-- Avadon Breastplate Part
(21740,4065,1,1,1,1800),-- Avadon Gaiters Material
(21740,1895,1,1,2,14286),-- Metallic Fiber
(21740,4040,1,1,2,3021),-- Mold Lubricant
(21740,4042,1,1,2,864),-- Enria
(21740,4043,1,1,2,1297),-- Asofe
(21740,4155,1,1,2,267),-- Recipe: Blue Wolf Breastplate
(21740,6670,1,1,2,8197),-- Deluxe Chest Key - Grade 6
(21740,8728,1,1,200,432),-- Life Stone: level 61
(21741,4040,1,1,-1,223170),-- Mold Lubricant
(21741,4090,1,1,-1,500943),-- Blue Wolf Boots Design
(21741,5161,1,1,-1,14878),-- Recipe: Spiritshot (A) Compressed Package (100%)
(21741,57,4510,9145,0,700000),-- Adena
(21741,79,1,1,1,26),-- Sword of Damascus
(21741,7889,1,1,1,26),-- Wizard\'s Tear
(21741,7883,1,1,1,26),-- Guardian Sword
(21741,8336,1,1,1,2272),-- Blade of the Wizard\'s Tear
(21741,8330,1,1,1,2057),-- Guardian Sword Blade
(21741,4114,1,1,1,7213),-- Sword of Damascus Blade
(21741,1866,1,1,2,250000),-- Suede
(21741,1871,1,3,2,166667),-- Charcoal
(21741,1877,1,1,2,13889),-- Adamantite Nugget
(21741,4040,1,1,2,7634),-- Mold Lubricant
(21741,4041,1,1,2,3268),-- Mold Hardener
(21741,5434,1,1,2,64),-- Recipe: Dragon Slayer(60%)
(21741,8729,1,1,200,4854),-- Life Stone: level 64
(21741,8739,1,1,200,1919),-- Mid-Grade Life Stone: level 64
(21741,8749,1,1,200,384),-- High-Grade Life Stone: level 64
(21742,4091,1,1,-1,510430),-- Doom Gloves Part
(21742,4096,1,1,-1,510430),-- Blue Wolf Gloves Fabric
(21742,5166,1,1,-1,15160),-- Recipe: Blessed Spiritshot (A) Compressed Package (100%)
(21742,57,4596,9318,0,700000),-- Adena
(21742,234,1,1,1,26),-- Demon Dagger
(21742,4119,1,1,1,7350),-- Demon Dagger Edge
(21742,1343,100,300,2,43478),-- Silver Arrow
(21742,1867,1,3,2,142857),-- Animal Skin
(21742,1872,2,6,2,111111),-- Animal Bone
(21742,1882,1,1,2,66667),-- Leather
(21742,1894,1,1,2,11236),-- Crafted Leather
(21742,4042,1,1,2,6098),-- Enria
(21742,5440,1,1,2,129),-- Recipe: Elysian(60%)
(21742,8729,1,1,200,4049),-- Life Stone: level 64
(21742,8739,1,1,200,1285),-- Mid-Grade Life Stone: level 64
(21742,8749,1,1,200,26),-- High-Grade Life Stone: level 64
(21743,1877,1,1,-1,17736),-- Adamantite Nugget
(21743,4041,1,1,-1,3856),-- Mold Hardener
(21743,4099,1,1,-1,19905),-- Doom Boots Part
(21743,57,179,363,0,700000),-- Adena
(21743,358,1,1,1,2),-- Blue Wolf Breastplate
(21743,2380,1,1,1,3),-- Blue Wolf Gaiters
(21743,2381,1,1,1,1),-- Doom Plate Armor
(21743,4078,1,1,1,319),-- Blue Wolf Breastplate Part
(21743,4080,1,1,1,449),-- Blue Wolf Gaiters Material
(21743,4081,1,1,1,216),-- Doom Plate Armor Temper
(21743,1866,1,1,2,5988),-- Suede
(21743,1871,1,1,2,13514),-- Charcoal
(21743,1877,1,1,2,874),-- Adamantite Nugget
(21743,4040,1,1,2,207),-- Mold Lubricant
(21743,4041,1,1,2,207),-- Mold Hardener
(21743,5442,1,1,2,9),-- Recipe: Soul Bow(60%)
(21743,8729,1,1,200,207),-- Life Stone: level 64
(21743,8739,1,1,200,103),-- Mid-Grade Life Stone: level 64
(21744,947,1,1,-1,414),-- Scroll: Enchant Weapon (Grade B)
(21744,4067,1,1,-1,37049),-- Zubei\'s Leather Gaiter Texture
(21744,4157,1,1,-1,5149),-- Recipe: Blue Wolf Gaiters
(21744,57,418,849,0,700000),-- Adena
(21744,2397,1,1,1,7),-- Tunic of Zubei
(21744,2402,1,1,1,11),-- Stockings of Zubei
(21744,2406,1,1,1,5),-- Avadon Robe
(21744,4069,1,1,1,1202),-- Tunic of Zubei Fabric
(21744,4070,1,1,1,1824),-- Stockings of Zubei Fabric
(21744,4071,1,1,1,923),-- Avadon Robe Fabric
(21744,1343,10,30,2,41667),-- Silver Arrow
(21744,1868,1,1,2,66667),-- Thread
(21744,1873,1,1,2,8929),-- Silver Nugget
(21744,1889,1,1,2,1590),-- Compound Braid
(21744,4044,1,1,2,1135),-- Thons
(21744,4157,1,1,2,218),-- Recipe: Blue Wolf Gaiters
(21744,5549,1,1,2,3067),-- Metallic Thread
(21745,4114,1,1,-1,46617),-- Sword of Damascus Blade
(21745,5446,1,1,-1,5820),-- Recipe: Bloody Orchid(60%)
(21745,5452,1,1,-1,5820),-- Recipe: Blood Tornado(60%)
(21745,57,4309,8724,0,700000),-- Adena
(21745,268,1,1,1,25),-- Bellion Cestus
(21745,4120,1,1,1,6873),-- Bellion Cestus Edge
(21745,1343,80,240,2,50000),-- Silver Arrow
(21745,1866,1,1,2,200000),-- Suede
(21745,1871,1,3,2,100000),-- Charcoal
(21745,1877,1,1,2,13158),-- Adamantite Nugget
(21745,4040,1,1,2,5076),-- Mold Lubricant
(21745,4041,1,1,2,3802),-- Mold Hardener
(21745,8729,1,1,200,4651),-- Life Stone: level 64
(21745,8739,1,1,200,1689),-- Mid-Grade Life Stone: level 64
(21745,8759,1,1,200,141),-- Top-Grade Life Stone: level 64
(21745,5460,1,1,2,141); -- Recipe: Dasparion\'s Staff(60%)

INSERT INTO `droplist` VALUES 
(21746,947,1,1,-1,3721),-- Scroll: Enchant Weapon (Grade B)
(21746,6670,1,3,-1,775155),-- Deluxe Chest Key - Grade 6
(21746,57,3753,7629,0,700000),-- Adena
(21746,6670,1,3,2,250000),-- Deluxe Chest Key - Grade 6
(21746,8728,1,1,200,45455),-- Life Stone: level 61
(21746,8738,1,1,200,12658),-- Mid-Grade Life Stone: level 61
(21746,8748,1,1,200,1965),-- High-Grade Life Stone: level 61
(21747,1894,1,1,-1,362536),-- Crafted Leather
(21747,4052,1,1,-1,365809),-- Earring of Black Ore Piece
(21747,6670,1,3,-1,861023),-- Deluxe Chest Key - Grade 6
(21747,57,4180,8463,0,700000),-- Adena
(21747,6670,1,3,2,250000),-- Deluxe Chest Key - Grade 6
(21747,8729,1,1,200,45455),-- Life Stone: level 64
(21747,8739,1,1,200,10638),-- Mid-Grade Life Stone: level 64
(21747,8749,1,1,200,1093),-- High-Grade Life Stone: level 64
(21748,947,1,1,-1,3668),-- Scroll: Enchant Weapon (Grade B)
(21748,948,1,1,-1,22926),-- Scroll: Enchant Armor (Grade B)
(21748,4129,1,1,-1,99246),-- Recipe: Earrings of Black Ore
(21748,57,3700,7521,0,700000),-- Adena
(21748,267,1,1,1,32),-- Arthro Nail
(21748,4111,1,1,1,7751),-- Arthro Nail Blade
(21748,947,1,1,2,110),-- Scroll: Enchant Weapon (Grade B)
(21748,948,1,1,2,991),-- Scroll: Enchant Armor (Grade B)
(21748,1867,1,3,2,111111),-- Animal Skin
(21748,1872,1,3,2,166667),-- Animal Bone
(21748,1875,1,1,2,18182),-- Stone of Purity
(21748,1881,1,1,2,37037),-- Coarse Bone Powder
(21748,4041,1,1,2,771),-- Mold Hardener
(21748,8728,1,1,200,4854),-- Life Stone: level 61
(21748,8738,1,1,200,2092),-- Mid-Grade Life Stone: level 61
(21748,8748,1,1,200,110),-- High-Grade Life Stone: level 61
(21749,947,1,1,-1,4362),-- Scroll: Enchant Weapon (Grade B)
(21749,948,1,1,-1,27262),-- Scroll: Enchant Armor (Grade B)
(21749,4053,1,1,-1,467420),-- Ring of Black Ore Gemstone
(21749,57,4408,8937,0,700000),-- Adena
(21749,601,1,1,1,101),-- Boots of Doom
(21749,2439,1,1,1,101),-- Blue Wolf Boots
(21749,2475,1,1,1,101),-- Doom Gloves
(21749,2487,1,1,1,101),-- Blue Wolf Gloves
(21749,4090,1,1,1,18072),-- Blue Wolf Boots Design
(21749,4091,1,1,1,18072),-- Doom Gloves Part
(21749,4096,1,1,1,18072),-- Blue Wolf Gloves Fabric
(21749,4099,1,1,1,18072),-- Doom Boots Part
(21749,947,1,1,2,645),-- Scroll: Enchant Weapon (Grade B)
(21749,948,1,1,2,1563),-- Scroll: Enchant Armor (Grade B)
(21749,8729,1,1,200,6452),-- Life Stone: level 64
(21749,8739,1,1,200,1269),-- Mid-Grade Life Stone: level 64
(21749,8749,1,1,200,293),-- High-Grade Life Stone: level 64
(21750,947,1,1,-1,3940),-- Scroll: Enchant Weapon (Grade B)
(21750,948,1,1,-1,24627),-- Scroll: Enchant Armor (Grade B)
(21750,5230,1,1,-1,13135),-- Recipe: Mithril Arrow (100%)
(21750,57,3985,8069,0,700000),-- Adena
(21750,110,1,1,1,115),-- Doom Shield
(21750,2416,1,1,1,81),-- Blue Wolf Helmet
(21750,2417,1,1,1,81),-- Doom Helmet
(21750,4077,1,1,1,21815),-- Doom Shield Fragment
(21750,4088,1,1,1,18328),-- Blue Wolf Helmet Design
(21750,4089,1,1,1,18328),-- Doom Helmet Pattern
(21750,947,1,1,2,115),-- Scroll: Enchant Weapon (Grade B)
(21750,948,1,1,2,1385),-- Scroll: Enchant Armor (Grade B)
(21750,8729,1,1,200,6250),-- Life Stone: level 64
(21750,8739,1,1,200,1965),-- Mid-Grade Life Stone: level 64
(21750,8749,1,1,200,577),-- High-Grade Life Stone: level 64
(21750,8759,1,1,200,115),-- Top-Grade Life Stone: level 64
(21751,729,1,1,-1,778),-- Scroll: Enchant Weapon (Grade A)
(21751,1876,1,3,-1,700393),-- Mithril Ore
(21751,1895,1,5,-1,667041),-- Metallic Fiber
(21751,57,2825,5748,0,700000),-- Adena
(21751,269,1,1,1,12),-- Blood Tornado
(21751,5539,1,1,1,3489),-- Blood Tornado Edge
(21751,1343,30,90,2,111111),-- Silver Arrow
(21751,1868,1,3,2,250000),-- Thread
(21751,1873,1,1,2,90909),-- Silver Nugget
(21751,1876,1,1,2,40000),-- Mithril Ore
(21751,1895,1,1,2,47619),-- Metallic Fiber
(21751,4042,1,1,2,1076),-- Enria
(21751,5434,1,1,2,110),-- Recipe: Dragon Slayer(60%)
(21751,8730,1,1,200,2155),-- Life Stone: level 67
(21751,8740,1,1,200,171),-- Mid-Grade Life Stone: level 67
(21752,5272,1,1,-1,3799),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21752,5522,1,1,-1,29108),-- Sealed Armor of Nightmare Pattern
(21752,5523,1,1,-1,29108),-- Sealed Majestic Plate Armor Pattern
(21752,57,3440,7020,0,700000),-- Adena
(21752,269,1,1,1,14),-- Blood Tornado
(21752,5539,1,1,1,4258),-- Blood Tornado Edge
(21752,1344,40,120,2,76923),-- Mithril Arrow
(21752,1867,1,3,2,90909),-- Animal Skin
(21752,1872,1,3,2,166667),-- Animal Bone
(21752,1875,1,1,2,12346),-- Stone of Purity
(21752,1881,1,1,2,25000),-- Coarse Bone Powder
(21752,4041,1,1,2,3137),-- Mold Hardener
(21752,5162,1,1,2,161),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21752,8731,1,1,200,953),-- Life Stone: level 70
(21753,5464,1,1,-1,2567),-- Recipe: Dark Legion\'s Edge(60%)
(21753,5520,1,1,-1,44699),-- Sealed Dark Crystal Breastplate Pattern
(21753,5521,1,1,-1,31541),-- Sealed Tallum Plate Armor Pattern
(21753,57,2793,5688,0,700000),-- Adena
(21753,5287,1,1,1,15),-- Sealed Dark Crystal breastplate
(21753,5288,1,1,1,24),-- Sealed Dark Crystal Gaiters
(21753,5293,1,1,1,11),-- Sealed Tallum Plate Armor
(21753,5520,1,1,1,2201),-- Sealed Dark Crystal Breastplate Pattern
(21753,5521,1,1,1,1553),-- Sealed Tallum Plate Armor Pattern
(21753,5524,1,1,1,3364),-- Sealed Dark Crystal Gaiters Pattern
(21753,1344,30,90,2,83333),-- Mithril Arrow
(21753,1864,1,3,2,200000),-- Stem
(21753,1869,1,1,2,200000),-- Iron Ore
(21753,1880,1,1,2,17241),-- Steel
(21753,4043,1,1,2,9615),-- Asofe
(21753,5550,1,1,2,3195),-- Durable Metal Plate
(21753,6901,1,1,2,293),-- Recipe: Shining Arrow (100%)
(21753,8731,1,1,200,1502),-- Life Stone: level 70
(21753,8741,1,1,200,456),-- Mid-Grade Life Stone: level 70
(21753,8751,1,1,200,72),-- High-Grade Life Stone: level 70
(21754,4042,1,1,-1,122379),-- Enria
(21754,5277,1,1,-1,3263),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21754,5527,1,1,-1,87790),-- Sealed Helm of Nightmare Design
(21754,57,2956,6030,0,700000),-- Adena
(21754,288,1,1,1,12),-- Carnage Bow
(21754,5535,1,1,1,3701),-- Carnage Bow Stave
(21754,1868,1,3,2,200000),-- Thread
(21754,1873,1,1,2,90909),-- Silver Nugget
(21754,1876,1,1,2,52632),-- Mithril Ore
(21754,1895,1,1,2,76923),-- Metallic Fiber
(21754,4042,1,1,2,3165),-- Enria
(21754,5167,1,1,2,160),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21754,8731,1,1,200,3876),-- Life Stone: level 70
(21754,8741,1,1,200,352),-- Mid-Grade Life Stone: level 70
(21755,729,1,1,-1,842),-- Scroll: Enchant Weapon (Grade A)
(21755,1874,1,1,-1,505293),-- Oriharukon Ore
(21755,4043,1,1,-1,252647),-- Asofe
(21755,57,3054,6219,0,700000),-- Adena
(21755,6323,1,1,1,43),-- Sealed Phoenix Necklace
(21755,6324,1,1,1,57),-- Sealed Phoenix Earring
(21755,6325,1,1,1,86),-- Sealed Phoenix Ring
(21755,6341,1,1,1,10321),-- Sealed Phoenix Earring Gemstone
(21755,6343,1,1,1,7979),-- Sealed Phoenix Necklace Beads
(21755,6345,1,1,1,14275),-- Sealed Phoenix Ring Gemstone
(21755,1808,1,1,2,166),-- Recipe: Soulshot: S Grade
(21755,1865,1,1,2,250000),-- Varnish
(21755,1870,1,1,2,250000),-- Coal
(21755,1874,1,1,2,14925),-- Oriharukon Ore
(21755,4039,1,1,2,8264),-- Mold Glue
(21755,4043,1,1,2,2584),-- Asofe
(21755,8731,1,1,200,2066),-- Life Stone: level 70
(21755,8741,1,1,200,517),-- Mid-Grade Life Stone: level 70
(21756,5466,1,1,-1,1679),-- Recipe: Sword of Miracles(60%)
(21756,5524,1,1,-1,44675),-- Sealed Dark Crystal Gaiters Pattern
(21756,5525,1,1,-1,70068),-- Sealed Dark Crystal Helmet Design
(21756,57,1828,3720,0,700000),-- Adena
(21756,288,1,1,1,8),-- Carnage Bow
(21756,5535,1,1,1,2284),-- Carnage Bow Stave
(21756,1868,1,1,2,250000),-- Thread
(21756,1873,1,1,2,58824),-- Silver Nugget
(21756,1876,1,1,2,26316),-- Mithril Ore
(21756,1895,1,1,2,47619),-- Metallic Fiber
(21756,4042,1,1,2,3311),-- Enria
(21756,5440,1,1,2,83),-- Recipe: Elysian(60%)
(21756,8730,1,1,200,2577),-- Life Stone: level 67
(21756,8740,1,1,200,736),-- Mid-Grade Life Stone: level 67
(21757,4041,1,1,-1,56546),-- Mold Hardener
(21757,5282,1,1,-1,2890),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21757,5528,1,1,-1,77747),-- Sealed Majestic Circlet Design
(21757,57,2617,5341,0,700000),-- Adena
(21757,235,1,1,1,11),-- Bloody Orchid
(21757,5536,1,1,1,3268),-- Bloody Orchid Head
(21757,1344,20,60,2,171502),-- Mithril Arrow
(21757,1866,1,1,2,182935),-- Suede
(21757,1871,1,3,2,91468),-- Charcoal
(21757,1877,1,1,2,10977),-- Adamantite Nugget
(21757,4040,1,1,2,5488),-- Mold Lubricant
(21757,4041,1,1,2,2387),-- Mold Hardener
(21757,5272,1,1,2,122),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21758,730,1,1,-1,4738),-- Scroll: Enchant Armor (Grade A)
(21758,5526,1,1,-1,87899),-- Sealed Tallum Helm Design
(21758,6335,1,1,-1,21870),-- Recipe: Sealed Majestic Necklace(70%)
(21758,57,2294,4666,0,700000),-- Adena
(21758,235,1,1,1,10),-- Bloody Orchid
(21758,5536,1,1,1,2857),-- Bloody Orchid Head
(21758,1343,20,60,2,171389),-- Silver Arrow
(21758,1865,1,3,2,79982),-- Varnish
(21758,1870,2,6,2,59987),-- Coal
(21758,1875,1,1,2,15997),-- Stone of Purity
(21758,1879,1,1,2,39991),-- Cokes
(21758,4040,1,1,2,4799),-- Mold Lubricant
(21758,5442,1,1,2,89),-- Recipe: Soul Bow(60%)
(21758,8740,1,1,200,4695),-- Mid-Grade Life Stone: level 67
(21759,5466,1,1,-1,2270),-- Recipe: Sword of Miracles(60%)
(21759,5524,1,1,-1,60409),-- Sealed Dark Crystal Gaiters Pattern
(21759,5525,1,1,-1,94746),-- Sealed Dark Crystal Helmet Design
(21759,57,2470,5029,0,700000),-- Adena
(21759,212,1,1,1,5),-- Dasparion\'s Staff
(21759,2504,1,1,1,5),-- Meteor Shower
(21759,5532,1,1,1,1526),-- Meteor Shower Head
(21759,5543,1,1,1,1541),-- Dasparion\'s Staff Edge
(21759,1344,20,60,2,161648),-- Mithril Arrow
(21759,1866,1,1,2,172424),-- Suede
(21759,1871,1,3,2,86212),-- Charcoal
(21759,1877,1,1,2,10346),-- Adamantite Nugget
(21759,3036,1,1,2,115),-- Recipe: Spiritshot S
(21759,4040,1,1,2,5173),-- Mold Lubricant
(21759,4041,1,1,2,2250),-- Mold Hardener
(21759,8731,1,1,200,4274),-- Life Stone: level 70
(21760,1885,1,2,-1,498859),-- High Grade Suede
(21760,5220,1,1,-1,239453),-- Metal Hardener
(21760,5532,1,1,-1,20187),-- Meteor Shower Head
(21760,57,2415,4913,0,700000),-- Adena
(21760,5290,1,1,1,39),-- Sealed Dark Crystal Gloves
(21760,5291,1,1,1,39),-- Sealed Dark Crystal Boots
(21760,5295,1,1,1,39),-- Sealed Tallum Gloves
(21760,5296,1,1,1,39),-- Sealed Tallum Boots
(21760,5496,1,1,1,5183),-- Sealed Dark Crystal Boots Lining
(21760,5497,1,1,1,5183),-- Sealed Tallum Boots Lining
(21760,5508,1,1,1,5183),-- Sealed Dark Crystal Gloves Design
(21760,5509,1,1,1,5183),-- Sealed Tallum Gloves Design
(21760,1866,1,1,2,125000),-- Suede
(21760,1871,1,1,2,200000),-- Charcoal
(21760,1885,1,1,2,10753),-- High Grade Suede
(21760,4041,1,1,2,5348),-- Mold Hardener
(21760,5220,1,1,2,5348),-- Metal Hardener
(21760,5448,1,1,2,110),-- Recipe: Soul Separator(60%)
(21760,8730,1,1,200,5348),-- Life Stone: level 67
(21760,8740,1,1,200,2674),-- Mid-Grade Life Stone: level 67
(21761,730,1,1,-1,5378),-- Scroll: Enchant Armor (Grade A)
(21761,5526,1,1,-1,99769),-- Sealed Tallum Helm Design
(21761,6335,1,1,-1,24823),-- Recipe: Sealed Majestic Necklace(70%)
(21761,57,2601,5296,0,700000),-- Adena
(21761,269,1,1,1,11),-- Blood Tornado
(21761,5539,1,1,1,3215),-- Blood Tornado Edge
(21761,1867,1,1,2,250000),-- Animal Skin
(21761,1872,1,3,2,125000),-- Animal Bone
(21761,1875,1,1,2,14493),-- Stone of Purity
(21761,1881,1,1,2,29412),-- Coarse Bone Powder
(21761,3957,1,1,2,142),-- Recipe: Blessed Spiritshot S
(21761,4041,1,1,2,1209),-- Mold Hardener
(21761,8731,1,1,200,2415),-- Life Stone: level 70
(21761,8741,1,1,200,1209),-- Mid-Grade Life Stone: level 70
(21761,8751,1,1,200,196),-- High-Grade Life Stone: level 70
(21762,5529,1,1,-1,19889),-- Dragon Slayer Edge
(21762,5533,1,1,-1,19885),-- Elysian Head
(21762,6901,1,1,-1,7249),-- Recipe: Shining Arrow (100%)
(21762,57,2918,5954,0,700000),-- Adena
(21762,269,1,1,1,12),-- Blood Tornado
(21762,5539,1,1,1,3612),-- Blood Tornado Edge
(21762,1867,1,3,2,166667),-- Animal Skin
(21762,1872,1,1,2,333333),-- Animal Bone
(21762,1882,1,1,2,58824),-- Leather
(21762,1894,1,1,2,10526),-- Crafted Leather
(21762,4042,1,1,2,3953),-- Enria
(21762,5277,1,1,2,159),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21762,8731,1,1,200,807),-- Life Stone: level 70
(21763,5535,1,1,-1,8315),-- Carnage Bow Stave
(21763,6337,1,1,-1,12497),-- Recipe: Sealed Majestic Earring(70%)
(21763,6339,1,1,-1,18746),-- Recipe: Sealed Majestic Ring(70%)
(21763,57,983,2000,0,700000),-- Adena
(21763,212,1,1,1,2),-- Dasparion\'s Staff
(21763,2504,1,1,1,2),-- Meteor Shower
(21763,8346,1,1,1,316),-- Hammer Piece of Destroyer
(21763,7899,1,1,1,2),-- Destroyer Hammer
(21763,8341,1,1,1,694),-- Spiritual Eye Piece
(21763,7894,1,1,1,2),-- Spiritual Eye
(21763,5532,1,1,1,607),-- Meteor Shower Head
(21763,5543,1,1,1,613),-- Dasparion\'s Staff Edge
(21763,1343,10,30,2,83333),-- Silver Arrow
(21763,1867,1,1,2,83333),-- Animal Skin
(21763,1872,1,1,2,83333),-- Animal Bone
(21763,1882,1,1,2,14706),-- Leather
(21763,1894,1,1,2,3158),-- Crafted Leather
(21763,4042,1,1,2,625),-- Enria
(21763,5450,1,1,2,34),-- Recipe: Dragon Grinder(60%)
(21763,6671,1,1,2,5000),-- Deluxe Chest Key - Grade 7
(21763,8730,1,1,200,1250),-- Life Stone: level 67
(21764,4040,1,2,-1,305197),-- Mold Lubricant
(21764,4042,1,1,-1,254331),-- Enria
(21764,5534,1,1,-1,41894),-- Soul Bow Stave
(21764,57,6142,12533,0,700000),-- Adena
(21764,269,1,1,1,25),-- Blood Tornado
(21764,5539,1,1,1,7602),-- Blood Tornado Edge
(21764,1344,20,60,2,333333),-- Mithril Arrow
(21764,1895,1,1,2,125000),-- Metallic Fiber
(21764,4040,1,1,2,8696),-- Mold Lubricant
(21764,4042,1,1,2,6536),-- Enria
(21764,4043,1,1,2,14706),-- Asofe
(21764,5282,1,1,2,83),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21764,8731,1,1,200,4545),-- Life Stone: level 70
(21764,8741,1,1,200,708),-- Mid-Grade Life Stone: level 70
(21764,8751,1,1,200,83),-- High-Grade Life Stone: level 70
(21765,1808,1,1,-1,6911),-- Recipe: Soulshot: S Grade
(21765,3036,1,1,-1,6911),-- Recipe: Spiritshot S
(21765,5537,1,1,-1,42658),-- Soul Separator Head
(21765,57,6259,12770,0,700000),-- Adena
(21765,288,1,1,1,26),-- Carnage Bow
(21765,5535,1,1,1,7835),-- Carnage Bow Stave
(21765,1868,3,9,2,166667),-- Thread
(21765,1873,1,1,2,200000),-- Silver Nugget
(21765,1889,1,1,2,30303),-- Compound Braid
(21765,4044,1,1,2,15625),-- Thons
(21765,5549,1,1,2,47619),-- Metallic Thread
(21765,6901,1,1,2,789),-- Recipe: Shining Arrow (100%)
(21765,8731,1,1,200,4444),-- Life Stone: level 70
(21765,8741,1,1,200,1495),-- Mid-Grade Life Stone: level 70
(21765,8751,1,1,200,124),-- High-Grade Life Stone: level 70
(21766,1875,1,1,-1,40424),-- Stone of Purity
(21766,4041,1,1,-1,5273),-- Mold Hardener
(21766,5538,1,1,-1,1663),-- Dragon Grinder Edge
(21766,57,244,498,0,700000),-- Adena
(21766,235,1,1,1,1),-- Bloody Orchid
(21766,5536,1,1,1,306),-- Bloody Orchid Head
(21766,1344,10,30,2,20408),-- Mithril Arrow
(21766,1808,1,1,2,12),-- Recipe: Soulshot: S Grade
(21766,1867,1,1,2,21739),-- Animal Skin
(21766,1872,1,1,2,22727),-- Animal Bone
(21766,1875,1,1,2,1178),-- Stone of Purity
(21766,1881,1,1,2,2494),-- Coarse Bone Powder
(21766,4041,1,1,2,277),-- Mold Hardener
(21766,8731,1,1,200,277),-- Life Stone: level 70
(21766,8741,1,1,200,139),-- Mid-Grade Life Stone: level 70
(21767,729,1,1,-1,161),-- Scroll: Enchant Weapon (Grade A)
(21767,4044,1,1,-1,48251),-- Thons
(21767,5536,1,1,-1,4925),-- Bloody Orchid Head
(21767,57,584,1188,0,700000),-- Adena
(21767,269,1,1,1,3),-- Blood Tornado
(21767,5539,1,1,1,721),-- Blood Tornado Edge
(21767,1868,1,1,2,83333),-- Thread
(21767,1873,1,1,2,17544),-- Silver Nugget
(21767,1889,1,1,2,2079),-- Compound Braid
(21767,4044,1,1,2,2151),-- Thons
(21767,5456,1,1,2,148),-- Recipe: Tallum Glaive(60%)
(21767,5549,1,1,2,4608),-- Metallic Thread
(21767,8730,1,1,200,445),-- Life Stone: level 67
(21767,8740,1,1,200,74),-- Mid-Grade Life Stone: level 67
(21768,5535,1,1,-1,50145),-- Carnage Bow Stave
(21768,6337,1,1,-1,75366),-- Recipe: Sealed Majestic Earring(70%)
(21768,6339,1,1,-1,113048),-- Recipe: Sealed Majestic Ring(70%)
(21768,57,5923,12058,0,700000),-- Adena
(21768,235,1,1,1,24),-- Bloody Orchid
(21768,5536,1,1,1,7385),-- Bloody Orchid Head
(21768,1866,1,3,2,90909),-- Suede
(21768,1871,1,3,2,200000),-- Charcoal
(21768,1877,1,1,2,18868),-- Adamantite Nugget
(21768,4040,1,1,2,8475),-- Mold Lubricant
(21768,4041,1,1,2,3817),-- Mold Hardener
(21768,5162,1,1,2,356),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21768,8731,1,1,200,4878),-- Life Stone: level 70
(21768,8741,1,1,200,533),-- Mid-Grade Life Stone: level 70
(21768,8751,1,1,200,356),-- High-Grade Life Stone: level 70
(21769,729,1,1,-1,1446),-- Scroll: Enchant Weapon (Grade A)
(21769,5478,1,1,-1,109401),-- Sealed Dark Crystal Leather Armor Pattern
(21769,6671,1,3,-1,722777),-- Deluxe Chest Key - Grade 7
(21769,57,5248,10676,0,700000),-- Adena
(21769,6671,1,1,2,500000),-- Deluxe Chest Key - Grade 7
(21769,8730,1,1,200,45455),-- Life Stone: level 67
(21769,8740,1,1,200,7463),-- Mid-Grade Life Stone: level 67
(21769,8750,1,1,200,365),-- High-Grade Life Stone: level 67
(21770,1894,1,2,-1,500270),-- Crafted Leather
(21770,6671,1,3,-1,792095),-- Deluxe Chest Key - Grade 7
(21770,57,5746,11699,0,700000),-- Adena
(21770,6671,1,3,2,250000),-- Deluxe Chest Key - Grade 7
(21770,8731,1,1,200,41667),-- Life Stone: level 70
(21770,8741,1,1,200,10870),-- Mid-Grade Life Stone: level 70
(21770,8751,1,1,200,2801),-- High-Grade Life Stone: level 70
(21771,729,1,1,-1,1425),-- Scroll: Enchant Weapon (Grade A)
(21771,730,1,1,-1,10688),-- Scroll: Enchant Armor (Grade A)
(21771,5479,1,1,-1,75894),-- Sealed Tallum Leather Armor Pattern
(21771,57,5173,10525,0,700000),-- Adena
(21771,235,1,1,1,21),-- Bloody Orchid
(21771,5536,1,1,1,6446),-- Bloody Orchid Head
(21771,729,1,1,2,133),-- Scroll: Enchant Weapon (Grade A)
(21771,730,1,1,2,465),-- Scroll: Enchant Armor (Grade A)
(21771,8730,1,1,200,6061),-- Life Stone: level 67
(21771,8740,1,1,200,1395),-- Mid-Grade Life Stone: level 67
(21771,8750,1,1,200,133),-- High-Grade Life Stone: level 67
(21772,729,1,1,-1,1657),-- Scroll: Enchant Weapon (Grade A)
(21772,730,1,1,-1,12428),-- Scroll: Enchant Armor (Grade A)
(21772,6901,1,1,-1,14913),-- Recipe: Shining Arrow (100%)
(21772,57,6003,12248,0,700000),-- Adena
(21772,212,1,1,1,13),-- Dasparion\'s Staff
(21772,2504,1,1,1,13),-- Meteor Shower
(21772,8346,1,1,1,1262),-- Hammer Piece of Destroyer
(21772,78998,1,1,1,13),-- Destroyer Hammer
(21772,8341,1,1,1,1727),-- Spiritual Eye Piece
(21772,7894,1,1,1,13),-- Spiritual Eye
(21772,5532,1,1,1,3713),-- Meteor Shower Head
(21772,5543,1,1,1,3748),-- Dasparion\'s Staff Edge
(21772,729,1,1,2,245),-- Scroll: Enchant Weapon (Grade A)
(21772,730,1,1,2,299),-- Scroll: Enchant Armor (Grade A)
(21772,8731,1,1,200,5556),-- Life Stone: level 70
(21772,8741,1,1,200,1733),-- Mid-Grade Life Stone: level 70
(21772,8751,1,1,200,120),-- High-Grade Life Stone: level 70
(21772,8761,1,1,200,60),-- Top-Grade Life Stone: level 70
(21773,729,1,1,-1,1510),-- Scroll: Enchant Weapon (Grade A)
(21773,730,1,1,-1,11328),-- Scroll: Enchant Armor (Grade A)
(21773,5482,1,1,-1,179418),-- Sealed Dark Crystalline Leggings Design
(21773,57,5478,11154,0,700000),-- Adena
(21773,288,1,1,1,22),-- Carnage Bow
(21773,5535,1,1,1,6851),-- Carnage Bow Stave
(21773,729,1,1,2,224),-- Scroll: Enchant Weapon (Grade A)
(21773,730,1,1,2,618),-- Scroll: Enchant Armor (Grade A)
(21773,8731,1,1,200,5495),-- Life Stone: level 70
(21773,8741,1,1,200,1992),-- Mid-Grade Life Stone: level 70
(21773,8751,1,1,200,137),-- High-Grade Life Stone: level 70
(21774,1876,1,3,-1,937757),-- Mithril Ore
(21774,1895,1,5,-1,893102),-- Metallic Fiber
(21774,5546,1,1,-1,28259),-- Sword of Miracles Edge
(21774,57,3773,7702,0,700000),-- Adena
(21774,305,1,1,1,11),-- Tallum Glaive
(21774,5541,1,1,1,3798),-- Tallum Glaive Edge
(21774,959,1,1,2,19),-- Scroll: Enchant Weapon (Grade S)
(21774,1868,3,9,2,111111),-- Thread
(21774,1873,1,1,2,111111),-- Silver Nugget
(21774,1876,1,1,2,50000),-- Mithril Ore
(21774,1895,1,1,2,90909),-- Metallic Fiber
(21774,4042,1,1,2,5208),-- Enria
(21774,8732,1,1,200,1608),-- Life Stone: level 76
(21774,8742,1,1,200,803),-- Mid-Grade Life Stone: level 76
(21775,960,1,1,-1,4468),-- Scroll: Enchant Armor (Grade S)
(21775,4041,1,1,-1,97124),-- Mold Hardener
(21775,5220,1,1,-1,446773),-- Metal Hardener
(21775,57,4493,9173,0,700000),-- Adena
(21775,5311,1,1,1,17),-- Sealed Armor of Nightmare
(21775,5316,1,1,1,17),-- Sealed Majestic Plate Armor
(21775,5522,1,1,1,2809),-- Sealed Armor of Nightmare Pattern
(21775,5523,1,1,1,2809),-- Sealed Majestic Plate Armor Pattern
(21775,959,1,1,2,22),-- Scroll: Enchant Weapon (Grade S)
(21775,1866,1,1,2,250000),-- Suede
(21775,1871,1,3,2,200000),-- Charcoal
(21775,1885,1,1,2,33333),-- High Grade Suede
(21775,4041,1,1,2,3690),-- Mold Hardener
(21775,5220,1,1,2,16129),-- Metal Hardener
(21775,8732,1,1,200,2874),-- Life Stone: level 76
(21775,8742,1,1,200,820),-- Mid-Grade Life Stone: level 76
(21776,6707,1,1,-1,45354),-- Sealed Draconic Leather Armor Part
(21776,6708,1,1,-1,130329),-- Sealed Draconic Leather Gloves Fabric
(21776,6709,1,1,-1,130329),-- Sealed Draconic Leather Boots Design
(21776,57,3684,7518,0,700000),-- Adena
(21776,5326,1,1,1,18),-- Sealed Robe of Nightmare
(21776,5329,1,1,1,18),-- Sealed Majestic Robe
(21776,5487,1,1,1,3045),-- Sealed Robe of Nightmare Fabric
(21776,5488,1,1,1,3045),-- Sealed Majestic Robe Fabric
(21776,960,1,1,2,498),-- Scroll: Enchant Armor (Grade S)
(21776,1344,40,120,2,90909),-- Mithril Arrow
(21776,1867,1,3,2,200000),-- Animal Skin
(21776,1872,1,3,2,125000),-- Animal Bone
(21776,1882,1,1,2,66667),-- Leather
(21776,1894,1,1,2,12987),-- Crafted Leather
(21776,4042,1,1,2,3650),-- Enria
(21776,8732,1,1,200,2488),-- Life Stone: level 76
(21776,8742,1,1,200,829),-- Mid-Grade Life Stone: level 76
(21776,8752,1,1,200,166),-- High-Grade Life Stone: level 76
(21777,6691,1,1,-1,20740),-- Angel Slayer Blade
(21777,6693,1,1,-1,21116),-- Dragon Hunter Axe Blade
(21777,6694,1,1,-1,21116),-- Saint Spear Blade
(21777,57,3860,7880,0,700000),-- Adena
(21777,5326,1,1,1,19),-- Sealed Robe of Nightmare
(21777,5329,1,1,1,19),-- Sealed Majestic Robe
(21777,5487,1,1,1,3192),-- Sealed Robe of Nightmare Fabric
(21777,5488,1,1,1,3192),-- Sealed Majestic Robe Fabric
(21777,960,1,1,2,163),-- Scroll: Enchant Armor (Grade S)
(21777,1344,30,90,2,125000),-- Mithril Arrow
(21777,1864,5,15,2,66667),-- Stem
(21777,1869,2,6,2,71429),-- Iron Ore
(21777,1880,1,1,2,26316),-- Steel
(21777,4043,1,1,2,12195),-- Asofe
(21777,5550,1,1,2,5051),-- Durable Metal Plate
(21777,8732,1,1,200,2890),-- Life Stone: level 76
(21778,960,1,1,-1,4004),-- Scroll: Enchant Armor (Grade S)
(21778,4042,1,1,-1,166816),-- Enria
(21778,4043,1,1,-1,333633),-- Asofe
(21778,57,4028,8221,0,700000),-- Adena
(21778,6326,1,1,1,38),-- Sealed Majestic Necklace
(21778,6327,1,1,1,51),-- Sealed Majestic Earring
(21778,6328,1,1,1,76),-- Sealed Majestic Ring
(21778,6342,1,1,1,11162),-- Sealed Majestic Earring Gemstone
(21778,6344,1,1,1,7224),-- Sealed Majestic Necklace Beads
(21778,6346,1,1,1,16043),-- Sealed Majestic Ring Gemstone
(21778,959,1,1,2,24),-- Scroll: Enchant Weapon (Grade S)
(21778,1895,1,1,2,100000),-- Metallic Fiber
(21778,4040,1,1,2,7042),-- Mold Lubricant
(21778,4042,1,1,2,4348),-- Enria
(21778,4043,1,1,2,19711),-- Asofe
(21778,8732,1,1,200,3802),-- Life Stone: level 76
(21778,8742,1,1,200,1629),-- Mid-Grade Life Stone: level 76
(21779,5162,1,1,-1,2697),-- Recipe: Spiritshot (S) Compressed Package (100%)
(21779,5167,1,1,-1,2697),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21779,6342,1,1,-1,137475),-- Sealed Majestic Earring Gemstone
(21779,57,2441,4984,0,700000),-- Adena
(21779,5320,1,1,1,12),-- Sealed Leather Armor of Nightmare
(21779,5323,1,1,1,12),-- Sealed Majestic Leather Armor
(21779,5480,1,1,1,2012),-- Sealed Leather Armor of Nightmare Fabric
(21779,5481,1,1,1,2012),-- Sealed Majestic Leather Armor Fabric
(21779,960,1,1,2,120),-- Scroll: Enchant Armor (Grade S)
(21779,1344,10,30,2,250000),-- Mithril Arrow
(21779,1894,1,1,2,5952),-- Crafted Leather
(21779,4039,1,1,2,8772),-- Mold Glue
(21779,4041,1,1,2,2809),-- Mold Hardener
(21779,4044,1,1,2,8403),-- Thons
(21779,8732,1,1,200,3506),-- Life Stone: level 76
(21779,8742,1,1,200,351),-- Mid-Grade Life Stone: level 76
(21780,959,1,1,-1,340),-- Scroll: Enchant Weapon (Grade S)
(21780,1894,1,1,-1,298162),-- Crafted Leather
(21780,4042,1,1,-1,141627),-- Enria
(21780,57,3419,6979,0,700000),-- Adena
(21780,6326,1,1,1,33),-- Sealed Majestic Necklace
(21780,6327,1,1,1,43),-- Sealed Majestic Earring
(21780,6328,1,1,1,65),-- Sealed Majestic Ring
(21780,6342,1,1,1,9477),-- Sealed Majestic Earring Gemstone
(21780,6344,1,1,1,6133),-- Sealed Majestic Necklace Beads
(21780,6346,1,1,1,13620),-- Sealed Majestic Ring Gemstone
(21780,959,1,1,2,17),-- Scroll: Enchant Weapon (Grade S)
(21780,1867,1,3,2,200000),-- Animal Skin
(21780,1872,1,3,2,111111),-- Animal Bone
(21780,1882,1,1,2,76923),-- Leather
(21780,1894,1,1,2,15625),-- Crafted Leather
(21780,4042,1,1,2,4348),-- Enria
(21780,8732,1,1,200,1739),-- Life Stone: level 76
(21781,4040,1,1,-1,152264),-- Mold Lubricant
(21781,4041,1,1,-1,66202),-- Mold Hardener
(21781,6344,1,1,-1,111606),-- Sealed Majestic Necklace Beads
(21781,57,3063,6253,0,700000),-- Adena
(21781,5311,1,1,1,11),-- Sealed Armor of Nightmare
(21781,5316,1,1,1,11),-- Sealed Majestic Plate Armor
(21781,5522,1,1,1,1915),-- Sealed Armor of Nightmare Pattern
(21781,5523,1,1,1,1915),-- Sealed Majestic Plate Armor Pattern
(21781,959,1,1,2,16),-- Scroll: Enchant Weapon (Grade S)
(21781,1866,1,1,2,166667),-- Suede
(21781,1871,1,1,2,250000),-- Charcoal
(21781,1877,1,1,2,10204),-- Adamantite Nugget
(21781,4040,1,1,2,5988),-- Mold Lubricant
(21781,4041,1,1,2,3260),-- Mold Hardener
(21781,8732,1,1,200,5988),-- Life Stone: level 76
(21781,8752,1,1,200,854),-- High-Grade Life Stone: level 76
(21782,6710,1,1,-1,131297),-- Sealed Draconic Leather Helmet Pattern
(21782,6711,1,1,-1,40103),-- Sealed Major Arcana Robe Part
(21782,6712,1,1,-1,115239),-- Sealed Major Arcana Gloves fabric
(21782,57,3257,6648,0,700000),-- Adena
(21782,5313,1,1,1,34),-- Sealed Gloves of Nightmare
(21782,5314,1,1,1,34),-- Sealed Boots of Nightmare
(21782,5318,1,1,1,34),-- Sealed Majestic Gloves
(21782,5319,1,1,1,34),-- Sealed Majestic Boots
(21782,5502,1,1,1,5047),-- Sealed Boots of Nightmare Lining
(21782,5503,1,1,1,5047),-- Sealed Majestic Boots Lining
(21782,5514,1,1,1,5047),-- Sealed Gloves of Nightmare Design
(21782,5515,1,1,1,5047),-- Sealed Majestic Gloves Design
(21782,960,1,1,2,137),-- Scroll: Enchant Armor (Grade S)
(21782,1344,30,90,2,111111),-- Mithril Arrow
(21782,1868,4,12,2,76923),-- Thread
(21782,1873,1,1,2,90909),-- Silver Nugget
(21782,1876,1,1,2,66667),-- Mithril Ore
(21782,1895,1,1,2,71429),-- Metallic Fiber
(21782,4042,1,1,2,898),-- Enria
(21782,8732,1,1,200,2695),-- Life Stone: level 76
(21783,5272,1,1,-1,3562),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21783,5480,1,1,-1,35965),-- Sealed Leather Armor of Nightmare Fabric
(21783,6346,1,1,-1,260950),-- Sealed Majestic Ring Gemstone
(21783,57,3224,6583,0,700000),-- Adena
(21783,5326,1,1,1,16),-- Sealed Robe of Nightmare
(21783,5329,1,1,1,16),-- Sealed Majestic Robe
(21783,5487,1,1,1,2666),-- Sealed Robe of Nightmare Fabric
(21783,5488,1,1,1,2666),-- Sealed Majestic Robe Fabric
(21783,960,1,1,2,136),-- Scroll: Enchant Armor (Grade S)
(21783,1344,20,60,2,142857),-- Mithril Arrow
(21783,1865,1,3,2,90909),-- Varnish
(21783,1870,2,6,2,62500),-- Coal
(21783,1875,1,1,2,15873),-- Stone of Purity
(21783,1879,1,1,2,34483),-- Cokes
(21783,4040,1,1,2,6024),-- Mold Lubricant
(21783,8732,1,1,200,2786),-- Life Stone: level 76
(21783,8752,1,1,200,1859),-- High-Grade Life Stone: level 76
(21784,959,1,1,-1,341),-- Scroll: Enchant Weapon (Grade S)
(21784,4041,1,1,-1,74112),-- Mold Hardener
(21784,4044,1,1,-1,284097),-- Thons
(21784,57,3430,7000,0,700000),-- Adena
(21784,5312,1,1,1,32),-- Sealed Helm of Nightmare
(21784,5315,1,1,1,46),-- Sealed Shield of Nightmare
(21784,5317,1,1,1,32),-- Sealed Majestic Circlet
(21784,5495,1,1,1,6372),-- Sealed Shield of Nightmare Fragment
(21784,5527,1,1,1,5017),-- Sealed Helm of Nightmare Design
(21784,5528,1,1,1,5017),-- Sealed Majestic Circlet Design
(21784,959,1,1,2,21),-- Scroll: Enchant Weapon (Grade S)
(21784,1894,1,1,2,15385),-- Crafted Leather
(21784,4039,1,1,2,9174),-- Mold Glue
(21784,4041,1,1,2,1531),-- Mold Hardener
(21784,4044,1,1,2,10753),-- Thons
(21784,8732,1,1,200,4587),-- Life Stone: level 76
(21784,8742,1,1,200,1020),-- Mid-Grade Life Stone: level 76
(21785,6695,1,1,-1,20090),-- Demon Splinter Blade
(21785,6696,1,1,-1,20802),-- Heavens Divider Edge
(21785,6697,1,1,-1,20090),-- Arcana Mace Head
(21785,57,3811,7780,0,700000),-- Adena
(21785,5313,1,1,1,40),-- Sealed Gloves of Nightmare
(21785,5314,1,1,1,40),-- Sealed Boots of Nightmare
(21785,5318,1,1,1,40),-- Sealed Majestic Gloves
(21785,5319,1,1,1,40),-- Sealed Majestic Boots
(21785,5502,1,1,1,5908),-- Sealed Boots of Nightmare Lining
(21785,5503,1,1,1,5908),-- Sealed Majestic Boots Lining
(21785,5514,1,1,1,5908),-- Sealed Gloves of Nightmare Design
(21785,5515,1,1,1,5908),-- Sealed Majestic Gloves Design
(21785,960,1,1,2,160),-- Scroll: Enchant Armor (Grade S)
(21785,1344,30,90,2,125000),-- Mithril Arrow
(21785,1868,4,12,2,71429),-- Thread
(21785,1873,1,1,2,111111),-- Silver Nugget
(21785,1889,1,1,2,23256),-- Compound Braid
(21785,4044,1,1,2,7937),-- Thons
(21785,5549,1,1,2,27778),-- Metallic Thread
(21785,8732,1,1,200,497),-- Life Stone: level 76
(21785,8742,1,1,200,1988),-- Mid-Grade Life Stone: level 76
(21786,5220,1,1,-1,130512),-- Metal Hardener
(21786,5277,1,1,-1,1450),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(21786,5481,1,1,-1,14641),-- Sealed Majestic Leather Armor Fabric
(21786,57,1313,2680,0,700000),-- Adena
(21786,6326,1,1,1,13),-- Sealed Majestic Necklace
(21786,6327,1,1,1,17),-- Sealed Majestic Earring
(21786,6328,1,1,1,25),-- Sealed Majestic Ring
(21786,6342,1,1,1,3639),-- Sealed Majestic Earring Gemstone
(21786,6344,1,1,1,2355),-- Sealed Majestic Necklace Beads
(21786,6346,1,1,1,5230),-- Sealed Majestic Ring Gemstone
(21786,1866,1,1,2,71429),-- Suede
(21786,1871,1,1,2,111111),-- Charcoal
(21786,1885,1,1,2,9174),-- High Grade Suede
(21786,4041,1,1,2,1397),-- Mold Hardener
(21786,5220,1,1,2,5000),-- Metal Hardener
(21786,6672,1,1,2,12346),-- Deluxe Chest Key - Grade 8
(21786,8732,1,1,200,1919),-- Life Stone: level 76
(21787,959,1,1,-1,798),-- Scroll: Enchant Weapon (Grade S)
(21787,960,1,1,-1,7976),-- Scroll: Enchant Armor (Grade S)
(21787,6688,1,1,-1,43102),-- Forgotten Blade Edge
(21787,57,8022,16377,0,700000),-- Adena
(21787,5312,1,1,1,75),-- Sealed Helm of Nightmare
(21787,5315,1,1,1,107),-- Sealed Shield of Nightmare
(21787,5317,1,1,1,75),-- Sealed Majestic Circlet
(21787,5495,1,1,1,14907),-- Sealed Shield of Nightmare Fragment
(21787,5527,1,1,1,11737),-- Sealed Helm of Nightmare Design
(21787,5528,1,1,1,11737),-- Sealed Majestic Circlet Design
(21787,959,1,1,2,40),-- Scroll: Enchant Weapon (Grade S)
(21787,1864,4,12,2,166667),-- Stem
(21787,1865,2,6,2,125000),-- Varnish
(21787,1866,3,9,2,76923),-- Suede
(21787,1868,10,30,2,62500),-- Thread
(21787,1873,4,12,2,31250),-- Silver Nugget
(21787,8732,1,1,200,5405),-- Life Stone: level 76
(21787,8742,1,1,200,1626),-- Mid-Grade Life Stone: level 76
(21787,8752,1,1,200,148),-- High-Grade Life Stone: level 76
(21788,6698,1,1,-1,399111),-- Sealed Tateossian Earring  Part
(21788,6699,1,1,-1,442867),-- Sealed Tateossian Ring Gem
(21788,6700,1,1,-1,243251),-- Sealed Tateossian Ring Wire
(21788,57,8174,16687,0,700000),-- Adena
(21788,81,1,1,1,8),-- Dragon Slayer
(21788,151,1,1,1,8),-- Sword of Miracles 
(21788,2500,1,1,1,8),-- Dark Legion\'s Edge
(21788,5529,1,1,1,2745),-- Dragon Slayer Edge
(21788,5545,1,1,1,2745),-- Dark Legion\'s Edge Blade
(21788,5546,1,1,1,3015),-- Sword of Miracles Edge
(21788,960,1,1,2,89),-- Scroll: Enchant Armor (Grade S)
(21788,1344,400,1200,2,17544),-- Mithril Arrow
(21788,1867,13,39,2,32258),-- Animal Skin
(21788,1869,2,6,2,125000),-- Iron Ore
(21788,1870,3,9,2,100000),-- Coal
(21788,1871,3,9,2,90909),-- Charcoal
(21788,1872,8,24,2,52632),-- Animal Bone
(21788,8732,1,1,200,4367),-- Life Stone: level 76
(21788,8742,1,1,200,1161),-- Mid-Grade Life Stone: level 76
(21788,8752,1,1,200,89),-- High-Grade Life Stone: level 76
(21789,959,1,1,-1,32),-- Scroll: Enchant Weapon (Grade S)
(21789,1894,1,1,-1,27802),-- Crafted Leather
(21789,4039,1,1,-1,26412),-- Mold Glue
(21789,57,319,651,0,700000),-- Adena
(21789,164,1,1,1,1),-- Elysian
(21789,213,1,1,1,1),-- Branch of The Mother Tree
(21789,8342,1,1,1,50),-- Flaming Dragon Skull Piece
(21789,7895,1,1,1,1),-- Flaming Dragon Skull
(21789,8349,1,1,1,50),-- Doom Crusher Head
(21789,7902,1,1,1,1),-- Doom Crusher
(21789,5533,1,1,1,161),-- Elysian Head
(21789,5544,1,1,1,161),-- Branch of The Mother Tree Head
(21789,959,1,1,2,2),-- Scroll: Enchant Weapon (Grade S)
(21789,1894,1,1,2,1205),-- Crafted Leather
(21789,4039,1,1,2,1314),-- Mold Glue
(21789,4041,1,1,2,110),-- Mold Hardener
(21789,4044,1,1,2,1534),-- Thons
(21789,8732,1,1,200,329),-- Life Stone: level 76
(21789,8742,1,1,200,54),-- Mid-Grade Life Stone: level 76
(21790,5282,1,1,-1,861),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21790,5487,1,1,-1,8727),-- Sealed Robe of Nightmare Fabric
(21790,5550,1,1,-1,25841),-- Durable Metal Plate
(21790,57,780,1592,0,700000),-- Adena
(21790,5313,1,1,1,9),-- Sealed Gloves of Nightmare
(21790,5314,1,1,1,9),-- Sealed Boots of Nightmare
(21790,5318,1,1,1,9),-- Sealed Majestic Gloves
(21790,5319,1,1,1,9),-- Sealed Majestic Boots
(21790,5502,1,1,1,1209),-- Sealed Boots of Nightmare Lining
(21790,5503,1,1,1,1209),-- Sealed Majestic Boots Lining
(21790,5514,1,1,1,1209),-- Sealed Gloves of Nightmare Design
(21790,5515,1,1,1,1209),-- Sealed Majestic Gloves Design
(21790,960,1,1,2,33),-- Scroll: Enchant Armor (Grade S)
(21790,1344,10,30,2,71429),-- Mithril Arrow
(21790,1864,1,1,2,125000),-- Stem
(21790,1869,1,1,2,62500),-- Iron Ore
(21790,1880,1,1,2,5814),-- Steel
(21790,4043,1,1,2,2004),-- Asofe
(21790,5550,1,1,2,950),-- Durable Metal Plate
(21790,8732,1,1,200,528),-- Life Stone: level 76
(21790,8742,1,1,200,317),-- Mid-Grade Life Stone: level 76
(21791,960,1,1,-1,7763),-- Scroll: Enchant Armor (Grade S)
(21791,1875,1,3,-1,646906),-- Stone of Purity
(21791,4040,1,1,-1,388144),-- Mold Lubricant
(21791,57,7810,15940,0,700000),-- Adena
(21791,164,1,1,1,11),-- Elysian
(21791,213,1,1,1,11),-- Branch of The Mother Tree
(21791,8342,1,1,1,2008),-- Flaming Dragon Skull Piece
(21791,7895,1,1,1,11),-- Flaming Dragon Skull
(21791,8349,1,1,1,2141),-- Doom Crusher Head
(21791,7902,1,1,1,11),-- Doom Crusher
(21791,5533,1,1,1,3932),-- Elysian Head
(21791,5544,1,1,1,3928),-- Branch of The Mother Tree Head
(21791,959,1,1,2,39),-- Scroll: Enchant Weapon (Grade S)
(21791,1865,4,12,2,90909),-- Varnish
(21791,1870,1,3,2,200000),-- Coal
(21791,1875,1,1,2,45455),-- Stone of Purity
(21791,1879,1,1,2,111111),-- Cokes
(21791,4040,1,1,2,13158),-- Mold Lubricant
(21791,8732,1,1,200,4651),-- Life Stone: level 76
(21791,8742,1,1,200,903),-- Mid-Grade Life Stone: level 76
(21791,8752,1,1,200,129),-- High-Grade Life Stone: level 76
(21792,1894,1,2,-1,611196),-- Crafted Leather
(21792,6672,1,3,-1,725795),-- Deluxe Chest Key - Grade 8
(21792,57,7008,14306,0,700000),-- Adena
(21792,6672,1,3,2,250000),-- Deluxe Chest Key - Grade 8
(21792,8732,1,1,200,50000),-- Life Stone: level 76
(21792,8742,1,1,200,13699),-- Mid-Grade Life Stone: level 76
(21792,8752,1,1,200,1364),-- High-Grade Life Stone: level 76
(21793,6672,1,3,-1,784501),-- Deluxe Chest Key - Grade 8
(21793,6689,1,1,-1,39930),-- Basalt Battlehammer Head
(21793,6690,1,1,-1,40774),-- Imperial Staff Head
(21793,57,7577,15464,0,700000),-- Adena
(21793,6672,1,3,2,333333),-- Deluxe Chest Key - Grade 8
(21793,8732,1,1,200,55556),-- Life Stone: level 76
(21793,8742,1,1,200,16949),-- Mid-Grade Life Stone: level 76
(21793,8752,1,1,200,1206),-- High-Grade Life Stone: level 76
(21794,959,1,1,-1,687),-- Scroll: Enchant Weapon (Grade S)
(21794,960,1,1,-1,6869),-- Scroll: Enchant Armor (Grade S)
(21794,1808,1,1,-1,7632),-- Recipe: Soulshot: S Grade
(21794,57,6908,14104,0,700000),-- Adena
(21794,81,1,1,1,7),-- Dragon Slayer
(21794,151,1,1,1,7),-- Sword of Miracles 
(21794,2500,1,1,1,7),-- Dark Legion\'s Edge
(21794,5529,1,1,1,2320),-- Dragon Slayer Edge
(21794,5545,1,1,1,2320),-- Dark Legion\'s Edge Blade
(21794,5546,1,1,1,2548),-- Sword of Miracles Edge
(21794,959,1,1,2,102),-- Scroll: Enchant Weapon (Grade S)
(21794,960,1,1,2,433),-- Scroll: Enchant Armor (Grade S)
(21794,8732,1,1,200,9804),-- Life Stone: level 76
(21794,8742,1,1,200,2169),-- Mid-Grade Life Stone: level 76
(21794,8752,1,1,200,433),-- High-Grade Life Stone: level 76
(21795,959,1,1,-1,780),-- Scroll: Enchant Weapon (Grade S)
(21795,960,1,1,-1,7795),-- Scroll: Enchant Armor (Grade S)
(21795,6691,1,1,-1,42123),-- Angel Slayer Blade
(21795,57,7840,16005,0,700000),-- Adena
(21795,289,1,1,1,22),-- Soul Bow
(21795,5534,1,1,1,7902),-- Soul Bow Stave
(21795,959,1,1,2,116),-- Scroll: Enchant Weapon (Grade S)
(21795,960,1,1,2,432),-- Scroll: Enchant Armor (Grade S)
(21795,8732,1,1,200,8850),-- Life Stone: level 76
(21795,8742,1,1,200,2088),-- Mid-Grade Life Stone: level 76
(21795,8752,1,1,200,504),-- High-Grade Life Stone: level 76
(21796,959,1,1,-1,718),-- Scroll: Enchant Weapon (Grade S)
(21796,960,1,1,-1,7180),-- Scroll: Enchant Armor (Grade S)
(21796,57,7224,14744,0,700000),-- Adena
(21796,289,1,1,1,20),-- Soul Bow
(21796,5534,1,1,1,7279),-- Soul Bow Stave
(21796,959,1,1,2,107),-- Scroll: Enchant Weapon (Grade S)
(21796,960,1,1,2,269),-- Scroll: Enchant Armor (Grade S)
(21796,8732,1,1,200,10638),-- Life Stone: level 76
(21796,8742,1,1,200,1795),-- Mid-Grade Life Stone: level 76
(21797,1867,1,3,-1,869731),-- Animal Skin
(21797,1869,1,4,-1,652298),-- Iron Ore
(21797,2961,1,1,-1,35519),-- Full Plate Boots Part
(21797,57,315,642,0,700000),-- Adena
(21797,2414,1,1,1,23),-- Full Plate Helmet
(21797,2497,1,1,1,33),-- Full Plate Shield
(21797,2959,1,1,1,2083),-- Full Plate Helmet Design
(21797,2966,1,1,1,3295),-- Full Plate Shield Fragment
(21797,1867,1,1,2,30303),-- Animal Skin
(21797,1869,1,1,2,26316),-- Iron Ore
(21797,1870,1,1,2,22727),-- Coal
(21797,1871,1,1,2,25000),-- Charcoal
(21797,1872,1,1,2,38462),-- Animal Bone
(21797,4175,1,1,2,404),-- Recipe: Avadon Boots
(21797,4972,1,1,2,40),-- Recipe: Great Axe (60%)
(21797,8726,1,1,200,807),-- Life Stone: level 55
(21798,5282,1,1,-1,2192),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21798,5487,1,1,-1,22211),-- Sealed Robe of Nightmare Fabric
(21798,5488,1,1,-1,22211),-- Sealed Majestic Robe Fabric
(21798,57,1980,4055,0,700000),-- Adena
(21798,12222,1,1,1,320),-- Common Item - Sealed Dark Crystal Leather Armor
(21798,12228,1,1,1,384),-- Common Item - Sealed Dark Crystalline Leggings
(21798,12230,1,1,1,160),-- Common Item - Sealed Tallum Leather Armor
(21798,5478,1,1,1,2042),-- Sealed Dark Crystal Leather Armor Pattern
(21798,5479,1,1,1,1437),-- Sealed Tallum Leather Armor Pattern
(21798,5482,1,1,1,3205),-- Sealed Dark Crystalline Leggings Design
(21798,1344,10,30,2,166667),-- Mithril Arrow
(21798,1895,1,1,2,45455),-- Metallic Fiber
(21798,4040,1,1,2,3135),-- Mold Lubricant
(21798,4042,1,1,2,2242),-- Enria
(21798,4043,1,1,2,4717),-- Asofe
(21798,5282,1,1,2,93),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(21798,6672,1,1,2,10753),-- Deluxe Chest Key - Grade 8
(21798,8732,1,1,200,992),-- Life Stone: level 76
(21798,8742,1,1,200,288),-- Mid-Grade Life Stone: level 76
(21798,8752,1,1,200,224),-- High-Grade Life Stone: level 76
(21799,1894,1,2,-1,318295),-- Crafted Leather
(21799,5495,1,1,-1,137748),-- Sealed Shield of Nightmare Fragment
(21799,6901,1,1,-1,9071),-- Recipe: Shining Arrow (100%)
(21799,57,3642,7457,0,700000),-- Adena
(21799,11946,1,1,1,116),-- Common Item - Dasparion\'s Staff
(21799,11949,1,1,1,116),-- Common Item - Meteor Shower
(21799,8346,1,1,1,723),-- Hammer Piece of Destroyer
(21799,11958,1,1,1,116),-- Common Item - Destroyer Hammer
(21799,8341,1,1,1,1001),-- Spiritual Eye Piece
(21799,11953,1,1,1,116),-- Common Item - Spiritual Eye
(21799,5532,1,1,1,2259),-- Meteor Shower Head
(21799,5543,1,1,1,2280),-- Dasparion\'s Staff Edge
(21799,6901,1,1,2,974),-- Recipe: Shining Arrow (100%)
(21799,8732,1,1,200,5882),-- Life Stone: level 76
(21799,8742,1,1,200,1016),-- Mid-Grade Life Stone: level 76
(21799,8752,1,1,200,381),-- High-Grade Life Stone: level 76
(21800,4042,1,1,-1,76772),-- Enria
(21800,5167,1,1,-1,2047),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(21800,5529,1,1,-1,12637),-- Dragon Slayer Edge
(21800,57,1853,3785,0,700000),-- Adena
(21800,11946,1,1,1,38),-- Common Item - Dasparion\'s Staff
(21800,11949,1,1,1,38),-- Common Item - Meteor Shower
(21800,8346,1,1,1,374),-- Hammer Piece of Destroyer
(21800,11958,1,1,1,38),-- Common Item - Destroyer Hammer
(21800,8341,1,1,1,436),-- Spiritual Eye Piece
(21800,11953,1,1,1,38),-- Common Item - Spiritual Eye
(21800,5532,1,1,1,1147),-- Meteor Shower Head
(21800,5543,1,1,1,1158),-- Dasparion\'s Staff Edge
(21800,1344,10,30,2,166667),-- Mithril Arrow
(21800,1867,1,3,2,90909),-- Animal Skin
(21800,1872,1,1,2,200000),-- Animal Bone
(21800,1882,1,1,2,28571),-- Leather
(21800,1894,1,1,2,4405),-- Crafted Leather
(21800,4042,1,1,2,17060),-- Enria
(21800,5272,1,1,2,114),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(21800,8732,1,1,200,1062),-- Life Stone: level 76
(21800,8742,1,1,200,228),-- Mid-Grade Life Stone: level 76
(21801,1864,1,1,-1,469914),-- Stem
(21801,1869,1,1,-1,234957),-- Iron Ore
(21801,1926,1,1,-1,23733),-- Piece Bone Breastplate Fragment
(21801,57,107,191,0,700000),-- Adena
(21801,39,1,1,1,753),-- Boots
(21801,51,1,1,1,753),-- Bracer
(21801,1930,1,1,1,11864),-- Boot Lining
(21801,1864,1,1,2,69403),-- Stem
(21801,1869,1,1,2,34702),-- Iron Ore
(21801,1878,1,1,2,13881),-- Braided Hemp
(21801,2151,1,1,2,9667),-- Recipe: Necklace of Devotion
(21802,1928,1,1,-1,40001),-- Piece Bone Gaiters Fragment
(21802,2151,1,1,-1,76492),-- Recipe: Necklace of Devotion
(21802,2176,1,1,-1,131391),-- Recipe: Leather Boots
(21802,57,122,225,0,700000),-- Adena
(21802,625,1,1,1,1258),-- Bone Shield
(21802,1148,1,1,1,882),-- Hard Leather Helmet
(21802,1865,1,1,2,40558),-- Varnish
(21802,1870,1,1,2,40558),-- Coal
(21802,1879,1,1,2,6760),-- Cokes
(21802,2176,1,1,2,19406),-- Recipe: Leather Boots
(21803,1871,1,1,-1,317362),-- Charcoal
(21803,1899,1,1,-1,49588),-- Necklace of Devotion Chain
(21803,5220,1,1,-1,12695),-- Metal Hardener
(21803,57,137,264,0,700000),-- Adena
(21803,40,1,1,1,427),-- Leather Boots
(21803,604,1,1,1,427),-- Crafted Leather Gloves
(21803,1931,1,1,1,5821),-- Leather Boot Lining
(21803,1934,1,1,1,5821),-- Crafted Leather Gloves Lining
(21803,1866,1,1,2,23749),-- Suede
(21803,1871,1,1,2,35623),-- Charcoal
(21803,1885,1,1,2,2969),-- High Grade Suede
(21803,2152,1,1,2,6272),-- Recipe: Enchanted Necklace
(21803,5220,1,1,2,1425),-- Metal Hardener
(21804,1061,1,1,-1,221263),-- Healing Potion
(21804,1900,1,1,-1,39943),-- Enchanted Necklace Chain
(21804,2250,1,1,-1,91271),-- Recipe: Bone Arrow
(21804,57,154,308,0,700000),-- Adena
(21804,46,1,1,1,400),-- Bronze Helmet
(21804,627,1,1,1,570),-- Aspis
(21804,1867,1,1,2,66502),-- Animal Skin
(21804,1872,1,1,2,66502),-- Animal Bone
(21804,1882,1,1,2,11084),-- Leather
(21804,2250,1,1,2,12469),-- Recipe: Bone Arrow
(21805,1901,1,1,-1,60152),-- Tiger\'s Eye Earring Stone
(21805,1938,1,1,-1,29277),-- Mithril Banded Mail Material
(21805,2154,1,1,-1,46627),-- Recipe: Elven Earring
(21805,57,184,368,0,700000),-- Adena
(21805,63,1,1,1,131),-- Gauntlets
(21805,553,1,1,1,131),-- Iron Boots
(21805,1940,1,1,1,4937),-- Iron Boots Design
(21805,2422,1,1,1,131),-- Reinforced Leather Boots
(21805,2423,1,1,1,131),-- Boots of Knowledge
(21805,2446,1,1,1,131),-- Reinforced Leather Gloves
(21805,2447,1,1,1,131),-- Gloves of Knowledge
(21805,1868,1,1,2,92622),-- Thread
(21805,1873,1,1,2,18525),-- Silver Nugget
(21805,1889,1,1,2,3088),-- Compound Braid
(21805,2154,1,1,2,4959),-- Recipe: Elven Earring
(21805,5549,1,1,2,4632),-- Metallic Thread
(21806,1864,1,3,-1,512417),-- Stem
(21806,1880,1,1,-1,51242),-- Steel
(21806,1939,1,1,-1,45008),-- Mithril Banded Gaiters Material
(21806,57,216,433,0,700000),-- Adena
(21806,47,1,1,1,358),-- Helmet
(21806,628,1,1,1,512),-- Hoplon
(21806,1864,1,1,2,108980),-- Stem
(21806,1869,1,1,2,54490),-- Iron Ore
(21806,1880,1,1,2,5449),-- Steel
(21806,2155,1,1,2,8747),-- Recipe: Elven Ring
(21806,5550,1,1,2,727),-- Durable Metal Plate
(21807,1902,1,1,-1,60588),-- Elven Earring Beads
(21807,2157,1,1,-1,49421),-- Recipe: Omen Beast\'s Eye Earring
(21807,2158,1,1,-1,74010),-- Recipe: Mithril Ring
(21807,57,253,504,0,700000),-- Adena
(21807,606,1,1,1,52),-- Rip Gauntlets
(21807,1124,1,1,1,52),-- Boots of Power
(21807,1946,1,1,1,2612),-- Rip Gauntlets Pattern
(21807,1948,1,1,1,2612),-- Boots of Power Pattern
(21807,2424,1,1,1,52),-- Manticore Skin Boots
(21807,2425,1,1,1,52),-- Brigandine Boots
(21807,2426,1,1,1,52),-- Elven Mithril Boots
(21807,2448,1,1,1,52),-- Manticore Skin Gloves
(21807,2449,1,1,1,52),-- Brigandine Gauntlets
(21807,2450,1,1,1,52),-- Elven Mithril Gloves
(21807,2923,1,1,1,2612),-- Manticore Skin Boot Lining
(21807,2924,1,1,1,2612),-- Brigandine Boots Design
(21807,2925,1,1,1,2612),-- Elven Mithril Boots Design
(21807,2926,1,1,1,2612),-- Manticore Skin Gloves Lining
(21807,2927,1,1,1,2612),-- Brigandine Gauntlets Pattern
(21807,2928,1,1,1,2612),-- Elven Mithril Gloves Design
(21807,1865,1,1,2,77281),-- Varnish
(21807,1870,1,1,2,77281),-- Coal
(21807,1874,1,1,2,5153),-- Oriharukon Ore
(21807,2157,1,1,2,6387),-- Recipe: Omen Beast\'s Eye Earring
(21808,1866,1,1,-1,460056),-- Suede
(21808,1871,1,1,-1,690083),-- Charcoal
(21808,1905,1,1,-1,67129),-- Omen Beast\'s Eye Earring Gemstone
(21808,57,282,570,0,700000),-- Adena
(21808,630,1,1,1,113),-- Square Shield
(21808,1956,1,1,1,6556),-- Square Shield Fragment
(21808,2412,1,1,1,79),-- Plate Helmet
(21808,2494,1,1,1,113),-- Plate Shield
(21808,2930,1,1,1,4656),-- Plate Helmet Design
(21808,2934,1,1,1,6556),-- Plate Shield Fragment
(21808,1866,1,1,2,57755),-- Suede
(21808,1871,1,1,2,86632),-- Charcoal
(21808,1877,1,1,2,3466),-- Adamantite Nugget
(21808,2160,1,1,2,5590),-- Recipe: Moonstone Earring
(21809,1908,1,1,-1,75095),-- Moonstone Earring Wire
(21809,2163,1,1,-1,40459),-- Recipe: Earring of Protection
(21809,4553,1,1,-1,3172),-- Greater Dye of Strength <Str+1 Con-1>
(21809,57,320,650,0,700000),-- Adena
(21809,62,1,1,1,34),-- Mithril Boots
(21809,1958,1,1,1,1872),-- Mithril Boots Design
(21809,2429,1,1,1,34),-- Chain Boots
(21809,2430,1,1,1,34),-- Karmian Boots
(21809,2431,1,1,1,34),-- Plated Leather Boots
(21809,2432,1,1,1,34),-- Dwarven Chain Boots
(21809,2433,1,1,1,34),-- Boots of Seal
(21809,2452,1,1,1,34),-- Reinforced Mithril Gloves
(21809,2453,1,1,1,34),-- Chain Gloves
(21809,2454,1,1,1,34),-- Karmian Gloves
(21809,2937,1,1,1,1872),-- Chain Boots Part
(21809,2938,1,1,1,1872),-- Karmian Boots Fabric
(21809,2939,1,1,1,1872),-- Plate Leather Boot Lining
(21809,2940,1,1,1,1872),-- Dwarven Chain Boots Part
(21809,2941,1,1,1,1872),-- Boots of Seal Pattern
(21809,2942,1,1,1,1872),-- Reinforced Mithril Gloves Design
(21809,2943,1,1,1,1872),-- Chain Gloves Design
(21809,2944,1,1,1,1872),-- Karmian Gloves Fabric
(21809,1867,1,1,2,83286),-- Animal Skin
(21809,1872,1,1,2,83286),-- Animal Bone
(21809,1875,1,1,2,4165),-- Stone of Purity
(21809,1881,1,1,2,8329),-- Coarse Bone Powder
(21809,2163,1,1,2,3188),-- Recipe: Earring of Protection
(21809,4041,1,1,2,544),-- Mold Hardener
(21809,8723,1,1,200,1742),-- Life Stone: level 46
(21810,1966,1,1,-1,32947),-- Plated Leather Fragment
(21810,1967,1,1,-1,49218),-- Plated Leather Gaiters Material
(21810,2166,1,1,-1,30590),-- Recipe: Earrings of Binding (100%)
(21810,57,364,740,0,700000),-- Adena
(21810,500,1,1,1,56),-- Great Helmet
(21810,632,1,1,1,79),-- Knight\'s Shield
(21810,1971,1,1,1,4080),-- Great Helmet Material
(21810,1972,1,1,1,5332),-- Knight\'s Shield Fragment
(21810,2496,1,1,1,86),-- Dwarven Chain Shield
(21810,2947,1,1,1,4760),-- Dwarven Chain Shield Fragment
(21810,1868,1,1,2,133280),-- Thread
(21810,1873,1,1,2,26656),-- Silver Nugget
(21810,1876,1,1,2,13328),-- Mithril Ore
(21810,1895,1,1,2,19040),-- Metallic Fiber
(21810,2166,1,1,2,2260),-- Recipe: Earrings of Binding (100%)
(21810,4042,1,1,2,1111),-- Enria
(21810,8724,1,1,200,2604),-- Life Stone: level 49
(21810,8734,1,1,200,1304),-- Mid-Grade Life Stone: level 49
(21811,1878,1,1,-1,409067),-- Braided Hemp
(21811,1884,1,1,-1,629333),-- Cord
(21811,4601,1,1,-1,2273),-- Greater Dye of Strength <Str+3 Con-3>
(21811,57,413,839,0,700000),-- Adena
(21811,64,1,1,1,87),-- Composite Boots
(21811,608,1,1,1,87),-- Mithril Gauntlets
(21811,1981,1,1,1,6220),-- Mithril Gauntlets Design
(21811,2956,1,1,1,6220),-- Composite Boots Part
(21811,1864,1,1,2,140971),-- Stem
(21811,1869,1,1,2,70486),-- Iron Ore
(21811,1878,1,1,2,28195),-- Braided Hemp
(21811,1884,1,1,2,43376),-- Cord
(21811,2169,1,1,2,1248),-- Recipe: Necklace of Binding
(21811,4039,1,1,2,2350),-- Mold Glue
(21811,8725,1,1,200,1305),-- Life Stone: level 52
(21812,1984,1,1,-1,24686),-- Theca Leather Armor Pattern
(21812,1985,1,1,-1,39710),-- Theca Leather Gaiters Pattern
(21812,4126,1,1,-1,19003),-- Recipe: Adamantite Earrings
(21812,57,464,945,0,700000),-- Adena
(21812,2414,1,1,1,40),-- Full Plate Helmet
(21812,2497,1,1,1,57),-- Full Plate Shield
(21812,2959,1,1,1,3677),-- Full Plate Helmet Design
(21812,2966,1,1,1,5817),-- Full Plate Shield Fragment
(21812,1865,1,1,2,70867),-- Varnish
(21812,1870,1,1,2,70867),-- Coal
(21812,1875,1,1,2,4725),-- Stone of Purity
(21812,1879,1,1,2,11812),-- Cokes
(21812,4040,1,1,2,1418),-- Mold Lubricant
(21812,4126,1,1,2,1170),-- Recipe: Adamantite Earrings
(21812,8726,1,1,200,4000),-- Life Stone: level 55
(21813,947,1,1,-1,516),-- Scroll: Enchant Weapon (Grade B)
(21813,4049,1,1,-1,67676),-- Adamantite Earring Gemstone
(21813,4129,1,1,-1,13964),-- Recipe: Earrings of Black Ore
(21813,57,519,1060,0,700000),-- Adena
(21813,12147,1,1,1,208),-- Common Item - Zubei\'s Boots
(21813,12149,1,1,1,208),-- Common Item - Avadon Boots
(21813,12146,1,1,1,208),-- Common Item - Zubei\'s Gauntlets
(21813,12148,1,1,1,208),-- Common Item - Avadon Gloves
(21813,4073,1,1,1,2479),-- Avadon Gloves Part
(21813,4097,1,1,1,2479),-- Zubei\'s Boots Design
(21813,4098,1,1,1,2479),-- Avadon Boots Design
(21813,4100,1,1,1,2479),-- Zubei\'s Gauntlets Part
(21813,1866,1,1,2,42347),-- Suede
(21813,1871,1,1,2,63521),-- Charcoal
(21813,1885,1,1,2,5294),-- High Grade Suede
(21813,4041,1,1,2,553),-- Mold Hardener
(21813,4129,1,1,2,688),-- Recipe: Earrings of Black Ore
(21813,5220,1,1,2,2541),-- Metal Hardener
(21813,8727,1,1,200,833),-- Life Stone: level 58
(21813,8737,1,1,200,833),-- Mid-Grade Life Stone: level 58
(21814,1867,1,5,-1,638926),-- Animal Skin
(21814,1872,1,5,-1,638926),-- Animal Bone
(21814,4613,1,1,-1,1983),-- Greater Dye of Strength <Str+4 Con-4>
(21814,57,580,1179,0,700000),-- Adena
(21814,12160,1,1,1,72),-- Common Item - Zubei\'s Helmet
(21814,12159,1,1,1,72),-- Common Item - Zubei\'s Shield
(21814,12171,1,1,1,72),-- Common Item - Avadon Shield
(21814,12170,1,1,1,72),-- Common Item - Avadon Circlet
(21814,4072,1,1,1,2385),-- Avadon Circlet Pattern
(21814,4101,1,1,1,3078),-- Zubei\'s Shield Fragment
(21814,4102,1,1,1,2385),-- Zubei\'s Helmet Design
(21814,4439,1,1,1,3662),-- Avadon Shield Fragment
(21814,1867,1,1,2,45455),-- Animal Skin
(21814,1872,1,1,2,66667),-- Animal Bone
(21814,1882,1,1,2,10309),-- Leather
(21814,1894,1,1,2,1724),-- Crafted Leather
(21814,4042,1,1,2,287),-- Enria
(21814,4130,1,1,2,862),-- Recipe: Ring of Black Ore
(21814,8728,1,1,200,1149),-- Life Stone: level 61
(21815,4052,1,1,-1,56535),-- Earring of Black Ore Piece
(21815,4053,1,1,-1,68445),-- Ring of Black Ore Gemstone
(21815,5230,1,1,-1,2129),-- Recipe: Mithril Arrow (100%)
(21815,57,646,1308,0,700000),-- Adena
(21815,12188,1,1,1,593),-- Common Item - Boots of Doom
(21815,12189,1,1,1,593),-- Common Item - Blue Wolf Boots
(21815,12187,1,1,1,593),-- Common Item - Doom Gloves
(21815,12190,1,1,1,593),-- Common Item - Blue Wolf Gloves
(21815,4090,1,1,1,2647),-- Blue Wolf Boots Design
(21815,4091,1,1,1,2647),-- Doom Gloves Part
(21815,4096,1,1,1,2647),-- Blue Wolf Gloves Fabric
(21815,4099,1,1,1,2647),-- Doom Boots Part
(21815,1868,1,1,2,100000),-- Thread
(21815,1873,1,1,2,26316),-- Silver Nugget
(21815,1889,1,1,2,4149),-- Compound Braid
(21815,4044,1,1,2,1779),-- Thons
(21815,5230,1,1,2,105),-- Recipe: Mithril Arrow (100%)
(21815,5549,1,1,2,6536),-- Metallic Thread
(21815,8739,1,1,200,1186),-- Mid-Grade Life Stone: level 64
(21816,4054,1,1,-1,34802),-- Necklace of Black Ore Beads
(21816,5438,1,1,-1,966),-- Recipe: Meteor Shower(60%)
(21816,5550,1,1,-1,23559),-- Durable Metal Plate
(21816,57,714,1448,0,700000),-- Adena
(21816,12183,1,1,1,262),-- Common Item - Doom Shield
(21816,12202,1,1,1,262),-- Common Item - Blue Wolf Helmet
(21816,12185,1,1,1,262),-- Common Item - Doom Helmet
(21816,4077,1,1,1,3913),-- Doom Shield Fragment
(21816,4088,1,1,1,3288),-- Blue Wolf Helmet Design
(21816,4089,1,1,1,3288),-- Doom Helmet Pattern
(21816,1864,1,1,2,100000),-- Stem
(21816,1869,1,1,2,50000),-- Iron Ore
(21816,1880,1,1,2,28571),-- Steel
(21816,4043,1,1,2,787),-- Asofe
(21816,5434,1,1,2,33),-- Recipe: Dragon Slayer(60%)
(21816,5550,1,1,2,1160),-- Durable Metal Plate
(21816,8729,1,1,200,787),-- Life Stone: level 64
(21816,8739,1,1,200,787),-- Mid-Grade Life Stone: level 64
(21817,1874,1,1,-1,129783),-- Oriharukon Ore
(21817,5434,1,1,-1,721),-- Recipe: Dragon Slayer(60%)
(21817,5478,1,1,-1,16370),-- Sealed Dark Crystal Leather Armor Pattern
(21817,57,786,1596,0,700000),-- Adena
(21817,12221,1,1,1,256),-- Common Item - Sealed Dark Crystal Gloves
(21817,12224,1,1,1,256),-- Common Item - Sealed Dark Crystal Boots
(21817,12229,1,1,1,256),-- Common Item - Sealed Tallum Gloves
(21817,12231,1,1,1,256),-- Common Item - Sealed Tallum Boots
(21817,5496,1,1,1,1686),-- Sealed Dark Crystal Boots Lining
(21817,5497,1,1,1,1686),-- Sealed Tallum Boots Lining
(21817,5508,1,1,1,1686),-- Sealed Dark Crystal Gloves Design
(21817,5509,1,1,1,1686),-- Sealed Tallum Gloves Design
(21817,1865,1,1,2,58824),-- Varnish
(21817,1870,1,1,2,62500),-- Coal
(21817,1874,1,1,2,4098),-- Oriharukon Ore
(21817,4039,1,1,2,2045),-- Mold Glue
(21817,4043,1,1,2,1024),-- Asofe
(21817,5440,1,1,2,36),-- Recipe: Elysian(60%)
(21817,8730,1,1,200,769),-- Life Stone: level 67
(21817,8740,1,1,200,769),-- Mid-Grade Life Stone: level 67
(21818,5440,1,1,-1,792),-- Recipe: Elysian(60%)
(21818,5479,1,1,-1,12647),-- Sealed Tallum Leather Armor Pattern
(21818,5482,1,1,-1,28211),-- Sealed Dark Crystalline Leggings Design
(21818,57,860,1756,0,700000),-- Adena
(21818,12226,1,1,1,438),-- Common Item - Sealed Dark Crystal Helmet
(21818,12225,1,1,1,438),-- Common Item - Sealed Dark Crystal Shield
(21818,12234,1,1,1,438),-- Common Item - Sealed Tallum Helmet
(21818,5494,1,1,1,2210),-- Sealed Dark Crystal Shield Fragment
(21818,5525,1,1,1,1627),-- Sealed Dark Crystal Helmet Design
(21818,5526,1,1,1,1627),-- Sealed Tallum Helm Design
(21818,1866,1,1,2,62500),-- Suede
(21818,1871,1,1,2,62500),-- Charcoal
(21818,1877,1,1,2,5102),-- Adamantite Nugget
(21818,4040,1,1,2,848),-- Mold Lubricant
(21818,4041,1,1,2,424),-- Mold Hardener
(21818,6901,1,1,2,106),-- Recipe: Shining Arrow (100%)
(21818,8730,1,1,200,329),-- Life Stone: level 67
(21819,1875,1,1,-1,155856),-- Stone of Purity
(21819,4041,1,1,-1,20329),-- Mold Hardener
(21819,5480,1,1,-1,10490),-- Sealed Leather Armor of Nightmare Fabric
(21819,57,941,1920,0,700000),-- Adena
(21819,12221,1,1,1,607),-- Common Item - Sealed Dark Crystal Gloves
(21819,12224,1,1,1,607),-- Common Item - Sealed Dark Crystal Boots
(21819,12229,1,1,1,607),-- Common Item - Sealed Tallum Gloves
(21819,12231,1,1,1,607),-- Common Item - Sealed Tallum Boots
(21819,5496,1,1,1,2024),-- Sealed Dark Crystal Boots Lining
(21819,5497,1,1,1,2024),-- Sealed Tallum Boots Lining
(21819,5508,1,1,1,2024),-- Sealed Dark Crystal Gloves Design
(21819,5509,1,1,1,2024),-- Sealed Tallum Gloves Design
(21819,1808,1,1,2,51),-- Recipe: Soulshot: S Grade
(21819,1867,1,1,2,83333),-- Animal Skin
(21819,1872,1,1,2,90909),-- Animal Bone
(21819,1875,1,1,2,2770),-- Stone of Purity
(21819,1881,1,1,2,14925),-- Coarse Bone Powder
(21819,4041,1,1,2,693),-- Mold Hardener
(21819,8731,1,1,200,1041),-- Life Stone: level 70
(21820,1808,1,1,-1,1132),-- Recipe: Soulshot: S Grade
(21820,5481,1,1,-1,11429),-- Sealed Majestic Leather Armor Fabric
(21820,6901,1,1,-1,2547),-- Recipe: Shining Arrow (100%)
(21820,57,1025,2092,0,700000),-- Adena
(21820,12226,1,1,1,513),-- Common Item - Sealed Dark Crystal Helmet
(21820,12225,1,1,1,513),-- Common Item - Sealed Dark Crystal Shield
(21820,12234,1,1,1,513),-- Common Item - Sealed Tallum Helmet
(21820,5494,1,1,1,2634),-- Sealed Dark Crystal Shield Fragment
(21820,5525,1,1,1,1939),-- Sealed Dark Crystal Helmet Design
(21820,5526,1,1,1,1939),-- Sealed Tallum Helm Design
(21820,1344,10,30,2,100000),-- Mithril Arrow
(21820,1868,1,1,2,200000),-- Thread
(21820,1873,1,1,2,38462),-- Silver Nugget
(21820,1876,1,1,2,135145),-- Mithril Ore
(21820,1895,1,1,2,30303),-- Metallic Fiber
(21820,3036,1,1,2,48),-- Recipe: Spiritshot S
(21820,4042,1,1,2,1919),-- Enria
(21821,4039,1,1,-1,92156),-- Mold Glue
(21821,4041,1,1,-1,24041),-- Mold Hardener
(21821,5487,1,1,-1,12450),-- Sealed Robe of Nightmare Fabric
(21821,57,1110,2273,0,700000),-- Adena
(21821,12264,1,1,1,178),-- Common Item - Sealed Gloves of Nightmare
(21821,12267,1,1,1,178),-- Common Item - Sealed Boots of Nightmare
(21821,12257,1,1,1,178),-- Common Item - Sealed Majestic Gloves
(21821,12260,1,1,1,178),-- Common Item - Sealed Majestic Boots
(21821,5502,1,1,1,1724),-- Sealed Boots of Nightmare Lining
(21821,5503,1,1,1,1724),-- Sealed Majestic Boots Lining
(21821,5514,1,1,1,1724),-- Sealed Gloves of Nightmare Design
(21821,5515,1,1,1,1724),-- Sealed Majestic Gloves Design
(21821,959,1,1,2,7),-- Scroll: Enchant Weapon (Grade S)
(21821,1894,1,1,2,5731),-- Crafted Leather
(21821,4039,1,1,2,5445),-- Mold Glue
(21821,4041,1,1,2,1421),-- Mold Hardener
(21821,4044,1,1,2,5445),-- Thons
(21821,8732,1,1,200,1427),-- Life Stone: level 76
(21822,6688,1,1,-1,6446),-- Forgotten Blade Edge
(21822,6689,1,1,-1,6325),-- Basalt Battlehammer Head
(21822,6690,1,1,-1,6459),-- Imperial Staff Head
(21822,57,1200,2449,0,700000),-- Adena
(21822,12269,1,1,1,12),-- Common Item - Sealed Helm of Nightmare (no drop info)
(21822,12268,1,1,1,16),-- Common Item - Sealed Shield of Nightmare
(21822,12261,1,1,1,12),-- Common Item - Sealed Majestic Circlet
(21822,5495,1,1,1,2230),-- Sealed Shield of Nightmare Fragment
(21822,5527,1,1,1,1756),-- Sealed Helm of Nightmare Design
(21822,5528,1,1,1,1756),-- Sealed Majestic Circlet Design
(21822,960,1,1,2,71),-- Scroll: Enchant Armor (Grade S)
(21822,1895,1,1,2,50341),-- Metallic Fiber
(21822,4040,1,1,2,3524),-- Mold Lubricant
(21822,4042,1,1,2,2937),-- Enria
(21822,4043,1,1,2,5874),-- Asofe
(21823,1894,1,1,-1,888656),-- Crafted Leather
(21823,4039,1,1,-1,844224),-- Mold Glue
(21823,5480,1,1,-1,113647),-- Sealed Leather Armor of Nightmare Fabric
(21823,57,10189,20801,0,700000),-- Adena
(21823,11975,1,1,1,3012),-- Common Item - Soul Bow
(21823,5534,1,1,1,10269),-- Soul Bow Stave
(21823,959,1,1,2,60),-- Scroll: Enchant Weapon (Grade S)
(21823,1894,1,1,2,28571),-- Crafted Leather
(21823,4039,1,1,2,25641),-- Mold Glue
(21823,4041,1,1,2,5435),-- Mold Hardener
(21823,8732,1,1,200,7813),-- Life Stone: level 76
(21823,8742,1,1,200,1808),-- Mid-Grade Life Stone: level 76
(21823,8752,1,1,200,603),-- High-Grade Life Stone: level 76
(21823,4044,1,1,2,38462); -- Thons


-- work in progress C5 mob drops - 22xxx mobs
INSERT INTO `droplist` VALUES 
(22001,57,163,306,0,700000),-- Adena
(22001,1872,1,1,2,69351),-- Animal Bone
(22001,1867,1,1,2,69351),-- Animal Skin
(22001,1882,1,1,2,11558),-- Leather
(22001,2152,1,1,2,9157),-- Recipe: Enchanted Necklace (Grade D)
(22001,391,1,1,1,432),-- Puma Skin Shirt (Grade D)
(22001,413,1,1,1,691),-- Puma Skin Gaiters (Grade D)
(22001,1867,1,1,-1,494275),-- Animal Skin
(22001,1899,1,1,-1,57923),-- Necklace of Devotion Chain
(22001,2152,1,1,-1,65265),-- Recipe: Enchanted Necklace (Grade D)
(22002,57,170,324,0,700000),-- Adena
(22002,1872,1,1,2,58440),-- Animal Bone
(22002,1867,1,1,2,58440),-- Animal Skin
(22002,1881,1,1,2,5844),-- Coarse Bone Powder
(22002,8301,1,1,2,1072),-- Recipe: Priest Sword
(22002,1875,1,1,2,2922),-- Stone of Purity
(22002,258,1,1,1,139),-- Bagh-Nakh (Grade D)
(22002,1931,1,1,-1,51043),-- Leather Boot Lining
(22002,2178,1,1,-1,62477),-- Recipe: Dark Stockings
(22002,1932,1,1,-1,61981),-- Bone Helmet Design
(22003,57,208,401,0,700000),-- Adena
(22003,1884,1,1,2,27753),-- Cord
(22003,1878,1,1,2,18039),-- Braided Hemp
(22003,1889,1,1,2,3007),-- Compound Braid
(22003,1874,1,1,2,3007),-- Oriharukon ore
(22003,5220,1,1,2,1804),-- Metal Hardener
(22003,274,1,1,1,172),-- Strengthened Bow (Grade D)
(22003,2180,1,1,2,3391),-- Recipe: Scale Mail
(22003,1874,1,1,-1,32142),-- Oriharukon ore
(22003,1878,1,1,-1,192855),-- Braided Hemp
(22003,1934,1,1,-1,63024),-- Crafted Leather Gloves Lining
(22004,57,247,477,0,700000),-- Adena
(22004,1061,1,1,2,27844),-- Healing Potion
(22004,1879,1,1,2,7657),-- Cokes
(22004,2181,1,1,2,4599),-- Recipe: White Tunic (Grade D)
(22004,1875,1,1,2,3063),-- Stone of Purity
(22004,5549,1,1,2,4594),-- Metallic Thread
(22004,5550,1,1,2,613),-- Durable Metal Plate
(22004,2017,1,1,2,534),-- Saber Blade
(22004,1885,1,1,2,3829),-- High Grade Suede
(22004,8327,1,1,2,675),-- Steel Sword Blade
(22004,5531,1,1,2,675),-- Heavy Sword Edge
(22004,8332,1,1,2,668),-- Priest Sword Edge
(22004,123,1,1,1,26),-- Saber
(22004,5285,1,1,1,26),-- Heavy Sword
(22004,7885,1,1,1,26),-- Priest Sword
(22004,7880,1,1,1,26),-- Steel Sword
(22004,2180,1,1,-1,43085),-- Recipe: Scale Mail (Grade D)
(22004,2017,1,1,-1,7615),-- Saber Blade
(22004,2181,1,1,-1,57361),-- Recipe: White Tunic (Grade D)
(22005,57,217,425,0,700000),-- Adena
(22005,1880,1,1,2,4741),-- Steel
(22005,1881,1,1,2,6321),-- Coarse Bone Powder
(22005,2182,1,1,2,5691),-- Recipe: Scale Gaiters (Grade D)
(22005,220,1,1,1,181),-- Crafted Dagger
(22005,1876,1,1,2,9481),-- Mithril Ore
(22005,1877,1,1,2,1896),-- Adamantite Nugget
(22005,1882,1,1,2,10535),-- Leather
(22005,2018,1,1,-1,6735),-- Assassin Knife Edge
(22005,1876,1,1,-1,101353),-- Mithril Ore
(22005,1880,1,1,-1,50680),-- Steel
(22006,57,225,445,0,700000),-- Adena
(22006,1878,1,1,2,19323),-- Braided Hemp
(22006,1884,1,1,2,29728),-- Cord
(22006,1889,1,1,2,3221),-- Compound Braid
(22006,292,1,1,1,117),-- Pike
(22006,1874,1,1,2,3221),-- Oriharukon ore
(22006,5220,1,1,2,1932),-- Metal Hardener
(22006,737,1,1,2,4831),-- Scroll of Resurrection
(22006,1061,1,1,-1,321463),-- Healing Potion
(22006,1900,1,1,-1,58032),-- Enchanted Necklace Chain
(22006,2153,1,1,-1,82490),-- Recipe: Tiger\'s Eye Earring
(22007,57,245,485,0,700000),-- Adena
(22007,86,1,1,1,13),-- Tomahawk
(22007,179,1,1,1,13),-- Mace of Prayer
(22007,180,1,1,1,13),-- Mace of Judgment
(22007,181,1,1,1,13),-- Mace of Miracle
(22007,182,1,1,1,13),-- Doom Hammer
(22007,183,1,1,1,13),-- Mystic Staff
(22007,184,1,1,1,13),-- Conjuror\'s Staff
(22007,185,1,1,1,13),-- Staff of Mana
(22007,1876,1,1,2,10532),-- Mithril Ore
(22007,1880,1,1,2,5266),-- Steel
(22007,1877,1,1,2,2106),-- Adamantite Nugget
(22007,1881,1,1,2,7021),-- Coarse Bone Powder
(22007,1882,1,1,2,11702),-- Leather
(22007,955,1,1,2,211),-- Scroll: Enchant Weapon (Grade D)
(22007,2022,1,1,2,492),-- Mace of Judgment Head
(22007,2023,1,1,2,392),-- Conjuror\'s Staff Head
(22007,1933,1,1,-1,68021),-- Dark Stocking Fabric
(22007,2183,1,1,-1,38038),-- Recipe: Mithril Banded Mail (Grade D)
(22007,1935,1,1,-1,32300),-- Scale Mail Fragment
(22008,57,234,468,0,700000),-- Adena
(22008,1876,1,1,2,10103),-- Mithril Ore
(22008,1880,1,1,2,5051),-- Steel
(22008,1877,1,1,2,2021),-- Adamantite Nugget
(22008,1881,1,1,2,6735),-- Coarse Bone Powder
(22008,1882,1,1,2,11225),-- Leather
(22008,956,1,1,2,1684),-- Scroll: Enchant Armor (Grade D)
(22008,1937,1,1,2,6510),-- Scale Gaiters Fragment
(22008,1935,1,1,2,4233),-- Scale Mail Fragment
(22008,348,1,1,1,148),-- Scale Mail
(22008,377,1,1,1,236),-- Scale Gaiters
(22008,955,1,1,-1,2219),-- Scroll: Enchant Weapon (Grade D)
(22008,1877,1,1,-1,22185),-- Adamantite Nugget
(22008,1881,1,1,-1,73951),-- Coarse Bone Powder
(22009,57,272,542,0,700000),-- Adena
(22009,848,1,1,1,535),-- Enchanted Earring (Grade D)
(22009,879,1,1,1,805),-- Enchanted Ring (Grade D)
(22009,911,1,1,1,402),-- Enchanted Necklace (Grade D)
(22009,1900,1,1,2,9599),-- Enchanted Necklace Chain
(22009,1875,1,1,2,3343),-- Stone of Purity
(22009,1879,1,1,2,8356),-- Cokes
(22009,1885,1,1,2,4178),-- High Grade Suede
(22009,5550,1,1,2,669),-- Durable Metal Plate
(22009,5549,1,1,2,5014),-- Metallic Thread
(22009,1829,1,1,2,20055),-- Scroll of Escape: Clan Hall
(22009,1832,1,1,2,55708),-- Greater Antidote
(22009,1832,1,1,-1,713613),-- Greater Antidote
(22009,1936,1,1,-1,52557),-- White Tunic Pattern
(22009,2184,1,1,-1,67463),-- Recipe: Mithril Banded Gaiters
(22010,57,286,575,0,700000),-- Adena
(22010,1868,1,1,2,148621),-- Thread
(22010,1873,1,1,2,29724),-- Silver Nugget
(22010,1829,1,1,2,29724),-- Scroll of Escape: Clan Hall
(22010,5549,1,1,2,7431),-- Metallic Thread
(22010,1889,1,1,2,4954),-- Compound Braid
(22010,46,1,1,1,968),-- Bronze Helmet (Grade D)
(22010,627,1,1,1,1381),-- Aspis (Grade D)
(22010,1937,1,1,-1,58413),-- Scale Gaiters Fragment
(22010,2185,1,1,-1,133844),-- Recipe: Iron Boots
(22010,2021,1,1,-1,8585),-- Spinebone Sword Blade
(22011,57,329,658,0,700000),-- Adena
(22011,1871,1,1,2,103504),-- Charcoal
(22011,1866,1,1,2,69002),-- Suede
(22011,2154,1,1,2,11082),-- Recipe: Elven Earring
(22011,1877,1,1,2,4140),-- Adamantite Nugget
(22011,58,1,1,1,84),-- Mithril Breastplate
(22011,59,1,1,1,121),-- Mithril Gaiters
(22011,349,1,1,1,84),-- Compound Scale Mail
(22011,350,1,1,1,84),-- Dwarven Scale Mail
(22011,351,1,1,1,76),-- Blast Plate
(22011,378,1,1,1,121),-- Compound Scale Gaiters
(22011,379,1,1,1,121),-- Dwarven Scale Gaiters
(22011,1871,1,1,-1,778674),-- Charcoal
(22011,1866,1,1,-1,519116),-- Suede
(22011,1901,1,1,-1,107552),-- Tiger\'s Eye Earring Stone
(22012,57,329,658,0,700000),-- Adena
(22012,1871,1,1,2,103504),-- Charcoal
(22012,1866,1,1,2,69003),-- Suede
(22012,1877,1,1,2,4140),-- Adamantite Nugget
(22012,2155,1,1,2,16614),-- Recipe: Elven Ring
(22012,58,1,1,1,84),-- Mithril Breastplate
(22012,59,1,1,1,121),-- Mithril Gaiters
(22012,349,1,1,1,84),-- Compound Scale Mail
(22012,350,1,1,1,84),-- Dwarven Scale Mail
(22012,351,1,1,1,76),-- Blast Plate
(22012,378,1,1,1,121),-- Compound Scale Gaiters
(22012,379,1,1,1,121),-- Dwarven Scale Gaiters
(22012,2154,1,1,-1,83370),-- Recipe: Elven Earring
(22012,2155,1,1,-1,124988),-- Recipe: Elven Ring
(22012,1938,1,1,-1,52348),-- Mithril Banded Mail Material
(22013,57,329,658,0,700000),-- Adena
(22013,1865,1,1,2,82803),-- Varnish
(22013,1870,1,1,2,82803),-- Coal
(22013,1879,1,1,2,13800),-- Cokes
(22013,2156,1,1,2,6624),-- Recipe: Elven Necklace
(22013,1875,1,1,2,5520),-- Stone of Purity
(22013,83,1,1,1,25),-- Sword of Magic
(22013,124,1,1,1,19),-- Two-Handed Sword
(22013,127,1,1,1,19),-- Crimson Sword
(22013,130,1,1,1,19),-- Elven Sword
(22013,144,1,1,1,25),-- Sword of Occult
(22013,2026,1,1,1,830),-- Two-Handed Sword Edge
(22013,1939,1,1,-1,69395),-- Mithril Banded Gaiters Material
(22013,1865,1,1,-1,778674),-- Varnish
(22013,1870,1,1,-1,778674),-- Coal
(22014,57,329,658,0,700000),-- Adena
(22014,1341,10,30,2,230008),-- Bone Arrow
(22014,1870,1,1,2,69002),-- Coal
(22014,1865,1,1,2,69002),-- Varnish
(22014,1879,1,1,2,11500),-- Cokes
(22014,1875,1,1,2,4600),-- Stone of Purity
(22014,2186,1,1,2,5862),-- Recipe: Brigandine Tunic
(22014,83,1,1,1,25),-- Sword of Magic
(22014,124,1,1,1,19),-- Two-Handed Sword
(22014,127,1,1,1,19),-- Crimson Sword
(22014,130,1,1,1,19),-- Elven Sword
(22014,144,1,1,1,25),-- Sword of Occult
(22014,2026,1,1,1,830),-- Two-Handed Sword Edge
(22014,1940,1,1,-1,116220),-- Iron Boots Design
(22014,2156,1,1,-1,62294),-- Recipe: Elven Necklace
(22014,2026,1,1,-1,9372),-- Two-Handed Sword Edge
(22015,57,329,658,0,700000),-- Adena
(22015,1884,1,1,2,42463),-- Cord
(22015,5220,1,1,2,2760),-- Metal Hardener
(22015,1878,1,1,2,27601),-- Braided Hemp
(22015,1889,1,1,2,4600),-- Compound Braid
(22015,1874,1,1,2,4600),-- Oriharukon ore
(22015,2187,1,1,2,3080),-- Recipe: Manticore Skin Shirt
(22015,260,1,1,1,111),-- Triple-Edged Jamadhr
(22015,1884,1,1,-1,479184),-- Cord
(22015,2186,1,1,-1,26043),-- Recipe: Brigandine Tunic
(22015,2027,1,1,-1,10201),-- Spiked Club Head
(22016,57,329,658,0,700000),-- Adena
(22016,1884,1,1,2,42463),-- Cord
(22016,1878,1,1,2,27601),-- Braided Hemp
(22016,2188,1,1,2,4929),-- Recipe: Manticore Skin Gaiters
(22016,1889,1,1,2,4600),-- Compound Braid
(22016,1874,1,1,2,4600),-- Oriharukon ore
(22016,5220,1,1,2,2760),-- Metal Hardener
(22016,260,1,1,1,111),-- Triple-Edged Jamadhr
(22016,1889,1,1,-1,51918),-- Compound Braid
(22016,2187,1,1,-1,34762),-- Recipe: Manticore Skin Shirt
(22016,2028,1,1,-1,9252),-- Shilen Knife Edge
(22017,57,249,499,0,700000),-- Adena
(22017,1864,1,1,2,129279),-- Stem
(22017,1869,1,1,2,64639),-- Iron Ore
(22017,1880,1,1,2,6464),-- Steel
(22017,737,1,1,2,6464),-- Scroll of Resurrection
(22017,2021,1,1,1,817),-- Spinebone Sword Blade
(22017,5550,1,1,2,862),-- Durable Metal Plate
(22017,125,1,1,1,26),-- Spinebone Sword
(22017,128,1,1,1,26),-- Knight\'s Sword
(22017,69,1,1,1,26),-- Bastard Sword
(22017,126,1,1,1,26),-- Artisan\'s Sword
(22017,1900,1,1,-1,64708),-- Enchanted Necklace Chain
(22017,2153,1,1,-1,91981),-- Recipe: Tiger\'s Eye Earring
(22017,1933,1,1,-1,69581),-- Dark Stocking Fabric
(22018,57,213,426,0,700000),-- Adena
(22018,1872,1,1,3,91942),-- Animal Bone
(22018,1867,1,1,2,91942),-- Animal Skin
(22018,1882,1,1,2,15324),-- Leather
(22018,392,1,1,1,359),-- Lion Skin Shirt
(22018,414,1,1,1,574),-- Lion Skin Gaiters
(22018,955,1,1,2,276),-- Scroll: Enchant Weapon (Grade D)
(22018,956,1,1,-1,16825),-- Scroll: Enchant Armor (Grade D)
(22018,1872,1,1,-1,672999),-- Animal Bone
(22018,1882,1,1,-1,112167),-- Leather
(22019,57,289,580,0,700000),-- Adena
(22019,1864,1,1,2,125107),-- Stem
(22019,1869,1,1,2,62554),-- Iron Ore
(22019,1061,1,1,2,37911),-- Healing Potion
(22019,1933,1,1,1,5520),-- Dark Stocking Fabric
(22019,1880,1,1,2,6255),-- Steel
(22019,1936,1,1,1,3840),-- White Tunic Pattern
(22019,956,1,1,2,2085),-- Scroll: Enchant Armor (Grade D)
(22019,5550,1,1,2,834),-- Durable Metal Plate
(22019,466,1,1,1,122),-- Elven Stockings
(22019,433,1,1,1,122),-- Elven Tunic
(22019,435,1,1,1,122),-- Mystic\'s Tunic
(22019,467,1,1,1,195),-- Dark Stockings
(22019,434,1,1,1,195),-- White Tunic
(22019,468,1,1,1,196),-- Mystic\'s Stockings
(22019,1834,1,1,-1,763138),-- Emergency Dressing
(22019,1935,1,1,-1,38370),-- Scale Mail Fragment
(22019,2183,1,1,-1,45186),-- Recipe: Mithril Banded Mail
(22020,57,239,479,0,700000),-- Adena
(22020,1872,1,1,2,80362),-- Animal Bone
(22020,1867,1,1,2,80362),-- Animal Skin
(22020,1881,1,1,2,8036),-- Coarse Bone Powder
(22020,1875,1,1,2,4018),-- Stone of Purity
(22020,8302,1,1,2,4313),-- Recipe: Sword of Magic Fog (100%)
(22020,378,1,1,1,88),-- Compound Scale Gaiters
(22020,59,1,1,1,88),-- Mithril Gaiters
(22020,351,1,1,1,55),-- Blast Plate
(22020,58,1,1,1,61),-- Mithril Breastplate
(22020,349,1,1,1,61),-- Compound Scale Mail
(22020,379,1,1,1,88),-- Dwarven Scale Gaiters
(22020,350,1,1,1,61),-- Dwarven Scale Mail
(22020,1901,1,1,-1,78285),-- Tiger\'s Eye Earring Stone
(22020,2154,1,1,-1,60684),-- Recipe: Elven Earring
(22020,1938,1,1,-1,38103),-- Mithril Banded Mail Material
(22021,57,291,580,0,700000),-- Adena
(22021,1341,10,30,2,174450),-- Bone Arrow
(22021,1884,1,1,2,32206),-- Cord
(22021,1878,1,1,2,20934),-- Braided Hemp
(22021,1874,1,1,2,3489),-- Oriharukon Ore
(22021,1889,1,1,2,3489),-- Compound Braid
(22021,2027,1,1,1,1599),-- Spiked Club Head
(22021,5220,1,1,2,2093),-- Metal Hardener
(22021,8295,1,1,2,374),-- Recipe: Titan Sword (100%)
(22021,157,1,1,1,33),-- Spiked Club
(22021,186,1,1,1,33),-- Staff of Magic
(22021,5220,1,1,-1,27560),-- Metal Hardener
(22021,1874,1,1,-1,45933),-- Oriharukon Ore
(22021,1939,1,1,-1,60518),-- Mithril Banded Gaiters Material
(22022,57,344,687,0,700000),-- Adena
(22022,1872,1,1,2,115606),-- Animal Bone
(22022,1867,1,1,2,115606),-- Animal Skin
(22022,1881,1,1,2,11561),-- Coarse Bone Powder
(22022,2156,1,1,2,6936),-- Recipe: Elven Necklace
(22022,1875,1,1,2,5780),-- Stone of Purity
(22022,2028,1,1,1,1030),-- Shilen Knife Edge
(22022,238,1,1,1,23),-- Dagger of Mana
(22022,223,1,1,1,23),-- Kukuri
(22022,239,1,1,1,23),-- Mystic Knife
(22022,241,1,1,1,23),-- Shilen Knife
(22022,2155,1,1,-1,130876),-- Recipe: Elven Ring
(22022,1940,1,2,-1,121696),-- Iron Boots Design
(22022,2156,1,1,-1,65229),-- Recipe: Elven Necklace
(22023,57,392,787,0,700000),-- Adena
(22023,1868,1,1,2,197994),-- Thread
(22023,1873,1,1,2,39599),-- Silver Nugget
(22023,5549,1,1,2,9900),-- Metallic Thread
(22023,1889,1,1,2,6600),-- Compound Braid
(22023,2026,1,1,1,993),-- Two-Handed Sword Edge
(22023,8295,1,1,2,707),-- Recipe: Titan Sword (100%)
(22023,83,1,1,1,30),-- Sword of Magic
(22023,124,1,1,1,22),-- Two-Handed Sword
(22023,127,1,1,1,22),-- Crimson Sword
(22023,130,1,1,1,22),-- Elven Sword
(22023,144,1,1,1,30),-- Sword of Occult
(22023,2026,1,1,-1,11205),-- Two-Handed Sword Edge
(22023,1868,1,3,-1,930959),-- Thread
(22023,1873,1,2,-1,372383),-- Silver Nugget
(22024,57,369,737,0,700000),-- Adena
(22024,1867,1,1,2,120476),-- Animal Skin
(22024,1872,1,1,2,120476),-- Animal Bone
(22024,1881,1,1,2,12048),-- Coarse Bone Powder
(22024,1943,1,1,1,8971),-- Manticore Skin Gaiters Pattern
(22024,1825,1,1,2,6454),-- Recipe: Oriharukon
(22024,1875,1,1,2,6024),-- Stone of Purity
(22024,1942,1,1,1,5761),-- Manticore Skin Shirt Texture
(22024,417,1,1,1,210),-- Manticore Skin Gaiters
(22024,395,1,1,1,131),-- Manticore Skin Shirt
(22024,6036,1,1,-1,29133),-- Greater Magic Haste Potion
(22024,1902,1,1,-1,88550),-- Elven Earring Beads
(22024,1825,1,1,-1,62428),-- Recipe: Oriharukon
(22025,57,369,736,0,700000),-- Adena
(22025,1872,1,1,2,120193),-- Animal Bone
(22025,1867,1,1,2,120193),-- Animal Skin
(22025,1882,1,1,2,20032),-- Leather
(22025,2145,1,1,2,6439),-- Recipe: Mithril Alloy
(22025,2918,1,1,1,4457),-- Mithril Scale Gaiters Material
(22025,2919,1,1,1,4457),-- Brigandine Gaiters Material
(22025,1941,1,1,1,2988),-- Brigandine Temper
(22025,1894,1,1,2,3163),-- Crafted Leather
(22025,34,1,1,1,105),-- Bronze Gaiters
(22025,395,1,1,1,65),-- Manticore Skin Shirt
(22025,417,1,1,1,105),-- Manticore Skin Gaiters
(22025,1903,1,1,-1,145444),-- Elven Ring Piece
(22025,2145,1,1,-1,62281),-- Recipe: Mithril Alloy
(22025,1904,1,1,-1,81375),-- Elven Necklace Beads
(22026,57,478,954,0,700000),-- Adena
(22026,1864,1,1,2,236710),-- Stem
(22026,1869,1,1,2,118355),-- Iron Ore
(22026,1884,1,1,2,72834),-- Cord
(22026,1878,1,1,2,47342),-- Braided Hemp
(22026,2146,1,1,2,8454),-- Recipe: Artisan\'s frame
(22026,8333,1,1,1,972),-- Sword of Magic Fog Blade 
(22026,8328,1,1,1,941),-- Titan Sword Blade 
(22026,2034,1,1,1,1042),-- Sword of Revolution Blade
(22026,129,1,1,1,18),-- Sword of Revolution
(22026,7886,1,1,1,18),-- Sword of Magic Fog
(22026,7881,1,1,1,18),-- Titan Sword
(22026,1864,1,5,-1,763202),-- Stem
(22026,1869,1,3,-1,572402),-- Iron Ore
(22026,6036,1,1,-1,38160); -- Greater Magic Haste Potion

INSERT INTO `droplist` VALUES
(22027,57,936,1864,0,700000),-- Adena
(22027,1870,1,1,2,228708),-- Coal
(22027,1865,1,1,2,228708),-- Varnish 
(22027,2147,1,1,2,16336),-- Recipe: Blacksmith\'s frame
(22027,1874,1,1,2,15247),-- Oriharukon Ore
(22027,6036,1,1,2,7624),-- Greater Magic Haste Potion
(22027,279,1,1,1,106),-- Strengthened Long Bow
(22027,2040,1,1,1,6616),-- Strengthened Long Bow Shaft
(22027,2146,1,1,-1,158015),-- Recipe: Artisan\'s frame
(22027,1941,1,1,-1,87978),-- Brigandine Temper
(22027,1942,1,1,-1,112839),-- Manticore Skin Shirt Texture
(22028,57,833,1662,0,700000),-- Adena
(22028,1864,3,9,2,68754),-- Stem
(22028,1869,1,1,2,206261),-- Iron Ore
(22028,1884,1,1,2,126930),-- Cord
(22028,1878,1,1,2,82504),-- Braided Hemp
(22028,2148,1,1,2,14733),-- Recipe: Crafted Leather
(22028,8333,1,1,1,1693),-- Sword of Magic Fog Blade 
(22028,8328,1,1,1,1641),-- Titan Sword Blade 
(22028,2034,1,1,1,1816),-- Sword of Revolution Blade
(22028,129,1,1,1,32),-- Sword of Revolution
(22028,7886,1,1,1,32),-- Sword of Magic Fog
(22028,7881,1,1,1,32),-- Titan Sword
(22028,1878,1,1,-1,798035),-- Braided Hemp
(22028,1943,1,1,-1,158466),-- Manticore Skin Gaiters Pattern
(22028,2147,1,1,-1,142506),-- Recipe: Blacksmith\'s frame
(22029,57,496,990,0,700000),-- Adena
(22029,1868,1,1,2,245638),-- Thread
(22029,5549,1,1,2,12282),-- Metallic Thread
(22029,1873,1,1,2,49128),-- Silver Nugget
(22029,1889,1,1,2,8188),-- Compound Braid
(22029,2149,1,1,1,8773),-- Recipe: Metallic Fiber
(22029,88,1,1,1,8),-- Morning Star
(22029,90,1,1,1,8),-- Goat Head Staff
(22029,158,1,1,1,8),-- Tarbar
(22029,169,1,1,1,8),-- Skull Breaker
(22029,172,1,1,1,8),-- Heavy Bone Club
(22029,7896,1,1,1,8),-- Titan Hammer
(22029,7890,1,1,1,8),-- Priest Mace
(22029,2031,1,1,1,444),-- Morning Star Head
(22029,2032,1,1,1,405),-- Goat Head Staff Head
(22029,2035,1,1,1,444),-- Tarbar Head
(22029,8343,1,1,1,444),-- Titan Hammer Piece
(22029,8337,1,1,1,444),-- Priest Mace Head
(22029,2036,1,1,1,444),-- Skull Breaker Head
(22029,2037,1,1,1,444),-- Heavy Bone Club Head
(22029,5549,1,1,-1,118798),-- Metallic Thread
(22029,1944,1,1,-1,60596),-- Mithril Tunic Fabric
(22029,6036,1,1,-1,39599),-- Greater Magic Haste Potion
(22030,57,504,1018,0,700000),-- Adena
(22030,1868,1,1,2,251345),-- Thread
(22030,1873,1,1,2,50269),-- Silver Nugget
(22030,1876,1,1,2,25134),-- Mithril Ore
(22030,1895,1,1,2,35906),-- Metalic Fibber
(22030,1953,1,1,1,10064),-- Sage's Rag Lining
(22030,1805,1,1,2,419),-- Recipe: Soulshot: C-Grade
(22030,438,1,1,1,171),-- Sage\'s Rag
(22030,1905,1,1,-1,121725),-- Omen Beast's Eye Earring Gemstone
(22030,2160,1,1,-1,80731),-- Recipe: Moonstone Earring
(22030,1805,1,1,-1,4171),-- Recipe: Soulshot: C-Grade
(22031,57,460,935,0,700000),-- Adena
(22031,1868,1,1,2,233870),-- Thread
(22031,1873,1,1,2,46774),-- Silver Nugget
(22031,1876,1,1,2,23387),-- Mithril Ore
(22031,1895,1,1,2,33410),-- Metalic Fibber
(22031,2054,1,1,1,2488),-- Light Crossbow Shaft
(22031,8303,1,1,2,511),-- Recipe: Mysterious Sword (100%)
(22031,280,1,1,1,42),-- Light Crossbow
(22031,1868,1,5,-1,776220),-- Thread
(22031,1873,1,1,-1,465732),-- Silver Nugget
(22031,1906,1,1,-1,160045),-- Mithril Ring Wire
(22032,57,573,1175,0,700000),-- Adena
(22032,1868,1,1,2,152909),-- Thread
(22032,5549,1,1,2,7645),-- Metallic Thread
(22032,1873,1,1,2,30582),-- Silver Nugget
(22032,1889,1,1,2,5097),-- Compound Braid
(22032,2163,1,1,1,3901),-- Recipe: Earring of Protection
(22032,71,1,1,1,13),-- Flamberge
(22032,72,1,1,1,13),-- Stormbringer
(22032,7887,1,1,1,13),-- 	Mysterious Sword
(22032,8334,1,1,1,911),-- Mysterious Sword Blade
(22032,2059,1,1,1,892),-- Flamberge Blade
(22032,2060,1,1,1,911),-- Stormbringer Blade
(22032,2163,1,1,-1,74282),-- Recipe: Earring of Protection
(22032,1908,1,1,-1,137871),-- Moonstone Earring wire
(22032,4553,1,1,-1,5824),-- Greater Dye of STR
(22032,8723,1,1,200,3058),-- Life Stone: level 46
(22032,8733,1,1,200,765),-- Mid-Grade Life Stone: level 46
(22032,8743,1,1,200,77),-- High-Grade Life Stone: level 46
(22032,8753,1,1,200,15),-- Top-Grade Life Stone: level 46
(22033,57,741,1505,0,700000),-- Adena
(22033,1872,1,1,2,118016),-- Animal Bone
(22033,1867,1,1,2,118016),-- Animal Skin
(22033,1881,1,1,2,11802),-- Coarse Bone Powder
(22033,1875,1,1,2,5901),-- Stone of Purity
(22033,4041,1,1,2,770),-- Mold Hardener
(22033,8723,1,1,200,3541),-- Life Stone: level 46
(22033,8733,1,1,200,885),-- Mid-Grade Life Stone: level 46
(22033,8743,1,1,200,89),-- High-Grade Life Stone: level 46
(22033,8753,1,1,200,18),-- Top-Grade Life Stone: level 46
(22033,439,1,1,1,152),-- Karmian Tunic
(22033,471,1,1,1,243),-- Karmian Stockings
(22033,1954,1,1,1,11765),-- Karmian Stocking Design
(22033,1962,1,1,1,7081),-- Karmian Tunic Pattern
(22033,2164,1,1,2,6809),-- Recipe: Ring of Protection
(22033,1909,1,1,-1,252912),-- Aquastone Ring Wire
(22033,2164,1,1,-1,144063),-- Recipe: Ring of Protection
(22033,1910,1,1,-1,141025),-- Aquastone Necklace Chain
(22034,57,481,871,0,700000),-- Adena
(22034,1872,1,1,2,108227),-- Animal Bone
(22034,1867,1,1,2,108227),-- Animal Skin
(22034,1881,1,1,2,10823),-- Coarse Bone Powder
(22034,1875,1,1,2,5411),-- Stone of Purity
(22034,4041,1,1,2,706),-- Mold Hardener
(22034,8723,1,1,200,3247),-- Life Stone: level 46
(22034,8733,1,1,200,812),-- Mid-Grade Life Stone: level 46
(22034,8743,1,1,200,81),-- High-Grade Life Stone: level 46
(22034,8753,1,1,200,16),-- Top-Grade Life Stone: level 46
(22034,439,1,1,1,139),-- Karmian Tunic
(22034,471,1,1,1,223),-- Karmian Stockings
(22034,1954,1,1,1,10789),-- Karmian Stocking Design
(22034,1962,1,1,1,6494),-- Karmian Tunic Pattern
(22034,8319,1,1,2,189),-- Recipe: Karik Horn (100%)
(22034,1909,1,1,-1,231934),-- Aquastone Ring Wire
(22034,2164,1,1,-1,132113),-- Recipe: Ring of Protection
(22034,1910,1,1,-1,129328),-- Aquastone Necklace Chain
(22035,57,741,1505,0,700000),-- Adena
(22035,1872,1,1,2,118016),-- Animal Bone
(22035,1867,1,1,2,118016),-- Animal Skin
(22035,1881,1,1,2,11802),-- Coarse Bone Powder
(22035,1875,1,1,2,5901),-- Stone of Purity
(22035,4041,1,1,2,770),-- Mold Hardener
(22035,8723,1,1,200,3541),-- Life Stone: level 46
(22035,8733,1,1,200,885),-- Mid-Grade Life Stone: level 46
(22035,8743,1,1,200,89),-- High-Grade Life Stone: level 46
(22035,8753,1,1,200,18),-- Top-Grade Life Stone: level 46
(22035,439,1,1,1,152),-- Karmian Tunic
(22035,471,1,1,1,243),-- Karmian Stockings
(22035,1954,1,1,1,11765),-- Karmian Stocking Design
(22035,1962,1,1,1,7081),-- Karmian Tunic Pattern
(22035,2213,1,1,2,1374),-- Recipe: Dwarven Chain Mail Shirt
(22035,1909,1,1,-1,252912),-- Aquastone Ring Wire
(22035,2164,1,1,-1,144063),-- Recipe: Ring of Protection
(22035,1910,1,1,-1,141025),-- Aquastone Necklace Chain
(22036,57,689,1193,0,700000),-- Adena
(22036,1872,1,1,2,83144),-- Animal Bone
(22036,1867,1,1,2,83144),-- Animal Skin
(22036,1881,1,1,2,8314),-- Coarse Bone Powder
(22036,1875,1,1,2,4157),-- Stone of Purity
(22036,4041,1,1,2,542),-- Mold Hardener
(22036,8723,1,1,200,2494),-- Life Stone: level 46
(22036,8733,1,1,200,624),-- Mid-Grade Life Stone: level 46
(22036,8743,1,1,200,62),-- High-Grade Life Stone: level 46
(22036,8753,1,1,200,12),-- Top-Grade Life Stone: level 46
(22036,439,1,1,1,107),-- Karmian Tunic
(22036,471,1,1,1,171),-- Karmian Stockings
(22036,1954,1,1,1,8289),-- Karmian Stocking Design
(22036,1962,1,1,1,4989),-- Karmian Tunic Pattern
(22036,2214,1,1,2,1547),-- Recipe: Dwarven Chain Gaiters
(22036,1909,1,1,-1,178179),-- Aquastone Ring Wire
(22036,2164,1,1,-1,101494),-- Recipe: Ring of Protection
(22036,1910,1,1,-1,99354),-- Aquastone Necklace Chain
(22037,57,572,1161,0,700000),-- Adena
(22037,1872,1,1,2,91097),-- Animal Bone
(22037,1867,1,1,2,91097),-- Animal Skin
(22037,1881,1,1,2,9110),-- Coarse Bone Powder
(22037,1875,1,1,2,4555),-- Stone of Purity
(22037,4041,1,1,2,594),-- Mold Hardener
(22037,8723,1,1,200,2733),-- Life Stone: level 46
(22037,8733,1,1,200,683),-- Mid-Grade Life Stone: level 46
(22037,8743,1,1,200,68),-- High-Grade Life Stone: level 46
(22037,8753,1,1,200,14),-- Top-Grade Life Stone: level 46
(22037,439,1,1,1,117),-- Karmian Tunic
(22037,471,1,1,1,188),-- Karmian Stockings
(22037,1954,1,1,1,9081),-- Karmian Stocking Design
(22037,1962,1,1,1,5466),-- Karmian Tunic Pattern
(22037,2215,1,1,1,870),-- Recipe: Robe of Seal
(22037,1875,1,1,-1,96375),-- Stone of Purity
(22037,1960,1,1,-1,79213),-- Chain Gaiters Part
(22037,4555,1,1,-1,5783),-- Greater Dye of CON
(22038,57,686,1395,0,700000),-- Adena
(22038,1871,1,1,2,76903),-- Charcoal
(22038,1866,1,1,2,51269),-- Suede
(22038,1877,1,1,2,3076),-- Adamantite Nugget
(22038,4040,1,1,2,1538),-- Mold Lubricant
(22038,4041,1,1,2,669),-- Mold Hardener
(22038,951,1,1,2,140),-- Scroll: Enchant Weapon (Grade C)
(22038,355,1,1,1,78),-- Dwarven Chain Mail Shirt
(22038,382,1,1,1,124),-- Dwarven Chain Gaiters
(22038,1968,1,1,1,4633),-- Dwarven Chain Mail Shirt Material
(22038,1969,1,1,1,7348),-- Dwarven Chain Gaiters Material
(22038,8724,1,1,200,2563),-- Life Stone: level 49
(22038,8734,1,1,200,641),-- Mid-Grade Life Stone: level 49
(22038,8744,1,1,200,64),-- High-Grade Life Stone: level 49
(22038,8754,1,1,200,13),-- Top-Grade Life Stone: level 49
(22038,1911,1,1,-1,129093),-- Earring of Protection Gemstone
(22038,2167,1,1,-1,88554),-- Recipe: Ring of Ages
(22038,2166,1,1,-1,58836),-- Recipe: Earrings of Binding (100%)
(22039,57,715,1457,0,700000),-- Adena
(22039,1871,1,1,2,72923),-- Charcoal
(22039,1866,1,1,2,48615),-- Suede
(22039,1877,1,1,2,2917),-- Adamantite Nugget
(22039,4040,1,1,2,1459),-- Mold Lubricant
(22039,4041,1,1,2,634),-- Mold Hardener
(22039,8724,1,1,200,2431),-- Life Stone: level 49
(22039,8734,1,1,200,608),-- Mid-Grade Life Stone: level 49
(22039,8744,1,1,200,61),-- High-Grade Life Stone: level 49
(22039,8754,1,1,200,12),-- Top-Grade Life Stone: level 49
(22039,952,1,1,2,972),-- Scroll: Enchant Armor (Grade C)
(22039,73,1,1,1,9),-- Shamshir
(22039,74,1,1,1,9),-- Katana
(22039,131,1,1,1,9),-- Spirit Sword
(22039,133,1,1,1,9),-- Raid Sword
(22039,2077,1,1,1,666),-- Shamshir Blade
(22039,2078,1,1,1,666),-- Katana Blade
(22039,2080,1,1,1,666),-- Spirit Sword Blade
(22039,2081,1,1,1,666),-- Raid Sword Blade
(22039,8318,1,1,2,119),-- Recipe: Dwarven Hammer (100%)
(22039,1911,1,1,-1,195087),-- Earring of Protection Gemstone
(22039,1877,1,1,-1,72416),-- Adamantite Nugget
(22039,951,1,1,-1,3292),-- Scroll: Enchant Weapon (Grade C)
(22040,57,549,1120,0,700000),-- Adena
(22040,1342,10,30,2,112006),-- Iron Arrow
(22040,1872,1,1,2,74670),-- Animal Bone
(22040,1867,1,1,2,74670),-- Animal Skin
(22040,1875,1,1,2,3734),-- Stone of Purity
(22040,1881,1,1,2,7467),-- Coarse Bone Powder
(22040,4041,1,1,2,487),-- Mold Hardener
(22040,2166,1,1,2,1898),-- Recipe: Earrings of Binding (100%)
(22040,282,1,1,1,14),-- Elemental Bow
(22040,285,1,1,1,14),-- Noble Elven Bow
(22040,2085,1,1,1,1066),-- Crossbow Shaft
(22040,2086,1,1,1,1066),-- Elven Bow of Nobility Shaft
(22040,8724,1,1,200,1867),-- Life Stone: level 49
(22040,8734,1,1,200,467),-- Mid-Grade Life Stone: level 49
(22040,8744,1,1,200,47),-- High-Grade Life Stone: level 49
(22040,8754,1,1,200,9),-- Top-Grade Life Stone: level 49
(22040,1913,1,1,-1,84545),-- Necklace of Protection Gemstone
(22040,4041,1,1,-1,12090),-- Mold Hardener
(22040,4589,1,1,-1,4634),-- Greater Dye of STR
(22041,57,917,1863,0,700000),-- Adena
(22041,1872,1,1,2,127785),-- Animal Bone
(22041,1867,1,1,2,127785),-- Animal Skin
(22041,1875,1,1,2,6389),-- Stone of Purity
(22041,1881,1,1,2,12778),-- Coarse Bone Powder
(22041,4041,1,1,2,833),-- Mold Hardener
(22041,2166,1,1,2,3249),-- Recipe: Earrings of Binding (100%)
(22041,162,1,1,1,4),-- War Axe
(22041,174,1,1,1,4),-- Nirvana Axe
(22041,196,1,1,1,4),-- Stick of Eternity
(22041,197,1,1,1,4),-- Paradia Staff
(22041,199,1,1,1,4),-- Pa\'agrian Hammer
(22041,200,1,1,1,4),-- Sage\'s Staff
(22041,201,1,1,1,4),-- Club of Nature
(22041,202,1,1,1,4),-- Mace of The Underworld
(22041,2097,1,1,1,321),-- War Axe Blade
(22041,2098,1,1,1,321),-- Nirvana Axe Blade
(22041,2099,1,1,1,338),-- Stick of Eternity Shaft
(22041,2100,1,1,1,338),-- Paradia Staff Head
(22041,2102,1,1,1,326),-- Pa\'agrian Hammer Head
(22041,2103,1,1,1,338),-- Sage\'s Staff Head
(22041,2104,1,1,1,338),-- Club of Nature Head
(22041,2105,1,1,1,321),-- Mace of The Underworld Head
(22041,8725,1,1,200,2662),-- Life Stone: level 52
(22041,8735,1,1,200,665),-- Mid-Grade Life Stone: level 52
(22041,8745,1,1,200,67),-- High-Grade Life Stone: level 52
(22041,8755,1,1,200,13),-- Top-Grade Life Stone: level 52
(22041,1913,1,1,-1,140926),-- Necklace of Protection Gemstone
(22041,4041,1,1,-1,20152),-- Mold Hardener
(22041,4589,1,1,-1,7725),-- Greater Dye of STR
(22042,57,841,1709,0,700000),-- Adena
(22042,1872,1,1,2,117186),-- Animal Bone
(22042,1867,1,1,2,117186),-- Animal Skin
(22042,1875,1,1,2,5859),-- Stone of Purity
(22042,1881,1,1,2,11719),-- Coarse Bone Powder
(22042,4041,1,1,2,764),-- Mold Hardener
(22042,2166,1,1,2,2979),-- Recipe: Earrings of Binding (100%)
(22042,162,1,1,1,3),-- War Axe
(22042,174,1,1,1,3),-- Nirvana Axe
(22042,196,1,1,1,3),-- Stick of Eternity
(22042,197,1,1,1,3),-- Paradia Staff
(22042,199,1,1,1,3),-- Pa\'agrian Hammer
(22042,200,1,1,1,3),-- Sage\'s Staff
(22042,201,1,1,1,3),-- Club of Nature
(22042,202,1,1,1,3),-- Mace of The Underworld
(22042,2097,1,1,1,294),-- War Axe Blade
(22042,2098,1,1,1,294),-- Nirvana Axe Blade
(22042,2099,1,1,1,310),-- Stick of Eternity Shaft
(22042,2100,1,1,1,310),-- Paradia Staff Head
(22042,2102,1,1,1,299),-- Pa\'agrian Hammer Head
(22042,2103,1,1,1,310),-- Sage\'s Staff Head
(22042,2104,1,1,1,310),-- Club of Nature Head
(22042,2105,1,1,1,294),-- Mace of The Underworld Head
(22042,8725,1,1,200,2441),-- Life Stone: level 52
(22042,8735,1,1,200,610),-- Mid-Grade Life Stone: level 52
(22042,8745,1,1,200,61),-- High-Grade Life Stone: level 52
(22042,8755,1,1,200,12),-- Top-Grade Life Stone: level 52
(22042,1913,1,1,-1,129237),-- Necklace of Protection Gemstone
(22042,4041,1,1,-1,18481),-- Mold Hardener
(22042,4589,1,1,-1,7084),-- Greater Dye of STR
(22043,57,917,1863,0,700000),-- Adena
(22043,1872,1,1,2,116168),-- Animal Bone
(22043,1867,1,1,2,116168),-- Animal Skin
(22043,1875,1,1,2,5808),-- Stone of Purity
(22043,1881,1,1,2,11617),-- Coarse Bone Powder
(22043,4041,1,1,2,758),-- Mold Hardener
(22043,1539,1,1,2,19362),-- Greater Healing Potion
(22043,2166,1,1,2,2953),-- Recipe: Earrings of Binding (100%)
(22043,162,1,1,1,4),-- War Axe
(22043,174,1,1,1,4),-- Nirvana Axe
(22043,196,1,1,1,4),-- Stick of Eternity
(22043,197,1,1,1,4),-- Paradia Staff
(22043,199,1,1,1,4),-- Pa\'agrian Hammer
(22043,200,1,1,1,4),-- Sage\'s Staff
(22043,201,1,1,1,4),-- Club of Nature
(22043,202,1,1,1,4),-- Mace of The Underworld
(22043,2097,1,1,1,321),-- War Axe Blade
(22043,2098,1,1,1,321),-- Nirvana Axe Blade
(22043,2099,1,1,1,338),-- Stick of Eternity Shaft
(22043,2100,1,1,1,338),-- Paradia Staff Head
(22043,2102,1,1,1,326),-- Pa\'agrian Hammer Head
(22043,2103,1,1,1,338),-- Sage\'s Staff Head
(22043,2104,1,1,1,338),-- Club of Nature Head
(22043,2105,1,1,1,321),-- Mace of The Underworld Head
(22043,8725,1,1,200,2420),-- Life Stone: level 52
(22043,8735,1,1,200,605),-- Mid-Grade Life Stone: level 52
(22043,8745,1,1,200,60),-- High-Grade Life Stone: level 52
(22043,8755,1,1,200,12),-- Top-Grade Life Stone: level 52
(22043,1913,1,1,-1,140926),-- Necklace of Protection Gemstone
(22043,4041,1,1,-1,20152),-- Mold Hardener
(22043,4589,1,1,-1,7725),-- Greater Dye of STR
(22044,57,646,1313,0,700000),-- Adena
(22044,1872,1,1,2,90026),-- Animal Bone
(22044,1867,1,1,2,90026),-- Animal Skin
(22044,1875,1,1,2,4501),-- Stone of Purity
(22044,1881,1,1,2,9003),-- Coarse Bone Powder
(22044,4041,1,1,2,587),-- Mold Hardener
(22044,2166,1,1,2,2289),-- Recipe: Earrings of Binding (100%)
(22044,162,1,1,1,3),-- War Axe
(22044,174,1,1,1,3),-- Nirvana Axe
(22044,196,1,1,1,3),-- Stick of Eternity
(22044,197,1,1,1,3),-- Paradia Staff
(22044,199,1,1,1,3),-- Pa\'agrian Hammer
(22044,200,1,1,1,3),-- Sage\'s Staff
(22044,201,1,1,1,3),-- Club of Nature
(22044,202,1,1,1,3),-- Mace of The Underworld
(22044,2097,1,1,1,226),-- War Axe Blade
(22044,2098,1,1,1,226),-- Nirvana Axe Blade
(22044,2099,1,1,1,238),-- Stick of Eternity Shaft
(22044,2100,1,1,1,238),-- Paradia Staff Head
(22044,2102,1,1,1,229),-- Pa\'agrian Hammer Head
(22044,2103,1,1,1,238),-- Sage\'s Staff Head
(22044,2104,1,1,1,238),-- Club of Nature Head
(22044,2105,1,1,1,226),-- Mace of The Underworld Head
(22044,8725,1,1,200,1876),-- Life Stone: level 52
(22044,8735,1,1,200,469),-- Mid-Grade Life Stone: level 52
(22044,8745,1,1,200,47),-- High-Grade Life Stone: level 52
(22044,8755,1,1,200,9),-- Top-Grade Life Stone: level 52
(22044,1913,1,1,-1,99284),-- Necklace of Protection Gemstone
(22044,4041,1,1,-1,14198),-- Mold Hardener
(22044,4589,1,1,-1,5442),-- Greater Dye of STR
(22045,57,204,407,0,700000),-- Adena
(22045,1870,1,1,2,50524),-- Coal
(22045,1865,1,1,2,50524),-- Varnish
(22045,1879,1,1,2,8421),-- Cokes
(22045,1875,1,1,2,3368),-- Stone of Purity
(22045,1825,1,1,2,2134),-- 	Recipe: Oriharukon
(22045,1948,1,1,1,2134),-- Boots of Power Pattern
(22045,2923,1,1,1,2134),-- Manticore Skin Boot Lining
(22045,2924,1,1,1,2134),-- Brigandine Boots Design
(22045,2925,1,1,1,2134),-- Elven Mithril Boots Design
(22045,2926,1,1,1,2134),-- Manticore Skin Gloves Lining
(22045,2927,1,1,1,2134),-- Brigandine Gauntlets Pattern
(22045,2928,1,1,1,2134),-- Elven Mithril Gloves Design
(22045,1946,1,1,1,2134),-- Rip Gauntlets Pattern
(22045,606,1,1,1,55),-- Rip Gauntlets
(22045,1124,1,1,1,55),-- Boots of Power
(22045,2424,1,1,1,55),-- Manticore Skin Boots
(22045,2425,1,1,1,55),-- Brigandine Boots
(22045,2426,1,1,1,55),-- Elven Mithril Boots
(22045,2448,1,1,1,55),-- Manticore Skin Gloves
(22045,2449,1,1,1,55),-- Brigandine Gauntlets
(22045,2450,1,1,1,55),-- Elven Mithril Gloves
(22045,1879,1,1,-1,81451),-- Cokes
(22045,1875,1,1,-1,49514),-- Stone of Purity
(22045,1902,1,1,-1,32580),-- Elven Earring Beads
(22046,57,216,442,0,700000),-- Adena
(22046,1871,1,1,2,25950),-- Charcoal 
(22046,1866,1,1,2,17300),-- Suede 
(22046,1885,1,1,2,2163),-- High Grade Suede
(22046,5220,1,1,2,1038),-- Metal Hardener
(22046,4041,1,1,2,226),-- Mold Hardener
(22046,8723,1,1,200,1038),-- Life Stone: level 46
(22046,8733,1,1,200,260),-- Mid-Grade Life Stone: level 46
(22046,8743,1,1,200,26),-- High-Grade Life Stone: level 46
(22046,8753,1,1,200,5),-- Top-Grade Life Stone: level 46
(22046,2163,1,1,2,1324),-- Recipe: Earring of Protection
(22046,96,1,1,1,5),-- Scythe
(22046,298,1,1,1,5),-- Orcish Glaive
(22046,302,1,1,1,5),-- Body Slasher
(22046,2062,1,1,1,339),-- Scythe Edge
(22046,2075,1,1,1,339),-- Orcish Glaive Blade
(22046,2076,1,1,1,339),-- Body Slasher Blade
(22046,2163,1,1,-1,28014),-- Recipe: Earring of Protection
(22046,1908,1,1,-1,51995),-- Moonstone Earring wire
(22046,4553,1,1,-1,2196),-- Greater Dye of STR
(22047,57,188,386,0,700000),-- Adena
(22047,1895,1,1,2,10758),-- Metalic Fibber
(22047,439,1,1,1,39),-- Karmian Tunic
(22047,471,1,1,1,62),-- Karmian Stockings
(22047,1954,1,1,1,3003),-- Karmian Stocking Design
(22047,1962,1,1,1,1807),-- Karmian Tunic Pattern
(22047,2166,1,1,1,2896),-- Recipe: Ring of Protection
(22047,8723,1,1,200,1506),-- Life Stone: level 46
(22047,8733,1,1,200,377),-- Mid-Grade Life Stone: level 46
(22047,8743,1,1,200,38),-- High-Grade Life Stone: level 46
(22047,8753,1,1,200,8),-- Top-Grade Life Stone: level 46
(22047,1910,1,1,-1,35994),-- Aquastone Necklace Chain
(22047,2166,1,1,-1,36770),-- Recipe: Ring of Protection
(22047,1909,1,1,-1,64552),-- Aquastone Ring Wire
(22048,57,183,370,0,700000),-- Adena
(22048,1879,1,1,2,6345),-- Cokes
(22048,1885,1,1,2,3172),-- High Grade Suede
(22048,1905,1,1,1,3703),-- Omen Beast's Eye Earring Gemstone
(22048,1907,1,1,1,2648),-- Necklace of Darkness Gem
(22048,5550,1,1,2,508),-- Durable Metal Plate
(22048,5549,1,1,2,3807),-- Metallic Thread
(22048,1805,1,1,2,127),-- Recipe: Soulshot: C-Grade
(22048,882,1,1,1,122),-- Mithril Ring
(22048,851,1,1,2,82),-- Omen Beast's Eye Earring 
(22048,914,1,1,2,61),-- Necklace of Darkness 
(22048,1906,1,1,1,5233),-- Mithril Ring Wire
(22048,1875,1,1,2,2538),-- Stone of Purity
(22048,1875,1,1,-1,30323),-- Stone of Purity
(22048,1879,1,1,-1,75807),-- Cokes
(22048,4469,1,1,-1,2599),-- Dye of STR <Str+1 Con-1>
(22049,57,246,502,0,700000),-- Adena
(22049,1872,1,1,2,36844),-- Animal Bone
(22049,1867,1,1,2,36844),-- Animal Skin
(22049,1971,1,1,1,2820),-- Great Helmet Material
(22049,2947,1,1,1,3290),-- Dwarven Chain Shield Fragment
(22049,1881,1,1,2,3684),-- Coarse Bone Powder
(22049,1972,1,1,1,3685),-- Knight's Shield Fragment
(22049,1875,1,1,2,1842),-- Stone of Purity
(22049,4041,1,1,2,240),-- Mold Hardener
(22049,8724,1,1,200,921),-- Life Stone: level 49
(22049,8734,1,1,200,230),-- Mid-Grade Life Stone: level 49
(22049,8744,1,1,200,23),-- High-Grade Life Stone: level 49
(22049,8754,1,1,200,5),-- Top-Grade Life Stone: level 49
(22049,500,1,1,1,50),-- Great Helmet
(22049,632,1,1,1,71),-- Knight\'s Shield
(22049,2496,1,1,1,77),-- Dwarven Chain Shield
(22049,951,1,1,2,50),-- Scroll: Enchant Weapon (C)
(22049,1911,1,1,-1,46387),-- Earring of Protection Gemstone
(22049,2166,1,1,-1,21141),-- Recipe: Earrings of Binding (100%)
(22049,1912,1,1,-1,67206),-- Ring of Protection Gemstone
(22050,57,628,1284,0,700000),-- Adena
(22050,1872,1,1,2,100438),-- Animal Bone
(22050,1867,1,1,2,100438),-- Animal Skin
(22050,1881,1,1,2,10044),-- Coarse Bone Powder
(22050,1875,1,1,2,5022),-- Stone of Purity
(22050,2163,1,1,2,3843),-- Recipe: Earring of Protection
(22050,2070,1,1,1,1495),-- Cursed Dagger Blade
(22050,8723,1,1,200,3013),-- Life Stone: level 46
(22050,8733,1,1,200,753),-- Mid-Grade Life Stone: level 46
(22050,8743,1,1,200,75),-- High-Grade Life Stone: level 46
(22050,8753,1,1,200,15),-- Top-Grade Life Stone: level 46
(22050,2072,1,1,1,1495),-- Dark Elven Dagger Edge
(22050,4041,1,1,2,655),-- Mold Hardener
(22050,226,1,1,1,21),-- Cursed Dagger
(22050,232,1,1,1,21),-- Dark Elven Dagger
(22050,1881,1,1,-1,212515),-- Coarse Bone Powder
(22050,2163,1,1,-1,81320),-- Recipe: Earring of Protection
(22050,1908,1,1,-1,150934),-- Moonstone Earring Wire
(22051,57,482,984,0,700000),-- Adena
(22051,2951,1,1,1,3794),-- Rind Leather Gloves Design
(22051,1974,1,1,1,3794),-- Crimson Boot Fabric
(22051,2948,1,1,1,3794),-- Rind Leather Boot Lining
(22051,2949,1,1,1,3794),-- Dwarven Chain Gloves Design
(22051,2950,1,1,1,3794),-- Gloves of Seal Pattern
(22051,2946,1,1,1,2521),-- Plate Leather Gloves Design
(22051,1894,1,1,2,2110),-- Crafted Leather
(22051,4039,1,1,2,2005),-- Mold Glue
(22051,4044,1,1,2,2005),-- Thons
(22051,8724,1,1,200,2005),-- Life Stone: level 49
(22051,8734,1,1,200,501),-- Mid-Grade Life Stone: level 49
(22051,8744,1,1,200,50),-- High-Grade Life Stone: level 49
(22051,8754,1,1,200,10),-- Top-Grade Life Stone: level 49
(22051,4041,1,1,2,523),-- Mold Hardener
(22051,1126,1,1,1,73),-- Crimson Boots
(22051,2434,1,1,2,73),-- Rind Leather Boots 
(22051,2455,1,1,2,79),-- Plated Leather Gloves 
(22051,2456,1,1,2,73),-- Dwarven Chain Gloves 
(22051,2457,1,1,2,73),-- Gloves of Seal 
(22051,2458,1,1,2,73),-- Rind Leather Gloves 
(22051,951,1,1,1,109),-- Scroll: Enchant Weapon (Grade C)
(22051,1894,1,1,-1,42869),-- Crafted Leather
(22051,4589,1,1,-1,4073),-- Greater Dye of STR <Str+2 Con-2>
(22051,1911,1,1,-1,90871),-- Earring of Protection Gemstone
(22052,57,707,1443,0,700000),-- Adena
(22052,1342,10,30,2,144321),-- Fine Steel Arrow
(22052,1873,1,1,2,28864),-- Silver Nugget
(22052,1895,1,1,2,20617),-- Metalic Fibber
(22052,1876,1,1,2,14432),-- Mithril Ore
(22052,1969,1,1,1,7584),-- Dwarven Chain Gaiters Material
(22052,1968,1,1,1,4782),-- Dwarven Chain Mail Shirt Material
(22052,952,1,1,2,962),-- Scroll: Enchant Armor (Grade C)
(22052,8724,1,1,200,2405),-- Life Stone: level 49
(22052,8734,1,1,200,601),-- Mid-Grade Life Stone: level 49
(22052,8744,1,1,200,50),-- High-Grade Life Stone: level 49
(22052,8754,1,1,200,10),-- Top-Grade Life Stone: level 49
(22052,4042,1,1,2,1203),-- Enria
(22052,8296,1,1,2,168),-- Recipe: Pa'agrian Sword (100%)
(22052,382,1,1,2,128),-- Dwarven Chain Gaiters 
(22052,355,1,1,2,80),-- Dwarven Chain mail Shirt 
(22052,1912,1,1,-1,193047),-- Ring of Protection Gemstone
(22052,2166,1,1,-1,60728),-- Recipe: Earrings of Binding (100%)
(22052,2167,1,1,-1,91402),-- Recipe: Ring of Ages
(22053,57,696,1415,0,700000),-- Adena
(22053,1871,1,1,2,77980),-- Charcoal
(22053,1866,1,1,2,51987),-- Suede
(22053,1972,1,1,1,10398),-- Knight's Shield Fragment
(22053,2947,1,1,1,9284),-- Dwarven Chain Shield Fragment
(22053,1971,1,1,1,7957),-- Great Helmet Material
(22053,1885,1,1,2,6498),-- High Grade Suede
(22053,5220,1,1,2,3119),-- Metal Hardener
(22053,2166,1,1,2,2643),-- Recipe: Earrings of Binding (100%)
(22053,8724,1,1,200,2599),-- Life Stone: level 49
(22053,8734,1,1,200,650),-- Mid-Grade Life Stone: level 49
(22053,8744,1,1,200,65),-- High-Grade Life Stone: level 49
(22053,8754,1,1,200,13),-- Top-Grade Life Stone: level 49
(22053,4041,1,1,2,678),-- Mold Hardener
(22053,632,1,1,2,200),-- Knight's Shield 
(22053,2496,1,1,2,216),-- Dwarven Chain Shield 
(22053,500,1,1,2,140),-- Great Helmet 
(22053,1866,1,3,-1,586654),-- Suede
(22053,1913,1,1,-1,107021),-- Necklace of Protection Gemstone
(22053,952,1,1,-1,23466),-- Scroll: Enchant Armor (Grade C)
(22054,57,718,1459,0,700000),-- Adena
(22054,1864,1,1,2,160855),-- Stem
(22054,1869,1,1,2,80428),-- Iron Ore
(22054,1884,1,1,2,49494),-- Cord
(22054,1878,1,1,2,32171),-- Braided Hemp
(22054,2949,1,1,1,5637),-- Dwarven Chain Gloves Design
(22054,2167,1,1,2,4104),-- Recipe: Ring of Ages
(22054,2948,1,1,1,5637),-- Rind Leather Boot Lining
(22054,2951,1,1,1,5637),-- Rind Leather Gloves Design
(22054,1974,1,1,1,5637),-- Crimson Boot Fabric
(22054,2950,1,1,1,5637),-- Gloves of Seal Pattern
(22054,2946,1,1,1,3746),-- Plate Leather Gloves Design
(22054,4039,1,1,2,2681),-- Mold Glue
(22054,8724,1,1,200,2681),-- Life Stone: level 49
(22054,8734,1,1,200,670),-- Mid-Grade Life Stone: level 49
(22054,8744,1,1,200,67),-- High-Grade Life Stone: level 49
(22054,8754,1,1,200,13),-- Top-Grade Life Stone: level 49
(22054,2455,1,1,1,117),-- Plated Leather Gloves
(22054,2434,1,1,2,108),-- Rind Leather Boots 
(22054,1126,1,1,2,108),-- Crimson Boots 
(22054,2458,1,1,2,108),-- Rind Leather Gloves 
(22054,2457,1,1,2,108),-- Gloves of Seal 
(22054,2456,1,1,2,108),-- Dwarven Chain Gloves 
(22054,1884,1,3,-1,558523),-- Cord
(22054,4590,1,1,-1,6051),-- Greater Dye of STR <Str+2 Dex-2>
(22054,1966,1,1,-1,66272),-- Plated Leather Fragment
(22055,57,770,1571,0,700000),-- Adena
(22055,1870,1,1,2,78585),-- Coal
(22055,1865,1,1,2,78585),-- Varnish
(22055,1539,1,1,2,17464),-- Greater Healing Potion
(22055,1879,1,1,2,13098),-- Cokes
(22055,1972,1,1,1,11526),-- Knight's Shield Fragment
(22055,2947,1,1,1,10291),-- Dwarven Chain Shield Fragment
(22055,1971,1,1,1,8820),-- Great Helmet Material
(22055,1875,1,1,2,5239),-- Stone of Purity
(22055,4040,1,1,2,1572),-- Mold Lubricant
(22055,8724,1,1,200,2619),-- Life Stone: level 49
(22055,8734,1,1,200,655),-- Mid-Grade Life Stone: level 49
(22055,8744,1,1,200,66),-- High-Grade Life Stone: level 49
(22055,8754,1,1,200,13),-- Top-Grade Life Stone: level 49
(22055,8318,1,1,2,128),-- Recipe: Dwarven Hammer (100%)
(22055,632,1,1,2,221),-- Knight's Shield 
(22055,2496,1,1,2,240),-- Dwarven Chain Shield 
(22055,500,1,1,2,155),-- Great Helmet
(22055,2168,1,1,-1,49644),-- Recipe: Necklace of Mermaid
(22055,1967,1,1,-1,106408),-- Plated Leather Gaiters Material
(22055,2222,1,1,-1,12194),-- Recipe: Composite Armor
(22056,57,693,1413,0,700000),-- Adena
(22056,1873,1,1,2,34562),-- Silver Nugget
(22056,1895,1,1,2,24687),-- Metalic Fibber
(22056,1876,1,1,2,17281),-- Mithril Ore
(22056,1969,1,1,1,7430),-- Dwarven Chain Gaiters Material
(22056,1968,1,1,1,4685),-- Dwarven Chain Mail Shirt Material
(22056,4042,1,1,2,1440),-- Enria
(22056,8724,1,1,200,2880),-- Life Stone: level 49
(22056,2222,1,1,2,540),-- Recipe: Composite Armor
(22056,8734,1,1,200,720),-- Mid-Grade Life Stone: level 49
(22056,382,1,1,2,126),-- Dwarven Chain Gaiters 
(22056,355,1,1,2,78),-- Dwarven Chain mail Shirt 
(22056,8744,1,1,200,72),-- High-Grade Life Stone: level 49
(22056,8754,1,1,200,14),-- Top-Grade Life Stone: level 49
(22056,1876,1,1,-1,351020),-- Mithril Ore
(22056,1968,1,1,-1,42292),-- Dwarven Chain Mail Shirt Material
(22056,951,1,1,-1,3191),-- Scroll: Enchant Weapon (Grade C)
(22057,57,549,1115,0,700000),-- Adena
(22057,1872,1,1,2,76491),-- Animal Bone
(22057,1867,1,1,2,76491),-- Animal Skin
(22057,1881,1,1,2,7649),-- Coarse Bone Powder
(22057,1915,1,1,1,8423),-- Ring of Ages Gemstone
(22057,1914,1,1,1,5371),-- Earring of Binding Gemstone
(22057,1916,1,1,1,4068),-- Necklace of Mermaid Teardrop
(22057,1875,1,1,2,3825),-- Stone of Purity
(22057,2169,1,1,2,1015),-- Recipe: Necklace of Binding
(22057,4041,1,1,2,499),-- Mold Hardener
(22057,8725,1,1,200,1594),-- Life Stone: level 52
(22057,8735,1,1,200,398),-- Mid-Grade Life Stone: level 52
(22057,8745,1,1,200,40),-- High-Grade Life Stone: level 52
(22057,8755,1,1,200,8),-- Top-Grade Life Stone: level 52
(22057,885,1,1,2,127),-- Ring of Ages 
(22057,917,1,1,2,63),-- Necklace of Mermaid 
(22057,854,1,1,2,84),-- Earring of Binding 
(22057,1539,1,1,-1,308276),-- Greater Healing Potion
(22057,1875,1,1,-1,92483),-- Stone of Purity
(22057,4601,1,1,-1,3083),-- Greater Dye of STR <Str+3 Con-3>
(22058,57,839,1705,0,700000),-- Adena
(22058,1342,10,30,2,159313),-- Fine Steel Arrow
(22058,1864,1,1,2,159312),-- Stem
(22058,1869,1,1,2,79656),-- Iron Ore
(22058,1915,1,1,1,12865),-- Ring of Ages Gemstone
(22058,1914,1,1,1,8203),-- Earring of Binding Gemstone
(22058,1916,1,1,1,6214),-- Necklace of Mermaid Teardrop
(22058,1880,1,1,2,7965),-- Steel
(22058,2170,1,1,2,1878),-- Recipe: Nassen's Earring
(22058,4043,1,1,2,2655),-- Asofe
(22058,5550,1,1,2,1062),-- Durable Metal Plate
(22058,8725,1,1,200,2213),-- Life Stone: level 52
(22058,885,1,1,2,193),-- Ring of Ages
(22058,854,1,1,2,129),-- Earring of Binding
(22058,917,1,1,2,97),-- Necklace of Mermaid
(22058,8735,1,1,200,553),-- Mid-Grade Life Stone: level 52
(22058,8745,1,1,200,55),-- High-Grade Life Stone: level 52
(22058,8755,1,1,200,11),-- Top-Grade Life Stone: level 52
(22058,1914,1,1,-1,119035),-- Earring of Binding Gemstone
(22058,2169,1,1,-1,37501),-- Recipe: Necklace of Binding
(22058,1915,1,1,-1,186681),-- Ring of Ages Gemstone
(22059,57,2198,4465,0,700000),-- Adena
(22059,1869,1,3,2,76509),-- Iron Ore
(22059,1864,1,3,2,229526),-- Stem
(22059,1880,1,1,2,22953),-- Steel
(22059,4043,1,1,1,7651),-- Asofe
(22059,2108,1,1,1,6147),-- Fisted Blade Piece
(22059,265,1,1,1,69),-- Fisted Blade
(22059,5550,1,1,2,3060),-- Durable Metal Plate
(22059,2231,1,1,2,989),-- Recipe: Full Plate Armor
(22059,1869,1,11,-1,925052),-- Iron Ore
(22059,2170,1,1,-1,130904),-- Recipe: Nassen's Earring
(22059,1916,1,1,-1,236183),-- Necklace of Mermaid Teardrop
(22059,8725,1,1,200,6376),-- Life Stone: level 52
(22059,8735,1,1,200,1594),-- Mid-Grade Life Stone: level 52
(22059,8745,1,1,200,159),-- High-Grade Life Stone: level 52
(22059,8755,1,1,200,32),-- Top-Grade Life Stone: level 52
(22060,57,1956,3952,0,700000),-- Adena
(22060,1865,1,1,2,203591),-- Varnish
(22060,1870,1,1,2,203591),-- Coal
(22060,4039,1,1,2,6786),-- Mold Glue
(22060,4043,1,1,2,6786),-- Asofe
(22060,1874,1,1,2,13573),-- Oriharukon Ore
(22060,8304,1,1,2,332),-- Recipe: Ecliptic Sword (100%)
(22060,8329,1,1,1,757),-- Pa'agrian Sword Blade
(22060,2091,1,1,1,793),-- Tsurugi Blade
(22060,2089,1,1,1,793),-- Caliburs Edge
(22060,2090,1,1,1,793),-- Delusional Blade
(22060,2095,1,1,1,793),-- Sword of Nightmare Blade
(22060,2096,1,1,1,793),-- Sword of Whispering Death Blade
(22060,2092,1,1,1,793),-- Homunkulus\'s Sword Blade
(22060,75,1,1,1,9),-- Caliburs
(22060,76,1,1,1,9),-- Sword of Delusion
(22060,77,1,1,1,9),-- Tsurugi
(22060,84,1,1,1,9),-- Homunkulus\'s Sword
(22060,134,1,1,1,9),-- Sword of Nightmare
(22060,145,1,1,1,9),-- Sword of Whispering Death
(22060,7882,1,1,1,9),-- Pa'agrian Sword
(22060,1870,1,11,-1,820524),-- Coal
(22060,4602,1,1,-1,10940),-- Greater Dye of STR <Str+3 Dex-3>
(22060,1539,1,3,-1,547016),-- Greater Healing Potion
(22060,8725,1,1,200,5655),-- Life Stone: level 52
(22060,8735,1,1,200,1414),-- Mid-Grade Life Stone: level 52
(22060,8745,1,1,200,141),-- High-Grade Life Stone: level 52
(22060,8755,1,1,200,28),-- Top-Grade Life Stone: level 52
(22062,57,1813,3664,0,700000),-- Adena
(22062,1882,1,1,2,41949),-- Leather
(22062,1867,1,3,2,125846),-- Animal Skin
(22062,1872,1,1,2,251692),-- Animal Bone
(22062,1894,1,1,2,6623),-- Crafted Leather
(22062,4042,1,1,2,3146),-- Enria
(22062,2234,1,1,2,2813),-- Recipe: Divine Stockings
(22062,2106,1,1,1,2572),-- Grace Dagger Edge
(22062,2107,1,1,1,2572),-- Dark Screamer Edge
(22062,231,1,1,1,29),-- Grace Dagger
(22062,233,1,1,1,29),-- Dark Screamer
(22062,8725,1,1,200,5243),-- Life Stone: level 52
(22062,8735,1,1,200,1311),-- Mid-Grade Life Stone: level 52
(22062,8745,1,1,200,131),-- High-Grade Life Stone: level 52
(22062,8755,1,1,200,26),-- Top-Grade Life Stone: level 52
(22062,1981,1,1,-1,268514),-- Mithril Gauntlets Design 
(22062,1894,1,1,-1,160166),-- Crafted Leather
(22062,2232,1,1,-1,26234),-- Recipe: Drake Leather Armor
(22063,57,2440,4930,0,700000),-- Adena
(22063,1873,1,1,2,112881),-- Silver Nugget
(22063,1866,1,1,2,188134),-- Suede
(22063,1864,3,9,2,94067),-- Stem
(22063,1865,2,6,2,70551),-- Varnish
(22063,8725,1,1,200,7839),-- Life Stone: level 52
(22063,8735,1,1,200,1960),-- Mid-Grade Life Stone: level 52
(22063,8745,1,1,200,196),-- High-Grade Life Stone: level 52
(22063,8755,1,1,200,39),-- Top-Grade Life Stone: level 52
(22063,2353,1,1,2,460),-- Recipe: Samurai Longsword
(22063,8329,1,1,1,945),-- Pa'agrian Sword Blade
(22063,2091,1,1,1,989),-- Tsurugi Blade
(22063,2089,1,1,1,989),-- Caliburs Edge
(22063,2090,1,1,1,989),-- Delusional Blade
(22063,2095,1,1,1,989),-- Sword of Nightmare Blade
(22063,2096,1,1,1,989),-- Sword of Whispering Death Blade
(22063,2092,1,1,1,989),-- Homunkulus\'s Sword Blade
(22063,75,1,1,1,11),-- Caliburs
(22063,76,1,1,1,11),-- Sword of Delusion
(22063,77,1,1,1,11),-- Tsurugi
(22063,84,1,1,1,11),-- Homunkulus\'s Sword
(22063,134,1,1,1,11),-- Sword of Nightmare
(22063,145,1,1,1,11),-- Sword of Whispering Death
(22063,7882,1,1,1,11),-- Pa'agrian Sword
(22063,1864,1,25,-1,944873),-- Stem
(22063,1539,1,3,-1,682408),-- Greater Healing Potion
(22063,4603,1,1,-1,13648),-- Greater Dye of CON
(22064,57,2294,4636,0,700000),-- Adena
(22064,1868,4,12,2,54274),-- Thread
(22064,1342,30,90,2,144732),-- Fine Steel Arrow
(22064,1873,1,1,2,86839),-- Silver Nugget
(22064,5549,1,1,2,21710),-- Metallic Thread
(22064,1889,1,1,2,14473),-- Compound Braid
(22064,4044,1,1,2,7237),-- Thons
(22063,8308,1,1,2,354),-- Recipe: Ecliptic Axe (100%)
(22064,2108,1,1,1,6396),-- Fisted Blade Piece
(22064,265,1,1,1,72),-- Fisted Blade
(22064,1983,1,1,-1,253278),-- Shining Circlet Pattern
(22064,2233,1,1,-1,53970),-- Recipe: Divine Tunic
(22064,2089,1,1,-1,31470),-- Caliburs Edge
(22064,8725,1,1,200,6031),-- Life Stone: level 52
(22064,8735,1,1,200,1508),-- Mid-Grade Life Stone: level 52
(22064,8745,1,1,200,151),-- High-Grade Life Stone: level 52
(22064,8755,1,1,200,30),-- Top-Grade Life Stone: level 52
(22065,57,2379,4833,0,700000),-- Adena
(22065,1873,1,1,2,98626),-- Silver Nugget
(22065,1895,1,1,2,70447),-- Metalic Fibber
(22065,1876,1,1,2,49313),-- Mithril Ore
(22065,4042,1,1,2,4109),-- Enria
(22065,2120,1,1,1,5330),-- Great Pata Blade
(22065,266,1,1,1,47),-- Great Pata
(22065,1806,1,1,2,493),-- Recipe: Soulshot: B-Grade
(22065,1806,1,1,-1,12020),-- Recipe: Soulshot: B-Grade
(22065,1895,1,3,-1,858579),-- Metalic Fibber
(22065,1918,1,1,-1,260288),-- Nassen's Earring Gemstone
(22065,8726,1,1,200,5708),-- Life Stone: level 55
(22065,8736,1,1,200,1427),-- Mid-Grade Life Stone: level 55
(22065,8746,1,1,200,143),-- High-Grade Life Stone: level 55
(22065,8756,1,1,200,29),-- Top-Grade Life Stone: level 55
(22066,57,1379,2277,0,700000),-- Adena
(22066,1869,1,1,1,125000),-- Iron Ore
(22066,1864,1,3,1,100000),-- Stem
(22066,1880,1,1,1,15873),-- Steel
(22066,4043,1,1,1,7963),-- Asofe
(22066,4042,1,1,-1,321527),-- Enria
(22066,1920,1,1,-1,97513),-- Necklace of Binding Chain
(22067,57,1327,2505,0,700000),-- Adena
(22067,1869,1,1,1,125000),-- Iron Ore
(22067,1864,1,3,1,100000),-- Stem
(22067,1880,1,1,1,15873),-- Steel
(22067,4043,1,1,1,7963),-- Asofe
(22067,4042,1,1,-1,321527),-- Enria
(22067,1920,1,1,-1,97513),-- Necklace of Binding Chain
(22068,57,1394,2444,0,700000),-- Adena
(22068,1869,1,1,1,125000),-- Iron Ore
(22068,1864,1,3,1,100000),-- Stem
(22068,1880,1,1,1,15873),-- Steel
(22068,4043,1,1,1,7963),-- Asofe
(22068,4042,1,1,-1,321527),-- Enria
(22068,1920,1,1,-1,97513),-- Necklace of Binding Chain
(22069,57,1873,2555,0,700000),-- Adena
(22069,1869,1,1,1,125000),-- Iron Ore
(22069,1864,1,3,1,100000),-- Stem
(22069,1880,1,1,1,15873),-- Steel
(22069,4043,1,1,1,7963),-- Asofe
(22069,4042,1,1,-1,321527),-- Enria
(22069,1920,1,1,-1,97513),-- Necklace of Binding Chain
(22070,57,1310,2602,0,700000),-- Adena
(22070,1869,1,1,1,125000),-- Iron Ore
(22070,1864,1,3,1,100000),-- Stem
(22070,1880,1,1,1,15873),-- Steel
(22070,4043,1,1,1,7963),-- Asofe
(22070,4042,1,1,-1,321527),-- Enria
(22070,1920,1,1,-1,97513),-- Necklace of Binding Chain
(22071,57,1666,2337,0,700000),-- Adena
(22071,1869,1,1,1,125000),-- Iron Ore
(22071,1864,1,3,1,100000),-- Stem
(22071,1880,1,1,1,15873),-- Steel
(22071,4043,1,1,1,7963),-- Asofe
(22071,4042,1,1,-1,321527),-- Enria
(22071,1920,1,1,-1,97513),-- Necklace of Binding Chain
(22072,57,1299,2638,0,700000),-- Adena
(22072,1869,1,1,1,121138),-- Iron Ore
(22072,1864,1,1,1,242276),-- Stem
(22072,1880,1,1,1,12114),-- Steel
(22072,4043,1,1,1,4038),-- Asofe
(22072,8726,1,1,200,2804),-- Life Stone: level 55
(22072,8736,1,1,200,701),-- Mid-Grade Life Stone: level 55
(22072,8746,1,1,200,70),-- High-Grade Life Stone: level 55
(22072,8756,1,1,200,14),-- Top-Grade Life Stone: level 55
(22072,5550,1,1,2,1615),-- Durable Metal Plate
(22072,203,1,1,1,5),-- Pa\'agrian Axe
(22072,204,1,1,1,4),-- Deadman\'s Staff
(22072,205,1,1,1,4),-- Ghoul\'s Staff
(22072,206,1,1,1,4),-- Demon\'s Staff
(22072,7891,1,1,1,4),-- Ecliptic Axe
(22072,7897,1,1,1,4),-- Dwarven Hammer
(22072,2112,1,1,1,419),-- Pa\'agrian Axe Blade
(22072,2116,1,1,1,407),-- Deadman\'s Staff Head
(22072,2117,1,1,1,407),-- Ghoul\'s Staff Head
(22072,2118,1,1,1,407),-- Demon\'s Staff Head
(22072,8338,1,1,1,416),-- Axe of Eclipse Head 
(22072,8344,1,1,1,395),-- Dwarven Hammer Piece
(22072,2503,1,1,1,4),-- Yaksa Mace
(22072,2969,1,1,1,416),-- Yaksa Mace Head
(22072,3034,1,1,2,242),-- Recipe: Spiritshot B
(22072,4042,1,1,-1,54680),-- Enria
(22072,1920,1,1,-1,112261),-- Necklace of Binding Chain
(22072,3034,1,1,-1,6562),-- Recipe: Spiritshot B
(22073,57,2208,2559,0,700000),-- Adena
(22073,1869,1,1,1,125000),-- Iron Ore
(22073,1864,1,3,1,100000),-- Stem
(22073,1880,1,1,1,15873),-- Steel
(22073,4043,1,1,1,7963),-- Asofe
(22073,4042,1,1,-1,321527),-- Enria
(22073,1920,1,1,-1,97513),-- Necklace of Binding Chain
(22074,57,2028,4120,0,700000),-- Adena
(22074,1868,1,3,2,189185),-- Thread
(22074,1873,1,1,2,75674),-- Silver Nugget
(22074,1895,1,1,2,54053),-- Metalic Fibber
(22074,1876,1,1,2,37837),-- Mithril Ore
(22074,4042,1,1,2,3153),-- Enria
(22074,2115,1,1,1,1445),-- Samurai Longsword Blade
(22074,5530,1,1,1,1446),-- Berserker Blade Edge
(22074,8335,1,1,1,1437),-- Ecliptic Sword Blade
(22074,11853,1,1,1,13),-- Samurai Longsword
(22074,11850,1,1,1,13),-- Ecliptic Sword
(22074,11843,1,1,1,13),-- Berserker Blade
(22074,4142,1,1,2,1455),-- Recipe: Avadon Gaiters
(22074,2115,1,1,-1,23447),-- Samurai Longsword Blade
(22074,4141,1,1,-1,24633),-- Recipe: Avadon Breastplate
(22074,2116,1,1,-1,24107),-- Deadman's Staff Head
(22074,8726,1,1,200,4379),-- Life Stone: level 55
(22074,8736,1,1,200,1095),-- Mid-Grade Life Stone: level 55
(22074,8746,1,1,200,109),-- High-Grade Life Stone: level 55
(22074,8756,1,1,200,22),-- Top-Grade Life Stone: level 55
(22075,57,1299,2624,0,700000),-- Adena
(22075,1895,1,1,2,42920),-- Metalic Fibber
(22075,4043,1,1,2,5007),-- Asofe
(22075,4040,1,1,2,3004),-- Mold Lubricant
(22075,2106,1,1,1,1842),-- Grace Dagger Edge
(22075,4042,1,1,2,2504),-- Enria
(22075,2107,1,1,1,1842),-- Dark Screamer Edge
(22075,231,1,1,1,20),-- Grace Dagger
(22075,233,1,1,1,20),-- Dark Screamer
(22075,2169,1,1,2,2659),-- Recipe: Necklace of Binding
(22075,4040,1,1,-1,65387),-- Mold Lubricant
(22075,4604,1,1,-1,7265),-- Greater Dye of CON <Con+3 Dex-3>
(22075,8308,1,1,-1,5333),-- Recipe: Ecliptic Axe (100%)
(22075,8725,1,1,200,4173),-- Life Stone: level 52
(22075,8735,1,1,200,1043),-- Mid-Grade Life Stone: level 52
(22075,8745,1,1,200,104),-- High-Grade Life Stone: level 52
(22075,8755,1,1,200,21),-- Top-Grade Life Stone: level 52
(22076,57,2379,4833,0,700000),-- Adena
(22076,1871,1,1,2,221909),-- Charcoal
(22076,1866,1,1,2,147940),-- Suede
(22076,1885,1,1,2,18492),-- High Grade Suede
(22076,5220,1,1,2,8876),-- Metal Hardener
(22076,5530,1,1,1,1696),-- Berserker Blade Edge
(22076,2115,1,1,1,1694),-- Samurai Longsword Blade
(22076,8335,1,1,1,1685),-- Ecliptic Sword Blade
(22076,4041,1,1,2,1930),-- Mold Hardener
(22076,8321,1,1,2,256),-- Recipe: Ice Storm Hammer (100%)
(22076,8726,1,1,200,5137),-- Life Stone: level 55
(22076,8736,1,1,200,1284),-- Mid-Grade Life Stone: level 55
(22076,8746,1,1,200,128),-- High-Grade Life Stone: level 55
(22076,8756,1,1,200,26),-- Top-Grade Life Stone: level 55
(22076,11853,1,1,1,16),-- Samurai Longsword
(22076,11850,1,1,1,16),-- Ecliptic Sword
(22076,11843,1,1,1,16),-- Berserker Blade
(22076,2117,1,1,-1,28277),-- Ghoul\'s Staff Head
(22076,8310,1,1,-1,6924),-- Recipe: Spell Breaker (60%)
(22076,2118,1,1,-1,28277),-- Demon's Staff Head
(22077,57,2469,5029,0,700000),-- Adena
(22077,1869,1,3,2,76949),-- Iron Ore
(22077,1864,1,3,2,230846),-- Stem
(22077,1880,1,1,2,23085),-- Steel
(22077,4043,1,1,2,7695),-- Asofe
(22077,5550,1,1,2,3078),-- Durable Metal Plate
(22077,8726,1,1,200,5344),-- Life Stone: level 55
(22077,8736,1,1,200,1336),-- Mid-Grade Life Stone: level 55
(22077,8746,1,1,200,134),-- High-Grade Life Stone: level 55
(22077,8756,1,1,200,27),-- Top-Grade Life Stone: level 55
(22077,8310,1,1,2,266),-- Recipe: Spell Breaker (60%)
(22077,203,1,1,1,9),-- Pa\'agrian Axe
(22077,204,1,1,1,7),-- Deadman\'s Staff
(22077,205,1,1,1,7),-- Ghoul\'s Staff
(22077,206,1,1,1,7),-- Demon\'s Staff
(22077,7891,1,1,1,7),-- Ecliptic Axe
(22077,7897,1,1,1,7),-- Dwarven Hammer
(22077,2112,1,1,1,799),-- Pa\'agrian Axe Blade
(22077,2116,1,1,1,776),-- Deadman\'s Staff Head
(22077,2117,1,1,1,776),-- Ghoul\'s Staff Head
(22077,2118,1,1,1,776),-- Demon\'s Staff Head
(22077,8338,1,1,1,792),-- Axe of Eclipse Head 
(22077,8344,1,1,1,753),-- Dwarven Hammer Piece
(22077,2503,1,1,1,7),-- Yaksa Mace
(22077,2969,1,1,1,792),-- Yaksa Mace Head
(22077,1869,1,13,-1,893152),-- Iron Ore
(22077,2119,1,1,-1,28489),-- Crystal Dagger Blade
(22077,8322,1,1,-1,7203),-- Recipe: Ice Storm Hammer (60%)
(22078,57,3630,4907,0,700000),-- Adena
(22078,1870,1,1,2,200000),-- Coal
(22078,1865,1,1,2,200000),-- Varnish
(22078,4039,1,1,2,4608),-- Mold Glue
(22078,4043,1,1,2,5525),-- Asofe
(22078,1986,1,1,1,8333),-- Full Plate Armor Temper
(22078,1874,1,1,2,12048),-- Oriharukon Ore
(22078,4144,1,1,-1,92182),-- Recipe: Zubei\'s Leather Gaiters
(22078,1880,1,1,-1,272544),-- Steel
(22078,2120,1,1,-1,75352),-- Great Pata Blade
(22079,57,559,1135,0,700000),-- Adena
(22079,1871,1,1,2,52120),-- Charcoal
(22079,1866,1,1,2,34747),-- Suede
(22079,1989,1,1,1,3927),-- Divine Stocking Pattern
(22079,1988,1,1,1,2504),-- Divine Tunic Fabric
(22079,1877,1,1,2,2085),-- Adamantite Nugget
(22079,4040,1,1,2,1042),-- Mold Lubricant
(22079,4126,1,1,2,860),-- Recipe: Adamantite Earrings
(22079,4041,1,1,2,453),-- Mold Hardener
(22079,473,1,1,1,51),-- Divine Stockings
(22079,442,1,1,1,32),-- Divine Tunic
(22079,2121,1,1,-1,6491),-- Eminence Bow Shaft
(22079,4145,1,1,-1,6191),-- Recipe: Avadon Leather Armor
(22079,2122,1,1,-1,6443),-- Orcish Poleaxe Blade
(22079,8726,1,1,200,1207),-- Life Stone: level 55
(22079,8736,1,1,200,302),-- Mid-Grade Life Stone: level 55
(22079,8746,1,1,200,30),-- High-Grade Life Stone: level 55
(22079,8756,1,1,200,6),-- Top-Grade Life Stone: level 55
(22080,57,613,1249,0,700000),-- Adena
(22080,1871,1,1,2,57342),-- Charcoal
(22080,1866,1,1,2,38228),-- Suede
(22080,1877,1,1,2,2294),-- Adamantite Nugget
(22080,4040,1,1,2,1147),-- Mold Lubricant
(22080,4041,1,1,2,499),-- Mold Hardener
(22080,4126,1,1,2,946),-- Recipe: Adamantite Earrings
(22080,2917,1,1,1,5489),-- Ring of Binding Gemstone
(22080,1918,1,1,1,4139),-- Nassen\'s Earring Gemstone
(22080,1920,1,1,1,3270),-- Necklace of Binding Chain
(22080,886,1,1,1,88),-- Ring of Binding
(22080,855,1,1,1,59),-- Nassen\'s Earring
(22080,119,1,1,1,44),-- Necklace of Binding
(22080,8726,1,1,200,1327),-- Life Stone: level 55
(22080,8736,1,1,200,332),-- Mid-Grade Life Stone: level 55
(22080,8746,1,1,200,33),-- High-Grade Life Stone: level 55
(22080,8756,1,1,200,7),-- Top-Grade Life Stone: level 55
(22080,1867,1,5,-1,690227),-- Animal Skin
(22080,1872,1,5,-1,690227),-- Animal Bone
(22080,2917,1,1,-1,89202),-- Ring of Binding Gemstone
(22081,57,583,1183,0,700000),-- Adena
(22081,1868,1,1,2,108675),-- Thread
(22081,1873,1,1,2,21735),-- Silver Nugget
(22081,5549,1,1,2,5434),-- Metallic Thread
(22081,1889,1,1,2,3622),-- Compound Braid
(22081,4044,1,1,2,1811),-- Thons
(22081,2955,1,1,1,2457),-- Theca Leather Gloves Texture
(22081,2954,1,1,1,2457),-- Theca Leather Boots Texture
(22081,2961,1,1,1,2114),-- Full Plate Boots Part
(22081,2964,1,1,1,2114),-- Divine Gloves Pattern
(22081,2963,1,1,1,2114),-- Full Plate Gauntlets Part
(22081,2960,1,1,1,2114),-- Drake Leather Boots Design
(22081,2962,1,1,1,2114),-- Drake Leather Gloves Design
(22081,4128,1,1,2,672),-- Recipe: Adamantite Necklace
(22081,2436,1,1,1,37),-- Theca Leather Boots
(22081,2437,1,1,1,37),-- Drake Leather Boots
(22081,2438,1,1,1,28),-- Full Plate Boots
(22081,2460,1,1,1,28),-- Theca Leather Gloves
(22081,2461,1,1,1,28),-- Drake Leather Gloves
(22081,2462,1,1,1,28),-- Full Plate Gauntlets
(22081,2463,1,1,1,28),-- Divine Gloves
(22081,4146,1,1,-1,9434),-- Recipe: Tunic of Zubei
(22081,2954,1,1,-1,93142),-- Theca Leather Boots Texture
(22081,4147,1,1,-1,15125),-- Recipe: Stockings of Zubei
(22081,8726,1,1,200,1258),-- Life Stone: level 55
(22081,8736,1,1,200,314),-- Mid-Grade Life Stone: level 55
(22081,8746,1,1,200,31),-- High-Grade Life Stone: level 55
(22081,8756,1,1,200,6),-- Top-Grade Life Stone: level 55
(22082,57,668,1361,0,700000),-- Adena
(22082,1868,1,1,2,124919),-- Thread
(22082,1873,1,1,2,24984),-- Silver Nugget
(22082,5549,1,1,2,6246),-- Metallic Thread
(22082,1889,1,1,2,4164),-- Compound Braid
(22082,4044,1,1,2,2082),-- Thons
(22082,4128,1,1,2,773),-- Recipe: Adamantite Necklace
(22082,2966,1,1,1,8544),-- Full Plate Shield Fragment
(22082,2959,1,1,1,5401),-- Full Plate Helmet Design
(22082,2497,1,1,1,108),-- Full Plate Shield
(22082,2414,1,1,1,76),-- Full Plate Helmet
(22082,8726,1,1,200,1446),-- Life Stone: level 55
(22082,8736,1,1,200,361),-- Mid-Grade Life Stone: level 55
(22082,8746,1,1,200,36),-- High-Grade Life Stone: level 55
(22082,8756,1,1,200,7),-- Top-Grade Life Stone: level 55
(22082,2955,1,1,-1,107064),-- Theca Leather Gloves Texture
(22082,4041,1,1,-1,14710),-- Mold Hardener
(22082,1872,1,5,-1,751825),-- Animal Bone
(22083,57,603,1228,0,700000),-- Adena
(22083,1871,1,1,2,56296),-- Charcoal
(22083,1866,1,1,2,37531),-- Suede
(22083,2917,1,1,1,5389),-- Ring of Binding Gemstone
(22083,1918,1,1,1,4063),-- Nassen\'s Earring Gemstone
(22083,1877,1,1,2,2252),-- Adamantite Nugget
(22083,1920,1,1,1,3210),-- Necklace of Binding Chain
(22083,4040,1,1,2,1126),-- Mold Lubricant
(22083,4041,1,1,2,489),-- Mold Hardener
(22083,4134,1,1,2,433),-- Recipe: Zubei\'s Gaiters
(22083,886,1,1,1,86),-- Ring of Binding
(22083,119,1,1,1,43),-- Necklace of Binding
(22083,855,1,1,1,58),-- Nassen\'s Earring
(22083,2959,1,1,-1,52739),-- Full Plate Helmet Design
(22083,4148,1,1,-1,6687),-- Recipe: Avadon Robe
(22083,2960,1,1,-1,83021),-- Drake Leather Boots Design
(22083,8726,1,1,200,1303),-- Life Stone: level 55
(22083,8736,1,1,200,326),-- Mid-Grade Life Stone: level 55
(22083,8746,1,1,200,33),-- High-Grade Life Stone: level 55
(22083,8756,1,1,200,7),-- Top-Grade Life Stone: level 55
(22084,57,625,1276,0,700000),-- Adena
(22084,1871,1,1,2,46791),-- Charcoal
(22084,1866,1,1,2,31194),-- Suede
(22084,1877,1,1,2,1872),-- Adamantite Nugget
(22084,4041,1,1,2,407),-- Mold Hardener
(22084,4040,1,1,2,936),-- Mold Lubricant
(22084,4134,1,1,2,360),-- Recipe: Zubei\'s Gaiters
(22084,4070,1,1,1,2792),-- Stockings of Zubei Fabric
(22084,4069,1,1,1,1839),-- Tunic of Zubei Fabric
(22084,4071,1,1,1,1413),-- Avadon Robe Fabric
(22084,12175,1,1,1,249),-- Common Item - Tunic of Zubei (no drop info)
(22084,12176,1,1,1,399),-- Common Item - Stockings of Zubei
(22084,12166,1,1,1,153),-- Common Item - Avadon Robe
(22084,4049,1,1,-1,83088),-- Adamantite Earring Gemstone
(22084,947,1,1,-1,634),-- Scroll: Enchant Weapon (Grade B)
(22084,8311,1,1,-1,1209),-- Recipe: Kaim Vanul\'s Bones (100%)
(22084,8726,1,1,200,1083),-- Life Stone: level 55
(22084,8736,1,1,200,271),-- Mid-Grade Life Stone: level 55
(22084,8746,1,1,200,27),-- High-Grade Life Stone: level 55
(22084,8756,1,1,200,5),-- Top-Grade Life Stone: level 55
(22085,57,552,1125,0,700000),-- Adena
(22085,1870,1,1,2,46886),-- Coal
(22085,1865,1,1,2,46886),-- Varnish
(22085,1874,1,1,2,3126),-- Oriharukon Ore
(22085,2917,1,1,1,4937),-- Ring of Binding Gemstone
(22085,1920,1,1,1,2941),-- Necklace of Binding Chain
(22085,1918,1,1,1,3722),-- Nassen\'s Earring Gemstone
(22085,4039,1,1,2,1563),-- Mold Glue
(22085,4043,1,1,2,1563),-- Asofe
(22085,1806,1,1,1,94),-- Recipe: Soulshot: B-Grade
(22085,886,1,1,1,79),-- Ring of Binding
(22085,855,1,1,1,53),-- Nassen\'s Earring
(22085,119,1,1,1,40),-- Necklace of Binding
(22085,1918,1,1,-1,60494),-- Nassen\'s Earring Gemstone
(22085,4043,1,1,-1,46560),-- Asofe
(22085,1874,1,1,-1,93121),-- Oriharukon Ore
(22085,8297,1,1,2,36),-- Recipe: Guardian\'s Sword (100%) 
(22085,8726,1,1,200,1085),-- Life Stone: level 55
(22085,8736,1,1,200,271),-- Mid-Grade Life Stone: level 55
(22085,8746,1,1,200,27),-- High-Grade Life Stone: level 55
(22085,8756,1,1,200,6),-- Top-Grade Life Stone: level 55
(22086,57,641,1309,0,700000),-- Adena
(22086,1870,1,1,2,43625),-- Coal
(22086,1865,1,1,2,43625),-- Varnish
(22086,1874,1,1,2,2908),-- Oriharukon Ore
(22086,4039,1,1,2,1454),-- Mold Glue
(22086,4043,1,1,2,1454),-- Asofe
(22086,1806,1,1,1,87),-- Recipe: Soulshot: B-Grade
(22086,8297,1,1,2,33),-- Recipe: Guardian\'s Sword (100%)
(22086,4067,1,1,1,2863),-- Zubei\'s Leather Gaiter Texture
(22086,4066,1,1,1,1886),-- Zubei\'s Leather Shirt Fabric
(22086,4068,1,1,1,1449),-- Avadon Leather Armor Lining
(22086,12151,1,1,1,256),-- Common Item - Zubei\'s Leather Shirt
(22086,12150,1,1,1,409),-- Common Item - Zubei\'s Leather Gaiters
(22086,12165,1,1,1,157),-- Common Item - Avadon Leather Armor
(22086,8726,1,1,200,1010),-- Life Stone: level 55
(22086,8736,1,1,200,252),-- Mid-Grade Life Stone: level 55
(22086,8746,1,1,200,25),-- High-Grade Life Stone: level 55
(22086,8756,1,1,200,5),-- Top-Grade Life Stone: level 55
(22086,4049,1,1,-1,85213),-- Adamantite Earring Gemstone
(22086,4130,1,1,-1,26373),-- Recipe: Ring of Black Ore
(22086,4131,1,1,-1,13208),-- Recipe: Necklace of Black Ore
(22087,57,1029,2101,0,700000),-- Adena
(22087,1868,1,1,2,140028),-- Thread
(22087,1864,1,1,2,140028),-- Stem
(22087,1865,1,1,2,70014),-- Varnish
(22087,1866,1,1,2,46676),-- Suede
(22087,1873,1,1,2,28006),-- Silver Nugget
(22087,4110,1,1,1,1085),-- Hell Knife Edge
(22087,4109,1,1,1,1085),-- Kris Edge
(22087,948,1,1,2,175),-- Scroll: Enchant Armor (Grade B)
(22087,8298,1,1,2,53),-- Recipe: Guardian\'s Sword (60%)
(22087,11915,1,1,1,110),-- Common Item - Kris
(22087,11919,1,1,1,110),-- Common Item - Hell Knife
(22087,4050,1,1,-1,161543),-- Adamantite Ring Wire
(22087,948,1,1,-1,6518),-- Scroll: Enchant Armor (Grade B)
(22087,1865,1,5,-1,869100),-- Varnish
(22087,8727,1,1,200,1400),-- Life Stone: level 58
(22087,8737,1,1,200,350),-- Mid-Grade Life Stone: level 58
(22087,8747,1,1,200,35),-- High-Grade Life Stone: level 58
(22087,8757,1,1,200,7),-- Top-Grade Life Stone: level 58
(22088,57,626,1278,0,700000),-- Adena
(22088,1868,1,1,2,85200),-- Thread
(22088,1864,1,1,2,85200),-- Stem
(22088,1865,1,1,2,42600),-- Varnish
(22088,1866,1,1,2,28400),-- Suede
(22088,1873,1,1,2,17040),-- Silver Nugget
(22088,948,1,1,2,106),-- Scroll: Enchant Armor (Grade B)
(22088,8298,1,1,2,32),-- Recipe: Guardian\'s Sword (60%)
(22088,4104,1,1,1,447),-- Great Sword Blade
(22088,4107,1,1,1,440),-- Keshanberk Blade
(22088,4108,1,1,1,440),-- Sword of Valhalla Blade
(22088,11889,1,1,1,45),-- Common Item - Great Sword
(22088,11916,1,1,1,45),-- Common Item - Keshanberk
(22088,11900,1,1,1,45),-- Common Item - Sword of Valhalla
(22088,8727,1,1,200,852),-- Life Stone: level 58
(22088,8737,1,1,200,213),-- Mid-Grade Life Stone: level 58
(22088,8747,1,1,200,21),-- High-Grade Life Stone: level 58
(22088,8757,1,1,200,4),-- Top-Grade Life Stone: level 58
(22088,1867,1,5,-1,705071),-- Animal Skin
(22088,4613,1,1,-1,2188),-- Greater Dye of Strength <Str+4 Con-4>
(22088,4051,1,1,-1,61524),-- Adamantite Necklace Chain
(22089,57,719,1469,0,700000),-- Adena
(22089,1868,1,1,2,107727),-- Thread
(22089,1873,1,1,2,21545),-- Silver Nugget
(22089,1895,1,1,2,15389),-- Metallic Fiber
(22089,1876,1,1,2,10773),-- Mithril Ore
(22089,4042,1,1,2,898),-- Enria
(22089,4105,1,1,1,385),-- Heavy War Axe Head
(22089,4106,1,1,1,379),-- Sprite\'s Staff Head
(22089,8339,1,1,1,385),-- Spell Breaker Head
(22089,8347,1,1,1,346),-- Ice Storm Hammer Head
(22089,8312,1,1,2,41),-- Recipe: Kaim Vanul\'s Bones (60%)
(22089,11918,1,1,1,39),-- Common Item - Heavy War Axe
(22089,11908,1,1,1,39),-- Common Item - Sprite\'s Staff
(22089,11902,1,1,1,39),-- Common Item - Spell Breaker
(22089,11904,1,1,1,39),-- Common Item - Ice Storm Hammer
(22089,4154,1,1,-1,21580),-- Recipe: Doom Shield
(22089,4155,1,1,-1,5663),-- Recipe: Blue Wolf Breastplate
(22089,4056,1,1,-1,33004),-- Zubei\'s Breastplate Part
(22089,8727,1,1,200,1077),-- Life Stone: level 58
(22089,8737,1,1,200,269),-- Mid-Grade Life Stone: level 58
(22089,8747,1,1,200,27),-- High-Grade Life Stone: level 58
(22089,8757,1,1,200,5),-- Top-Grade Life Stone: level 58
(22090,57,747,1521,0,700000),-- Adena
(22090,1868,1,1,2,111720),-- Thread
(22090,1873,1,1,2,22344),-- Silver Nugget
(22090,1895,1,1,2,15960),-- Metallic Fiber
(22090,1876,1,1,2,11172),-- Mithril Ore
(22090,4042,1,1,2,931),-- Enria
(22090,8312,1,1,2,43),-- Recipe: Kaim Vanul\'s Bones (60%)
(22090,4105,1,1,1,400),-- Heavy War Axe Head
(22090,4106,1,1,1,393),-- Sprite\'s Staff Head
(22090,8347,1,1,1,359),-- Ice Storm Hammer Head
(22090,8339,1,1,1,400),-- Spell Breaker Head
(22090,11918,1,1,1,40),-- Common Item - Heavy War Axe
(22090,11908,1,1,1,40),-- Common Item - Sprite\'s Staff
(22090,11902,1,1,1,40),-- Common Item - Spell Breaker
(22090,11904,1,1,1,40),-- Common Item - Ice Storm Hammer
(22090,8727,1,1,200,1117),-- Life Stone: level 58
(22090,8737,1,1,200,279),-- Mid-Grade Life Stone: level 58
(22090,8747,1,1,200,28),-- High-Grade Life Stone: level 58
(22090,8757,1,1,200,5),-- Top-Grade Life Stone: level 58
(22090,4057,1,1,-1,52897),-- Zubei\'s Gaiter Material
(22090,1866,1,3,-1,630360),-- Suede
(22090,947,1,1,-1,756),-- Scroll: Enchant Weapon (Grade B)
(22091,57,787,1603,0,700000),-- Adena
(22091,1870,1,1,2,58842),-- Coal
(22091,1871,1,1,2,58842),-- Charcoal
(22091,1872,1,1,2,78456),-- Animal Bone
(22091,1867,1,1,2,78456),-- Animal Skin
(22091,1869,1,1,2,58842),-- Iron Ore
(22091,4057,1,1,1,2057),-- Zubei\'s Gaiter Material
(22091,4065,1,1,1,2057),-- Avadon Gaiters Material
(22091,4064,1,1,1,1331),-- Avadon Breastplate Part
(22091,4056,1,1,1,1331),-- Zubei\'s Breastplate Part
(22091,4154,1,1,2,696),-- Recipe: Doom Shield
(22091,12161,1,1,1,176),-- Common Item - Zubei\'s Breastplate
(22091,12152,1,1,1,282),-- Common Item - Zubei\'s Gaiters
(22091,12173,1,1,1,176),-- Common Item - Avadon Breastplate
(22091,12172,1,1,1,282),-- Common Item - Avadon Gaiters
(22091,1869,1,5,-1,664016),-- Iron Ore
(22091,4614,1,1,-1,2748),-- Greater Dye of Strength <Str+4 Dex-4>
(22091,8306,1,1,-1,1521),-- Recipe: Wizard\'s Tear (60%)
(22091,8727,1,1,200,1177),-- Life Stone: level 58
(22091,8737,1,1,200,294),-- Mid-Grade Life Stone: level 58
(22091,8747,1,1,200,30),-- High-Grade Life Stone: level 58
(22091,8757,1,1,200,6),-- Top-Grade Life Stone: level 58
(22092,57,816,1662,0,700000),-- Adena
(22092,1870,1,1,2,61033),-- Coal
(22092,1871,1,1,2,61033),-- Charcoal
(22092,1872,1,1,2,81378),-- Animal Bone
(22092,1867,1,1,2,81378),-- Animal Skin
(22092,1869,1,1,2,61033),-- Iron Ore
(22092,4154,1,1,2,722),-- Recipe: Doom Shield
(22092,4104,1,1,1,582),-- Great Sword Blade
(22092,4108,1,1,1,573),-- Sword of Valhalla Blade
(22092,4107,1,1,1,573),-- Keshanberk Blade
(22092,11889,1,1,1,58),-- Common Item - Great Sword
(22092,11916,1,1,1,58),-- Common Item - Keshanberk
(22092,11900,1,1,1,58),-- Common Item - Sword of Valhalla
(22092,8727,1,1,200,1221),-- Life Stone: level 58
(22092,8737,1,1,200,305),-- Mid-Grade Life Stone: level 58
(22092,8747,1,1,200,31),-- High-Grade Life Stone: level 58
(22092,8757,1,1,200,6),-- Top-Grade Life Stone: level 58
(22092,4157,1,1,-1,10280),-- Recipe: Blue Wolf Gaiters
(22092,4065,1,1,-1,57797),-- Avadon Gaiters Material
(22092,8314,1,1,-1,1129),-- Recipe: Spiritual Eye (60%)
(22093,57,1193,2426,0,700000),-- Adena
(22093,1868,1,1,2,162039),-- Thread
(22093,1343,10,30,2,115742),-- Silver Arrow
(22093,1873,1,1,2,32408),-- Silver Nugget
(22093,1895,1,1,2,23148),-- Metallic Fiber
(22093,1876,1,1,2,16204),-- Mithril Ore
(22093,4067,1,1,1,5317),-- Zubei\'s Leather Gaiter Texture
(22093,4066,1,1,1,3502),-- Zubei\'s Leather Shirt Fabric
(22093,4068,1,1,1,2690),-- Avadon Leather Armor Lining
(22093,12151,1,1,1,475),-- Common Item - Zubei\'s Leather Shirt
(22093,12150,1,1,1,760),-- Common Item - Zubei\'s Leather Gaiters
(22093,12165,1,1,1,291),-- Common Item - Avadon Leather Armor
(22093,4042,1,1,2,1350),-- Enria
(22093,4157,1,1,2,403),-- Recipe: Blue Wolf Gaiters
(22093,4615,1,1,-1,4162),-- Greater Dye of Constitution <Con+4 Str-4>
(22093,8300,1,1,-1,1649),-- Recipe: Infernal Master (60%)
(22093,8316,1,1,-1,1117),-- Recipe: Flaming Dragon Skull (60%)
(22093,8728,1,1,200,1350),-- Life Stone: level 61
(22093,8738,1,1,200,337),-- Mid-Grade Life Stone: level 61
(22093,8748,1,1,200,34),-- High-Grade Life Stone: level 61
(22093,8758,1,1,200,7),-- Top-Grade Life Stone: level 61
(22094,57,726,1472,0,700000),-- Adena
(22094,1868,1,1,2,108197),-- Thread
(22094,1873,1,1,2,21639),-- Silver Nugget
(22094,1895,1,1,2,15457),-- Metallic Fiber
(22094,1876,1,1,2,10820),-- Mithril Ore
(22094,4042,1,1,2,902),-- Enria
(22094,4157,1,1,2,269),-- Recipe: Blue Wolf Gaiters
(22094,4105,1,1,1,387),-- Heavy War Axe Head
(22094,8339,1,1,1,387),-- Spell Breaker Head
(22094,4106,1,1,1,381),-- Sprite\'s Staff Head
(22094,8347,1,1,1,348),-- Ice Storm Hammer Head
(22094,11918,1,1,1,39),-- Common Item - Heavy War Axe
(22094,11908,1,1,1,39),-- Common Item - Sprite\'s Staff
(22094,11902,1,1,1,39),-- Common Item - Spell Breaker
(22094,11904,1,1,1,39),-- Common Item - Ice Storm Hammer
(22094,8728,1,1,200,902),-- Life Stone: level 61
(22094,8738,1,1,200,225),-- Mid-Grade Life Stone: level 61
(22094,8748,1,1,200,22),-- High-Grade Life Stone: level 61
(22094,8758,1,1,200,4),-- Top-Grade Life Stone: level 61
(22094,4042,1,1,-1,30524),-- Enria
(22094,948,1,1,-1,4579),-- Scroll: Enchant Armor (B)
(22094,4066,1,1,-1,43180),-- 	Zubei's Leather Shirt Fabric
(22095,57,753,1526,0,700000),-- Adena
(22095,1343,10,30,2,66784),-- Silver Arrow
(22095,1870,1,1,2,46749),-- Coal
(22095,1865,1,1,2,46749),-- Varnish
(22095,1879,1,1,2,7791),-- Cokes
(22095,1875,1,1,2,3117),-- Stone of Purity
(22095,4040,1,1,2,935),-- Mold Lubricant
(22095,4109,1,1,1,790),-- Kris Edge
(22095,4110,1,1,1,790),-- Hell Knife Edge
(22095,4159,1,1,2,132),-- Recipe: Blue Wolf Leather Armor
(22095,8306,1,1,2,36),-- Recipe: Wizard\'s Tear (60%)
(22095,11915,1,1,1,80),-- Common Item - Kris
(22095,11919,1,1,1,80),-- Common Item - Hell Knife
(22095,4159,1,1,-1,5380),-- Recipe: Blue Wolf Leather Armor
(22095,4160,1,1,-1,5380),-- Recipe: Leather Armor of Doom
(22095,4068,1,1,-1,34399),-- Avadon Leather Armor Lining
(22095,8728,1,1,200,779),-- Life Stone: level 61
(22095,8738,1,1,200,195),-- Mid-Grade Life Stone: level 61
(22095,8748,1,1,200,19),-- High-Grade Life Stone: level 61
(22095,8758,1,1,200,4),-- Top-Grade Life Stone: level 61
(22096,57,780,1580,0,700000),-- Adena
(22096,1870,1,1,2,52815),-- Coal
(22096,1865,1,1,2,52815),-- Varnish
(22096,1879,1,1,2,8803),-- Cokes
(22096,1875,1,1,2,3521),-- Stone of Purity
(22096,4040,1,1,2,1056),-- Mold Lubricant
(22096,4159,1,1,2,150),-- Recipe: Blue Wolf Leather Armor
(22096,8306,1,1,2,40),-- Recipe: Wizard\'s Tear (60%)
(22096,4117,1,1,1,254),-- Art of Battle Axe Blade
(22096,4116,1,1,1,254),-- Deadman\'s Glory Stone
(22096,4118,1,1,1,240),-- Evil Spirit Head
(22096,8348,1,1,1,253),-- Star Buster Head
(22096,11935,1,1,1,19),-- Common Item - Deadman\'s Glory
(22096,11938,1,1,1,19),-- Common Item - Art of Battle Axe
(22096,11934,1,1,1,19),-- Common Item - Staff of Evil Spirits
(22096,11941,1,1,1,19),-- Common Item - Kaim Vanul's Bones
(22096,11936,1,1,1,19),-- Common Item - Star Buster
(22096,8340,1,1,1,254),-- Bones Head of Kaim Vanul
(22096,8728,1,1,200,880),-- Life Stone: level 61
(22096,8738,1,1,200,220),-- Mid-Grade Life Stone: level 61
(22096,8748,1,1,200,22),-- High-Grade Life Stone: level 61
(22096,8758,1,1,200,4),-- Top-Grade Life Stone: level 61
(22096,4052,1,1,-1,69634),-- Earring of Black Ore Piece
(22096,1875,1,1,-1,131122),-- Stone of Purity
(22096,4041,1,1,-1,17103),-- Mold Hardener
(22097,57,910,1844,0,700000),-- Adena
(22097,1343,10,30,2,74493),-- Silver Arrow
(22097,1870,1,1,2,52145),-- Coal
(22097,1865,1,1,2,52145),-- Varnish
(22097,1879,1,1,2,8691),-- Cokes
(22097,1875,1,1,2,3476),-- Stone of Purity
(22097,4087,1,1,1,1988),-- Stockings of Doom Pattern
(22097,4086,1,1,1,1988),-- Blue Wolf Stockings Pattern
(22097,4084,1,1,1,1473),-- Blue Wolf Tunic Fabric
(22097,4085,1,1,1,1473),-- Tunic of Doom Pattern
(22097,4040,1,1,2,1043),-- Mold Lubricant
(22097,3035,1,1,2,69),-- Recipe: Spiritshot A
(22097,8305,1,1,2,40),-- Recipe: Wizard\'s Tear (100%)
(22097,8326,1,1,2,19),-- Recipe: Doom Crusher (60%)
(22097,12201,1,1,1,150),-- Common Item - Blue Wolf Tunic
(22097,12192,1,1,1,150),-- Common Item - Tunic of Doom
(22097,12203,1,1,1,241),-- Common Item - Blue Wolf Stockings
(22097,12205,1,1,1,241),-- Common Item - Stockings of Doom
(22097,4054,1,1,-1,45202),-- Necklace of Black Ore Beads
(22097,4053,1,1,-1,98368),-- Ring of Black Ore Gemstone
(22097,1807,1,1,-1,3060),-- Recipe: Soulshot: A-Grade
(22097,8729,1,1,200,695),-- Life Stone: level 64
(22097,8739,1,1,200,174),-- Mid-Grade Life Stone: level 64
(22097,8748,1,1,200,17),-- High-Grade Life Stone: level 64
(22097,8758,1,1,200,3),-- Top-Grade Life Stone: level 64
(22098,57,942,1908,0,700000),-- Adena
(22098,1870,1,1,2,58479),-- Coal
(22098,1865,1,1,2,58479),-- Varnish
(22098,1879,1,1,2,9746),-- Cokes
(22098,1875,1,1,2,3899),-- Stone of Purity
(22098,4040,1,1,2,1170),-- Mold Lubricant
(22098,3035,1,1,2,78),-- Recipe: Spiritshot A
(22098,8305,1,1,2,45),-- Recipe: Wizard\'s Tear (100%)
(22098,8326,1,1,2,22),-- Recipe: Doom Crusher (60%)
(22098,8336,1,1,1,512),-- Blade of the Wizard\'s Tear
(22098,4114,1,1,1,512),-- Sword of Damascus Blade
(22098,8330,1,1,1,511),-- Guardian Sword Blade 
(22098,11930,1,1,1,38),-- Common Item - Sword of Damascus
(22098,11943,1,1,1,38),-- Common Item - Wizard\'s Tear
(22098,11929,1,1,1,38),-- Common Item - Guardian Sword
(22098,8729,1,1,200,780),-- Life Stone: level 64
(22098,8739,1,1,200,195),-- Mid-Grade Life Stone: level 64
(22098,8748,1,1,200,20),-- High-Grade Life Stone: level 64
(22098,8758,1,1,200,4),-- Top-Grade Life Stone: level 64
(22098,3035,1,1,-1,3168),-- Recipe: Spiritshot A
(22098,4077,1,1,-1,106869),-- Doom Shield Fragment
(22098,4039,1,1,-1,79190),-- Mold Glue
(22105,57,4141,8456,0,700000),-- Adena
(22105,1867,1,3,2,137819),-- Animal Skin
(22105,1881,1,1,2,41346),-- Coarse Bone Powder
(22105,1872,1,3,2,206729),-- Animal Bone
(22105,1875,1,1,2,20673),-- Stone of Purity
(22105,4041,1,1,2,2697),-- Mold Hardener
(22105,5543,1,1,1,1320),-- Dasparion\'s Staff Edge
(22105,8346,1,1,1,1307),-- Hammer Piece of Destroyer 
(22105,5532,1,1,1,1307),-- Meteor Shower Head
(22105,8341,1,1,1,1306),-- Spiritual Eye Piece
(22105,212,1,1,1,5),-- Dasparion\'s Staff
(22105,2504,1,1,1,5),-- Meteor Shower
(22105,7894,1,1,1,5),-- Spiritual Eye
(22105,7899,1,1,1,5),-- Destroyer Hammer
(22105,3957,1,1,1,138),-- Recipe: Blessed Spiritshot S
(22105,8731,1,1,200,2871),-- Life Stone: level 70
(22105,8741,1,1,200,718),-- Mid-Grade Life Stone: level 70
(22105,8751,1,1,200,72),-- High-Grade Life Stone: level 70
(22105,8761,1,1,200,14),-- Top-Grade Life Stone: level 70
(22105,1875,1,1,-1,559892),-- Stone of Purity
(22105,5478,1,1,-1,70622),-- Sealed Dark Crystal Leather Armor Pattern
(22105,5434,1,1,-1,3111),-- Recipe: Dragon Slayer(60%)
(22106,57,4276,8707,0,700000),-- Adena
(22106,1867,1,3,2,129188),-- Animal Skin
(22106,1344,40,120,2,90836),-- Mithril Arrow
(22106,1872,1,3,2,193782),-- Animal Bone
(22106,1881,1,1,2,38756),-- Coarse Bone Powder
(22106,1875,1,1,2,19378),-- Stone of Purity
(22106,4041,1,1,2,2527),-- Mold Hardener
(22106,5543,1,1,1,1361),-- Dasparion\'s Staff Edge
(22106,8341,1,1,1,1346),-- Spiritual Eye Piece
(22106,8346,1,1,1,1348),-- Hammer Piece of Destroyer 
(22106,5532,1,1,1,1348),-- Meteor Shower Head
(22106,212,1,1,1,6),-- Dasparion\'s Staff
(22106,2504,1,1,1,6),-- Meteor Shower
(22106,7894,1,1,1,6),-- Spiritual Eye
(22106,7899,1,1,1,6),-- Destroyer Hammer
(22106,3957,1,1,1,129),-- Recipe: Blessed Spiritshot S
(22106,8731,1,1,200,2691),-- Life Stone: level 70
(22106,8741,1,1,200,673),-- Mid-Grade Life Stone: level 70
(22106,8751,1,1,200,67),-- High-Grade Life Stone: level 70
(22106,8761,1,1,200,13),-- Top-Grade Life Stone: level 70
(22106,5479,1,1,-1,51240),-- Sealed Tallum Leather Armor Pattern
(22106,5440,1,1,-1,3207),-- Recipe: Elysian(60%)
(22106,5482,1,1,-1,114269),-- Sealed Dark Crystalline Leggings Design
(22107,57,4401,8971,0,700000),-- Adena
(22107,1894,1,1,2,12839),-- Crafted Leather
(22107,4044,1,1,2,12197),-- Thons
(22107,4039,1,1,2,12197),-- Mold Glue
(22107,4041,1,1,2,3182),-- Mold Hardener
(22107,5542,1,1,1,5553),-- Halberd Edge
(22107,98,1,1,1,23),-- Halberd
(22107,3957,1,1,1,163),-- Recipe: Blessed Spiritshot S
(22107,8731,1,1,200,3388),-- Life Stone: level 70
(22107,8741,1,1,200,847),-- Mid-Grade Life Stone: level 70
(22107,8751,1,1,200,85),-- High-Grade Life Stone: level 70
(22107,8761,1,1,200,17),-- Top-Grade Life Stone: level 70
(22107,729,1,1,-1,991),-- Scroll: Enchant Weapon(Grade A)
(22107,4039,1,1,-1,297302),-- Mold Glue
(22107,4041,1,1,-1,77557),-- Mold Hardener
(22108,57,6139,12513,0,700000),-- Adena
(22108,1869,2,6,2,164038),-- Iron Ore
(22108,1864,5,15,2,131231),-- Stem
(22108,1880,1,1,2,65615),-- Steel
(22108,1344,180,540,2,45567),-- Mithril Arrow
(22108,4043,1,1,2,21872),-- Asofe
(22108,5550,1,1,2,8749),-- Durable Metal Plate
(22108,10302,1,1,2,2625),-- Transform Sealbook - Heretic
(22108,5482,1,1,1,10103),-- Sealed Dark Crystalline Leggings Design
(22108,5479,1,1,1,4529),-- Sealed Tallum Leather Armor Pattern
(22108,5478,1,1,1,6437),-- Sealed Dark Crystal Leather Armor Pattern
(22108,5297,1,1,1,57),-- Sealed Dark Crystal Leather Armor
(22108,5298,1,1,1,92),-- Sealed Dark Crystalline Leggings
(22108,5301,1,1,1,35),-- Sealed Tallum Leather Armor
(22108,5485,1,1,-1,104604),-- Sealed Tallum Tunic Texture
(22108,5442,1,1,-1,4607),-- Recipe: Soul Bow(60%)
(22108,5486,1,1,-1,73607),-- Sealed Dark Crystal Robe Fabric
(22109,57,6057,12346,0,700000),-- Adena
(22109,1864,21,63,2,35968),-- Stem
(22109,1869,2,6,2,188831),-- Iron Ore
(22109,1884,2,6,2,92693),-- Cord
(22109,1878,3,9,2,50355),-- Braided Hemp
(22109,4039,1,1,2,25177),-- Mold Glue
(22109,10302,1,1,2,3021),-- Transform Sealbook - Heretic
(22109,5524,1,1,1,7445),-- Sealed Dark Crystal Gaiters Pattern
(22109,5521,1,1,1,3437),-- Sealed Tallum Plate Armor Pattern
(22109,5520,1,1,1,4871),-- Sealed Dark Crystal Breastplate Pattern
(22109,5287,1,1,1,42),-- Sealed Dark Crystal breastplate
(22109,5288,1,1,1,38),-- Sealed Dark Crystal Gaiters
(22109,5293,1,1,1,26),-- Sealed Tallum Plate Armor
(22109,5448,1,1,-1,4546),-- Recipe: Soul Separator(60%)
(22109,5489,1,1,-1,162002),-- Sealed Tallum Stockings Fabric
(22109,730,1,1,-1,10228),-- Scroll: Enchant Armor(Grade A)
(22110,57,4588,9352,0,700000),-- Adena
(22110,1871,1,3,2,163462),-- Charcoal
(22110,1866,2,6,2,81731),-- Suede
(22110,1344,100,300,2,61299),-- Mithril Arrow
(22110,1885,1,1,2,40866),-- High Grade Suede
(22110,5220,1,1,2,19615),-- Metal Hardener
(22110,4041,1,1,2,4264),-- Mold Hardener
(22110,5524,1,1,1,5639),-- Sealed Dark Crystal Gaiters Pattern
(22110,5521,1,1,1,2604),-- Sealed Tallum Plate Armor Pattern
(22110,5520,1,1,1,3690),-- Sealed Dark Crystal Breastplate Pattern
(22110,10302,1,1,2,1962),-- Transform Sealbook - Heretic
(22110,5287,1,1,1,32),-- Sealed Dark Crystal breastplate
(22110,5288,1,1,1,51),-- Sealed Dark Crystal Gaiters
(22110,5293,1,1,1,20),-- Sealed Tallum Plate Armor
(22110,1885,1,1,-1,774742),-- High Grade Suede
(22110,5494,1,1,-1,195252),-- Sealed Dark Crystal Shield Fragment
(22110,4041,1,1,-1,80843),-- Mold Hardener
(22111,57,4417,9013,0,700000),-- Adena
(22111,1870,1,3,2,165327),-- Coal
(22111,1865,1,3,2,110218),-- Varnish
(22111,1874,1,1,2,22043),-- Oriharukon Ore
(22111,4039,1,1,2,11022),-- Mold Glue
(22111,4043,1,1,2,11022),-- Asofe
(22111,5272,1,1,1,147),-- Recipe: Greater Soulshot(S) Compressed Package(100%)
(22111,5543,1,1,1,1407),-- Dasparion\'s Staff Edge
(22111,5532,1,1,1,1394),-- Meteor Shower Head
(22111,212,1,1,1,6),-- Dasparion\'s Staff
(22111,2504,1,1,1,6),-- Meteor Shower
(22111,7894,1,1,1,6),-- Spiritual Eye
(22111,7899,1,1,1,6),-- Destroyer Hammer
(22111,8346,1,1,1,1394),-- Hammer Piece of Destroyer
(22111,8341,1,1,1,1392),-- Spiritual Eye Piece
(22111,8731,1,1,200,3062),-- Life Stone: level 70
(22111,8741,1,1,200,766),-- Mid-Grade Life Stone: level 70
(22111,8751,1,1,200,76),-- High-Grade Life Stone: level 70
(22111,8761,1,1,200,15),-- Top-Grade Life Stone: level 70
(22111,1808,1,1,-1,3980),-- Recipe: Soulshot: S Grade
(22111,3036,1,1,-1,3980),-- Recipe: Spiritshot S
(22111,5480,1,1,-1,40184),-- Sealed Leather Armor of Nightmare Fabric
(22112,57,5608,11431,0,700000),-- Adena
(22112,1867,7,21,2,62161),-- Animal Skin
(22112,1872,4,12,2,116552),-- Animal Bone
(22112,1881,1,1,2,93242),-- Coarse Bone Powder
(22112,1875,1,1,2,46621),-- Stone of Purity
(22112,5494,1,1,1,14688),-- Sealed Dark Crystal Shield Fragment
(22112,5526,1,1,1,10811),-- Sealed Tallum Helm Design
(22112,5525,1,1,1,10811),-- Sealed Dark Crystal Helmet Design
(22112,4041,1,1,2,6081),-- Mold Hardener
(22112,10302,1,1,2,2797),-- Transform Sealbook - Heretic
(22112,5289,1,1,1,105),-- Sealed Dark Crystal Helmet
(22112,5294,1,1,1,105),-- Sealed Tallum Helmet
(22112,5292,1,1,1,150),-- Sealed Dark Crystal Shield
(22112,5478,1,1,-1,95559),-- Sealed Dark Crystal Leather Armor Pattern
(22112,729,1,1,-1,1263),-- Scroll: Enchant Weapon(Grade A)
(22112,4041,1,1,-1,98816),-- Mold Hardener
(22113,57,7035,14354,0,700000),-- Adena
(22113,1344,40,120,2,149594),-- Mithril Arrow
(22113,1881,1,1,2,63826),-- Coarse Bone Powder
(22113,1872,5,15,2,63826),-- Animal Bone
(22113,1867,9,27,2,35459),-- Animal Skin
(22113,1875,1,1,2,31913),-- Stone of Purity
(22113,4041,1,1,2,4163),-- Mold Hardener
(22113,5547,1,1,1,2985),-- Elemental Sword Edge
(22113,8331,1,1,1,2994),-- Infernal Master Blade
(22113,5548,1,1,1,2985),-- Tallum Blade Edge
(22113,5277,1,1,2,213),-- Recipe: Greater Spiritshot(S) Compressed Package(100%)
(22113,80,1,1,1,12),-- Tallum Blade
(22113,150,1,1,1,12),-- Elemental Sword
(22113,7884,1,1,1,12),-- Infernal Master
(22113,8731,1,1,200,4432),-- Life Stone: level 70
(22113,8741,1,1,200,1108),-- Mid-Grade Life Stone: level 70
(22113,8751,1,1,200,111),-- High-Grade Life Stone: level 70
(22113,8761,1,1,200,22),-- Top-Grade Life Stone: level 70
(22113,1875,1,3,-1,475374),-- Stone of Purity
(22113,5522,1,1,-1,63993),-- Sealed Armor of Nightmare Pattern
(22113,3957,1,1,-1,6338),-- Recipe: Blessed Spiritshot S
(22114,57,4723,9638,0,700000),-- Adena
(22114,1868,3,9,2,117853),-- Thread
(22114,1895,1,1,2,101017),-- Metallic Fiber
(22114,1873,1,1,2,141423),-- Silver Nugget
(22114,1876,1,1,2,70712),-- Mithril Ore
(22114,4042,1,1,2,5893),-- Enria
(22114,5547,1,1,1,2005),-- Elemental Sword Edge
(22114,8331,1,1,1,2011),-- Infernal Master Blade
(22114,5548,1,1,1,2005),-- Tallum Blade Edge
(22114,80,1,1,1,8),-- Tallum Blade
(22114,150,1,1,1,8),-- Elemental Sword
(22114,7884,1,1,1,8),-- Infernal Master
(22114,1808,1,1,2,157),-- Recipe: Soulshot: S Grade
(22114,8731,1,1,200,3274),-- Life Stone: level 70
(22114,8741,1,1,200,819),-- Mid-Grade Life Stone: level 70
(22114,8751,1,1,200,82),-- High-Grade Life Stone: level 70
(22114,8761,1,1,200,16),-- Top-Grade Life Stone: level 70
(22114,5157,1,1,-1,4256),-- Recipe: Soulshot(S) Compressed Package(100%)
(22114,5487,1,1,-1,43119),-- Sealed Robe of Nightmare Fabric
(22114,5481,1,1,-1,42968),-- Sealed Majestic Leather Armor Fabric
(22115,57,3877,7938,0,700000),-- Adena
(22115,1867,1,3,2,129306),-- Animal Skin
(22115,1344,40,120,2,90918),-- Mithril Arrow
(22115,1881,1,1,2,38792),-- Coarse Bone Powder
(22115,1872,1,3,2,193960),-- Animal Bone
(22115,1875,1,1,2,19396),-- Stone of Purity
(22115,4041,1,1,2,2530),-- Mold Hardener
(22115,8331,1,1,1,1654),-- Infernal Master Blade
(22115,5547,1,1,1,1650),-- Elemental Sword Edge
(22115,5548,1,1,1,1650),-- Tallum Blade Edge
(22115,3036,1,1,2,129),-- Recipe: Spiritshot S
(22115,150,1,1,1,7),-- Elemental Sword
(22115,7884,1,1,1,7),-- Infernal Master
(22115,80,1,1,1,7),-- Tallum Blade
(22115,8732,1,1,200,2425),-- Life Stone: level 76
(22115,8742,1,1,200,606),-- Mid-Grade Life Stone: level 76
(22115,8752,1,1,200,61),-- High-Grade Life Stone: level 76
(22115,4041,1,1,-1,68518),-- Mold Hardener
(22115,1875,1,1,-1,525307),-- Stone of Purity
(22115,5488,1,1,-1,35483),-- Sealed Majestic Robe Fabric
(22116,57,3997,8166,0,700000),-- Adena
(22116,1871,1,3,2,166337),-- Charcoal
(22116,1866,1,1,2,221783),-- Suede
(22116,1885,1,1,2,27723),-- High Grade Suede
(22116,5220,1,1,2,13307),-- Metal Hardener
(22116,5539,1,1,1,5049),-- Blood Tornado Edge
(22116,4041,1,1,2,2893),-- Mold Hardener
(22116,269,1,1,1,21),-- Blood Tornado
(22116,8732,1,1,200,2772),-- Life Stone: level 76
(22116,8742,1,1,200,693),-- Mid-Grade Life Stone: level 76
(22116,8752,1,1,200,69),-- High-Grade Life Stone: level 76
(22116,3957,1,1,1,148),-- Recipe: Blessed Spiritshot S
(22116,5277,1,1,-1,3604),-- Recipe: Compressed Package of Spiritshots: S-grade
(22116,5495,1,2,-1,123133),-- Sealed Shield of Nightmare Fragment
(22116,5167,1,1,-1,3604),-- Recipe: Compressed Package of Blessed Spiritshots: S-grade
(22117,57,4113,8395,0,700000),-- Adena
(22117,1344,30,90,2,128304),-- Mithril Arrow
(22117,1869,2,6,2,76983),-- Iron Ore
(22117,1864,5,15,2,61583),-- Stem
(22117,1880,1,1,2,30793),-- Steel
(22117,4043,1,1,2,10264),-- Asofe
(22117,5550,1,1,2,4106),-- Durable Metal Plate
(22117,5542,1,1,1,5192),-- Halberd Edge
(22117,98,1,1,1,22),-- Halberd
(22117,5157,1,1,2,137),-- Recipe: Soulshot(S) Compressed Package(100%)
(22117,5502,1,1,-1,140851),-- Sealed Boots of Nightmare Lining
(22117,5550,1,1,-1,111197),-- Durable Metal Plate
(22117,4043,1,1,-1,277993),-- Asofe
(22117,8732,1,1,200,2566),-- Life Stone: level 76
(22117,8742,1,1,200,642),-- Mid-Grade Life Stone: level 76
(22117,8752,1,1,200,64),-- High-Grade Life Stone: level 76
(22118,57,5754,11742,0,700000),-- Adena
(22118,1882,1,1,2,106387),-- Leather
(22118,1867,4,12,2,79791),-- Animal Skin
(22118,1872,1,3,2,212775),-- Animal Bone
(22118,6346,1,1,1,23377),-- Sealed Majestic Ring Gemstone
(22118,6344,1,1,1,10526),-- Sealed Majestic Necklace Beads
(22118,1894,1,1,2,16798),-- Crafted Leather
(22118,4042,1,1,2,7979),-- Enria
(22118,6342,1,1,1,16265),-- Sealed Majestic Earring Gemstone
(22118,6326,1,1,2,72),-- Sealed Majestic Necklace
(22118,6327,1,1,2,96),-- Sealed Majestic Earring
(22118,6328,1,1,2,144),-- Sealed Majestic Ring
(22118,959,1,1,1,19),-- Scroll: Enchant Weapon(Grade S)
(22118,5503,1,1,-1,197084),-- Sealed Majestic Boots Lining
(22118,5272,1,1,-1,5186),-- Recipe: Greater Soulshot(S) Compressed Package(100%)
(22118,5514,1,1,-1,197084),-- Sealed Gloves of Nightmare Design
(22118,8732,1,1,200,3989),-- Life Stone: level 76
(22118,8742,1,1,200,997),-- Mid-Grade Life Stone: level 76
(22118,8752,1,1,200,100),-- High-Grade Life Stone: level 76
(22119,57,4258,8689,0,700000),-- Adena
(22119,1868,4,12,2,79711),-- Thread
(22119,1344,150,450,2,26571),-- Mithril Arrow
(22119,1895,1,1,2,91098),-- Metallic Fiber
(22119,1876,1,1,2,63769),-- Mithril Ore
(22119,4042,1,1,2,5314),-- Enria
(22119,6346,1,1,1,17299),-- Sealed Majestic Ring Gemstone
(22119,6342,1,1,1,12036),-- Sealed Majestic Earring Gemstone
(22119,6344,1,1,1,7789),-- Sealed Majestic Necklace Beads
(22119,6326,1,1,2,53),-- Sealed Majestic Necklace
(22119,6327,1,1,2,71),-- Sealed Majestic Earring
(22119,6328,1,1,2,106),-- Sealed Majestic Ring
(22119,960,1,1,2,128),-- Scroll: Enchant Armor (Grade S)
(22119,1873,1,1,2,127538),-- Silver Nugget
(22119,8732,1,1,200,2657),-- Life Stone: level 76
(22119,8742,1,1,200,664),-- Mid-Grade Life Stone: level 76
(22119,8752,1,1,200,66),-- High-Grade Life Stone: level 76
(22119,5515,1,1,-1,145843),-- Sealed Majestic Gloves Design
(22119,5277,1,1,-1,3838),-- Recipe: Greater Spiritshot(S) Compressed Package(100%)
(22119,4042,1,1,-1,143923),-- Enria
(22120,57,5463,11149,0,700000),-- Adena
(22120,1870,2,6,2,90912),-- Coal
(22120,1871,3,9,2,75760),-- Charcoal
(22120,1872,7,21,2,43291),-- Animal Bone
(22120,1867,12,36,2,25253),-- Animal Skin
(22120,1869,2,6,2,113640),-- Iron Ore
(22120,5514,1,1,1,8367),-- Sealed Gloves of Nightmare Design
(22120,5502,1,1,1,8367),-- Sealed Boots of Nightmare Lining
(22120,5515,1,1,1,8367),-- Sealed Majestic Gloves Design
(22120,5503,1,1,1,8367),-- Sealed Majestic Boots Lining
(22120,5314,1,1,1,75),-- Sealed Boots of Nightmare
(22120,5318,1,1,1,75),-- Sealed Majestic Gloves
(22120,5313,1,1,1,75),-- Sealed Gloves of Nightmare
(22120,5319,1,1,1,75),-- Sealed Majestic Boots
(22120,6901,1,1,2,455),-- Recipe: Shining Arrow(100%)
(22120,5522,1,1,-1,56599),-- Sealed Armor of Nightmare Pattern
(22120,5282,1,1,-1,7387),-- Recipe: Greater Blessed Spiritshot(S) Compressed Package(100%)
(22120,8732,1,1,200,3788),-- Life Stone: level 76
(22120,8742,1,1,200,947),-- Mid-Grade Life Stone: level 76
(22120,8752,1,1,200,95),-- High-Grade Life Stone: level 76
(22121,57,4237,8649,0,700000),-- Adena
(22121,1878,1,1,2,181329),-- Braided Hemp
(22121,1864,4,12,2,113331),-- Stem
(22121,1869,2,6,2,90664),-- Iron Ore
(22121,1884,3,9,2,46495),-- Cord
(22121,1344,210,630,2,26984),-- Mithril Arrow
(22121,4039,1,1,2,15111),-- Mold Glue
(22121,5487,1,1,1,3572),-- Sealed Robe of Nightmare Fabric
(22121,5488,1,1,1,3572),-- Sealed Majestic Robe Fabric
(22121,5326,1,1,1,24),-- Sealed Robe of Nightmare
(22121,5329,1,1,1,24),-- Sealed Majestic Robe
(22121,959,1,1,2,18),-- Scroll: Enchant Weapon(Grade S)
(22121,5523,1,1,-1,29268),-- Sealed Majestic Plate Armor Pattern
(22121,1808,1,1,-1,3820),-- Recipe: Soulshot: S Grade
(22121,5527,1,1,-1,102753),-- Sealed Helm of Nightmare Design
(22122,57,5688,11612,0,700000),-- Adena
(22122,1870,7,21,2,47337),-- Coal
(22122,1879,1,1,2,118341),-- Cokes
(22122,1865,4,12,2,88756),-- Varnish
(22122,1875,1,1,2,47337),-- Stone of Purity
(22122,4040,1,1,2,14201),-- Mold Lubricant
(22122,5514,1,1,1,8994),-- Sealed Gloves of Nightmare Design
(22122,5502,1,1,1,8994),-- Sealed Boots of Nightmare Lining
(22122,5515,1,1,1,8994),-- Sealed Majestic Gloves Design
(22122,5503,1,1,1,8994),-- Sealed Majestic Boots Lining
(22122,960,1,1,2,284),-- Scroll: Enchant Armor (Grade S)
(22122,5314,1,1,1,79),-- Sealed Boots of Nightmare
(22122,5319,1,1,1,79),-- Sealed Majestic Boots
(22122,5313,1,1,1,79),-- Sealed Gloves of Nightmare
(22122,5318,1,1,1,79),-- Sealed Majestic Gloves
(22122,4040,1,1,-1,230766),-- Mold Lubricant
(22122,6346,1,1,-1,375658),-- Sealed Majestic Ring Gemstone
(22122,6901,1,1,-1,11538),-- Recipe: Shining Arrow (100%)
(22123,57,5741,11756,0,700000),-- Adena
(22123,1895,1,1,2,153871),-- Metallic Fiber
(22123,4043,1,1,2,17592),-- Asofe
(22123,4040,1,1,2,10771),-- Mold Lubricant
(22123,4042,1,1,2,8976),-- Enria
(22123,5495,1,1,1,10904),-- Sealed Shield of Nightmare Fragment
(22123,5527,1,1,1,8585),-- Sealed Helm of Nightmare Design
(22123,5528,1,1,1,8585),-- Sealed Majestic Circlet Design
(22123,6901,1,1,2,538),-- Recipe: Shining Arrow (100%)
(22123,5312,1,1,1,71),-- Sealed Helm of Nightmare
(22123,5315,1,1,1,101),-- Sealed Shield of Nightmare
(22123,5317,1,1,1,71),-- Sealed Majestic Circlet
(22123,6342,1,1,-1,264323),-- Sealed Majestic Earring Gemstone
(22123,5282,1,1,-1,5186),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(22123,6344,1,1,-1,171056),-- Sealed Majestic Necklace Beads
(22123,8732,1,1,200,4488),-- Life Stone: level 76
(22123,8742,1,1,200,1122),-- Mid-Grade Life Stone: level 76
(22123,8752,1,1,200,112),-- High-Grade Life Stone: level 76
(22124,57,4553,9324,0,700000),-- Adena
(22124,1868,3,9,2,126558),-- Thread
(22124,1895,1,1,2,108478),-- Metallic Fiber
(22124,1876,1,1,2,75938),-- Mithril Ore
(22124,1873,1,1,2,151870),-- Silver Nugget
(22124,6346,1,1,1,18540),-- Sealed Majestic Ring Gemstone
(22124,6342,1,1,1,12899),-- Sealed Majestic Earring Gemstone
(22124,4042,1,1,2,6328),-- Enria
(22124,6344,1,1,1,8348),-- Sealed Majestic Necklace Beads
(22124,959,1,1,2,15),-- Scroll: Enchant Weapon (Grade S)
(22124,6327,1,1,1,76),-- Sealed Majestic Earring
(22124,6326,1,1,1,57),-- Sealed Majestic Necklace
(22124,6328,1,1,1,114),-- Sealed Majestic Ring
(22124,4040,1,1,-1,185092),-- Mold Lubricant
(22124,5480,1,1,-1,41527),-- Sealed Leather Armor of Nightmare Fabric
(22124,1808,1,1,-1,4113),-- Recipe: Soulshot: S Grade
(22124,8732,1,1,200,3164),-- Life Stone: level 76
(22124,8742,1,1,200,791),-- Mid-Grade Life Stone: level 76
(22124,8752,1,1,200,79),-- High-Grade Life Stone: level 76
(22125,57,4561,9343,0,700000),-- Adena
(22125,1895,1,1,2,195619),-- Metallic Fiber
(22125,4043,1,1,2,22822),-- Asofe
(22125,4040,1,1,2,13693),-- Mold Lubricant
(22125,4042,1,1,2,11411),-- Enria
(22125,5541,1,1,1,4693),-- Tallum Glaive Edge
(22125,960,1,1,2,274),-- Scroll: Enchant Armor (Grade S)
(22125,305,1,1,1,17),-- Tallum Glaive
(22125,5481,1,1,-1,41603),-- Sealed Majestic Leather Armor Fabric
(22125,5487,1,1,-1,41751),-- Sealed Robe of Nightmare Fabric
(22125,3036,1,1,-1,4121),-- Recipe: Spiritshot S
(22126,57,4672,9570,0,700000),-- Adena
(22126,1873,1,1,2,155853),-- Silver Nugget
(22126,1868,3,9,2,129877),-- Thread
(22126,5549,1,1,2,38963),-- Metallic Thread
(22126,1889,1,1,2,25976),-- Compound Braid
(22126,4044,1,1,2,12988),-- Thons
(22126,5523,1,1,1,2986),-- Sealed Majestic Plate Armor Pattern
(22126,5522,1,1,1,2986),-- Sealed Armor of Nightmare Pattern
(22126,6901,1,1,2,390),-- Recipe: Shining Arrow (100%)
(22126,5311,1,1,1,20),-- Sealed Armor of Nightmare
(22126,5316,1,1,1,20),-- Sealed Majestic Plate Armor
(22126,4043,1,1,-1,316576),-- Asofe
(22126,5488,1,1,-1,42767),-- Sealed Majestic Robe Fabric
(22126,5550,1,1,-1,126630),-- Durable Metal Plate
(22126,8732,1,1,200,3247),-- Life Stone: level 76
(22126,8742,1,1,200,812),-- Mid-Grade Life Stone: level 76
(22126,8752,1,1,200,81),-- High-Grade Life Stone: level 76
(22127,57,4689,9571,0,700000),-- Adena
(22127,164,1,1,1,4),-- Elysian
(22127,213,1,1,1,4),-- Branch of The Mother Tree
(22127,7895,1,1,1,4),-- Flaming Dragon Skull
(22127,7902,1,1,1,4),-- Doom Crusher
(22127,1867,1,3,2,260060),-- Animal Skin
(22127,1881,1,1,2,78018),-- Coarse Bone Powder
(22127,1872,4,12,2,97522),-- Animal Bone
(22127,1875,1,1,2,39009),-- Stone of Purity
(22127,4041,1,1,2,5088),-- Mold Hardener
(22127,8342,1,1,1,1204),-- Flaming Dragon Skull Piece
(22127,8349,1,1,1,1204),-- Doom Crusher Head
(22127,5533,1,1,1,1204),-- Elysian Head
(22127,5544,1,1,1,1203),-- Branch of The Mother Tree Head
(22127,959,1,1,2,24),-- Scroll: Enchant Weapon(Grade S)
(22127,4041,1,1,-1,82682),-- Mold Hardener
(22127,6688,1,1,-1,20552),-- Forgotten Blade Edge
(22127,6901,1,1,-1,9508),-- Recipe: Shining Arrow (100%)
(22128,57,1380,2816,0,700000),-- Adena
(22128,1864,1,3,2,172163),-- Stem
(22128,1869,1,1,2,172163),-- Iron Ore
(22128,1880,1,1,2,17216),-- Steel
(22128,4043,1,1,2,5739),-- Asofe
(22128,5550,1,1,2,2296),-- Durable Metal Plate
(22128,5534,1,1,1,1418),-- Soul Bow Stave
(22128,960,1,1,2,69),-- Scroll: Enchant Armor (Grade S)
(22128,289,1,1,1,5),-- Soul Bow
(22128,1875,1,1,-1,186510),-- Stone of Purity
(22128,4041,1,1,-1,24327),-- Mold Hardener
(22128,6695,1,1,-1,5933),-- Demon Splinter Blade
(22129,57,4923,10046,0,700000),-- Adena
(22129,1895,1,1,2,210655),-- Metallic Fiber
(22129,8056,1,1,3,32258),-- Key of Splendor Room
(22129,4043,1,1,2,34576),-- Asofe
(22129,4040,1,1,2,14746),-- Mold Lubricant
(22129,4042,1,1,2,12288),-- Enria
(22129,5480,1,1,1,4136),-- Sealed Leather Armor of Nightmare Fabric
(22129,5481,1,1,1,4136),-- Sealed Majestic Leather Armor Fabric
(22129,6901,1,1,2,737),-- Recipe: Shining Arrow (100%)
(22129,5320,1,1,1,28),-- Sealed Leather Armor of Nightmare
(22129,5323,1,1,1,28),-- Sealed Majestic Leather Armor
(22129,6698,1,1,-1,196115),-- Sealed Tateossian Earring  Part
(22129,6699,1,1,-1,217615),-- Sealed Tateossian Ring Gem
(22129,6700,1,1,-1,119528),-- Sealed Tateossian Necklace Chain
(22130,57,7234,14769,0,700000),-- Adena
(22130,5549,1,1,2,60205),-- Metallic Thread
(22130,1873,2,6,2,60205),-- Silver Nugget
(22130,1889,1,1,2,40137),-- Compound Braid
(22130,1868,3,9,2,200683),-- Thread
(22130,8056,1,1,3,32258),-- Key of Splendor Room
(22130,4044,1,1,2,20068),-- Thons
(22130,5537,1,1,1,7434),-- Soul Separator Head
(22130,236,1,1,1,26),-- Soul Separator
(22130,959,1,1,2,24),-- Scroll: Enchant Weapon (Grade S)
(22130,1894,1,1,-1,514911),-- Crafted Leather
(22130,4039,1,1,-1,489165),-- Mold Glue
(22130,959,1,1,-1,587),-- Scroll: Enchant Weapon (Grade S)
(22130,8732,1,1,200,5017),-- Life Stone: level 76
(22130,8742,1,1,200,1254),-- Mid-Grade Life Stone: level 76
(22130,8752,1,1,200,125),-- High-Grade Life Stone: level 76
(22131,57,7789,15902,0,700000),-- Adena
(22131,1894,1,1,2,40942),-- Crafted Leather
(22131,8056,1,1,3,31250),-- Key of Splendor Room
(22131,4039,1,1,2,38895),-- Mold Glue
(22131,4044,1,1,2,38895),-- Thons
(22131,4041,1,1,2,10146),-- Mold Hardener
(22131,5488,1,1,1,6567),-- Sealed Majestic Robe Fabric
(22131,5487,1,1,1,6567),-- Sealed Robe of Nightmare Fabric
(22131,960,1,1,2,467),-- Scroll: Enchant Armor (Grade S)
(22131,5326,1,1,1,44),-- Sealed Robe of Nightmare
(22131,5329,1,1,1,44),-- Sealed Majestic Robe
(22131,6706,1,1,-1,256322),-- Sealed Imperial Crusader Helmet Pattern
(22131,6705,1,1,-1,220361),-- Sealed Imperial Crusader Shield Part
(22131,6704,1,1,-1,224973),-- Sealed Imperial Crusader Boots Design
(22132,57,5056,10350,0,700000),-- Adena
(22132,1867,1,3,2,187342),-- Animal Skin
(22132,1872,4,12,2,70253),-- Animal Bone
(22132,1881,1,1,2,56203),-- Coarse Bone Powder
(22132,1875,1,1,2,28101),-- Stone of Purity
(22132,8056,1,1,2,33333),-- Key of Splendor Room
(22132,4041,1,1,2,3666),-- Mold Hardener
(22132,5546,1,1,1,1905),-- Sword of Miracles Edge
(22132,5529,1,1,1,1734),-- Dragon Slayer Edge
(22132,5545,1,1,1,1734),-- Dark Legion\'s Edge Blade
(22132,6901,1,1,2,422),-- Recipe: Shining Arrow (100%)
(22132,81,1,1,1,6),-- Dragon Slayer
(22132,151,1,1,1,6),-- Sword of Miracles 
(22132,2500,1,1,1,6),-- Dark Legion\'s Edge
(22132,6702,1,1,-1,70417),-- Sealed Imperial Crusader Gaiters Pattern
(22132,6701,1,1,-1,54702),-- Sealed Imperial Crusader Breastplate Part
(22132,13099,1,1,-1,143288),-- Sealed Arcana Sickle Piece
(22132,8732,1,1,200,3513),-- Life Stone: level 76
(22132,8742,1,1,200,878),-- Mid-Grade Life Stone: level 76
(22132,8752,1,1,200,88),-- High-Grade Life Stone: level 76
(22133,57,1966,4024,0,700000),-- Adena
(22133,1867,1,3,2,109244),-- Animal Skin
(22133,1872,1,3,2,163866),-- Animal Bone
(22133,1881,1,1,2,32773),-- Coarse Bone Powder
(22133,8056,1,1,3,12821),-- Key of Splendor Room
(22133,1875,1,1,2,16387),-- Stone of Purity
(22133,5537,1,1,1,2023),-- Soul Separator Head
(22133,4041,1,1,2,2137),-- Mold Hardener
(22133,236,1,1,1,7),-- Soul Separator
(22133,959,1,1,2,10),-- Scroll: Enchant Weapon (Grade S)
(22133,6688,1,1,-1,8634),-- Forgotten Blade Edge
(22133,6689,1,1,-1,8471),-- Basalt Battlehammer Head
(22133,6690,1,1,-1,8650),-- Imperial Staff Head
(22134,57,5142,10540,0,700000),-- Adena
(22134,8056,1,1,3,33333),-- Key of Splendor Room
(22134,1894,1,1,2,27097),-- Crafted Leather
(22134,4044,1,1,2,25742),-- Thons
(22134,4039,1,1,2,25742),-- Mold Glue
(22134,4041,1,1,2,6715),-- Mold Hardener
(22134,5529,1,1,1,1765),-- Dragon Slayer Edge
(22134,5545,1,1,1,1765),-- Dark Legion\'s Edge Blade
(22134,5546,1,1,1,1939),-- Sword of Miracles Edge
(22134,960,1,1,2,309),-- Scroll: Enchant Armor (Grade S)
(22134,81,1,1,1,6),-- Dragon Slayer
(22134,151,1,1,1,6),-- Sword of Miracles 
(22134,2500,1,1,1,6),-- Dark Legion\'s Edge
(22134,6709,1,1,-1,148895),-- Sealed Draconic Leather Boots Design
(22134,6707,1,1,-1,51815),-- Sealed Draconic Leather Armor Part
(22134,6708,1,1,-1,148895),-- Sealed Draconic Leather Gloves Fabric
(22135,57,8288,16989,0,700000),-- Adena
(22135,1895,1,1,2,222272),-- Metallic Fiber
(22135,6346,1,1,1,33767),-- Sealed Majestic Ring Gemstone
(22135,4043,1,1,2,25932),-- Asofe
(22135,8056,1,1,3,32258),-- Key of Splendor Room
(22135,4040,1,1,2,15559),-- Mold Lubricant
(22135,6342,1,1,1,23494),-- Sealed Majestic Earring Gemstone
(22135,4042,1,1,2,12966),-- Enria
(22135,6344,1,1,1,15204),-- Sealed Majestic Necklace Beads
(22135,6901,1,1,2,778),-- Recipe: Shining Arrow (100%)
(22135,6326,1,1,1,104),-- Sealed Majestic Necklace
(22135,6327,1,1,1,138),-- Sealed Majestic Earring
(22135,6328,1,1,1,208),-- Sealed Majestic Ring
(22135,6694,1,1,-1,37093),-- Saint Spear Blade
(22135,6691,1,1,-1,36433),-- Angel Slayer Blade
(22135,6693,1,1,-1,37093),-- Dragon Hunter Axe Blade
(22135,8732,1,1,200,6483),-- Life Stone: level 76
(22135,8742,1,1,200,1621),-- Mid-Grade Life Stone: level 76
(22135,8752,1,1,200,162),-- High-Grade Life Stone: level 76
(22137,57,492,1008,0,700000),-- Adena
(22137,1344,10,30,2,65985),-- Mithril Arrow
(22137,1871,1,1,2,52788),-- Charcoal
(22137,1866,1,1,2,35192),-- Suede
(22137,1877,1,1,2,2112),-- Adamantite Nugget
(22137,5541,1,1,1,507),-- Tallum Glaive Edge
(22137,4040,1,1,2,1056),-- Mold Lubricant
(22137,4041,1,1,2,459),-- Mold Hardener
(22137,960,1,1,2,21),-- Scroll: Enchant Armor (Grade S)
(22137,11976,1,1,1,33),-- Common Item - Tallum Glaive
(22137,4040,1,1,-1,20015),-- Mold Lubricant
(22137,4041,1,1,-1,8702),-- Mold Hardener
(22137,5480,1,1,-1,4491),-- Sealed Leather Armor of Nightmare Fabric
(22138,57,545,1113,0,700000),-- Adena
(22138,1344,10,30,2,51042),-- Mithril Arrow
(22138,1873,1,1,2,16333),-- Silver Nugget
(22138,1868,1,1,2,81688),-- Thread
(22138,1895,1,1,2,11667),-- Metallic Fiber
(22138,1876,1,1,2,8167),-- Mithril Ore
(22138,4042,1,1,2,681),-- Enria
(22138,5523,1,1,1,348),-- Sealed Majestic Plate Armor Pattern
(22138,5522,1,1,1,348),-- Sealed Armor of Nightmare Pattern
(22138,6901,1,1,2,41),-- Recipe: Shining Arrow (100%)
(22138,12263,1,1,1,43),-- Common Item - Sealed Armor of Nightmare
(22138,12262,1,1,1,43),-- Common Item - Sealed Majestic Plate Armor
(22138,8732,1,1,200,340),-- Life Stone: level 76
(22138,8742,1,1,200,85),-- Mid-Grade Life Stone: level 76
(22138,8752,1,1,200,9),-- High-Grade Life Stone: level 76
(22138,6901,1,1,-1,1106),-- Recipe: Shining Arrow (100%)
(22138,6688,1,1,-1,2390),-- Forgotten Blade Edge
(22139,57,4525,9265,0,700000),-- Adena
(22139,1344,20,60,2,353700),-- Mithril Arrow
(22139,1894,1,1,2,19857),-- Crafted Leather
(22139,4039,1,1,2,18864),-- Mold Glue
(22139,4044,1,1,2,18864),-- Thons
(22139,5541,1,1,1,4656),-- Tallum Glaive Edge
(22139,305,1,1,1,16),-- Tallum Glaive
(22139,4041,1,1,2,4921),-- Mold Hardener
(22139,959,1,1,2,22),-- Scroll: Enchant Weapon (Grade S)
(22139,5515,1,1,-1,155316),-- Sealed Majestic Gloves Design
(22139,5522,1,1,-1,31318),-- Sealed Armor of Nightmare Pattern
(22139,3957,1,1,-1,4087),-- Recipe: Blessed Spiritshot S
(22140,8273,6,6,3,1000000),-- Anteroom Key
(22140,57,4415,9013,0,700000),-- Adena
(22140,1871,1,3,2,157453),-- Charcoal
(22140,1866,2,6,2,78727),-- Suede
(22140,1344,100,300,2,59045),-- Mithril Arrow
(22140,1877,1,1,2,18894),-- Adamantite Nugget
(22140,4040,1,1,2,9447),-- Mold Lubricant
(22140,5503,1,1,1,6980),-- Sealed Majestic Boots Lining
(22140,5502,1,1,1,6080),-- Sealed Boots of Nightmare Lining
(22140,5515,1,1,1,6980),-- Sealed Majestic Gloves Design
(22140,5514,1,1,1,6980),-- Sealed Gloves of Nightmare Design
(22140,4041,1,1,2,4107),-- Mold Hardener
(22140,960,1,1,2,189),-- Scroll: Enchant Armor (Grade S)
(22140,5313,1,1,1,61),-- Sealed Gloves of Nightmare
(22140,5314,1,1,1,61),-- Sealed Boots of Nightmare
(22140,5318,1,1,1,61),-- Sealed Majestic Gloves
(22140,5319,1,1,1,61),-- Sealed Majestic Boots
(22140,5495,1,1,-1,135982),-- Sealed Shield of Nightmare Fragment
(22140,5488,1,1,-1,40326),-- Sealed Majestic Robe Fabric
(22140,730,1,1,-1,7463),-- Scroll: Enchant Armor (Grade A)
(22141,57,4566,9322,0,700000),-- Adena
(22141,1344,100,300,2,42751),-- Mithril Arrow
(22141,1865,1,3,2,114002),-- Varnish
(22141,1870,2,6,2,85502),-- Coal
(22141,1879,1,1,2,57001),-- Cokes
(22141,1875,1,1,2,22800),-- Stone of Purity
(22141,5495,1,1,1,8655),-- Sealed Shield of Nightmare Fragment
(22141,5527,1,1,1,6815),-- Sealed Helm of Nightmare Design
(22141,4040,1,1,2,6840),-- Mold Lubricant
(22141,5528,1,1,1,6815),-- Sealed Majestic Circlet Design
(22141,6901,1,1,2,342),-- Recipe: Shining Arrow (100%)
(22141,8732,1,1,200,2850),-- Life Stone: level 76
(22141,8742,1,1,200,713),-- Mid-Grade Life Stone: level 76
(22141,8752,1,1,200,71),-- High-Grade Life Stone: level 76
(22141,5315,1,1,1,80),-- Sealed Shield of Nightmare
(22141,5312,1,1,1,56),-- Sealed Helm of Nightmare
(22141,5317,1,1,1,56),-- Sealed Majestic Circlet
(22141,5488,1,1,-1,41711),-- Sealed Majestic Robe Fabric
(22141,730,1,1,-1,7719),-- Scroll: Enchant Armor (Grade A)
(22141,5480,1,1,-1,41564),-- Sealed Leather Armor of Nightmare Fabric
(22142,8274,1,1,3,1000000),-- Chapel Key
(22142,57,4584,9387,0,700000),-- Adena
(22142,1866,1,3,2,127419),-- Suede
(22142,1871,1,3,2,286692),-- Charcoal
(22142,1885,1,1,2,47782),-- High Grade Suede
(22142,5220,1,1,2,22935),-- Metal Hardener
(22142,5495,1,1,1,8706),-- Sealed Shield of Nightmare Fragment
(22142,5527,1,1,1,6855),-- Sealed Helm of Nightmare Design
(22142,5528,1,1,1,6855),-- Sealed Majestic Circlet Design
(22142,4041,1,1,2,4986),-- Mold Hardener
(22142,5315,1,1,1,81),-- Sealed Shield of Nightmare
(22142,5317,1,1,1,56),-- Sealed Majestic Circlet
(22142,5312,1,1,1,56),-- Sealed Helm of Nightmare
(22142,959,1,1,2,23),-- Scroll: Enchant Weapon (Grade S)
(22142,5220,1,1,-1,372700),-- Metal Hardener
(22142,5495,1,1,-1,141485),-- Sealed Shield of Nightmare Fragment
(22142,3957,1,1,-1,4141),-- Recipe: Blessed Spiritshot S
(22143,8274,1,1,3,990000),-- Chapel Key
(22143,57,5331,10918,0,700000),-- Adena
(22143,1869,1,3,2,222260),-- Iron Ore
(22143,1864,4,12,2,166695),-- Stem
(22143,1880,1,1,2,66678),-- Steel
(22143,4043,1,1,2,22226),-- Asofe
(22143,5503,1,1,1,8446),-- Sealed Majestic Boots Lining
(22143,5514,1,1,1,8446),-- Sealed Gloves of Nightmare Design
(22143,5515,1,1,1,8446),-- Sealed Majestic Gloves Design
(22143,5502,1,1,1,8446),-- Sealed Boots of Nightmare Lining
(22143,5550,1,1,2,8890),-- Durable Metal Plate
(22143,960,1,1,1,267),-- Scroll: Enchant Armor (Grade S)
(22143,5318,1,1,1,74),-- Sealed Majestic Gloves
(22143,5314,1,1,1,74),-- Sealed Boots of Nightmare
(22143,5313,1,1,1,74),-- Sealed Gloves of Nightmare
(22143,5319,1,1,1,74),-- Sealed Majestic Boots
(22143,5502,1,1,-1,182996),-- Sealed Boots of Nightmare Lining
(22143,5503,1,1,-1,182996),-- Sealed Majestic Boots Lining
(22143,5157,1,1,-1,4816),-- Recipe: Soulshot(S) Compressed Package(100%)
(22144,57,4643,9510,0,700000),-- Adena
(22144,1344,40,120,2,108894),-- Mithril Arrow
(22144,1882,1,1,1,77436),-- Leather
(22144,1867,5,15,2,46461),-- Animal Skin
(22144,1872,1,3,2,154871),-- Animal Bone
(22144,1894,1,1,2,12227),-- Crafted Leather
(22144,4042,1,1,2,5808),-- Enria
(22144,5529,1,1,1,1593),-- Dragon Slayer Edge
(22144,5546,1,1,1,1750),-- Sword of Miracles Edge
(22144,5545,1,1,1,1593),-- Dark Legion's Edge Blade
(22144,6901,1,1,2,348),-- Recipe: Shining Arrow (100%)
(22144,8732,1,1,200,2904),-- Life Stone: level 76
(22144,8742,1,1,200,726),-- Mid-Grade Life Stone: level 76
(22144,8752,1,1,200,73),-- High-Grade Life Stone: level 76
(22144,81,1,1,1,6),-- Dragon Slayer
(22144,151,1,1,1,6),-- Sword of Miracles 
(22144,2500,1,1,1,6),-- Dark Legion\'s Edge
(22144,1894,1,1,-1,331138),-- Crafted Leather
(22144,4042,1,1,-1,157290),-- Enria
(22144,5514,1,1,-1,159389),-- Sealed Gloves of Nightmare Design
(22145,57,4457,9126,0,700000),-- Adena
(22145,1344,150,450,2,39816),-- Mithril Arrow
(22145,1873,1,1,2,191120),-- Silver Nugget
(22145,1868,4,12,2,119450),-- Thread
(22145,5549,1,1,2,47780),-- Metallic Thread
(22145,1889,1,1,2,31854),-- Compound Braid
(22145,6346,1,1,2,18146),-- Sealed Majestic Ring Gemstone
(22145,6342,1,1,1,12626),-- Sealed Majestic Earring Gemstone
(22145,4044,1,1,2,15927),-- Thons
(22145,6344,1,1,1,8171),-- Sealed Majestic Necklace Beads
(22145,6328,1,1,1,112),-- Sealed Majestic Ring
(22145,6327,1,1,1,74),-- Sealed Majestic Earring
(22145,6326,1,1,1,56),-- Sealed Majestic Necklace
(22145,959,1,1,2,19),-- Scroll: Enchant Weapon (Grade S)
(22145,5162,1,1,-1,4026),-- Recipe: Spiritshot (S) Compressed Package (100%)
(22145,5515,1,1,-1,152986),-- Sealed Majestic Gloves Design
(22145,5167,1,1,-1,4026),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(22146,57,4956,10114,0,700000),-- Adena
(22146,1864,4,12,2,154647),-- Stem
(22146,1865,2,6,2,123718),-- Varnish
(22146,8275,1,1,2,83333),-- Key of Darkness
(22146,1866,3,9,2,68732),-- Suede
(22146,1868,10,30,2,58913),-- Thread
(22146,1873,4,12,2,30929),-- Silver Nugget
(22146,5487,1,1,1,4178),-- Sealed Robe of Nightmare Fabric
(22146,5488,1,1,1,4178),-- Sealed Majestic Robe Fabric
(22146,960,1,1,1,248),-- Scroll: Enchant Armor (Grade S)
(22146,5326,1,1,1,28),-- Sealed Robe of Nightmare
(22146,5329,1,1,1,28),-- Sealed Majestic Robe
(22146,6688,1,1,-1,21727),-- Forgotten Blade Edge
(22146,960,1,1,-1,4021),-- Scroll: Enchant Armor (Grade S)
(22147,57,4937,10077,0,700000),-- Adena
(22147,1867,13,39,2,18254),-- Animal Skin
(22147,1344,40,120,2,115511),-- Mithril Arrow
(22147,1869,2,6,2,73927),-- Iron Ore
(22147,1870,3,9,2,61606),-- Coal
(22147,1871,3,9,2,52805),-- Charcoal
(22147,6342,1,1,1,13954),-- Sealed Majestic Earring Gemstone
(22147,1872,8,24,2,30803),-- Animal Bone
(22147,6344,1,1,1,9030),-- Sealed Majestic Necklace Beads
(22147,6346,1,1,1,20055),-- Sealed Majestic Ring Gemstone
(22147,6328,1,1,1,123),-- Sealed Majestic Ring
(22147,6327,1,1,1,82),-- Sealed Majestic Earring
(22147,6326,1,1,1,62),-- Sealed Majestic Necklace
(22147,6901,1,1,2,370),-- Recipe: Shining Arrow (100%)
(22147,6689,1,1,-1,21231),-- Basalt Battlehammer Head
(22147,6901,1,1,-1,10011),-- Recipe: Shining Arrow (100%)
(22147,8732,1,1,200,3080),-- Life Stone: level 76
(22147,8742,1,1,200,770),-- Mid-Grade Life Stone: level 76
(22147,8752,1,1,200,77),-- High-Grade Life Stone: level 76
(22148,57,3873,7932,0,700000),-- Adena
(22148,1867,12,36,2,26912),-- Animal Skin
(22148,1869,2,6,2,121104),-- Iron Ore
(22148,1870,2,6,2,96883),-- Coal
(22148,1871,3,9,2,80736),-- Charcoal
(22148,1872,7,21,2,46135),-- Animal Bone
(22148,5538,1,1,1,3986),-- Dragon Grinder Edge
(22148,959,1,1,2,19),-- Scroll: Enchant Weapon (Grade S)
(22148,270,1,1,1,14),-- Dragon Grinder
(22148,5481,1,1,-1,35322),-- Sealed Majestic Leather Armor Fabric
(22148,5480,1,1,-1,35322),-- Sealed Leather Armor of Nightmare Fabric
(22148,1808,1,1,-1,3499),-- Recipe: Soulshot: S Grade
(22149,8273,6,6,3,1000000),-- Anteroom Key
(22149,57,4643,9510,0,700000),-- Adena
(22149,1895,1,1,2,99559),-- Metallic Fiber
(22149,1868,4,12,2,87115),-- Thread
(22149,1876,1,1,2,69692),-- Mithril Ore
(22149,1344,150,450,2,29039),-- Mithril Arrow
(22149,1873,1,1,2,139383),-- Silver Nugget
(22149,4042,1,1,2,5808),-- Enria
(22149,5529,1,1,1,1593),-- Dragon Slayer Edge
(22149,5546,1,1,1,1750),-- Sword of Miracles Edge
(22149,5545,1,1,1,1593),-- Dark Legion's Edge Blade
(22149,81,1,1,1,6),-- Dragon Slayer
(22149,151,1,1,1,6),-- Sword of Miracles 
(22149,2500,1,1,1,6),-- Dark Legion\'s Edge
(22149,960,1,1,2,139),-- Scroll: Enchant Armor (Grade S)
(22149,4042,1,1,-1,157290),-- Enria
(22149,1876,1,3,-1,943743),-- Mithril Ore
(22149,5487,1,1,-1,42498),-- Sealed Robe of Nightmare Fabric
(22149,8732,1,1,200,2904),-- Life Stone: level 76
(22149,8742,1,1,200,726),-- Mid-Grade Life Stone: level 76
(22149,8752,1,1,200,73),-- High-Grade Life Stone: level 76
(22150,57,4087,8341,0,700000),-- Adena
(22150,1873,2,6,2,50994),-- Silver Nugget
(22150,1895,1,1,2,145698),-- Metallic Fiber
(22150,1868,3,9,2,169981),-- Thread
(22150,1876,1,1,2,101988),-- Mithril Ore
(22150,4042,1,1,2,8499),-- Enria
(22150,6901,1,1,2,510),-- Recipe: Shining Arrow (100%)
(22150,5529,1,1,1,1399),-- Dragon Slayer Edge
(22150,5546,1,1,1,1536),-- Sword of Miracles Edge
(22150,5545,1,1,1,1399),-- Dark Legion's Edge Blade
(22150,81,1,1,1,5),-- Dragon Slayer
(22150,151,1,1,1,5),-- Sword of Miracles 
(22150,2500,1,1,1,5),-- Dark Legion\'s Edge
(22150,6901,1,1,-1,8287),-- Recipe: Shining Arrow (100%)
(22150,6688,1,1,-1,17911),-- Forgotten Blade Edge
(22151,57,5752,11742,0,700000),-- Adena
(22151,1868,3,9,2,239335),-- Thread
(22151,1873,2,6,2,71801),-- Silver Nugget
(22151,8275,1,1,3,41667),-- Key of Darkness
(22151,5549,1,1,2,71801),-- Metallic Thread
(22151,4044,1,1,2,23933),-- Thons
(22151,1889,1,1,2,47867),-- Compound Braid
(22151,5529,1,1,1,1969),-- Dragon Slayer Edge
(22151,5546,1,1,1,2163),-- Sword of Miracles Edge
(22151,5545,1,1,1,1969),-- Dark Legion's Edge Blade
(22151,959,1,1,2,28),-- Scroll: Enchant Weapon (Grade S)
(22151,81,1,1,1,7),-- Dragon Slayer
(22151,151,1,1,1,7),-- Sword of Miracles 
(22151,2500,1,1,1,7),-- Dark Legion\'s Edge
(22151,5549,1,3,-1,583380),-- Metallic Thread
(22151,6689,1,1,-1,24744),-- Basalt Battlehammer Head
(22151,959,1,1,-1,467),-- Scroll: Enchant Weapon (Grade S)
(22152,57,5060,10326,0,700000),-- Adena
(22152,1344,30,90,2,157878),-- Mithril Arrow
(22152,1873,1,1,2,151563),-- Silver Nugget
(22152,1868,4,12,2,94727),-- Thread
(22152,5549,1,1,2,37891),-- Metallic Thread
(22152,1889,1,1,2,25260),-- Compound Braid
(22152,4044,1,1,2,12630),-- Thons
(22152,5537,1,1,1,5199),-- Soul Separator Head
(22152,236,1,1,1,18),-- Soul Separator
(22152,960,1,1,2,152),-- Scroll: Enchant Armor (Grade S)
(22152,1889,1,1,-1,684135),-- Compound Braid
(22152,6690,1,1,-1,22223),-- Imperial Staff Head
(22152,6901,1,1,-1,10262),-- Recipe: Shining Arrow (100%)
(22152,8732,1,1,200,3157),-- Life Stone: level 76
(22152,8742,1,1,200,790),-- Mid-Grade Life Stone: level 76
(22152,8752,1,1,200,79),-- High-Grade Life Stone: level 76
(22153,57,4892,9983,0,700000),-- Adena
(22153,1344,30,90,2,152635),-- Mithril Arrow
(22153,1873,1,1,2,146529),-- Silver Nugget
(22153,1868,4,12,2,91581),-- Thread
(22153,1889,1,1,2,24422),-- Compound Braid
(22153,5549,1,1,2,36632),-- Metallic Thread
(22153,4044,1,1,2,12211),-- Thons
(22153,8732,1,1,200,3053),-- Life Stone: level 76
(22153,8742,1,1,200,763),-- Mid-Grade Life Stone: level 76
(22153,8752,1,1,200,76),-- High-Grade Life Stone: level 76
(22153,5546,1,1,1,1839),-- Sword of Miracles Edge
(22153,5545,1,1,1,1674),-- Dark Legion's Edge Blade
(22153,5529,1,1,1,1674),-- Dragon Slayer Edge
(22153,81,1,1,1,6),-- Dragon Slayer
(22153,151,1,1,1,6),-- Sword of Miracles 
(22153,2500,1,1,1,6),-- Dark Legion\'s Edge
(22153,6901,1,1,2,366),-- Recipe: Shining Arrow (100%)
(22153,6693,1,1,-1,21833),-- Dragon Hunter Axe Blade
(22153,6901,1,1,-1,9921),-- Recipe: Shining Arrow (100%)
(22153,6691,1,1,-1,21445),-- Angel Slayer Blade
(22154,57,4935,10074,0,700000),-- Adena
(22154,1868,4,12,2,132001),-- Thread
(22154,1876,1,1,2,105601),-- Mithril Ore
(22154,1895,1,1,2,150858),-- Metallic Fiber
(22154,1344,150,450,2,44000),-- Mithril Arrow
(22154,6342,1,1,1,13952),-- Sealed Majestic Earring Gemstone
(22154,6344,1,1,1,9029),-- Sealed Majestic Necklace Beads
(22154,1873,3,9,2,35200),-- Silver Nugget
(22154,4042,1,1,2,8800),-- Enria
(22154,6346,1,1,1,20053),-- Sealed Majestic Ring Gemstone
(22154,6328,1,1,1,123),-- Sealed Majestic Ring
(22154,6327,1,1,1,82),-- Sealed Majestic Earring
(22154,6326,1,1,1,62),-- Sealed Majestic Necklace
(22154,959,1,1,2,21),-- Scroll: Enchant Weapon (Grade S)
(22154,1895,1,7,-1,715000),-- Metallic Fiber
(22154,4042,1,1,-1,166833),-- Enria
(22154,960,1,1,-1,4004),-- Scroll: Enchant Armor (Grade S)
(22155,57,7054,14402,0,700000),-- Adena
(22155,1868,15,45,2,58709),-- Thread
(22155,1873,2,6,2,88063),-- Silver Nugget
(22155,5549,1,1,2,88063),-- Metallic Thread
(22155,1889,1,1,2,58709),-- Compound Braid
(22155,4044,1,1,2,29354),-- Thons
(22155,5326,1,1,1,40),-- Sealed Robe of Nightmare
(22155,5329,1,1,1,40),-- Sealed Majestic Robe
(22155,5487,1,1,1,5948),-- Sealed Robe of Nightmare Fabric
(22155,5488,1,1,1,5948),-- Sealed Majestic Robe Fabric
(22155,960,1,1,2,352),-- Scroll: Enchant Armor (Grade S)
(22155,6706,1,1,-1,232140),-- Sealed Imperial Crusader Helmet Pattern
(22155,6705,1,1,-1,199571),-- Sealed Imperial Crusader Shield Part
(22155,6704,1,1,-1,203748),-- Sealed Imperial Crusader Boots Design
(22156,57,6418,13138,0,700000),-- Adena
(22156,1873,2,6,2,53505),-- Silver Nugget
(22156,1868,3,9,2,178351),-- Thread
(22156,5549,1,1,2,53505),-- Metallic Thread
(22156,6346,1,1,1,26127),-- Sealed Majestic Ring Gemstone
(22156,1889,1,1,2,35670),-- Compound Braid
(22156,4044,1,1,2,17835),-- Thons
(22156,6342,1,1,1,18178),-- Sealed Majestic Earring Gemstone
(22156,6344,1,1,1,11764),-- Sealed Majestic Necklace Beads
(22156,8732,1,1,200,4459),-- Life Stone: level 76
(22156,8742,1,1,200,1115),-- Mid-Grade Life Stone: level 76
(22156,8752,1,1,200,111),-- High-Grade Life Stone: level 76
(22156,6328,1,1,1,161),-- Sealed Majestic Ring
(22156,6327,1,1,1,107),-- Sealed Majestic Earring
(22156,6326,1,1,1,80),-- Sealed Majestic Necklace
(22156,6901,1,1,2,535),-- Recipe: Shining Arrow (100%)
(22156,6689,1,1,-1,27659),-- Basalt Battlehammer Head
(22156,13099,1,1,-1,181883),-- Sealed Arcana Sickle Piece
(22156,6688,1,1,-1,28190),-- Forgotten Blade Edge
(22157,57,6157,12570,0,700000),-- Adena
(22157,1868,15,45,2,51242),-- Thread
(22157,1895,3,9,2,36602),-- Metallic Fiber
(22157,1876,1,1,2,153727),-- Mithril Ore
(22157,1873,2,6,2,76863),-- Silver Nugget
(22157,6346,1,1,1,25022),-- Sealed Majestic Ring Gemstone
(22157,6342,1,1,1,17409),-- Sealed Majestic Earring Gemstone
(22157,6344,1,1,1,11266),-- Sealed Majestic Necklace Beads
(22157,6328,1,1,1,154),-- Sealed Majestic Ring
(22157,6327,1,1,1,103),-- Sealed Majestic Earring
(22157,6326,1,1,1,77),-- Sealed Majestic Necklace
(22157,959,1,1,2,31),-- Scroll: Enchant Weapon (Grade S)
(22157,4042,1,1,2,12810),-- Enria
(22157,4039,1,1,-1,416344),-- Mold Glue
(22157,1894,1,1,-1,438256),-- Crafted Leather
(22157,959,1,1,-1,500),-- Scroll: Enchant Weapon (Grade S)
(22158,57,6280,12821,0,700000),-- Adena
(22158,1866,1,3,2,116145),-- Suede
(22158,1871,1,3,2,261325),-- Charcoal
(22158,1877,1,1,2,20906),-- Adamantite Nugget
(22158,4040,1,1,2,10453),-- Mold Lubricant
(22158,5515,1,1,1,9930),-- Sealed Majestic Gloves Design
(22158,5502,1,1,1,9930),-- Sealed Boots of Nightmare Lining
(22158,5514,1,1,1,9930),-- Sealed Gloves of Nightmare Design
(22158,5503,1,1,1,9930),-- Sealed Majestic Boots Lining
(22158,4041,1,1,2,4545),-- Mold Hardener
(22158,8732,1,1,200,4355),-- Life Stone: level 76
(22158,8742,1,1,200,1089),-- Mid-Grade Life Stone: level 76
(22158,8752,1,1,200,109),-- High-Grade Life Stone: level 76
(22158,960,1,1,2,209),-- Scroll: Enchant Armor (Grade S)
(22158,5318,1,1,1,87),-- Sealed Majestic Gloves
(22158,5314,1,1,1,87),-- Sealed Boots of Nightmare
(22158,5313,1,1,1,87),-- Sealed Gloves of Nightmare
(22158,5319,1,1,1,87),-- Sealed Majestic Boots
(22158,4040,1,1,-1,254792),-- Mold Lubricant
(22158,1877,1,1,-1,509585),-- Adamantite Nugget
(22158,960,1,1,-1,5096),-- Scroll: Enchant Armor (Grade S)
(22159,57,7209,14758,0,700000),-- Adena
(22159,1882,1,1,2,133560),-- Leather
(22159,1867,4,12,2,100170),-- Animal Skin
(22159,1872,7,21,2,53424),-- Animal Bone
(22159,1894,1,1,2,21088),-- Crafted Leather
(22159,4042,1,1,2,10017),-- Enria
(22159,8732,1,1,200,5008),-- Life Stone: level 76
(22159,8742,1,1,200,1252),-- Mid-Grade Life Stone: level 76
(22159,8752,1,1,200,125),-- High-Grade Life Stone: level 76
(22159,5487,1,1,1,6089),-- Sealed Robe of Nightmare Fabric
(22159,5488,1,1,1,6089),-- Sealed Majestic Robe Fabric
(22159,5326,1,1,1,41),-- Sealed Robe of Nightmare
(22159,5329,1,1,1,41),-- Sealed Majestic Robe
(22159,6901,1,1,2,601),-- Recipe: Shining Arrow (100%)
(22159,6703,1,1,-1,208583),-- Sealed Imperial Crusader Gauntlets Design
(22159,6702,1,1,-1,100403),-- Sealed Imperial Crusader Gaiters Pattern
(22159,6701,1,1,-1,77997),-- Sealed Imperial Crusader Breastplate Part
(22160,57,6418,13138,0,700000),-- Adena
(22160,1895,1,1,2,275170),-- Metallic Fiber
(22160,6342,1,1,1,18178),-- Sealed Majestic Earring Gemstone
(22160,6344,1,1,1,11764),-- Sealed Majestic Necklace Beads
(22160,6346,1,1,1,26127),-- Sealed Majestic Ring Gemstone
(22160,4042,1,1,2,16052),-- Enria
(22160,4043,1,1,2,32103),-- Asofe
(22160,4040,1,1,2,19262),-- Mold Lubricant
(22160,6328,1,1,1,161),-- Sealed Majestic Ring
(22160,6327,1,1,1,107),-- Sealed Majestic Earring
(22160,6326,1,1,1,80),-- Sealed Majestic Necklace
(22160,959,1,1,2,38),-- Scroll: Enchant Weapon (Grade S)
(22160,6693,1,1,-1,28701),-- Dragon Hunter Axe Blade
(22160,6694,1,1,-1,28701),-- Saint Spear Blade
(22160,6691,1,1,-1,28190),-- Angel Slayer Blade
(22161,57,7282,14866,0,700000),-- Adena
(22161,1895,1,1,2,155834),-- Metallic Fiber
(22161,1868,4,12,2,136355),-- Thread
(22161,1876,1,1,2,109084),-- Mithril Ore
(22161,1873,3,9,2,36361),-- Silver Nugget
(22161,1344,150,450,2,45451),-- Mithril Arrow
(22161,5495,1,1,1,13803),-- Sealed Shield of Nightmare Fragment
(22161,4042,1,1,2,9090),-- Enria
(22161,5527,1,1,1,10868),-- Sealed Helm of Nightmare Design
(22161,5528,1,1,1,10868),-- Sealed Majestic Circlet Design
(22161,5317,1,1,1,89),-- Sealed Majestic Circlet
(22161,5312,1,1,1,89),-- Sealed Helm of Nightmare
(22161,5315,1,1,1,128),-- Sealed Shield of Nightmare
(22161,960,1,1,2,218),-- Scroll: Enchant Armor (Grade S)
(22161,1894,1,1,-1,518308),-- Crafted Leather
(22161,4039,1,1,-1,492392),-- Mold Glue
(22161,959,1,1,-1,591),-- Scroll: Enchant Weapon (Grade S)
(22161,8732,1,1,200,4545),-- Life Stone: level 76
(22161,8742,1,1,200,1136),-- Mid-Grade Life Stone: level 76
(22161,8752,1,1,200,114),-- High-Grade Life Stone: level 76
(22162,57,6280,12821,0,700000),-- Adena
(22162,1894,1,1,2,33010),-- Crafted Leather
(22162,4044,1,1,2,31359),-- Thons
(22162,4039,1,1,2,31359),-- Mold Glue
(22162,5546,1,1,1,2361),-- Sword of Miracles Edge
(22162,4041,1,1,2,8181),-- Mold Hardener
(22162,5529,1,1,1,2150),-- Dragon Slayer Edge
(22162,5545,1,1,1,2150),-- Dark Legion's Edge Blade
(22162,6901,1,1,2,941),-- Recipe: Shining Arrow (100%)
(22162,81,1,1,1,8),-- Dragon Slayer
(22162,151,1,1,1,8),-- Sword of Miracles 
(22162,2500,1,1,1,8),-- Dark Legion\'s Edge
(22162,1877,1,1,-1,509585),-- Adamantite Nugget
(22162,4040,1,1,-1,254792),-- Mold Lubricant
(22162,960,1,1,-1,5096),-- Scroll: Enchant Armor (Grade S)
(22163,57,8351,17094,0,700000),-- Adena
(22163,1866,1,3,2,154701),-- Suede
(22163,1871,4,12,2,87019),-- Charcoal
(22163,4040,1,1,2,13923),-- Mold Lubricant
(22163,4041,1,1,2,6053),-- Mold Hardener
(22163,1877,1,1,2,27846),-- Adamantite Nugget
(22163,236,1,1,1,30),-- Soul Separator
(22163,959,1,1,2,28),-- Scroll: Enchant Weapon (Grade S)
(22163,5537,1,1,1,8596),-- Soul Separator Head
(22163,8732,1,1,200,5801),-- Life Stone: level 76
(22163,8742,1,1,200,1450),-- Mid-Grade Life Stone: level 76
(22163,8752,1,1,200,145),-- High-Grade Life Stone: level 76
(22163,6707,1,1,-1,84077),-- Sealed Draconic Leather Armor Part
(22163,6708,1,1,-1,241600),-- Sealed Draconic Leather Gloves Fabric
(22163,6709,1,1,-1,241600),-- Sealed Draconic Leather Boots Design
(22164,57,6418,13138,0,700000),-- Adena
(22164,1870,7,21,2,53505),-- Coal
(22164,1879,1,1,2,133763),-- Cokes
(22164,1865,4,12,2,100323),-- Varnish
(22164,1875,1,1,2,53505),-- Stone of Purity
(22164,6346,1,1,1,26127),-- Sealed Majestic Ring Gemstone
(22164,6342,1,1,1,18178),-- Sealed Majestic Earring Gemstone
(22164,6344,1,1,1,11764),-- Sealed Majestic Necklace Beads
(22164,4040,1,1,2,16051),-- Mold Lubricant
(22164,960,1,1,2,321),-- Scroll: Enchant Armor (Grade S)
(22164,6328,1,1,1,161),-- Sealed Majestic Ring
(22164,6327,1,1,1,107),-- Sealed Majestic Earring
(22164,6326,1,1,1,80),-- Sealed Majestic Necklace
(22164,6697,1,1,-1,27659),-- Arcana Mace Head
(22164,6696,1,1,-1,28638),-- Heavens Divider Edge
(22164,6695,1,1,-1,27659),-- Demon Splinter Blade
(22165,57,6157,12570,0,700000),-- Adena
(22165,1871,1,3,2,256212),-- Charcoal
(22165,1866,1,3,2,113872),-- Suede
(22165,1885,1,1,2,42702),-- High Grade Suede
(22165,5220,1,1,2,20497),-- Metal Hardener
(22165,5514,1,1,1,9736),-- Sealed Gloves of Nightmare Design
(22165,5503,1,1,1,9736),-- Sealed Majestic Boots Lining
(22165,5515,1,1,1,9736),-- Sealed Majestic Gloves Design
(22165,5502,1,1,1,9736),-- Sealed Boots of Nightmare Lining
(22165,4041,1,1,2,4456),-- Mold Hardener
(22165,6901,1,1,2,512),-- Recipe: Shining Arrow (100%)
(22165,8732,1,1,200,4270),-- Life Stone: level 76
(22165,8742,1,1,200,1068),-- Mid-Grade Life Stone: level 76
(22165,8752,1,1,200,106),-- High-Grade Life Stone: level 76
(22165,5318,1,1,1,85),-- Sealed Majestic Gloves
(22165,5314,1,1,1,85),-- Sealed Boots of Nightmare
(22165,5313,1,1,1,85),-- Sealed Gloves of Nightmare
(22165,5319,1,1,1,85),-- Sealed Majestic Boots
(22165,1894,1,1,-1,438256),-- Crafted Leather
(22165,4039,1,1,-1,416344),-- Mold Glue
(22165,959,1,1,-1,500),-- Scroll: Enchant Weapon (Grade S)
(22166,57,6157,12570,0,700000),-- Adena
(22166,1869,7,21,2,51242),-- Iron Ore
(22166,1864,4,12,2,192159),-- Stem
(22166,1880,1,1,2,76864),-- Steel
(22166,6346,1,1,1,25022),-- Sealed Majestic Ring Gemstone
(22166,4043,1,1,2,25621),-- Asofe
(22166,6342,1,1,1,17409),-- Sealed Majestic Earring Gemstone
(22166,6344,1,1,1,11266),-- Sealed Majestic Necklace Beads
(22166,5550,1,1,2,10248),-- Durable Metal Plate
(22166,6328,1,1,1,154),-- Sealed Majestic Ring
(22166,6327,1,1,1,103),-- Sealed Majestic Earring
(22166,6326,1,1,1,77),-- Sealed Majestic Necklace
(22166,959,1,1,2,31),-- Scroll: Enchant Weapon (Grade S)
(22166,1877,1,1,-1,499612),-- Adamantite Nugget
(22166,4040,1,1,-1,249806),-- Mold Lubricant
(22166,960,1,1,-1,4996),-- Scroll: Enchant Armor (Grade S)
(22167,57,7370,15108,0,700000),-- Adena
(22167,4039,1,1,2,36897),-- Mold Glue
(22167,1894,1,1,2,38839),-- Crafted Leather
(22167,4044,1,1,2,36897),-- Thons
(22167,5487,1,1,1,6230),-- Sealed Robe of Nightmare Fabric
(22167,5488,1,1,1,6230),-- Sealed Majestic Robe Fabric
(22167,4041,1,1,2,9625),-- Mold Hardener
(22167,960,1,1,2,443),-- Scroll: Enchant Armor (Grade S)
(22167,5326,1,1,1,42),-- Sealed Robe of Nightmare
(22167,5329,1,1,1,42),-- Sealed Majestic Robe
(22167,6712,1,1,-1,213418),-- Sealed Major Arcana Gloves fabric
(22167,6711,1,1,-1,74269),-- Sealed Major Arcana Robe Part
(22167,6710,1,1,-1,243157),-- Sealed Draconic Leather Helmet Pattern
(22168,57,6433,13186,0,700000),-- Adena
(22168,1869,18,54,2,22364),-- Iron Ore
(22168,1870,2,6,2,161023),-- Coal
(22168,1871,3,9,2,134185),-- Charcoal
(22168,1872,7,21,2,76677),-- Animal Bone
(22168,1867,12,36,2,44729),-- Animal Skin
(22168,5481,1,1,1,5420),-- Sealed Majestic Leather Armor Fabric
(22168,5480,1,1,1,5420),-- Sealed Leather Armor of Nightmare Fabric
(22168,6901,1,1,2,805),-- Recipe: Shining Arrow (100%)
(22168,5320,1,1,1,37),-- Sealed Leather Armor of Nightmare
(22168,5323,1,1,1,37),-- Sealed Majestic Leather Armor
(22168,6700,1,1,-1,285159),-- Sealed Tateossian Ring wire
(22168,6698,1,1,-1,256985),-- Sealed Tateossian Earring  Part
(22168,6699,1,1,-1,156628),-- Sealed Tateossian Ring Gem
(22169,57,7442,15233,0,700000),-- Adena
(22169,1344,400,1200,2,17449),-- Mithril Arrow
(22169,1864,5,15,2,111673),-- Stem
(22169,1865,3,9,2,93061),-- Varnish
(22169,1866,3,9,2,53177),-- Suede
(22169,1868,12,36,2,46530),-- Thread
(22169,1873,4,12,2,24816),-- Silver Nugget
(22169,5522,1,1,1,4754),-- Sealed Armor of Nightmare Pattern
(22169,5523,1,1,1,4754),-- Sealed Majestic Plate Armor Pattern
(22169,5311,1,1,1,32),-- Sealed Armor of Nightmare
(22169,5316,1,1,1,32),-- Sealed Majestic Plate Armor
(22169,959,1,1,2,22),-- Scroll: Enchant Weapon (Grade S)
(22169,4039,1,1,-1,504077),-- Mold Glue
(22169,1894,1,1,-1,530607),-- Crafted Leather
(22169,13099,1,1,-1,210896),-- Sealed Arcana Sickle Piece
(22169,8732,1,1,200,4653),-- Life Stone: level 76
(22169,8742,1,1,200,1163),-- Mid-Grade Life Stone: level 76
(22169,8752,1,1,200,116),-- High-Grade Life Stone: level 76
(22170,57,6418,13138,0,700000),-- Adena
(22170,4039,1,1,2,32103),-- Mold Glue
(22170,1894,1,1,2,33793),-- Crafted Leather
(22170,4044,1,1,2,32103),-- Thons
(22170,5487,1,1,1,5421),-- Sealed Robe of Nightmare Fabric
(22170,5488,1,1,1,5421),-- Sealed Majestic Robe Fabric
(22170,4041,1,1,2,8375),-- Mold Hardener
(22170,960,1,1,2,385),-- Scroll: Enchant Armor (Grade S)
(22170,5326,1,1,1,36),-- Sealed Robe of Nightmare
(22170,5329,1,1,1,36),-- Sealed Majestic Robe
(22170,1877,1,1,-1,521677),-- Adamantite Nugget
(22170,4040,1,1,-1,260839),-- Mold Lubricant
(22170,960,1,1,-1,5217),-- Scroll: Enchant Armor (Grade S)
(22171,57,8537,17499,0,700000),-- Adena
(22171,1895,3,9,2,50878),-- Metallic Fiber
(22171,1876,1,1,2,213688),-- Mithril Ore
(22171,1873,2,6,2,106844),-- Silver Nugget
(22171,1868,15,45,2,71229),-- Thread
(22171,4042,1,1,2,17808),-- Enria
(22171,8342,1,1,1,2198),-- Flaming Dragon Skull Piece
(22171,5533,1,1,1,2198),-- Elysian Head
(22171,5544,1,1,1,2196),-- Branch of The Mother Tree Head
(22171,8349,1,1,1,2198),-- Doom Crusher Head
(22171,6901,1,1,2,1068),-- Recipe: Shining Arrow (100%)
(22171,164,1,1,1,8),-- Elysian
(22171,213,1,1,1,8),-- Branch of The Mother Tree
(22171,7895,1,1,1,8),-- Flaming Dragon Skull
(22171,7902,1,1,1,8),-- Doom Crusher
(22171,6713,1,1,-1,247201),-- Sealed Major Arcana Boots Design
(22171,6714,1,1,-1,281647),-- Sealed Major Arcana Circlet Pattern
(22171,7579,1,1,-1,37962),-- Draconic Bow Shaft
(22172,57,7442,15233,0,700000),-- Adena
(22172,1872,5,15,2,74448),-- Animal Bone
(22172,1867,9,27,2,41360),-- Animal Skin
(22172,1875,1,1,2,37224),-- Stone of Purity
(22172,1881,1,1,2,74448),-- Coarse Bone Powder
(22172,1344,40,120,2,174487),-- Mithril Arrow
(22172,5527,1,1,1,11126),-- Sealed Helm of Nightmare Design
(22172,5495,1,1,1,14131),-- Sealed Shield of Nightmare Fragment
(22172,5528,1,1,1,11126),-- Sealed Majestic Circlet Design
(22172,4041,1,1,2,4856),-- Mold Hardener
(22172,959,1,1,2,22),-- Scroll: Enchant Weapon (Grade S)
(22172,5317,1,1,1,92),-- Sealed Majestic Circlet
(22172,5312,1,1,1,92),-- Sealed Helm of Nightmare
(22172,5315,1,1,1,131),-- Sealed Shield of Nightmare
(22172,8732,1,1,200,4653),-- Life Stone: level 76
(22172,8742,1,1,200,1163),-- Mid-Grade Life Stone: level 76
(22172,8752,1,1,200,116),-- High-Grade Life Stone: level 76
(22172,6714,1,1,-1,245313),-- Sealed Major Arcana Circlet Pattern
(22172,6901,1,1,-1,15122),-- Recipe: Shining Arrow (100%)
(22172,959,1,1,-1,605),-- Scroll: Enchant Weapon (Grade S)
(22173,57,6293,12881,0,700000),-- Adena
(22173,1868,3,9,2,262291),-- Thread
(22173,1873,2,6,2,78687),-- Silver Nugget
(22173,5549,1,1,2,78687),-- Metallic Thread
(22173,1889,1,1,2,52458),-- Compound Braid
(22173,4044,1,1,2,26229),-- Thons
(22173,5487,1,1,1,5315),-- Sealed Robe of Nightmare Fabric
(22173,5488,1,1,1,5315),-- Sealed Majestic Robe Fabric
(22173,5326,1,1,1,36),-- Sealed Robe of Nightmare
(22173,5329,1,1,1,36),-- Sealed Majestic Robe
(22173,960,1,1,2,315),-- Scroll: Enchant Armor (Grade S)
(22173,4039,1,1,-1,426223),-- Mold Glue
(22173,1894,1,1,-1,448656),-- Crafted Leather
(22173,959,1,1,-1,511),-- Scroll: Enchant Weapon (Grade S)
(22174,57,6280,12821,0,700000),-- Adena
(22174,1344,40,120,2,209993),-- Mithril Arrow
(22174,1881,1,1,2,89597),-- Coarse Bone Powder
(22174,1872,5,15,2,89597),-- Animal Bone
(22174,1867,9,27,2,49776),-- Animal Skin
(22174,1875,1,1,2,44799),-- Stone of Purity
(22174,6901,1,1,2,672),-- Recipe: Shining Arrow (100%)
(22174,4041,1,1,2,5843),-- Mold Hardener
(22174,5487,1,1,1,5295),-- Sealed Robe of Nightmare Fabric
(22174,5488,1,1,1,5295),-- Sealed Majestic Robe Fabric
(22174,5326,1,1,1,36),-- Sealed Robe of Nightmare
(22174,5329,1,1,1,36),-- Sealed Majestic Robe
(22174,1877,1,1,-1,509585),-- Adamantite Nugget
(22174,4040,1,1,-1,254792),-- Mold Lubricant
(22174,960,1,1,-1,5096),-- Scroll: Enchant Armor (Grade S)
(22175,57,250,511,0,700000),-- Adena
(22175,1868,1,1,2,62538),-- Thread
(22175,1873,1,1,2,12508),-- Silver Nugget
(22175,1876,1,1,2,6254),-- Mithril Ore
(22175,1895,1,1,2,8934),-- Metallic Fiber
(22175,4042,1,1,2,521),-- Enria
(22175,5522,1,1,1,160),-- Sealed Armor of Nightmare Pattern
(22175,5523,1,1,1,160),-- Sealed Majestic Plate Armor Pattern
(22175,5311,1,1,1,1),-- Sealed Armor of Nightmare
(22175,5316,1,1,1,1),-- Sealed Majestic Plate Armor
(22175,959,1,1,2,1),-- Scroll: Enchant Weapon (Grade S)
(22175,6689,1,1,-1,1078),-- Basalt Battlehammer Head
(22175,960,1,1,-1,203),-- Scroll: Enchant Armor (Grade S)
(22175,4042,1,1,-1,8469),-- Enria
(22176,57,250,511,0,700000),-- Adena
(22176,1344,10,30,2,33502),-- Mithril Arrow
(22176,1871,1,1,2,26802),-- Charcoal
(22176,1866,1,1,2,17868),-- Suede
(22176,5481,1,1,1,211),-- Sealed Majestic Leather Armor Fabric
(22176,5480,1,1,1,211),-- Sealed Leather Armor of Nightmare Fabric
(22176,960,1,1,2,11),-- Scroll: Enchant Armor (Grade S)
(22176,5320,1,1,1,1),-- Sealed Leather Armor of Nightmare
(22176,5323,1,1,1,1),-- Sealed Majestic Leather Armor
(22176,1877,1,1,2,1072),-- Adamantite Nugget
(22176,4041,1,1,2,233),-- Mold Hardener
(22176,4040,1,1,2,536),-- Mold Lubricant
(22176,1877,1,1,-1,20325),-- Adamantite Nugget
(22176,6901,1,1,-1,508),-- Recipe: Shining Arrow (100%)
(22176,6690,1,1,-1,1100),-- Imperial Staff Head
(22188,57,5136,10528,0,700000),-- Adena
(22188,4039,1,1,2,16070),-- Mold Glue
(22188,1894,1,1,2,16916),-- Crafted Leather
(22188,4044,1,1,2,16070),-- Thons
(22188,4041,1,1,2,4192),-- Mold Hardener
(22188,5529,1,1,1,1763),-- Dragon Slayer Edge
(22188,5545,1,1,1,1763),-- Dark Legion's Edge Blade
(22188,5546,1,1,1,1936),-- Sword of Miracles Edge
(22188,959,1,1,2,19),-- Scroll: Enchant Weapon (Grade S)
(22188,81,1,1,1,6),-- Dragon Slayer
(22188,151,1,1,1,6),-- Sword of Miracles 
(22188,2500,1,1,1,6),-- Dark Legion\'s Edge
(22188,8732,1,1,200,4018),-- Life Stone: level 76
(22188,8742,1,1,200,1004),-- Mid-Grade Life Stone: level 76
(22188,8752,1,1,200,100),-- High-Grade Life Stone: level 76
(22188,1894,1,1,-1,366512),-- Crafted Leather
(22188,6901,1,1,-1,10446),-- Recipe: Shining Arrow (100%)
(22188,6688,1,1,-1,22578),-- Forgotten Blade Edge
(22189,57,4145,8484,0,700000),-- Adena
(22189,4039,1,1,2,20732),-- Mold Glue
(22189,1894,1,1,2,21823),-- Crafted Leather
(22189,4044,1,1,2,20732),-- Thons
(22189,4041,1,1,2,5408),-- Mold Hardener
(22189,960,1,1,2,249),-- Scroll: Enchant Armor (Grade S)
(22189,8342,1,1,1,1066),-- Flaming Dragon Skull Piece
(22189,5533,1,1,1,1066),-- Elysian Head
(22189,5544,1,1,1,1065),-- Branch of The Mother Tree Head
(22189,8349,1,1,1,1066),-- Doom Crusher Head
(22189,164,1,1,1,4),-- Elysian
(22189,213,1,1,1,4),-- Branch of The Mother Tree
(22189,7895,1,1,1,4),-- Flaming Dragon Skull
(22189,7902,1,1,1,4),-- Doom Crusher
(22189,6690,1,1,-1,18240),-- Imperial Staff Head
(22189,6689,1,1,-1,17862),-- Basalt Battlehammer Head
(22189,6901,1,1,-1,8422),-- Recipe: Shining Arrow (100%)
(22190,57,7411,7610,0,700000),-- Adena
(22190,1895,1,1,2,350000),-- Metallic Fiber
(22196,57,13330,27213,0,700000),-- Adena
(22196,1879,1,1,2,184884),-- Cokes
(22196,1885,1,1,2,92442),-- High Grade Suede
(22196,9628,1,1,2,8700),-- Leonard
(22196,9630,1,1,2,5839),-- Orichalcum
(22196,9629,1,1,2,4823),-- Adamantine
(22196,9573,1,1,200,7704),-- Life Stone: level 80
(22196,9574,1,1,200,1926),-- Mid-Grade Life Stone: level 80
(22196,9575,1,1,200,193),-- High-Grade Life Stone: level 80
(22196,9576,1,1,200,39),-- Top-Grade Life Stone: level 80
(22196,5529,1,1,1,4563),-- Dragon Slayer Edge
(22196,5545,1,1,1,4563),-- Dark Legion\'s Edge Blade
(22196,5546,1,1,1,5012),-- Sword of Miracles Edge
(22196,81,1,1,1,16),-- Dragon Slayer
(22196,151,1,1,1,16),-- Sword of Miracles 
(22196,2500,1,1,1,16),-- Dark Legion\'s Edge
(22196,13099,1,1,-1,377090),-- Sealed Arcana Sickle Piece
(22196,6688,1,1,-1,58445),-- Forgotten Blade Edge
(22196,6901,1,1,-1,27039),-- Recipe: Shining Arrow (100%)
(22197,57,5813,11868,0,700000),-- Adena
(22197,1879,1,1,2,80628),-- Cokes
(22197,1885,1,1,2,40314),-- High Grade Suede
(22197,9628,1,1,2,3794),-- Leonard
(22197,9630,1,1,2,2546),-- Orichalcum
(22197,9629,1,1,2,2103),-- Adamantine
(22197,9573,1,1,200,3360),-- Life Stone: level 80
(22197,9574,1,1,200,840),-- Mid-Grade Life Stone: level 80
(22197,9575,1,1,200,84),-- High-Grade Life Stone: level 80
(22197,9576,1,1,200,17),-- Top-Grade Life Stone: level 80
(22197,5546,1,1,1,2186),-- Sword of Miracles Edge
(22197,5529,1,1,1,1990),-- Dragon Slayer Edge
(22197,5545,1,1,1,1990),-- Dark Legion\'s Edge Blade
(22197,81,1,1,1,7),-- Dragon Slayer
(22197,151,1,1,1,7),-- Sword of Miracles 
(22197,2500,1,1,1,7),-- Dark Legion\'s Edge
(22197,6690,1,1,-1,25536),-- Imperial Staff Head
(22197,6689,1,1,-1,25008),-- Basalt Battlehammer Head
(22197,6901,1,1,-1,11792),-- Recipe: Shining Arrow (100%)
(22198,57,13330,27213,0,700000),-- Adena
(22198,1879,1,1,2,184884),-- Cokes
(22198,1885,1,1,2,92442),-- High Grade Suede
(22198,9628,1,1,2,8700),-- Leonard
(22198,9630,1,1,2,5839),-- Orichalcum
(22198,9629,1,1,2,4823),-- Adamantine
(22198,9573,1,1,200,7704),-- Life Stone: level 80
(22198,9574,1,1,200,1926),-- Mid-Grade Life Stone: level 80
(22198,9575,1,1,200,193),-- High-Grade Life Stone: level 80
(22198,9576,1,1,200,39),-- Top-Grade Life Stone: level 80
(22198,5529,1,1,1,4563),-- Dragon Slayer Edge
(22198,5546,1,1,1,5012),-- Sword of Miracles Edge
(22198,5545,1,1,1,4563),-- Dark Legion\'s Edge Blade
(22198,81,1,1,1,16),-- Dragon Slayer
(22198,151,1,1,1,16),-- Sword of Miracles 
(22198,2500,1,1,1,16),-- Dark Legion\'s Edge
(22198,13099,1,1,-1,565534),-- Sealed Arcana Sickle Piece
(22198,959,1,1,-1,1622),-- Scroll: Enchant Weapon (Grade S)
(22199,57,22768,46483,0,700000),-- Adena
(22199,6346,1,1,1,92524),-- Sealed Majestic Ring Gemstone
(22199,6342,1,1,1,64376),-- Sealed Majestic Earring Gemstone
(22199,6344,1,1,1,41661),-- Sealed Majestic Necklace Beads
(22199,1879,1,3,2,157901),-- Cokes
(22199,1885,1,1,2,157901),-- High Grade Suede
(22199,9628,1,1,2,14861),-- Leonard
(22199,9630,1,1,2,9973),-- Orichalcum
(22199,9629,1,1,2,8238),-- Adamantine
(22199,9573,1,1,200,13159),-- Life Stone: level 80
(22199,9574,1,1,200,3290),-- Mid-Grade Life Stone: level 80
(22199,9575,1,1,200,329),-- High-Grade Life Stone: level 80
(22199,9576,1,1,200,66),-- Top-Grade Life Stone: level 80
(22199,6328,1,1,1,569),-- Sealed Majestic Ring
(22199,6327,1,1,1,379),-- Sealed Majestic Earring
(22199,6326,1,1,1,285),-- Sealed Majestic Necklace
(22199,6693,1,1,-1,101640),-- Dragon Hunter Axe Blade
(22199,6691,1,1,-1,99831),-- Angel Slayer Blade
(22199,6901,1,1,-1,46186),-- Recipe: Shining Arrow (100%)
(22200,57,6033,12317,0,700000),-- Adena
(22200,1879,1,1,2,83682),-- Cokes
(22200,1885,1,1,2,41841),-- High Grade Suede
(22200,9628,1,1,2,3938),-- Leonard
(22200,9630,1,1,2,2643),-- Orichalcum
(22200,9629,1,1,2,2183),-- Adamantine
(22200,9573,1,1,200,3487),-- Life Stone: level 80
(22200,9574,1,1,200,872),-- Mid-Grade Life Stone: level 80
(22200,9575,1,1,200,87),-- High-Grade Life Stone: level 80
(22200,9576,1,1,200,17),-- Top-Grade Life Stone: level 80
(22200,8342,1,1,1,1550),-- Flaming Dragon Skull Piece
(22200,8349,1,1,1,1549),-- Doom Crusher Head
(22200,5533,1,1,1,1550),-- Elysian Head
(22200,5544,1,1,1,1548),-- Branch of The Mother Tree Head
(22200,164,1,1,1,5),-- Elysian
(22200,213,1,1,1,5),-- Branch of The Mother Tree
(22200,7895,1,1,1,5),-- Flaming Dragon Skull
(22200,7902,1,1,1,5),-- Doom Crusher
(22200,6694,1,1,-1,26933),-- Saint Spear Blade
(22200,6901,1,1,-1,12238),-- Recipe: Shining Arrow (100%)
(22200,960,1,1,-1,4895),-- Scroll: Enchant Armor (Grade S)
(22201,57,6033,12317,0,700000),-- Adena
(22201,1879,1,1,2,83682),-- Cokes
(22201,1885,1,1,2,41841),-- High Grade Suede
(22201,9628,1,1,2,3938),-- Leonard
(22201,9630,1,1,2,2643),-- Orichalcum
(22201,9629,1,1,2,2183),-- Adamantine
(22201,9573,1,1,200,3487),-- Life Stone: level 80
(22201,9574,1,1,200,872),-- Mid-Grade Life Stone: level 80
(22201,9575,1,1,200,87),-- High-Grade Life Stone: level 80
(22201,9576,1,1,200,17),-- Top-Grade Life Stone: level 80
(22201,5544,1,1,1,1548),-- Branch of The Mother Tree Head
(22201,8342,1,1,1,1550),-- Flaming Dragon Skull Piece
(22201,5533,1,1,1,1550),-- Elysian Head
(22201,8349,1,1,1,1549),-- Doom Crusher Head
(22201,164,1,1,1,5),-- Elysian
(22201,213,1,1,1,5),-- Branch of The Mother Tree
(22201,7895,1,1,1,5),-- Flaming Dragon Skull
(22201,7902,1,1,1,5),-- Doom Crusher
(22201,1895,1,7,-1,874174),-- Metallic Fiber
(22201,13099,1,1,-1,170678),-- Sealed Arcana Sickle Piece
(22201,4040,1,1,-1,244769),-- Mold Lubricant
(22202,57,6033,12317,0,700000),-- Adena
(22202,1879,1,1,2,83682),-- Cokes
(22202,1885,1,1,2,41841),-- High Grade Suede
(22202,9628,1,1,2,3938),-- Leonard
(22202,9630,1,1,2,2643),-- Orichalcum
(22202,9629,1,1,2,2183),-- Adamantine
(22202,9573,1,1,200,3487),-- Life Stone: level 80
(22202,9574,1,1,200,872),-- Mid-Grade Life Stone: level 80
(22202,9575,1,1,200,87),-- High-Grade Life Stone: level 80
(22202,9576,1,1,200,17),-- Top-Grade Life Stone: level 80
(22202,8342,1,1,1,1550),-- Flaming Dragon Skull Piece
(22202,5533,1,1,1,1550),-- Elysian Head
(22202,5544,1,1,1,1548),-- Branch of The Mother Tree Head
(22202,8349,1,1,1,1549),-- Doom Crusher Head
(22202,164,1,1,1,5),-- Elysian
(22202,213,1,1,1,5),-- Branch of The Mother Tree
(22202,7895,1,1,1,5),-- Flaming Dragon Skull
(22202,7902,1,1,1,5),-- Doom Crusher
(22202,6696,1,1,-1,26873),-- Heavens Divider Edge
(22202,6901,1,1,-1,12238),-- Recipe: Shining Arrow (100%)
(22203,57,5934,12114,0,700000),-- Adena
(22203,1879,1,1,2,82302),-- Cokes
(22203,1885,1,1,2,41151),-- High Grade Suede
(22203,9628,1,1,2,3873),-- Leonard
(22203,9630,1,1,2,2599),-- Orichalcum
(22203,9629,1,1,2,2147),-- Adamantine
(22203,9573,1,1,200,3429),-- Life Stone: level 80
(22203,9574,1,1,200,857),-- Mid-Grade Life Stone: level 80
(22203,9575,1,1,200,86),-- High-Grade Life Stone: level 80
(22203,9576,1,1,200,17),-- Top-Grade Life Stone: level 80
(22203,5534,1,1,1,6101),-- Soul Bow Stave
(22203,289,1,1,1,21),-- Soul Bow
(22203,1875,1,1,-1,802440),-- Stone of Purity
(22203,6697,1,1,-1,25527),-- Arcana Mace Head
(22203,959,1,1,-1,481),-- Scroll: Enchant Weapon (Grade S)
(22204,57,5786,11812,0,700000),-- Adena
(22204,1879,1,1,2,80251),-- Cokes
(22204,1885,1,1,2,40126),-- High Grade Suede
(22204,9628,1,1,2,3777),-- Leonard
(22204,9630,1,1,2,2534),-- Orichalcum
(22204,9629,1,1,2,2093),-- Adamantine
(22204,9573,1,1,200,3344),-- Life Stone: level 80
(22204,9574,1,1,200,836),-- Mid-Grade Life Stone: level 80
(22204,9575,1,1,200,84),-- High-Grade Life Stone: level 80
(22204,9576,1,1,200,17),-- Top-Grade Life Stone: level 80
(22204,5534,1,1,1,5949),-- Soul Bow Stave
(22204,289,1,1,1,21),-- Soul Bow
(22204,1885,1,3,-1,489029),-- High Grade Suede
(22204,6698,1,1,-1,230538),-- Sealed Tateossian Earring Part
(22204,6901,1,1,-1,11737),-- Recipe: Shining Arrow (100%)
(22205,57,5934,12114,0,700000),-- Adena
(22205,1879,1,1,2,82302),-- Cokes
(22205,1885,1,1,2,41151),-- High Grade Suede
(22205,9628,1,1,2,3873),-- Leonard
(22205,9630,1,1,2,2599),-- Orichalcum
(22205,9629,1,1,2,2147),-- Adamantine
(22205,9573,1,1,200,3429),-- Life Stone: level 80
(22205,9574,1,1,200,857),-- Mid-Grade Life Stone: level 80
(22205,9575,1,1,200,86),-- High-Grade Life Stone: level 80
(22205,9576,1,1,200,17),-- Top-Grade Life Stone: level 80
(22205,5534,1,1,1,6101),-- Soul Bow Stave
(22205,289,1,1,1,21),-- Soul Bow
(22205,13099,1,1,-1,167863),-- Sealed Arcana Sickle Piece
(22205,6699,1,1,-1,262349),-- Sealed Tateossian Ring Gem
(22205,6901,1,1,-1,12037),-- Recipe: Shining Arrow (100%)
(22208,57,6577,13427,0,700000),-- Adena
(22208,5537,1,1,1,6758),-- Soul Separator Head
(22208,236,1,1,1,24),-- Soul Separator
(22208,8954,1,1,2,2815),-- Blue Primeval Crystal
(22208,8955,1,1,2,1408),-- Green Primeval Crystal
(22208,8956,1,1,2,704),-- Red Primeval Crystal
(22208,960,1,1,2,282),-- Scroll: Enchant Armor (Grade S)
(22208,959,1,1,2,28),-- Scroll: Enchant Weapon (Grade S)
(22208,8752,1,1,200,147),-- High-Grade Life Stone: level 76
(22208,8762,1,1,200,29),-- Top-Grade Life Stone: level 76
(22208,6703,1,1,-1,189953),-- Sealed Imperial Crusader Gauntlets Design
(22208,6901,1,1,-1,13341),-- Recipe: Shining Arrow (100%)
(22208,959,1,1,-1,534),-- Scroll: Enchant Weapon (Grade S)
(22209,57,6047,12344,0,700000),-- Adena
(22209,5537,1,1,1,6213),-- Soul Separator Head
(22209,236,1,1,1,22),-- Soul Separator
(22209,8954,1,1,2,2588),-- Blue Primeval Crystal
(22209,8955,1,1,2,1294),-- Green Primeval Crystal
(22209,8956,1,1,2,647),-- Red Primeval Crystal
(22209,960,1,1,2,259),-- Scroll: Enchant Armor (Grade S)
(22209,959,1,1,2,26),-- Scroll: Enchant Weapon (Grade S)
(22209,8752,1,1,200,135),-- High-Grade Life Stone: level 76
(22209,8762,1,1,200,27),-- Top-Grade Life Stone: level 76
(22209,1894,1,1,-1,430376),-- Crafted Leather
(22209,4042,1,1,-1,204429),-- Enria
(22209,6704,1,1,-1,174638),-- Sealed Imperial Crusader Boots Design
(22210,57,6577,13427,0,700000),-- Adena
(22210,5537,1,1,1,6758),-- Soul Separator Head
(22210,236,1,1,1,24),-- Soul Separator
(22210,8954,1,1,2,2815),-- Blue Primeval Crystal
(22210,8955,1,1,2,1408),-- Green Primeval Crystal
(22210,8956,1,1,2,704),-- Red Primeval Crystal
(22210,960,1,1,2,282),-- Scroll: Enchant Armor (Grade S)
(22210,959,1,1,2,28),-- Scroll: Enchant Weapon (Grade S)
(22210,8752,1,1,200,147),-- High-Grade Life Stone: level 76
(22210,8762,1,1,200,29),-- Top-Grade Life Stone: level 76
(22210,6705,1,1,-1,186058),-- Sealed Imperial Crusader Shield Part
(22210,6901,1,1,-1,13341),-- Recipe: Shining Arrow (100%)
(22211,57,6191,12640,0,700000),-- Adena
(22211,5538,1,1,1,6361),-- Dragon Grinder Edge
(22211,270,1,1,1,22),-- Dragon Grinder
(22211,8954,1,1,2,3710),-- Blue Primeval Crystal
(22211,8955,1,1,2,1855),-- Green Primeval Crystal
(22211,8956,1,1,2,927),-- Red Primeval Crystal
(22211,960,1,1,2,371),-- Scroll: Enchant Armor (Grade S)
(22211,959,1,1,2,37),-- Scroll: Enchant Weapon (Grade S)
(22211,13099,1,1,-1,175157),-- Sealed Arcana Sickle Piece
(22211,6706,1,1,-1,203741),-- Sealed Imperial Crusader Helmet Pattern
(22211,960,1,1,-1,5024),-- Scroll: Enchant Armor (Grade S)
(22212,57,5939,12126,0,700000),-- Adena
(22212,5538,1,1,1,6102),-- Dragon Grinder Edge
(22212,270,1,1,1,21),-- Dragon Grinder
(22212,8954,1,1,2,3559),-- Blue Primeval Crystal
(22212,8955,1,1,2,1779),-- Green Primeval Crystal
(22212,8956,1,1,2,890),-- Red Primeval Crystal
(22212,960,1,1,2,356),-- Scroll: Enchant Armor (Grade S)
(22212,959,1,1,2,36),-- Scroll: Enchant Weapon (Grade S)
(22212,4041,1,1,-1,104767),-- Mold Hardener
(22212,6707,1,1,-1,59697),-- Sealed Draconic Leather Armor Part
(22212,6901,1,1,-1,12048),-- Recipe: Shining Arrow (100%)
(22213,57,6191,12640,0,700000),-- Adena
(22213,5538,1,1,1,6361),-- Dragon Grinder Edge
(22213,270,1,1,1,22),-- Dragon Grinder
(22213,8954,1,1,2,3710),-- Blue Primeval Crystal
(22213,8955,1,1,2,1855),-- Green Primeval Crystal
(22213,8956,1,1,2,927),-- Red Primeval Crystal
(22213,960,1,1,2,371),-- Scroll: Enchant Armor (Grade S)
(22213,959,1,1,2,37),-- Scroll: Enchant Weapon (Grade S)
(22213,6708,1,1,-1,178823),-- Sealed Draconic Leather Gloves Fabric
(22213,6709,1,1,-1,178823),-- Sealed Draconic Leather Boots Design
(22213,6901,1,1,-1,12560),-- Recipe: Shining Arrow (100%)
(22214,57,5863,11970,0,700000),-- Adena
(22214,5514,1,1,1,9271),-- Sealed Gauntlets of Nightmare Design
(22214,5502,1,1,1,9271),-- Sealed Boots of Nightmare Lining
(22214,5515,1,1,1,9271),-- Sealed Majestic Gauntlets Design
(22214,5503,1,1,1,9271),-- Sealed Majestic Boots Lining
(22214,5313,1,1,1,81),-- Sealed Gloves of Nightmare
(22214,5314,1,1,1,81),-- Sealed Boots of Nightmare
(22214,5318,1,1,1,81),-- Sealed Majestic Gloves
(22214,5319,1,1,1,81),-- Sealed Majestic Boots
(22214,8954,1,1,2,3513),-- Blue Primeval Crystal
(22214,8955,1,1,2,1757),-- Green Primeval Crystal
(22214,8956,1,1,2,878),-- Red Primeval Crystal
(22214,960,1,1,2,351),-- Scroll: Enchant Armor (Grade S)
(22214,959,1,1,2,35),-- Scroll: Enchant Weapon (Grade S)
(22214,5549,1,3,-1,594682),-- Metallic Thread
(22214,13099,1,1,-1,165689),-- Sealed Arcana Sickle Piece
(22214,959,1,1,-1,476),-- Scroll: Enchant Weapon (Grade S)
(22215,57,68305,139448,0,700000),-- Adena
(22215,5541,1,1,1,70147),-- Tallum Glaive Edge
(22215,305,1,1,1,247),-- Tallum Glaive
(22215,1879,3,9,2,157901),-- Cokes
(22215,1885,2,6,2,118426),-- High Grade Suede
(22215,9628,1,1,2,44584),-- Leonard
(22215,9630,1,1,2,29918),-- Orichalcum
(22215,9629,1,1,2,24715),-- Adamantine
(22215,9573,1,1,200,39475),-- Life Stone: level 80
(22215,9574,1,1,200,9869),-- Mid-Grade Life Stone: level 80
(22215,9575,1,1,200,987),-- High-Grade Life Stone: level 80
(22215,9576,1,1,200,198),-- Top-Grade Life Stone: level 80
(22215,6711,1,1,-1,686527),-- Sealed Major Arcana Robe Part
(22215,6710,1,5,-1,749227),-- Sealed Draconic Leather Helmet Pattern
(22215,6901,1,1,-1,138558),-- Recipe: Shining Arrow (100%)
(22216,57,68305,139448,0,700000),-- Adena
(22216,5541,1,1,1,70147),-- Tallum Glaive Edge
(22216,305,1,1,1,247),-- Tallum Glaive
(22216,1879,3,9,2,157901),-- Cokes
(22216,1885,2,6,2,118426),-- High Grade Suede
(22216,9628,1,1,2,44584),-- Leonard
(22216,9630,1,1,2,29918),-- Orichalcum
(22216,9629,1,1,2,24715),-- Adamantine
(22216,9573,1,1,200,39475),-- Life Stone: level 80
(22216,9574,1,1,200,9869),-- Mid-Grade Life Stone: level 80
(22216,9575,1,1,200,987),-- High-Grade Life Stone: level 80
(22216,9576,1,1,200,198),-- Top-Grade Life Stone: level 80
(22216,6712,1,5,-1,657594),-- Sealed Major Arcana Gloves fabric
(22216,6901,1,1,-1,138558),-- Recipe: Shining Arrow (100%)
(22216,960,1,1,-1,55423),-- Scroll: Enchant Armor (Grade S)
(22217,57,68305,139448,0,700000),-- Adena
(22217,5541,1,1,1,70147),-- Tallum Glaive Edge
(22217,305,1,1,1,247),-- Tallum Glaive
(22217,1879,3,9,2,157901),-- Cokes
(22217,1885,2,6,2,118426),-- High Grade Suede
(22217,9628,1,1,2,44584),-- Leonard
(22217,9630,1,1,2,29918),-- Orichalcum
(22217,9629,1,1,2,24715),-- Adamantine
(22217,9573,1,1,200,39475),-- Life Stone: level 80
(22217,9574,1,1,200,9869),-- Mid-Grade Life Stone: level 80
(22217,9575,1,1,200,987),-- High-Grade Life Stone: level 80
(22217,9576,1,1,200,198),-- Top-Grade Life Stone: level 80
(22217,4043,1,9,-1,923722),-- Asofe
(22217,4042,1,5,-1,769768),-- Enria
(22217,6713,1,5,-1,657594),-- Sealed Major Arcana Boots Design
(22218,57,5469,11164,0,700000),-- Adena
(22218,1879,1,1,2,75850),-- Cokes
(22218,1885,1,1,2,37925),-- High Grade Suede
(22218,9628,1,1,2,3569),-- Leonard
(22218,9630,1,1,2,2395),-- Orichalcum
(22218,9629,1,1,2,1979),-- Adamantine
(22218,9573,1,1,200,3160),-- Life Stone: level 80
(22218,9574,1,1,200,790),-- Mid-Grade Life Stone: level 80
(22218,9575,1,1,200,79),-- High-Grade Life Stone: level 80
(22218,9576,1,1,200,16),-- Top-Grade Life Stone: level 80
(22218,5529,1,1,1,1872),-- Dragon Slayer Edge
(22218,5546,1,1,1,2056),-- Sword of Miracles Edge
(22218,5545,1,1,1,1872),-- Dark Legion\'s Edge Blade
(22218,81,1,1,1,7),-- Dragon Slayer
(22218,151,1,1,1,7),-- Sword of Miracles 
(22218,2500,1,1,1,7),-- Dark Legion\'s Edge
(22218,13099,1,1,-1,154703),-- Sealed Arcana Sickle Piece
(22218,6688,1,1,-1,23977),-- Forgotten Blade Edge
(22218,6901,1,1,-1,11093),-- Recipe: Shining Arrow (100%)
(22219,57,5492,11212,0,700000),-- Adena
(22219,1879,1,1,2,76173),-- Cokes
(22219,1885,1,1,2,38087),-- High Grade Suede
(22219,9628,1,1,2,3585),-- Leonard
(22219,9630,1,1,2,2406),-- Orichalcum
(22219,9629,1,1,2,1987),-- Adamantine
(22219,9573,1,1,200,3174),-- Life Stone: level 80
(22219,9574,1,1,200,793),-- Mid-Grade Life Stone: level 80
(22219,9575,1,1,200,79),-- High-Grade Life Stone: level 80
(22219,9576,1,1,200,16),-- Top-Grade Life Stone: level 80
(22219,8342,1,1,1,1410),-- Flaming Dragon Skull Piece
(22219,5533,1,1,1,1410),-- Elysian Head
(22219,5544,1,1,1,1409),-- Branch of The Mother Tree Head
(22219,8349,1,1,1,1410),-- Doom Crusher Head
(22219,164,1,1,1,5),-- Elysian
(22219,213,1,1,1,5),-- Branch of The Mother Tree
(22219,7895,1,1,1,5),-- Flaming Dragon Skull
(22219,7902,1,1,1,5),-- Doom Crusher
(22219,6694,1,1,-1,24516),-- Saint Spear Blade
(22219,6901,1,1,-1,11140),-- Recipe: Shining Arrow (100%)
(22219,960,1,1,-1,4456),-- Scroll: Enchant Armor (Grade S)
(22220,57,4894,9982,0,700000),-- Adena
(22220,1879,1,1,2,67866),-- Cokes
(22220,1885,1,1,2,33933),-- High Grade Suede
(22220,9628,1,1,2,3194),-- Leonard
(22220,9630,1,1,2,2143),-- Orichalcum
(22220,9629,1,1,2,1770),-- Adamantine
(22220,9573,1,1,200,2828),-- Life Stone: level 80
(22220,9574,1,1,200,707),-- Mid-Grade Life Stone: level 80
(22220,9575,1,1,200,71),-- High-Grade Life Stone: level 80
(22220,9576,1,1,200,14),-- Top-Grade Life Stone: level 80
(22220,5534,1,1,1,5030),-- Soul Bow Stave
(22220,289,1,1,1,18),-- Soul Bow
(22220,1875,1,1,-1,661693),-- Stone of Purity
(22220,6697,1,1,-1,21049),-- Arcana Mace Head
(22220,959,1,1,-1,397),-- Scroll: Enchant Weapon (Grade S)
(22221,57,5310,10842,0,700000),-- Adena
(22221,1879,1,1,2,73657),-- Cokes
(22221,1885,1,1,2,36829),-- High Grade Suede
(22221,9628,1,1,2,3466),-- Leonard
(22221,9630,1,1,2,2326),-- Orichalcum
(22221,9629,1,1,2,1921),-- Adamantine
(22221,9573,1,1,200,3069),-- Life Stone: level 80
(22221,9574,1,1,200,767),-- Mid-Grade Life Stone: level 80
(22221,9575,1,1,200,77),-- High-Grade Life Stone: level 80
(22221,9576,1,1,200,15),-- Top-Grade Life Stone: level 80
(22221,5537,1,1,1,5457),-- Soul Separator Head
(22221,236,1,1,1,19),-- Soul Separator
(22221,6703,1,1,-1,153376),-- Sealed Imperial Crusader Gauntlets Design
(22221,6901,1,1,-1,10772),-- Recipe: Shining Arrow (100%)
(22221,959,1,1,-1,431),-- Scroll: Enchant Weapon (Grade S)
(22222,57,5157,10528,0,700000),-- Adena
(22222,5538,1,1,1,5297),-- Dragon Grinder Edge
(22222,270,1,1,1,19),-- Dragon Grinder
(22222,8954,1,1,2,2207),-- Blue Primeval Crystal
(22222,8955,1,1,2,1104),-- Green Primeval Crystal
(22222,8956,1,1,2,552),-- Red Primeval Crystal
(22222,960,1,1,2,221),-- Scroll: Enchant Armor (Grade S)
(22222,959,1,1,2,22),-- Scroll: Enchant Weapon (Grade S)
(22222,8752,1,1,200,115),-- High-Grade Life Stone: level 76
(22222,8762,1,1,200,23),-- Top-Grade Life Stone: level 76
(22222,1875,1,1,-1,697420),-- Stone of Purity
(22222,6706,1,1,-1,169702),-- Sealed Imperial Crusader Helmet Pattern
(22222,960,1,1,-1,4185),-- Scroll: Enchant Armor (Grade S)
(22223,57,5707,11651,0,700000),-- Adena
(22223,5529,1,1,1,1872),-- Dragon Slayer Edge
(22223,5546,1,1,1,2056),-- Sword of Miracles Edge
(22223,5545,1,1,1,1872),-- Dark Legion\'s Edge Blade
(22223,81,1,1,1,7),-- Dragon Slayer
(22223,151,1,1,1,7),-- Sword of Miracles 
(22223,2500,1,1,1,7),-- Dark Legion\'s Edge
(22223,13099,1,1,-1,161444),-- Sealed Arcana Sickle Piece
(22223,6688,1,1,-1,25022),-- Forgotten Blade Edge
(22223,6901,1,1,-1,11576),-- Recipe: Shining Arrow (100%)
(22224,57,5638,11511,0,700000),-- Adena
(22224,959,1,1,2,34),-- Scroll: Enchant Weapon (Grade S)
(22224,960,1,1,1,338),-- Scroll: Enchant Armor (Grade S)
(22224,8955,1,1,2,1689),-- Green Primeval Crystal
(22224,8954,1,1,2,3378),-- Blue Primeval Crystal
(22224,8956,1,1,2,845),-- Red Primeval Crystal
(22224,8342,1,1,1,1448),-- Flaming Dragon Skull Piece
(22224,5533,1,1,1,1448),-- Elysian Head
(22224,5544,1,1,1,1447),-- Branch of The Mother Tree Head
(22224,8349,1,1,1,1448),-- Doom Crusher Head
(22224,164,1,1,1,5),-- Elysian
(22224,213,1,1,1,5),-- Branch of The Mother Tree
(22224,7895,1,1,1,5),-- Flaming Dragon Skull
(22224,7902,1,1,1,5),-- Doom Crusher
(22224,6694,1,1,-1,25169),-- Saint Spear Blade
(22224,6901,1,1,-1,11437),-- Recipe: Shining Arrow (100%)
(22224,960,1,1,-1,4575),-- Scroll: Enchant Armor (Grade S)
(22225,57,5357,10937,0,700000),-- Adena
(22225,1879,1,1,2,74304),-- Cokes
(22225,1885,1,1,2,37152),-- High Grade Suede
(22225,9628,1,1,2,3497),-- Leonard
(22225,9630,1,1,2,2346),-- Orichalcum
(22225,9629,1,1,2,1938),-- Adamantine
(22225,9573,1,1,200,3096),-- Life Stone: level 80
(22225,9574,1,1,200,774),-- Mid-Grade Life Stone: level 80
(22225,9575,1,1,200,77),-- High-Grade Life Stone: level 80
(22225,9576,1,1,200,16),-- Top-Grade Life Stone: level 80
(22225,5534,1,1,1,5508),-- Soul Bow Stave
(22225,289,1,1,1,19),-- Soul Bow
(22225,1875,1,1,-1,724469),-- Stone of Purity
(22225,6697,1,1,-1,23046),-- Arcana Mace Head
(22225,959,1,1,-1,435),-- Scroll: Enchant Weapon (Grade S)
(22226,57,5881,12007,0,700000),-- Adena
(22226,5537,1,1,1,6044),-- Soul Separator Head
(22226,236,1,1,1,21),-- Soul Separator
(22226,8954,1,1,2,2517),-- Blue Primeval Crystal
(22226,8955,1,1,2,1259),-- Green Primeval Crystal
(22226,8956,1,1,2,629),-- Red Primeval Crystal
(22226,960,1,1,2,252),-- Scroll: Enchant Armor (Grade S)
(22226,959,1,1,2,25),-- Scroll: Enchant Weapon (Grade S)
(22226,8752,1,1,200,131),-- High-Grade Life Stone: level 76
(22226,8762,1,1,200,26),-- Top-Grade Life Stone: level 76
(22226,6703,1,1,-1,169868),-- Sealed Imperial Crusader Gauntlets Design
(22226,6901,1,1,-1,11931),-- Recipe: Shining Arrow (100%)
(22226,959,1,1,-1,477),-- Scroll: Enchant Weapon (Grade S)
(22227,57,5800,11842,0,700000),-- Adena
(22227,5538,1,1,1,5958),-- Dragon Grinder Edge
(22227,270,1,1,1,21),-- Dragon Grinder
(22227,8954,1,1,2,3475),-- Blue Primeval Crystal
(22227,8955,1,1,2,1738),-- Green Primeval Crystal
(22227,8956,1,1,2,869),-- Red Primeval Crystal
(22227,960,1,1,2,348),-- Scroll: Enchant Armor (Grade S)
(22227,959,1,1,2,35),-- Scroll: Enchant Weapon (Grade S)
(22227,1875,1,1,-1,784408),-- Stone of Purity
(22227,6706,1,1,-1,190869),-- Sealed Imperial Crusader Helmet Pattern
(22227,960,1,1,-1,4706),-- Scroll: Enchant Armor (Grade S)
(22228,57,8,12,0,700000),-- Adena
(22228,116,1,1,1,27027),-- Magic Ring
(22228,112,1,1,1,23255),-- Apprentice\'s Earring
(22228,118,1,1,1,15384),-- Necklace of Magic
(22228,1864,1,1,2,6993),-- Stem
(22228,1868,1,1,2,3984),-- Thread
(22228,1666,1,1,2,3984),-- Recipe: Wooden Arrow
(22228,1865,1,1,2,2994),-- Varnish
(22228,1866,1,1,2,1996),-- Suede
(22228,1873,1,1,2,997),-- Silver Nugget
(22229,57,8,12,0,700000),-- Adena
(22229,17,10,30,2,20000),-- Wooden Arrow
(22229,28,1,1,1,19230),-- Pants
(22229,21,1,1,1,9708),-- Shirt
(22229,1866,1,1,2,3267),-- Suede
(22229,1786,1,1,2,3194),-- Recipe: Broad Sword
(22229,1871,1,1,2,2544),-- Charcoal
(22230,57,8,13,0,700000),-- Adena
(22230,18,1,1,1,50000),-- Leather Shield
(22230,41,1,1,1,34482),-- Cloth Cap
(22230,1870,1,1,2,6536),-- Coal
(22230,1865,1,1,2,5988),-- Varnish
(22230,1787,1,1,2,5181),-- Recipe: Willow Staff
(22231,57,13,20,0,700000),-- Adena
(22231,1060,1,1,2,14285),-- Lesser Healing Potion
(22231,13,1,1,1,9174),-- Short Bow
(22231,1871,1,1,2,6622),-- Charcoal
(22231,1788,1,1,2,5154),-- Recipe: Bow
(22231,1866,1,1,2,4608),-- Suede
(22232,57,19,29,0,700000),-- Adena
(22232,28,1,1,1,45454),-- Pants
(22232,21,1,1,1,28571),-- Shirt
(22232,1864,1,1,2,25000),-- Stem
(22232,1869,1,1,2,11494),-- Iron Ore
(22232,1666,1,1,2,3333),-- Recipe: Wooden Arrow
(22233,57,30,42,0,700000),-- Adena
(22233,1865,1,1,2,19607),-- Varnish
(22233,1870,1,1,2,18867),-- Coal
(22233,1786,1,1,2,18518),-- Recipe: Broad Sword
(22233,13,1,1,1,15873),-- Short Bow
(22234,57,41,58,0,700000),-- Adena
(22234,1871,1,1,2,16129),-- Charcoal
(22234,1866,1,1,2,9804),-- Suede
(22234,1789,1,1,2,3105),-- Recipe: Cedar Staff
(22234,29,1,1,1,2506),-- Leather Pants
(22234,22,1,1,1,1603),-- Leather Shirt
(22234,2005,1,1,-1,5553),-- Broadsword Blade
(22235,57,51,71,0,700000),-- Adena
(22235,17,10,30,2,71428),-- Wooden Arrow
(22235,1871,1,1,2,15151),-- Charcoal
(22235,1866,1,1,2,9615),-- Suede
(22235,1790,1,1,2,2631),-- Recipe: Dirk
(22235,2007,1,1,1,2114),-- Bow Shaft
(22235,14,1,1,1,482),-- Bow
(22235,1060,1,1,-1,36285),-- Lesser Healing Potion
(22235,1877,1,1,-1,5414),-- Adamantite Nugget
(22236,57,66,90,0,700000),-- Adena
(22236,1872,1,1,2,33333),-- Animal Bone
(22236,1867,1,1,2,31250),-- Animal Skin
(22236,1791,1,1,2,4762),-- Recipe: Brandish
(22236,2007,1,1,1,3247),-- Bow Shaft
(22236,14,1,1,1,433),-- Bow
(22236,2005,1,1,-1,15553),-- Broadsword Blade
(22236,1790,1,1,-1,12631),-- Recipe: Dirk
(22236,2007,1,1,-1,12114),-- Bow Shaft
(22237,57,79,107,0,700000),-- Adena
(22237,1795,1,1,2,83333),-- Recipe:Leather Shoes
(22237,1060,1,1,2,45454),-- Lesser Healing Potion
(22237,1865,1,1,2,23255),-- Varnish
(22237,1870,1,1,2,19607),-- Coal
(22237,2005,1,1,1,1915),-- Broadsword Blade
(22237,3,1,1,1,958),-- Broadsword
(22237,1865,1,1,-1,129607),-- Varnish
(22238,57,95,129,0,700000),-- Adena
(22238,1796,1,1,2,41666),-- Recipe: Leather Tunic
(22238,1870,1,1,2,35714),-- Coal
(22238,1865,1,1,2,31250),-- Varnish
(22238,2006,1,1,2,2164),-- Willow Staff Head
(22238,8,1,1,1,786),-- Willow Staff
(22238,152,1,1,1,589),-- Heavy Chisel
(22238,5,1,1,1,197),-- Mace
(22238,1060,1,1,-1,101285),-- Lesser Healing Potion
(22238,4039,1,1,-1,3441),-- Mold Glue
(22238,2007,1,1,-1,12114),-- Bow Shaft
(22239,57,108,145,0,700000),-- Adena
(22239,1872,1,1,2,55555),-- Animal Bone
(22239,1867,1,1,2,52631),-- Animal Skin
(22239,736,1,1,2,16666),-- Scroll of Escape
(22239,2010,1,1,1,1658),-- Brandish Blade
(22239,1333,1,1,1,1106),-- Brandish
(22239,1896,1,1,-1,92812),-- Necklace of Anguish Chain
(22240,57,110,152,0,700000),-- Adena
(22240,1864,1,1,2,76923),-- Stem
(22240,1869,1,1,2,38461),-- Iron Ore
(22240,1792,1,1,2,3003),-- Recipe: Short Spear
(22240,2008,1,1,1,1689),-- Cedar Staff Head
(22240,9,1,1,1,241),-- Cedar Staff
(22240,154,1,1,1,161),-- Dwarven Mace
(22240,1869,1,2,-1,11494),-- Iron Ore
(22240,1921,1,1,-1,53911),-- Leather Shoes Texture
(22240,1878,1,1,-1,33261),-- Braided Hem
(22241,57,138,196,0,700000),-- Adena
(22241,1871,1,1,2,52631),-- Charcoal
(22241,1866,1,1,2,35714),-- Suede
(22241,1793,1,1,1,3448),-- Recipe: Sword of Reflection
(22241,271,1,1,1,441),-- Hunting Bow
(22241,1831,1,1,-1,668170),-- Antidote
(22241,1922,1,1,-1,59085),-- Leather Tunic Pattern
(22241,1794,1,1,-1,17030),-- Recipe: Forest Bow
(22242,57,112,166,0,700000),-- Adena
(22242,1831,1,1,2,90909),-- Antidote
(22242,1865,1,1,2,33333),-- Varnish
(22242,1870,1,1,2,31250),-- Coal
(22242,1794,1,1,2,2427),-- Recipe: Forest Bow
(22242,2010,1,1,1,1567),-- Brandish Blade
(22242,66,1,1,1,207),-- Gladius
(22242,1333,1,1,1,118),-- Brandish
(22242,1798,1,1,-1,30882),-- Recipe: Leather Helmet
(22242,1923,1,1,-1,12202),-- Leather Stocking Pattern
(22242,2008,1,1,-1,9411),-- Cedar Staff Head
(22243,57,115,175,0,700000),-- Adena
(22243,1868,1,1,2,90909),-- Thread
(22243,1798,1,1,2,47619),-- Recipe: Leather Helmet
(22243,1873,1,1,2,18867),-- Silver Nugget
(22243,2386,1,1,1,1788),-- Wooden Gaiters
(22243,30,1,1,1,1368),-- Hard Leather Pant
(22243,23,1,1,1,789),-- Wooden Breastplate
(22243,1833,1,2,-1,502269),-- Bandage
(22243,1868,1,2,-1,371441),-- Thread
(22243,1873,1,1,-1,84381),-- Silver Nugget
(22244,57,116,181,0,700000),-- Adena
(22244,1870,1,1,2,45454),-- Coal
(22244,1865,1,1,2,43478),-- Varnish
(22244,1925,1,1,1,28571),-- Leather Gloves Lining
(22244,1800,1,1,2,13888),-- Recipe: Piece Bone Breastplate
(22244,50,1,1,1,1056),-- Leather Gloves
(22244,38,1,1,1,704),-- Low Boots
(22244,1897,1,1,-1,91243),-- Necklace of Wisdom Chain
(22244,1800,1,1,-1,76634),-- Recipe: Piece Bone Breastplate
(22244,1924,1,1,-1,76634),-- Leather Helmet Design
(22245,57,119,190,0,700000),-- Adena
(22245,1865,1,1,2,50000),-- Varnish
(22245,1870,1,1,2,43478),-- Coal
(22245,1801,1,1,2,24390),-- Recipe: Piece Bone Gaiters
(22245,2501,1,1,1,105),-- Bone Club
(22245,1870,1,1,-1,135721),-- Coal
(22245,1925,1,1,-1,91252),-- Leather Gloves Lining
(22245,1801,1,1,-1,119680),-- Recipe: Piece Bone Gaiters
(22246,57,120,198,0,700000),-- Adena
(22246,1870,1,1,2,50000),-- Coal
(22246,1865,1,1,2,47619),-- Varnish
(22246,1924,1,1,1,9174),-- Leather Helmet Design
(22246,1817,1,1,2,7575),-- Recipe: Cord
(22246,945,1,1,1,1897),-- Skeleton Buckler
(22246,102,1,1,1,1579),-- Round Shield
(22246,44,1,1,1,316),-- Leather Helmet
(22246,1865,1,2,-1,253000),-- Varnish
(22246,734,1,1,-1,43053),-- Haste Potion
(22257,57,1771,3615,0,700000),-- Adena
(22257,1868,1,1,2,236365),-- Thread
(22257,9593,1,1,100,200000),-- Yellow Seed Of Evil - Shard
(22257,1873,1,1,2,47273),-- Silver Nugget
(22257,1895,1,1,2,33766),-- Metallic Fiber
(22257,1876,1,1,2,23637),-- Mithril Ore
(22257,4042,1,1,2,1970),-- Enria
(22257,9546,1,1,200,945),-- Fire Stone
(22257,9573,1,1,200,821),-- Life Stone: level 80
(22257,9574,1,1,200,205),-- Mid-Grade Life Stone: level 80
(22257,9575,1,1,200,20),-- High-Grade Life Stone: level 80
(22257,9576,1,1,200,4),-- Top-Grade Life Stone: level 80
(22257,5509,1,1,1,3810),-- Sealed Tallum Gloves Design
(22257,5497,1,1,1,3810),-- Sealed Tallum Boots Lining
(22257,5508,1,1,1,3810),-- Sealed Dark Crystal Gloves Design
(22257,5496,1,1,1,3810),-- Sealed Dark Crystal Boots Lining
(22257,12229,1,1,1,708),-- Common Item - Sealed Tallum Gloves
(22257,12231,1,1,1,708),-- Common Item - Sealed Tallum Boots
(22257,12221,1,1,1,708),-- Common Item - Sealed Dark Crystal Gloves
(22257,12224,1,1,1,708),-- Common Item - Sealed Dark Crystal Boots
(22257,1808,1,1,2,52),-- Recipe: Soulshot: S Grade
(22257,1885,1,1,-1,15799),-- Sealed Majestic Leather Armor Fabric
(22257,5503,1,1,-1,15799),-- Sealed Leather Armor of Nightmare Fabric
(22257,1808,1,1,-1,1565),-- Recipe: Soulshot: S Grade
(22258,57,1821,3717,0,700000),-- Adena
(22258,9593,1,1,100,200000),-- Yellow Seed Of Evil - Shard
(22258,1894,1,1,2,7428),-- Crafted Leather
(22258,4039,1,1,2,7819),-- Mold Glue
(22258,4041,1,1,2,1938),-- Mold Hardener
(22258,4044,1,1,2,7428),-- Thons
(22258,9546,1,1,200,1783),-- Fire Stone
(22258,959,1,1,2,9),-- Scroll: Enchant Weapon (Grade S)
(22258,5545,1,1,1,611),-- Dark Legion's Edge Blade
(22258,5546,1,1,1,671),-- Sword of Miracles Edge
(22258,5529,1,1,1,611),-- Dragon Slayer Edge
(22258,11964,1,1,1,40),-- Common Item - Dark Legion's Edge
(22258,11971,1,1,1,40),-- Common Item - Sword of Miracles 
(22258,11968,1,1,1,40),-- Common Item - Dragon Slayer
(22258,1894,1,1,-1,95236),-- Crafted Leather
(22258,5487,1,1,-1,12222),-- Sealed Nightmare Robe Fabric
(22258,3036,1,1,-1,1206),-- Recipe: Spiritshot: S Grade
(22259,57,1988,4059,0,700000),-- Adena
(22259,9593,1,1,100,250000),-- Yellow Seed Of Evil - Shard
(22259,1871,1,1,2,182499),-- Charcoal
(22259,1866,1,1,2,121666),-- Suede
(22259,1344,20,60,2,114062),-- Mithril Arrow
(22259,1877,1,1,2,7300),-- Adamantite Nugget
(22259,4040,1,1,2,3650),-- Mold Lubricant
(22259,4041,1,1,2,1587),-- Mold Hardener
(22259,9546,1,1,200,1460),-- Fire Stone
(22259,5544,1,1,1,500),-- Branch of The Mother Tree Head
(22259,5533,1,1,1,501),-- Elysian Head
(22259,8342,1,1,1,501),-- Flaming Dragon Skull Piece
(22259,8349,1,1,1,501),-- Doom Crusher Head
(22259,11970,1,1,1,32),-- Common Item - Branch of The Mother Tree
(22259,11974,1,1,1,32),-- Common Item - Elysian
(22259,11969,1,1,1,32),-- Common Item - Flaming Dragon Skull
(22259,11966,1,1,1,32),-- Common Item - Doom Crusher
(22259,960,1,1,2,73),-- Scroll: Enchant Armor (Grade S)
(22259,4040,1,1,-1,59275),-- Mold Lubricant
(22259,5488,1,1,-1,13346),-- Sealed Majestic Robe Fabric
(22259,3957,1,1,-1,1317),-- Recipe: Blessed Spiritshot S
(22260,57,1647,3363,0,700000),-- Adena
(22260,9593,1,1,100,250000),-- Yellow Seed Of Evil - Shard
(22260,10557,1,1,2,20853),-- Forgotten Scroll - Song of Windstorm
(22260,5522,1,1,1,1030),-- Sealed Armor of Nightmare Pattern
(22260,5523,1,1,1,1030),-- Sealed Majestic Plate Armor Pattern
(22260,12263,1,1,1,127),-- Common Item - Sealed Armor of Nightmare
(22260,12262,1,1,1,127),-- Common Item - Sealed Majestic Plate Armor
(22260,5502,1,1,-1,41467),-- Sealed Boots of Nightmare Lining
(22260,5495,1,1,-1,37283),-- Sealed Shield of Nightmare Fragment
(22260,5157,1,1,-1,1091),-- Recipe: Soulshot (S) Compressed Package (100%)
(22261,57,1696,3473,0,700000),-- Adena
(22261,9594,1,1,100,125000),-- Green Seed of Evil - Shard
(22261,5534,1,1,1,1713),-- Soul Bow Stave
(22261,11975,1,1,1,111),-- Common Item - Soul Bow
(22261,10558,1,1,2,21514),-- Forgotten Scroll - Dance of Blade Storm
(22261,1885,1,1,-1,211091),-- High Grade Suede
(22261,5503,1,1,-1,42781),-- Sealed Majestic Boots Lining
(22261,4041,1,1,-1,22027),-- Mold Hardener
(22262,57,1717,3516,0,700000),-- Adena
(22262,9594,1,1,100,111111),-- Green Seed of Evil - Shard
(22262,1895,1,1,2,36095),-- Metallic Fiber
(22262,4040,1,1,2,2527),-- Mold Lubricant
(22262,9628,1,1,2,991),-- Leonard
(22262,9630,1,1,2,665),-- Orichalcum
(22262,9546,1,1,200,1011),-- Fire Stone
(22262,8349,1,1,1,433),-- Doom Crusher Head
(22262,5533,1,1,1,433),-- Elysian Head
(22262,5544,1,1,1,433),-- Branch Of The Mother Tree Head
(22262,8342,1,1,1,433),-- Flaming Dragon Skull Piece
(22262,9573,1,1,200,877),-- Life Stone: level 80
(22262,9574,1,1,200,219),-- Mid-Grade Life Stone: level 80
(22262,9575,1,1,200,22),-- High-Grade Life Stone: level 80
(22262,9576,1,1,200,4),-- Top-Grade Life Stone: level 80
(22262,960,1,1,2,51),-- Scroll: Enchant Armor (Grade S)
(22262,11970,1,1,1,28),-- Common Item - Branch of The Mother Tree
(22262,11974,1,1,1,28),-- Common Item - Elysian
(22262,11969,1,1,1,28),-- Common Item - Flaming Dragon Skull
(22262,11966,1,1,1,28),-- Common Item - Doom Crusher
(22262,5514,1,1,-1,43313),-- Sealed Gauntlets Of Nightmare Design
(22262,5162,1,1,-1,1140),-- Recipe: Spiritshot (S) Compressed Package (100%)
(22262,5167,1,1,-1,1140),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(22263,57,1978,4052,0,700000),-- Adena
(22263,5538,1,1,1,1997),-- Dragon Grinder Edge
(22263,9594,1,1,100,142857),-- Green Seed of Evil - Shard
(22263,1895,1,1,2,41593),-- Metallic Fiber
(22263,4040,1,1,2,2911),-- Mold Lubricant
(22263,9628,1,1,2,1142),-- Leonard
(22263,9630,1,1,2,766),-- Orichalcum
(22263,9546,1,1,200,1165),-- Fire Stone
(22263,9573,1,1,200,1011),-- Life Stone: level 80
(22263,9574,1,1,200,253),-- Mid-Grade Life Stone: level 80
(22263,9575,1,1,200,25),-- High-Grade Life Stone: level 80
(22263,9576,1,1,200,5),-- Top-Grade Life Stone: level 80
(22263,960,1,1,2,58),-- Scroll: Enchant Armor (Grade S)
(22263,11967,1,1,1,129),-- Common Item - Dragon Grinder
(22263,1895,1,1,-1,844332),-- Metallic Fiber
(22263,5515,1,1,-1,59103),-- Sealed Majestic Gauntlets Design
(22263,4040,1,1,-1,49910),-- Mold Lubricant
(22264,57,1759,3602,0,700000),-- Adena
(22264,1868,1,1,2,369713),-- Thread
(22264,9594,1,1,100,125000),-- Green Seed of Evil - Shard
(22264,1873,1,1,2,73943),-- Silver Nugget
(22264,5549,1,1,2,18486),-- Metallic Thread
(22264,1889,1,1,2,12324),-- Compound Braid
(22264,4044,1,1,2,6162),-- Thons
(22264,9546,1,1,200,1479),-- Fire Stone
(22264,959,1,1,2,7),-- Scroll: Enchant Weapon (Grade S)
(22264,5534,1,1,1,1776),-- Soul Bow Stave
(22264,11975,1,1,1,115),-- Common Item - Soul Bow
(22264,5522,1,1,-1,8946),-- Sealed Armor of Nightmare Pattern
(22264,5523,1,1,-1,8946),-- Sealed Majestic Plate Armor Pattern
(22264,5272,1,1,-1,1167),-- Recipe: Greater Soulshot (S) Compressed Package (100%)
(22265,57,1778,3566,0,700000),-- Adena
(22265,9596,1,1,100,58824),-- Red Seed of Evil - Shard
(22265,1895,1,1,2,42755),-- Metallic Fiber
(22265,4040,1,1,2,2993),-- Mold Lubricant
(22265,9628,1,1,2,1174),-- Leonard
(22265,9630,1,1,2,788),-- Orichalcum
(22265,9546,1,1,200,1197),-- Fire Stone
(22265,9573,1,1,200,1039),-- Life Stone: level 80
(22265,9574,1,1,200,260),-- Mid-Grade Life Stone: level 80
(22265,9575,1,1,200,26),-- High-Grade Life Stone: level 80
(22265,9576,1,1,200,5),-- Top-Grade Life Stone: level 80
(22265,960,1,1,2,60),-- Scroll: Enchant Armor (Grade S)
(22265,5537,1,1,1,2053),-- Soul Separator Head
(22265,11973,1,1,1,133),-- Common Item - Soul Separator
(22265,6901,1,1,-1,3038),-- Recipe: Shining Arrow (100%)
(22265,6688,1,1,-1,6566),-- Forgotten Blade Edge
(22265,959,1,1,-1,122),-- Scroll: Enchant Weapon (Grade S)
(22266,57,1584,3176,0,700000),-- Adena
(22266,1344,10,30,2,238056),-- Mithril Arrow
(22266,9596,1,1,100,50000),-- Red Seed of Evil - Shard
(22266,1895,1,1,2,54413),-- Metallic Fiber
(22266,4040,1,1,2,3809),-- Mold Lubricant
(22266,9628,1,1,2,1494),-- Leonard
(22266,9630,1,1,2,1002),-- Orichalcum
(22266,9546,1,1,200,1523),-- Fire Stone
(22266,5537,1,1,1,1829),-- Soul Separator Head
(22266,11973,1,1,1,118),-- Common Item - Soul Separator
(22266,6901,1,1,2,191),-- Recipe: Shining Arrow (100%)
(22266,1895,1,1,-1,773207),-- Metallic Fiber
(22266,6689,1,1,-1,5739),-- Basalt Battlehammer Head
(22266,4040,1,1,-1,54124),-- Mold Lubricant
(22267,57,24706,49558,0,700000),-- Adena
(22267,9596,1,1,100,800000),-- Red Seed of Evil - Shard
(22267,1895,3,9,2,198804),-- Metallic Fiber
(22267,4040,1,1,2,83195),-- Mold Lubricant
(22267,9628,1,1,2,32625),-- Leonard
(22267,9630,1,1,2,21893),-- Orichalcum
(22267,5541,1,1,1,28519),-- Tallum Glaive Edge
(22267,11976,1,1,1,1846),-- Common Item - Tallum Glaive
(22267,959,1,1,2,166),-- Scroll: Enchant Weapon (Grade S)
(22267,6690,1,1,-1,91435),-- Imperial Staff Head
(22267,6901,1,1,-1,42222),-- Recipe: Shining Arrow (100%)
(22268,57,1377,2821,0,700000),-- Adena
(22268,1344,10,30,2,158318),-- Mithril Arrow
(22268,1868,1,3,2,126665),-- Thread
(22268,9595,1,1,100,100000),-- Blue Seed of Evil - Shard
(22268,1873,1,1,2,50662),-- Silver Nugget
(22268,1895,1,1,2,36187),-- Metallic Fiber
(22268,1876,1,1,2,253319),-- Mithril Ore
(22268,4042,1,1,2,2111),-- Enria
(22268,9548,1,1,200,1013),-- Earth Stone
(22268,5480,1,1,1,1137),-- Sealed Leather Armor of Nightmare Fabric
(22268,5481,1,1,1,1137),-- Sealed Majestic Leather Armor Fabric
(22268,960,1,1,2,51),-- Scroll: Enchant Armor (S)
(22268,12265,1,1,1,141),-- Common Item - Sealed Leather Armor of Nightmare
(22268,12258,1,1,1,141),-- Common Item - Sealed Majestic Leather Armor
(22268,1895,1,1,-1,587678),-- Metallic Fiber
(22268,1876,1,1,-1,411375),-- Mithril Ore
(22268,5480,1,1,-1,9230),-- Sealed Leather Armor of Nightmare Fabric
(22269,57,1391,2849,0,700000),-- Adena
(22269,1872,1,1,2,194914),-- Animal Bone
(22269,1867,1,1,2,194914),-- Animal Skin
(22269,9595,1,1,100,100000),-- Blue Seed of Evil - Shard
(22269,1881,1,1,2,19491),-- Coarse Bone Powder
(22269,1875,1,1,2,9746),-- Stone of Purity
(22269,4041,1,1,2,1271),-- Mold Hardener
(22269,9548,1,1,200,1169),-- Earth Stone
(22269,5522,1,1,1,871),-- Sealed Armor of Nightmare Pattern
(22269,5523,1,1,1,871),-- Sealed Majestic Plate Armor Pattern
(22269,6901,1,1,2,146),-- Recipe: Shining Arrow (100%) (S)
(22269,12261,1,1,1,107),-- Common Item - Sealed Armor of Nightmare
(22269,12262,1,1,1,107),-- Common Item - Sealed Majestic Plate Armor
(22269,1808,1,1,-1,923),-- Recipe: Soulshot: S Grade
(22269,5481,1,1,-1,9321),-- Sealed Majestic Leather Armor Fabric
(22269,5487,1,1,-1,9354),-- Sealed Nightmare Robe Fabric
(22270,9689,1,1,101,1000000),-- Spirit of the Lake
(22270,57,1537,3147,0,700000),-- Adena
(22270,9595,1,1,100,100000),-- Blue Seed of Evil - Shard
(22270,1344,10,30,2,117768),-- Mithril Arrow
(22270,1868,1,1,2,188428),-- Thread
(22270,1873,1,1,2,37686),-- Silver Nugget
(22270,5549,1,1,2,9421),-- Metallic Thread
(22270,1889,1,1,2,6281),-- Compound Braid
(22270,4044,1,1,2,3140),-- Thons
(22270,9549,1,1,200,754),-- Wind Stone
(22270,5487,1,1,1,1273),-- Sealed Robe of Nightmare Fabric
(22270,5488,1,1,1,1273),-- Sealed Majestic Robe Fabric
(22270,12266,1,1,1,158),-- Common Item - Sealed Robe of Nightmare
(22270,12259,1,1,1,158),-- Common Item - Sealed Majestic Robe
(22270,959,1,1,2,4),-- Scroll: Enchant Weapon (Grade S)
(22270,9573,1,1,200,654),-- Life Stone: level 80
(22270,9574,1,1,200,164),-- Mid-Grade Life Stone: level 80
(22270,9575,1,1,200,16),-- High-Grade Life Stone: level 80
(22270,9576,1,1,200,3),-- Top-Grade Life Stone: level 80
(22270,4044,1,1,-1,76502),-- Thons
(22270,3036,1,1,-1,1020),-- Recipe: Spiritshot: S Grade
(22270,5488,1,1,-1,10335),-- Sealed Majestic Robe Fabric
(22271,9689,1,1,101,1000000),-- Spirit of the Lake
(22271,57,1537,3147,0,700000),-- Adena
(22271,9595,1,1,100,100000),-- Blue Seed of Evil - Shard
(22271,1895,1,1,2,53837),-- Metallic Fiber
(22271,4040,1,1,2,3769),-- Mold Lubricant
(22271,9628,1,1,2,1478),-- Leonard
(22271,9630,1,1,2,992),-- Orichalcum
(22271,9549,1,1,200,1507),-- Wind Stone
(22271,6346,1,1,1,6135),-- Sealed Majestic Ring Gemstone
(22271,6342,1,1,1,4268),-- Sealed Majestic Earring Gemstone
(22271,6344,1,1,1,2762),-- Sealed Majestic Necklace Beads
(22271,12350,1,1,1,348),-- Common Item - Sealed Majestic Necklace
(22271,12352,1,1,1,464),-- Common Item - Sealed Majestic Earring
(22271,12351,1,1,1,695),-- Common Item - Sealed Majestic Ring
(22271,960,1,1,2,75),-- Scroll: Enchant Armor (S)
(22271,1895,1,1,-1,655731),-- Metallic Fiber
(22271,5495,1,1,-1,34850),-- Sealed Shield of Nightmare Fragment
(22271,3957,1,1,-1,1020),-- Recipe: Blessed Spiritshot (S)
(22272,57,2190,4484,0,700000),-- Adena
(22272,1344,30,90,2,90909),-- Mithril Arrow
(22272,1868,4,12,2,66667),-- Thread
(22272,1873,1,1,2,100000),-- Silver Nugget
(22272,1876,1,1,2,40000),-- Mithril Ore
(22272,1895,1,1,2,62500),-- Metallic Fiber
(22272,4042,1,1,2,3731),-- Enria
(22272,6346,1,1,1,7463),-- Sealed Majestic Ring Gemstone
(22272,6342,1,1,1,7194),-- Sealed Majestic Earring Gemstone
(22272,6344,1,1,1,3448),-- Sealed Majestic Necklace Beads
(22272,12351,1,1,1,287),-- Common Item - Sealed Majestic Ring
(22272,6901,1,1,2,287),-- Recipe: Shining Arrow (100%) (S)
(22272,5157,1,1,-1,10000),-- Recipe: Soulshot (S) Compressed Package (100%)
(22272,5503,1,1,-1,130000),-- Sealed Majestic Boots Lining
(22272,5502,1,1,-1,90000),-- Sealed Boots of Nightmare Lining
(22273,57,2117,4336,0,700000),-- Adena
(22273,4040,1,1,2,3774),-- Mold Lubricant
(22273,9628,1,1,2,1350),-- Leonard
(22273,9630,1,1,2,1079),-- Orichalcum
(22273,9573,1,1,200,539),-- Life Stone: level 80
(22273,5495,1,1,1,4049),-- Sealed Shield of Nightmare Fragment
(22273,5527,1,1,1,3774),-- Sealed Helm of Nightmare Design
(22273,5528,1,1,1,1618),-- Sealed Majestic Circlet Design
(22273,12268,1,1,1,270),-- Common Item - Sealed Shield of Nightmare
(22273,1895,1,3,-1,390000),-- Metallic Fiber
(22273,4040,1,1,-1,40000),-- Mold Lubricant
(22273,5514,1,1,-1,60000),-- Sealed Gloves of Nightmare Design
(22274,57,2161,4425,0,700000),-- Adena
(22274,1344,20,60,2,142857),-- Mithril Arrow
(22274,1866,1,1,2,166667),-- Suede
(22274,1871,1,3,2,76923),-- Charcoal
(22274,4041,1,1,2,1631),-- Mold Hardener
(22274,1877,1,1,2,7092),-- Adamantite Nugget
(22274,4040,1,1,2,4082),-- Mold Lubricant
(22274,5495,1,1,1,4630),-- Sealed Shield of Nightmare Fragment
(22274,5527,1,1,1,2174),-- Sealed Helm of Nightmare Design
(22274,5528,1,1,1,2717),-- Sealed Majestic Circlet Design
(22274,960,1,1,2,272),-- Scroll: Enchant Armor (S)
(22274,5167,1,1,-1,10000),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(22274,5515,1,1,-1,110000),-- Sealed Majestic Gloves Design
(22275,57,12348,25286,0,700000),-- Adena
(22275,1895,1,3,2,216303),-- Metallic Fiber
(22275,4043,1,1,2,50471),-- Asofe
(22275,4040,1,1,2,30282),-- Mold Lubricant
(22275,4042,1,1,2,25235),-- Enria
(22275,9546,1,1,200,12113),-- Fire Stone
(22275,8342,1,1,1,3115),-- Flaming Dragon Skull Piece
(22275,5533,1,1,1,3115),-- Elysian Head
(22275,8349,1,1,1,3114),-- Doom Crusher Head
(22275,5544,1,1,1,3112),-- Branch of the Mother Tree Head
(22275,6901,1,1,2,1514),-- Recipe: Shining Arrow (100%)
(22275,7902,1,1,1,11),-- Doom Crusher
(22275,7895,1,1,1,11),-- Flaming Dragon Skull
(22275,213,1,1,1,11),-- Branch of the Mother Tree
(22275,164,1,1,1,11),-- Elysian
(22275,1895,1,11,-1,878191),-- Metallic Fiber
(22275,4040,1,1,-1,368840),-- Mold Lubricant
(22275,5522,1,1,-1,62805),-- Sealed Armor of Nightmare Pattern
(22276,57,3787,7755,0,700000),-- Adena
(22276,1344,10,30,2,316635),-- Mithril Arrow
(22276,1895,1,1,2,72373),-- Metallic Fiber
(22276,4040,1,1,2,5066),-- Mold Lubricant
(22276,9546,1,1,200,2026),-- Fire Stone
(22276,9628,1,1,2,1987),-- Leonard
(22276,9573,1,1,200,1759),-- Life Stone: level 80
(22276,9630,1,1,2,1333),-- Orichalcum
(22276,8342,1,1,1,955),-- Flaming Dragon Skull Piece
(22276,8349,1,1,1,955),-- Doom Crusher Head
(22276,5533,1,1,1,955),-- Elysian Head
(22276,5544,1,1,1,954),-- Branch of the Mother Tree Head
(22276,9574,1,1,200,440),-- Mid-Grade Life Stone: level 80
(22276,9575,1,1,200,44),-- High-Grade Life Stone: level 80
(22276,959,1,1,2,10),-- Scroll: Enchant Weapon (S)
(22276,9576,1,1,200,9),-- Top-Grade Life Stone: level 80
(22276,7902,1,1,1,3),-- Doom Crusher
(22276,7895,1,1,1,3),-- Flaming Dragon Skull
(22276,213,1,1,1,3),-- Branch of the Mother Tree
(22276,164,1,1,1,3),-- Elysian
(22276,5527,1,1,-1,67627),-- Sealed Helm of Nightmare Design
(22276,5523,1,1,-1,19263),-- Sealed Majestic Plate Armor Pattern
(22276,5272,1,1,-1,2514),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(22277,57,12348,25286,0,700000),-- Adena
(22277,1873,2,6,2,129782),-- Silver Nugget
(22277,1868,15,45,2,86521),-- Thread
(22277,1895,3,9,2,61801),-- Metallic Fiber
(22277,1876,3,9,2,37081),-- Mithril Ore
(22277,4042,1,1,2,21630),-- Enria
(22277,5534,1,1,1,12471),-- Soul Bow Stave
(22277,9547,1,1,200,10382),-- Water Stone
(22277,960,1,1,2,519),-- Scroll: Enchant Armor (S)
(22277,289,1,1,1,44),-- Soul Bow
(22277,4042,1,1,-1,307367),-- Enria
(22277,5528,1,1,-1,220493),-- Sealed Majestic Circlet Design
(22277,5277,1,1,-1,8196),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(22278,57,3787,7755,0,700000),-- Adena
(22278,1344,20,60,2,248786),-- Mithril Arrow
(22278,1894,1,1,2,13967),-- Crafted Leather
(22278,4039,1,1,2,13269),-- Mold Glue
(22278,4044,1,1,2,13269),-- Thons
(22278,5534,1,1,1,3825),-- Soul Bow Stave
(22278,4041,1,1,2,3461),-- Mold Hardener
(22278,9547,1,1,200,3184),-- Water Stone
(22278,6901,1,1,2,398),-- Recipe: Shining Arrow (100%)
(22278,289,1,1,1,13),-- Soul Bow
(22278,4039,1,1,-1,188545),-- Mold Glue
(22278,5529,1,1,-1,15518),-- Dragon Slayer Edge
(22278,5282,1,1,-1,2514),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(22279,57,3707,7592,0,700000),-- Adena
(22279,1866,1,1,2,259769),-- Suede
(22279,1871,1,3,2,194827),-- Charcoal
(22279,1877,1,1,2,15586),-- Adamantite Nugget
(22279,4040,1,1,2,7793),-- Mold Lubricant
(22279,4041,1,1,2,3388),-- Mold Hardener
(22279,9546,1,1,200,3118),-- Fire Stone
(22279,5546,1,1,1,1369),-- Sword of Miracles Edge
(22279,5529,1,1,1,1247),-- Dragon Slayer Edge
(22279,5545,1,1,1,1247),-- Dark Legion\'s Edge Blade
(22279,959,1,1,2,16),-- Scroll: Enchant Weapon (S)
(22279,151,1,1,1,4),-- Sword of Miracles
(22279,2500,1,1,1,4),-- Dark Legion\'s Edge
(22279,81,1,1,1,4),-- Dragon Slayer
(22279,5534,1,1,-1,15201),-- Soul Bow Stave
(22279,5533,1,1,-1,15188),-- Elysian Head
(22279,1808,1,1,-1,2461),-- Recipe: Soulshot: S Grade
(22280,57,2109,4321,0,700000),-- Adena
(22280,1894,1,1,2,5446),-- Crafted Leather
(22280,4039,1,1,2,5174),-- Mold Glue
(22280,4044,1,1,2,5174),-- Thons
(22280,5515,1,1,1,3277),-- Sealed Majestic Gauntlets Design
(22280,5503,1,1,1,3277),-- Sealed Majestic Boots Lining
(22280,5514,1,1,1,3277),-- Sealed Gauntlets of Nightmare Design
(22280,5502,1,1,1,3277),-- Sealed Boots of Nightmare Lining
(22280,4041,1,1,2,1350),-- Mold Hardener
(22280,9546,1,1,200,1242),-- Fire Stone
(22280,9573,1,1,200,1078),-- Life Stone: level 80
(22280,12260,1,1,1,528),-- Common Item - Sealed Majestic Boots
(22280,12257,1,1,1,528),-- Common Item - Sealed Majestic Gauntlet
(22280,12267,1,1,1,528),-- Common Item - Sealed Boots of Nightmare
(22280,12264,1,1,1,528),-- Common Item - Sealed Gauntlet of Nightmare
(22280,9574,1,1,200,269),-- Mid-Grade Life Stone: level 80
(22280,960,1,1,2,62),-- Scroll: Enchant Armor (S)
(22280,9575,1,1,200,27),-- High-Grade Life Stone: level 80
(22280,9576,1,1,200,5),-- Top-Grade Life Stone: level 80
(22280,4044,1,1,-1,105027),-- Thons
(22280,4041,1,1,-1,27398),-- Mold Hardener
(22280,5537,1,1,-1,8644),-- Soul Separator Head
(22281,57,1266,2593,0,700000),-- Adena
(22281,1895,1,1,2,44360),-- Metallic Fiber
(22281,4040,1,1,2,3105),-- Mold Lubricant
(22281,5541,1,1,1,1277),-- Tallum Glaive Edge
(22281,9546,1,1,200,1242),-- Fire Stone
(22281,9628,1,1,2,1218),-- Leonard
(22281,9630,1,1,2,817),-- Orichalcum
(22281,6901,1,1,2,155),-- Recipe: Shining Arrow (100%)
(22281,11976,1,1,1,83),-- Common Item - Tallum Glaive
(22281,5538,1,1,-1,5187),-- Dragon Grinder Edge
(22281,3957,1,1,-1,840),-- Recipe: Blessed Spiritshot S
(22281,3036,1,1,-1,840),-- Recipe: Spiritshot S
(22282,57,3707,7592,0,700000),-- Adena
(22282,1869,1,3,2,129885),-- Iron Ore
(22282,1864,4,12,2,97413),-- Stem
(22282,1880,1,1,2,38965),-- Steel
(22282,4043,1,1,2,12989),-- Asofe
(22282,5550,1,1,2,5195),-- Durable Metal Plate
(22282,9546,1,1,200,3117),-- Fire Stone
(22282,5481,1,1,1,3060),-- Sealed Majestic Leather Armor Fabric
(22282,5480,1,1,1,3060),-- Sealed Leather Armor of Nightmare Fabric
(22282,5323,1,1,1,21),-- Sealed Majestic Leather Armor
(22282,5320,1,1,1,21),-- Sealed Leather Armor of Nightmare
(22282,959,1,1,2,16),-- Scroll: Enchant Weapon (S)
(22282,4043,1,1,-1,184566),-- Asofe
(22282,5550,1,1,-1,73826),-- Durable Metal Plate
(22282,5541,1,1,-1,15185),-- Tallum Glaive Edge
(22283,57,1266,2593,0,700000),-- Adena
(22283,1895,1,1,2,44360),-- Metallic Fiber
(22283,4040,1,1,2,3105),-- Mold Lubricant
(22283,9546,1,1,200,1242),-- Fire Stone
(22283,9628,1,1,2,1218),-- Leonard
(22283,9630,1,1,2,817),-- Orichalcum
(22283,5522,1,1,1,793),-- Sealed Armor of Nightmare Pattern
(22283,5523,1,1,1,793),-- Sealed Majestic Plate Armor Pattern
(22283,12262,1,1,1,98),-- Common Item - Sealed Majestic Plate Armor
(22283,12263,1,1,1,98),-- Common Item - Sealed Armor of Nightmare
(22283,960,1,1,2,62),-- Scroll: Enchant Armor (S)
(22283,5545,1,1,-1,5188),-- Dark Legion\'s Edge Blade
(22283,5544,1,1,-1,5182),-- Branch of the Mother Tree Head
(22283,5157,1,1,-1,840),-- Recipe: Soulshot (S) Compressed Package (100%)
(22284,57,3552,7274,0,700000),-- Adena
(22284,1868,1,3,2,237568),-- Thread
(22284,1873,1,1,2,95027),-- Silver Nugget
(22284,5549,1,1,2,23757),-- Metallic Thread
(22284,1889,1,1,2,15838),-- Compound Braid
(22284,4044,1,1,2,7919),-- Thons
(22284,5487,1,1,1,2942),-- Sealed Nightmare Robe Fabric
(22284,5488,1,1,1,2942),-- Sealed Majestic Robe Fabric
(22284,9547,1,1,200,1901),-- Water Stone
(22284,9573,1,1,200,1650),-- Life Stone: level 80
(22284,9574,1,1,200,412),-- Mid-Grade Life Stone: level 80
(22284,6901,1,1,2,238),-- Recipe: Shining Arrow (100%)
(22284,9575,1,1,200,41),-- High-Grade Life Stone: level 80
(22284,5329,1,1,1,20),-- Sealed Majestic Robe
(22284,5326,1,1,1,20),-- Sealed Nightmare Robe
(22284,9576,1,1,200,8),-- Top-Grade Life Stone: level 80
(22284,5549,1,1,-1,530489),-- Metallic Thread
(22284,5546,1,1,-1,15986),-- Sword of Miracles Edge
(22284,5162,1,1,-1,2358),-- Recipe: Spiritshot (S) Compressed Package (100%)
(22285,57,3835,7853,0,700000),-- Adena
(22285,1873,1,1,2,161228),-- Silver Nugget
(22285,1864,4,12,2,100767),-- Stem
(22285,1865,2,6,2,80614),-- Varnish
(22285,1866,3,9,2,44786),-- Suede
(22285,1868,10,30,2,38388),-- Thread
(22285,6346,1,1,1,15308),-- Sealed Majestic Ring Gemstone
(22285,6342,1,1,1,10651),-- Sealed Majestic Earring Gemstone
(22285,6344,1,1,1,6893),-- Sealed Majestic Necklace Beads
(22285,9547,1,1,200,3225),-- Water Stone
(22285,6328,1,1,1,94),-- Sealed Majestic Ring
(22285,6327,1,1,1,63),-- Sealed Majestic Earring
(22285,6326,1,1,1,47),-- Sealed Majestic Necklace
(22285,959,1,1,2,16),-- Scroll: Enchant Weapon (S)
(22285,6342,1,1,-1,194619),-- Sealed Majestic Earring Gemstone
(22285,5167,1,1,-1,3818),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(22286,57,1266,2593,0,700000),-- Adena
(22286,1870,1,1,2,133078),-- Coal
(22286,1871,1,1,2,133078),-- Charcoal
(22286,1872,1,3,2,88719),-- Animal Bone
(22286,1867,1,3,2,59146),-- Animal Skin
(22286,1869,2,6,2,33270),-- Iron Ore
(22286,5515,1,1,1,1967),-- Sealed Majestic Gauntlets Design
(22286,5503,1,1,1,1967),-- Sealed Majestic Boots Lining
(22286,5514,1,1,1,1967),-- Sealed Gauntlets of Nightmare Design
(22286,5502,1,1,1,1967),-- Sealed Boots of Nightmare Lining
(22286,9546,1,1,200,1064),-- Fire Stone
(22286,12257,1,1,1,317),-- Common Item - Sealed Majestic Gauntlet
(22286,12267,1,1,1,317),-- Common Item - Sealed Boots of Nightmare
(22286,12264,1,1,1,317),-- Common Item - Sealed Gauntlet of Nightmare
(22286,12260,1,1,1,317),-- Common Item - Sealed Majestic Boots
(22286,960,1,1,2,53),-- Scroll: Enchant Armor (S)
(22286,6346,1,1,-1,61568),-- Sealed Majestic Ring Gemstone
(22286,6344,1,1,-1,27722),-- Sealed Majestic Necklace Beads
(22286,5272,1,1,-1,840),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(22287,57,4398,9008,0,700000),-- Adena
(22287,1894,1,1,2,18923),-- Crafted Leather
(22287,4039,1,1,2,17977),-- Mold Glue
(22287,4044,1,1,2,17977),-- Thons
(22287,5495,1,1,1,8189),-- Sealed Shield of Nightmare Fragment
(22287,5527,1,1,1,6448),-- Sealed Helm of Nightmare Design
(22287,5528,1,1,1,6448),-- Sealed Majestic Circlet Design
(22287,4041,1,1,2,4690),-- Mold Hardener
(22287,9546,1,1,200,4314),-- Fire Stone
(22287,12268,1,1,1,1399),-- Common Item - Sealed Shield of Nightmare
(22287,12261,1,1,1,979),-- Common Item - Sealed Majestic Circlet
(22287,12269,1,1,1,979),-- Common Item - Sealed Helm of Nightmare
(22287,6901,1,1,2,539),-- Recipe: Shining Arrow (100%)
(22287,1894,1,1,-1,230482),-- Crafted Leather
(22287,4039,1,1,-1,218958),-- Mold Glue
(22287,8342,1,1,-1,18018),-- Flaming Dragon Skull Piece
(22288,57,4830,9894,0,700000),-- Adena
(22288,1868,3,9,2,169252),-- Thread
(22288,1895,1,1,2,145073),-- Metallic Fiber
(22288,1876,1,1,2,101551),-- Mithril Ore
(22288,1873,2,6,2,50776),-- Silver Nugget
(22288,5495,1,1,1,8995),-- Sealed Shield of Nightmare Fragment
(22288,4042,1,1,2,8463),-- Enria
(22288,5528,1,1,1,7083),-- Sealed Majestic Circlet Design
(22288,5527,1,1,1,7082),-- Sealed Helm of Nightmare Design
(22288,9546,1,1,200,4062),-- Fire Stone
(22288,12268,1,1,1,1537),-- Common Item - Sealed Shield of Nightmare
(22288,12261,1,1,1,1075),-- Common Item - Sealed Majestic Circlet
(22288,12269,1,1,1,1075),-- Common Item - Sealed Helm of Nightmare
(22288,959,1,1,2,20),-- Scroll: Enchant Weapon (S)
(22288,8349,1,1,-1,19787),-- Doom Crusher Head
(22288,5282,1,1,-1,3207),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(22288,5277,1,1,-1,3207),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(22289,57,4706,9640,0,700000),-- Adena
(22289,1873,1,1,2,125925),-- Silver Nugget
(22289,1868,3,9,2,104938),-- Thread
(22289,1895,1,1,2,89947),-- Metallic Fiber
(22289,1876,1,1,2,62963),-- Mithril Ore
(22289,4042,1,1,2,5247),-- Enria
(22289,9546,1,1,200,2518),-- Fire Stone
(22289,9573,1,1,200,2186),-- Life Stone: level 80
(22289,8342,1,1,1,1187),-- Flaming Dragon Skull Piece
(22289,8349,1,1,1,1187),-- Doom Crusher Head
(22289,5533,1,1,1,1187),-- Elysian Head
(22289,5544,1,1,1,1186),-- Branch of the Mother Tree Head
(22289,9574,1,1,200,547),-- Mid-Grade Life Stone: level 80
(22289,960,1,1,2,126),-- Scroll: Enchant Armor (S)
(22289,11966,1,1,1,77),-- Common Item - Doom Crusher
(22289,11969,1,1,1,77),-- Common Item - Flaming Dragon Skull
(22289,11970,1,1,1,77),-- Common Item - Branch of the Mother Tree
(22289,11974,1,1,1,77),-- Common Item - Elysian
(22289,9575,1,1,200,55),-- High-Grade Life Stone: level 80
(22289,9576,1,1,200,11),-- Top-Grade Life Stone: level 80
(22289,1876,1,3,-1,702978),-- Mithril Ore
(22289,1895,1,5,-1,669503),-- Metallic Fiber
(22289,5480,1,1,-1,31544),-- Sealed Leather Armor of Nightmare Fabric
(22290,57,14094,28869,0,700000),-- Adena
(22290,5549,1,1,2,148149),-- Metallic Thread
(22290,1889,1,1,2,98766),-- Compound Braid
(22290,1868,15,45,2,98766),-- Thread
(22290,4044,1,1,2,49383),-- Thons
(22290,1873,12,36,2,24692),-- Silver Nugget
(22290,9546,1,1,200,11852),-- Fire Stone
(22290,8342,1,1,1,3556),-- Flaming Dragon Skull Piece
(22290,5533,1,1,1,3556),-- Elysian Head
(22290,8349,1,1,1,3555),-- Doom Crusher Head
(22290,5544,1,1,1,3552),-- Branch of the Mother Tree Head
(22290,6901,1,1,2,1482),-- Recipe: Shining Arrow (100%)
(22290,11966,1,1,1,230),-- Common Item - Doom Crusher
(22290,11969,1,1,1,230),-- Common Item - Flaming Dragon Skull
(22290,11970,1,1,1,230),-- Common Item - Branch of the Mother Tree
(22290,11974,1,1,1,230),-- Common Item - Elysian
(22290,5487,1,1,-1,94799),-- Sealed Nightmare Robe Fabric
(22290,5481,1,1,-1,94465),-- Sealed Majestic Leather Armor Fabric
(22290,1808,1,1,-1,9356),-- Recipe: Soulshot: S Grade
(22291,57,12128,24842,0,700000),-- Adena
(22291,1873,2,6,2,127485),-- Silver Nugget
(22291,5549,1,1,2,127485),-- Metallic Thread
(22291,1889,1,1,2,84990),-- Compound Braid
(22291,1868,15,45,2,84990),-- Thread
(22291,4044,1,1,2,42495),-- Thons
(22291,5534,1,1,1,12250),-- Soul Bow Stave
(22291,9546,1,1,200,10199),-- Fire Stone
(22291,11975,1,1,1,793),-- Common Item - Soul Bow
(22291,959,1,1,2,51),-- Scroll: Enchant Weapon (S)
(22291,1889,1,3,-1,603856),-- Compound Braid
(22291,5488,1,1,-1,81576),-- Sealed Majestic Robe Fabric
(22291,3036,1,1,-1,8051),-- Recipe: Spiritshot S
(22292,57,3458,7082,0,700000),-- Adena
(22292,1895,1,1,2,72687),-- Metallic Fiber
(22292,4040,1,1,2,5088),-- Mold Lubricant
(22292,5534,1,1,1,3492),-- Soul Bow Stave
(22292,9546,1,1,200,2035),-- Fire Stone
(22292,9628,1,1,2,1995),-- Leonard
(22292,9573,1,1,200,1767),-- Life Stone: level 80
(22292,9630,1,1,2,1339),-- Orichalcum
(22292,9574,1,1,200,442),-- Mid-Grade Life Stone: level 80
(22292,11975,1,1,1,226),-- Common Item - Soul Bow
(22292,960,1,1,2,102),-- Scroll: Enchant Armor (S)
(22292,9575,1,1,200,44),-- High-Grade Life Stone: level 80
(22292,9576,1,1,200,9),-- Top-Grade Life Stone: level 80
(22292,4040,1,1,-1,103289),-- Mold Lubricant
(22292,5495,1,1,-1,78421),-- Sealed Shield of Nightmare Fragment
(22292,3957,1,1,-1,2295),-- Recipe: Blessed Spiritshot S
(22293,57,4030,8254,0,700000),-- Adena
(22293,1871,1,3,2,211795),-- Charcoal
(22293,1866,1,3,2,94131),-- Suede
(22293,1877,1,1,2,16944),-- Adamantite Nugget
(22293,4040,1,1,2,8472),-- Mold Lubricant
(22293,5541,1,1,1,4065),-- Tallum Glaive Edge
(22293,4041,1,1,2,3683),-- Mold Hardener
(22293,9546,1,1,200,3389),-- Fire Stone
(22293,6901,1,1,2,424),-- Recipe: Shining Arrow (100%)
(22293,11976,1,1,1,264),-- Common Item - Tallum Glaive
(22293,5502,1,1,-1,101659),-- Sealed Boots of Nightmare Lining
(22293,5503,1,1,-1,101659),-- Sealed Majestic Boots Lining
(22293,5157,1,1,-1,2675),-- Recipe: Soulshot (S) Compressed Package (100%)
(22294,57,3458,7082,0,700000),-- Adena
(22294,1895,1,1,2,121146),-- Metallic Fiber
(22294,4040,1,1,2,8480),-- Mold Lubricant
(22294,5541,1,1,1,3489),-- Tallum Glaive Edge
(22294,9546,1,1,200,3392),-- Fire Stone
(22294,9628,1,1,2,3326),-- Leonard
(22294,9630,1,1,2,2232),-- Orichalcum
(22294,11976,1,1,1,226),-- Common Item - Tallum Glaive
(22294,959,1,1,2,17),-- Scroll: Enchant Weapon (S)
(22294,1895,1,3,-1,737778),-- Metallic Fiber
(22294,4040,1,1,-1,103289),-- Mold Lubricant
(22294,5514,1,1,-1,87223),-- Sealed Gauntlets of Nightmare Design
(22295,57,10074,20633,0,700000),-- Adena
(22295,1895,1,1,2,352959),-- Metallic Fiber
(22295,4043,1,1,2,41178),-- Asofe
(22295,4040,1,1,2,24707),-- Mold Lubricant
(22295,4042,1,1,2,20589),-- Enria
(22295,9546,1,1,200,9883),-- Fire Stone
(22295,5480,1,1,1,8315),-- Sealed Leather Armor of Nightmare Fabric
(22295,5481,1,1,1,8315),-- Sealed Majestic Leather Armor Fabric
(22295,12258,1,1,1,1035),-- Common Item - Sealed Majestic Leather Armor
(22295,12265,1,1,1,1035),-- Common Item - Sealed Leather Armor of Nightmare
(22295,960,1,1,2,494),-- Scroll: Enchant Armor (S)
(22295,5515,1,1,-1,254123),-- Sealed Majestic Gauntlets Design
(22295,5167,1,1,-1,6687),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(22295,5162,1,1,-1,6687),-- Recipe: Spiritshot (S) Compressed Package (100%)
(22296,57,10596,21703,0,700000),-- Adena
(22296,1866,2,6,2,162422),-- Suede
(22296,1344,100,300,2,121816),-- Mithril Arrow
(22296,1871,9,27,2,54140),-- Charcoal
(22296,1877,1,1,2,38981),-- Adamantite Nugget
(22296,4040,1,1,2,19491),-- Mold Lubricant
(22296,4041,1,1,2,8474),-- Mold Hardener
(22296,9546,1,1,200,7796),-- Fire Stone
(22296,5522,1,1,1,6638),-- Sealed Armor of Nightmare Pattern
(22296,5523,1,1,1,6638),-- Sealed Majestic Plate Armor Pattern
(22296,6901,1,1,2,974),-- Recipe: Shining Arrow (100%)
(22296,12262,1,1,1,817),-- Common Item - Sealed Majestic Plate Armor
(22296,12263,1,1,1,817),-- Common Item - Sealed Armor of Nightmare
(22296,1877,1,1,-1,633054),-- Adamantite Nugget
(22296,4041,1,1,-1,137620),-- Mold Hardener
(22296,5522,1,1,-1,53897),-- Sealed Armor of Nightmare Pattern
(22297,57,10429,21361,0,700000),-- Adena
(22297,1866,1,3,2,243607),-- Suede
(22297,1871,4,12,2,137029),-- Charcoal
(22297,1877,1,1,2,43849),-- Adamantite Nugget
(22297,4040,1,1,2,21924),-- Mold Lubricant
(22297,4041,1,1,2,9533),-- Mold Hardener
(22297,9546,1,1,200,8770),-- Fire Stone
(22297,5487,1,1,1,8639),-- Sealed Nightmare Robe Fabric
(22297,5488,1,1,1,8639),-- Sealed Majestic Robe Fabric
(22297,12259,1,1,1,1072),-- Common Item - Sealed Majestic Robe
(22297,12266,1,1,1,1072),-- Common Item - Sealed Nightmare Robe
(22297,959,1,1,2,44),-- Scroll: Enchant Weapon (S)
(22297,5527,1,1,-1,186245),-- Sealed Helm of Nightmare Design
(22297,5523,1,1,-1,53050),-- Sealed Majestic Plate Armor Pattern
(22297,5272,1,1,-1,6923),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(22298,57,23997,49152,0,700000),-- Adena
(22298,1344,150,450,2,122616),-- Mithril Arrow
(22298,1873,3,9,2,98094),-- Silver Nugget
(22298,6346,1,1,1,95808),-- Sealed Majestic Ring Gemstone
(22298,6342,1,1,1,66661),-- Sealed Majestic Earring Gemstone
(22298,1895,3,9,2,60057),-- Metallic Fiber
(22298,1868,32,96,2,45981),-- Thread
(22298,6344,1,1,1,43139),-- Sealed Majestic Necklace Beads
(22298,1876,4,12,2,32698),-- Mithril Ore
(22298,4042,1,1,2,24523),-- Enria
(22298,9546,1,1,200,11771),-- Fire Stone
(22298,12351,1,1,1,10859),-- Common Item - Sealed Majestic Ring
(22298,9573,1,1,200,10218),-- Life Stone: level 80
(22298,12352,1,1,1,7239),-- Common Item - Sealed Majestic Earring
(22298,12350,1,1,1,5435),-- Common Item - Sealed Majestic Necklace
(22298,9574,1,1,200,2555),-- Mid-Grade Life Stone: level 80
(22298,960,1,1,2,589),-- Scroll: Enchant Armor (S)
(22298,9575,1,1,200,255),-- High-Grade Life Stone: level 80
(22298,9576,1,1,200,51),-- Top-Grade Life Stone: level 80
(22298,4042,1,1,-1,597389),-- Enria
(22298,5528,1,1,-1,428543),-- Sealed Majestic Circlet Design
(22298,5277,1,1,-1,15930),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(22299,57,34465,70594,0,700000),-- Adena
(22299,4039,1,1,2,169062),-- Mold Glue
(22299,4044,1,1,2,169062),-- Thons
(22299,1894,1,3,2,88980),-- Crafted Leather
(22299,5503,1,1,1,53536),-- Sealed Majestic Boots Lining
(22299,5515,1,1,1,53536),-- Sealed Majestic Gauntlets Design
(22299,5514,1,1,1,53536),-- Sealed Gauntlets of Nightmare Design
(22299,5502,1,1,1,53536),-- Sealed Boots of Nightmare Lining
(22299,4041,1,1,2,44103),-- Mold Hardener
(22299,12260,1,1,1,8626),-- Common Item - Sealed Majestic Boots
(22299,12257,1,1,1,8626),-- Common Item - Sealed Majestic Gauntlet
(22299,12267,1,1,1,8626),-- Common Item - Sealed Boots of Nightmare
(22299,12264,1,1,1,8626),-- Common Item - Sealed Gauntlet of Nightmare
(22299,6901,1,1,2,5072),-- Recipe: Shining Arrow (100%)
(22299,4041,1,1,-1,447646),-- Mold Hardener
(22299,5529,1,1,-1,141233),-- Dragon Slayer Edge
(22299,5282,1,1,-1,22880),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(22300,57,5597,11465,0,700000),-- Adena
(22300,1866,1,3,2,152535),-- Suede
(22300,1871,4,12,2,85801),-- Charcoal
(22300,1877,1,1,2,27456),-- Adamantite Nugget
(22300,4040,1,1,2,13728),-- Mold Lubricant
(22300,5495,1,1,1,10423),-- Sealed Shield of Nightmare Fragment
(22300,5527,1,1,1,8207),-- Sealed Helm of Nightmare Design
(22300,5528,1,1,1,8207),-- Sealed Majestic Circlet Design
(22300,4041,1,1,2,5969),-- Mold Hardener
(22300,12268,1,1,1,1781),-- Common Item - Sealed Shield of Nightmare
(22300,12261,1,1,1,1246),-- Common Item - Sealed Majestic Circlet
(22300,12269,1,1,1,1246),-- Common Item - Sealed Helm of Nightmare
(22300,959,1,1,2,27),-- Scroll: Enchant Weapon (S)
(22300,5534,1,1,-1,22952),-- Soul Bow Stave
(22300,5533,1,1,-1,22932),-- Elysian Head
(22300,1808,1,1,-1,3716),-- Recipe: Soulshot: S Grade
(22301,57,29719,60872,0,700000),-- Adena
(22301,1875,1,1,2,208257),-- Stone of Purity
(22301,1879,3,9,2,86773),-- Cokes
(22301,4040,1,1,2,62477),-- Mold Lubricant
(22301,1865,28,84,2,55783),-- Varnish
(22301,1870,60,180,2,26032),-- Coal
(22301,9546,1,1,200,24991),-- Fire Stone
(22301,5546,1,1,1,10983),-- Sword of Miracles Edge
(22301,5529,1,1,1,9999),-- Dragon Slayer Edge
(22301,5545,1,1,1,9999),-- Dark Legion\'s Edge Blade
(22301,960,1,1,2,1250),-- Scroll: Enchant Armor (S)
(22301,11971,1,1,1,648),-- Common Item - Sword of Miracles
(22301,11964,1,1,1,648),-- Common Item - Dark Legion
(22301,11968,1,1,1,648),-- Common Item - Dragon Slayer
(22301,4040,1,1,-1,887802),-- Mold Lubricant
(22301,1875,1,7,-1,739835),-- Stone of Purity
(22301,5537,1,1,-1,121784),-- Soul Separator Head
(22302,57,17859,36581,0,700000),-- Adena
(22302,1895,1,3,2,375453),-- Metallic Fiber
(22302,4040,1,1,2,52564),-- Mold Lubricant
(22302,9628,1,1,2,20613),-- Leonard
(22302,9630,1,1,2,13832),-- Orichalcum
(22302,8342,1,1,1,4506),-- Flaming Dragon Skull Piece
(22302,5533,1,1,1,4506),-- Elysian Head
(22302,8349,1,1,1,4505),-- Doom Crusher Head
(22302,5544,1,1,1,4501),-- Branch of the Mother Tree Head
(22302,6901,1,1,2,2628),-- Recipe: Shining Arrow (100%)
(22302,11966,1,1,1,292),-- Common Item - Doom Crusher
(22302,11969,1,1,1,292),-- Common Item - Flaming Dragon Skull
(22302,11970,1,1,1,292),-- Common Item - Branch of the Mother Tree
(22302,11974,1,1,1,292),-- Common Item - Elysian
(22302,5538,1,1,-1,73171),-- Dragon Grinder Edge
(22302,3957,1,1,-1,11856),-- Recipe: Blessed Spiritshot S
(22302,3036,1,1,-1,11856),-- Recipe: Spiritshot S
(22303,57,18630,38159,0,700000),-- Adena
(22303,1869,7,21,2,152311),-- Iron Ore
(22303,1864,24,72,2,95194),-- Stem
(22303,4043,1,1,2,76155),-- Asofe
(22303,1880,3,9,2,32638),-- Steel
(22303,5550,1,1,2,30462),-- Durable Metal Plate
(22303,5538,1,1,1,18801),-- Dragon Grinder Edge
(22303,11967,1,1,1,1218),-- Common Item - Dragon Grinder
(22303,959,1,1,2,91),-- Scroll: Enchant Weapon (S)
(22303,4043,1,1,-1,927572),-- Asofe
(22303,5550,1,1,-1,371029),-- Durable Metal Plate
(22303,5541,1,1,-1,76313),-- Tallum Glaive Edge
(22304,57,18705,38313,0,700000),-- Adena
(22304,1867,4,12,2,229388),-- Animal Skin
(22304,1872,7,21,2,122340),-- Animal Bone
(22304,1882,3,9,2,50975),-- Leather
(22304,1894,1,1,2,48292),-- Crafted Leather
(22304,4042,1,1,2,22939),-- Enria
(22304,5534,1,1,1,18893),-- Soul Bow Stave
(22304,9573,1,1,200,9558),-- Life Stone: level 80
(22304,9574,1,1,200,2389),-- Mid-Grade Life Stone: level 80
(22304,11975,1,1,1,1223),-- Common Item - Soul Bow
(22304,960,1,1,2,550),-- Scroll: Enchant Armor (S)
(22304,9575,1,1,200,239),-- High-Grade Life Stone: level 80
(22304,9576,1,1,200,48),-- Top-Grade Life Stone: level 80
(22304,5545,1,1,-1,76652),-- Dark Legion\'s Edge Blade
(22304,5544,1,1,-1,76561),-- Branch of the Mother Tree Head
(22304,5157,1,1,-1,12418),-- Recipe: Soulshot (S) Compressed Package (100%)
(22305,57,4339,8887,0,700000),-- Adena
(22305,1895,1,1,2,152016),-- Metallic Fiber
(22305,4040,1,1,2,10641),-- Mold Lubricant
(22305,5534,1,1,1,4382),-- Soul Bow Stave
(22305,9547,1,1,200,4257),-- Water Stone
(22305,9628,1,1,2,4173),-- Leonard
(22305,9630,1,1,2,2800),-- Orichalcum
(22305,6901,1,1,2,532),-- Recipe: Shining Arrow (100%)
(22305,11975,1,1,1,284),-- Common Item - Soul Bow
(22305,1895,1,3,-1,925777),-- Metallic Fiber
(22305,5546,1,1,-1,19529),-- Sword of Miracles Edge
(22305,5162,1,1,-1,2880),-- Recipe: Spiritshot (S) Compressed Package (100%)
(22306,57,9969,20420,0,700000),-- Adena
(22306,1866,3,9,2,116433),-- Suede
(22306,1868,10,30,2,99799),-- Thread
(22306,1873,4,12,2,52395),-- Silver Nugget
(22306,1864,36,108,2,29108),-- Stem
(22306,1865,25,75,2,20958),-- Varnish
(22306,5537,1,1,1,10062),-- Soul Separator Head
(22306,9547,1,1,200,8383),-- Water Stone
(22306,11973,1,1,1,652),-- Common Item - Soul Separator
(22306,959,1,1,2,42),-- Scroll: Enchant Weapon (S)
(22306,6342,1,1,-1,505968),-- Sealed Majestic Earring Gemstone
(22306,5167,1,1,-1,9927),-- Recipe: Blessed Spiritshot (S) Compressed Package (100%)
(22307,57,24123,49411,0,700000),-- Adena
(22307,1879,1,1,2,268938),-- Cokes
(22307,1885,1,1,2,134469),-- High Grade Suede
(22307,5537,1,1,1,24348),-- Soul Separator Head
(22307,9547,1,1,200,12909),-- Water Stone
(22307,9628,1,1,2,12656),-- Leonard
(22307,9573,1,1,200,11206),-- Life Stone: level 80
(22307,9630,1,1,2,8493),-- Orichalcum
(22307,9629,1,1,2,7016),-- Adamantine
(22307,9574,1,1,200,2801),-- Mid-Grade Life Stone: level 80
(22307,11973,1,1,1,1578),-- Common Item - Soul Separator
(22307,960,1,1,2,646),-- Scroll: Enchant Armor (S)
(22307,9575,1,1,200,280),-- High-Grade Life Stone: level 80
(22307,9576,1,1,200,56),-- Top-Grade Life Stone: level 80
(22307,6346,1,3,-1,586559),-- Sealed Majestic Ring Gemstone
(22307,6344,1,1,-1,528217),-- Sealed Majestic Necklace Beads
(22307,5272,1,1,-1,16014),-- Recipe: Greater Soulshot (S) Compressed Package(100%)
(22308,57,3967,8126,0,700000),-- Adena
(22308,1879,1,1,2,69501),-- Cokes
(22308,1885,1,1,2,34750),-- High Grade Suede
(22308,9547,1,1,200,3336),-- Water Stone
(22308,5481,1,1,1,3275),-- Sealed Majestic Leather Armor Fabric
(22308,5480,1,1,1,3275),-- Sealed Leather Armor of Nightmare Fabric
(22308,9628,1,1,2,3271),-- Leonard
(22308,9630,1,1,2,2195),-- Orichalcum
(22308,9629,1,1,2,1813),-- Adamantine
(22308,6901,1,1,2,417),-- Recipe: Shining Arrow (100%)
(22308,12258,1,1,1,408),-- Common Item - Sealed Majestic Leather Armor
(22308,12265,1,1,1,408),-- Common Item - Sealed Leather Armor of Nightmare
(22308,8342,1,1,-1,48761),-- Flaming Dragon Skull Piece
(22309,57,5443,11149,0,700000),-- Adena
(22309,1895,1,1,2,190723),-- Metallic Fiber
(22309,4040,1,1,2,13351),-- Mold Lubricant
(22309,9547,1,1,200,5340),-- Water Stone
(22309,9628,1,1,2,5235),-- Leonard
(22309,5480,1,1,1,4493),-- Sealed Leather Armor of Nightmare Fabric
(22309,5481,1,1,1,4493),-- Sealed Majestic Leather Armor Fabric
(22309,9630,1,1,2,3513),-- Orichalcum
(22309,12258,1,1,1,559),-- Common Item - Sealed Majestic Leather Armor
(22309,12265,1,1,1,559),-- Common Item - Sealed Leather Armor of Nightmare
(22309,959,1,1,2,27),-- Scroll: Enchant Weapon (S)
(22309,8349,1,1,-1,22297),-- Doom Crusher Head
(22309,5282,1,1,-1,3614),-- Recipe: Greater Blessed Spiritshot (S) Compressed Package(100%)
(22309,5277,1,1,-1,3614),-- Recipe: Greater Spiritshot (S) Compressed Package(100%)
(22310,57,5119,10485,0,700000),-- Adena
(22310,1895,1,1,2,153728),-- Metallic Fiber
(22310,1344,60,180,2,112093),-- Mithril Arrow
(22310,4040,1,1,2,10761),-- Mold Lubricant
(22310,9547,1,1,200,4304),-- Water Stone
(22310,5487,1,1,1,4240),-- Sealed Nightmare Robe Fabric
(22310,5488,1,1,1,4240),-- Sealed Majestic Robe Fabric
(22310,9628,1,1,2,4220),-- Leonard
(22310,9630,1,1,2,2832),-- Orichalcum
(22310,12259,1,1,1,526),-- Common Item - Sealed Majestic Robe
(22310,12266,1,1,1,526),-- Common Item - Sealed Nightmare Robe
(22310,960,1,1,2,215),-- Scroll: Enchant Armor (S)
(22310,1876,1,3,-1,764565),-- Mithril Ore
(22310,1895,1,5,-1,728157),-- Metallic Fiber
(22310,5480,1,1,-1,34308),-- Sealed Leather Armor of Nightmare Fabric
(22311,57,15512,31115,0,700000),-- Adena
(22311,1873,2,6,2,118713),-- Silver Nugget
(22311,5549,1,1,2,118713),-- Metallic Thread
(22311,1889,1,1,2,79142),-- Compound Braid
(22311,1868,15,45,2,79142),-- Thread
(22311,6346,1,1,1,70857),-- Sealed Majestic Ring Gemstone
(22311,6342,1,1,1,49301),-- Sealed Majestic Earring Gemstone
(22311,4044,1,1,2,39571),-- Thons
(22311,6344,1,1,1,31905),-- Sealed Majestic Necklace Beads
(22311,9546,1,1,200,9498),-- Fire Stone
(22311,10483,1,1,200,8244),-- Life Stone - level 82
(22311,12351,1,1,1,8031),-- Common Item - Sealed Majestic Ring
(22311,12352,1,1,1,5354),-- Common Item - Sealed Majestic Earring
(22311,12350,1,1,1,4019),-- Common Item - Sealed Majestic Necklace
(22311,10484,1,1,200,2061),-- Mid-Grade Life Stone - level 82
(22311,6901,1,1,2,1187),-- Recipe: Shining Arrow (100%)
(22311,10485,1,1,200,206),-- High-Grade Life Stone - level 82
(22311,10486,1,1,200,41),-- Top-Grade Life Stone - level 82
(22311,6688,1,1,-1,57298),-- Forgotten Blade Edge
(22311,6689,1,1,-1,56218),-- Basalt Battlehammer Head
(22311,6901,1,1,-1,26509),-- Recipe: Shining Arrow (100%)
(22312,57,19156,38424,0,700000),-- Adena
(22312,1895,1,3,2,383952),-- Metallic Fiber
(22312,4040,1,1,2,53754),-- Mold Lubricant
(22312,5515,1,1,1,34044),-- Sealed Majestic Gauntlets Design
(22312,5503,1,1,1,34044),-- Sealed Majestic Boots Lining
(22312,5514,1,1,1,34044),-- Sealed Gauntlets of Nightmare Design
(22312,5502,1,1,1,34044),-- Sealed Boots of Nightmare Lining
(22312,9546,1,1,200,21501),-- Fire Stone
(22312,9628,1,1,2,21080),-- Leonard
(22312,9630,1,1,2,14146),-- Orichalcum
(22312,12260,1,1,1,5485),-- Common Item - Sealed Majestic Boots
(22312,12257,1,1,1,5485),-- Common Item - Sealed Majestic Gauntlet
(22312,12267,1,1,1,5485),-- Common Item - Sealed Boots of Nightmare
(22312,12264,1,1,1,5485),-- Common Item - Sealed Gauntlet of Nightmare
(22312,959,1,1,2,107),-- Scroll: Enchant Weapon (S)
(22312,6690,1,1,-1,70893),-- Imperial Staff Head
(22312,6901,1,1,-1,32736),-- Recipe: Shining Arrow (100%)
(22312,960,1,1,-1,13094),-- Scroll: Enchant Armor (S)
(22313,57,5245,10520,0,700000),-- Adena
(22313,1868,4,12,2,137975),-- Thread
(22313,1876,1,1,2,110380),-- Mithril Ore
(22313,1344,150,450,2,45992),-- Mithril Arrow
(22313,1873,3,9,2,36793),-- Silver Nugget
(22313,1895,3,9,2,22527),-- Metallic Fiber
(22313,5495,1,1,1,11174),-- Sealed Shield of Nightmare Fragment
(22313,4042,1,1,2,9198),-- Enria
(22313,5528,1,1,1,8798),-- Sealed Majestic Circlet Design
(22313,5527,1,1,1,8798),-- Sealed Helm of Nightmare Design
(22313,9546,1,1,200,4415),-- Fire Stone
(22313,12268,1,1,1,1909),-- Common Item - Sealed Shield of Nightmare
(22313,12261,1,1,1,1336),-- Common Item - Sealed Majestic Circlet
(22313,12269,1,1,1,1336),-- Common Item - Sealed Helm of Nightmare
(22313,960,1,1,2,221),-- Scroll: Enchant Armor (S)
(22313,1876,1,3,-1,896287),-- Mithril Ore
(22313,4042,1,1,-1,149381),-- Enria
(22313,6691,1,1,-1,19373),-- Angel Slayer Blade
(22314,57,5131,10293,0,700000),-- Adena
(22314,1895,1,1,2,205703),-- Metallic Fiber
(22314,4040,1,1,2,14399),-- Mold Lubricant
(22314,9546,1,1,200,5760),-- Fire Stone
(22314,9628,1,1,2,5647),-- Leonard
(22314,9630,1,1,2,3789),-- Orichalcum
(22314,8342,1,1,1,1481),-- Flaming Dragon Skull Piece
(22314,8349,1,1,1,1481),-- Doom Crusher Head
(22314,5533,1,1,1,1481),-- Elysian Head
(22314,5544,1,1,1,1480),-- Branch of the Mother Tree Head
(22314,6901,1,1,2,720),-- Recipe: Shining Arrow (100%)
(22314,11966,1,1,1,96),-- Common Item - Doom Crusher
(22314,11969,1,1,1,96),-- Common Item - Flaming Dragon Skull
(22314,11970,1,1,1,96),-- Common Item - Branch of the Mother Tree
(22314,11974,1,1,1,96),-- Common Item - Elysian
(22314,6693,1,1,-1,19298),-- Dragon Hunter Axe Blade
(22314,6901,1,1,-1,8769),-- Recipe: Shining Arrow (100%)
(22315,57,3052,6121,0,700000),-- Adena
(22315,1344,10,30,2,291926),-- Mithril Arrow
(22315,1895,1,1,2,66726),-- Metallic Fiber
(22315,4040,1,1,2,4671),-- Mold Lubricant
(22315,5534,1,1,1,3527),-- Soul Bow Stave
(22315,9546,1,1,200,1868),-- Fire Stone
(22315,9628,1,1,2,1832),-- Leonard
(22315,10483,1,1,200,1622),-- Life Stone - level 82
(22315,9630,1,1,2,1229),-- Orichalcum
(22315,10484,1,1,200,406),-- Mid-Grade Life Stone - level 82
(22315,11975,1,1,1,228),-- Common Item - Soul Bow
(22315,10485,1,1,200,40),-- High-Grade Life Stone - level 82
(22315,959,1,1,2,9),-- Scroll: Enchant Weapon (S)
(22315,10486,1,1,200,8),-- Top-Grade Life Stone - level 82
(22315,4040,1,1,-1,104299),-- Mold Lubricant
(22315,6694,1,1,-1,11476),-- Saint Spear Blade
(22315,959,1,1,-1,209),-- Scroll: Enchant Weapon (S)
(22316,57,8835,17721,0,700000),-- Adena
(22316,1895,1,1,2,354164),-- Metallic Fiber
(22316,4043,1,1,2,41319),-- Asofe
(22316,4040,1,1,2,24792),-- Mold Lubricant
(22316,4042,1,1,2,20659),-- Enria
(22316,5537,1,1,1,10202),-- Soul Separator Head
(22316,9546,1,1,200,9917),-- Fire Stone
(22316,11973,1,1,1,661),-- Common Item - Soul Separator
(22316,960,1,1,2,496),-- Scroll: Enchant Armor (S)
(22316,4042,1,1,-1,251633),-- Enria
(22316,6695,1,1,-1,32019),-- Demon Splinter Blade
(22316,6901,1,1,-1,15098),-- Recipe: Shining Arrow (100%)
(22317,57,1260,2528,0,700000),-- Adena
(22317,1868,1,1,2,176808),-- Thread
(22317,1344,10,30,2,110505),-- Mithril Arrow
(22317,1873,1,1,2,35362),-- Silver Nugget
(22317,1895,1,1,2,25258),-- Metallic Fiber
(22317,1876,1,1,2,17681),-- Mithril Ore
(22317,5495,1,1,1,2685),-- Sealed Shield of Nightmare Fragment
(22317,5527,1,1,1,2114),-- Sealed Helm of Nightmare Design
(22317,5528,1,1,1,2114),-- Sealed Majestic Circlet Design
(22317,4042,1,1,2,1474),-- Enria
(22317,9546,1,1,200,707),-- Fire Stone
(22317,10483,1,1,200,614),-- Life Stone - level 82
(22317,12268,1,1,1,459),-- Common Item - Sealed Shield of Nightmare
(22317,12261,1,1,1,321),-- Common Item - Sealed Majestic Circlet
(22317,12269,1,1,1,321),-- Common Item - Sealed Helm of Nightmare
(22317,10484,1,1,200,153),-- Mid-Grade Life Stone - level 82
(22317,6901,1,1,2,88),-- Recipe: Shining Arrow (100%)
(22317,10485,1,1,200,15),-- High-Grade Life Stone - level 82
(22317,10486,1,1,200,3),-- Top-Grade Life Stone - level 82
(22317,6696,1,1,-1,4729),-- Heavens Divider Edge
(22317,6697,1,1,-1,4567),-- Arcana Mace Head
(22317,6901,1,1,-1,2154),-- Recipe: Shining Arrow (100%)
(22320,57,16445,32986,0,700000),-- Adena
(22320,9599,1,3,100,721000),-- Ancient Tome of the Demon
(22320,9674,4,12,101,98885),-- Darion's Badge
(22320,5522,1,1,1,11787),-- Sealed Armor of Nightmare Pattern
(22320,5523,1,1,1,11787),-- Sealed Majestic Plate Armor Pattern
(22320,5311,1,1,1,79),-- Sealed Armor of Nightmare
(22320,5316,1,1,1,79),-- Sealed Majestic Plate Armor
(22320,1895,1,3,2,188353),-- Metallic Fiber
(22320,4040,1,1,2,39554),-- Mold Lubricant
(22320,9628,1,1,2,15511),-- Leonard
(22320,9630,1,1,2,10409),-- Orichalcum
(22320,9551,1,1,200,15822),-- Divine Stone
(22320,959,1,1,2,79),-- Scroll: Enchant Weapon (S)
(22320,1895,1,17,-1,892165),-- Metallic Fiber
(22320,4040,1,1,-1,562064),-- Mold Lubricant
(22320,960,1,1,-1,11241),-- Scroll: Enchant Armor (S)
(22321,57,15784,31661,0,700000),-- Adena
(22321,9599,1,3,100,721000),-- Ancient Tome of the Demon
(22321,9674,3,9,101,98426),-- Darion's Badge
(22321,5487,1,1,1,14957),-- Sealed Robe of Nightmare Fabric
(22321,5488,1,1,1,14957),-- Sealed Majestic Robe Fabric
(22321,5326,1,1,1,101),-- Sealed Robe of Nightmare
(22321,5329,1,1,1,101),-- Sealed Majestic Robe
(22321,1344,900,2700,2,20506),-- Mithril Arrow
(22321,1866,16,48,2,30758),-- Suede
(22321,1885,1,1,2,123033),-- High Grade Suede
(22321,1871,10,30,2,70304),-- Charcoal
(22321,4041,1,1,2,12838),-- Mold Hardener
(22321,5220,1,1,2,59056),-- Metal Hardener
(22321,960,1,1,2,591),-- Scroll: Enchant Armor (S)
(22321,9551,1,1,200,11811),-- Divine Stone
(22321,6698,1,1,-1,529832),-- Sealed Tateossian Earring Part
(22321,6901,1,1,-1,26974),-- Recipe: Shining Arrow (100%)
(22321,6699,1,2,-1,587920),-- Sealed Tateossian Ring Gem
(22324,57,3249,6517,0,700000),-- Adena
(22324,9599,1,1,100,284900),-- Ancient Tome of the Demon
(22324,9674,1,1,101,136749),-- Darion's Badge
(22324,5495,1,1,1,6922),-- Sealed Shield of Nightmare Fragment
(22324,5527,1,1,1,5450),-- Sealed Helm of Nightmare Design
(22324,5528,1,1,1,5450),-- Sealed Majestic Circlet Design
(22324,960,1,1,2,137),-- Scroll: Enchant Armor (S)
(22324,5317,1,1,1,45),-- Sealed Majestic Circlet
(22324,5312,1,1,1,45),-- Sealed Helm of Nightmare
(22324,5315,1,1,1,64),-- Sealed Shield of Nightmare
(22324,1864,4,12,2,85468),-- Stem
(22324,1865,2,6,2,68375),-- Varnish
(22324,1866,3,9,2,37986),-- Suede
(22324,1868,10,30,2,32560),-- Thread
(22324,1873,4,12,2,17094),-- Silver Nugget
(22324,9551,1,1,200,2734),-- Divine Stone
(22324,6901,1,1,-1,5552),-- Recipe: Shining Arrow (100%)
(22324,6702,1,1,-1,38051),-- Sealed Imperial Crusader Gaiters Pattern
(22325,57,2913,5843,0,700000),-- Adena
(22325,9599,1,1,100,255400),-- Ancient Tome of the Demon
(22325,9674,1,1,101,108987),-- Darion's Badge
(22325,5495,1,1,1,6206),-- Sealed Shield of Nightmare Fragment
(22325,5527,1,1,1,4886),-- Sealed Helm of Nightmare Design
(22325,5528,1,1,1,4886),-- Sealed Majestic Circlet Design
(22325,6901,1,1,2,273),-- Recipe: Shining Arrow (100%)
(22325,5317,1,1,1,40),-- Sealed Majestic Circlet
(22325,5312,1,1,1,40),-- Sealed Helm of Nightmare
(22325,5315,1,1,1,57),-- Sealed Shield of Nightmare
(22325,1344,40,120,2,85146),-- Mithril Arrow
(22325,1867,1,3,2,121096),-- Animal Skin
(22325,1869,2,6,2,54494),-- Iron Ore
(22325,1870,3,9,2,45411),-- Coal
(22325,1871,3,9,2,38924),-- Charcoal
(22325,1872,8,24,2,22706),-- Animal Bone
(22325,9551,1,1,200,2180),-- Divine Stone
(22325,6703,1,1,-1,106314),-- Sealed Imperial Crusader Gauntlets Design
(22325,960,1,1,-1,2987),-- Scroll: Enchant Armor (S)
(22327,57,32178,64546,0,700000),-- Adena
(22327,9599,2,6,100,705400),-- Ancient Tome of the Demon
(22327,9674,25,75,101,27089),-- Darion's Badge
(22327,8349,1,1,1,9286),-- Doom Crusher Head
(22327,5533,1,1,1,9288),-- Elysian Head
(22327,5544,1,1,1,9279),-- Branch Of The Mother Tree Head
(22327,8342,1,1,1,9288),-- Flaming Dragon Skull Piece
(22327,960,1,1,2,1354),-- Scroll: Enchant Armor (S)
(22327,164,1,1,1,33),-- Elysian
(22327,213,1,1,1,33),-- Branch of The Mother Tree
(22327,7895,1,1,1,33),-- Flaming Dragon Skull
(22327,7902,1,1,1,33),-- Doom Crusher
(22327,1868,198,594,2,17102),-- Thread
(22327,1873,18,54,2,37624),-- Silver Nugget
(22327,1876,4,12,2,84653),-- Mithril Ore
(22327,1895,3,9,2,138209),-- Metallic Fiber
(22327,4042,1,1,2,56435),-- Enria
(22327,9551,1,1,200,27089),-- Divine Stone
(22327,6706,1,1,-1,892056),-- Sealed Imperial Crusader Helmet Pattern
(22327,6705,1,1,-1,766903),-- Sealed Imperial Crusader Shield Part
(22327,6901,1,1,-1,54991),-- Recipe: Shining Arrow (100%)
(22328,57,13003,26083,0,700000),-- Adena
(22328,9599,1,3,100,570100),-- Ancient Tome of the Demon
(22328,9674,1,3,101,182446),-- Darion's Badge
(22328,5538,1,1,1,15014),-- Dragon Grinder Edge
(22328,270,1,1,1,53),-- Dragon Grinder
(22328,1879,1,1,2,152038),-- Cokes
(22328,1885,1,1,2,76019),-- High Grade Suede
(22328,9628,1,1,2,7155),-- Leonard
(22328,9630,1,1,2,4801),-- Orichalcum
(22328,9629,1,1,2,3966),-- Adamantine
(22328,6901,1,1,1,912),-- Recipe: Shining Arrow (100%)
(22328,9551,1,1,200,7298),-- Divine Stone
(22328,10483,1,1,200,6335),-- Life Stone - level 82
(22328,10484,1,1,200,1584),-- Mid-Grade Life Stone - level 82
(22328,10485,1,1,200,159),-- High-Grade Life Stone - level 82
(22328,10486,1,1,200,32),-- Top-Grade Life Stone - level 82
(22328,1885,1,3,-1,925910),-- High Grade Suede
(22328,13099,1,1,-1,309907),-- Sealed Arcana Sickle Piece
(22328,959,1,1,-1,889),-- Scroll: Enchant Weapon (Grade S)
(22329,57,33226,66648,0,700000),-- Adena
(22329,9599,2,6,100,728400),-- Ancient Tome of the Demon
(22329,9674,14,42,101,49949),-- Darion's Badge
(22329,5534,1,1,1,38396),-- Soul Bow Stave
(22329,289,1,1,1,135),-- Soul Bow
(22329,1344,360,1080,2,121403),-- Mithril Arrow
(22329,1895,20,60,2,24974),-- Metallic Fiber
(22329,4040,1,1,2,69928),-- Mold Lubricant
(22329,9628,1,1,2,27423),-- Leonard
(22329,9630,1,1,2,18402),-- Orichalcum
(22329,9551,1,1,200,27971),-- Divine Stone
(22329,959,1,1,2,140),-- Scroll: Enchant Weapon (Grade S)
(22329,6707,1,1,-1,281342),-- Sealed Draconic Leather Armor Part
(22329,6708,1,1,-1,808455),-- Sealed Draconic Leather Gloves Fabric
(22329,6901,1,1,-1,56782),-- Recipe: Shining Arrow (100%)
(22330,57,2044,4100,0,700000),-- Adena
(22330,9599,1,1,100,179200),-- Ancient Tome of the Demon
(22330,5537,1,1,1,2361),-- Soul Separator Head
(22330,236,1,1,1,8),-- Soul Separator
(22330,1879,1,1,2,28679),-- Cokes
(22330,1885,1,1,2,14340),-- High Grade Suede
(22330,9628,1,1,2,1350),-- Leonard
(22330,9630,1,1,2,906),-- Orichalcum
(22330,9629,1,1,2,748),-- Adamantine
(22330,960,1,1,2,69),-- Scroll: Enchant Armor (S)
(22330,9573,1,1,200,1195),-- Life Stone: level 80
(22330,9574,1,1,200,299),-- Mid-Grade Life Stone: level 80
(22330,9575,1,1,200,30),-- High-Grade Life Stone: level 80
(22330,9576,1,1,200,6),-- Top-Grade Life Stone: level 80
(22330,6709,1,1,-1,49735),-- Sealed Draconic Leather Boots Design
(22330,960,1,1,-1,1397),-- Scroll: Enchant Armor (S)
(22330,6901,1,1,-1,3493),-- Recipe: Shining Arrow (100%)
(22335,57,13117,26259,0,700000),-- Adena
(22335,9599,2,6,100,574300),-- Ancient Tome of the Demon
(22335,10012,4,12,101,34457),-- Scorpion Poison stinger
(22335,5487,1,1,1,12412),-- Sealed Robe of Nightmare Fabric
(22335,5488,1,1,1,12412),-- Sealed Majestic Robe Fabric
(22335,5326,1,1,1,83),-- Sealed Robe of Nightmare
(22335,5329,1,1,1,83),-- Sealed Majestic Robe
(22335,1868,15,45,2,91887),-- Thread
(22335,1873,2,6,2,137830),-- Silver Nugget
(22335,1876,3,9,2,39380),-- Mithril Ore
(22335,1895,3,9,2,65633),-- Metallic Fiber
(22335,4042,1,1,2,22972),-- Enria
(22335,9549,1,1,200,11026),-- Wind Stone
(22335,959,1,1,2,55),-- Scroll: Enchant Weapon (Grade S)
(22335,6714,1,1,-1,363104),-- Sealed Major Arcana Circlet Pattern
(22335,7579,1,1,-1,48941),-- Draconic Bow Shaft
(22335,6901,1,1,-1,22384),-- Recipe: Shining Arrow (100%)
(22337,57,4080,8167,0,700000),-- Adena
(22337,9599,1,1,100,357200),-- Ancient Tome of the Demon
(22337,5514,1,1,1,7240),-- Sealed Gloves of Nightmare Design
(22337,5503,1,1,1,7240),-- Sealed Majestic Boots Lining
(22337,5515,1,1,1,7240),-- Sealed Majestic Gloves Design
(22337,5502,1,1,1,7240),-- Sealed Boots of Nightmare Lining
(22337,5318,1,1,1,63),-- Sealed Majestic Gloves
(22337,5314,1,1,1,63),-- Sealed Boots of Nightmare
(22337,5313,1,1,1,63),-- Sealed Gloves of Nightmare
(22337,5319,1,1,1,63),-- Sealed Majestic Boots
(22337,1871,1,3,2,244959),-- Charcoal
(22337,1866,1,3,2,108871),-- Suede
(22337,1877,1,1,2,19597),-- Adamantite Nugget
(22337,4040,1,1,2,9799),-- Mold Lubricant
(22337,4041,1,1,2,4260),-- Mold Hardener
(22337,6901,1,1,1,490),-- Recipe: Shining Arrow (100%)
(22337,9549,1,1,200,3919),-- Wind Stone
(22337,8712,1,1,-1,16897),-- Sirra's Blade Edge
(22337,8713,1,1,-1,16897),-- Sword of Ipos Blade
(22337,6901,1,1,-1,6962),-- Recipe: Shining Arrow (100%)
(22339,57,2112,4227,0,700000),-- Adena
(22339,9599,1,1,100,184900),-- Ancient Tome of the Demon
(22339,9676,1,1,101,19724),-- Mark of Betrayal
(22339,5545,1,1,1,811),-- Dark Legion's Edge Blade
(22339,5546,1,1,1,891),-- Sword of Miracles Edge
(22339,5529,1,1,1,811),-- Dragon Slayer Edge
(22339,960,1,1,2,118),-- Scroll: Enchant Armor (S)
(22339,81,1,1,1,3),-- Dragon Slayer
(22339,151,1,1,1,3),-- Sword of Miracles 
(22339,2500,1,1,1,3),-- Dark Legion\'s Edge
(22339,1895,1,1,2,84530),-- Metallic Fiber
(22339,4040,1,1,2,5917),-- Mold Lubricant
(22339,9628,1,1,2,2320),-- Leonard
(22339,9630,1,1,2,1557),-- Orichalcum
(22339,1885,1,1,-1,450440),-- High Grade Suede
(22339,8715,1,1,-1,13120),-- Behemoth\'s Tuning Fork Piece
(22340,57,12708,25440,0,700000),-- Adena
(22340,9599,1,3,100,556400),-- Ancient Tome of the Demon
(22340,8349,1,1,1,3662),-- Doom Crusher Head
(22340,5533,1,1,1,3663),-- Elysian Head
(22340,5544,1,1,1,3659),-- Branch Of The Mother Tree Head
(22340,8342,1,1,1,3663),-- Flaming Dragon Skull Piece
(22340,1895,1,3,2,254341),-- Metallic Fiber
(22340,4040,1,1,2,35608),-- Mold Lubricant
(22340,9628,1,1,2,13964),-- Leonard
(22340,9630,1,1,2,9370),-- Orichalcum
(22340,9549,1,1,200,14244),-- Wind Stone
(22340,6901,1,1,2,1780),-- Recipe: Shining Arrow (100%)
(22340,164,1,1,1,13),-- Elysian
(22340,213,1,1,1,13),-- Branch of The Mother Tree
(22340,7895,1,1,1,13),-- Flaming Dragon Skull
(22340,7902,1,1,1,13),-- Doom Crusher
(22340,6901,1,1,-1,21685),-- Recipe: Shining Arrow (100%)
(22340,8716,1,1,-1,52634),-- Naga Storm Piece
(22349,57,28003,56059,0,700000),-- Adena
(22349,9599,2,6,100,613000),-- Ancient Tome of the Demon
(22349,9680,1,1,101,13699),-- Dim Life Force
(22349,9681,1,1,101,3012),-- Life Force
(22349,5480,1,1,1,26410),-- Sealed Leather Armor of Nightmare Fabric
(22349,5481,1,1,1,26410),-- Sealed Majestic Leather Armor Fabric
(22349,960,1,1,2,1345),-- Scroll: Enchant Armor (S)
(22349,5320,1,1,1,178),-- Sealed Leather Armor of Nightmare
(22349,5323,1,1,1,178),-- Sealed Majestic Leather Armor
(22349,1895,12,36,2,40033),-- Metallic Fiber
(22349,4040,1,1,2,67259),-- Mold Lubricant
(22349,1344,300,900,2,140117),-- Mithril Arrow
(22349,9628,1,1,2,26375),-- Leonard
(22349,9630,1,1,2,17699),-- Orichalcum
(22349,9549,1,1,200,26902),-- Wind Stone
(22349,6901,1,1,-1,47785),-- Recipe: Shining Arrow (100%)
(22349,6690,1,1,-1,103484),-- Imperial Staff Head
(22350,57,29112,58280,0,700000),-- Adena
(22350,9599,2,6,100,637300),-- Ancient Tome of the Demon
(22350,9680,1,1,101,25641),-- Dim Life Force
(22350,9681,1,1,101,6803),-- Life Force
(22350,5480,1,1,1,27456),-- Sealed Leather Armor of Nightmare Fabric
(22350,5481,1,1,1,27456),-- Sealed Majestic Leather Armor Fabric
(22350,6901,1,1,2,3496),-- Recipe: Shining Arrow (100%)
(22350,5320,1,1,1,185),-- Sealed Leather Armor of Nightmare
(22350,5323,1,1,1,185),-- Sealed Majestic Leather Armor
(22350,1879,3,9,2,97111),-- Cokes
(22350,1885,1,1,2,291334),-- High Grade Suede
(22350,9628,1,1,2,27420),-- Leonard
(22350,9630,1,1,2,18400),-- Orichalcum
(22350,9629,1,1,2,15200),-- Adamantine
(22350,9551,1,1,200,27968),-- Divine Stone
(22350,13099,1,1,-1,692815),-- Sealed Arcana Sickle Piece
(22350,6691,1,1,-1,107379),-- Angel Slayer Blade
(22350,959,1,1,-1,1987),-- Scroll: Enchant Weapon (Grade S)
(22351,57,29112,58280,0,700000),-- Adena
(22351,9599,2,6,100,637300),-- Ancient Tome of the Demon
(22351,9680,1,1,101,55556),-- Dim Life Force
(22351,9681,1,1,101,9174),-- Life Force
(22351,5545,1,1,1,11185),-- Dark Legion's Edge Blade
(22351,5546,1,1,1,12286),-- Sword of Miracles Edge
(22351,5529,1,1,1,11185),-- Dragon Slayer Edge
(22351,81,1,1,1,39),-- Dragon Slayer
(22351,151,1,1,1,39),-- Sword of Miracles 
(22351,2500,1,1,1,39),-- Dark Legion\'s Edge
(22351,1868,15,45,2,148316),-- Thread
(22351,1873,16,48,2,27809),-- Silver Nugget
(22351,1876,3,9,2,63564),-- Mithril Ore
(22351,1895,3,9,2,105940),-- Metallic Fiber
(22351,4042,1,1,2,37079),-- Enria
(22351,959,1,1,2,89),-- Scroll: Enchant Weapon (Grade S)
(22351,9548,1,1,200,17798),-- Earth Stone
(22351,10483,1,1,200,15449),-- Life Stone - level 82
(22351,10484,1,1,200,3862),-- Mid-Grade Life Stone - level 82
(22351,10485,1,1,200,386),-- High-Grade Life Stone - level 82
(22351,10486,1,1,200,77),-- Top-Grade Life Stone - level 82
(22351,6901,1,1,-1,49678),-- Recipe: Shining Arrow (100%)
(22351,1895,1,29,-1,946253),-- Metallic Fiber
(22351,6693,1,1,-1,109325),-- Dragon Hunter Axe Blade
(22352,57,28120,56293,0,700000),-- Adena
(22352,9599,2,6,100,615600),-- Ancient Tome of the Demon
(22352,9680,1,1,101,520000),-- Dim Life Force
(22352,9681,1,1,101,125000),-- Life Force
(22352,5545,1,1,1,10804),-- Dark Legion's Edge Blade
(22352,5546,1,1,1,11867),-- Sword of Miracles Edge
(22352,5529,1,1,1,10804),-- Dragon Slayer Edge
(22352,81,1,1,1,38),-- Dragon Slayer
(22352,151,1,1,1,38),-- Sword of Miracles 
(22352,2500,1,1,1,38),-- Dark Legion\'s Edge
(22352,1868,135,405,2,25014),-- Thread
(22352,1873,16,48,2,42210),-- Silver Nugget
(22352,1876,3,9,2,96481),-- Mithril Ore
(22352,1895,3,9,2,160802),-- Metallic Fiber
(22352,4042,1,1,2,56281),-- Enria
(22352,960,1,1,2,1351),-- Scroll: Enchant Armor (S)
(22352,9546,1,1,200,27015),-- Fire Stone
(22352,6901,1,1,-1,47985),-- Recipe: Shining Arrow (100%)
(22352,6695,1,1,-1,101764),-- Demon Splinter Blade
(22352,6694,1,1,-1,105598),-- Saint Spear Blade
(22353,57,121177,242584,0,700000),-- Adena
(22353,9599,6,18,100,884200),-- Ancient Tome of the Demon
(22353,9682,1,1,101,1000000),-- Contained Life Force
(22353,5534,1,1,1,139828),-- Soul Bow Stave
(22353,9551,1,1,200,116413),-- Divine Stone
(22353,6901,1,1,2,14552),-- Recipe: Shining Arrow (100%)
(22353,4042,6,18,2,20211),-- Enria
(22353,1867,308,924,2,31497),-- Animal Skin
(22353,1872,60,180,2,161686),-- Animal Bone
(22353,1894,5,15,2,51059),-- Crafted Leather
(22353,1882,27,81,2,59884),-- Leather
(22353,289,1,1,1,491),-- Soul Bow
(22353,1894,1,15,-1,906932),-- Crafted Leather
(22353,4042,1,7,-1,861586),-- Enria
(22353,960,1,1,-1,82712),-- Scroll: Enchant Armor (S)
(22359,57,4226,8463,0,700000),-- Adena
(22359,9599,1,1,100,370200),-- Ancient Tome of the Demon
(22359,5534,1,1,1,4879),-- Soul Bow Stave
(22359,1895,1,1,2,169236),-- Metallic Fiber
(22359,4040,1,1,2,11847),-- Mold Lubricant
(22359,9628,1,1,2,4646),-- Leonard
(22359,9630,1,1,2,3117),-- Orichalcum
(22359,960,1,1,2,237),-- Scroll: Enchant Armor (S)
(22359,289,1,1,1,17),-- Soul Bow
(22359,9551,1,1,200,4739),-- Divine Stone
(22359,1895,1,5,-1,687097),-- Metallic Fiber
(22359,6701,1,1,-1,38411),-- Sealed Imperial Crusader Breastplate Part
(22359,960,1,1,-1,2886),-- Scroll: Enchant Armor (S)
(22360,57,12288,24611,0,700000),-- Adena
(22360,9599,1,3,100,538300),-- Ancient Tome of the Demon
(22360,1345,50,150,0,114829),-- Shining Arrow
(22360,1864,28,84,2,30758),-- Stem
(22360,1869,9,27,2,47845),-- Iron Ore
(22360,1880,1,1,2,86122),-- Steel
(22360,4043,1,1,2,28707),-- Asofe
(22360,5534,1,1,1,14186),-- Soul Bow Stave
(22360,5550,1,1,2,11483),-- Durable Metal Plate
(22360,9551,1,1,200,6890),-- Divine Stone
(22360,289,1,1,1,50),-- Soul Bow
(22360,14166,1,1,200,4161),-- Life Stone - level 84
(22360,14167,1,1,200,1039),-- Mid-Grade Life Stone - level 84
(22360,6901,1,1,2,861),-- Recipe: Shining Arrow (100%)
(22360,14168,1,1,200,104),-- High-Grade Life Stone - level 84
(22360,14169,1,1,200,21),-- Top-Grade Life Stone - level 84
(22360,6702,1,2,-1,143782),-- Sealed Imperial Crusader Gaiters Pattern
(22360,5550,1,1,-1,279723),-- Durable Metal Plate
(22360,6901,1,1,-1,20979),-- Recipe: Shining Arrow (100%)
(22361,57,46478,93087,0,700000),-- Adena
(22361,9599,3,9,100,678600),-- Ancient Tome of the Demon
(22361,9714,1,1,101,500000),-- Key of the Evil Eye
(22361,5522,1,1,1,33282),-- Sealed Armor of Nightmare Pattern
(22361,5523,1,1,1,33282),-- Sealed Majestic Plate Armor Pattern
(22361,4042,1,1,2,93068),-- Enria
(22361,1867,28,84,2,132954),-- Animal Skin
(22361,1872,60,180,2,62045),-- Animal Bone
(22361,1882,27,81,2,22980),-- Leather
(22361,1894,5,15,2,19593),-- Crafted Leather
(22361,9551,1,1,200,44672),-- Divine Stone
(22361,959,1,1,2,223),-- Scroll: Enchant Weapon (Grade S)
(22361,5311,1,1,1,222),-- Sealed Armor of Nightmare
(22361,5316,1,1,1,222),-- Sealed Majestic Plate Armor
(22361,6703,1,3,-1,564888),-- Sealed Imperial Crusader Gauntlets Design
(22361,6704,1,3,-1,564888),-- Sealed Imperial Crusader Boots Design
(22361,6901,1,1,-1,79350),-- Recipe: Shining Arrow (100%)
(22362,57,119181,238696,0,700000),-- Adena
(22362,9599,6,20,100,803100),-- Ancient Tome of the Demon
(22362,8349,1,1,1,34361),-- Doom Crusher Head
(22362,5533,1,1,1,34368),-- Elysian Head
(22362,5544,1,1,1,34334),-- Branch Of The Mother Tree Head
(22362,8342,1,1,1,34368),-- Flaming Dragon Skull Piece
(22362,164,1,1,1,121),-- Elysian
(22362,213,1,1,1,121),-- Branch of The Mother Tree
(22362,7895,1,1,1,121),-- Flaming Dragon Skull
(22362,7902,1,1,1,121),-- Doom Crusher
(22362,1345,2100,6300,0,35356),-- Shining Arrow
(22362,1895,12,36,2,132583),-- Metallic Fiber
(22362,9628,1,1,2,87349),-- Leonard
(22362,9630,1,1,2,58615),-- Orichalcum
(22362,9551,1,1,200,89095),-- Divine Stone
(22362,4040,4,12,2,27842),-- Mold Lubricant
(22362,960,1,1,2,4455),-- Scroll: Enchant Armor (S)
(22362,8702,1,1,2,1577),-- Recipe: Shyeed\'s Bow (60%)
(22362,6897,1,1,2,1141),-- Recipe: Heavens Divider (60%)
(22362,1895,1,123,-1,937658),-- Metallic Fiber
(22362,4040,1,9,-1,813887),-- Mold Lubricant
(22362,959,1,1,-1,8139),-- Scroll: Enchant Weapon (Grade S)
(22363,57,34159,68413,0,700000),-- Adena
(22363,9685,1,1,101,200000),-- Gate Key: Darkness
(22363,5545,1,1,1,13130),-- Dark Legion\'s Edge Blade
(22363,5546,1,1,1,14422),-- Sword of Miracles Edge
(22363,5529,1,1,1,13130),-- Dragon Slayer Edge
(22363,1879,3,9,2,99749),-- Cokes
(22363,1885,1,1,2,299247),-- High Grade Suede
(22363,9628,1,1,2,28164),-- Leonard
(22363,9630,1,1,2,18900),-- Orichalcum
(22363,9629,1,1,2,15613),-- Adamantine
(22363,9551,1,1,200,28727),-- Divine Stone
(22363,6861,1,1,2,6367),-- Recipe: Sealed Imperial Crusader Shield (60%)
(22363,959,1,1,2,144),-- Scroll: Enchant Weapon (Grade S)
(22363,81,1,1,1,46),-- Dragon Slayer
(22363,151,1,1,1,46),-- Sword of Miracles 
(22363,2500,1,1,1,46),-- Dark Legion\'s Edge
(22363,6706,1,1,-1,946016),-- Sealed Imperial Crusader Helmet Pattern
(22363,6705,1,1,-1,813293),-- Sealed Imperial Crusader Shield Part
(22363,6901,1,1,-1,58317),-- Recipe: Shining Arrow (100%)
(22364,57,9939,19905,0,700000),-- Adena
(22364,9685,1,1,101,30000),-- Gate Key: Darkness
(22364,1879,1,1,2,174135),-- Cokes
(22364,1885,1,1,2,87067),-- High Grade Suede
(22364,9628,1,1,2,8194),-- Leonard
(22364,9630,1,1,2,5499),-- Orichalcum
(22364,9629,1,1,2,4543),-- Adamantine
(22364,9551,1,1,200,8359),-- Divine Stone
(22364,8349,1,1,1,2865),-- Doom Crusher Head
(22364,5533,1,1,1,2866),-- Elysian Head
(22364,5544,1,1,1,2863),-- Branch Of The Mother Tree Head
(22364,8342,1,1,1,2866),-- Flaming Dragon Skull Piece
(22364,13100,1,1,2,1853),-- Recipe: Sealed Arcana Sigil (60%)
(22364,959,1,1,2,42),-- Scroll: Enchant Weapon (Grade S)
(22364,164,1,1,1,10),-- Elysian
(22364,213,1,1,1,10),-- Branch of The Mother Tree
(22364,7895,1,1,1,10),-- Flaming Dragon Skull
(22364,7902,1,1,1,10),-- Doom Crusher
(22364,6707,1,1,-1,84071),-- Sealed Draconic Leather Armor Part
(22364,960,1,1,-1,6787),-- Scroll: Enchant Armor (S)
(22364,6901,1,1,-1,16968),-- Recipe: Shining Arrow (100%)
(22365,57,21314,42688,0,700000),-- Adena
(22365,9685,1,1,101,100000),-- Gate Key: Darkness
(22365,1894,1,1,2,62896),-- Crafted Leather
(22365,4044,1,1,2,59751),-- Thons
(22365,4041,1,1,2,15587),-- Mold Hardener
(22365,4039,1,1,2,59751),-- Mold Glue
(22365,5534,1,1,1,24607),-- Soul Bow Stave
(22365,289,1,1,1,86),-- Soul Bow
(22365,6893,1,1,2,184),-- Recipe: Saint Spear (60%)
(22365,960,1,1,2,717),-- Scroll: Enchant Armor (S)
(22365,14166,1,1,200,8660),-- Life Stone - level 84
(22365,14167,1,1,200,2162),-- Mid-Grade Life Stone - level 84
(22365,14168,1,1,200,216),-- High-Grade Life Stone - level 84
(22365,14169,1,1,200,43),-- Top-Grade Life Stone - level 84
(22365,1894,1,3,-1,638392),-- Crafted Leather
(22365,4044,1,3,-1,606472),-- Thons
(22365,6708,1,1,-1,518094),-- Sealed Draconic Leather Gloves Fabric
(22367,57,7242,14504,0,700000),-- Adena
(22367,1345,10,30,0,312342),-- Shining Arrow
(22367,1879,1,1,2,78085),-- Cokes
(22367,1885,1,1,2,39043),-- High Grade Suede
(22367,9628,1,1,2,3675),-- Leonard
(22367,9630,1,1,2,2466),-- Orichalcum
(22367,9629,1,1,2,2037),-- Adamantine
(22367,9551,1,1,200,3748),-- Divine Stone
(22367,5480,1,1,1,6833),-- Sealed Leather Armor of Nightmare Fabric
(22367,5481,1,1,1,6833),-- Sealed Majestic Leather Armor Fabric
(22367,5320,1,1,1,46),-- Sealed Leather Armor of Nightmare
(22367,5323,1,1,1,46),-- Sealed Majestic Leather Armor
(22367,959,1,1,2,19),-- Scroll: Enchant Weapon (Grade S)
(22367,8690,1,1,2,66),-- Recipe: Sirra\'s Blade (60%)
(22367,14166,1,1,200,2263),-- Life Stone - level 84
(22367,14167,1,1,200,565),-- Mid-Grade Life Stone - level 84
(22367,14168,1,1,200,56),-- High-Grade Life Stone - level 84
(22367,14169,1,1,200,11),-- Top-Grade Life Stone - level 84
(22367,4042,1,1,-1,206067),-- Enria
(22367,6710,1,1,-1,200569),-- Sealed Draconic Leather Helmet Pattern
(22367,959,1,1,-1,495),-- Scroll: Enchant Weapon (Grade S)
(22368,57,81028,162283,0,700000),-- Adena
(22368,1895,12,36,2,115893),-- Metallic Fiber
(22368,4040,2,6,2,38940),-- Mold Lubricant
(22368,9628,1,1,2,76353),-- Leonard
(22368,9630,1,1,2,51237),-- Orichalcum
(22368,960,1,1,2,3894),-- Scroll: Enchant Armor (S)
(22368,9551,1,1,200,77880),-- Divine Stone
(22368,6883,1,1,2,997),-- Recipe: Basalt Battlehammer (60%)
(22368,5514,1,1,1,143859),-- Sealed Gloves of Nightmare Design
(22368,5503,1,1,1,143859),-- Sealed Majestic Boots Lining
(22368,5515,1,1,1,143859),-- Sealed Majestic Gloves Design
(22368,5502,1,1,1,143859),-- Sealed Boots of Nightmare Lining
(22368,5318,1,1,1,1257),-- Sealed Majestic Gloves
(22368,5314,1,1,1,1257),-- Sealed Boots of Nightmare
(22368,5313,1,1,1,1257),-- Sealed Gloves of Nightmare
(22368,5319,1,1,1,1257),-- Sealed Majestic Boots
(22368,4040,1,5,-1,922230),-- Mold Lubricant
(22368,6711,1,1,-1,685418),-- Sealed Major Arcana Robe Part
(22368,6901,1,1,-1,138334),-- Recipe: Shining Arrow (100%)
(22369,57,8024,16070,0,700000),-- Adena
(22369,9687,1,1,101,500000),-- Gate Key: Blood
(22369,8349,1,1,1,2313),-- Doom Crusher Head
(22369,5533,1,1,1,2314),-- Elysian Head
(22369,5544,1,1,1,2312),-- Branch Of The Mother Tree Head
(22369,8342,1,1,1,2314),-- Flaming Dragon Skull Piece
(22369,1868,15,45,2,56234),-- Thread
(22369,1873,2,6,2,84350),-- Silver Nugget
(22369,1889,1,1,2,56234),-- Compound Braid
(22369,4044,1,1,2,28117),-- Thons
(22369,5549,1,1,2,84350),-- Metallic Thread
(22369,9551,1,1,200,6748),-- Divine Stone
(22369,6901,1,1,2,844),-- Recipe: Shining Arrow (100%)
(22369,8692,1,1,2,120),-- Recipe: Sword of Ipos (60%)
(22369,164,1,1,1,8),-- Elysian
(22369,213,1,1,1,8),-- Branch of The Mother Tree
(22369,7895,1,1,1,8),-- Flaming Dragon Skull
(22369,7902,1,1,1,8),-- Doom Crusher
(22369,6713,1,1,-1,195038),-- Sealed Major Arcana Boots Design
(22369,6712,1,1,-1,195038),-- Sealed Major Arcana Gloves Design
(22369,6901,1,1,-1,13698),-- Recipe: Shining Arrow (100%)
(22370,57,29949,59981,0,700000),-- Adena
(22370,9686,1,1,101,200000),-- Gate Key: Destruction
(22370,9849,1,1,102,980000),-- Fiery Demon Blood
(22370,1879,3,9,2,87455),-- Cokes
(22370,1885,1,1,2,262365),-- High Grade Suede
(22370,9628,1,1,2,24693),-- Leonard
(22370,9630,1,1,2,16571),-- Orichalcum
(22370,9629,1,1,2,13689),-- Adamantine
(22370,9551,1,1,200,25187),-- Divine Stone
(22370,6861,1,1,2,5582),-- Recipe: Sealed Imperial Crusader Shield (60%)
(22370,5495,1,1,1,63742),-- Sealed Shield of Nightmare Fragment
(22370,5527,1,1,1,50188),-- Sealed Helm of Nightmare Design
(22370,5528,1,1,1,50188),-- Sealed Majestic Circlet Design
(22370,5317,1,1,1,413),-- Sealed Majestic Circlet
(22370,5312,1,1,1,413),-- Sealed Helm of Nightmare
(22370,5315,1,1,1,590),-- Sealed Shield of Nightmare
(22370,959,1,1,2,126),-- Scroll: Enchant Weapon (Grade S)
(22370,1885,1,9,-1,852162),-- High Grade Suede
(22370,13099,1,1,-1,713056),-- Sealed Arcana Sickle Piece
(22370,960,1,1,-1,20452),-- Scroll: Enchant Armor (S)
(22371,57,9801,19630,0,700000),-- Adena
(22371,1345,20,60,0,228966),-- Shining Arrow
(22371,1879,1,1,2,114483),-- Cokes
(22371,1885,1,1,2,57241),-- High Grade Suede
(22371,9628,1,1,2,5387),-- Leonard
(22371,9630,1,1,2,3615),-- Orichalcum
(22371,9629,1,1,2,2986),-- Adamantine
(22371,9551,1,1,200,5495),-- Divine Stone
(22371,5480,1,1,1,9248),-- Sealed Leather Armor of Nightmare Fabric
(22371,5481,1,1,1,9248),-- Sealed Majestic Leather Armor Fabric
(22371,5320,1,1,1,62),-- Sealed Leather Armor of Nightmare
(22371,5323,1,1,1,62),-- Sealed Majestic Leather Armor
(22371,14166,1,1,200,3318),-- Life Stone - level 84
(22371,14167,1,1,200,828),-- Mid-Grade Life Stone - level 84
(22371,14168,1,1,200,83),-- High-Grade Life Stone - level 84
(22371,14169,1,1,200,16),-- Top-Grade Life Stone - level 84
(22371,960,1,1,2,275),-- Scroll: Enchant Armor (S)
(22371,6714,1,1,-1,271438),-- Sealed Major Arcana Circlet Pattern
(22371,7579,1,1,-1,36586),-- Draconic Bow Shaft
(22371,6901,1,1,-1,16733),-- Recipe: Shining Arrow (100%)
(22372,57,24979,50028,0,700000),-- Adena
(22372,9849,1,1,102,1000000),-- Fiery Demon Blood
(22372,1344,240,720,2,136769),-- Mithril Arrow
(22372,1894,1,1,2,92139),-- Crafted Leather
(22372,4044,1,1,2,87532),-- Thons
(22372,4041,1,1,2,22834),-- Mold Hardener
(22372,4039,1,1,2,87532),-- Mold Glue
(22372,9551,1,1,200,21008),-- Divine Stone
(22372,6901,1,1,2,2626),-- Recipe: Shining Arrow (100%)
(22372,6881,1,1,2,269),-- Recipe: Forgotten Blade (60%)
(22372,5546,1,1,1,10546),-- Sword of Miracles Edge
(22372,5529,1,1,1,9602),-- Dragon Slayer Edge
(22372,5545,1,1,1,9602),-- Dark Legion's Edge Blade
(22372,81,1,1,1,34),-- Dragon Slayer
(22372,151,1,1,1,34),-- Sword of Miracles 
(22372,2500,1,1,1,34),-- Dark Legion\'s Edge
(22372,8712,1,1,-1,103509),-- Sirra\'s Blade Edge
(22372,6901,1,1,-1,42646),-- Recipe: Shining Arrow (100%)
(22372,959,1,1,-1,1706),-- Scroll: Enchant Weapon (Grade S)
(22373,57,7603,15210,0,700000),-- Adena
(22373,9599,1,2,100,333333),-- Ancient Tome of the Demon
(22373,5480,1,1,1,8000),-- Sealed Leather Armor of Nightmare Fabric
(22373,5481,1,1,1,8000),-- Sealed Majestic Leather Armor Fabric
(22373,1895,1,1,2,250000),-- Metallic Fiber
(22373,4040,1,1,2,11905),-- Mold Lubricant
(22373,9628,1,1,2,8000),-- Leonard
(22373,9630,1,1,2,15625),-- Orichalcum
(22373,9551,1,1,200,8000),-- Divine Stone
(22373,6853,1,1,2,1996),-- Recipe: Sealed Imperial Crusader Breastplate (60%)
(22373,6849,1,1,2,1996),-- Recipe: Sealed Tateossian Ring (70%)
(22373,1895,1,7,-1,226000),-- Metallic Fiber
(22373,4040,1,1,-1,56000),-- Mold Lubricant
(22373,8713,1,1,-1,10000),-- Sword of Ipos Blade
(22374,57,21260,42000,0,700000),-- Adena
(22374,9599,2,6,100,500000),-- Ancient Tome of the Demon
(22374,5514,1,1,1,31250),-- Sealed Gloves of Nightmare Design
(22374,5503,1,1,1,31250),-- Sealed Majestic Boots Lining
(22374,5515,1,1,1,31250),-- Sealed Majestic Gloves Design
(22374,5502,1,1,1,31250),-- Sealed Boots of Nightmare Lining
(22374,1344,296,812,2,71429),-- Mithril Arrow
(22374,4041,1,1,2,31250),-- Mold Hardener
(22374,1881,5,8,2,52632),-- Coarse Bone Powder
(22374,1872,84,96,2,31250),-- Animal Bone
(22374,1867,9,27,2,142857),-- Animal Skin
(22374,1875,1,1,2,71429),-- Stone of Purity
(22374,9551,1,1,200,20408),-- Divine Stone
(22374,960,1,1,2,979),-- Scroll: Enchant Armor (S)
(22374,6865,1,1,2,979),-- Recipe: Sealed Draconic Leather Armor (60%)
(22374,6847,1,1,2,2941),-- Recipe: Sealed Tateossian Earring (70%)
(22374,8714,1,1,-1,300000),-- Barakiel\'s Axe Piece
(22375,57,45040,89930,0,700000),-- Adena
(22375,9599,3,9,100,680000),-- Ancient Tome of the Demon
(22375,8349,1,1,1,10000),-- Doom Crusher Head
(22375,5533,1,1,1,10000),-- Elysian Head
(22375,5544,1,1,1,10000),-- Branch Of The Mother Tree Head
(22375,8342,1,1,1,10000),-- Flaming Dragon Skull Piece
(22375,1868,126,351,2,35714),-- Thread
(22375,1873,12,36,2,66667),-- Silver Nugget
(22375,1889,4,10,2,30303),-- Compound Braid
(22375,4044,1,1,2,142857),-- Thons
(22375,5549,3,9,2,50000),-- Metallic Thread
(22375,9551,1,1,200,35714),-- Divine Stone
(22375,6901,1,1,2,5848),-- Recipe: Shining Arrow (100%)
(22375,6851,1,1,2,1464),-- Recipe: Sealed Tateossian Necklace(70%)
(22375,8715,1,1,-1,150000),-- Behemoth\'s Tuning Fork Piece
(22375,4044,1,5,-1,700000),-- Thons
(22376,57,121720,237530,0,700000),-- Adena
(22376,9599,7,21,100,780000),-- Ancient Tome of the Demon
(22376,6346,1,1,1,570000),-- Sealed Majestic Ring Gemstone
(22376,6342,1,1,1,333333),-- Sealed Majestic Earring Gemstone
(22376,6344,1,1,1,200000),-- Sealed Majestic Necklace Beads
(22376,4041,1,1,2,111111),-- Mold Hardener
(22376,1881,43,102,2,25641),-- Coarse Bone Powder
(22376,1872,249,706,2,76923),-- Animal Bone
(22376,1867,997,1689,2,18868),-- Animal Skin
(22376,1875,5,11,2,142857),-- Stone of Purity
(22376,9551,1,1,200,90909),-- Divine Stone
(22376,6855,1,1,2,16949),-- Recipe: Sealed Imperial Crusader Gaiters (60%)
(22376,1875,1,29,-1,600000),-- Stone of Purity
(22376,8716,1,1,-1,500000),-- Naga Storm Piece
(22418,57,3655,7487,0,700000),-- Adena
(22418,1873,1,1,2,153680),-- Silver Nugget
(22418,1868,3,9,2,128067),-- Thread
(22418,5549,1,1,2,38420),-- Metallic Thread
(22418,1889,1,1,2,25613),-- Compound Braid
(22418,4044,1,1,2,12807),-- Thons
(22418,5503,1,1,1,5678),-- Sealed Majestic Boots Lining
(22418,5515,1,1,1,5678),-- Sealed Majestic Gauntlets Design
(22418,5514,1,1,1,5678),-- Sealed Gauntlets of Nightmare Design
(22418,5502,1,1,1,5678),-- Sealed Boots of Nightmare Lining
(22418,9547,1,1,200,3074),-- Water Stone
(22418,5318,1,1,1,50),-- Sealed Majestic Gauntlets
(22418,5314,1,1,1,50),-- Sealed Boots of Nightmare
(22418,5313,1,1,1,50),-- Sealed Gauntlets of Nightmare
(22418,5319,1,1,1,50),-- Sealed Majestic Boots
(22418,959,1,1,2,15),-- Scroll: Enchant Weapon (S)
(22418,1889,1,1,-1,363965),-- Compound Braid
(22418,5488,1,1,-1,24584),-- Sealed Majestic Robe Fabric
(22418,3957,1,1,-1,2426),-- Recipe: Blessed Spiritshot S
(22419,57,3428,7022,0,700000),-- Adena
(22419,1895,1,1,2,72075),-- Metallic Fiber
(22419,4040,1,1,2,5045),-- Mold Lubricant
(22419,9547,1,1,200,2018),-- Water Stone
(22419,9628,1,1,2,1979),-- Leonard
(22419,10483,1,1,200,1752),-- Life Stone - level 82
(22419,9630,1,1,2,1328),-- Orichalcum
(22419,5546,1,1,1,1267),-- Sword of Miracles Edge
(22419,5529,1,1,1,1153),-- Dragon Slayer Edge
(22419,5545,1,1,1,1153),-- Dark Legion\'s Edge Blade
(22419,10484,1,1,200,438),-- Mid-Grade Life Stone - level 82
(22419,960,1,1,2,101),-- Scroll: Enchant Armor (S)
(22419,10485,1,1,200,44),-- High-Grade Life Stone - level 82
(22419,10486,1,1,200,9),-- Top-Grade Life Stone - level 82
(22419,151,1,1,1,4),-- Sword of Miracles
(22419,2500,1,1,1,4),-- Dark Legion\'s Edge
(22419,81,1,1,1,4),-- Dragon Slayer
(22419,5502,1,1,-1,86487),-- Sealed Boots of Nightmare Lining
(22419,5495,1,1,-1,77761),-- Sealed Shield of Nightmare Fragment
(22419,5157,1,1,-1,2276),-- Recipe: Soulshot (S) Compressed Package (100%)
(22420,57,3748,7678,0,700000),-- Adena
(22420,1873,1,1,2,137906),-- Silver Nugget
(22420,1868,3,9,2,114922),-- Thread
(22420,1895,1,1,2,98505),-- Metallic Fiber
(22420,1876,1,1,2,68953),-- Mithril Ore
(22420,1345,40,120,2,57461),-- Shining Arrow
(22420,4042,1,1,2,5746),-- Enria
(22420,9547,1,1,200,2758),-- Water Stone
(22420,8342,1,1,1,946),-- Flaming Dragon Skull Piece
(22420,8349,1,1,1,946),-- Doom Crusher Head
(22420,5533,1,1,1,946),-- Elysian Head
(22420,5544,1,1,1,945),-- Branch of the Mother Tree Head
(22420,6901,1,1,2,345),-- Recipe: Shining Arrow (100%)
(22420,7902,1,1,1,3),-- Doom Crusher
(22420,7895,1,1,1,3),-- Flaming Dragon Skull
(22420,213,1,1,1,3),-- Branch of the Mother Tree
(22420,164,1,1,1,3),-- Elysian
(22420,1895,1,3,-1,799855),-- Metallic Fiber
(22420,5503,1,1,-1,94561),-- Sealed Majestic Boots Lining
(22420,4042,1,1,-1,93316),-- Enria
(22449,57,20620,41297,0,700000),-- Adena
(22449,9599,2,4,100,601900),-- Ancient Tome of the Demon
(22449,1895,3,9,2,137570),-- Metallic Fiber
(22449,4040,1,1,2,57779),-- Mold Lubricant
(22449,9628,1,1,2,22659),-- Leonard
(22449,9630,1,1,2,15205),-- Orichalcum
(22449,9551,1,1,200,23112),-- Divine Stone
(22449,55382,1,1,1,23773),-- Dragon Grinder Edge
(22449,270,1,1,1,84),-- Dragon Grinder
(22449,6901,1,1,2,2889),-- Recipe: Shining Arrow (100%)
(22449,6689,1,1,-1,74624),-- Basalt Battlehammer Head
(22449,6901,1,1,-1,35188),-- Recipe: Shining Arrow (100%)
(22449,6690,1,1,-1,76202),-- Imperial Staff Head
(22450,57,987,1976,0,700000),-- Adena
(22450,9599,1,1,100,86400),-- Ancient Tome of the Demon
(22450,1868,1,1,2,276536),-- Thread
(22450,1873,1,1,2,55307),-- Silver Nugget
(22450,1895,1,1,2,39505),-- Metallic Fiber
(22450,5534,1,1,1,1139),-- Soul Bow Stave
(22450,11975,1,1,1,74),-- Common Item - Soul Bow
(22450,959,1,1,2,6),-- Scroll: Enchant Weapon (S)
(22450,1876,1,1,2,27654),-- Mithril Ore
(22450,4042,1,1,2,2304),-- Enria
(22450,1895,1,1,-1,481173),-- Metallic Fiber
(22450,4042,1,1,-1,28068),-- Enria
(22450,960,1,1,-1,674); -- Scroll: Enchant Armor (S)


-- Category Explanation (RaidBoss & their minions)
--   0 is full armor and armor mats
--   1 is full weapon and weapon mats
--   2 is anything else (scrolls,arrows,etc)
INSERT INTO `droplist` VALUES
(25001,348,1,1,0,272694),-- Scale Mail
(25001,377,4,12,0,54507),-- Scale Gaiters
(25001,1935,53,157,0,74358),-- Scale Mail Fragment
(25001,1937,24,72,0,250135),-- Scale Gaiters Fragment
(25001,69,1,1,1,67692),-- Bastard Sword
(25001,125,1,1,1,67692),-- Spinebone Sword
(25001,126,1,1,1,67692),-- Artisan\'s Sword
(25001,128,1,1,1,67692),-- Knight\'s Sword
(25001,2021,3,9,1,352920),-- Spinebone Sword Blade
(25001,956,12,36,2,251620),-- Scroll: Enchant Armor (Grade D)
(25001,6575,1,1,2,40259),-- Blessed Scroll: Enchant Weapon (Grade D)
(25001,6576,1,1,2,362332),-- Blessed Scroll: Enchant Armor (Grade D)
(25002,17,10,30,2,919333),-- Wooden Arrow
(25003,1060,1,1,2,408593),-- Lesser Healing Potion
(25004,352,1,1,0,143869),-- Brigandine Tunic
(25004,1941,140,420,0,23469),-- Brigandine Temper
(25004,2377,6,16,0,20921),-- Mithril Scale Gaiters
(25004,2378,1,1,0,230131),-- Brigandine Gaiters
(25004,2918,40,120,0,122541),-- Mithril Scale Gaiters Material
(25004,2919,81,243,0,60514),-- Brigandine Gaiters Material
(25004,224,1,1,1,69232),-- Maingauche
(25004,1660,1,1,1,69232),-- Cursed Maingauche
(25004,2038,24,72,1,81836),-- Maingauche Edge
(25004,2044,8,22,1,261874),-- Cursed Maingauche Edge
(25004,6387,60,180,2,321287),-- Blessed Scroll of Resurrection for Pets
(25005,6036,1,1,2,8594),-- Greater Magic Haste Potion
(25006,1341,10,30,2,859422),-- Bone Arrow
(25007,398,1,1,0,115013),-- Plated Leather
(25007,399,1,1,0,106046),-- Rind Leather Armor
(25007,418,7,10,0,14147),-- Plated Leather Gaiters
(25007,419,1,1,0,169820),-- Rind Leather Gaiters
(25007,1966,192,576,0,18753),-- Plated Leather Fragment
(25007,1967,116,346,0,46571),-- Plated Leather Gaiters Material
(25007,1975,23,67,0,143973),-- Rind Leather Armor Design
(25007,1976,60,180,0,99952),-- Rind Leather Gaiters Material
(25007,282,1,1,1,52558),-- Elemental Bow
(25007,285,1,1,1,52558),-- Noble Elven Bow
(25007,2085,24,72,1,85282),-- Crossbow Shaft
(25007,2086,8,22,1,272902),-- Elven Bow of Nobility Shaft
(25007,952,12,36,2,852299),-- Scroll: Enchant Armor (Grade C)
(25008,6036,1,1,2,16213),-- Greater Magic Haste Potion
(25009,1341,20,60,2,810667),-- Bone Arrow
(25010,2397,1,1,0,64704),-- Tunic of Zubei
(25010,2402,1,1,0,103503),-- Stockings of Zubei
(25010,2406,1,1,0,39800),-- Avadon Robe
(25010,4069,81,243,0,56359),-- Tunic of Zubei Fabric
(25010,4070,40,120,0,173285),-- Stockings of Zubei Fabric
(25010,4071,14,42,0,250509),-- Avadon Robe Fabric
(25010,300,1,1,1,52171),-- Great Axe
(25010,4113,12,36,1,373636),-- Great Axe Head
(25010,4622,1,3,2,389481),-- Greater Dye of Mental Strength <Men+4 Wit-4>
(25010,4623,2,4,2,259654),-- Greater Dye of Wit <Wit+4 Int-4>
(25010,4624,2,6,2,194740),-- Greater Dye of Wit <Wit+4 Men-4>
(25010,6572,1,1,2,150599),-- Blessed Scroll: Enchant Armor (Grade B)
(25011,1539,1,1,2,188978),-- Greater Healing Potion
(25012,1342,20,60,2,850400),-- Fine Steel Arrow
(25013,2384,1,1,0,54316),-- Zubei\'s Leather Shirt
(25013,2388,1,1,0,86886),-- Zubei\'s Leather Gaiters
(25013,2390,1,1,0,33411),-- Avadon Leather Armor
(25013,4066,81,243,0,47311),-- Zubei\'s Leather Shirt Fabric
(25013,4067,40,120,0,145465),-- Zubei\'s Leather Gaiter Texture
(25013,4068,14,42,0,210290),-- Avadon Leather Armor Lining
(25013,300,1,1,1,43795),-- Great Axe
(25013,4113,12,36,1,313650),-- Great Axe Head
(25013,952,21,63,2,150500),-- Scroll: Enchant Armor (Grade C)
(25013,4604,2,6,2,263377),-- Greater Dye of Constitution <Con+3 Dex-3>
(25013,4605,3,7,2,210701),-- Greater Dye of Dexterity <Dex+3 Str-3>
(25013,4606,3,9,2,175584),-- Greater Dye of Dexterity <Dex+3 Con-3>
(25014,1539,1,1,2,154489),-- Greater Healing Potion
(25015,1342,20,60,2,695200),-- Fine Steel Arrow
(25016,601,1,1,0,96753),-- Boots of Doom
(25016,2439,1,1,0,96753),-- Blue Wolf Boots
(25016,2475,1,1,0,96753),-- Doom Gloves
(25016,2487,1,1,0,96753),-- Blue Wolf Gloves
(25016,4090,116,346,0,58177),-- Blue Wolf Boots Design
(25016,4091,192,576,0,34997),-- Doom Gloves Part
(25016,4096,293,877,0,22974),-- Blue Wolf Gloves Fabric
(25016,4099,60,180,0,111992),-- Doom Boots Part
(25016,268,1,1,1,35597),-- Bellion Cestus
(25016,4120,12,36,1,327029),-- Bellion Cestus Edge
(25016,947,1,1,2,232830),-- Scroll: Enchant Weapon (Grade B)
(25016,4619,5,15,2,80287),-- Greater Dye of Intelligence <Int+4 Men-4>
(25016,4620,2,4,2,267621),-- Greater Dye of Intelligence <Int+4 Wit-4>
(25016,4621,2,6,2,200716),-- Greater Dye of Mental Strength <Men+4 Int-4>
(25017,1539,1,1,2,227733),-- Greater Healing Potion
(25018,1342,60,180,2,341600),-- Fine Steel Arrow
(25019,393,1,1,0,167392),-- Mithril Banded Mail
(25019,394,1,1,0,167392),-- Reinforced Leather Shirt
(25019,415,4,12,0,33444),-- Mithril Banded Gaiters
(25019,416,5,13,0,29728),-- Reinforced Leather Gaiters
(25019,1938,53,157,0,62696),-- Mithril Banded Mail Material
(25019,1939,24,72,0,179171),-- Mithril Banded Gaiters Material
(25019,316,1,1,1,79024),-- Blood of Saints
(25019,317,1,1,1,79024),-- Tome of Blood
(25019,2030,3,9,1,572656),-- Tome of Blood Page
(25019,956,12,36,2,793367),-- Scroll: Enchant Armor (Grade D)
(25020,396,1,1,0,210544),-- Salamander Skin Mail
(25020,1952,12,36,0,515065),-- Salamander Skin Mail Texture
(25020,159,1,1,1,16695),-- Bonebreaker
(25020,187,1,1,1,16695),-- Atuba Hammer
(25020,188,1,1,1,16695),-- Ghost Staff
(25020,189,1,1,1,16695),-- Staff of Life
(25020,190,1,1,1,16695),-- Atuba Mace
(25020,2048,2,6,1,239325),-- Bonebreaker Head
(25020,2049,3,7,1,191461),-- Atuba Hammer Head
(25020,2050,7,21,1,72504),-- Ghost Staff Head
(25020,2051,12,36,1,42294),-- Staff of Life Shaft
(25020,2058,3,9,1,159548),-- Atuba Mace Head
(25020,6574,1,1,2,428808),-- Blessed Scroll: Enchant Armor (Grade C)
(25021,6036,1,1,2,10720),-- Greater Magic Haste Potion
(25022,1341,20,60,2,536000),-- Bone Arrow
(25023,353,1,1,0,189125),-- Half Plate Armor
(25023,380,4,12,0,37825),-- Plate Gaiters
(25023,1950,53,157,0,87926),-- Half Plate Temper
(25023,1951,24,72,0,299138),-- Plate Gaiters Part
(25023,262,1,1,1,123047),-- Scallop Jamadhr
(25023,2053,12,36,1,293992),-- Scallop Jamadhr Edge
(25023,951,3,9,2,335203),-- Scroll: Enchant Weapon (Grade C)
(25024,6036,1,1,2,10720),-- Greater Magic Haste Potion
(25025,1341,20,60,2,536000),-- Bone Arrow
(25026,119,1,1,0,96888),-- Necklace of Binding
(25026,855,1,1,0,129184),-- Nassen\'s Earring
(25026,886,1,1,0,193777),-- Ring of Binding
(25026,1918,81,243,0,56259),-- Nassen\'s Earring Gemstone
(25026,1920,140,420,0,25717),-- Necklace of Binding Chain
(25026,2917,40,120,0,151092),-- Ring of Binding Gemstone
(25026,286,1,1,1,40182),-- Eminence Bow
(25026,2121,3,9,1,725814),-- Eminence Bow Shaft
(25026,4607,1,3,2,340993),-- Greater Dye of Intelligence <Int+3 Men-3>
(25026,4608,2,4,2,227328),-- Greater Dye of Intelligence <Int+3 Wit-3>
(25026,4609,2,6,2,170496),-- Greater Dye of Mental Strength <Men+3 Int-3>
(25026,6573,1,1,2,20459),-- Blessed Scroll: Enchant Weapon (Grade C)
(25027,1342,20,60,2,666400),-- Fine Steel Arrow
(25028,1539,1,1,2,148089),-- Greater Healing Potion
(25029,503,1,1,0,95783),-- Zubei\'s Helmet
(25029,633,1,1,0,136840),-- Zubei\'s Shield
(25029,673,1,1,0,136840),-- Avadon Shield
(25029,2415,1,1,0,95783),-- Avadon Circlet
(25029,4072,293,877,0,22010),-- Avadon Circlet Pattern
(25029,4101,116,346,0,71944),-- Zubei\'s Shield Fragment
(25029,4102,192,576,0,33531),-- Zubei\'s Helmet Design
(25029,4439,60,180,0,164759),-- Avadon Shield Fragment
(25029,284,1,1,1,51495),-- Dark Elven Long Bow
(25029,4112,12,36,1,405775),-- Dark Elven Longbow Shaft
(25029,947,1,1,2,222972),-- Scroll: Enchant Weapon (Grade B)
(25029,4613,5,15,2,76887),-- Greater Dye of Strength <Str+4 Con-4>
(25029,4614,2,4,2,256290),-- Greater Dye of Strength <Str+4 Dex-4>
(25029,4615,2,6,2,192217),-- Greater Dye of Constitution <Con+4 Str-4>
(25030,1539,1,1,2,174578),-- Greater Healing Potion
(25031,1342,20,60,2,785600),-- Fine Steel Arrow
(25032,5297,1,1,0,37701),-- Sealed Dark Crystal Leather Armor
(25032,5298,1,1,0,60299),-- Sealed Dark Crystalline Leggings
(25032,5301,1,1,0,23197),-- Sealed Tallum Leather Armor
(25032,5478,9,27,0,235217),-- Sealed Dark Crystal Leather Armor Pattern
(25032,5479,20,60,0,74482),-- Sealed Tallum Leather Armor Pattern
(25032,5482,28,84,0,118670),-- Sealed Dark Crystalline Leggings Design
(25032,288,1,1,1,32142),-- Carnage Bow
(25032,5535,12,36,1,332119),-- Carnage Bow Stave
(25032,729,1,1,2,163170),-- Scroll: Enchant Weapon (Grade A)
(25032,6570,1,1,2,163170),-- Blessed Scroll: Enchant Armor (Grade A)
(25033,1343,20,60,2,871429),-- Silver Arrow
(25034,1343,20,60,2,871429),-- Silver Arrow
(25035,5313,1,1,0,54308),-- Sealed Gloves of Nightmare
(25035,5314,1,1,0,54308),-- Sealed Boots of Nightmare
(25035,5318,1,1,0,54308),-- Sealed Majestic Gloves
(25035,5319,1,1,0,54308),-- Sealed Majestic Boots
(25035,305,1,1,1,21287),-- Tallum Glaive
(25035,5541,12,36,1,252257),-- Tallum Glaive Edge
(25035,5502,116,346,0,26901),-- Sealed Boots of Nightmare Lining
(25035,5503,23,67,0,138092),-- Sealed Majestic Boots Lining
(25035,5514,60,180,0,51784),-- Sealed Gloves of Nightmare Design
(25035,5515,16,48,0,194191),-- Sealed Majestic Gloves Design
(25035,959,1,1,2,38156),-- Scroll: Enchant Weapon (Grade S)
(25035,960,1,1,2,381565),-- Scroll: Enchant Armor (Grade S)
(25035,6577,1,1,2,6360),-- Blessed Scroll: Enchant Weapon (Grade S)
(25036,1344,20,60,2,898500),-- Mithril Arrow
(25037,1344,20,60,2,898500),-- Mithril Arrow
(25038,849,4,10,0,93126),-- Tiger\'s Eye Earring
(25038,880,3,7,0,195168),-- Black Pearl Ring
(25038,912,4,12,0,61083),-- Near Forest Necklace
(25038,1901,72,216,0,154441),-- Tiger\'s Eye Earring Stone
(25038,83,1,1,1,58466),-- Sword of Magic
(25038,124,1,1,1,43313),-- Two-Handed Sword
(25038,127,1,1,1,43313),-- Crimson Sword
(25038,130,1,1,1,43313),-- Elven Sword
(25038,144,1,1,1,58466),-- Sword of Occult
(25038,2026,3,9,1,322992),-- Two-Handed Sword Edge
(25038,955,3,9,2,417444),-- Scroll: Enchant Weapon (Grade D)
(25039,6036,1,1,2,7812),-- Greater Magic Haste Potion
(25040,1341,10,30,2,781200),-- Bone Arrow
(25041,61,9,25,0,10511),-- Mithril Gloves
(25041,607,9,27,0,9927),-- Ogre Power Gauntlets
(25041,1125,10,28,0,9405),-- Assault Boots
(25041,1949,112,336,0,33477),-- Mithril Gloves Design
(25041,1957,320,960,0,11717),-- Assault Boots Part
(25041,1963,203,607,0,18516),-- Ogre Power Gauntlets Part
(25041,2427,10,30,0,8935),-- Salamander Skin Boots
(25041,2428,1,1,0,178695),-- Plate Boots
(25041,2451,1,1,0,178695),-- Sage\'s Worn Gloves
(25041,2931,28,82,0,136342),-- Salamander Skin Boot Lining
(25041,2932,36,108,0,104150),-- Plate Boots Design
(25041,2933,46,136,0,82406),-- Sage\'s Worn Gloves Lining
(25041,262,1,1,1,87193),-- Scallop Jamadhr
(25041,2053,12,36,1,208326),-- Scallop Jamadhr Edge
(25041,6574,1,1,2,447912),-- Blessed Scroll: Enchant Armor (Grade C)
(25042,1341,20,60,2,565333),-- Bone Arrow
(25043,6036,1,1,2,11307),-- Greater Magic Haste Potion
(25044,2436,1,1,0,79200),-- Theca Leather Boots
(25044,2437,1,1,0,60919),-- Drake Leather Boots
(25044,2438,1,1,0,60919),-- Full Plate Boots
(25044,2460,1,1,0,79200),-- Theca Leather Gloves
(25044,2461,1,1,0,60919),-- Drake Leather Gloves
(25044,2462,1,1,0,60919),-- Full Plate Gauntlets
(25044,2463,1,1,0,60919),-- Divine Gloves
(25044,2954,144,432,0,18415),-- Theca Leather Boots Texture
(25044,2955,255,765,0,10399),-- Theca Leather Gloves Texture
(25044,2960,396,1188,0,5761),-- Drake Leather Boots Design
(25044,2961,30,90,0,76045),-- Full Plate Boots Part
(25044,2962,39,117,0,58496),-- Drake Leather Gloves Design
(25044,2963,49,147,0,46558),-- Full Plate Gauntlets Part
(25044,2964,60,180,0,38023),-- Divine Gloves Pattern
(25044,228,1,1,1,37332),-- Crystal Dagger
(25044,2119,3,9,1,668197),-- Crystal Dagger Blade
(25044,952,21,63,2,90517),-- Scroll: Enchant Armor (Grade C)
(25044,4604,2,6,2,158406),-- Greater Dye of Constitution <Con+3 Dex-3>
(25044,4605,3,7,2,126724),-- Greater Dye of Dexterity <Dex+3 Str-3>
(25044,4606,3,9,2,105604),-- Greater Dye of Dexterity <Dex+3 Con-3>
(25045,1342,20,60,2,612000),-- Fine Steel Arrow
(25046,1539,1,1,2,136000),-- Greater Healing Potion
(25047,2436,1,1,0,85245),-- Theca Leather Boots
(25047,2437,1,1,0,65568),-- Drake Leather Boots
(25047,2438,1,1,0,65568),-- Full Plate Boots
(25047,2460,1,1,0,85245),-- Theca Leather Gloves
(25047,2461,1,1,0,65568),-- Drake Leather Gloves
(25047,2462,1,1,0,65568),-- Full Plate Gauntlets
(25047,2463,1,1,0,65568),-- Divine Gloves
(25047,2954,144,432,0,19820),-- Theca Leather Boots Texture
(25047,2955,255,765,0,11193),-- Theca Leather Gloves Texture
(25047,2960,395,1188,0,6200),-- Drake Leather Boots Design
(25047,2961,30,90,0,81849),-- Full Plate Boots Part
(25047,2962,39,117,0,62961),-- Drake Leather Gloves Design
(25047,2963,49,147,0,50113),-- Full Plate Gauntlets Part
(25047,2964,60,180,0,40925),-- Divine Gloves Pattern
(25047,228,1,1,1,40182),-- Crystal Dagger
(25047,2119,3,9,1,719198),-- Crystal Dagger Blade
(25047,4610,1,3,2,340993),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(25047,4611,2,4,2,227329),-- Greater Dye of Wit <Wit+3 Int-3>
(25047,4612,2,6,2,170496),-- Greater Dye of Wit <Wit+3 Men-3>
(25047,6574,1,1,2,175367),-- Blessed Scroll: Enchant Armor (Grade C)
(25048,1342,20,60,2,666400),-- Fine Steel Arrow
(25049,1539,1,1,2,148089),-- Greater Healing Potion
(25050,2384,1,1,0,34356),-- Zubei\'s Leather Shirt
(25050,2388,1,1,0,54956),-- Zubei\'s Leather Gaiters
(25050,2390,1,1,0,21132),-- Avadon Leather Armor
(25050,4066,9,27,0,269319),-- Zubei\'s Leather Shirt Fabric
(25050,4067,40,120,0,92007),-- Zubei\'s Leather Gaiter Texture
(25050,4068,14,42,0,133009),-- Avadon Leather Armor Lining
(25050,300,1,1,1,27701),-- Great Axe
(25050,4113,12,36,1,198387),-- Great Axe Head
(25050,4610,1,3,2,333178),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(25050,4611,2,4,2,222119),-- Greater Dye of Wit <Wit+3 Int-3>
(25050,4612,2,6,2,166589),-- Greater Dye of Wit <Wit+3 Men-3>
(25050,6574,1,1,2,171348),-- Blessed Scroll: Enchant Armor (Grade C)
(25051,5287,1,1,0,26967),-- Sealed Dark Crystal breastplate
(25051,5288,1,1,0,43151),-- Sealed Dark Crystal Gaiters
(25051,5293,1,1,0,16595),-- Sealed Tallum Plate Armor
(25051,5520,9,27,0,172957),-- Sealed Dark Crystal Breastplate Pattern
(25051,5521,3,7,0,437076),-- Sealed Tallum Plate Armor Pattern
(25051,5524,28,84,0,84529),-- Sealed Dark Crystal Gaiters Pattern
(25051,235,1,1,1,30660),-- Bloody Orchid
(25051,5536,12,36,1,306377),-- Bloody Orchid Head
(25051,729,1,1,2,103763),-- Scroll: Enchant Weapon (Grade A)
(25051,730,1,1,2,778226),-- Scroll: Enchant Armor (Grade A)
(25051,6569,1,1,2,12452),-- Blessed Scroll: Enchant Weapon (Grade A)
(25052,1343,20,60,2,871429),-- Silver Arrow
(25053,1343,20,60,2,871429),-- Silver Arrow
(25054,6676,1,1,0,15739),-- Sealed Imperial Crusader Gauntlet
(25054,6677,1,1,0,15739),-- Sealed Imperial Crusader Boots
(25054,6681,1,1,0,15739),-- Sealed Draconic Leather Glove
(25054,6682,1,1,0,15739),-- Sealed Draconic Leather Boots
(25054,6685,1,1,0,15739),-- Sealed Major Arcana Glove
(25054,6686,1,1,0,15739),-- Sealed Major Arcana Boots
(25054,6703,8,24,0,223739),-- Sealed Imperial Crusader Gauntlets Design
(25054,6704,14,40,0,132586),-- Sealed Imperial Crusader Boots Design
(25054,6708,20,60,0,89496),-- Sealed Draconic Leather Gloves Fabric
(25054,6709,28,82,0,65087),-- Sealed Draconic Leather Boots Design
(25054,6712,36,108,0,49720),-- Sealed Major Arcana Gloves fabric
(25054,6713,46,136,0,39340),-- Sealed Major Arcana Boots Design
(25054,6371,1,1,1,10584),-- Demon Splinter
(25054,6695,12,36,1,199953),-- Demon Splinter Blade
(25054,959,1,1,2,39110),-- Scroll: Enchant Weapon (Grade S)
(25054,960,1,1,2,391104),-- Scroll: Enchant Armor (Grade S)
(25054,6577,1,1,2,6519),-- Blessed Scroll: Enchant Weapon (Grade S)
(25055,1344,60,180,2,349167),-- Mithril Arrow
(25056,1344,60,180,2,349167),-- Mithril Arrow
(25057,64,1,1,0,286396),-- Composite Boots
(25057,608,1,1,0,286396),-- Mithril Gauntlets
(25057,1981,53,157,0,151462),-- Mithril Gauntlets Design
(25057,2956,192,576,0,41415),-- Composite Boots Part
(25057,231,1,1,1,24546),-- Grace Dagger
(25057,233,1,1,1,24546),-- Dark Screamer
(25057,2106,4,12,1,276250),-- Grace Dagger Edge
(25057,2107,8,22,1,147333),-- Dark Screamer Edge
(25057,952,3,9,2,584108),-- Scroll: Enchant Armor (Grade C)
(25057,4604,2,6,2,146027),-- Greater Dye of Constitution <Con+3 Dex-3>
(25057,4605,3,7,2,116821),-- Greater Dye of Dexterity <Dex+3 Str-3>
(25057,4606,3,9,2,97351),-- Greater Dye of Dexterity <Dex+3 Con-3>
(25058,1342,20,60,2,559200),-- Fine Steel Arrow
(25059,1539,1,1,2,124267),-- Greater Healing Potion
(25060,605,4,12,0,278795),-- Leather Gauntlets
(25060,1123,8,22,0,148691),-- Blue Buckskin Boots
(25060,275,1,1,1,115232),-- Long Bow
(25060,276,1,1,1,115232),-- Elven Bow
(25060,2024,12,36,1,169448),-- Elven Bow Shaft
(25060,955,4,10,2,105736),-- Scroll: Enchant Weapon (Grade D)
(25060,956,8,22,2,411195),-- Scroll: Enchant Armor (Grade D)
(25060,6387,24,72,2,154198),-- Blessed Scroll of Resurrection for Pets
(25061,1061,1,1,2,116364),-- Healing Potion
(25062,1832,1,1,2,213333),-- Greater Antidote
(25063,630,6,16,0,30926),-- Square Shield
(25063,1956,40,120,0,190655),-- Square Shield Fragment
(25063,2412,1,1,0,238251),-- Plate Helmet
(25063,2494,6,18,0,28349),-- Plate Shield
(25063,2930,140,420,0,38686),-- Plate Helmet Design
(25063,2934,81,243,0,94151),-- Plate Shield Fragment
(25063,297,1,1,1,87193),-- Glaive
(25063,2055,12,36,1,208326),-- Glaive Edge
(25063,951,1,3,2,712587),-- Scroll: Enchant Weapon (Grade C)
(25064,852,1,1,0,258667),-- Moonstone Earring
(25064,883,6,16,0,35288),-- Aquastone Ring
(25064,915,1,1,0,194081),-- Aquastone Necklace
(25064,1908,81,243,0,90193),-- Moonstone Earring Wire
(25064,1909,480,1440,0,21705),-- Aquastone Ring Wire
(25064,1910,140,420,0,41495),-- Aquastone Necklace Chain
(25064,89,1,1,1,9846),-- Big Hammer
(25064,160,1,1,1,9846),-- Battle Axe
(25064,161,1,1,1,9846),-- Silver Axe
(25064,173,1,1,1,9846),-- Skull Graver
(25064,191,1,1,1,9846),-- Heavy Doom Hammer
(25064,192,1,1,1,9846),-- Crystal Staff
(25064,193,1,1,1,9846),-- Stick of Faith
(25064,194,1,1,1,9846),-- Heavy Doom Axe
(25064,2061,2,6,1,167705),-- Big Hammer Head
(25064,2063,3,7,1,134164),-- Battle Axe Head
(25064,2064,3,9,1,111803),-- Silver Axe Head
(25064,2065,4,10,1,95832),-- Skull Graver Head
(25064,2066,10,30,1,34035),-- Heavy Doom Hammer Head
(25064,2067,4,12,1,78187),-- Crystal Staff Head
(25064,2068,5,13,1,69499),-- Stick of Faith Shaft
(25064,2069,17,49,1,20623),-- Heavy Doom Axe Blade
(25064,951,3,7,2,81822),-- Scroll: Enchant Weapon (Grade C)
(25064,4591,1,3,2,375022),-- Greater Dye of Constitution <Con+2 Str-2>
(25064,4592,2,4,2,250015),-- Greater Dye of Constitution <Con+2 Dex-2>
(25064,4593,2,6,2,187511),-- Greater Dye of Dexterity <Dex+2 Str-2>
(25065,1341,20,60,2,736000),-- Bone Arrow
(25066,6036,1,1,2,14720),-- Greater Magic Haste Potion
(25067,2397,1,1,0,63241),-- Tunic of Zubei
(25067,2402,1,1,0,101162),-- Stockings of Zubei
(25067,2406,1,1,0,38900),-- Avadon Robe
(25067,4069,81,243,0,55084),-- Tunic of Zubei Fabric
(25067,4070,40,120,0,169366),-- Stockings of Zubei Fabric
(25067,4071,14,42,0,244843),-- Avadon Robe Fabric
(25067,78,1,1,1,16997),-- Great Sword
(25067,142,1,1,1,16997),-- Keshanberk
(25067,148,1,1,1,16997),-- Sword of Valhalla
(25067,4104,5,15,1,324888),-- Great Sword Blade
(25067,4107,9,27,1,197697),-- Keshanberk Blade
(25067,4108,14,42,1,114234),-- Sword of Valhalla Blade
(25067,952,21,63,2,175229),-- Scroll: Enchant Armor (Grade C)
(25067,4604,2,6,2,306653),-- Greater Dye of Constitution <Con+3 Dex-3>
(25067,4605,3,7,2,245322),-- Greater Dye of Dexterity <Dex+3 Str-3>
(25067,4606,3,9,2,204435),-- Greater Dye of Dexterity <Dex+3 Con-3>
(25068,1539,1,1,2,167822),-- Greater Healing Potion
(25069,1342,20,60,2,755200),-- Fine Steel Arrow
(25070,856,1,1,0,96655),-- Adamantite Earring
(25070,887,1,1,0,144983),-- Adamantite Ring
(25070,918,1,1,0,72492),-- Adamantite Necklace
(25070,4049,81,243,0,72901),-- Adamantite Earring Gemstone
(25070,4050,40,120,0,174379),-- Adamantite Ring Wire
(25070,4051,140,420,0,31186),-- Adamantite Necklace Chain
(25070,78,1,1,1,10111),-- Great Sword
(25070,142,1,1,1,10111),-- Keshanberk
(25070,148,1,1,1,10111),-- Sword of Valhalla
(25070,4104,5,15,1,193268),-- Great Sword Blade
(25070,4107,9,27,1,105707),-- Keshanberk Blade
(25070,4108,14,42,1,67955),-- Sword of Valhalla Blade
(25070,947,1,1,2,175123),-- Scroll: Enchant Weapon (Grade B)
(25070,948,1,3,2,547259),-- Scroll: Enchant Armor (Grade B)
(25070,6571,1,1,2,12509),-- Blessed Scroll: Enchant Weapon (Grade B)
(25071,1539,1,1,2,188978),-- Greater Healing Potion
(25072,1342,20,60,2,850400),-- Fine Steel Arrow
(25073,5326,1,1,0,22190),-- Sealed Robe of Nightmare
(25073,5329,1,1,0,22190),-- Sealed Majestic Robe
(25073,5487,4,12,0,412731),-- Sealed Robe of Nightmare Fabric
(25073,5488,8,22,0,219932),-- Sealed Majestic Robe Fabric
(25073,289,1,1,1,21193),-- Soul Bow
(25073,5534,12,36,1,251406),-- Soul Bow Stave
(25073,959,1,1,2,37986),-- Scroll: Enchant Weapon (Grade S)
(25073,960,1,1,2,379865),-- Scroll: Enchant Armor (Grade S)
(25073,6577,1,1,2,6331),-- Blessed Scroll: Enchant Weapon (Grade S)
(25074,1344,20,60,2,870500),-- Mithril Arrow
(25075,1344,20,60,2,870500),-- Mithril Arrow
(25076,58,1,1,0,118972),-- Mithril Breastplate
(25076,59,1,3,0,85999),-- Mithril Gaiters
(25076,349,1,1,0,118972),-- Compound Scale Mail
(25076,350,1,1,0,118972),-- Dwarven Scale Mail
(25076,351,1,1,0,107471),-- Blast Plate
(25076,378,2,4,0,57333),-- Compound Scale Gaiters
(25076,379,1,1,0,171998),-- Dwarven Scale Gaiters
(25076,293,1,1,1,118535),-- War Hammer
(25076,296,1,1,1,118535),-- Dwarven Pike
(25076,6575,1,1,2,126939),-- Blessed Scroll: Enchant Weapon (Grade D)
(25077,1834,1,1,2,225319),-- Emergency Dressing
(25078,17,20,60,2,506967),-- Wooden Arrow
(25079,437,1,1,0,288198),-- Mithril Tunic
(25079,470,4,12,0,57533),-- Mithril Stockings
(25079,1944,53,157,0,120402),-- Mithril Tunic Fabric
(25079,1945,216,648,0,44557),-- Mithril Stocking Design
(25079,318,1,1,1,138464),-- Crucifix of Blood
(25079,2042,12,36,1,286294),-- Crucifix of Blood Piece
(25079,955,3,9,2,642575),-- Scroll: Enchant Weapon (Grade D)
(25080,6036,1,1,2,8594),-- Greater Magic Haste Potion
(25081,1341,10,30,2,859422),-- Bone Arrow
(25082,439,1,1,0,154890),-- Karmian Tunic
(25082,471,1,1,0,247518),-- Karmian Stockings
(25082,1954,24,72,0,249607),-- Karmian Stocking Design
(25082,1962,53,157,0,68682),-- Karmian Tunic Pattern
(25082,71,1,1,1,38351),-- Flamberge
(25082,72,1,1,1,38351),-- Stormbringer
(25082,2059,8,22,1,167403),-- Flamberge Blade
(25082,2060,4,12,1,335526),-- Stormbringer Blade
(25082,4589,1,3,2,486900),-- Greater Dye of Strength <Str+2 Con-2>
(25082,4590,2,4,2,324600),-- Greater Dye of Strength <Str+2 Dex-2>
(25082,6574,1,1,2,166937),-- Blessed Scroll: Enchant Armor (Grade C)
(25083,1341,20,60,2,698667),-- Bone Arrow
(25084,6036,1,1,2,13973),-- Greater Magic Haste Potion
(25085,854,7,19,0,45880),-- Earring of Binding
(25085,885,6,16,0,81315),-- Ring of Ages
(25085,917,7,21,0,31956),-- Necklace of Mermaid
(25085,1914,81,243,0,234348),-- Earring of Binding Gemstone
(25085,1915,280,840,0,106320),-- Ring of Ages Gemstone
(25085,1916,240,720,0,59908),-- Necklace of Mermaid Teardrop
(25085,951,3,7,2,143345),-- Scroll: Enchant Weapon (Grade C)
(25085,4601,6,18,2,73000),-- Greater Dye of Strength <Str+3 Con-3>
(25085,4602,2,4,2,292000),-- Greater Dye of Strength <Str+3 Dex-3>
(25085,4603,2,6,2,219000),-- Greater Dye of Constitution <Con+3 Str-3>
(25086,6036,1,1,2,17813),-- Greater Magic Haste Potion
(25087,1341,20,60,2,890667),-- Bone Arrow
(25088,355,1,1,0,161490),-- Dwarven Chain Mail Shirt
(25088,382,1,1,0,258374),-- Dwarven Chain Gaiters
(25088,1968,53,157,0,91817),-- Dwarven Chain Mail Shirt Material
(25088,1969,192,576,0,39281),-- Dwarven Chain Gaiters Material
(25088,227,1,1,1,54318),-- Stiletto
(25088,242,1,1,1,54318),-- Soulfire Dirk
(25088,2083,24,72,1,83075),-- Stiletto Edge
(25088,2084,8,22,1,265841),-- Soulfire Dirk Blade
(25088,6573,1,1,2,103727),-- Blessed Scroll: Enchant Weapon (Grade C)
(25089,864,1,1,0,71286),-- Earring of Black Ore
(25089,895,1,1,0,106929),-- Ring of Black Ore
(25089,926,1,1,0,53465),-- Necklace of Black Ore
(25089,4052,81,243,0,55386),-- Earring of Black Ore Piece
(25089,4053,28,84,0,193973),-- Ring of Black Ore Gemstone
(25089,4054,20,60,0,124789),-- Necklace of Black Ore Beads
(25089,171,1,1,1,7534),-- Deadman\'s Glory
(25089,175,1,1,1,7534),-- Art of Battle Axe
(25089,210,1,1,1,7534),-- Staff of Evil Spirits
(25089,4116,5,15,1,166115),-- Deadman\'s Glory Stone
(25089,4117,9,27,1,92286),-- Art of Battle Axe Blade
(25089,4118,2,6,1,393462),-- Evil Spirit Head
(25089,4616,1,3,2,254878),-- Greater Dye of Constitution <Con+4 Dex-4>
(25089,4617,2,4,2,169919),-- Greater Dye of Dexterity <Dex+4 Str-4>
(25089,4618,2,6,2,127439),-- Greater Dye of Dexterity <Dex+4 Con-4>
(25089,6572,1,1,2,98553),-- Blessed Scroll: Enchant Armor (Grade B)
(25090,1539,1,1,2,219556),-- Greater Healing Potion
(25091,1342,20,60,2,988000),-- Fine Steel Arrow
(25092,5312,1,1,0,48286),-- Sealed Helm of Nightmare
(25092,5315,1,1,0,68999),-- Sealed Shield of Nightmare
(25092,5317,1,1,0,48286),-- Sealed Majestic Circlet
(25092,5495,40,120,0,93119),-- Sealed Shield of Nightmare Fragment
(25092,5527,9,27,0,325858),-- Sealed Helm of Nightmare Design
(25092,5528,14,42,0,209480),-- Sealed Majestic Circlet Design
(25092,305,1,1,1,21287),-- Tallum Glaive
(25092,5541,12,36,1,252257),-- Tallum Glaive Edge
(25092,730,1,3,2,596190),-- Scroll: Enchant Armor (Grade A)
(25092,6578,1,1,2,63594),-- Blessed Scroll: Enchant Armor (Grade S)
(25093,1344,20,60,2,898500),-- Mithril Arrow
(25094,1344,20,60,2,898500),-- Mithril Arrow
(25095,395,1,1,0,193209),-- Manticore Skin Shirt
(25095,417,4,12,0,38570),-- Manticore Skin Gaiters
(25095,1942,53,157,0,80718),-- Manticore Skin Shirt Texture
(25095,1943,24,72,0,274954),-- Manticore Skin Gaiters Pattern
(25095,279,1,1,1,92827),-- Strengthened Long Bow
(25095,2040,12,36,1,240310),-- Strengthened Long Bow Shaft
(25095,6576,1,3,2,646178),-- Blessed Scroll: Enchant Armor (Grade D)
(25096,6036,1,1,2,8212),-- Greater Magic Haste Potion
(25097,1341,10,30,2,821156),-- Bone Arrow
(25098,396,1,1,0,219924),-- Salamander Skin Mail
(25098,1952,12,36,0,538011),-- Salamander Skin Mail Texture
(25098,70,1,1,1,43596),-- Claymore
(25098,2047,4,12,1,332241),-- Claymore Blade
(25098,2499,1,1,1,43576),-- Elven Long Sword
(25098,2967,8,22,1,174797),-- Elven Long Sword Blade
(25098,952,3,9,2,580626),-- Scroll: Enchant Armor (Grade C)
(25098,4589,2,6,2,217735),-- Greater Dye of Strength <Str+2 Con-2>
(25098,4590,3,7,2,174187),-- Greater Dye of Strength <Str+2 Dex-2>
(25099,64,1,1,0,275300),-- Composite Boots
(25099,608,1,1,0,275300),-- Mithril Gauntlets
(25099,1981,53,157,0,145593),-- Mithril Gauntlets Design
(25099,2956,192,576,0,39810),-- Composite Boots Part
(25099,75,1,1,1,7865),-- Caliburs
(25099,76,1,1,1,7865),-- Sword of Delusion
(25099,77,1,1,1,7865),-- Tsurugi
(25099,84,1,1,1,7865),-- Homunkulus\'s Sword
(25099,134,1,1,1,7865),-- Sword of Nightmare
(25099,145,1,1,1,7865),-- Sword of Whispering Death
(25099,2089,8,24,1,44258),-- Caliburs Edge
(25099,2090,14,40,1,26228),-- Delusional Blade
(25099,2091,2,6,1,177035),-- Tsurugi Blade
(25099,2092,3,7,1,141628),-- Homunkulus\'s Sword Blade
(25099,2095,3,9,1,118024),-- Sword of Nightmare Blade
(25099,2096,4,10,1,101163),-- Sword of Whispering Death Blade
(25099,952,3,9,2,561477),-- Scroll: Enchant Armor (Grade C)
(25099,4604,2,6,2,140396),-- Greater Dye of Constitution <Con+3 Dex-3>
(25099,4605,3,7,2,112295),-- Greater Dye of Dexterity <Dex+3 Str-3>
(25099,4606,3,9,2,93580),-- Greater Dye of Dexterity <Dex+3 Con-3>
(25100,6036,1,1,2,17813),-- Greater Magic Haste Potion
(25101,1341,20,60,2,890667),-- Bone Arrow
(25102,854,1,1,0,159081),-- Earring of Binding
(25102,885,1,1,0,238568),-- Ring of Ages
(25102,917,1,1,0,119323),-- Necklace of Mermaid
(25102,1914,81,243,0,62505),-- Earring of Binding Gemstone
(25102,1915,280,840,0,28357),-- Ring of Ages Gemstone
(25102,1916,20,60,0,191744),-- Necklace of Mermaid Teardrop
(25102,326,1,1,1,49092),-- Heathen\'s Book
(25102,2110,12,36,1,190501),-- Heathen\'s Book Page
(25102,4607,1,3,2,292053),-- Greater Dye of Intelligence <Int+3 Men-3>
(25102,4608,2,4,2,194702),-- Greater Dye of Intelligence <Int+3 Wit-3>
(25102,4609,2,6,2,146027),-- Greater Dye of Mental Strength <Men+3 Int-3>
(25102,6573,1,1,2,17523),-- Blessed Scroll: Enchant Weapon (Grade C)
(25103,554,1,1,0,84625),-- Zubei\'s Boots
(25103,600,1,1,0,84625),-- Avadon Boots
(25103,612,1,1,0,84625),-- Zubei\'s Gauntlets
(25103,2464,1,1,0,84625),-- Avadon Gloves
(25103,4073,23,67,0,195295),-- Avadon Gloves Part
(25103,4097,60,180,0,73236),-- Zubei\'s Boots Design
(25103,4098,116,346,0,38045),-- Avadon Boots Design
(25103,4100,192,576,0,22886),-- Zubei\'s Gauntlets Part
(25103,91,1,1,1,15167),-- Heavy War Axe
(25103,92,1,1,1,15167),-- Sprite\'s Staff
(25103,4105,4,12,1,362385),-- Heavy War Axe Head
(25103,4106,8,22,1,190101),-- Sprite\'s Staff Head
(25103,948,1,3,2,820891),-- Scroll: Enchant Armor (Grade B)
(25103,6572,1,1,2,175123),-- Blessed Scroll: Enchant Armor (Grade B)
(25104,1539,1,1,2,188978),-- Greater Healing Potion
(25105,1342,20,60,2,850400),-- Fine Steel Arrow
(25106,5290,1,1,0,48676),-- Sealed Dark Crystal Gloves
(25106,5291,1,1,0,48676),-- Sealed Dark Crystal Boots
(25106,5295,1,1,0,48676),-- Sealed Tallum Gloves
(25106,5296,1,1,0,48676),-- Sealed Tallum Boots
(25106,5496,11,31,0,241845),-- Sealed Dark Crystal Boots Lining
(25106,5497,23,67,0,112862),-- Sealed Tallum Boots Lining
(25106,5508,60,180,0,42323),-- Sealed Dark Crystal Gloves Design
(25106,5509,16,48,0,158711),-- Sealed Tallum Gloves Design
(25106,212,1,1,1,9219),-- Dasparion\'s Staff
(25106,2504,1,1,1,9219),-- Meteor Shower
(25106,5532,8,22,1,146088),-- Meteor Shower Head
(25106,5543,4,12,1,276498),-- Dasparion\'s Staff Edge
(25106,6570,1,1,2,93596),-- Blessed Scroll: Enchant Armor (Grade A)
(25106,6571,1,1,2,24068),-- Blessed Scroll: Enchant Weapon (Grade B)
(25107,1343,20,60,2,813714),-- Silver Arrow
(25108,1343,20,60,2,813714),-- Silver Arrow
(25109,5311,1,1,0,17049),-- Sealed Armor of Nightmare
(25109,5316,1,1,0,17049),-- Sealed Majestic Plate Armor
(25109,5522,4,12,0,319488),-- Sealed Armor of Nightmare Pattern
(25109,5523,8,22,0,170394),-- Sealed Majestic Plate Armor Pattern
(25109,305,1,1,1,21711),-- Tallum Glaive
(25109,5541,12,36,1,257274),-- Tallum Glaive Edge
(25109,6570,1,1,2,162147),-- Blessed Scroll: Enchant Armor (Grade A)
(25109,6578,1,1,2,64859),-- Blessed Scroll: Enchant Armor (Grade S)
(25110,1344,60,180,2,339000),-- Mithril Arrow
(25111,1344,60,180,2,339000),-- Mithril Arrow
(25112,850,6,18,0,26510),-- Elven Earring
(25112,881,5,15,0,47752),-- Elven Ring
(25112,913,1,1,0,238758),-- Elven Necklace
(25112,1902,81,243,0,71526),-- Elven Earring Beads
(25112,1903,40,120,0,238459),-- Elven Ring Piece
(25112,1904,154,462,0,34653),-- Elven Necklace Beads
(25112,93,1,1,1,47918),-- Winged Spear
(25112,294,1,1,1,47918),-- War Pick
(25112,2033,4,12,1,358808),-- Winged Spear Blade
(25112,2041,8,22,1,191365),-- War Pick Head
(25112,956,12,36,2,926570),-- Scroll: Enchant Armor (Grade D)
(25113,6036,1,1,2,8594),-- Greater Magic Haste Potion
(25114,1341,10,30,2,859422),-- Bone Arrow
(25115,62,11,33,0,7593),-- Mithril Boots
(25115,1958,220,660,0,16567),-- Mithril Boots Design
(25115,2429,12,34,0,7263),-- Chain Boots
(25115,2430,12,36,0,6960),-- Karmian Boots
(25115,2431,13,37,0,6682),-- Plated Leather Boots
(25115,2432,13,39,0,6424),-- Dwarven Chain Boots
(25115,2433,14,40,0,6187),-- Boots of Seal
(25115,2452,14,42,0,5966),-- Reinforced Mithril Gloves
(25115,2453,15,43,0,5760),-- Chain Gloves
(25115,2454,1,1,0,167038),-- Karmian Gloves
(25115,2937,378,1134,0,9643),-- Chain Boots Part
(25115,2938,26,78,0,140186),-- Karmian Boots Fabric
(25115,2939,35,105,0,104137),-- Plate Leather Boot Lining
(25115,2940,45,135,0,80996),-- Dwarven Chain Boots Part
(25115,2941,56,168,0,65086),-- Boots of Seal Pattern
(25115,2942,68,204,0,53600),-- Reinforced Mithril Gloves Design
(25115,2943,81,243,0,44998),-- Chain Gloves Design
(25115,2944,95,285,0,38366),-- Karmian Gloves Fabric
(25115,263,1,1,1,124211),-- Chakram
(25115,2073,12,36,0,352609),-- Chakram Edge
(25115,952,21,63,2,112639),-- Scroll: Enchant Armor (Grade C)
(25115,4594,2,6,2,295679),-- Greater Dye of Dexterity <Dex+2 Con-2>
(25115,4595,3,7,2,236542),-- Greater Dye of Intelligence <Int+2 Men-2>
(25115,4596,3,9,2,197119),-- Greater Dye of Intelligence <Int+2 Wit-2>
(25116,1341,20,60,2,736000),-- Bone Arrow
(25117,6036,1,1,2,14720),-- Greater Magic Haste Potion
(25118,353,1,1,0,134016),-- Half Plate Armor
(25118,380,1,1,0,214426),-- Plate Gaiters
(25118,1950,53,157,0,62305),-- Half Plate Temper
(25118,1951,24,72,0,211973),-- Plate Gaiters Part
(25118,159,1,1,1,17439),-- Bonebreaker
(25118,187,1,1,1,17439),-- Atuba Hammer
(25118,188,1,1,1,17439),-- Ghost Staff
(25118,189,1,1,1,17439),-- Staff of Life
(25118,190,1,1,1,17439),-- Atuba Mace
(25118,2048,18,54,1,27777),-- Bonebreaker Head
(25118,2049,3,7,1,199993),-- Atuba Hammer Head
(25118,2050,7,21,1,75735),-- Ghost Staff Head
(25118,2051,12,36,1,44179),-- Staff of Life Shaft
(25118,2058,3,9,1,166660),-- Atuba Mace Head
(25118,4591,1,3,2,326602),-- Greater Dye of Constitution <Con+2 Str-2>
(25118,4592,2,4,2,217735),-- Greater Dye of Constitution <Con+2 Dex-2>
(25118,4593,2,6,2,163301),-- Greater Dye of Dexterity <Dex+2 Str-2>
(25118,6573,1,1,2,13064),-- Blessed Scroll: Enchant Weapon (Grade C)
(25119,357,1,1,0,30553),-- Zubei\'s Breastplate
(25119,383,1,1,0,48876),-- Zubei\'s Gaiters
(25119,2376,1,1,0,30553),-- Avadon Breastplate
(25119,2379,1,1,0,48876),-- Avadon Gaiters
(25119,4056,16,48,0,137903),-- Zubei\'s Breastplate Part
(25119,4057,60,180,0,56833),-- Zubei\'s Gaiter Material
(25119,4064,23,67,0,98065),-- Avadon Breastplate Part
(25119,4065,11,31,0,324760),-- Avadon Gaiters Material
(25119,78,1,1,1,14598),-- Great Sword
(25119,142,1,1,1,14598),-- Keshanberk
(25119,148,1,1,1,14598),-- Sword of Valhalla
(25119,4104,5,15,1,279039),-- Great Sword Blade
(25119,4107,9,27,1,152620),-- Keshanberk Blade
(25119,4108,14,42,1,98112),-- Sword of Valhalla Blade
(25119,4607,5,15,2,105351),-- Greater Dye of Intelligence <Int+3 Men-3>
(25119,4608,2,4,2,351169),-- Greater Dye of Intelligence <Int+3 Wit-3>
(25119,4609,2,6,2,263376),-- Greater Dye of Mental Strength <Men+3 Int-3>
(25119,6573,1,1,2,31605),-- Blessed Scroll: Enchant Weapon (Grade C)
(25120,1539,1,1,2,154489),-- Greater Healing Potion
(25121,1342,20,60,2,695200),-- Fine Steel Arrow
(25122,2391,1,1,0,23097),-- Blue Wolf Leather Armor
(25122,2392,1,1,0,23097),-- Leather Armor of Doom
(25122,4082,24,72,0,82019),-- Blue Wolf Leather Armor Texture
(25122,4083,8,22,0,262459),-- Leather Armor of Doom Design
(25122,268,1,1,1,20716),-- Bellion Cestus
(25122,4120,3,9,1,761244),-- Bellion Cestus Edge
(25122,948,1,3,2,423425),-- Scroll: Enchant Armor (Grade B)
(25122,4616,2,4,2,155742),-- Greater Dye of Constitution <Con+4 Dex-4>
(25122,4617,2,6,2,116806),-- Greater Dye of Dexterity <Dex+4 Str-4>
(25122,4618,3,7,2,93445),-- Greater Dye of Dexterity <Dex+4 Con-4>
(25123,1539,1,1,2,196267),-- Greater Healing Potion
(25124,1342,20,60,2,883200),-- Fine Steel Arrow
(25125,5304,1,1,0,35962),-- Sealed Tallum Tunic
(25125,5305,1,1,0,57518),-- Sealed Tallum Stockings
(25125,5308,1,1,0,22127),-- Sealed Dark Crystal Robe
(25125,5485,9,27,0,224370),-- Sealed Tallum Tunic Texture
(25125,5486,20,60,0,71047),-- Sealed Dark Crystal Robe Fabric
(25125,5489,28,84,0,113198),-- Sealed Tallum Stockings Fabric
(25125,212,1,1,1,15330),-- Dasparion\'s Staff
(25125,2504,1,1,1,15330),-- Meteor Shower
(25125,5532,8,22,1,242934),-- Meteor Shower Head
(25125,5543,4,12,1,459799),-- Dasparion\'s Staff Edge
(25125,6569,1,1,2,18677),-- Blessed Scroll: Enchant Weapon (Grade A)
(25125,6570,1,1,2,155645),-- Blessed Scroll: Enchant Armor (Grade A)
(25126,6678,1,1,0,30143),-- Sealed Imperial Crusader Shield
(25126,6679,1,1,0,21099),-- Sealed Imperial Crusader Helmet
(25126,6683,1,1,0,21099),-- Sealed Draconic Leather Helmet
(25126,6687,1,1,0,21099),-- Sealed Major Arcana Circlet
(25126,6705,23,67,0,156706),-- Sealed Imperial Crusader Shield Part
(25126,6706,60,180,0,68355),-- Sealed Imperial Crusader Helmet Pattern
(25126,6710,116,346,0,35509),-- Sealed Draconic Leather Helmet Pattern
(25126,6714,16,48,0,256330),-- Sealed Major Arcana Circlet Pattern
(25126,6365,1,1,1,3548),-- Basalt Battlehammer
(25126,6366,1,1,1,3548),-- Imperial Staff
(25126,6369,1,1,1,3548),-- Dragon Hunter Axe
(25126,6579,1,1,1,3548),-- Arcana Mace
(25126,6689,16,48,1,50264),-- Basalt Battlehammer Head
(25126,6690,11,31,1,78212),-- Imperial Staff Head
(25126,6693,6,18,1,139086),-- Dragon Hunter Axe Blade
(25126,6697,3,7,1,321688),-- Arcana Mace Head
(25126,3936,1,3,2,491603),-- Blessed Scroll of Resurrection
(25126,6578,1,1,2,87396),-- Blessed Scroll: Enchant Armor (Grade S)
(25127,46,2,4,0,495636),-- Bronze Helmet
(25127,627,4,12,0,265321),-- Aspis
(25127,222,1,1,1,345695),-- Poniard Dagger
(25127,955,1,3,2,370072),-- Scroll: Enchant Weapon (Grade D)
(25127,6575,1,1,2,41119),-- Blessed Scroll: Enchant Weapon (Grade D)
(25127,6576,1,1,2,370072),-- Blessed Scroll: Enchant Armor (Grade D)
(25128,851,6,18,0,22046),-- Omen Beast\'s Eye Earring
(25128,882,5,15,0,39682),-- Mithril Ring
(25128,914,1,1,0,198411),-- Necklace of Darkness
(25128,1905,81,243,0,74105),-- Omen Beast\'s Eye Earring Gemstone
(25128,1906,40,120,0,212048),-- Mithril Ring Wire
(25128,1907,154,462,0,27874),-- Necklace of Darkness Gem
(25128,159,1,1,1,16016),-- Bonebreaker
(25128,187,1,1,1,16016),-- Atuba Hammer
(25128,188,1,1,1,16016),-- Ghost Staff
(25128,189,1,1,1,16016),-- Staff of Life
(25128,190,1,1,1,16016),-- Atuba Mace
(25128,2048,2,6,1,229594),-- Bonebreaker Head
(25128,2049,3,7,1,183676),-- Atuba Hammer Head
(25128,2050,7,21,1,69556),-- Ghost Staff Head
(25128,2051,12,36,1,40574),-- Staff of Life Shaft
(25128,2058,3,9,1,153061),-- Atuba Mace Head
(25128,951,1,1,2,436305),-- Scroll: Enchant Weapon (Grade C)
(25128,952,3,9,2,533261),-- Scroll: Enchant Armor (Grade C)
(25128,6573,1,1,2,15998),-- Blessed Scroll: Enchant Weapon (Grade C)
(25129,6036,1,1,2,10107),-- Greater Magic Haste Potion
(25130,1341,20,60,2,505333),-- Bone Arrow
(25131,2397,1,1,0,34264),-- Tunic of Zubei
(25131,2402,1,1,0,54810),-- Stockings of Zubei
(25131,2406,1,1,0,21076),-- Avadon Robe
(25131,4069,9,27,0,268604),-- Tunic of Zubei Fabric
(25131,4070,40,120,0,91763),-- Stockings of Zubei Fabric
(25131,4071,14,42,0,132656),-- Avadon Robe Fabric
(25131,91,1,1,1,13814),-- Heavy War Axe
(25131,92,1,1,1,13814),-- Sprite\'s Staff
(25131,4105,4,12,1,330058),-- Heavy War Axe Head
(25131,4106,8,22,1,173149),-- Sprite\'s Staff Head
(25131,4610,1,3,2,332294),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(25131,4611,2,4,2,221529),-- Greater Dye of Wit <Wit+3 Int-3>
(25131,4612,2,6,2,166147),-- Greater Dye of Wit <Wit+3 Men-3>
(25131,6574,1,1,2,170893),-- Blessed Scroll: Enchant Armor (Grade C)
(25132,1539,1,1,2,154489),-- Greater Healing Potion
(25133,1342,20,60,2,695200),-- Fine Steel Arrow
(25134,631,6,16,0,27504),-- Eldarake
(25134,1964,40,120,0,155854),-- Eldarake Temper
(25134,2413,1,1,0,211799),-- Chain Hood
(25134,2495,6,18,0,25212),-- Chain Shield
(25134,2936,140,420,0,40445),-- Chain Hood Pattern
(25134,2945,81,243,0,76965),-- Chain Shield Fragment
(25134,281,1,1,1,78771),-- Crystallized Ice Bow
(25134,2074,12,36,1,204903),-- Crystallized Ice Bow Shaft
(25134,4597,1,3,2,375022),-- Greater Dye of Mental Strength <Men+2 Int-2>
(25134,4598,2,4,2,250015),-- Greater Dye of Mental Strength <Men+2 Wit-2>
(25134,4599,2,6,2,187511),-- Greater Dye of Wit <Wit+2 Int-2>
(25134,6573,1,1,2,15001),-- Blessed Scroll: Enchant Weapon (Grade C)
(25135,1341,20,60,2,736000),-- Bone Arrow
(25136,6036,1,1,2,14720),-- Greater Magic Haste Potion
(25137,503,1,1,0,56421),-- Zubei\'s Helmet
(25137,633,1,1,0,80606),-- Zubei\'s Shield
(25137,673,1,1,0,80606),-- Avadon Shield
(25137,2415,1,1,0,56421),-- Avadon Circlet
(25137,4072,23,67,0,168546),-- Avadon Circlet Pattern
(25137,4101,116,346,0,42379),-- Zubei\'s Shield Fragment
(25137,4102,192,576,0,19751),-- Zubei\'s Helmet Design
(25137,4439,60,180,0,97051),-- Avadon Shield Fragment
(25137,267,1,1,1,30333),-- Arthro Nail
(25137,4111,12,36,1,241586),-- Arthro Nail Blade
(25137,4613,1,3,2,226452),-- Greater Dye of Strength <Str+4 Con-4>
(25137,4614,2,4,2,150968),-- Greater Dye of Strength <Str+4 Dex-4>
(25137,4615,1,1,2,452204),-- Greater Dye of Constitution <Con+4 Str-4>
(25137,6571,1,1,2,9382),-- Blessed Scroll: Enchant Weapon (Grade B)
(25138,1539,1,1,2,188978),-- Greater Healing Potion
(25139,1342,20,60,2,850400),-- Fine Steel Arrow
(25140,6323,1,1,0,71683),-- Sealed Phoenix Necklace
(25140,6324,1,1,0,95578),-- Sealed Phoenix Earring
(25140,6325,1,1,0,143366),-- Sealed Phoenix Ring
(25140,6341,81,243,0,81999),-- Sealed Phoenix Earring Gemstone
(25140,6343,140,420,0,36675),-- Sealed Phoenix Necklace Beads
(25140,6345,40,120,0,229665),-- Sealed Phoenix Ring Gemstone
(25140,98,1,1,1,30660),-- Halberd
(25140,5542,12,36,1,303675),-- Halberd Edge
(25140,729,1,1,2,103763),-- Scroll: Enchant Weapon (Grade A)
(25140,730,1,1,2,778226),-- Scroll: Enchant Armor (Grade A)
(25140,6569,1,1,2,12452),-- Blessed Scroll: Enchant Weapon (Grade A)
(25141,1343,20,60,2,871429),-- Silver Arrow
(25142,1343,20,60,2,871429),-- Silver Arrow
(25143,6680,1,1,0,26701),-- Sealed Draconic Leather Armor
(25143,6707,12,36,0,429334),-- Sealed Draconic Leather Armor Part
(25143,6371,1,1,1,14590),-- Demon Splinter
(25143,6695,12,36,1,275646),-- Demon Splinter Blade
(25143,959,1,1,2,53915),-- Scroll: Enchant Weapon (Grade S)
(25143,960,1,1,2,539155),-- Scroll: Enchant Armor (Grade S)
(25143,6577,1,1,2,8986),-- Blessed Scroll: Enchant Weapon (Grade S)
(25144,1344,60,180,2,391833),-- Mithril Arrow
(25145,1344,60,180,2,391833),-- Mithril Arrow
(25146,848,3,9,0,133493),-- Enchanted Earring
(25146,879,3,7,0,241022),-- Enchanted Ring
(25146,911,4,10,0,85882),-- Enchanted Necklace
(25146,1900,96,288,0,74853),-- Enchanted Necklace Chain
(25146,275,1,1,1,106040),-- Long Bow
(25146,276,1,1,1,106040),-- Elven Bow
(25146,2024,12,36,1,155932),-- Elven Bow Shaft
(25146,6575,1,1,2,56759),-- Blessed Scroll: Enchant Weapon (Grade D)
(25146,6576,1,1,2,510833),-- Blessed Scroll: Enchant Armor (Grade D)
(25147,735,1,1,2,13720),-- Potion of Alacrity
(25148,6035,1,1,2,13720),-- Magic Haste Potion
(25149,436,2,4,0,167392),-- Tunic of Knowledge
(25149,469,1,3,0,401327),-- Stockings of Knowledge
(25149,83,1,1,1,53336),-- Sword of Magic
(25149,124,1,1,1,39512),-- Two-Handed Sword
(25149,127,1,1,1,39512),-- Crimson Sword
(25149,130,1,1,1,39512),-- Elven Sword
(25149,144,1,1,1,53336),-- Sword of Occult
(25149,2026,3,9,1,294651),-- Two-Handed Sword Edge
(25149,6576,1,3,2,571224),-- Blessed Scroll: Enchant Armor (Grade D)
(25150,1060,1,1,2,450637),-- Lesser Healing Potion
(25151,1831,1,1,2,540764),-- Antidote
(25152,438,1,1,0,219924),-- Sage\'s Rag
(25152,1953,12,36,0,538011),-- Sage\'s Rag Lining
(25152,262,1,1,1,87193),-- Scallop Jamadhr
(25152,2053,12,36,1,208326),-- Scallop Jamadhr Edge
(25152,4594,1,3,2,326603),-- Greater Dye of Dexterity <Dex+2 Con-2>
(25152,4595,2,4,2,217735),-- Greater Dye of Intelligence <Int+2 Men-2>
(25152,4596,2,6,2,163301),-- Greater Dye of Intelligence <Int+2 Wit-2>
(25152,6574,1,1,2,111978),-- Blessed Scroll: Enchant Armor (Grade C)
(25153,1341,20,60,2,565333),-- Bone Arrow
(25154,6036,1,1,2,11307),-- Greater Magic Haste Potion
(25155,397,1,1,0,250826),-- Mithril Shirt
(25155,1961,53,157,0,111224),-- Mithril Shirt Fabric
(25155,2387,5,13,0,44536),-- Tempered Mithril Gaiters
(25155,2935,192,576,0,50526),-- Tempered Mithril Gaiters Fragment
(25155,226,1,1,1,62105),-- Cursed Dagger
(25155,232,1,1,1,62105),-- Dark Elven Dagger
(25155,2070,24,72,1,90557),-- Cursed Dagger Blade
(25155,2072,8,22,1,289781),-- Dark Elven Dagger Edge
(25155,4600,3,9,2,394238),-- Greater Dye of Wit <Wit+2 Men-2>
(25155,6574,1,1,2,405502),-- Blessed Scroll: Enchant Armor (Grade C)
(25156,1341,20,60,2,736000),-- Bone Arrow
(25157,6036,1,1,2,14720),-- Greater Magic Haste Potion
(25158,400,1,1,0,88525),-- Theca Leather Armor
(25158,401,1,1,0,41919),-- Drake Leather Armor
(25158,420,1,1,0,141703),-- Theca Leather Gaiters
(25158,1984,81,243,0,37192),-- Theca Leather Armor Pattern
(25158,1985,28,84,0,173071),-- Theca Leather Gaiters Pattern
(25158,1987,20,60,0,107203),-- Drake Leather Armor Texture
(25158,135,1,1,1,26834),-- Samurai Longsword
(25158,2115,8,22,1,193453),-- Samurai Longsword Blade
(25158,5286,1,1,1,26834),-- Berserker Blade
(25158,5530,4,12,1,362619),-- Berserker Blade Edge
(25158,4610,5,15,2,91087),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(25158,4611,2,4,2,303622),-- Greater Dye of Wit <Wit+3 Int-3>
(25158,4612,2,6,2,227717),-- Greater Dye of Wit <Wit+3 Men-3>
(25158,6574,1,1,2,234223),-- Blessed Scroll: Enchant Armor (Grade C)
(25159,357,1,1,0,20511),-- Zubei\'s Breastplate
(25159,383,1,1,0,32812),-- Zubei\'s Gaiters
(25159,2376,1,1,0,20511),-- Avadon Breastplate
(25159,2379,1,1,0,32812),-- Avadon Gaiters
(25159,4056,16,48,0,92580),-- Zubei\'s Breastplate Part
(25159,4057,60,180,0,38154),-- Zubei\'s Gaiter Material
(25159,4064,23,67,0,65835),-- Avadon Breastplate Part
(25159,4065,11,31,0,218024),-- Avadon Gaiters Material
(25159,284,1,1,1,29401),-- Dark Elven Long Bow
(25159,4112,12,36,1,234678),-- Dark Elven Longbow Shaft
(25159,4619,1,3,2,219494),-- Greater Dye of Intelligence <Int+4 Men-4>
(25159,4620,2,4,2,146330),-- Greater Dye of Intelligence <Int+4 Wit-4>
(25159,4621,1,1,2,438988),-- Greater Dye of Mental Strength <Men+4 Int-4>
(25159,6571,1,1,2,9094),-- Blessed Scroll: Enchant Weapon (Grade B)
(25160,1539,1,1,2,181689),-- Greater Healing Potion
(25161,1342,20,60,2,817600),-- Fine Steel Arrow
(25162,110,1,1,0,12285),-- Doom Shield
(25162,2416,1,1,0,85994),-- Blue Wolf Helmet
(25162,2417,1,1,0,85994),-- Doom Helmet
(25162,4077,40,120,0,224438),-- Doom Shield Fragment
(25162,4088,81,243,0,93115),-- Blue Wolf Helmet Design
(25162,4089,140,420,0,53873),-- Doom Helmet Pattern
(25162,287,1,1,1,35597),-- Bow of Peril
(25162,4121,12,36,1,325231),-- Bow of Peril Shaft
(25162,948,6,18,2,121265),-- Scroll: Enchant Armor (Grade B)
(25162,4622,2,4,2,267622),-- Greater Dye of Mental Strength <Men+4 Wit-4>
(25162,4623,2,6,2,200717),-- Greater Dye of Wit <Wit+4 Int-4>
(25162,4624,3,7,2,160573),-- Greater Dye of Wit <Wit+4 Men-4>
(25163,5320,1,1,0,22289),-- Sealed Leather Armor of Nightmare
(25163,5323,1,1,0,22289),-- Sealed Majestic Leather Armor
(25163,5480,4,12,0,412748),-- Sealed Leather Armor of Nightmare Fabric
(25163,5481,8,22,0,220132),-- Sealed Majestic Leather Armor Fabric
(25163,81,1,1,1,7096),-- Dragon Slayer
(25163,151,1,1,1,7096),-- Sword of Miracles 
(25163,2500,1,1,1,7096),-- Dark Legion\'s Edge
(25163,5529,9,27,1,112160),-- Dragon Slayer Edge
(25163,5545,14,42,1,72103),-- Dark Legion\'s Edge Blade
(25163,5546,5,15,1,221755),-- Sword of Miracles Edge
(25163,959,1,1,2,38156),-- Scroll: Enchant Weapon (Grade S)
(25163,960,1,1,2,381565),-- Scroll: Enchant Armor (Grade S)
(25163,6577,1,1,2,6360),-- Blessed Scroll: Enchant Weapon (Grade S)
(25166,849,4,10,0,132967),-- Tiger\'s Eye Earring
(25166,880,3,7,0,278665),-- Black Pearl Ring
(25166,912,4,12,0,87215),-- Near Forest Necklace
(25166,1901,72,216,0,220513),-- Tiger\'s Eye Earring Stone
(25166,157,1,1,1,123685),-- Spiked Club
(25166,186,1,1,1,123685),-- Staff of Magic
(25166,2027,12,36,1,250976),-- Spiked Club Head
(25166,6387,60,180,2,298017),-- Blessed Scroll of Resurrection for Pets
(25167,1833,1,1,2,540764),-- Bandage
(25168,734,1,1,2,33798),-- Haste Potion
(25169,606,10,30,0,10795),-- Rip Gauntlets
(25169,1124,11,31,0,10281),-- Boots of Power
(25169,1946,180,540,0,23260),-- Rip Gauntlets Pattern
(25169,1948,314,940,0,13355),-- Boots of Power Pattern
(25169,2424,11,33,0,9814),-- Manticore Skin Boots
(25169,2425,12,34,0,9387),-- Brigandine Boots
(25169,2426,12,36,0,8996),-- Elven Mithril Boots
(25169,2448,13,37,0,8636),-- Manticore Skin Gloves
(25169,2449,13,39,0,8303),-- Brigandine Gauntlets
(25169,2450,14,40,0,7997),-- Elven Mithril Gloves
(25169,2923,24,72,0,174446),-- Manticore Skin Boot Lining
(25169,2924,33,97,0,128821),-- Brigandine Boots Design
(25169,2925,42,126,0,99683),-- Elven Mithril Boots Design
(25169,2926,53,157,0,79746),-- Manticore Skin Gloves Lining
(25169,2927,64,192,0,65417),-- Brigandine Gauntlets Pattern
(25169,2928,77,229,0,54727),-- Elven Mithril Gloves Design
(25169,129,1,1,1,138464),-- Sword of Revolution
(25169,2034,12,36,1,327341),-- Sword of Revolution Blade
(25169,4553,5,15,2,96386),-- Greater Dye of Strength <Str+1 Con-1>
(25169,4554,2,4,2,321288),-- Greater Dye of Strength <Str+1 Dex-1>
(25169,4555,2,6,2,240966),-- Greater Dye of Constitution <Con+1 Str-1>
(25169,6575,1,1,2,53548),-- Blessed Scroll: Enchant Weapon (Grade D)
(25170,631,6,16,0,25658),-- Eldarake
(25170,1964,40,120,0,145399),-- Eldarake Temper
(25170,2413,1,1,0,197590),-- Chain Hood
(25170,2495,1,1,0,282245),-- Chain Shield
(25170,2936,140,420,0,37732),-- Chain Hood Pattern
(25170,2945,81,243,0,71802),-- Chain Shield Fragment
(25170,226,1,1,1,36743),-- Cursed Dagger
(25170,232,1,1,1,36743),-- Dark Elven Dagger
(25170,2070,4,12,1,321455),-- Cursed Dagger Blade
(25170,2072,8,22,1,171443),-- Dark Elven Dagger Edge
(25170,951,1,3,2,763333),-- Scroll: Enchant Weapon (Grade C)
(25171,1341,20,60,2,664000),-- Bone Arrow
(25172,6036,1,1,2,13280),-- Greater Magic Haste Potion
(25173,64,1,1,0,286396),-- Composite Boots
(25173,608,1,1,0,286396),-- Mithril Gauntlets
(25173,1981,53,157,0,151462),-- Mithril Gauntlets Design
(25173,2956,192,576,0,41415),-- Composite Boots Part
(25173,326,1,1,1,49092),-- Heathen\'s Book
(25173,2110,12,36,1,190501),-- Heathen\'s Book Page
(25173,4610,1,3,2,292053),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(25173,4611,2,4,2,194702),-- Greater Dye of Wit <Wit+3 Int-3>
(25173,4612,2,6,2,146027),-- Greater Dye of Wit <Wit+3 Men-3>
(25173,6574,1,1,2,150198),-- Blessed Scroll: Enchant Armor (Grade C)
(25174,1342,20,60,2,559200),-- Fine Steel Arrow
(25175,1539,1,1,2,124267),-- Greater Healing Potion
(25176,2384,1,1,0,37620),-- Zubei\'s Leather Shirt
(25176,2388,1,1,0,60178),-- Zubei\'s Leather Gaiters
(25176,2390,1,1,0,23140),-- Avadon Leather Armor
(25176,4066,9,27,0,294912),-- Zubei\'s Leather Shirt Fabric
(25176,4067,40,120,0,100750),-- Zubei\'s Leather Gaiter Texture
(25176,4068,14,42,0,145649),-- Avadon Leather Armor Lining
(25176,284,1,1,1,30333),-- Dark Elven Long Bow
(25176,4112,12,36,1,239022),-- Dark Elven Longbow Shaft
(25176,4616,1,3,2,226452),-- Greater Dye of Constitution <Con+4 Dex-4>
(25176,4617,2,4,2,150968),-- Greater Dye of Dexterity <Dex+4 Str-4>
(25176,4618,1,1,2,452904),-- Greater Dye of Dexterity <Dex+4 Con-4>
(25176,6572,1,1,2,87561),-- Blessed Scroll: Enchant Armor (Grade B),
(25177,1539,1,1,2,188978),-- Greater Healing Potion
(25178,1342,20,60,2,850400),-- Fine Steel Arrow
(25179,2391,1,1,0,25854),-- Blue Wolf Leather Armor
(25179,2392,1,1,0,25854),-- Leather Armor of Doom
(25179,4082,24,72,0,91811),-- Blue Wolf Leather Armor Texture
(25179,4083,8,22,0,293793),-- Leather Armor of Doom Design
(25179,234,1,1,1,23489),-- Demon\'s Sword
(25179,4119,12,36,1,213393),-- Demon\'s Sword Edge
(25179,947,1,1,2,202229),-- Scroll: Enchant Weapon (Grade B)
(25179,948,1,3,2,631966),-- Scroll: Enchant Armor (Grade B)
(25179,6571,1,1,2,14445),-- Blessed Scroll: Enchant Weapon (Grade B)
(25180,1539,1,1,2,227733),-- Greater Healing Potion
(25181,1342,60,180,2,341600),-- Fine Steel Arrow
(25182,5290,1,1,0,52614),-- Sealed Dark Crystal Gloves
(25182,5291,1,1,0,52614),-- Sealed Dark Crystal Boots
(25182,5295,1,1,0,52614),-- Sealed Tallum Gloves
(25182,5296,1,1,0,52614),-- Sealed Tallum Boots
(25182,5496,116,346,0,23765),-- Sealed Dark Crystal Boots Lining
(25182,5497,23,67,0,121994),-- Sealed Tallum Boots Lining
(25182,5508,60,180,0,45747),-- Sealed Dark Crystal Gloves Design
(25182,5509,16,48,0,171552),-- Sealed Tallum Gloves Design
(25182,80,1,1,1,9965),-- Tallum Blade
(25182,150,1,1,1,9965),-- Elemental Sword
(25182,5547,8,22,1,159315),-- Elemental Sword Edge
(25182,5548,4,12,1,298716),-- Tallum Blade Edge
(25182,729,1,1,2,101169),-- Scroll: Enchant Weapon (Grade A)
(25182,6570,1,1,2,101169),-- Blessed Scroll: Enchant Armor (Grade A)
(25183,1343,20,60,2,871429),-- Silver Arrow
(25184,1343,20,60,2,871429),-- Silver Arrow
(25185,851,6,18,0,24004),-- Omen Beast\'s Eye Earring
(25185,882,5,15,0,43207),-- Mithril Ring
(25185,914,1,1,0,216035),-- Necklace of Darkness
(25185,1905,81,243,0,80688),-- Omen Beast\'s Eye Earring Gemstone
(25185,1906,40,120,0,230882),-- Mithril Ring Wire
(25185,1907,154,462,0,30350),-- Necklace of Darkness Gem
(25185,280,1,1,1,87193),-- Light Crossbow
(25185,2054,12,36,1,214425),-- Light Crossbow Shaft
(25185,951,3,7,2,71258),-- Scroll: Enchant Weapon (Grade C)
(25185,4597,1,3,2,326602),-- Greater Dye of Mental Strength <Men+2 Int-2>
(25185,4598,2,4,2,217735),-- Greater Dye of Mental Strength <Men+2 Wit-2>
(25185,4599,2,6,2,163301),-- Greater Dye of Wit <Wit+2 Int-2>
(25186,1341,20,60,2,565333),-- Bone Arrow
(25187,6036,1,1,2,11307),-- Greater Magic Haste Potion
(25188,629,5,15,0,37932),-- Kite Shield
(25188,1947,336,1008,0,24505),-- Kite Shield Fragment
(25188,2411,1,1,0,265656),-- Brigandine Helmet
(25188,2493,6,16,0,34483),-- Brigandine Shield
(25188,2922,27,81,0,81566),-- Brigandine Helmet Design
(25188,2929,60,180,0,137227),-- Brigandine Shield Fragment
(25188,88,1,1,1,19167),-- Morning Star
(25188,90,1,1,1,19167),-- Goat Head Staff
(25188,158,1,1,1,19167),-- Tarbar
(25188,169,1,1,1,19167),-- Skull Breaker
(25188,172,1,1,1,19167),-- Heavy Bone Club
(25188,2031,7,21,1,74424),-- Morning Star Head
(25188,2032,3,9,1,158525),-- Goat Head Staff Head
(25188,2035,12,36,1,43415),-- Tarbar Head
(25188,2036,18,54,1,28943),-- Skull Breaker Head
(25188,2037,3,7,1,208389),-- Heavy Bone Club Head
(25188,4556,5,15,2,66713),-- Greater Dye of Constitution <Con+1 Dex-1>
(25188,4557,2,4,2,222377),-- Greater Dye of Dexterity <Dex+1 Str-1>
(25188,4558,2,6,2,166783),-- Greater Dye of Dexterity <Dex+1 Con-1>
(25188,6576,1,1,2,333565),-- Blessed Scroll: Enchant Armor (Grade D)
(25189,438,1,1,0,210544),-- Sage\'s Rag
(25189,1953,12,36,0,515065),-- Sage\'s Rag Lining
(25189,280,1,1,1,83474),-- Light Crossbow
(25189,2054,12,36,1,205280),-- Light Crossbow Shaft
(25189,952,12,36,2,416897),-- Scroll: Enchant Armor (Grade C)
(25190,6036,1,1,2,10720),-- Greater Magic Haste Potion
(25191,1341,20,60,2,536000),-- Bone Arrow
(25192,440,1,1,0,165586),-- Robe of Seal
(25192,1970,12,36,0,465225),-- Robe of Seal Fabric
(25192,325,1,1,1,67881),-- Horn of Glory
(25192,2088,12,36,1,220235),-- Horn of Glory Fragment
(25192,6574,1,1,2,555541),-- Blessed Scroll: Enchant Armor (Grade C)
(25193,6036,1,1,2,16987),-- Greater Magic Haste Potion
(25194,1341,20,60,2,849333),-- Bone Arrow
(25198,5311,1,1,0,16717),-- Sealed Armor of Nightmare
(25198,5316,1,1,0,16717),-- Sealed Majestic Plate Armor
(25198,5522,6,18,0,313255),-- Sealed Armor of Nightmare Pattern
(25198,5523,11,31,0,179003),-- Sealed Majestic Plate Armor Pattern
(25198,164,1,1,1,7096),-- Elysian
(25198,213,1,1,1,7096),-- Branch of The Mother Tree
(25198,5533,16,48,1,94616),-- Elysian Head
(25198,5544,23,67,1,67215),-- Branch of The Mother Tree Head
(25198,6569,1,1,2,19078),-- Blessed Scroll: Enchant Weapon (Grade A)
(25198,6578,1,1,2,63594),-- Blessed Scroll: Enchant Armor (Grade S)
(25199,6326,1,1,0,74440),-- Sealed Majestic Necklace
(25199,6327,1,1,0,99287),-- Sealed Majestic Earring
(25199,6328,1,1,0,148880),-- Sealed Majestic Ring
(25199,6342,90,270,0,140381),-- Sealed Majestic Earring Gemstone
(25199,6344,240,720,0,4068),-- Sealed Majestic Necklace Beads
(25199,6346,308,924,0,58957),-- Sealed Majestic Ring Gemstone
(25199,270,1,1,1,7171),-- Dragon Grinder
(25199,5538,5,13,1,339970),-- Dragon Grinder Edge
(25199,6570,1,1,2,160666),-- Blessed Scroll: Enchant Armor (Grade A)
(25199,6578,1,1,2,64267),-- Blessed Scroll: Enchant Armor (Grade S)
(25200,1344,20,60,2,957000),-- Mithril Arrow
(25201,1344,20,60,2,957000),-- Mithril Arrow
(25202,5326,1,1,0,18186),-- Sealed Robe of Nightmare
(25202,5329,1,1,0,18186),-- Sealed Majestic Robe
(25202,5487,7,21,0,289680),-- Sealed Robe of Nightmare Fabric
(25202,5488,12,36,0,168980),-- Sealed Majestic Robe Fabric
(25202,81,1,1,1,5790),-- Dragon Slayer
(25202,151,1,1,1,5790),-- Sword of Miracles 
(25202,2500,1,1,1,5790),-- Dark Legion\'s Edge
(25202,5529,25,75,1,49416),-- Dragon Slayer Edge
(25202,5545,33,99,1,37437),-- Dark Legion\'s Edge Blade
(25202,5546,18,54,1,75387),-- Sword of Miracles Edge
(25202,959,1,1,2,38915),-- Scroll: Enchant Weapon (Grade S)
(25202,960,1,1,2,389154),-- Scroll: Enchant Armor (Grade S)
(25202,6577,1,1,2,6486),-- Blessed Scroll: Enchant Weapon (Grade S)
(25203,1344,60,180,2,339000),-- Mithril Arrow
(25204,1344,60,180,2,339000),-- Mithril Arrow
(25205,6678,1,1,0,48238),-- Sealed Imperial Crusader Shield
(25205,6679,1,1,0,33766),-- Sealed Imperial Crusader Helmet
(25205,6683,1,1,0,33766),-- Sealed Draconic Leather Helmet
(25205,6687,1,1,0,33766),-- Sealed Major Arcana Circlet
(25205,6705,315,945,0,26869),-- Sealed Imperial Crusader Shield Part
(25205,6706,66,198,0,149168),-- Sealed Imperial Crusader Helmet Pattern
(25205,6710,126,378,0,78136),-- Sealed Draconic Leather Helmet Pattern
(25205,6714,208,624,0,47332),-- Sealed Major Arcana Circlet Pattern
(25205,6371,1,1,1,3785),-- Demon Splinter
(25205,6695,5,15,1,257418),-- Demon Splinter Blade
(25205,3936,1,3,2,491699),-- Blessed Scroll of Resurrection
(25205,6578,1,1,2,87413),-- Blessed Scroll: Enchant Armor (Grade S)
(25206,1344,60,180,2,359667),-- Mithril Arrow
(25207,1344,60,180,2,359667),-- Mithril Arrow
(25208,354,1,1,0,119116),-- Chain Mail Shirt
(25208,381,1,1,0,190566),-- Chain Gaiters
(25208,439,1,1,0,238600),-- Karmian Tunic
(25208,471,1,1,0,381288),-- Karmian Stockings
(25208,1954,192,576,0,48063),-- Karmian Stocking Design
(25208,1962,53,157,0,105802),-- Karmian Tunic Pattern
(25208,1959,12,36,0,259755),-- Chain Mail Shirt Material
(25208,1960,105,315,0,60389),-- Chain Gaiters Part
(25208,951,1,1,2,545486),-- Scroll: Enchant Weapon (Grade C)
(25208,952,12,36,2,166676),-- Scroll: Enchant Armor (Grade C)
(25208,6573,1,1,2,20001),-- Blessed Scroll: Enchant Weapon (Grade C)
(25209,1341,20,60,2,736000),-- Bone Arrow
(25210,6036,1,1,2,14720),-- Greater Magic Haste Potion
(25211,851,6,18,0,25077),-- Omen Beast\'s Eye Earring
(25211,882,5,15,0,45138),-- Mithril Ring
(25211,914,1,1,0,225691),-- Necklace of Darkness
(25211,1905,81,243,0,84294),-- Omen Beast\'s Eye Earring Gemstone
(25211,1906,40,120,0,241203),-- Mithril Ring Wire
(25211,1907,154,462,0,31707),-- Necklace of Darkness Gem
(25211,262,1,1,1,91090),-- Scallop Jamadhr
(25211,2053,12,36,1,217639),-- Scallop Jamadhr Edge
(25211,6573,1,1,2,42938),-- Blessed Scroll: Enchant Weapon (Grade C)
(25212,1341,20,60,2,597333),-- Bone Arrow
(25213,6036,1,1,2,11947),-- Greater Magic Haste Potion
(25214,439,1,1,0,159067),-- Karmian Tunic
(25214,471,1,1,0,254192),-- Karmian Stockings
(25214,1954,24,72,0,256338),-- Karmian Stocking Design
(25214,1962,53,157,0,70534),-- Karmian Tunic Pattern
(25214,96,1,1,1,26257),-- Scythe
(25214,298,1,1,1,26257),-- Orcish Glaive
(25214,302,1,1,1,26257),-- Body Slasher
(25214,2062,5,15,1,181525),-- Scythe Edge
(25214,2075,9,27,1,100847),-- Orcish Glaive Blade
(25214,2076,14,42,1,64831),-- Body Slasher Blade
(25214,6574,1,1,2,514316),-- Blessed Scroll: Enchant Armor (Grade C)
(25215,1341,20,60,2,736000),-- Bone Arrow
(25216,6036,1,1,2,14720),-- Greater Magic Haste Potion
(25217,856,1,1,0,88033),-- Adamantite Earring
(25217,887,1,1,0,132049),-- Adamantite Ring
(25217,918,1,1,0,66024),-- Adamantite Necklace
(25217,4049,81,243,0,66398),-- Adamantite Earring Gemstone
(25217,4050,40,120,0,158823),-- Adamantite Ring Wire
(25217,4051,14,42,0,284041),-- Adamantite Necklace Chain
(25217,267,1,1,1,27627),-- Arthro Nail
(25217,4111,12,36,1,220035),-- Arthro Nail Blade
(25217,951,3,7,2,108751),-- Scroll: Enchant Weapon (Grade C)
(25217,4601,1,3,2,332294),-- Greater Dye of Strength <Str+3 Con-3>
(25217,4602,2,4,2,221530),-- Greater Dye of Strength <Str+3 Dex-3>
(25217,4603,2,6,2,166147),-- Greater Dye of Constitution <Con+3 Str-3>
(25218,1539,1,1,2,154489),-- Greater Healing Potion
(25219,1342,20,60,2,695200),-- Fine Steel Arrow
(25220,5320,1,1,0,22627),-- Sealed Leather Armor of Nightmare
(25220,5323,1,1,0,22627),-- Sealed Majestic Leather Armor
(25220,5480,4,12,0,419012),-- Sealed Leather Armor of Nightmare Fabric
(25220,5481,8,22,0,223473),-- Sealed Majestic Leather Armor Fabric
(25220,289,1,1,1,21611),-- Soul Bow
(25220,5534,12,36,1,256363),-- Soul Bow Stave
(25220,959,1,1,2,38735),-- Scroll: Enchant Weapon (Grade S)
(25220,960,1,1,2,387355),-- Scroll: Enchant Armor (Grade S)
(25220,6577,1,1,2,6456),-- Blessed Scroll: Enchant Weapon (Grade S)
(25221,1344,20,60,2,986500),-- Mithril Arrow
(25222,1344,20,60,2,986500),-- Mithril Arrow
(25223,61,9,25,0,10511),-- Mithril Gloves
(25223,607,9,27,0,9927),-- Ogre Power Gauntlets
(25223,1125,10,28,0,9405),-- Assault Boots
(25223,1949,112,336,0,33477),-- Mithril Gloves Design
(25223,1957,320,960,0,11717),-- Assault Boots Part
(25223,1963,203,607,0,18516),-- Ogre Power Gauntlets Part
(25223,2427,10,30,0,8935),-- Salamander Skin Boots
(25223,2428,1,1,0,178695),-- Plate Boots
(25223,2451,1,1,0,178695),-- Sage\'s Worn Gloves
(25223,2931,28,82,0,136342),-- Salamander Skin Boot Lining
(25223,2932,36,108,0,104150),-- Plate Boots Design
(25223,2933,46,136,0,82406),-- Sage\'s Worn Gloves Lining
(25223,225,1,1,1,87193),-- Mithril Dagger
(25223,2052,12,36,1,218498),-- Mithril Dagger Blade
(25223,952,15,45,2,174188),-- Scroll: Enchant Armor (Grade C)
(25223,4600,2,6,2,326602),-- Greater Dye of Wit <Wit+2 Men-2>
(25224,1341,20,60,2,565333),-- Bone Arrow
(25225,6036,1,1,2,11307),-- Greater Magic Haste Potion
(25226,6323,1,1,0,70713),-- Sealed Phoenix Necklace
(25226,6324,1,1,0,94285),-- Sealed Phoenix Earring
(25226,6325,1,1,0,141426),-- Sealed Phoenix Ring
(25226,6341,81,243,0,80890),-- Sealed Phoenix Earring Gemstone
(25226,6343,140,420,0,36179),-- Sealed Phoenix Necklace Beads
(25226,6345,40,120,0,226558),-- Sealed Phoenix Ring Gemstone
(25226,80,1,1,1,15123),-- Tallum Blade
(25226,150,1,1,1,15123),-- Elemental Sword
(25226,5547,8,22,1,241786),-- Elemental Sword Edge
(25226,5548,4,12,1,453348),-- Tallum Blade Edge
(25226,729,1,1,2,102359),-- Scroll: Enchant Weapon (Grade A)
(25226,730,1,1,2,767967),-- Scroll: Enchant Armor (Grade A)
(25226,6569,1,1,2,12283),-- Blessed Scroll: Enchant Weapon (Grade A)
(25227,1539,1,1,2,244444),-- Greater Healing Potion
(25228,1342,60,180,2,366667),-- Fine Steel Arrow
(25229,6678,1,1,0,22482),-- Sealed Imperial Crusader Shield
(25229,6679,1,1,0,15737),-- Sealed Imperial Crusader Helmet
(25229,6683,1,1,0,15737),-- Sealed Draconic Leather Helmet
(25229,6687,1,1,0,15737),-- Sealed Major Arcana Circlet
(25229,6705,23,67,0,116879),-- Sealed Imperial Crusader Shield Part
(25229,6706,60,180,0,50982),-- Sealed Imperial Crusader Helmet Pattern
(25229,6710,11,31,0,291329),-- Sealed Draconic Leather Helmet Pattern
(25229,6714,16,48,0,191185),-- Sealed Major Arcana Circlet Pattern
(25229,7575,1,1,1,10584),-- Draconic Bow
(25229,7579,12,36,1,206149),-- Draconic Bow Shaft
(25229,3936,1,1,2,733315),-- Blessed Scroll of Resurrection
(25229,6578,1,1,2,65184),-- Blessed Scroll: Enchant Armor (Grade S)
(25230,358,1,1,0,19346),-- Blue Wolf Breastplate
(25230,2380,1,1,0,30949),-- Blue Wolf Gaiters
(25230,2381,1,1,0,11905),-- Doom Plate Armor
(25230,4078,9,27,0,194279),-- Blue Wolf Breastplate Part
(25230,4080,28,84,0,87938),-- Blue Wolf Gaiters Material
(25230,4081,20,60,0,58991),-- Doom Plate Armor Temper
(25230,97,1,1,1,21351),-- Lance
(25230,4115,12,36,1,195823),-- Lance Blade
(25230,948,3,9,2,290942),-- Scroll: Enchant Armor (Grade B)
(25230,6572,1,1,2,186203),-- Blessed Scroll: Enchant Armor (Grade B)
(25231,1539,1,1,2,203911),-- Greater Healing Potion
(25232,1342,20,60,2,917600),-- Fine Steel Arrow
(25233,6326,1,1,0,29742),-- Sealed Majestic Necklace
(25233,6327,1,1,0,39670),-- Sealed Majestic Earring
(25233,6328,1,1,0,59485),-- Sealed Majestic Ring
(25233,6342,9,27,0,373928),-- Sealed Majestic Earring Gemstone
(25233,6344,20,60,0,108894),-- Sealed Majestic Necklace Beads
(25233,6346,28,84,0,172744),-- Sealed Majestic Ring Gemstone
(25233,236,1,1,1,12893),-- Soul Separator
(25233,5537,3,9,1,611370),-- Soul Separator Head
(25233,6569,1,1,2,11555),-- Blessed Scroll: Enchant Weapon (Grade A)
(25233,6578,1,1,2,38516),-- Blessed Scroll: Enchant Armor (Grade S)
(25234,5289,1,1,0,46286),-- Sealed Dark Crystal Helmet
(25234,5292,1,1,0,66137),-- Sealed Dark Crystal Shield
(25234,5294,1,1,0,46286),-- Sealed Tallum Helmet
(25234,5494,40,120,0,81170),-- Sealed Dark Crystal Shield Fragment
(25234,5525,9,27,0,265532),-- Sealed Dark Crystal Helmet Design
(25234,5526,14,42,0,170700),-- Sealed Tallum Helm Design
(25234,212,1,1,1,9869),-- Dasparion\'s Staff
(25234,2504,1,1,1,9869),-- Meteor Shower
(25234,5532,8,22,1,156395),-- Meteor Shower Head
(25234,5543,4,12,1,296007),-- Dasparion\'s Staff Edge
(25234,729,1,1,2,66800),-- Scroll: Enchant Weapon (Grade A)
(25234,730,1,1,2,501003),-- Scroll: Enchant Armor (Grade A)
(25234,6569,1,1,2,8016),-- Blessed Scroll: Enchant Weapon (Grade A)
(25235,5313,1,1,0,54882),-- Sealed Gloves of Nightmare
(25235,5314,1,1,0,54882),-- Sealed Boots of Nightmare
(25235,5318,1,1,0,54882),-- Sealed Majestic Gloves
(25235,5319,1,1,0,54882),-- Sealed Majestic Boots
(25235,5502,116,346,0,27185),-- Sealed Boots of Nightmare Lining
(25235,5503,23,67,0,139553),-- Sealed Majestic Boots Lining
(25235,5514,60,180,0,52332),-- Sealed Gloves of Nightmare Design
(25235,5515,16,48,0,196246),-- Sealed Majestic Gloves Design
(25235,289,1,1,1,21513),-- Soul Bow
(25235,5534,12,36,1,255202),-- Soul Bow Stave
(25235,959,1,1,2,38560),-- Scroll: Enchant Weapon (Grade S)
(25235,960,1,1,2,385601),-- Scroll: Enchant Armor (Grade S)
(25235,6577,1,1,2,6427),-- Blessed Scroll: Enchant Weapon (Grade S)
(25236,1344,20,60,2,957000),-- Mithril Arrow
(25237,1344,20,60,2,957000),-- Mithril Arrow
(25238,5289,1,1,0,49902),-- Sealed Dark Crystal Helmet
(25238,5292,1,1,0,71304),-- Sealed Dark Crystal Shield
(25238,5294,1,1,0,49902),-- Sealed Tallum Helmet
(25238,5494,40,120,0,87511),-- Sealed Dark Crystal Shield Fragment
(25238,5525,9,27,0,286277),-- Sealed Dark Crystal Helmet Design
(25238,5526,14,42,0,184036),-- Sealed Tallum Helm Design
(25238,80,1,1,1,10640),-- Tallum Blade
(25238,150,1,1,1,10640),-- Elemental Sword
(25238,5547,8,22,1,170118),-- Elemental Sword Edge
(25238,5548,4,12,1,318971),-- Tallum Blade Edge
(25238,729,1,1,2,72019),-- Scroll: Enchant Weapon (Grade A)
(25238,730,1,1,2,540144),-- Scroll: Enchant Armor (Grade A)
(25238,6569,1,1,2,8642),-- Blessed Scroll: Enchant Weapon (Grade A)
(25239,1343,20,60,2,932000),-- Silver Arrow
(25240,1343,20,60,2,932000),-- Silver Arrow
(25241,357,1,1,0,36396),-- Zubei\'s Breastplate
(25241,383,1,1,0,58222),-- Zubei\'s Gaiters
(25241,2376,1,1,0,36396),-- Avadon Breastplate
(25241,2379,1,1,0,58222),-- Avadon Gaiters
(25241,4056,16,48,0,164277),-- Zubei\'s Breastplate Part
(25241,4057,60,180,0,67702),-- Zubei\'s Gaiter Material
(25241,4064,23,67,0,116819),-- Avadon Breastplate Part
(25241,4065,116,346,0,35170),-- Avadon Gaiters Material
(25241,229,1,1,1,26086),-- Kris
(25241,243,1,1,1,26086),-- Hell Knife
(25241,4109,24,72,1,102267),-- Kris Edge
(25241,4110,8,22,1,327255),-- Hell Knife Edge
(25241,947,1,1,2,225898),-- Scroll: Enchant Weapon (Grade B)
(25241,4619,5,15,2,77896),-- Greater Dye of Intelligence <Int+4 Men-4>
(25241,4620,2,4,2,259653),-- Greater Dye of Intelligence <Int+4 Wit-4>
(25241,4621,2,6,2,194740),-- Greater Dye of Mental Strength <Men+4 Int-4>
(25242,1539,1,1,2,188978),-- Greater Healing Potion
(25243,1342,20,60,2,850400),-- Fine Steel Arrow
(25244,6680,1,1,0,19369),-- Sealed Draconic Leather Armor
(25244,6707,12,36,0,311439),-- Sealed Draconic Leather Armor Part
(25244,6367,1,1,1,10584),-- Angel Slayer
(25244,6691,12,36,1,203791),-- Angel Slayer Blade
(25244,959,1,1,2,39110),-- Scroll: Enchant Weapon (Grade S)
(25244,960,1,1,2,391104),-- Scroll: Enchant Armor (Grade S)
(25244,6577,1,1,2,6519),-- Blessed Scroll: Enchant Weapon (Grade S)
(25245,6680,1,1,0,25961),-- Sealed Draconic Leather Armor
(25245,6707,12,36,0,417437),-- Sealed Draconic Leather Armor Part
(25245,7575,1,1,1,14186),-- Draconic Bow
(25245,7579,12,36,1,276311),-- Draconic Bow Shaft
(25245,959,1,1,2,52421),-- Scroll: Enchant Weapon (Grade S)
(25245,960,1,1,2,524215),-- Scroll: Enchant Armor (Grade S)
(25245,6577,1,1,2,8737),-- Blessed Scroll: Enchant Weapon (Grade S)
(25246,1344,60,180,2,381000),-- Mithril Arrow
(25247,1344,60,180,2,381000),-- Mithril Arrow
(25248,5312,1,1,0,48796),-- Sealed Helm of Nightmare
(25248,5315,1,1,0,69729),-- Sealed Shield of Nightmare
(25248,5317,1,1,0,48796),-- Sealed Majestic Circlet
(25248,5495,40,120,0,94103),-- Sealed Shield of Nightmare Fragment
(25248,5527,81,243,0,36589),-- Sealed Helm of Nightmare Design
(25248,5528,14,42,0,211964),-- Sealed Majestic Circlet Design
(25248,236,1,1,1,21513),-- Soul Separator
(25248,5537,12,36,1,255028),-- Soul Separator Head
(25248,730,1,3,2,602496),-- Scroll: Enchant Armor (Grade A)
(25248,6578,1,1,2,64267),-- Blessed Scroll: Enchant Armor (Grade S)
(25249,6674,1,1,0,29507),-- Sealed Imperial Crusader Breastplate
(25249,6675,1,1,0,47213),-- Sealed Imperial Crusader Gaiters
(25249,6701,53,157,0,95615),-- Sealed Imperial Crusader Breastplate Part
(25249,6702,24,72,0,269241),-- Sealed Imperial Crusader Gaiters Pattern
(25249,3936,1,1,2,733315),-- Blessed Scroll of Resurrection
(25249,6578,1,1,2,65184),-- Blessed Scroll: Enchant Armor (Grade S)
(25250,1344,60,180,2,349167),-- Mithril Arrow
(25251,1344,60,180,2,349167),-- Mithril Arrow
(25252,5326,1,1,0,22289),-- Sealed Robe of Nightmare
(25252,5329,1,1,0,22289),-- Sealed Majestic Robe
(25252,5487,4,12,0,414216),-- Sealed Robe of Nightmare Fabric
(25252,5488,8,22,0,220916),-- Sealed Majestic Robe Fabric
(25252,270,1,1,1,21287),-- Dragon Grinder
(25252,5538,12,36,1,252310),-- Dragon Grinder Edge
(25252,959,1,1,2,38156),-- Scroll: Enchant Weapon (Grade S)
(25252,960,1,1,2,381565),-- Scroll: Enchant Armor (Grade S)
(25252,6577,1,1,2,6360),-- Blessed Scroll: Enchant Weapon (Grade S)
(25253,1344,20,60,2,898500),-- Mithril Arrow
(25254,1344,20,60,2,898500),-- Mithril Arrow
(25255,5297,1,1,0,35962),-- Sealed Dark Crystal Leather Armor
(25255,5298,1,1,0,57518),-- Sealed Dark Crystalline Leggings
(25255,5301,1,1,0,22127),-- Sealed Tallum Leather Armor
(25255,5478,9,27,0,224370),-- Sealed Dark Crystal Leather Armor Pattern
(25255,5479,20,60,0,71047),-- Sealed Tallum Leather Armor Pattern
(25255,5482,28,84,0,113198),-- Sealed Dark Crystalline Leggings Design
(25255,269,1,1,1,30660),-- Blood Tornado
(25255,5539,12,36,1,303675),-- Blood Tornado Edge
(25255,6569,1,1,2,18677),-- Blessed Scroll: Enchant Weapon (Grade A)
(25255,6570,1,1,2,155645),-- Blessed Scroll: Enchant Armor (Grade A)
(25256,358,1,1,0,21011),-- Blue Wolf Breastplate
(25256,2380,1,1,0,33613),-- Blue Wolf Gaiters
(25256,2381,1,1,0,12930),-- Doom Plate Armor
(25256,6323,1,1,1,58211),-- Sealed Phoenix Necklace
(25256,6324,1,1,1,77615),-- Sealed Phoenix Earring
(25256,6325,1,1,1,116422),-- Sealed Phoenix Ring
(25256,6341,81,243,1,665880),-- Sealed Phoenix Earring Gemstone
(25256,6343,140,420,1,29782),-- Sealed Phoenix Necklace Beads
(25256,6345,40,120,1,186503),-- Sealed Phoenix Ring Gemstone
(25256,4078,9,27,0,211000),-- Blue Wolf Breastplate Part
(25256,4080,28,84,0,95506),-- Blue Wolf Gaiters Material
(25256,4081,20,60,0,64068),-- Doom Plate Armor Temper
(25256,6571,1,1,2,21667),-- Blessed Scroll: Enchant Weapon (Grade B)
(25256,6572,1,1,2,202230),-- Blessed Scroll: Enchant Armor (Grade B)
(25257,1539,1,1,2,227733),-- Greater Healing Potion
(25258,1342,60,180,2,341600),-- Fine Steel Arrow
(25259,2397,1,1,0,64704),-- Tunic of Zubei
(25259,2402,1,1,0,103503),-- Stockings of Zubei
(25259,2406,1,1,0,39800),-- Avadon Robe
(25259,97,1,1,1,34637),-- Lance  
(25259,4115,12,36,1,316759),-- Lance Blade  
(25259,4069,81,243,0,56359),-- Tunic of Zubei Fabric
(25259,4070,40,120,0,173285),-- Stockings of Zubei Fabric
(25259,4071,14,42,0,250509),-- Avadon Robe Fabric
(25259,948,6,18,2,117655),-- Scroll: Enchant Armor (Grade B)
(25259,4622,2,4,2,259654),-- Greater Dye of Mental Strength <Men+4 Wit-4>
(25259,4623,2,6,2,194740),-- Greater Dye of Wit <Wit+4 Int-4>
(25259,4624,3,7,2,155792),-- Greater Dye of Wit <Wit+4 Men-4>
(25260,103,7,19,0,15941),-- Tower Shield
(25260,107,7,21,0,14802),-- Composite Shield
(25260,517,1,1,0,145109),-- Composite Helmet
(25260,1149,1,1,0,145109),-- Shining Circlet
(25260,1978,60,180,0,109762),-- Tower Shield Fragment
(25260,1983,23,67,0,200267),-- Shining Circlet Pattern
(25260,2957,192,576,0,23469),-- Composite Helmet Design
(25260,4055,116,346,0,57019),-- Composite Shield Fragment
(25260,75,1,1,1,12437),-- Caliburs
(25260,76,1,1,1,12437),-- Sword of Delusion
(25260,77,1,1,1,12437),-- Tsurugi
(25260,84,1,1,1,12437),-- Homunkulus\'s Sword
(25260,134,1,1,1,12437),-- Sword of Nightmare
(25260,145,1,1,1,12437),-- Sword of Whispering Death
(25260,2089,8,24,1,69984),-- Caliburs Edge
(25260,2090,14,40,1,41472),-- Delusional Blade
(25260,2091,20,60,1,27994),-- Tsurugi Blade
(25260,2092,3,7,1,223950),-- Homunkulus\'s Sword Blade
(25260,2095,3,9,1,186625),-- Sword of Nightmare Blade
(25260,2096,4,10,1,159964),-- Sword of Whispering Death Blade
(25260,951,3,7,2,145286),-- Scroll: Enchant Weapon (Grade C)
(25260,4601,6,18,2,73988),-- Greater Dye of Strength <Str+3 Con-3>
(25260,4602,2,4,2,295953),-- Greater Dye of Strength <Str+3 Dex-3>
(25260,4603,2,6,2,221965),-- Greater Dye of Constitution <Con+3 Str-3>
(25261,1342,20,60,2,559200),-- Fine Steel Arrow
(25262,1539,1,1,2,124267),-- Greater Healing Potion
(25263,5297,1,1,0,36252),-- Sealed Dark Crystal Leather Armor
(25263,5298,1,1,0,57982),-- Sealed Dark Crystalline Leggings
(25263,5301,1,1,0,22306),-- Sealed Tallum Leather Armor
(25263,5478,9,27,0,226178),-- Sealed Dark Crystal Leather Armor Pattern
(25263,5479,20,60,0,71620),-- Sealed Tallum Leather Armor Pattern
(25263,5482,28,84,0,114109),-- Sealed Dark Crystalline Leggings Design
(25263,212,1,1,1,15454),-- Dasparion\'s Staff
(25263,2504,1,1,1,15454),-- Meteor Shower
(25263,5532,8,22,1,244891),-- Meteor Shower Head
(25263,5543,4,12,1,4635030),-- Dasparion\'s Staff Edge
(25263,729,1,1,2,156899),-- Scroll: Enchant Weapon (Grade A)
(25263,6570,1,1,2,156899),-- Blessed Scroll: Enchant Armor (Grade A)
(25264,1343,20,60,2,932000),-- Silver Arrow
(25265,1343,20,60,2,932000),-- Silver Arrow
(25266,6684,1,1,0,19369),-- Sealed Major Arcana Robe
(25266,6711,12,36,0,311439),-- Sealed Major Arcana Robe Part
(25266,6370,1,1,1,10584),-- Saint Spear
(25266,6694,12,36,1,207484),-- Saint Spear Blade
(25266,959,1,1,2,39110),-- Scroll: Enchant Weapon (Grade S)
(25266,960,1,1,2,391104),-- Scroll: Enchant Armor (Grade S)
(25266,6577,1,1,2,6519),-- Blessed Scroll: Enchant Weapon (Grade S)
(25267,1344,60,180,2,349167),-- Mithril Arrow
(25268,1344,60,180,2,349167),-- Mithril Arrow
(25269,6326,1,1,0,49107),-- Sealed Majestic Necklace
(25269,6327,1,1,0,65498),-- Sealed Majestic Earring
(25269,6328,1,1,0,98215),-- Sealed Majestic Ring
(25269,6342,81,243,0,68599),-- Sealed Majestic Earring Gemstone
(25269,6344,20,60,0,179792),-- Sealed Majestic Necklace Beads
(25269,6346,28,84,0,285214),-- Sealed Majestic Ring Gemstone
(25269,289,1,1,1,21288),-- Soul Bow
(25269,5534,12,36,1,252231),-- Soul Bow Stave
(25269,730,1,3,2,596190),-- Scroll: Enchant Armor (Grade A)
(25269,6578,1,1,2,63594),-- Blessed Scroll: Enchant Armor (Grade S)
(25270,1344,20,60,2,898500),-- Mithril Arrow
(25271,1344,20,60,2,898500),-- Mithril Arrow
(25272,63,3,9,0,78438),-- Gauntlets
(25272,553,4,10,0,64233),-- Iron Boots
(25272,1940,60,180,0,114437),-- Iron Boots Design
(25272,2422,4,12,0,58829),-- Reinforced Leather Boots
(25272,2423,5,13,0,52292),-- Boots of Knowledge
(25272,2446,5,15,0,47063),-- Reinforced Leather Gloves
(25272,2447,6,16,0,42784),-- Gloves of Knowledge
(25272,157,1,1,1,86625),-- Spiked Club
(25272,186,1,1,1,86625),-- Staff of Magic
(25272,2027,12,36,1,175775),-- Spiked Club Head
(25272,956,12,36,2,869675),-- Scroll: Enchant Armor (Grade D)
(25276,6698,81,243,0,60970),-- Sealed Tateossian Earring  Part
(25276,6699,40,120,0,137001),-- Sealed Tateossian Ring Gem
(25276,6700,14,42,0,214999),-- Sealed Tateossian Ring Wire
(25276,6724,1,1,0,30755),-- Sealed Tateossian Earring 
(25276,6725,1,1,0,46133),-- Sealed Tateossian Ring
(25276,6726,1,1,0,23061),-- Sealed Tateossian Necklace
(25276,6364,1,1,1,5292),-- Forgotten Blade
(25276,6372,1,1,1,5292),-- Heaven\'s Divider
(25276,6688,8,22,1,163036),-- Forgotten Blade Edge
(25276,6696,4,12,1,310549),-- Heavens Divider Edge
(25276,3936,1,1,2,733315),-- Blessed Scroll of Resurrection
(25276,6578,1,1,2,65184),-- Blessed Scroll: Enchant Armor (Grade S)
(25277,554,1,1,0,122181),-- Zubei\'s Boots
(25277,600,1,1,0,122181),-- Avadon Boots
(25277,612,1,1,0,122181),-- Zubei\'s Gauntlets
(25277,2464,1,1,0,122181),-- Avadon Gloves
(25277,4073,293,877,0,21689),-- Avadon Gloves Part
(25277,4097,60,180,0,105738),-- Zubei\'s Boots Design
(25277,4098,116,346,0,54929),-- Avadon Boots Design
(25277,4100,192,576,0,33043),-- Zubei\'s Gauntlets Part
(25277,284,1,1,1,43795),-- Dark Elven Long Bow
(25277,4112,12,36,1,345098),-- Dark Elven Longbow Shaft
(25277,952,21,63,2,150500),-- Scroll: Enchant Armor (Grade C)
(25277,4604,2,6,2,263377),-- Greater Dye of Constitution <Con+3 Dex-3>
(25277,4605,3,7,2,210701),-- Greater Dye of Dexterity <Dex+3 Str-3>
(25277,4606,3,9,2,175584),-- Greater Dye of Dexterity <Dex+3 Con-3>
(25278,1539,1,1,2,154489),-- Greater Healing Potion
(25279,1342,20,60,2,695200),-- Fine Steel Arrow
(25280,856,1,1,0,166240),-- Adamantite Earring
(25280,887,1,1,0,249359),-- Adamantite Ring
(25280,918,1,1,0,124680),-- Adamantite Necklace
(25280,4049,81,243,0,125385),-- Adamantite Earring Gemstone
(25280,4050,440,1320,0,27265),-- Adamantite Ring Wire
(25280,4051,140,420,0,53638),-- Adamantite Necklace Chain
(25280,300,1,1,1,52171),-- Great Axe
(25280,4113,12,36,1,373636),-- Great Axe Head
(25280,947,1,1,2,301198),-- Scroll: Enchant Weapon (Grade B)
(25280,948,3,9,2,313749),-- Scroll: Enchant Armor (Grade B)
(25280,6571,1,1,2,21514),-- Blessed Scroll: Enchant Weapon (Grade B)
(25281,5313,1,1,0,54308),-- Sealed Gloves of Nightmare
(25281,5314,1,1,0,54308),-- Sealed Boots of Nightmare
(25281,5318,1,1,0,54308),-- Sealed Majestic Gloves
(25281,5319,1,1,0,54308),-- Sealed Majestic Boots
(25281,5502,116,346,0,26901),-- Sealed Boots of Nightmare Lining
(25281,5503,23,67,0,138092),-- Sealed Majestic Boots Lining
(25281,5514,60,180,0,51784),-- Sealed Gloves of Nightmare Design
(25281,5515,16,48,0,194191),-- Sealed Majestic Gloves Design
(25281,236,1,1,1,21287),-- Soul Separator
(25281,5537,12,36,1,252359),-- Soul Separator Head
(25281,959,1,1,2,38156),-- Scroll: Enchant Weapon (Grade S)
(25281,960,1,1,2,381565),-- Scroll: Enchant Armor (Grade S)
(25281,6577,1,1,2,6360),-- Blessed Scroll: Enchant Weapon (Grade S)
(25282,6676,1,1,0,15739),-- Sealed Imperial Crusader Gauntlet
(25282,6677,1,1,0,15739),-- Sealed Imperial Crusader Boots
(25282,6681,1,1,0,15739),-- Sealed Draconic Leather Glove
(25282,6682,1,1,0,15739),-- Sealed Draconic Leather Boots
(25282,6685,1,1,0,15739),-- Sealed Major Arcana Glove
(25282,6686,1,1,0,15739),-- Sealed Major Arcana Boots
(25282,6703,8,24,0,223739),-- Sealed Imperial Crusader Gauntlets Design
(25282,6704,14,40,0,132586),-- Sealed Imperial Crusader Boots Design
(25282,6708,20,60,0,89496),-- Sealed Draconic Leather Gloves Fabric
(25282,6709,28,82,0,65807),-- Sealed Draconic Leather Boots Design
(25282,6712,36,108,0,49720),-- Sealed Major Arcana Gloves fabric
(25282,6713,46,136,0,39340),-- Sealed Major Arcana Boots Design
(25282,6365,1,1,1,2646),-- Basalt Battlehammer
(25282,6366,1,1,1,2646),-- Imperial Staff
(25282,6369,1,1,1,2646),-- Dragon Hunter Axe
(25282,6579,1,1,1,2646),-- Arcana Mace
(25282,6689,2,6,1,299966),-- Basalt Battlehammer Head
(25282,6690,11,31,1,58345),-- Imperial Staff Head
(25282,6693,6,18,1,103756),-- Dragon Hunter Axe Blade
(25282,6697,3,7,1,239975),-- Arcana Mace Head
(25282,959,1,1,2,39110),-- Scroll: Enchant Weapon (Grade S)
(25282,960,1,1,2,391104),-- Scroll: Enchant Armor (Grade S)
(25282,6577,1,1,2,6519),-- Blessed Scroll: Enchant Weapon (Grade S)
(25283,6364,1,1,1,6285),-- Forgotten Blade
(25283,6372,1,1,1,6285),-- Heaven\'s Divider
(25283,6688,8,22,1,193620),-- Forgotten Blade Edge
(25283,6696,4,12,1,368805),-- Heavens Divider Edge
(25283,7575,1,1,1,18854),-- Draconic Bow
(25283,7579,12,36,1,367231),-- Draconic Bow Shaft
(25283,3936,1,3,2,653164),-- Blessed Scroll of Resurrection
(25283,6578,1,1,2,116118),-- Blessed Scroll: Enchant Armor (Grade S)
(25284,1344,60,180,2,403000),-- Mithril Arrow
(25285,1344,60,180,2,403000),-- Mithril Arrow
(25286,6365,1,1,1,3142),-- Basalt Battlehammer
(25286,6366,1,1,1,3142),-- Imperial Staff
(25286,6367,1,1,1,18854),-- Angel Slayer
(25286,6369,1,1,1,3142),-- Dragon Hunter Axe
(25286,6579,1,1,1,3142),-- Arcana Mace
(25286,6689,16,48,1,44521),-- Basalt Battlehammer Head
(25286,6690,11,31,1,69277),-- Imperial Staff Head
(25286,6691,12,36,1,363031),-- Angel Slayer Blade
(25286,6693,6,18,1,123197),-- Dragon Hunter Axe Blade
(25286,6697,3,7,1,284938),-- Arcana Mace Head
(25286,959,1,1,2,69671),-- Scroll: Enchant Weapon (Grade S)
(25286,960,1,1,2,696707),-- Scroll: Enchant Armor (Grade S)
(25286,6577,1,1,2,11612),-- Blessed Scroll: Enchant Weapon (Grade S)
(25286,10296,1,1,3,50000),-- Transform Sealbook - Anakim
(25287,1344,60,180,2,403000),-- Mithril Arrow
(25288,1344,60,180,2,403000),-- Mithril Arrow
(25289,1344,60,180,2,403000),-- Mithril Arrow
(25290,6674,1,1,0,15820),-- Sealed Imperial Crusader Breastplate
(25290,6675,1,1,0,25313),-- Sealed Imperial Crusader Gaiters
(25290,6701,8,22,0,358823),-- Sealed Imperial Crusader Breastplate Part
(25290,6702,24,72,0,144350),-- Sealed Imperial Crusader Gaiters Pattern
(25290,6367,1,1,1,14186),-- Angel Slayer
(25290,6691,12,36,1,273152),-- Angel Slayer Blade
(25290,3936,1,3,2,491453),-- Blessed Scroll of Resurrection
(25290,6578,1,1,2,87369),-- Blessed Scroll: Enchant Armor (Grade S)
(25291,1344,60,180,2,381000),-- Mithril Arrow
(25292,1344,60,180,2,381000),-- Mithril Arrow
(25293,6684,1,1,0,25961),-- Sealed Major Arcana Robe
(25293,6674,1,1,0,23730),-- Sealed Imperial Crusader Breastplate
(25293,6675,1,1,0,37696),-- Sealed Imperial Crusader Gaiters
(25293,6701,53,157,0,76894),-- Sealed Imperial Crusader Breastplate Part
(25293,6702,24,72,0,216526),-- Sealed Imperial Crusader Gaiters Pattern
(25293,6711,12,36,0,417437),-- Sealed Major Arcana Robe Part
(25293,959,1,1,2,52421),-- Scroll: Enchant Weapon (Grade S)
(25293,960,1,1,2,524215),-- Scroll: Enchant Armor (Grade S)
(25293,6577,1,1,2,8737),-- Blessed Scroll: Enchant Weapon (Grade S)
(25294,1344,60,180,2,381000),-- Mithril Arrow
(25295,1344,60,180,2,381000),-- Mithril Arrow
(25296,6326,1,1,0,50084),-- Sealed Majestic Necklace
(25296,6327,1,1,0,66801),-- Sealed Majestic Earring
(25296,6328,1,1,0,100168),-- Sealed Majestic Ring
(25296,6342,81,243,0,69963),-- Sealed Majestic Earring Gemstone
(25296,6344,20,60,0,183368),-- Sealed Majestic Necklace Beads
(25296,6346,28,84,0,290887),-- Sealed Majestic Ring Gemstone
(25296,164,1,1,1,10855),-- Elysian
(25296,213,1,1,1,10855),-- Branch of The Mother Tree
(25296,5533,4,12,1,385997),-- Elysian Head
(25296,5544,8,22,1,205662),-- Branch of The Mother Tree Head
(25296,730,1,3,2,608047),-- Scroll: Enchant Armor (Grade A)
(25296,6578,1,1,2,64859),-- Blessed Scroll: Enchant Armor (Grade S)
(25297,1344,60,180,2,339000),-- Mithril Arrow
(25298,1344,60,180,2,339000),-- Mithril Arrow
(25299,6680,1,1,0,13446),-- Sealed Draconic Leather Armor
(25299,6707,12,36,0,216212),-- Sealed Draconic Leather Armor Part
(25299,6367,1,1,1,7348),-- Angel Slayer
(25299,6691,3,9,1,565930),-- Angel Slayer Blade
(25299,3936,1,1,2,509091),-- Blessed Scroll of Resurrection
(25299,6578,1,1,2,45253),-- Blessed Scroll: Enchant Armor (Grade S)
(25300,1344,60,180,2,403000),-- Mithril Arrow
(25301,1344,60,180,2,403000),-- Mithril Arrow
(25302,6684,1,1,0,14897),-- Sealed Major Arcana Robe
(25302,6711,12,36,0,239529),-- Sealed Major Arcana Robe Part
(25302,6370,1,1,1,8140),-- Saint Spear
(25302,6694,3,9,1,638313),-- Saint Spear Blade
(25302,3936,1,1,2,563996),-- Blessed Scroll of Resurrection
(25302,6578,1,1,2,50133),-- Blessed Scroll: Enchant Armor (Grade S)
(25303,1344,60,180,2,447667),-- Mithril Arrow
(25304,1344,60,180,2,447667),-- Mithril Arrow
(25305,6698,81,243,0,50316),-- Sealed Tateossian Earring  Part
(25305,6699,40,120,0,113061),-- Sealed Tateossian Ring Gem
(25305,6700,14,42,0,177429),-- Sealed Tateossian Ring Wire
(25305,6724,1,1,0,25381),-- Sealed Tateossian Earring 
(25305,6725,1,1,0,38072),-- Sealed Tateossian Ring
(25305,6726,1,1,0,19032),-- Sealed Tateossian Necklace
(25305,6370,1,1,1,8734),-- Saint Spear
(25305,6694,3,9,1,684917),-- Saint Spear Blade
(25305,959,1,1,2,32276),-- Scroll: Enchant Weapon (Grade S)
(25305,960,1,1,2,322761),-- Scroll: Enchant Armor (Grade S)
(25305,6577,1,1,2,5380),-- Blessed Scroll: Enchant Weapon (Grade S)
(25306,6676,1,1,0,9092),-- Sealed Imperial Crusader Gauntlet
(25306,6677,1,1,0,9092),-- Sealed Imperial Crusader Boots
(25306,6681,1,1,0,9092),-- Sealed Draconic Leather Glove
(25306,6682,1,1,0,9092),-- Sealed Draconic Leather Boots
(25306,6685,1,1,0,9092),-- Sealed Major Arcana Glove
(25306,6686,1,1,0,9092),-- Sealed Major Arcana Boots
(25306,6703,8,24,0,129405),-- Sealed Imperial Crusader Gauntlets Design
(25306,6704,14,40,0,76590),-- Sealed Imperial Crusader Boots Design
(25306,6708,20,60,0,51698),-- Sealed Draconic Leather Gloves Fabric
(25306,6709,28,82,0,37598),-- Sealed Draconic Leather Boots Design
(25306,6712,36,108,0,28721),-- Sealed Major Arcana Gloves fabric
(25306,6713,4,10,0,295417),-- Sealed Major Arcana Boots Design
(25306,6364,1,1,1,3057),-- Forgotten Blade
(25306,6372,1,1,1,3057),-- Heaven\'s Divider
(25306,6688,2,4,1,470926),-- Forgotten Blade Edge
(25306,6696,4,12,1,179404),-- Heavens Divider Edge
(25306,3936,1,1,2,423622),-- Blessed Scroll of Resurrection
(25306,6578,1,1,2,37655),-- Blessed Scroll: Enchant Armor (Grade S)
(25307,1344,60,180,2,481167),-- Mithril Arrow
(25308,1344,60,180,2,481167),-- Mithril Arrow
(25309,6674,1,1,0,8194),-- Sealed Imperial Crusader Breastplate
(25309,6675,1,1,0,13111),-- Sealed Imperial Crusader Gaiters
(25309,6367,1,1,1,7348),-- Angel Slayer
(25309,6691,3,9,1,565930),-- Angel Slayer Blade
(25309,6701,8,22,0,185867),-- Sealed Imperial Crusader Breastplate Part
(25309,6702,4,12,0,448612),-- Sealed Imperial Crusader Gaiters Pattern
(25309,959,1,1,2,27152),-- Scroll: Enchant Weapon (Grade S)
(25309,960,1,1,2,271517),-- Scroll: Enchant Armor (Grade S)
(25309,6577,1,1,2,4525),-- Blessed Scroll: Enchant Weapon (Grade S)
(25310,1344,60,180,2,403000),-- Mithril Arrow
(25311,1344,60,180,2,403000),-- Mithril Arrow
(25312,6698,9,27,0,422035),-- Sealed Tateossian Earring  Part
(25312,6699,40,120,0,105368),-- Sealed Tateossian Ring Gem
(25312,6700,14,42,0,165359),-- Sealed Tateossian Ring Wire
(25312,6724,1,1,0,23654),-- Sealed Tateossian Earring 
(25312,6725,1,1,0,35481),-- Sealed Tateossian Ring
(25312,6726,1,1,0,17736),-- Sealed Tateossian Necklace
(25312,6364,1,1,1,4070),-- Forgotten Blade
(25312,6372,1,1,1,4070),-- Heaven\'s Divider
(25312,6688,8,22,1,125392),-- Forgotten Blade Edge
(25312,6696,4,12,1,238844),-- Heavens Divider Edge
(25312,959,1,1,2,30080),-- Scroll: Enchant Weapon (Grade S)
(25312,960,1,1,2,300800),-- Scroll: Enchant Armor (Grade S)
(25312,6577,1,1,2,5014),-- Blessed Scroll: Enchant Weapon (Grade S)
(25313,1344,60,180,2,447667),-- Mithril Arrow
(25314,1344,60,180,2,447667),-- Mithril Arrow
(25315,6678,1,1,0,18553),-- Sealed Imperial Crusader Shield
(25315,6679,1,1,0,12987),-- Sealed Imperial Crusader Helmet
(25315,6683,1,1,0,12987),-- Sealed Draconic Leather Helmet
(25315,6687,1,1,0,12987),-- Sealed Major Arcana Circlet
(25315,6705,23,67,0,96456),-- Sealed Imperial Crusader Shield Part
(25315,6706,60,180,0,42074),-- Sealed Imperial Crusader Helmet Pattern
(25315,6710,11,31,0,240421),-- Sealed Draconic Leather Helmet Pattern
(25315,6714,16,48,0,157777),-- Sealed Major Arcana Circlet Pattern
(25315,6365,1,1,1,2184),-- Basalt Battlehammer
(25315,6366,1,1,1,2184),-- Imperial Staff
(25315,6369,1,1,1,2184),-- Dragon Hunter Axe
(25315,6579,1,1,1,2184),-- Arcana Mace
(25315,6689,2,6,1,247550),-- Basalt Battlehammer Head
(25315,6690,11,31,1,48150),-- Imperial Staff Head
(25315,6693,6,18,1,85626),-- Dragon Hunter Axe Blade
(25315,6697,3,7,1,198041),-- Arcana Mace Head
(25315,959,1,1,2,32276),-- Scroll: Enchant Weapon (Grade S)
(25315,960,1,1,2,322761),-- Scroll: Enchant Armor (Grade S)
(25315,6577,1,1,2,5380),-- Blessed Scroll: Enchant Weapon (Grade S)
(25316,6680,1,1,0,11189),-- Sealed Draconic Leather Armor
(25316,6707,3,9,0,719639),-- Sealed Draconic Leather Armor Part
(25316,6371,1,1,1,6114),-- Demon Splinter
(25316,6695,3,9,1,462042),-- Demon Splinter Blade
(25316,3936,1,1,2,423622),-- Blessed Scroll of Resurrection
(25316,6578,1,1,2,37655),-- Blessed Scroll: Enchant Armor (Grade S)
(25317,1344,60,180,2,481167),-- Mithril Arrow
(25318,1344,60,180,2,481167),-- Mithril Arrow
(25319,6676,1,1,0,23965),-- Sealed Imperial Crusader Gauntlet
(25319,6677,1,1,0,23965),-- Sealed Imperial Crusader Boots
(25319,6681,1,1,0,23965),-- Sealed Draconic Leather Glove
(25319,6682,1,1,0,23965),-- Sealed Draconic Leather Boots
(25319,6685,1,1,0,23965),-- Sealed Major Arcana Glove
(25319,6686,1,1,0,23965),-- Sealed Major Arcana Boots
(25319,6703,112,336,0,24334),-- Sealed Imperial Crusader Gauntlets Design
(25319,6704,14,40,0,201883),-- Sealed Imperial Crusader Boots Design
(25319,6708,20,60,0,136271),-- Sealed Draconic Leather Gloves Fabric
(25319,6709,28,82,0,99106),-- Sealed Draconic Leather Boots Design
(25319,6712,36,108,0,75706),-- Sealed Major Arcana Gloves fabric
(25319,6713,46,136,0,59899),-- Sealed Major Arcana Boots Design
(25319,6365,1,1,1,4029),-- Basalt Battlehammer
(25319,6366,1,1,1,4029),-- Imperial Staff
(25319,6369,1,1,1,4029),-- Dragon Hunter Axe
(25319,6579,1,1,1,4029),-- Arcana Mace
(25319,6689,16,48,1,57084),-- Basalt Battlehammer Head
(25319,6690,11,31,1,88825),-- Imperial Staff Head
(25319,6693,6,18,1,157960),-- Dragon Hunter Axe Blade
(25319,6697,3,7,1,365340),-- Arcana Mace Head
(25319,3936,1,3,2,558313),-- Blessed Scroll of Resurrection
(25319,6578,1,1,2,99255),-- Blessed Scroll: Enchant Armor (Grade S)
(25320,1344,60,180,2,458833),-- Mithril Arrow
(25321,1344,60,180,2,458833),-- Mithril Arrow
(25322,6323,1,1,0,71974),-- Sealed Phoenix Necklace
(25322,6324,1,1,0,95966),-- Sealed Phoenix Earring
(25322,6325,1,1,0,143948),-- Sealed Phoenix Ring
(25322,212,1,1,1,15392),-- Dasparion\'s Staff
(25322,2504,1,1,1,15392),-- Meteor Shower
(25322,5532,8,22,1,243921),-- Meteor Shower Head
(25322,5543,4,12,1,461666),-- Dasparion\'s Staff Edge
(25322,6341,81,243,0,82332),-- Sealed Phoenix Earring Gemstone
(25322,6343,140,420,0,36284),-- Sealed Phoenix Necklace Beads
(25322,6345,40,120,0,230598),-- Sealed Phoenix Ring Gemstone
(25322,729,1,1,2,104185),-- Scroll: Enchant Weapon (Grade A)
(25322,730,1,1,2,781387),-- Scroll: Enchant Armor (Grade A)
(25322,6569,1,1,2,12502),-- Blessed Scroll: Enchant Weapon (Grade A)
(25323,1343,20,60,2,901714),-- Silver Arrow
(25324,1343,20,60,2,901714),-- Silver Arrow
(25325,5312,1,1,0,48286),-- Sealed Helm of Nightmare
(25325,5315,1,1,0,68999),-- Sealed Shield of Nightmare
(25325,5317,1,1,0,48286),-- Sealed Majestic Circlet
(25325,5311,1,1,1,25075),-- Sealed Armor of Nightmare
(25325,5316,1,1,1,25075),-- Sealed Majestic Plate Armor
(25325,5522,4,12,1,469886),-- Sealed Armor of Nightmare Pattern
(25325,5523,8,22,1,250606),-- Sealed Majestic Plate Armor Pattern
(25325,5495,40,120,0,93119),-- Sealed Shield of Nightmare Fragment
(25325,5527,9,27,0,325858),-- Sealed Helm of Nightmare Design
(25325,5528,14,42,0,209480),-- Sealed Majestic Circlet Design
(25325,6570,1,1,2,158985),-- Blessed Scroll: Enchant Armor (Grade A)
(25325,6578,1,1,2,63594),-- Blessed Scroll: Enchant Armor (Grade S)
(25326,1344,20,60,2,898500),-- Mithril Arrow
(25327,1344,20,60,2,898500),-- Mithril Arrow
(25328,5311,1,1,0,16822),-- Sealed Armor of Nightmare
(25328,5316,1,1,0,16822),-- Sealed Majestic Plate Armor
(25328,5522,4,12,0,315229),-- Sealed Armor of Nightmare Pattern
(25328,5523,8,22,0,168122),-- Sealed Majestic Plate Armor Pattern
(25328,81,1,1,1,7140),-- Dragon Slayer
(25328,151,1,1,1,7140),-- Sword of Miracles 
(25328,2500,1,1,1,7140),-- Dark Legion\'s Edge
(25328,5529,9,27,1,112866),-- Dragon Slayer Edge
(25328,5545,14,42,1,72557),-- Dark Legion\'s Edge Blade
(25328,5546,5,15,1,223150),-- Sword of Miracles Edge
(25328,730,1,3,2,599942),-- Scroll: Enchant Armor (Grade A)
(25328,6578,1,1,2,63994),-- Blessed Scroll: Enchant Armor (Grade S)
(25329,1344,20,60,2,928000),-- Mithril Arrow
(25330,1344,20,60,2,928000),-- Mithril Arrow
(25331,1344,20,60,2,928000),-- Mithril Arrow
(25332,1344,20,60,2,928000),-- Mithril Arrow
(25333,47,2,4,0,256450),-- Helmet
(25333,628,4,12,0,137451),-- Hoplon
(25333,260,1,1,1,181912),-- Triple-Edged Jamadhr
(25333,6575,1,1,2,97404),-- Blessed Scroll: Enchant Weapon (Grade D)
(25334,397,1,1,0,103876),-- Mithril Shirt
(25334,1961,8,22,0,322432),-- Mithril Shirt Fabric
(25334,2387,1,1,0,165996),-- Tempered Mithril Gaiters
(25334,439,1,1,0,155814),-- Karmian Tunic
(25334,471,1,1,0,248995),-- Karmian Stockings
(25334,1954,24,72,0,251097),-- Karmian Stocking Design
(25334,1962,53,157,0,69092),-- Karmian Tunic Pattern
(25334,2935,24,72,0,167397),-- Tempered Mithril Gaiters Fragment
(25334,952,12,36,2,326537),-- Scroll: Enchant Armor (Grade C)
(25335,356,1,1,0,47648),-- Full Plate Armor
(25335,1986,3,9,0,750202),-- Full Plate Armor Temper
(25335,203,1,1,1,6950),-- Pa\'agrian Axe
(25335,204,1,1,1,5422),-- Deadman\'s Staff
(25335,205,1,1,1,5422),-- Ghoul\'s Staff
(25335,206,1,1,1,5422),-- Demon\'s Staff
(25335,2112,7,21,1,44202),-- Pa\'agrian Axe Blade
(25335,2116,2,6,1,150309),-- Deadman\'s Staff Head
(25335,2117,3,7,1,120246),-- Ghoul\'s Staff Head
(25335,2118,3,9,1,100205),-- Demon\'s Staff Head
(25335,2503,1,1,1,5422),-- Yaksa Mace
(25335,2969,2,4,1,204613),-- Yaksa Mace Head
(25335,951,1,1,2,376489),-- Scroll: Enchant Weapon (Grade C)
(25335,4601,1,3,2,230076),-- Greater Dye of Strength <Str+3 Con-3>
(25335,4602,2,4,2,153385),-- Greater Dye of Strength <Str+3 Dex-3>
(25335,4603,2,6,2,115038),-- Greater Dye of Constitution <Con+3 Str-3>
(25336,2398,1,1,0,13930),-- Blue Wolf Tunic
(25336,2399,1,1,0,13930),-- Tunic of Doom
(25336,2403,1,1,0,22294),-- Blue Wolf Stockings
(25336,2404,1,1,0,22294),-- Stockings of Doom
(25336,4084,16,48,0,70244),-- Blue Wolf Tunic Fabric
(25336,4085,23,67,0,49951),-- Tunic of Doom Pattern
(25336,4086,6,18,0,252921),-- Blue Wolf Stockings Pattern
(25336,4087,11,31,0,144527),-- Stockings of Doom Pattern
(25336,79,1,1,1,15380),-- Sword of Damascus
(25336,4114,3,9,1,566134),-- Sword of Damascus Blade
(25336,4613,1,3,2,173441),-- Greater Dye of Strength <Str+4 Con-4>
(25336,4614,1,1,2,346882),-- Greater Dye of Strength <Str+4 Dex-4>
(25336,4615,1,1,2,346882),-- Greater Dye of Constitution <Con+4 Str-4>
(25336,6571,1,1,2,7186),-- Blessed Scroll: Enchant Weapon (Grade B)
(25337,5311,1,1,0,6954),-- Sealed Armor of Nightmare
(25337,5316,1,1,0,6954),-- Sealed Majestic Plate Armor
(25337,5522,4,12,0,130321),-- Sealed Armor of Nightmare Pattern
(25337,5523,2,4,0,347523),-- Sealed Majestic Plate Armor Pattern
(25337,270,1,1,1,8856),-- Dragon Grinder
(25337,5538,3,9,1,419852),-- Dragon Grinder Edge
(25337,6577,1,1,2,3968),-- Blessed Scroll: Enchant Weapon (Grade S)
(25337,6578,1,1,2,26456),-- Blessed Scroll: Enchant Armor (Grade S)
(25338,6698,9,27,0,258007),-- Sealed Tateossian Earring  Part
(25338,6699,40,120,0,64416),-- Sealed Tateossian Ring Gem
(25338,6700,14,42,0,101090),-- Sealed Tateossian Ring Wire
(25338,6724,1,1,0,14461),-- Sealed Tateossian Earring 
(25338,6725,1,1,0,21691),-- Sealed Tateossian Ring
(25338,6726,1,1,0,10843),-- Sealed Tateossian Necklace
(25338,6370,1,1,1,4976),-- Saint Spear
(25338,6694,3,9,1,390226),-- Saint Spear Blade
(25338,3936,1,1,2,344793),-- Blessed Scroll of Resurrection
(25338,6578,1,1,2,30648),-- Blessed Scroll: Enchant Armor (Grade S)
(25339,6678,1,1,0,11220),-- Sealed Imperial Crusader Shield
(25339,6679,1,1,0,7854),-- Sealed Imperial Crusader Helmet
(25339,6683,1,1,0,7854),-- Sealed Draconic Leather Helmet
(25339,6687,1,1,0,7854),-- Sealed Major Arcana Circlet
(25339,6705,23,67,0,58331),-- Sealed Imperial Crusader Shield Part
(25339,6706,6,18,0,254438),-- Sealed Imperial Crusader Helmet Pattern
(25339,6710,11,31,0,145393),-- Sealed Draconic Leather Helmet Pattern
(25339,6714,16,48,0,95414),-- Sealed Major Arcana Circlet Pattern
(25339,6371,1,1,1,5282),-- Demon Splinter
(25339,6695,3,9,1,399164),-- Demon Splinter Blade
(25339,959,1,1,2,19519),-- Scroll: Enchant Weapon (Grade S)
(25339,960,1,1,2,195186),-- Scroll: Enchant Armor (Grade S)
(25339,6577,1,1,2,3253),-- Blessed Scroll: Enchant Weapon (Grade S)
(25340,1344,60,180,2,414167),-- Mithril Arrow
(25341,1344,60,180,2,414167),-- Mithril Arrow
(25342,6680,1,1,0,9666),-- Sealed Draconic Leather Armor
(25342,6707,3,9,0,621706),-- Sealed Draconic Leather Armor Part
(25342,7575,1,1,1,5282),-- Draconic Bow
(25342,7579,3,9,1,411515),-- Draconic Bow Shaft
(25342,3936,1,1,2,365972),-- Blessed Scroll of Resurrection
(25342,6578,1,1,2,32531),-- Blessed Scroll: Enchant Armor (Grade S)
(25343,1344,60,180,2,414167),-- Mithril Arrow
(25344,1344,60,180,2,414167),-- Mithril Arrow
(25345,1344,60,180,2,414167),-- Mithril Arrow
(25346,6674,1,1,0,5890),-- Sealed Imperial Crusader Breastplate
(25346,6675,1,1,0,9425),-- Sealed Imperial Crusader Gaiters
(25346,6701,8,22,0,133615),-- Sealed Imperial Crusader Breastplate Part
(25346,6702,4,12,0,322495),-- Sealed Imperial Crusader Gaiters Pattern
(25346,6367,1,1,1,5282),-- Angel Slayer
(25346,6691,3,9,1,406832),-- Angel Slayer Blade
(25346,959,1,1,2,19519),-- Scroll: Enchant Weapon (Grade S)
(25346,960,1,1,2,195186),-- Scroll: Enchant Armor (Grade S)
(25346,6577,1,1,2,3253),-- Blessed Scroll: Enchant Weapon (Grade S)
(25347,1344,60,180,2,414167),-- Mithril Arrow
(25348,1344,60,180,2,414167),-- Mithril Arrow
(25349,6684,1,1,1,9666),-- Sealed Major Arcana Robe
(25349,6678,1,1,0,16830),-- Sealed Imperial Crusader Shield
(25349,6679,1,1,0,11781),-- Sealed Imperial Crusader Helmet
(25349,6683,1,1,0,11871),-- Sealed Draconic Leather Helmet
(25349,6687,1,1,0,11871),-- Sealed Major Arcana Circlet
(25349,6705,23,67,0,87496),-- Sealed Imperial Crusader Shield Part
(25349,6706,60,180,0,38165),-- Sealed Imperial Crusader Helmet Pattern
(25349,6710,11,31,0,218088),-- Sealed Draconic Leather Helmet Pattern
(25349,6714,16,48,0,143121),-- Sealed Major Arcana Circlet Pattern
(25349,6711,3,9,1,621706),-- Sealed Major Arcana Robe Part
(25349,6578,1,1,2,32351),-- Blessed Scroll: Enchant Armor (Grade S)
(25349,3936,1,1,3,365972),-- Blessed Scroll of Resurrection
(25350,1344,60,180,2,414167),-- Mithril Arrow
(25351,1344,60,180,2,414167),-- Mithril Arrow
(25352,395,1,1,0,199474),-- Manticore Skin Shirt
(25352,417,4,12,0,39821),-- Manticore Skin Gaiters
(25352,1942,53,157,0,83336),-- Manticore Skin Shirt Texture
(25352,1943,24,72,0,283869),-- Manticore Skin Gaiters Pattern
(25352,261,1,1,1,95837),-- Bich\'Hwa
(25352,2039,12,36,1,2170726),-- Bich\'Hwa Edge
(25352,4559,3,7,2,133426),-- Greater Dye of Intelligence <Int+1 Men-1>
(25352,4560,3,9,2,111189),-- Greater Dye of Intelligence <Int+1 Wit-1>
(25352,4561,4,10,2,95305),-- Greater Dye of Mental Strength <Men+1 Int-1>
(25352,6387,12,36,2,277791),-- Blessed Scroll of Resurrection for Pets
(25353,6036,1,1,2,8594),-- Greater Magic Haste Potion
(25354,630,6,16,0,30926),-- Square Shield
(25354,1956,40,120,0,190655),-- Square Shield Fragment
(25354,2412,1,1,0,238251),-- Plate Helmet
(25354,2494,6,18,0,28349),-- Plate Shield
(25354,397,1,1,1,207794),-- Mithril Shirt
(25354,1961,53,157,1,92142),-- Mithril Shirt Fabric
(25354,2387,1,1,1,332060),-- Tempered Mithril Gaiters
(25354,2935,192,576,1,41858),-- Tempered Mithril Gaiters Fragment
(25354,2930,140,420,0,38686),-- Plate Helmet Design
(25354,2934,81,243,0,94151),-- Plate Shield Fragment
(25354,951,1,1,2,475085),-- Scroll: Enchant Weapon (Grade C)
(25354,952,12,36,2,154157),-- Scroll: Enchant Armor (Grade C)
(25354,6573,1,1,2,17419),-- Blessed Scroll: Enchant Weapon (Grade C)
(25355,1341,20,60,2,565333),-- Bone Arrow
(25356,6036,1,1,2,11307),-- Greater Magic Haste Potion
(25357,605,4,12,0,256557),-- Leather Gauntlets
(25357,1123,8,22,0,136830),-- Blue Buckskin Boots
(25357,222,1,1,1,318120),-- Poniard Dagger
(25357,955,4,10,2,97301),-- Scroll: Enchant Weapon (Grade D)
(25357,956,8,22,2,378396),-- Scroll: Enchant Armor (Grade D)
(25357,6387,24,72,2,141898),-- Blessed Scroll of Resurrection for Pets
(25358,1061,1,1,2,99782),-- Healing Potion
(25359,1832,1,1,2,182933),-- Greater Antidote
(25360,58,1,1,0,122830),-- Mithril Breastplate
(25360,59,1,3,0,88788),-- Mithril Gaiters
(25360,349,1,1,0,122830),-- Compound Scale Mail
(25360,350,1,1,0,122830),-- Dwarven Scale Mail
(25360,351,1,1,0,110956),-- Blast Plate
(25360,378,2,4,0,59192),-- Compound Scale Gaiters
(25360,379,2,6,0,44393),-- Dwarven Scale Gaiters
(25360,316,1,1,1,81587),-- Blood of Saints
(25360,317,1,1,1,81587),-- Tome of Blood
(25360,2030,3,9,1,591224),-- Tome of Blood Page
(25360,6576,1,3,2,589746),-- Blessed Scroll: Enchant Armor (Grade D)
(25361,1341,10,30,2,710311),-- Bone Arrow
(25362,433,6,16,0,16522),-- Elven Tunic
(25362,434,1,1,0,181736),-- White Tunic
(25362,435,1,1,0,181736),-- Mystic\'s Tunic
(25362,466,4,12,0,36394),-- Elven Stockings
(25362,467,5,13,0,32351),-- Dark Stockings
(25362,468,5,15,0,29116),-- Mystic\'s Stockings
(25362,1933,24,72,0,171268),-- Dark Stocking Fabric
(25362,1936,53,157,0,54460),-- White Tunic Pattern
(25362,86,1,1,1,33846),-- Tomahawk
(25362,179,1,1,1,33846),-- Mace of Prayer
(25362,180,1,1,1,33846),-- Mace of Judgment
(25362,181,1,1,1,33846),-- Mace of Miracle
(25362,182,1,1,1,33846),-- Doom Hammer
(25362,183,1,1,1,33846),-- Mystic Staff
(25362,184,1,1,1,33846),-- Conjuror\'s Staff
(25362,185,1,1,1,33846),-- Staff of Mana
(25362,2022,4,12,1,163267),-- Mace of Judgment Head
(25362,2023,2,4,1,346365),-- Conjuror\'s Staff Head
(25362,955,4,10,2,103523),-- Scroll: Enchant Weapon (Grade D)
(25362,956,8,22,2,402589),-- Scroll: Enchant Armor (Grade D)
(25362,6387,24,72,2,150971),-- Blessed Scroll of Resurrection for Pets
(25363,1831,1,1,2,490311),-- Antidote
(25364,1833,1,1,2,490311),-- Bandage
(25365,436,2,4,0,172819),-- Tunic of Knowledge
(25365,469,1,3,0,414341),-- Stockings of Knowledge
(25365,293,1,1,1,122380),-- War Hammer
(25365,296,1,1,1,122380),-- Dwarven Pike
(25365,6387,12,36,2,98291),-- Blessed Scroll of Resurrection for Pets
(25366,848,3,9,0,142028),-- Enchanted Earring
(25366,879,3,7,0,256432),-- Enchanted Ring
(25366,911,4,10,0,91373),-- Enchanted Necklace
(25366,1900,96,288,0,79638),-- Enchanted Necklace Chain
(25366,259,1,1,1,338459),-- Single-Edged Jamadhr
(25366,6575,1,1,2,60388),-- Blessed Scroll: Enchant Weapon (Grade D)
(25366,6576,1,1,2,543494),-- Blessed Scroll: Enchant Armor (Grade D)
(25367,734,1,1,2,30644),-- Haste Potion
(25368,735,1,1,2,15322),-- Potion of Alacrity
(25369,63,3,9,0,71556),-- Gauntlets
(25369,553,4,10,0,61334),-- Iron Boots
(25369,1940,60,180,0,104396),-- Iron Boots Design
(25369,2422,4,12,0,53667),-- Reinforced Leather Boots
(25369,2423,5,13,0,47704),-- Boots of Knowledge
(25369,2446,5,15,0,42934),-- Reinforced Leather Gloves
(25369,2447,6,16,0,39030),-- Gloves of Knowledge
(25369,260,1,1,1,237073),-- Triple-Edged Jamadhr
(25369,955,3,9,2,3808162),-- Scroll: Enchant Weapon (Grade D)
(25370,735,1,1,2,16899),-- Potion of Alacrity
(25371,6035,1,1,2,16899),-- Magic Haste Potion
(25372,391,2,4,0,374353),-- Puma Skin Shirt
(25372,413,4,12,0,224798),-- Puma Skin Gaiters
(25372,123,1,1,1,128884),-- Saber
(25372,2017,8,22,1,179838),-- Saber Blade
(25372,5285,1,1,1,128884),-- Heavy Sword
(25372,5531,24,72,1,71005),-- Heavy Sword Edge
(25372,955,4,10,2,100243),-- Scroll: Enchant Weapon (Grade D)
(25372,956,8,22,2,389836),-- Scroll: Enchant Armor (Grade D)
(25372,6387,24,72,2,146188),-- Blessed Scroll of Resurrection for Pets
(25373,46,2,4,0,456101),-- Bronze Helmet
(25373,627,4,12,0,244158),-- Aspis
(25373,315,1,1,1,318120),-- Divine Tome
(25373,6387,12,36,2,283795),-- Blessed Scroll of Resurrection for Pets
(25373,6575,1,1,2,37839),-- Blessed Scroll: Enchant Weapon (Grade D)
(25373,6576,1,1,2,340555),-- Blessed Scroll: Enchant Armor (Grade D)
(25374,1834,1,1,2,182933),-- Emergency Dressing
(25375,347,2,4,0,281012),-- Ring Mail Breastplate
(25375,376,4,12,0,168739),-- Iron Plate Gaiters
(25375,156,1,1,1,171845),-- Hand Axe
(25375,167,1,1,1,171845),-- Scalpel
(25375,178,1,1,1,171846),-- Bone Staff
(25375,955,1,3,2,350848),-- Scroll: Enchant Weapon (Grade D)
(25375,6575,1,1,2,38983),-- Blessed Scroll: Enchant Weapon (Grade D)
(25375,6576,1,1,2,350848),-- Blessed Scroll: Enchant Armor (Grade D)
(25376,17,10,30,2,795667),-- Wooden Arrow
(25377,1060,1,1,2,353630),-- Lesser Healing Potion
(25378,432,2,4,0,374353),-- Cursed Tunic
(25378,465,4,12,0,224798),-- Cursed Stockings
(25378,258,1,1,1,515537),-- Bagh-Nakh
(25378,955,4,10,2,100243),-- Scroll: Enchant Weapon (Grade D)
(25378,956,8,22,2,389836),-- Scroll: Enchant Armor (Grade D)
(25378,6387,24,72,2,146188),-- Blessed Scroll of Resurrection for Pets
(25379,1831,1,1,2,424356),-- Antidote
(25380,392,2,4,0,227754),-- Lion Skin Shirt
(25380,414,1,3,0,547320),-- Lion Skin Gaiters
(25380,292,1,1,1,318120),-- Pike
(25380,955,4,10,2,97301),-- Scroll: Enchant Weapon (Grade D)
(25380,956,8,22,2,378396),-- Scroll: Enchant Armor (Grade D)
(25380,6387,24,72,2,141898),-- Blessed Scroll of Resurrection for Pets
(25381,1833,1,1,2,424356),-- Bandage
(25382,734,1,1,2,26522),-- Haste Potion
(25383,851,6,18,0,22980),-- Omen Beast\'s Eye Earring
(25383,882,5,15,0,41364),-- Mithril Ring
(25383,914,1,1,0,206821),-- Necklace of Darkness
(25383,1905,81,243,0,77246),-- Omen Beast\'s Eye Earring Gemstone
(25383,1906,40,120,0,221035),-- Mithril Ring Wire
(25383,1907,154,462,0,29055),-- Necklace of Darkness Gem
(25383,225,1,1,1,83474),-- Mithril Dagger
(25383,2052,12,36,1,2091797),-- Mithril Dagger Blade
(25383,6573,1,1,2,50028),-- Blessed Scroll: Enchant Weapon (Grade C)
(25384,6036,1,1,2,10720),-- Greater Magic Haste Potion
(25385,61,9,25,0,10981),-- Mithril Gloves
(25385,607,9,27,0,10371),-- Ogre Power Gauntlets
(25385,1125,10,28,0,9825),-- Assault Boots
(25385,1949,112,336,0,34972),-- Mithril Gloves Design
(25385,1957,320,960,0,12240),-- Assault Boots Part
(25385,1963,203,607,0,19342),-- Ogre Power Gauntlets Part
(25385,2427,10,30,0,9334),-- Salamander Skin Boots
(25385,2428,11,31,0,8890),-- Plate Boots
(25385,2451,1,1,0,186677),-- Sage\'s Worn Gloves
(25385,2931,28,82,0,142432),-- Salamander Skin Boot Lining
(25385,2932,36,108,0,108802),-- Plate Boots Design
(25385,2933,46,136,0,86085),-- Sage\'s Worn Gloves Lining
(25385,280,1,1,1,91090),-- Light Crossbow
(25385,2054,12,36,1,224010),-- Light Crossbow Shaft
(25385,6574,1,1,2,467933),-- Blessed Scroll: Enchant Armor (Grade C)
(25386,1341,20,60,2,597333),-- Bone Arrow
(25387,6036,1,1,2,11947),-- Greater Magic Haste Potion
(25388,396,1,1,0,219924),-- Salamander Skin Mail
(25388,1952,12,36,0,538011),-- Salamander Skin Mail Texture
(25388,297,1,1,1,87193),-- Glaive
(25388,2055,12,36,1,208326),-- Glaive Edge
(25388,6574,1,1,2,447912),-- Blessed Scroll: Enchant Armor (Grade C)
(25389,1341,20,60,2,565333),-- Bone Arrow
(25390,6036,1,1,2,11307),-- Greater Magic Haste Potion
(25391,61,9,25,0,9654),-- Mithril Gloves
(25391,607,9,27,0,9117),-- Ogre Power Gauntlets
(25391,1125,10,28,0,8638),-- Assault Boots
(25391,1949,112,336,0,30746),-- Mithril Gloves Design
(25391,1957,320,960,0,10761),-- Assault Boots Part
(25391,1963,203,607,0,17005),-- Ogre Power Gauntlets Part
(25391,2427,10,30,0,8206),-- Salamander Skin Boots
(25391,2428,1,1,0,164117),-- Plate Boots
(25391,2451,1,1,0,164117),-- Sage\'s Worn Gloves
(25391,2931,28,82,0,125220),-- Salamander Skin Boot Lining
(25391,2932,36,108,0,95653),-- Plate Boots Design
(25391,2933,46,136,0,756838),-- Sage\'s Worn Gloves Lining
(25391,297,1,1,1,80080),-- Glaive
(25391,2055,12,36,1,191332),-- Glaive Edge
(25391,6574,1,1,2,411373),-- Blessed Scroll: Enchant Armor (Grade C)
(25392,352,1,1,0,102760),-- Brigandine Tunic
(25392,1941,14,42,0,167627),-- Brigandine Temper
(25392,2377,1,1,0,164373),-- Mithril Scale Gaiters
(25392,2378,1,1,0,164373),-- Brigandine Gaiters
(25392,2918,40,120,0,87526),-- Mithril Scale Gaiters Material
(25392,2919,81,243,0,43222),-- Brigandine Gaiters Material
(25392,279,1,1,1,98899),-- Strengthened Long Bow
(25392,2040,12,36,1,256030),-- Strengthened Long Bow Shaft
(25392,955,3,7,2,137691),-- Scroll: Enchant Weapon (Grade D)
(25392,4562,1,3,2,344224),-- Greater Dye of Mental Strength <Men+1 Wit-1>
(25392,4563,2,4,2,229483),-- Greater Dye of Wit <Wit+1 Int-1>
(25392,4564,2,6,2,172112),-- Greater Dye of Wit <Wit+1 Men-1>
(25393,1341,10,30,2,954667),-- Bone Arrow
(25394,354,1,1,0,47697),-- Chain Mail Shirt
(25394,381,1,1,0,76225),-- Chain Gaiters
(25394,1959,12,36,0,135168),-- Chain Mail Shirt Material
(25394,1960,15,45,0,219971),-- Chain Gaiters Part
(25394,96,1,1,1,10519),-- Scythe
(25394,298,1,1,1,10519),-- Orcish Glaive
(25394,302,1,1,1,10519),-- Body Slasher
(25394,2062,5,15,1,94459),-- Scythe Edge
(25394,2075,2,4,1,314863),-- Orcish Glaive Blade
(25394,2076,2,6,1,236147),-- Body Slasher Blade
(25394,6573,1,1,2,31224),-- Blessed Scroll: Enchant Weapon (Grade C)
(25395,103,1,1,0,136338),-- Tower Shield
(25395,107,1,1,0,136338),-- Composite Shield
(25395,517,1,1,0,95466),-- Composite Helmet
(25395,1149,1,1,0,95466),-- Shining Circlet
(25395,1978,60,180,0,72211),-- Tower Shield Fragment
(25395,1983,23,67,0,131752),-- Shining Circlet Pattern
(25395,2957,192,576,0,15440),-- Composite Helmet Design
(25395,4055,116,346,0,37512),-- Composite Shield Fragment
(25395,162,1,1,1,6137),-- War Axe
(25395,174,1,1,1,6137),-- Nirvana Axe
(25395,196,1,1,1,6137),-- Stick of Eternity
(25395,197,1,1,1,6137),-- Paradia Staff
(25395,199,1,1,1,6137),-- Pa\'agrian Hammer
(25395,200,1,1,1,6137),-- Sage\'s Staff
(25395,201,1,1,1,6137),-- Club of Nature
(25395,202,1,1,1,6137),-- Mace of The Underworld
(25395,2097,4,10,1,77575),-- War Axe Blade
(25395,2098,4,12,1,67878),-- Nirvana Axe Blade
(25395,2099,10,30,1,28575),-- Stick of Eternity Shaft
(25395,2100,17,49,1,17318),-- Paradia Staff Head
(25395,2102,3,9,1,91832),-- Pa\'agrian Hammer Head
(25395,2103,2,6,1,142873),-- Sage\'s Staff Head
(25395,2104,3,7,1,114299),-- Club of Nature Head
(25395,2105,5,13,1,60336),-- Mace of The Underworld Head
(25395,952,3,9,2,584108),-- Scroll: Enchant Armor (Grade C)
(25395,4604,2,6,2,146027),-- Greater Dye of Constitution <Con+3 Dex-3>
(25395,4605,3,7,2,116821),-- Greater Dye of Dexterity <Dex+3 Str-3>
(25395,4606,3,9,2,97351),-- Greater Dye of Dexterity <Dex+3 Con-3>
(25396,1342,20,60,2,559200),-- Fine Steel Arrow
(25397,1539,1,1,2,124267),-- Greater Healing Potion
(25398,353,1,1,0,134016),-- Half Plate Armor
(25398,380,1,1,0,214426),-- Plate Gaiters
(25398,1950,53,157,0,62305),-- Half Plate Temper
(25398,1951,24,72,0,211973),-- Plate Gaiters Part
(25398,70,1,1,1,43596),-- Claymore
(25398,2047,4,12,1,332241),-- Claymore Blade
(25398,2499,1,1,1,43596),-- Elven Long Sword
(25398,2967,8,22,1,174797),-- Elven Long Sword Blade
(25398,951,1,3,2,712587),-- Scroll: Enchant Weapon (Grade C)
(25399,1341,20,60,2,565333),-- Bone Arrow
(25400,6036,1,1,2,11307),-- Greater Magic Haste Potion
(25401,437,1,1,0,205848),-- Mithril Tunic
(25401,470,4,12,0,41093),-- Mithril Stockings
(25401,1944,53,157,0,85998),-- Mithril Tunic Fabric
(25401,1945,24,72,0,292940),-- Mithril Stocking Design
(25401,224,1,1,1,49450),-- Maingauche
(25401,1660,1,1,1,49450),-- Cursed Maingauche
(25401,2038,4,12,1,350710),-- Maingauche Edge
(25401,2044,8,22,1,187045),-- Cursed Maingauche Edge
(25401,956,12,36,2,956178),-- Scroll: Enchant Armor (Grade D)
(25402,6036,1,1,2,9547),-- Greater Magic Haste Potion
(25403,1341,10,30,2,954667),-- Bone Arrow
(25404,630,6,16,0,28403),-- Square Shield
(25404,1956,40,120,0,175102),-- Square Shield Fragment
(25404,2412,1,1,0,218815),-- Plate Helmet
(25404,2494,6,18,0,26036),-- Plate Shield
(25404,2930,140,420,0,35530),-- Plate Helmet Design
(25404,2934,81,243,0,86470),-- Plate Shield Fragment
(25404,70,1,1,1,40040),-- Claymore
(25404,2047,4,12,1,305138),-- Claymore Blade
(25404,2499,1,1,1,40040),-- Elven Long Sword
(25404,2967,8,22,1,160538),-- Elven Long Sword Blade
(25404,6573,1,1,2,47994),-- Blessed Scroll: Enchant Weapon (Grade C)
(25405,6036,1,1,2,10107),-- Greater Magic Haste Potion
(25406,1341,20,60,2,505333),-- Bone Arrow
(25407,2398,1,1,0,21002),-- Blue Wolf Tunic
(25407,2399,1,1,0,21002),-- Tunic of Doom
(25407,2403,1,1,0,33613),-- Blue Wolf Stockings
(25407,2404,1,1,0,33613),-- Stockings of Doom
(25407,4084,16,48,0,105909),-- Blue Wolf Tunic Fabric
(25407,4085,23,67,0,75313),-- Tunic of Doom Pattern
(25407,4086,60,180,0,38134),-- Blue Wolf Stockings Pattern
(25407,4087,11,31,0,217908),-- Stockings of Doom Pattern
(25407,97,1,1,1,23189),-- Lance
(25407,4115,12,36,1,212676),-- Lance Blade
(25407,947,1,1,2,151672),-- Scroll: Enchant Weapon (Grade B)
(25407,4613,1,3,2,261503),-- Greater Dye of Strength <Str+4 Con-4>
(25407,4614,2,4,2,174335),-- Greater Dye of Strength <Str+4 Dex-4>
(25407,4615,2,6,2,130752),-- Greater Dye of Constitution <Con+4 Str-4>
(25408,1539,1,1,2,227733),-- Greater Healing Potion
(25409,1342,60,180,2,341600),-- Fine Steel Arrow
(25410,852,1,1,0,258667),-- Moonstone Earring
(25410,883,6,16,0,35288),-- Aquastone Ring
(25410,915,1,1,0,194081),-- Aquastone Necklace
(25410,1908,81,243,0,90193),-- Moonstone Earring Wire
(25410,1909,480,1440,0,21705),-- Aquastone Ring Wire
(25410,1910,140,420,0,41495),-- Aquastone Necklace Chain
(25410,71,1,1,1,39385),-- Flamberge
(25410,72,1,1,1,39385),-- Stormbringer
(25410,2059,8,22,1,171917),-- Flamberge Blade
(25410,2060,4,12,1,344573),-- Stormbringer Blade
(25410,6573,1,1,2,60004),-- Blessed Scroll: Enchant Weapon (Grade C)
(25411,1341,20,60,2,736000),-- Bone Arrow
(25412,2414,1,1,0,142161),-- Full Plate Helmet
(25412,2497,1,1,0,203101),-- Full Plate Shield
(25412,2959,53,157,0,96614),-- Full Plate Helmet Design
(25412,2966,24,72,0,334335),-- Full Plate Shield Fragment
(25412,299,1,1,1,15949),-- Orcish Poleaxe
(25412,301,1,1,1,15949),-- Scorpion
(25412,303,1,1,1,15949),-- Widow Maker
(25412,2113,5,15,1,142023),-- Scorpion Blade
(25412,2114,9,27,1,78902),-- Widowmaker Head
(25412,2122,2,6,1,334655),-- Orcish Poleaxe Blade
(25412,4607,1,3,2,316811),-- Greater Dye of Intelligence <Int+3 Men-3>
(25412,4608,2,4,2,211207),-- Greater Dye of Intelligence <Int+3 Wit-3>
(25412,4609,2,6,2,158406),-- Greater Dye of Mental Strength <Men+3 Int-3>
(25412,6573,1,1,2,19009),-- Blessed Scroll: Enchant Weapon (Grade C)
(25413,1342,20,60,2,612000),-- Fine Steel Arrow
(25414,1539,1,1,2,136000),-- Greater Healing Potion
(25415,438,1,1,0,195393),-- Sage\'s Rag
(25415,1953,12,36,0,478000),-- Sage\'s Rag Lining
(25415,159,1,1,1,15493),-- Bonebreaker
(25415,187,1,1,1,15493),-- Atuba Hammer
(25415,188,1,1,1,15493),-- Ghost Staff
(25415,189,1,1,1,15493),-- Staff of Life
(25415,190,1,1,1,15493),-- Atuba Mace
(25415,2048,2,6,1,222103),-- Bonebreaker Head
(25415,2049,3,7,1,177603),-- Atuba Hammer Head
(25415,2050,7,21,1,67286),-- Ghost Staff Head
(25415,2051,12,36,1,39250),-- Staff of Life Shaft
(25415,2058,3,9,1,148067),-- Atuba Mace Head
(25415,952,12,36,2,386987),-- Scroll: Enchant Armor (Grade C)
(25416,1341,20,60,2,565333),-- Bone Arrow
(25417,6036,1,1,2,11307),-- Greater Magic Haste Potion
(25418,103,1,1,0,131056),-- Tower Shield
(25418,107,1,1,0,131056),-- Composite Shield
(25418,517,1,1,0,91767),-- Composite Helmet
(25418,1149,1,1,0,91767),-- Shining Circlet
(25418,1978,60,180,0,69413),-- Tower Shield Fragment
(25418,1983,23,67,0,126648),-- Shining Circlet Pattern
(25418,2957,192,576,0,14842),-- Composite Helmet Design
(25418,4055,116,346,0,36058),-- Composite Shield Fragment
(25418,162,1,1,1,5899),-- War Axe
(25418,174,1,1,1,5899),-- Nirvana Axe
(25418,196,1,1,1,5899),-- Stick of Eternity
(25418,197,1,1,1,5899),-- Paradia Staff
(25418,199,1,1,1,5899),-- Pa\'agrian Hammer
(25418,200,1,1,1,5899),-- Sage\'s Staff
(25418,201,1,1,1,5899),-- Club of Nature
(25418,202,1,1,1,5899),-- Mace of The Underworld
(25418,2097,4,10,1,74566),-- War Axe Blade
(25418,2098,4,12,1,65245),-- Nirvana Axe Blade
(25418,2099,10,30,1,27466),-- Stick of Eternity Shaft
(25418,2100,2,4,1,183108),-- Paradia Staff Head
(25418,2102,3,9,1,88270),-- Pa\'agrian Hammer Head
(25418,2103,2,6,1,137332),-- Sage\'s Staff Head
(25418,2104,3,7,1,109895),-- Club of Nature Head
(25418,2105,5,13,1,57992),-- Mace of The Underworld Head
(25418,4607,1,3,2,280738),-- Greater Dye of Intelligence <Int+3 Men-3>
(25418,4608,2,4,2,187159),-- Greater Dye of Intelligence <Int+3 Wit-3>
(25418,4609,2,6,2,140369),-- Greater Dye of Mental Strength <Men+3 Int-3>
(25418,6573,1,1,2,16844),-- Blessed Scroll: Enchant Weapon (Grade C)
(25419,6036,1,1,2,17813),-- Greater Magic Haste Potion
(25420,442,1,1,0,73761),-- Divine Tunic
(25420,473,1,1,0,117977),-- Divine Stockings
(25420,1988,8,22,0,389745),-- Divine Tunic Fabric
(25420,1989,24,72,0,191045),-- Divine Stocking Pattern
(25420,266,1,1,1,38731),-- Great Pata
(25420,2120,3,9,1,730779),-- Great Pata Blade
(25420,952,21,63,2,93908),-- Scroll: Enchant Armor (Grade C)
(25420,4604,2,6,2,164340),-- Greater Dye of Constitution <Con+3 Dex-3>
(25420,4605,3,7,2,131472),-- Greater Dye of Dexterity <Dex+3 Str-3>
(25420,4606,3,9,2,109560),-- Greater Dye of Dexterity <Dex+3 Con-3>
(25421,1342,20,60,2,638400),-- Fine Steel Arrow
(25422,1539,1,1,2,141867),-- Greater Healing Potion
(25423,2398,1,1,0,16783),-- Blue Wolf Tunic
(25423,2399,1,1,0,16783),-- Tunic of Doom
(25423,2403,1,1,0,26861),-- Blue Wolf Stockings
(25423,2404,1,1,0,26861),-- Stockings of Doom
(25423,4084,16,48,0,84631),-- Blue Wolf Tunic Fabric
(25423,4085,23,67,0,60182),-- Tunic of Doom Pattern
(25423,4086,6,18,0,304725),-- Blue Wolf Stockings Pattern
(25423,4087,11,31,0,174129),-- Stockings of Doom Pattern
(25423,234,1,1,1,18530),-- Demon\'s Sword
(25423,4119,3,9,1,682092),-- Demon\'s Sword Edge
(25423,4622,1,3,2,208965),-- Greater Dye of Mental Strength <Men+4 Wit-4>
(25423,4623,1,1,2,417930),-- Greater Dye of Wit <Wit+4 Int-4>
(25423,4624,2,4,2,139310),-- Greater Dye of Wit <Wit+4 Men-4>
(25423,6572,1,1,2,80799),-- Blessed Scroll: Enchant Armor (Grade B)
(25424,1539,1,1,2,196267),-- Greater Healing Potion
(25425,1342,20,60,2,883200),-- Fine Steel Arrow
(25426,47,2,4,0,334212),-- Helmet
(25426,628,4,12,0,179130),-- Hoplon
(25426,278,1,1,1,118536),-- Gastraphetes
(25426,2029,12,36,1,219011),-- Gastraphetes Shaft
(25426,956,12,36,2,793367),-- Scroll: Enchant Armor (Grade D)
(25427,1061,1,1,2,122901),-- Healing Potion
(25428,1832,1,1,2,225319),-- Greater Antidote
(25429,393,1,1,0,125544),-- Mithril Banded Mail
(25429,394,1,1,0,125544),-- Reinforced Leather Shirt
(25429,415,1,1,0,200664),-- Mithril Banded Gaiters
(25429,416,1,1,0,200664),-- Reinforced Leather Gaiters
(25429,1938,60,180,0,164579),-- Mithril Banded Mail Material
(25429,1939,28,84,0,460777),-- Mithril Banded Gaiters Material
(25429,223,1,1,1,19756),-- Kukuri
(25429,238,1,1,1,19756),-- Dagger of Mana
(25429,239,1,1,1,19756),-- Mystic Knife
(25429,241,1,1,1,19756),-- Shilen Knife
(25429,2028,27,81,1,64638),-- Shilen Knife Edge
(25429,6575,1,1,2,126939),-- Blessed Scroll: Enchant Weapon (Grade D)
(25430,6035,1,1,2,15322),-- Magic Haste Potion
(25431,60,1,1,0,84722),-- Composite Armor
(25431,1977,15,45,0,384443),-- Composite Armor Temper
(25431,265,1,1,1,31460),-- Fist Blade
(25431,2108,12,36,1,173998),-- Fist Blade Piece
(25431,4610,1,3,2,280738),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(25431,4611,2,4,2,187156),-- Greater Dye of Wit <Wit+3 Int-3>
(25431,4612,2,6,2,140369),-- Greater Dye of Wit <Wit+3 Men-3>
(25431,6574,1,1,2,144379),-- Blessed Scroll: Enchant Armor (Grade C)
(25432,1341,20,60,2,890667),-- Bone Arrow
(25433,6036,1,1,2,17813),-- Greater Magic Haste Potion
(25434,554,1,1,0,84625),-- Zubei\'s Boots
(25434,600,1,1,0,84625),-- Avadon Boots
(25434,612,1,1,0,84625),-- Zubei\'s Gauntlets
(25434,2464,1,1,0,84625),-- Avadon Gloves
(25434,4073,23,67,0,195295),-- Avadon Gloves Part
(25434,4097,60,180,0,73236),-- Zubei\'s Boots Design
(25434,4098,116,346,0,38045),-- Avadon Boots Design
(25434,4100,192,576,0,22886),-- Zubei\'s Gauntlets Part
(25434,78,1,1,1,10111),-- Great Sword
(25434,142,1,1,1,10111),-- Keshanberk
(25434,148,1,1,1,10111),-- Sword of Valhalla
(25434,4104,5,15,1,193268),-- Great Sword Blade
(25434,4107,9,27,1,105707),-- Keshanberk Blade
(25434,4108,14,42,1,67955),-- Sword of Valhalla Blade
(25434,948,1,3,2,547258),-- Scroll: Enchant Armor (Grade B)
(25434,6572,1,1,2,116749),-- Blessed Scroll: Enchant Armor (Grade B)
(25435,1539,1,1,2,188978),-- Greater Healing Potion
(25436,1342,20,60,2,850400),-- Fine Steel Arrow
(25437,60,1,1,0,88136),-- Composite Armor
(25437,1977,12,36,0,333282),-- Composite Armor Temper
(25437,265,1,1,1,49092),-- Fist Blade
(25437,2108,3,9,1,724049),-- Fist Blade Piece
(25437,4607,1,3,2,292053),-- Greater Dye of Intelligence <Int+3 Men-3>
(25437,4608,2,4,2,194702),-- Greater Dye of Intelligence <Int+3 Wit-3>
(25437,4609,2,6,2,146027),-- Greater Dye of Mental Strength <Men+3 Int-3>
(25437,6573,1,1,2,17523),-- Blessed Scroll: Enchant Weapon (Grade C)
(25438,60,1,1,0,84722),-- Composite Armor
(25438,1977,12,36,0,320369),-- Composite Armor Temper
(25438,283,1,1,1,47190),-- Akat Long Bow
(25438,2109,3,9,1,728579),-- Akat Longbow Shaft
(25438,951,3,7,2,91878),-- Scroll: Enchant Weapon (Grade C)
(25438,4601,1,3,2,280738),-- Greater Dye of Strength <Str+3 Con-3>
(25438,4602,2,4,2,187159),-- Greater Dye of Strength <Str+3 Dex-3>
(25438,4603,2,6,2,140369),-- Greater Dye of Constitution <Con+3 Str-3>
(25439,1341,20,60,2,890667),-- Bone Arrow
(25440,6036,1,1,2,17813),-- Greater Magic Haste Potion
(25441,854,1,1,0,159081),-- Earring of Binding
(25441,885,1,1,0,238568),-- Ring of Ages
(25441,917,1,1,0,119323),-- Necklace of Mermaid
(25441,1914,81,243,0,62505),-- Earring of Binding Gemstone
(25441,1915,280,840,0,28357),-- Ring of Ages Gemstone
(25441,1916,20,60,0,191744),-- Necklace of Mermaid Teardrop
(25441,283,1,1,1,49092),-- Akat Long Bow
(25441,2109,12,36,1,189485),-- Akat Longbow Shaft
(25441,4610,1,3,2,292053),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(25441,4611,2,4,2,194702),-- Greater Dye of Wit <Wit+3 Int-3>
(25441,4612,2,6,2,146027),-- Greater Dye of Wit <Wit+3 Men-3>
(25441,6574,1,1,2,150198),-- Blessed Scroll: Enchant Armor (Grade C)
(25442,1342,20,60,2,559200),-- Fine Steel Arrow
(25443,1539,1,1,2,124267),-- Greater Healing Potion
(25444,5287,1,1,0,15381),-- Sealed Dark Crystal breastplate
(25444,5288,1,1,0,24612),-- Sealed Dark Crystal Gaiters
(25444,5293,1,1,0,9466),-- Sealed Tallum Plate Armor
(25444,5520,9,27,0,98138),-- Sealed Dark Crystal Breastplate Pattern
(25444,5521,3,7,0,249300),-- Sealed Tallum Plate Armor Pattern
(25444,5524,4,12,0,337493),-- Sealed Dark Crystal Gaiters Pattern
(25444,288,1,1,1,17488),-- Carnage Bow
(25444,5535,3,9,1,71021),-- Carnage Bow Stave
(25444,729,1,1,2,59184),-- Scroll: Enchant Weapon (Grade A)
(25444,730,1,1,2,443880),-- Scroll: Enchant Armor (Grade A)
(25444,6569,1,1,2,7102),-- Blessed Scroll: Enchant Weapon (Grade A)
(25445,1343,20,60,2,871429),-- Silver Arrow
(25446,1343,20,60,2,871429),-- Silver Arrow
(25447,5326,1,1,0,14002),-- Sealed Robe of Nightmare
(25447,5329,1,1,0,14002),-- Sealed Majestic Robe
(25447,5487,4,12,0,260220),-- Sealed Robe of Nightmare Fabric
(25447,5488,8,22,0,138784),-- Sealed Majestic Robe Fabric
(25447,164,1,1,1,6687),-- Elysian
(25447,213,1,1,1,6687),-- Branch of The Mother Tree
(25447,5533,4,12,1,237763),-- Elysian Head
(25447,5544,8,22,1,126682),-- Branch of The Mother Tree Head
(25447,959,1,1,2,23971),-- Scroll: Enchant Weapon (Grade S)
(25447,960,1,1,2,239707),-- Scroll: Enchant Armor (Grade S)
(25447,6577,1,1,2,3995),-- Blessed Scroll: Enchant Weapon (Grade S)
(25448,1344,20,60,2,928000),-- Mithril Arrow
(25449,1344,20,60,2,928000),-- Mithril Arrow
(25450,6676,1,1,0,20517),-- Sealed Imperial Crusader Gauntlet
(25450,6677,1,1,0,20517),-- Sealed Imperial Crusader Boots
(25450,6681,1,1,0,20517),-- Sealed Draconic Leather Glove
(25450,6682,1,1,0,20517),-- Sealed Draconic Leather Boots
(25450,6685,1,1,0,20517),-- Sealed Major Arcana Glove
(25450,6686,1,1,0,20517),-- Sealed Major Arcana Boots
(25450,6703,112,336,0,20833),-- Sealed Imperial Crusader Gauntlets Design
(25450,6704,14,40,0,172836),-- Sealed Imperial Crusader Boots Design
(25450,6708,20,60,0,116664),-- Sealed Draconic Leather Gloves Fabric
(25450,6709,28,82,0,84846),-- Sealed Draconic Leather Boots Design
(25450,6712,36,108,0,64813),-- Sealed Major Arcana Gloves fabric
(25450,6713,46,136,0,51280),-- Sealed Major Arcana Boots Design
(25450,6364,1,1,1,6899),-- Forgotten Blade
(25450,6372,1,1,1,6899),-- Heaven\'s Divider
(25450,6688,8,22,1,212535),-- Forgotten Blade Edge
(25450,6696,4,12,1,404834),-- Heavens Divider Edge
(25450,959,1,1,2,50985),-- Scroll: Enchant Weapon (Grade S)
(25450,960,1,1,2,509847),-- Scroll: Enchant Armor (Grade S)
(25450,6577,1,1,2,8498),-- Blessed Scroll: Enchant Weapon (Grade S)
(25451,1344,60,180,2,381000),-- Mithril Arrow
(25452,1344,60,180,2,381000),-- Mithril Arrow
(25453,5320,1,1,0,22289),-- Sealed Leather Armor of Nightmare
(25453,5323,1,1,0,22289),-- Sealed Majestic Leather Armor
(25453,5480,4,12,0,412748),-- Sealed Leather Armor of Nightmare Fabric
(25453,5481,8,22,0,220132),-- Sealed Majestic Leather Armor Fabric
(25453,305,1,1,1,21287),-- Tallum Glaive
(25453,5541,12,36,1,252257),-- Tallum Glaive Edge
(25453,959,1,1,2,38156),-- Scroll: Enchant Weapon (Grade S)
(25453,960,1,1,2,381565),-- Scroll: Enchant Armor (Grade S)
(25453,6577,1,1,2,6360),-- Blessed Scroll: Enchant Weapon (Grade S)
(25454,1344,20,60,2,898500),-- Mithril Arrow
(25455,1344,20,60,2,898500),-- Mithril Arrow
(25456,2414,1,1,0,153011),-- Full Plate Helmet
(25456,2497,1,1,0,218603),-- Full Plate Shield
(25456,266,1,1,1,40182),-- Great Pata
(25456,2120,3,9,1,758154),-- Great Pata Blade
(25456,2959,53,157,0,103989),-- Full Plate Helmet Design
(25456,2966,192,576,0,44982),-- Full Plate Shield Fragment
(25456,951,3,7,2,111598),-- Scroll: Enchant Weapon (Grade C)
(25456,4601,1,3,2,340993),-- Greater Dye of Strength <Str+3 Con-3>
(25456,4602,2,4,2,227329),-- Greater Dye of Strength <Str+3 Dex-3>
(25456,4603,1,1,2,170497),-- Greater Dye of Constitution <Con+3 Str-3>
(25457,1342,20,60,2,666400),-- Fine Steel Arrow
(25458,1539,1,1,2,148089),-- Greater Healing Potion
(25459,1342,20,60,2,666400),-- Fine Steel Arrow
(25460,357,1,1,0,19325),-- Zubei\'s Breastplate
(25460,383,1,1,0,30914),-- Zubei\'s Gaiters
(25460,2376,1,1,0,19325),-- Avadon Breastplate
(25460,2379,1,1,0,30914),-- Avadon Gaiters
(25460,4056,16,48,0,87226),-- Zubei\'s Breastplate Part
(25460,4057,60,180,0,35948),-- Zubei\'s Gaiter Material
(25460,4064,23,67,0,62027),-- Avadon Breastplate Part
(25460,4065,11,31,0,205415),-- Avadon Gaiters Material
(25460,300,1,1,1,27701),-- Great Axe
(25460,4113,12,36,1,198387),-- Great Axe Head
(25460,951,3,7,2,109040),-- Scroll: Enchant Weapon (Grade C)
(25460,4601,1,3,2,33178),-- Greater Dye of Strength <Str+3 Con-3>
(25460,4602,2,4,2,222119),-- Greater Dye of Strength <Str+3 Dex-3>
(25460,4603,2,6,2,166589),-- Greater Dye of Constitution <Con+3 Str-3>
(25461,1539,1,1,2,161067),-- Greater Healing Potion
(25462,1342,20,60,2,724800),-- Fine Steel Arrow
(25463,2391,1,1,0,23097),-- Blue Wolf Leather Armor
(25463,2392,1,1,0,23097),-- Leather Armor of Doom
(25463,234,1,1,1,20716),-- Demon\'s Sword
(25463,4119,3,9,1,762545),-- Demon\'s Sword Edge
(25463,4082,24,72,0,82019),-- Blue Wolf Leather Armor Texture
(25463,4083,8,22,0,262459),-- Leather Armor of Doom Design
(25463,947,1,1,2,180661),-- Scroll: Enchant Weapon (Grade B)
(25463,948,1,3,2,564564),-- Scroll: Enchant Armor (Grade B)
(25463,6571,1,1,2,12904),-- Blessed Scroll: Enchant Weapon (Grade B)
(25464,1539,1,1,2,196267),-- Greater Healing Potion
(25465,1342,20,60,2,883200),-- Fine Steel Arrow
(25466,1539,1,1,2,196267),-- Greater Healing Potion
(25467,5289,1,1,0,40082),-- Sealed Dark Crystal Helmet
(25467,5292,1,1,0,57273),-- Sealed Dark Crystal Shield
(25467,5294,1,1,0,40082),-- Sealed Tallum Helmet
(25467,5494,40,120,0,70291),-- Sealed Dark Crystal Shield Fragment
(25467,5525,9,27,0,229945),-- Sealed Dark Crystal Helmet Design
(25467,5526,14,42,0,147822),-- Sealed Tallum Helm Design
(25467,269,1,1,1,17093),-- Blood Tornado
(25467,5539,3,9,1,677176),-- Blood Tornado Edge
(25467,729,1,1,2,57847),-- Scroll: Enchant Weapon (Grade A)
(25467,730,1,1,2,433857),-- Scroll: Enchant Armor (Grade A)
(25467,6569,1,1,2,6942),-- Blessed Scroll: Enchant Weapon (Grade A)
(25468,1343,20,60,2,842286),-- Silver Arrow
(25469,1343,20,60,2,842286),-- Silver Arrow
(25470,5290,1,1,0,47169),-- Sealed Dark Crystal Gloves
(25470,5291,1,1,0,47169),-- Sealed Dark Crystal Boots
(25470,5295,1,1,0,47169),-- Sealed Tallum Gloves
(25470,5296,1,1,0,47169),-- Sealed Tallum Boots
(25470,5496,11,31,0,234357),-- Sealed Dark Crystal Boots Lining
(25470,5497,23,67,0,109367),-- Sealed Tallum Boots Lining
(25470,5508,60,180,0,41012),-- Sealed Dark Crystal Gloves Design
(25470,5509,16,48,0,153796),-- Sealed Tallum Gloves Design
(25470,98,1,1,1,17866),-- Halberd
(25470,5542,3,9,1,707824),-- Halberd Edge
(25470,6569,1,1,2,10884),-- Blessed Scroll: Enchant Weapon (Grade A)
(25470,6570,1,1,2,90698),-- Blessed Scroll: Enchant Armor (Grade A)
(25471,1343,20,60,2,901714),-- Silver Arrow
(25472,1343,20,60,2,901714),-- Silver Arrow
(25473,856,1,1,0,87763),-- Adamantite Earring
(25473,887,1,1,0,131644),-- Adamantite Ring
(25473,918,1,1,0,65822),-- Adamantite Necklace
(25473,4049,81,243,0,66195),-- Adamantite Earring Gemstone
(25473,4050,40,120,0,158336),-- Adamantite Ring Wire
(25473,4051,14,42,0,283171),-- Adamantite Necklace Chain
(25473,91,1,1,1,13771),-- Heavy War Axe
(25473,92,1,1,1,13771),-- Sprite\'s Staff
(25473,4105,4,12,1,329047),-- Heavy War Axe Head
(25473,4106,8,22,1,172612),-- Sprite\'s Staff Head
(25473,4607,1,3,2,331276),-- Greater Dye of Intelligence <Int+3 Men-3>
(25473,4608,2,4,2,220850),-- Greater Dye of Intelligence <Int+3 Wit-3>
(25473,4609,2,6,2,165638),-- Greater Dye of Mental Strength <Men+3 Int-3>
(25473,6573,1,1,2,19876),-- Blessed Scroll: Enchant Weapon (Grade C)
(25474,1539,1,1,2,167822),-- Greater Healing Potion
(25475,357,1,1,0,21162),-- Zubei\'s Breastplate
(25475,383,1,1,0,33852),-- Zubei\'s Gaiters
(25475,2376,1,1,0,21162),-- Avadon Breastplate
(25475,2379,1,1,0,33852),-- Avadon Gaiters
(25475,4056,16,48,0,95515),-- Zubei\'s Breastplate Part
(25475,4057,60,180,0,39364),-- Zubei\'s Gaiter Material
(25475,4064,23,67,0,67922),-- Avadon Breastplate Part
(25475,4065,11,31,0,224936),-- Avadon Gaiters Material
(25475,78,1,1,1,10111),-- Great Sword
(25475,142,1,1,1,10111),-- Keshanberk
(25475,148,1,1,1,10111),-- Sword of Valhalla
(25475,4104,5,15,1,193268),-- Great Sword Blade
(25475,4107,9,27,1,105707),-- Keshanberk Blade
(25475,4108,14,42,1,67955),-- Sword of Valhalla Blade
(25475,947,1,1,2,175123),-- Scroll: Enchant Weapon (Grade B)
(25475,948,1,3,2,547260),-- Scroll: Enchant Armor (Grade B)
(25475,4617,2,4,2,201291),-- Greater Dye of Dexterity <Dex+4 Str-4>
(25476,1539,1,1,2,188978),-- Greater Healing Potion
(25477,1342,20,60,2,850400),-- Fine Steel Arrow
(25478,5304,1,1,0,20512),-- Sealed Tallum Tunic
(25478,5305,1,1,0,32807),-- Sealed Tallum Stockings
(25478,5308,1,1,0,12621),-- Sealed Dark Crystal Robe
(25478,5485,9,27,0,127973),-- Sealed Tallum Tunic Texture
(25478,5486,3,7,0,324182),-- Sealed Dark Crystal Robe Fabric
(25478,5489,28,84,0,64564),-- Sealed Tallum Stockings Fabric
(25478,235,1,1,1,17488),-- Bloody Orchid
(25478,5536,3,9,1,698990),-- Bloody Orchid Head
(25478,729,1,1,2,88776),-- Scroll: Enchant Weapon (Grade A)
(25478,6570,1,1,2,88776),-- Blessed Scroll: Enchant Armor (Grade A)
(25479,1343,20,60,2,871429),-- Silver Arrow
(25480,1343,20,60,2,871429),-- Silver Arrow
(25481,2384,1,1,0,35288),-- Zubei\'s Leather Shirt
(25481,2388,1,1,0,56446),-- Zubei\'s Leather Gaiters
(25481,2390,1,1,0,21705),-- Avadon Leather Armor
(25481,4066,9,27,0,276625),-- Zubei\'s Leather Shirt Fabric
(25481,4067,40,120,0,94503),-- Zubei\'s Leather Gaiter Texture
(25481,4068,14,42,0,136618),-- Avadon Leather Armor Lining
(25481,229,1,1,1,14226),-- Kris
(25481,243,1,1,1,14226),-- Hell Knife
(25481,4109,4,12,1,334649),-- Kris Edge
(25481,4110,8,22,1,178480),-- Hell Knife Edge
(25481,948,1,3,2,384995),-- Scroll: Enchant Armor (Grade B)
(25481,4616,2,4,2,141607),-- Greater Dye of Constitution <Con+4 Dex-4>
(25481,4617,2,6,2,106205),-- Greater Dye of Dexterity <Dex+4 Str-4>
(25481,4618,3,7,2,84694),-- Greater Dye of Dexterity <Dex+4 Con-4>
(25482,1539,1,1,0,174578),-- Greater Healing Potion
(25483,1342,20,60,2,785600),-- Fine Steel Arrow
(25484,503,1,1,0,51388),-- Zubei\'s Helmet
(25484,633,1,1,0,73415),-- Zubei\'s Shield
(25484,673,1,1,0,73415),-- Avadon Shield
(25484,2415,1,1,0,51388),-- Avadon Circlet
(25484,4072,23,67,0,153509),-- Avadon Circlet Pattern
(25484,4101,116,346,0,38598),-- Zubei\'s Shield Fragment
(25484,4102,16,48,0,215872),-- Zubei\'s Helmet Design
(25484,4439,60,180,0,88393),-- Avadon Shield Fragment
(25484,229,1,1,1,13814),-- Kris
(25484,243,1,1,1,13814),-- Hell Knife
(25484,4109,4,12,1,324945),-- Kris Edge
(25484,4110,8,22,1,173304),-- Hell Knife Edge
(25484,4607,1,3,2,332294),-- Greater Dye of Intelligence <Int+3 Men-3>
(25484,4608,2,4,2,221529),-- Greater Dye of Intelligence <Int+3 Wit-3>
(25484,4609,2,6,2,166147),-- Greater Dye of Mental Strength <Men+3 Int-3>
(25484,6573,1,1,2,19938),-- Blessed Scroll: Enchant Weapon (Grade C)
(25485,1539,1,1,2,154489),-- Greater Healing Potion
(25486,1342,20,60,2,695200),-- Fine Steel Arrow
(25487,62,11,33,0,4815),-- Mithril Boots
(25487,1958,220,660,0,10507),-- Mithril Boots Design
(25487,2429,12,34,0,4606),-- Chain Boots
(25487,2430,12,36,0,4414),-- Karmian Boots
(25487,2431,13,37,0,4237),-- Plated Leather Boots
(25487,2432,13,39,0,4074),-- Dwarven Chain Boots
(25487,2433,1,1,0,105934),-- Boots of Seal
(25487,2452,1,1,0,105934),-- Reinforced Mithril Gloves
(25487,2453,1,1,0,105934),-- Chain Gloves
(25487,2454,1,1,0,105934),-- Karmian Gloves
(25487,2937,378,1134,0,6115),-- Chain Boots Part
(25487,2938,26,78,0,88904),-- Karmian Boots Fabric
(25487,2939,35,105,0,66043),-- Plate Leather Boot Lining
(25487,2940,45,135,0,51367),-- Dwarven Chain Boots Part
(25487,2941,56,168,0,41277),-- Boots of Seal Pattern
(25487,2942,68,204,0,33993),-- Reinforced Mithril Gloves Design
(25487,2943,81,243,0,28538),-- Chain Gloves Design
(25487,2944,95,285,0,24333),-- Karmian Gloves Fabric
(25487,89,1,1,1,9846),-- Big Hammer
(25487,160,1,1,1,9846),-- Battle Axe
(25487,161,1,1,1,9846),-- Silver Axe
(25487,173,1,1,1,9846),-- Skull Graver
(25487,191,1,1,1,9846),-- Heavy Doom Hammer
(25487,192,1,1,1,9846),-- Crystal Staff
(25487,193,1,1,1,9846),-- Stick of Faith
(25487,194,1,1,1,9846),-- Heavy Doom Axe
(25487,2061,2,6,1,167705),-- Big Hammer Head
(25487,2063,3,7,1,134164),-- Battle Axe Head
(25487,2064,3,9,1,111803),-- Silver Axe Head
(25487,2065,4,10,1,95832),-- Skull Graver Head
(25487,2066,10,30,1,34035),-- Heavy Doom Hammer Head
(25487,2067,4,12,1,78187),-- Crystal Staff Head
(25487,2068,5,13,1,69499),-- Stick of Faith Shaft
(25487,2069,17,49,1,20623),-- Heavy Doom Axe Blade
(25487,6574,1,1,2,514316),-- Blessed Scroll: Enchant Armor (Grade C)
(25488,6036,1,1,2,14720),-- Greater Magic Haste Potion
(25489,1341,20,60,2,736000),-- Bone Arrow
(25490,631,6,16,0,27504),-- Eldarake
(25490,1964,40,120,0,155854),-- Eldarake Temper
(25490,2413,1,1,0,211799),-- Chain Hood
(25490,2495,6,18,0,25212),-- Chain Shield
(25490,2936,140,420,0,40445),-- Chain Hood Pattern
(25490,2945,81,243,0,76965),-- Chain Shield Fragment
(25490,263,1,1,1,78771),-- Chakram
(25490,2073,12,36,1,223615),-- Chakram Edge
(25490,951,1,3,2,818230),-- Scroll: Enchant Weapon (Grade C)
(25491,6036,1,1,2,14720),-- Greater Magic Haste Potion
(25492,1341,20,60,2,736000),-- Bone Arrow
(25493,503,1,1,0,56421),-- Zubei\'s Helmet
(25493,633,1,1,0,80606),-- Zubei\'s Shield
(25493,673,1,1,0,80606),-- Avadon Shield
(25493,2415,1,1,0,56421),-- Avadon Circlet
(25493,4072,23,67,0,168546),-- Avadon Circlet Pattern
(25493,4101,116,346,0,42379),-- Zubei\'s Shield Fragment
(25493,4102,192,576,0,19751),-- Zubei\'s Helmet Design
(25493,4439,60,180,0,97051),-- Avadon Shield Fragment
(25493,91,1,1,1,15167),-- Heavy War Axe
(25493,92,1,1,1,15167),-- Sprite\'s Staff
(25493,4105,4,12,1,362385),-- Heavy War Axe Head
(25493,4106,8,22,1,190101),-- Sprite\'s Staff Head
(25493,947,1,1,2,131342),-- Scroll: Enchant Weapon (Grade B)
(25493,4613,1,3,2,226452),-- Greater Dye of Strength <Str+4 Con-4>
(25493,4614,2,4,2,150968),-- Greater Dye of Strength <Str+4 Dex-4>
(25493,4615,1,1,2,452905),-- Greater Dye of Constitution <Con+4 Str-4>
(25494,1539,1,1,2,188978),-- Greater Healing Potion
(25495,1342,20,60,2,850400),-- Fine Steel Arrow
(25496,554,1,1,0,76839),-- Zubei\'s Boots
(25496,600,1,1,0,76839),-- Avadon Boots
(25496,612,1,1,0,76839),-- Zubei\'s Gauntlets
(25496,2464,1,1,0,76839),-- Avadon Gloves
(25496,4073,23,67,0,177329),-- Avadon Gloves Part
(25496,4097,60,180,0,66498),-- Zubei\'s Boots Design
(25496,4098,116,346,0,34545),-- Avadon Boots Design
(25496,4100,192,576,0,20781),-- Zubei\'s Gauntlets Part
(25496,267,1,1,1,27543),-- Arthro Nail
(25496,4111,12,36,1,219361),-- Arthro Nail Blade
(25496,4610,1,3,2,331275),-- Greater Dye of Mental Strength <Men+3 Wit-3>
(25496,4611,2,4,2,220850),-- Greater Dye of Wit <Wit+3 Int-3>
(25496,4612,2,6,2,165638),-- Greater Dye of Wit <Wit+3 Men-3>
(25496,6574,1,1,2,170370),-- Blessed Scroll: Enchant Armor (Grade C)
(25497,1342,20,60,2,755200),-- Fine Steel Arrow
(25498,854,1,1,0,159081),-- Earring of Binding
(25498,885,1,1,0,238568),-- Ring of Ages
(25498,917,1,1,0,119323),-- Necklace of Mermaid
(25498,1914,81,243,0,62505),-- Earring of Binding Gemstone
(25498,1915,280,840,0,28357),-- Ring of Ages Gemstone
(25498,1916,20,60,0,191744),-- Necklace of Mermaid Teardrop
(25498,231,1,1,1,24546),-- Grace Dagger
(25498,233,1,1,1,24546),-- Dark Screamer
(25498,2106,4,12,1,276250),-- Grace Dagger Edge
(25498,2107,8,22,1,147333),-- Dark Screamer Edge
(25498,951,3,7,2,95581),-- Scroll: Enchant Weapon (Grade C)
(25498,4601,1,3,2,292054),-- Greater Dye of Strength <Str+3 Con-3>
(25498,4602,2,4,2,194703),-- Greater Dye of Strength <Str+3 Dex-3>
(25498,4603,2,6,2,146027),-- Greater Dye of Constitution <Con+3 Str-3>
(25499,1342,20,60,2,559200),-- Fine Steel Arrow
(25500,1539,1,1,2,124267),-- Greater Healing Potion 
(25501,2038,4,12,1,339851),-- Maingauche Edge
(25501,4553,1,3,3,333565),-- Greater Dye of STR <Str+1 Con-1>
(25501,1943,24,72,2,283869),-- Manticore Skin Gaiters Pattern
(25501,4554,2,4,3,222377),-- Greater Dye of STR <Str+1 Dex-1>
(25501,395,1,1,2,199474),-- Manticore Skin Shirt
(25501,2044,8,22,1,181254),-- Cursed Maingauche Edge
(25501,4555,2,6,3,166783),-- Greater Dye of CON <Con+1 Str-1>
(25501,1942,53,157,2,83336),-- Manticore Skin Shirt Texture
(25501,224,1,1,1,47918),-- Maingauche
(25501,1660,1,1,1,47918),-- Cursed Maingauche
(25501,417,4,12,2,39821),-- Manticore Skin Gaiters
(25501,6575,1,1,4,37063),-- Blessed Scroll: Enchant Weapon (D)
(25504,951,1,1,3,531163),-- Scroll: Enchant Weapon (C)
(25504,2076,2,6,1,441895),-- Body Slasher Blade
(25504,2062,5,15,1,176758),-- Scythe Edge
(25504,952,12,36,3,162300),-- Scroll: Enchant Armor (C)
(25504,2433,1,1,2,103153),-- Boots of Seal
(25504,2452,1,1,2,103153),-- Reinforced Mithril Gloves
(25504,2453,1,1,2,103153),-- Chain Gloves
(25504,2454,1,1,2,103153),-- Karmian Gloves
(25504,2075,9,27,1,98199),-- Orcish Glaive Blade
(25504,2938,26,78,2,86570),-- Karmian Boots Fabric
(25504,2939,35,105,2,64309),-- Plate Leather Boot Lining
(25504,2940,45,135,2,50019),-- Dwarven Chain Boots Part
(25504,2941,56,168,2,40193),-- Boots of Seal Pattern
(25504,2942,68,204,2,33100),-- Reinforced Mithril Gloves Design
(25504,2943,81,243,2,27788),-- Chain Gloves Design
(25504,96,1,1,1,25568),-- Scythe
(25504,298,1,1,1,25568),-- Orcish Glaive
(25504,302,1,1,1,25568),-- Body Slasher
(25504,2944,95,285,2,23694),-- Karmian Gloves Fabric
(25504,6573,1,1,3,19476),-- Blessed Scroll: Enchant Weapon (C)
(25504,1958,220,660,2,10231),-- Mithril Boots Design
(25504,2937,378,1134,2,5955),-- Chain Boots Part
(25504,62,11,33,2,4689),-- Mithril Boots
(25504,2429,12,34,2,4485),-- Chain Boots
(25504,2430,12,36,2,4298),-- Karmian Boots
(25504,2431,13,37,2,4126),-- Plated Leather Boots
(25504,2432,13,39,2,3967),-- Dwarven Chain Boots
(25506,6574,1,1,3,460338),-- Blessed Scroll: Enchant Armor (C)
(25506,2935,24,72,2,229435),-- Tempered Mithril Gaiters Fragment
(25506,2387,1,1,2,227514),-- Tempered Mithril Gaiters
(25506,2074,12,36,2,183398),-- Crystallized Ice Bow Shaft
(25506,397,1,1,2,142372),-- Mithril Shirt
(25506,281,1,1,1,70504),-- Crystallized Ice Bow
(25506,1961,53,157,2,63132),-- Mithril Shirt Fabric
(25509,4615,1,1,4,424821),-- Greater Dye of CON <Con+4 Str-4>
(25509,4106,2,6,1,334340),-- Sprite\'s Staff Head
(25509,4069,9,27,2,276625),-- Tunic of Zubei Fabric
(25509,8347,3,7,1,244529),-- Ice Storm Hammer Head
(25509,4613,1,3,4,212410),-- Greater Dye of STR <Str+4 Con-4>
(25509,4614,2,4,4,141607),-- Greater Dye of STR <Str+4 Dex-4>
(25509,4071,14,42,2,136618),-- Avadon Robe Fabric
(25509,947,1,1,3,123197),-- Scroll: Enchant Weapon (B)
(25509,4105,6,18,1,113306),-- Heavy War Axe Head
(25509,4070,40,120,2,94503),-- Stockings of Zubei Fabric
(25509,8339,11,31,1,64746),-- Spell Breaker Head
(25509,2402,1,1,2,56446),-- Stockings of Zubei
(25509,2397,1,1,2,35288),-- Tunic of Zubei
(25509,2406,1,1,2,21705),-- Avadon Robe
-- (25509,9814,1,3,101,8800),-- Memento Mori
-- (25509,8176,1,1,100,8800),-- Destruction Tombstone
(25509,91,1,1,1,7113),-- Heavy War Axe
(25509,92,1,1,1,7113),-- Sprite\'s Staff
(25509,7892,1,1,1,7113),-- Spell Breaker
(25509,7900,1,1,1,7113),-- Ice Storm Hammer
(25512,4108,2,6,1,431942),-- Sword of Valhalla Blade
(25512,4065,11,31,2,204242),-- Avadon Gaiters Material
(25512,4104,5,15,1,175496),-- Great Sword Blade
(25512,4604,2,6,3,165638),-- Greater Dye of CON <Con+3 Dex-3>
(25512,4605,3,7,3,132510),-- Greater Dye of DEX <Dex+3 Str-3>
(25512,4606,3,9,3,110425),-- Greater Dye of DEX <Dex+3 Con-3>
(25512,4107,9,27,1,95987),-- Keshanberk Blade
(25512,952,21,63,4,94650),-- Scroll: Enchant Armor (C)
(25512,4056,16,48,2,86728),-- Zubei\'s Breastplate Part
(25512,4064,23,67,2,61673),-- Avadon Breastplate Part
(25512,4057,60,180,2,35742),-- Zubei\'s Gaiter Material
(25512,383,1,1,2,30738),-- Zubei\'s Gaiters
(25512,2379,1,1,2,30738),-- Avadon Gaiters
(25512,357,1,1,2,19215),-- Zubei\'s Breastplate
(25512,2376,1,1,2,19215),-- Avadon Breastplate
(25512,78,1,1,1,9181),-- Great Sword
(25512,142,1,1,1,9181),-- Keshanberk
(25512,148,1,1,1,9181),-- Sword of Valhalla
-- (25512,9817,1,3,101,8500),-- Nonliving Nucleus
-- (25512,8176,1,1,100,8500),-- Destruction Tombstone
(25514,6691,3,9,1,565930),-- Angel Slayer Blade
(25514,3936,1,1,3,509091),-- Blessed Scroll of Resurrection
(25514,6710,11,31,2,202250),-- Sealed Draconic Leather Helmet Pattern
(25514,6714,16,48,2,132727),-- Sealed Major Arcana Circlet Pattern
(25514,6705,23,67,2,81141),-- Sealed Imperial Crusader Shield Part
(25514,6578,1,1,3,45253),-- Blessed Scroll: Enchant Armor (S)
(25514,6706,60,180,2,35394),-- Sealed Imperial Crusader Helmet Pattern
(25514,6678,1,1,2,15608),-- Sealed Imperial Crusader Shield
(25514,6679,1,1,2,10925),-- Sealed Imperial Crusader Helmet
(25514,6683,1,1,2,10925),-- Sealed Draconic Leather Helmet
(25514,6687,1,1,2,10925),-- Sealed Major Arcana Circlet
(25514,6367,1,1,1,7348),-- Angel Slayer
(25515,1344,20,60,2,403000),-- Mithril Arrow 
(25516,1344,20,60,2,403000),-- Mithril Arrow
(25523,6710,11,31,2,288537),-- Sealed Draconic Leather Helmet Pattern
(25523,6691,12,36,1,201838),-- Angel Slayer Blade
(25523,6714,16,48,2,189352),-- Sealed Major Arcana Circlet Pattern
(25523,6570,1,1,3,161398),-- Blessed Scroll: Enchant Armor (A)
(25523,6705,23,67,2,115759),-- Sealed Imperial Crusader Shield Part
(25523,6578,1,1,3,64559),-- Blessed Scroll: Enchant Armor (S)
(25523,6706,60,180,2,50494),-- Sealed Imperial Crusader Helmet Pattern
(25523,6678,1,1,2,22267),-- Sealed Imperial Crusader Shield
(25523,6679,1,1,2,15586),-- Sealed Imperial Crusader Helmet
(25523,6683,1,1,2,15586),-- Sealed Draconic Leather Helmet
(25523,6687,1,1,2,15586),-- Sealed Major Arcana Circlet
(25523,6367,1,1,1,10483),-- Angel Slayer
-- (25523,9817,3,7,101,8300),-- Nonliving Nucleus
-- (25523,8176,1,4,100,8300),-- Destruction Tombstone
-- (25523,8619,1,1,102,5600),-- Ancient Book - Divine Inspiration (Original Language Version)
(25524,960,1,1,3,524479),-- Scroll: Enchant Armor (S)
(25524,5522,4,12,2,430587),-- Sealed Armor of Nightmare Pattern
(25524,5546,5,15,1,304812),-- Sword of Miracles Edge
(25524,5523,8,22,2,229647),-- Sealed Majestic Plate Armor Pattern
(25524,5529,9,27,1,154169),-- Dragon Slayer Edge
(25524,5545,14,42,1,99109),-- Dark Legion\'s Edge Blade
(25524,959,1,1,3,52448),-- Scroll: Enchant Weapon (S)
(25524,5311,1,1,2,22978),-- Sealed Armor of Nightmare
(25524,5316,1,1,2,22978),-- Sealed Majestic Plate Armor
(25524,81,1,1,1,9754),-- Dragon Slayer
(25524,151,1,1,1,9754),-- Sword of Miracles
(25524,2500,1,1,1,9754),-- Dark Legion\'s Edge
(25524,6577,1,1,3,8742),-- Blessed Scroll: Enchant Weapon (S)
(25525,1344,60,180,2,359667),-- Mithril Arrow 
(25526,1344,60,180,2,359667),-- Mithril Arrow
(25527,6689,2,6,1,289231),-- Basalt Battlehammer Head
(25527,6697,3,7,1,231387),-- Arcana Mace Head
(25527,6700,14,42,2,207304),-- Sealed Tateossian Necklace Chain
(25527,6699,40,120,2,132098),-- Sealed Tateossian Ring Gem
(25527,6693,6,18,1,100043),-- Dragon Hunter Axe Blade
(25527,6698,81,243,2,58788),-- Sealed Tateossian Earring Part
(25527,6690,11,31,1,56257),-- Imperial Staff Head
(25527,6725,1,1,2,44482),-- Sealed Tateossian Ring
(25527,6724,1,1,2,29654),-- Sealed Tateossian Earring
(25527,6726,1,1,2,22236),-- Sealed Tateossian Necklace
(25527,6365,1,1,1,2551),-- Basalt Battlehammer
(25527,6366,1,1,1,2551),-- Imperial Staff
(25527,6369,1,1,1,2551),-- Dragon Hunter Axe
(25527,6579,1,1,1,2551),-- Arcana Mace
(25531,7579,3,9,1,583776),-- Draconic Bow Shaft
(25531,6710,11,31,2,206253),-- Sealed Draconic Leather Helmet Pattern
(25531,6714,16,48,2,135354),-- Sealed Major Arcana Circlet Pattern
(25531,6705,23,67,2,82748),-- Sealed Imperial Crusader Shield Part
(25531,6706,60,180,2,36094),-- Sealed Imperial Crusader Helmet Pattern
(25531,6578,1,1,3,23074),-- Blessed Scroll: Enchant Armor (S)
(25531,6678,1,1,2,15917),-- Sealed Imperial Crusader Shield
(25531,6679,1,1,2,11141),-- Sealed Imperial Crusader Helmet
(25531,6683,1,1,2,11141),-- Sealed Draconic Leather Helmet
(25531,6687,1,1,2,11141),-- Sealed Major Arcana Circlet
(25531,7575,1,1,1,7493),-- Draconic Bow
(25531,9468,1,1,3,6671),-- Dynasty Necklace
(25531,9474,1,1,3,6671),-- Dynasty Necklace
(25531,9477,1,1,3,6671),-- Dynasty Necklace
(25532,6713,4,10,2,290537),-- Sealed Major Arcana Boots Design
(25532,6690,2,4,1,232024),-- Imperial Staff Head
(25532,6689,2,6,1,170414),-- Basalt Battlehammer Head
(25532,6697,3,7,1,136331),-- Arcana Mace Head
(25532,6703,8,24,2,127110),-- Sealed Imperial Crusader Gauntlets Design
(25532,6704,14,40,2,75325),-- Sealed Imperial Crusader Boots Design
(25532,6693,6,18,1,58945),-- Dragon Hunter Axe Blade
(25532,6708,20,60,2,50844),-- Sealed Draconic Leather Gloves Fabric
(25532,6709,28,82,2,36977),-- Sealed Draconic Leather Boots Design
(25532,6712,36,108,2,28246),-- Sealed Major Arcana Gloves fabric
(25532,6578,1,1,3,14813),-- Blessed Scroll: Enchant Armor (S)
(25532,6676,1,1,2,8942),-- Sealed Imperial Crusader Gauntlet
(25532,6677,1,1,2,8942),-- Sealed Imperial Crusader Boots
(25532,6681,1,1,2,8942),-- Sealed Draconic Leather Glove
(25532,6682,1,1,2,8942),-- Sealed Draconic Leather Boots
(25532,6685,1,1,2,8942),-- Sealed Major Arcana Glove
(25532,6686,1,1,2,8942),-- Sealed Major Arcana Boots
(25532,9459,1,1,3,4283),-- Dynasty Necklace
(25532,9462,1,1,3,4283),-- Dynasty Necklace
(25532,9465,1,1,3,4283),-- Dynasty Necklace
(25532,9471,1,1,3,4283),-- Dynasty Necklace
(25532,6365,1,1,1,1503),-- Basalt Battlehammer
(25532,6366,1,1,1,1503),-- Imperial Staff
(25532,6369,1,1,1,1503),-- Dragon Hunter Axe
(25532,6579,1,1,1,1503),-- Arcana Mace
(25532,10300,1,1,0,50000),-- Transform Sealbook - Kechi (custom chance)
(25534,6707,12,36,2,352969),-- Sealed Draconic Leather Armor Part
(25534,7579,12,36,1,233638),-- Draconic Bow Shaft
(25534,6680,1,1,2,21952),-- Sealed Draconic Leather Armor
(25534,7575,1,1,1,11995); -- Draconic Bow

INSERT INTO `droplist` VALUES 
(18342,57,7,13,0,1000000); -- Adena

-- Category Explanation (GrandBoss)
-- 0+ is defined catagories
-- Any other mobs within this range (minions, etc) use the default drop categories (most often 2)
INSERT INTO `droplist` VALUES
(29001,70,1,1,0,60000),-- Claymore
(29001,159,1,1,0,55000),-- Bonebreaker
(29001,188,1,1,0,55000),-- Ghost Staff
(29001,189,1,1,0,55000),-- Staff of Life
(29001,225,1,1,0,55000),-- Mithril Dagger
(29001,262,1,1,0,50000),-- Scallop Jamadhr
(29001,280,1,1,0,55000),-- Light Crossbow
(29001,297,1,1,0,55000),-- Glaive
(29001,2499,1,1,0,60000),-- Elven Long Sword
(29001,851,1,2,1,350000),-- Omen Beast\'s Eye Earring
(29001,882,1,2,1,350000),-- Mithril Ring
(29001,914,1,2,1,300000),-- Necklace of Darkness
(29001,353,1,2,2,375000),-- Half Plate Armor
(29001,380,1,2,2,375000),-- Plate Gaiters
(29001,396,1,2,3,425000),-- Salamander Skin Mail
(29001,438,1,2,3,425000),-- Sage\'s Rag
(29001,607,1,3,4,200000),-- Ogre Power Gauntlets
(29001,630,1,3,4,200000),-- Square Shield
(29001,955,1,3,4,200000),-- Scroll: Enchant Weapon (Grade D)
(29001,1125,1,3,4,200000),-- Assault Boots
(29001,2412,1,3,4,200000),-- Plate Helmet
(29001,57,140000,170000,5,1000000),-- Adena
(29001,57,140000,170000,6,1000000),-- Adena
(29001,57,140000,170000,7,1000000),-- Adena
(29001,57,140000,170000,8,1000000),-- Adena
(29001,57,140000,170000,9,1000000),-- Adena
(29001,57,140000,170000,10,1000000),-- Adena
(29001,57,120000,150000,11,1000000),-- Adena
(29001,6660,1,1,12,300000),-- Ring of Queen Ant
(29006,75,1,1,0,48000),-- Caliburs
(29006,84,1,1,0,40000),-- Homunkulus\'s Sword
(29006,162,1,1,0,48000),-- War Axe
(29006,174,1,1,0,40000),-- Nirvana Axe
(29006,200,1,1,0,40000),-- Sage\'s Staff
(29006,233,1,1,0,48000),-- Dark Screamer
(29006,265,1,1,0,40000),-- Fist Blade
(29006,283,1,1,0,48000),-- Akat Long Bow
(29006,301,1,1,0,48000),-- Scorpion
(29006,60,1,1,1,120000),-- Composite Armor
(29006,400,1,1,1,120000),-- Theca Leather Armor
(29006,420,1,1,1,120000),-- Theca Leather Gaiters
(29006,441,1,1,1,120000),-- Demon\'s Tunic
(29006,472,1,1,1,120000),-- Demon\'s Stockings
(29006,107,1,1,2,140000),-- Composite Shield
(29006,517,1,1,2,140000),-- Composite Helmet
(29006,951,1,1,2,140000),-- Scroll: Enchant Weapon (Grade C)
(29006,2435,1,1,2,140000),-- Demon\'s Boots
(29006,2459,1,1,2,140000),-- Demon\'s Gloves
(29006,57,100000,150000,3,1000000),-- Adena
(29006,57,100000,150000,4,1000000),-- Adena
(29006,57,100000,150000,5,1000000),-- Adena
(29006,57,100000,150000,6,1000000),-- Adena
(29006,57,100000,150000,7,1000000),-- Adena
(29006,6662,1,1,8,300000),-- Ring of Core
(29006,8754,1,2,200,100000),-- Top-Grade Life Stone: level 49
(29006,8744,2,4,200,450000),-- High-Grade Life Stone: level 49
(29006,8745,2,4,200,400000),-- High-Grade Life Stone: level 52
(29006,8755,1,2,200,50000),-- Top-Grade Life Stone: level 52
(29014,76,1,1,0,42000),-- Sword of Delusion
(29014,145,1,1,0,35000),-- Sword of Whispering Death
(29014,162,1,1,0,42000),-- War Axe
(29014,199,1,1,0,35000),-- Pa\'agrian Hammer
(29014,201,1,1,0,35000),-- Club of Nature
(29014,231,1,1,0,42000),-- Grace Dagger
(29014,265,1,1,0,35000),-- Fist Blade
(29014,283,1,1,0,42000),-- Akat Long Bow
(29014,303,1,1,0,42000),-- Widow Maker
(29014,854,1,2,1,350000),-- Earring of Binding
(29014,885,1,2,1,350000),-- Ring of Ages
(29014,917,1,2,1,300000),-- Necklace of Mermaid
(29014,60,1,1,2,100000),-- Composite Armor
(29014,400,1,1,2,100000),-- Theca Leather Armor
(29014,420,1,1,2,100000),-- Theca Leather Gaiters
(29014,441,1,1,2,100000),-- Demon\'s Tunic
(29014,472,1,1,2,100000),-- Demon\'s Stockings
(29014,64,1,2,3,100000),-- Composite Boots
(29014,103,1,2,3,100000),-- Tower Shield
(29014,517,1,2,3,100000),-- Composite Helmet
(29014,608,1,2,3,100000),-- Mithril Gauntlets
(29014,951,1,2,3,100000),-- Scroll: Enchant Weapon (Grade C)
(29014,57,110000,150000,4,1000000),-- Adena
(29014,57,110000,150000,5,1000000),-- Adena
(29014,57,110000,150000,6,1000000),-- Adena
(29014,57,110000,150000,7,1000000),-- Adena
(29014,57,110000,150000,8,1000000),-- Adena
(29014,6661,1,1,9,300000),-- Earring of Orfen
(29014,8754,1,2,200,100000),-- Top-Grade Life Stone: level 49
(29014,8744,2,4,200,450000),-- High-Grade Life Stone: level 49
(29014,8745,2,4,200,400000),-- High-Grade Life Stone: level 52
(29014,8755,1,2,200,50000),-- Top-Grade Life Stone: level 52
(29019,80,1,2,0,250000),-- Tallum Blade
(29019,98,1,2,0,250000),-- Halberd
(29019,269,1,2,0,250000),-- Blood Tornado
(29019,2504,1,2,0,250000),-- Meteor Shower
(29019,150,1,2,1,250000),-- Elemental Sword
(29019,212,1,2,1,250000),-- Dasparion\'s Staff
(29019,235,1,2,1,250000),-- Bloody Orchid
(29019,288,1,2,1,250000),-- Carnage Bow
(29019,81,1,2,2,250000),-- Dragon Slayer
(29019,151,1,2,2,250000),-- Sword of Miracles 
(29019,164,1,2,2,250000),-- Elysian
(29019,2500,1,2,2,250000),-- Dark Legion\'s Edge
(29019,213,1,2,3,200000),-- Branch of The Mother Tree
(29019,236,1,2,3,200000),-- Soul Separator
(29019,270,1,2,3,200000),-- Dragon Grinder
(29019,289,1,2,3,200000),-- Soul Bow
(29019,305,1,2,3,200000),-- Tallum Glaive
(29019,6364,1,1,4,170000),-- Forgotten Blade
(29019,6365,1,1,4,170000),-- Basalt Battlehammer
(29019,6369,1,1,4,170000),-- Dragon Hunter Axe
(29019,6370,1,1,4,170000),-- Saint Spear
(29019,6372,1,1,4,160000),-- Heaven\'s Divider
(29019,6580,1,1,4,160000),-- Tallum Blade*Dark Legion\'s Edge
(29019,6366,1,1,5,200000),-- Imperial Staff
(29019,6367,1,1,5,200000),-- Angel Slayer
(29019,6371,1,1,5,200000),-- Demon Splinter
(29019,6579,1,1,5,200000),-- Arcana Mace
(29019,7575,1,1,5,200000),-- Draconic Bow
(29019,6326,1,5,6,300000),-- Sealed Majestic Necklace
(29019,6327,1,5,6,350000),-- Sealed Majestic Earring
(29019,6328,1,5,6,350000),-- Sealed Majestic Ring
(29019,6323,1,5,7,300000),-- Sealed Phoenix Necklace
(29019,6324,1,5,7,350000),-- Sealed Phoenix Earring
(29019,6325,1,5,7,350000),-- Sealed Phoenix Ring
(29019,6724,1,1,8,350000),-- Sealed Tateossian Earring 
(29019,6725,1,1,8,350000),-- Sealed Tateossian Ring
(29019,6726,1,1,8,300000),-- Sealed Tateossian Necklace
(29019,5287,1,3,9,500000),-- Sealed Dark Crystal breastplate
(29019,5297,1,3,9,250000),-- Sealed Dark Crystal Leather Armor
(29019,5304,1,3,9,250000),-- Sealed Tallum Tunic
(29019,5311,1,3,10,350000),-- Sealed Armor of Nightmare
(29019,5320,1,3,10,350000),-- Sealed Leather Armor of Nightmare
(29019,5326,1,3,10,300000),-- Sealed Robe of Nightmare
(29019,5293,1,3,11,250000),-- Sealed Tallum Plate Armor
(29019,5301,1,3,11,250000),-- Sealed Tallum Leather Armor
(29019,5308,1,3,11,250000),-- Sealed Dark Crystal Robe
(29019,5316,1,3,11,80000),-- Sealed Majestic Plate Armor
(29019,5323,1,3,11,80000),-- Sealed Majestic Leather Armor
(29019,5329,1,3,11,90000),-- Sealed Majestic Robe
(29019,5288,1,3,12,350000),-- Sealed Dark Crystal Gaiters
(29019,5298,1,3,12,350000),-- Sealed Dark Crystalline Leggings
(29019,5305,1,3,12,300000),-- Sealed Tallum Stockings
(29019,6674,1,1,13,150000),-- Sealed Imperial Crusader Breastplate
(29019,6675,1,1,13,150000),-- Sealed Imperial Crusader Gaiters
(29019,6680,1,1,13,350000),-- Sealed Draconic Leather Armor
(29019,6684,1,1,13,350000),-- Sealed Major Arcana Robe
(29019,5289,1,3,14,250000),-- Sealed Dark Crystal Helmet
(29019,5294,1,3,14,250000),-- Sealed Tallum Helmet
(29019,5312,1,3,14,250000),-- Sealed Helm of Nightmare
(29019,5317,1,3,14,250000),-- Sealed Majestic Circlet
(29019,5291,1,3,15,250000),-- Sealed Dark Crystal Boots
(29019,5296,1,3,15,250000),-- Sealed Tallum Boots
(29019,5314,1,3,15,250000),-- Sealed Boots of Nightmare
(29019,5319,1,3,15,250000),-- Sealed Majestic Boots
(29019,5290,1,3,16,250000),-- Sealed Dark Crystal Gloves
(29019,5295,1,3,16,250000),-- Sealed Tallum Gloves
(29019,5313,1,3,16,250000),-- Sealed Gloves of Nightmare
(29019,5318,1,3,16,250000),-- Sealed Majestic Gloves
(29019,5292,1,3,17,400000),-- Sealed Dark Crystal Shield
(29019,5315,1,3,17,400000),-- Sealed Shield of Nightmare
(29019,6678,1,1,17,200000),-- Sealed Imperial Crusader Shield
(29019,6676,1,1,18,340000),-- Sealed Imperial Crusader Gauntlet
(29019,6681,1,1,18,330000),-- Sealed Draconic Leather Glove
(29019,6685,1,1,18,330000),-- Sealed Major Arcana Glove
(29019,6677,1,1,19,340000),-- Sealed Imperial Crusader Boots
(29019,6682,1,1,19,330000),-- Sealed Draconic Leather Boots
(29019,6686,1,1,19,330000),-- Sealed Major Arcana Boots
(29019,6679,1,1,20,340000),-- Sealed Imperial Crusader Helmet
(29019,6683,1,1,20,330000),-- Sealed Draconic Leather Helmet
(29019,6687,1,1,20,330000),-- Sealed Major Arcana Circlet
(29019,729,1,19,21,1000000),-- Scroll: Enchant Weapon (Grade A)
(29019,1538,1,59,22,1000000),-- Blessed Scroll of Escape
(29019,3936,1,39,23,1000000),-- Blessed Scroll of Resurrection
(29019,57,14000000,18000000,24,1000000),-- Adena
(29019,57,14000000,18000000,25,1000000),-- Adena
(29019,57,14000000,18000000,26,1000000),-- Adena
(29019,57,14000000,18000000,27,1000000),-- Adena
(29019,57,14000000,18000000,28,1000000),-- Adena
(29019,57,14000000,18000000,29,1000000),-- Adena
(29019,57,9000000,13000000,30,1000000),-- Adena
(29019,6656,1,1,31,1000000),-- Earring of Antharas
(29019,10549,1,1,32,333334),-- Forgotten Scroll - Protection of Rune
(29019,10550,1,1,32,333333),-- Forgotten Scroll - Protection of Elemental
(29019,10551,1,1,32,333333),-- Forgotten Scroll - Protection of Alignment
(29019,13893,1,2,33,1000000),-- Sealed Holy Spirit's Cloak
(29019,8751,2,4,200,450000),-- High-Grade Life Stone: level 70
(29019,8761,1,2,200,100000),-- Top-Grade Life Stone: level 70
(29019,8752,2,4,200,400000),-- High-Grade Life Stone: level 76
(29019,8762,1,2,200,50000),-- Top-Grade Life Stone: level 76
(29020,80,1,1,0,60000),-- Tallum Blade
(29020,98,1,1,0,60000),-- Halberd
(29020,150,1,1,0,60000),-- Elemental Sword
(29020,288,1,1,0,60000),-- Carnage Bow
(29020,6364,1,1,0,12000),-- Forgotten Blade
(29020,6365,1,1,0,12000),-- Basalt Battlehammer
(29020,6367,1,1,0,9000),-- Angel Slayer
(29020,6369,1,1,0,9000),-- Dragon Hunter Axe
(29020,6370,1,1,0,9000),-- Saint Spear
(29020,6580,1,1,0,9000),-- Tallum Blade*Dark Legion\'s Edge
(29020,212,1,1,1,60000),-- Dasparion\'s Staff
(29020,235,1,1,1,60000),-- Bloody Orchid
(29020,269,1,1,1,60000),-- Blood Tornado
(29020,2504,1,1,1,60000),-- Meteor Shower
(29020,6366,1,1,1,12000),-- Imperial Staff
(29020,6371,1,1,1,12000),-- Demon Splinter
(29020,6372,1,1,1,12000),-- Heaven\'s Divider
(29020,6579,1,1,1,12000),-- Arcana Mace
(29020,7575,1,1,1,12000),-- Draconic Bow
(29020,6658,1,1,2,1000000),-- Ring of Baium
(29020,8751,1,2,200,450000),-- High-Grade Life Stone: level 70
(29020,8761,1,1,200,100000),-- Top-Grade Life Stone: level 70
(29020,8752,1,2,200,400000),-- High-Grade Life Stone: level 76
(29020,8762,1,1,200,50000),-- Top-Grade Life Stone: level 76
(29022,135,1,1,0,48000),-- Samurai Longsword
(29022,204,1,1,0,12000),-- Deadman\'s Staff
(29022,205,1,1,0,12000),-- Ghoul\'s Staff
(29022,206,1,1,0,15000),-- Demon\'s Staff
(29022,228,1,1,0,48000),-- Crystal Dagger
(29022,266,1,1,0,30000),-- Great Pata
(29022,286,1,1,0,39000),-- Eminence Bow
(29022,299,1,1,0,30000),-- Orcish Poleaxe
(29022,2503,1,1,0,36000),-- Yaksa Mace
(29022,5286,1,1,0,30000),-- Berserker Blade
(29022,78,1,1,1,10000),-- Great Sword
(29022,91,1,1,1,12000),-- Heavy War Axe
(29022,92,1,1,1,13000),-- Sprite\'s Staff
(29022,142,1,1,1,8000),-- Keshanberk
(29022,148,1,1,1,8000),-- Sword of Valhalla
(29022,229,1,1,1,8000),-- Kris
(29022,243,1,1,1,8000),-- Hell Knife
(29022,267,1,1,1,10000),-- Arthro Nail
(29022,284,1,1,1,13000),-- Dark Elven Long Bow
(29022,300,1,1,1,10000),-- Great Axe
(29022,357,1,1,2,50000),-- Zubei\'s Breastplate
(29022,2376,1,1,2,50000),-- Avadon Breastplate
(29022,2384,1,1,2,50000),-- Zubei\'s Leather Shirt
(29022,2397,1,1,2,50000),-- Tunic of Zubei
(29022,383,1,1,3,50000),-- Zubei\'s Gaiters
(29022,2379,1,1,3,50000),-- Avadon Gaiters
(29022,2388,1,1,3,50000),-- Zubei\'s Leather Gaiters
(29022,2402,1,1,3,50000),-- Stockings of Zubei
(29022,2390,1,1,4,100000),-- Avadon Leather Armor
(29022,2406,1,1,4,100000),-- Avadon Robe
(29022,503,1,1,5,30000),-- Zubei\'s Helmet
(29022,554,1,1,5,30000),-- Zubei\'s Boots
(29022,600,1,1,5,30000),-- Avadon Boots
(29022,612,1,1,5,30000),-- Zubei\'s Gauntlets
(29022,633,1,1,5,30000),-- Zubei\'s Shield
(29022,673,1,1,5,30000),-- Avadon Shield
(29022,947,1,1,5,60000),-- Scroll: Enchant Weapon (Grade B)
(29022,2415,1,1,5,30000),-- Avadon Circlet
(29022,2464,1,1,5,30000),-- Avadon Gloves
(29022,57,180000,220000,6,1000000),-- Adena
(29022,57,180000,220000,7,1000000),-- Adena
(29022,57,180000,220000,8,1000000),-- Adena
(29022,57,180000,220000,9,1000000),-- Adena
(29022,57,180000,220000,10,1000000),-- Adena
(29022,57,180000,220000,11,1000000),-- Adena
(29022,57,180000,220000,12,1000000),-- Adena
(29022,6659,1,1,13,1000000),-- Zaken\'s Earring
(29022,8922,1,1,14,100000),-- Pirate Hat
(29022,10295,1,1,15,50000),-- Transform Sealbook - Zaken
(29022,8747,2,4,200,450000),-- High-Grade Life Stone: level 58
(29022,8757,1,2,200,100000),-- Top-Grade Life Stone: level 58
(29022,8748,2,4,200,400000),-- High-Grade Life Stone: level 61
(29022,8758,1,2,200,50000),-- Top-Grade Life Stone: level 61
(29023,1061,4,8,2,200000),-- Healing Potion
(29024,1061,4,8,2,200000),-- Healing Potion
(29026,1061,4,8,2,200000),-- Healing Potion
(29027,1061,4,8,2,200000),-- Healing Potion
(29028,80,1,2,0,250000),-- Tallum Blade
(29028,98,1,2,0,250000),-- Halberd
(29028,269,1,2,0,250000),-- Blood Tornado
(29028,2504,1,2,0,250000),-- Meteor Shower
(29028,150,1,2,1,250000),-- Elemental Sword
(29028,212,1,2,1,250000),-- Dasparion\'s Staff
(29028,235,1,2,1,250000),-- Bloody Orchid
(29028,288,1,2,1,250000),-- Carnage Bow
(29028,81,1,2,2,250000),-- Dragon Slayer
(29028,151,1,2,2,250000),-- Sword of Miracles 
(29028,164,1,2,2,250000),-- Elysian
(29028,2500,1,2,2,250000),-- Dark Legion\'s Edge
(29028,213,1,2,3,200000),-- Branch of The Mother Tree
(29028,236,1,2,3,200000),-- Soul Separator
(29028,270,1,2,3,200000),-- Dragon Grinder
(29028,289,1,2,3,200000),-- Soul Bow
(29028,305,1,2,3,200000),-- Tallum Glaive
(29028,6364,1,1,4,170000),-- Forgotten Blade
(29028,6365,1,1,4,170000),-- Basalt Battlehammer
(29028,6369,1,1,4,170000),-- Dragon Hunter Axe
(29028,6370,1,1,4,170000),-- Saint Spear
(29028,6372,1,1,4,160000),-- Heaven\'s Divider
(29028,6580,1,1,4,160000),-- Tallum Blade*Dark Legion\'s Edge
(29028,6366,1,1,5,200000),-- Imperial Staff
(29028,6367,1,1,5,200000),-- Angel Slayer
(29028,6371,1,1,5,200000),-- Demon Splinter
(29028,6579,1,1,5,200000),-- Arcana Mace
(29028,7575,1,1,5,200000),-- Draconic Bow
(29028,6326,1,5,6,300000),-- Sealed Majestic Necklace
(29028,6327,1,5,6,350000),-- Sealed Majestic Earring
(29028,6328,1,5,6,350000),-- Sealed Majestic Ring
(29028,6323,1,5,7,300000),-- Sealed Phoenix Necklace
(29028,6324,1,5,7,350000),-- Sealed Phoenix Earring
(29028,6325,1,5,7,350000),-- Sealed Phoenix Ring
(29028,6724,1,1,8,350000),-- Sealed Tateossian Earring 
(29028,6725,1,1,8,350000),-- Sealed Tateossian Ring
(29028,6726,1,1,8,300000),-- Sealed Tateossian Necklace
(29028,5287,1,3,9,500000),-- Sealed Dark Crystal breastplate
(29028,5297,1,3,9,250000),-- Sealed Dark Crystal Leather Armor
(29028,5304,1,3,9,250000),-- Sealed Tallum Tunic
(29028,5311,1,3,10,350000),-- Sealed Armor of Nightmare
(29028,5320,1,3,10,350000),-- Sealed Leather Armor of Nightmare
(29028,5326,1,3,10,300000),-- Sealed Robe of Nightmare
(29028,5293,1,3,11,250000),-- Sealed Tallum Plate Armor
(29028,5301,1,3,11,250000),-- Sealed Tallum Leather Armor
(29028,5308,1,3,11,250000),-- Sealed Dark Crystal Robe
(29028,5316,1,3,11,80000),-- Sealed Majestic Plate Armor
(29028,5323,1,3,11,80000),-- Sealed Majestic Leather Armor
(29028,5329,1,3,11,90000),-- Sealed Majestic Robe
(29028,5288,1,3,12,350000),-- Sealed Dark Crystal Gaiters
(29028,5298,1,3,12,350000),-- Sealed Dark Crystalline Leggings
(29028,5305,1,3,12,300000),-- Sealed Tallum Stockings
(29028,6674,1,1,13,150000),-- Sealed Imperial Crusader Breastplate
(29028,6675,1,1,13,150000),-- Sealed Imperial Crusader Gaiters
(29028,6680,1,1,13,350000),-- Sealed Draconic Leather Armor
(29028,6684,1,1,13,350000),-- Sealed Major Arcana Robe
(29028,5289,1,3,14,250000),-- Sealed Dark Crystal Helmet
(29028,5294,1,3,14,250000),-- Sealed Tallum Helmet
(29028,5312,1,3,14,250000),-- Sealed Helm of Nightmare
(29028,5317,1,3,14,250000),-- Sealed Majestic Circlet
(29028,5291,1,3,15,250000),-- Sealed Dark Crystal Boots
(29028,5296,1,3,15,250000),-- Sealed Tallum Boots
(29028,5314,1,3,15,250000),-- Sealed Boots of Nightmare
(29028,5319,1,3,15,250000),-- Sealed Majestic Boots
(29028,5290,1,3,16,250000),-- Sealed Dark Crystal Gloves
(29028,5295,1,3,16,250000),-- Sealed Tallum Gloves
(29028,5313,1,3,16,250000),-- Sealed Gloves of Nightmare
(29028,5318,1,3,16,250000),-- Sealed Majestic Gloves
(29028,5292,1,3,17,400000),-- Sealed Dark Crystal Shield
(29028,5315,1,3,17,400000),-- Sealed Shield of Nightmare
(29028,6678,1,1,17,200000),-- Sealed Imperial Crusader Shield
(29028,6676,1,1,18,340000),-- Sealed Imperial Crusader Gauntlet
(29028,6681,1,1,18,330000),-- Sealed Draconic Leather Glove
(29028,6685,1,1,18,330000),-- Sealed Major Arcana Glove
(29028,6677,1,1,19,340000),-- Sealed Imperial Crusader Boots
(29028,6682,1,1,19,330000),-- Sealed Draconic Leather Boots
(29028,6686,1,1,19,330000),-- Sealed Major Arcana Boots
(29028,6679,1,1,20,340000),-- Sealed Imperial Crusader Helmet
(29028,6683,1,1,20,330000),-- Sealed Draconic Leather Helmet
(29028,6687,1,1,20,330000),-- Sealed Major Arcana Circlet
(29028,729,1,19,21,1000000),-- Scroll: Enchant Weapon (Grade A)
(29028,1538,1,59,22,1000000),-- Blessed Scroll of Escape
(29028,3936,1,39,23,1000000),-- Blessed Scroll of Resurrection
(29028,57,14000000,18000000,24,1000000),-- Adena
(29028,57,14000000,18000000,25,1000000),-- Adena
(29028,57,14000000,18000000,26,1000000),-- Adena
(29028,57,14000000,18000000,27,1000000),-- Adena
(29028,57,14000000,18000000,28,1000000),-- Adena
(29028,57,14000000,18000000,29,1000000),-- Adena
(29028,57,14000000,18000000,30,1000000),-- Adena
(29028,57,14000000,18000000,31,1000000),-- Adena
(29028,57,14000000,18000000,32,1000000),-- Adena
(29028,57,2000000,6000000,33,1000000),-- Adena
(29028,6657,1,1,34,1000000),-- Necklace of Valakas
(29028,10549,1,1,35,333334),-- Forgotten Scroll - Protection of Rune
(29028,10550,1,1,35,333333),-- Forgotten Scroll - Protection of Elemental
(29028,10551,1,1,35,333333),-- Forgotten Scroll - Protection of Alignment
(29028,13893,1,2,36,1000000),-- Sealed Holy Spirit's Cloak
(29028,8751,2,4,200,450000),-- High-Grade Life Stone: level 70
(29028,8761,1,2,200,100000),-- Top-Grade Life Stone: level 70
(29028,8752,2,4,200,400000),-- High-Grade Life Stone: level 76
(29028,8762,1,2,200,50000),-- Top-Grade Life Stone: level 76
(29047,57,9000000,11000000,0,1000000),-- Adena
(29047,57,9000000,11000000,19,1000000),-- Adena
(29047,1538,1,30,14,1000000),-- Blessed Scroll of Escape
(29047,3936,1,10,13,1000000),-- Blessed Scroll of Resurrection
(29047,8191,1,1,12,1000000),-- Frintezza's Necklace
(29047,8751,2,4,200,450000),-- High-Grade Life Stone: level 70
(29047,8761,1,2,200,100000),-- Top-Grade Life Stone: level 70
(29047,8752,2,4,200,400000),-- High-Grade Life Stone: level 76
(29047,8762,1,2,200,50000),-- Top-Grade Life Stone: level 76
(29047,5304,1,1,9,210000),-- Sealed Tallum Tunic
(29047,5292,1,1,8,250000),-- Sealed Dark Crystal Shield 
(29047,6326,1,1,11,210000),-- Sealed Majestic Necklace
(29047,5305,1,1,9,175000),-- Sealed Tallum Stockings
(29047,5326,1,1,7,175000),-- Sealed Nightmare Robe
(29047,6684,1,1,17,100000),-- Sealed Major Arcana Robe
(29047,6675,1,1,16,100000),-- Sealed Imperial Crusader Gaiters
(29047,6724,1,1,18,140000),-- Sealed Tateossian Earring
(29047,5291,1,1,10,150000),-- Sealed Dark Crystal Boots
(29047,5308,1,1,10,108000),-- Sealed Dark Crystal Robe
(29047,150,1,1,4,100000),-- Elemental Sword
(29047,5287,1,1,10,210000),-- Sealed Dark Crystal breastplate
(29047,6686,1,1,17,136000),-- Sealed Major arcana boots
(29047,5315,1,1,8,150000),-- Sealed Shield of Nightmare
(29047,2504,1,1,4,100000),-- Meteor Shower
(29047,6676,1,1,16,132000),-- Sealed Imperial Crusade Gauntlet
(29047,6327,1,1,11,180000),-- Sealed Majestic Earring
(29047,6323,1,1,11,175000),-- Sealed Phoenix Necklace
(29047,6683,1,1,1,132000),-- Sealed Draconic Leather Helmet
(29047,5311,1,1,7,150000),-- Sealed Armor of Nightmare
(29047,5293,1,1,9,120000),-- Sealed Tallum Plate Armor
(29047,98,1,1,4,100000),-- Halberd
(29047,6679,1,1,16,132000),-- Sealed Imperial Crusade Helmet
(29047,5314,1,1,7,100000),-- Sealed Boots of Nightmare
(29047,5317,1,1,15,100000),-- Sealed Majestic Circlet
(29047,6726,1,1,18,140000),-- Sealed Tateossian Necklace
(29047,6685,1,1,17,136000),-- Sealed Major Arcana Glove
(29047,6687,1,1,17,136000),-- Sealed Major Arcana Circlet
(29047,5288,1,1,10,150000),-- Sealed Dark Crystal Gaiters
(29047,164,1,1,5,75000),-- Elysian
(29047,6725,1,1,18,120000),-- Sealed Tateossian Ring
(29047,6328,1,1,11,210000),-- Sealed Majestic Ring
(29047,5297,1,1,10,180000),-- Sealed Dark Crystal Leather Armor
(29047,6682,1,1,1,132000),-- Sealed Draconic Leather Boots
(29047,5298,1,1,10,175000),-- Sealed Dark Crystal Leggings
(29047,5320,1,1,7,175000),-- Sealed Leather Armor of Nightmare
(29047,5289,1,1,10,150000),-- Sealed Dark Crystal Helmet
(29047,6325,1,1,11,175000),-- Sealed Phoenix Ring
(29047,5294,1,1,9,150000),-- Sealed Tallum Helmet
(29047,6680,1,1,1,100000),-- Sealed Draconic Leather Armor
(29047,288,1,1,4,100000),-- Carnage Bow
(29047,5319,1,1,15,100000),-- Sealed Majestic Boots
(29047,5329,1,1,15,102000),-- Sealed Majestic Robe
(29047,5312,1,1,7,100000),-- Sealed Helm of Nightmare
(29047,80,1,1,4,100000),-- Tallum Blade
(29047,6681,1,1,1,132000),-- Sealed Draconic Leather Glove
(29047,6324,1,1,11,150000),-- Sealed Phoenix Earring
(29047,6677,1,1,16,132000),-- Sealed Imperial Crusade Boots
(29047,5313,1,1,7,100000),-- Sealed Gauntlets of Nightmare
(29047,289,1,1,5,60000),-- Soul Bow
(29047,5290,1,1,10,150000),-- Sealed Dark Crystal Gloves
(29047,5296,1,1,9,150000),-- Sealed Tallum Boots
(29047,5295,1,1,9,150000),-- Sealed Tallum Gloves
(29047,6369,1,1,6,36000),-- Dragon Hunter Axe
(29047,2500,1,1,5,75000),-- Dark Legion's Edge
(29047,235,1,1,4,100000),-- Bloody Orchid
(29047,269,1,1,4,100000),-- Blood Tornado
(29047,5323,1,1,15,102000),-- Sealed Majestic Leather Armor
(29047,212,1,1,4,100000),-- Dasparion's Staff
(29047,5301,1,1,9,108000),-- Sealed Tallum Leather Armor
(29047,213,1,1,5,60000),-- Branch of the Mother Tree
(29047,5318,1,1,15,100000),-- Sealed Majestic Gauntlets
(29047,5316,1,1,15,60000),-- Sealed Majestic Plate Armor
(29047,270,1,1,5,60000),-- Dragon Grinder
(29047,6674,1,1,16,100000),-- Sealed Imperial Crusade Breastplate
(29047,151,1,1,5,75000),-- Sword of Miracles
(29047,7575,1,1,6,40000),-- Draconic Bow
(29047,6372,1,1,6,36000),-- Heaven's Divider
(29047,6370,1,1,6,36000),-- Saint Spear
(29047,6365,1,1,6,36000),-- Basalt Battlegammer
(29047,81,1,1,5,75000),-- Dragon Slayer
(29047,6580,1,1,6,20000),-- Tallum Blade*Dark Legion's Edge
(29047,6678,1,1,8,100000),-- Sealed Imperial Crusader Shield
(29047,6366,1,1,6,40000),-- Imperial staff
(29047,305,1,1,5,60000),-- Tallum Glaive
(29047,6371,1,1,6,40000),-- Demon Splinter
(29047,6579,1,1,6,40000),-- Arcana Mace
(29047,236,1,1,5,60000),-- Soul Separator
(29047,6367,1,1,6,40000),-- Angel Slayer
(29047,6364,1,1,6,36000),-- Forgotten Blade
(29047,10549,1,1,20,333334),-- Forgotten Scroll - Protection of Rune
(29047,10550,1,1,20,333333),-- Forgotten Scroll - Protection of Elemental
(29047,10551,1,1,20,333333),-- Forgotten Scroll - Protection of Alignment
(29047,13893,1,2,21,700000);-- Sealed Holy Spirit's Cloak

INSERT INTO `droplist` VALUES
(29116,6713,4,10,2,252577),-- Sealed Major Arcana Boots Design
(29116,6703,8,24,2,10503),-- Sealed Imperial Crusader Gauntlets Design
(29116,6704,14,40,2,65483),-- Sealed Imperial Crusader Boots Design
(29116,6708,20,60,2,44201),-- Sealed Draconic Leather Gloves Fabric
(29116,6578,1,1,1,35718),-- Blessed Scroll: Enchant Armor (S)
(29116,6709,28,82,2,32146),-- Sealed Draconic Leather Boots Design
(29116,6712,36,108,2,24556),-- Sealed Major Arcana Gloves fabric
(29116,6676,1,1,2,6938),-- Sealed Imperial Crusader Gauntlet
(29116,6677,1,1,2,6938),-- Sealed Imperial Crusader Boots
(29116,6681,1,1,2,6938),-- Sealed Draconic Leather Glove
(29116,6682,1,1,2,6938),-- Sealed Draconic Leather Boots
(29116,6685,1,1,2,6938),-- Sealed Major Arcana Glove
(29116,6686,1,1,2,6938),-- Sealed Major Arcana Boots
(29116,6577,1,1,1,5358),-- Blessed Scroll: Enchant Weapon (S)
(29116,9458,1,1,3,3332),-- Dynasty Earrings
(29116,9461,1,1,3,3332),-- Dynasty Earrings
(29116,9464,1,1,3,3332),-- Dynasty Earrings
(29116,9467,1,1,3,3332),-- Dynasty Earrings
(29116,9470,1,1,3,3332),-- Dynasty Earrings
(29116,9473,1,1,3,3332),-- Dynasty Earrings
(29116,9476,1,1,3,3332),-- Dynasty Earrings
(29117,6710,11,31,2,287788),-- Sealed Draconic Leather Helmet Pattern
(29117,6714,16,48,2,188861),-- Sealed Major Arcana Circlet Pattern
(29117,6705,23,67,2,115459),-- Sealed Imperial Crusader Shield Part
(29117,10170,1,1,3,84497),-- Baylor\'s Earring
(29117,6578,1,1,1,71436),-- Blessed Scroll: Enchant Armor (S)
(29117,6706,60,180,2,50363),-- Sealed Imperial Crusader Helmet Pattern
(29117,6678,1,1,2,19817),-- Sealed Imperial Crusader Shield
(29117,6679,1,1,2,13876),-- Sealed Imperial Crusader Helmet
(29117,6683,1,1,2,13876),-- Sealed Draconic Leather Helmet
(29117,6687,1,1,2,13876),-- Sealed Major Arcana Circlet
(29117,6577,1,1,1,10715); -- Blessed Scroll: Enchant Weapon (S)

INSERT INTO `droplist` VALUES
(29030,6652,2,6,2,186739),-- Amulet: Protection of Valakas
(29030,6654,18,54,2,64840),-- Amulet: Flames of Valakas
(29030,6655,14,42,2,41683),-- Amulet: Slay Valakas
(29033,6652,2,6,2,189407),-- Amulet: Protection of Valakas
(29033,6654,18,54,2,65766),-- Amulet: Flames of Valakas
(29033,6655,14,42,2,42279),-- Amulet: Slay Valakas
(29036,6652,2,6,2,192247),-- Amulet: Protection of Valakas
(29036,6654,18,54,2,66752),-- Amulet: Flames of Valakas
(29036,6655,14,42,2,42912),-- Amulet: Slay Valakas
(29037,6652,2,6,2,194987),-- Amulet: Protection of Valakas
(29037,6654,18,54,2,67704),-- Amulet: Flames of Valakas
(29037,6655,14,42,2,43524),-- Amulet: Slay Valakas
(29040,6652,2,6,2,197626),-- Amulet: Protection of Valakas
(29040,6654,18,54,2,68620),-- Amulet: Flames of Valakas
(29040,6655,14,42,2,44113); -- Amulet: Slay Valakas

INSERT INTO `droplist` VALUES
(29095,6711,15,29,2,700000),-- Sealed Major Arcane Robe Part
(29095,6684,1,1,2,71428),-- Sealed Major Arcane Robe
(29095,6695,3,9,3,930000),-- Demon Splinter Blade
(29095,6371,1,1,3,23810),-- Demon Splinter
(29095,10298,1,1,5,50000),-- Transform Sealbook - Gordon
(29095,960,1,1,4,450000),-- Scroll: Enchant Armor (Grade S)
(29104,10015,1,1,1,1000000); -- Prison Gate Key - unknown chance

INSERT INTO `droplist` VALUES 
-- RB drops for clan skill items (25% chance drop until chances are confirmed)
-- admins who have drop multipliers might want to edit chances
-- Memento Mori
(25026,9814,1,3,100,250000),-- Katu Van Leader Atui (49)
(25044,9814,1,3,100,250000),-- Barion (47)
(25047,9814,1,3,100,250000),-- Karte (49)
(25155,9814,1,4,100,250000),-- Shaman King Selu (40)
(25335,9814,1,2,100,250000),-- Anakazel (48)
(25410,9814,1,2,100,250000),-- Road Scavenger Leader (40)
(25412,9814,1,3,100,250000),-- Necrosentinel Royal Guard (47)
(25418,9814,1,3,100,250000),-- Dread Avenger Kraven (44)
(25437,9814,1,3,100,250000),-- Timak Orc Gosmos (45)
(25438,9814,1,3,100,250000),-- Thief Kelbar (44)
(25013,9814,2,5,100,250000),-- Ghost of Peasant Leader (50)
(25050,9814,1,3,100,250000),-- Verfa (51)
(25106,9814,1,4,100,250000),-- Ghost of the Well Lidia (60)
(25131,9814,1,3,100,250000),-- Carnage Lord Gato (50)
(25137,9814,1,4,100,250000),-- Beleth's Seer Sephia (55)
(25176,9814,1,4,100,250000),-- Black Lily (55)
(25256,9814,1,4,100,250000),-- Taik High Prefect Arak (60)
(25336,9814,1,3,100,250000),-- Anakazel (58)
(25407,9814,1,4,100,250000),-- Lord Ishka (60)
(25434,9814,1,4,100,250000),-- Bandit Leader Barda (55)
(25460,9814,1,3,100,250000),-- Deadman Ereve (51)
(25475,9814,1,4,100,250000),-- Ghost Knight Kabed (55)
(25035,9814,3,8,100,250000),-- Shilen's Messenger Cabrio (70)
(25051,9814,3,7,100,250000),-- Rahha (65)
(25073,9814,2,8,100,250000),-- Bloody Priest Rudelto (69)
(25092,9814,3,7,100,250000),-- Korim (70)
(25109,9814,3,7,100,250000),-- Antharas Priest Cloe (74)
(25220,9814,3,7,100,250000),-- Death Lord Hallate (73)
(25233,9814,2,4,100,250000),-- Spirit of Andras,the Betrayer (69)
(25235,9814,3,7,100,250000),-- Vanor Chief Kandra (72)
(25248,9814,3,7,100,250000),-- Doom Blade Tanatos (72)
(25281,9814,3,7,100,250000),-- Anakim's Nemesis Zakaron (70)
(25322,9814,3,7,100,250000),-- Demon's Agent Falston (66)
(25328,9814,3,7,100,250000),-- Eilhalder von Hellmann (71)
(25337,9814,1,3,100,250000),-- Anakazel (68)
(25444,9814,1,4,100,250000),-- Enmity Ghost Ramdal (65)
(25447,9814,2,4,100,250000),-- Immortal Savior Mardil (71)
(25481,9814,1,3,100,250000),-- Magus Kenishee (53)
(25484,9814,1,3,100,250000),-- Zaken Chief Mate Tillion (50)
(25182,9814,1,4,100,250000),-- Demon Kurikups (59)
(25032,9814,2,6,100,250000),-- Evas Guardian Millenu (58)
(25277,9814,2,5,100,250000),-- Liliths Witch Marilion (50)

-- Dragon's Heart
(25415,9815,1,2,100,250000),-- Nakondas (40)
(25441,9815,1,3,100,250000),-- Evil Spirit Cyrion (45)
(25234,9815,1,4,100,250000),-- Ancient Weird Drake (65)
(25238,9815,1,4,100,250000),-- Abyss Brukunt (67)
(25163,9815,3,7,100,250000),-- Roaring Skylancer (70)
(25198,9815,3,7,100,250000),-- Fafurion's Herald Lokness (70)
(25453,9815,3,7,100,250000),-- Meanas Anor (70)
(25241,9815,2,6,100,250000),-- Harit Hero Tamash (55)

-- Earth Egg
(25057,9816,1,3,100,250000),-- Biconne of Blue Sky (45)
(25085,9816,2,4,100,250000),-- Timak Orc Chief Ranger (44)
(25099,9816,1,3,100,250000),-- Rotten Tree Repiro (44)
(25134,9816,1,2,100,250000),-- Leto Chief Talkin (40)
(25192,9816,1,2,100,250000),-- Earth Protector Panathen (43)
(25208,9816,1,2,100,250000),-- Water Couatle Ateka (40)
(25420,9816,1,3,100,250000),-- Orfen's Handmaiden (48)
(25490,9816,1,2,100,250000),-- Gwindorr (40)
(25498,9816,1,3,100,250000),-- Fafurion's Henchman Istary (45)
(25067,9816,2,6,100,250000),-- Captain of Red Flag Shaka (52)
(25089,9816,1,4,100,250000),-- Soulless Wild Boar (59)
(25103,9816,2,3,100,250000),-- Sorcerer Isirr (55)
(25122,9816,1,4,100,250000),-- Refugee Hopeful Leo (56)
(25217,9816,1,3,100,250000),-- Cursed Clara (50)
(25230,9816,1,4,100,250000),-- Timak Seer Ragoth (57)
(25463,9816,1,4,100,250000),-- Harit Guardian Garangky (56)
(25473,9816,1,3,100,250000),-- Grave Robber Kim (52)
(25493,9816,1,4,100,250000),-- Eva's Spirit Niniel (55)
(25125,9816,3,7,100,250000),-- Fierce Tiger King Angel (65)
(25199,9816,3,7,100,250000),-- Water Dragon Seer Sheshark (72)
(25202,9816,3,7,100,250000),-- Krokian Padisha Sobekk (74)
(25226,9816,3,7,100,250000),-- Roaring Lord Kastor (62)
(25252,9816,3,7,100,250000),-- Palibati Queen Themis (70)
(25263,9816,3,7,100,250000),-- Kernon's Faithful Servant Kelone (67)
(25478,9816,1,5,100,250000),-- Shilen's Priest Hisilrome (65)
(25496,9816,1,3,100,250000),-- Fafurion's Envoy Pingolpin (52)
(25102,9816,1,3,100,250000),-- Shacram (45)
(25173,9816,1,3,100,250000),-- Tiger King Karuta (45)
(25007,9816,1,5,100,250000),-- Retreat Spider Cletu (42)

-- Nonliving Nucleus
(25115,9817,1,4,100,250000),-- Icarus Sample 1 (40)
(25158,9817,2,4,100,250000),-- King Tarlk (48)
(25214,9817,1,2,100,250000),-- Fafurion's Page Sika (40)
(25260,9817,2,4,100,250000),-- Iron Giant Totem (45)
(25162,9817,2,6,100,250000),-- Giant Marpanak (60)
(25179,9817,1,4,100,250000),-- Guardian of the Statue of Giant Karum (60)
(25140,9817,3,7,100,250000),-- Hekaton Prime (65)
(25255,9817,3,7,100,250000),-- Gargoyle Lord Tiphon (65)
(25269,9817,3,7,100,250000),-- Beast Lord Behemoth (70)
(25296,9817,3,7,100,250000),-- Icicle Emperor Bumbalump (74)
(25467,9817,1,2,100,250000),-- Gorgolos (64)
(25470,9817,1,5,100,250000),-- Last Titan Utenus (66)
(25523,9817,3,7,100,250000),-- Plague Golem (73)
(25259,9817,2,6,100,250000),-- Zaken Butcher Krantz (55)
(25029,9817,2,6,100,250000),-- Atraiban (53)
(25273,9817,2,5,100,250000),-- Carnamakos (50)
(25395,9817,1,3,100,250000),-- Archon Suscepter (45)
(25016,9817,2,7,100,250000),-- The 3rd Underwater Guardian (60)
(25088,9817,2,4,100,250000),-- Crazy Mechanic Golem (43)
(25431,9817,1,3,100,250000),-- Flamestone Golem (44)

-- Angelic Essence
(25064,8170,1,3,100,250000),-- Wizard of Storm Teruk (40)
(25487,8170,1,2,100,250000),-- Water Spirit Lian (40)
(25010,8171,2,7,100,250000),-- Furious Thieles (55)
(25070,8171,1,4,100,250000),-- Enchanted Forest Watcher Ruell (55)
(25159,8171,1,3,100,250000),-- Paniel The Unicorn (54)
(25280,8171,2,6,100,250000),-- Pagan Watcher Cerberon (55)
(25325,8172,3,7,100,250000),-- Flame of Splendor Barakiel (70)
(25423,8172,1,4,100,250000),-- Fairy Queen Timiniel (56)
(25119,8172,2,5,100,250000),-- Messenger of Fairy Queen Berun (50)
(25456,8172,1,4,100,250000),-- Mirror of Oblivion (49)

-- Destruction Tombstone
(25010,8176,1,3,101,250000),-- Furious Thieles (55)
(25013,8176,1,3,101,250000),-- Ghost of Peasant Leader (50)
(25016,8176,1,3,101,250000),-- The 3rd Underwater Guardian (60)
(25026,8176,1,1,101,250000),-- Katu Van Leader Atui (49)
(25035,8176,2,3,101,250000),-- Shilen's Messenger Cabrio (70)
(25044,8176,1,1,101,250000),-- Barion (47)
(25050,8176,1,1,101,250000),-- Verfa (51)
(25051,8176,2,3,101,250000),-- Rahha (65)
(25057,8176,1,1,101,250000),-- Biconne of Blue sky (45)
(25064,8176,1,1,101,250000),-- Wizard of Storm Teruk (40)
(25067,8176,1,3,101,250000),-- Captain of Red Flag Shaka (52)
(25070,8176,1,1,101,250000),-- Enchanted Forest Watcher Ruell (55)
(25073,8176,1,4,101,250000),-- Bloody Priest Rudelto (69)
(25088,8176,1,2,101,250000),-- Crazy Mechanic Golem (43)
(25089,8176,1,2,101,250000),-- Soulless Wild Boar (59) 
(25092,8176,1,4,101,250000),-- Korim (70)
(25102,8176,1,1,101,250000),-- Shacram (45) 
(25103,8176,1,1,101,250000),-- Sorcerer Isirr (55)
(25106,8176,1,2,101,250000),-- Ghost of the Well Lidia (60)
(25109,8176,1,4,101,250000),-- Antharas Priest Cloe (74)
(25115,8176,1,2,101,250000),-- Icarus Sample 1 (40) 
(25122,8176,1,2,101,250000),-- Refugee Hopeful Leo (56)
(25125,8176,1,4,101,250000),-- Fierce Tiger King Angel (65)
(25131,8176,1,1,101,250000),-- Carnage Lord Gato (50)
(25134,8176,1,1,101,250000),-- Leto Chief Talkin (40) 
(25140,8176,1,4,101,250000),-- Hekaton Prime (65)
(25155,8176,1,2,101,250000),-- Shaman King Selu (55) 
(25158,8176,1,2,101,250000),-- King Tarlk (48)
(25159,8176,1,1,101,250000),-- Paniel the Unicorn (54)
(25162,8176,1,3,101,250000),-- Giant Marpanak (60)
(25163,8176,1,4,101,250000),-- Roaring Skylancer (70)
(25176,8176,1,1,101,250000),-- Black Lily (55)
(25179,8176,1,1,101,250000),-- Guardian of the Statue of Giant Karum (60)
(25192,8176,1,1,101,250000),-- Earth Protector Panathen (43) 
(25198,8176,1,4,101,250000),-- Fafurion''s Herald Lokness (70)
(25199,8176,1,4,101,250000),-- Water Dragon Seer Sheshark (72)
(25202,8176,1,4,101,250000),-- Krokian Padisha Sobekk (74)
(25208,8176,1,1,101,250000),-- Water Couatle Ateka (40)
(25217,8176,1,1,101,250000),-- Cursed Clara (50)
(25220,8176,1,4,101,250000),-- Death Lord Hallate (73)
(25226,8176,1,4,101,250000),-- Roaring Lord Kastor (62)
(25230,8176,1,2,101,250000),-- Timak Seer Ragoth (57)
(25233,8176,1,2,101,250000),-- Spirit of Andras,the Betrayer (69)
(25234,8176,1,2,101,250000),-- Ancient Weird Drake (60)
(25235,8176,1,4,101,250000),-- Vanor Chief Kandra (72)
(25238,8176,1,2,101,250000),-- Abyss Brukunt (59)
(25248,8176,1,4,101,250000),-- Doom Blade tanatos (72)
(25252,8176,1,4,101,250000),-- Palibati Queen Themis (70)
(25255,8176,1,4,101,250000),-- Gargoyle Lord Tiphon (65)
(25256,8176,1,2,101,250000),-- Taik High Prefect Arak (60)
(25263,8176,1,4,101,250000),-- Kernon's Faithful Servant Kelone (67)
(25269,8176,1,4,101,250000),-- Beast Lord Behemoth (70)
(25281,8176,1,4,101,250000),-- Anakim's Nemesis Zakaron (70)
(25296,8176,1,4,101,250000),-- Icicle Emperor Bumbalump (74)
(25322,8176,1,4,101,250000),-- Demon's Agent Falston (66)
(25325,8176,1,4,101,250000),-- Flame of Splendor Barakiel (70)
(25328,8176,1,4,101,250000),-- Eilhalder von Hellmann (71)
(25335,8176,1,1,101,250000),-- Anakazel (48) 
(25336,8176,1,1,101,250000),-- Anakazel (58)
(25337,8176,1,1,101,250000),-- Anakazel (68)
(25407,8176,1,2,101,250000),-- Lord Ishka (60)
(25410,8176,1,1,101,250000),-- Road Scavenger Leader (40) 
(25412,8176,1,1,101,250000),-- Necrosentinel Royal Guard (47) 
(25415,8176,1,1,101,250000),-- Nakondas (40) 
(25418,8176,1,1,101,250000),-- Dread Avenger Kraven (44)
(25420,8176,1,1,101,250000),-- Orfen's Handmaiden (48)
(25423,8176,1,2,101,250000),-- Fairy Queen Timiniel (56)
(25434,8176,1,2,101,250000),-- Bandit Leader Barda (55)
(25437,8176,1,1,101,250000),-- Timak Orc Gosmos (45)
(25438,8176,1,1,101,250000),-- Thief Kelbar (44)
(25441,8176,1,1,101,250000),-- Evil Spirit Cyrion (45)
(25444,8176,1,2,101,250000),-- Enmity Ghost Ramdal (65)
(25447,8176,1,2,101,250000),-- Immortal Savior Mardil (71)
(25453,8176,1,4,101,250000),-- Meanas Anor (70)
(25456,8176,1,2,101,250000),-- Mirror of Oblivion (49)
(25460,8176,1,1,101,250000),-- Deadman Ereve (51)
(25467,8176,1,2,101,250000),-- Gorgolos (64)
(25470,8176,1,2,101,250000),-- Last Titan utenus (66)
(25473,8176,1,1,101,250000),-- Grave Rober kim (52)
(25475,8176,1,1,101,250000),-- Ghost Knight Kabed (55)
(25478,8176,1,2,101,250000),-- Shilen's Priest Hisilrome (65)
(25487,8176,1,1,101,250000),-- Water Spirit Lian (40) 
(25490,8176,1,1,101,250000),-- Gwindorr (40) 
(25493,8176,1,2,101,250000),-- Eva's Spirit Niniel (55)
(25498,8176,1,1,101,250000),-- Fafurion's Henchman Istary (45)
(25523,8176,1,4,101,250000),-- Plague Golem (73)
(25395,8176,1,1,101,250000),-- Archon Suscepter (45)
(25029,8176,1,3,101,250000),-- Atraiban (53)
(25137,8176,1,4,101,250000),-- Beleth Seer Sephia (55)
(25273,8176,1,3,101,250000),-- Carnamakos (50)
(25182,8176,1,2,101,250000),-- Demon Kurikupss (59)
(25032,8176,1,3,101,250000),-- Eva's Guardian Millenu (58)
(25496,8176,1,1,101,250000),-- Fafurion's Envoy Pingolpin (52)
(25214,8176,1,1,101,250000),-- Fafurion's Page Sika (40)
(25431,8176,1,1,101,250000),-- Flamestone Golem (44)
(25463,8176,1,2,101,250000),-- Harit Guardian Garanky (56)
(25241,8176,1,3,101,250000),-- Harit Hero Tamash (55)
(25260,8176,1,2,101,250000),-- Iron Giant Totem (45)
(25047,8176,1,1,101,250000),-- Karte (49)
(25277,8176,1,3,101,250000),-- Lilith Witch Marilion (50)
(25481,8176,1,1,101,250000),-- Magus Kenishee (53)
(25119,8176,1,2,101,250000),-- Messenger of Fairy Queen Berun (50)
(25280,8176,1,3,101,250000),-- Pagan Watcher Cerberon (55)
(25007,8176,1,2,101,250000),-- Retreat Spider Cletu (42)
(25099,8176,1,2,101,250000),-- Rotten Tree Repiro (44)
(25173,8176,1,1,101,250000),-- Tiger King Karuta (45)
(25085,8176,1,2,101,250000),-- Timak Orc Chief Ranger (44)
(25259,8176,1,3,101,250000),-- Zaken Butcher Krantz (55)
(25484,8176,1,1,101,250000),-- Zaken Chief Mate Tillion (50)

-- Ancient Book - Divine Inspiration (Original Language Version)
(25453,8619,1,1,102,250000),-- Meanas Anor (70)
(25198,8619,1,1,102,250000),-- Fafurion's Herald Lokness (70)
(25163,8619,1,1,102,250000),-- Roaring Skylancer (70)
(25220,8619,1,1,102,250000),-- Death Lord Hallate (73)
(25035,8619,1,1,102,250000),-- Shilen's Messenger Cabrio (70)
(25248,8619,1,1,102,250000),-- Doom Blade Tanatos (72)
(25092,8619,1,1,102,250000),-- Korim (70)
(25109,8619,1,1,102,250000),-- Antharas Priest Cloe (74)
(25233,8619,1,1,102,250000),-- Spirit of Andras\, the Betrayer (69)
(25073,8619,1,1,102,250000),-- Bloody Priest Rudelto (69)
(25337,8619,1,1,102,250000),-- Anakazel (68)
(25235,8619,1,1,102,250000),-- Vanor Chief Kandra (72)
(25328,8619,1,1,102,250000),-- Eilhalder von Hellmann (71)
(25447,8619,1,1,102,250000),-- Immortal Savior Mardil (71)
(25252,8619,1,1,102,250000),-- Palibati Queen Themis (70)
(25199,8619,1,1,102,250000),-- Water Dragon Seer Sheshark (72)
(25202,8619,1,1,102,250000),-- Krokian Padisha Sobekk (74)
(25269,8619,1,1,102,250000),-- Beast Lord Behemoth (70)
(25325,8619,1,1,102,250000),-- Flame of Splendor Barakiel (70)
(25470,8619,1,1,102,250000),-- Last Titan Utenus (66)
(25296,8619,1,1,102,250000),-- Icicle Emperor Bumbalump (74)

-- Ancient Book - Divine Inspiration (Manuscript)
(29019,8620,12,29,102,1000000),-- Antharas (79)
(29020,8620,5,15,102,800000),-- Baium (75)
(29028,8620,12,19,102,1000000),-- Valakas (85)
(29047,8620,6,25,102,1000000); -- Scarlet Van Halisha (90)

INSERT INTO `droplist` VALUES
-- Shanty Fortress
(35670,9912,1,3,1,1000000),-- Knight's Epaulette
(35671,9912,1,3,1,1000000),-- Knight's Epaulette
(35672,9912,1,3,1,1000000),-- Knight's Epaulette
(35673,9912,2,6,1,1000000),-- Knight's Epaulette
(35674,9912,2,6,1,1000000),-- Knight's Epaulette
(35676,9912,1,3,1,1000000),-- Knight's Epaulette
(35677,9912,127,127,1,1000000),-- Knight's Epaulette
(35678,9912,4,7,1,1000000),-- Knight's Epaulette
(35679,9912,4,7,1,1000000),-- Knight's Epaulette
(35680,9912,1,3,1,1000000),-- Knight's Epaulette
(35681,9912,3,6,1,1000000),-- Knight's Epaulette
(35682,9912,2,8,1,1000000),-- Knight's Epaulette
(35683,9912,1,3,1,1000000),-- Knight's Epaulette
(35684,9912,3,7,1,1000000),-- Knight's Epaulette
-- Southern Fortress
(35702,9912,1,3,1,1000000),-- Knight's Epaulette
(35703,9912,7,7,1,1000000),-- Knight's Epaulette
(35704,9912,3,6,1,1000000),-- Knight's Epaulette
(35705,9912,2,6,1,1000000),-- Knight's Epaulette
(35706,9912,2,6,1,1000000),-- Knight's Epaulette
(35711,9912,35,35,1,100000),-- Knight's Epaulette
(35712,9912,1,3,1,1000000),-- Knight's Epaulette
(35713,9912,124,128,1,1000000),-- Knight's Epaulette
(35714,9912,2,4,1,1000000),-- Knight's Epaulette
(35715,9912,2,4,1,1000000),-- Knight's Epaulette
(35716,9912,76,150,1,1000000),-- Knight's Epaulette
(35717,9912,2,8,1,1000000),-- Knight's Epaulette
(35718,9912,3,8,1,1000000),-- Knight's Epaulette
(35719,9912,138,138,1,1000000),-- Knight's Epaulette
(35720,9912,2,8,1,1000000),-- Knight's Epaulette
(35721,9912,35,41,1,1000000),-- Knight's Epaulette
(35722,9912,1,3,1,1000000),-- Knight's Epaulette
-- Hive Fortress
(35739,9912,1,3,1,1000000),-- Knight's Epaulette
(35740,9912,7,7,1,1000000),-- Knight's Epaulette
(35741,9912,3,5,1,1000000),-- Knight's Epaulette
(35742,9912,2,6,1,1000000),-- Knight's Epaulette
(35743,9912,2,6,1,1000000),-- Knight's Epaulette
(35745,9912,1,3,1,1000000),-- Knight's Epaulette
(35746,9912,97,97,1,1000000),-- Knight's Epaulette
(35747,9912,1,3,1,1000000),-- Knight's Epaulette
(35748,9912,1,3,1,1000000),-- Knight's Epaulette
(35749,9912,73,83,1,1000000),-- Knight's Epaulette
(35750,9912,2,8,1,1000000),-- Knight's Epaulette
(35751,9912,2,7,1,1000000),-- Knight's Epaulette
(35752,9912,105,139,1,1000000),-- Knight's Epaulette
(35753,9912,2,8,1,1000000),-- Knight's Epaulette
-- Valley Fortress
(35771,9912,1,3,1,1000000),-- Knight's Epaulette
(35772,9912,2,8,1,1000000),-- Knight's Epaulette
(35773,9912,2,6,1,1000000),-- Knight's Epaulette
(35774,9912,2,6,1,1000000),-- Knight's Epaulette
(35775,9912,2,6,1,1000000),-- Knight's Epaulette
(35780,9912,34,34,1,100000),-- Knight's Epaulette
(35781,9912,2,2,1,1000000),-- Knight's Epaulette
(35782,9912,150,150,1,1000000),-- Knight's Epaulette
(35783,9912,4,4,1,1000000),-- Knight's Epaulette
(35784,9912,4,4,1,1000000),-- Knight's Epaulette
(35785,9912,73,143,1,1000000),-- Knight's Epaulette
(35786,9912,2,8,1,1000000),-- Knight's Epaulette
(35787,9912,2,8,1,1000000),-- Knight's Epaulette
(35788,9912,86,147,1,1000000),-- Knight's Epaulette
(35789,9912,2,8,1,1000000),-- Knight's Epaulette
(35790,9912,43,59,1,1000000),-- Knight's Epaulette
(35791,9912,1,3,1,1000000),-- Knight's Epaulette
-- Ivory Tower Fortress
(35808,9912,1,3,1,1000000),-- Knight's Epaulette
(35809,9912,1,3,1,1000000),-- Knight's Epaulette
(35810,9912,2,6,1,1000000),-- Knight's Epaulette
(35811,9912,2,6,1,1000000),-- Knight's Epaulette
(35812,9912,2,5,1,1000000),-- Knight's Epaulette
(35814,9912,1,3,1,1000000),-- Knight's Epaulette
(35815,9912,105,105,1,1000000),-- Knight's Epaulette
(35816,9912,1,3,1,1000000),-- Knight's Epaulette
(35817,9912,1,3,1,1000000),-- Knight's Epaulette
(35818,9912,134,134,1,1000000),-- Knight's Epaulette
(35819,9912,3,7,1,1000000),-- Knight's Epaulette
(35820,9912,3,7,1,1000000),-- Knight's Epaulette
(35821,9912,1,3,1,1000000),-- Knight's Epaulette
(35822,9912,4,8,1,1000000),-- Knight's Epaulette
-- Narsell Fortress
(35839,9912,1,3,1,1000000),-- Knight's Epaulette
(35840,9912,2,2,1,1000000),-- Knight's Epaulette
(35841,9912,4,4,1,1000000),-- Knight's Epaulette
(35842,9912,2,6,1,1000000),-- Knight's Epaulette
(35843,9912,2,6,1,1000000),-- Knight's Epaulette
(35845,9912,1,3,1,1000000),-- Knight's Epaulette
(35846,9912,117,117,1,1000000),-- Knight's Epaulette
(35847,9912,1,3,1,1000000),-- Knight's Epaulette
(35848,9912,1,3,1,1000000),-- Knight's Epaulette
(35849,9912,86,114,1,1000000),-- Knight's Epaulette
(35850,9912,2,8,1,1000000),-- Knight's Epaulette
(35851,9912,4,5,1,1000000),-- Knight's Epaulette
(35852,9912,126,126,1,1000000),-- Knight's Epaulette
(35853,9912,3,8,1,1000000),-- Knight's Epaulette
-- Bayou Fortress
(35871,9912,1,3,1,1000000),-- Knight's Epaulette
(35872,9912,1,3,1,1000000),-- Knight's Epaulette
(35873,9912,1,3,1,1000000),-- Knight's Epaulette
(35874,9912,3,6,1,1000000),-- Knight's Epaulette
(35875,9912,2,6,1,1000000),-- Knight's Epaulette
(35880,9912,1,3,1,1000000),-- Knight's Epaulette
(35881,9912,1,3,1,1000000),-- Knight's Epaulette
(35882,9912,1,3,1,1000000),-- Knight's Epaulette
(35883,9912,1,3,1,1000000),-- Knight's Epaulette
(35884,9912,1,3,1,1000000),-- Knight's Epaulette
(35885,9912,1,3,1,1000000),-- Knight's Epaulette
(35886,9912,3,8,1,1000000),-- Knight's Epaulette
(35887,9912,3,8,1,1000000),-- Knight's Epaulette
(35888,9912,1,3,1,1000000),-- Knight's Epaulette
(35889,9912,3,5,1,1000000),-- Knight's Epaulette
(35890,9912,1,3,1,1000000),-- Knight's Epaulette
(35891,9912,1,3,1,1000000),-- Knight's Epaulette
-- White Sand Fortress
(35908,9912,1,3,1,1000000),-- Knight's Epaulette
(35909,9912,8,8,1,1000000),-- Knight's Epaulette
(35910,9912,3,6,1,1000000),-- Knight's Epaulette
(35911,9912,2,6,1,1000000),-- Knight's Epaulette
(35912,9912,2,5,1,1000000),-- Knight's Epaulette
(35914,9912,1,3,1,1000000),-- Knight's Epaulette
(35915,9912,1,3,1,1000000),-- Knight's Epaulette
(35916,9912,5,7,1,1000000),-- Knight's Epaulette
(35917,9912,5,7,1,1000000),-- Knight's Epaulette
(35918,9912,1,3,1,1000000),-- Knight's Epaulette
(35919,9912,3,8,1,1000000),-- Knight's Epaulette
(35920,9912,3,7,1,1000000),-- Knight's Epaulette
(35921,9912,1,3,1,1000000),-- Knight's Epaulette
(35922,9912,2,6,1,1000000),-- Knight's Epaulette
-- Borderland Fortress
(35940,9912,1,3,1,1000000),-- Knight's Epaulette
(35941,9912,3,3,1,1000000),-- Knight's Epaulette
(35942,9912,3,6,1,1000000),-- Knight's Epaulette
(35943,9912,3,6,1,1000000),-- Knight's Epaulette
(35944,9912,4,6,1,1000000),-- Knight's Epaulette
(35949,9912,1,3,1,1000000),-- Knight's Epaulette
(35950,9912,1,3,1,1000000),-- Knight's Epaulette
(35951,9912,1,3,1,1000000),-- Knight's Epaulette
(35952,9912,1,3,1,1000000),-- Knight's Epaulette
(35953,9912,1,3,1,1000000),-- Knight's Epaulette
(35954,9912,1,3,1,1000000),-- Knight's Epaulette
(35955,9912,3,3,1,1000000),-- Knight's Epaulette
(35956,9912,6,7,1,1000000),-- Knight's Epaulette
(35957,9912,79,79,1,1000000),-- Knight's Epaulette
(35958,9912,4,4,1,1000000),-- Knight's Epaulette
(35959,9912,1,3,1,1000000),-- Knight's Epaulette
(35960,9912,1,3,1,1000000),-- Knight's Epaulette
-- Swamp Fortress
(35978,9912,1,3,1,1000000),-- Knight's Epaulette
(35979,9912,2,8,1,1000000),-- Knight's Epaulette
(35980,9912,2,6,1,1000000),-- Knight's Epaulette
(35981,9912,2,6,1,1000000),-- Knight's Epaulette
(35982,9912,2,6,1,1000000),-- Knight's Epaulette
(35987,9912,23,23,1,100000),-- Knight's Epaulette
(35988,9912,1,3,1,1000000),-- Knight's Epaulette
(35989,9912,83,109,1,1000000),-- Knight's Epaulette
(35990,9912,1,3,1,1000000),-- Knight's Epaulette
(35991,9912,1,3,1,1000000),-- Knight's Epaulette
(35992,9912,83,125,1,1000000),-- Knight's Epaulette
(35993,9912,2,7,1,1000000),-- Knight's Epaulette
(35994,9912,2,7,1,1000000),-- Knight's Epaulette
(35995,9912,82,139,1,1000000),-- Knight's Epaulette
(35996,9912,3,8,1,1000000),-- Knight's Epaulette
(35997,9912,40,71,1,1000000),-- Knight's Epaulette
(35998,9912,27,27,1,1000000),-- Knight's Epaulette
-- Archaic Fortress
(36015,9912,1,3,1,1000000),-- Knight's Epaulette
(36016,9912,3,6,1,1000000),-- Knight's Epaulette
(36017,9912,2,4,1,1000000),-- Knight's Epaulette
(36018,9912,2,6,1,1000000),-- Knight's Epaulette
(36019,9912,2,6,1,1000000),-- Knight's Epaulette
(36021,9912,5,6,1,1000000),-- Knight's Epaulette
(36022,9912,2,6,1,1000000),-- Knight's Epaulette
(36023,9912,2,6,1,1000000),-- Knight's Epaulette
(36024,9912,1,3,1,1000000),-- Knight's Epaulette
(36025,9912,87,87,1,1000000),-- Knight's Epaulette
(36026,9912,2,8,1,1000000),-- Knight's Epaulette
(36027,9912,3,7,1,1000000),-- Knight's Epaulette
(36028,9912,76,138,1,1000000),-- Knight's Epaulette
(36029,9912,2,8,1,1000000),-- Knight's Epaulette
-- Floran Fortress
(36047,9912,1,3,1,1000000),-- Knight's Epaulette
(36048,9912,1,3,1,1000000),-- Knight's Epaulette
(36049,9912,2,5,1,1000000),-- Knight's Epaulette
(36050,9912,2,6,1,1000000),-- Knight's Epaulette
(36051,9912,2,6,1,1000000),-- Knight's Epaulette
(36056,9912,23,23,1,200000),-- Knight's Epaulette
(36057,9912,2,3,1,1000000),-- Knight's Epaulette
(36058,9912,1,3,1,1000000),-- Knight's Epaulette
(36059,9912,6,7,1,1000000),-- Knight's Epaulette
(36060,9912,6,7,1,1000000),-- Knight's Epaulette
(36061,9912,1,3,1,1000000),-- Knight's Epaulette
(36062,9912,2,8,1,1000000),-- Knight's Epaulette
(36063,9912,2,8,1,1000000),-- Knight's Epaulette
(36064,9912,1,3,1,1000000),-- Knight's Epaulette
(36065,9912,2,8,1,1000000),-- Knight's Epaulette
(36066,9912,1,3,1,1000000),-- Knight's Epaulette
(36067,9912,1,3,1,1000000),-- Knight's Epaulette
(36069,9912,1,3,1,1000000),-- Knight's Epaulette
-- Cloud Mountain Fortress
(36085,9912,1,3,1,1000000),-- Knight's Epaulette
(36086,9912,2,7,1,1000000),-- Knight's Epaulette
(36087,9912,2,5,1,1000000),-- Knight's Epaulette
(36088,9912,1,3,1,1000000),-- Knight's Epaulette
(36089,9912,2,5,1,1000000),-- Knight's Epaulette
(36094,9912,13,13,1,200000),-- Knight's Epaulette
(36095,9912,1,3,1,1000000),-- Knight's Epaulette
(36096,9912,147,147,1,1000000),-- Knight's Epaulette
(36097,9912,3,3,1,1000000),-- Knight's Epaulette
(36098,9912,3,3,1,1000000),-- Knight's Epaulette
(36099,9912,1,3,1,1000000),-- Knight's Epaulette
(36100,9912,3,6,1,1000000),-- Knight's Epaulette
(36101,9912,3,6,1,1000000),-- Knight's Epaulette
(36102,9912,1,3,1,1000000),-- Knight's Epaulette
(36103,9912,5,6,1,1000000),-- Knight's Epaulette
-- Tanor Fortress
(36122,9912,1,3,1,1000000),-- Knight's Epaulette
(36123,9912,4,4,1,1000000),-- Knight's Epaulette
(36124,9912,3,6,1,1000000),-- Knight's Epaulette
(36125,9912,2,6,1,1000000),-- Knight's Epaulette
(36126,9912,2,6,1,1000000),-- Knight's Epaulette
(36128,9912,1,3,1,1000000),-- Knight's Epaulette
(36129,9912,1,3,1,1000000),-- Knight's Epaulette
(36130,9912,1,3,1,1000000),-- Knight's Epaulette
(36131,9912,1,3,1,1000000),-- Knight's Epaulette
(36132,9912,1,3,1,1000000),-- Knight's Epaulette
(36133,9912,3,6,1,1000000),-- Knight's Epaulette
(36134,9912,5,7,1,1000000),-- Knight's Epaulette
(36135,9912,1,3,1,1000000),-- Knight's Epaulette
(36136,9912,5,6,1,1000000),-- Knight's Epaulette
-- Dragonspine Fortress
(36153,9912,28,28,1,1000000),-- Knight's Epaulette
(36153,9581,1,1,1,100000),-- Scale Shirt
(36154,9912,3,7,1,1000000),-- Knight's Epaulette
(36155,9912,2,6,1,1000000),-- Knight's Epaulette
(36156,9912,2,6,1,1000000),-- Knight's Epaulette
(36157,9912,2,6,1,1000000),-- Knight's Epaulette
(36159,9912,1,3,1,1000000),-- Knight's Epaulette
(36160,9912,120,144,1,1000000),-- Knight's Epaulette
(36161,9912,2,7,1,1000000),-- Knight's Epaulette
(36162,9912,2,7,1,1000000),-- Knight's Epaulette
(36163,9912,103,103,1,1000000),-- Knight's Epaulette
(36164,9912,2,8,1,1000000),-- Knight's Epaulette
(36165,9912,2,7,1,1000000),-- Knight's Epaulette
(36166,9912,79,101,1,1000000),-- Knight's Epaulette
(36167,9912,2,8,1,1000000),-- Knight's Epaulette
-- Antharas Fortress
(36185,9912,1,3,1,1000000),-- Knight's Epaulette
(36186,9912,5,6,1,1000000),-- Knight's Epaulette
(36187,9912,3,6,1,1000000),-- Knight's Epaulette
(36188,9912,2,6,1,1000000),-- Knight's Epaulette
(36189,9912,2,6,1,1000000),-- Knight's Epaulette
(36194,9912,14,14,1,200000),-- Knight's Epaulette
(36195,9912,2,3,1,1000000),-- Knight's Epaulette
(36196,9912,130,130,1,1000000),-- Knight's Epaulette
(36197,9912,1,3,1,1000000),-- Knight's Epaulette
(36198,9912,1,3,1,1000000),-- Knight's Epaulette
(36199,9912,131,131,1,1000000),-- Knight's Epaulette
(36200,9912,2,8,1,1000000),-- Knight's Epaulette
(36201,9912,3,8,1,1000000),-- Knight's Epaulette
(36202,9912,133,133,1,1000000),-- Knight's Epaulette
(36203,9912,2,6,1,1000000),-- Knight's Epaulette
(36204,9912,42,75,1,1000000),-- Knight's Epaulette
(36205,9912,64,64,1,1000000),-- Knight's Epaulette
-- Western Guard Fortress
(36223,9912,1,3,1,1000000),-- Knight's Epaulette
(36224,9912,6,6,1,1000000),-- Knight's Epaulette
(36225,9912,4,5,1,1000000),-- Knight's Epaulette
(36226,9912,2,6,1,1000000),-- Knight's Epaulette
(36227,9912,2,5,1,1000000),-- Knight's Epaulette
(36232,9912,13,13,1,200000),-- Knight's Epaulette
(36233,9912,1,3,1,1000000),-- Knight's Epaulette
(36234,9912,1,3,1,1000000),-- Knight's Epaulette
(36235,9912,4,8,1,1000000),-- Knight's Epaulette
(36236,9912,4,8,1,1000000),-- Knight's Epaulette
(36237,9912,1,3,1,1000000),-- Knight's Epaulette
(36238,9912,2,8,1,1000000),-- Knight's Epaulette
(36239,9912,3,8,1,1000000),-- Knight's Epaulette
(36240,9912,1,3,1,1000000),-- Knight's Epaulette
(36241,9912,4,5,1,1000000),-- Knight's Epaulette
(36242,9912,1,3,1,1000000),-- Knight's Epaulette
(36243,9912,1,3,1,1000000),-- Knight's Epaulette
-- Hunter's Fortress
(36261,9912,1,3,1,1000000),-- Knight's Epaulette
(36262,9912,1,3,1,1000000),-- Knight's Epaulette
(36263,9912,3,5,1,1000000),-- Knight's Epaulette
(36264,9912,2,6,1,1000000),-- Knight's Epaulette
(36265,9912,2,6,1,1000000),-- Knight's Epaulette
(36270,9912,16,16,1,200000),-- Knight's Epaulette
(36271,9912,3,3,1,1000000),-- Knight's Epaulette
(36272,9912,1,3,1,1000000),-- Knight's Epaulette
(36273,9912,4,4,1,1000000),-- Knight's Epaulette
(36274,9912,4,4,1,1000000),-- Knight's Epaulette
(36275,9912,1,3,1,1000000),-- Knight's Epaulette
(36276,9912,2,8,1,1000000),-- Knight's Epaulette
(36277,9912,2,8,1,1000000),-- Knight's Epaulette
(36278,9912,123,123,1,1000000),-- Knight's Epaulette
(36279,9912,4,8,1,1000000),-- Knight's Epaulette
(36280,9912,1,3,1,1000000),-- Knight's Epaulette
(36281,9912,1,3,1,1000000),-- Knight's Epaulette
-- Aaru Fortress
(36298,9912,1,3,1,1000000),-- Knight's Epaulette
(36299,9912,7,8,1,1000000),-- Knight's Epaulette
(36300,9912,2,5,1,1000000),-- Knight's Epaulette
(36301,9912,2,6,1,1000000),-- Knight's Epaulette
(36302,9912,2,6,1,1000000),-- Knight's Epaulette
(36304,9912,1,3,1,1000000),-- Knight's Epaulette
(36305,9912,127,127,1,1000000),-- Knight's Epaulette
(36306,9912,1,3,1,1000000),-- Knight's Epaulette
(36307,9912,1,3,1,1000000),-- Knight's Epaulette
(36308,9912,119,147,1,1000000),-- Knight's Epaulette
(36309,9912,2,8,1,1000000),-- Knight's Epaulette
(36310,9912,4,7,1,1000000),-- Knight's Epaulette
(36311,9912,1,3,1,1000000),-- Knight's Epaulette
(36312,9912,3,7,1,1000000),-- Knight's Epaulette
-- Demon Fortress
(36330,9912,1,3,1,1000000),-- Knight's Epaulette
(36331,9912,3,8,1,1000000),-- Knight's Epaulette
(36332,9912,3,5,1,1000000),-- Knight's Epaulette
(36333,9912,3,6,1,1000000),-- Knight's Epaulette
(36334,9912,2,5,1,1000000),-- Knight's Epaulette
(36339,9912,1,3,1,1000000),-- Knight's Epaulette
(36340,9912,1,3,1,1000000),-- Knight's Epaulette
(36341,9912,1,3,1,1000000),-- Knight's Epaulette
(36342,9912,1,3,1,1000000),-- Knight's Epaulette
(36343,9912,1,3,1,1000000),-- Knight's Epaulette
(36344,9912,1,3,1,1000000),-- Knight's Epaulette
(36345,9912,2,7,1,1000000),-- Knight's Epaulette
(36346,9912,4,8,1,1000000),-- Knight's Epaulette
(36347,9912,1,3,1,1000000),-- Knight's Epaulette
(36348,9912,5,8,1,1000000),-- Knight's Epaulette
(36349,9912,1,3,1,1000000),-- Knight's Epaulette
(36350,9912,1,3,1,1000000),-- Knight's Epaulette
-- Monastic Fortress
(36368,9912,1,3,1,1000000),-- Knight's Epaulette
(36369,9912,1,3,1,1000000),-- Knight's Epaulette
(36370,9912,1,3,1,1000000),-- Knight's Epaulette
(36371,9912,2,6,1,1000000),-- Knight's Epaulette
(36372,9912,2,6,1,1000000),-- Knight's Epaulette
(36377,9912,1,3,1,1000000),-- Knight's Epaulette
(36378,9912,1,3,1,1000000),-- Knight's Epaulette
(36379,9912,1,3,1,1000000),-- Knight's Epaulette
(36380,9912,1,3,1,1000000),-- Knight's Epaulette
(36381,9912,1,3,1,1000000),-- Knight's Epaulette
(36382,9912,1,3,1,1000000),-- Knight's Epaulette
(36383,9912,3,8,1,1000000),-- Knight's Epaulette
(36384,9912,6,8,1,1000000),-- Knight's Epaulette
(36385,9912,1,3,1,1000000),-- Knight's Epaulette
(36386,9912,4,6,1,1000000),-- Knight's Epaulette
(36387,9912,1,3,1,1000000),-- Knight's Epaulette
(36388,9912,1,3,1,1000000); -- Knight's Epaulette